      subroutine comparesepar ( ibuffr, buffer, ivectr, kprob, &
                                intmat, namefile, text, threshold )
! ====================================================================== &
!
!        programmer    Guus Segal
!        version  1.2  date 20-06-2013 New definition file ref numbers
!        version  1.1  date 09-11-2012 Long length ibuffr
!        version  1.0  date 06-09-2004
!
!   copyright (c) 2004-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compare sepran array inside the program with sepran array stored in
!     unformatted form in a file and print the differences if they are
!     larger than the threshold value
!     The differences are printed to a file called namefile_dif
! **********************************************************************
!
!                       KEYWORDS
!
!     array
!     compare
!     file
!     sepran_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*)
      integer ivectr(5), kprob(*), intmat(*)
      double precision buffer(*), threshold
      character (len=*) namefile, text

!     buffer        i/o   Buffer array
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     intmat         i    Standard SEPRAN array of length 5 containing
!                         information concerning the structure of the matrix.
!                         See the Programmer's Guide 13.5 for a
!                         complete description.
!     ivectr         i    standard sepran array denoting USOL or vector of
!                         special structure defined in nodal points
!     kprob          i    Standard SEPRAN array, containing information of
!                         the problem
!     namefile       i    String variable containing name of the file
!                         from which the array to be compared is read
!     text           i    String variable containing text to be written in the
!                         heading
!     threshold      i    threshold value
!                         Only values with difference larger than threshold
!                         value are stored
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) iparray
      integer length, mmarray, lenname, ivectsav, iref
      character (len=80) namewrfile
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     iparray        Starting address of array
!     iref           File reference number
!     ivectsav       Help parameter to store the value of ivect(3)
!     length         Help parameter to indicate the length of arrays
!     lenname        Length of name
!     mmarray        Memory management sequence number of array
!     namewrfile     Name of the output file
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inidgttext
      integer inidln, inisetnewref

!     DIFSEPARS      Compute the difference between 2 sepran arrays and store
!                    difference in first one
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INI060TEXT     Reactivate existing array in IBUFFR
!     INI066TEXT     delete temporary array
!     INIDGTTEXT     Compute pointer of array in BUFFER
!     INIDLN         Compute length of array in BUFFER
!     INIRMREF       Remove entry iref from the set of reference numbers used
!     INISETNEWREF   Create new reference number
!     PRINMTTHRES    print large matrix
!                    all elements smaller than the threshld value are suppressed
!     PRINRLTHRES    print vector
!                    all elements smaller than the threshld value are suppressed
!     READSEPAR2FILE Read sepran real array stored in buffer (unformatted) from
!                    file named namefile
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1153   vector INVEC1 corresponds to a wrong type of vector, &
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      call eropen ( 'comparesepar' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from comparesepar'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      lenname = len(namefile)
      namewrfile = namefile(1:lenname)//'_dif'
      iref = inisetnewref ()
      open ( unit = iref, file = namewrfile(1:lenname+4) )

!     --- Read array and store difference

      if ( ivectr(2)==103 ) then

!     --- Vector is of type matrix, compare inner matrix

         mmarray = 0
         call readsepar2file ( ibuffr, buffer, mmarray, namefile )
         call difsepars ( ibuffr, buffer, ivectr(3), mmarray )
         irefwr = iref
         ivectsav = ivectr(3)
         ivectr(3) = mmarray
         call prinmtthres ( ibuffr, buffer, intmat, ivectr, kprob, &
                            threshold )
         ivectr(3) = ivectsav

      else if ( ivectr(2)==110 .or. ivectr(2)==115 ) then

!     --- Vector is of type solution

         mmarray = 0
         call readsepar2file ( ibuffr, buffer, mmarray, namefile )
         call difsepars ( ibuffr, buffer, ivectr(1), mmarray )
         call ini060text ( ibuffr, mmarray, 'dif_array' )
         iparray = inidgttext ( mmarray, 'dif_array' )
         length = inidln(mmarray)
         irefwr = iref
         call prinrlthres ( buffer(iparray), length, threshold, text )

      else

!     --- Vector is of wrong type

         call errint ( ivectr(2), 1 )
         call errsub ( 1153, 1, 0, 0 )
         go to 1000

      end if
      irefwr = 6

      call inirmref ( iref )  ! close file
      close(iref)

!     --- Remove temporary array

      call ini066text ( ibuffr, mmarray, 'array' )

1000  call erclos ( 'comparesepar' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End comparesepar'

      end if

      end
