      subroutine copydfbf ( ibuffr, buffer, ivec1, ivec2, kprob1, &
                            kprob2, idf1, idf2, kmesh )
! ====================================================================== &
!
!        programmer    Frans van de Vosse
!        version  1.4  date 29-05-2014 remove call to time subroutine
!        version  1.3  date 09-11-2012 Long length ibuffr
!        version  1.2  date 28-06-2006 iniget --> inigettext
!
!   copyright (c) 1998-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     copydfbf copies the contents of one degree of freedom in array ivec1
!     belonging to kprob1 into another degree of freedom in array ivec2
!     belonging to kprob2
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*)
      integer ivec1(*), ivec2(*), kprob1(*), kprob2(*), idf1, idf2, &
              kmesh(*)
      double precision buffer(*)

!     buffer    i/o   Real buffer array in which all large data is stored
!     ibuffr    i/o   Integer buffer array in which all large data is stored
!     idf1       i    degree of freedom in ivec1 to be copied
!     idf2       i    degree of freedom in ivec2 to be filled
!     ivec1      i    old array (must be an excisting SEPRAN array)
!     ivec2     i/o   new array (must be an excisting SEPRAN array)
!     kmesh      i    Standard SEPRAN array containing information of the mesh
!     kprob1     i    kprob of old array
!     kprob2     i    kprob of new array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8)  ipvec1, ipvec2, ipkpf1, ipkpf2, ipkph1, ipkph2
      integer itypv1, itypv2, inkpf1, inkpf2, inkph1, inkph2, &
              inkpp1, inkpp2, nphys1, nphys2
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     inkpf1         Index of kprob part f
!     inkpf2         Index of kprob part f
!     inkph1         Index of kprob part h
!     inkph2         Index of kprob part h
!     inkpp1         Index of kprob part p
!     inkpp2         Index of kprob part p
!     ipkpf1         Pointer to kprob part f
!     ipkpf2         Pointer to kprob part f
!     ipkph1         Pointer to kprob part h
!     ipkph2         Pointer to kprob part h
!     ipvec1         Pointer to vec2
!     ipvec2         Pointer to vec2
!     itypv1         Type number of vector 1
!     itypv2         Type number of vector 2
!     nphys1         Maximal number of degrees of freedom per point or the
!                    number of physical variables if defined
!     nphys2         Maximal number of degrees of freedom per point or the
!                    number of physical variables if defined
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inigettext, inidgttext

!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERCPDF01       Copies the contents of one degree of freedom in array vec1
!                    belonging to kprob1 into another degree of freedom in
!                    array vec2
!                    belonging to kprob2
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INI065         compute pointers to degrees of freedom of vectors
!     INIDGTTEXT     Compute pointer of array in BUFFER
!     INIGETTEXT     Compute pointer of array in IBUFFR
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      13   array iold has not been filled.
!      26   Array KPROB has not been filled
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      call eropen ( 'copydfbf' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from copydfbf'
  1      format ( a, 1x, (10i6) )
         call prinin ( ivec1, 5, 'ivec1' )
         call prinin ( ivec2, 5, 'ivec2' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      itypv1 = ivec1(2)
      if ( itypv1/=110 .and. itypv1/=115 .and. itypv1/=116 ) then
         call errint ( itypv1, 1 )
         call errsub ( 13, 1, 0, 0 )
      end if
      itypv2 = ivec2(2)
      if ( itypv2/=110 .and. itypv2/=115 .and. itypv2/=116 ) then
         call errint ( itypv2, 1 )
         call errsub ( 13, 1, 0, 0 )
      end if
      if ( kprob1(2)/=101 ) then
         call errint ( kprob1(1), 1 )
         call errsub ( 26, 1, 0, 0 )
      end if
      if ( kprob2(2)/=101 ) then
         call errint ( kprob2(1), 1 )
         call errsub ( 26, 1, 0, 0 )
      end if
      if ( ierror/=0 ) go to 1000

!     --- End tests

      call ini065 ( ibuffr, ivec1, kprob1, inkpf1, inkph1, inkpp1, &
                    nphys1 )
      call ini065 ( ibuffr, ivec2, kprob2, inkpf2, inkph2, inkpp2, &
                    nphys2 )

      ipvec1 = inidgttext ( ivec1(1), 'vector1' )
      ipvec2 = inidgttext ( ivec2(1), 'vector2' )
      ipkpf1=1
      ipkpf2=1
      ipkph1=1
      ipkph2=1
      if ( inkpf1>0 ) ipkpf1 = inigettext(inkpf1, 'kprobf1')
      if ( inkpf2>0 ) ipkpf2 = inigettext(inkpf2, 'kprobf2')
      if ( inkph1>0 ) ipkph1 = inigettext(inkph1, 'kprobh1')
      if ( inkph2>0 ) ipkph2 = inigettext(inkph2, 'kprobh2')
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'inkpf1, inkph1, inkpp1, nphys1', &
                          inkpf1, inkph1, inkpp1, nphys1
         write(irefwr,1) 'inkpf2, inkph2, inkpp2, nphys2', &
                          inkpf2, inkph2, inkpp2, nphys2

      end if  ! ( debug )

      call ercpdf01 ( buffer(ipvec1), ibuffr(ipkpf1), ibuffr(ipkph1), &
                      inkpf1, inkph1, nphys1, idf1, &
                      buffer(ipvec2), ibuffr(ipkpf2), ibuffr(ipkph2), &
                      inkpf2, inkph2, nphys2, idf2, kmesh(8) )

1000  call erclos ( 'copydfbf' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End copydfbf'

      end if  ! ( debug )

      end
