      subroutine er0027 ( ichois, icurvs, ncurvs, n, kmeshm, iwork, &
                          kmesh, kmeshq )
! ====================================================================== &
!
!        programmer    Guus Segal
!        version  4.3  date 02-11-2012 Change error in warning
!        version  4.2  date 16-06-2005 Set n at input
!        version  4.1  date 18-02-2004 Debug statements
!
!   copyright (c) 1985-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill nodal points of curves as indicated by ichois and icurvs
!     in array iwork
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     node
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichois, ncurvs, n, kmeshm(*), iwork(*), icurvs(*), &
              kmesh(*), kmeshq(*)

!     ichois         i    choice parameter.  Possibilities:
!                          0   only one curve is considered
!                         -1   curves icrv1 to icrv2 are considered
!                         -2   curves icrv1 to icrv2 are considered in
!                              reversed order
!                         n>0  n curves icrv1, icrv2, ... , icrvn are considered
!     icurvs         i    array containing the curve numbers (see ichois)
!     iwork          o    array containing the nodal point numbers of the curves
!                         subsequently
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     kmeshm         i    KMESH part m, contains nodal point numbers of
!                         the curves
!     kmeshq         i    KMESH part q, contains the subcurve numbers of
!                         each curve
!     n             i/o   number of nodal points on the curves
!                         At input length of array iwork
!     ncurvs         i    Number of curves in mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istar3, icrv1, jcrv1, k, jdirec, j1, j2, leniwork, &
              ilast, j, ifirst, i, nsubcv, istore(ncurvs)
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     icrv1          First curve number
!     ifirst         First nodal point in curve
!     ilast          Last nodal point in curve
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     istore         work array of length ncurvs to store all subcurves
!     j              Counting variable
!     j1             First point of curve in KMESH part m
!     j2             Last point of curve in KMESH part m
!     jcrv1          First curve number
!     jdirec         Defines in which direction the curves must be considered
!                    idirec = 1, from low to high, idirec = -1: reversed order
!     k              Counting variable
!     leniwork       length of array iwork
!     nsubcv         Number of subcurves of a curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     ERSTORECURVS   Fill array istore with (sub) curve numbers of the curves
!                    stored in array icurvs using the parameter ichois
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      45   first point in part of boundary is not equal to last point
!     139   curve number does not exist
!     557   At least one of the curves that are used has not been filled.
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      call eropen ( 'er0027' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from er0027'
         call prinin ( icurvs, ncurvs, 'icurvs' )

      end if
      if ( ierror/=0 ) go to 1000
      leniwork = n

!     --- First store all (sub) curves in array istore

      call erstorecurvs ( ichois, kmesh, kmeshq, nsubcv, istore, &
                          ncurvs, icurvs )
      if ( debug ) then
         write(irefwr,*) 'ichois, nsubcv, ncurvs ', &
                          ichois, nsubcv, ncurvs
         call prinin ( istore, nsubcv, 'istore' )
      end if

!     --- Fill nodes in iwork

      istar3 = kmeshm(2)-1
      n = 1
      do k = 1, nsubcv
         icrv1 = istore(k)
         jcrv1 = abs(icrv1)
         if ( jcrv1==0 .or. jcrv1>ncurvs ) then
            call errint ( jcrv1, 1 )
            call errint ( ncurvs, 2 )
            call errsub ( 139, 2, 0, 0 )
            go to 1000
         end if
         if ( icrv1>0 ) then
            jdirec = 1
            j1 = kmeshm(4+jcrv1)
            j2 = kmeshm(5+jcrv1)-1
         else
            jdirec = -1
            j1 = kmeshm(5+jcrv1)-1
            j2 = kmeshm(4+jcrv1)
         end if
         if ( n>1 ) ilast = iwork(n)
         if ( debug ) &
            write(irefwr,*) 'k, icrv1, j1, j2, jdirec, ilast ', &
                             k, icrv1, j1, j2, jdirec, ilast

!        --- fill array iwork

         do j = j1, j2, jdirec
            iwork(n+jdirec*(j-j1)) = kmeshm(istar3+j)
         end do
         ifirst = iwork(n)
         n = n+(j2-j1)*jdirec
         call inichkarlen ( n, leniwork, 'iwork' )
         if ( ierror/=0 ) go to 1000
         if ( debug ) write(irefwr,*) 'ifirst, n ', ifirst, n
         if ( k/=1 ) then
            if ( ifirst/=ilast ) then
               call errint ( ifirst, 1 )
               call errint ( ilast, 2 )
               call errint ( k, 3 )
               call errint ( 0, 4 )
               call errsub ( 45, 4, 0, 0 )
            end if

         end if

      end do

!     --- Check if the curves had been filled by subroutine mesh
!         If so all positions in iwork must correspond to a nodal point number

      do i = 1, n
         if ( iwork(i)<=0 ) then
            call errint ( i, 1 )
            call errwar ( 557, 1, 0, 0 )
            if ( nwarn>=10 ) go to 1000
         end if
      end do

1000  call erclos ( 'er0027' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End er0027'

      end if

      end
