      subroutine ercomnor ( kmesh, norm, weight, icurvs, coor, &
                            normaldr, kmeshn, kmeshs )
! ====================================================================== &
!
!        programmer    Guus Segal
!        version  3.5  date 09-01-2008 Set isurnr
!        version  3.4  date 05-05-2004 Debug statements
!        version  3.3  date 22-01-2003 Correction empty surfaces
!        version  3.2  date 26-08-2002 Replace error message by warning
!        version  3.1  date 01-08-2002 Minor adaptation error message
!        version  3.0  date 18-03-2002 Remove elssurfs
!        version  2.0  date 06-10-2000 Use icurvs; new body
!        version  1.1  date 30-10-1998 Correction inext
!        version  1.0  date 05-10-1998
!
!   copyright (c) 1998-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the averaged normals in all points on the surfaces isrf1 to isrf2
!
! **********************************************************************
!
!                       KEYWORDS
!
!     average
!     normal_vector
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision norm(3,*), weight(*), coor(3,*)
      integer kmesh(*), icurvs(*), normaldr, kmeshn(*), kmeshs(*)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect
!                         to the global numbering
!     icurvs         i    Integer array containing the surface numbers
!                         Pos. 1 indicates how array icurvs is filled.
!                         Possible values:
!                          0   only one surface is used. The
!                              surface number is stored in icurvs(2)
!                         -1   The solution is used along the
!                              surfaces icrv1 to icrv2. icrv1 is stored in
!                              icurvs(2) and icrv2 in icurvs(3).
!                         n>0  The surface numbers are stored
!                              in icurvs(2) to icurvs(n+1).
!                              That sequence is used.
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     kmeshn         i    KMESH part n, contains nodal point numbers of the
!                         surfaces
!     kmeshs         i    Kmesh part s, contains information about the various
!                         surfaces. See the PG 13.1
!     norm           o    Array of length ndim * npoint containing the normal in
!                         the nodal points
!     normaldr       i    If 1 the outward directed normal is required, if -1
!                         the inward directed normal
!     weight              Help array of length npoint to store the weights
!                         for the averaging
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer isurnr, i, k, npoint, nodp, ichoice, isrf1, isrf2, nsurfs
      double precision h
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     h              Help variable to compute the length of a vector
!     i              Counting variable
!     ichoice        Choice parameter
!     isrf1          First surface number
!     isrf2          Last surface number
!     isurnr         Surface sequence number
!     k              Counting variable
!     nodp           Nodal point sequence number
!     npoint         Number of nodal points
!     nsurfs         number of surfaces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERCOMNOR1      Body of subroutine ERCOMNOR
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2337   Element incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear arrays norms and weights
!     For isurnr := isrf1 (1) isrf2
!        For all elements ielem in surface isurnr
!           For all nodes nodp in ielem do
!              Compute the normal in the point using the right-handed screw
!              direction and the direction of the elements
!              Add normal to array norms
!              Add 1 to array weights
!     Divide all entries in norms by corresponding weights
!     Normalize the entries in norms
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      call eropen ( 'ercomnor' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ercomnor'
         call prinin ( icurvs, 3, 'icurvs' )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Clear arrays norms and weights

      npoint = kmesh(8)
      nsurfs = kmesh(12)
      do i = 1, npoint
         weight(i) = 0d0
         norm(1,i) = 0d0
         norm(2,i) = 0d0
         norm(3,i) = 0d0
      end do

!     --- For isurnr := isrf1 (1) isrf2

      ichoice = icurvs(1)
      if ( ichoice>=0 ) then

!     --- ichoice>0;  ichoice surfaces

         do k = 1, max(1,ichoice)

            isurnr = icurvs(k+1)
            if ( kmeshn(isurnr)>0 ) &
               call ercomnor1 ( kmesh, norm, weight, coor, normaldr, &
                                kmeshn, kmeshs(nsurfs+2), isurnr )

         end do

      else

!     --- ichoice<0;  surfaces icrv1 to icrv2

         isrf1 = icurvs(2)
         isrf2 = icurvs(3)
         isurnr = isrf1

         do isurnr = isrf1, isrf2

            if ( kmeshn(isurnr)>0 ) &
               call ercomnor1 ( kmesh, norm, weight, coor, normaldr, &
                                kmeshn, kmeshs(nsurfs+2), isurnr )

         end do
         isurnr = isrf1

      end if

!     --- Divide all entries in norms by corresponding weights
!         Normalize the entries in norms

      do nodp = 1, npoint

         if ( weight(nodp)>0d0 ) then

!        --- Point is in surface

            h = 1d0/weight(nodp)
            do k = 1, 3
               norm(k,nodp) = norm(k,nodp)*h
            end do

            h = sqrt(norm(1,nodp)**2+norm(2,nodp)**2+norm(3,nodp)**2)

            if ( h>0d0 ) then
               h = 1d0/h
            else
               call errint ( nodp, 1 )
               call errint ( 0, 2 )
               call errint ( isurnr, 3 )
               call errwar ( 2337, 3, 0, 0 )
               h = 1d0
            end if

            do k = 1, 3
               norm(k,nodp) = norm(k,nodp)*h
            end do

         end if

      end do

1000  call erclos ( 'ercomnor' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End ercomnor'

      end if

      end
