      subroutine iinputseq ( ibuffr, buffer, number, iseqnr, iinput, &
                             rinput, leninp, lenrnp, name )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.3  date 21-11-2012 Long integers
!        version  2.2  date 10-10-2012 Correction empty rinput
!        version  2.1  date 30-06-2011 Allow negative iseqnr
!
!   copyright (c) 2004-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read contents of input block with sequence number iseqnr corresponding
!     to main keyword name (sequence number: number)
!     Store results in arrays iinput and rinput
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     input
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), number, iseqnr, iinput(*), leninp, lenrnp
      double precision buffer(*), rinput(*)
      character *(*) name

!     buffer        i/o  Real buffer array in which all large data is stored
!     ibuffr        i/o  Integer buffer array in which all large data is stored
!     iinput         o    integer input array in which information is stored
!     iseqnr         i    Sequence number with respect to the storage in array
!                         ISEPIN
!                         If negative the routine returns without warning
!                         with leninp = 0 if no input can be found, otherwise
!                         (positive) an error message is given.
!     leninp        i/o   Computed length of integer input array
!                         At input: declared length
!                         At output: actual length
!     lenrnp        i/o   Computed length of real input array
!                         At input: declared length
!                         At output: actual length
!     name           i    Name of specific input block
!     number         i    Sequence number of specific input block in ISEPIN
!     rinput         o    Real input array in which information is stored
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipintm, iprntm
      integer iinptm, irintm, leninpact, lenrnpact, lenname
      character (len=80) namecopy
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     iinptm         Array sequence number of array IINPUT
!     ipintm         Starting address of array IINPUT in array IBUFFR
!     iprntm         Starting address of array RINPUT in array IBUFFR
!     irintm         Array sequence number of array RINPUT
!     leninpact      Computed length of integer input array
!     lenname        Length of name
!     lenrnpact      Computed length of real input array
!     namecopy       String with copy of name
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inigettext, inidgttext
      integer inilen, inidln

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INI015         Extract infor entries of integer and real input arrays from
!                    ISEPIN
!     INI060TEXT     Reactivate existing array in IBUFFR
!     INICHINPSOL    Adapt input array with respect to variables
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     INIDGTTEXT     Compute pointer of array in BUFFER
!     INIDLN         Compute length of array in BUFFER
!     INIGETTEXT     Compute pointer of array in IBUFFR
!     INILEN         Compute length of array in IBUFFR
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2494   No input block of specific name found
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'iinputseq' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from iinputseq'
         write(irefwr,1) 'number, iseqnr', number, iseqnr
         write(irefwr,1) 'leninp, lenrnp', leninp, lenrnp
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      call ini015 ( ibuffr, buffer, number, abs(iseqnr), iinptm, &
                    irintm )

      if ( iinptm==0 ) then

!     --- Array iintim has not been filled, i.e. no block name
!         has been found

         if ( iseqnr<0 ) then
            leninp = 0
            go to 1000   ! jump if negative seqnr
         end if  ! ( iseqnr<0 )
         call errint ( iseqnr, 1 )
         call errchr ( name, 1 )
         call errsub ( 2494, 1, 0, 1 )
         go to 1000

      end if

      call ini060text ( ibuffr, -iinptm, 'iinputcop' )
      if ( ierror/=0 ) go to 1000
      ipintm = inigettext ( iinptm, 'iinputcop' )
      leninpact = inilen(iinptm)
      if ( debug ) write(irefwr,1) 'leninpact', leninpact

      lenname = min(len(name),80)
      namecopy = name(1:lenname)

      call inichkarlen ( leninpact, leninp, 'iin_'//namecopy )
      leninp = leninpact
      iinput(1:leninp) = ibuffr(ipintm:ipintm+leninp-1)

      if ( irintm>0 ) then

!     --- irintm>0, hence array rinput must be filled

         call ini060text ( ibuffr, -irintm, 'rinputcop' )
         iprntm = inidgttext ( irintm, 'rinputcop' )
         lenrnpact = inidln(irintm)
         call inichkarlen ( lenrnpact, lenrnp, 'rin_'//namecopy )
         if ( debug ) write(irefwr,1) 'lenrnpact', lenrnpact
         lenrnp = lenrnpact
         rinput(1:lenrnp) = buffer(iprntm:iprntm+lenrnp-1)

      else

!     --- irintm = 0, hence array rinput must not be filled

         lenrnp = 0

      end if  ! ( irintm>0 )

      if ( ierror/=0 ) go to 1000

!     --- Adapt variables

      call inichinpsol ( iinput, leninp )

1000  call erclos ( 'iinputseq' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'leninp, lenrnp', leninp, lenrnp
         write(irefwr,*) 'End iinputseq'

      end if

      end
