      subroutine ini062 ( ibuffr, length, ipstar )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.10 date 21-11-2012 Long integers
!        version  1.9  date 28-09-2012 Correction largest integer
!        version  1.8  date 04-06-2007 Fortran 90
!
!   copyright (c) 1997-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute starting position of a temporary array of length LENGTH in
!     array ibuffr
!     If space is available IBFREE is not adjusted, which means that INI062
!     may only be called once during a process because otherwise the same
!     start address might be found.
!     If more arrays must be reserved in one time it is necessary to give the
!     sum of the length of all these arrays.
!     Remember: use first all reals and then the integers!
!     if no space is available in array IBUFFR, INI062 tries to
!     create space in the following ways:
!       -  by writing low priority arrays to backing storage
!       -  if this is not sufficient by compressing
!          array ibuffr after writing of all arrays with priority
!          smaller than 10 to backing storage
!     As a consequence starting positions that have been computed before may
!     be changed and should be recomputed
!     INI062 is identical to subroutine INI052 as described in the Programmer's
!     Guide, except that the parameter name is not longer used.
!     The name of the calling subroutine is extracted from common blocks
!     CMESSC and CMESSG, so it is supposed that the subroutine EROPEN has been
!     used
! **********************************************************************
!
!                       KEYWORDS
!
!     memory_management
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmessgf90'
      include 'SPcommon/cmesscf90'
      include 'SPcommon/cbuffrf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cdebugf90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/comcons3f90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer (kind=8) ipstar
      integer length, ibuffr(*)

!     ibuffr i/o  Integer buffer array in which all large data is stored
!     ipstar  o  computed starting position in array IBUFFR
!     length  o  length of the array in words
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) i
      integer iextra

!     i         Help variable to store new length
!     iextra    Extra space needed at both sides of the array for control
!               integers
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRINTLONG     Put long integer in error message
!     ERRSUB         Error messages
!     INI049         Create space in IBUFFR by writing low priority arrays to
!                    file 3, or to compress IBUFFR
!     INICHK1        Check IBUFFR
!     PRINBF         Print and check length of array IBUFFR
! **********************************************************************
!
!                       I/O
!
!     If necessary arrays are written to file 3 (iref3)
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1765   length of array less than one
!    2270   integer incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
!     If ( idebug>0 ) then
!        Set iextra = extra space needed
!        check all control integers
!     If ( enough space in IBUFFR after IBFREE ) then
!         IPSTAR = IBFREE
!     else
!         Create space by writing low priority arrays to file 3
!         or compressing IBUFFR
!     If ( idebug>0 ) then
!        place control integers for this array
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ini062' )
      if ( ierror/=0 ) go to 1000

      if ( idebug==0 ) then
         iextra = 0
      else
         iextra = intlen
         ifirws = 0
         call inichk1(ibuffr)
      end if
      if ( length<=0 ) then

!     --- length<=0, error

         call errint ( length, 1 )
         call errchr ( 'temporary_array', 1 )
         call errsub ( 1765, 1, 0, 1 )

      end if
      if ( ierror/=0 ) go to 1000

      ipstar=1
      if ( ibfree-1>nbuffr .or. ibfree<=0 ) then

!     --- ibfree out of range

         call errintlong ( ibfree, 1 )
         call errchr ( 'ibfree', 1 )
         call errsub ( 2270, 1, 0, 1 )
         go to 1000

      end if
      if ( length>1e11 .or. length<=0 ) then

!     --- length out of range

         call errint ( length, 1 )
         call errchr ( 'length', 1 )
         call errsub ( 2270, 1, 0, 1 )
         go to 1000

      end if
      if ( nbuffr>1e11 .or. nbuffr<=0 ) then

!     --- nbuffr out of range

         call errintlong ( nbuffr, 1 )
         call errchr ( 'nbuffr', 1 )
         call errsub ( 2270, 1, 0, 1 )
         go to 1000

      end if

      i = ibfree+length+iextra*2-1
      if ( i>0 .and. i<=nbuffr ) then

!     --- enough free space available   ( standard situation )

         ipstar = ibfree

      else

!     --- try to create free space by writing low priority arrays
!         to backing storage ( file 3 ) or compressing ibuffr

         call ini049 ( ibuffr, length, ipstar )
         if ( ierror/=0 ) go to 1000

      end if

      if ( idebug>0 ) then

         nbufw1 = ipstar-1
         ibuffr(nbufw1) = numarrs+1
         nbufw2 = ipstar+length
         ibuffr(nbufw2) = numarrs+1

      end if
      call prinbf ( ibfree-1 )

      if ( ioutp>2 ) then
         write(irefwr,*) ' Create work space with starting address ', &
                         ipstar,' and length ', length, &
                         ' in subroutine ', &
                         namsub(levsub-1)//':ini062'
      end if

1000  call erclos ( 'ini062' )

      end
