      subroutine ini065 ( ibuffr, isol, kprob, indprf, indprh, indprp, &
                          nphys )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.7  date 07-07-2006 Debug statements
!        version  1.6  date 09-12-2005 Fortran 90
!        version  1.5  date 03-02-2003 Replace ini060 by ini060text
!        version  1.4  date 06-02-2002 Extension with eropen/erclos
!        version  1.3  date 05-11-1999 Extension with type 801
!        version  1.2  date 19-10-1999 Extension with type 127
!        version  1.1  date 22-11-1998 Include common
!        version  1.0  date 23-07-1997
!
!   copyright (c) 1997-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!  Memory management subroutine which simplifies the computation of the starting
!  addresses of an existing SEPRAN solution array and KPROB parts f, h and p
!     If these arrays exist, it is checked if they are positioned in
!     in array IBUFFR (Blank common) or not.
!     If one or more of these arrays are not available
!     these array are read from file 3 and space is reserved in array IBUFFR.
!     The priority number is made equal to 10.
!     if no space is available in array IBUFFR, INI065 tries to
!     create space in the following ways:
!       -  by writing low priority arrays to backing storage
!       -  if this is not sufficient by compressing
!          array ibuffr after writing of all arrays with priority
!          smaller than 10 to backing storage
!     INI065 replaces the old subroutine INI103 or INI002 (for existing arrays)
!     If array KPROB part f (or its corresponding counterpart) does exist, &
!     INDPRF gets the correct index number referring to array INFOR in
!     common block CARRAY.
!     Afterwards the starting address of KPROB part f is given by
!     INFOR(1,INDFOR)
!     In the same way INDPRH and INDPRP refer to KPROB parts h and p
!     respectively
!     INI065 is identical to subroutine INI055 as described in the Programmer's
!     Guide, except that the parameter name is not longer used.
!     The name of the calling subroutine is extracted from common blocks
!     CMESSC and CMESSG, so it is supposed that the subroutine EROPEN has been
!     used
! **********************************************************************
!
!                       KEYWORDS
!
!     memory_management
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isol(5), kprob(*), indprf, indprh, indprp, nphys, &
              ibuffr(*)

!     ibuffr i/o Integer buffer array in which all large data is stored
!     indprf  o  At output INDPRF contains information concerning the KPROB
!                part f corresponding to ISOL.
!                If no KPROB part f corresponds to ISOL, INDPRF = 0, otherwise
!                (INDPRF>0) the starting address of KPROB part f is given
!                by INFOR(1,INDPRF)
!     indprh  o  At output INDPRH contains information concerning the KPROB
!                part h corresponding to ISOL.
!                If no KPROB part h corresponds to ISOL, INDPRH = 0, otherwise
!                (INDPRH>0) the starting address of KPROB part h is given
!                by INFOR(1,INDPRH)
!     indprp  o  At output INDPRP contains information concerning the KPROB
!                part p corresponding to ISOL.
!                If no KPROB part p corresponds to ISOL, INDPRP = 0, otherwise
!                (INDPRP>0) the starting address of KPROB part p is given
!                by INFOR(1,INDPRP)
!     isol    i  The standard SEPRAN solution array or array of special
!                structure of type 115 from which the starting positions
!                must be computed and the relevant information found.
!                Other types of arrays are not allowed
!     kprob   i  Standard SEPRAN array containing information of the problems
!                to be solved.
!     nphys   o  At output NPHYS contains the maximal number of degrees of
!                freedom per point or the number of physical variables if
!                defined
!                If ISOL is a vector of special structure and the number of
!                degrees of freedom per point is not constant then NPHYS = 0
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, itypvc, iprob, ipkprb, ivec, kpivec
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Loop variable
!     ipkprb         Starting address of part of KPROB with respect to
!                    a complete problem in array KPROB (minus one)
!     iprob          Problem number
!     itypvc         Type number of solution vector (110=solution vector, &
!                    115=vector of special structure)
!     ivec           Indication of the structure of a vector of special
!                    structure
!     kpivec         Starting address of information concerning arrays of
!                    special structure in KPROB
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INI060TEXT     Reactivate existing array in IBUFFR
! **********************************************************************
!
!                       I/O
!
!     none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     744   Fatal error: Only arrays of type 110 and 115 are permitted
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Compute problem number (IPROB), type of solution vector (ITYPVC)
!     and for a vector of special structure the vector number (IVEC)
!     Read solution array in IBUFFR (INI070)
!     If ( ISOL is of type 110 ) then
!        if ( KPROB part f has been filled ) then
!           Set INDPRF and read KPROB part f in IBUFFR
!        if ( KPROB part h has been filled ) then
!           Set INDPRH and read KPROB part h in IBUFFR
!        if ( KPROB part p has been filled ) then
!           Set INDPRP and read KPROB part p in IBUFFR
!     else if ( ISOL is of type 115 ) then
!        if ( KPROB part f has been filled ) then
!           Set INDPRF and read KPROB part f in IBUFFR
!     else if ( ISOL is of type 116 ) then
!        Compute nphys
!     else if ( ISOL is of type 127 ) then
!        Compute nphys
!     else
!        wrong type of vector found: error
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ini065' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ini065'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      indprf = 0
      indprh = 0
      indprp = 0
      itypvc = isol(2)
      if ( itypvc/=110 .and. itypvc/=115 .and. itypvc/=116 .and. &
           itypvc/=127 .and. itypvc/=801 ) then

!     --- Array isol has not been filled

         call errint ( itypvc, 1 )
         call errsub ( 744, 1, 0, 0 )

      end if
      if ( debug ) write(irefwr,1) 'itypvc', itypvc

!     --- Activate isol

      call ini060text ( ibuffr, isol(1), 'isol' )
      if ( ierror/=0 ) go to 1000

!     --- Compute IPROB and offset for array KPROB

      iprob  = 1 + isol(4)/1000
      ipkprb = 1
      do i = 2, iprob
         ipkprb = ipkprb + kprob ( 2+ipkprb )
      end do

      if ( itypvc==110 ) then

!     --- ISOL is of type 110:  solution vector

         nphys = max ( kprob(ipkprb+3), kprob(ipkprb+32) )
         indprf = kprob(ipkprb+18)
         indprh = kprob(ipkprb+20)
         indprp = kprob(ipkprb+34)
         if ( indprf>0 ) call ini060text ( ibuffr, -indprf, &
            'kprobf' )
         if ( indprh>0 ) call ini060text ( ibuffr, -indprh, &
            'kprobh' )
         if ( indprp>0 ) call ini060text ( ibuffr, -indprp, &
            'kprobp' )

      else if ( itypvc==115 ) then

!     --- ISOL is of type 115:  array of special structure

         ivec   = mod(isol(4),1000)
         kpivec = kprob(ipkprb+17)-1+kprob(ipkprb+38)*kprob(ipkprb+5)+ &
                  ivec + ipkprb-1
         nphys  = kprob(kpivec)
         if ( debug ) &
            write(irefwr,1) 'ivec, kpivec, nphys', &
                             ivec, kpivec, nphys
         if ( nphys<0 ) then

!        --- nphys<0, constant number of degrees of freedom per point

            nphys = -nphys

         else

!        --- nphys>0, number of degrees of freedom per point not constant

            indprf = nphys
            kpivec = kprob(ipkprb+17)-1+(ivec-1)*kprob(ipkprb+38) + &
                     ipkprb-1
            nphys  = 0
            do i = 1, kprob(ipkprb+38)
               nphys = max ( nphys, kprob(kpivec+i) )
            end do
            call ini060text ( ibuffr, -indprf, 'kprobe' )

         end if

      else if ( itypvc==116 ) then

!     --- ISOL is of type 116:  array of special structure defined per element

         nphys = mod(isol(4),1000)

      else if ( itypvc==127 ) then

!     --- ISOL is of type 127:  array defined per edge or face

         nphys = mod(isol(4),1000)

      else if ( itypvc==801 ) then

!     --- ISOL is of type 801:  vector of type capacity vector

         nphys = mod(isol(4),1000)

      end if

1000  call erclos ( 'ini065' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End ini065'

      end if  ! ( debug )

      end
