      subroutine ini101 ( ibuffr, length, ipstar, iwork, ilast )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.7  date 21-11-2012 Long integers
!        version  2.6  date 28-09-2012 Correction largest integer
!        version  2.5  date 15-04-2005 Extra information in error message
!
!   copyright (c) 1986-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   create space by either looking for contiguous space in array ibuffr
!   or by writing arrays to file 3
!   at least length succeeding positions are needed
! **********************************************************************
!
!                       KEYWORDS
!
!     memory_management
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/comcons3f90'
      include 'SPcommon/carrayf90'
      include 'SPcommon/cbuffrf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cdebugf90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer (kind=8) ipstar, ilast
      integer length, iwork(*), ibuffr(*)

!    ibuffr  i/o  Integer array in which all large data is stored
!    ilast    i   last position in ibuffr that may be used
!    ipstar   o   starting position if there is space free
!                 if there is no space available ipstar gets the value 0
!    iwork        work array of length 1500, in which the entries in infor
!                 are sorted for increasing starting address
!                 iwork(1) contains the index of the array with the smallest
!                          starting address, &
!                 iwork(2) with the next starting address etc.
!    length   i   free space needed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipfirs, iplast, ipnext, ispace, i
      integer iextra, numarr, k, inext, &
              loclen, kprio, mwrite, iprio, j, iprint
      logical debug
      save iprint

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              General loop variable
!     iextra         Extra space needed at both sides of the array for control
!                    integers
!     inext          Sequence number of next array in IBUFFR with respect
!                    to INFOR
!     ipfirs         Starting address of present array in IBUFFR
!     iplast         Ending address of array in IBUFFR
!     ipnext         Starting address of array inext in IBUFFR
!     iprint         Indicates if the warning with respect to backong storage
!                    has been printed (1) or not
!     iprio          Priority number
!     ispace         Defines the available contiguous space in IBUFFR
!     j              General loop variable
!     k              General loop variable
!     kprio          Loop variable for priorities
!     loclen         Length of present array to be considered
!     mwrite         First sequence number of arrays to be written to file 3
!     numarr         Number of arrays that are stored in IBUFFR
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRINTLONG     Put long integer in error message
!     ERRSUB         Error messages
!     INI100         write one array from IBUFFR to backing storage
!     INI102         Sort the sequence of the starting addresses in IBUFFR
!     PRINBUFARS     Print the active arrays present in array buffer
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2375   Swapping must be carried out, but noswap activated
! **********************************************************************
!
!                       PSEUDO CODE
!
!    ipstar = 0
!    If ( idebug>0 ) then
!       Set iextra = extra space needed
!    Sort array infor with respect to increasing starting addresses
!    Put sorted sequence in iwork
!    Check the contiguous space between these entries
!    If ( contiguous space found ) then
!       set ipstar
!    else if ( space at the end of IBUFFR is sufficient ) then
!       set ipstar at end
!    else if ( total free space enough ) then
!       compress IBUFFR as much as necessary
!       set ipstar
!    else
!       ready := false
!       for iprio := 1 (1) 9, while (not ready) do
!          write arrays with priority<iprio to backing storage until
!          contiguous space is found
!          if ( contiguous space is found ) then
!             set ipstar
!             ready = true
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data iprint /0/
! ======================================================================
!
      call eropen ( 'ini101' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ini101'

      end if

!     --- check if there are free succeeding positions in
!         ibuffr(1),...,ibuffr(ibfree)
!         sort the sequence in infor and store in array iwork

      ipstar = 0
      if ( idebug==0 ) then
         iextra = 0
      else
         iextra = intlen
      end if
      call ini102 ( iwork, numarr )
      if ( debug ) call prinin ( iwork, numarr, 'iwork')
      if ( ierror/=0 ) go to 1000

!     --- Check contiguous free space

      ipfirs = numarrs+1 + iextra
      ispace = 0
      do k = 1, numarr
         inext  = iwork(k)
         iplast = min(nbuffr+1,infor(inext)) - iextra - 1
         ispace = ispace + max(0,iplast-ipfirs)
         if ( iplast-ipfirs>length+iextra ) then

!        --- sufficient contiguous space found

            ipstar = ipfirs
            go to 1000

         else

!        --- not enough space, raise ipfirs

            ipfirs = infor(inext)+infor1(1,inext)+iextra*2
            if ( intlen==2 .and. mod(ipfirs,2_8)==0 ) ipfirs=ipfirs+1

         end if

      end do

      ispace = ispace + ilast - iextra - ipfirs + 1
      if ( debug ) write(irefwr,*) ispace

!     --- no free contiguous space found
!         Check if there is space at the end of IBUFFR

      i = ibfree+length+iextra*2-1
      if ( i>0 .and. i<=ilast ) then

!     --- enough free space available

         ipstar = ibfree
         ibfree = ibfree+length+iextra*2
         if ( intlen==2 .and. mod (ibfree,2_8)==0 ) ibfree = ibfree+1

      else if ( ispace>length ) then

!     --- enough space available, compress IBUFFR from the start, &
!         until enough contiguous free space is found

         ipfirs = numarrs+1 + iextra
         do k = 1, numarr
            inext  = iwork(k)
            ipnext = infor(inext) - iextra
            if ( ipnext-ipfirs-1>length+iextra ) then
               ipstar = ipfirs
               if ( idebug==2 ) then
                  ibuffr(ipstar:ipnext-1) = iinfin
               end if
               go to 1000
            end if
            loclen = infor1(1,inext) + 2*iextra
            if ( ipnext>ipfirs ) then

!           --- move array inext to starting address ipfirs

               if ( iextra==0 ) then

!              --- iextra = 0, standard case

                  ibuffr(ipfirs:ipfirs+loclen-1) = &
                     ibuffr(ipnext:ipnext+loclen-1)

               else

!              --- iextra = intlen, debug case

                  ipnext = infor(inext)-1
                  ibuffr(ipfirs:ipfirs+infor1(1,inext)+1) = &
                     ibuffr(ipnext:ipnext+infor1(1,inext)+1)

               end if
               infor(inext) = ipfirs

            end if
            ipfirs = ipfirs + loclen
            if ( intlen==2 .and. mod(ipfirs,2_8)==0 ) ipfirs=ipfirs+1

         end do

!        --- end has been reached

         ipstar = ipfirs
         ibfree = ipstar + length + iextra*2
         if ( intlen==2 .and. mod (ibfree,2_8)==0 ) ibfree = ibfree+1

      else

!     --- no free contiguous space found; try to find succeeding space by
!         writing low priority arrays to file 3
!         Decrease numarr in order to circumvene interference with ini079

         k = numarr
         do j = k, 1, -1
           if ( infor(iwork(j)) + infor1(1,iwork(j)) + iextra >= &
                nbuffr ) then
              numarr = j - 1
           else
              goto 210
           end if
         end do

!        --- Loop over the priorities

210      if ( noswap==1 ) then

!        --- Swapping is not permitted

            call errintlong ( ibfree, 1 )
            call errint ( length, 2 )
            call errintlong ( nbuffr, 3 )
            call errintlong ( ibfree+length-nbuffr, 4 )
            call errsub ( 2375, 4, 0, 0 )
            call prinbufars
            write(irefwr,215) length
215         format ( 'length required = ', i10 )
            go to 1000

         end if

         if ( iprint==0 .and. ioutp>=0 ) then
            write(irefwr,220)
220         format(///' ********** Warning ************'// &
                      ' There is not enough space in IBUFFR'/ &
                      ' The SEPRAN swapping mechanism is activated'/ &
                      ' This may lead to a loss of performance'/ &
                      ' It is advised to enlarge array IBUFFR if', &
                      ' possible'/// )
            iprint = 1
         end if

         do kprio = 1, 9
            ipfirs = numarrs+1+iextra
            mwrite = 0

!           --- Loop over all arrays in IBUFFR

            do k = 1, numarr
               inext  = iwork(k)
               iplast = min(nbuffr+1,infor(inext)) - iextra - 1
               iprio  = infor1(2,inext)
               if ( iprio<=kprio ) then

!              --- only arrays with priority<=kprio are considered

                  if ( mwrite==0 ) mwrite = k
                  if ( k==numarr ) then
                     iplast = ilast-iextra-1
                  else
                     iplast = min(nbuffr+1,infor(iwork(k+1)))-1-iextra
                  end if
                  if ( iplast-ipfirs>=length+iextra) then

!                 --- Enough contiguous space found
!                     write arrays mwrite to k   to file 3

                     do j = mwrite, k
                        call ini100 ( ibuffr, iwork(j) )
                     end do

!                    --- Recompute ibfree

                     ibfree = numarrs+1 + iextra
                     do j = 1, numarr
                        ibfree = max(ibfree, &
                                     infor(iwork(j))+ &
                                     infor1(1,iwork(j))+2*iextra)
                     end do

                     ipstar = ipfirs
                     ibfree = max ( ibfree, ipstar+length+iextra*2 )
                     if ( intlen==2 .and. mod(ibfree,2_8)==0 ) &
                          ibfree=ibfree+1
                     go to 1000

                  end if

               else

!              --- priority of present array not enough
!                  The space starts after the present one

                  ipfirs = iplast+infor1(1,inext)+1+3*iextra
                  if ( intlen==2 .and. mod(ipfirs,2_8)==0) &
                       ipfirs = ipfirs+1
                  mwrite = 0

               end if

            end do

         end do

      end if
1000  call erclos ( 'ini101' )
      if ( debug ) write(irefwr,*) 'End ini101'

      end
