      subroutine iniclearbnsol ( ibuffr, buffer, kmesh, kprob, isol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 21-11-2012 Long integers
!        version  1.2  date 27-09-2010 Make work arrays local
!        version  1.1  date 30-03-2007 Extension wih 3d
!
!   copyright (c) 2007-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Set all values of the solution array equal to 0 at the boundary
!
! **********************************************************************
!
!                       KEYWORDS
!
!     clear
!     solution_vector
!     outer_boundary
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), kmesh(*), kprob(*), isol(*)
      double precision buffer(*)

!     buffer        i/o   Buffer array
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     isol           i    Standard SEPRAN array containing information of the
!                         solution array
!                         The value of this vector at the boundary is set to 0
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     kprob          i    Standard SEPRAN array, containing information of
!                         the problem
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipioutercv, ipnodes, ipusol, ipkprf, ipkmsm, &
                       ipkmsq, ipkmsn
      integer npoint, indprf, indprp, nunkp, nphys, mmnodes, ndim, &
              nnodes
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     indprf         indication if kprob part f has been filled (>0)
!                    or not (0)
!     indprp         Indication if array kprob part p has been filled (1)
!                    or not (0)
!     ipioutercv     Starting address of array ioutercv
!     ipkmsm         Starting address of array KMESH part m
!     ipkmsn         Starting address of array KMESH part n
!     ipkmsq         Starting address of array KMESH part q
!     ipkprf         Starting address of array KPROB part f
!     ipnodes        Starting address of array INODES
!     ipusol         Starting address of solution array
!     mmnodes        Memory management sequence number of array nodes
!     ndim           Dimension of the space.
!     nnodes         Number of nodes
!     nphys          Number of physical unknowns, or maximal degrees of
!                    freedom per point
!     npoint         Number of nodal points
!     nunkp          Maximum number of unknowns in a point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inigettext, inidgttext

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     EROUTCURVNOD   Find all nodes in the outer curves and store in array nodes
!     EROUTSURFNOD   Find all nodes in the outer surfaces and store in array
!                    nodes
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INI060TEXT     Reactivate existing array in IBUFFR
!     INI061         compute pointers to degrees of freedom of vectors
!     INI066TEXT     delete temporary array
!     INI081         Create permanent space in array IBUFFR
!     INIACTMK       General memory management subroutine to activate or create
!                    a permanent array
!     INICLEARBOUN   Make solution array equal to 0 for all nodes
!     INIDGTTEXT     Compute pointer of array in BUFFER
!     INIGETTEXT     Compute pointer of array in IBUFFR
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     419   NDIM incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Store all nodes of the boundary in array nodes
!     Make solution equal to 0 for all these nodes
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'iniclearbnsol' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from iniclearbnsol'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get constants

      ndim = kmesh(6)
      npoint = kmesh(8)

!     --- create or activate kmesh part ah with outer boundary

      call iniactmk ( ibuffr, 1, 58, kmesh, kmesh, ibuffr )

!     --- Activate arrays

      if ( ndim==2 ) then
         call ini060text ( ibuffr, kmesh(27), 'kmeshm' )
         if ( kmesh(31)>0 ) &
            call ini060text ( ibuffr, kmesh(31), 'kmeshq' )
      else
         call ini060text ( ibuffr, kmesh(28), 'kmeshn' )
      end if  ! ( ndim==2 )

      call ini061 ( ibuffr, isol, kprob, indprf, indprp, &
                    nunkp, nphys )

!     --- Reserve space for temporary arrays

      mmnodes = 0   ! semi-permanent array nodes
      call ini081 ( ibuffr, mmnodes, npoint, 'nodes' )

!     --- Compute starting addresses

      ipkprf = 1
      ipkmsq = 1

      if ( ndim==2 ) then
         ipkmsm = inigettext ( kmesh(27), 'kmeshm' )
         if ( kmesh(31)>0 ) ipkmsq = inigettext ( kmesh(31), 'kmeshq' )
      else
         ipkmsn = inigettext ( kmesh(28), 'kmeshn' )
      end if  ! ( ndim==2 )
      ipioutercv = inigettext ( kmesh(58), 'outercurves' )
      ipnodes = inigettext ( mmnodes, 'nodes' )
      if ( indprf>0 ) ipkprf = inigettext ( indprf, 'kprobf' )
      ipusol = inidgttext ( isol(1), 'usol' )

!     --- Store all nodes of the boundary in array nodes

      if ( ndim==2 ) then

!     --- ndim = 2

         call eroutcurvnod ( kmesh, ibuffr(ipkmsm), ibuffr(ipkmsq), &
                             ibuffr(ipioutercv), nnodes, &
                             ibuffr(ipnodes) )

      else if ( ndim==3 ) then

!     --- ndim = 3

         call eroutsurfnod ( kmesh, ibuffr(ipkmsn), ibuffr(ipioutercv), &
                             nnodes, ibuffr(ipnodes) )

      else

!     --- Other dimensions not yet implemented

         call errint ( ndim, 1 )
         call errint ( 2, 2 )
         call errint ( 2, 3 )
         call errsub ( 419, 3, 0, 0 )
         go to 1000

      end if  ! ( ndim==2 )

!     --- Make solution equal to 0 for all these nodes

      call iniclearboun ( buffer(ipusol), ibuffr(ipkprf), indprf, nunkp, &
                          ibuffr(ipnodes), nnodes )

!     --- Remove semi-permanent arrays

      call ini066text ( ibuffr, mmnodes, 'nodes' )

1000  call erclos ( 'iniclearbnsol' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End iniclearbnsol'

      end if  ! ( debug )

      end
