      subroutine inistrelgrp ( kmesh, kprob, number, ivec, jvec )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 03-12-2013 Layout
!        version  1.0  date 09-12-2005
!
!   copyright (c) 2005-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if vector of special structure has a constant number of
!     number unknowns per point for all non-empty elements
! **********************************************************************
!
!                       KEYWORDS
!
!     find
!     structure
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmesh(*), kprob(*), number, ivec, jvec

!     ivec           o    Sequence number of vector of special structure
!                         if 0, no vector found
!     jvec           i    present sequence number of vector of special stucture
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     kprob          i    Standard SEPRAN array, containing information of
!                         the problem
!     number         i    The required number of unknowns per point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inpelm, nelgrp, kelma, ikelm, kprobc, kprobe, kelgrp, &
              ielgrp, itype, jnunkp, i
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     ielgrp         element group number
!     ikelm          Last position used for element group
!     inpelm         number of points per element
!     itype          type number of element
!     jnunkp         Number of degrees of freedom in point
!     kelgrp         sum over all standard elements of number of points in
!                    standard elements
!     kelma          Indicates if KMESH part a has been filled (>0)
!                    or not (0)
!                    If kelma>0, kelma is the starting address of KMESH
!                    part a in KMESH
!     kprobc         Starting address of KPROB part c in KPROB
!     kprobe         starting address in kprob part e
!     nelgrp         Number of element groups
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For all element groups do
!        jnunkp := number of degrees of freedom per point in vector
!                  of special structure jvec
!        if ( jnunkp#number ) then
!           return
!     ivec := jvec
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'inistrelgrp' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from inistrelgrp'
         write(irefwr,1) 'number, jvec', number, jvec
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Extract constants

      inpelm = kmesh(4)   ! number of points per element
      nelgrp = kmesh(5)   ! number of element groups
      kelma = kmesh(15)   ! start of kmesh part a
      kprobc = kprob(16)  ! start of kprob part c
      kprobe = kprob(18)  ! start of kprob part e
      kelgrp = kprob(39)  ! sum of number of points in all element groups
      ivec = 0            ! return value of ivec
      if ( debug ) &
         write(irefwr,1) 'inpelm, nelgrp, kelma, kprobc, kprobe', &
                          inpelm, nelgrp, kelma, kprobc, kprobe

!     --- For all element groups do

      do ielgrp = 1, nelgrp

!     --- inunkp := number of degrees of freedom per point in solution vector

         itype = kprob(kprobc+ielgrp-1)  ! type number of element
         if ( itype<0 ) go to 200       ! skip elements with type number<1
         if ( kelma==0) then

!        --- kelma = 0, constant number of nodes per element

            ikelm = (ielgrp-1)*inpelm

         else

!        --- kelma>0, variable number of nodes per element

            ikelm = kmesh(kelma+ielgrp-1)
            inpelm = kmesh(kelma+ielgrp)-ikelm

         end if
         if ( debug ) &
          write(irefwr,1) 'ielgrp, itype, ikelm, inpelm', &
                           ielgrp, itype, ikelm, inpelm

!        --- jnunkp := number of degrees of freedom per point in vector
!                     of special structure jvec

         do i = 1, inpelm

            jnunkp = kprob(kprobe-1+ikelm+kelgrp*(jvec-1)+i)
            if ( debug ) write(irefwr,1) 'i, jnunkp',i, jnunkp
            if ( jnunkp/=number ) then

!           --- jnunkp#number
!               return

               go to 1000

            end if  ! ( jnunkp/= number )

         end do  ! i = 1, inpelm

200   end do  ! ielgrp = 1, nelgrp

!     --- ivec := jvec

      ivec = jvec    ! apparantly vector found

1000  call erclos ( 'inistrelgrp' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End inistrelgrp'

      end if

      end
