      subroutine intx05 ( dnumbr, maxnum, inum, readnx, positive, &
                          maxval, code, istartinfo )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.6  date 10-01-2015 New call to readreal
!        version  2.5  date 06-05-2014 Extra debug
!        version  2.4  date 04-11-2009 Adaptation for vectors
!
!   copyright (c) 1997-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   subroutine INTX05 is used to read a series of integers
!   The series is closed by a ) sign, end of record or a text
!   The reading of numbers is stopped if
!    a ) sign is found
!    an end of record is found
!    maxnum numbers have been read
!   After reading the integers, icolum points to the next position to be read
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cseprdf90'
      include 'SPcommon/cseprdnewf90'
      include 'SPcommon/creadinfof90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer maxnum, inum, maxval, istartinfo
      double precision dnumbr(maxnum)
      logical readnx, positive
      character (len=*) code

!     code           i    String containing the corresponding keyword
!     dnumbr         o    Array to be filled with the numbers
!     inum           o    Number of numbers that have been actually read
!     istartinfo     i    Starting address to put information in array
!                         infotabout
!     maxnum         i    Maximum number of numbers to be read
!     maxval         i    Maximum value allowed for the integer read
!     positive       i    If true all numbers must be positive
!     readnx        i/o   Indicates if the next item has not been read (true)
!                         or has been (false)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical finished, debug, openfound
      integer kcolsv, ival, lastitemsav

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     finished       Indicates if a while loop must be terminated (true)
!                    or not (false)
!     ival           Integer value read
!     kcolsv         Help parameter to save the old value of kcolum
!     lastitemsav    Help parameter to save the value of lastitem
!     openfound      Indicates if an openings bracket is found before
!                    reading the number or expression
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     READNAMEVEC    Read name of vector in common cpost5
!     READREAL       Read a real or real expression from the input file
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1041   Real number read, whereas integer number is expected
!    1870   Integer number read out of range
!    1899   Too few or too much numbers read
! **********************************************************************
!
!                       PSEUDO CODE
!
!   finished  := false
!   inum   := 0
!   ispecl := 4
!   while ( not finished ) do
!      Read next item (ASKNXT)
!      if ( ityprd = 2 ) then
!         inum := inum+1
!      else if ( ityprd = 3 ) then
!         error message
!      else if ( ityprd = 0, 1, 4, 5, 6 or 7 ) then
!         finished := true
!      if ( inum = maxnum ) then
!         finished := true
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intx05' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intx05'
         write(irefwr,1) 'lastitem', lastitem
         call prinin ( typeitems, nitems, 'typeitems' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Make some initializations

      finished  = .false.
      inum   = 0
      ispecl = 4
      kcolsv = kcolum

!     --- Check openings bracket

      if ( typeitems(lastitem+1)==5 ) then

!     --- Typeitems = 5, openings bracket
!         if followed by string, skip bracket

         if ( typeitems(lastitem+2)==3 ) lastitem = lastitem+1

      end if  ! ( typeitems(lastitem+1)==5 )
      if ( debug ) write(irefwr,1) 'lastitem/1', lastitem

200   if ( .not. finished ) then

!     --- While loop
!         Read next item

         if ( debug ) then
            write(irefwr,*) 'before readreal'
            write(irefwr,1) 'lastitem', lastitem
         end if  ! ( debug )

         openfound = .false.
         if ( readnx ) call readreal ( openfound, .false. )
         readnx = .true.
         if ( debug ) &
            write(irefwr,1) 'ityprd, lastitem', ityprd, lastitem

         if ( ityprd==2 ) then

!        --- ityprd = 2, integer number found

            ival = nint(realrd)
            if ( debug ) write(irefwr,1) 'ival', ival
            if ( positive ) then

!           --- only positive integers

               if ( ival<1 .or. ival>maxval ) then

!              --- Error integer out of range

                  call errint ( ival, 1 )
                  call errint ( 1, 2 )
                  call errint ( maxval, 3 )
                  call errchr ( code, 1 )
                  call errsub ( 1870, 3, 0, 1 )
                  call errcrd

               end if

            else

!           --- positive and negative integers

               if ( abs(ival)>maxval ) then

!              --- Error integer out of range

                  call errint ( ival, 1 )
                  call errint ( -maxval, 2 )
                  call errint ( maxval, 3 )
                  call errchr ( code, 1 )
                  call errsub ( 1870, 3, 0, 1 )
                  call errcrd

               end if

            end if
            inum = inum + 1
            dnumbr(inum) = realrd

         else if ( ityprd==3 ) then

!        --- ityprd = 3, i.e. real found

            call erreal ( realrd, 1 )
            call errsub ( 1041, 0, 1, 0 )
            call errcrd
            readnx = .false.
            finished = .true.

         else if ( ityprd==4 ) then

!        --- ityprd = 4, check if it is a vector name

            call readnamevec
            if ( ityprd==2 ) then

!           --- Vector name found

               inum = inum + 1
               dnumbr(inum) = realrd

            else

!           --- No vector name found

               if ( istop/=2 ) readnx = .false.
               finished = .true.

            end if  ! ( ityprd==2 )

         else if ( ityprd==11 ) then

!        --- ityprd = 11, i.e. scalar found

            inum = inum + 1
            dnumbr(inum) = realrd
            infotabout(inum+istartinfo-1) = ityprd

         else if ( ityprd<=1 .or. ityprd>=4 .and. ityprd<=7 ) then

!        --- ityprd = 0, 1 or 4-7, i.e. end of file, end of record or
!            ) sign found

            if ( istop/=2 ) readnx = .false.
            finished = .true.

         else if ( ityprd==20 ) then

!        --- ityprd = 20, no items left

            if ( debug ) write(irefwr,1) 'lastitem/1', lastitem
            if ( lastitem<=nitems ) then

!           --- lastitem<=nitems, read next quantity
!               skip , or = sign

               lastitemsav = lastitem
               if ( typeitems(lastitem)==13 .or. &
                    typeitems(lastitem)==26 ) lastitem = lastitem+1
               if ( lastitem>nitems .or. typeitems(lastitem)/=3 ) then

!              --- lastitem >nitems, finished

                  finished = .true.
                  lastitem = lastitemsav

               else

!              --- string found, check if it is a vector name

                  call readnamevec
                  if ( ityprd==2 ) then

!                 --- Vector name found

                     inum = inum + 1
                     dnumbr(inum) = realrd

                  else

!                 --- No vector name found

                     lastitem = lastitemsav
                     finished = .true.

                  end if  ! ( ityprd==2 )

              end if  ! ( lastitem>nitems )

            else

!           --- lastitem >nitems, finished

               finished = .true.

            end if  ! ( lastitem<=nitems )

         end if

         if ( inum>maxnum ) then

!        --- inum = maxnum, stop reading

            call errint ( inum, 1 )
            call errint ( 1, 2 )
            call errint ( maxnum, 3 )
            call errsub ( 1899, 3, 0, 0 )
            readnx = .false.
            finished = .true.

         end if

         go to 200

      end if
      ispecl = 0
      kcolum = min ( kcolum, kcolsv )

1000  call erclos ( 'intx05' )
      if ( debug ) then

!     --- Debug information

         call prinrl ( dnumbr, inum, 'dnumbr' )
         write(irefwr,*) 'readnx', readnx
         write(irefwr,*) 'End intx05'

      end if  ! ( debug )

      end
