      subroutine manbndbf ( ibuffr, buffer, iinman, rinman, kmesh, &
                            kprob, ivectr, icurvs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 21-11-2012 Long integers
!        version  1.1  date 23-12-2011 New call to er0081
!        version  1.0  date 02-04-1999
!
!   copyright (c) 1999-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute minimum or maximum of a SEPRAN vector along one or more
!     boundaries
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     manipulation
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinman(*), ivectr(*), kmesh(*), kprob(*), icurvs(*), &
              ibuffr(*)
      double precision rinman(*), buffer(*)

!     buffer    i/o   Buffer array
!     ibuffr    i/o   Integer buffer array in which all large data is stored
!     icurvs     i    Integer array containing the curve or surface numbers
!                     that must be printed.
!                     Pos. 1 indicates how array icurvs is filled.
!                     Possible values:
!                      0   only one curve or surface is used. The curve or
!                          surface number is stored in icurvs(2)
!                     -1   The solution is considered along the curves or
!                          surfaces icrv1 to icrv2. icrv1 is stored in icurvs(2)
!                          and icrv2 in icurvs(3).
!                     -2   See -1, however, the sequence of the curves is
!                          reversed. In case of curves this means that the
!                          curves are printed from the end point of icrv2 to
!                          the initial point of icrv2, then the end point of
!                          icrv2-1 to the begin point of icrv2-1 and so on.
!                          In case of surfaces ichois=-2 is treated as ichois=-1
!                     n>0  The curve numbers or surface numbers are stored in
!                          icurvs(2) to icurvs(n+1). That sequence is used.
!                     Remark: a negative curve number means that the curve is
!                          considered from end point to initial point, hence in
!                          reversed order. Negative surface numbers are not yet
!                          available.
!     iinman    i     Integer array in which the user indicates which actions
!                     must be performed by subroutine manbndbf
!                     The entries of IINMAN must be filled (by the user) as
!                     follows
!                     If the user sets a value equal to 0, the default value
!                     is used
!                     1:  Number of entries filled by the user
!                     2:  ICHOIS, indicates the type of action
!                         Possibilities:
!                         1:   The minimum and maximum are computed
!                              The minimum is stored in rinman(1), the
!                              maximum in rinman(2)
!                         2:   The minimum and maximum of a function of the
!                              degree of freedom and the co-ordinates are
!                              computed
!                              The minimum is stored in rinman(1), the
!                              maximum in rinman(2)
!                              The user written subroutine funbnd is called
!                     3:  IDIM, indicates if curves (idim=2) or surfaces
!                         (idim=3) must be considered.
!                         The curve or surface numbers must be stored in array
!                         ICURVS
!                         Default value: 2
!                     4:  IDEGFD   Depending on the value of IDEGFD, the
!                                  algebraic manipulations will be performed
!                                  with respect to all degrees of freedom of
!                                  the vectors (IDEGFD=0), with all degrees
!                                  of freedom, or with the IDEGFD th degree of
!                                  freedom only (IDEGFD>0).
!                         Default value: IDEGFD = 0
!     ivectr    i     Standard SEPRAN array referring to the vector to be
!                     manipulated
!     kmesh     i     Standard SEPRAN array containing information of the mesh
!     kprob     i     Standard SEPRAN array, containing information of the
!                     problem
!     rinman    o     Array containing the computed output as described in
!                     iinman
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nparms
      parameter (nparms=4)
      integer (kind=8) ipusol, ipcoor, ipkprf
      integer inpcop(nparms), i, ndim, ichois, idim, iquant, number, &
              isolnw, indknw, nparts, normal, indtan, ncurvs, &
              nnodes, iregion, iseq, error, npoint
      integer, allocatable :: nodes(:), ipoint(:)
      logical compl

!     compl   indication whether the arithmetic is real (false)
!             or complex (true)
!     error   Return value of allocate or deallocate
!     i       Counting variable
!     ichois  Choice parameter
!     idim    dimension of the space
!     indknw  Infor entry with respect to the array containing the
!             starting addresses of the nodal points in the new
!             solution array
!             If negative the number of unknowns in the array is constant
!             and nunkp = -indknw
!     indtan  Infor entry with respect to the array containing the
!             tangential vectors along the boundaries
!             This array is only used in case of curves
!     inpcop  Copy of integer input array extended with defaults
!     ipcoor  Starting address of array COOR into array IBUFFR
!     ipkprf  Starting address of array KPROBF into array IBUFFR
!     ipoint  dummy
!     ipusol  Starting address of solution array in IBUFFR
!     iquant  indicates which quantity must be considered
!             Possible values:
!             0:  The contents of the vector are considered as stored in
!                 the vector corresponding to IVECTR i.e. without
!                 transformation.
!             1:  The normal component of a vector along the curve
!                 or surface is considered .
!             2:  The tangential component(s) of a vector along the
!                 curve or surface is considered .
!     iregion If 0 the complete region is taken into account
!     iseq    Defines sequence of nodal points with respect to increasing
!             co-ordinates
!     isolnw  Infor entry with respect to new, reduced solution array
!     ncurvs  Number of curves
!     ndim    Dimension of the space.
!     nnodes  Number of nodes in isolnw
!     nodes   Contains the node numbers that must be filled
!     normal  Infor entry with respect to the array containing the
!             normal vectors along the boundaries
!             This array is only used in case of curves
!     nparms  Number of entries of input array
!     nparts  number of parts the boundary consists of
!     npoint  Number of nodal points
!     number  The number-th unknown in each nodal point of the array
!             is considered. If the array is complex number=2*i-1
!             corresponds to the real part of array i and number=2*i
!             to the imaginary part of array i
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inigettext, inidgttext

!     ER0081         Store solution along curves or surface into help array
!     ER0085         Compute minimum and maximum of vector in nodal points given
!                    by array nodes
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INI060TEXT     Reactivate existing array in IBUFFR
!     INI066TEXT     delete temporary array
!     INIDGTTEXT     Compute pointer of array in BUFFER
!     INIGETTEXT     Compute pointer of array in IBUFFR
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!       1   ICHOIS is incorrect
!      26   Array KPROB has not been filled
!      75   Array KMESH has not been filled
!     182   parameter idim has incorrect value
!     386   complex quantities are not permitted in this subroutine
!    1516   IDIM>NDIM
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Compute nodal points along the curves (surfaces) and store in array inodes
!   Store corresponding degrees of freedom in in array isolnw
!   Compute minima and maxima
!   Remove superfluous arrays
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data inpcop / 4, 1, 2, 0 /
! ======================================================================
!
      call eropen ( 'manbndbf' )

!     --- Standard checks

      if ( kmesh(2)/=100 ) then
         call errint ( kmesh(2), 1 )
         call errsub ( 75, 1, 0, 0 )
      end if
      if ( kprob(2)/=101 ) then
         call errint ( kprob(2), 1 )
         call errsub ( 26, 1, 0, 0 )
      end if
      compl = mod(ivectr(3),10)==1
      if ( compl ) call errsub ( 386, 0, 0, 0 )
      if ( ierror/=0 ) go to 1000
      ndim   = kmesh(6)

!     --- Fill array inpcop

      do i = 1, min(nparms, inpcop(1))
         inpcop(i) = iinman(i)
      end do

!     --- Check parameters

      ichois = inpcop(2)
      idim   = inpcop(3)
      iquant = 0
      number = inpcop(4)
      if ( ichois<1 .or. ichois>2 ) then
         call errint ( ichois, 1 )
         call errsub ( 1, 1, 0, 0 )
      end if
      if ( idim<2 .or. idim>3 ) then
         call errint ( idim, 1 )
         call errsub ( 182, 1, 0, 0 )
      end if
      if ( idim>ndim ) then
         call errint ( idim, 1 )
         call errint ( ndim, 2 )
         call errsub ( 1516, 2, 0, 0 )
      end if
      if ( ierror/=0 ) then
         ierror = 0
         go to 1000
      end if

      ncurvs = kmesh(11)
      npoint = kmesh(8)
      allocate ( nodes(npoint), ipoint(ncurvs+1), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'nodes' )
      if ( ierror/=0 ) go to 1000

!     --- Compute nodal point numbers in which the vector must be printed
!         Store quantities to be printed in a work array

      iregion = 0
      iseq = 1
      call er0081 ( ibuffr, buffer, kmesh, kprob, ivectr, icurvs, idim, &
                    iquant, number, isolnw, indknw, ipoint, &
                    nparts, normal, indtan, iregion, iseq, 2, nodes, &
                    nnodes )
      if ( ierror/=0 ) go to 1000

!     --- Compute minima and maxima

      call ini060text ( ibuffr, kmesh(23), 'coor' )
      ipusol = inigettext ( isolnw, 'isolnew')
      ipcoor = inidgttext ( kmesh(23), 'coor' )
      ipkprf = 1
      if ( indknw>0 ) ipkprf = inigettext ( indknw, 'indknw' )
      if ( ierror/=0 ) go to 1000
      call er0085 ( ichois, ibuffr(ipusol), indknw, ibuffr(ipkprf), &
                    nodes, buffer(ipcoor), rinman(1), &
                    rinman(2), ndim, nnodes )

!     --- Delete superfluous arrays

      call ini066text ( ibuffr, isolnw, 'usolnw' )
      if ( indknw>0 ) call ini066text ( ibuffr, indknw, 'indknw' )

      deallocate ( nodes, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'nodes' )

1000  call erclos ( 'manbndbf' )
      ierror = 0
      end
