      subroutine prcopysepuser ( ibuffr, buffer, isol, newvec, npoint, &
                                 kprob, namevec, iseq )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 21-11-2012 Long integers
!        version  1.0  date 05-04-2006
!
!   copyright (c) 2006-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy a sepran solution vector with one component per point into the
!     iseq^th component of a user vector of size npoint x number_of_components
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), isol(*), npoint, kprob(*), iseq
      double precision buffer(*), newvec(npoint,*)
      character (len=*) namevec

!     buffer        i/o   Buffer array
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     iseq           i    Sequence number in newvec to which solution in isol
!                         must be copied
!     isol           i    Standard SEPRAN array containing information of the
!                         solution array
!                         isol correpsponds to a vector with one component
!     kprob          i    Standard SEPRAN array, containing information of
!                         the problem
!     namevec        i    name of vector
!     newvec        i/o   array of length npoint x nunkp from which the iseq_th
!                         component is filled
!     npoint         i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipusol
      integer inunkp, nphys, indprp, indprf
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     indprf         indication if kprob part f has been filled (>0)
!                    or not (0)
!     indprp         Indication if array kprob part p has been filled (1)
!                    or not (0)
!     inunkp         Number of degrees of freedom in point
!     ipusol         Starting address of solution array
!     nphys          Number of physical unknowns, or maximal degrees of
!                    freedom per point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inidgttext

!     DCOPY          Copy double precision dx to double precision dy
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INI061         compute pointers to degrees of freedom of vectors
!     INIDGTTEXT     Compute pointer of array in BUFFER
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1059   number of degrees of freedom incorrect
!    2829   Number of degrees of freedom per point not constant for input vector
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcopysepuser' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcopysepuser'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Activate arrays

      call ini061 ( ibuffr, isol, kprob, indprf, indprp, &
                    inunkp, nphys )
      if ( ierror/=0 ) go to 1000

      if ( indprf>0 ) then

!     --- Error indprf>0, has not been implemented

         call errsub ( 2829, 0, 0, 0 )
         go to 1000

      end if  ! ( indprf==0 )

      if ( inunkp/=1 ) then

!     --- Error inunkp#1, has not been implemented

         call errint ( inunkp, 1 )
         call errint ( 1, 2 )
         call errchr ( namevec, 1 )
         call errsub ( 1059, 2, 0, 1 )
         go to 1000

      end if  ! ( indprf==0 )

!     --- Compute starting addresses

      ipusol = inidgttext ( isol(1), 'usol' )

!     --- Copy vector

      call dcopy ( npoint, buffer(ipusol), 1, newvec(1,iseq), 1 )

1000  call erclos ( 'prcopysepuser' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prcopysepuser'

      end if  ! ( debug )

      end
