      subroutine prevalveccexp ( vecout, vectors, lenvec, numvecrhs, &
                                 istore, store, lenistore, lenstore, &
                                 text )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-12-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Evaluate vector expression and fill actual values of vectors
!     Result is stored in vecout (complex variant)
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     expression
!     vector
!     complex
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/comcons1f90'
      include 'SPcommon/cusconsf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer lenvec, numvecrhs, lenistore, istore(lenistore), &
              lenstore
      double complex vecout(lenvec), vectors(lenvec,numvecrhs)
      double precision store(lenstore)
      character (len=*) text

!     istore         i    Contains the expression as stored in common
!                         cexpression, where all scalars and reals
!                         have been computed and stored in store
!                         References to these values are given by
!                         their position in store plus 1000
!                         The values of the vectors are stored in
!                         array vectors
!                         References to these vectors are given by
!                         their position in vectors plus 10000
!     lenistore      i    length of array istore
!     lenstore       i    Length of array store
!     lenvec         i    Length of all vectors
!     numvecrhs      i    Number of vectors in array vectors
!     store          i    Contains values of reals ans scalars, see istore
!     text           i    String variable containing some text for error message
!     vecout         o    Output vector, result of the evaluation
!     vectors        i    Contains the values of all vectors at the right-hand
!                         side of the expressiom
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer maxlevels, lenwork
      parameter ( maxlevels=10, lenwork=50 )
      integer iwork(lenwork), levels(maxlevels), nlevel, icount, ncount, &
              indintexpr, ival, endintexp, iseq, &
              irefwork(lenwork), error
      double complex, allocatable :: work(:,:)
      logical debug, finished

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     endintexp      Last position in integer part of expression
!                    that is to be used
!     error          Return value of allocate or deallocate
!     finished       Indicates if the loop has finished (true) or not
!                    (false)
!     icount         Counter
!     indintexpr     index of address in inpsep
!     irefwork       integer work array of length lenwork
!                    if irefwork(i) = 0, it concerns a constant
!                    otherwise (1) a vector
!     iseq           Sequence number of expression to be evaluated
!                    Corrected with 200000000
!     ival           Integer value in inpsep
!     iwork          integer work array of length lenwork
!                    iwork is used to store the integer information of the
!                    expression
!                    For a description see subroutine READEXPR
!     lenwork        Declared length of array work
!     levels         Array defining the starting addresses of each level
!                    Each time an opening bracket ( is found a new level is
!                    created, each time a closing bracket is found the
!                    level is decreased by one
!     maxlevels      Declared length of array levels
!     ncount         Counter
!     nlevel         Number of levels stored in levels
!     work           Work array of length lenwork
!                    work is used to store the real information of the
!                    expression
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PREVALVECCEXP1 Read subexpression not containing any brackets and
!                    translate into a vector. (complex variant)
!     PREVALVECCEXP2 Interpret last part of a vecor expression, &
!                    possibly closed by brackets (complex variant)
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2800   Incorrect vector inpsep
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Copied and adapted from readexpression
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prevalveccexp' )
      debug = .false.

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prevalveccexp'
         write(irefwr,1) 'lenvec, numvecrhs', lenvec, numvecrhs
         call prinin ( istore, lenistore, 'istore' )
         call prinrl ( store, lenstore, 'store' )
  1      format ( a, 1x, (10i10) )

      end if
      if ( ierror/=0 ) go to 1000

      allocate ( work(lenvec,lenwork), stat = error )
      if ( error/=0 ) call eralloc ( error, lenvec*lenwork, 'work' )
      if ( ierror/=0 ) go to 1000

      nlevel = 1   ! level for evaluation between brackets
      icount = 1   ! counter for iwork, work
      levels(1) = 1  ! Start of array levels
      indintexpr = 1  ! next position in inpsep
      endintexp = lenistore
      finished = .false.  ! define when loop is finished

      if ( debug ) &
         write(irefwr,1) 'indintexpr, endintexp', indintexpr, endintexp

      do while ( .not. finished )

         if ( indintexpr>endintexp ) then

!        --- end of inpsep has been reached, so we are at the end

            ncount = icount-1
            if ( debug ) then
               write(irefwr,*) 'end of inpsep has been reached'
               write(irefwr,1) 'icount, ncount', icount, ncount
               call prinin ( iwork, ncount, 'iwork' )
               call prinin ( irefwork, ncount, 'irefwork')
            end if

            call prevalveccexp1 ( work, iwork, ncount, vecout, text, &
                                  lenvec, irefwork )
            finished = .true.

         else if ( istore(indintexpr)==-4 ) then

!        --- Closing bracket found

            if ( debug ) then

               write(irefwr,*) 'Closing bracket found ', indintexpr
               write(irefwr,1) 'nlevel, icount', nlevel, icount

            end if  ! ( debug )

            call prevalveccexp2 ( levels, nlevel, icount, iwork, work, &
                                  text, lenvec, irefwork )
            indintexpr = indintexpr+1

            if ( debug ) then

!           --- Debug information

               write(irefwr,1) 'indintexpr, nlevel, icount', &
                                indintexpr, nlevel, icount
               call prinin ( levels, nlevel, 'levels' )
               call prinin ( iwork, icount-1, 'iwork' )
               call prinrl ( work, icount-1, 'work' )

            end if  ! ( debug )

         else if ( istore(indintexpr)==-3 ) then

!        --- Opening bracket found, raise levels and icount

            if ( debug ) &
               write(irefwr,*) 'Opening bracket found ', indintexpr
            nlevel = nlevel+1
            levels(nlevel) = icount
            indintexpr = indintexpr+1

!           --- If the next item is a function raise icount

            if ( istore(indintexpr)>=10 ) icount = icount+1

            if ( debug ) then

               write(irefwr,1) 'indintexpr, icount, nlevel', &
                                indintexpr, icount, nlevel
               call prinin ( levels, nlevel, 'levels' )

            end if  ! ( debug )

         else if ( istore(indintexpr)<1000 ) then

!        --- Operator found, fill iwork

            call inichkarlen ( icount, lenwork, 'work' )
            if ( ierror/=0 ) go to 1000

            iwork(icount) = istore(indintexpr)
            indintexpr = indintexpr+1
            if ( debug ) then
               write(irefwr,*) 'operator found'
               write(irefwr,1) 'indintexpr, icount', indintexpr, icount
            end if  ! ( debug ) then

!           --- Check if next item is number or vector

            ival = istore(indintexpr)
            if ( debug ) write(irefwr,1) 'ival', ival
            if ( ival>10000 ) then

!           --- Vector found

               irefwork(icount) = 1   ! vector
               iseq = istore(indintexpr)-10000
               work(1:lenvec,icount) = vectors(1:lenvec,iseq)
               indintexpr = indintexpr+1

            else if ( ival>=1000 ) then

!           --- Number found

               irefwork(icount) = 0   ! number
               iseq = istore(indintexpr)-1000
               work(1:lenvec,icount) = store(iseq)
               indintexpr = indintexpr+1

            end if  ! ( ival>10000 )

            if ( debug ) then

!           --- Debug information

               write(irefwr,*) 'Operator found ', indintexpr
               write(irefwr,1) 'ival', ival
               call prinin ( levels, nlevel, 'levels' )
               call prinin ( iwork, icount, 'iwork' )
               call prinin ( irefwork, icount, 'work' )
               call prinrl ( work(1,icount), lenvec, 'work' )

            end if  ! ( debug )

            icount = icount+1

         else

!        --- Error, this situation should not occur

            call errint ( indintexpr, 1 )
            call errint ( istore(indintexpr), 2 )
            call errsub ( 2800, 2, 0, 0 )
            call errcrd
            call prinin ( istore, lenistore, 'istore' )
            go to 1000

         end if  ! ( indintexpr>=endintexp )

      end do !  while ( .not. finished )

      deallocate ( work, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'work' )

1000  call erclos ( 'prevalveccexp' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prevalveccexp'

      end if

      end
