      subroutine prevalveccexp2 ( levels, nlevel, icount, iwork, work, &
                                  text, lenvec, irefwork )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 31-12-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpret last part of a vector expression, possibly closed by brackets
!     complex case
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     complex
!     vector
!     expression
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer levels(*), nlevel, icount, iwork(*), lenvec, irefwork(*)
      double complex work(lenvec,*)
      character (len=*) text

!     icount        i/o   Counter
!                         Defines present position in iwork
!     irefwork      i/o   integer work array of length lenwork
!                         if irefwork(i) = 0, it concerns a constant
!                         otherwise (1) a vector
!     iwork         i/o   integer work array
!                         For a description see subroutine readexpression
!     lenvec         i    Length of left-hand side vector in expression
!     levels        i/o   Array defining the starting addresses of each level
!                         Each time an opening bracket ( is found a new level is
!                         created, each time a closing bracket is found the
!                         level is decreased by one
!     nlevel        i/o   Number of levels stored in levels
!     text           i    String variable containing some text for error message
!     work          i/o   Work array
!                         work contains the numbers that must be used to apply
!                         the operation
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istart, ncount, i
      double complex result(lenvec)
      logical debug, extradebug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     extradebug     If true extra debugging statements are carried out
!     i              Counting variable
!     istart         First position used at a part
!     ncount         Counter
!     result         Array to store the result of a vector expression
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PREVALVECCEXP1 Read subexpression not containing any brackets and
!                    translate into a vector. (complex variant)
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prevalveccexp2' )
      debug = .false.
      extradebug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prevalveccexp2'
         write(irefwr,1) 'icount', icount
         call prinin ( levels, nlevel, 'levels' )
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      istart = levels(nlevel)
      ncount = icount-istart

      if ( debug ) then
         write(irefwr,*) 'istart, ncount ', istart, ncount
         call prinin ( iwork, istart+ncount-1, 'iwork' )
         if ( extradebug ) &
            call prinrl1 ( work, istart+ncount-1, lenvec, 'work')
      end if

      if ( ncount>1 ) then

!     --- ncount>1, evaluate expression

         call  prevalveccexp1 ( work(1,istart), iwork(istart), ncount, &
                                result, text, lenvec, irefwork(istart) )
         if ( ierror/=0 ) go to 1000

         if ( debug ) then
            if ( extradebug ) then
               call prinrl ( result, lenvec, 'result' )
               call prinrl1 ( work, istart+ncount-1, lenvec, 'work')
            end if  ! ( extradebug ) then
            call prinin ( iwork, istart+ncount-1, 'iwork' )
            write(irefwr,*) 'istart, ncount ', istart, ncount
         end if

!        --- Clear end of iwork and work, &
!            Decrease icount and nlevel
!            Store result in work

         do i = 0, ncount-1
            iwork(istart+i) = 0
            work(1:lenvec,istart+i) = 0d0
            irefwork(istart+i) = 0
         end do
         icount = max(istart,2)
         work(1:lenvec,icount-1) = result(1:lenvec)
         if ( icount==2 .and. iwork(1)==0 ) iwork(1) = 1

      else if ( ncount==1 .and. icount>2 )  then

!     --- ncount = 1

         if ( iwork(icount-1)==1 ) then

!        --- iwork(icount-1) = 1, hence addition
!            decrease icount and update work

            work(1:lenvec,icount-2) = work(1:lenvec,icount-1)
            iwork(icount-1) = 0
            irefwork(icount-1) = 0
            icount = icount-1

         end if  ! ( iwork(icount-1)==1 )

      end if  ! ( ncount>1 )
      nlevel = nlevel-1

1000  call erclos ( 'prevalveccexp2' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'icount, nlevel ', icount, nlevel
         call prinin ( levels, nlevel, 'levels' )
         call prinin ( iwork, icount-1, 'iwork' )
         if ( extradebug ) &
            call prinrl1 ( work, icount-1, lenvec, 'work')
         write(irefwr,*) 'End prevalveccexp2'

      end if

      end
