      subroutine prevalvecexpr2 ( levels, nlevel, icount, iwork, work, &
                                  text, lenvec, irefwork )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 08-09-2015 extension with subtraction
!        version  1.0  date 31-12-2013
!
!   copyright (c) 2013-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpret last part of a vector expression, possibly closed by brackets
!
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     vector
!     expression
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer levels(*), nlevel, icount, iwork(*), lenvec, irefwork(*)
      double precision work(lenvec,*)
      character (len=*) text

!     icount        i/o   Counter
!                         Defines present position in iwork
!     irefwork      i/o   integer work array of length lenwork
!                         if irefwork(i) = 0, it concerns a constant
!                         otherwise (1) a vector
!     iwork         i/o   integer work array
!                         For a description see subroutine readexpression
!     lenvec         i    Length of left-hand side vector in expression
!     levels        i/o   Array defining the starting addresses of each level
!                         Each time an opening bracket ( is found a new level is
!                         created, each time a closing bracket is found the
!                         level is decreased by one
!     nlevel        i/o   Number of levels stored in levels
!     text           i    String variable containing some text for error message
!     work          i/o   Work array
!                         work contains the numbers that must be used to apply
!                         the operation
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istart, ncount, i, irefworksav
      double precision result(lenvec)
      logical debug, extradebug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     extradebug     If true extra debugging statements are carried out
!     i              Counting variable
!     irefworksav    Help parameter to save the value of irefwork(istart)
!     istart         First position used at a part
!     ncount         Counter
!     result         Array to store the result of a vector expression
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PREVALVECEXPR1 Read subexpression not containing any brackets
!                    and translate into a vector
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prevalvecexpr2' )
      debug = .false.
      extradebug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prevalvecexpr2'
         write(irefwr,1) 'icount', icount
         call prinin ( levels, nlevel, 'levels' )
  1      format ( a, 1x, (10i6) )
  4      format ( a, 1x, i3, 1x, (5d12.4) )

      end if
      if ( ierror/=0 ) go to 1000

      istart = levels(nlevel)
      ncount = icount-istart

      if ( debug ) then
         write(irefwr,*) 'istart, ncount ', istart, ncount
         call prinin ( iwork, istart+ncount-1, 'iwork' )
         call prinin ( irefwork, istart+ncount-1, 'irefwork' )
         if ( extradebug ) then
            call prinrl1 ( work, istart+ncount-1, lenvec, 'work' )
         else
            call prinrl ( work(1,istart+ncount-1), min(5,lenvec), &
                          'work(istart+ncount-1)')
         end if  ! ( extradebug )
      end if  ! ( debug )

      if ( ncount>1 ) then

!     --- ncount>1, evaluate expression

         call  prevalvecexpr1 ( work(1,istart), iwork(istart), ncount, &
                                result, text, lenvec, irefwork(istart) )
         if ( ierror/=0 ) go to 1000

         if ( debug ) then
            if ( extradebug ) then
               call prinrl ( result, lenvec, 'result' )
               call prinrl1 ( work, istart+ncount-1, lenvec, 'work')
            end if  ! ( extradebug ) then
            call prinin ( iwork, istart+ncount-1, 'iwork' )
            call prinin ( irefwork, istart+ncount-1, 'irefwork' )
            write(irefwr,1) 'istart, ncount, icount', &
                             istart, ncount, icount
         end if

!        --- Clear end of iwork and work, &
!            Decrease icount and nlevel
!            Store result in work

         irefworksav = irefwork(istart)
         do i = 0, ncount-1
            iwork(istart+i) = 0
            irefwork(istart+i) = 0
            work(1:lenvec,istart+i) = 0d0
         end do
         icount = max(istart,2)
         work(1:lenvec,icount-1) = result(1:lenvec)
         irefwork(icount-1) = irefworksav
         if ( icount==2 .and. iwork(1)==0 ) iwork(1) = 1

      else if ( ncount==1 .and. icount>2 )  then

!     --- ncount = 1

         if ( iwork(icount-1)==1 ) then

!        --- iwork(icount-1) = 1, hence addition
!            decrease icount and update work

            icount = icount-1
            work(1:lenvec,icount-1) = work(1:lenvec,icount)
            iwork(icount) = 0
            irefwork(icount-1) = irefwork(icount)

         else if ( iwork(icount-1)==-1 ) then

!        --- iwork(icount-1) = -1, hence subtraction
!            decrease icount and update work

            icount = icount-1
            work(1:lenvec,icount-1) = -work(1:lenvec,icount)
            iwork(icount) = 0
            irefwork(icount-1) = irefwork(icount)

         end if  ! ( iwork(icount-1)==1 )

      end if  ! ( ncount>1 )
      nlevel = nlevel-1

1000  call erclos ( 'prevalvecexpr2' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'icount, nlevel ', icount, nlevel
         call prinin ( levels, nlevel, 'levels' )
         call prinin ( iwork, icount-1, 'iwork' )
         call prinin ( irefwork, icount-1, 'irefwork' )
         if ( extradebug ) then
            call prinrl1 ( work, icount-1, lenvec, 'work')
         else
            do i = 1, icount-1
               write(irefwr,4) 'work ', i, work(1:min(5,lenvec),i)
            end do  ! i = 1, ncount
         end if  ! ( extradebug )
         write(irefwr,*) 'End prevalvecexpr2'

      end if

      end
