      subroutine prfillnodints ( ibuffr, kmesh, numsurfs, surfaces, &
                                 nodes, nnodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-10-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the common nodes of each couple of surfaces in the set
!     surfaces(1:numsurfs)
! **********************************************************************
!
!                       KEYWORDS
!
!     common
!     intersection
!     node
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), kmesh(*), numsurfs, surfaces(numsurfs), &
              nodes(*), nnodes

!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     nnodes         o    Number of nodes
!     nodes          o    Contains the node numbers
!     numsurfs       i    number of surfaces in array surfaces
!     surfaces       i    Contains teh surface numbers from which the
!                         intersection must be computed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer error, npoint, i, j, k, nfound, nnodesi, nnodesj, &
              inpsep(2), inode
      integer, allocatable :: iwork(:), istorei(:), istorej(:), &
                              commonset(:)
      logical debug

!     commonset      Contains common nodes of sets istorei and istorej
!     debug          If true debug statements are carried out otherwise
!                    they are not
!     error          Return value of allocate or deallocate
!     i              Counting variable
!     inode          Node sequence number
!     inpsep         The integer input read is stored in array INPSEP
!     istorei        Contains all nodes in surface i
!     istorej        Contains all nodes in surface j
!     iwork          integer work array of length npoint to mark the nodes
!                    that have been filled in nodes
!                    0:  not filled
!                    1:  filled
!     j              Counting variable
!     k              Counting variable
!     nfound         Number of common nodes of sets istorei and istorej
!     nnodesi        Number of nodes in istorei
!     nnodesj        Number of nodes in istorej
!     npoint         Number of nodal points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INICOMMONARRAY Compute common entries of two sets of arrays and
!                    store in new set
!     PRFILLNODSURFS Fill array nodes with nodal points as defined in inpsep
!                    Special case: nodes in surfaces
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     iwork(1:npoint) = 0
!     for i := 1 (1) numsurfs-1 do
!        istorei := all nodes in surfaces(i)
!        for j := i+1 (1) numsurfs do
!           istorej := all nodes in surfaces(j)
!           store nodes in intersection istorei and istorej satisfying iwork=0
!           set iwork = 1 for these nodes
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prfillnodints' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prfillnodints'
         call prinin ( surfaces, numsurfs, 'surfaces' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get parameters

      npoint = kmesh(8)

!     --- allocate work arrays

      allocate ( iwork(npoint), istorei(npoint), istorej(npoint), &
                 commonset(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, 3*npoint, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- iwork(1:npoint) = 0

      iwork = 0

      nnodes = 0

!     --- for i := 1 (1) numsurfs-1 do

      do i = 1, numsurfs-1

!     --- istorei := all nodes in surfaces(i)

         inpsep(1) = 1            ! only one surface
         inpsep(2) = surfaces(i)  ! surface number
         call prfillnodsurfs ( ibuffr, kmesh, 0, inpsep, &
                               istorei, nnodesi, 3 )

!        --- for j := i+1 (1) numsurfs do

         do j = i+1, numsurfs

!        --- istorej := all nodes in surfaces(j)

            inpsep(2) = surfaces(j)  ! surface number
            call prfillnodsurfs ( ibuffr, kmesh, 0, inpsep, &
                                  istorej, nnodesj, 3 )

!           --- store nodes in intersection istorei and istorej satisfying
!               iwork=0
!               set iwork = 1 for these nodes
!               First store intersection in commonset of length nfound

            call inicommonarray ( istorei, nnodesi, istorej, nnodesj, &
                                  npoint, nfound, commonset )

            do k = 1, nfound

               inode = commonset(k)      ! node in commonset
               if ( iwork(inode)==0) then

!              --- Node has not been stored yet

                  nnodes = nnodes+1      ! raise nnodes
                  nodes(nnodes) = inode  ! store node
                  iwork(inode) = 1       ! mark node as stored

               end if  ! ( iwork(inode)==0)

            end do  ! k = 1, nfound

         end do  ! j = i+1, numsurfs

      end do  ! i = 1, numsurfs-1

!     --- deallocate work arrays

      deallocate ( iwork, istorei, istorej, commonset, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  call erclos ( 'prfillnodints' )
      if ( debug ) then

!     --- Debug information

         call prinin ( nodes, nnodes, 'nodes' )
         write(irefwr,*) 'End prfillnodints'

      end if  ! ( debug )

      end
