      subroutine prmakevector ( ibuffr, buffer, kmesh, kprob, isol, &
                                iinvec, numvct, ioutvec )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 12-02-2015 Extra parameter ioutvec
!        version  1.1  date 21-11-2012 Long integers
!        version  1.0  date 02-03-2011
!
!   copyright (c) 2011-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Make new vector out of two old vectors
!     The components of the first vector are the first ones, &
!     the components of the second vector are the last
! **********************************************************************
!
!                       KEYWORDS
!
!     vector
!     create
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/comcons2f90'
      include 'SPcommon/cpost5f90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), kmesh(*), kprob(*), isol(5,*), iinvec(*), &
              numvct
      integer, intent(in) :: ioutvec
      double precision buffer(*)

!     buffer        i/o   Buffer array
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     iinvec         i    integer input array, &
!                         Contents See subroutine prmanv
!     ioutvec        i    Sequence number of output vector in isol
!     isol          i/o   Standard SEPRAN array containing information of the
!                         solution arrays
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     kprob          i    Standard SEPRAN array, containing information of
!                         the problem
!     numvct        i/o   Largest sequence number of vector that has been filled
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipvec2, ipvec3, ipvec1
      integer ivec1, ivec2, indprf1, indprh1, indprp1, nunkp1, &
              nphys1, indprf2, indprh2, indprp2, nunkp2, nphys2, &
              number, npoint, indprf, indprp, &
              nphys, itypvc, ivec, iprob, itemp(5)
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     indprf         indication if kprob part f has been filled (>0)
!                    or not (0)
!     indprf1        indication if kprob1 part f has been filled (>0)
!                    or not (0)
!     indprf2        indication if kprob2 part f has been filled (>0)
!                    or not (0)
!     indprh1        indication if kprob part h has been filled (>0)
!                    or not (0)
!     indprh2        indication if kprob part h has been filled (>0)
!                    or not (0)
!     indprp         Indication if array kprob part p has been filled (1)
!                    or not (0)
!     indprp1        Indication if array kprob1 part p has been filled (1)
!                    or not (0)
!     indprp2        Indication if array kprob2 part p has been filled (1)
!                    or not (0)
!     iprob          Actual problem number
!     ipvec1         Starting address of array VECTOR1
!     ipvec2         Starting address of array VECTOR2
!     ipvec3         Starting address of array VECTOR3
!     itemp          Temporaray array to store information of new vector
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     ivec           Sequence number of output vector
!     ivec1          Sequence number of first vector
!     ivec2          Sequence number of second vector
!     nphys          Number of physical unknowns, or maximal degrees of
!                    freedom per point
!     nphys1         Number of physical unknowns, or maximal degrees of
!                    freedom per point in first mesh
!     nphys2         Number of physical unknowns, or maximal degrees of
!                    freedom per point in second mesh
!     npoint         Number of nodal points
!     number         The number-th unknown in each nodal point of the array
!                    is considered. If the array is complex number=2*i-1
!                    corresponds to the real part of array i and
!                    number=2*i to the imaginary part of array i
!     nunkp1         maximum number of unknowns per point in vector 1
!     nunkp2         maximum number of unknowns per point in vector 2
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inidgttext

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     INI066TEXT     delete temporary array
!     INI068         compute pointers to degrees of freedom of vectors
!     INI086         compute pointers to degrees of freedom of vectors
!     INIDGTTEXT     Compute pointer of array in BUFFER
!     INISTREXT      Find the structure of the vector with exactly
!                    NUMBER unknowns per point
!     PRININ         print 1d integer array
!     PRMAKEVECTOR1  Create new vector from two separate vectors
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3071   Number of degrees of freedom not constant in one of the two vectors
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prmakevector' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prmakevector'
         call prinin ( iinvec, 13, 'iinvec' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get parameters

      ivec1 = iinvec(12)
      ivec2 = iinvec(13)
      if ( debug ) then
         write(irefwr,1) 'ioutvec, ivec1, ivec2', ioutvec, ivec1, ivec2
         write(irefwr,1) 'isol(1,ivec1)', isol(1,ivec1)
         write(irefwr,1) 'isol(1,ivec2)', isol(1,ivec2)
         write(irefwr,1) 'isol(1,ioutvec)', isol(1,ioutvec)
      end if  ! ( debug )

!     --- Activate arrays

      call ini068 ( ibuffr, isol(1,ivec1), kprob, indprf1, indprh1, &
                    indprp1, nunkp1, nphys1 )
      call ini068 ( ibuffr, isol(1,ivec2), kprob, indprf2, indprh2, &
                    indprp2, nunkp2, nphys2 )
      if ( ierror/=0 ) go to 1000

      if ( debug ) then
         write(irefwr,1) 'indprf1, indprh1, indprp1, nunkp1, nphys1', &
                          indprf1, indprh1, indprp1, nunkp1, nphys1
         write(irefwr,1) 'indprf2, indprh2, indprp2, nunkp2, nphys2', &
                          indprf2, indprh2, indprp2, nunkp2, nphys2
      end if  ! ( debug )

      if ( indprf1>0 .or. indprf2>0 ) then

!     --- indprf1>0 or indprf2>0, not yet implemented

         call errsub ( 3071, 0, 0, 0 )
         go to 1000

      end if  ! ( indprf1>0 .or. indprf2>0 )

!     --- Create a new vector with nunkp1+nunkp2 degrees of freedom

      number = nunkp1+nunkp2

      if ( debug ) write(irefwr,1) 'number', number

      call inistrext ( number, kprob, kmesh, itypvc, ivec, iprob )

      if ( debug ) write(irefwr,1) 'ivec, iprob', ivec, iprob

      if ( ierror/=0 ) go to 1000

!     --- Create output vector

      if ( ioutvec==ivec1 .or. ioutvec==ivec2 ) then

!     --- Create temporary vector in itemp

         itemp(1:5) = 0
         call ini086 ( ibuffr, itemp, itypvc, ivec, iprob, &
                       kmesh, kprob, indprf, indprp, nphys, &
                       names(ioutvec) )
         ivec = itemp(1)

      else

!     --- Create new vector

         call ini086 ( ibuffr, isol(1,ioutvec), itypvc, ivec, iprob, &
                       kmesh, kprob, indprf, indprp, nphys, &
                       names(ioutvec) )
         ivec = isol(1,ioutvec)

      end if  ! ( ioutvec==ivec1 .or. ioutvec==ivec2 )
      if ( ierror/=0 ) go to 1000

!     --- activate vectors

      ipvec1 = inidgttext ( isol(1,ivec1), names(ivec1) )
      ipvec2 = inidgttext ( isol(1,ivec2), names(ivec2) )
      ipvec3 = inidgttext ( ivec, names(ioutvec) )
      if ( debug ) &
         write(irefwr,1) 'ipvec1, ipvec2, ipvec3', &
                          ipvec1, ipvec2, ipvec3

!     --- Create vector

      npoint = kmesh(8)
      call prmakevector1 ( buffer(ipvec1), npoint, nunkp1, &
                           buffer(ipvec2), nunkp2, buffer(ipvec3) )

      if ( ioutvec==ivec1 .or. ioutvec==ivec2 ) then

!     --- Remove array

         call ini066text ( ibuffr, isol(1,ioutvec), names(ioutvec) )
         isol(1:5,ioutvec) = itemp(1:5)

      end if  ! ( ioutvec==ivec1 .or. ioutvec==ivec2 )

      numvct = max(numvct,ioutvec)

1000  call erclos ( 'prmakevector' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prmakevector'

      end if  ! ( debug )

      end
