      subroutine prsimplegcr ( ibuffr, buffer, qmat,
     +                         gmat, intmat, ru, inpsol, qmatpre,
     +                         dmat, projvc, rmat, rmatpre,
     +                         rinsol, u, normrhsd, infmatvec, ninpsol1,
     +                         nrinsol1, qmatprime, intmatpre,
     +                         intmatpredg, kprob, cmat )
! ======================================================================
!
!        programmer    Guus Segal
!        version  6.3  date 20-08-2009 Better termination criterion
!        version  6.2  date 20-08-2009 Extra call for performance checking
!        version  6.1  date 14-07-2009 New call to prsimpleiter
!        version  6.0  date 28-04-2009 Extra parameter cmat
!        version  5.0  date 24-10-2008 New parameter list
!        version  4.0  date 24-07-2008 Change of parameter list
!        version  3.1  date 14-11-2007 Extension with msimpler
!
!   copyright (c) 2003-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Solve a system of linear equations of special form using the SIMPLE method
!     The method is accelerated by the GCR process
! **********************************************************************
!
!                       KEYWORDS
!
!     simple
!     linear_solver
!     matrix
!     real
!     krylov
!     conjugate_gradients
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpi'
      include 'SPcommon/cinout'
      include 'SPcommon/cconst'
      include 'SPcommon/citerinfo'
      include 'SPcommon/cdctim'
      include 'SPcommon/cmcdpr'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer intmat(*), infmatvec(*), ibuffr(*), ninpsol1, nrinsol1,
     +        intmatpre(*), intmatpredg(*), kprob(*), inpsol(ninpsol1,*)
      double precision qmat(*), gmat(*), ru(*), dmat(*), buffer(*),
     +                 qmatpre(*), projvc(*), rmat(*), rmatpre(*),
     +                 rinsol(*), u(*), normrhsd, qmatprime(*), cmat(*)

!     buffer        i/o   Buffer array
!     cmat           i    Matrix C (Pressure part of large matrix)
!     dmat           i    Matrix D (divergence matrix)
!     gmat           i    Matrix G (Gradient matrix)
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     infmatvec      i    Array to indicate information about the matrix-vector
!                         multiplication and other solver related quantities
!                         See subroutine prsimple for a description
!     inpsol         i    Integer input array for subroutine solvel
!                         See read03 for a description
!     intmat         i    First part of matrix structure in case of Schur
!                         storage.
!                         The description can be found in incommatsimple
!                         as intmatnew
!     intmatpre      i    See intmat, now with respect to preconditioner
!     intmatpredg    i    Standard SEPRAN array of length 5 containing
!                         information concerning the structure of the
!                         DG preconditioning matrix
!     kprob          i    Standard SEPRAN array, containing information of
!                         the problem
!     ninpsol1       i    Length of array inpsol per (sub)equation
!     normrhsd       i    Norm of right-hand side vector
!     nrinsol1       i    Length of array rinsol per (sub)equation
!     projvc         i    Double precision array containing the projection
!                         vectors.
!                         See subroutine prpjspar4 for a description
!     qmat           i    Matrix Q
!     qmatpre        i    Preconditioning of matrix Q
!     qmatprime      i    Diagonal of global matrix Q
!     rinsol         i    Contains information concerning the linear solver
!                         See the Programmers Guide for a description
!     rmat           i    Matrix R
!     rmatpre        i    Preconditioning of matrix R
!     ru            i/o   Residual of system of equations
!     u             i/o   Solution vector
!                         v during the iteration
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer k, i, nusol, ireler, iprint, iter, niteru, niterp,
     +        maxiter, itimesav, niteruacc, niterpacc, nu, np,
     +        niteracc, niterutot, niterptot, error, ngcr, ipinnercontr,
     +        ipirowd, ipicold
      double precision, allocatable :: svec(:,:), vvec(:,:), rucc(:),
     +                                 diagpmm(:)
      double precision norm, eps, normu, normp, epsu, timebefore, rferr
      logical debug, matlab, converged, scaling

!     converged      If true the process is converged
!     debug          If true debug statements are carried out otherwise
!                    they are not
!     diagpmm        Work array to store the diagonal of the pressure
!                    scaling matrix to check the accuracy
!     eps            Local accuracy
!     epsu           Local accuracy with respect to velocity
!     error          Return value of allocate or deallocate
!     i              Counting variable
!     ipicold        Starting address of column numbers of D matrix
!     ipinnercontr   Starting address of array innercontr
!     ipirowd        Starting address of number of non-zero entries per row
!                    of D matrix (accumulated)
!     iprint         Print level for information
!     ireler         Defines how the error is measured
!     iter           Iteration sequence number
!     itimesav       Help parameter to save the parameter itime
!     k              Counting variable
!     matlab         If true matlab input is generated
!     maxiter        Maximum number of iterations
!     ngcr           maximum number of GCR iterations
!     niteracc       Accumulated number of GCR iterations
!     niterp         Number of iterations to solve the p-equations
!     niterpacc      Accumulated number of iterations to solve the p-equations
!     niterptot      Total number of p iterations in all GCR steps
!     niteru         Number of iterations to solve the u-equations
!     niteruacc      Accumulated number of iterations to solve the u-equations
!     niterutot      Total number of u iterations in all GCR steps
!     norm           norm of residual
!     normp          norm of p-vector
!     normu          norm of u-vector
!     np             Number of pressure unknowns
!     nu             Number of velocity unknowns
!     nusol          Total number of unknowns
!     rferr          Maximal accuracy possible for this matrix, due to
!                    round-off
!     rucc           Work array to store ru temporarily
!     scaling        If true the residual is scaled before computing the
!                    termination criterion
!     svec           Work array of size ngcr x n_u to store the vectors
!                    s during the iteration
!     timebefore     Time at entrance of subroutine
!     vvec           Work array of size ngcr x n_u to store the vectors
!                    v during the iteration
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision prpardnrm2

!     ASKCLK         print elapsed time
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     ERSETTIME      Get the CPU time at start of subroutine
!     PRGCRCYCLE     Perform one step of the gcr process
!     PRINRL         Print 1d real vector
!     PRLSCITER      Perform a lsc iteration as part of some outer loop
!     PRPARDNRM2     Parallel Blas routine to compute the 2-norm of a vector
!     PRSCHURSCALR   Compute the scaling matrix for the residual
!                    for the PMM approach
!     PRSIMPLEITER   Perform a simple iteration as part of some outer loop
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     676   The CG-process has terminated because the number of iterations
!           exceeds the maximum value
!     677   The maximal accuracy with respect to the condition of the
!           matrix is reached
!    2910   Not enough space
!    2911   Not possible to deallocate
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Start with The matrices Q, R, D and the vectors u, p, ru and r_p
!     For k := 0 (1) ngcr do
!        Perform one step of simple to compute du and dp
!        s^(k+1) := (du,dp), r = (ru,r_p)
!        v^(k+1) := A s^(k+1)
!        For i := 1 (1) k do
!           alpha := ( v^(k+1), v^i)
!           v^(k+1) := v^(k+1)- alpha v^i
!           s^(k+1) := s^(k+1)- alpha s^i
!        v^(k+1) := v^(k+1)/||v^(k+1)||
!        s^(k+1) := s^(k+1)/||v^(k+1)||
!        alpha := ( r, v^(k+1))
!        (u,p) := (u,p) + alpha s^(k+1)
!        r := r - alpha v^(k+1)
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data niteruacc /0/, niterpacc /0/, niteracc /0/
! ======================================================================
!
      call eropen ( 'prsimplegcr' )
      call ersettime ( timebefore )
      ngcr = inpsol(13,1)   ! (maximum) number of gcr iterations
      nu = infmatvec(11)
      np = infmatvec(12)
      allocate(svec(nu+np,0:ngcr+1) , vvec(nu+np,0:ngcr+1), rucc(nu+np),
     +         diagpmm(np), stat = error)
      if ( error/= 0 ) then

!     --- error#0, so not enough space

         call errint ( (nu+np)*(ngcr+2),1 )
         call errint ( error, 2 )
         call errchr ( 'svec', 1 )
         call errsub ( 2910, 2, 0, 1 )
         go to 1000

      end if  ! ( error/= 0 )

      debug = .false.
      matlab = .false.
      scaling = .true.      ! the residual is scaled to get a better termination
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prsimplegcr'
         write(irefwr,1) 'nu, np ', nu, np
         call prinrl ( ru, nu+np, 'ru' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if
      if ( ierror/=0 ) go to 1000

      converged = .false.
      itimesav = itime
      itime = 0
      ipinnercontr = infmatvec(4)     ! starting address innercontr

      if ( matlab ) then
         open ( unit=19, file = 'fu.dat' )
         do i = 1, nu
            write(19,10) ru(i)
 10         format ( e15.8 )
         end do
         close ( unit=19 )
         open ( unit=19, file = 'fp.dat' )
         do i = 1, np
            write(19,10) ru(nu+i)
         end do
         close ( unit=19 )
      end if

      nusol = nu+np
      norm = prpardnrm2 (  nusol, ru, 1, ibuffr(ipinnercontr) )
      normu = prpardnrm2 (  nu, ru, 1, ibuffr(ipinnercontr) )
      rferr = 1d3*normrhsd*epsmac
      if ( debug )
     +   write(irefwr,2) 'norm, normu', norm, normu

!     --- Check if accuracy has been reached

      ireler = inpsol(11,1)
      if ( ireler==10 ) ireler = 1
      eps = rinsol(1)
      maxiter = inpsol(5,1)
      if ( maxiter==0 ) maxiter = nusol
      if ( debug ) write(irefwr,*) 'ireler, eps', ireler, eps
      if ( ireler==-1 ) then

!     --- Relative error compared to rhs

         eps = eps * normrhsd

      else if ( ireler==1 ) then

!     --- Relative error compared to initial residual

         epsu =eps * normu
         eps = eps * norm

      end if
      if ( ireler/=0 .and. norm<=eps ) then

!     --- Initial residual already small

         go to 1000

      end if
      iprint = inpsol(6,1)
      niterutot = 0
      niterptot = 0

      if ( scaling ) then

!     --- scaling, create scaling matrix for P part

         ipirowd  = intmat(2)
         ipicold  = ipirowd+np+1
         call prschurscalr ( diagpmm, np, dmat, qmat, intmat(ipirowd),
     +                       intmat(ipicold) )

         if ( ireler==1 ) then

!        --- ireler = 1, eps depends on initial residual;  reset

            eps = eps/norm
            epsu =eps/normu

         end if  ! ( ireler==1 )

!        --- Since we are using scaling the initial residual should be scaled
!            too
!            Compute scaled norm

         rucc = ru(1:nu+np)/qmat(1:nu)
         rucc(nu+1:nu+np) = ru(nu+1:nu+np)/diagpmm(1:np)
         norm = prpardnrm2 ( nu+np, rucc, 1, ibuffr(ipinnercontr) )
         normu = prpardnrm2 (  nu, rucc, 1, ibuffr(ipinnercontr) )

         if ( iprint>=2 ) then
            normp = prpardnrm2 (  np, rucc(nu+1), 1,
     +                            ibuffr(ipinnercontr+nu) )
         end if  ! ( iprint>=2 )

         if ( ireler==1 ) then

!        --- ireler = 1, eps depends on initial residual;  recompute

            eps = eps*norm   ! reset eps
            epsu = eps*normu

         end if  ! ( ireler==1 )

      end if  ! ( scaling )

!     --- Print heading if iprint>0

      if ( iprint>0 ) then
         if ( inpsol(3,1)==6 ) then
            write ( irefwr, 100 )   ! simple
100         format(//' Output for the Simple method'//)
         else if ( inpsol(3,1)==8 ) then
            write ( irefwr, 101 )   ! simpler
101         format(//' Output for the Simpler method'//)
         else if ( inpsol(3,1)==12 ) then
            write ( irefwr, 102 )   ! hsimple
102         format(//' Output for the HSimpler method'//)
         else if ( inpsol(3,1)==13 ) then
            write ( irefwr, 103 )   ! msimple
103         format(//' Output for the MSimpler method'//)
         else if ( inpsol(3,1)==14 ) then
            write ( irefwr, 104 )   ! lsc
104         format(//' Output for the LSC method'//)
         else
            write ( irefwr, 105 )   ! qlsc
105         format(//' Output for the QLSC method'//)
         end if  ! ( inpsol(3,1)==6 )
      end if  ! ( iprint>0 ) then

      if ( iprint==1 ) then

!     --- iprint = 1,  Print information

         write ( irefwr, 110 ) norm
110      format ( ' The 2-norm of the initial residual', ss, 1pe17.2e2 )

      else if ( iprint>=2 ) then

!     --- iprint>=2,  Maximal output

         if ( .not. scaling )
     +      normp = prpardnrm2 ( np, ru(nu+1), 1,
     +                           ibuffr(ipinnercontr+nu) )
         write ( irefwr, 120 ) 0, norm, normu, normp, 0, 0
120      format ( '  Iter    residual  res_u     res_p      iter_u',
     +            '   iter_p '/
     +            ' number   |r_k|_2   |ru_k|_2  |rp_k|_2' //
     +              i5, 3x, ss, 3(1pe10.2e2), 2i8 )

      end if
      iter = 0

!     --- For k := 0 (1) maxiter-1 do

      do k = 0, maxiter-1

!     --- Perform one step of simple to compute du and dp
!         s^(k+1) := (du,dp), r = (ru,r_p)

         iter = iter+1
         if ( inpsol(3,1)<14 ) then

!        --- SIMPLE iteration

            call prsimpleiter ( ibuffr, nu, np, qmat, gmat,
     +                          dmat, intmat, ru, svec(1,0), inpsol,
     +                          qmatpre, projvc, rmat, rmatpre,
     +                          rinsol, normrhsd, niterp,
     +                          infmatvec, ninpsol1, nrinsol1,
     +                          qmatprime, iter, intmatpre,
     +                          intmatpredg, kprob, cmat )

         else

!        --- LSC iteration

            call prlsciter ( ibuffr, nu, np, qmat, gmat,
     +                       dmat, intmat, ru, svec(1,0), inpsol,
     +                       qmatpre, projvc, rmat, rmatpre,
     +                       rinsol, normrhsd, niterp,
     +                       infmatvec, ninpsol1, nrinsol1,
     +                       qmatprime, intmatpre,
     +                       intmatpredg, kprob )

         end if  ! ( inpsol(3,1)<14 )
         if ( ierror/=0 ) go to 1000

         niteru = inpsol(1,2)       ! number of iterations carried out

!        --- Perform one step of the gcr process.

         call prgcrcycle ( ibuffr, buffer, nu, np, qmat, gmat,
     +                     dmat, svec, vvec, intmat, ngcr,
     +                     k, norm, ru, u, infmatvec, cmat )
         if ( ierror/=0 ) go to 1000

         if ( scaling ) then

!        --- scaling = true, copy ru into rucc and adapt
!            The residual is scaled in order to make the termination
!            criterion more reliable

            rucc = ru(1:nu+np)
            ru(1:nu) = ru(1:nu)/qmat(1:nu)
            ru(nu+1:nu+np) = ru(nu+1:nu+np)/diagpmm(1:np)
            norm = prpardnrm2 (  nu+np, ru, 1, ibuffr(ipinnercontr) )

         end if  ! ( scaling )

         normu = prpardnrm2 (  nu, ru, 1, ibuffr(ipinnercontr) )

         if ( iprint>=2 ) then

!        --- iprint>=2,  Maximal output

            normp = prpardnrm2 (  np, ru(nu+1), 1,
     +              ibuffr(ipinnercontr+nu) )
            write ( irefwr, 400 ) iter, norm, normu, normp,
     +              niteru, niterp
400         format ( i5, 3x, ss, 3(1pe10.2e2), 2i8 )

         end if

         if ( scaling ) then

!        --- scaling = true, reset ru

            ru(1:nu+np) = rucc

         end if  ! ( scaling )

         niterutot = niterutot+niteru
         niterptot = niterptot+niterp

!        --- In case of stokes and relative accuracy the convergence
!            may depend on the viscosity
!            To avoid that dependence we also have to check the norm
!            of the velocity part

         converged = norm<=eps

!        --- print *,epsu

         if ( converged .and. ireler==1 ) converged = normu<=epsu
         if ( converged ) go to 500
         if ( normu<rferr .and. iter>3 ) then

!        --- maximum accuracy has been reached

            call erreal ( rferr, 1 )
            call errwar ( 677, 0, 1, 0 )
            converged = .true.
            go to 500

         end if  ! ( normu< rferr )

      end do  ! k = 1, ngcr

!     --- If the subroutine reaches this stage, too much iterations have
!         been carried out

      call errint ( iter, 1 )
      if ( inpsol(22,1)==1) then
         inpsol(1,1) = iter
         call errwar ( 676, 1, 0, 0 )
      else
         call errsub ( 676, 1, 0, 0 )
      end if

500   niteracc = niteracc + iter
      niteruacc = niteruacc + niterutot
      niterpacc = niterpacc + niterptot

!     --- call prinrl (  u,nu+np , 'u' )

      if ( iprint>=1 ) then

!     --- iprint = 1  Print information

         write ( irefwr, 510 ) iter, norm, niterutot, niterptot,
     +                         niteracc, niteruacc, niterpacc
510      format('The number of executed iterations in the',
     +          ' GCR-process is', i5/
     +          'The 2-norm of the residual is:', ss,1pe12.2e2/
     +          'The number of u iterations is ', i6/
     +          'The number of p iterations is ', i6/
     +          'The accumulated number of GCR-iterations is', i8/
     +          'The accumulated number of u-iterations is', i8/
     +          'The accumulated number of p-iterations is', i8 )

      end if

!     --- Fill information in citerinfo
!         This is only used for performance checking

      niterout = niteracc
      niteruout = niteruacc
      niterpout = niterpacc
      call askclk
      timesubr = tim3-timebefore

1000  deallocate ( svec, vvec, rucc, diagpmm, stat = error )
      if ( error/= 0 ) then

!     --- error#0, problem with deallocate

         call errint ( error, 1 )
         call errchr ( 'work', 1 )
         call errsub ( 2911, 1, 0, 1 )

      end if  ! ( error/= 0 )

      call erclos ( 'prsimplegcr' )

      if ( debug ) then
         call prinrl ( u, nu, 'u' )
         call prinrl ( u(nu+1), np, 'p' )
         write(irefwr,*) 'End prsimplegcr'
      end if  ! ( debug )
      itime = itimesav

      end
