      subroutine prstorevec1 ( ibuffr, buffer, kmesh, kprob, isolut, &
                               ivecin, iseqvec, numvct )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-02-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create new vector by computing a vector expression
!
! **********************************************************************
!
!                       KEYWORDS
!
!     vector
!     expression
!     create
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/comcons2f90'
      include 'SPcommon/cpost5f90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: kmesh(*), kprob(*), ivecin
      integer, intent(out) :: iseqvec
      integer, intent(inout) :: ibuffr(*), numvct, isolut(5,*)
      double precision, intent(inout) :: buffer(*)

!     buffer        i/o   Buffer array
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     iseqvec        o    Starting address of sequence numbers of vectors
!     isolut        i/o   Standard SEPRAN array containing information of
!                         the solution vectors
!     ivecin         i    Sequence number of vector expression
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     kprob          i    Standard SEPRAN array, containing information of
!                         the problem
!     numvct        i/o   Largest sequence number of vector that has been filled
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: icount, istore(100), vecs(100), numvecrhs, numrealrhs, &
                 iprob, ivec, itypvc, indprf, indprp, nphys, lenvec
      double precision :: store(100)
      logical :: debug, complexvec

!     complexvec     If true the vector to be filled is complex
!     debug          If true debug statements are carried out otherwise
!                    they are not
!     icount         Counter
!     indprf         indication if kprob part f has been filled (>0)
!                    or not (0)
!     indprp         Indication if array kprob part p has been filled (1)
!                    or not (0)
!     iprob          Actual problem number
!     istore         Contains information of the expression
!                    See subroutine READEXPR, array iwork
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     ivec           Sequence number of vector of special structure
!     lenvec         Length of left-hand side vector in expression
!     nphys          Number of physical unknowns, or maximal degrees of
!                    freedom per point
!     numrealrhs     Number of reals stored in store
!     numvecrhs      Number of vectors in right-hand side
!                    Length of array vecs
!     store          Contains real information of the expression
!                    See subroutine READEXPR, array work
!     vecs           In this array of size numvecrhs the sequence numbers
!                    of the right-hand side vectors are stored
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     INI086         compute pointers to degrees of freedom of vectors
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRVECCHECKLEN  Check if all vectors at right-hand side have the same
!                    length as the vector on the right-hand side
!     PRVECCOMBINAT1 Evaluate right-hand side of vector expression
!                    Complete real vector
!     PRVECFIND1     Find the type of vector to be created for the left-hand side
!                    This depends on the right-hand side
!     PRVECSTRHS     Translate expression to a set of reals and a new set of
!                    temporary vectors
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     222   input vector is complex:  not yet allowed.
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prstorevec1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prstorevec1'
         write(irefwr,1) 'ivecin', ivecin
  1      format ( a, 1x, (10i10) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Translate expression to a set of reals and a new set of
!         temporary vectors

      call prvecstrhs ( ivecin-200000000, numvecrhs, numrealrhs, &
                        icount, istore, store, vecs )
      call inichkarlen ( icount, 100, 'istore' )
      call inichkarlen ( numrealrhs, 100, 'store' )
      call inichkarlen ( numvecrhs, 100, 'vecs' )
      if ( ierror/=0 ) go to 1000

      if ( debug ) then
         write(irefwr,1) 'numvecrhs, numrealrhs, icount', &
                          numvecrhs, numrealrhs, icount
         call prinin ( istore, icount, 'istore' )
         call prinrl ( store, numrealrhs, 'store' )
         call prinin ( vecs, numvecrhs, 'vecs' )
      end if  ! ( debug )

!     --- find vector depending on right-hand side

      numvct = numvct+1
      iseqvec = numvct  ! new temporary vector

      if ( debug ) write(irefwr,1) 'numvct, iseqvec', numvct, iseqvec

      call prvecfind1 ( ibuffr, kmesh, kprob, iprob, ivec, itypvc, &
                        iseqvec, vecs, isolut, numvecrhs, complexvec )
      if ( debug ) write(irefwr,*) 'complexvec/2', complexvec

      if ( complexvec ) then

!     --- Error: complex vector

         call errsub ( 222, 0, 0, 0 )
         go to 1000

      end if  ! ( complexvec )

!     --- create temporary vector

      call ini086 ( ibuffr, isolut(1,iseqvec), itypvc, ivec, iprob, &
                    kmesh, kprob, indprf, indprp, nphys, &
                    names(iseqvec) )

!     --- Check if all vectors at the right-hand side have the same length
!         as the left-hand side

      call prvecchecklen ( isolut, iseqvec, vecs, numvecrhs, kmesh, &
                           nphys, .true. )

!     --- Compute length of vector

      lenvec = isolut(5,iseqvec)

!     --- Compute vector

      call prveccombinat1 ( ibuffr, buffer, isolut, iseqvec, vecs, &
                            numvecrhs, kmesh, lenvec, istore, &
                            store, icount, numrealrhs, nphys )

1000  call erclos ( 'prstorevec1' )
      if ( debug ) then

!     --- Debug information

         call prinin ( isolut(1,iseqvec), 5, 'isol_temp' )
         write(irefwr,*) 'End prstorevec1'

      end if  ! ( debug )

      end subroutine prstorevec1
