      subroutine prveccombinat3 ( ibuffr, buffer, isol, iseqvec, vecs, &
                                  numvecrhs, kmesh, kprob, &
                                  istore, store, icount, numrealrhs, &
                                  inpsep, indprf )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 17-01-2015 new options implemented
!        version  1.1  date 11-03-2014 new call to prveccombinat4
!        version  1.0  date 23-12-2013
!
!   copyright (c) 2013-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Evaluate right-hand side of vector expression
!     Special case: a part of the real vector must be computed
! **********************************************************************
!
!                       KEYWORDS
!
!     manipulation
!     vector
!     expression
!     real
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/comcons2f90'
      include 'SPcommon/cpost5f90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), isol(5,*), iseqvec, vecs(*), numvecrhs, &
              kmesh(*), istore(*), icount, numrealrhs, &
              kprob(*), inpsep(*), indprf
      double precision buffer(*), store(*)

!     buffer        i/o   Buffer array
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     icount         i    Computed length of array istore
!     indprf         i    indication if kprob part f has been filled (>0)
!                         or not (0)
!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine ASKVEC
!     iseqvec        i    sequence number of vector in left-hand side with
!                         to isol
!     isol           i    Standard SEPRAN array containing information of the
!                         respect solution array
!     istore         i    Contains the expression as stored in common
!                         cexpression, where all scalars and reals
!                         have been computed and stored in store
!                         References to these values are given by
!                         their position in store plus 1000
!                         The values of the vectors are stored in
!                         array vectors
!                         References to these vectors are given by
!                         their position in vectors plus 10000
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     kprob          i    Standard SEPRAN array, containing information of
!                         the problem
!     numrealrhs     i    Number of reals stored in store
!     numvecrhs      i    Number of vectors in right-hand side
!                         Length of array vecs
!     store          i    Contains real information of the expression
!                         See subroutine READEXPR, array work
!     vecs           i    In this array of size numvecrhs the sequence numbers
!                         of the right-hand side vectors are stored
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer npoint, error, nnodes
      integer, allocatable :: nodes(:)
      logical debug, allnodes

!     allnodes       If true all nodes of themesh must be used
!     debug          If true debug statements are carried out otherwise
!                    they are not
!     error          Return value of allocate or deallocate
!     nnodes         Number of nodes
!     nodes          Contains the node numbers that must be filled
!     npoint         Number of nodal points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRVECCOMBINAT4 Evaluate right-hand side of vector expression
!                    a part of the real vector must be computed
!                    the number of unknowns per point is constant
!     PRVECCOMBINAT5 Evaluate right-hand side of vector expression
!                    a part of the real vector must be computed
!                    the number of unknowns per point is constant
!     PRVECCOMBINAT6 Evaluate right-hand side of vector expression
!                    a part of the real vector must be computed
!                    the number of unknowns per point is variable
!     PRVECFILLNODS  Fill array nodes with the nodes that must be used in the
!                    creation of a vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3162   Option not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prveccombinat3' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prveccombinat3'
         write(irefwr,1) 'indprf', indprf
         call prinin ( inpsep, 20, 'inpsep' )
   1     format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Check if nodal points must be filled or not

      allnodes = inpsep(14)==0 .and. inpsep(15)==0

      if ( debug ) write(irefwr,*) 'allnodes ', allnodes

      if ( allnodes ) then

!     --- All nodes are used

         if ( indprf==0 ) then

!        --- The number of degrees of freedom in each node is the same

            call prveccombinat5 ( ibuffr, buffer, isol, iseqvec, vecs, &
                                  numvecrhs, kmesh, kprob, inpsep(5), &
                                  istore, store, icount, numrealrhs )

         else

!        --- The number of degrees of freedom is variable

            call errsub (3162, 0, 0, 0 )
            go to 1000

         end if  ! ( indprf==0 )

      else

!     --- Not all nodes are used
!         Fill array nodes with the nodes that must be used

         npoint = kmesh(8)
         allocate ( nodes(npoint), stat = error )
         if ( error/=0 ) call eralloc ( error, npoint, 'nodes' )
         if ( ierror/=0 ) go to 1000

         call prvecfillnods ( ibuffr, kmesh, kprob, nodes, nnodes, &
                              inpsep )

         if ( indprf==0 ) then

!        --- The number of degrees of freedom in each node is the same

            call prveccombinat4 ( ibuffr, buffer, isol, iseqvec, vecs, &
                                  numvecrhs, kmesh, kprob, inpsep(5), &
                                  istore, store, icount, numrealrhs, &
                                  nodes, nnodes )

         else

!        --- The number of degrees of freedom is variable

            call prveccombinat6 ( ibuffr, buffer, isol, iseqvec, vecs, &
                                  numvecrhs, kmesh, kprob, inpsep(5), &
                                  istore, store, icount, numrealrhs, &
                                  nodes, nnodes )

         end if  ! ( indprf==0 )

         deallocate ( nodes, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'nodes' )

      end if  ! ( allnodes )

1000  call erclos ( 'prveccombinat3' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prveccombinat3'

      end if  ! ( debug )

      end
