      subroutine prvecfill4 ( ibuffr, buffer, isol, iseqvec, vecs, &
                              numvecrhs, kmesh, vectors, length, &
                              kprob, nunkp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-01-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill all right-hand side vectors in expression where a fixed
!     number of degrees of freedom per point must be filled
!     whole region
! **********************************************************************
!
!                       KEYWORDS
!
!     vector
!     expression
!     right_hand_side
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/comcons2f90'
      include 'SPcommon/cpost5f90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), isol(5,*), iseqvec, vecs(*), numvecrhs, &
              kmesh(*), length, kprob(*), nunkp
      double precision buffer(*), vectors(length,numvecrhs)

!     buffer        i/o   Buffer array
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     iseqvec        i    sequence number of vector in left-hand side with
!                         to isol
!     isol           i    Standard SEPRAN array containing information of the
!                         solution arrays
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     kprob          i    Standard SEPRAN array, containing information of
!                         the problem
!     length         i    Length of the vectors to be filled
!     numvecrhs      i    Number of vectors in right-hand side
!                         Length of array vecs
!     nunkp          i    Maximum number of unknowns in a point
!     vecs           i    In this array of size numvecrhs the sequence numbers
!                         of the right-hand side vectors are stored
!     vectors        o    Array of size length x numvecrhs to store the vectors
!                         of the right-hand side in the nodes required
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, ndim, iseqv, indprf, nunkpv, indprp, nphys, npoint
      integer (kind=8) :: ipvec
      logical :: debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     indprf         indication if kprob part f has been filled (>0)
!                    or not (0)
!     indprp         Indication if array kprob part p has been filled (1)
!                    or not (0)
!     ipvec          Starting address of output vector in buffer
!     iseqv          Sequence number of vector in isol
!     ndim           Dimension of the space.
!     nphys          Number of physical unknowns, or maximal degrees of
!                    freedom per point
!     npoint         Number of nodal points
!     nunkpv         Number of unknowns in right-hand-side vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) :: inidgttext

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INI060TEXT     Reactivate existing array in IBUFFR
!     INI061         compute pointers to degrees of freedom of vectors
!     INIDGTTEXT     Compute pointer of array in BUFFER
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3162   Option not yet implemented
!    3163   Vector has wrong sequence number
!    3166   Number of degrees of freedom in rhs vector less than in rhs vector
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prvecfill4' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prvecfill4'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get parameters

      ndim = kmesh(6)           ! dimension of space
      npoint = kmesh(8)         ! number of nodes

!     --- Loop over vectors

      do i = 1, numvecrhs

         iseqv = vecs(i)     ! sequence number of vector
         if ( iseqv<=maxvc1 ) then

!        --- Standard sepran vector
!            activate array

            call ini061 ( ibuffr, isol(1,iseqv), kprob, indprf, indprp, &
                          nunkpv, nphys )

            if ( nunkpv<nunkp ) then

!           --- Error nunkpv too small

               call errint ( i, 1 )
               call errint ( nunkpv, 2 )
               call errint ( nunkp, 3 )
               call errchr ( names(iseqv), 1 )
               call errchr ( names(iseqvec), 2 )
               call errsub ( 3166, 3, 0, 2 )
               go to 1000

            end if  ! ( nunkpv<nunkp )

!           --- Find starting address

            ipvec = inidgttext ( isol(1,iseqv), names(iseqv) )

!           --- Copy vector

            if ( indprf==0 ) then

!           --- indprf = 0, number of degrees of freedom fixed
!               Copy first degree of freedom into vectors(i)

               vectors(1:npoint*nunkp,i) = &
                  buffer(ipvec:ipvec+npoint*nunkp-1)

            else

!           --- indprf>0, number of degrees of freedom variable

               call errsub (3162, 0, 0, 0 )
               go to 1000

           end if  ! ( indprf==0 )

         else if ( iseqv==maxvc1+1 ) then

!        --- Coordinates
!            activate array

            call ini060text ( ibuffr, kmesh(23), 'coor' )
            if ( ndim<nunkp ) then

!           --- Error ndim too small

               call errint ( i, 1 )
               call errint ( ndim, 2 )
               call errint ( nunkp, 3 )
               call errchr ( 'coor', 1 )
               call errchr ( names(iseqvec), 2 )
               call errsub ( 3166, 3, 0, 2 )
               go to 1000

            end if  ! ( nunkpv<nunkp )

!           --- Find starting address

            ipvec = inidgttext ( kmesh(23), 'coor' )

!           --- Copy vector

            vectors(1:npoint*ndim,i) = &
                  buffer(ipvec:ipvec+ndim*npoint-1)

         else if ( iseqv<maxvc1+1+ndim ) then

!        --- One of the coordinates
!            activate array

            iseqv = iseqv-maxvc1-1
            call ini060text ( ibuffr, kmesh(23), 'coor' )
            if ( 1<nunkp ) then

!           --- Error ndim too small

               call errint ( i, 1 )
               call errint ( 1, 2 )
               call errint ( nunkp, 3 )
               call errchr ( 'coor_x', 1 )
               call errchr ( names(iseqvec), 2 )
               call errsub ( 3166, 3, 0, 2 )
               go to 1000

            end if  ! ( nunkpv<nunkp )

!           --- Find starting address

            ipvec = inidgttext ( kmesh(23), 'coor' )

!           --- Copy vector

            vectors(1:npoint,i) = &
                  buffer(ipvec:ipvec+npoint-1)

         else

!        --- Unknown vector

            call errint ( iseqv, 1 )
            call errchr ( names(iseqvec), 1 )
            call errsub ( 3163, 1, 0, 1 )
            go to 1000

         end if

      end do  ! i = 1, numvecrhs

1000  call erclos ( 'prvecfill4' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prvecfill4'

      end if  ! ( debug )

      end
