      subroutine readcompute ( istrf, ivecnm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 20-04-2012 Extra debug
!        version  2.0  date 29-09-2008 Extra parameter ivecnm
!        version  1.6  date 12-03-2008 New way of defining scalars
!
!   copyright (c) 1999-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Help subroutine for the program SEPPOST.
!   This subroutine reads the input after the keyword compute until one of
!   the keywords indicating the type of compute statement is found
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     post_processing
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/comcons2f90'
      include 'SPcommon/csepchf90'
      include 'SPcommon/cseprdf90'
      include 'SPcommon/cseprdnewf90'
      include 'SPcommon/cpost1f90'
      include 'SPcommon/cpost5f90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istrf, ivecnm

!     istrf          i    Vector number read
!     ivecnm        i/o   vector sequencer number with respect to isol
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nncode
      parameter ( nncode=11 )
      character (len=19) code(nncode)
      integer ivec, ncode(nncode), ierpri, lcode, ncodmx(nncode), ispecs
      logical debug, readvec

!     code           Table with keywords
!     debug          If true debug statements are carried out otherwise
!                    they are not
!     ierpri         Indicates if intx00 must give an error message (1) or
!                    not (0)
!     ispecs         Help variable to save the value of ispecl
!     ivec           Sequence number of vector of special structure
!     lcode          return code produced by intx00
!     ncode          Table containing the number of significant characters
!                    in the keyword table code
!     ncodmx         Table containing the maximal number of  characters in
!                    the keyword table code
!     nncode         Number of entries in code
!     readvec        if true vector number must be read
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer intx00

!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INTX00         Compare text in string with table (variable length)
!     PRGETNAME      Get sequence number of vector with specific name
!     PRSETNAME      Set sequence number of vector with specific name if it
!                    does not exist otherwise get sequence number
!     READNAMEVEC    Read name of vector in common cpost5
!     READVECTNUM    Read vector name or Vi or V%vectorname
! **********************************************************************
!
!                       I/O
!
!     The subroutine reads from the standard input file
!     The input after compute is interpreted, i.e.
!
!     compute [Vi] = stream
!     compute  Vi  = velocity
!     compute  Vi  = intersection
!     compute [Vi] = mach_number
!     compute [Vi] = total_enthalpy
!     compute [Vi] = stagnation_pressure
!     compute [Vi] = scaled_pressure
!     compute [Vi] = entropy
!     compute  Vi  = function_of
!     compute  Vi  = interpolation
!     compute  Vi  = normal_stress
!
!     The rest of the input is interpreted by read06
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     672   The number of vectors or constants per time step to be created
!           is too large
!     712   Wrong letter in the part PLANE(..X+..Y+..Z=..)
!     716   No number found after COMPUTE C or COMPUTE V
!     717   The number j corresponding to the statement COMPUTE Cj or
!           COMPUTE Vj is out of range (<0 or>1000 )
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Take next item
!    If ( next item = 'V' ) then
!       take next item
!       if ( next item is not integer number ) then
!          error
!    if ( next item is integer number ) then
!       Set istrf and check this parameter
!       take next item
!    else ( next item not an integer )
!       istrf := -1
!    if ( next item is a keyword ) then
!       Find the corresponding keyword
!       Fill iinput
!       if ( istrf = -1 ) then
!          Give istrf a value depending on type of keyword
!    else
!       error, not a keyword found after compute [Vj]
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code / 'STREAM', 'VELOCITY', 'INTERSECTION', 'MACH_NUMBER', &
                  'TOTAL_ENTHALPY', 'STAGNATION_PRESSURE', &
                  'SCALED_PRESSURE', 'ENTROPY', 'FUNCTION_OF', &
                  'INTERPOLATION', 'NORMAL_STRESS' /
      data ncode  / 6, 8,12,11,14, 19,15, 7,11,13, &
                   13 /
      data ncodmx / 6, 8,12,11,14, 19,15, 7,11,13, &
                   13 /

!    code     Table with keywords
!              Contents
!              1:  STREAM
!              2:  VELOCITY
!              3:  INTERSECTION
!              4:  MACH_NUMBER
!              5:  TOTAL_ENTHALPY
!              6:  STAGNATION_PRESSURE
!              7:  SCALED_PRESSURE
!              8:  ENTROPY
!              9:  FUNCTION_OF
!             10:  INTERPOLATION
!             11:  NORMAL_STRESS
! ======================================================================
!
      call eropen ( 'readcompute' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readcompute'
         write(irefwr,1) 'ityprd, lastitem/1', ityprd, lastitem
         write(irefwr,1) 'numvec/1', numvec
         if ( ityprd==4 ) write(irefwr,*) 'string ', string(1:nchar)
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Take next item

      istrf = -1
      ispecs = ispecl
      ispecl = 11
      call asknxt
      if ( debug ) then
         write(irefwr,1) 'ityprd, nchar, lastitem/2', &
                          ityprd, nchar, lastitem
         write(irefwr,1) 'numvec/2', numvec
         if ( ityprd==4 ) write(irefwr,*) 'string ', string(1:nchar)
      end if  ! ( debug )

      if ( ityprd==4 ) then

!     --- word found, check if it is V

         if ( nchar==1 .and. string(1:1)=='V' ) then

!        --- next item = 'V'
!            take next item

            if ( debug ) write(irefwr,*) 'V found'
            call asknxt
            if ( debug ) then
               write(irefwr,1) 'ityprd, lastitem/2a', ityprd, lastitem
               write(irefwr,1) 'numvec/3', numvec
            end if  ! ( debug )

            if ( ityprd/=2 ) then

!           --- Not a number found after V

               call errsub ( 716, 0, 0, 0 )
               call errcrd
               go to 1000

            end if

         else

!        --- Check if it is a main keyword

            ierpri = 0
            lcode = intx00 ( code, ncode, nncode, ierpri, ncodmx )
            if ( debug ) write(irefwr,1) 'lcode', lcode

            if ( lcode==0 .and. ityprd==4 ) then

!           --- Interprete this string as a new vector

               lastitem = lastitembef      ! reread, since name may contain
                                           ! a number
               ispecl = 9                  ! set ispecl in order to include
                                           ! number
               call asknxt                 ! read again

               if ( debug ) then
                  write(irefwr,1) 'ityprd, lastitem/3', ityprd, lastitem
                  write(irefwr,*) 'string ', string(1:nchar)
                  write(irefwr,1) 'numvec/4', numvec
               end if  ! ( debug )

               call readnamevec    ! first check if it is a vector name

               if ( debug ) then
                  write(irefwr,1) 'numvec/5', numvec
               end if  ! ( debug )

               ispecl = 11         ! reset ispecl

               if (  ityprd==4 ) then

!              --- Interprete this string as a new vector

                  call prsetname ( string(1:nchar), istrf )
                  call asknxt  ! read new item
                  if ( debug ) then
                     write(irefwr,1) 'istrf, ityprd, lastitem/3', &
                                      istrf, ityprd, lastitem
                     write(irefwr,1) 'numvec/6', numvec
                  end if  ! ( debug )

               end if  ! (  ityprd==4 )

            end if  ! ( lcode==0 )

         end if  ! ( nchar==1 .and. string(1:1)=='V' )

      end if

      if ( ityprd==2 ) then

!     --- next item is integer number
!         Set istrf and check this parameter

         istrf = nint(realrd)
         if ( istrf<0 .or. istrf>1000 ) then

!        --- istrf out of range

            call errint ( istrf, 1 )
            call errint ( 1000, 2 )
            call errsub ( 717, 2, 0, 0 )
            call errcrd
            goto 1000

         end if
         if ( irefar(istrf)==0 ) then

!        --- irefar(istrf) = 0, so a new vector must be created

            numvec = numvec + 1
            if ( numvec>maxvc1 ) then

!           --- Too much vectors per time step

               call errint ( numvec, 1 )
               call errint ( maxvc1, 2 )
               call errsub ( 672, 2, 0, 0 )
               call errcrd
               goto 1000
            end if
            irefar(istrf) = numvec

         end if
         if ( debug ) write(irefwr,1) 'numvec/7', numvec

!        --- take next item

         call asknxt

      end if
      if ( debug ) write(irefwr,1) 'numvec/8', numvec

      if ( ityprd==4 ) then

!     --- next item is a keyword
!         Find the corresponding keyword
!         Fill iinput
!         if ( istrf = -1 ) then
!            Give istrf a value depending on type of keyword

         ierpri = 1
         lcode = intx00 ( code, ncode, nncode, ierpri, ncodmx )
         if ( debug ) write(irefwr,1) 'lcode', lcode
         readvec = .false.

         select case ( lcode )

            case (0)

!           --- incorrect keyword found

               go to 1000

            case (1)

!           --- Compute Vj = Stream function  Vi found

               iinput(1) = 203
               call asknxt  ! read next item
               if ( ityprd/=4 .or. string(1:nchar)/='FUNCTION' ) &
                  lastitem = lastitembef    ! next item # FUNCTION
               if ( istrf==-1 ) &
                  call prsetname ( 'STREAM_FUNCTION', istrf )
               readvec = .true.  ! Vj must be read
               if ( debug ) write(irefwr,1) 'numvec/9', numvec

            case (2, 3)

!           --- Compute Vj = Velocity profile Vi
!               or Compute Vj = INTERSECTION Vi found

               iinput(1) = 201
               readvec = .true.  ! Vj must be read

            case (4)

!           --- Compute Vj = MACH_NUMBER found
!               Set default values for velocity and enthalpy vector

               iinput(1) = 204
               ivec = 5
               call prgetname ( 'VELOCITY', ivec )
               iinput(2) = irefar(ivec)
               ivec = 3
               call prgetname ( 'ENTHALPY', ivec )
               iinput(3) = irefar(ivec)
               if ( istrf==-1 ) &
                  call prsetname ( 'MACH_NUMBER', istrf )
               if ( debug ) write(irefwr,1) 'numvec/10', numvec

            case (5)

!           --- Compute Vj = TOTAL_ENTHALPY found
!               Set default values for velocity and enthalpy vector

               iinput(1) = 205
               ivec = 5
               call prgetname ( 'VELOCITY', ivec )
               iinput(2) = irefar(ivec)
               ivec = 3
               call prgetname ( 'ENTHALPY', ivec )
               iinput(3) = irefar(ivec)
               if ( istrf==-1 ) &
                  call prsetname ( 'TOTAL_ENTHALPY', istrf )
               if ( debug ) write(irefwr,1) 'numvec/11', numvec

            case (6)

!           --- Compute Vj = STAGNATION_PRESSURE found
!               Set default values for pressure and mach_number vector

               iinput(1) = 206
               ivec = 2
               call prgetname ( 'PRESSURE', ivec )
               iinput(2) = irefar(ivec)
               ivec = 10
               call prgetname ( 'MACH_NUMBER', ivec )
               iinput(3) = irefar(ivec)
               if ( istrf==-1 ) &
                  call prsetname ( 'STAGNATION_PRESSURE', istrf )
               if ( debug ) write(irefwr,1) 'numvec/12', numvec

            case (7)

!           --- Compute Vj = SCALED_PRESSURE found
!               Set default values for pressure

               iinput(1) = 207
               ivec = 2
               call prgetname ( 'PRESSURE', ivec )
               iinput(2) = irefar(ivec)
               if ( istrf==-1 ) &
                  call prsetname ( 'SCALED_PRESSURE', istrf )
               if ( debug ) write(irefwr,1) 'numvec/13', numvec

            case (8)

!           --- Compute Vj = ENTROPY found
!               Set default values for pressure and density

               iinput(1) = 208
               ivec = 2
               call prgetname ( 'PRESSURE', ivec )
               iinput(2) = irefar(ivec)
               ivec = 4
               call prgetname ( 'DENSITY', ivec )
               iinput(3) = irefar(ivec)
               if ( istrf==-1 ) &
                  call prsetname ( 'ENTROPY', istrf )
               if ( debug ) write(irefwr,1) 'numvec/14', numvec

            case (9)

!           --- Compute Vj = Function_of Vi

               iinput(1) = 225
               if ( istrf==-1 ) &
                  call prsetname ( 'NORMAL_STRESS', istrf )
               readvec = .true.  ! Vj must be read
               if ( debug ) write(irefwr,1) 'numvec/15', numvec

            case (10)

!           --- Compute Vj = INTERPOLATION Vi

               iinput(1) = 209
               readvec = .true.  ! Vj must be read

            case (11)

!           --- Compute Vj = NORMAL_STRESS

               iinput(1) = 210
               readvec = .true.  ! Vj must be read

          end select  ! case ( lcode )

          if ( readvec .and. typeitems(lastitem+1)==3 ) then

!         --- readvec = true, hence the vector from which we have to compute
!             must be read and stored in ivecnm
!             The next item is a string

             call readvectnum ( ivecnm )
             if ( debug ) write(irefwr,1) 'numvec/16', numvec

          end if  ! ( readvec )

      else

!     --- error, not a keyword found after compute [Vj]

         call errchr ( string(1:1), 1 )
         call errsub ( 712, 0, 0, 1 )
         call errcrd
         go to 1000

      end if
      iinput(16) = irefar(istrf)
      ispecl = ispecs

1000  call erclos ( 'readcompute' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'numvec', numvec
         write(irefwr,*) 'End readcompute'

      end if  ! ( debug )

      end
