      subroutine readscbouncond ( ibuffr, buffer, istart, inpsep,
     +                            tabout, icode, nout, strout, nprob,
     +                            numvec, rinsep, istrin, kmesh )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 26-06-2011 extension with zero_level_set
!        version  1.0  date 04-10-2010
!
!   copyright (c) 2010-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read the input following the following standard keywords with respect
!     to subroutine readsc
!             1:  PRESCRIBE_BOUNDARY_CONDITIONS
!     The input is stored in array inpsep and the real array rinsep
!     If this subroutine is adapted also adapt ini016
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     output
!     boundary_condition
!     change_boundary
!     capacity
!     inverse_problem
!     pressure_correction
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpi'
      include 'SPcommon/cconst'
      include 'SPcommon/comcons1'
      include 'SPcommon/comcons2'
      include 'SPcommon/cuscons'
      include 'SPcommon/cusname'
      include 'SPcommon/cpost5'
      include 'SPcommon/cseprdnew'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*),  istart, inpsep(*), icode, nout, nprob, numvec,
     +        istrin, kmesh(*)
      double precision buffer(*), tabout(*), rinsep(*)
      character (len=*) strout(*)

!     buffer        i/o   Buffer array
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     icode          i    Indicates which of the options in readsc is found
!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         This part contains:
!                         2  ISEQNR   Sequence number corresponding to input
!                                     If iseqnr = -10, position 5 ... are used
!                         3  IPROB    Problem sequence number
!                                     If iprob = -1, the next one will be used
!                         4  IVECT    Vector sequence number (first vector)
!                                     If ivector = -1, the same value as iprob
!                                     will be used
!                         5  ICOMPL   If 0, then real else complex
!                         6  IFUNC    Function sequence number with respect to
!                                     subroutine FUNBC
!                                     If 0 no filling takes place
!                                     If <0 the value is given stored in
!                                     RINSEP(-IFUNC)
!                         7  IPLACE   Position where boundary conditions
!                                     are given
!                                     0:  whole domain
!                                     1:  points
!                                     2:  curves
!                                     3:  surfaces
!                         8  INUM     Number of curves
!                         9  IPHYS    Physical quantity
!                        10  IVEC     Defines type of output vector
!                        11-11+INUM   Curve numbers
!     istart        i/o   Next free position in inpsep
!     istrin        i/o   Starting address of information for this subroutine
!                         in RINSEP
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     nout           i    Number of entries in tabout
!     nprob          i    Number of various problems
!     numvec        i/o   Number of vectors stored in isol
!     rinsep        i/o   Array containing the real input for this subroutine
!     strout        i/o   Contains series of strings
!     tabout        i/o   Contains series of numbers
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer errorcode, istartbef, istrinbef
      logical debug, outfnd

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
!       PRESCRIBE_BOUNDARY_CONDITIONS, options (all in one line)
!           where options may have the following contents
!               SEQUENCE_NUMBER = s  (Default the next one)
!               VECTOR = s           (Default the same as problem)
!               PROBLEM = s          (Default 1)
!               VALUE = v            (No default)
!               FUNC = f             (No default)
!               QUADRATIC, MAX = a, MEAN_VALUE = v, BOUNDARY_LAYER_WIDTH = b,
!                 DIFFERENCE = delta_T
!               HALF_QUADRATIC, MAX = a
!               POINTS = (Pi, Pj, Pk )
!               CURVES = (Ci, Cj, Ck )
!               SURFACES = (Si, Sj, Sk )
!               ZERO_LEVELSET i
!               OLD_VECTOR = m
!               SEQ_VECTORS = V1, V2, ...
!      In case of CREATE_VECTOR, extra
!               OUTPUT_VECTOR = ivec
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readscbouncond' )
      debug = .true.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readscbouncond'
         write(irefwr,1) 'istart, istrin', istart, istrin
         call prinin ( typeitems, nitems, 'typeitems' )
         write(irefwr,1) 'lastitem, istart', lastitem, istart
         istartbef = istart
         istrinbef = istrin
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- First we read the standard keywords and check if there are
!         no other keywords

      errorcode = 1   ! return when an unknown keyword is found
      call readscstandard ( istart, inpsep, tabout, outfnd, icode,
     +                      nout, strout, nprob, numvec, errorcode )
      if ( debug )
     +   write(irefwr,1) 'errorcode, lastitem', errorcode, lastitem

      if ( errorcode<0 ) then

!     --- errorcode = -1, hence other keyword found
!         read rest of keywords with standard routine

         call readscboun1 ( ibuffr, buffer, icode, kmesh, inpsep,
     +                      istart, rinsep, istrin, nprob )

      end if  ! ( errorcode<0 )

1000  call erclos ( 'readscbouncond' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'istart', istart
         call prinin ( inpsep(istartbef), istart-istartbef, 'inpsep' )
         if ( istrin>istrinbef )
     +     call prinrl ( rinsep(istrinbef), istrin-istrinbef, 'rinsep' )
         write(irefwr,*) 'End readscbouncond'

      end if  ! ( debug )

      end
