      subroutine msh480 ( kbound, nbound, line, nlind, coor, ivolnr, &
                          shift, iswitch )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  5.0  date 06-12-2013 Extra parameter ivolnr
!        version  4.1  date 10-01-2011 Add variable icheck to header
!        version  4.0  date 03-09-2010 New release
!
!   copyright (c) 1992-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subroutine to check whether a line of the boundary intersects
!     one of the other triangles. If so then exit routine with
!     an error message and stop the program.
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     intersection
!     boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nbound, kbound(3,nbound), line(*), nlind, ivolnr, iswitch
      double precision coor(3,*), shift(4)

!     coor       i    array of length ndim x npoint containing the co-ordinates
!                     of the nodal points with respect to the global numbering
!     iswitch    i    if 0 error message 3066 else 3067
!     ivolnr     i    Volume sequence number
!     kbound     i    actual array with boundary triangles
!     line            help array to store the lines of the boundary
!     nbound     i    number of triangles in kbound
!     nlind           declared length of array line
!     shift      i    Contains the vector xmint, ymint, zmint and tran
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ia, ib, il, k1, k2, k3, ks, nline, ncheck, i, icheck
      double precision xmin, xmax, ymin, ymax, zmin, zmax, t, x, &
              xa, ya, za, xb, yb, zb, &
              x1, x2, x3, y1, y2, y3, z1, z2, z3

!     debug      if true debug statements are carried out otherwise
!     i          Counting variable
!     ia         begin point line
!     ib         end point of line
!     icheck     if 0, no error found, if 1 error.
!     il         line loop number
!     k1         triangle node
!     k2         triangle node
!     k3         triangle node
!     ks         loop variable
!     ncheck     variable to store length of array line used
!     nline      number of lines stored in array line
!     t          parameter for crossing point
!     x          help parameter to store backtransformed coordinate
!     x1         x-coordinate
!     x2         x-coordinate
!     x3         x-coordinate
!     xa         x-coordinate first point line
!     xb         x-coordinate last point line
!     xmax       help variable for surrounding cube
!     xmin       help variable for surrounding cube
!     y1         y-coordinate
!     y2         y-coordinate
!     y3         y-coordinate
!     ya         y-coordinate
!     yb         y-coordinate
!     ymax       y-coordinate
!     ymin       y-coordinate
!     z1         z-coordinate
!     z2         z-coordinate
!     z3         z-coordinate
!     za         z-coordinate
!     zb         z-coordinate
!     zmax       z-coordinate
!     zmin       z-coordinate
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSH480EXT      perform extra check on intersection of line and triangle
!     MSH610         Determine intersection of triangle and line
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     928   Length help array not enough
!    3066   Boundary intersects itself
!    3067   Boundary intersects itself with internal surfaces or curves
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Determine all lines in boundary
!     Run through all triangles and check for all lines
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen( 'msh480' )
      if ( ierror/=0 ) go to 1000

!     --- Set debugvalue:

      debug = .false.

!     --- Fill array line with all lines in boundary
!         (It is assumed that all lines are double in the boundary
!          and that all triangles are ordered, hence one line k1-k2 in
!          a boundary triangle is always coupled to a line k2-k1 in another
!          boundary triangle. See msh637)

      icheck = 0
      nline  = 0

!     --- Run through all triangles:

      do ks = 1 , nbound
           k1 = kbound( 1, ks )
           k2 = kbound( 2, ks )
           k3 = kbound( 3, ks )
           if ( debug ) &
              write(irefwr,*) 'msh480: Triangle',ks,'nodes',k1,k2,k3
           if ( k1>k2 ) then
                line(2*nline+1) = k1
                line(2*nline+2) = k2
                nline = nline + 1
           end if
           if ( k2>k3 ) then
                line(2*nline+1) = k2
                line(2*nline+2) = k3
                nline = nline + 1
           end if
           if ( k3>k1 ) then
                line(2*nline+1) = k3
                line(2*nline+2) = k1
                nline = nline + 1
           end if
      end do

!     --- Length of array line enough ?

      ncheck = 2 * nline

!     --- Give info:

      if ( debug ) then
         write(irefwr,*) 'In msh480 line length needed =',ncheck
      end if

!     --- Check:

      if ( ncheck>nlind ) then
           call errint( ncheck , 1 )
           call errint( nlind  , 2 )
           call errint( 0  , 3 )
           call errsub ( 928, 3, 0, 0 )
           go to 1000
      end if

!     --- Check whether all lines have no points in common with
!         boundary triangles
!         Run through all lines skip all lines outside "block" , and
!         check for all the boundary triangles

      do ks = 1 , nbound
         k1 = kbound( 1, ks )
         k2 = kbound( 2, ks )
         k3 = kbound( 3, ks )

!        --- Node k1

         x1 = coor( 1, k1 )
         y1 = coor( 2, k1 )
         z1 = coor( 3, k1 )

!        --- Node k2

         x2 = coor( 1, k2 )
         y2 = coor( 2, k2 )
         z2 = coor( 3, k2 )

!        --- Node k3

         x3 = coor( 1, k3 )
         y3 = coor( 2, k3 )
         z3 = coor( 3, k3 )

!        --- Determine the extreme values for the surrounding cube

         xmin = min( x1, x2, x3 )
         ymin = min( y1, y2, y3 )
         zmin = min( z1, z2, z3 )
         xmax = max( x1, x2, x3 )
         ymax = max( y1, y2, y3 )
         zmax = max( z1, z2, z3 )

!        --- Check all lines from array line:

         do il = 1 , nline

!        --- Nodes:

            ia = line(2*il-1)
            ib = line(2*il  )

!           --- Compare:

            if ( ia/=k1 .and. ia/=k2 .and. ia/=k3 .and. &
                 ib/=k1 .and. ib/=k2 .and. ib/=k3 ) then

!           --- Coordinates Node A:

               xa = coor( 1,ia )
               ya = coor( 2,ia )
               za = coor( 3,ia )

!              --- Node B:

               xb = coor( 1,ib )
               yb = coor( 2,ib )
               zb = coor( 3,ib )

!              --- Intersection?

               if ( min(xa,xb)<=xmax .and. &
                    max(xa,xb)>=xmin .and. &
                    min(ya,yb)<=ymax .and. &
                    max(ya,yb)>=ymin .and. &
                    min(za,zb)<=zmax .and. &
                    max(za,zb)>=zmin .and. &
                    icheck==0 ) then

!              --- Compute:

                    call msh610( xa,ya,za, xb,yb,zb, x1,y1,z1, &
                                 x2,y2,z2, x3,y3,z3, t )

!                   --- t value:

                    if ( t<1d0 + 1d-10 ) then

!                   --- Line intersects triangle
!                       First extra check

                       call msh480ext ( coor(1,ia), coor(1,ib), &
                                        coor(1,k1), coor(1,k2), &
                                        coor(1,k3) )

                       call errint ( ia, 2 )
                       call errint ( ib, 3 )
                       do i = 1, 3
                          call errint ( kbound(i,ks), 3+i )
                          x = (coor(i,ia)-1d0)*shift(4)+shift(i)
                          call erreal ( x, i )
                          x = (coor(i,ib)-1d0)*shift(4)+shift(i)
                          call erreal ( x, i+3 )
                          x = (coor(i,k1)-1d0)*shift(4)+shift(i)
                          call erreal ( x, i+6 )
                          x = (coor(i,k2)-1d0)*shift(4)+shift(i)
                          call erreal ( x, i+9 )
                          x = (coor(i,k3)-1d0)*shift(4)+shift(i)
                          call erreal ( x, i+12 )
                       end do  ! i = 1, 3
                       call errint ( ivolnr, 1 )
                       if ( iswitch==0 ) then
                          call errsub ( 3066, 6, 15, 0 )
                       else
                          call errsub ( 3067, 6, 15, 0 )
                       end if  ! ( iswitch==0 )

                       icheck = 1
                       go to 1000
                    end if
               end if
            end if
         end do
      end do

!     --- End of routine:

1000  call erclos( 'msh480' )
      end
