      subroutine msh642( kelem, nelem, coor, nipnt, npoint, istart, &
                         kbound, nbound )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  2.0  date 08-09-2010 New release
!        version  1.2  date 20-08-2010 dynamic allocation helparray
!        version  1.1  date 29-04-2005 new norms
!        version  1.0  date 17-04-1991
!
!   copyright (c) 1991-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subroutine to create and place the centerpoints in tetrahedrons
!     when MESHTP = 15
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     3d
!     mesh_generation
!     tetrahedron
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kbound(*), kelem(*), nelem, nipnt, npoint, istart(nipnt), &
              nbound
      double precision coor(*)

!     coor          i/o   array of length ndim x npoint containing the
!                         co-ordinates of the nodal points
!     istart              helparray of length npoint containing for each
!                         node the number of faces it is the smallest
!                         node number of
!     kbound         i    array containing the boundary triangles
!     kelem         i/o   array containing the tetrahedrons
!     nbound         i    Number of boundary triangles
!     nelem          i    Number of elements
!     nipnt          i    Number of nodes in all boundary elements
!     npoint        i/o   Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, i1, i2, i3, i4, i7, ih, ip, il, j, j1, j2, j3, ndrie, &
              ist, ien, jh
      double precision val, basis
      integer,          dimension   (:), allocatable :: knoop
      double precision, dimension   (:), allocatable :: getal

!     basis      Base value for identification
!     getal      real helparray for ordering of faces of tetrahedrons
!     i          Counting variable
!     i1         Node number
!     i2         Node number
!     i3         Node number
!     i4         Node number
!     i7         Node number
!     ien        End number
!     ih         Node number
!     il         Node number
!     ip         Node number
!     ist        Start number
!     j          Counting variable
!     j1         Node number
!     j2         Node number
!     j3         Node number
!     jh         Help node number
!     knoop      help array to store the old and new face node numbers
!     ndrie      Total number of faces
!     val        Help value for identification of face
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MSH643         Determine for triangle j1-j2-j3 its sequencenumber
!                    and place this number in array GETAL and place value
!                    -1 in array KNOOP.
!     MSH644         Subroutine to determine the relative number of face
!                    j1-j2-j3 in order to place a new node in the barycenter
!     MSH645         Subroutine to create and place the coordinates of the
!                    center point of triangle i1,i2,i3 on position il (in 3D)
!     PLT363         Help subroutine to determine the order for the values
!                    ( i1 , i2 , i3 )  result ( j1 , j2 , j3 )
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen('msh642')

!     --- Start to fill array istart

      do 100 i = 1 , nipnt
          istart(i) = 0
100   continue

!     --- Run through all elements to determine the smallest node of each
!            triangle

      do 200 i = 1 , nelem
          i1 = kelem(10*i - 9)
          i2 = kelem(10*i - 7)
          i3 = kelem(10*i - 5)
          i4 = kelem(10*i    )

          ih = min( i1, i2, i3 )
          istart(ih) = istart(ih) + 1
          ih = min( i1, i2, i4 )
          istart(ih) = istart(ih) + 1
          ih = min( i1, i3, i4 )
          istart(ih) = istart(ih) + 1
          ih = min( i2, i3, i4 )
          istart(ih) = istart(ih) + 1
200   continue

!     --- Run through all boundary elements

      do 220 i = 1 , nbound
          i1 = kbound(7*i - 6)
          i2 = kbound(7*i - 4)
          i3 = kbound(7*i - 2)

          ih = min( i1, i2, i3 )
          istart(ih) = istart(ih) + 1
220   continue

!     --- Determine the start positions for the triangles

      ip = 1

      do 250 i = 1 , nipnt
          il = istart(i) / 2
          istart(i) = ip
          ip = ip + il
250   continue

!     --- Make all references and all node numbers zero

      ndrie = istart(nipnt) - 1

!     --- Allocate space for getal array

      ih = 0
      allocate ( getal( ndrie ), knoop( ndrie ), stat = ih )
      if ( ih/=0 ) call eralloc (  ih, 2*ndrie, 'getal' )
      if ( ierror/=0 ) go to 1000

!     --- Initialize arrays getal and knoop

      do 300 i = 1 , ndrie
          getal(i) = 0
          knoop(i) = 0
300   continue

!     --- Set basis value (dependent on npoint)

      basis = npoint + 10d0

!     --- Determine for each triangle its number

      do 350 i = 1 , nelem

!     --- Find the four cornernodes:

          i1 = kelem(10*i-9)
          i2 = kelem(10*i-7)
          i3 = kelem(10*i-5)
          i4 = kelem(10*i  )

          call plt363( i1,i2,i3, j1,j2,j3 )
          call msh643( j1,j2,j3, istart,getal,basis,knoop )

          call plt363( i1,i2,i4, j1,j2,j3 )
          call msh643( j1,j2,j3, istart,getal,basis,knoop )

          call plt363( i1,i3,i4, j1,j2,j3 )
          call msh643( j1,j2,j3, istart,getal,basis,knoop )

          call plt363( i2,i3,i4, j1,j2,j3 )
          call msh643( j1,j2,j3, istart,getal,basis,knoop )
350   continue

!     --- Run through all boundary triangles and place existing
!         node numbers (of barycentre of triangle)

      do 400 i = 1 , nbound

          i1 = kbound(7*i-6)
          i2 = kbound(7*i-4)
          i3 = kbound(7*i-2)
          i7 = kbound(7*i  )

          call plt363( i1,i2,i3, j1,j2,j3 )

          ist = istart(j3)
          ien = istart(j3+1) - 1
          val = j2 * basis + j1

          jh = 0

          do 380 j = ist , ien
              if ( abs(getal(j)-val)<1d-1 ) then
                 jh = j
                 goto 390
              end if
380       continue

!         --- jh indicates triangle number

390       if ( jh>0 ) knoop(jh) = i7

!         --- ready for this triangle

400   continue

!     --- Run through all knoopvalues and determine new nodes
!         Initialize the npoint value

      nipnt = npoint

      do 500 i = 1 , ndrie
          j = knoop(i)

          if ( j==-2 ) then

!         --- Internal triangle: place new point number

             npoint   = npoint + 1
             knoop(i) = npoint
          end if
500   continue

!     --- Run through kelem and fill all values

      do 600 i = nelem , 1 , -1

          ih = 10 * i + 1

          i1   = kelem( ih - 10 )
          i2   = kelem( ih -  8 )
          i3   = kelem( ih -  6 )
          i4   = kelem( ih -  1 )

          jh   = 14 * i - 3

          do 590 il = 1 , 10
              kelem( jh - il ) = kelem( ih - il )
590       continue

!         --- Determine the extra centre points

          call plt363( i1,i2,i3, j1,j2,j3 )
          call msh644( j1,j2,j3, istart,getal,basis,il )
          if ( knoop(il)>nipnt ) &
               call msh645( coor, i1, i2, i3, knoop(il) )
          kelem( jh   ) = knoop( il )
          call plt363( i1,i2,i4, j1,j2,j3 )
          call msh644( j1,j2,j3, istart,getal,basis,il )
          if ( knoop(il)>nipnt ) &
               call msh645( coor, i1, i2, i4, knoop(il) )
          kelem( jh+1 ) = knoop( il )
          call plt363( i2,i3,i4, j1,j2,j3 )
          call msh644( j1,j2,j3, istart,getal,basis,il )
          if ( knoop(il)>nipnt ) &
               call msh645( coor, i2, i3, i4, knoop(il) )
          kelem( jh+2 ) = knoop( il )
          call plt363( i1,i3,i4, j1,j2,j3 )
          call msh644( j1,j2,j3, istart,getal,basis,il )
          if ( knoop(il)>nipnt ) &
               call msh645( coor, i1, i3, i4, knoop(il) )
          kelem( jh+3 ) = knoop( il )
600   continue

!     --- Deallocate array getal

      deallocate( getal, knoop, stat = ih )
      if ( ih>0 ) call erdealloc ( ih, 'getal' )

1000  call erclos('msh642')

      end
