      subroutine msh652 ( kmeshc, nelem, coor, npoint, numelm )
! ======================================================================
!
!        programmer    niek praagman
!        version  3.0  date 01-12-2010 New release
!        version  2.2  date 06-05-2010 eps disturbation coords barycentre
!        version  2.1  date 26-03-2008 Textual update
!
!   copyright (c) 2004-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check whether all tetrahedrons of kmeshc are disjoint
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh_generation
!     3d
!     tetrahedron
!     check
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nelem, kmeshc(4,nelem), npoint, numelm
      double precision coor(3,npoint+1)

!     coor       i    array of length ndim x npoint containing the co-ordinates
!                     of the nodal points with respect to the global numbering
!     kmeshc     i    actual array with all tetrahedrons
!     nelem      i    number of tetrahedrons to be considered
!     npoint     i    length of array coor, number of nodes
!     numelm     o    indicator which element is wrong
!                      0: all elements are disjunct
!                     >0: element with number numelm belongs also to other
!                      element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision eps, fact, volume, ratio, t, &
                       vol1, vol2, vol3, vol4, vol5, xinf, &
                       xa, xb, x1, x2, x3, x4, &
                       ya, yb, y1, y2, y3, y4, &
                       za, zb, z1, z2, z3, z4
      integer iel, jel, i1, i2, i3, i4, j, j1, j2, j3, j4, ja

!     eps         accuracy
!     fact        helpfactor for barycentre
!     i1          first node
!     i2          second node
!     i3          third node
!     i4          fourth node
!     iel         loopvariable for element to be checked
!     j           counter
!     j1          helpvariable node loop
!     j2          helpvariable node loop
!     j3          helpvariable node loop
!     j4          helpvariable node loop
!     ja          indicator whether point is inside (1) element or not
!     jel         loopvariable for element to be considered
!     ratio       helpvariable to store ratio of element considered
!     t           helpvariable
!     vol1        volume
!     vol2        volume
!     vol3        volume
!     vol4        volume
!     vol5        volume
!     volume      help to store volume of tetrahedron
!     x1          x-coordinate
!     x2          x-coordinate
!     x3          x-coordinate
!     x4          x-coordinate
!     xa          x-coordinate
!     xb          x-coordinate
!     xinf        x value 'infinity'
!     y1          y-coordinate
!     y2          y-coordinate
!     y3          y-coordinate
!     y4          y-coordinate
!     ya          y-coordinate
!     yb          y-coordinate
!     z1          z-coordinate
!     z2          z-coordinate
!     z3          z-coordinate
!     z4          z-coordinate
!     za          z-coordinate
!     zb          z-coordinate
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     MSH610         Determine intersection of triangle and line
!     MSH632         compute volume of tetrahedron
! **********************************************************************
!
!                       I/O
!
!     None
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     None
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Run through all elements (loop iel)
!       Determine barycenter of element
!       Run through all elements (loop jel) and check that
!          barycenter iel does not belong to element jel if jel # iel
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
!     None
! ======================================================================
!
!     --- Set the two variables xinf and numelm:

      xinf   = -1000.10d0
      numelm = 0
      eps    = 1d2 * epsmac

!     --- Run through all elements and determine barycenter of tetrahedron

      do iel = 1, nelem
         i1 = kmeshc( 1, iel )
         i2 = kmeshc( 2, iel )
         i3 = kmeshc( 3, iel )
         i4 = kmeshc( 4, iel )

!        --- Determine coordinates

         fact = 1d0 / 4d0

!        --- Mean value:

         xb = fact * ( coor( 1, i1 ) + coor( 1, i2 ) + &
                       coor( 1, i3 ) + coor( 1, i4 ) )
         yb = fact * ( coor( 2, i1 ) + coor( 2, i2 ) + &
                       coor( 2, i3 ) + coor( 2, i4 ) )
         zb = fact * ( coor( 3, i1 ) + coor( 3, i2 ) + &
                       coor( 3, i3 ) + coor( 3, i4 ) )

!        --- Place barycenter temporary in coor:

         coor(1,npoint+1) = xb
         coor(2,npoint+1) = yb
         coor(3,npoint+1) = zb

!        --- Use point in infinity:

         xa =  0.1 * xinf
         ya =  yb + 1.234d-6
         za =  zb + 0.456d-6

!        --- Run through all elements and check if (xb,yb,zb) is inside:

         do jel = 1, nelem

!        --- Find nodes element jel:

            j1 = kmeshc( 1, jel )
            j2 = kmeshc( 2, jel )
            j3 = kmeshc( 3, jel )
            j4 = kmeshc( 4, jel )

!           --- Coords j1:

            x1 = coor( 1, j1 )
            y1 = coor( 2, j1 )
            z1 = coor( 3, j1 )

!           --- Coords j2:

            x2 = coor( 1, j2 )
            y2 = coor( 2, j2 )
            z2 = coor( 3, j2 )

!           --- Coords j3:

            x3 = coor( 1, j3 )
            y3 = coor( 2, j3 )
            z3 = coor( 3, j3 )

!           --- Coords j4:

            x4 = coor( 1, j4 )
            y4 = coor( 2, j4 )
            z4 = coor( 3, j4 )

!           --- Check common points line a - b with the four faces

            t = - 1000d0
            ja = 0

!           --- Face 1 - 2 - 3

            call msh610(xa,ya,za,xb,yb,zb,x1,y1,z1,x2,y2,z2, &
                        x3,y3,z3,t)
            if ( t<1d0 ) ja = ja + 1

!           --- Face 1 - 2 - 4

            call msh610(xa,ya,za,xb,yb,zb,x1,y1,z1,x2,y2,z2, &
                        x4,y4,z4,t)
            if ( t<1d0 ) ja = ja + 1

!           --- Face 1 - 3 - 4

            call msh610(xa,ya,za,xb,yb,zb,x1,y1,z1,x3,y3,z3, &
                        x4,y4,z4,t)
            if ( t<1d0 ) ja = ja + 1

!           --- Face 2 - 3 - 4

            call msh610(xa,ya,za,xb,yb,zb,x2,y2,z2,x3,y3,z3, &
                        x4,y4,z4,t)
            if ( t<1d0 ) ja = ja + 1

!           --- Check and if needed give message:

            if ( ja==1 .and. iel/=jel ) then

!           --- For savety: compute barycentric coordinates:

               call msh632( coor,j1,j2,j3,      j4,volume )
               call msh632( coor,j1,j2,j3,npoint+1,vol1   )
               call msh632( coor,j2,j4,j3,npoint+1,vol2   )
               call msh632( coor,j3,j4,j1,npoint+1,vol3   )
               call msh632( coor,j1,j4,j2,npoint+1,vol4   )
               vol1 = vol1 / volume
               vol2 = vol2 / volume
               vol3 = vol3 / volume
               vol4 = vol4 / volume
               vol5 = 1d0 - vol1 - vol2 - vol3 - vol4
               if ( -eps<vol1 .and. -eps<vol2 .and. -eps<vol3 .and. &
                    -eps<vol4 .and. -eps<vol5 .and. 1D0+eps>vol1 .and. &
                    1D0+eps>vol2 .and. 1D0+eps>vol3 .and. &
                    1D0+eps>vol4 .and. 1D0+eps>vol5 ) then
                  numelm = max( iel, jel )
                  write(irefwr,*) 'Bary coordinates'
                  write(irefwr,*) 'vol 1 ',vol1
                  write(irefwr,*) 'vol 2 ',vol2
                  write(irefwr,*) 'vol 3 ',vol3
                  write(irefwr,*) 'vol 4 ',vol4
                  write(irefwr,*) 'vol 5 ',vol5
                  write(irefwr,*) 'Barycentre',iel,'in elmnt ',jel
                  call msh632( coor, j1, j2, j3, j4, volume)
                  call msh661( coor, j1, j2, j3, j4, ratio )
                  write(irefwr,*) 'Elemnt',jel,' nodes',j1,j2,j3,j4
                  write(irefwr,*) 'cord',j1,': ',x1,y1,z1
                  write(irefwr,*) 'cord',j2,': ',x2,y2,z2
                  write(irefwr,*) 'cord',j3,': ',x3,y3,z3
                  write(irefwr,*) 'cord',j4,': ',x4,y4,z4
                  write(irefwr,*) 'Volume = ',volume,'Ratio =',ratio
                  call msh632( coor, i1, i2, i3, i4, volume)
                  call msh661( coor, i1, i2, i3, i4, ratio )
                  write(irefwr,*) 'Elemnt',iel,' nodes: ',i1,i2,i3,i4
                  write(irefwr,*) 'cord',i1,(coor(j,i1),j=1,3)
                  write(irefwr,*) 'cord',i2,(coor(j,i2),j=1,3)
                  write(irefwr,*) 'cord',i3,(coor(j,i3),j=1,3)
                  write(irefwr,*) 'cord',i4,(coor(j,i4),j=1,3)
                  write(irefwr,*) 'bary',iel,xb,yb,zb
                  write(irefwr,*) 'Volume = ',volume,'Ratio =',ratio
               end if
            end if
         end do
      end do

!     --- Ready msh652

      end
