      subroutine msh656( npoint, coor   , xstart , ystart, zstart, &
                         dismax, numnodextcurvs, curvenumbers, &
                         coorcurvs, nextracrvs, icube , &
                         nx    , ny     , kelem , nelem , &
                         kstapl, kstap  , itri, kbndpt, lkbndpt )
! ======================================================================
!
!        programmer    niek praagman
!        version  2.0  date 09-09-2010 New release
!        version  1.3  date 15-04-2009 Adjust fac value
!        version  1.2  date 08-04-2009 Use dot and crossproduct to
!                                      determine orientation of tetrahedrons
!        version  1.1  date 27-03-2009 Several adjustments (ws)
!        version  1.0  date 29-08-2008
!
!   copyright (c) 2008-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Place special fixed lines and their fixed nodes and place elements
!   around these special nodes in a volume.
!   Check whether all elements are inside volume etc.
! **********************************************************************
!
!                       KEYWORDS
!
!     fixed_lines
!     plaxis
!     mesh
!     3d
!     mesh_generation
!     tetrahedron
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint, kelem(4,*), icube(*), nelem, nx, ny, kstap, &
              nextracrvs, kstapl(3,*), itri(*), &
              numnodextcurvs(nextracrvs), curvenumbers(nextracrvs), &
              lkbndpt, kbndpt(*)
      double precision coor(3,*), coorcurvs(3,*), &
              xstart, ystart, zstart, dismax

!     coor             i   coordinate array
!     coorcurvs        i   array with coordinates of nodes on internal
!                          fixed lines
!     curvenumbers     i   usernumbers of given fixed-curves
!     dismax           i   helpdistance for blockdetermination
!     icube            i   array with for each node the number of the
!                          quadrangle that node belongs to
!     itri            i/o  helparray containing information indicating
!                          whether a point is in the (actual) boundary or not
!     kbndpt          i/o  array with extra nodenumbers for internal curves
!     kelem           i/o  element array (created triangles)
!     kstap           i/o  length of boundary elements array
!     kstapl          i/o  array with nodes of boundary elements
!     lkbndpt         i/o  adresses of extra nodes internal lines in kbndpt
!     nelem           i/o  number of created elements in interior in total
!     nextracrvs       i   number of extra internal curves
!     npoint           i   number of points in line elements of contour
!     numnodextcurvs   i   sequentially number of nodes in the extra
!                          internal curves of this volume
!     nx               i   number of blocks in x-direction
!     ny               i   number of blocks in y-direction
!     xstart           i   x-coordinate left under in enveloping block
!     ystart           i   y-coordinate left under in enveloping block
!     zstart           i   z-coordinate left under in enveloping block
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision dis, dist, fac, dx, dy, dz, xp, yp, zp, &
              xq, yq, zq, xh, yh, zh, xc, yc, zc, eps, &
              dx1, dy1, dz1, dx2, dy2, dz2, volume
      integer dir, i, i1, i2, i3, i4, iex, istart, itry, j, ja1, ja2, &
              n1, n2, n3, nc, numnodes, nipnt, ppoint, qpoint
      integer npunt

!     debug        parameter to indicate yes/no (true/false) debugging
!     dir          main direction of line
!     dis          distance
!     dist         helpvariable to store local meshsize
!     dx           x-difference
!     dx1          x-difference
!     dx2          x-difference
!     dy           y-difference
!     dy1          y-difference
!     dy2          y-difference
!     dz           z-difference
!     dz1          z-difference
!     dz2          z-difference
!     eps          accuracy parameter
!     fac          factor
!     i            loop variable
!     i1           node number
!     i2           node number
!     i3           node number
!     i4           node number
!     iex          loopvariable
!     istart       starting position for next internal line
!     itry         number of tries
!     j            loopvariable
!     ja1          indicator whether point is inside area or not
!     ja2          indicator whether point is inside area or not
!     n1           helpvariable x-number of block
!     n2           helpvariable y-number of block
!     n3           helpvariable z-number of block
!     nc           block number
!     nipnt        number of initial points
!     numnodes     helpvariable to get number of nodes on linepiece
!     ppoint       point number of first point of line piece
!     qpoint       point number of last point of line piece
!     volume       helpvariable to store volume of tet
!     xc           x-coordinate
!     xh           x-coordinate
!     xp           x-coordinate
!     xq           x-coordinate
!     yc           y-coordinate
!     yh           y-coordinate
!     yp           y-coordinate
!     yq           y-coordinate
!     zc           z-coordinate
!     zh           z-coordinate
!     zp           z-coordinate
!     zq           z-coordinate
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSH607         check whether point is inside or outside volume
!     MSH632         compute volume of tetrahedron
!     MSH652         Check if all tetrahedrons of kmeshc are disjoint
! **********************************************************************
!
!                       I/O
!
!    None
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2913   Error with barycenters
!    2971   Node of line outside region
!    2972   Internal line too close to boundary
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Run through all special internal linepieces
!
!    Check whether tetrahedrons are permitted.
!    If not permitted then
!       stop, error
!    Else place nodes and elements and adjust arrays.
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
!    None
! ======================================================================
!
      call eropen ( 'msh656' )

!     --- Initialize debug

      debug = .false.

!     --- Set accuracy

      eps = 100 * epsmac

!     --- Run through all (extra) fixed pieces:

      istart = 0
      npunt  = npoint
      if ( debug ) write(irefwr,*) 'Start msh656: npunt =',npunt

      do i = 1, nextracrvs

!     --- Determine for each extra curve the number of nodes in that line:

         numnodes = numnodextcurvs(i)

!        --- Run through pieces of this line:

         do j = 1, numnodes - 1
            nipnt  = npoint
            qpoint = 0

!           --- Find coordinates begin and endpoint:

            xp = coorcurvs(1,istart + j     )
            yp = coorcurvs(2,istart + j     )
            zp = coorcurvs(3,istart + j     )

            xq = coorcurvs(1,istart + j + 1 )
            yq = coorcurvs(2,istart + j + 1 )
            zq = coorcurvs(3,istart + j + 1 )

!           --- Check if first point of first line piece is already a node

            if ( j==1 ) then
               ppoint = 0

!              --- Check nodes so far:

               do iex = 1 , npoint

                  dx = xp - coor(1,iex)
                  dy = yp - coor(2,iex)
                  dz = zp - coor(3,iex)

                  dis = sqrt(dx*dx + dy*dy + dz*dz)

                  if ( dis<eps .and. ppoint==0 ) then

!                 --- Old point: nodenumber is iex

                     ppoint = iex
                     ja1 = 1
                  end if
               end do

!              --- If node is new then check whether node is inside:

               if ( ppoint==0 ) then

!              --- Check node:

                  call msh607( xp, yp, zp, xstart, coor, kstapl, kstap, &
                               ja1 )

!                 --- Place first node of piece:

                  npoint = npoint + 1

                  coor(1,npoint) = xp
                  coor(2,npoint) = yp
                  coor(3,npoint) = zp

                  ppoint = npoint

                  n1 = int ( (xp-xstart)/dismax )
                  n2 = int ( (yp-ystart)/dismax )
                  n3 = int ( (zp-zstart)/dismax )

!                 --- This point belongs to quadrangle with number

                  nc = 1 + n1 + n2*nx + n3*nx*ny

                  icube( npoint ) = nc
               end if
               lkbndpt = lkbndpt + 1
               kbndpt(lkbndpt) = ppoint
            else
               ja1 = 1

!              --- First point of each following line piece is the same as
!                  the last point of its previous line piece

               ppoint = nipnt - 4
            end if

!           --- Check if the last point of the last line piece is already in
!               the boundary of the volume:

            if ( j==numnodes - 1 ) then
               qpoint = 0

!              --- Check nodes so far:

               do iex = 1 , npoint

                  dx = xq - coor(1,iex)
                  dy = yq - coor(2,iex)
                  dz = zq - coor(3,iex)

                  dis = sqrt(dx*dx + dy*dy + dz*dz)

                  if ( dis<eps .and. qpoint==0 ) then

!                 --- Old point

                     qpoint = iex
                     ja2 = 1
                  end if
               end do
            end if
            if ( qpoint==0 ) then

!           --- Check whether node is inside:

               call msh607( xq, yq, zq, xstart, coor, kstapl, kstap, &
                            ja2 )
               npoint = npoint + 1
               qpoint = npoint

               coor(1,npoint) = xq
               coor(2,npoint) = yq
               coor(3,npoint) = zq

               n1 = int ( (xq-xstart)/dismax )
               n2 = int ( (yq-ystart)/dismax )
               n3 = int ( (zq-zstart)/dismax )

!              --- This point belongs to quadrangle with number

               nc = 1 + n1 + n2*nx + n3*nx*ny
               icube( npoint ) = nc
            end if

            lkbndpt = lkbndpt + 1
            kbndpt(lkbndpt) = qpoint

            if ( ja1==0 .or. ja2==0 ) then

!           --- Stop: User submitted line/point that is not inside region

               call errint ( curvenumbers(i), 1 )
               call errsub ( 2971, 1, 0, 0 )
               go to 1000
            end if

!           --- Next place extra nodes:

            dx = xq - xp
            dy = yq - yp
            dz = zq - zp

            dist = sqrt(dx*dx + dy*dy + dz*dz)

!           --- Check main direction of line xp-xq

            dir = 3

            if ( abs(dx) >1.1*abs(dy).and.abs(dx)>1.1*abs(dz) ) dir = 1
            if ( abs(dy) >1.1*abs(dx).and.abs(dy)>1.1*abs(dz) ) dir = 2

!           --- Define

            if ( dir==1 ) then

!           --- Check whether exactly in x-direction:

               if ( abs(abs(dx)-dist)<1d-5*dist ) then

!              --- Special situation:

                  dx1 =    0
                  dy1 = dx / abs(dx)
                  dz1 =    0
               else

!              --- Use ( 0, -dz, dy ) and ( dy**2+dz**2, -dx*dy, -dx*dz )

                  dx1 =    0
                  dy1 =  -dz
                  dz1 =   dy
               end if
            else if ( dir==2 ) then

!           --- Check whether exactly in y-direction:

               if ( abs(abs(dy)-dist)<1d-5*dist ) then

!              --- Special situation:

                  dx1 =    0
                  dy1 =    0
                  dz1 = dy / abs(dy)

               else

!              --- Use ( dz, 0, -dx ) and ( -dy*dx, dz*dz+dx*dx, -dy*dz )

                  dx1 =   dz
                  dy1 =    0
                  dz1 =  -dx
               end if
            else if ( dir==3 ) then

!           --- Check whether exactly in y-direction:

               if ( abs(abs(dz)-dist)<1d-5*dist ) then

!              --- Special situation:

                  dx1 = dz / abs(dz)
                  dy1 =    0
                  dz1 =    0
               else

!              --- Use ( -dy, dx, 0 ) and ( -dx*dx, -dz*dy, dx*dx+dy*dy )

                  dx1 =  -dy
                  dy1 =   dx
                  dz1 =    0
               end if
            end if

!           --- Compute third direction using vector product:

            dx2 = dy * dz1 - dz * dy1
            dy2 = dz * dx1 - dx * dz1
            dz2 = dx * dy1 - dy * dx1

!           --- Normalize helpvectors using variable dist:

            dis = sqrt( dx1*dx1+dy1*dy1+dz1*dz1 )
            fac = 0.5 * dist / dis

            dx1 = fac * dx1
            dy1 = fac * dy1
            dz1 = fac * dz1

            dis = sqrt( dx2*dx2+dy2*dy2+dz2*dz2 )

            fac = 0.5 * dist / dis

            dx2 = fac * dx2
            dy2 = fac * dy2
            dz2 = fac * dz2

!           --- Define middle point on linepiece:

            xh = (xp+xq)/2d0
            yh = (yp+yq)/2d0
            zh = (zp+zq)/2d0

            n1 = int ( (xh-xstart)/dismax )
            n2 = int ( (yh-ystart)/dismax )
            n3 = int ( (zh-zstart)/dismax )

!           --- This point belongs to quadrangle with number

            nc = 1 + n1 + n2*nx + n3*nx*ny

!           --- Define four additional nodes:

            ja2 = 0

!           --- First node:

            itry = 0

            xc = xh + dx1
            yc = yh + dy1
            zc = zh + dz1

100         call msh607( xc, yc, zc, xstart, coor, kstapl, kstap, ja1 )

            if ( ja1==0 .and. itry<10 ) then

               xc = xh + 0.5 * ( xc - xh )
               yc = yh + 0.5 * ( yc - yh )
               zc = zh + 0.5 * ( zc - zh )

               itry = itry + 1

               goto 100

            else if ( ja1==0 .and. itry==10 ) then

               ja2 = 1
               goto 500

            else

               npoint = npoint + 1

               coor(1,npoint) = xc
               coor(2,npoint) = yc
               coor(3,npoint) = zc

               icube( npoint ) = nc

            end if

!           --- Second node:

            itry = 0

            xc = xh + dx2
            yc = yh + dy2
            zc = zh + dz2

200         call msh607( xc, yc, zc, xstart, coor, kstapl, kstap, ja1 )

            if ( ja1==0 .and. itry<10 ) then

               xc = xh + 0.5 * ( xc - xh )
               yc = yh + 0.5 * ( yc - yh )
               zc = zh + 0.5 * ( zc - zh )

               itry = itry + 1

               goto 200

            else if ( ja1==0 .and. itry==10 ) then

               ja2 = 1
               goto 500

            else

               npoint = npoint + 1

               coor(1,npoint) = xc
               coor(2,npoint) = yc
               coor(3,npoint) = zc

               icube( npoint ) = nc

            end if

!           --- Third node:

            itry = 0

            xc = xh - dx1
            yc = yh - dy1
            zc = zh - dz1

300         call msh607( xc, yc, zc, xstart, coor, kstapl, kstap, ja1 )

            if ( ja1==0 .and. itry<10 ) then

               xc = xh + 0.5 * ( xc - xh )
               yc = yh + 0.5 * ( yc - yh )
               zc = zh + 0.5 * ( zc - zh )

               itry = itry + 1

               goto 300

            else if ( ja1==0 .and. itry==10 ) then

               ja2 = 1
               goto 500

            else

               npoint = npoint + 1

               coor(1,npoint) = xc
               coor(2,npoint) = yc
               coor(3,npoint) = zc

               icube( npoint ) = nc

            end if

!           --- Fourth node:

            itry = 0

            xc = xh - dx2
            yc = yh - dy2
            zc = zh - dz2

400         call msh607( xc, yc, zc, xstart, coor, kstapl, kstap, ja1 )

            if ( ja1==0 .and. itry<10 ) then

               xc = xh + 0.5 * ( xc - xh )
               yc = yh + 0.5 * ( yc - yh )
               zc = zh + 0.5 * ( zc - zh )

               itry = itry + 1

               goto 400

            else if ( ja1==0 .and. itry==10 ) then

               ja2 = 1
               goto 500

            else

               npoint = npoint + 1

               coor(1,npoint) = xc
               coor(2,npoint) = yc
               coor(3,npoint) = zc

               icube( npoint ) = nc

            end if

500         if ( ja2==1 ) then

!           --- One of the new points is outside region or too
!               near to the boundary: error

               call errint ( curvenumbers(i), 1 )
               call errsub ( 2972, 1, 0, 0 )
               go to 1000

            end if

!           --- Place four elements:

            nelem          = nelem  + 1
            kelem(1,nelem) = ppoint
            kelem(2,nelem) = qpoint
            kelem(3,nelem) = npoint
            kelem(4,nelem) = npoint - 3

            call msh632( coor, ppoint, qpoint, npoint, npoint-3, &
                         volume )

            nelem          = nelem  + 1
            kelem(1,nelem) = ppoint
            kelem(2,nelem) = qpoint
            kelem(3,nelem) = npoint - 3
            kelem(4,nelem) = npoint - 2

            call msh632( coor, ppoint, qpoint, npoint-3, npoint-2, &
                         volume )

            nelem          = nelem  + 1
            kelem(1,nelem) = ppoint
            kelem(2,nelem) = qpoint
            kelem(3,nelem) = npoint - 2
            kelem(4,nelem) = npoint - 1

            call msh632( coor, ppoint, qpoint, npoint-2, npoint-1, &
                         volume )

            nelem          = nelem  + 1
            kelem(1,nelem) = ppoint
            kelem(2,nelem) = qpoint
            kelem(3,nelem) = npoint - 1
            kelem(4,nelem) = npoint

            call msh632( coor, ppoint, qpoint, npoint-1, npoint, &
                         volume )

!           --- Adjust boundary array

            kstap           = kstap  + 1
            kstapl(1,kstap) = npoint - 3
            kstapl(2,kstap) = ppoint
            kstapl(3,kstap) = npoint - 2

            kstap           = kstap  + 1
            kstapl(1,kstap) = npoint - 2
            kstapl(2,kstap) = ppoint
            kstapl(3,kstap) = npoint - 1

            kstap           = kstap  + 1
            kstapl(1,kstap) = npoint - 1
            kstapl(2,kstap) = ppoint
            kstapl(3,kstap) = npoint

            kstap           = kstap  + 1
            kstapl(1,kstap) = npoint
            kstapl(2,kstap) = ppoint
            kstapl(3,kstap) = npoint - 3

            kstap           = kstap  + 1
            kstapl(1,kstap) = npoint - 1
            kstapl(2,kstap) = qpoint
            kstapl(3,kstap) = npoint - 2

            kstap           = kstap  + 1
            kstapl(1,kstap) = npoint - 2
            kstapl(2,kstap) = qpoint
            kstapl(3,kstap) = npoint - 3

            kstap           = kstap  + 1
            kstapl(1,kstap) = npoint - 3
            kstapl(2,kstap) = qpoint
            kstapl(3,kstap) = npoint

            kstap           = kstap  + 1
            kstapl(1,kstap) = npoint
            kstapl(2,kstap) = qpoint
            kstapl(3,kstap) = npoint - 1

!           --- Adjust array itri

            itri(ppoint    ) = itri(ppoint  ) + 4
            itri(qpoint    ) = itri(qpoint  ) + 4
            itri(npoint - 3) = itri(npoint-3) + 4
            itri(npoint - 2) = itri(npoint-2) + 4
            itri(npoint - 1) = itri(npoint-1) + 4
            itri(npoint    ) = itri(npoint  ) + 4
         end do
         istart = istart+numnodes
      end do  ! i = 1, nextracrvs
1000  call erclos ( 'msh656' )
      if ( debug ) then
         write(irefwr,*) 'End of msh656',npoint,' nodes '
         do i=1, npoint
            xc = coor(1,i)
            yc = coor(2,i)
            zc = coor(3,i)
            write(irefwr,*) i,'crds',xc,yc,zc
         end do

         write(irefwr,*) 'End of msh656',nelem,' elements'
         do i=1, nelem
            i1 = kelem(1,i)
            i2 = kelem(2,i)
            i3 = kelem(3,i)
            i4 = kelem(4,i)
            call msh632( coor, i1, i2, i3, i4, volume )
            write(irefwr,*) 'elmnt',i,'volume=',volume
            write(irefwr,*) i1,'crd',(coor(j,i1),j=1,3)
            write(irefwr,*) i2,'crd',(coor(j,i2),j=1,3)
            write(irefwr,*) i3,'crd',(coor(j,i3),j=1,3)
            write(irefwr,*) i4,'crd',(coor(j,i4),j=1,3)
         end do

         nc = 0
         call msh652( kelem, nelem, coor, npoint, nc )
         write(irefwr,*) 'After call to msh652'
         write(irefwr,*) 'Problems in element',nc
         if (nc>0) then
            call errint ( nc, 1 )
            call errsub ( 2913, 1, 0, 0 )
         end if
         write(irefwr,*) 'end debug msh656'
      end if

      end
