      subroutine msh665 ( kelem, nelem, nelmfix, jelem, ja, &
                          coor, npoint )
! ======================================================================
!
!        programmer    niek praagman
!
!        version  3.1  date 22-08-2011 Add extra routine msh678
!        version  3.0  date 09-09-2010 New release
!        version  2.2  date 20-08-2010 Add extra routines msh673,4,5
!
!   copyright (c) 2006-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Skip bad tetrahedron by rearranging the nodes. (Tetrahedron is "flat").
!     Try to redivide a bunch of three or four tetrahedrons.
!     First find all extra nodes ii1, ii2, ii3 and ii4 and next find
!     the best redivision.
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh_generation
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nelem, nelmfix, npoint, ja, jelem, kelem(4,*)
      double precision coor(3,*)

!     coor       i    array of length ndim x npoint containing the co-ordinates
!                     of the nodal points with respect to the global numbering
!     ja         o    indicator type of tetrahedron jelem
!                       ja = 0 => Boundary involved, no improvement possible
!                       ja = 1 => Redivision of three old elements. Three
!                                 elements removed, two new ones
!                       ja = 2 => Redivision based on four old tetrahedrons
!                                 but now with better ratio realized
!                       ja = 3 => Difficult situation: no solution yet
!     jelem      i    element number of "bad" tetrahedron
!     kelem     i/o   element array
!     nelem      i    number of elements in mesh
!     nelmfix    i    the first nelmfix elements should remain fixed
!     npoint    i/o   number of nodes in mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i1, i2, i3, i4, ia, ib, ic, id, ichng, ik, il, &
              ielem, ii1, ii2, ii3, ii4, j1, j2, j3, j4, &
              jel1, jel2, jel3, jel4, iiel1, iiel2, iiel3, iiel4, &
              iel12i1i2, iel13i1i3, iel14i2i3, &
              iel23i1i4, iel24i2i4, iel34i3i4
      double precision vol, rat1, rat2, ratio
      double precision vol1, vol2

!     i1             node number
!     i2             node number
!     i3             node number
!     i4             node number
!     ia             node number
!     ib             node number
!     ic             node number
!     ichng          changement indicator
!     id             node number
!     iel12i1i2      extra element number for special case
!     iel13i1i3      extra element number for special case
!     iel14i2i3      extra element number for special case
!     iel23i1i4      extra element number for special case
!     iel24i2i4      extra element number for special case
!     iel34i3i4      extra element number for special case
!     ielem          loop variable
!     ii1            node number
!     ii2            node number
!     ii3            node number
!     ii4            node number
!     iiel1          element change number
!     iiel2          element change number
!     iiel3          element change number
!     iiel4          element change number
!     ik             element number
!     il             loop variable
!     j1             indicator
!     j2             indicator
!     j3             indicator
!     j4             indicator
!     jel1           element number of the 4 extra tetrahedrons
!     jel2           element number of the 4 extra tetrahedrons
!     jel3           element number of the 4 extra tetrahedrons
!     jel4           element number of the 4 extra tetrahedrons
!     rat1           help variable to store ratio's
!     rat2           help variable to store ratio's
!     ratio          ratio of element to be improved
!     vol            volume of tetrahedron
!     vol1           volume of tetrahedron
!     vol2           volume of tetrahedron
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSH632         compute volume of tetrahedron
!     MSH661         Compute ratio of inner and outer sphere of tetrahedron
!     MSH666         Redivide bunch of four tets
!     MSH673         Redivide bunch of seven tets
!     MSH674         Redivide bunch of seven tets
!     MSH675         Redivide bunch of seven tets
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Find all extra nodes connected to the four sides of the tetrahedron
!
!    Check whether two of these extra nodes concide and if so
!    make redivision
!
!    Check whether tetrahedron has three neighbouring tetrahedrons that
!    together can be redivided and if so check whether the ultimate ratio
!    is improved
!
!    Check whether special situation for three neighbours and find best
!    improvement
!
!    Return
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh665' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh665'
      end if
      debug = .false.
      if ( ierror/=0 ) go to 1000

!     --- Set the nodenumbers
!         Suppose ja=0 i.e. no changements will be made

      ja = 0

!     --- Node numbers of jelem:

      i1 = kelem(1,jelem)
      i2 = kelem(2,jelem)
      i3 = kelem(3,jelem)
      i4 = kelem(4,jelem)

!     --- Consider element i1 - i2 - i3 -i4

      call msh661( coor, i1, i2, i3, i4, ratio )

!     --- Find extra tetrahedrons, change combinations of tetrahedrons
!         and skip element jelem

      jel1 = 0
      jel2 = 0
      jel3 = 0
      jel4 = 0

      ii1  = 0
      ii2  = 0
      ii3  = 0
      ii4  = 0

      do ielem = nelmfix+1, nelem

         ia = kelem(1,ielem)
         ib = kelem(2,ielem)
         ic = kelem(3,ielem)
         id = kelem(4,ielem)

         j1 = 0
         j2 = 0
         j3 = 0
         j4 = 0

         if (i1==ia .or. i1==ib .or. i1==ic .or. i1==id) j1 = 1
         if (i2==ia .or. i2==ib .or. i2==ic .or. i2==id) j2 = 1
         if (i3==ia .or. i3==ib .or. i3==ic .or. i3==id) j3 = 1
         if (i4==ia .or. i4==ib .or. i4==ic .or. i4==id) j4 = 1

         if ( j1 + j2 + j3 + j4==3 ) then

!        --- Extra tetrahedron found
!            Find extra node either ii1, ii2, ii3 or ii4

            if ( j1 + j2 + j3==3 ) then

!           --- id is new (i1-i2-i3 and ii1)

               jel1 = ielem
               if ( ia/=i1 .and. ia/=i2 .and. ia/=i3 ) then
                  ii1 = ia
               else if ( ib/=i1 .and. ib/=i2 .and. ib/=i3 ) then
                  ii1 = ib
               else if ( ic/=i1 .and. ic/=i2 .and. ic/=i3 ) then
                  ii1 = ic
               else
                  ii1 = id
               end if
            else if ( j1 + j2 + j4==3 ) then
               jel2 = ielem

!              --- ic is new (i1-i2-i4 and ii2)

               if ( ia/=i1 .and. ia/=i2 .and. ia/=i4 ) then
                  ii2 = ia
               else if ( ib/=i1 .and. ib/=i2 .and. ib/=i4 ) then
                  ii2 = ib
               else if ( ic/=i1 .and. ic/=i2 .and. ic/=i4 ) then
                  ii2 = ic
               else
                  ii2 = id
               end if
            else if ( j1 + j3 + j4==3 ) then

!           --- ib is new (i1-i3-i4 and ii3)

               jel3 = ielem
               if ( ia/=i1 .and. ia/=i3 .and. ia/=i4 ) then
                  ii3 = ia
               else if ( ib/=i1 .and. ib/=i3 .and. ib/=i4 ) then
                  ii3 = ib
               else if ( ic/=i1 .and. ic/=i3 .and. ic/=i4 ) then
                  ii3 = ic
               else
                  ii3 = id
               end if
            else if ( j2 + j3 + j4==3 ) then

!           --- ia is new (i2-i3-i4 and ii4)

               jel4 = ielem
               if ( ia/=i2 .and. ia/=i3 .and. ia/=i4 ) then
                  ii4 = ia
               else if ( ib/=i2 .and. ib/=i3 .and. ib/=i4 ) then
                  ii4 = ib
               else if ( ic/=i2 .and. ic/=i3 .and. ic/=i4 ) then
                  ii4 = ic
               else
                  ii4 = id
               end if
            end if
         end if
      end do

!     --- Change tetrahedrons:

      call msh661( coor, i1,i2,i3,i4, ratio )

!     --- Parameter ichng: indicator performed action or not

      ichng = 0
      if ( ii1==ii2 .and. ii1/=0 .and. ichng==0 ) then

!     --- Change diagonal 1 - 2 to diagonal 3 - 4
!         Compute new ratio's

         call msh661( coor, i1, i3, i4, ii1, rat1 )
         call msh661( coor, i2, i4, i3, ii2, rat2 )

!        --- Check

         if ( min(rat1,rat2)> ratio ) then

!        --- Improvement

            kelem(1,jel1) = i1
            kelem(2,jel1) = i3
            kelem(3,jel1) = i4
            kelem(4,jel1) = ii1

            kelem(1,jel2) = i2
            kelem(2,jel2) = i4
            kelem(3,jel2) = i3
            kelem(4,jel2) = ii2

            if ( debug ) then
               write(irefwr,*) 'Msh665: Case 1 ii1 =',ii1,' ii2 =',ii2
               write(irefwr,*) 'New ratio = ',min(rat1,rat2)
               call msh632(coor,i1,i3,i4,ii1,vol)
               vol1 = vol
               call msh632(coor,i2,i4,i3,ii2,vol)
               vol1 = vol + vol1
               call msh632(coor,i1,i3,i2,ii1,vol)
               vol2 = vol
               call msh632(coor,i1,i2,i4,ii2,vol)
               vol2 = vol2 + vol
               call msh632(coor,i1,i2,i3,i4,vol)
               vol2 = vol2 + vol
               write(irefwr,*) 'Volumes: New = ',vol1,' Old = ',vol2
            end if

            ichng = 1
            ja    = 1
         end if
      end if

      if ( ii1==ii3 .and. ii1/=0 .and. ichng==0 ) then

!     --- Change diagonal 1-3 to 2-4

         call msh661( coor, i1, i4, i2, ii1, rat1 )
         call msh661( coor, i2, i4, i3, ii3, rat2 )

!        --- Check

         if ( min(rat1,rat2)>ratio ) then

            kelem(1,jel1) = i1
            kelem(2,jel1) = i4
            kelem(3,jel1) = i2
            kelem(4,jel1) = ii1

            kelem(1,jel3) = i2
            kelem(2,jel3) = i4
            kelem(3,jel3) = i3
            kelem(4,jel3) = ii3

            if ( debug ) then
               write(irefwr,*) 'msh665: Case 2 ii1 =',ii1,' ii3 =',ii3
               write(irefwr,*) 'New ratio = ',min(rat1,rat2)
               call msh632(coor,i1,i4,i2,ii1,vol)
               vol1 = vol
               call msh632(coor,i2,i4,i3,ii3,vol)
               vol1 = vol + vol1
               call msh632(coor,i1,i3,i2,ii1,vol)
               vol2 = vol
               call msh632(coor,i1,i4,i3,ii3,vol)
               vol2 = vol2 + vol
               call msh632(coor,i1,i2,i3,i4,vol)
               vol2 = vol2 + vol
               write(irefwr,*) 'Volumes: New = ',vol1,' Old = ',vol2
            end if

            ichng = 1
            ja    = 1

         end if

      end if

      if ( ii1==ii4 .and. ii1/=0 .and. ichng==0 ) then


         call msh661( coor, i1, i4, i2, ii1, rat1 )
         call msh661( coor, i1, i3, i4, ii4, rat2 )

!        --- Check:

         if ( min(rat1,rat2)>ratio ) then

            kelem(1,jel1) = i1
            kelem(2,jel1) = i4
            kelem(3,jel1) = i2
            kelem(4,jel1) = ii1

            kelem(1,jel4) = i1
            kelem(2,jel4) = i3
            kelem(3,jel4) = i4
            kelem(4,jel4) = ii4

            if ( debug ) then
               write(irefwr,*) 'msh665: Case 3 ii1 =',ii1,' ii4 =',ii4
               write(irefwr,*) 'New ratio = ',min(rat1,rat2)
               call msh632(coor,i1,i4,i2,ii1,vol)
               vol1 = vol
               call msh632(coor,i1,i3,i4,ii4,vol)
               vol1 = vol + vol1
               call msh632(coor,i2,i1,i3,ii1,vol)
               vol2 = vol
               call msh632(coor,i2,i3,i4,ii4,vol)
               vol2 = vol2 + vol
               call msh632(coor,i1,i2,i3,i4,vol)
               vol2 = vol2 + vol
               write(irefwr,*) 'Volumes: New = ',vol1,' Old = ',vol2
            end if

            ichng = 1
            ja    = 1

         end if

      end if

      if ( ii2==ii3 .and. ii2/=0 .and. ichng==0 ) then

         call msh661( coor, i1, i2, i3, ii2, rat1 )
         call msh661( coor, i2, i4, i3, ii3, rat2 )

         if ( min(rat1,rat2)>ratio ) then

            kelem(1,jel2) = i1
            kelem(2,jel2) = i2
            kelem(3,jel2) = i3
            kelem(4,jel2) = ii2

            kelem(1,jel3) = i2
            kelem(2,jel3) = i4
            kelem(3,jel3) = i3
            kelem(4,jel3) = ii3

            if ( debug ) then

               write(irefwr,*) 'msh665: Case 4 ii2 =',ii2,' ii3 =',ii3
               write(irefwr,*) 'New ratio = ',min(rat1,rat2)
               call msh632(coor,i1,i2,i3,ii2,vol)
               vol1 = vol
               call msh632(coor,i2,i4,i3,ii3,vol)
               vol1 = vol + vol1
               call msh632(coor,i1,i2,i4,ii2,vol)
               vol2 = vol
               call msh632(coor,i1,i4,i3,ii3,vol)
               vol2 = vol2 + vol
               call msh632(coor,i1,i2,i3,i4,vol)
               vol2 = vol2 + vol
               write(irefwr,*) 'Volumes: New = ',vol1,' Old = ',vol2

            end if

            ichng = 1
            ja    = 1

         end if

      end if

      if ( ii2==ii4 .and. ii2/=0 .and. ichng==0 ) then

         call msh661( coor, i1, i2, i3, ii2, rat1 )
         call msh661( coor, i1, i3, i4, ii4, rat2 )

         if ( min(rat1,rat2)>ratio ) then

            kelem(1,jel2) = i1
            kelem(2,jel2) = i2
            kelem(3,jel2) = i3
            kelem(4,jel2) = ii2

            kelem(1,jel4) = i1
            kelem(2,jel4) = i3
            kelem(3,jel4) = i4
            kelem(4,jel4) = ii4

            if ( debug ) then

               write(irefwr,*) 'msh665: Case 5 ii2 =',ii2,' ii4 =',ii4
               write(irefwr,*) 'New ratio = ',min(rat1,rat2)
               call msh632(coor,i1,i2,i3,ii2,vol)
               vol1 = vol
               call msh632(coor,i1,i3,i4,ii4,vol)
               vol1 = vol + vol1
               call msh632(coor,i1,i2,i4,ii2,vol)
               vol2 = vol
               call msh632(coor,i2,i3,i4,ii4,vol)
               vol2 = vol2 + vol
               call msh632(coor,i1,i2,i3,i4,vol)
               vol2 = vol2 + vol
               write(irefwr,*) 'Volumes New = ',vol1,' Old = ',vol2

            end if

            ichng = 1
            ja    = 1

         end if

      end if

      if ( ii3==ii4 .and. ii3/=0 .and. ichng==0 ) then

         call msh661( coor, i1, i2, i3, ii3, rat1 )
         call msh661( coor, i1, i4, i2, ii4, rat2 )

         if ( min(rat1,rat2)>ratio ) then

            kelem(1,jel3) = i1
            kelem(2,jel3) = i2
            kelem(3,jel3) = i3
            kelem(4,jel3) = ii3

            kelem(1,jel4) = i1
            kelem(2,jel4) = i4
            kelem(3,jel4) = i2
            kelem(4,jel4) = ii4

            if ( debug ) then

               write(irefwr,*) 'msh665: Case 6 ii3 =',ii3,' ii4 =',ii4
               write(irefwr,*) 'New ratio = ',min(rat1,rat2)
               call msh632(coor,i1,i2,i3,ii3,vol)
               vol1 = vol
               call msh632(coor,i1,i4,i2,ii4,vol)
               vol1 = vol + vol1
               call msh632(coor,i3,i1,i4,ii3,vol)
               vol2 = vol
               call msh632(coor,i3,i4,i2,ii4,vol)
               vol2 = vol2 + vol
               call msh632(coor,i1,i2,i3,i4,vol)
               vol2 = vol2 + vol
               write(irefwr,*) 'Volumes: New = ',vol1,' Old = ',vol2

            end if

            ichng = 1
            ja    = 1

         end if

      end if
      ! --- Initialize extra values:
      iel12i1i2 = 0
      iel13i1i3 = 0
      iel14i2i3 = 0
      iel23i1i4 = 0
      iel24i2i4 = 0
      iel34i3i4 = 0

      if ( ichng==0 .and. ii1/=ii2 .and. ii1/=0 .and. ii2/=0 ) then

!     --- Find better division for four tetraeders
!         Run through all elements and find elementnumber of
!         i1-i2-ii2-ii1 if it exists:

         do ielem = 1, nelem
            if ( ichng==0 ) then
!           --- Find element to be replaced
               ia = kelem(1,ielem)
               ib = kelem(2,ielem)
               ic = kelem(3,ielem)
               id = kelem(4,ielem)

               j1 = 0
               j2 = 0
               j3 = 0
               j4 = 0

               if ( i1==ia.or. i1==ib.or. i1==ic.or. i1==id) j1 = 1
               if ( i2==ia.or. i2==ib.or. i2==ic.or. i2==id) j2 = 1
               if (ii1==ia.or.ii1==ib.or.ii1==ic.or.ii1==id) j3 = 1
               if (ii2==ia.or.ii2==ib.or.ii2==ic.or.ii2==id) j4 = 1

               if ( j1+j2+j3+j4==4 ) then
                  ! Element found, consider alternative divisions:
                  ! Combination of jelem, jel1, jel2 and ielem
                  iiel1 = jelem
                  iiel2 = jel1
                  iiel3 = jel2
                  iiel4 = ielem
                  if ( debug ) then
                     write(irefwr,*) 'msh666: case1 new'
                     write(irefwr,*) 'nodes: ii1 and ii2 out of'
                     write(irefwr,*) ii1,ii2,ii3,ii4
                     write(irefwr,*) 'old ratio = ',ratio
                  end if
                  call msh666( iiel1, iiel2, iiel3, iiel4, kelem, &
                               coor, i3,i4,ii1,ii2,i1,i2, ichng )

                  if ( ichng==0 .and. 0>1 ) then
                     if ( debug ) write(irefwr,*) 'msh678 Case 1'
                     call msh678( iiel1, iiel2, iiel3, iiel4, kelem, &
                                  coor, i3,i4,ii1,ii2,i1,i2, &
                                  npoint, nelem, ichng )
                  end if

                  if ( ichng==0 ) then
!                 --- Element still exists no change yet:
                     iel12i1i2 = ielem
                  else
                     ja = 2
                  end if
               end if
            end if
         end do
      end if

      if ( ichng==0 .and. ii1/=ii3 .and. ii1/=0 .and. ii3/=0 ) then
!     --- Find better division for four tetraeders
!         Run through all elements and find elementnumber of
!         i1-i3-ii1-ii3
         do ielem = nelmfix+1, nelem
            if ( ichng==0 ) then
!           --- Find element to be replaced
               ia = kelem(1,ielem)
               ib = kelem(2,ielem)
               ic = kelem(3,ielem)
               id = kelem(4,ielem)

               j1 = 0
               j2 = 0
               j3 = 0
               j4 = 0

               if ( i1==ia.or. i1==ib.or. i1==ic.or. i1==id) j1 = 1
               if ( i3==ia.or. i3==ib.or. i3==ic.or. i3==id) j2 = 1
               if (ii1==ia.or.ii1==ib.or.ii1==ic.or.ii1==id) j3 = 1
               if (ii3==ia.or.ii3==ib.or.ii3==ic.or.ii3==id) j4 = 1

               if ( j1+j2+j3+j4==4 ) then
                  ! Element found, consider alternative divisions:
                  ! Combination of jelem, jel1, jel3 and ielem
                  iiel1 = jelem
                  iiel2 = jel1
                  iiel3 = jel3
                  iiel4 = ielem

                  if ( debug ) then
                     write(irefwr,*) 'msh666: case 2 new'
                     write(irefwr,*) 'Nodes: ii1 and ii3 out of'
                     write(irefwr,*) ii1,ii2,ii3,ii4
                     write(irefwr,*) 'old ratio = ',ratio
                  end if

                  call msh666( iiel1, iiel2, iiel3, iiel4, kelem, &
                               coor, i4,i2,ii3,ii1,i1,i3, ichng )

                  if ( ichng==0 .and. 0>1 ) then
                     call msh678( iiel1, iiel2, iiel3, iiel4, kelem, &
                                  coor, i4,i2,ii3,ii1,i1,i3, &
                                  npoint, nelem, ichng )
                  end if

                  if ( ichng==0 ) then
!                 --- Element exists but no changings yet
                     iel13i1i3 = ielem
                  else
                     ja = 2
                  end if
               end if
            end if
         end do
      end if

      if ( ichng==0 .and. ii1/=ii4 .and. ii1/=0 .and. ii4/=0 ) then
!     --- Find better division for four tetraeders
!         Run through all elements and find elementnumber of
!         i2-i3-ii4-ii1 if it exists:
         do ielem = nelmfix+1, nelem
            if ( ichng==0 ) then
!           --- Find element to be replaced
               ia = kelem(1,ielem)
               ib = kelem(2,ielem)
               ic = kelem(3,ielem)
               id = kelem(4,ielem)

               j1 = 0
               j2 = 0
               j3 = 0
               j4 = 0

               if ( i2==ia.or. i2==ib.or. i2==ic.or. i2==id) j1 = 1
               if ( i3==ia.or. i3==ib.or. i3==ic.or. i3==id) j2 = 1
               if (ii4==ia.or.ii4==ib.or.ii4==ic.or.ii4==id) j3 = 1
               if (ii1==ia.or.ii1==ib.or.ii1==ic.or.ii1==id) j4 = 1

               if ( j1+j2+j3+j4==4 ) then
!              --- Element found, consider alternative divisions:
!                  Combination of jelem, jel1, jel4 and ielem

                  iiel1 = jelem
                  iiel2 = jel1
                  iiel3 = jel4
                  iiel4 = ielem

                  if ( debug ) then
                     write(irefwr,*) 'msh666: case 3 new'
                     write(irefwr,*) 'Nodes: ii1 and ii4 out of'
                     write(irefwr,*) ii1,ii2,ii3,ii4
                     write(irefwr,*) 'old ratio = ',ratio
                  end if

                  call msh666( iiel1, iiel2, iiel3, iiel4, kelem, &
                               coor, i1,i4,ii1,ii4,i2,i3, ichng )

                  if ( ichng==0 .and. 0>1 ) then
                     if ( debug ) write(irefwr,*) 'msh678 Case 3'
                     call msh678( iiel1, iiel2, iiel3, iiel4, kelem, &
                                  coor, i1,i4,ii1,ii4,i2,i3, &
                                  npoint, nelem, ichng )
                  end if

                  if ( ichng==0 ) then
!                 --- Element exists but no changings yet:
                     iel14i2i3 = ielem
                  else
                     ja = 2
                  end if
               end if
            end if
         end do
      end if

      if ( ichng==0 .and. ii2/=ii3 .and. ii2/=0 .and. ii3/=0 ) then
!     --- Find better division for four tetraeders
!         extra basiselement is i1-i4-ii3-ii2
!         --- Run through all elements and find elementnumber of
!             i1-i4-ii3-ii2 if it exists:
         do ielem = nelmfix+1, nelem
            if ( ichng==0 ) then
!           --- Find element to be replaced
               ia = kelem(1,ielem)
               ib = kelem(2,ielem)
               ic = kelem(3,ielem)
               id = kelem(4,ielem)

               j1 = 0
               j2 = 0
               j3 = 0
               j4 = 0

               if ( i1==ia.or. i1==ib.or. i1==ic.or. i1==id) j1 = 1
               if ( i4==ia.or. i4==ib.or. i4==ic.or. i4==id) j2 = 1
               if (ii2==ia.or.ii2==ib.or.ii2==ic.or.ii2==id) j3 = 1
               if (ii3==ia.or.ii3==ib.or.ii3==ic.or.ii3==id) j4 = 1

               if ( j1+j2+j3+j4==4 ) then
!              --- Element found, consider alternative divisions:
!                  Combination of jelem, jel2, jel3 and ielem
                  iiel1 = jelem
                  iiel2 = jel2
                  iiel3 = jel3
                  iiel4 = ielem

                  if ( debug ) then
                     write(irefwr,*) 'msh666: case 4 new'
                     write(irefwr,*) 'Nodes: ii2 and ii3 out of'
                     write(irefwr,*) ii1,ii2,ii3,ii4
                     write(irefwr,*) 'old ratio = ',ratio
                  end if

                  call msh666( iiel1, iiel2, iiel3, iiel4, kelem, &
                               coor, i2,i3,ii2,ii3,i1,i4, ichng )

                  if ( ichng==0 .and. 0>1 ) then
                     if ( debug ) write(irefwr,*) 'msh678 Case 4'
                     call msh678( iiel1, iiel2, iiel3, iiel4, kelem, &
                                  coor, i2,i3,ii2,ii3,i1,i4, &
                                  npoint, nelem, ichng )
                  end if

                  if ( ichng==0 ) then
!                 --- Element exists but no changings yet:
                     iel23i1i4 = ielem
                  else
                       ja = 2
                  end if
               end if
            end if
         end do
      end if

      if ( ichng==0 .and. ii2/=ii4 .and. ii2/=0 .and. ii4/=0 ) then
!     --- Find better division for four tetraeders
!         Run through all elements and find elementnumber of
!         i2-i4-ii2-ii4 if it exists:
         do ielem = nelmfix+1, nelem
            if ( ichng==0 ) then
!           --- Find element to be replaced
               ia = kelem(1,ielem)
               ib = kelem(2,ielem)
               ic = kelem(3,ielem)
               id = kelem(4,ielem)

               j1 = 0
               j2 = 0
               j3 = 0
               j4 = 0

               if ( i2==ia.or. i2==ib.or. i2==ic.or. i2==id) j1 = 1
               if ( i4==ia.or. i4==ib.or. i4==ic.or. i4==id) j2 = 1
               if (ii2==ia.or.ii2==ib.or.ii2==ic.or.ii2==id) j3 = 1
               if (ii4==ia.or.ii4==ib.or.ii4==ic.or.ii4==id) j4 = 1

               if ( j1+j2+j3+j4==4 ) then
!              --- Element found, consider alternative divisions:
!                  Combination of jelem, jel2, jel4 and ielem
                  iiel1 = jelem
                  iiel2 = jel2
                  iiel3 = jel4
                  iiel4 = ielem

                  if ( debug ) then
                     write(irefwr,*) 'msh666: case 5 new'
                     write(irefwr,*) 'Nodes: ii2 and ii4 out of'
                     write(irefwr,*) ii1,ii2,ii3,ii4
                     write(irefwr,*) 'old ratio = ',ratio
                  end if

                  call msh666( iiel1, iiel2, iiel3, iiel4, kelem, &
                               coor, i3,i1,ii4,ii2,i2,i4, ichng )

                  if ( ichng==0 .and. 0>1 ) then
                     if ( debug ) write(irefwr,*) 'msh678 Case 5'
                     call msh678( iiel1, iiel2, iiel3, iiel4, kelem, &
                                  coor, i3,i1,ii4,ii2,i2,i4, &
                                  npoint, nelem, ichng )
                  end if

                  if ( ichng==0 ) then
!                 --- Element exists but no changings yet:
                     iel24i2i4 = ielem
                  else
                     ja = 2
                  end if
               end if
            end if
         end do
      end if

      if ( ichng==0 .and. ii3/=ii4 .and. ii3/=0 .and. ii4/=0 ) then
!     --- Find better division for four tetraeders
!         Run through all elements and find elementnumber of
!         i3-i4-ii4-ii3
         do ielem = nelmfix+1, nelem
            if ( ichng==0 ) then
!           --- Find element to be replaced
               ia = kelem(1,ielem)
               ib = kelem(2,ielem)
               ic = kelem(3,ielem)
               id = kelem(4,ielem)

               j1 = 0
               j2 = 0
               j3 = 0
               j4 = 0

               if ( i3==ia.or. i3==ib.or. i3==ic.or. i3==id) j1 = 1
               if ( i4==ia.or. i4==ib.or. i4==ic.or. i4==id) j2 = 1
               if (ii3==ia.or.ii3==ib.or.ii3==ic.or.ii3==id) j3 = 1
               if (ii4==ia.or.ii4==ib.or.ii4==ic.or.ii4==id) j4 = 1

               if ( j1+j2+j3+j4==4 ) then
!              --- Element found, consider alternative divisions:
!                  Combination of jelem, jel3, jel4 and ielem
                  iiel1 = jelem
                  iiel2 = jel3
                  iiel3 = jel4
                  iiel4 = ielem

                  if ( debug ) then
                     write(irefwr,*) 'msh666: case 6 new'
                     write(irefwr,*) 'Nodes: ii3 and ii4 out of'
                     write(irefwr,*) ii1,ii2,ii3,ii4
                     write(irefwr,*) 'old ratio = ',ratio
                  end if

                  call msh666( iiel1, iiel2, iiel3, iiel4, kelem, &
                               coor, i1,i2,ii3,ii4,i3,i4, ichng )

                  if ( ichng==0 .and. 0>1 ) then
                     if ( debug ) write(irefwr,*) 'msh678 Case 6'
                     call msh678( iiel1, iiel2, iiel3, iiel4, kelem, &
                                  coor, i1,i2,ii3,ii4,i3,i4, &
                                  npoint, nelem, ichng )
                  end if

                  if ( ichng==0 ) then
!                 --- Element exists but no changings yet
                     iel34i3i4 = ielem
                  else
                     ja = 2
                  end if
               end if
            end if
         end do
      end if

!     --- Check existence of number of tetraeders:

      ik = 0

      if ( iel12i1i2>0 ) ik = ik + 1
      if ( iel13i1i3>0 ) ik = ik + 1
      if ( iel14i2i3>0 ) ik = ik + 1
      if ( iel23i1i4>0 ) ik = ik + 1
      if ( iel24i2i4>0 ) ik = ik + 1
      if ( iel34i3i4>0 ) ik = ik + 1

      call msh661( coor, i1 , i2 , i3 , i4 , ratio  )

      if ( ichng == 0 .and. ik == 1 .and. ratio < 0.15d0 ) then
         if ( debug ) then
            write(irefwr,*) 'check possibility eight tetrahedrons'
            write(irefwr,*) i1,i2,i3,i4,' ratio ',ratio
         end if

         if ( iel12i1i2>0 ) then
            if ( debug ) write(irefwr,*) 'msh678 Case 1'
            call msh678( iiel1, iiel2, iiel3, iiel4, kelem, &
                         coor, i3,i4,ii1,ii2,i1,i2, &
                         npoint, nelem, ichng )
         else if ( iel13i1i3>0 ) then
            if ( debug ) write(irefwr,*) 'msh678 Case 2'
            call msh678( iiel1, iiel2, iiel3, iiel4, kelem, &
                         coor, i4,i2,ii3,ii1,i1,i3, &
                         npoint, nelem, ichng )
         else if ( iel14i2i3>0 ) then
            if ( debug ) write(irefwr,*) 'msh678 Case 3'
            call msh678( iiel1, iiel2, iiel3, iiel4, kelem, &
                         coor, i1,i4,ii1,ii4,i2,i3, &
                         npoint, nelem, ichng )
         else if ( iel23i1i4>0 ) then
            if ( debug ) write(irefwr,*) 'msh678 Case 4'
            call msh678( iiel1, iiel2, iiel3, iiel4, kelem, &
                         coor, i2,i3,ii2,ii3,i1,i4, &
                         npoint, nelem, ichng )
         else if ( iel24i2i4>0 ) then
            if ( debug ) write(irefwr,*) 'msh678 Case 5'
            call msh678( iiel1, iiel2, iiel3, iiel4, kelem, &
                         coor, i3,i1,ii4,ii2,i2,i4, &
                         npoint, nelem, ichng )
         else if ( iel34i3i4>0 ) then
            if ( debug ) write(irefwr,*) 'msh678 Case 6'
            call msh678( iiel1, iiel2, iiel3, iiel4, kelem, &
                         coor, i1,i2,ii3,ii4,i3,i4, &
                         npoint, nelem, ichng )
         end if
      end if

      if ( ichng==0 .and. ik>1 ) then
!     --- Not yet changed: find ratio of bad tet:
         call msh661( coor, i1, i2, i3, i4, ratio )

!        --- Check for combinations of six tetrahedrons:

         if ( ichng==0 .and. ratio<0.2d0 .and. &
              iel12i1i2>0 .and. iel34i3i4>0 ) then

!        --- Try new division using barycenter and 12 tetrahedrons

            if ( debug ) then
               write(irefwr,*) 'New division: iel12i1i2 en iel34i3i4'
               write(irefwr,*) 'basetet',i1,i2,i3,i4
               write(irefwr,*) 'jel1',(kelem(il,jel1),il=1,4)
               write(irefwr,*) 'jel2',(kelem(il,jel2),il=1,4)
               ielem = iel12i1i2
               write(irefwr,*) 'i1i2',(kelem(il,ielem),il=1,4)
               write(irefwr,*) 'Combined with'
               write(irefwr,*) 'jel3',(kelem(il,jel3),il=1,4)
               write(irefwr,*) 'jel4',(kelem(il,jel4),il=1,4)
               ielem = iel34i3i4
               write(irefwr,*) 'i3i4',(kelem(il,ielem),il=1,4)
               write(irefwr,*) 'Check'
            end if
            il = nelem
            call msh675( kelem, nelem, jelem, jel1, jel2, jel3, jel4, &
                         iel12i1i2, iel34i3i4, coor, npoint )
            if ( nelem>il ) ichng = 3
         end if
         if ( ichng==0 .and. ratio<0.2d0 .and. &
              iel13i1i3>0 .and. iel24i2i4>0 ) then

!        --- Try new division using barycenter and 12 tetrahedrons

            if ( debug ) then
               write(irefwr,*) 'New division: iel13i1i3 en iel24i2i4'
               write(irefwr,*) 'basetet',i1,i2,i3,i4
               write(irefwr,*) 'jel1',(kelem(il,jel1),il=1,4)
               write(irefwr,*) 'jel3',(kelem(il,jel3),il=1,4)
               ielem = iel13i1i3
               write(irefwr,*) 'i1i3',(kelem(il,ielem),il=1,4)
               write(irefwr,*) 'Combined with'
               write(irefwr,*) 'jel2',(kelem(il,jel2),il=1,4)
               write(irefwr,*) 'jel4',(kelem(il,jel4),il=1,4)
               ielem = iel24i2i4
               write(irefwr,*) 'i2i4',(kelem(il,ielem),il=1,4)
               write(irefwr,*) 'Check'
            end if
            il = nelem
            call msh673( kelem, nelem, jelem, jel1, jel2, jel3, jel4, &
                         iel13i1i3, iel24i2i4, coor, npoint )
            if ( nelem>il ) ichng = 3
         end if
         if ( ichng==0 .and. ratio<0.2d0 .and. &
              iel14i2i3>0 .and. iel23i1i4>0 ) then

!        --- Try new division using barycenter and 12 tetrahedrons

            if ( debug ) then
               write(irefwr,*) 'New division: iel14i2i3 en iel23i1i4'
               write(irefwr,*) 'basetet',i1,i2,i3,i4
               write(irefwr,*) 'jel1',(kelem(il,jel1),il=1,4)
               write(irefwr,*) 'jel4',(kelem(il,jel4),il=1,4)
               ielem = iel14i2i3
               write(irefwr,*) 'i2i3',(kelem(il,ielem),il=1,4)
               write(irefwr,*) 'Combined with'
               write(irefwr,*) 'jel2',(kelem(il,jel2),il=1,4)
               write(irefwr,*) 'jel3',(kelem(il,jel3),il=1,4)
               ielem = iel23i1i4
               write(irefwr,*) 'i1i4',(kelem(il,ielem),il=1,4)
               write(irefwr,*) 'Check'
            end if
            il = nelem
            call msh674( kelem, nelem, jelem, jel1, jel2, jel3, jel4, &
                         iel14i2i3, iel23i1i4, coor, npoint )
            if ( nelem>il ) ichng = 3
         end if

      end if

!     --- Finally, if needed, remove "bad" element:

      if ( ja==1 ) then
         do ielem = jelem, nelem-1
            kelem(1,ielem) = kelem(1,ielem+1)
            kelem(2,ielem) = kelem(2,ielem+1)
            kelem(3,ielem) = kelem(3,ielem+1)
            kelem(4,ielem) = kelem(4,ielem+1)
         end do
         nelem = nelem - 1
      end if

1000  call erclos ( 'msh665' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Final check at end of msh665'
         write(irefwr,*) 'volumes and ratios'
         vol = 0d0
         ia  = 0
         do ielem = 1, nelem
            i1 = kelem(1,ielem)
            i2 = kelem(2,ielem)
            i3 = kelem(3,ielem)
            i4 = kelem(4,ielem)

            call msh661( coor, i1,i2,i3,i4, ratio )
            call msh632( coor, i1,i2,i3,i4, vol1   )

            if ( vol1<=0 ) then
               write(irefwr,*) 'element',ielem,'vol1 =',vol1,'rat',ratio
               ia = 1
            end if
            vol = vol + vol1
         end do

         if ( ia==1 ) write(irefwr,*) 'End of msh665, volume = ',vol/6d0
      end if

      end
