      subroutine msh678 ( jel1, jel2, jel3, jel4, kelem, coor, &
                          i2, i3, ii2, ii3, i1, i4, npoint, &
                          nelem, isuccs )
! ======================================================================
!
!        programmer    niek praagman
!        version  1.0  date 22-08-2011
!
!   copyright (c) 2011-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check whether a "bad" division of four coupled tetrahedrons with a
!     closed boundary of eight connected triangles can be improved for
!     the overall ratio's by adding the barycenter and constructing eight
!     new tets. If so store the four of the new elements on the old
!     positions and add four new tets in kelem
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh_generation
!     3d
!     adjustment
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer jel1, jel2, jel3, jel4, kelem(4,*), &
              i1, i2, i3, i4, ii2, ii3, isuccs, nelem, npoint
      double precision coor(3,*)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     i1             i    point of original "flat" element
!     i2             i    point of original "flat" element
!     i3             i    point of original "flat" element
!     i4             i    point of original "flat" element
!     ii1            i    node number of extra edge extra element
!     ii2            i    node number of extra edge extra element
!     isuccs         o    indicator new elements (==1) or not (==0)
!     jel1           i    element position in array kelem to be refilled
!     jel2           i    element position in array kelem to be refilled
!     jel3           i    element position in array kelem to be refilled
!     jel4           i    element position in array kelem to be refilled
!     kelem         i/o   New KMESH part c with respect to the surface elements
!                         All elements have the same number of nodes
!     nelem         i/o   actual number of elements in mesharray kelem
!     npoint        i/o   actual number of nodes in coordinatearray coor
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer  il, j
      double precision ratio, rat1, rat2, vol, vol1, vol2

!     il             loop variable
!     j              loop variable
!     rat1           help ratio for division
!     rat2           help ratio for division
!     ratio          ratio to be improved
!     vol            helpvariable to store volume
!     vol1           helpvariable to store first alternative volume
!     vol2           helpvariable to store second alternative volume
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     MSH632         compute volume of tetrahedron
!     MSH661         Compute ratio of inner and outer sphere of tetrahedron
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Compare the total volumes of the old four and the new eight elements
!    If the volumes are the same and the overall ratio improves then
!    Refill the coordinate and element positions in a straightforward way
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh678' )
      debug = .false.
      if ( debug ) then
!     --- Debug information
         write(irefwr,*) 'Debug information from msh678'
      end if
      if ( ierror/=0 ) go to 1000
!     --- Assume positive result
      isuccs = 1
!     --- Compute old ratio
      call msh661( coor, i1, i2, i3, i4, ratio )
!     --- Try new division using four tetrahedrons:
      if ( debug ) then
         write(irefwr,*) 'New division:'
         write(irefwr,*) 'basetet',i1,i2,i3,i4,'ratio =',ratio
         write(irefwr,*) 'jel1',(kelem(il,jel1),il=1,4)
         write(irefwr,*) 'jel2',(kelem(il,jel2),il=1,4)
         write(irefwr,*) 'jel3',(kelem(il,jel3),il=1,4)
         write(irefwr,*) 'jel4',(kelem(il,jel4),il=1,4)
!        --- Make a redivision for these four tets:
         write(irefwr,*) 'elementnumbers',jel1,jel2,jel3,jel4
         write(irefwr,*) 'nodes',i2,i3,ii2,ii3,i1,i4
      end if
!     --- First compute old volume and old overall ratio:
      vol1 = 0d0
      rat1 = 1d0
      call msh632( coor, i1 , i2 , i3 , i4 , vol   )
      call msh661( coor, i1 , i2 , i3 , i4 , ratio )
      vol1 = vol1 + vol
      if ( ratio < rat1 ) rat1 = ratio
      call msh632( coor, i3 , i4 , ii3, i1 , vol   )
      call msh661( coor, i3 , i4 , ii3, i1 , ratio )
      if ( ratio < rat1 ) rat1 = ratio
      vol1 = vol1 + vol
      call msh632( coor, i1 , i2 , i4 , ii2, vol   )
      call msh661( coor, i1 , i2 , i4 , ii2, ratio )
      if ( ratio < rat1 ) rat1 = ratio
      vol1 = vol1 + vol
      call msh632( coor, ii2, i1 , ii3, i4 , vol   )
      call msh661( coor, ii2, i1 , ii3, i4 , ratio )
      if ( ratio < rat1 ) rat1 = ratio
      vol1 = vol1 + vol
      if ( debug ) write(irefwr,*) 'vol1 = ',vol1,' rat1 = ',rat1
!     --- Place barycenter:
      do j=1,3
         coor(j,npoint+1) = &
             ( coor(j,i1) + coor(j,i2) + coor(j,i3) + coor(j,i4) + &
               coor(j,ii2)+ coor(j,ii3) ) / 6d0
      end do
!     --- Next: find eight boundary triangles and corresponding new tets
      vol2 = 0d0
      rat2 = 1d0
      call msh661( coor, i1 , i2 , i3 , npoint+1, ratio )
      call msh632( coor, i1 , i2 , i3 , npoint+1, vol   )
      if ( debug ) write(irefwr,*) 'vol = ',vol,' ratio = ',ratio
      vol2 = vol2 + abs(vol)
      if ( ratio < rat2 ) rat2 = ratio
      call msh661( coor, i3 , i2 , i4 , npoint+1, ratio )
      call msh632( coor, i3 , i2 , i4 , npoint+1, vol   )
      if ( debug ) write(irefwr,*) 'vol = ',vol,' ratio = ',ratio
      vol2 = vol2 + abs(vol)
      if ( ratio < rat2 ) rat2 = ratio
      call msh661( coor, i3 , i4 , ii3, npoint+1, ratio )
      call msh632( coor, i3 , i4 , ii3, npoint+1, vol   )
      if ( debug ) write(irefwr,*) 'vol = ',vol,' ratio = ',ratio
      vol2 = vol2 + abs(vol)
      if ( ratio < rat2 ) rat2 = ratio
      call msh661( coor, i1 , ii2, i2 , npoint+1, ratio )
      call msh632( coor, i1 , ii2, i2 , npoint+1, vol   )
      if ( debug ) write(irefwr,*) 'vol = ',vol,' ratio = ',ratio
      vol2 = vol2 + abs(vol)
      if ( ratio < rat2 ) rat2 = ratio
      call msh661( coor, i1 , ii3, ii2, npoint+1, ratio )
      call msh632( coor, i1 , ii3, ii2, npoint+1, vol   )
      if ( debug ) write(irefwr,*) 'vol = ',vol,' ratio = ',ratio
      vol2 = vol2 + abs(vol)
      if ( ratio < rat2 ) rat2 = ratio
      call msh661( coor, i1 , i3 , ii3, npoint+1, ratio )
      call msh632( coor, i1 , i3 , ii3, npoint+1, vol   )
      if ( debug ) write(irefwr,*) 'vol = ',vol,' ratio = ',ratio
      vol2 = vol2 + abs(vol)
      if ( ratio < rat2 ) rat2 = ratio
      call msh661( coor, ii2, i4 , i2 , npoint+1, ratio )
      call msh632( coor, ii2, i4 , i2 , npoint+1, vol   )
      if ( debug ) write(irefwr,*) 'vol = ',vol,' ratio = ',ratio
      vol2 = vol2 + abs(vol)
      if ( ratio < rat2 ) rat2 = ratio
      call msh661( coor, ii2, ii3, i4 , npoint+1, ratio )
      call msh632( coor, ii2, ii3, i4 , npoint+1, vol   )
      if ( debug ) write(irefwr,*) 'vol = ',vol,' ratio = ',ratio
      vol2 = vol2 + abs(vol)
      if ( ratio < rat2 ) rat2 = ratio
      if ( debug ) then
         write(irefwr,*) 'vol2 = ',vol2,' rat2 = ',rat2
         write(irefwr,*) 'difference vol1 - vol2',vol1-vol2
      end if

      if ( abs(vol1 - vol2) < 1d-10 *abs(vol1) &
                .and. rat2 > rat1 ) then
!     --- New point and new elements:
         npoint = npoint + 1
!        --- Elmts:
         kelem(1,jel1) = i1
         kelem(2,jel1) = i2
         kelem(3,jel1) = i3
         kelem(4,jel1) = npoint

         kelem(1,jel2) = i3
         kelem(2,jel2) = i2
         kelem(3,jel2) = i4
         kelem(4,jel2) = npoint

         kelem(1,jel3) = i3
         kelem(2,jel3) = i4
         kelem(3,jel3) = ii3
         kelem(4,jel3) = npoint

         kelem(1,jel4) = i1
         kelem(2,jel4) = ii2
         kelem(3,jel4) = i2
         kelem(4,jel4) = npoint

!        --- Four new elements:
         kelem(1,nelem+1) = i1
         kelem(2,nelem+1) = ii3
         kelem(3,nelem+1) = ii2
         kelem(4,nelem+1) = npoint

         kelem(1,nelem+2) = i1
         kelem(2,nelem+2) = i3
         kelem(3,nelem+2) = ii3
         kelem(4,nelem+2) = npoint

         kelem(1,nelem+3) = ii2
         kelem(2,nelem+3) = i4
         kelem(3,nelem+3) = i2
         kelem(4,nelem+3) = npoint

         kelem(1,nelem+4) = ii2
         kelem(2,nelem+4) = ii3
         kelem(3,nelem+4) = i4
         kelem(4,nelem+4) = npoint

         nelem = nelem + 4
      else
!     --- Keep old division
         isuccs = 0
      end if

1000  call erclos ( 'msh678' )

      debug = .false.
      if ( debug ) then
!     --- Debug information
         write(irefwr,*) 'End msh678'
      end if

      end
