      subroutine sepbuild ( iinbld, matr, intmat, kmesh, kprob, &
                            irhsd, massmt, iusol, islold, iuser, user )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 31-07-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     the large matrix and vector are computed
!     the effect of boundary conditions is taken into account
! **********************************************************************
!
!                       KEYWORDS
!
!     diagonal_matrix
!     mass_matrix
!     right_hand_side
!     stiffness_matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuf
      use sepmodulebuild
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cdebug_routsf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinbld(*), matr(5,*), intmat(5,*), kmesh(*), kprob(*), &
              irhsd(5,*), iusol(5), iuser(*), islold(5,*), massmt(*)
      double precision user(*)

!    iinbld    i   In this array it is indicated how the matrix and right-hand
!                  side, must be filled.
!                  IINBLD must be filled as follows
!                   1 Number of entries of IINBLD that are filled by the user.
!                     If IINBLD(1) = 0, only defaults are used
!                   2 JCHOIS    general choice parameter, possibilities:
!                            1  Both the large matrix and large vector are built
!                            2  Only the large vector is built.
!                            3  The large matrix is built and the large vector
!                               is made equal to zero.
!                               If EFFBOUNCOND = 0 the large vector may be
!                               unequal to zero, due to
!                               non-homogeneous essential boundary conditions.
!                            4  No large matrix nor large vector is build
!                            5  The large matrix is not built;
!                               the large vector is made equal to zero.
!                               If EFFBOUNCOND = 0 the large vector may be
!                               unequal to zero, due to
!                               non-homogeneous essential boundary conditions.
!                            6  The large matrix is not built;
!                               The large vector is created by multiplication of
!                               the matrix consisting of all boundary elements
!                               by the solution vector
!                               In this way an extra term for the reaction force
!                               is created
!                           14  The large matrix is built;
!                               no right-hand-side vector is made.
!                               Default value: 1
!                   3 EFFBOUNCOND   Indication whether the effect of essential
!                               boundary  conditions must be taken into
!                               account (0) or not (1).
!                               Default value: 0
!                   4 IMAS      Indication if the mass matrix must be computed
!                               (>1) or not (0). The storage of the mass matrix
!                               depends on the parameter IMAS in the following
!                               way:
!                               0: No mass matrix
!                               1: The mass matrix is stored as a diagonal
!                                  matrix
!                               2: The mass matrix is stored as a large matrix
!                                  with exactly the same structure as the
!                                  stiffness matrix
!                               3: The mass matrix is stored as a large matrix, &
!                                  however, its storage scheme may differ
!                                  from the one used for the stiffness matrix
!                                  The structure of this matrix must be
!                                  stored in INTMAT(.,2)
!                               Default value: 0
!                   5 IPREC     Indication if the old solution is stored in
!                               ISLOLD (1) or not (0)
!                               IPREC is not used if IINBLD(10)>0
!                               Default value: 0
!                   6 ALLELEMENTS  Indication if all standard element
!                               sequence numbers are used or not
!                               Possible values:
!                               -2: skip all internal elements
!                               -1: skip all boundary elements
!                                0: do not skip any element
!                               >0 skip some element groups
!                               In that case the positions 16 to 15+NELGRP+ &
!                               NUMNATBND   (NELGRP is number of element groups
!                               and NUMNATBND the number of boundary element
!                               groups) are considered as an array IELHLP to be
!                               filled by the user in the following way:
!                               In IELHLP(IELGRP)  (1<=IELGRP<=NELGRP+NUMNATBND)
!                               the user must store, whether elements with
!                               standard element sequence number IELGRP (or
!                               IELGRP = NELGRP + IBNGRP for boundary elements)
!                               are added to the large matrix and large vector
!                               or not.
!                               When IELHLP(IELGRP) = 0 the elements with
!                               standard element sequence number IELGRP are
!                               skipped, when IELHLP(IELGRP) = 1 these elements
!                               are added.
!                               Default value: 0
!                   7 ADDMATVEC   When ADDMATVEC = 0 the large matrix and vector
!                               are cleared before the assembling process.
!                               When ADDMATVEC = 1 the right-hand-side vector
!                               and the large matrix are not cleared, &
!                               but added to the already existing right-hand
!                               side vector and large matrix.
!                               Hence        RHSD := RHSD + DELTA  RHSD
!                                            MATRIX := MATRIX +  DELTA  MATRIX
!                               Default value: 0
!                   8  CLEARLASTRHSD  When CLEARLASTRHSD = 0 the last part of
!                               the right-hand-side vector corresponding to the
!                               boundary conditions is cleared
!                               (standard situation)
!                               when CLEARLASTRHSD = 1 the standard assembling
!                               process is also applied to this last part of the
!                               right-hand-side vector. For most applications
!                               CLEARLASTRHSD = 0 suffices.
!                               Default value: 0
!                   9  IPROB    Problem definition number
!                               Default value: defined by INTMAT
!                  10  NUMOLD   Number of arrays that have been filled in array
!                               ISLOLD
!                               Default value: 0
!                  11  ISPECTRAL Indication if a spectral element method is used
!                               needing extra information (1) or not (0)
!                               Default value: 0
!                  12  NRIGHTHS Number of right-hand sides that must be computed
!                               by subroutine BUILD in this call. This parameter
!                               is only used if JCHOIS = 1 or 2.
!                               For all these vectors the other parameters in
!                               IINBLD are used in exactly the same way.
!                               The information about the right-hand sides is
!                               stored in the following way:
!                               f  refers to rhsd(.,i)  + = 1 (1) 5
!                                i
!                               Default value: 1
!                  13  ISEQIN   Sequence number of old solution in array ISLOLD
!                               Default value: 1
!                  14  IDEF_COR indicates if defect correction should be used
!                               0: no defect correction
!                               1: defect correction
!                               Default value: 0
!                  15  ISUBDIVIDE indicates if quadratic elements must be
!                               treated as a conglomerate of linear elements (1)
!                               or as quadratic elements itself (0)
!                               Default value: 0
!    intmat    i   Standard SEPRAN array of length 5 containing information
!                  concerning the structure of the matrix. See the Programmer's
!                  Guide 13.5 for a complete description.
!                  In the case of a non-diagonal mass matrix array intmat
!                  must contain the information about the structure of
!                  the mass matrix in intmat(1-5,2)
!    irhsd     o   Standard SEPRAN array of length 5 containing information
!                  concerning the right-hand-side vector. See the Programmer's
!                  Guide 13.4 for a complete description.
!                  In case of defect correction the right-hand side consists of
!                  two parts:
!                  irhsd(.,i) refers to the right-hand side corresponding to
!                             matr(.,i)
!    islold    i   If an iterative procedure is used, for example for
!                  non-linear problems the solution in a preceding iteration
!                  may be necessary. In that case information concerning the
!                  preceding solution must be stored in ISLOLD.
!                  If the preceding solution is not necessary, a dummy parameter
!                  may be used.
!                  Another possibility is that coefficients depend on solution
!                  arrays or arrays of special stricture that have been computed
!                  before.
!                  ISLOLD is used if either IPREC = 1 or NUMOLD>0
!                  If NUMOLD=0 and IPREC=1 ISLOLD must be a solution vector
!                  of exactly the same type and corresponding to the same
!                  problem as ISOL
!                  If NUMOLD>0, ISLOLD must be a two-dimensional array of size
!                  (5,k), with k at least NUMOLD.
!                  The first NUMOLD vectors ISLOLD(.,i) must have been filled
!                  with solution vectors or vectors of special structure not
!                  necessarily corresponding to problem IPROB
!    iuser     i   In this array together with array user information concerning
!                  coefficients etc. for the differential equation must be
!                  stored.
!                  For standard problems that are contained in the library, &
!                  the arrays IUSER and USER must be filled according to
!                  fixed rules (see the manual Standard Problems).
!                  When the user uses his own elements he may fill and use
!                  IUSER and USER in his own way, since these arrays are only
!                  used by the element subroutines.
!    iusol     i   Standard SEPRAN array of length 5 containing information
!                  concerning the solution vector. See the Programmer's
!                  Guide 13.3 for a complete description.
!                  Essential boundary conditions must be filled in the
!                  solution vector corresponding to ISOL if necessary.
!                  (Depends on the value of ICHOIS).
!    kmesh     i   Standard SEPRAN array containing information of the mesh
!    kprob     i   Standard SEPRAN array containing information of the problem
!                  definition. See the Programmer's Guide 13.2 for a complete
!                  description
!    massmt    o   This array contains the Mass matrix at output, where the
!                  mass matrix is considered to be a diagonal matrix
!                  massmt is only filled when imas = 1
!    matr      o   Standard SEPRAN array of length 5 containing information
!                  concerning the large matrix. See the Programmer's
!                  Guide 13.6 for a complete description.
!                  In case of defect correction the matrix consists of two parts
!                  matr(.,1) refers to the matrix to be inverted for the defect
!                            correction, for example the upwind matrix
!                  matr(.,2) refers to the matrix for the correction in the
!                            defect correction, for example the central
!                            difference matrix
!    user      i   See array iuser
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lniincommat, nbld
      parameter ( lniincommat=25, nbld=15 )
      integer ihelp, nprob, k, iincommat(lniincommat)
      logical debug, changed
      integer jinbld(nbld)
      double precision timebefore(2)

!     changed        If true the position of lagrangian multipliers in ficitious
!                    domain elements has been changed
!     debug          If true debug statements are carried out otherwise
!                    they are not
!     ihelp          Starting address of array IELHLP in array IINBLD
!     iincommat      Input array for subroutine incommat
!     jinbld         Copy of iinbld with all defaults filled
!     k              General loop variable
!     lniincommat    Length of iincommat
!     nbld           Length of array jinbld
!     nprob          Number of various problems
!     timebefore     Time at entrance of subroutine
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer iniprb

!     ERCLMN         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERSETTIME      Get the CPU time at start of subroutine
!     INCOMMAT       compute integer information of the large matrix
!     INIPRB         Get value of ipkprb as function of iprob
!     INIPRI         Decrease priorities of arrays
!     PRININ         print 1d integer array
!     PRINMTBF       Print matrix
!     PRINTVBF       Print a SEPRAN array
!     PROBFICT       Store the fluid elements corresponding to each lambda point
!                    in array kprobfict
!     TO0050         Performs the actual building of matrix and right-hand side
!     TOADDRHSPAR    Add parts of right-hand-side at interface to lower blocks
!     TOBUILD        Perform those actions in build that are not carried out
!                    by TO0050
!     TORENUMSOL     Renumber solution array from standard sequence used to
!                    sequence used by matrix
!     TORENUMSOLBACK Renumber solution array from used by matrix back
!                    to standard sequence
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      26   Array KPROB has not been filled
!      75   Array KMESH has not been filled
!     699   Problem number incorrect
!    1246   IINBLD(1)<NELGRP + NUMNAT, whereas IINBLD(6)=1
!    1776   IINBLD(4) has wrong value
!    1777   INTMAT(2,2) has not been filled
!    1778   INTMAT(2,1) corresponds to the wrong problem number
!    1908   INTMAT(5,1) # INTMAT(5,2)
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Perform some tests
!    Compute starting address of kprob
!    If ( there are unknowns ) then
!       Compute starting address of array ihelp
!       Fill array jinbld with defaults
!       Perform some tests
!       Call actual building subroutine
!       Compute indprh
!       If ( there are global unknowns ) then
!          Build part of matrix and right-hand side corresponding to global
!          unknowns
!       renumber right-hand side
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepbuild' )
      debug = debug_buildbf
      if ( itime>0 ) call ersettime ( timebefore )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepbuild'
         write(irefwr,*) 'parallel ', parallel
         call prinin ( iinbld, iinbld(1), 'iinbld' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Perform some tests

      if ( kprob(2)/=101 ) then
         call errint ( kprob(1), 1 )
         call errsub ( 26, 1, 0, 0 )
      end if
      if ( kmesh(2)/=100 ) then
         call errint ( kmesh(1), 1 )
         call errsub ( 75, 1, 0, 0 )
      end if
      nprob = max ( 1, kprob(40) )
      if ( abs(intmat(2,1))/=102 ) then

!     --- Array intmat has not been filled, fill with default values
!         Detect iprob from iusol

         if ( iusol(2)==110 ) then
            iprob = iusol(4)/1000+1
         else if ( iinbld(1)>0 .and. iinbld(9)>0 ) then
            iprob = iinbld(9)
         else
            iprob = 1
         end if  ! ( iusol(2)==110 )

         iincommat = 0
         iincommat(2) = 2  ! jmethod
         iincommat(5) = iprob
         call incommat ( ibuffr, iincommat, kmesh, kprob, intmat )

      end if  ! ( intmat(2,1)/=102 )

      iprob = intmat(1,1)/1000 + 1
      if ( iprob<1 .or. iprob>nprob ) then
         call errint ( iprob, 1 )
         call errint ( nprob, 2 )
         call errsub ( 699, 2, 0, 0 )
      end if
      if ( ierror/=0 ) go to 1000

      if ( debug ) write(irefwr,1) 'iprob', iprob

!     --- Compute starting address of kprob

      ipkprb = iniprb(iprob,kprob)

!     --- If there are no unknowns the matrix must not be filled and
!         everything is skipped

      if ( kprob(ipkprb+5)-kprob(ipkprb+10)-kprob(ipkprb+34)==0 ) &
         go to 1000

      ihelp = 1
      if ( iinbld(1)>5 .and. iinbld(6)>=1 ) then

!     --- iinbld(6) = 1, hence array IELHLP should be stored from position
!                        16 to 15 + NELGRP + NUMNATBND

         if ( iinbld(1)<15+kmesh(5)+kprob(ipkprb+12) ) then

!        --- iinbld(1) too small to store IELHLP

            call errint ( iinbld(1), 1 )
            call errint ( 15+kmesh(5)+kprob(ipkprb+12), 2 )
            call errint ( kmesh(5), 3 )
            call errint ( kprob(ipkprb+12), 4 )
            call errsub ( 1246, 4, 0, 0 )

         else

            ihelp = 16

         end if

      end if
      if ( ierror/=0 ) go to 1000

!     --- Fill array jinbld with defaults

      jinbld(3:nbld) = 0
      jinbld(1) = nbld
      jinbld(2) = 1
      jinbld(12) = 1
      jinbld(13) = 1
      do k = 2, min(nbld,iinbld(1))
         if ( iinbld(k)/=0 ) jinbld(k) = iinbld(k)
      end do

!     --- Perform some tests

      imas = jinbld(4)
      if ( imas<0 .or. imas>3 ) then

!     --- Error 1776: imas has wrong value

         call errint ( imas, 1 )
         call errsub ( 1776, 1, 0, 0 )

      end if
      if ( imas==3 ) then

         if ( intmat(2,2)/=102 ) then

            call errint ( 2, 1 )
            call errint ( intmat(2,2), 2 )
            call errsub ( 1777, 2, 0, 0 )

         end if

         if ( intmat(1,2)/1000+1/=iprob ) then

            call errint ( 2, 1 )
            call errint ( intmat(1,2), 2 )
            call errint ( iprob, 3 )
            call errsub ( 1778, 3, 0, 0 )

         end if

         if ( mod(intmat(1,1),1000)==mod(intmat(1,2),1000) ) then

!        --- intmat(.,1) and intmat(.,2) correspond to the same storage
!            scheme, check if intmat(5,1) = intmat(5,2)

            if ( intmat(5,1)/=intmat(5,2) ) then

!           --- intmat(5,1) # intmat(5,2)

               call errint ( mod(intmat(1,1),1000), 1 )
               call errint ( intmat(1,1)/1000+1, 2 )
               call errint ( intmat(5,1), 3 )
               call errint ( intmat(5,2), 4 )
               call errsub ( 1908, 4, 0, 0 )

            end if

         end if

      end if
      if ( ierror/=0 ) go to 1000

!     --- Change matrix structure for moving fictitous elements

      if ( kprob(ipkprb+51)>0 ) then

!     --- fictitous elements present
!         Store the fluid elements corresponding to each lambda point in array
!         kprobfict
!         This is done because the position of the surface on which the
!         ficitious domain elements are lying may have been changed during
!         the computation

         jmethod = mod(intmat(1,iprob),100)
         changed = .false.
         call probfict ( ibuffr, buffer, kmesh, kprob, changed )
         if ( ierror/=0 ) go to 1000

!        --- recompute intmat if updated fictitious points are associated
!            with new fluid elements

         if ( changed ) then

!        --- fictituous points have been moved

            iincommat = 0
            iincommat(2) = jmethod
            iincommat(5) = iprob
            call incommat ( ibuffr, iincommat, kmesh, kprob, intmat )

         end if

      end if
      if ( ierror/=0 ) go to 1000

      if ( iinbld(2)==6 ) then

!     --- iinbld(2) = 6, reorder solution array to same sequence as in matrix

         call torenumsol ( ibuffr, buffer, iusol, kprob )

      end if  ! ( iinbld(2)==6 )

!     --- Call actual building subroutine

      addvec = .false.
      if ( debug ) write(irefwr,*) 'before to0050'
      if ( ierror==0 .and. imas>0 .or. jinbld(2)/=4 ) &
          call to0050 ( jinbld, matr, intmat, kmesh, &
                        kprob, irhsd, iusol, iuser, user, islold, &
                        iinbld(ihelp), massmt, 5 )
      if ( debug ) write(irefwr,*) 'after to0050'
      if ( ierror/=0 ) go to 1000

!     --- Rest of building of matrix

      call tobuild ( kmesh, kprob, intmat, matr, &
                     irhsd, iuser, user, iusol, jinbld, &
                     iinbld(ihelp) )
      if ( debug ) write(irefwr,*) 'after tobuild'

      if ( iinbld(2)==6 ) then

!     --- iinbld(2) = 6, order solution array back to old sequence

         call torenumsolback ( ibuffr, buffer, iusol, kprob )

      end if  ! ( iinbld(2)==6 )
      if ( debug ) write(irefwr,*) 'after torenumsolback'

      if ( parallel .and. kprob(69)>0 ) then

!     --- Parallel case, new storage
!         Add right-hand side from interface unknowns to lower blocks

         call toaddrhspar ( ibuffr, buffer, intmat, irhsd, kprob(ipkprb+1) )

      end if  ! ( parallel .and. kprob(69)>0 )
      call inipri

1000  call erclmn ( 'sepbuild', timebefore )
      if ( debug ) then

!     --- debug = true, print matrix and rhsd

         if ( irhsd(1,1)>0 ) &
            call printvbf ( ibuffr, buffer, irhsd, kmesh, kprob, 'rhs' )
         call prinmtbf ( ibuffr, buffer, intmat, matr, kprob )
         write(irefwr,*) 'End sepbuild'

      end if

      end
