      subroutine eroutsurfnod ( nnodes, nodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 27-09-2010 Make work arrays local
!        version  1.0  date 30-03-2007
!
!   copyright (c) 2007-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find all nodes in the outer surfaces and store in array nodes
!
! **********************************************************************
!
!                       KEYWORDS
!
!     node
!     surface
!     outer_boundary
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nnodes, nodes(*)

!     nnodes         o    Number of nodes in array nodes
!     nodes          o    Contains the node numbers of the outer boundary
!                         at output
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nparts, j, istart, nsurfaces, isurnr, &
              npointssurf, k, inodp, ipart
      integer, allocatable :: iwork(:), iwork1(:)

!     inodp          Nodal point number
!     ipart          Sequence number of part
!     istart         First position used at a part
!     isurnr         Surface sequence number
!     iwork          integer work array of length npoint
!                    Is filled by subroutine ERSURNOD:
!                    array containing the nodal point numbers of the
!                    surfes subsequently
!                    If point does not belong to a node, the address of
!                    the point with respect to surfes provided with a
!                    minus sign is stored
!     iwork1         integer work array of length npoint
!                    iwork1 is to mark points that have been stored
!     j              Counting variable
!     k              Counting variable
!     nparts         number of parts the boundary consists of
!     npointssurf    number of points in surface
!     nsurfaces      Number of surfaces in part
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPSURNOD      Store all nodes in the surfaces ifirst to ilast in array
!                    nodes
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'eroutsurfnod' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from eroutsurfnod'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( iwork(npoint), iwork1(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, 2*npoint, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Clear array iwork1

      iwork1 = 0

      nparts = outerboun(1)    ! number of parts in the outer boundary
      nnodes = 0

!     --- For all parts do

      do ipart = 1, nparts

         istart = outerboun(ipart+1)
         nsurfaces = outerboun(istart)
         if ( debug ) &
            write(irefwr,1) 'ipart, nsurfaces', ipart, nsurfaces

         do j = 1, nsurfaces

            isurnr = abs(outerboun(istart+j))
            if ( debug ) write(irefwr,1) 'j, isurnr', j, isurnr
            npointssurf = npoint
            call sepsurnod ( isurnr, isurnr, iwork, npointssurf )

            do k = 1, npointssurf

               inodp = iwork(k)
               if ( inodp>0 ) then

!              --- Node number is positive

                  if ( iwork1(inodp)==0 ) then

!                 --- Node number has not been used yet
!                     add to array and mark as used

                     nnodes = nnodes+1
                     nodes(nnodes) = inodp
                     iwork1(inodp) = 1

                  end if  ! ( iwork1(inodp)==0 )

               end if  ! ( inodp>0 )

            end do  ! k = 1, npointssurf

         end do  ! k = 1, nsurfaces

      end do  ! ipart = 1, nparts

      deallocate ( iwork, iwork1, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         call prinin ( nodes, nnodes, 'nodes' )
         write(irefwr,*) 'End eroutsurfnod'

      end if  ! ( debug )
      call erclos ( 'eroutsurfnod' )

      end
