      subroutine prevalveccexpf ( work, iwork, ncount, result, &
                                  lenvec, irefwork  )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-12-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Evaluate vector expression and fill actual values of vectors
!     Result is stored in vecout (complex version)
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     expression
!     function
!     vector
!     complex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(*), ncount, lenvec, irefwork(ncount)
      double complex work(lenvec,*), result(lenvec)

!     irefwork      i/o   integer work array of length ncount
!                         if irefwork(i) = 0, it concerns a constant
!                         otherwise (1) a vector
!     iwork         i/o   integer work array of length ncount
!                         iwork is used to store the integer information of the
!                         expression
!     lenvec         i    Length of all vectors
!     ncount         i    Length of iwork
!     result         o    Result vector
!     work          i/o   Work array of length ncount
!                         work is used to store the real information of the
!                         expression
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j
      double precision value
      logical funcfound

!     funcfound      If true a function has been found
!     i              Counting variable
!     j              Counting variable
!     value          Real value if it concerns a scalar
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL2        Print a real two-dimensional array
!     READEVALFUNC   Evaluate function defined by ifunc
!     READEVALFUNCV  Evaluate vector function defined by ifunc
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prevalveccexpf' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prevalveccexpf'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      funcfound = .false.

!     --- For i := 1(1)ncount do

100   do i = 1, ncount

!     --- First all functions are evaluated in order to satisfy the
!         priority rules

         if ( iwork(i)>=10 ) then

!        --- 10<=iwork(i)<100: function
!            work(i-1) := sqrt(work(i))
!            shift work and iwork

             funcfound = .true.

             if ( irefwork(i)==0 ) then

!            --- function of number

                call readevalfunc ( iwork(i), work(1,i), value )

                if ( i==1 ) then

!               --- First function

                  iwork(1) = 1
                  irefwork(1) = 0         ! number
                  work(1:lenvec,1) = value

                else

!               --- function in other position

                   work(1:lenvec,i-1) = value
                   irefwork(i-1) = 0     ! number
                   ncount = ncount-1
                   do j = i, ncount
                      work(1:lenvec,j) = work(1:lenvec,j+1)
                      iwork(j) = iwork(j+1)
                      irefwork(j) = irefwork(j+1)
                   end do
                   work(1:lenvec,ncount+1) = (0d0,0d0)
                   iwork(ncount+1) = 0
                   irefwork(ncount+1) = 0
                   do while ( ncount>1 .and. iwork(ncount)==0 )
                      ncount = ncount-1
                      work(1:lenvec,ncount) = work(1:lenvec,ncount+1)
                   end do  ! while ( ncount>1 .and. iwork(ncount)==0 )
                end if  ! ( i==1 )

             else

!            --- function of vector

                call readevalfuncv ( iwork(i), work(1,i), result, &
                                     lenvec )

                if ( i==1 ) then
                  iwork(1) = 1
                  irefwork(1) = 1
                  work(1:lenvec,1) = result(:)
                else
                   work(1:lenvec,i-1) = result(:)
                   irefwork(i-1) = 1
                   ncount = ncount-1
                   do j = i, ncount
                      work(1:lenvec,j) = work(1:lenvec,j+1)
                      iwork(j) = iwork(j+1)
                      irefwork(j) = irefwork(j+1)
                   end do
                   work(1:lenvec,ncount+1) = (0d0,0d0)
                   iwork(ncount+1) = 0
                   irefwork(ncount+1) = 0
                   do while ( ncount>1 .and. iwork(ncount)==0 )
                      ncount = ncount-1
                      work(1:lenvec,ncount) = work(1:lenvec,ncount+1)
                   end do  ! while ( ncount>1 .and. iwork(ncount)==0 )
                end if  ! ( i==1 )

             end if  ! ( irefwork(i)==0 )
             if ( ierror/=0 ) go to 1000
             go to 100

         end if

      end do

1000  call erclos ( 'prevalveccexpf' )
      if ( debug ) then

!     --- Debug information

         if ( funcfound ) then
            call prinin ( iwork, ncount, &
                          'after evaluation of functions' )
            call prinrl2 ( work, lenvec, ncount, &
                           'after evaluation of functions' )
         end if  ! ( funcfound )
         write(irefwr,*) 'End prevalveccexpf'

      end if  ! ( debug )

      end
