      subroutine readserkeyw ( inumbr, maxnum, inum, positive, maxval, &
                               code, keywordlc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 25-11-2013 Give error message if inum>maxnum
!        version  1.2  date 09-03-2013 Correction length keyword
!        version  1.1  date 01-10-2008 Adaptation constants
!
!   copyright (c) 2008-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read number of integers preceded by the same keyword
!     Reading stops if closing bracket is found or end of record
!     Or if the maximum number is reached
!     A minus sign for the keyword is interpreted as minus sign for the integer
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleread
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer maxnum, inum, inumbr(*), maxval
      logical positive
      character (len=*) code, keywordlc

!     code           i    String containing the corresponding keyword
!     inum           o    Number of numbers that have been actually read
!     inumbr         o    Array to be filled with the numbers
!     keywordlc      i    Keyword that is expected before number
!     maxnum         i    |maxnum| defines the maximum number of numbers to be
!                         read
!                         If maxnum>0, texts are skipped
!                         If maxnum<0, texts are used as separater
!     maxval         i    Maximum value allowed for the integer read
!     positive       i    If true all numbers must be positive
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer kcolsv, ival, lenkeyw
      logical finished, minsgn

!     finished       Indicates if a while loop must be terminated (true)
!                    or not (false)
!     ival           Integer value read
!     kcolsv         Help parameter to save the old value of kcolum
!     lenkeyw        Length of keyword
!     minsgn         Indicates if a minus sign has been found (true)
!                    or not (false)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     READREALCONS   Check if last string read corresponds to constant
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1439   Number of items more than 100
!    1870   Integer number read out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readserkeyw' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readserkeyw'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Clear array inumbr

      inumbr(1:abs(maxnum)) = 0

      finished  = .false.
      inum   = 0
      ispecl = 4
      minsgn = .false.
      kcolsv = kcolum

      do while ( .not. finished )

!     --- While loop
!         Read next item

         call asknxt
         if ( ierror/=0 ) go to 1000
         if ( ityprd==4 ) then

!        --- ityprd = 4, text found

            lenkeyw = min(nchar,len(keywordlc))  ! to prevent wrong bounds
            if ( string(1:nchar)/=keywordlc(1:lenkeyw) ) then

!           --- string is not keyword, but might be a constant

               call readrealcons
               if ( ityprd/=2 .and. ityprd/=3 ) finished = .true.

            end if  ! ( string(1:nchar)/=keyword(1:nchar) )

         end if  ! ( ityprd==4 ) then

         if ( ityprd==2 .or. ityprd==3 .or. ityprd==11 ) then

!        --- ityprd = 2 or 3, number found

            inum = inum + 1

            if ( inum>maxnum ) then

!           --- Number of items read more than available space

               call errint ( inum, 1 )
               call errint ( maxnum, 2 )
               call errsub ( 1439, 2, 0, 0 )
               go to 1000

            end if  ! ( inum>maxnum )
            if ( minsgn .and. realrd>0 ) realrd = - realrd
            minsgn = .false.
            inumbr(inum) = nint(realrd)
            ival = nint(realrd)
            if ( positive ) then

!           --- only positive integers

               if ( ival<1 .or. ival>maxval ) then

!              --- Error integer out of range

                  call errint ( ival, 1 )
                  call errint ( 1, 2 )
                  call errint ( maxval, 3 )
                  call errchr ( code, 1 )
                  call errsub ( 1870, 3, 0, 1 )
                  call errcrd

               end if

            else

!           --- positive and negative integers

               if ( abs(ival)>maxval ) then

!              --- Error integer out of range

                  call errint ( ival, 1 )
                  call errint ( -maxval, 2 )
                  call errint ( maxval, 3 )
                  call errchr ( code, 1 )
                  call errsub ( 1870, 3, 0, 1 )
                  call errcrd

               end if

            end if

         else if ( ityprd==6 ) then

!        --- ityprd = 6, minus sign found

            minsgn = .true.

         else if ( ityprd<=1 .or. ityprd==7 ) then

!        --- ityprd = 0, 1 or 7, i.e. end of file, end of record or
!            ) sign found

            finished = .true.

         end if

      end do  ! while ( .not. finished )

      ispecl = 0
      kcolum = min ( kcolum, kcolsv )

1000  call erclos ( 'readserkeyw' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readserkeyw'

      end if  ! ( debug )

      end
