      subroutine sepfillnodelms ( inpsep, nodes, nnodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-10-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array nodes for set of elements
!     Old subroutine prfillnodelms
! **********************************************************************
!
!                       KEYWORDS
!
!     create
!     vector
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), nodes(*), nnodes

!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine prfillnodes
!     nnodes        i/o   Number of nodes
!     nodes         i/o   Contains the node numbers that must be filled
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ifirst, last, numnod, ifirgr, &
              ielgrp, lastgr, istart, inpelm, jnpelm, ifelem
      integer, allocatable :: iwork(:)

!                    they are not
!     ielgrp         element group number
!     ifelem         First element number
!     ifirgr         First element group number
!     ifirst         First element number
!     inpelm         number of points per element
!     istart         First position used at a part
!     iwork          integer work array of length npoint
!                    iwork(i) = 0, means node i has not yet been filled
!                    iwork(i)>0, means node i has been filled
!                    The value defines the new node number
!     jnpelm         Number of nodes in an element
!     last           Last number to be read from inpsep
!     lastgr         Last element group number
!     numnod         Number of nodes that should be used locally in an element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     ERFILLNODSKMC1 Fill array nodes for elements 1 to inelem
!     ERFILLNODSKMC2 Fill array nodes for elements 1 to inelem
!                    Special case oly a restricted number of nodes per element
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1123   The elements in which the vector must be created do not all have
!           the same number of nodal points per element, whereas the vector must
!           be created in only one call.
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillnodelms' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillnodelms'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get parameters

      allocate ( iwork(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Clear array iwork

      iwork = 0

      nnodes = 0

!     --- Get parameters from inpsep

      ifirst = inpsep(1)
      last   = inpsep(2)
      numnod = inpsep(3)

!     --- Set first and last group as well as first element in first group

      ifirgr = 1
      lastgr = nelgrp
      ifelem = ifirst

      if ( nelgrp>1 ) then

!     --- More than one element group, find range of element groups

         inelem = 0

!        --- Loop over element groups

         do ielgrp = 1, nelgrp

            inelem = inelem + sepgetinelem ( ielgrp )

            if ( inelem<ifirst ) then

!           --- inelem<ifirst, so first group is at least ielgrp + 1

               ifirgr = ielgrp + 1
               ifelem = ifirst - inelem

            end if  ! ( inelem<ifirst )

            if ( inelem<last ) then

!           --- inelem<last, so last group is at least ielgrp + 1

               lastgr = ielgrp + 1

            else if ( inelem==last ) then

!           --- inelem = last, so last group is ielgrp

               lastgr = ielgrp

            end if  ! ( inelem<last )

         end do  ! ielgrp = 1, nelgrp

      end if  ! ( nelgrp>1 )

!     --- Compute starting value istart

      istart = 0
      do ielgrp = 1, ifirgr-1
         inelem = sepgetinelem ( ielgrp )
         inpelm = sepgetinpelm ( ielgrp )
         istart = istart + inelem * inpelm
      end do
      inpelm = sepgetinelem ( ifirgr )
      istart = istart + inpelm * (ifelem-1)

!     --- Check inpelm (must be equal for all elment groups to be considered)

      do ielgrp = ifirgr+1, lastgr

         jnpelm = sepgetinelem ( ielgrp )
         if ( inpelm/=jnpelm ) then
            call errint ( inpelm, 1 )
            call errint ( jnpelm, 2 )
            call errint ( ielgrp, 3 )
            call errsub ( 1123, 3, 0, 0 )
         end if

      end do
      if ( ierror/=0 ) go to 1000

      inelem = last - ifirst + 1

      if ( numnod==0 ) then

!     --- numnod = 0, fill all nodes per element

         call erfillnodskmc1 ( nodes, nnodes, kmeshc(istart+1), &
                               inelem, inpelm, iwork )

      else

!     --- numnod>0, fill some nodes per element

         call erfillnodskmc2 ( nodes, nnodes, kmeshc, inelem, &
                               inpelm, iwork, numnod, inpsep(4), &
                               istart )

      end if  ! ( numnod==0 )

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  call erclos ( 'sepfillnodelms' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepfillnodelms'

      end if  ! ( debug )

      end subroutine sepfillnodelms
