      subroutine sepfillnodes ( iplace, inum, inpsep, &
                                nodes, nnodes, ipoint, nparts )
! ======================================================================
!
!
!        programmer    Guus Segal
!        version  1.0  date 11-08-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array nodes with nodal points as defined in array inpsep
!     Old subroutine prfillnodes
!     It is assumed that sepfillkmesh has been called before
! **********************************************************************
!
!                       KEYWORDS
!
!     node
!     curve
!     surface
!     volume
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecpack
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iplace, inpsep(*), nodes(*), nnodes, &
              inum, ipoint(*), nparts

!     inpsep         i    The integer input read is stored in array INPSEP
!                         The input depends in inum
!                         if inum>0, inpsep(1:inum) contains inum
!                         curve numbers
!                         if inum = 0 then
!                         Pos. 1 indicates how array inpsep is filled.
!                         Possible values:
!                          0   only one curve is used. The curve
!                              number is stored in inpsep(2)
!                         -1   The solution is printed along the curves
!                              icrv1 to icrv2. icrv1 is stored in
!                              inpsep(2) and icrv2 in inpsep(3).
!                         -2   See -1, however, the sequence of the printing is
!                              reversed. In case of curves this means that the
!                              curves are printed from the end point of icrv2 to
!                              the initial point of icrv2, then the end point of
!                              icrv2-1 to the begin point of icrv2-1 and so on.
!                         n>0  The curve numbers numbers are stored
!                              in inpsep(2) to inpsep(n+1).
!                              That sequence is used.
!                         curve may be replaced by surface or volume
!     inum           i    See array inpsep
!     iplace         i    Position where nodes must be filled
!                          0:  whole domain
!                          1:  user points
!                          2:  curves
!                          3:  surfaces
!                          4:  The nodes are filled from node1 to node2 with step istep
!                          5:  The nodes are filled in the elements i to j
!                          6:  The nodes are filled in the element group ielgrp
!                          7:  The nodes are filled in the volumes V1 to V2
!                          8:  The nodes are filled in the user points P1 to P2
!                          9:  The nodes are filled in the obstacle i
!                         10:  The nodes are filled in the contact i
!                         11:  The nodes are filled in the no-contact i
!                         12:  The nodes are filled using a file with node
!                              numbers
!                         13:  The nodes are filled using a file with elements
!                         14:  The nodes are filled using a file with capacities
!                         15:  The global unknowns are filled for global group
!                              iglgrp
!                         16:  All nodes in the obstacle are filled
!                         17:  Only the inner nodes in the obstacle are filled
!                         18:  Only the boundary nodes in the obstacle are
!                              filled
!                         19:  Only the boundary nodes on the obstacle are
!                              filled
!                         20:  zero levelset
!                         22:  Series of curves
!                         23:  Series of nodes
!                         24:  Series of surfaces
!                         25:  Series of volumes
!                         26:  Intersection of a series of surfaces
!     ipoint         o    Array of length nparts+1 containing the starting
!                         addresses in array inodes of the various parts
!                         the boundary consists of.
!                         ipoint(1) = 1, &
!                         number of nodes at part i = ipoint(i+1)-ipoint(i)
!     nnodes         o    Number of nodes in array nodes
!     nodes          o    Contains the node numbers that must be filled
!     nparts         o    number of parts the boundary consists of
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, icount

!     i              Counting variable
!     icount         Counter
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRFILLNODNODS  Fill array nodes for set of nodal points
!     PRININ         print 1d integer array
!     SEPFILLNLEVSET Fill nodal points at boundray of obstacle or zero-level set
!     SEPFILLNODCONT Fill array nodes for contact problems depending on kfill
!     SEPFILLNODCURS Fill array nodes with nodal points as defined in inpsep
!                    Special case: nodes in curves
!     SEPFILLNODELGR Fill array nodes for element groups
!     SEPFILLNODELMS Fill array nodes for set of elements
!     SEPFILLNODINTS Fill the common nodes of each couple of surfaces in the set
!                    surfaces(1:numsurfs)
!     SEPFILLNODOBS  Fill array nodes for obstacle problems depending on kfill
!     SEPFILLNODOBST Fill array nodes for obstacle problems (curves only)
!     SEPFILLNODSRFS Fill array nodes with nodal points as defined in inpsep
!                    Special case: nodes in surfaces
!     SEPFILLNODVOLM Fill array nodes with nodal points as defined in inpsep
!                    Special case: nodes in volumes
!     SEPUSERPNODES  Store all nodes of a set of user points in array nodes
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2208   There are no obstacles, whereas essential boundary conditions
!           for the obstacle are required
!    2764   Integer parameter has wrong value
! **********************************************************************
!
!                       PSEUDO CODE
!
! ======================================================================
!
      call eropen ( 'sepfillnodes' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillnodes'
         write(irefwr,1) 'iplace, inum', iplace, inum
         call prinin ( inpsep, max(inum,2), 'inpsep' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      select case ( iplace )

         case ( 0 )

!        --- All nodes in mesh

            nnodes = npoint
            do i = 1, nnodes
               nodes(i) = i
            end do  ! i = 1, nnodes

         case ( 1, 8 )

!        --- User points

            call sepuserpnodes ( inpsep, nodes, nnodes, iplace )
            nparts = nnodes

         case ( 2, 22 )

!        --- Nodes in set of curves

            call sepfillnodcurs ( inum, inpsep, nodes, &
                                  nnodes, ipoint, nparts, iplace )

         case ( 3, 24 )

!        --- Nodes in set of surfaces

            call sepfillnodsrfs ( inum, inpsep, nodes, nnodes, iplace )

         case ( 4, 23 )

!        --- Nodes node1 to node2 with step istep
!            or series of nodes

            call prfillnodnods ( inpsep, nodes, nnodes, npoint, iplace )

         case ( 5 )

!        --- Elements i to j

            call sepfillnodelms ( inpsep, nodes, nnodes )

         case ( 6 )

!        --- Element group ielgrp

            call sepfillnodelgr ( inpsep, nodes, nnodes )

         case ( 7, 25 )

!        --- Nodes in set of volumes

            call sepfillnodvolm ( inum, inpsep, nodes, nnodes, iplace )

         case ( 9 )

!        --- Fill obstacles

            call sepfillnodobst ( inpsep, nodes, nnodes )

         case ( 10, 11 )

!        --- kfill = 10, 11 Fill contact or no-contact

            call sepfillnodcont ( inpsep, nodes, nnodes, iplace )

         case ( 16:19 )

!        --- Nodes with respect to obstacles

            call sepfillnodobs ( inpsep, nodes, nnodes, iplace )

         case ( 20 )

!        --- Nodes in boundary points on obstacle (or zero levelset)

            if ( numobst==0 ) then

!           --- Error no obstacles present

               call errsub ( 2208, 0, 0, 0 )
               go to 1000

            end if
            call sepfillnlevset ( inpsep(1), nodes, nnodes )

         case ( 26 )

!        --- Nodes in intersection of a set of surfaces

            call sepfillnodints ( inpsep(1), inpsep(2), nodes, nnodes )

         case default

            call errint ( iplace, 1 )
            call errchr ( 'iplace', 1 )
            call errsub ( 2764, 1, 0, 1 )

      end select  ! case ( iplace )

      if ( parallel ) then

!     --- Parallel case, remove negative node numbers

         icount = 0
         do i = 1, nnodes

            if ( nodes(i)>0 ) then
               icount = icount+1
               nodes(icount) = nodes(i)
            end if  ! ( nodes(i)>0 )

         end do  ! i = 1, nnodes
         nnodes = icount

      end if  ! ( parallel )

      if ( iplace/=2 ) nparts = nnodes

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'nparts', nparts
         call prinin ( nodes, nnodes, 'nodes' )
         write(irefwr,*) 'End sepfillnodes'

      end if  ! ( debug )
      call erclos ( 'sepfillnodes' )

      end subroutine sepfillnodes
