      subroutine sepfillnodvolm ( inum, inpsep, nodes, nnodes, iplace )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-10-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array nodes with nodal points as defined in array inpsep
!     Special case: nodes in volumes
!     Old subroutine prfillnodvolms
! **********************************************************************
!
!                       KEYWORDS
!
!     node
!     volume
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), nodes(*), nnodes, inum, iplace

!     inpsep         i    The integer input read is stored in array INPSEP
!                         The input depends in inum
!                         if inum>0, inpsep(1:inum) contains inum
!                         volume numbers
!                         if inum = 0 then
!                         Pos. 1 indicates how array inpsep is filled.
!                         Possible values:
!                          0   only one volume is used. The
!                              volume number is stored in icurvs(2)
!                         -1   The solution is printed along the
!                              volumes icrv1 to icrv2. icrv1 is stored in
!                              icurvs(2) and icrv2 in icurvs(3).
!                         n>0  The volume numbers are stored
!                              in icurvs(2) to icurvs(n+1).
!                              That sequence is used.
!     inum           i    See inpsep
!     iplace         i    Position where nodes must be filled
!                         See subroutine prfillnodes
!     nnodes         o    Number of nodes
!     nodes          o    Contains the node numbers that must be filled
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nvolumes, ivlm1, ivlm2, numnod
      integer, allocatable :: volumes(:)

!                    they are not
!     i              Counting variable
!     ivlm1          First volume number
!     ivlm2          Last volume number
!     numnod         If>0, only numnod nodes per element must be used
!                    In that case ihelp is used
!                    If 0 all nodes are used
!     nvolumes       Number of volumes in array volumes
!     volumes        Contains volume numbers
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPVOLMSNOD    Store all nodes in the volumes stored in array volumes
!                    in array nodes
!     SEPVOLMSNOD1   Store all nodes in the volumes stored in array volumes
!                    in array nodes
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     296   first volume number has incorrect value
!     297   last volume number has incorrect value
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillnodvolm' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillnodvolm'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( iplace==25 ) then

!     --- iplace = 25, series of volumes with respect to create

         numnod = inpsep(2)
         if ( numnod==0 ) then

!        --- Standard case, all nodes in volumes

            call sepvolmsnod ( inpsep(1), nodes, nnodes, inpsep(3) )

         else

!        --- Special case, restricted set of nodes in volumes

            call sepvolmsnod1 ( inpsep(1), nodes, nnodes, inpsep(3), numnod, &
                                inpsep(3+inpsep(1)) )

         end if  ! ( numnod==0 )

      else if ( inum>0 ) then

!     --- inum>0, Use inpsep

         call sepvolmsnod ( inum, nodes, nnodes, inpsep )

      else

!     --- inum = 0, inpsep contains old array icurvs

         nvolumes = inpsep(1)
         if ( nvolumes==0 ) nvolumes = 1
         if ( nvolumes>0 ) then

!        --- inpsep(1)>= 0, volume numbers stored from position 2

            call sepvolmsnod ( nvolumes, nodes, nnodes, inpsep(2) )

         else

!        --- inpsep(1)<0, volume numbers from inpsep(2 to 3)

            ivlm1 = inpsep(2)
            ivlm2 = inpsep(3)
            if ( ivlm1<=0 .or. ivlm1>nvolumes ) then
               call errint ( ivlm1, 1 )
               call errint ( nvolms, 2 )
               call errsub ( 296, 2, 0, 0 )
               ivlm1 = 1
            end if
            if ( ivlm2<=ivlm1 .or. ivlm2>nvolms ) then
               call errint ( ivlm2, 1 )
               call errint ( ivlm1, 2 )
               call errint ( nvolms, 3 )
               call errsub ( 297, 3, 0, 0 )
               ivlm2 = ivlm1
            end if
            nvolumes = ivlm2-ivlm1+1
            allocate ( volumes(nvolumes), stat = error )
            if ( error/=0 ) &
               call eralloc ( error, nvolumes, 'volumes' )
            if ( ierror/=0 ) go to 1000

            do i = 1, nvolumes
               volumes(i) = ivlm1+i-1
            end do  ! i = 1, nvolumes

            call sepvolmsnod ( nvolumes, nodes, nnodes, volumes )

            deallocate ( volumes, stat = error )
            if ( error/=0 ) call erdealloc ( error, 'volumes' )

         end if  ! ( ichoice>0 )

      end if  ! ( inum>0 )

1000  call erclos ( 'sepfillnodvolm' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepfillnodvolm'

      end if  ! ( debug )

      end subroutine sepfillnodvolm
