      subroutine sepreadcurves ( finished, readnext, ispeclsav, subkey, &
                                 numkey, jump, numchars, num )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-04-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read part ( Ci, Cj, ... Cl ) in input record
!     Old subroutine intx02curves
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ispeclsav, numkey, jump, num, numchars
      logical finished, readnext
      character (len=*) subkey

!     finished      i/o   Indicates if the loop has finished (true) or not
!                         (false)
!     ispeclsav      i    Help parameter to save the value of ispecl
!     jump          i/o   if 0 no error has been found, if 1 array tabout
!                         is too small
!     num            o    Number of integers found and stored in tabout
!     numchars       i    Number of significant characters in subkey
!     numkey         i    Number of significant characters in sub keyword
!     readnext      i/o   If true the next item must be read
!     subkey         i    Sub (second) keyword
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, iprev, numval, ivalmin, ivalread
      logical ::  close, tofound

!     close          Indicates if the end of a sequence of numbers has been
!                    found (true) or not (false)
!     i              Counting variable
!     iprev          Previous value used
!     ivalmin        Minimum integer value
!     ivalread       Integer value read
!     numval         Number of values
!     tofound        If true the keyword TO has been found
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINRL         Print 1d real vector
!     READSPLITKEYW  check whether the keyword contains numbers
!                    If so split the keyword into a part without number and
!     READSTRCONST   Check if name read corresponds to one of the integer or
!                    real constants
!     READSTRSCALS   Check if name read corresponds to one of the scalars
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     574   Not an integer number found after the keyword
!    1870   Integer number read out of range
!    2125   C1 preceded by minus sign in C1 to C2
!    2126   C2 preceded by minus sign in C1 to C2
!    2127   C2 smaller than in C1 to C2
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepreadcurves' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepreadcurves'
         write(irefwr,1) 'iposut, icode', iposut, icode
         write(irefwr,1) 'ispeclsav, jump, numkey, numchars', &
                          ispeclsav, jump, numkey, numchars
         write(irefwr,*) 'finished, readnext ', finished, readnext
         call prinrl ( tabout, iposut, 'tabout' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      close = .false.
      num = 0
      readnx = .false.
      tofound = .false.

      do while ( .not. close .and. .not. finished )

!     --- End of items not yet found

         ispecl = 9
         if ( readnext ) call asknxt
         ispecl = ispeclsav
         readnext = .true.

         if ( debug ) then
            write(irefwr,1) 'ityprd/4, lastitem', ityprd, lastitem
            if ( ityprd==4 ) &
               write(irefwr,*) 'string ', string(1:nchar)
         end if  ! ( debug )

         if ( ityprd==4 ) then

!        --- String found, check if it is a constant or a variable

            call readstrconst
            if ( ityprd==4 ) call readstrscals

            if ( ityprd==4 ) then

!           --- Not a constant but a string found

               call readsplitkeyw ( numchar, keyword )
               lastitem = lastitembef+1   ! Only string is used
               nchar = numchar

            end if  ! ( ityprd==4 )

         end if  ! ( ityprd==4 )

         if ( debug ) then
            write(irefwr,1) 'ityprd/4a, lastitem', ityprd, lastitem
            if ( ityprd==4 ) &
               write(irefwr,*) 'string ', string(1:nchar)
         end if  ! ( debug )

         if ( ityprd==2 .or. ityprd==11 ) then

!        --- Integer number found

            ivalread = nint(realrd)
            if ( debug ) write(irefwr,1) 'ivalread', ivalread
            ivalmin = 1
            if ( iprogram==3 .and. icode==111 ) ivalmin = 0
            if ( ityprd==2 .and. &
                 (ivalread<ivalmin .or. ivalread>tabmax(icode)) ) then

!           --- Error integer out of range

               call errint ( ivalread, 1 )
               call errint ( ivalmin, 2 )
               call errint ( nint(tabmax(icode)), 3 )
               call errchr ( code(icode), 1 )
               call errsub ( 1870, 3, 0, 1 )
               call errcrd

            end if

            if ( tofound ) then

!           --- ci to cj found, do some checks

               if ( debug ) write(irefwr,*) 'tofound'
               if ( tabout(iposut)<0 ) then

!              --- First curve number<0

                  call errint ( -nint(tabout(iposut)), 1 )
                  call errchr ( subkey(1:numkey), 1 )
                  call errsub ( 2125, 1, 0, 1 )

               end if
               if ( minusg<0 ) then

!              --- Second curve number<0

                  call errint ( ivalread, 1 )
                  call errchr ( subkey(1:numkey), 1 )
                  call errsub ( 2126, 1, 0, 1 )

               end if
               if ( ivalread<nint(tabout(iposut)) ) then

!              --- Second curve number<first one

                  call errint ( nint(tabout(iposut)), 1 )
                  call errint ( ivalread, 1 )
                  call errchr ( subkey(1:numkey), 1 )
                  call errchr ( subkey(1:numkey), 2 )
                  call errsub ( 2127, 2, 0, 2 )

               end if
               if ( tabmin(icode)>2d0 ) then

!              --- All numbers must be stored

                  iprev = nint(tabout(iposut))
                  numval = ivalread-iprev
                  if ( iposut+numval>nout ) then
                     jump = 1
                     go to 1000
                  end if
                  do i = 1, numval
                     num = num + 1
                     iposut = iposut + 1
                     tabout(iposut) = iprev+i
                  end do

               else

!              --- Only the last number must be stored

                  num = num + 1
                  iposut = iposut + 1
                  if ( iposut>nout ) then
                     jump = 1
                     go to 1000
                  end if
                  tabout(iposut) = ivalread

               end if

            else

               num = num + 1
               iposut = iposut + 1
               if ( iposut>nout ) then
                  jump = 1
                  go to 1000
               end if
               tabout(iposut) = minusg * ivalread
               if ( debug ) &
                  write(irefwr,1) 'iposut, minusg, ivalread', &
                                   iposut, minusg, ivalread

            end if

!           --- Read next item

            minusg = 1
120         ispecl = ispeclsav
            call asknxt
            if ( debug ) then
               write(irefwr,1) 'ityprd/5, lastitem', ityprd, lastitem
               if ( ityprd==4 ) &
                  write(irefwr,*) 'string ', string(1:nchar)
            end if  ! ( debug )
            if ( ityprd==4 .and. .not. tofound ) then
               if ( nchar==2 .and. string(1:2)=='TO' ) then
                  tofound = .true.
                  go to 120
               end if
            end if
            if ( ityprd==6 ) then
               minusg = -1
               call asknxt
               if ( debug ) then
                  write(irefwr,1) 'ityprd/6, lastitem', ityprd, lastitem
                  write(irefwr,1) 'minusg', minusg
                  if ( ityprd==4 ) &
                     write(irefwr,*) 'string ', string(1:nchar)
               end if  ! ( debug )
            end if
            if ( ityprd==7 ) then

!           --- closing bracket found

               call asknxt
               if ( num>0 ) close = .true.
               if ( debug ) then
                  write(irefwr,*) 'closing bracket'
                  write(irefwr,1) 'ityprd/7, lastitem', ityprd, lastitem
                  if ( ityprd==4 ) &
                     write(irefwr,*) 'string ', string(1:nchar)
               end if  ! ( debug )

            end if

            if ( debug ) write(irefwr,*) 'close ', close
            if ( .not. close .and. ityprd==4 ) then

!           --- String found

               if ( debug ) then
                  write(irefwr,1) 'nchar, numchars, numkey', &
                                   nchar, numchars, numkey
                  write(irefwr,*) 'string ', string(1:numchars)
                  write(irefwr,*) 'subkey ', subkey(1:numkey)
               end if  ! ( debug )

               if (string(1:numchars)/=subkey(1:numkey)) then

!              --- String not found

                  close = .true.
                  readnx = .false.

               end if

            else

!           --- not a string

               close = .true.
               readnx = .false.

            end if

         else

!        --- Not an integer number found after the keyword

            call errchr ( code(icode), 1 )
            call errsub ( 574, 0, 0, 1 )
            call errcrd
            finished = .true.

         end if

      end do  ! while ( .not. close .and. .not. finished )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'num, jump', num, jump
         write(irefwr,*) 'readnext ', readnext
         call prinrl ( tabout, iposut, 'tabout' )
         write(irefwr,*) 'End sepreadcurves'

      end if  ! ( debug )
      call erclos ( 'sepreadcurves' )

      end subroutine sepreadcurves
