      subroutine sepuserpnodes ( inpsep, nodes, nnodes, kfill )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-10-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Store all nodes of a set of user points in array nodes
!     Old subroutine eruserpnodes
! **********************************************************************
!
!                       KEYWORDS
!
!     node
!     user_point
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), nodes(*), nnodes, kfill

!     inpsep         i    The integer input read is stored in array INPSEP
!                         In this case it contains the user point numbers
!     kfill          i    Defines which user points are used
!                         1: standard
!                         8: iuserp1 to iuserp2 with step istep
!                            These values are stored in inpsep(1:3)
!     nnodes        i/o   Number of nodes in array nodes at output
!                         At input it contains the number of user points to
!                         be considered
!                         if nnodes = 0 at input, the number of user points
!                         is stored in inpsep(1) and the user point numbers
!                         from position 2 in inpsep
!     nodes          o    Contains the node numbers
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, iuserp, ishift, iuserp1, iuserp2, istep

!     i              Counting variable
!     ishift         Shift in inpsep
!     istep          Step for USER POINTS
!     iuserp         User point sequence number
!     iuserp1        First user point number of P1 to P2
!     iuserp2        Second user point number of P1 to P2
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     254   nodal point number in user point out of range
!    1116   First user point number out of range
!    1117   Second user point number out of range
!    1118   Step size ISTEP in user point P1 to P2 with step ISTEP negative
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepuserpnodes' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepuserpnodes'
         write(irefwr,1) 'kfill, nnodes', kfill, nnodes
         call prinin ( inpsep, 3, 'inpsep' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( kfill==8 ) then

!     --- kfill = 8, iuserp1 to iuserp2 with step istep

         iuserp1  = inpsep(1)
         iuserp2  = inpsep(2)
         istep  = inpsep(3)

         if ( debug ) &
            write(irefwr,1) 'iuserp1, iuserp2, istep', &
                             iuserp1, iuserp2, istep

         if ( iuserp1<=0 .or. iuserp1>nuspnt ) then
            call errint ( iuserp1, 1 )
            call errint ( nuspnt, 2 )
            call errsub ( 1116, 2, 0, 0 )
         end if
         if ( iuserp2<iuserp1 .or. iuserp2>nuspnt ) then
            call errint ( iuserp2, 1 )
            call errint ( iuserp1, 2 )
            call errint ( nuspnt, 3 )
            call errsub ( 1117, 3, 0, 0 )
         end if
         if ( istep<0 ) then
            call errint ( istep, 1 )
            call errsub ( 1118, 1, 0, 0 )
         end if

         istep = max(istep,1)
         nnodes = 0

         do i = iuserp1, iuserp2, istep

            nnodes = nnodes+1
            nodes(nnodes) = kmeshl(i)

         end do  ! i = iuserp1, iuserp2, istep

      else

!     --- kfill = 1, standard case

         if ( nnodes==0 ) then

!        --- nnodes = 0, hence number of user points in inpsep(1)

            nnodes = inpsep(1)  ! number of user points
            ishift = 1          ! shift in inpsep

         else

!        --- nnodes>0, hence user points from inpsep(1)

            ishift = 0  ! shift in inpsep

         end if  ! ( nnodes==0 )

         if ( debug ) write(irefwr,1) 'ishift, nnodes', ishift, nnodes

         do i = 1, nnodes

            iuserp = inpsep(i+ishift)

            if ( iuserp<=0 .or. iuserp>nuspnt ) then

!           --- iuserp out of range

               call errint ( iuserp, 1 )
               call errint ( nuspnt, 2 )
               call errsub ( 254, 2, 0, 0 )
               iuserp = 1

            end if  ! ( iuserp1<=0 .or. iuserp1>nuspnt )
            nodes(i) = kmeshl(iuserp)

         end do  ! i = 1, nnodes

      end if  ! ( kfill==8 )

1000  call erclos ( 'sepuserpnodes' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepuserpnodes'

      end if  ! ( debug )

      end subroutine sepuserpnodes
