      subroutine adapbndb ( inpada, rinada, icurves, iread, changed, &
                            changeinput, numvecs, iseqvecs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 20-08-2016 use sepmoduleinput
!        version  2.4  date 15-11-2012 Long integers
!        version  2.3  date 28-06-2005 Remove markers at boundary
!
!   copyright (c) 1997-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   This subroutine is used to adapt the boundary of a two-dimensional mesh
!   depending on the solution
!   It forms the body of subroutine adapboun
!   It is an extension of the previous subroutine ADAPBN
!   Input may be read either from the standard input file or given by the
!   parameter list
! **********************************************************************
!
!                       KEYWORDS
!
!     adapt_boundary
!     free_boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemain
      use sepmoduleinput
      use sepmoduletime
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpada(*), icurves(*), iread, numvecs, iseqvecs(*)
      double precision rinada(*)
      logical changed, changeinput

!     changed       o    If true the number of nodes along the boundary has been
!                        changed
!     changeinput   o    If true the input arrays iinput and rinput have been
!                        changed
!     icurves       i    Integer array containing the curve numbers for the
!                        adaptation of the boundary. A negative curve
!                        number indicates that the curve must be computed in
!                        the reversed direction.
!                        The curves including their sign must form one
!                        contiguous curve
!                        The number of curves is stored in array inpada
!                        icurves is only used if iread = -1
!                        If iredist>100, also the curve numbers of
!                        which the distribution must be changed are stored
!     inpada        i    Integer array to be filled by the user, indicating how
!                        the boundary must be adapted.
!                        INPADA is only used if iread = -1
!                        INPADA must be filled as follows:
!                        Pos. 1   Position of last entry that has been filled by
!                                 the user. If inpada(1) = 0 or 1, only defaults
!                                 are used.
!                        Pos. 2   ncurves, indicates the number of curves
!                                 in array icurves that must be used for the
!                                 adaptation of the mesh
!                                 Default value: 1
!                        Pos. 3   Not yet used. Must be set equal to 0
!                                 Default value: 0
!                        Pos. 4   iadapt Indicates how the boundary must be
!                                 adapted
!                                 Possible values:
!                                 1.  The curves are adapted by:
!                                     xnew(i) = xold(i) + alpha(i) n(i)
!                                     with xold(i) the old co-ordinates of the
!                                     i-th node and xnew(i) the new ones.
!                                     n(i) is the normal at the boundary of the
!                                     i-th node and alpha(i) the number-th
!                                     degree of freedom in this node
!                                     The normal is computed assuming a counter- &
!                                     clockwise direction of the boundary
!                                 2.  The curves are adapted by the user
!                                     provided function subroutine FUNCCR
!                                 3.  See 1, however, now alpha is computed
!                                     according to the 'film method'
!                                     See: Caswell and Viriyayuthakorn, &
!                                     Finite element simulation of die swell for
!                                     a Maxwell fluid, J. of Non-Newt. Fluid
!                                     Mech.  Vol 12, 13-29(1983).
!                                     alpha(i) = factor*(psi(i)-psi(0))/|vt(i)|, &
!                                                      xi
!                                     psi(i)-psi(0) = /  v.n dx
!                                                    x0
!                                     The vector isol must be the velocity
!                                     vector with degrees of freedom number and
!                                     number+1 from isol.  Cartesian coordinates
!                                     Starting flow rate integration along free
!                                     boundary with flow rate zero in first
!                                     point of curves.
!                                 4.  Other method according to:
!                                     xnew(i) = xold(i) + v(i) * factor
!                                     Time-dependent case if imult=1 see
!                                     inpada(11)
!                                     In the axisymmetric case we use:
!                                     (r,z)new(i) = (r,z)old(i) + &
!                                                    (vr,vz)(i) dt /(2 pi r(i))
!                                 5.  See 1. Now alpha is computed according to
!                                     alpha=v.n * factor
!                                 6.  See 1. Now alpha is computed according to
!                                     alpha=(grad c).n * factor
!                                     It is assumed that grad c is stored in
!                                     a vector of special structure defined per
!                                     element
!                                 7.  See 6. The more accurate scheme described
!                                     in Segal, Vuik and Vermolen is used
!                                     Use omega, eps and maxiter as stored in
!                                     rinada(7), rinada(6) and inpada(18)
!                                 8.  Constant velocity i.e.
!                                     xnew(i) = xold(i) + u * factor
!                                     where u is a given velocity vector
!                                     Time-dependent case if imult=1
!                                     (see inpada(11))
!                                 9.  Using FUNCSOLCR
!                                 Default value: 1
!                        Pos. 5   idirec Indicates the direction of the
!                                 computation
!                                 If idirec = 1 the computation is performed
!                                 from first to last point, &
!                                 if idirec = -1 from last to first point
!                                 It is sometimes required to alternate between
!                                 1 and -1 during iteration.
!                                 This option is only used for iadapt = 3
!                                 Default value: 1
!                        Pos. 6   number, the number-th degree of freedom of
!                                 isol (and higher) are used for the computation
!                                 of alpha. See iadapt.
!                                 Default value: 1
!                        Pos. 7   linear. Indicates if the elements along the
!                                 boundary must be treated as linear or as
!                                 quadratic elements.
!                                 Possible values:
!                                 1: linear
!                                 2: quadratic
!                                 Default value: to be computed by the
!                                 subroutine in the following way:
!                                 If all internal elements are quadratic, the
!                                 boundary elements are assumed quadratic, &
!                                 otherwise linear
!                        Pos. 8   jcart defines the type of co-ordinate system
!                                 to be used. This is of importance for
!                                 iadapt = 3, 4 and 5
!                                 Possible values:
!                                 1:  Cartesian co-ordinates
!                                 2:  Axi-symmetric co-ordinates
!                                 Default value: 1
!                        Pos. 9   iexlude_1: indicates which of the co-ordinates
!                                 of the first point of the first curve must be
!                                 adapted.
!                                 Possible values:
!                                 0  Both co-ordinates must be adapted
!                                 1  Only the second co-ordinate must be
!                                    adapted, the first one remains unchanged
!                                 2  Only the first co-ordinate must be adapted, &
!                                    the second one remains unchanged
!                                 3  Both co-ordinates remain unchanged
!                                 Default value: 0
!                        Pos. 10  iexlude_2: indicates which of the co-ordinates
!                                 of the last point of the last curve must be
!                                 adapted.
!                                 Possible values:
!                                 0  Both co-ordinates must be adapted
!                                 1  Only the second co-ordinate must be adapted
!                                    the first one remains unchanged
!                                 2  Only the first co-ordinate must be adapted, &
!                                    the second one remains unchanged
!                                 3  Both co-ordinates remain unchanged
!                                 Default value: 0
!                        Pos. 11  imult: indicates how the value of factor
!                                 must be computed. (iadapt>2)
!                                 Possible values:
!                                 0  Multiplication factor is 1
!                                 1  Multiplication factor is dt from
!                                    common ctime
!                                 2  Multiplication factor is stored in
!                                    rinada(1)
!                                 3  Multiplication factor is dt * rinada(1)
!                                 Default value: 0
!                        Pos. 12  ithres: indicates if a threshold value is used
!                                 in Caswell's method and where to find it.
!                                 Possible values:
!                                 0  Threshold value = 0
!                                 1  Threshold value is stored in rinada(2)
!                                 Default value: 0
!                        Pos. 13  iangle_1: Indicates if angle of freesurface at
!                                 first point is prescribed or not
!                                 Possible values:
!                                 0  angle is not given
!                                 1  angle is stored in rinada(3)
!                                <0  The first point must be projected onto
!                                    the curve c |iangle_1|
!                                 Default value: 0
!                        Pos. 14  iangle_2: Indicates if angle of freesurface at
!                                 last point is prescribed or not
!                                 Possible values:
!                                 0  angle is not given
!                                 1  angle is stored in rinada(4)
!                                <0  The last point must be projected onto
!                                    the curve c |iangle_2|
!                                 Default value: 0
!                        Pos. 15  redistribute nodes
!                                 Consists of two parts
!                                 iredist_type and ncurves_redist according to:
!                                 iredist = iredist_type + 100*ncurves_redist
!                                 Possible values:
!                                 iredist_type:
!                                 0  no node redistribution
!                                 1  nodes are redistributed, nelm constant
!                                 2  nodes are redistributed, nelm recomputed
!                                    according to coarseness
!                                 ncurves_redist: number of curves that
!                                 must be redistributed
!                                 The curve numbers are stored in icurvs
!                                 positions ncurves+1 to ncurves+ncurves_redist
!                                 This option is only available in combination
!                                 with iredist>0
!                                 Default value: 0
!                        Pos. 16  Plot_boun
!                                 Consists of 2 parts plotboun and plotmark
!                                 according to plotboun+10*plotmark
!                                 plotboun:
!                                 0  do not plot new boundary
!                                 1  plot new boundary
!                                 2  plot new boundary, use xmin, xmax, ymin and
!                                    ymax from rinada(8-11)
!                                 3  plot new boundary, use
!                                    yfact from rinada(14)
!                                 4  plot new boundary, use xmin, xmax, ymin and
!                                    ymax from rinada(8-11) and
!                                    yfact from rinada(14)
!                                 plotmark:
!                                 0  do not plot markers along the boundary
!                                 1  plot markers along the boundary
!                                 Default value: 0
!                        Pos. 17  Numobst (Number of obstacles)
!                                 Each obstacle must be closed and consist of
!                                 one curve only (This may be a curve of curves)
!                                 At most 10 obstacles may be used.
!                                 The curve numbers of the Numobst obstacles are
!                                 stored in array icurvs positions:
!                                 ncurves+ncurves_redist+1 ... .+Numobst
!                                 Default value: 0
!                        Pos. 18  Maxiter, maximum number of iterations in
!                                 iteration process
!                                 Default value: 10
!                        Pos. 19  Prin_boun
!                                 Possible values:
!                                 0  do not plot new boundary
!                                 1  print new boundary
!                                 2  print new boundary, use xmin, xmax, ymin
!                                    and ymax from rinada(8-11)
!                                 Default value: 10
!                        Pos. 20  check_direction
!                                 Possible values:
!                                -1  the direction of boundary must be
!                                    negative relative to the outer normal
!                                 0  no checking
!                                 1  the adaptation of boundary must be
!                                    in the direction of the outer normal
!                                 Default value: 0
!     iread      i       consists of two parts iread_act and ncall according to:
!                        iread = sign(iread_act)*(ireadact+1000*ncall) with
!                        iread_act   Defines if parameters should be read or
!                           are given in array INPADA through the parameter list
!                           Possible values:
!                           0   The input is read from the standard input file
!                           >0  The input as read from the standard input file
!                                bysubroutine SEPSTM is used. The value of iread
!                               defines the sequence number
!                           -1  No input is read, all information from array
!                                INPADAis used
!                        ncall   Number of subsequent calls to adapboun
!                                If zero 1 is assumed
!     iseqvecs   i       Contains sequence numbers of vectors with respect to
!                        isol
!     numvecs    i       Number of vectors in iseqvecs
!     rinada     i       Contains real information concerning the adaptation of
!                        the boundary. See array inpada
!                         1:  multiplication factor
!                         2:  threshold value
!                         3:  angle_1
!                         4:  angle_2
!                         5:  alpha for splines
!                         6:  accuracy eps for non-linear iteration
!                         7:  relaxation parameter omega
!                         8-13:  plot region (xmin,xmax,ymin,ymax)
!                             positions 12,13 reserved for zmin, zmax
!                        14:  yfact
!                        15-17:  velocity u
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nadapt, nrladapt
      parameter ( nadapt=20, nrladapt=17 )
      integer i, lengin, lengrl, numcrv, iseqnr, &
              iinada, irlada, curves, linear, &
              ireadact, ncall, icall, jmaxsave, ichois, ivectr, &
              plotmark, plotboun, maxcall, ipos
      double precision yfact, dum
      integer, pointer :: icopad(:), iinput(:), icurvs(:)
      integer, allocatable :: iwork(:)
      double precision, pointer :: rcopada(:), rinput(:)
      save icall

!     curves         memory management reference number of array curves
!     dum            dummy
!     i              Counting variable
!     icall          Calling sequence number. Once icall=ncall it is reset
!     ichois         Choice parameter
!     icopad         Copy of inpada extended with default values
!     icurvs         Integer array containing the curve numbers that
!                    must be adapted. It is assumed that these curves form
!                    one continuous curve.
!     iinada         Memory management sequence number of array inpada
!     iinput         integer input array
!     ipos           Position of input arrays in sepmoduleinput
!     ireadact       See iread
!     irlada         Memory management sequence number of array rinada
!     iseqnr         Sequence number with respect to the storage in array ISEPIN
!     ivectr         sequence number of first vector to be used in isol
!     iwork          work array of length ncurvs to mark curves that must
!                    be plotted or printed
!     jmaxsave       Help variable to save the value of jmax
!     lengin         Length of integer array iinput
!     lengrl         Length of real array rinput
!                    (in double precision variables)
!     linear         Parameter iadapt as described in inpada(7)
!     maxcall        Maximum number of inputs read
!     nadapt         Length of array icopad
!     ncall          See iread
!     nrladapt       Length of array rcopada
!     numcrv         number of parts where curves are given for generation
!                    of boundary elements
!     plotboun       See inpada(16)
!     plotmark       See inpada(16)
!     rcopada        Copy of rinada
!     rinput         Real input array
!     yfact          Multiplication factor in y-direction
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INI018         Check uniqueness of curve numbers
!     INICHINPSOL    Adapt input array with respect to variables
!     MSH008         Fill copy of inpada and extend with defaults
!     MSH056         Body of subroutines ADAPBN and CHANBN
!     PLTBN2         Plot boundary of two-dimensional mesh
!     PLTINTCV       fill array iwork for all curves that must be plotted
!     PRINCURV       Print coordinates of curves
!     READADBN       Read the input for the subroutine ADAPBOUN
!     SEPGETINPFINF  Find maxcall and ipos from sepmoduleinput as function of
!                    isubr and iseqnr
! **********************************************************************
!
!                       I/O
!
!    Depending on the parameter iread input is read from the standard
!    input file by subroutine readadbn
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     419   NDIM # 2
!    1948   No input with the main keyword has been read
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Perform standard tests
!    if ( iread=-1 ) then
!       Fill icopad with default values
!       Copy contents from inpada into icopad
!    else if ( iread=0 ) then
!       Read input from the standard input file
!    else ( iread>0 )
!       Copy input read before from ibuffr into icopad
!    Check values
!    Call actual subroutine
!    If ( boundary must be plotted ) then
!       plot boundary
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data icall /0/
! ======================================================================
!
!     --- Standard tests

      call eropen ( 'adapbndb' )

      icall = icall + 1
      if ( ndim/=2 ) then
         call errint ( ndim, 1 )
         call errint ( 2, 2 )
         call errint ( 2, 3 )
         call errsub ( 419, 3, 0, 0 )
      end if

!     --- Get arrays iinput and rinput and copy to end of ibuffr
!         Put control integers around both arrays

      lengin = size(iinputkm)
      lengrl = size(rinputkm)

      allocate ( iinput(lengin), rinput(lengrl), stat = error )
      if ( error/=0 ) call eralloc ( error, lengin+lengrl, 'iinput' )
      if ( ierror/=0 ) go to 1000
      ireadact = mod(iread,1000)
      ncall = max(1,abs(iread/1000))
      ivectr = iseqvecs(1)

      if ( ireadact<=0 ) then

!     --- Input has not been read before, allocate icopad and rcopada

         allocate ( icopad(nadapt), rcopada(nrladapt), stat = error )
         if ( error/=0 ) call eralloc ( error, nadapt, 'icopad' )
         if ( ierror/=0 ) go to 1000

      end if  ! ( ireadact<=0 )

      if ( ireadact==-1 ) then

!     --- ireadact = -1, fill icopad from inpada

         call msh008 ( 2, inpada, isolut2d(ivectr), icopad, &
                       nadapt, icurves )

!        --- Copy array icurves to ibuffr

         numcrv = icopad(2)+icopad(15)/100+icopad(17)

!        --- Reserve space for array icurves
!            This array is put at the end of IBUFFR so that it can not
!            be destroyed by memory management subroutines

         allocate ( icurvs(numcrv), stat = error )
         if ( error/=0 )  call eralloc ( error, numcrv, 'icurvs' )
         if ( ierror/=0 ) go to 1000
         icurvs = icurves(1:numcrv)
         rcopada(1:nrladapt) = rinada(1:nrladapt)

      else if ( ireadact==0 ) then

!     --- ireadact = 0, fill icopad by reading from the standard input file

         iseqnr = 1
         allocate ( icurvs(ncurvs*2), stat = error )
         if ( error/=0 )  call eralloc ( error, numcrv, 'icurvs' )
         if ( ierror/=0 ) go to 1000
         call readadbn ( icopad, rcopada, icurvs, 1, iseqnr )

      else

!     --- ireadact>0, fill icopad by copying from sepmoduleinput
!         First find starting position in array in

         iseqnr = ireadact
         call sepgetinpfinf ( 16, iseqnr, ipos, maxcall )

         if ( in(ipos)%used ) then

!        --- Input block for adapbn found

            icopad => in(ipos)%iinput
            rcopada => re(ipos)%rinput
            numcrv = icopad(2)+icopad(15)/100+icopad(17)
            allocate ( icurvs(numcrv), stat = error )
            if ( error/=0 )  call eralloc ( error, numcrv, 'icurvs' )
            if ( ierror/=0 ) go to 1000
            do i = 1, numcrv
               icurvs(i) = icopad(i+nadapt)
            end do

         else

!        --- No input block for adapbn found

            call errchr ( 'ADAPT_BOUNDARY', 1 )
            call errsub ( 1948, 0, 0, 1 )
            go to 1000

         end if

      end if

!     --- Adapt variables

      call inichinpsol ( icopad, nadapt )
      call inichinpsol ( icurvs, numcrv )
      plotboun = mod(icopad(16),10)   ! indicates if boundary must be plotted
      plotmark = icopad(16)/10        ! indicates if markers must be set
                                      ! along boundary

      if ( icopad(7)==0 ) then

!     --- The type of elements has not yet been detected
!         detect if all volume elements are quadratic or not

         linear = 2
         do i = 1, nelgrp
            ishape = kmeshf(i)
            if ( ishape==3 .or. ishape==5 ) linear = 1
         end do
         icopad(7) = linear

      end if

!     --- Check uniqueness of curve numbers

      call ini018 ( icurvs, ncurvs, &
                    numcrv-icopad(17), icopad(2) )

      iinput = iinputkm
      rinput = rinputkm

!     --- Call actual subroutine

      call msh056 ( icopad, rcopada, isolut2d, iinput, rinput, icurvs, &
                    ncall, icall, changed, changeinput, numvecs, &
                    iseqvecs )

      if ( .not. changeinput ) then

!     --- Copy iinput and rinput back and reset nbuffr

         iinputkm = iinput
         rinputkm = rinput

      end if

      if ( (plotboun>=1 .or. icopad(19)>=1) .and. &
           (itimloop==0 .or. ioutact==1) ) then

!     --- Plot boundary of mesh


         allocate ( iwork(ncurvs), stat = error )
         if ( error/=0 ) call eralloc ( error, ncurvs, 'iwork' )
         if ( ierror/=0 ) go to 1000

!        --- Fill array iwork with curves that must be plotted

         call pltintcv ( icurvs, numcrv-icopad(17), iwork )

         if ( plotboun==2 .or. plotboun==4 .or. &
              icopad(19)>9 ) then
            jmaxsave = jmax
            jmax = 1
            xmin = rcopada(8)
            xmax = rcopada(9)
            ymin = rcopada(10)
            ymax = rcopada(11)
         end if
         if ( plotboun>=1 ) then

!        --- Plot boundary

            if ( plotboun>=3 ) then
               yfact = rcopada(14)
            else
               yfact = 1d0
            end if
            call pltbn2 ( 1+10*plotmark, icurvsmsh, curvesmsh, &
                          ncurvs, 15d0, yfact, nuspnt, dum, &
                          iwork )

         end if
         if ( icopad(19)>=1 ) then

!        --- Print boundary

            ichois = icopad(19)+1
            call princurv ( ichois, icurvsmsh, curvesmsh, &
                            ncurvs, 2, iwork, 'curves' )

         end if
         if ( plotboun==2 ) jmax = jmaxsave

         deallocate ( iwork, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'iwork' )
         if ( ierror/=0 ) go to 1000

      end if
      if ( icall==ncall ) icall = 0

      deallocate ( iinput, rinput, icurvs, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iinput' )
      if ( ierror/=0 ) go to 1000

      if ( ireadact<=0 ) then

!     --- ireadact<=0, deallocate arrays

         deallocate ( icopad, rcopada, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'icopad' )

      else

!     --- ireadact>0, uncouple icopad and rcopada

         nullify ( icopad, rcopada )

      end if  ! ( ireadact<=0 )

1000  call erclos ( 'adapbndb' )

      end subroutine adapbndb
