      subroutine ercopyinput ( isubr, iseqnr, namesubr, iinput, &
                               rinput, inforiinput, leninp, lenrin, &
                               leninf )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 26-08-2016 use sepmoduleinput
!        version  1.1  date 28-10-2015 Extra debug
!        version  1.0  date 26-08-2011
!
!   copyright (c) 2011-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy input read by a read subroutine into buffer array
!     information is stored in isepi2/isepi3
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleinput
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isubr, iseqnr, leninp, iinput(leninp), lenrin, leninf, &
              inforiinput(*)
      double precision rinput(*)
      character (len=*) namesubr

!     iinput         i    integer input array
!     inforiinput    i    Array of length of iinput that contains a 0 if the
!                         number stored in rincre is a real and a reference
!                         to scalars if it is a scalar
!     iseqnr         i    Sequence number with respect to the storage in array
!                         ISEPIN
!     isubr          i    Indicates the type of calling subroutine
!                         Possible values
!                         1:  CREATE
!                         If negative the input block may be reused in
!                         sepmoduleinput
!     leninf         i    Length of array inforiinput
!     leninp         i    Computed length of integer input array
!     lenrin         i    Computed length of array user
!     namesubr       i    Name of subroutine
!     rinput         i    Real input array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ipos, maxcall
      logical :: reuse

!     ipos           Position of input arrays in sepmoduleinput
!     maxcall        Maximum value of icall in sepmoduleinp
!     reuse          If true the input block may be overwritten
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     SEPGETINPFINF  Find maxcall and ipos from sepmoduleinput as function of
!                    isubr and iseqnr
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1872   Sequence number used more than once
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ercopyinput' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ercopyinput'
         write(irefwr,1) 'isubr, iseqnr', isubr, iseqnr
         write(irefwr,1) 'leninp, lenrin, leninf', &
                          leninp, lenrin, leninf
         call prinrl ( rinput, lenrin, 'rinput' )
         call prinin ( iinput, leninp, 'iinput' )
         call prinin ( inforiinput, leninf, 'inforiinput' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000


      if ( isubr<0 ) then

!     --- The input block in sepmoduleinput may be overwritten

         reuse = .true.
         isubr = -isubr

      else

!     --- The input block in sepmoduleinput may not be overwritten

         reuse = .false.

      end if  ! ( isubr<0 )

      if ( iseqnr<=iinputarrs(isubr)%maxsub ) &
         iinputarrs(isubr)%maxcall = max ( iinputarrs(isubr)%maxcall, iseqnr )

      call sepgetinpfinf ( isubr, iseqnr, ipos, maxcall )

      if ( debug ) &
         write(irefwr,1) 'isubr, iseqnr, ipos, maxcall', &
                          isubr, iseqnr, ipos, maxcall

      if ( in(ipos)%used ) then

         if ( .not. reuse ) then

!        --- Error sequence number already in use

             call errint ( iseqnr, 1 )
             call errchr ( namesubr, 1 )
             call errsub ( 1872, 1, 0, 1 )
             go to 1000

         else

!        --- Remove old vectors

            nullify ( in(ipos)%iinput )
            if ( lenrin>0 ) nullify ( re(ipos)%rinput )

         end if  ! ( .not. reuse )

      end if  ! ( in(ipos)%used )

      in(ipos)%used = .true.

      allocate ( in(ipos)%iinput(leninp), stat=error )
      if ( error/=0 ) call eralloc ( error, leninp, 'iinput' )

      in(ipos)%iinput = iinput(1:leninp)

      if ( lenrin>0 ) then

         allocate ( re(ipos)%rinput(lenrin), stat=error )
         if ( error/=0 ) call eralloc ( error, lenrin, 'rinput' )

         re(ipos)%rinput = rinput(1:lenrin)

      end if  ! ( lenrin>0 )

      if ( leninf>0 ) then

         allocate ( inf(ipos)%iinputf(lenrin), stat=error )
         if ( error/=0 ) call eralloc ( error, lenrin, 'inforiinput' )

         inf(ipos)%iinputf = inforiinput(1:lenrin)

      end if  ! ( lenrin>0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End ercopyinput'

      end if  ! ( debug )
      call erclos ( 'ercopyinput' )

      end
