      subroutine insp01
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 06-01-2016 Use sepmoduleelem
!        version  3.0  date 06-01-2016 Use sepmoduleelem
!        version  2.11 date 09-03-2013 Jump if nothing to be done
!
!   copyright (c) 1990-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     subroutine INSP01 is a help subroutine of SEPPS1
!     It is used to compute vectors that must be computed from other vectors
!     At these moment the vectors to be computed must be defined by
!     so called COMPUTE statements like:
!     compute Vi = stream function Vj ....
!     compute Vi = velocity profile Vj ....
!     compute Vi = intersection Vj ....
! **********************************************************************
!
!                       KEYWORDS
!
!     compute
!     post_processing
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulepost
      use sepmodulevecs
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulemain
      use sepmoduleplot
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ivec, ichoice, ivalue, jvec, imesh, idummy, i, &
              iseqent, lvec, &
              iseqdens, iseqmach, iseqvel, iinvec(5), &
              indsol, jndsol, velndsol, machndsol, entndsol, densndsol
      double precision rinvec(1)

!     densndsol      Sequence number in isol of density vector
!     entndsol       Sequence number in isol of enthalpy vector
!     i              Counting variable
!     ichoice        Choice parameter
!     idummy         Dummy parameter
!     iinvec         User input array for subroutine MANVEC
!     imesh          Mesh sequence number
!     indsol         Sequence number of vector in isol
!     iseqdens       Sequence number of density vector in isol
!     iseqent        Sequence number of enthalpy vector in isol
!     iseqmach       Sequence number of mach vector in isol
!     iseqvel        Sequence number of velocity vector in isol
!     ivalue         Integer value
!     ivec           Sequence number of vector of special structure
!     jndsol         Sequence number of vector in isol
!     jvec           Sequence number of originating array to be processed
!     lvec           Defines type of output vector for stream function
!     machndsol      Sequence number in isol of mach vector
!     rinvec         Real user input array for subroutine MANVEC
!                    The contents depend on iinvec
!     velndsol       Sequence number in isol of velocity vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERCOMPVECTOR   Compute the normal stress vector along a set of curves and
!                    store in a solution vector
!     ERINTPOL       Interpolate element values to nodal points
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     PRCPCOEF       Compute scaled pressure
!     PRENTROPY      Compute the entropy in all points of the mesh, from the
!                    given pressure and density
!     PRMACHNO       Compute mach number
!     PRSTAGPRES     Compute the stagnation pressure from mach number
!                    and pressure
!     PRTOTENTH      Compute the total enthalpy from velocity and enthalpy
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPINTP2D      Interpolate 2D region with straight line
!     SEPINTP3D      Interpolate 3D region with plane
!     SEPMANVEC      Vector manipulation subroutine
!     SEPSTREAM      Actual body of STREAM
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2073   Velocity vector has not been filled
!    2074   Vector to be used for computation not available
!    2230   solution array has not been filled
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'insp01' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from insp01'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ivec = iinputpst(2)
      jvec = ispec(ivec,2)
      if ( debug ) write(irefwr,1) 'ivec, jvec', ivec, jvec

      if ( jvec==0 ) go to 1000  ! nothing to be done

      jndsol = imapsol(jvec,itimst)
      if ( debug ) write(irefwr,1) 'jndsol', jndsol

      if ( jndsol==0 ) then

!     --- Create new entry in isol

         lastmapsol = lastmapsol+1
         imapsol(jvec,itimst) = lastmapsol
         jndsol = lastmapsol

      end if
      if ( debug ) &
         write(irefwr,1) 'jndsol, ispec(jvec,1)', jndsol, ispec(jvec,1)
      if ( mod(ispec(jvec,1),100)<=3 ) then

!     --- Vector to be computed corresponds to 2D-mesh
!         Check if the originating vector must be computed

         if ( mod(ispec(jvec,1),100)==3 .and. isolut2d(jndsol)==0 ) then

!        --- Vector to be computed corresponds to a 3d-mesh and has not yet been
!            filled

            ichoice = ispec(jvec,4)
            imesh = ispec(jvec,3)
            indsol = imapsol(ispec(jvec,2),itimst)
            if ( indsol==0 ) then

!           --- Array has not been filled

               call errint ( ispec(jvec,2), 1 )
               call errint ( itimst, 2 )
               call errsub ( 2230, 2, 0, 0 )
               go to 1000

            end if
            if ( debug ) write(irefwr,*) 'call intp3d/1'
            call sepintp3d ( ichoice, valint(1,jvec), indsol, jndsol, &
                             numint(jvec), imesh )

         else if ( mod(ispec(jvec,1),100)==1 .and. &
                   isolut2d(jndsol)==0 ) then

!        --- The 1D-intersection of a stream function is required, whereas
!            the stream function has not yet been filled
!            filled

            imesh = ispec(jvec,3)
            kmesh => kmesh2d(imesh)
            call sepfillkmesh(kmesh)
            lvec = ispec(jvec,4)
            if ( iinputpst(32)>=1 ) then

!           --- mmiskip>0, Element groups must be skipped
!               Fill array iskip and store temporary at the end of ibuffr

              ichoice = 1

            else

!           --- mmiskip = 0, No element groups must be skipped

               ichoice = 0

            end if
            indsol = imapsol(ispec(jvec,2),itimst)
            if ( indsol==0 ) then

!           --- Array has not been filled

               call errint ( ispec(jvec,2), 1 )
               call errint ( itimst, 2 )
               call errsub ( 2230, 2, 0, 0 )
               go to 1000

            end if
            if ( debug ) write(irefwr,*) 'call to0040/1'
            kprob => kprob2d(imesh)
            call sepstream ( ichoice, ichels, lvec, &
                             isolut2d(jndsol), jpoint, psiphi, &
                             isolut2d(indsol), iskiparr )

         end if

      end if
      if ( ierror/=0 ) go to 1000

!     --- Compute the vector itself

      imesh = ispec(ivec,3)
      indsol = imapsol(ivec,itimst)
      kmesh => kmesh2d(imesh)
      kprob => kprob2d(imesh)
      if ( indsol==0 ) then

!     --- Create new entry in isol

         lastmapsol = lastmapsol+1
         imapsol(ivec,itimst) = lastmapsol
         indsol = lastmapsol

      end if
      jndsol = imapsol(jvec,itimst)
      if ( jndsol==0 ) then

!     --- Array has not been filled

         call errint ( jvec, 1 )
         call errint ( itimst, 2 )
         call errsub ( 2230, 2, 0, 0 )
         go to 1000

      end if
      if ( mod(ispec(ivec,1),100)==1 ) then

!     --- isol may be computed by subroutine STREAM

         lvec = ispec(ivec,4)
         if ( isolut2d(jndsol)==0 ) then

!        --- velocity vector has not been filled

            call errchr ( 'velocity', 1 )
            call errchr ( names(jvec), 2 )
            call errwar ( 2073, 0, 0, 2 )
            go to 1000

         end if
         if ( iinputpst(32)>=1 ) then

!        --- mmiskip>0, Element groups must be skipped
!            Fill array iskip and store temporary at the end of ibuffr

            ichoice = 1

         else

!        --- mmiskip = 0, No element groups must be skipped

            ichoice = 0

         end if

         if ( debug ) write(irefwr,*) 'call to0040/2'
         call sepstream ( ichoice, ichels, lvec, &
                          isolut2d(indsol), jpoint, psiphi, &
                          isolut2d(jndsol), iskiparr )

      else if ( mod(ispec(ivec,1),100)==2 ) then

!     --- isol may be computed by subroutine INTP2D

         if ( debug ) write(irefwr,*) 'call intp2d'
         isolut => isolut2d(jndsol)
         call sepintp2d  ( ichint, ivalue, valint(1,ivec), &
                           numint(ivec), isolut2d(indsol), iskip, idummy )

      else if ( mod(ispec(ivec,1),100)==3 ) then

!     --- isol may be computed by subroutine INTP3D

         ichoice = ispec(ivec,4)
         if ( debug ) write(irefwr,1) 'imesh, intp3d/2', imesh
         kprob => kprob2d(1)
         call sepintp3d ( ichoice, valint(1,ivec), jndsol, &
                          indsol, numint(ivec), imesh )
         kprob => kprob2d(imesh)

      else if ( mod(ispec(ivec,1),100)==4 ) then

!     --- Store mach number in ISOL

         if ( isolut2d(jndsol)==0 ) then

!        --- velocity vector has not been filled

            call errchr ( 'velocity', 1 )
            call errchr ( names(jvec), 2 )
            call errwar ( 2073, 0, 0, 2 )
            go to 1000

         end if

         iseqent = ispec(ivec,4)
         entndsol = imapsol(iseqent,itimst)
         if ( entndsol==0 ) then

!        --- Array has not been filled

            call errint ( iseqent, 1 )
            call errint ( itimst, 2 )
            call errsub ( 2230, 2, 0, 0 )
            go to 1000

         end if
         if ( isolut2d(entndsol)==0 ) then

!        --- enthalpy vector has not been filled

            call errchr ( 'enthalpy', 1 )
            call errchr ( names(iseqent), 2 )
            call errwar ( 2073, 0, 0, 2 )
            go to 1000

         end if
         call prmachno ( isolut2d(jndsol), isolut2d(entndsol), &
                         isolut2d(indsol), ipost, rpost )

      else if ( mod(ispec(ivec,1),100)==5 ) then

!     --- Store total enthalpy in ISOL

         iseqent = ispec(ivec,4)
         entndsol = imapsol(iseqent,itimst)
         if ( entndsol==0 ) then

!        --- Array has not been filled

            call errint ( iseqent, 1 )
            call errint ( itimst, 2 )
            call errsub ( 2230, 2, 0, 0 )
            go to 1000

         end if
         if ( isolut2d(jndsol)==0 ) then

!        --- velocity vector has not been filled

            call errchr ( 'velocity', 1 )
            call errchr ( names(jvec), 2 )
            call errwar ( 2073, 0, 0, 2 )
            go to 1000

         end if
         if ( isolut2d(entndsol)==0 ) then

!        --- enthalpy vector has not been filled

            call errchr ( 'enthalpy', 1 )
            call errchr ( names(iseqent), 2 )
            call errwar ( 2073, 0, 0, 2 )
            go to 1000

         end if
         call prtotenth ( isolut2d(jndsol), isolut2d(entndsol), &
                          isolut2d(indsol), ipost, rpost )

      else if ( mod(ispec(ivec,1),100)==6 ) then

!     --- Store stagnation pressure in ISOL

         if ( isolut2d(jndsol)==0 ) then

!        --- pressure vector has not been filled

            call errchr ( 'pressure', 1 )
            call errchr ( names(jvec), 2 )
            call errwar ( 2073, 0, 0, 2 )
            go to 1000

         end if

         iseqmach = ispec(ivec,4)
         machndsol = imapsol(iseqmach,itimst)
         if ( machndsol==0 ) then

!        --- Array has not been filled

            call errint ( iseqmach, 1 )
            call errint ( itimst, 2 )
            call errsub ( 2230, 2, 0, 0 )
            go to 1000

         end if
         if ( isolut2d(machndsol)==0 ) then

!        --- mach_number has not been filled
!            try to fill this vector

           iseqent = ispec(iseqmach,4)
           if ( iseqent==0 ) then

!          --- There is no vector with the name enthalpy

               call errchr ( 'ENTHALPY', 1 )
               call errwar ( 2074, 0, 0, 1 )
               go to 1000

            end if
            entndsol = imapsol(iseqent,itimst)
            if ( entndsol==0 ) then

!           --- Array has not been filled

               call errint ( iseqent, 1 )
               call errint ( itimst, 2 )
               call errsub ( 2230, 2, 0, 0 )
               go to 1000

            end if
            if ( isolut2d(entndsol)==0 ) then

!           --- enthalpy vector has not been filled

               call errchr ( 'enthalpy', 1 )
               call errchr ( names(iseqent), 2 )
               call errwar ( 2073, 0, 0, 2 )
               go to 1000

            end if

            iseqvel = ispec(iseqmach,2)
            if ( iseqvel==0 ) then

!           --- There is no vector with the name velocity

               call errchr ( 'VELOCITY', 1 )
               call errwar ( 2074, 0, 0, 1 )
               go to 1000

            end if
            velndsol = imapsol(iseqvel,itimst)
            if ( velndsol==0 ) then

!           --- Array has not been filled

               call errint ( iseqvel, 1 )
               call errint ( itimst, 2 )
               call errsub ( 2230, 2, 0, 0 )
               go to 1000

            end if
            if ( isolut2d(velndsol)==0 ) then

!           --- velocity vector has not been filled

               call errchr ( 'velocity', 1 )
               call errchr ( names(iseqvel), 2 )
               call errwar ( 2073, 0, 0, 2 )
               go to 1000

            end if

!           --- Compute mach number

            call prmachno ( isolut2d(velndsol), isolut2d(entndsol), &
                            isolut2d(machndsol), ipost, rpost )

         end if

         call prstagpres ( isolut2d(jndsol), isolut2d(machndsol), &
                           isolut2d(indsol), rpost )

      else if ( mod(ispec(ivec,1),100)==7 ) then

!     --- Store cp in ISOL

         call prcpcoef ( isolut2d(jndsol), isolut2d(indsol), ipost, rpost )

      else if ( mod(ispec(ivec,1),100)==8 ) then

!     --- Store entropy in ISOL
!         Number corresponding to density equation;
!         should be done in a more proper manner
!         (iseqdens = ndim+1+nturb+ntrnsp+1)

         iseqdens = 5
         densndsol = imapsol(iseqdens,itimst)
         call prentropy ( isolut2d(jndsol), isolut2d(densndsol), &
                          isolut2d(indsol), ipost, rpost )

      else if ( mod(ispec(ivec,1),100)==9 ) then

!     --- Interpolate from cell quantities to nodal quantities

         iinvec(1) = ispec(ivec,4)
         iinvec(2) = jndsol
         iinvec(3) = indsol
         iinvec(4) = 3  !!moet aangepast
         call erintpol ( iinvec, isolut2d )

      else if ( mod(ispec(ivec,1),100)==10 ) then

!     --- Compute normal stress

         iinvec(1) = 5              ! number of entries filled
         iinvec(2) = jndsol         ! iseqstress
         iinvec(3) = indsol         ! iseqnormstress
         iinvec(4) = mod(ispec(ivec,4),10)  ! idim
         call ercompvector ( isolut2d, iinvec, maxsoluts )

      else if ( mod(ispec(ivec,1),100)==25 ) then

!     --- use subroutine MANVEC to compute solution

         iinvec(1) = 5
         iinvec(2) = 43
         iinvec(3) = 0
         iinvec(4) = 0
         iinvec(5) = ispec(ivec,4)
         call sepmanvec ( iinvec, rinvec, isolut2d(jndsol), &
                          isolut2d(jndsol), isolut2d(indsol) )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End insp01'

      end if  ! ( debug )
      call erclos ( 'insp01' )

      end subroutine insp01
