      subroutine insp02 ( user, iuser )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.5  date 06-03-2012 Initialize indsol
!        version  4.4  date 22-11-2006 Extra debug
!        version  4.3  date 30-08-2006 New definition of cpost5
!
!   copyright (c) 1990-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     INSP02 is used to perform the print and plot commands for
!     solution vectors and derived quantities
!     subroutine INSP02 is a help subroutine of subroutine SEPPS1
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     post_processing
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulepost
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulemain
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iuser(105)
      double precision user(105)

!     iuser          i    Help array for integer coefficients (particle tracking
!                         and plot field)
!     user           i    Help array for real coefficients (particle tracking
!                         and plot field)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ivec, kader, jtimessv, indsol
      integer, target :: isolloc
      real :: rwork(100)
      logical mapped
      integer, pointer :: kmeshlc, kproblc

!     indsol         Sequence number of vector in isol
!     isolloc        Local array isol, contains interpolated values
!     ivec           Sequence number of array to be processed
!     jtimessv       Help variable to store the input value of jtimes
!     kader          Value of jkader at input (save parameter)
!     kmeshlc        Local array KMESH (corresponds to rectangular grid)
!     kproblc        Local array KPROB (corresponds to rectangular grid)
!     mapped         if true the mapping from 3d to 2d plane is carried out
!     rwork          Work array for subroutine PLAF05
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     INSP02SUB      Actual printing and plotting subroutine
!     MSHMAPRECT     Map the solution defined on an arbitrary mesh onto a
!                    rectangular grid
!     MSHMAPTOPLANE  Map the solution defined on a plane in a 3d mesh onto
!                    a plane in a 2d mesh
!     PLAFEP         General SEPRAN low level plotting subroutine
!     PLAFP7         Plot text using absolute co-ordinates
!     PLLEGN         Plot bottom text and axis
!     PRININ         print 1d integer array
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPFILLKPROB   Fill parameters in module sepmodulekprob
!     SEPRMMESHINFO  Remove array with sequence number iseq from km
!     SEPRMPROBINFO  Remove array with sequence number kproblc from kp
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2072   Vector has not been filled
!    2230   solution array has not been filled
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'insp02' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from insp02'
         call prinin ( iinputpst, 100, 'iinputpst' )
         if ( icurvs(1)>0 ) then
            call prinin ( icurvs(2), icurvs(1), 'curves/surfaces' )
         end if  ! ( icurvs(1)>0 )
         write(irefwr,1) 'itimst, jtimes', itimst, jtimes
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Set some constpstants and fill first position of local arrays

      kader = jkader
      jtimessv = jtimes
      kmeshlc => kmesh2d(maxmesh+1)
      kproblc => kprob2d(maxmesh+1)
      isolloc = 0
      call sepfillkmesh ( kmesh )
      call sepfillkprob

      ivec = iinputpst(2)
      indsol = 1
      if ( debug ) write(irefwr,1) 'ivec, iinputpst(1)', ivec, iinputpst(1)
      if ( ivec<=1000 .and. &
           ( iinputpst(1)<=124 .or. iinputpst(1)>130 ) ) then
         indsol = imapsol(ivec,itimst)
         if ( debug ) write(irefwr,1) 'indsol', indsol
         if ( indsol==0 ) then

!        --- Array has not been filled

            call errint ( ivec, 1 )
            call errint ( itimst, 2 )
            call errsub ( 2230, 2, 0, 0 )
            go to 1000

         end if
         if ( debug ) write(irefwr,1) 'isolut2d(indsol)', isolut2d(indsol)
         if ( (iinputpst(1)<125 .or. iinputpst(1)>126 ) .and. &
              isolut2d(indsol)==0 ) then

!        --- Array has not been filled, skip this operation

            call errchr ( names(ivec), 1 )
            call errwar ( 2072, 0, 0, 1 )
            go to 1000

         end if

      end if
      if ( ierror/=0 ) go to 1000

!     --- Perform actual plotting or printing

      if ( iinputpst(91)/=0 ) then

!     --- Special case, map on rectangular grid

         if ( debug ) then
            write(irefwr,*) 'indsol ', indsol
            call prinin ( isolut2d(indsol), 5, 'isol' )
         end if  ! ( debug )
         call mshmaprect ( isolut2d(indsol), isolloc, &
                           rinputpst(31), iinputpst(67), icurvs )

!        --- Perform printing or plotting

         kmesh => kmeshlc
         kprob => kproblc
         isolut => isolloc
         call insp02sub ( user, iuser, iinputpst, rinputpst, &
                          iconstpst, constpst, ivec )

!        --- Remove local arrays

         call seprmmeshinfo ( kmeshlc )
         call seprmprobinfo ( kprob )
         call seprmsolinfo ( isolloc )

      else if ( ndim==3 .and. iinputpst(67)==3 .and. &
                iinputpst(1)>=110 .and. iinputpst(1)<=115 ) then

!     --- Special case, plot surface in 3d
!         The surface is mapped onto a plane if possible and
!         a 2d plot is made
!         At this moment only a plane is allowed

         call mshmaptoplane ( kmeshlc, isolut2d(indsol), &
                              isolloc, mapped, icurvs(2), icurvs(1) )
         if ( debug ) write(irefwr,*) 'mapped ', mapped

         if ( mapped ) then

!        --- mapped is true, so surfaces are mapped onto plane
!            continue with local arrays

            kmesh => kmeshlc
            kprob => kproblc
            isolut => isolloc
            call insp02sub ( user, iuser, iinputpst, rinputpst, &
                             iconstpst, constpst, ivec )

!           --- Remove local arrays

            call seprmmeshinfo ( kmeshlc )
            call seprmprobinfo ( kproblc )
            call seprmsolinfo ( isolloc )

         else

!        --- Standard case

            isolut => isolut2d(indsol)
            call insp02sub ( user, iuser, iinputpst, rinputpst, &
                             iconstpst, constpst, ivec )

         end if  ! ( mapped )

      else

!     --- Standard case

         isolut => isolut2d(indsol)
         call insp02sub ( user, iuser, iinputpst, rinputpst, &
                          iconstpst, constpst, ivec )

      end if  ! ( iinputpst(91)==0 )

      if ( debug ) write(irefwr,1) 'jtimes, jtimessv', jtimes, jtimessv
      jtimes = jtimessv

!     --- Plot bottom text and axis if necessary

      if ( jtimessv==11 .and. ( iinputpst(1)<120 .or. &
           iinputpst(1)>121 .or. iinputpst(22)==iinputpst(23) ) ) then

!     --- The bottom text is plotted when jtimes = 11, &
!         except if plotfn (iinputpst(1)=120) or plotln (iinputpst(1)=121)
!         is called for a picture number (iinputpst(22)) smaller than
!         the last picture number (iinputpst(23))

         if ( iinputpst(1)>=103 .and. iinputpst(1)<120 .and. &
              iinputpst(71)==0  ) then
            ntextx = iinputpst(37)
            ntexty = iinputpst(38)
            ntext  = iinputpst(39)
            if ( ntext==0 ) then
               if ( iinputpst(1)==110 ) then
                  ntext = 15
                  textid(1)(1:ntext) = 'Vector plot of '
               else if ( iinputpst(1)==112 .or. iinputpst(1)==113 ) then
                  ntext = 18
                  textid(1)(1:ntext) = 'Contour levels of '
               else if ( iinputpst(1)==115 .or. iinputpst(1)==116 ) then
                  ntext = 11
                  textid(1)(1:ntext) = '3D plot of '
               else
                  ntext = 1
                  textid(1)(1:1) = ' '
               end if
               if ( ntext>1 ) then
                  textid(1)(ntext+1:ntext+30) = names(ivec)
                  ntext = ntext + 30
               end if
            end if
            call pllegn ( rinputpst(16) )
         end if
         if ( ierror/=0 ) go to 1000
         if ( iinputpst(1)>=101 .and. iinputpst(1)<130 .and. &
              iinputpst(44)/=0 ) then
            if ( iinputpst(44)==1 ) then
               iinputpst(44) = 2
            end if
            call plafp7 ( xorgid, yorgid, identf, 0d0, .25d0, &
                          iinputpst(45) )
         end if

      end if
      if ( ierror/=0 ) go to 1000

!     --- Update jtimes

      if ( debug ) write(irefwr,1) 'jtimes, jtimst/1', jtimes, jtimst
      jtimes = jtimst
      if ( jtimes==11 .and. iinputpst(1)>=101 .and. &
           iinputpst(1)<130 ) jtimes = 12
      if ( debug ) write(irefwr,1) 'jtimes, jtimst/2', jtimes, jtimst

!     --- The picture is closed when jtimes = 10, &
!         except if plotfn (iinputpst(1)=120) or plotln (iinputpst(1)=121)
!         is called for a picture number (iinputpst(22)) smaller than
!         the last picture number (iinputpst(23))

      if ( jtimes==10  .and. iinputpst(1)>=110 .and. &
           iinputpst(1)<130  .and. ( iinputpst(1)<120 .or. &
           iinputpst(1)>121 .or. iinputpst(22)==iinputpst(23) ) ) &
           call plafep ( 2, 0d0, 0d0, 0d0, 0d0, 0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'jtimes', jtimes
         write(irefwr,*) 'End insp02'

      end if
      call erclos ( 'insp02' )

      jkader = kader

      end subroutine insp02
