      subroutine plottimehis ( inpsep, rinsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 28-03-2015 Extension with scalars
!        version  2.0  date 16-04-2014 Extra parameters isolut2d, kprob
!        version  1.3  date 25-11-2012 Long integers
!
!   copyright (c) 2010-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Plot time history for one vector or scalar
!
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     time_history
!     scalar
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletimhis
      use sepmodulemain
      use sepmodulemnsub
      use sepmodulekprob
      use sepmodulecpack
      use sepmoduleplot
      use sepmoduletexts
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*)
      double precision rinsep(*)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         For a description see subroutine readscplottim
!     rinsep         i    Array containing the real input for this subroutine
!                         For a description see subroutine readscplottim
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ivect, nch1, istrin, ncolours, idegfd, &
              iscal, num, i, numtim, istartscal
      integer, allocatable :: iwork(:)
      double precision, allocatable :: work(:,:)
      character (len=80) namear, name

!                    they are not
!     i              Counting variable
!     idegfd         Degree of freedom sequence number
!     iscal          Sequence number of scalar
!     istartscal     Start of scalars in inpsep
!     istrin         Starting address of information for this subroutine
!                    in RINSEP
!     ivect          Vector sequence number
!     iwork          integer work array to store the sequence numbers of the
!                    scalars
!     name           Name of array
!     namear         Name of array
!     nch1           Number of characters in name
!     ncolours       Number of colors that are stored in inpsep
!     num            Number of scalars
!     numtim         Number of time steps
!     work           Work array to store the time history of all scalars
!                    to be plotted as well as the time
!                    work(1,:) contains the time
!                    work(1+i,:) contains the i^th scalar
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PLOTTIMEHIS1   Plot contents of time history array
!     PLOTTIMEHIS2   ?
!     PRININ         print 1d integer array
!     PRINRL1        Print 2d real vector
!     PRINTIMEHIS3   Store contents of time history of one scalar in array work
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3051   Degree of freedom out of range
!    3052   No time history has been stored for vector
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plottimehis' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plottimehis'
         write(irefwr,1) 'istartmn, inodenr', istartmn, inodenr
         write(irefwr,*) 'parallel ', parallel
         call prinin ( inpsep, 11, 'inpsep' )
  1      format ( a, 1x, (10i9) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ivect = inpsep(1)    ! vector sequence number
      ncolours = inpsep(3) ! number of colours given

!     --- skip in parallel case and not on host

      if ( parallel .and. inodenr/=1 ) then

!     --- Parallel but not on host: jump

         num = 0
         if ( ivect<-10000 ) num = -ivect-10000
         go to 900

      end if  ! ( parallel .and. inodenr/=1 )

!     --- Get parameters

      istrin = inpsep(2)   ! starting address in rinsep
      idegfd = inpsep(5)   ! degree of freedom
      nphysi = 1            ! number of degrees of freedom per point
      num = 0

      if ( ivect<-10000 ) then

!     --- scalars to be plotted

         num = -ivect-10000  ! number of scalars
         idegfd = inpsep(6)  ! skip zero
         numtim = istephiscal-idegfd  ! number of timesteps
         istartscal = inpsep(7)

         if ( debug ) &
            write(irefwr,1) 'num, numtim, idegfd, istartscal', &
                             num, numtim, idegfd, istartscal

         allocate ( work(num+1,numtim), iwork(num), stat = error )
         if ( error/=0 ) call eralloc ( error, num*numtim, 'work' )
         if ( ierror/=0 ) go to 1000

!        --- Store scalars in work array

         do i = 1, num

            iscal = inpsep(istartscal-1+i)
            iwork(i) = iscal
            rvectim => scaltim(iscal)%arr
            call printimehis3 ( work, i, idegfd, num )

         end do  ! i = 1, num

         if ( debug ) then
            call prinin ( iwork, num, 'iwork' )
            call prinrl1 ( work, numtim, num+1, 'work' )
         end if  ! ( debug )

!        --- Plot function

         call plottimehis2 ( work, num, rinsep(istrin), inpsep, &
                             inpsep(11), numtim, iwork )

         deallocate ( iwork, work, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'iwork' )
         if ( ierror/=0 ) go to 1000

         go to 900

      else if ( ivect<-1000 ) then

         select case (ivect)
         case(-1001)
            namear = 'x'
         case(-1002)
            namear = 'y'
         case(-1003)
            namear = 'z'
         end select  ! case (ivect)
         name = namear
         nch1 = 1

         ivect = maxvc1-ivect-1000

      else if ( ivect==-1000 ) then

         ivect = maxvc1+1
         namear = 'x'
         name = namear
         nch1 = 1

      else

         nch1  = index(names(ivect),' ')-1
         if ( debug ) write(irefwr,1) 'nch1', nch1
         namear = names(ivect)(1:nch1)//'_timehis'
         name = names(ivect)

         call sepactsolbf2 ( isolut2d(ivect) )

      end if  ! ( ivect<-1000 )

      if ( vecttim(ivect)%npoint_hist<=0 ) then

!     --- Error: There is no time history for this vector

         call errint ( ivect, 1 )
         call errchr ( namear, 1 )
         call errsub ( 3052, 10, 0, 1 )
         go to 1000

      end if  ! ( vecttim(ivect)%npoint_hist<=0 )

      if ( idegfd>nphysi ) then

!     --- Degree of freedom out of range

         call errint ( idegfd, 1 )
         call errint ( nphysi, 2 )
         call errchr ( namear, 1 )
         call errsub ( 3051, 2, 0, 1 )
         go to 1000

      end if  ! ( idegfd>nphysi )

      rvectim => vecttim(ivect)%arr

      npoint_hist = vecttim(ivect)%npoint_hist

      call plottimehis1 ( rinsep(istrin), name, inpsep, inpsep(11), idegfd )

!     --- Raise istartmn

900   istartmn = istartmn + 11+ncolours+num

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'istartmn', istartmn
         write(irefwr,*) 'End plottimehis'

      end if  ! ( debug )
      call erclos ( 'plottimehis' )

      end
