      subroutine pravs1 ( ichois, scale, iref, coor, ndim, npoint, &
                          nelem, npelm, kmeshc, nunkp, iseq, time, &
                          ntime )
! ======================================================================
!
!        programmer    Martien Hulsen
!        version  2.1  date 25-11-2012 Long integers
!        version  2.0  date 17-06-1993  scaling of coordinates
!        version  1.0  date 06-06-1993
!
!   copyright (c) 1993-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Write coordinates and topology to file sepavs.xxx for (x,y,t) data.
!   A 2D mesh is repeated ntime times to contruct a 3D mesh consisting
!   of prisms. The actual solution vectors are written by a separate
!   routine PRAVS2. The mesh must be 2D but the data may be contain 3D vectors.
!   This possibility is meant for a SEPRAN-AVS interface
!   The array should already have been mapped on a triangular mesh.
!   It is assumed that the solution vector contains only scalars in each
!   point or vectors
!   There are no points to be skipped
! **********************************************************************
!
!                       KEYWORDS
!
!     avs
!     output
!     vector
!     write
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
!    - &
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichois, iref, ndim, npoint, nelem, nunkp, npelm, iseq, &
              kmeshc(npelm,nelem), ntime
      double precision scale, coor(ndim,npoint), time(ntime)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     ichois         i    Choice parameter
!                         0:  Only time axis is scaled. Scaling according
!                             to the minimum of xmax-xmin, ymax-ymin.
!                         1:  time axis and either x or y is scaled. The
!                             coordinate direction with minimum width and
!                             the time axis is scaled to SCALE * maximum width.
!     iref           i    File reference number
!     iseq           i    Sequence number of vector
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     ndim           i    Dimension of the space.
!     nelem          i    Number of elements
!     npelm          i    Number of points per element
!     npoint         i    Number of nodal points
!     ntime          i    Number of time-steps
!     nunkp          i    Maximum number of unknowns in a point
!     scale          i    Scale factor for ichois=1.
!     time           i    Array of actual times.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, m, n, itime, ncomp
      double precision xmin, xmax, ymin, ymax, tfac, xfac, yfac

!     i              Counting variable
!     itime          Integer time sequence number
!     j              Counting variable
!     k              Counting variable
!     m
!     n
!     ncomp          Number of components (for AVS) in a nodal point.
!                    It is 1 for a scalar and 3 for a vector (2D and 3D).
!     tfac           Scale factor for the time. The time scaling is such that
!                    the length of the t-axis is equal to
!                    minimum ( xmax - xmin, ymax - ymin )
!     xfac           Scale factor for the x co-ordinate
!     xmax           Maximum x-coordinate
!     xmin           Minimum x-coordinate
!     yfac           Scale factor for the y co-ordinate
!     ymax           Maximum y-coordinate
!     ymin           Minimum y-coordinate
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
!    This subroutine writes the co-ordinates and the topology
!    to the file indicated by iref.
!    This file is in standard formatted AVS format
!    Shape of the file:                 meaning
!    #   SEPRAN - AVS OUTPUT FILE       comments
!    #
!    #   (x,y,t) data
!    #   time interval = (..,..)
!    #
!    npoint, nelem, ncomp, 0, 0         1. dimensions
!    1     x1 y1 z1                     2. node-id and co-ordinates
!    2     x2 y2 z2
!          .
!          .
!          .
!    1  1  cel-type i1 i2 i3 ...        3. node connectivity list
!    2  1  cel-type i1 i2 i3 ...
!          .
!          .
!          .
!    1  ncomp                           4. number of components of vec
!    vector iseq,                       5. component label, units label (empty)
!    1  u1,1 u1,2 u1,3                  6. data values
!    2  u2,1 u2,2 u2,3
!          .
!          .
!          .
!    Part 6 is not written in this routine and must be written
!    separately by ntime calls to pravs2.
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Set ncomp
!   Write comments to file
!   Write dimension parameters to file
!   Write co-ordinates to file
!   Write topology to file
!   Write information of solution to file
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Set ncomp

      if ( nunkp==1 ) then

!     --- scalar

         ncomp = 1
      else

!     --- vector (2D or 3D)

         ncomp = 3
      end if

!     --- Write comments to file

      write(iref,100) time(1), time(ntime)
100   format ( '#   SEPRAN - AVS OUTPUT FILE'/ &
               '#'/ &
               '#   (x,y,t) data'/ &
               '#   time interval ', e11.3, '  --', e11.3/ &
               '#' )

!     --- Write dimension parameters to file

      write(iref,110) npoint * ntime, nelem * ( ntime - 1 ) , ncomp
110   format ( 3i6, ' 0 0' )

!     --- Find co-ordinates span

      xmin = 0
      xmax = 0
      ymin = 0
      ymax = 0
      do 30 i = 1, npoint
         xmin = min(xmin,coor(1,i))
         xmax = max(xmax,coor(1,i))
         ymin = min(ymin,coor(2,i))
         ymax = max(ymax,coor(2,i))
 30   continue
      if ( ichois==0 ) then

!     --- Only time axis scaling

         xfac = 1d0
         tfac = min( xmax - xmin, ymax - ymin ) &
                        / ( time(ntime) - time(1) )
         yfac = 1d0
      else if ( ichois==1 ) then

!     --- Scale time and one coordinate axis

         if ( xmax - xmin<ymax - ymin ) then
            xfac = scale * ( ymax - ymin ) / ( xmax - xmin )
            yfac = 1d0
            tfac = scale * ( ymax - ymin ) / ( time(ntime) - time(1) )
         else
            xfac = 1d0
            yfac = scale * ( xmax - xmin ) / ( ymax - ymin )
            tfac = scale * ( xmax - xmin ) / ( time(ntime) - time(1) )
         end if
      end if

!     --- Write co-ordinates to file

      do 10 itime = 1, ntime
         k = ( itime - 1 ) * npoint
         write(iref,200) ( i+k, xfac*coor(1,i), yfac*coor(2,i), &
                           tfac*time(itime), i=1,npoint )
200      format ( i6, 3e12.4 )
 10   continue

!     --- Write topology to file

      do 20 itime = 1, ntime - 1
         k = ( itime - 1 ) * npoint
         m = k + npoint
         n = ( itime - 1 ) * nelem
         write(iref,300) ( i+n, (kmeshc(j,i)+k, j=1,3), &
                                (kmeshc(j,i)+m, j=1,3), i=1,nelem )
300      format ( i6, ' 1 prism ', 6i6 )
 20   continue

!     --- Write information of solution to file

      write(iref,400) ncomp
400   format ( '1', i3 )
      write(iref,410) iseq
410   format ( 'vector', i3, ',' )
      end
