      function prevalboolexpr ( inpsep, leninpsep, rinsep, text )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.1  date 08-12-2013 Extension with integer scalars
!        version  3.0  date 14-11-2013 Extra parameter text
!        version  2.1  date 20-04-2012 Indexed scalars
!
!   copyright (c) 2008-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Evaluate boolean expression and return with result
!
! **********************************************************************
!
!                       KEYWORDS
!
!     boolean_expression
!     boolean
!     evaluate
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), leninpsep
      double precision rinsep(*)
      logical prevalboolexpr
      character (len=*) text

!     inpsep         i    Contains the integer information corresponding
!                         to the boolean expression
!                         See subroutine READBOOLEXPR
!     leninpsep      i    Length of array inpsep
!     prevalboolexpr o    Result of the boolean expression
!     rinsep         i    Array containing the real input for this subroutine
!     text           i    String variable containing some text for error message
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer maxlevels, lenwork
      parameter ( maxlevels=10, lenwork=200 )
      integer ibwork(lenwork), levels(maxlevels)
      integer ipos, nlevel, numreal, icount, nexpr, jstart, ivalue
      double precision reals(2)
      logical finished, bwork(lenwork), bool, integerfound

!     bool           help variable to store a boolean
!     bwork          Array to store boolean values corresponding to ibwork
!     finished       Indicates if the loop has finished (true) or not
!                    (false)
!     ibwork         In this array the boolean operators are stored
!                    Must be used together with bwork
!                    Length icount
!                    ibwork(i) contains the operator
!                    bwork(i) the value of the boolean
!                    polish notation
!     icount         Counter
!     integerfound   If true an integer has been found
!     ipos           Position in array
!     ivalue         Integer value
!     jstart         Position in array
!     lenwork        Length of array work
!     levels         Work array to store position of openings brackets
!     maxlevels      Declared length of array lvels
!     nexpr          Number of items in bwork and ibwork to be considered
!     nlevel         Number of levels stored in levels
!     numreal        Number of reals read, at both sides of a boolean operator
!     reals          Array of size 2 to store reals read
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      logical prevalbool, prevalboolend

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERSCALVAL      Fill value of scalar defined by ipos into either rvalue or
!                    ivalue
!     PREVALBOOL     Evaluate the expression real1 boolean_operator real2
!     PREVALBOOLEND  Evaluate final boolean expression
!     PREVALEXPR     Evaluate expression and fill actual values of scalars
!     PRINBOOL       Print an array of booleans
!     PRININ         print 1d integer array
!     PRININLARGE    Print 1d integer vector (large numbers)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     499   Internal SEPRAN error
!    2940   Number of reals before boolean operator not 1
!    2941   No real found after boolean operator
!    2942   More than one real found, without a boolean operator in between
! **********************************************************************
!
!                       PSEUDO CODE
!
!     prevalboolexpr := false
!     finished := false; boolean := false
!     ipos := istart+2
!     nlevel := 1; numreal:=0; icount:=1
!     levels(1) := 1;
!     while ( not finished ) do
!        if ( inpsep(ipos)==11 ) then
!           nlevel := nlevel+1;  ipos:= ipos+1
!           levels(nlevel) := icount;
!        else if ( inpsep(ipos)==12 ) then
!           evaluate expression
!           adapt ibwork, bwork and levels
!        else if ( inpsep(ipos)==100 ) then
!           finished = true
!           evaluate expression
!        else if ( inpsep(ipos)<0 ) then
!           numreal := numreal+1
!           reals(numreal) := evaluate number
!        else if ( inpsep(ipos)<7 ) then
!           bool := result operator on 2 reals
!           numreal := 0
!        else if ( inpsep(ipos)>6 and inpsep(ipos)<10 ) then
!           ibwork(icount) := boolean_operator sequence number
!           bwork(icount) := bool
!           icount := icount+1
!        else
!           error
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prevalboolexpr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prevalboolexpr'
         write(irefwr,1) 'leninpsep', leninpsep
         call prininlarge ( inpsep, leninpsep, 'inpsep' )
  1      format ( a, 1x, (10i10) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- prevalboolexpr := false
!         finished := false; boolean := false
!         ipos := istart+2
!         nlevel := 1; numreal:=0; icount:=0
!         levels(1) := 1;

      prevalboolexpr = .false.
      finished = .false.
      ipos = 1
      nlevel = 1
      numreal = 0
      icount = 1
      levels(1) = 1
      ibwork(1) = 0
      reals = 0d0

!     --- while ( not finished ) do

      do while ( .not. finished )

         if ( debug ) &
            write(irefwr,1) 'ipos, inpsep(ipos)', ipos, inpsep(ipos)

         if ( ipos>leninpsep) then

!        --- end reached
!            finished = true
!            evaluate expression

            if ( icount==1 ) then

!           --- icount = 1, Boolean result is stored in bwork(1)

               prevalboolexpr = bwork(1)

            else

!           --- icount>1, Boolean must be evaluated

               prevalboolexpr = prevalboolend ( bwork, ibwork, icount )

            end if  ! ( icount==1 )
            finished = .true.

         else if ( inpsep(ipos)==11 ) then

!        --- inpsep(ipos)=11, i.e. openings bracket found
!            nlevel := nlevel+1;  ipos:= ipos+1
!            levels(nlevel) := icount

            ipos = ipos+1
            icount = icount+1  ! raise icount
            levels(nlevel) = icount
            ibwork(icount) = 0  ! dummy operator
            nlevel = nlevel+1

            if ( debug ) then
               write(irefwr,1) 'open bracket'
               write(irefwr,1) 'nlevel, ipos, icount', &
                                nlevel, ipos, icount
               call prinbool ( bwork, icount, 'bwork' )
               call prinin ( ibwork, icount, 'ibwork' )
            end if  ! ( debug )

         else if ( inpsep(ipos)==12 ) then

!        --- inpsep(ipos)==12, i.e. closing bracket found
!            evaluate expression
!            adapt ibwork, bwork and levels

            if ( debug ) then
               write(irefwr,1) 'close bracket'
               write(irefwr,1) 'nlevel, ipos, icount', &
                                nlevel, ipos, icount
               call prinbool ( bwork, icount, 'bwork' )
               call prinin ( ibwork, icount, 'ibwork' )
            end if  ! ( debug )

            nlevel = nlevel-1     ! one level less
            nexpr = icount-levels(nlevel)
            icount = levels(nlevel)  ! reset icount
            if ( nexpr>1 ) then

!           --- Evaluate expression between brackets and put into bwork

               bwork(icount-1) = &
                  prevalboolend ( bwork(icount), ibwork(icount), nexpr )

            else

!           --- No expression, only boolean, put in bwork

               bwork(icount-1) =bwork(icount)

            end if  ! ( icount-levels(nlevel)>1 )

            icount = icount-1     ! reset icount
            ipos = ipos+1         ! raise positin in inpsep

            if ( debug ) then
               write(irefwr,1) 'nlevel, ipos, icount', &
                                nlevel, ipos, icount
               call prinbool ( bwork, icount, 'bwork' )
               call prinin ( ibwork, icount, 'ibwork' )
            end if  ! ( debug )

         else if ( inpsep(ipos)==100 ) then

!        --- inpsep(ipos)==100 end of expression found
!            finished = true
!            evaluate expression

            if ( icount==1 ) then

!           --- icount = 1, Boolean result is stored in bwork(1)

               prevalboolexpr = bwork(1)

            else

!           --- icount>1, Boolean must be evaluated

               prevalboolexpr = prevalboolend ( bwork, ibwork, icount )

            end if  ! ( icount==1 )
            finished = .true.

         else if ( inpsep(ipos)>1000 ) then

!        --- inpsep(ipos)>1000, real or real expression found
!            numreal := numreal+1
!            reals(numreal) := evaluate number

            if ( numreal/=0 ) then

!           --- The number of reals read before is not 0: error

               call errint ( numreal, 1 )
               call errsub ( 2942, 1, 0, 0 )
               go to 1000

            else if ( inpsep(ipos)>=200000000 ) then

!           --- evaluate expression

               call prevalexpr ( inpsep(ipos), reals(1), text )

            else if ( inpsep(ipos)>=100000000 ) then

!           --- scalar found

               call erscalval ( inpsep(ipos)-100000000, reals(1), &
                                ivalue, integerfound )

            else if ( inpsep(ipos)>1000 ) then

!           --- Real found

               reals(1) = rinsep(inpsep(ipos)-1000)

            end if  ! ( numreal/=0 )

            if ( debug ) write(irefwr,2) 'reals(1)', reals(1)

            numreal = 1
            ipos = ipos+1

         else if ( inpsep(ipos)<7 ) then

!        --- inpsep(ipos)<7 boolean operator between 2 reals
!            bool := result operator on 2 reals
!            numreal := 0

            if ( debug ) then
               write(irefwr,*) 'boolean operator between reals'
               write(irefwr,1) 'numreal, inpsep(ipos+1)', &
                                numreal, inpsep(ipos+1)
            end if  ! ( debug )
            if ( numreal/=1 ) then

!           --- The number of reals read before is not 1: error

               call errint ( numreal, 1 )
               call errint ( inpsep(ipos), 2 )
               call errsub ( 2940, 2, 0, 0 )
               go to 1000

            else if ( inpsep(ipos+1)>=200000000 ) then

!           --- evaluate expression

               call prevalexpr ( inpsep(ipos+1), reals(2), text )

            else if ( inpsep(ipos+1)>100000000 ) then

!           --- Scalar found

               reals(2) = scalars(inpsep(ipos+1)-100000000)

            else if ( inpsep(ipos+1)>1000 ) then

!           --- Real found

               reals(2) = rinsep(inpsep(ipos+1)-1000)

            else if ( inpsep(ipos+1)>=0 ) then

!           --- The next item is not a real

               call errint ( inpsep(ipos+1), 1 )
               call errint ( inpsep(ipos), 2 )
               call errsub ( 2941, 2, 0, 0 )
               go to 1000

            else

!           --- real or real expression found
!               reals(2) := evaluate number

               jstart = -inpsep(ipos+1)
               leninpsep = inpsep(jstart)
               if ( debug ) &
                  write(irefwr,1) 'jstart, leninpsep', jstart, leninpsep
               call prevalexpr ( inpsep(jstart+1), reals(2), text )

            end if  ! ( numreal/=1 )
            if ( debug ) write(irefwr,2) 'reals', reals

            bool = prevalbool ( reals, inpsep(ipos) )
            numreal = 0   ! reset number of reals
            ipos = ipos+2
            bwork(icount) = bool
            if ( debug ) then
               write(irefwr,*) 'bool', bool
               write(irefwr,1) 'ipos, icount', ipos, icount
               call prinbool ( bwork, icount, 'bwork' )
               call prinin ( ibwork, icount, 'ibwork' )
            end if  ! ( debug )

         else if ( inpsep(ipos)<10 ) then

!        --- inpsep(ipos)>6 and inpsep(ipos)<10 and, or, not
!            ibwork(icount) := boolean_operator sequence number
!            bwork(icount) := bool
!            icount := icount+1

            icount = icount+1
            ibwork(icount) = inpsep(ipos)
            if ( debug ) then
               write(irefwr,*) 'boolean operator and/or/not'
               write(irefwr,1) 'ipos, icount', ipos, icount
               call prinbool ( bwork, icount, 'bwork' )
               call prinin ( ibwork, icount, 'ibwork' )
            end if  ! ( debug )
            ipos = ipos+1

         else

!        --- internal error

            call errsub ( 499, 0, 0, 0 )
            go to 1000

         end if  ! ( inpsep(ipos)==11 )
         if ( ierror/=0 ) go to 1000

      end do  ! while ( .not. finished )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'prevalboolexpr ', prevalboolexpr
         write(irefwr,*) 'End prevalboolexpr'

      end if  ! ( debug )
      call erclos ( 'prevalboolexpr' )

      end
