      subroutine prevalexpr ( iseqin, realrd, text )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 08-12-2013 Extension with integer scalars
!        version  2.0  date 14-11-2013 Extra parameter text
!        version  1.5  date 14-04-2013 New call to erscalval
!
!   copyright (c) 2005-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Evaluate expression and fill actual values of scalars
!     Result is stored in realrd
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulexpr
      use sepmodulecons
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iseqin
      double precision realrd
      character (len=*) text

!     iseqin         i    Sequence number of expression to be evaluated
!     realrd         o    Result of expression
!     text           i    String variable containing some text for error message
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer maxlevels, lenwork
      parameter ( maxlevels=10, lenwork=200 )
      integer iwork(lenwork), levels(maxlevels), nlevel, icount, ncount, &
              indintexpr, ival, ivalue, endintexp, iseq
      double precision work(lenwork), rvalue
      logical finished, integerfound

!     endintexp      Last position in integer part of expression
!                    that is to be used
!     finished       Indicates if the loop has finished (true) or not
!                    (false)
!     icount         Counter
!     indintexpr      index of address in inpsep
!     integerfound   If true an integer has been found
!     iseq           Sequence number of expression to be evaluated
!                    Corrected with 200000000
!     ival           Integer value in inpsep
!     ivalue         Integer value
!     iwork          integer work array of length lenwork
!                    iwork is used to store the integer information of the
!                    expression
!                    For a description see subroutine READEXPR
!     lenwork        Declared length of array work
!     levels         Array defining the starting addresses of each level
!                    Each time an opening bracket ( is found a new level is
!                    created, each time a closing bracket is found the
!                    level is decreased by one
!     maxlevels      Declared length of array levels
!     ncount         Counter
!     nlevel         Number of levels stored in levels
!     rvalue         Real value
!     work           Work array of length lenwork
!                    work is used to store the real information of the
!                    expression
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERSCALVAL      Fill value of scalar defined by ipos into either rvalue or
!                    ivalue
!     PRININ         print 1d integer array
!     PRININLARGE    Print 1d integer vector (large numbers)
!     PRINRL         Print 1d real vector
!     READEXPR       Read subexpression not containing any brackets and
!                    translate into a real or integer number
!     READEXPRCLOS   Interpret last part of an expression, &
!                    possibly closed by brackets
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2800   Incorrect vector inpsep
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Copied and adapted from readexpression
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prevalexpr' )
      debug = .false.

      iseq = mod(iseqin,200000000)
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prevalexpr'
         write(irefwr,1) 'iseq', iseq
         call prinin ( istexpress(0), iseq+1, 'istexpress' )
         call prininlarge ( intexpress, istexpress(iseq), &
                            'intexpress' )
         call prinrl ( realexpress, lastrealexpr, 'realexpress' )
         work = 0d0
  1      format ( a, 1x, (10i10) )
  2      format ( a, 1x, (5d12.4) )

      end if
      if ( ierror/=0 ) go to 1000

      nlevel = 1   ! level for evaluation between brackets
      icount = 1   ! counter for iwork, work
      levels(1) = 1  ! Start of array levels
      indintexpr = istexpress(iseq-1)+1  ! next position in inpsep
      endintexp = istexpress(iseq)
      finished = .false.  ! define when loop is finished

      if ( debug ) &
         write(irefwr,1) 'indintexpr, endintexp', indintexpr, endintexp

      do while ( .not. finished )

         if ( indintexpr>endintexp ) then

!        --- end of inpsep has been reached, so we are at the end

            ncount = icount-1
            if ( debug ) then
               write(irefwr,*) 'end of inpsep has been reached'
               write(irefwr,1) 'icount, ncount', icount, ncount
               call prinin ( iwork, ncount, 'iwork' )
               call prinrl ( work, ncount, 'work')
            end if

            call readexpr ( work, iwork, ncount, realrd, text )
            finished = .true.

         else if ( intexpress(indintexpr)==-4 ) then

!        --- Closing bracket found

            if ( debug ) then

               write(irefwr,*) 'Closing bracket found ', indintexpr
               write(irefwr,1) 'nlevel, icount', nlevel, icount

            end if  ! ( debug )

            call readexprclos ( levels, nlevel, icount, iwork, work, &
                                text )
            indintexpr = indintexpr+1

            if ( debug ) then

!           --- Debug information

               write(irefwr,1) 'indintexpr, nlevel, icount', &
                                indintexpr, nlevel, icount
               call prinin ( levels, nlevel, 'levels' )
               call prinin ( iwork, icount-1, 'iwork' )
               call prinrl ( work, icount-1, 'work' )

            end if  ! ( debug )

         else if ( intexpress(indintexpr)==-3 ) then

!        --- Opening bracket found, raise levels and icount

            if ( debug ) &
               write(irefwr,*) 'Opening bracket found ', indintexpr
            nlevel = nlevel+1
            levels(nlevel) = icount
            indintexpr = indintexpr+1

!           --- If the next item is a function raise icount

            if ( intexpress(indintexpr)>=10 ) icount = icount+1

            if ( debug ) then

               write(irefwr,1) 'indintexpr, icount, nlevel', &
                                indintexpr, icount, nlevel
               call prinin ( levels, nlevel, 'levels' )

            end if  ! ( debug )

         else if ( intexpress(indintexpr)<1000 ) then

!        --- Operator found, fill iwork

            iwork(icount) = intexpress(indintexpr)
            indintexpr = indintexpr+1
            if ( debug ) then
               write(irefwr,*) 'operator found'
               write(irefwr,1) 'indintexpr, icount', indintexpr, icount
            end if  ! ( debug ) then

!           --- Check if next item is number or scalar

            ival = intexpress(indintexpr)
            if ( debug ) write(irefwr,1) 'ival', ival
            if ( ival>=100000000 ) then

!           --- Scalar found

               call erscalval ( ival, rvalue, ivalue, integerfound )
               if ( integerfound ) then
                  work(icount) = ivalue
               else
                  work(icount) = rvalue
               end if  ! ( integerfound )
               indintexpr = indintexpr+1

            else if ( ival>1000 ) then

!           --- Number found

               work(icount) = realexpress(ival-1000)
               indintexpr = indintexpr+1

               if ( debug ) then
                  write(irefwr,*) 'number found'
                  write(irefwr,1) 'indintexpr, icount', &
                                   indintexpr, icount
                  write(irefwr,2) 'work', work(icount)
               end if  ! ( debug )

            end if  ! ( ival>100000000 )
            icount = icount+1
            if ( debug ) then

!           --- Debug information

               write(irefwr,*) 'Operator found ', indintexpr
               write(irefwr,1) 'ival', ival
               call prinin ( levels, nlevel, 'levels' )
               call prinin ( iwork, icount-1, 'iwork' )
               call prinrl ( work, icount-1, 'work' )

            end if  ! ( debug )

         else

!        --- Error, this situation should not occur

            call errint ( indintexpr, 1 )
            call errint ( intexpress(indintexpr), 2 )
            call errsub ( 2800, 2, 0, 0 )
            call errcrd
            call prinin ( intexpress(istexpress(iseq-1)+1), &
                          endintexp, 'intexpress' )
            go to 1000

         end if  ! ( indintexpr>=endintexp )

      end do !  while ( .not. finished )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,2) 'realrd', realrd
         write(irefwr,*) 'End prevalexpr'

      end if
      call erclos ( 'prevalexpr' )

      end
