      subroutine prinversemap1d ( npoint1, npoint2, coor1, coor2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 19-04-2013 Correction for one point
!        version  1.0  date 22-02-2012
!
!   copyright (c) 2012-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpolate node in new mesh from nodes in old mesh
!     1d case (linear elements)
! **********************************************************************
!
!                       KEYWORDS
!
!     mapping_array
!     map
!     1d
!     linear
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemap
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint1, npoint2
      double precision coor1(npoint1), coor2(npoint2)

!     coor1          i    Array containing the coordinates of the first mesh
!     coor2          i    Array containing the coordinates of the second mesh
!     npoint1        i    number of points in first mesh
!     npoint2        i    number of points in second mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, inode, nelms, jnode, icount, inpelm, &
              ielem, ikelmc, k, ielems(10)
      double precision dx, x, x1, x2, eps, alpha1
      logical coupled, found

!     alpha1         multiplication factor
!     coupled        If true old node is coupled to itself
!     dx             length of element
!     eps            Local accuracy
!     found          If true the node is part of the element
!     i              Counting variable
!     icount         Counter
!     ielem          Element number
!     ielems         Contains all elements a node is lying in
!     ikelmc         position in kmesh part c
!     inode          Node sequence number
!     inpelm         number of points per element
!     j              Counting variable
!     jnode          Array containing the node numbers of an element
!     k              Counting variable
!     nelms          Number of elements in ielems
!     x              x co-ordinate of point
!     x1             x-coordinate of first point in element
!     x2             x-coordinate of last point in element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer getstartkmeshc
      logical checknodeinelm

!     CHECKNODEINELM Check if node is in element
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     GETELMSNODEH1  Find all elements corresponding to nodal point inode
!     GETSTARTKMESHC Compute starting address in kmeshc and information of
!                    element
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1382   Value of NPELM is not yet implemented for this subroutine
!    3095   Node in old mesh not coupled to same node in new mesh
!    3096   Node in new mesh is interpolated by more than 2 nodes in new mesh
!    3097   Node in old mesh is not interpolated
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prinversemap1d' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prinversemap1d'
         call prinin ( imap(1,1), npoint2, 'imap(:,1)' )
         call prinin ( imap2(1,1), npoint2, 'imap2(:,1)' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( npelm>2 ) then

!     --- Error: elements of shape/=1 not yet implemented

         call errint ( npelm, 1 )
         call errsub ( 1382, 1, 0, 0 )
         go to 1000

      end if  ! ( npelm>2 )

      eps = 1d-10

      do i = 1, npoint2

         if ( imap2(i,1)==0 ) then

!        --- Node i has not yet been interpolated
!            Check if new node is coupled to old node

            coupled = .false.
            do j = 1, imap(i,1)
               inode = imap(i,j+1)
               if ( inode==i ) then

!              --- new node is coupled to old one

                  coupled = .true.
                  exit

               end if  ! ( inode==node )

            end do  ! j = 1, imap(i,1)

            if ( .not. coupled ) then

!           --- New node not coupled to old one: not yet implemented

               call errint ( inode, 1 )
               call errsub ( 3095, 1, 0, 0 )
               go to 1000

            end if  ! ( .not. coupled )

!           --- Find all elements in new mesh containing node inode
!               Only elements with highest dimension are applicable

            nelms = 10
            call getelmsnodeh1 ( inode, ielems, nelms )

            if ( debug ) then
               write(irefwr,1) 'inode', inode
               call prinin ( ielems, nelms, 'ielems' )
            end if  ! ( debug )

            if ( imap(i,1)==2 ) then

!           --- Remove all elements containing the other node
!               This is the case that the new node is interpolated between
!               2 old nodes.

               if ( imap(i,2)==inode ) then
                  jnode = imap(i,3)
               else
                  jnode = imap(i,2)
               end if  ! ( imap(i,2)==inode )

               if ( debug ) write(irefwr,1) 'jnode', jnode

               icount = 0
               do j = 1, nelms

                  ielem = ielems(j)
                  ikelmc = &
                     getstartkmeshc ( inpelm, ielem )
                  found = &
                     checknodeinelm ( kmeshc(ikelmc), inpelm, jnode )
                  if ( debug ) then
                     write(irefwr,1) 'ielem', ielem
                     call prinin ( kmeshc(ikelmc), 2, 'element' )
                     write(irefwr,*) 'found', found
                  end if  ! ( debug )ielem

                  if ( .not. found ) then

!                 --- Node not in element, add element to list

                     icount = icount+1
                     ielems(icount) = ielem

                  end if  ! ( .not. found )

               end do  ! j = 1, nelms

               nelms = max(1,icount)
               if ( debug ) call prinin ( ielems, nelms, 'ielems/2' )

               x = coor2(inode)

               do j = 1, nelms

                  ielem = ielems(j)
                  ikelmc = &
                     getstartkmeshc ( inpelm, ielem )

!                 --- Interpolate old coordinate in new elements

                  x1 = coor1(kmeshc(ikelmc))
                  x2 = coor1(kmeshc(ikelmc+1))

                  if ( debug ) then

!                 --- Debug information

                     write(irefwr,1) 'inode, ielem', inode, ielem
                     write(irefwr,2) 'x, x1, x2', x, x1, x2

                  end if  ! ( debug )

                  if ( x>=x1-eps .and. x<=x2+eps .or. &
                       x<=x1+eps .and. x>=x2-eps ) then

!                 --- x in element: interpolate

                     dx = 1d0/(x2-x1)
                     imap2(inode,1) = 2
                     imap2(inode,2) = kmeshc(ikelmc)
                     imap2(inode,3) = kmeshc(ikelmc+1)
                     alpha1 = (x-x1)*dx
                     rmap2(inode,1) = 1d0-alpha1
                     rmap2(inode,2) = alpha1

                  end if  ! ( x>=x1-eps .and. x<=x2+eps .or. &

                  if ( debug ) then
                     write(irefwr,1) 'ielem', ielem
                     write(irefwr,2) 'x, x1, x2', x, x1, x2
                  end if  ! ( debug )

                  if ( imap2(inode,1)>0 ) then

                     if ( debug ) then
                        write(irefwr,100) inode, &
                           ( imap2(inode,k+1), rmap2(inode,k), &
                             k=1,imap2(inode,1) )
100                     format ( i6, ':', 20 ( i6, d12.3 ) )
                     end if  ! ( debug )

                     exit

                  end if  ! ( imap2(inode,1)>0 )

               end do  ! j = 1, nelms

            else

!           --- Other situations are not available yet

               call errint ( inode, 1 )
               call errint ( imap(i,1), 2 )
               call errsub ( 3096, 2, 0, 0 )
               go to 1000

            end if  ! ( imap(i,1)==2 )

         end if  ! ( imap(i,1)==1 )

      end do  ! i = 1, npoint2

!     --- Check if there are any nodes that have not been interpolated

      do i = 1, npoint2

         if ( imap2(i,1)==0 ) then

!        --- Node i has not yet been interpolated

            call errint ( i, 1 )
            call errsub ( 3097, 1, 0, 0 )
            go to 1000

         end if  ! ( imap2(i,1)==0 )

      end do  ! i = 1, npoint2

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prinversemap1d'

      end if  ! ( debug )
      call erclos ( 'prinversemap1d' )

      end
