      subroutine prmakesignvec ( inpsep, numvct )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 25-11-2012 Long integers
!        version  1.0  date 28-05-2011
!
!   copyright (c) 2011-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create new vector V by either taking the sign of the input vector
!     or multiplication of the sign of input vector 1 by input vector 2
! **********************************************************************
!
!                       KEYWORDS
!
!     sign
!     multiplication
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulemain
      use sepmodulekprob
      use sepmodulekmesh
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), numvct

!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine prmanv for a description
!     numvct        i/o   Largest sequence number of vector that has been filled
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ivec, itypvc, ivecout, ivecsign, ivecmult

!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     ivec           Sequence number of vector of special structure
!     ivecmult       ivec for multiplication vector
!     ivecout        ivec for output vector
!     ivecsign       ivec for sign vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRMAKESIGNVEC1 Create new vector V by taking the sign of the input vector
!     PRMAKESIGNVEC2 Create new vector V by either multiplication of the sign of
!                    input vector 1 by input vector 2
!     PRMAKESIGNVEC3 Create new vector V by division of
!                    input vector 1 by input vector 2 (componentwise)
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
!     SEPCREASOLBF1  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3079   sign function dos not correspond to 1 degree of freedom per point
!    3080   multiplication vector does not have constant number of degrees of
!           freedom per point
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prmakesignvec' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prmakesignvec'
         call prinin ( inpsep, 13, 'inpsep' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- get constants

      ivecout = inpsep(3)     ! sequence number of output vector
      if ( inpsep(4)==107 ) then

!     --- inpsep(4) = 107, multiplication

         ivecsign = inpsep(9)    ! sequence number of input vector for sign
         ivecmult = inpsep(10)   ! sequence number of input vector for mult

      else

!     --- inpsep(4) = 108, division

         ivecsign = inpsep(10)   ! sequence number of input vector for sign
         ivecmult = inpsep(9)    ! sequence number of input vector for div

      end if  ! ( inpsep(4)==107 )

!     --- activate arrays

      call sepactsolbf2 ( isolut2d(ivecsign) )
      if ( ierror/=0 ) go to 1000

      if ( ivecmult==0 ) then

!     --- Only the sign of the vector is stored
!         in and output vector have exactly the same structure

         itypvc = ks(isolut2d(ivecsign))%typevec
         ivec = ks(isolut2d(ivecsign))%ivec
         iprob  = ks(isolut2d(ivecsign))%iprob
         call sepcreasolbf1 ( isolut2d(ivecout), itypvc, ivec, &
                              names(ivecout), &
                              ks(isolut2d(ivecsign))%compl )
         if ( ierror/=0 ) go to 1000

         vectout => ks(isolut2d(ivecout))%sol
         vectin => ks(isolut2d(ivecsign))%sol
         call prmakesignvec1 ( vectin, vectout, size(vectout) )

      else

!     --- Multiply vector with the sign of the other vector
!         multipication and output vector have exactly the same structure

         if ( indprfi>0 .or. nphysi/=1 .or. nunkpi/=1 ) then

!        --- Error: either indprfi>0 or nphysi#1

            call errint ( indprfi, 1 )
            call errint ( nunkpi, 2 )
            call errint ( nphysi, 3 )
            call errsub ( 3079, 3, 0, 0 )
            go to 1000

         end if  ! ( indprfi>0 .or. nphysi/=1 .or. nunkpi/=1 )

         call sepactsolbf2 ( isolut2d(ivecmult) )
         indprfo = indprfi
         nphyso = nphysi
         if ( ierror/=0 ) go to 1000

         itypvc = ks(isolut2d(ivecmult))%typevec
         ivec = ks(isolut2d(ivecmult))%ivec
         iprob = ks(isolut2d(ivecmult))%iprob
         call sepcreasolbf1 ( isolut2d(ivecout), itypvc, ivec, &
                              names(ivecout), &
                              ks(isolut2d(ivecmult))%compl )
         if ( ierror/=0 ) go to 1000

         if ( indprfo>0 .or. nphyso/=nphysi ) then

!        --- Error: either indprfo>0 or nphyso#nphysi

            call errint ( indprfo, 1 )
            call errint ( nphysi, 2 )
            call errint ( nphyso, 3 )
            call errsub ( 3080, 3, 0, 0 )
            go to 1000

         end if  ! ( indprf>0 .or. nphys/=1 .or. nunkp/=1 )

         vectin => ks(isolut2d(ivecsign))%sol
         vectin2 => ks(isolut2d(ivecmult))%sol
         vectout => ks(isolut2d(ivecout))%sol
         length = size(vectin2)
         if ( inpsep(4)==107 ) then

!        --- inpsep(4) = 107, multiplication

            call prmakesignvec2 ( vectin, vectout, length, vectin2, nphysi, &
                                  inpsep )

         else

!        --- inpsep(4) = 108, division

            call prmakesignvec3 ( vectin2, vectout, length, vectin, nphysi, &
                                  inpsep )

         end if  ! ( inpsep(4)==107 )

      end if  ! ( ivecmult==0 )

      numvct = max(numvct,ivecout)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prmakesignvec'

      end if  ! ( debug )
      call erclos ( 'prmakesignvec' )

      end subroutine prmakesignvec
