      function prmeanval ( inpsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 25-11-2012 Long integers
!        version  1.0  date 03-01-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find mean value of vector as defined by inpsep
!
! **********************************************************************
!
!                       KEYWORDS
!
!     level_set
!     value
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulemain
      use sepmodulekprob
      use sepmodulekmesh
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*)
      double precision prmeanval

!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine prmanv iinvec
!     prmeanval      o    mean value computed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ichoice, ilev, iseqvec, nnodes, inum, nparts, idegfd
      integer, allocatable :: ipoint(:)

!     ichoice        See inpsep(7)
!     idegfd         Degree of freedom sequence number
!     ilev           See inpsep(6)
!     inum           Number of numbers in inpsep
!     ipoint         Array of length nparts+1 containing the starting
!                    addresses in array inodes of the various parts
!                    the boundary consists of.
!                    ipoint(1) = 1, &
!                    number of nodes at part i = ipoint(i+1)-ipoint(i)
!     iseqvec        Starting address of sequence numbers of vectors
!     nnodes         Number of nodes
!     nparts         number of parts the boundary consists of
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision prmeanvalue, prmeanvalue0

!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRMEANVALUE    Find mean value of vector
!     PRMEANVALUE0   Find mean value of vector (all nodes are used)
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
!     SEPFILLNODES   Fill array nodes with nodal points as defined in array
!                    inpsep
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!       1   ICHOIS is incorrect
!     183   idegfd incorrect
!    1653   Number of unknowns not constant in node
!    2951   Sequence number of vectors out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prmeanval' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prmeanval'
         call prinin ( inpsep, 12, 'inpsep' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      prmeanval = 0   ! initialization

!     --- Get parameters

      idegfd = inpsep(3)   ! degree of freedom
      ichoice = inpsep(5)  ! type of region
      ilev = inpsep(4)     ! level set number
      iseqvec = inpsep(12)  ! vector sequence number
      inum = 1  ! number of obstacles

      if ( iseqvec<1 .or. iseqvec>maxvc1 ) then

!     --- sequence number out of range

         call errint ( iseqvec, 1 )
         call errint ( maxvc1, 2 )
         call errsub ( 2951, 2, 0, 0 )
         go to 1000

      end if  ! ( iseqvec<1 .or. iseqvec>maxvc1 )

      if ( debug ) write(irefwr,1) 'ichoice, ilev', ichoice, ilev

!     --- activate arrays

      call sepactsolbf2 ( isolut2d(iseqvec) )
      if ( ierror/=0 ) go to 1000

      if ( ichoice>0 .and. indprfi/=0 ) then

!     --- number of unknowns not constant, not yet implemented

         call errsub ( 1653, 0, 0, 0 )

      end if  ! ( nunkpi>1 .or. indprfi/=0 )

      if ( idegfd<0 .or. idegfd>nunkpi ) then

!     --- number of unknowns incorrect

         call errint ( idegfd, 1 )
         call errint ( 1, 2 )
         call errint ( nunkpi, 3 )
         call errsub ( 183, 3, 0, 0 )

      end if  ! ( idegfd<0 .or. idegfd>nunkpi )
      if ( ierror/=0 ) go to 1000

      if ( ichoice>0 ) then

!     --- activate or create ipointobst

         allocate ( nodes(npoint), ipoint(ncurvs+1), stat = error )
         if ( error/=0 ) call eralloc ( error, npoint+ncurvs, 'nodes' )
         if ( ierror/=0 ) go to 1000

         call sepfillnodes ( ichoice, inum, inpsep(4), &
                             nodes, nnodes, ipoint, nparts )

      end if  ! ( ichoice==1 )

!     --- get starting addresses

      usoltemp => ks(isolut2d(iseqvec))%sol
      vectin => usoltemp

      select case (ichoice)

      case (0)

!     --- ichoice = 0, whole vector

         nnodes = ks(isolut2d(iseqvec))%nusol/nunkpi
         prmeanval = prmeanvalue0 ( vectin, nnodes, nunkpi, idegfd )

      case (20)

!     --- ichoice = 20, zero_level_set

         prmeanval = prmeanvalue ( vectin, nodes, nnodes, nunkpi, idegfd )

         deallocate ( nodes, ipoint, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'nodes' )

      case default

!     --- Option not yet implemented

         call errint ( ichoice, 1 )
         call errsub ( 1, 1, 0, 0 )

      end select  ! case (ichoice)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prmeanval'

      end if  ! ( debug )
      call erclos ( 'prmeanval' )

      end function prmeanval
