      function prmeanvalue ( usol, nodes, nnodes, nunkp, idegfd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 28-03-2015 Allow parallel
!        version  1.1  date 21-09-2012 Allow nnodes = 0
!        version  1.0  date 12-12-2011
!
!   copyright (c) 2011-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find mean value of vector
!     Ony a limited number of nodes are used
! **********************************************************************
!
!                       KEYWORDS
!
!     value
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecpack
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nnodes, nodes(nnodes), nunkp, idegfd
      double precision usol(nunkp,*), prmeanvalue

!     idegfd         i    Degree of freedom sequence number
!     nnodes         i    Number of nodes
!     nodes          i    Contains the node numbers that must be used
!     nunkp          i    Maximum number of unknowns in a point
!     prmeanvalue    o    mean value
!     usol           i    Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: vsend(2), value, vrec(2)

!     value          Help variabl to store a real
!     vrec           array to store quantities to be received from other nodes
!     vsend          array to store quantities to be sent to other nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRPARGLSUM     This function performs a global sum of a quantitity
!                    or vector across all processes
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prmeanvalue' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prmeanvalue'
         write(irefwr,1) 'nnodes, idegfd', nnodes, idegfd
         call prinin ( nodes, nnodes, 'nodes' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      value = 0d0

      if ( idegfd==0 ) then

!     --- All degrees of freedom

         if ( nnodes>0 ) value = sum(usol(:,nodes(1:nnodes)))

      else

!     --- Only one degree of freedom

         if ( nnodes>0 ) value = sum(usol(idegfd,nodes(1:nnodes)))

      end if  ! ( idegfd==0 )

      if ( debug ) write(irefwr,2) 'value', value

      if ( parallel ) then

!     --- Parallel computation

         vsend(1) = value
         vsend(2) = nnodes
         call prparglsum ( vsend, vrec, 2, SPDOUBLE )

         if ( idegfd==0 ) then

!        --- All degrees of freedom

            prmeanvalue = vrec(1)/(vrec(2)*nunkp)

         else

!        --- Only one degree of freedom

            prmeanvalue = vrec(1)/vrec(2)

         end if  ! ( idegfd==0 )

      else

!     --- Serial computation

         if ( idegfd==0 ) then

!        --- All degrees of freedom

            prmeanvalue = value/(nnodes*nunkp)

         else

!        --- Only one degree of freedom

            prmeanvalue = value/nnodes

         end if  ! ( idegfd==0 )

      end if  ! ( parallel )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,2) 'prmeanvalue', prmeanvalue
         write(irefwr,*) 'End prmeanvalue'

      end if  ! ( debug )
      call erclos ( 'prmeanvalue' )

      end
