      subroutine prmoveobstac ( iinput, rinput )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.6  date 25-11-2012 Long integers
!        version  1.5  date 22-11-2004 Fortran 90
!        version  1.4  date 06-08-2004 Extension with origin
!
!   copyright (c) 2001-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Move obstacle over a distance u * dt
!    In R^3 it may also be rotated over an angle
! **********************************************************************
!
!                       KEYWORDS
!
!     obstacle
!     time_dependent
!     displacement
!     rotation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemain
      use sepmoduletime
      use sepmodulecons
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*)
      double precision rinput(*)

!     iinput         i    integer input array
!                         At this moment the storage is as follows:
!                          1: iobst   obstacle sequence number
!                          2: istrin  starting address in rinsep
!                          3: if>0 reference to scalar w.r.t. u_veloc
!                          4: if>0 reference to scalar w.r.t. v_veloc
!                          5: if>0 reference to scalar w.r.t. w_veloc
!                          6: if>0 reference to scalar w.r.t. phi_veloc
!                          7: Rotation plane 1: (xy) 2: (yz) 3: (xz)
!                          8: Indicates if positions 4 to 6 refer to a
!                             velocity (0) or displacement (1)
!                          9: if>0 reference to scalar w.r.t. x_orig
!                         10: if>0 reference to scalar w.r.t. y_orig
!                         11: if>0 reference to scalar w.r.t. z-orig
!     rinput         i    Real input array
!                         At this moment the storage is as follows:
!                         1:  u_x  displacement in x-direction
!                         2:  u_y  displacement in y-direction
!                         3:  u_z  displacement in z-direction
!                         4:  u_phi rotation velocity
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iobst, i, istrin, npointlc, lenin, lenrl, &
              rotation_dir, iprob, nprob, iseqprob
      double precision uvel(3), uphi, origin(3)

!     i              Counting variable
!     iobst          Obstacle sequence number
!     iprob          Actual problem number
!     iseqprob       Sequence number of problem input
!     istrin         Starting address in array RINSEP
!     lenin          Number of integers in one row of kmeshx
!     lenrl          Number of reals in one row of rmeshx
!     npointlc       Local number of points
!     nprob          Number of various problems
!     origin         Contains the coordinates of the origin of the axis of the
!                    obstacle
!     rotation_dir   Rotation direction
!                    1: rotation in x-y plane
!                    2: rotation in y-z plane
!                    3: rotation in x-z plane
!     uphi           rotation velocity in radians
!     uvel           Contains the velocity vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INCOMGEN       Compute matrix structure for free boundary problem
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRMOVEOBSTAC1  Move obstacle over a distance
!     PRMOVEOBSTAC2  Move and rotate obstacle over a distance (R^3)
!     PROBDFEXT      Reads and stores problem definition
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2209   Obstacle number out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
!     iobst := iinput(1)
!     ndim := kmesh(6)
!     u := rinput(1..ndim)*tstep
!     numobst := kmesh(42)
!     Activate arrays
!     Compute starting addresses
!     Raise xmin, xmax, ymin, ymax
!     Raise sequence number of obstacle in kmesh ad if it exists
!     Adapt kprob
!     Adapt intmat
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prmoveobstac' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prmoveobstac'
         call prinin ( iinput, 11, 'iinput' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- iobst := iinput(1)
!          ndim := kmesh(6)
!          u := rinput(1..ndim)*tstep
!          numobst := kmesh(42)

      iobst = iinput(1)
      istrin = iinput(2)-1
      if ( debug ) then
         call prinrl ( rinput(istrin+1), 7, 'rinput' )
         write(irefwr,*) 'iobst, istrin ',iobst, istrin+1
      end if  ! ( debug )

!     --- Check if the value is given as a scalar
!         If so substitute that value

      do i = 1, ndim

         if ( iinput(2+i)>0 ) rinput(istrin+i) = scalars(iinput(2+i))
         if ( iinput(8)==0 ) then

!        --- velocity found, multiply by dt

            uvel(i) = rinput(istrin+i)*tstep

         else

!        --- displacement found

            uvel(i) = rinput(istrin+i)

         end if

      end do  ! i = 1, ndim
      istrin = istrin+ndim
      if ( iinput(6)/=0 ) then

!     --- uphi is given

         if ( iinput(6)>0 ) rinput(istrin+1) = scalars(iinput(6))
         uphi = rinput(istrin+1)*tstep
         istrin = istrin+1

      else

!     --- uphi is not given

         uphi = 0d0

      end if
      if ( debug ) write(irefwr,*) 'uphi ', uphi

!     --- origin

      do i = 1, ndim

         if ( iinput(8+i)>0 ) rinput(istrin+i) = scalars(iinput(8+i))
         origin(i) = rinput(istrin+i)

      end do  ! i = 1, ndim
      if ( debug ) call prinrl ( origin, 3, 'origin' )

      rotation_dir = iinput(7)

      if ( iobst<=0 .or. iobst>numobst ) then

!     --- obstacle number out of range

         call errint ( iobst, 1 )
         call errint ( numobst, 2 )
         call errsub ( 2209, 2, 0, 0 )

      end if
      if ( ndim==2 ) then

!     --- ndim = 2, the obstacle consists of curves
!         Raise xmin, xmax, ymin, ymax

         call prmoveobstac1 ( iobst, uvel )

      else

!     --- ndim = 3, the obstacle consists of a surface
!         Activate arrays

         lenrl = 9
         lenin = 12

!        --- Compute starting addresses

         npointlc = kmeshx(12,iobst)

         if ( ierror/=0 ) go to 1000

!        --- Update coordinates

         call prmoveobstac2 ( iobst, uvel, uphi, rmeshx, &
                              coorobst, npointlc, rotation_dir, &
                              kmeshx, origin )

         if ( abs(uphi)>0d0 .and. kmeshx(6,iobst)>0 ) then

!        --- uphi # 0, hence obstacle has been rotated
!            it is necessary to recover the obstacle with a grid
!            This is done simply by removing the old iobgrid array

            iseqkmobst = kmeshx(6,iobst)
            if ( ndim==2 ) then
               deallocate ( km(iseqkmobst)%kmobst(iobst)%iobgrid2d, &
                            km(iseqkmobst)%kmobst(iobst)%iobgrid2d1 )
            else
               deallocate ( km(iseqkmobst)%kmobst(iobst)%iobgrid3d )
            end if  ! ( ndim==2 )
            kmeshx(6,iobst) = 0

         end if

      end if

!     --- Raise sequence number of obstacle in kmesh ad if it exists

      if ( kelmad>0 ) then

!     --- Kmesh part ad exists, raise position 5

         kmeshad(5) = kmeshad(5)+1

      end if

!     --- Adapt kprob

      iseqprob = 1
      call probdfext ( 3, iseqprob )
      if ( ierror/=0 ) go to 1000

!     --- Adapt intmat

      call incomgen ( -1, intmat1d )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prmoveobstac'

      end if
      call erclos ( 'prmoveobstac' )

      end
