      subroutine prnlns ( inpsep, rinsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.1  date 11-12-2017 New storage of intmat
!        version  4.0  date 20-08-2016 New parameter list
!        version  3.2  date 06-02-2012 Allow indexed vectors
!
!   copyright (c) 1993-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Call subroutine NLNPRB to solve a non-linear problem
!    Fill array IINSOL with the values stored in INPSEP
! **********************************************************************
!
!                       KEYWORDS
!
!     non_linear
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesol
      use sepmodulekprob
      use sepmodulemnsub
      use sepmodulemain
      use sepmoduleinput
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*)
      double precision rinsep(*)

!     inpsep         i    See subroutine readscnlnsol
!     rinsep         i    Array containing the real input for this subroutine
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseqlc, ivec, iseqnr, ivectr, iinsol(51), iproblc, &
              iprobr, idummy(1), iseqreac, ipos, iseqcoef, maxcall
      double precision dummy(1), rinsol(10)
      logical inputfound

!                    they are not
!     dummy          Dummy array
!     idummy         Dummy array
!     iinsol         Input array defined by the user
!                    With this array the user may indicate which actions
!                    he wants to be performed by LINPRB
!                    See subroutine TO0091
!     inputfound     If true an input blokc non_linear has been found
!     ipos           Position of input arrays in sepmoduleinput
!     iproblc        Actual problem number
!     iprobr         Problem number read
!     iseqcoef       Sequence number of coefficients
!                    If <=0, no sequence number given
!     iseqlc         Sequence number read
!     iseqnr         Sequence number with respect to the storage in array ISEPIN
!                    for coefficients
!     iseqreac       Sequence number of reaction vector in array ISOL
!                    If <0 also the right-hand side contribution is added
!     ivec           Sequence number of vector of read
!     ivectr         Sequence number of first vector
!     maxcall        Maximum number of inputs read
!     rinsol         Contains accuracy
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inivectr

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIVECTR       Return with sequence number of vector
!     PRININ         print 1d integer array
!     SEPGETINPFINF  Extract infor entries of integer and real input arrays from
!                    ISEPIN
!     TO0091         Actual solution of system of non-linear equations
!     TO0095         Solve system of non-linear equations
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data iseqnr /0/, ivectr /0/, iproblc /0/
! ======================================================================
!
      call eropen ( 'prnlns' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prnlns'
         write(irefwr,1) 'istartmn', istartmn
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      iseqnr = 1
      iseqlc   = inpsep(1)
      if ( iseqlc>0 ) iseqnr = iseqlc
      rinsol = 0d0

      if ( debug ) write(irefwr,1) 'iseqlc, numvecmn', iseqlc, numvecmn

      if ( iseqlc/=-10 ) then

!     --- Standard case, use sequence number

         if ( debug ) call prinin ( inpsep, 5, 'inpsep' )

!        --- Check if sequence number corresponds to input block

         call sepgetinpfinf ( 12, iseqnr, ipos, maxcall )
         inputfound = maxcall>0
         if ( inputfound ) inputfound = in(ipos)%used

         if ( inputfound ) then

!        --- Standard case

            iproblc  = min(iproblc+1,nprob)
            ivectr = iproblc
            ivec   = inivectr(inpsep(3))
            if ( ivec/=-1 ) ivectr = ivec
            if ( isolut2d(ivectr)>=1 .and. isolut2d(ivectr)<=lastsol ) then
               if ( ks(isolut2d(ivectr))%typevec==110 ) &
                 iproblc = ks(isolut2d(ivectr))%iprob
            end if  ! ( isolut2d(ivectr)>=1 .and. ... )
            iprobr = inpsep(2)
            if ( iprobr/=-1 ) iproblc = iprobr
            iinsol(1) = 1
            iinsol(2) = 0
            iinsol(3) = iseqnr
            iinsol(4) = iproblc
            iinsol(5) = ivectr
            iinsol(6) = numvecmn
            iinsol(7) = inpsep(4)
            call to0095 ( iinsol )
            numvecmn = max(numvecmn,ivectr,iinsol(7))

         else

!        --- No input found, fill defaults

            iinsol = 0           ! clear vector
            iinsol(1) = 1        ! ncopeq
            iinsol(2) = 20       ! maxiter
            iinsol(5) = 2        ! miniter
            iinsol(6) = 1        ! icriter
            iinsol(7) = 1        ! iseqsol
            iinsol(8) = 1        ! ivectr
            iinsol(10) = numvecmn  ! maxvec
            rinsol(1) = 1d-2     ! accuracy
            iseqreac = 0         ! reaction force
            iseqcoef = 0         ! seq_coef

         end if  ! ( inputfound )

         istartmn = istartmn+5

      else

!     --- New case, all input in inpsep and rinsep

         if ( debug ) call prinin ( inpsep, 12, 'inpsep' )

         iinsol(9:51) = 0           ! clear vector
         iinsol(1) = 1              ! ncopeq
         iinsol(2:6) = inpsep(5:9)  ! maxiter ... icriter
         iinsol(7) = 1              ! iseqsol
         iinsol(8) = max(1,inpsep(3))  ! ivectr
         iinsol(9) = 0              ! callusr
         iinsol(10) = numvecmn        ! maxvec
         iinsol(11) = 0             ! iseqout
         iinsol(13) = inpsep(10)    ! method
         iinsol(26) = 27            ! starting address
         iinsol(27) = inpsep(9)     ! icriter
         rinsol(1) = rinsep(inpsep(11))  ! accuracy
         iseqreac = inpsep(4)       ! reaction force
         iseqcoef = inpsep(12)      ! seq_coef
         istartmn = istartmn+13
         if ( debug ) call prinin ( iinsol, 26, 'iinsol' )

         inputfound = .false.

      end if  ! ( iseqlc/=-10 )

      if ( .not. inputfound ) then

!     --- input has been stored in iinsol/rinsol

         if ( iseqcoef==0 ) then

!        --- Check if an input block coefficients can be found

            call sepgetinpfinf ( 6, 1, ipos, maxcall )
            inputfound = maxcall>0
            if ( inputfound ) inputfound = in(ipos)%used
            if ( inputfound ) iseqcoef = 1

         end if  ! ( iseqcoef==0 ) then

         iinsol(31) = iseqcoef   ! coefficients

         call to0091 ( iinsol, rinsol, iseqreac )

      end if  ! ( .not. inputfound )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prnlns'

      end if  ! ( debug )
      call erclos ( 'prnlns' )

      end
