      subroutine readc0 ( inpcre, rincre, numint, nreal, icompl, &
                          inforinpcre, iprob, ivec, isubr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  7.1  date 26-06-2014 extra debug
!        version  7.0  date 11-03-2014 extra parameter isubr
!        version  6.1  date 12-10-2012 Allow level_set
!
!   copyright (c) 1989-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Reads the part of the standard input file for subroutine CREATE
!    that corresponds to a usual data record
!    and stores the information in the arrays INPCRE and RINCRE
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     create
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleread1
      use sepmodulework
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numint, nreal, inpcre(*), icompl, ivec, &
              inforinpcre(*), iprob
      integer, intent(in) :: isubr
      double precision rincre(*)

!     icompl         i    Indicates if vector is real (0) or complex (1)
!     inforinpcre    o    Array of length of rincre that contains a 0 if the
!                         number stored in rincre is a real and a reference
!                         to scalars if it is a scalar
!     inpcre         o    The integer input read is stored in array INPCRE from
!                         position 1. Hence in the call INPCRE(ISTART), with
!                         ISTART the starting position must be used.
!                         INPCRE will be filled in correspondence with array
!                         INPCRE for subroutine CREATV
!     iprob          o    Actual problem number
!     isubr          i    Indicates calling subroutine
!                         1: readcr
!                         2: readscboun1
!     ivec           o    Sequence number of vector of special structure
!     nreal         i/o   Number of reals stored in RINCRE
!     numint         o    Number of integers that are stored in INPCRE
!     rincre        i/o   The value alpha of the real parameter in VALUE = alpha
!                         is stored in RINCRE
!                         The same remarks as for INPCRE hold
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: ncodsb=6, nncode1=41
      integer nread, ndegfd, nscods(ncodsb), &
              ncode1(nncode1), ncodmx1(nncode1)
      character (len=20) code1(nncode1)
      character (len=1) codesb(ncodsb)

!     code1          Table with keywords
!     codesb         Table with subkeywords
!     iwork          is filled as follows
!                    1:  Defines how the vector must be filled
!                        0 value = v found
!                        1<k<1000:  func = k
!                        1000<k<2000:  special_function = k-1000
!                        2000<k<3000:  old_function = k-2000
!                        3000<k<4000:  old_vector = k-3000
!                           Position 111 contains m
!                                    111+i i=1...m the vector numbers
!                        4000<k<5000:  vector = k-4000
!                        9001: quadratic bc
!                        >200000000: vector expression
!                    2:  Defines the region
!                        1: points
!                        2: curves
!                        3: surfaces
!                        4: nodes
!                        5: elements
!                        6: groups
!                        7: volumes
!                        8: user_points
!                        9: obstacle
!                       10: contact
!                    3:  Nummber of points etc
!                        Sequence numbers stored in work(100..99+m)
!                        or step
!                    4:  First point number
!                    5:  Last point number
!                    100-99+ndegfd:  Degrees of freedom
!     ncode1         Table containing the number of significant characters in
!                    the keyword table code1
!     ncodmx1        Table containing the maximal number of  characters in
!                    the keyword table code1
!     ncodsb         Number of entries in codesb
!     ndegfd         Number of degrees of freedom per point
!     nncode1        Number of entries in code1
!     nread          Number of items read
!     nscods         Number of significant letters in subkeywords
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     READC0EXPR     Check if the next information found corresponds to an
!                    expression
!     READC0FILL     Fill information read by readc0 and interpreted by
!                    readc0interpr in inpcre
!     READC0INTERPR  interpret all information read by readc0
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYCODESUB Copy codemn2 and related arrays into codesub
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!     READC0 requires input from the standard input file
!     This input must be stored into array CARDS by the calling subroutine
!     The following input is recognized:
!     [functional description] [degrees of freedom] [location part]
!     in arbitrary order
!     functional description may be of the shape:
!     [VALUE = [alpha] [(alpha, beta)] ] [FUNC = k] [SPECIAL_FUNC = l]
!     [OLD_FUNC = k ] [OLD_VECTOR = m] [SEQ_VECTORS = V1, V2, ...]  [VECTOR = j]
!     [QUADRATIC, MAX = a, MEAN_VALUE = v, BOUNDARY_LAYER_WIDTH = b, &
!                 DIFFERENCE = delta_T ]
!     [HALF_QUADRATIC, MAX = a]
!     only one of these possibilities is permitted
!     Default: VALUE = 0
!     Degrees of freedom has the following shape
!     DEGFD k [,DEGFD l [, DEGFD m ....] ]
!     If omitted DEGFD 0 is assumed
!     The location part has one of the following structures:
!     POINTS ( Pk, Pl, + . + , Pm)
!     USER POINTS ( Pk to Pl [,step = s] )
!     CURVES [l] (Cj [to Cm] )
!     OUTER_CURVES
!     SURFACES [i1, i2, i3, ... ] (Sj [to Sm] )
!     OUTER_SURFACES
!     VOLUMES [i1, i2, i3, ... ] (Vj [to Vm] )
!     NODES (Nj [to Nm [, step = s ] ] )
!     ELEMENTS i [to j] [(RN1, RN2, RN3, ... )]
!     GROUP ielgrp [(RN1, RN2, RN3, ... )]
!     GLOBAL_GROUP iglgrp
!     CONTACT i
!     NO_CONTACT i
!     OBSTACLE i
!     IN_ALL_OBSTACLE i
!     IN_INNER_OBSTACLE i
!     IN_BOUN_OBSTACLE i
!     ON_BOUN_OBSTACLE i
!     ZERO_LEVELSET i
!     FILE_NODAL_VALUES = 'file_name'
!     FILE_ELEMENT_VALUES = 'file_name'
!     FILE_CAPACITY_VALUES = 'file_name'
!     IVEC
!     OUTPUT_VECTOR = I
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code1 / 'VALUE', 'FUNCTION', 'SPECIAL_FUNCTION', &
                  'OLD_FUNCTION', 'OLD_VECTOR', 'SEQ_VECTORS', 'DEGFD', &
                  'POINTS', 'USER', 'STEP', 'CURVES', 'TYPE', &
                  'SURFACES', 'VOLUMES', 'NODES', 'ELEMENTS', &
                  'GROUP', 'OBSTACLE', 'CONTACT', 'NO_CONTACT', &
                  'QUADRATIC', 'MAX', 'FILE_NODAL_VALUES', &
                  'FILE_ELEMENT_VALUES', 'HALF_QUADRATIC', &
                  'FILE_CAPACITY_VALUES', 'GLOBAL_GROUP', &
                  'IN_ALL_OBSTACLE', 'IN_INNER_OBSTACLE', &
                  'IN_BOUN_OBSTACLE', 'ON_BOUN_OBSTACLE', &
                  'OUTER_CURVES', 'OUTER_SURFACES', 'MEAN_VALUE', &
                  'BOUNDARY_LAYER_WIDTH', 'DIFFERENCE', &
                  'ZERO_LEVELSET', 'VECTOR', 'PROBLEM', 'IVEC', &
                  'OUTPUT_VECTOR' /
      data ncode1 /  3, 3, 3, 5, 7,    3, 5, 4, 4, 4, &
                    4, 4, 4, 4, 5,    4, 4, 4, 4, 7, &
                    5, 3, 9, 9, 8,    9, 9,10,12,12, &
                   12, 9, 9, 4,12,    4, 8, 3, 4, 4, &
                    4/
      data ncodmx1/  5, 8,16,12,10,   11, 5, 6, 4, 4, &
                    6, 4, 8, 7, 5,    8, 5, 8, 7,10, &
                    9, 3,17,19,14,   20,12,15,17,17, &
                   17,12,14,10,20,   10,14, 6, 7, 4, &
                   13/
      data codesb/ 'V', 'P', 'C', 'S', 'N', 'E' /
      data nscods /6*1/

!    code1      Contains keywords that are recognized
!              Possible values:
!              1:  VALUE
!              2:  FUNCTION
!              3:  SPECIAL_FUNCTION
!              4:  OLD_FUNCTION
!              5:  OLD_VECTOR
!              6:  SEQ_VECTORS
!              7:  DEGFD
!              8:  POINTS
!              9:  USER_POINT
!             10:  STEP
!             11:  CURVES
!             12:  TYPE
!             13:  SURFACES
!             14:  VOLUMES
!             15:  NODES
!             16:  ELEMENT
!             17:  GROUP
!             18:  OBSTACLE
!             19:  CONTACT
!             20:  NO_CONTACT
!             21:  QUADRATIC
!             22:  MAX
!             23:  FILE_NODAL_VALUES
!             24:  FILE_ELEMENT_VALUES
!             25:  HALF_QUADRATIC
!             26:  FILE_CAPACITY_VALUES
!             27:  GLOBAL_GROUP
!             28:  IN_ALL_OBSTACLE
!             29:  IN_INNER_OBSTACLE
!             30:  IN_BOUN_OBSTACLE
!             31:  ON_BOUN_OBSTACLE
!             32:  OUTER_CURVES
!             33:  OUTER_SURFACES
!             34:  MEAN_VALUE
!             35:  BOUNDARY_LAYER_WIDTH
!             36:  DIFFERENCE
!             37:  ZERO_LEVELSET
!             38:  VECTOR
!             39:  PROBLEM
!             40:  IVEC
!             41:  OUTPUT_VECTOR
! ======================================================================
!
      call eropen ( 'readc0' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readc0'
         write(irefwr,1) 'lastitem, isubr, icode', lastitem, isubr, icode
         call prinin ( typeitems, nitems, 'typeitems' )
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      allocate ( iwork(200), work(200) )

      call sepcopycode ( code1, nncode1, ncode1, ncodmx1 )
      call sepcopycodesub ( codesb, ncodsb, nscods )

!     --- Initialize some parameters

      numint = 0
      nread  = 0
      ndegfd = 0
      iwork(1:10) = 0

      if ( isubr==2 .and. typeitems(lastitem+1)==13 ) then

!     --- equals sign found, check if it is an expression or a real

         call readc0expr

         if ( debug ) write(irefwr,1) 'iwork(1)', iwork(1)
         if ( iwork(1)==1 ) then

!        --- real found, interpret as value = ...

            nread = 1
            rincre(nreal+1) = work(1)
            inforinpcre(nreal+1) = 0

         end if  ! ( iwork(1)==1 )

      end if  ! ( typeitems(lastitem+1)==13 )

!     --- Set table values

      table(1:nncode) = -1d20
      itype(1:nncode) = 0
      tabmin(1:nncode) = 0d0
      tabmax(1:nncode) = 0d0

      itype(1) = 19         ! VALUE
      tabmin(1) = icompl+1
      tabmax(1) = 1d10
      tabmin(2:5) = 1d0     ! FUNCTION,SPECIAL_FUNCTION,OLD_FUNCTION,OLD_VECTOR
      tabmax(2:5) = 999d0
      itype(6) = 40         ! SEQ_VECTORS
      table(6) = -1
      itype(7) = 14         ! DEGFD
      tabmin(7) = 100d0
      tabmax(7) = 11d0
      itype(8) = 18          ! POINTS
      table(8) = -2
      tabmin(8) = 97d0
      tabmax(8) = nuspnt
      itype(9) = 2           ! USER
      tabmax(10) = nuspnt ! STEP
      itype(11) = 31         ! CURVES
      table(11) = -3
      tabmin(11) = 20d0
      tabmax(11:12) = ncurvs ! CURVES, TYPE
      tabmin(12:14) = npelm ! TYPE, SURFACES, VOLUMES
      itype(13:14) = 32        ! SURFACES, VOLUMES
      table(13) = -4     ! SURFACES
      tabmax(13) = nsurfs
      table(14) = -1     ! VOLUMES
      tabmax(14) = nvolms
      itype(15) = 18           ! NODES
      table(15) = -5
      tabmin(15) = 2d0
      tabmax(15) = npoint
      itype(16:17) = 33
      tabmin(16) = 2d0         ! ELEMENT
      tabmax(16:17) = npelm
      tabmin(17) = nelgrp    ! GROUP
      tabmin(18:20) = 1d0
      tabmax(18) = 100d0       ! OBSTACLE
      tabmax(19:20) = 10d0     ! CONTACT, NO_CONTACT
      itype(21) = 2            ! QUADRATIC
      table(22) = -1d21        ! MAX
      itype(22) = 1
      tabmin(22) = -1d9
      tabmax(22) = 1d9
      itype(23:24) = 6         ! FILE_NODAL_VALUES, FILE_ELEMENT_VALUES
      itype(25) = 2            ! HALF_QUADRATIC
      itype(26) = 6            ! FILE_CAPACITY_VALUES
      tabmax(27) = 1d3         ! GLOBAL_GROUP
      tabmin(28:31) = 1d0      ! IN_ALL_OBSTACLE, ... , ON_BOUN_OBSTACLE
      tabmax(28:31) = numobst
      itype(32:33) = 2         ! OUTER_CURVES, OUTER_SURFACES
      tabmin(34) = -1d10
      itype(34:36) = 1         ! MEAN_VALUE, BOUNDARY_LAYER_WIDTH, DIFFERENCE
      tabmax(34:36) = 1d10
      tabmin(36) = -1d10
      tabmin(37:39) = 1d0
      tabmax(37) = 1d0         ! ZERO_LEVELSET
      tabmax(38) = 100d0       ! VECTOR
      table(39) = -1d0         ! PROBLEM
      tabmax(39) = iprob
      tabmin(40:41) = 1d0         ! ivec
      tabmax(40:41) = 1d2
      table(40:41) = 0d0

      iretur = 0

      call sepreadsetkeyw
      icode = iretur
      if ( debug ) then
         call prinrl ( table, nncode, 'table' )
         call prinrl ( tabmax, nncode, 'tabmax' )
      end if  ! ( debug )
      if ( icode>0 ) go to 1000

!     --- interpret the contents read and store in iwork, work

      call readc0interpr ( nread, nreal, inforinpcre, icompl, &
                           rincre, ndegfd )

!     --- Fill information in INPCRE
!         First per point

      call readc0fill ( numint, ndegfd, nread, icompl, &
                        inforinpcre, rincre, inpcre, nreal )

!     --- fill problem number

      iprob = nint(table(39))
      ivec = nint(table(40))

      if ( ivec==0 .and. table(41)>0d0 ) ivec = nint(table(41))
      deallocate ( iwork, work )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'numint, nreal', numint, nreal
         write(irefwr,1) 'iprob, icode, nmain', iprob, icode, nmain
         call prinin ( inpcre, numint, 'inpcre' )
         call prinrl ( rincre, nreal, 'rincre' )
         write(irefwr,*) 'End readc0'

      end if
      call erclos ( 'readc0' )

      ispecl = 0

      end subroutine readc0
