      subroutine readinfr ( iinput, rinput, isubr, iseqnr, maxlen, nrinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.9  date 26-08-2008 Fortran 90
!        version  1.8  date 09-02-2001 Extension of iinput
!        version  1.7  date 06-07-2000 Extra warning
!
!   copyright (c) 1996-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!  Read the input for the instationary free boundary subroutine statfree
!  The input is stored in the arrays iinput and rinput
!  The input must be enclosed between INSTATIONARY_FREE_BOUNDARY and END.
! **********************************************************************
!
!                       KEYWORDS
!
!     adapt_mesh
!     change_mesh
!     free_boundary
!     read
!     time_dependent
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer maxlen, nrinp, iinput(*), iseqnr, isubr
      double precision rinput(*)

!    iinput  o   The integer input read is stored in array IINPUT
!                This array must be filled as follows:
!                1:  LASTIINP  Position of last entry that has been filled
!                2:  IPRINT    Defines the amount of output produced
!                              Possible values:
!                              0:  No special information is printed
!                3:  ISEQMESH  Sequence number of adapt mesh to be used
!                4:  INDEX_VECTR   defines the starting address in
!                              IINPUT of the vectors that are used in the update
!                              of the free boundary
!                5:  INTERPOLATE_SOL: defines the number of solution
!                              vectors that must be interpolated
!                6:  INDEX_SOL: defines the starting address in
!                              IINPUT of the vectors to be interpolated
!                              In positions IINPUT(INDEX_SOL) to
!                              IINPUT(INDEX_SOL+INTERPOLATE_SOL-1) the
!                              sequence numbers of the solution vectors
!                              to be interpolated are stored.
!                              If INDEX_SOL = 0, the vectors 1 to
!                              INTERPOLATE_SOL are interpolated
!                              If INDEX_SOL<0, the vectors
!                              -INDEX_SOL to -INDEX_SOL+INTERPOLATE_SOL-1
!                              are interpolated
!                7:  IVEC_MESHVEL Consists of two parts iseq and ivec according
!                              to ISEQMESH = ISEQ + 1000 x IVEC, with
!                       ISEQ   Sequence number of vector in which
!                              the mesh velocity must be stored
!                              If 0, no mesh velocity is stored
!                       IVEC   Defines the type of vector to be used for
!                              the mesh velocity vector
!                              Possible values:
!                              0:  The type of vector is found by ADAPMESH
!                              1:  The vector is of the type solution vector
!                              2:  The vector is a vector of special structure
!                                  of type IVEC-1
!                8:  ICHECK_BOUN indication of how the boundary must be checked
!                              Possible values:
!                          0   No checking of the boundary, except in the
!                              case that general is applied
!                              If general is applied all boundaries of the
!                              surfaces generated by general are checked
!                              with respect to the general requirements
!                          1   The first and last element of each curve
!                              in array icurves is compared with the
!                              original length stored in array kmesh p
!                              If 1/a<dx/dx_orig<a the curve is
!                              accepted, otherwise icheck is set equal to 1
!                              dx is the actual length of the element
!                              dx_orig is the original length of the element
!                              a must be stored in rinput(1)
!                              As soon as one of the curves returns with
!                              icheck = 1, checking is stopped
!                9:  ICHECK_MESH indication of how the internal elements must be
!                              checked. This is only applied if the boundary
!                              check returns with icheck = 0
!                              Possible values:
!                          0   No checking of the internal elements
!                          1   All angles of the elements are checked
!                              If the smallest angle of an element<rinput(2)
!                              or the largest angle of an element>rinput(3)
!                              icheck is set equal to 2 and checking is stopped
!               10:  IWRMESH:  Indicates if the mesh must be written to
!                              the file meshoutput
!                              Possible values:
!                              0: do not write the mesh
!                              1: over write meshoutput as soon as the last time
!                                 step has been reached
!               11:  NUMVEC:   Number of vectors used to compute the update
!                              of the free boundary
!    iseqnr i/o  Sequence number with respect to the storage in array ISEPIN
!    isubr   i   Indicates by which subroutine READSTFR is called
!                Possible values:
!                1:  ...      read record with STATIONARY_FREE_BOUNDARY
!                2:  ERSTATFR,  do not read record with STATIONARY_FREE_BOUNDARY
!    maxlen i/o  Length of integer input array
!                At input available length, at output computed length
!    nrinp  i/o  Length of real input array
!                At input available length, at output computed length
!    rinput  o   Contains the real input for the instationary free boundary
!                subroutine
!                At this moment the input is defined as follows
!                1:  alpha_boun
!                2:  angle_min
!                3:  angle_max
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode1=13
      integer ncode1(nncode1), itype1(nncode1), ncodmx1(nncode1)
      integer k, lastinp, m, ifirst
      double precision :: tabmin1(nncode1), tabmax1(nncode1)
      character (len=20) code1(nncode1)
      logical sequent

!     code1      Table with keywords corresponding to options
!     ifirst     First array to be interpolated
!     itype1     itype array for code1 (See intx02)
!     k          Counting variable
!     lastinp    Last position filled in array iinput
!     m          Number of arrays to be interpolated
!     ncode1     Table containing the number of significant characters in
!                the keyword table code1
!     ncodmx1    Table containing the number of maximum characters in
!                the keyword table code1
!     nncode1    Number of entries in table code1
!     sequent    If true the arrays to be interpolated are subsequent arrays
!     tabmax1    Table with maximum values of numbers corresponding to code1
!     tabmin1    Table with minimum values of numbers corresponding to code1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYTYPE    Copy itype2 and related arrays into itype
!     SEPREADSEQNR   Read sequence number iseqnr from record
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
!     SEPSETMAINEND  Fill keyword END as first main keyword and set nmain to 1
! **********************************************************************
!
!                       I/O
!
!     READINFR requires input from the standard input file
!     The following input is recognized:
!     INSTATIONARY_FREE_BOUNDARY [, SEQUENCE_NUMBER = s] (mandatory):
!         start of input
!      optional records:
!          print_level = p        (Default 0)
!          adapt_mesh = i         (Default value 1)
!          seq_vectors = (i1,i2,i3...)     (Default 1)
!          mesh_velocity = i      (Default 0)
!          check_boundary = mb    (Default 0)
!          check_mesh = mm        (Default 0)
!          alpha_boun = a         (Default 2)
!          angle_min = a1         (Default 10)
!          angle_max = a2         (Default 120)
!          interpolate_solution = ( v1, v2, ... , vn )   not
!          write_mesh             (Default not)
!          no_write_mesh
!          struct_mesh_velocity = i (Default defined by subroutine)
!    END   (mandatory):  end of input
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     329   Array to store the input too small
!     550   array length too short
!     650   First record read by subroutine is not equal to INSTATIONARY_FREE
!    2190   check_mesh not combined with interpolate_solution
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Initialize arrays and parameters with default values
!    If ( isubr=1 ) then
!       Read first record
!       Analyse the first word (stationary_free_boundary)
!    Read rest of first record (sequence number)
!    Read the rest of the records by intx02 until END is found
!    set maxlen and nrinp to final values
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code1 / 'PRINT_LEVEL', &
                  'ADAPT_MESH', 'SEQ_VECTORS', 'MESH_VELOCITY', &
                  'CHECK_BOUNDARY', 'CHECK_MESH', 'ALPHA_BOUN', &
                  'ANGLE_MIN', 'ANGLE_MAX', 'INTERPOLATE_SOLUTION', &
                  'WRITE_MESH', 'NO_WRITE_MESH', &
                  'STRUCT_MESH_VELOCITY' /
      data ncode1  / 3, 5, 6, 8,  9,  9,  8, 8, 8, 5, 9, 11, 5 /
      data ncodmx1 /11,10,11,13, 14, 10, 10, 9, 9,20,10, 12,20 /
      data itype1  / 0, 0,11,38,  0,  0,  1, 1, 1,18, 2,  2, 0 /
      data tabmin1 / 0d0, 1d0, 1d1, 1d0, 0d0, 0d0, 1d-2, 1d0,  90d0, &
                    0d0, 0d0, 0d0, 0d0 /
      data tabmax1 / 0d0, 1d1, 1d2, 1d2, 2d0, 2d0,  1d2,45d0, 170d0, &
                    0d0, 0d0, 0d0, 1d2 /

!     code1    Table with keywords corresponding to options
!             Values:
!             1:  print_level
!             2:  adapt_mesh
!             3:  seq_vectors
!             4:  mesh_velocity
!             5:  check_boundary
!             6:  check_mesh
!             7:  alpha_boun
!             8:  angle_min
!             9:  angle_max
!            10:  interpolate_solution
!            11:  write_mesh
!            12:  no_write_mesh
!            13:  struct_mesh_velocity
! ======================================================================
!
!     --- Initialize arrays and parameters

      call eropen ( 'readinfr' )

      lastinp = 12
      if ( maxlen<lastinp ) then

!     --- maxlen too small

         call errint ( lastinp, 1 )
         call errint ( maxlen, 2 )
         call errsub ( 329, 2, 0, 0 )
         go to 1000

      end if
      if ( nrinp<3 ) then

!     --- nrinp too small

         call errint ( 1, 1 )
         call errint ( nrinp, 2 )
         call errsub ( 329, 2, 0, 0 )

      end if
      iinput(2:lastinp) = 0
      iinput(3) = 1
      iinput(4) = lastinp
      iinput(11) = 1
      iinput(lastinp) = 1
      rinput(1) = 2d0
      rinput(2) = 10d0
      rinput(3) = 120d0

      if ( isubr==1 ) then

!     --- Call by INSTFREE, read first record  (INSTATIONARY_FREE_BOUNDARY)

         ityprd = 1
         call asknxt
         if ( ityprd/=4 ) then
            call errchr ( 'INSTATIONARY_FREE_BOUNDARY', 1 )
            call errsub ( 650, 0, 0, 1 )
            call errcrd
            go to 1000
         else if ( string(1:6)/='INSTAT' ) then
            call errchr ( 'INSTATIONARY_FREE_BOUNDARY', 1 )
            call errsub ( 650, 0, 0, 1 )
            call errcrd
            go to 1000
         end if

      end if

      call sepreadseqnr (iseqnr  )   ! read sequence number
      call sepcopycode ( code1, nncode1, ncode1, ncodmx1 )
      call sepcopytype ( nncode1, itype1, tabmin1, tabmax1 )
      call sepsetmainend                  ! Fill END as main keyword

      codesub(1) = 'V'
      ncodesub(1) =  1

      table(1:nncode) = -1d20
      table(10) = -1d0
      table(13) = -1d0
      tabmin(10) = dble(nout)
      tabmax(10) = 1d2
      istop = 2
      iretur = 0
      call sepreadsetkeyw
      istop = 0
      if ( table(2)>0d0 ) iinput(3) = nint(table(2))
      if ( table(3)>0d0 ) then

!     --- iseq_vectors found, store number of vectors in iinput(11) and
!         sequence numbers from position lastinp

         iposut = nint(table(3))
         m = nint(tabmax(3))
         iinput(11) = m
         if ( lastinp+m-1>maxlen ) then
            call errint ( lastinp+m-1, 1 )
            call errint ( maxlen, 2 )
            call errchr ( 'iinput', 1 )
            call errsub ( 550, 2, 0, 1 )
            go to 1000
         end if
         do k = 0, m-1
            iinput(lastinp+k) = nint(tabout(iposut+k))
         end do
         lastinp = lastinp+m-1

      end if
      if ( table(4)>0d0 ) iinput(7) = nint(table(4))
      if ( table(5)>0d0 ) iinput(8) = nint(table(5))
      if ( table(6)>0d0 ) iinput(9) = nint(table(6))
      if ( table(7)>0d0 ) rinput(1) = table(7)
      if ( table(8)>0d0 ) rinput(2) = table(8)
      if ( table(9)>0d0 ) rinput(3) = table(9)
      if ( table(10)>0d0 ) then

!     --- interpolate solution found

         iposut = nint(table(10))
         m = nint(tabmax(10))
         iinput(5) = m

!        --- Check the sequence numbers stored in tabout

         ifirst = nint(tabout(iposut))
         sequent = .true.
         do k = 1, m-1
            if ( nint(tabout(iposut+k))/=nint(tabout(iposut+k-1))-1 ) &
               sequent = .false.
         end do
         if ( sequent ) then
            if ( ifirst>1 ) iinput(6) = -ifirst
         else
            iinput(6) = lastinp+1
            if ( lastinp+m>maxlen ) then
               call errint ( lastinp+m, 1 )
               call errint ( maxlen, 2 )
               call errchr ( 'iinput', 1 )
               call errsub ( 550, 2, 0, 1 )
               go to 1000
            end if
            do k = 1, m
               iinput(lastinp+k) = nint(tabout(iposut+k-1))
            end do
            lastinp = lastinp+m
         end if
      end if
      if ( table(11)>0d0 ) iinput(10) = 1
      iinput(7) = iinput(7)+1000*(1+nint(table(13)))
      iinput(1) = lastinp
      if ( iinput(9)>0 .and. iinput(5)==0 ) then

!     --- iinput(9)>0, whereas iinput(5) = 0
!         This means that checking of mesh is applied withour interpolation

         call errwar ( 2190, 0, 0, 0 )

      end if

1000  call erclos ( 'readinfr' )
      end subroutine readinfr
