      subroutine readinverse ( iinput, rinput, isubr, iseqnr, niinp, nrinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-10-1999
!
!   copyright (c) 1999-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!  Read the input for the subroutine compinverse
!  The input is stored in the arrays iinput and rinput
!  The input must be enclosed between INVERSE_PROBLEM and END.
! **********************************************************************
!
!                       KEYWORDS
!
!     loop
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer niinp, nrinp, iinput(*), iseqnr, isubr
      double precision rinput(*)

!    iinput  o   The integer input read is stored in array IINPUT
!                This array must be filled as follows:
!                 1: Largest entry that has been filled by the user
!                    Default value: 11
!                 2: iseqsl:  sequence number of input for linear
!                             solver
!                    Default value:
!                 3: iseqcf:  sequence number of input for coefficients
!                    Default value: 1
!                 4: iveccapacity: sequence number of the capacity
!                                  vector in array isolut
!                    Default value: 3
!                 5: ibegincv: curve number of first electrode
!                    Default value: 1
!                 6: iendcv : curve number of last electrode
!                    Default value: 2
!                 7: ivecsol: sequence number of the solution
!                             vector in array isolut
!                    Default value: 1
!                 8: ivecreact: sequence number of the reaction
!                               vector in array isolut
!                    Default value: 2
!                 9: ielgrploc: element group sequence number
!                               corresponding to inverse problem
!                    Default value: 1
!                10: iveceps: sequence number in array solut of
!                             array in which the values of epsilon
!                             are stored
!                    Default value: 4
!                11: method: type of method to solve the inverse problem
!                    Default value: 1
!    iseqnr i/o  Sequence number with respect to the storage in array ISEPIN
!    isubr   i   Indicates by which subroutine readinverse is called
!                Possible values:
!                1:  ...      read record with INVERSE_PROBLEM
!                2:  EREIGV,  do not read record with INVERSE_PROBLEM
!    niinp  i/o  Length of integer input array
!                At input available length, at output computed length
!    nrinp  i/o  Length of real input array
!                At input available length, at output computed length
!    rinput  o   Contains the real input for the eigenvalue subroutine
!                At this moment he input is defined as follows
!                1: value of delta_eps
!                2: value of regularization parameter mu
!                3: value of eps_ref
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode1=13
      integer :: ncode1(nncode1), ncodmx1(nncode1), itype1(nncode1)
      integer k
      double precision :: tabmin1(nncode1), tabmax1(nncode1)
      character (len=21) code1(nncode1)

!     code1      Table with keywords corresponding to options
!     itype1     itype array for code1 (See intx02)
!     k          Counting variable
!     ncode1     Table containing the number of significant characters in
!                the keyword table code1
!     ncodmx1    Table containing the number of maximum characters in
!                the keyword table code1
!     nncode1    Number of entries in table code1
!     tabmax1    Table with maximum values of numbers corresponding to code1
!     tabmin1    Table with minimum values of numbers corresponding to code1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYTYPE    Copy itype2 and related arrays into itype
!     SEPREADSEQNR   Read sequence number iseqnr from record
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
!     SEPSETMAINEND  Fill keyword END as first main keyword and set nmain to 1
! **********************************************************************
!
!                       I/O
!
!     readinverse requires input from the standard input file
!     The following input is recognized:
!     INVERSE_PROBLEM [, SEQUENCE_NUMBER = s] (mandatory):
!         start of input
!      optional records:
!          lin_solver = i             (Default 1)
!          curve_begin = j            (Default 1)
!          curve_end = j              (Default 2)
!          solution_vector = s        (Default 1)
!          capacity_vector = s        (Default 3)
!          reaction_vector = s        (Default 2)
!          epsilon_vector  = s        (Default 4)
!          seq_coef = m               (Default 1)
!          element_group = i          (Default 1)
!          method = m                 (Default capacity_simple=1)
!          regular_parm = r           (Default 0.01)
!          delta_eps = e              (Default 0.1)
!          eps_ref = e                (Default 1)
!     END   (mandatory):  end of input
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     329   Array to store the input too small
!     650   First record read by subroutine is not equal to INVERSE_PROBLEM
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Initialize arrays and parameters with default values
!    If ( isubr=1 ) then
!       Read first record
!       Analyse the first word (INVERSE_PROBLEM)
!    Read rest of first record (sequence number)
!    Read the rest of the records by intx02 until END is found
!    set niinp and nrinp to final values
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code1 / 'LIN_SOLVER', 'SEQ_COEF', 'CAPACITY_VECTOR', &
                  'CURVE_BEGIN', 'CURVE_END', 'SOLUTION_VECTOR', &
                  'REACTION_VECTOR', 'ELEMENT_GROUP', 'EPSILON_VECTOR', &
                  'METHOD', 'REGULAR_PARM', 'DELTA_EPS', 'EPS_REF' /
      data ncode1  / 3, 6, 5, 9, 9, 5, 5, 5,11, 6, &
                    6, 7, 7 /
      data ncodmx1 /10, 8,15,11, 9,15,15,13,14, 6, &
                   12, 9, 7 /
      data itype1  / 0, 0, 0, 0, 0, 0, 0, 0, 0, 5, &
                    1, 1, 1 /
      data tabmin1 / 1d0, 1d0, 1d0, 1d0, 2d0, 1d0, 1d0, 1d0, 1d0, 1d0, &
                   1d-10, 1d-5, 1d-10 /
      data tabmax1 / 1d2, 1d2, 1d2, 1d3, 1d3, 1d2, 1d2, 1d3, 1d2, 1d0, &
                    1d1, 1d1, 1d10 /

!     code1   Table with keywords corresponding to options
!             Values:
!             1:  LIN_SOLVER
!             2:  SEQ_COEF
!             3:  CAPACITY_VECTOR
!             4:  CURVE_BEGIN
!             5:  CURVE_END
!             6:  SOLUTION_VECTOR
!             7:  REACTION_VECTOR
!             8:  ELEMENT_GROUP
!             9:  EPSILON_VECTOR
!            10:  METHOD
!            11:  REGULAR_PARM
!            12:  DELTA_EPS
!            13:  EPS_REF
! ======================================================================
!
!     --- Initialize arrays and parameters

      call eropen ( 'readinverse' )

      if ( niinp<nncode1-2 ) then

!     --- niinp too small

         call errint ( nncode1-1, 1 )
         call errint ( niinp, 2 )
         call errsub ( 329, 2, 0, 0 )
         go to 1000

      end if
      if ( nrinp<3 ) then

!     --- nrinp too small

         call errint ( 2, 1 )
         call errint ( nrinp, 2 )
         call errsub ( 329, 2, 0, 0 )

      end if
      iinput(1) = nncode+1
      iinput(2:niinp) = 1
      iinput(4) = 3
      iinput(6) = 2
      iinput(8) = 2
      iinput(9) = 4
      rinput(1) = 0.01d0
      rinput(2) = 0.1d0
      rinput(3) = 1d0

      if ( isubr==1 ) then

!     --- Call by xxxxx, read first record  (INVERSE_PROBLEM)

         ityprd = 1
         call asknxt
         if ( ityprd/=4 ) then
            call errchr ( 'INVERSE_PROBLEM', 1 )
            call errsub ( 650, 0, 0, 1 )
            call errcrd
            go to 1000
         else if ( string(1:7)/='INVERSE' ) then
            call errchr ( 'INVERSE_PROBLEM', 1 )
            call errsub ( 650, 0, 0, 1 )
            call errcrd
            go to 1000
         end if

      end if

      call sepreadseqnr (iseqnr  )   ! read sequence number
      call sepcopycode ( code1, nncode1, ncode1, ncodmx1 )
      call sepcopytype ( nncode1, itype1, tabmin1, tabmax1 )
      call sepsetmainend                  ! Fill END as main keyword

      codesub(1) = 'CAPACITY_SIMPLE'
      ncodesub(1) =  13

      table(1:nncode) = -1d20
      istop = 2
      iretur = 0
      call sepreadsetkeyw
      istop = 0
      if ( table(1)>0d0 ) iinput(2) = nint(table(1))
      if ( table(2)>0d0 ) iinput(3) = nint(table(2))
      if ( table(3)>0d0 ) iinput(4) = nint(table(3))
      if ( table(4)>0d0 ) iinput(5) = nint(table(4))
      if ( table(5)>0d0 ) iinput(6) = nint(table(5))
      if ( table(6)>0d0 ) iinput(7) = nint(table(6))
      if ( table(7)>0d0 ) iinput(8) = nint(table(7))
      if ( table(8)>0d0 ) iinput(9) = nint(table(8))
      if ( table(9)>0d0 ) iinput(10) = nint(table(9))
      if ( table(10)>0d0 ) iinput(11) = nint(table(10))
      if ( table(11)>0d0 ) rinput(2) = table(11)
      if ( table(12)>0d0 ) rinput(1) = table(12)
      if ( table(13)>0d0 ) rinput(3) = table(13)

1000  call erclos ( 'readinverse' )
      end subroutine readinverse
