      subroutine readoutp ( ivectr, ndtvecold )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.3  date 29-05-2014 remove call to time subroutine
!        version  2.2  date 25-11-2012 Long integers
!        version  2.1  date 17-05-2012 Correction error message
!
!   copyright (c) 1999-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     read real/complex sepran solution array from the file sepcomp.out
!     This subroutine is meant to be used in combination with subroutine
!     prinoutp
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulemain
      use sepmodulekmesh
      use sepmodulevecs
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ivectr, ndtvecold

!     ivectr         o    integer array denoting the array to be read
!     ndtvecold      i    Value of ndtvec at start of calling subroutine
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical compl
      integer ihelp(8), ios, i, itypvc, ivec, k, nlines, number, icompl

!     compl          indication whether the arithmetic is real (false)
!                    or complex (true)
!     i              Counting variable
!     ihelp          integer work array of length 8 to store
!                    quantities corresponding to ivectr
!                    1: compl
!                    2: iprob
!                    3: ivec
!                    4: jcart
!                    5: nphys
!                    6: nusol
!                    7: typevec
!                    8: iseqkp
!     ihelp          integer work array to store the first 5 integers of
!                    the vector to be read
!     ios            IOSTAT variable, returns from reading or writing
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     ivec           Sequence number of vector of special structure
!     k              Counting variable
!     nlines         number of parts where line elements are created
!     number         number of unknowns per point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INISTREXT      Find the structure of the vector with exactly
!                    NUMBER unknowns per point
!     PRININ         print 1d integer array
!     READHEADING    Read second part of information of solution array from
!                    sepcomp.out
!     READOUTCVEC    read real sepran solution array from the file sepcomp.out
!     READOUTVEC     read complex sepran solution array from the file
!                    sepcomp.out
!     SEPCREASOLBF1  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
!     SEPFILLKMESHAA Fill first part of array KMESH part aa
!     SEPFILLNMESHA  Fill array nmesha
! **********************************************************************
!
!                       I/O
!
!    The solution vector or vector of special structure is read from the
!    file with reference number iref74
!    If iref74>0, the file is formatted, otherwise it is unformatted
!    The first record contain the 5 integers that must be stored into array
!    IVECTR, the second record contains the real values
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      60   Input solution array has not been filled or is of the wrong type
!    2023   The mesh does not correspond to the vectors read
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readoutp' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readoutp'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- read heading, ihelp is an old fashioned array of length 5
!         Do not change

      call readheading ( ios, ihelp, nlines, nglunk, nfictunk )
      if ( ierror/=0 ) go to 1000

      itypvc = ihelp(7)
      ivec = ihelp(3)
      nusolo = ihelp(6)
      iseqkp = ihelp(8)
      if(ihelp(1)==0) then
         compl = .false.
      else
         compl = .true.
      end if
      if ( debug ) then
         write(irefwr,1) 'itypvc, ivec, nusolo', itypvc, ivec, nusolo
         call prinin ( ihelp, 8, 'ihelp' )
      end if  ! ( debug )
      if ( itypvc==127 ) then

!     --- Vector consisting of faces
!         Create faces if necessary

         if ( kelmaa==0 ) call sepfillkmeshaa
         if ( nmesh(31)==0 ) call sepfillnmesha

      else if ( itypvc/=110 .and. itypvc/=115 .and. &
                itypvc/=116 .and. itypvc/=129 .and. &
                itypvc/=801 ) then

!     --- Error: this type of array has not yet been implemented

         call errint ( itypvc, 1 )
         call errsub ( 60, 1, 0, 0 )

      end if
      iprob  = ihelp(2)
      if ( itypvc==129 ) then

!     --- Special case itypvc = 129, vector defined per integration point

         nphyso = ihelp(6)

      end if ! ( itypvc==129 )

      if ( mainsubr==4 .and. itypvc==115 ) then

!     --- Vector of special structure, check if this is an extension

         iprob = ihelp(2)
         ivec = ihelp(3)

         if ( debug ) &
            write(irefwr,1) 'iprob, ivec, ndtvecold', iprob, ivec, ndtvecold
         if ( iprob==1 .and. ivec>ndtvecold ) then

!        --- It concerns a new array

            number = ihelp(6)/npoint
            itypvc = 115
            call inistrext ( number, itypvc, ivec )
            if ( debug ) &
               write(irefwr,1) 'number, ivec, iprob', &
                                number, ivec, iprob

         end if  ! ( iprob==1 .and. ivec>ndtvecold )

      end if  ! ( itypvc==115 )

      if ( itypvc==801 ) nphyso = 1
      icompl = ihelp(1)
      iseqkp = kprob
      kprobpart => kp(iseqkp)%kprobsub(iprob)
      kprobloc => kprobpart%kprobloc
      call sepcreasolbf1 ( ivectr, itypvc, ivec, 'vector', icompl )
      if ( itypvc==801 ) then
         nphyso = ivec
      else if ( itypvc==129 ) then
         nphyso = ihelp(5)
      end if  ! ( itypvc==801 )
      if ( debug ) then
         write(irefwr,1) 'ivectr', ivectr, ks(ivectr)%compl
      end if  ! ( debug )

      if ( ierror/=0 )  go to 1000
      usoltemp => ks(ivectr)%sol
      length = size ( usoltemp )
      if ( compl ) length = length/2
      if ( length/=nusolo ) then

!     --- Error: incorrect length, probably incorrect mesh

         call errint ( length, 1 )
         call errint ( nusolo, 2 )
         call errsub ( 2023, 2, 0, 0 )
         go to 1000

      end if

      if ( ks(ivectr)%typevec==110 .or. ks(ivectr)%typevec==115 ) then
         nlines = npoint
      else if ( ks(ivectr)%typevec==127 ) then
         nlines = nmesh(4)
      else if ( ks(ivectr)%typevec==801 ) then
         nlines = (nphyso*(nphyso-1))/2
         nphyso = length/nlines
      else
         nlines = nelem
      end if

      if ( compl ) then

!     --- Complex vector

         call readoutcvec ( usoltemp, nlines )

      else

!     --- Real vector

         call readoutvec ( usoltemp, nlines )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readoutp'

      end if  ! ( debug )
      call erclos ( 'readoutp' )

      end subroutine readoutp
