      subroutine readscpnlnconc ( istart, inpsep, rinsep, istrin )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 28-12-2014 New call to readseqint
!        version  1.0  date 24-04-2013
!
!   copyright (c) 2013-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read the input following the following the keywords
!     PART_NONLINEAR_CONCENTRATION
!     with respect to subroutine readsc
!     The input is stored in array inpsep and the real array rinsep
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     non_linear
!     concentration
!     convection_diffusion
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleread1
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istart, inpsep(*), istrin
      double precision :: rinsep(*)

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         This part contains:
!                          2: seq_conc Sequence number of concentration
!                          3: seq_vn   Sequence number of vn
!                             in 1d sequence number of vnboun
!                          4: seq_g    Sequence number of rhs_bound
!                          5: seq_time_int    Sequence number of time integrat.
!                          6: numcomps Number of components
!                          7: seq_cinf Sequence number of cinf
!                          8: seq_xinf Sequence number of xinf
!                          9: seq_cpart Sequence number of cpart
!                         10: seq_k    Sequence number of k
!                         11: seq_sub  Sequence number of substitutional
!                         12: maxiter  Maximum number of iteration
!                         13: iteration_method  type of iteration method
!                             Possible values
!                             1: picard
!                             2: aitken_picard
!                             3: vec_aitken_picard
!                         14: print_level  amount of output
!                         15: position of eps in rinsep
!                         16: seq_curv Sequence number of curvature
!                         17: seq_zeta Sequence number zeta
!                         18: seq_kinf Sequence number of kinf
!                         19: seq_numiter Sequence number of numiter
!                         20: seq_cs
!     istart        i/o   Next free position in inpsep
!     istrin        i/o   Next free position in rinsep
!     rinsep        i/o   The information of the plane is stored in
!                         rinsep from position istrin
!                         Pos. 1   eps
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode2=19, nncod3=3
      integer ncode2(nncode2), ncode3(nncod3), &
              ncodmx2(nncode2), istrinsav, numcomps, itypeg
      character (len=20) code2(nncode2)
      character (len=17) code3(nncod3)

!     code2          Table with keywords
!     code3          Table with keywords
!     istrinsav      Value of istrin at input
!     itypeg         Indicates if the parameter g should be scalar (1)
!                    or vector (2)
!     ncode2         Table containing the number of significant characters
!                    in the keyword table code2
!     ncode3         Table containing the number of significant letters in
!                    code3
!     ncodmx2         Table containing the maximal number of  characters in
!                    the keyword table code
!     nncod3         Number of entries in code3
!     nncode2        Number of entries in code
!     numcomps       Number of components
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer getintparm

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHKDIM      Check dimension of array or scalar
!     GETINTPARM     Return with real/scalar defined by inpsep/rinsep
!     PRININLARGE    Print 1d integer vector (large numbers)
!     PRINRL         Print 1d real vector
!     READSEQINT     Put the value of the integer variable read in iinput
!     READSEQSCALAR  Put the value of the sequence number read in iinput
!     READSEQVAR     Put the value of the variable read in rinput
!     READSEQVECTOR  Put the value of the sequence number read in iinput
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYCODESUB Copy codemn2 and related arrays into codesub
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!       PART_NONLINEAR_CONCENTRATION, options (all in one line)
!       with options:
!          concentration           default: concentration
!          accuracy = eps          default: 1e-3 or epsloop
!          max_iter = m            default: 100 or max_iter
!          normal_velocity = vn    default: vn
!           in 1d: normal_velocity = vnboun    default: vnboun
!          rhs_bound = g           default: g
!          curvature = c           default: curvature
!          seq_time_integration=i  default: 1 or seq_time_integration
!          numcomps = n            default: numcomps
!          cinf = c                default: cinf
!          xinf = x                default: xinf
!          cpart = c               default: cpart
!          K = k                   default: k
!          Kinf = kinf             default: kinf
!          zeta = z                default: zeta
!          substitutional = s      default: substitutional
!          iteration_method = p    default: picard_aitken
!          print_level = p         default: 0 or print_level
!          cs = cs                 default: cs or none
!          num_iter = numiter      default: numiter (must be scalar)
!             Possibilities:
!             picard
!             aitken_picard
!             vec_aitken_picard
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code2 / 'CONCENTRATION', 'NORMAL_VELOCITY', 'RHS_BOUND', &
                  'SEQ_TIME_INTEGRATION', 'NUMCOMPS', 'CINF', 'XINF', &
                  'CPART' , 'K', 'SUBSTITUTIONAL', 'MAX_ITER', &
                  'ITERATION_METHOD', 'ACCURACY', 'PRINT_LEVEL', &
                  'CURVATURE', 'ZETA', 'KINF', 'NUM_ITER', &
                  'CS' /
      data ncode2 /  4,  4,  3,  8,  4,   4,  4,  4,  1, 3, &
                    3,  4,  3,  4,  4,   4,  4,  2,  4 /
      data ncodmx2 /13, 15,  9, 20,  8,   4,  4,  5,  1,14, &
                    8, 16,  8, 11,  9,   4,  4,  2,  4 /
      data code3 / 'PICARD', 'AITKEN_PICARD', 'VEC_AITKEN_PICARD' /
      data ncode3 / 3,  3,  3 /

!     code2   Table with sub keywords corresponding to keyword
!            PART_NONLINEAR_CONCENTRATION
!             Values:
!             1:   concentration
!             2:   normal_velocity
!             3:   rhs_bound
!             4:   seq_time_integration
!             5:   numcomps
!             6:   cinf
!             7:   xinf
!             8:   cpart
!             9:   k
!            10:   substitutional
!            11:   max_iter
!            12:   iteration_method
!            13:   accuracy
!            14:   print_level
!            15:   curvature
!            16:   zeta
!            17:   kinf
!            18:   num_iter
!            19:   cs
!     code3  Table with subsub keywords corresponding to keywords in code2
!             1:   PICARD
!             2:   AITKEN_PICARD
!             3:   VEC_AITKEN_PICARD
! ======================================================================
!
      call eropen ( 'readscpnlnconc' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readscpnlnconc'
         istrinsav = istrin

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call sepcopycode ( code2, nncode2, ncode2, ncodmx2 )
      call sepcopycodesub ( code3, nncod3, ncode3 )

      inpsep(1) = icode

      itype(1:nncode) = 43                  ! vector or scalar
      tabmin(1:nncode) = 1d0
      tabmax(1:nncode) = 10d0
      table(1:nncode) = -1d0

      itype(4:5) = 0              ! seq_time_integration/max_iter

      tabmax(11) = 1d4            ! max_iter

      itype(12) = 5               ! iteration_method
      tabmin(12) = 1d0
      tabmax(12) = tabmin(12)+2d0

      itype(13) = 1               ! accuracy
      tabmin(13) = 1d-12
      tabmax(13) = 0.5d0

      itype(14) = 0               ! print_level
      tabmin(14) = 0d0
      tabmax(14) = 3d0

      itype(19) = 2               ! output_cs
      table(19) = 0d0

      nseqkeyw = nncode
      nmain = 0
      call sepreadsetkeyw
      if ( debug ) call prinrl ( table, nncode, 'table' )

      inpsep(13) = 2    ! iteration_method

      call readseqvector ( table(1), 'concentration', inpsep(2), -1 )
      if ( ndim==1 ) then

!     --- ndim = 1, g must be scalar

         call readseqscalar ( table(2), 'vnboun', inpsep(3), -1 )
         call readseqscalar ( table(3), 'g', inpsep(4), -1 )
         call readseqscalar ( table(19), 'cs', inpsep(20), 0 )
         itypeg = 1

      else

!     --- ndim>1, g must be vector

         call readseqvector ( table(2), 'vn', inpsep(3), -1 )
         call readseqvector ( table(3), 'g', inpsep(4), -1 )
         call readseqvector ( table(19), 'cs', inpsep(20), 0 )
         itypeg = 2

      end if  ! ( ndim==1 )

      icode = 4
      call readseqint ( inpsep(5), 1 )
      call readseqscalar ( table(5), 'numcomps', inpsep(6), -1 )
      call readseqscalar ( table(6), 'cinf', inpsep(7), -1 )
      call readseqscalar ( table(7), 'xinf', inpsep(8), -1 )
      call readseqscalar ( table(8), 'cpart', inpsep(9), -1 )
      call readseqscalar ( table(9), 'k', inpsep(10), -1 )
      call readseqscalar ( table(10), 'substitutional', inpsep(11), -1 )
      icode = 11
      call readseqint ( inpsep(12), 100 )
      if ( table(12)>0d0 ) inpsep(13) = nint(table(12))  ! iteration_method
      icode = 14
      call readseqint ( inpsep(14), 0 )
      call readseqvector ( table(15), 'curvature', inpsep(16), 0 )
      call readseqscalar ( table(16), 'zeta', inpsep(17), 0 )
      call readseqscalar ( table(17), 'kinf', inpsep(18), -1 )
      call readseqscalar ( table(18), 'numiter', inpsep(19), -1 )

      icode = 13
      call readseqvar ( 'epsloop', rinsep, inpsep(15), istrin, 1d-3 )

      if ( ierror/=0 ) go to 1000

!     --- Check dimensions of arrays and scalars

      numcomps = getintparm(inpsep(6))
      call errchkdim ( numcomps, inpsep(2), 2, 'concentration' )
      call errchkdim ( numcomps, inpsep(4), itypeg, 'rhs_bound' )
      call errchkdim ( numcomps, inpsep(7), 1, 'cinf' )
      call errchkdim ( numcomps, inpsep(8), 1, 'xinf' )
      call errchkdim ( numcomps, inpsep(9), 1, 'cpart' )
      call errchkdim ( numcomps, inpsep(10), 1, 'K' )

1000  if ( debug ) then

!     --- Debug information

         call prininlarge ( inpsep, 19, 'inpsep' )
         if ( istrin>istrinsav ) &
            call prinrl ( rinsep(istrinsav), istrinsav-istrin, &
                          'rinsep' )
         write(irefwr,*) 'End readscpnlnconc'

      end if  ! ( debug )
      call erclos ( 'readscpnlnconc' )
      istart = istart+20

      end subroutine readscpnlnconc
