      subroutine readscstandard ( istart, inpsep, outfnd, nprob )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.5  date 15-05-2014 Extra debug
!        version  2.4  date 15-03-2014 Adaptation new vectors
!        version  2.3  date 18-12-2013 Extension for READ_SOLUTIONS
!
!   copyright (c) 2002-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Read the input following the following standard keywords with respect
!   to subroutine readsc
!             1:  PRESCRIBE_BOUNDARY_CONDITIONS
!             5:  OUTPUT
!            14:  COPY_COOR
!            19:  SOLVE_TIME_DEPENDENT_PROBLEM
!            41:  COMPUTE_CONTACT_SURFACE
!            48:  CHANGE_COORDINATES
!            50:  CHANGE_PROBLEM
!            51:  COMPUTE_CAPACITY
!            52:  SOLVE_INVERSE_PROBLEM
!            71:  ENTHALPY_INTEGRATION
!            72:  COMPUTE_ENTHALPY
!            77:  PRESSURE_CORRECTION
!            78:  NAVIER_STOKES
!            79:  SOLVE_BEARING
!            80:  READ_SOLUTIONS
!           108:  CHANGE_VECTOR
!   The input is stored in array inpsep and the real array rinsep
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     output
!     boundary_condition
!     change_boundary
!     capacity
!     inverse_problem
!     pressure_correction
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      use sepmoduleplot
      use sepmoduletexts
      use sepmodulepost
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical outfnd
      integer istart, inpsep(0:*), nprob

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         This part contains:
!                         0  kcode
!                         1  ISEQNR Sequence number corresponding to input
!                                   If iseqnr = -1, the next one will be used
!                         2  IPROB  Problem sequence number
!                                   If iprob = -1, the next one will be used
!                         3  IVECTORVector sequence number (first vector)
!                                   If ivector = -1, the same value as iprob
!                                   will be used
!     istart        i/o   Next free position in inpsep
!     nprob          i    Number of various problems
!     outfnd        i/o   Call to output found (true) or not (false)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncod2=5
      integer ncode2(nncod2), ncod2m(nncod2), m, ivec, kcode
      character (len=15) code2(nncod2)

!     code2          Table with keywords
!     ivec           Sequence number of vector with respect to array ISOL
!     kcode          Code defined by readsc
!     m              number of items read
!     ncod2m         Table containing the number of maximum characters in
!                    the keyword table code2
!     ncode2         Table containing the number of significant letters in
!                    code2
!     nncod2         Number of entries in code2
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRKEYWEXCL    Print error message concerning two keywords that are
!                    mutually exclusive
!     ERRWAR         Print warnings
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!       PRESCRIBE_BOUNDARY_CONDITIONS, options (all in one line)
!           where options may have the following contents
!               SEQUENCE_NUMBER = s  (Default the next one)
!               VECTOR = s           (Default the same as problem)
!       OUTPUT, options (all in one line)
!           where options may have the following contents
!               SEQUENCE_NUMBER = s  (Default the next one)
!               VECTOR = s           (Default 1)
!               FILE = file_name     (Default sepcomp)
!       SOLVE_TIME_DEPENDENT_PROBLEM, options (all in one line)
!           where options may have the following contents
!               SEQUENCE_NUMBER = s  (Default the next one)
!               VECTOR = s           (Default the same as problem)
!       COMPUTE_CONTACT_SURFACE, sequence_number = k (Default 1)
!       CHANGE_COORDINATES, sequence_number = k (Default 1)
!       CHANGE_PROBLEM, vector=i, problem=p (Default 1,1)
!       COMPUTE_CAPACITY, sequence_number = k (Default 1)
!       SOLVE_INVERSE_PROBLEM, sequence_number = k (Default 1)
!       ENTHALPY_INTEGRATION, sequence_number = k (Default 1)
!       COMPUTE_ENTHALPY, sequence_number = k (Default 1)
!       PRESSURE_CORRECTION, sequence_number = k (Default 1)
!       COPY_COOR, vector = i, problem=p (Default 1)
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1886   Vector has not yet been filled
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code2 / 'SEQUENCE_NUMBER', 'VECTOR', 'V', 'PROBLEM', 'FILE' /
      data ncod2m /15, 6, 1, 7, 4 /
      data ncode2 / 6, 6, 1, 4, 4 /

!     code2   Table with sub keywords corresponding to some keywords
!             Values:
!             1:  SEQUENCE
!             2:  VECTOR
!             3:  V
!             4:  PROBLEM
!             5:  FILE
! ======================================================================
!
      call eropen ( 'readscstandard' )
      debug = .false. .and. ioutp>=0
      kcode = icode
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readscstandard'
         write(irefwr,1) 'kcode, istart, lastitem', &
                          kcode, istart, lastitem
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call sepcopycode ( code2, nncod2, ncode2, ncod2m )

      if ( kcode==5 .or. kcode==19 ) outfnd = .true.
      itype(1:nncode) = 0
      table(1:nncode) = -1d0
      tabmin(1:nncode) = 1d0
      tabmax(1:nncode) = 1d2
      tabmax(1) = 100d0
      tabmax(4) = nprob
      itype(5) = 6  ! file
      iretur = -10
      if ( kcode==1 .or. kcode==19 .or. &
           kcode==51 .or. kcode==52 ) then
         m = 3
      else if ( kcode==5  .or. kcode==80 ) then
         m = nncod2
      else if ( kcode==41 .or. kcode==48 .or. kcode>=71 .and. &
                kcode/=14 .and. kcode<108 ) then
         m = 1
      else
         m = 4
      end if
      nseqkeyw = nncod2
      nmain = 0
      nncode = m

      if ( debug ) write(irefwr,1) 'before intx02ext1: m =', m

      call sepreadsetkeyw
      nncode = nncod2

      if ( debug ) then
         write(irefwr,*) 'after intx02ext1'
         write(irefwr,1) 'iretur, errorcode, ierror', &
                          iretur, errorcode, ierror
         call prinin ( iseqkeyw, -iretur-10, 'iseqkeyw' )
         call prinrl ( table, nncod2, 'table' )

      end if  ! ( debug )

      if ( table(2)>0d0 .and. table(3)>0d0 ) &
         call errkeywexcl ( code2, ncod2m, 3, 2 )

      table(2) = max(table(2),table(3))
      if ( iretur<-10 ) then

!     --- Stand alone number found.
!         This is only allowed if there is only one, if it is the first item
!         read and moreover if not vector is found
!         Put number read into table(2), i.e. ivec

         if ( debug ) &
            write(irefwr,1) 'ierror, iretur, iseqkeyw(1)', &
                             ierror, iretur, iseqkeyw(1)

         if ( ierror==0 .and. iretur==-11 .and. iseqkeyw(1)<0 ) &
            table(2) = tabout(-iseqkeyw(1))

      end if
      inpsep(0) = kcode
      inpsep(1) = nint(table(1))

      if ( kcode==48 .or. kcode>=71 .and. kcode<108 .and. kcode/=80 ) &
         then

!     --- CHANGE_COORDINATES, ENTHALPY_INTEGRATION, COMPUTE_ENTHALPY or
!         PRESSURE_CORRECTION found
!         only sequence number required

         istart = istart+2
         go to 1000

      end if

      inpsep(2) = nint(table(4))
      inpsep(3) = nint(table(2))
      ivec = inpsep(3)

      if ( debug ) write(irefwr,1) 'kcode, ivec', kcode, ivec

      if ( ivec>0 ) then
         if ( kcode==1 .or. kcode==3 .or. kcode==4 .or. kcode==8 .or. &
              kcode==14 .or. kcode==19 .or. kcode>=51 ) then

!        --- vector is new

            if ( ivec<=maxvec ) then
               numvec = min ( max(numvec,ivec), maxvec )
               ivectors(ivec) = 1
            end if  ! ( ivec<=maxvec ) then

         else if ( ivectors(ivec)==0 ) then

!        --- vector does not exist

            call errint ( ivec, 1 )
            call errchr ( names(ivec), 1 )
            call errwar ( 1886, 1, 0, 1 )
            call errcrd

         end if
      end if
      if ( kcode==5 .or. kcode==80 ) then

!     --- OUTPUT consider name of output files

         inpsep(2) = 0
         if ( table(5)>0 ) then
            inpsep(2) = nextxt
            texts(nextxt) = strout(1)
            nextxt = nextxt+1
         end if

      end if
      istart = istart + 4

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'istart', istart
         call prinin ( inpsep, 4, 'inpsep' )
         write(irefwr,*) 'End readscstandard'

      end if  ! ( debug )
      call erclos ( 'readscstandard' )

      end
