      subroutine readscstoc ( inpsep, istart, rinsep, istrin )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 07-03-2012 New call to readseqvector
!        version  1.0  date 04-11-2009
!
!   copyright (c) 2009-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read the input following the keyword
!     stochastic_investment
!     with respect to subroutine readsc
!     The input is stored in array inpsep and rinsep
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     stochastic
!     investment
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleread1
      use sepmoduleplot
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), istart, istrin
      double precision :: rinsep(*)

!     inpsep         o    The integer input read is stored in array INPSEP
!                         INPSEP is filled as follows
!                          1: Number of vectors: n
!                          2: maxiter
!                             Default value: 100
!                          3: dimension of problem
!                          4: starting address in rinsep
!                          5: sequence number of reaction force
!                          6: Print level
!                          7-10: Not yet used
!                             Default value: 0
!                          11   -10+n: seq_vectors for each decision
!                          11+n -10+2n: seq_boun_cond for each decision
!                          11+2n-10+3n: seq_coeff for each decision
!     istart        i/o   Next position in array inpsep
!                         The parameter is raised in this subroutine
!     istrin        i/o   Next free position in rinsep
!     rinsep         o    Array containing the real input for this subroutine
!                         Contents:
!                         1:   accuracy: eps
!                             Default value: 0.001
!                         2-n:   investments for n-1 decisions
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode1=9
      integer ncode1(nncode1), itype1(nncode1), ncodmx1(nncode1), &
              jstart, i, num, numveclc, ivec
      double precision :: tabmin1(nncode1), tabmax1(nncode1)
      character (len=14) code1(nncode1)

!     code1          Table with keywords
!     i              Counting variable
!     itype1         Table containing information about the numbers behind
!                    the keywords in the table code1
!     ivec           Sequence number of vector of special structure
!     jstart         starting position
!     ncode1         Table containing the number of significant characters
!                    in the keyword table code1
!     ncodmx1        Table containing the maximal number of  characters in
!                    the keyword table code1
!     nncode1        Number of entries in code1
!     num            Number of numbers read
!     numveclc         Number of vectors stored in isol
!     tabmax1        Contains maximum values allowed in table
!     tabmin1        Contains minimum values allowed in table
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     READSEQVECTOR  Put the value of the sequence number read in iinput
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYTYPE    Copy itype2 and related arrays into itype
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!       STOCHASTIC_INVESTMENT, options
!           where options may have the following contents
!           VECTORS = (.. ,..)
!           SEQ_BOUN = ( .. )
!           SEQ_COEF = (.. )
!           ACCURACY = eps
!           INVESTEMENTS = (... )
!           MAX_ITER = ...
!           REACTION_FORCE = ...
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3029   number of boundary conditions is not number of vectors
!    3030   number of coefficients is not number of vectors
!    3031   number of investements is not number of vectors minus 1
!    3032   No investements found
!    3033   No vectors found
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code1 / 'VECTORS', 'DIMENSION', 'SEQ_BOUN', 'SEQ_COEF', &
                  'ACCURACY', 'INVESTEMENTS', 'MAX_ITER', &
                  'REACTION_FORCE', 'PRINT_LEVEL' /
      data ncode1  / 6,  3, 4, 4, 5, 5, 5, 5, 5 /
      data ncodmx1 / 7,  9, 8, 8, 8,12, 8,14,11 /
      data itype1  / 28, 0,28,28, 1,19, 0,38, 0 /
      data tabmin1 / 1d0, 1d0, 1d0, 1d0, 1d-10, 1d1, 1d0, 0d0, 0d0 /
      data tabmax1 / 1d2, 3d0, 1d2, 1d2, 1d0,  1d10, 1d4, 0d0, 3d0 /

!     code   Table with sub keywords corresponding to keyword
!            stochastic_investment
!             Values:
!             1:  VECTORS
!             2:  DIMENSION
!             3:  SEQ_BOUN
!             4:  SEQ_COEF
!             5:  ACCURACY
!             6:  INVESTEMENTS
!             7:  MAX_ITER
!             8:  REACTION_FORCE
!             9:  PRINT_LEVEL
! ======================================================================
!
      call eropen ( 'readscstoc' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readscstoc'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      inpsep(istart) = icode      ! standard first item
      call sepcopycode ( code1, nncode1, ncode1, ncodmx1 )
      call sepcopytype ( nncode1, itype1, tabmin1, tabmax1 )

!     --- Clear table

      table(1:nncode) = -1d0
      table(2) = dble(ndim)     ! default dimension

      tabmin(1) = 10d0              ! maximum number of vectors
      tabmax(1) = dble(maxvec)  ! maximum number of vectors
      tabmin(3:4) = tabmin(1)
      tabmin(6) = tabmin(1)
      tabmax(3:4) = tabmax(1)
      tabmax(6) = tabmax(1)

      iretur = 0
      nmain = 0
      call sepreadsetkeyw
      if ( ierror/=0 ) go to 1000

      inpsep(istart+2) = 100      ! maxiter
      inpsep(istart+4) = istrin   ! starting address rinsep
      inpsep(istart+6) = 0        ! print level

!     --- Interpret the input

      if ( table(1)>0 ) then

!     --- vectors

         iposut = nint(table(1))      ! starting position of vectors
         numveclc = nint(tabmax(1))     ! number of vectors read
         inpsep(istart+1) = numveclc    ! number of vectors
         inpsep(istart+3) = 0         ! dimension of space (not yet filled)

      else

!     --- Error: no vectors found

         call errsub ( 3033, 0, 0, 0 )
         numveclc = 0
         go to 1000

      end if  ! ( table(1)>0 )

      inpsep(istart+3) = nint(table(2))    ! dimension of space

!     --- Store vector numbers

      jstart = istart+10           ! starting position minus 1
      inpsep(jstart+1:jstart+numveclc) = &
         nint(tabout(iposut:iposut+numveclc-1))
      jstart = jstart+numveclc       ! raise starting position

      if ( table(3)>0 ) then

!     --- boundary conditions

         iposut = nint(table(3))   ! starting position of boundary conditions
         num = nint(tabmax(3))     ! number of boundary conditions read

         if ( num/=numveclc ) then

!        --- Error num#numveclc

            call errint ( num, 1 )
            call errint ( numveclc, 2 )
            call errsub ( 3029, 2, 0, 0 )

         end if  ! ( num/=numveclc )

!        --- Store boundary conditions

         inpsep(jstart+1:jstart+numveclc) = &
            nint(tabout(iposut:iposut+numveclc-1))

      else

!     --- no boundary conditions found, use defaults

         do i = 1, numveclc
            inpsep(jstart+i) = i
         end do  ! i = 1, numveclc

      end if  ! ( table(3)>0 )

      jstart = jstart+numveclc     ! raise starting position

      if ( table(4)>0 ) then

!     --- coefficients

         iposut = nint(table(4))   ! starting position of coefficients
         num = nint(tabmax(4))     ! number of coefficients read

         if ( num/=numveclc ) then

!        --- Error num#numveclc

            call errint ( num, 1 )
            call errint ( numveclc, 2 )
            call errsub ( 3030, 2, 0, 0 )

         end if  ! ( num/=numveclc )

!        --- Store coefficients

         inpsep(jstart+1:jstart+numveclc) = &
            nint(tabout(iposut:iposut+numveclc-1))

      else

!     --- no coefficients found, use defaults

         do i = 1, numveclc
            inpsep(jstart+i) = i
         end do  ! i = 1, numveclc

      end if  ! ( table(4)>0 )

      jstart = jstart+numveclc     ! raise starting position

      if ( table(5)>0d0 ) then

!     --- accuracy

         rinsep(istrin) = table(5)

      else

         rinsep(istrin) = 1d-3  ! default value

      end if  ! ( table(5)>0d0 )

      if ( table(6)>0d0 ) then

!     --- investements

         iposut = nint(table(6))   ! starting position of investements
         num = nint(tabmax(6))     ! number of investements read

         if ( num/=numveclc-1 ) then

!        --- Error num#numveclc-1

            call errint ( num, 1 )
            call errint ( numveclc, 2 )
            call errsub ( 3031, 2, 0, 0 )

         end if  ! ( num/=numveclc-1 )

!        --- Store investements

         rinsep(istrin+1:istrin+numveclc-1) = &
            tabout(iposut:iposut+numveclc-2)

      else

!     --- No investements found

         call errint ( numveclc-1, 1 )
         call errsub ( 3032, 1, 0, 0 )

      end if  ! ( table(5)>0d0 )

      if ( table(7)>0d0 ) inpsep(istart+2) = nint(table(7))  ! maxiter
      if ( table(9)>0d0 ) inpsep(istart+6) = nint(table(9))  ! print level

!     --- reaction force vector

      call readseqvector ( table(8), 'reac', ivec, 2 )
      inpsep(istart+5) = ivec  ! reaction force

1000  if ( debug ) then

!     --- Debug information

         call prinin ( inpsep(istart), jstart+1-istart, 'inpsep' )
         call prinrl ( rinsep(istrin), numveclc, 'rinsep' )
         write(irefwr,*) 'End readscstoc'

      end if  ! ( debug )
      call erclos ( 'readscstoc' )

!     --- Raise pointers

      istart = jstart+1
      istrin = istrin+numveclc

      end subroutine readscstoc
