      subroutine readscstring ( istart, inpsep, icode, rinsep, istrin )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 08-12-2013 Extension with integer scalars
!        version  1.0  date 01-06-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read information following string = &
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     string
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons
      use sepmoduleread
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istart, inpsep(*), icode, istrin
      double precision rinsep(*)

!     icode          i    Indicates which of the options in readsc is found
!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         This part contains:
!                          2: sequence number of string at left-hand side
!                          3: nparts (number of parts the string consists of)
!                          For each part followed by 2 positions with information:
!                          1: Type of string the part consists of
!                             Possible values
!                             1: string from the list of strings
!                             2: integer constant
!                             3: real constant
!                             4: scalar
!                             5: string constant (between '')
!                             6: integer
!                             7: real
!                             8: vector name
!                          2: depends on type
!                             1: sequence number of string
!                             2: sequence number of integer constant
!                             3: sequence number of real constant
!                             4: sequence number of scalar
!                             5: sequence number of string in array texts
!                             6: value of integer
!                             7: position in rinsep
!                             8: sequence number of vector name
!     istart        i/o   Next free position in inpsep
!     istrin        i/o   Next free position in rinsep
!     rinsep        i/o   The information of the plane is stored in
!                         rinsep from position istrin
!                         Pos. 1   eps
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nparts

!     nparts         number of parts the string consists of
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     READREALSCAL   Read integer, real or scalar
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2143   Sequence number already in use
!    2144   No fluid_groups found
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readscstring' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readscstring'
         write(irefwr,1) 'lastitem, typeitems(lastitem)', &
                          lastitem, typeitems(lastitem)
         write(irefwr,1) 'icode, ityprd', icode, ityprd
         write(irefwr,1) 'index_rdarr(1)', index_rdarr(1)
         write(irefwr,*) 'string ', string(1:nchar)
         call prinin ( typeitems, nitems, 'typeitems' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d16.8) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      inpsep(1) = icode  ! 107 to indicate string = ...
      inpsep(2) = index_rdarr(1)  ! sequence number string (lhs)

      nparts = 0  ! number of parts found

      do while ( ityprd>1 )

!     --- Read next item

         call asknxt
         if ( debug ) then
            write(irefwr,1) 'ityprd', ityprd
            write(irefwr,2) 'realrd', realrd
            write(irefwr,1) 'lastitem/2',lastitem
            write(irefwr,*) 'string ', string(1:nchar)
         end if  ! ( debug )

         select case ( ityprd )

            case(0,1)

!           --- End of record found, exit

               exit

            case(2)

!           --- Integer found

               nparts = nparts+1
               inpsep(2+2*nparts) = 6  ! integer
               inpsep(3+2*nparts) = nint(realrd)

            case(3)

!           --- Real found

               nparts = nparts+1
               inpsep(2+2*nparts) = 7   ! real
               inpsep(3+2*nparts) = istrin
               rinsep(istrin) = realrd
               istrin = istrin+1

            case(4)

!           --- String found, check type of string

               lastitem = lastitem-1  ! reset since it is read again
               call readrealscal
               if ( debug ) then
                  write(irefwr,1) 'ityprd/2', ityprd
                  write(irefwr,1) 'lastitem/3',lastitem
               end if  ! ( debug )

               select case ( ityprd )

               case(12)

!              --- integer found

                  nparts = nparts+1
                  inpsep(2+2*nparts) = 2   ! integer constant
                  inpsep(3+2*nparts) = index_rdarr(1)

               case(13)

!              --- real found

                  nparts = nparts+1
                  inpsep(2+2*nparts) = 3   ! integer constant
                  inpsep(3+2*nparts) = index_rdarr(1)

               case(11, 14)

!              --- scalar found

                  nparts = nparts+1
                  inpsep(2+2*nparts) = 4   ! scalar
                  inpsep(3+2*nparts) = nint(realrd)  ! reference to scalar

               case(15, 16)

!              --- string found

                  nparts = nparts+1
                  inpsep(2+2*nparts) = 1   ! string
                  inpsep(3+2*nparts) = index_rdarr(1)

               case(102)

!              --- vector name found

                  nparts = nparts+1
                  inpsep(2+2*nparts) = 8   ! vector name
                  inpsep(3+2*nparts) = nint(realrd)

               case default

!              --- string not recognized

                  call errchr ( string(1:nchar), 1 )
                  call errsub ( 2144, 0, 0, 1 )
                  call errcrd
                  go to 1000


               end select  ! case ( ityprd )

            case(5)

!           --- String between quotes found

               nparts = nparts+1
               inpsep(2+2*nparts) = 5   ! string
               inpsep(3+2*nparts) = nextxt
               texts(nextxt) = string(1:nchar)
               nextxt = nextxt+1

            case default

!           --- ityprd has wrong value

               call errint ( ityprd, 1 )
               call errchr ( 'ityprd', 1 )
               call errsub ( 2143, 1, 0, 1 )
               call errcrd
               go to 1000

         end select  ! case ( ityprd )

      end do  ! while ( ityprd>1 )
      inpsep(3) = nparts
      istart = istart+3+2*nparts

1000  call erclos ( 'readscstring' )
      if ( debug ) then

!     --- Debug information

         call prinin ( inpsep, 3+2*nparts, 'inpsep' )
         write(irefwr,*) 'End readscstring'

      end if  ! ( debug )

      end
