      subroutine readseqvars ( names, rinput, iinput, istrin, &
                               defvalues, numvals )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-04-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Put the values of the variables read in rinput
!     If no sequence number is found, check if the variables are known
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     variable
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: numvals
      integer, intent(out) :: iinput(numvals)
      integer, intent(inout) :: istrin
      double precision, intent(in) :: defvalues(numvals)
      double precision, intent(inout) :: rinput(*)
      character (len=*) :: names(numvals)

!     defvalues      i    Default values
!     iinput         o    integer input array referring to variable
!                         If value stored in table(icode) iinput contains the
!                         entry in rinput where the value is stored
!                         If the variable is a scalar minus the entry
!                         in scalars is stored in iinput
!     istrin        i/o   Starting address of information for this subroutine
!                         in RINPUT
!                         If a value is stored in rinput, istrin is raised
!     names          i    Names of variables
!                         If no input is found, it is checked if this name
!                         is stored in the set of scalars
!     numvals        i    Number of values to be stored
!     rinput        i/o   Real input array
!                         If a real is read the value is stored in
!                         rinput(istrin)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: m, i, iseq

!     i              Counting variable
!     iseq           Sequence number
!     m              number of parameters read
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: getnamevar, getnameint, getnamereal

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     GETNAMEINT     Return with sequence number of integer in incons
!     GETNAMEREAL    Return with sequence number of real in rlcons
!     GETNAMEVAR     Return with sequence number of variable in scalars
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1533   Number of items read after keyword incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readseqvars' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readseqvars'
         write(irefwr,1) 'icode, numvals', icode, numvals
         do i = 1, numvals
            write(irefwr,*) 'i, names(i) ', i, names(i)
         end do  ! i = 1, numvals
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iposut = nint(table(icode))
      if ( debug ) write(irefwr,1) 'iposut', iposut

      if ( iposut==0 ) then

!     --- No input found, check if names are known

         do i = 1, numvals

            iseq = getnamevar ( names(i) )
            if ( debug ) write(irefwr,1) 'iseq/1', iseq

            if ( iseq>0 ) then

!           --- iseq>0, variable name found

               iinput(i) = iseq+maxints+maxreals+100000000

            else

!           --- iseq = 0, variable name not found
!               Check if it is an integer or real

               iseq = getnameint ( names(i) )
               if ( debug ) write(irefwr,1) 'iseq/2', iseq
               if ( iseq>0 ) then
                  iinput(i) = iseq+100000000  ! integer found
               else
                  iseq = getnamereal ( names(i) )
                  if ( debug ) write(irefwr,1) 'iseq/3', iseq
                  if ( iseq>0 ) then
                     iinput(i) = iseq+maxints+100000000  ! real found
                  end if  ! ( iseq>0 )
               end if  ! ( iseq>0 )

               if ( iseq==0 ) then

!              --- Use default value

                  if ( defvalues(i)>1d9 ) then

!                 --- No default value, fill with 0

                     iinput(i) = 0

                  else

!                 --- No default value, fill with value

                     rinput(istrin) = defvalues(i)
                     iinput(i) = -istrin
                     istrin = istrin+1

                  end if  ! ( defvalues(i)>1d9 )

               end if  ! ( iseq==0 )

            end if  ! ( iseq>0 )

         end do  ! i = 1, numvals

      else

!     --- iposut>0, hence input found

         m = nint(tabmax(icode))
         if ( m/=numvals ) then

!        --- Number of items read is not equal to numvals

            call errchr ( code(icode), 1 )
            call errint ( m, 1 )
            call errint ( numvals, 2 )
            call errint ( numvals, 3 )
            call errsub ( 1533, 3, 0, 1 )
            go to 1000

         end if  ! ( m/=numvals )

         do i = 1, numvals

            if ( infotabout(iposut+i-1)==11 ) then

!           --- variable found

               iinput(i) = nint(tabout(iposut+i-1))+maxints+maxreals

            else

!           --- real found

               rinput(istrin) = tabout(iposut+i-1)
               iinput(i) = -istrin
               istrin = istrin+1

            end if  ! ( infotabout(iposut+i-1)==11 )

         end do  ! i = 1, numvals

      end if  ! ( iposut==0 )

1000  call erclos ( 'readseqvars' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readseqvars'

      end if  ! ( debug )

      end
