      subroutine readstfr ( iinput, rinput, isubr, iseqnr, niinp, nrinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.5  date 13-11-2010 error 1488 --> errkeywexcl
!        version  1.4  date 16-06-2008 Fortran 90
!        version  1.3  date 10-08-2001 Enlarge maxiter
!
!   copyright (c) 1996-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read the input for the stationary free boundary subroutine statfree
!     The input is stored in the arrays iinput and rinput
!     The input must be enclosed between STATIONARY_FREE_BOUNDARY and END.
! **********************************************************************
!
!                       KEYWORDS
!
!     adapt_mesh
!     change_mesh
!     free_boundary
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer niinp, nrinp, iinput(*), iseqnr, isubr
      double precision rinput(*)

!    iinput  o   The integer input read is stored in array IINPUT
!                This array must be filled as follows:
!                1:  MAXITER   Maximum number of iterations allowed
!                2:  IPRINT    Defines the amount of output produced
!                              Possible values:
!                             -1:  All output information is suppressed, &
!                                  even the message of no-convergence
!                              0:  No information about the iteration
!                                  process is printed except in the case
!                                  of an error
!                              1:  The number of iterations performed is
!                                  printed
!                              2:  Some extra information about the iteration
!                                  process is printed for each iteration
!                              3:  See 2, in this case also the ratio of two
!                                  succeeding iterations is printed
!                3:  IRETURN   Indicates if the subroutine must stop if
!                              no convergence has been found (0) or that
!                              control must be given back to the main
!                              program (1)
!                4:  MINITER   Minimum number of iterations allowed
!                5:  ICRITER   Type of criterion to be used for the
!                              termination
!                              Possible values:
!                              0:  Absolute difference between two succeeding
!                                  iterations
!                              1:  Relative difference between two succeeding
!                                  iterations
!                6:  ISEQMESH  Sequence number of adapt mesh to be used
!                7:  INDEX_VECTR   defines the starting address in
!                              IINPUT of the vectors that are used in the update
!                              of the free boundary
!                8:  IWRMESH   If 1 the mesh is written to the file meshoutput
!                              once convergence has occurred.
!                              If 0, the mesh is not written
!                9:  NUMVEC:   Number of vectors used to compute the update
!                              of the free boundary
!    iseqnr i/o  Sequence number with respect to the storage in array ISEPIN
!    isubr   i   Indicates by which subroutine READSTFR is called
!                Possible values:
!                1:  ...      read record with STATIONARY_FREE_BOUNDARY
!                2:  ERSTATFR,  do not read record with STATIONARY_FREE_BOUNDARY
!    niinp  i/o  Length of integer input array
!                At input available length, at output computed length
!    nrinp  i/o  Length of real input array
!                At input available length, at output computed length
!    rinput  o   Contains the real input for the stationary free boundary
!                subroutine
!                At this moment he input is defined as follows
!                1:  accuracy eps
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode1=10, nncod2=4
      integer ncode1(nncode1), ncode2(nncod2), itype1(nncode1), ncodmx1(nncode1)
      integer k, m, lastinp
      double precision :: tabmin1(nncode1), tabmax1(nncode1)
      character (len=15) code1(nncode1), code2(nncod2)

!     code1          Table with keywords corresponding to options
!     code2          Table with the keywords following the subkeywords
!     itype1         itype array for code1  (See intx02)
!     k              Counting variable
!     lastinp        Last position filled in array iinput
!     m              number of integers read
!     ncode1         Table containing the number of significant characters in
!                    the keyword table code1
!     ncode2         See ncode, now for table code2
!     ncodmx1        Table containing the number of maximum characters in
!                    the keyword table code1
!     nncod2         Number of entries in table code2
!     nncode1        Number of entries in table code1
!     tabmax1        Table with maximum values of numbers corresponding to code1
!     tabmin1        Table with minimum values of numbers corresponding to code1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRKEYWEXCL    Print error message concerning two keywords that are
!                    mutually exclusive
!     ERRSUB         Error messages
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYCODESUB Copy codemn2 and related arrays into codesub
!     SEPCOPYTYPE    Copy itype2 and related arrays into itype
!     SEPREADSEQNR   Read sequence number iseqnr from record
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
!     SEPSETMAINEND  Fill keyword END as first main keyword and set nmain to 1
! **********************************************************************
!
!                       I/O
!
!     READSTFR requires input from the standard input file
!     The following input is recognized:
!     STATIONARY_FREE_BOUNDARY [, SEQUENCE_NUMBER = s] (mandatory):
!         start of input
!      optional records:
!          maxiter = m      (Default 20)
!          miniter = m      (Default 2)
!          accuracy = eps   (Default 1d-3)
!          print_level = p  (Default 0)
!          criterion = c    (Default abs)
!          adapt_mesh = i   (Default value 1)
!          at_error = e     (Default stop)
!          seq_vectors = (i1,i2,i3...)     (Default 1)
!          write_mesh       (Default write)
!          no_write_mesh    (Default write)
!                      Possible values for c:   absolute
!                                               relative
!                      Possible values for e:   stop
!                                               return
!     END   (mandatory):  end of input
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     329   Array to store the input too small
!     550   array length too short
!     650   First record read by subroutine is not equal to STATIONARY_FREE
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Initialize arrays and parameters with default values
!    If ( isubr=1 ) then
!       Read first record
!       Analyse the first word (stationary_free_boundary)
!    Read rest of first record (sequence number)
!    Read the rest of the records by intx02 until END is found
!    set niinp and nrinp to final values
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code1 / 'MAXITER', 'MINITER', 'PRINT_LEVEL', 'ACCURACY', &
                  'CRITERION', 'ADAPT_MESH', 'AT_ERROR', &
                  'SEQ_VECTORS', 'WRITE_MESH', 'NOWRITE_MESH' /
      data ncode1  / 3, 3, 5, 3, 4, 5, 5, 6, 9, 11 /
      data ncodmx1 / 7, 7,11, 8, 9,10, 8,11,10, 12 /
      data itype1  / 0, 0, 0, 1, 5, 0, 5,11, 2,  2 /
      data tabmin1 / 1d0, 1d0,-1d0, 0d0, 1d0, 1d0, 3d0, 1d1, 0d0, 0d0 /
      data tabmax1 / 1d4, 1d4, 3d0, 1d1, 2d0, 1d1, 4d0, 1d2, 0d0, 0d0 /
      data code2 / 'ABSOLUTE', 'RELATIVE', 'STOP', 'RETURN' /
      data ncode2 / 3, 3, 3, 3 /

!     code1    Table with keywords corresponding to options
!             Values:
!             1:  MAXITER
!             2:  MINITER
!             3:  PRINT_LEVEL
!             4:  ACCURACY
!             5:  CRITERION
!             6:  ADAPT_MESH
!             7:  AT_ERROR
!             8:  SEQ_VECTORS
!             9:  WRITE_MESH
!            10:  NO_WRITE_MESH
!     code2   Table with the keywords following the subkeywords
!             Values:
!             1:  ABSOLUTE
!             2:  RELATIVE
!             3:  STOP
!             4:  RETURN
! ======================================================================
!
!     --- Initialize arrays and parameters

      call eropen ( 'readstfr' )

      lastinp = 10
      if ( niinp<lastinp ) then

!     --- niinp too small

         call errint ( lastinp, 1 )
         call errint ( niinp, 2 )
         call errsub ( 329, 2, 0, 0 )
         go to 1000

      end if
      if ( nrinp<1 ) then

!     --- nrinp too small

         call errint ( 1, 1 )
         call errint ( nrinp, 2 )
         call errsub ( 329, 2, 0, 0 )

      end if
      iinput(1:niinp) = 0
      iinput(1) = 20
      iinput(4) =  2
      iinput(6) =  1
      iinput(7) = lastinp
      iinput(8) =  1
      iinput(9) =  1
      iinput(lastinp) =  1
      rinput(1) =  1d-3

      if ( isubr==1 ) then

!     --- Call by STATFREE, read first record  (STATIONARY_FREE_BOUNDARY)

         ityprd = 1
         call asknxt
         if ( ityprd/=4 ) then
            call errchr ( 'STATIONARY_FREE_BOUNDARY', 1 )
            call errsub ( 650, 0, 0, 1 )
            call errcrd
            go to 1000
         else if ( string(1:4)/='STAT' ) then
            call errchr ( 'STATIONARY_FREE_BOUNDARY', 1 )
            call errsub ( 650, 0, 0, 1 )
            call errcrd
            go to 1000
         end if

      end if

!     --- Read rest of record

      call sepreadseqnr(iseqnr)


      call sepcopycode ( code1, nncode1, ncode1, ncodmx1 )
      call sepcopytype ( nncode1, itype1, tabmin1, tabmax1 )
      call sepcopycodesub ( code2, nncod2, ncode2 )
      call sepsetmainend

      table(1:nncode) = -1d20
      tabmax(1) = 1d4
      istop = 2
      iretur = 0
      call sepreadsetkeyw
      istop = 0
      if ( table(1)>0d0 ) iinput(1) = nint(table(1))
      if ( table(2)>0d0 ) iinput(4) = nint(table(2))
      if ( table(3)>0d0 ) iinput(2) = nint(table(3))
      if ( table(4)>0d0 ) rinput(1) = table(4)
      if ( table(5)>0d0 ) iinput(5) = nint(table(5))-1
      if ( table(6)>0d0 ) iinput(6) = nint(table(6))
      if ( table(7)>0d0 ) iinput(3) = nint(table(7))-1
      if ( table(8)>0d0 ) then

!     --- iseq_vectors found, store number of vectors in iinput(11) and
!         sequence numbers from position lastinp

         iposut = nint(table(8))
         m = nint(tabmax(8))
         iinput(9) = m
         if ( lastinp+m-1>niinp ) then
            call errint ( lastinp+m-1, 1 )
            call errint ( niinp, 2 )
            call errchr ( 'iinput', 1 )
            call errsub ( 550, 2, 0, 1 )
            go to 1000
         end if
         do k = 0, m-1
            iinput(lastinp+k) = nint(tabout(iposut+k))
         end do
         lastinp = lastinp+m-1

      end if
      if ( table(10)>0d0 ) iinput(8) = 0
      if ( table(9)>0d0 .and. table(10)>0d0 ) &
         call errkeywexcl ( code, ncodmx, 9, 10 )

      niinp = lastinp

1000  call erclos ( 'readstfr' )

      end subroutine readstfr
