      subroutine readtmtheta ( iintim, ndt, rintim, leniintim, istrin, method )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-10-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill information concerning theta in iintim/rintim
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     time_integration
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: method, ndt
      integer, intent(out) :: leniintim
      integer, intent(inout) :: istrin, iintim(*)
      double precision, intent(inout) :: rintim(*)

!     iintim        i/o   Contains the input for time integration
!                         See subroutine readtm1
!     istrin        i/o   Starting address in array RINSEP
!     leniintim     i/o   Computed length of array iintim
!     method         i    Type of time integration method
!     ndt            o    Number of different time steps
!     rintim        i/o   Contains real input for time integration
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, nfrac, iseq
      double precision :: alpha, beta, theta

!     alpha          Parameter alpha for generalized theta method
!     beta           Parameter beta for generalized theta method
!     i              Counting variable
!     iseq           Sequence number of scalar
!     nfrac          Number of fractional steps
!     theta          Parameter theta for generalized theta method
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: sepgetnamevar

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     READSERVAR     Store numitems reals or variables in iinput
!     SEPGETNAMEVAR  Get sequence number of scalar, real or integer
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1589   Too much thetas read
!    1590   Number of thetas read not even
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readtmtheta' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readtmtheta'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( table(4)>0d0 ) then
         nfrac = nint(tabmax(4))
         iposut = nint(table(4))
         if ( debug ) write(irefwr,1) 'nfrac, iposut', nfrac, iposut
      else
         nfrac = 0
      end if  ! ( table(4)>0d0 )

      leniintim = leniintim+1
      iintim(53) = leniintim  ! start information theta

      if ( method==9 ) then

!     --- Theta method

         if ( nfrac>ndt ) then
            call errint ( nfrac, 1 )
            call errint ( ndt, 2 )
            call errsub ( 1589, 2, 0, 0 )
         end if

         if ( nfrac>0 ) then

            call readservar ( table(4), iintim(leniintim), rintim, nfrac, &
                              istrin )
            leniintim = leniintim+nfrac-1

         else

            iseq = sepgetnamevar ( 'theta' )
            if ( iseq>maxints+maxreals ) iseq = iseq+100000000  ! scalar found
            if ( iseq>0 ) then

!           --- iseq>0, variable name found

               iintim(leniintim) = iseq

            else

!           --- iseq=0, use default

               iintim(leniintim:leniintim+ndt-1) = -istrin
               rintim(istrin) = 0.5d0
               istrin = istrin+1

            end if  ! ( iseq>0 )
         end if
         nfrac = 1

      else if ( method==10 .or. method==12 ) then

!     --- fractional step or generalized theta

         if ( nfrac>10 ) then
            call errint ( nfrac, 1 )
            call errint ( 10, 2 )
            call errsub ( 1589, 2, 0, 0 )
         end if

         if ( mod(nfrac,2)/=0 ) then

!        --- Not an even number of thetas given

            call errint ( nfrac, 1 )
            call errsub ( 1590, 1, 0, 0 )

         end if

         if ( nfrac>0 ) then

            call readservar ( table(4), iintim(leniintim), rintim, nfrac, istrin )

         else if ( method==10 ) then

!        --- Default values for fractional step method

            nfrac = 6
            theta = 1d0-0.5d0*sqrt(2d0)
            alpha = (1d0-2d0*theta)/(1d0-theta)
            beta  = theta/(1d0-theta)
            rintim(istrin) = beta*theta
            rintim(istrin+1) = alpha*theta
            rintim(istrin+2) = alpha*(1d0-2d0*theta)
            rintim(istrin+3) = beta*(1d0-2d0*theta)
            rintim(istrin+4:istrin+5) = rintim(istrin:istrin+1)

            do i = 0, 5
               iintim(leniintim+i) = -istrin-i
            end do  ! i = 0, 5
            istrin = istrin+6

         else

!        --- Default values for generalized theta method

            nfrac = 6
            beta  = sqrt(3d0)
            alpha = 1d0/(1d0+2d0/beta)
            rintim(istrin) = alpha*0.5d0
            rintim(istrin+1) = alpha*beta/6d0
            rintim(istrin+2) = 0d0
            rintim(istrin+3) = alpha*beta/3d0
            rintim(istrin+4:istrin+5) = rintim(istrin:istrin+1)

            do i = 0, 5
               iintim(leniintim+i) = -istrin-i
            end do  ! i = 0, 5
            istrin = istrin+6

         end if
         leniintim = leniintim+nfrac-1

      else

         if ( nfrac>0 ) then
            call errint ( nfrac, 1 )
            call errint ( 1, 2 )
            call errsub ( 1589, 2, 0, 0 )
         end if
         nfrac = 0

      end if
      iintim(17) = nfrac

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readtmtheta'

      end if  ! ( debug )
      call erclos ( 'readtmtheta' )

      end subroutine readtmtheta
