      subroutine sepenthalpy ( finished, iinput, rewind, iprint, &
                               iinenthalpy, rinenthalpy, changestep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-11-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Subroutine to be solve the time dependent temperature equation with
!    freezing front by a enthalpy method
!    It performs one time-step
!    This is a very special subroutine in the SEPRAN context, since it
!    assumes that the user has made a loop himself
!    sepenthalpy should be called in this loop
!    It returns with a value for finished indicating if the loop may be finished
!    or not.
!    The following tasks are performed:
!    -  In first call: set time integration constants
!    -  Perform one time step
!    -  In last time step: set finished and reset first
!    In contrast to adapmesh input is only possible through the input file
!    and not through the parameter list
!    Warning:  Experimental version
!              Both the parameter list as well as the input may change
!              Check this in each new update of SEPRAN until this
!              subroutine is part of the manuals
!     Old subroutine prenthalpy
! **********************************************************************
!
!                       KEYWORDS
!
!     free_boundary
!     time_dependent
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesol
      use sepmodulemnsub
      use sepmodulemain
      use sepmoduleintmat
      use sepmoduletime
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iinput(*), iinenthalpy(*)
      integer, intent(inout) :: iprint
      double precision, intent(in) :: rinenthalpy(*)
      logical, intent(in) :: rewind
      logical, intent(inout) :: finished
      logical, intent(out) :: changestep

!     changestep     o    If true the time step may be changed
!     finished      i/o   Indicates if the loop has been finished (true) or not
!                         (false)
!     iinenthalpy    i    Integer input array with respect to enthalpy
!                         computations
!                         For a definition of this array see subroutine
!                         readenthalpy (array iinput)
!     iinput         i    Integer input array with the following contents
!                         1: IREAD Defines how the input must be read
!                            Possible values:
!                            0   The input is read from the standard input file
!                            >0  The input as read from the standard input file
!                                by subroutine SEPSTM is used. The value of
!                                iread defines the sequence number
!     iprint         i    Print level, 0 is no printing
!     rewind         i    When true, the current time level is rewinded
!                         to a previous time level
!     rinenthalpy    i    Real input array with respect to enthalpy
!                         computations
!                         For a definition of this array see subroutine
!                         readenthalpy (array iinput)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iseqintmat, iseqtemp, isubr, iseqnr
      double precision :: tstepsav
      logical :: first
      save first, iseqintmat

!     first          If true it concerns the first step in a loop
!     iseqintmat     Sequence number with respect to array intmat
!     iseqnr         Sequence number with respect to the storage in array
!                    ISEPIN
!     iseqtemp       Sequence number of temperature in solution vector
!     isubr          Indicates the type of calling subroutine
!     tstepsav       Temporary copy of tstep
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     SEPTIMEFR02    Adapt time parameters at end of time step for timefrbf
!     SEPTIMEFR03    solves time-dependent enthalpy problem
!     SEPTIMEHIS4    Fill time step for time history
!     SEPTIMSEQINTM  Fill array intmat if necessary and initialize iseqintmat
! **********************************************************************
!
!                       I/O
!
!   Depending on the values in IREAD input may be read from the standard
!   input file by subroutine READFRST
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    if ( first call of loop ) then
!       Perform standard tests
!       Fill time integration arrays
!       Set t, t0, tend and so on
!    Perform one time step
!    Raise time t
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data first /.true./
! ======================================================================
!
      call eropen ( 'sepenthalpy' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepenthalpy'
         call prinin ( iinenthalpy, 10, 'iinenthalpy' )
         call prinrl ( rinenthalpy, 10, 'rinenthalpy' )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      numvecmn = max(numvecmn,iinenthalpy(2))
      iprint = iinenthalpy(8)

!     --- Temporarily save a copy of tstep. This is done to compensate
!         for the fact that in case of multiple time loops, tstep
!         is reset to the initial value given in the prb by subroutine
!         prtimefro1 for subsequent time loops

      tstepsav = tstep

!     --- Put parameters in iintim

      iseqnr = iinenthalpy(4)
      iseqtemp = max(1,iinenthalpy(3))

!     --- Fill iseqintmat and intmat if necessary

      iprobtim = ks(isolut2d(iseqtemp))%iprob
      call septimseqintm ( iseqintmat )

!     --- correct time step size at end when needed

      tstep = tstepsav

      if ( t + tstep>tend ) then

         if ( debug ) then
            write(irefwr,2) 't, tstep, tend', t, tstep, tend
            write(irefwr,*) 'Correcting t to end at tend'
         end if  ! ( debug )

         tstep = tend - t
         if ( debug ) write(irefwr,2) 'tstep', tstep

      end if

!     --- Actual call to time subroutine

      isubr = 3
      intmat => intmatar(iseqintmat)
      call septimefr03 ( finished, first, isubr, iinput(1), &
                         iinenthalpy, rinenthalpy, rewind, changestep )
      if ( ierror/=0 ) go to 1000

!     --- Adapt time parameters

      call septimefr02

!     --- write time step to time history

      call septimehis4

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'rewind', rewind
         write(irefwr,*) 'End sepenthalpy'

      end if  ! ( debug )
      call erclos ( 'sepenthalpy' )

      end
