      subroutine sepenthalpymn ( finished, iinput )
! ======================================================================
!
!        programmer    John Brusche
!        version  1.0  date 12-11-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Wrapper routine for sepenthalpy. Added feature is the time step
!     control.
!     Old subroutine prenthalpymain
! **********************************************************************
!
!                       KEYWORDS
!
!     free_boundary
!     time_dependent
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemnsub
      use sepmodulemain
      use sepmodulevecs
      use sepmoduletime
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iinput(*)
      logical, intent(inout) :: finished

!     finished      i/o   Indicates if the loop has been finished (true) or not
!                         (false)
!     iinput         i    Integer input array with the following contents
!                         1: IREAD Defines how the input must be read
!                            Possible values:
!                            0   The input is read from the standard input file
!                            >0  The input as read from the standard input file
!                                by subroutine SEPSTM is used. The value of
!                                iread defines the sequence number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: niinpent=10, nrinpent=10
      integer :: irewind, iprint, ioutactsav, iseqnr, &
                 iinenthalpy(niinpent), iseqtime, leninp, lenrnp, &
                 isetctimen
      double precision :: tstep_user, toutnextsav, rinenthalpy(nrinpent)
      logical :: rewind, first, changestep, again
      save tstep_user, irewind, iinenthalpy, rinenthalpy, first, &
           isetctimen

!     again          If true repeat step
!     changestep     If true the time step may be changed
!     first          If true it concerns the first step
!     iinenthalpy    Integer input array with respect to enthalpy
!                    computations
!                    For a definition of this array see subroutine
!                    readenthalpy (array iinput)
!     ioutactsav     Help variable to store the value of ioutact
!     iprint         Print level, 0 is no printing
!     irewind        Counting variable that keeps tracks of the number of
!                    time steps that is taken for the current time step size
!     iseqnr         Sequence number with respect to the storage in array
!                    ISEPIN
!     iseqtime       Sequence number for input time integration
!     isetctimen     Value of parameter use_ctimen at start of subroutine
!     leninp         Computed length of integer input array
!     lenrnp         Computed length of real input array
!     niinpent       Declared length of array iinenthalpy
!     nrinpent       Declared length of array rinenthalpy
!     rewind         When true, the current time level is rewinded
!                    to a previous time level
!     rinenthalpy    Real input array with respect to enthalpy
!                    computations
!                    For a definition of this array see subroutine
!                    readenthalpy (array iinput)
!     toutnextsav    Help variable to store the value of toutnext
!     tstep_user     Initial and maximum time step size as set by the user
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     IINPUTSEQ      Read contents of input block with sequence number iseqnr
!                    corresponding to main keyword name
!     PRENTHPLOTMSH  Plot the mesh. Nodal point numbers are shown.
!     PRININ         print 1d integer array
!     PRTIMENTHALADP Adapt time step for enthalpy integration if necessary
!     SEPENTHALPY    Solve the time dependent temperature equation
!                    with  freezing front by a enthalpy method
!     SEPFILLTIMINP  Fill module sepmoduletime with information stored in
!                    inpsep/rinsep and ibuffr
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Standard tests

      call eropen ( 'sepenthalpymn' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepenthalpymn'
         call prinin ( isolut2d, 10, 'isolut' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if
      if ( ierror/=0 ) go to 1000

      if ( itimloop<=1 ) then

!     --- New time loop, set first

         first = .true.

!        --- Read input for enthalpy problem and store in arrays
!            iinenthalpy, rinenthalpy

         iseqnr = iinput(1)
         leninp = niinpent
         lenrnp = nrinpent
         call iinputseq ( 26, iseqnr, iinenthalpy, &
                          rinenthalpy, leninp, lenrnp, &
                          'ENTHALPY_INTEGRATION' )
         iseqtime = iinenthalpy(4)  ! sequence number time integration

         if (debug) write(irefwr,1) 'iseqtime', iseqtime

      else

         first = .false.

      end if

      if (debug)  write(irefwr,*) 'itimloop, first ', itimloop, first

!     --- Save some parameters for the case of rewinding

      ioutactsav = ioutact
      toutnextsav = toutnext

!     --- Save copy of initial tstep

      if (first) then

!     --- first is true, i.e. first time step
!         We change the time integration input such that it is set to
!         use_ctimen
!         In this way the time step may be changed in prenthalpy

         if (debug) call prenthplotmsh

         itiminterval = 1
         call sepfilltiminp ( iseqtime, 3, iinenthalpy(3) )
         tstep = arrtimestep(1)
         usectimen = .true.

         if ( debug ) write(irefwr,2) 'tinit, tstep, tend', t0, tstep, tend
         first = .false.

      end if
      numoldtim = max(1,numvecmn)
      tstep_user = tstep

!     --- Save a copy of the old temperature, so we can revert to it in case
!         we need to go back one time step

      allocate ( uold(ks(isolut2d(1))%nusol) )
      usol => ks(isolut2d(1))%sol
      uold = ks(isolut2d(1))%sol

  !    call sepcopyvc ( isolut2d(1), tempold )

666   rewind = .false.
      complrun = .false.

!     --- correct time step size at end when needed

      call sepenthalpy ( finished, iinput, rewind, iprint, iinenthalpy, &
                         rinenthalpy, changestep )
      if ( changestep ) then

!     --- changestep = true, time steps may be changed

         call prtimenthaladp ( rewind, iprint, tstep_user, &
                               finished, irewind, ioutactsav, &
                               toutnextsav, isolut2d, again )
         if ( again ) go to 666

      end if  ! ( changestep )

!     --- Remove array

      deallocate ( uold )

      itimloop = 2
      if ( finished ) then

!     --- finished is true, i.e. end of time loop
!         reset time integration parameter use_ctimen to old value

         usectimen = .false.
         itimloop = 0

      end if  ! ( finished )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepenthalpymn'

      end if
      call erclos ( 'sepenthalpymn' )

      end
