      subroutine sepextend ( inpsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 02-08-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy mesh corresponding to axi-symmetric problem onto complete
!     mesh defined on both sides of the symmetry axis
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     mesh
!     2d
!     symmetric
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmoduleinput
      use sepmodulekmesh
      use sepmodulesymm
      use sepmodulemnsub
      use sepmodulemain
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: inpsep(*)

!     inpsep         i    See subroutine readscextend
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: imesh1
      integer :: iseqprob

!     imesh1         sequence number of input mesh
!     iseqprob       Sequence number of problem input
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PROBDFEXT      Reads and stores problem definition
!     SEPCOPCOORSYMM Copy coordinates of axi-symmetric mesh and fill a part
!                    of new kmesh
!     SEPCOPCURVESYM Copy curves of axi-symmetric mesh and fill a part
!                    of new kmesh
!     SEPCOPKMESH    Delete old kmesh2 if it exists and copy essential
!                    information from kmesh1 to kmesh2
!     SEPCOPKMESHBUF Copy new arrays into ibuffr
!     SEPCOPTOPOLSYM Copy topology of axi-symmetric mesh and fill a part
!                    of new kmesh
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPSYMMNODES   Find all nodes on symmetry axis and store in listsymm
!                    if any
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3217   copying of axy-symmetric mesh only possible for 2d meshes
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Find which nodes are positioned on the symmetry axis
!     Copy coordinates of all other nodes
!     Create new topology
!     Create new curves
!     Fill new kprob
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepextend' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepextend'
         call prinin ( inpsep, 2, 'inpsep' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      imesh1 = inpsep(1)
      meshsymm = inpsep(2)

!     --- Activate input mesh and store information in sepmodulekmesh

      kmesh => kmesh2d(imesh1)
      kprob => kprob2d(imesh1)
      call sepfillkmesh ( kmesh )

      if ( ndim/=2 ) then

!     --- Error: copying of axy-symmetric mesh only possible for 2d meshes

         call errint ( ndim, 1 )
         call errsub ( 3217, 1, 0, 0 )
         go to 1000

      end if  ! ( ndim/=2 )

!     --- Delete contents of old kmesh2 if it exists
!         and copy some information of old kmesh

      call sepcopkmesh ( kmesh, kmesh2d(meshsymm) )

!     --- Find which nodes are positioned on the symmetry axis

      call sepsymmnodes

!     --- Copy coordinates of all other nodes

      kmesh => kmesh2d(meshsymm)
      iseqkm = kmesh
      call sepcopcoorsymm

!     --- Create new topology

      call sepcoptopolsym

!     --- Create new curves

      call sepcopcurvesym

!     --- Copy new arrays in ibuffr

      call sepcopkmeshbuf

      call sepfillkmesh ( kmesh )

!     --- Fill new kprob

      iseqkp = kprob
      iinputtmp => kp(iseqkp)%iinputkp
      rinputtmp => kp(iseqkp)%rinputkp

      iseqprob = 1
      kprob => kprob2d(meshsymm)
      call probdfext ( 1, iseqprob )

      kmesh =>  kmesh2d(imesh1)
      call sepfillkmesh ( kmesh )

      deallocate ( iworksymm, iworkcur, listsymmc, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iworksymm' )

      if ( numsymm>0 ) deallocate ( listsymm )

      istartmn = istartmn+3

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepextend'

      end if  ! ( debug )
      call erclos ( 'sepextend' )

      end subroutine sepextend
