      subroutine sepmatsol ( iinsol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-12-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     constructs the matrix and right-hand side for a linear problem and
!     solves it
!     Old subroutine to0089
! **********************************************************************
!
!                       KEYWORDS
!
!     linear
!     linear_solver
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulematr
      use sepmodulesmf
      use sepmodulemnsub
      use sepmodulemain
      use sepmodulesolve
      use sepmodulecons
      use sepmodulekprob
      use sepmoduleplot
      use sepmoduledebug
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinsol(*)

!     iinsol         i    Contains information concerning the linear solver
!                         See the Programmers Guide for a description
!                  With this array the user may indicate which actions
!                  he wants to be performed by this subroutine
!                  IINSOL must be filled as follows:
!                  Pos. 1: isubr
!                       2: IREAD Defines how the input must be read
!                                Possible values:
!                                0:  All SEPRAN input has been read by
!                                    subroutine SEPSTN until END_OF_SEPRAN_INPUT
!                                    or end of file has been found
!                                    This input is used
!                                1:  The input is read as described for LINSOL
!                                2:  The last input read as described for LINSOL
!                                    is used (equivalent with LINSTM ICTIME=1)
!                          Default value: 0
!                       3: ISEQCF, This parameter is only used if IREAD=0
!                          It indicates the sequence number of the input
!                          for the coefficients to be used
!                          Default value: 1
!                       4: ISEQSL, This parameter is only used if IREAD=0
!                          It indicates the sequence number of the input
!                          for the linear solver to be used
!                          Default value: 1
!                       5: IPROB, Problem sequence number
!                          Default value: 1
!                       6: IVECTR, sequence number of solution vector with
!                          respect to array isolut
!                          Default value: iprob
!                       7: maxvc1, Maximum number of vectors stored in isolut
!                          Default value: 1
!                       8: IDEF_COR   indicates if defect correction should be
!                                     used
!                          0: no defect correction
!                          1: defect correction
!                          Default value: 0
!                       9: LINEAR_SUB indicates if quadratic elements should be
!                          treated as a cluster of linear subelements
!                          0: no subelements
!                          1: subelements
!                          Default value: 0
!                      10: IVEC_REACT indicates if reaction forces must be
!                          computed (>0) or not (0)
!                          If >0, IVEC_REACT defines the sequence number of the
!                          vector in which the reaction forces must be stored
!                          Default value: 0
!                      11: IVEC_FEM is only used in case of a spectral mesh
!                          It indicates if also the fem solution vector
!                          must be stored besides the sem solution vector
!                          if>0, IVEC_FEM defines the sequence number
!                          of the fem solution vector
!                          if<0, it has a special meaning
!                          In this case isolutut(1,-IVEC_FEM) contains a
!                          right-hand side vector that must be added to an
!                          existing rhs vector
!                          Default value: 0
!                      12: Indicates if a FEM preconditioning must be applied
!                          in case of a spectral mesh
!                          Possible values:
!                          0: no fem preconditioning
!                          1: fem preconditioning
!                          Default value: 0
!                      13: KEEP indicates if the matrix and right-hand side
!                          must be kept, destroyed or reused
!                          Possible values:
!                          0: new and destroy
!                          1: new and keep
!                          2: old and destroy
!                          3: old and keep
!                      14: IVEC_REACT_SP indicates if reaction forces must be
!                          computed (>0) or not (0)
!                          The difference with 10 is that maver is called with 9
!                          instead of 10. This implies that the vector must have
!                          been filled before and moreover that matrix times
!                          solution without the effect of essential boundary
!                          conditions are stored. The rest of the vector is
!                          unchanged
!                          If >0, IVEC_REACT_SP defines the sequence number of
!                          the vector in which the reaction forces must be
!                          stored
!                      15: Is only used if iinsol(10)>0, hence if reaction
!                          forces are computed
!                          If 0, no action is carried out
!                          Possibility 1 is meant for the special case of
!                          slip boundary conditions where a part of the
!                          slip is stored in a boundary matrix
!                          This part multiplied by the solution must be
!                          subtracted from the reaction force to get the
!                          real reaction force
!                      16: KEEP_MAT indicates if the matrix
!                          must be kept, destroyed or reused
!                          Possible values:
!                          0: new and destroy
!                          1: new and keep
!                          2: old and destroy
!                          3: old and keep
!                      17: KEEP_RHS indicates if the right-hand side
!                          must be kept, destroyed or reused
!                          Possible values:
!                          0: new and destroy
!                          1: new and keep
!                          2: old and destroy
!                          3: old and keep
!                      18: iseq_test_force sequence number of test force
!                          vector. If 0 no test force is computed
!                      19: iseq_rhs sequence number of right-hand side
!                          vector. If 0 the standard vector is used
!                      20: iseq_intmat1d, sequence number of intmat1d entry
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision rinsol1(nrinsol)
      integer ictime, inpsol1(ninpsol), i
      save inpsol1, rinsol1

!     i              Counting variable
!     ictime         Indication if information must be read from standard
!                    input file ( 0 ) or that information read before is
!                    used ( 1 )
!     inpsol1        Contains integer information concerning the solution
!                    process for subroutine SOLVEL.
!                    If iread = -2, inpsol1 is filled by SOLVEL, if iread = -1
!                    it is used
!     rinsol1        Array containing the real user input if iread=-1
!                    The storage of rinsol1 depends on iinsol.
!                    The following positions are used
!                    1. eps      Required accuracy for the iteration process
!                    2. droptol  Drop tolerance with respect to Y12M
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPBUILD       Build matrix and right-hand side
!     SEPCOPYVC      Copy one SEPRAN vector into another one
!     SEPFILLCOEFFS  Fill coefficients at the end of the buffer array
!                    with starting addresses ipiusr and ipuser
!     SEPFILLIINBLD  Extract parameters from array iinsol
!                    Make relation between SMF and arrays
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPLINSOL      Body of linear solver
!     SEPLINSOLKPROB Fill relevant parameters into sepmodulekprob for
!                    subroutine linsol
!     SEPRMMATRIX    Remove array with sequence number mmindex from buf
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
!     SEPSOLVEINPUT  Fill arrays inpsol and rinsol
!     TO0089REAC     Compute the reaction forces in the linear system of
!                    equations
!     TO0089SEM      Compute SEM solution by FEM preconditioning
!     TO0089TEXT1    Write some text information for subroutine to0089 (solve)
!                    depending on output parameters
!     TOCOMPTSTFORCE Compute test force in outer boundary
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepmatsol' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepmatsol'
         call prinin ( iinsol, 20, 'iinsol' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill coefficients in IUSER and USER

      if ( debug ) write(irefwr,*) 'before sepfillcoeffs'

      iprob = max(1,iinsol(5))
      kmesh => kmesh2d(meshnr)
      call sepfillkmesh(kmesh)

      call sepfillcoeffs ( iinsol(1), iinsol(3), iinsol(20) )
      if ( ierror/=0 ) go to 1000

!     --- Store information of iinsol into SMF
!         It concerns iinbld and references to all arrays

      call sepfilliinbld ( iinsol )

!     --- Actual building of system of equations
!         In case of local transformations both the matrix and
!         right-hand side correspond to the local coordinate system

      if ( debug ) write(irefwr,*) 'before sepbuild'
      call sepbuild
      if ( ierror/=0 ) go to 1000
      stiffmat => stiffm(iprob:maxprb)
      if ( debug ) call prinin ( stiffmat, 2, 'matr/1' )
      kmesh => kmesh2d(meshnr)
      call sepfillkmesh(kmesh)

      if ( iinsol(18)>0 ) then

!     --- iinsol(18)>0, hence right-hand side is stored in isolut
!         we copy the vector into irhsd

         call sepcopyvc ( isolut2d(iinsol(18)), irhsd )

      end if  ! ( iinsol(18)>0 )

!     --- Print some information and fill iread for solver

      if ( debug ) write(irefwr,*) 'before to0089text1'
      call to0089text1 ( iinsol )

      if ( iinsol(11)<0 ) then

!     --- Special case: iinsol(11)<0, add extra rhsd to rhsd
!         Both correspond to the local coordinate system

         rhsd => ks(irhsd(1))%sol
         rhsdsp => ks(isolut2d(-iinsol(11)))%sol
         rhsd = rhsdsp + rhsd

      end if  ! ( iinsol(11)<0 )

!     --- Solve system of linear equations
!         In case of a spectral method, the solution of the finite element
!         system is computed which is used later on as start vector for the
!         spectral preconditioning
!         In case of local transformations, the output solution and reaction
!         force are in the Cartesian coordinate system, not in the local one

      if ( debug ) write(irefwr,*) 'before solvelbf'

      if ( irhsd(1)>0 ) then

!     --- There is right-hand side: solve

         inpsol1(1) = ninpsol
         rinsol1(1) = dble(nrinsol1)
         intmat => intmat1d(iseqintmat)
         iseqmt = max(stiffmat(1),1)
         call sepsolveinput ( ireadinpsol, inpsol1, rinsol1 )
         if ( iinsol(12)>0 ) keep = 1
         call seplinsolkprob ( irhsd )
         call seplinsol ( stiffmat, isolut2d, irhsd )

      else

!     --- Fill reaction forces, depending on iinsol

         if ( debug ) write(irefwr,*) 'before to0089reac'
         call to0089reac ( iinsol, iseqsol, &
                           iinbld, iseqreac, iinsol(11) )

      end if  ! ( irhsd(1)>0 )
      if ( ierror/=0 ) go to 1000

      if ( iinsol(11)>0 ) then

!     --- Fill FEM vector

         write(irefwr,200) iinsol(11)
200      format( ' Compute vector ', i3, ' as FEM solution vector' )
         call sepcopyvc ( isolut2d(iseqsol), isolut2d(iinsol(11)) )

      end if
      if ( ierror/=0 ) go to 1000

      if ( iinsol(12)>0 ) then

!     --- iinsol(12)>0, Compute SEM solution by FEM preconditioning

         keep = 2
         call to0089sem ( iseqsol, iinsol(4) )

      end if
      if ( ierror/=0 ) go to 1000

      if ( iinsol(17)>0 ) then

!     --- Special option to compute the test force
!         First copy solution array to temporary array and set all
!         boundary values to zero
!         Then replace the solution at the outer boundary by 0
!         Finally compute the test force

         call tocomptstforce ( iinsol, iseqsol, iinbld, iinsol(11) )

      end if  ! ( iinsol(17)>0 )

!     --- Delete arrays matr and irhsd

      ictime = iinsol(20)

      if ( iinsol(15)==2 .or. iinsol(15)==0 .and. &
           ( ictime<2 .or. ictime>4 .and. ictime/=6 ) ) then
         if ( stiffm(iprob)/=0 ) call seprmmatrix ( stiffm(iprob) )
         if ( debug ) write(irefwr,*) 'Destroy matrix'
      end if  ! ( iinsol(15)==2 .or. iinsol(15)==0 .. )

      if ( iinsol(16)==2 .or. iinsol(16)==0 .and. &
           ( ictime<2 .or. ictime>4 .and. ictime/=6 ) ) then
         if ( irhsd(1)>0 ) &
            call seprmsolinfo ( irhsd(1) )
         if ( debug ) write(irefwr,*) 'Destroy right-hand side'
      end if  ! ( iinsol(16)==2 .or. iinsol(16)==0 .. )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepmatsol'

      end if  ! ( debug )
      call erclos ( 'sepmatsol' )

      end subroutine sepmatsol
