      subroutine sepnavstokes ( inpsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 30-11-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Solve the time dependent Navier-Stokes equations
!     Only one time-step is carried out
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     navier_stokes_equation
!     time_integration
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemnsub
      use sepmodulemain
      use sepmoduletime
      use sepmodulepost
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: inpsep(*)

!     inpsep         i    integer input array
!                         Contains specific input for the navier-stokes
!                         integration
!                         contents:
!                         1:  sequence number of navier-stokes  input
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: niinnavstokes=10, nrinnavstokes=1
      integer :: ichoice, iinnavstokes(niinnavstokes), iseqnr, leninp, &
                 lenrnp, istart
      double precision :: rinnavstokes(nrinnavstokes)

!     ichoice        Integer choice parameter, defines type of method
!                     1:  standard
!                     2:  pressure correction
!                     3:  standard time integration, convection through
!                         explicit substepping
!     iinnavstokes   Integer input array for Navier-Stokes
!                    See subroutine readnavstokes for a description
!     iseqnr         Sequence number with respect to the storage in array
!                    ISEPIN
!     istart         Dummy
!     leninp         Computed length of integer input array
!     lenrnp         Computed length of real input array
!     niinnavstokes  Declared length of array iinnavstokes
!     nrinnavstokes  Declared length of array rinnavstokes
!     rinnavstokes   Real input array for Navier-Stokes
!                    See subroutine readnavstokes for a description
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     IINPUTSEQ      Read contents of input block with sequence number iseqnr
!                    corresponding to main keyword name
!     PRININ         print 1d integer array
!     SEPCONVSUBSTEP Solve one step of the time-dependent Navier-Stokes
!                    equations
!                    Use an explicit method for the convection with
!                    smaller time steps (Substepping)
!     SEPPRESSCORR   Perform one or more time steps with the pressure correction
!                    method to solve the Navier-Stokes equations
!     SEPTIMEFREE    Actual body of subroutine TIMEFREE
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     ichoice := type of solution method
!     if ( ichoice=1 ) then
!        perform one time step of the standard computation
!     else if ( ichoice=2 ) then
!        perform one time step pressure correction
!     else if ( ichoice=3 ) then
!        perform one time step with substepping for the convective term
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepnavstokes' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepnavstokes'
         call prinin ( inpsep, 1, 'inpsep' )
100      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Read input for Navier-Stokes and store in array iinnavstokes

      iseqnr = inpsep(1)    ! sequence number input
      leninp = niinnavstokes
      lenrnp = nrinnavstokes
      call iinputseq ( 30, iseqnr, iinnavstokes, &
                       rinnavstokes, leninp, lenrnp, 'navstokes' )
      if ( debug ) call prinin ( iinnavstokes, leninp, 'iinnavstokes' )

!     --- ichoice := type of solution method

      ichoice = iinnavstokes(1)  ! type of solution method
      if ( debug ) write(irefwr,100) 'ichoice', ichoice

      if ( ichoice==1 ) then

!     --- ichoice=1, perform one time step of the standard computation

         complrun = .false.
         istart = istartmn
         istartmn = 0
         call septimefree ( timefinished, iinnavstokes(2), rinnavstokes )
         istartmn = istart

      else if ( ichoice==2 ) then

!     --- ichoice=2, perform one time step pressure correction

         complrun = .false.
         call seppresscorr ( timefinished, iinnavstokes(5) )

      else

!     --- ichoice=3, perform one time step with substepping
!         for the convective term

         call sepconvsubstep ( timefinished, iinnavstokes )

      end if !  ( ichoice==1 )

      istartmn = istartmn + 2  ! raise istartmn

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepnavstokes'

      end if
      call erclos ( 'sepnavstokes' )

      end
