      subroutine statfree ( niter, ready, iread )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 30-01-2016 Replace back
!        version  2.0  date 10-04-1999 Replace body by statfrbf
!        version  1.6  date 24-07-1998 include commons
!
!   copyright (c) 1996-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Subroutine to be used for stationary free surfaces
!    It checks for convergence and if necessary adapts the mesh
!    This is a very special subroutine in the SEPRAN context, since it
!    assumes that the user has made a loop himself
!    statfree should be called in this loop
!    It returns with a value for ready indicating if the loop may be finished
!    or not.
!    The following tasks are performed:
!    - If niter>1, the difference between old and new solution is
!      computed
!      If this difference satisfies the convergence criterion the
!      subroutine returns with ready = true
!      At the users option the final mesh is written
!    - If the subroutine has not been finished then the mesh
!      is adapted using subroutine adapmesh
!      See subroutine adapmesh for a description
!    In contrast to adapmesh input is only possible through the input file
!    and not through the parameter list
!    Warning:  Experimental version
!              Both the parameter list as well as the input may change
!              Check this in each new update of SEPRAN until this
!              subroutine is part of the manuals
! **********************************************************************
!
!                       KEYWORDS
!
!     change_boundary
!     change_mesh
!     mesh
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesol
      use sepmodulekmesh
      use sepmodulemain
      use sepmoduleinput
      use sepmodulemach
      use sepmodulemesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: niter, iread
      logical ready

!     iread   i    Defines how the parameters should be read
!                  Possible values:
!                  0   The input is read from the standard input file
!                  >0  The input as read from the standard input file by
!                      subroutine SEPSTM is used. The value of iread
!                      defines the sequence number
!                  iread is only used if niter=1
!     niter   i/o  Iteration sequence number.
!                  In the first call niter should have the value 1
!                  It is raised each next iteration and reset to 1 if
!                  convergence has been achieved
!     ready   o    Indicates if convergence has been achieved (true) or not
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer niinp, nrinp
      parameter ( niinp=50, nrinp=1 )
      integer iinput(niinp), itypvc, ifirst, ipos, &
              maxitr, minitr, iinvec(6), iretur, iref, icrit, &
              iprint, ivectr, iwrmesh, inpada(1), &
              icurvs(1), inpmsh(1), iseqnr, iseqmsh, islold(5), &
              numvecs, istvectr, leniinp, maxcal
      double precision rinput(nrinp), diffsl(3), eps, rinvec(2), &
                       rinada(1), rinmsh(1)
      logical diver
      save ifirst, islold, diffsl, iinput, rinput

!     diffsl     In this array information about the convergence of the
!                iteration process is stored
!                DIFFSL is filled as follows:
!                diffsl(1)   maximum difference between present and preceding
!                            iteration, i.e. || u(n) - u(n-1) ||
!                diffsl(2)   value of diffsl(i,1) in the preceding
!                            iteration, i.e. || u(n-1) - u(n-2) ||
!                diffsl(3)   value of diffsl(i,2) in the preceding
!                            iteration, i.e. || u(n-2) - u(n-3) ||
!     diver      Indicates if the iteration process diverges (true) or not
!     eps        Local accuracy
!     icrit      Type of criterion to be used for the termination
!                Possible values:
!                0:  Absolute difference between two succeeding iterations
!                1:  Relative difference between two succeeding iterations
!     icurvs     array containing the number of points in the curves
!                accumulated
!     ifirst     First curve (surface) number read
!     iinput     integer input array
!     iinvec     User input array for subroutine MANVEC
!     inpada     User input array for subroutine ADAPBOUN
!     inpmsh     User input array for subroutine ADAPMESH
!     ipos           Position of input arrays in sepmoduleinput
!     iprint     If 1 information must be printed
!     iref       File reference number
!     iretur     Return value of subroutine INTX02
!                Possible values:
!                -1:  error has been found
!                 0:  normal return
!     iseqmsh    Sequence number of adap_mesh input
!     iseqnr     Sequence number with respect to the storage in array ISEPIN
!     islold     Standard SEPRAN array containing information of the solution
!                array at the previous iteration
!     istvectr   Starting position of array iseqvecs in iinput
!     itypvc     Indication of the type of solution vector:
!                110: solution vector
!                115, 116: vectors of special structure
!     ivectr     standard sepran array denoting usol or vector of special
!                structure defined in nodal points
!     iwrmesh    If 1 the mesh must be written if 0 not
!     leniinp    Length of array iinput
!     maxcal     Maximum number of coefficient inputs read
!     maxitr     Maximum number of iterations
!     minitr     Minimum number of iterations
!     niinp      Length of array iinput
!     nrinp      Length of array rinput
!     numvecs    Number of vectors in iseqvecs
!     rinada     User input array for subroutine ADAPBOUN
!     rinmsh     User input array for subroutine ADAPMESH
!     rinput     Real input array
!     rinvec     Real user input array for subroutine MANVEC
!                The contents depend on iinvec
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     INICHINPSOL    Adapt input array with respect to variables
!     INIFIL         Open files
!     MSH068         write information of the mesh to the file indicated by iref
!     MSHADAP        Adapt mesh in free boundary problem
!     READSTFR       Read the input for the subroutine STATFREE
!     SEPCOPYVC      Copy one SEPRAN vector into another one
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPGETINPFINF  Find maxcall and ipos from sepmoduleinput as function of
!                    isubr and iseqnr
!     SEPMANVEC      Vector manipulation subroutine
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
! **********************************************************************
!
!                       I/O
!
!   Depending on the values in IREAD input may be read from the standard
!   input file by subroutine READFRST
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      60   Input solution array has not been filled or is of the wrong
!     419   NDIM # 2
!    1906   Divergence of iteration process
!    1907   No convergence before maximum number of iterations has been reached
!    1945   iter#1, whereas ifirst=0
!    1948   No input with the main keyword has been read
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Perform standard tests
!    Fill arrays iinput and rinput with information about the process
!    if ( niter>1 ) then
!       ready := solution has been converged
!       If ( ready ) then
!          if mesh must be written, write mesh
!          reaset niter
!    else
!       ready := false
!    if ( not ready ) then
!       adapt mesh
!       raise niter
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data ifirst /0/, islold/5*0/
! ======================================================================
!
      call eropen ( 'statfree' )

      if ( ifirst==0 .and. niter/=1 ) then

!     --- ifirst = 0, whereas niter#1

         call errint ( niter, 1 )
         call errwar ( 1945, 1, 0, 0 )
         niter = 1

      end if

      call sepfillkmesh ( kmesh )

!     --- Fill arrays iinput and rinput

      if ( niter==1 ) then

!     --- First iteration
!         Standard tests

         if ( ndim/=2 ) then
            call errint ( ndim, 1 )
            call errint ( 2, 2 )
            call errint ( 2, 3 )
            call errsub ( 419, 3, 0, 0 )
         end if
         itypvc = ks(isolut2d(1))%typevec
         if ( itypvc/=110 .and. itypvc/=115 ) then
            call errint ( itypvc, 1 )
            call errsub ( 60, 1, 0, 0)
         end if
         if ( iread==0 ) then

!        --- iread = 0, fill iinput by reading from the standard input file

            leniinp = niinp
            iseqnr = 1
            call readstfr ( iinput, rinput, 1, iseqnr, leniinp, nrinp )

         else

!        --- iread>0, fill iinput and rinput by copying from ibuffr

            iseqnr = iread
            call sepgetinpfinf ( 18, iseqnr, ipos, maxcal )
            if ( maxcal==0 ) then
               call errchr ( 'STATIONARY_FREE_BOUNDARY', 1 )
               call errsub ( 1948, 0, 0, 1 )
               go to 1000
            end if
            length = size( in(ipos)%iinput )
            iinput(1:length) = in(ipos)%iinput
            length = size( re(ipos)%rinput )
            rinput(1:length) = re(ipos)%rinput

         end if

!        --- Adapt variables

         call inichinpsol ( iinput, niinp )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Set constants

      maxitr  = iinput(1)
      iprint  = iinput(2)
      iretur  = iinput(3)
      minitr  = iinput(4)
      icrit   = iinput(5)
      iseqmsh = iinput(6)
      istvectr = iinput(7)
      iwrmesh = iinput(8)
      numvecs = iinput(9)
      ivectr  = iinput(istvectr)
      eps     = rinput(1)
      diver = .false.
      if ( niter==1 ) then

!     --- niter = 1, first iteration

         ready = .false.
         diffsl(1) = 0d0
         diffsl(2) = 0d0
         diffsl(3) = 0d0

      else

!     --- niter>1, next iteration
!         Check convergence
!         Compute norm(s) of difference

         iinvec(1) = 6
         iinvec(2) = 5
         iinvec(3) = 0
         iinvec(4) = 0
         iinvec(6) = 3
         iinvec(5) = 1
         call sepmanvec ( iinvec, rinvec, isolut2d(ivectr), &
                          islold, isolut2d )
         diffsl(1) = rinvec(1)
         if ( icrit==1 ) then

!        --- Relative criterion, put norm into diffsl(.,1)

            iinvec(5) = 0
            call sepmanvec ( iinvec, rinvec, isolut2d(ivectr), &
                             isolut2d(ivectr), isolut2d )
            if ( rinvec(1)<=0d0 ) rinvec(1) = eps
            diffsl(1) = diffsl(1)/rinvec(1)

         end if
         if ( ierror/=0 ) go to 1000
         if ( niter>=minitr .and. diffsl(1)<eps ) ready = .true.
         if ( .not. ready .and. niter>2 .and. niter>=minitr ) then

!        --- Check divergence

            if ( diffsl(2)<diffsl(1) .and. &
                 diffsl(3)<diffsl(2) ) diver = .true.
            if ( diver ) then
               call errint ( niter, 1 )
               if ( iretur==0 ) then
                  call errsub ( 1906, 1, 0, 0 )
               else
                  call errwar ( 1906, 1, 0, 0 )
               end if
               write ( irefwr, 100 ) diffsl(1), diffsl(2)
100            format ( 10x, ss, 1pe13.2e2, 10x, 1pe13.2e2 )

            end if

         end if

      end if
      if ( abs(iprint)>=1 ) then

!     --- Print output required

         if ( abs(iprint)>=2 ) then

!        --- iprint>=2, print extra information

            if ( niter==1 ) write(irefwr,200)
200         format ( /' Information about the iteration process'/ &
                      ' Iteration     || u(n) - u(n-1) || ')
            if ( niter>1 ) write (irefwr,210) niter-1, diffsl(1)
210         format ( 2x, i5, 7x, ss, 1pe13.2e2 )

         end if

         if ( ready ) then
            write(irefwr,220) niter
220         format ( ' Convergence has been reached after ', i3, &
                     ' steps' )
         end if

      end if

!     --- Copy diffsl(1) into 2

      diffsl(3) = diffsl(2)
      diffsl(2) = diffsl(1)

      if ( diver ) ready = .true.
      if ( .not. ready .and. niter>=maxitr ) then
         call errint ( niter, 1 )
         if ( iretur==0 ) then
            call errsub ( 1907, 1, 0, 0 )
         else
            call errwar ( 1907, 1, 0, 0 )
         end if
         ready = .true.
      end if

      if ( .not. ready ) then

!     --- Process is not yet finished, raise niter and call adapmesh

         ifirst = 1
         niter  = niter+1
         call mshadap ( inpada, rinada, inpmsh, rinmsh, &
                        icurvs, iseqmsh, numvecs, &
                        iinput(istvectr) )

!        --- Copy solution into islold

         call sepcopyvc ( isolut2d(ivectr), islold )

      else

!     --- Process is finished

         ifirst = 0
         niter  = 1
         if ( iwrmesh==1 ) then

!        --- write mesh for post processing purposes

            call inifil ( 10 )
            call msh068 ( iref10 )
            iref = abs(iref10)
            close ( iref )

         end if
         if ( islold(1)>0 ) &
            call seprmsolinfo ( islold(1) )

      end if

1000  call erclos ( 'statfree' )

      end subroutine statfree
