      subroutine to0090 ( iinsol )
! ======================================================================
!
!        programmer    Zdenek Vlasek
!        version  6.4  date 11-12-2017 New storage of intmat
!        version  6.3  date 30-01-2016 Replace meshwrbf by meshwr
!        version  6.2  date 17-12-2015 New call to incommat
!
!   copyright (c) 1989-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Solve second order elliptic partial differential equation
!   with one degree of freedom ( see SP 3.1 ) if necessary by local refinement
! **********************************************************************
!
!                       KEYWORDS
!
!     adaptive
!     linear
!     linear_solver
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesol
      use sepmoduleuser
      use sepmoduleintmat
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulemain
      use sepmodulemach
      use sepmoduleplot
      use sepmodulecadapt
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iinsol(*)

!     iinsol         i    Input array defined by the user
!                         With this array the user may indicate which actions
!                         he wants to be performed by LINPRB
!                         See subroutine linprbnew
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer mxelhl, lniincommat
      parameter ( mxelhl=100, lniincommat=25 )
      integer i, mxmesh, miter, n400, iref, iconst(1), &
              ichois, numol0, ichder, intarray(4), isoltmp(5), &
              iincommt(lniincommat), iplot, iprint, &
              icheld, ix, jdegfd, ivec, iseqnr, isubrtm, &
              ictime, jmesh, imesh, nelemlc, &
              iter, ito, idum(1), nvert, icol1, &
              ielgrp, ielset, nelset, ielhlp(mxelhl), iseqprob
      double precision :: rinput(5)
      double precision epsilo, plotfm, yfact, &
             const(2), avergV, avergE, ernorm, unorm, sumsqr(1), &
             eta, dum(1)
      integer deb(10)
      logical isopen
      save deb, isoltmp

!     averge         r.m.s average of local error estimators for elements
!     avergv         r.m.s average of local error estimators for vertices
!     const          array of length 2, constants for vecman
!     deb
!     dum            dummy variable
!     epsilo         same as rinlrf(1), demanded precision
!     ernorm         global semi-energy norm of error
!     eta            when inplrf(2)=1: relative global semi-energy norm of error
!                    when inplrf(2)=2: global semi-energy norm of error
!     i              Counting variable
!     ichder         parameter for deriva
!     icheld         parameter for deriva
!     ichois         one component of  ichder  for deriva
!                    parameter in other calls
!     icol1          isoltmp(*,icol1) is passed to deriva as inpvc1
!     iconst         Help array with respect to subroutine vecman
!     ictime         parameter for to0089 and probdf
!     idum           dummy variable
!     ielgrp         counter of element groups
!     ielhlp         array of length nelgrp ( = kmesh(5) ), sets of element
!                    groups to be processed is marked by 1, the others by 0
!     ielset         counter of sets of element groups
!     iincommt       Integer input array containing information concerning
!                    the structure of the large matrix
!     imesh          loop variable to count the meshes
!     intarray       array used to pass the options into refloc
!     iplot          indicates amount of plots of meshes
!     iprint         indicates amount of prints to standard output
!     iref           File reference number
!     iseqnr         Sequence number
!     iseqprob       Sequence number of problem input
!     isoltmp        array 5 * 5, each column is one Sepran array
!                    column 1  :  scalar per node:     solution  u
!                           2  :  vector per node:
!                                    components of smooth ( a * grad u )
!                           3  :  two scalars per element
!                                 --- first scalar
!                                    temporary: error estimators
!                                    finaly:    error indicators for element
!                                 --- second scalar
!                                    local semi-energy norm of solution
!                           4  :  temporary: vector per node:
!                                    old smooth ( a * grad u )
!                                 temporary: scalar per vertex:
!                                    error estimators
!                                 finaly:    scalar per vertex:
!                                    error indicators for vertices
!                           5  :  scalar per vertex:
!                                    local refinement criterium for refloc
!     isopen         answer of inquire, is file 10 open ?
!     isubrtm        Sequence number of subroutine
!     iter           counter of calls of deriva
!     ito            what column of isoltmp should be filled by subr deriva
!     ivec           parameter for deriva
!     ix             parameter for deriva
!     jdegfd         parameter for vecman
!     jmesh          the mesh for which the solution was passed via isoltmp(*,1)
!     lniincommat    Length of iincommat
!     miter          same as inplrf(7)
!     mxelhl         5 + upper bound for number of element groups + &
!                    upperbound for number of natural boundary conditions
!                    see PG 5.4.2
!     mxmesh         same as inplrf(3)
!     n400           counter for the loop goto 400
!     nelemlc        Local number of elements
!     nelset         number of sets of element groups
!     numol0         one component of parameter ichder for deriva
!     nvert          number of vertices in a mesh
!     plotfm         parameter for plotm2
!     rinput         Real dummy array
!     sumsqr         sum of squares of some local errors
!     unorm          global semi-energy norm of solution
!     yfact          parameter for plotm2
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ER0034         Performs the actual tasks for CREAT and PRESDF
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIFIL         Open files
!     PROBDFEXT      Reads and stores problem definition
!     SEPCOMMAT      Actual body of COMMAT and MATSTRUC
!     SEPCOPYVC      Copy one SEPRAN vector into another one
!     SEPDERIVA      calculate values derived from solution
!     SEPMESHWR      fill the file meshoutput with the mesh
!     SEPPLOTM1      plot 1D mesh
!     SEPPLOTM2      plot 2D mesh
!     SEPPRINOV      Print SEPRAN solution vector
!     SEPREFLOC      Refine mesh locally
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
!     SEPVECMAN      Sepran vector manipulations
!     TO0089         Build system of equations and solves linear system of
!                    equations
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   1423
!   1425
!   1426
! **********************************************************************
!
!                       PSEUDO CODE
!
!     "Zienkiewicz-Zhu simple error estimator"
!     see e.g. Zhu J.Z., Zienkiewicz A.O.:
!              Adaptive techniques in the finite element method,
!              Comm. in Appl. Num. Methods, Vol. 4, 197-204 (1988)
!                    PSEUDO CODE
!   repeat
!     solve problem by to0089
!     calculate the a-posteriori local error estimators in semi-energy
!               norm by "Zienkiewicz-Zhu simple error estimator"
!     calculate indicators of local distribution of this error,
!               by Zienkiewicz-Zhu
!     calculate a local refinement criterium in vertices, for refloc
!     perform a local mesh refinement by refloc
!     update data in memory
!   until prescribed precision is achieved
!         or maximum number of meshes is reached
!   update file 'meshoutput'
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data deb / 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 /, &
           isoltmp/5*0/
! ======================================================================
!
      call eropen('to0090')

!     --- check inplrf(*)  and  rinlrf(*)

      nelemlc = 0
      iconst(1) = 0
      if ( inplrf(1)<8 &
         .or. inplrf(2)/=1 .and. inplrf(2)/=2 &
         .or. inplrf(3)<1 &
         .or. inplrf(4)<0 .or. inplrf(4)>2 &
         .or. inplrf(5)<0 .or. inplrf(5)>2 &
         .or. inplrf(6)<0 .or. inplrf(6)>1 &
         .or. inplrf(7)<0 .or. inplrf(8)<0 &
         .or. rinlrf(1)<0d0 ) then

         do i=1,8
           call errint ( inplrf(i), i )
         end do

         call erreal ( rinlrf(1), 1 )
         call errsub ( 1423, 8, 1, 0 )
         go to 1000

      end if

!     --- unpack  inplrf(*)  and  rinlrf(*)

      mxmesh = inplrf(3)
      iplot  = inplrf(4)
      iprint = inplrf(5)
      miter  = inplrf(7)
      jmesh  = inplrf(8)

      epsilo = rinlrf(1)
      call sepcopyvc ( isolut2d, isoltmp(1) )

!     --- main refinement loop

      optim = .false.

      do imesh = max(1,jmesh), mxmesh

         if ( iprint>=1 ) write (irefwr,'(/1x, a, i3 / )') &
           '----------------- to0090 ------------------- mesh number =', &
           imesh

         if ( iplot==1 .and. imesh==1 &
            .or. iplot==2 ) then

!        --- plot the current mesh, see PG 9.3 and 9.1

            ichois = 0
            plotfm = 15d0
            yfact = 1d0
            jmax = 0
            jkader = -1
            jtimes = 0
            if ( ndim==1 ) then
               jmark = 1
               call sepplotm1 ( ichois, idum, plotfm )
            else if ( ndim==2 ) then
               jmark = 5
               call sepplotm2 ( ichois, idum, plotfm, yfact )
            else

!           --- not implemented

            end if

         end if
         if ( imesh>jmesh ) then

!        --- The solution is not available
!            Get essential boundary conditions, see PG 5.5.3

            if ( imesh==1 ) then

!           --- Read them from a file

               ictime = 0

            else

!           --- Use the e.b.c. stored in ibuffr

               ictime = 1

            end if

            iseqnr = 1
            isubrtm = -2
            call er0034 ( isubrtm, ictime, isolut2d, iseqnr, isolut2d, 100 )

            iprob = ks(isolut2d(1))%iprob

!           --- Assemble the system and solve it, see IN 5.3.3 - linsol

            call to0089 ( ictime, iinsol )
            call sepcopyvc ( isolut2d, isoltmp(1) )

!           --- result :  isolut ... solution u  in nodes

         end if

!        --- calculate a-posteriori error estimators for elements,
!            for each of sets of element groups
!            ----------------- by deriva --------  icheld 8-10
!            loop over element sets (groups?)
!            loop over icheld = 10, [11].. , 13, 12
!            icheld=8/10:  calculate the smooth  a * grad u
!                       - it is the first approximation, lumped mass matrix used
!                       - independently for each element group (ichois=10),
!                         on the common boundary of element groups the values
!                         may differ
!                       - clear the output vector for each group
!            icheld=9/13:  local error estimators - attached to elements
!                       - for refinement criterium on elements
!                     local norm of the solution - attached to elements
!                       - independently for each element group (ichois=10)
!                       - clear the output vector only once(ichois=10 then 30)
!            icheld=10/12:  local error estimators - distributed to vertices
!                       - for all element groups at once (ichois=0),
!                         so this part runs only for the last group
!                       - clear the output vector
!            endloop
!            endloop
!            --- see SP 3.1.3 for ichois, icheld, ix, jdegfd, ivec

         jdegfd = 1
         ix = 0
         ivec = 1

!        --- one set = one element group

         nelset = nelgrp
         do ielset = 1, nelset

!        --- fill ielhlp(*), first 5 positions are reserved,
!                            first position is length

            ielhlp(1) = mxelhl

            do ielgrp = 1, nelgrp + kprobloc ( 12 )
               ielhlp ( 5 + ielgrp ) = 0
            end do
            ielhlp ( 5 + ielset ) = 1

!           --- loop, three or more calls of deriva

            do iter = 0, miter + 2

               if ( iter==0 ) then

!              --- the first smoothing, with the lumped mass matrix
!                                       or some other diagonal matrix

                  icheld = 8!10
                  ichois = 10
                  numol0 = 1
                  ito = 2

               else if ( iter<=miter ) then

!              --- 1<=iter<=miter
!                  perform one iteration to improve smoothing

                  icheld = 9
                  ichois = 10
                  numol0 = 2
                  ito = 4

               else if ( iter==miter+1 ) then

!              --- iter = miter + 1
!                  local error estimators and norm of solution,
!                  both attached to element

                  icheld = 9

                  if ( ielset==1 ) then
                    ichois = 10
                  else
                    ichois = 30
                  end if

                  numol0 = 2
                  ito = 3

               else

!              --- iter = miter + 2
!                  local error estimators, distributed to vertices
!                  only for the last set of element groups,
!                  ichois=0, so all groups should be processed

                  icheld = 10
                  ichois = 0
                  numol0 = 3
                  ito = 4

               end if

!              --- when ndim=1:  inpvc0 is isoltmp(*,1)
!                                inpvc1 is isoltmp(*,2), numol0=0
!                  when ndim=2:  inpvc0 is isoltmp(*,1..numol0)
!                                inpvc1 is not used, dummy column icol1=1

               if ( ndim==1 ) then
                  numol0 = 0
                  if ( icheld==8 ) then
                    icol1 = 1
                  else
                    icol1 = 2
                  end if
               else
                  icol1 = 1
               end if

               iprob = ks(isolut2d(1))%iprob

               iuser => iuser1(iprob)%iuser
               user => iuser1(iprob)%user

               ichder = ichois + 100 * numol0

               if ( icheld/=10 .or. ielset==nelset ) then

                  if ( deb(1)/=0 ) &
                     write ( irefwr, '( / 10(a,1x,i2) )' ) &
                     'to0090 calls deriva:', icheld, '=icheld', &
                     numol0, '=numol0', ito, '=ito', ichois, '=ichois', &
                     ielset, '=ielset', icol1, '=icol1'

                  call sepderiva ( ichder, icheld, ix, &
                                jdegfd, ivec, isoltmp(ito), &
                                isoltmp(1), isoltmp(icol1), ielhlp )
                  if ( deb(2)/=0 .and. icheld==10 ) &
                      call sepprinov ( isoltmp(ito), &
                                       1, 'error in vertices', dum, 0 )

               end if

            end do

         end do

!        --- calculate sum of squares of local error estimators
!                                        on elements,
!            ( use the first "jdegfd" )

         jdegfd = 1
         call sepvecman ( 7, jdegfd, isoltmp(3), &
                          isoltmp(3), idum, sumsqr, iconst )
         nelemlc = iconst(1)
         if ( nelemlc/=iconst(1) ) then
            call errint ( nelemlc, 1 )
            call errint ( iconst(1), 2 )
            call errsub ( 1425, 2, 0, 0 )
            go to 1000
         end if

!        --- calculate global semi-energy norm of error estimator

         ernorm = sqrt ( sumsqr(1) )

         if ( iprint>=1 ) write ( irefwr, '( / ( 1x, a, f14.8 ) )' ) &
            'estimator of global semi-energy norm of error  =', ernorm

!        --- the same global error corresponds to an average value avergE

         avergE = sqrt ( sumsqr(1) / dble(nelemlc) )

!        --- global semi-energy norm of  u
!            ( use the second "jdegfd" )

         jdegfd = 2
         call sepvecman ( 7, jdegfd, isoltmp(3), &
                          isoltmp(3), idum, sumsqr, iconst )
         nelemlc = iconst(1)

         unorm = sqrt ( sumsqr(1) )

         if ( inplrf(2)==1 ) then

!        --- relative error, see [AiZhCrZi89], p.2168

            if ( unorm>0d0 ) then
               eta = ernorm / sqrt ( unorm**2 + ernorm**2 )
            else

!           --- both  unorm  and  ernorm  are zero,
!               avoid crash by zerodivide

               eta = 0d0
            end if

         else if ( inplrf(2)==2 ) then

!        --- absolute error in semi-energy norm

            eta = ernorm
         end if

         if ( iprint>=1 ) then

            write ( irefwr, '( ( / 1x, a, f14.8 ) )' ) &
            '                    semi-energy norm of  u     =', unorm, &
            'estimator of (relative) error, semi-energy n.  =', eta, &
            'r.m.s. average of local error per element      =', avergE

         end if

!        --- calculate sum of squares of local error estimator
!                                        in vertices

         jdegfd = 1
         call sepvecman ( 7, jdegfd, isoltmp(4), &
                          isoltmp(4), idum,  sumsqr, iconst )
         nvert = iconst(1)

         if ( iprint>=1 ) write( irefwr, '( / 1x,a,i7,5x,a,i7)' ) &
            'to0090: nelem=', nelemlc, ' nvert=', nvert

         if ( nelemlc/=nelem ) write (irefwr,*) &
           'to0090: internal bug, nelem:', nelem, '=nelem'

!        --- calculate r.m.s averaged local error estimator, vertices

         avergV = sqrt ( sumsqr(1) / dble(nvert) )

         if ( iprint>=1 ) write( irefwr, '( / ( 1x, a, f14.8 / ) )' ) &
            'r.m.s. average of local error per vertex       =', avergV

         if ( eta<=epsilo ) then

!        --- precision is O.K., leave the refinement loop

            if ( iprint>=1 ) write(irefwr,*) &
               ' >>>>> this mesh satisfies the user''s demand'
            goto 600

         end if

!        --- replace the error estimators by error indicators,
!            which tell
!                "how many times should be error on element smaller"
!            ( error indicators are calculated even when the next
!              refinement is not allowed, to enable restart )
!            ratio  eta/epsilo  takes care for global error
!            denominator  avergV  resp.  avergE  takes care for a regular
!                         distribution of the local error, which is
!                         needed to avoid a lower convergence rate
!                         caused by singularities

         if ( optim ) then

!        --- some of previous meshes was nearly optimal
!            ( the local error distribution is regular )
!            we need to refine all mesh

            const ( 1 ) = (eta/epsilo) / avergV

         else

!        --- we do not care much about the global error at this stage
!            but we look for nearly optimal mesh

            const ( 1 ) = (eta/epsilo)**0.2 / avergV

         end if

!        --- multiply vector  isoltmp(*,4) := const(1) * isoltmp(*,4)

         const ( 2 ) = 0d0

         jdegfd = 1
         call sepvecman ( 3, jdegfd, isoltmp(4), &
                           isoltmp(4), isoltmp(4), const, iconst )

         if ( deb(3)/=0 ) &
            call sepprinov ( isoltmp(4), &
                             1, 'rel.error in vertices', dum, 0 )

!        --- put in isoltmp(*,5) the local refinement criterium for refloc:
!                             relative lengths of new edges
!            refloc refines the edges for which the average of this
!            criterium in the endpoints is>1.5
!            expected rate of convergence ( it gives some dependency of
!            local error on local size of edges ), is taken into account
!            in the subroutine FUNADA called via vecman

         jdegfd = 1
         call sepvecman ( 19, jdegfd, isoltmp(4), &
                          isoltmp(4), isoltmp(5), const, iconst )

         if ( deb(4)/=0 ) &
            call sepprinov ( isoltmp(5), &
                             1, 'criterium for refloc', dum, 0 )

         if ( imesh<mxmesh ) then

            n400 = 0

!           --- we may try to refine the mesh, &
!               using  isoltmp(*,5)  as criterium

400        ichois = 1

!          --- length of intarray(*)

            intarray(1) = 4

!           --- local criterium

            intarray(2) = 1

!           --- relative criterium

            intarray(3) = 1

!           --- maxnumber of refine steps ( halving )

            intarray(4) = 2

            call seprefloc ( isoltmp(5), dum, intarray )

!           --- some values must be updated after refinement,
!               see UM 4.1, 5.2

            ichois = 3

            iseqprob = 1
            call probdfext ( ichois, iseqprob )

            iseqim = intmat1d(1)
            iprob = intmt(iseqim)%iprob
            do i = 3, lniincommat
               iincommt(i) = 0
            end do  ! i = 3, lniincommat
            iincommt(1) = 9
            iincommt(2) = intmt(iseqim)%jmethod
            iincommt(4) = 0
            iincommt(5) = iprob
            call sepcommat ( iincommt )

!           --- now, the new mesh is ready for next loop

            if ( nelemlc==nelem ) then

!           --- mesh was not changed during this refinement

               if ( iprint>=1 ) write(irefwr,*) &
                 ' mesh was not changed during this refinement'

               if ( optim ) then

!              --- deadlock: global error is too big
!                            but mesh was not refined
!                  reason: refinement follows the criterium in a very
!                          rough way and is unable to achieve crit(*)=<1
!                  solution: enlarge the criterium, how much ?

                 const ( 1 ) = 1.5d0
                 const ( 2 ) = 0d0

                 jdegfd = 1
                 call sepvecman ( 3, jdegfd, &
                                 isoltmp(5), isoltmp(5), &
                                 isoltmp(5), const, iconst )

!                --- the mesh is not counted, we repeat the refinement

                 n400 = n400 + 1

                 if ( n400<=4 ) goto 400

                 call errsub ( 1426, 0, 0, 0 )
                 go to 1000

              else

!             --- nearly optimal mesh was found
!                 but the global error remains too large, so from now
!                 crit(*) should take into account the global error
!                 ( in fact, instead of p-refinement we apply
!                   some kind of regular h-refinement )

                 optim = .true.
                 if ( iprint>=1 ) &
                    write (irefwr,*) 'this was nearly optimal mesh'

              end if

            else

!           --- mesh was changed during this refinement,
!               the old solution does not correspond to new mesh,
!               we must solve the problem again

            end if

         else

!        --- global semi-energy norm is too big,
!            but  imesh = mxmesh, next refinement is not allowed,
!            loop ends normaly

         end if

      end do

      imesh = mxmesh

!     --- fill output parameters
!         restore the upper end of the buffer

600   inplrf(8) = imesh
      rinlrf(2) = eta

!     --- plot the last mesh if needed

      if ( iplot==1 .and. imesh>1 ) then

         if ( ndim==1 ) then
            jmark = 1
            call sepplotm1 ( ichois, idum, plotfm )
         else if ( ndim==2 ) then
            jmark = 5
            call sepplotm2 ( ichois, idum, plotfm, yfact )
         else

!        --- not implemented

        end if

      end if

!     --- rewrite file 'meshoutput'
!         it is not needed here but later, in the postprocessing

      if ( imesh>1 ) then

!     --- re-open the file 'meshoutput', to record the updated mesh

         iref = abs(iref10)
         inquire ( iref, OPENED = isopen )
         if ( isopen ) close ( iref )
         call inifil ( 10 )

         call sepmeshwr ( iref10 )

         if ( iprint>=1 ) write ( irefwr, '( / 1x, a / 1x, a/ )' ) &
            'to0090: the file "meshoutput" has been just updated', &
        '        the original mesh is lost ! '

      end if
      do i = 1, 5
         if ( isoltmp(i)>0 ) &
            call seprmsolinfo ( isoltmp(i) )
      end do

1000  call erclos ('to0090')

      end subroutine to0090
