      subroutine toloop ( niter, finished, iread, iseqlp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 20-08-2016 use sepmoduleinput
!        version  2.5  date 15-11-2012 Long integers
!        version  2.4  date 19-09-2012 Extension with cloop
!
!   copyright (c) 1998-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Subroutine to be used for loops
!    It checks for convergence
!    This is a very special subroutine in the SEPRAN context, since it
!    assumes that the user has made a loop himself
!    toloop should be called in this loop
!    It returns with a value for finished indicating if the loop may be finished
!    or not.
!    The following tasks are performed:
!    - If niter>1, the difference between old and new solution is
!      computed
!      If this difference satisfies the convergence criterion the
!      subroutine returns with finished = true
!    input is only possible through the input file
!    and not through the parameter list
!    Warning:  Experimental version
!              Both the parameter list as well as the input may change
!              Check this in each new update of SEPRAN until this
!              subroutine is part of the manuals
! **********************************************************************
!
!                       KEYWORDS
!
!     loop
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmoduleloop
      use sepmoduleinput
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: niter, iread, iseqlp
      logical finished

!     finished      i/o   Indicates if the loop has finished (true) or not
!                         (false)
!     iread          i    Defines how the input must be read
!                         Possible values:
!                         -1:  No input is required
!                          0:  All SEPRAN input has been read by
!                              subroutine SEPSTN until
!                              END_OF_SEPRAN_INPUT or
!                              end of file has been found
!                              This input is used
!                          1:  The input is read as described for
!                              EIGENVAL
!     iseqlp         i    Present level of loop
!     niter         i/o   Iteration sequence number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer niinp, nrinp
      parameter ( niinp=7, nrinp=1 )
      integer iinput(niinp), itypvc, ipos, &
              maxitr, minitr, iinvec(6), iretur, icrit, &
              iprint, ivectr, iseqnr, maxcal
      double precision rinput(nrinp), diffsl(3,10), eps, rinvec(2)
      logical diver, checkdiv
      save diffsl, iinput, rinput

!     checkdiv       If true check divergence
!                    they are not
!     diffsl         In this array information about the convergence of the
!                    iteration process is stored
!                    DIFFSL is filled as follows:
!                    diffsl(1)   maximum difference between present and
!                                preceding iteration, i.e. || u(n) - u(n-1) ||
!                    diffsl(2)   value of diffsl(i,1) in the preceding
!                                iteration, i.e. || u(n-1) - u(n-2) ||
!                    diffsl(3)   value of diffsl(i,2) in the preceding
!                                iteration, i.e. || u(n-2) - u(n-3) ||
!     diver          Indicates if the iteration process diverges (true) or not
!     eps            Local accuracy
!     icrit          Type of criterion to be used for the termination
!                    Possible values:
!                    0:  Absolute difference between two succeeding iterations
!                    1:  Relative difference between two succeeding iterations
!     iinput         integer input array with the following contents
!                    1:  MAXITER   Maximum number of iterations allowed
!                    2:  IPRINT    Defines the amount of output produced
!                                  Possible values:
!                                 -1:  All output information is suppressed, &
!                                      even the message of no-convergence
!                                  0:  No information about the iteration
!                                      process is printed except in the case
!                                      of an error
!                                  1:  The number of iterations performed is
!                                      printed
!                                  2:  Some extra information about the
!                                      iteration process is printed for each
!                                      iteration
!                                  3:  See 2, in this case also the ratio of two
!                                      succeeding iterations is printed
!                    3:  IRETURN   Indicates if the subroutine must stop if
!                                  no convergence has been found (0) or that
!                                  control must be given back to the main
!                                  program (1)
!                    4:  MINITER   Minimum number of iterations allowed
!                    5:  ICRITER   Type of criterion to be used for the
!                                  termination
!                                  Possible values:
!                                  0:  Absolute difference between two
!                                      succeeding iterations
!                                  1:  Relative difference between two
!                                      succeeding iterations
!                    6:  IVECTR    Sequence number of vector to be used for
!                                  comparison
!                    7:  ICHKDIV   Indicates if divergence must be tested
!                                  0: yes
!                                  1: no
!     iinvec         User input array for subroutine MANVEC
!     ipos           Position of input arrays in sepmoduleinput
!     iprint         If 1 information must be printed
!     iretur         Return value of subroutine INTX02
!                    Possible values:
!                    -1:  error has been found
!                     0:  normal return
!     iseqnr         Sequence number with respect to the storage in array ISEPIN
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     ivectr         standard sepran array denoting usol or vector of special
!                    structure defined in nodal points
!     maxcal         Maximum number of coefficient inputs read
!     maxitr         Maximum number of iterations
!     minitr         Minimum number of iterations
!     niinp          Length of array iinput
!     nrinp          Length of array rinput
!     rinput         Real input array
!     rinvec         Real user input array for subroutine MANVEC
!                    The contents depend on iinvec
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     INICHINPSOL    Adapt input array with respect to variables
!     PRININ         print 1d integer array
!     SEPCOPYVC      Copy one SEPRAN vector into another one
!     SEPGETINPFINF  Find maxcall and ipos from sepmoduleinput as function of
!                    isubr and iseqnr
!     SEPMANVEC      Vector manipulation subroutine
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
! **********************************************************************
!
!                       I/O
!
!   Depending on the values in IREAD input may be read from the standard
!   input file by subroutine READFRST
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1906   Divergence of iteration process
!    1907   No convergence before maximum number of iterations has been reached
!    1945   iter#1, whereas ifirstlp=0
!    1948   No input with the main keyword has been read
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Perform standard tests
!    Fill arrays iinput and rinput with information about the process
!    if ( niter>1 ) then
!       finished := solution has been converged
!       If ( finished ) then
!          if mesh must be written, write mesh
!          reaset niter
!    else
!       finished := false
!    if ( not finished ) then
!       adapt mesh
!       raise niter
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'toloop' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from toloop'
         write(irefwr,1) 'iread, iseqlp', iread, iseqlp
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ifirstlp(iseqlp)==0 .and. niter/=0 ) then

!     --- ifirstlp = 0, whereas niter#1

         call errint ( niter, 1 )
         call errwar ( 1945, 1, 0, 0 )
         niter = 0

      end if

!     --- Fill arrays iinput and rinput

!     --- iread>0, fill iinput and rinput by copying from ibuffr

      iseqnr = iread
      call sepgetinpfinf ( 21, iseqnr, ipos, maxcal )
      if ( maxcal==0 ) then
         call errchr ( 'LOOP_INPUT', 1 )
         call errsub ( 1948, 0, 0, 1 )
         go to 1000
      end if
      iinput = in(ipos)%iinput(1:niinp)
      rinput = re(ipos)%rinput(1:nrinp)

!     --- Adapt variables

      call inichinpsol ( iinput, niinp )
      if ( debug ) call prinin ( iinput, 7, 'iinput' )

      if ( ierror/=0 ) go to 1000

!     --- Set constants

      maxitr  = iinput(1)
      iprint  = iinput(2)
      iretur  = iinput(3)
      minitr  = iinput(4)
      icrit   = iinput(5)
      ivectr  = iinput(6)
      eps     = rinput(1)
      diver = .false.
      checkdiv = iinput(7)==0

      if ( niter==0 ) then

!     --- niter = 1, first iteration

         finished = .false.
         diffsl(1,iseqlp) = 0d0
         diffsl(2,iseqlp) = 0d0
         diffsl(3,iseqlp) = 0d0

      else

!     --- niter>1, next iteration
!         Check convergence
!         Compute norm(s) of difference

         iinvec(1) = 6
         iinvec(2) = 5
         iinvec(3) = 0
         iinvec(4) = 0
         iinvec(6) = 3
         iinvec(5) = 1
         call sepmanvec ( iinvec, rinvec, isolut2d(ivectr), &
                          isollp(1,iseqlp), isolut2d )
         diffsl(1,iseqlp) = rinvec(1)
         if ( debug ) write(irefwr,2) 'rinvec(1)', rinvec(1)
         if ( icrit==1 ) then

!        --- Relative criterion, put norm into diffsl(.,1)

            iinvec(5) = 0
            call sepmanvec ( iinvec, rinvec, &
                             isolut2d(ivectr), isolut2d(ivectr), isolut2d )
            if ( rinvec(1)<=0d0 ) rinvec(1) = eps
            diffsl(1,iseqlp) = diffsl(1,iseqlp)/rinvec(1)

         end if  ! ( icrit==1 )
         if ( ierror/=0 ) go to 1000
         if ( niter>=minitr .and. diffsl(1,iseqlp)<eps ) &
            finished = .true.
         if ( checkdiv .and. .not. finished .and. niter>2 .and. &
              niter>=minitr ) then

!        --- Check divergence

            if ( diffsl(2,iseqlp)<diffsl(1,iseqlp) .and. &
                 diffsl(3,iseqlp)<diffsl(2,iseqlp) ) diver = .true.
            if ( diver ) then
               call errint ( niter, 1 )
               if ( iretur==0 ) then
                  call errsub ( 1906, iseqlp, 0, 0 )
               else
                  call errwar ( 1906, iseqlp, 0, 0 )
               end if
               write ( irefwr, 100 ) diffsl(1,iseqlp), diffsl(2,iseqlp)
100            format ( 10x, ss, 1pe13.2e2, 10x, 1pe13.2e2 )

            end if

         end if

      end if
      if ( abs(iprint)>=1 ) then

!     --- Print output required

         if ( abs(iprint)>=2 ) then

!        --- iprint>=2, print extra information

            if ( niter==0 ) write(irefwr,200)
200         format ( /' Information about the iteration process'/ &
                      ' Loop,  Iteration     || u(n) - u(n-1) || ')
            if ( niter>0 ) &
                write (irefwr,210) iseqlp, niter, diffsl(1,iseqlp)
210         format ( 2x, i5, 2x, i5, 7x, ss, 1pe13.2e2 )

         end if

         if ( finished ) then
            write(irefwr,220) iseqlp, niter
220         format ( ' Convergence for loop ', i2, &
                     ' has been reached after ', i3, ' steps' )
         end if

      end if

!     --- Copy diffsl(1) into 2

      diffsl(3,iseqlp) = diffsl(2,iseqlp)
      diffsl(2,iseqlp) = diffsl(1,iseqlp)

      if ( diver ) finished = .true.
      if ( .not. finished .and. niter>=maxitr ) then
         call errint ( niter, 1 )
         if ( iretur==0 ) then
            call errsub ( 1907, 1, 0, 0 )
         else
            call errwar ( 1907, 1, 0, 0 )
         end if
         finished = .true.
      end if

      if ( .not. finished ) then

!     --- Process is not yet finished, raise niter

         ifirstlp(iseqlp) = 1
         niter  = niter+1

!        --- Copy solution into isollp

         call sepcopyvc ( isolut2d(ivectr), isollp(1,iseqlp) )

      else

!     --- Process is finished

         ifirstlp(iseqlp) = 0
         niter  = 1
         if ( isollp(1,iseqlp)>0 ) &
            call seprmsolinfo ( isollp(1,iseqlp) )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End toloop'

      end if  ! ( debug )
      call erclos ( 'toloop' )

      end
