      subroutine msh009 ( bcord, bcorlc, t1, t2, t3, trans, bcorut, bcord3d )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.8  date 22-01-2014 Increase accuracy
!        version  4.7  date 09-09-2013 Increase accuracy
!        version  4.6  date 27-11-2008 Decrease accuracy plabe
!
!   copyright (c) 1984-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    find transformation and inverse transformation for a plane
!    in three dimensions
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     plane
!     transformation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision bcord(ibcrds,*), bcorlc(ibcrds-1,*), &
                       t1(3), t2(3), t3(3), trans(3,3), &
                       bcorut(3), bcord3d(3,nbndpt)

!    bcord    i  array containing the x, y, z co-ordinates of the boundary
!                bcord(1,i) = xi etc.
!    bcord3d  o  copy of bcord
!    bcorlc   o  array containing the ksi-eta co-ordinates
!                bcorlc(1,i) = ksi i bcorlc(2,i) = eta i
!    bcorut   o  contains the co-ordinates of the first point
!    t1       o  first tangential vectors in plane (normalized)
!    t2       o  second tangential vectors in plane (normalized and orthogonal
!                to t1)
!    t3       o  normal vector on the plane (normalised)
!    trans    o  array containing the inverse of the transformation
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nodp, nodsav
      double precision som, sqr, produc, prodsv, det, sumsav, maxsum

!     det            Determinant of tranformation
!     i              General loop variable
!     maxsum         Maximum difference between coordinates in boundary
!                    before and after transformation
!     nodp           Nodal point number
!     nodsav         Nodal point number to be saved
!     prodsv         Inner product netween two tangential vectors to be saved
!     produc         Inner product netween two tangential vectors
!     som            help variable to compute a sum
!     sqr            Local corrected accuracy
!     sumsav         help variable to store the norm of the vector t2
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision ercdet

!     ERCDET         Compute a 3x3 determinant
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERCOMPNORMVEC  Compute the normal vector with respect to a plane, given
!                    two orthogonal tangential vectors in this plane
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHCOPBOUN2PL  Copy 3d boundary onto plane
!     MSHTRANSMAT    Compute the inverse transformation matrix to be used to
!                    map a 3d plane onto 2d coordinates, given 2 tangential
!                    vectors and the normal
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     278   Number of points on boundary of submesh less than 2
!     279   Points on the boundaries of a surface in r3 are not lying in a plane
!    1788   The angle between the first point at the boundary and all other
!           points at the boundary is too smal
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Define t1 = (x2-x1) / ||x2-x1|| (First tangential vector)
!    Find i such that |(t2*,t1)|<0.5  with t2* = (xi-x1) / ||xi-x1||, i.e.
!    Each vector x in the plane may be written as:
!     x = lambda t1 + mu t2* &
!    Define second tangential vector orthogonal to t1:
!    t2 = lambda t1 + mu t2*, such that (t2,t1) = 0:
!    lambda (t1,t1) + mu (t2*,t1) = 0, hence
!    mu=1, lambda=-(t2*,t1)
!    t2:=t2/||t2||
!                                          | t12 t23 - t13 t22 |
!    n = t1 x t2 / ||t1 x t2||;  t1 x t2 = | t13 t21 - t11 t23 |
!                                          | t11 t22 - t12 t21 |
!    Representation of x in plane:  x = ksi t1 + eta t2
!         |x|                      |ksi|
!   or    |y|   = |  t1  t2  n  |  |eta|
!         |z|                      | 0 |
!   The inverse transformation is given by:
!       |ksi|                      -1   |x|
!       |eta|   =   |  t1  t2  n  |     |y|
!       | 0 |                           |z|
!   Inverse of |t1 t2 n|:  |det( t1 t2 n )| = 1
!      -1   | t22 n3 - t23 n2    t23 n1 - t21 n3    t21 n2 - t22 n1 |
!     T     | n2 t13 - n3 t12    n3 t11 - n1 t13    n1 t12 - n2 t11 |
!           | t12 t23- t13 t22   t13 t21- t11 t23   t11 t22- t12 t21|
!   hence:
!  ksi = (t22 n3 - t23 n2) x + (t23 n1 - t21 n3) y + (t21 n2 - t22 n1) z
!  eta = (t13 n2 - t12 n3) x + (t11 n3 - t13 n1) y + (t12 n1 - t11 n2) z
!                  PSEUDO CODE
!    Check if nbndpt>2
!    Compute and normalize t1: t1:=(x2-x1)/||x2-x1||
!    Compute and normalize t2:
!      found := true
!      For i := nbndpt (-1) 2 while (not found) do
!         t2 := (xi-x1) / ||xi-x1||
!         if ( |(t2,t1)|<0.5 ) found := true
!      If ( not found ) then
!         error: all points on a line
!      t2 := t2 - (t2,t1) t1
!      t2 := t2 / ||t2||
!    Compute and normalize n: n := t1 x t2 / || t1 x t2 ||
!    Compute determinant of transformation:
!      det := t11 ( t22 n3 - t23 n2 ) - t21 ( t12 n3 - t13 n2 ) + &
!             n1 ( t12 t23 - t13 t21 )
!    If ( det<0 ) n := -n
!    Compute inverse of transformation
!    Fill array bcorlc with transformed coordinates
!    Check if all points are in a plane
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh009' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh009'
         write(irefwr,1) 'nbndpt', nbndpt
         call prinrl1 ( bcord, nbndpt, ibcrds, 'bcord' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Check if nbndpt>2

      if ( nbndpt<=2 ) then
         call errint ( nbndpt, 1 )
         call errint ( isurf, 2 )
         call errsub ( 278, 2, 0, 0 )
         go to 1000
      end if

!     --- construct and normalise t1

      bcord3d(1:3,1:nbndpt) = bcord(1:3,1:nbndpt)

      t1 = bcord(1:3,2)-bcord(1:3,1)
      som = dot_product(t1,t1)
      t1 = t1/sqrt(som)
      sqr = 1d-5*som

      if ( debug ) write(irefwr,2) 'sqr, som', sqr, som

!     --- find t2* &

      prodsv = rinfin
      do nodp = 2, nbndpt-1
         t2 = bcord(1:3,nodp)-bcord(1:3,1)
         som = dot_product(t2,t2)
         if ( som<=0d0 ) go to 160
         som = sqrt(som)
         t2 = t2/som

!        --- check angle between t1 and t2* &

         produc = 0d0
         do i = 1, 3
            produc = produc+t1(i)*t2(i)
         end do
         if ( abs(prodsv)>abs(produc) ) then
            prodsv = produc
            nodsav = nodp
            sumsav = som
         end if

         if ( abs(produc)<0.5d0 ) goto 210

160   end do

!     --- No point found such that the angle is large enough

      if ( abs(prodsv)<0.999999d0 ) then

!     --- Inner product less than .99
!         use nodsav as nodal point to compute second tangential vector

         t2 = bcord(1:3,nodsav) - bcord(1:3,1)

         t2 = t2 / sumsav
         produc = prodsv

      else

!     --- All inner products are larger than .99, so all angles are too small

         call errint ( isurf, 1 )
         call errsub ( 1788, 1, 0, 0 )

      end if

!     --- t2* found   compute t2

210   som = 0d0

      t2 = t2-produc*t1
      som = sqrt(dot_product(t2,t2))
      t2 = t2/som

!     --- find normal vector

      call ercompnormvec ( t1, t2, t3 )

      det = ercdet ( t1, t2, t3 )  ! determinant

      if ( det<0 ) t3 = -t3

!     --- compute inverse of transformation

      call mshtransmat ( t1, t2, t3, trans )

      if ( debug ) then
         call prinrl ( t1, 3, 't1' )
         call prinrl ( t2, 3, 't2' )
         call prinrl ( t3, 3, 't3' )
      end if  ! ( debug )

!     --- fill array bcorlc with transformed co-ordinates
!         check whether points are in a plane

      maxsum = 0d0
      do nodp = 1, nbndpt

         do i = 1, 2

            bcorlc(i,nodp) = &
               sum((bcord(1:3,nodp)-bcord(1:3,1))*trans(i,1:3))

!           --- check whether all points are in a plane

            if ( nodp/=1 ) then
               som = &
                  sum((bcord(1:3,nodp)-bcord(1:3,nodp-1))*trans(3,1:3))

               maxsum = max(maxsum,abs(som))

               if ( debug ) then
                  write(irefwr,1) 'nodp', nodp
                  write(irefwr,2) 'som', som
               end if  ! ( debug )

            end if

         end do

         if (ibcrds==5) then

!        --- in case of paver copy additional info per point

            bcorlc(3,nodp) = bcord(4,nodp)
            bcorlc(4,nodp) = bcord(5,nodp)

         end if

      end do  ! nodp = 1, nbndpt

      if ( maxsum>5d6*sqr ) then

!     --- maxsum>sqr, give error message

         call erreal ( maxsum, 1 )
         call erreal ( sqr, 2 )
         call errint ( isurf, 1 )
         call errsub ( 279, 1, 2, 0 )

      else  if ( maxsum>sqr ) then

!     --- surface close to plane, map boundary onto plane

         call mshcopboun2pl ( bcord, nbndpt, trans, t3, t1, t2 )

!        --- Map 3d boundary onto plane

         do nodp = 1, nbndpt

            do i = 1, 2

               bcorlc(i,nodp) = &
                  sum((bcord(1:3,nodp)-bcord(1:3,1))*trans(i,1:3))

            end do  ! i = 1, 2

         end do  ! nodp = 1, nbndpt

      end if  ! ( maxsum>sqr )

!     --- Store first point into bcorut

      bcorut = bcord(1:3,1)

1000  call erclos ( 'msh009' )
      if ( debug ) then

!     --- Debug information

         call prinrl1 ( bcorlc, nbndpt, 2, 'bcorlc' )
         call prinrl ( t1, 3, 't1' )
         call prinrl ( t2, 3, 't2' )
         write(irefwr,*) 'End msh009'

      end if  ! ( debug )

      end
