      subroutine msh011 ( rpin, &
                          holeinfo, maxholes, nholes, new_surf, &
                          boundary, numcurvboun, doublecurvs, &
                          iinput )
! ======================================================================
!
!        programmer    Guus Segal
!        version 15.1  date 30-11-2014 Change isurf 23 in 99
!        version 15.0  date 25-11-2012 Remove inplem from parameter list
!        version 14.0  date 02-09-2010 Remove intarmsh
!
!   copyright (c) 1982-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill local array bcord with the coordinates
!     For isurf = 19 the thickness and factor are also filled.
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer maxholes, holeinfo(2,0:maxholes+1), &
              nholes, boundary(2,ncurvs+10), numcurvboun, iinput(*)
      double precision rpin(4,*)
      logical new_surf, doublecurvs

!     boundary       o    Array of size 2 x numcurvboun containing
!                         information of the boundary of the surface
!                         (1,i) contains the i-th curve number (including sign)
!                         (2,i) contains the starting address in kbndpt
!     doublecurvs    o    If true the boundary contains double curves
!     holeinfo       i    Integer array of size 2 x (nholes+2)
!                         Contains information about holes in surfaces and where
!                         a new part of the boundary starts
!                         Pos (1,i-1) contains the local sequence number of the
!                               first curve of part i provided with a sign
!                               If the sign is positive the boundary is
!                               created counter clockwise, otherwise clockwise
!                         Pos (2,i-1) contains the local sequence number of the
!                               first node of of part i
!                         Pos (1,nholes+1) contains the number of curves of the
!                         boundary
!     iinput         i    integer input array, standard for subroutine MESH
!     maxholes       i    Maximum number of holes found in a boundary of a
!                         surface
!     new_surf       i    If true it concerns a new surface, &
!                         otherwise it is a surface to be copied
!     nholes         i    Number of holes in the boundary of a surface
!     numcurvboun    i    Number of curves in the boundary
!     rpin           i    Thickness and factor stored per curve, each curve
!                    i    has 2 factors and thicknesses dependent on side of
!                         curve
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iex, nbc, nrep, i2, ja, j, k, ns, istart, lenbcord, &
              jsurf, ndefsf, irep, i, jcurv, i1, nloc, iprec, &
              isurfc, jump(4), ithk, ifac, iwork(ncurvs)
      double precision dx, d1, d2, d3, e1, e2, e3, xbegin, ybegin, &
                       zbegin, xend, yend, zend, tend, fend
      logical holepermitted

!     d1
!     d2
!     d3
!     dx             length of vector in x-direction
!     e1
!     e2
!     e3
!     fend           End value of parameter f
!     holepermitted  If true it is possible that the outer boundary consists
!                    of disjoint parst
!     i              Counting variable
!     i1             First point of curve in KMESH part m
!     i2             Last point of curve in KMESH part m
!     iex
!     ifac           Indicates if second or fourth position of array rpin
!                    must be used
!     iprec          Surface number of surface to be copied etc.
!     irep           Loop variable
!     istart         First position used at a part
!     isurfc         Type of the surface to be copied
!     ithk           Indicates if first or third position of array rpin
!                    must be used
!     iwork          integer work array of length ncurvs to mark curves
!     j              Counting variable
!     ja
!     jcurv          Absolute value of curve number
!     jsurf
!     jump           In this array the relative nodal point number of the last
!                    point in a closed boundary is stored
!     k              Counting variable
!     lenbcord       Length of array bcord
!     nbc            Number of points along a side
!     ndefsf         Number of curves in surface
!     nloc
!     nrep           Number of parts of curves the boundary consists of
!                    For a closed boundary nrep =1, for a pipe surface 3 or 4
!     ns             Starting address of information of specific surface in
!                    array KMESHS
!     tend           End value of parameter t
!     xbegin         x-coordinate of begin point of curve
!     xend           x-coordinate of end point of curve
!     ybegin         y-coordinate of begin point of curve
!     yend           xycoordinate of end point of curve
!     zbegin         z-coordinate of begin point of curve
!     zend           z-coordinate of end point of curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     MSH090         Check whether the points with coordinates (xend,yend,zend)
!                    and (curves (ih+1,2,3)) are different
!     MSHFILLEXTCRVS Fill coordinates of extra curves in bcord
!     PRININ1        print 2d integer array
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     622   coinciding points in boundary of surface
!     909   More than four subdomains in TRIANGLE command
!     922   Surface generator SIMILAR is used while the number of boundary
!           points in the surface to be copied is not the same as the number
!           of points in the new boundary.
!    1764   first point in part of boundary is not equal to last point
!    3224   Array length too small
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh011' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh011'
         write(irefwr,1) 'isurnr, nbndpt, nsurfs', isurnr, nbndpt, nsurfs
         write(irefwr,1) 'maxholes', maxholes
         call prinrl1 ( curvesmsh, size(curvesmsh,2), ndim, 'curvesmsh' )
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Get constants

      iex  = 0
      nbc  = 1
      nrep = 1
      ns = nsurfs+2+(isurnr-1)*8
      zbegin = 0d0
      zend = 0d0
      istart = kmeshs(isurnr)
      jsurf = kmeshs(ns)
      if ( jsurf==24 ) jsurf = 7    ! compound surface
      isurfc = jsurf
      lenbcord = size(bcordtmp,2)
      if ( debug ) &
         write(irefwr,1) 'isurnr, jsurf, isurfc', isurnr, jsurf, isurfc
      holepermitted = jsurf==3 .or. jsurf==19 .or. jsurf==8 .or. &
                      jsurf==9 .or. jsurf==14 .or. jsurf==16 .or. &
                      jsurf==7 .or. jsurf==11 .or. jsurf==22 .or. &
                      jsurf==99
      numcurvboun = 0        ! number of curves in boundary
      doublecurvs = .false.  ! no double curves yet
      iwork = 0              ! Clear array iwork

      if ( jsurf==7 .or. jsurf==11 ) then

!     --- jsurf = 7 or 11   surface of surfaces

         ndefsf = kmeshs(ns+2)

      else if ( jsurf==10 .or. jsurf==12 ) then

!     --- jsurf = 10 or 12

         if ( kmeshs(ns+5)==0 ) then
            nrep = 3
         else
            nrep = 4
         end if

      else if ( jsurf==8 .or. jsurf==9 .or. jsurf==14 .or. &
                jsurf==16 ) then

!     --- jsurf = 8, 9, 14, surface must be copied

         iprec  = kmeshs(ns+3)
         isurfc = kmeshs(nsurfs+2+(iprec-1)*8)
         if ( debug ) &
            write(irefwr,1) 'iprec, isurfc', iprec, isurfc
         if ( isurfc==10 .or. isurfc==12 ) then

!        --- Originating surface is of type pipe surface, set nrep

            if ( kmeshs(nsurfs+2+(iprec-1)*8+5)==0 ) then
               nrep = 3
            else
               nrep = 4
            end if
            ns = ns + 2

         end if
         ndefsf = kmeshs(isurnr+1) - istart

      else

         ndefsf = kmeshs(isurnr+1) - istart

      end if

      do irep = 1, nrep
         if ( nrep>1 .and. (jsurf==10 .or. jsurf==12 .or. &
              isurfc==10 ) ) ndefsf = kmeshs(ns+1+irep)
         do i = 1, ndefsf
            icurnr = kmeshs(istart+i)
            jcurv = abs(icurnr)

!           --- Store curve number with sign in boundary

            numcurvboun = numcurvboun+1      ! raise number of curves in bound
            call inichkarlen ( numcurvboun, ncurvs+10, 'boundary' )
            if ( ierror/=0 ) go to 1000
            boundary(1,numcurvboun) = icurnr ! store curve number
            if ( iwork(jcurv)==0 ) then

!           --- curve icurnr not yet found

               iwork(jcurv) = 1 ! mark curve as found

            else

!           --- curve icurnr already found

               doublecurvs = .true. ! double curve found

            end if  ! ( iwork(jcurv)==0 )

            if ( debug ) then

!           --- Debug information

               write(irefwr,1) 'irep, i, icurnr', irep, i, icurnr

            end if

            if ( icurnr<0 ) then
               ithk = 1
               ifac   = 2
            else
               ithk = 3
               ifac   = 4
            end if

            if ( jcurv==1 ) then

!           --- jcurv = 1

               i1 = 0
               i2 = icurvsmsh(1)
               nloc = i2

            else

!           --- jcurv>1

               i1 = icurvsmsh(jcurv-1)
               i2 = icurvsmsh(jcurv)
               nloc = i2-i1

            end if
            if ( debug ) write(irefwr,1) 'i1, i2, nloc', i1, i2, nloc

            if ( i>1 ) then

!           --- i>1

              nbc = nbc-1
              xend = bcordtmp(1,nbc)
              yend = bcordtmp(2,nbc)
              if ( ndim==3 ) zend = bcordtmp(3,nbc)
              if ( new_surf .and. jsurf==19 ) then
                 tend = bcordtmp(ndim+1,nbc)
                 fend = bcordtmp(ndim+2,nbc)
              end if

            else

!           --- i = 1

              if ( new_surf .and. jsurf==19 ) then
                 tend = rpin(ithk,jcurv)
                 fend = rpin(ifac,jcurv)
              end if

            end if

            if ( icurnr>0 ) then

!           --- icurnr>0 copy bcordtmp forwards

               if ( holepermitted .and. i>1 ) then

!              --- Check for new point

                  ja = 1

                  call msh090 ( curvesmsh, i1+1, ndim, xend, yend, zend, &
                                ja )

                  if ( ja==1 ) then

!                 --- jump in boundary

                     iex = iex + 1
                     holeinfo(1,iex) = i
                     holeinfo(2,iex) = nbc
                     nbc = nbc + 1

!                    --- negative thickness indicates start of new loop
!                        t = (-3t +t ) / 2 = -t

                     if ( new_surf .and. jsurf==19 ) then
                        tend = -3d0 * rpin(ithk,jcurv)
                        fend = rpin(ifac,jcurv)
                     end if

                  end if

               end if

               if ( new_surf .and. jsurf==19 ) then

!              --- fill thickness and factor, first point of subcurve
!                  has average with last

                  if ( nbc+nloc-1> lenbcord ) then
                     call errchr ( 'bcordtmp', 1 )
                     call errint ( lenbcord, 1 )
                     call errint ( nbc+nloc-1, 2 )
                     call errsub ( 3224, 2, 0, 1 )
                     go to 1000
                  end if  ! ( nbc+nloc-1> lenbcord )
                  bcordtmp(ndim+1,nbc) = (rpin(ithk,jcurv)+tend) / 2d0
                  bcordtmp(ndim+2,nbc) = (rpin(ifac,jcurv)+fend) / 2d0
                  do j = 1, nloc-1
                     bcordtmp(ndim+1,j+nbc) = rpin(ithk,jcurv)
                     bcordtmp(ndim+2,j+nbc) = rpin(ifac,jcurv)
                  end do

               end if

               if ( nbc+nloc-1> lenbcord ) then
                  call errchr ( 'bcordtmp', 1 )
                  call errint ( lenbcord, 1 )
                  call errint ( nbc+nloc-1, 2 )
                  call errsub ( 3224, 2, 0, 1 )
                  go to 1000
               end if  ! ( nbc+nloc-1> lenbcord )
               do j = 0, nloc-1
                  do k = 1, ndim
                     bcordtmp(k,j+nbc) = curvesmsh(k,j+i1+1)
                  end do
               end do

            else

!           --- icurnr<0  copy bcordtmp backwards

               if ( holepermitted .and. i>1 ) then

!              --- Check for new point

                  ja = -1

                  call msh090 ( curvesmsh, i2, ndim, xend, yend, &
                                zend, ja )

                  if ( ja==1 ) then

!                 --- jump in boundary

                     nbc = nbc + 1
                     iex = iex + 1
                     holeinfo(1,iex) = i
                     holeinfo(2,iex) = nbc

!                    --- negative thickness indicates start of new loop
!                        t = (-3t +t ) / 2 = -t

                     if ( new_surf .and. jsurf==19 ) then
                        tend = -3d0 * rpin(ithk,jcurv)
                        fend = rpin(ifac,jcurv)
                     end if

                  end if

               end if

               if ( new_surf .and. jsurf==19 ) then

!              --- fill thickness and factor, first point of subcurve
!                  has average with last

                  if ( nbc+nloc-1> lenbcord ) then
                     call errchr ( 'bcordtmp', 1 )
                     call errint ( lenbcord, 1 )
                     call errint ( nbc+nloc-1, 2 )
                     call errsub ( 3224, 2, 0, 1 )
                     go to 1000
                  end if  ! ( nbc+nloc-1> lenbcord )
                  bcordtmp(ndim+1,nbc) = (rpin(ithk,jcurv)+tend) / 2d0
                  bcordtmp(ndim+2,nbc) = (rpin(ifac,jcurv)+fend) / 2d0
                  do j = 1, nloc-1
                     bcordtmp(ndim+1,j+nbc) = rpin(ithk,jcurv)
                     bcordtmp(ndim+2,j+nbc) = rpin(ifac,jcurv)
                  end do

               end if

               if ( nbc+nloc-1> lenbcord ) then
                  call errchr ( 'bcordtmp', 1 )
                  call errint ( lenbcord, 1 )
                  call errint ( nbc+nloc-1, 2 )
                  call errsub ( 3224, 2, 0, 1 )
                  go to 1000
               end if  ! ( nbc+nloc-1> lenbcord )
               do j = 0, nloc-1
                  do k = 1, ndim
                     bcordtmp(k,nbc+j) = curvesmsh(k,i2-j)
                  end do
               end do

            end if

            if ( i>1 .and. .not. holepermitted ) then

!           --- i>1  check first and end point

               xbegin = bcordtmp(1,nbc)
               ybegin = bcordtmp(2,nbc)
               if ( ndim==3 ) zbegin = bcordtmp(3,nbc)
               d1 = abs(xbegin-xend)
               d2 = abs(ybegin-yend)
               e1 = 1d-4*(abs(xbegin)+abs(xend))
               e2 = 1d-4*(abs(ybegin)+abs(yend))
               if ( e1<epsmac) e1 = sqreps
               if ( e2<epsmac) e2 = sqreps
               d3 = 0d0
               e3 = d3
               if ( ndim==3 ) then

!              --- ndim = 3

                  d3 = abs(zbegin-zend)
                  e3 = sqreps*(abs(zbegin)+abs(zend))
                  if ( e3<epsmac ) e3 = sqreps

               end if

               if ( d1>e1.or. d2>e2 .or. d3>e3 ) then
                  call errint(1, 1)
                  call errint(nbc, 2)
                  call errint(jcurv, 3)
                  call errint(isurnr, 4)
                  call erreal(xbegin, 1)
                  call erreal(ybegin, 2)
                  call erreal(zbegin, 3)
                  call erreal(xend, 4)
                  call erreal(yend, 5)
                  call erreal(zend, 6)
                  call errsub ( 1764, 4, 6, 0)
               end if

            end if
            boundary(2,numcurvboun) = nbc  ! store starting address of curve
                                           ! in boundary
            nbc = nbc+nloc

         end do  ! i = 1, ndefsf
         istart = istart + ndefsf
         jump(irep) = nbc-1

      end do  ! irep = 1, nrep

      if ( .not. (holepermitted .or. isurfc==3 .or. isurfc==7 .or. &
           isurfc==10 .or. isurfc==12 .or. isurfc==19 .or. &
           isurfc==22 .or. isurfc==99) ) then

!     --- jsurf # 3, 7, 8, 9, 10, 12  Check initial and end point

         xbegin = bcordtmp(1,1)
         ybegin = bcordtmp(2,1)
         xend = bcordtmp(1,nbc-1)
         yend = bcordtmp(2,nbc-1)
         d1 = abs(xbegin-xend)
         d2 = abs(ybegin-yend)
         e1 = 1d-4*(abs(xbegin)+abs(xend))
         e2 = 1d-4*(abs(ybegin)+abs(yend))
         if ( e1<epsmac ) e1 = sqreps
         if ( e2<epsmac ) e2 = sqreps
         d3 = 0d0
         e3 = d3
         if ( ndim==3 ) then

!        --- ndim  = 3

            zbegin = bcordtmp(3,1)
            zend = bcordtmp(3,nbc-1)
            d3 = abs(zbegin-zend)
            e3 = sqreps*(abs(zbegin)+abs(zend))
            if ( e3<epsmac) e3 = sqreps

         end if

         if ( d1>e1 .or. d2>e2 .or. d3>e3 ) then
            call errint(1, 1)
            call errint(nbc, 2)
            call errint(jcurv, 3)
            call errint(isurnr, 4)
            call erreal(xbegin, 1)
            call erreal(ybegin, 2)
            call erreal(zbegin, 3)
            call erreal(xend, 4)
            call erreal(yend, 5)
            call erreal(zend, 6)
            call errsub ( 1764, 4, 6, 0)

         end if
         if ( debug ) write(irefwr,1) 'irep, nbc', irep, nbc

      end if

!     --- check for double points

      icurnr = 0
      irep = 1

      do i = 1, nbc-1-nrep
         icurnr = icurnr + 1
         dx = abs(bcordtmp(1,icurnr)-bcordtmp(1,icurnr+1))
         do k = 2, ndim
            dx = dx+abs(bcordtmp(k,icurnr)-bcordtmp(k,icurnr+1))
         end do
         if ( dx<=0d0 ) then
            call errint ( isurnr, 1 )
            call errint ( i, 2 )
            call erreal ( bcordtmp(1,icurnr), 1 )
            call erreal ( 0d0, 2 )
            call erreal ( 0d0, 3 )
            if ( ndim>1 ) call erreal ( bcordtmp(2,icurnr), 2 )
            if ( ndim>2 ) call erreal ( bcordtmp(3,icurnr), 3 )
            call errsub ( 622, 2, 3, 0 )
         end if
         if ( isurfc==10 .or. isurfc==12 ) then

!        --- Pipe surface, jump if end of part is reached

            if ( icurnr==jump(irep)-1 ) then
               irep = irep + 1
               icurnr = icurnr + 1
            end if
         end if

      end do

!     --- Special check for isurfc = 3 and isurfc = 14

      if ( isurfc==3 .or. isurfc==19 .or. isurfc==22 .or. &
           isurfc==99 ) then

         if ( debug ) then
             write(irefwr,1) 'nbndpt, maxholes, iex', &
                              nbndpt, maxholes, iex
         end if  ! ( debug )
         nbndpt = nbndpt - maxholes + iex
         if ( debug ) write(irefwr,1) 'nbndpt/2', nbndpt

         if ( iex>maxholes ) then

!        --- Remark : Extra length for isurf = 3 is maxholes
!                     points. If more is needed than also MSH064 has to be
!                     adjusted.

            call errint ( iex, 1 )
            call errint ( maxholes, 2 )
            call errsub ( 909, 2, 0, 0)

         end if

      else if ( isurfc==14 ) then

         if ( nbndpt/=nbc-1 ) then

!        --- Remark : Number of boundary points in second mesh
!                     differs from number of points in boundary
!                     of first surface.

            call errint ( isurnr, 1 )
            call errsub ( 922, 1, 0, 0 )

         end if

      end if
      if ( debug ) then

!     --- debug information

         if ( iex>0 ) call prinin1 ( holeinfo, iex+2, 2, 'holeinfo' )
         call prinrl1 ( bcordtmp, nbndpt, ibcrds, 'bcordtmp' )

      end if
      nholes = iex
      holeinfo(1,0) = 1
      holeinfo(2,0) = 1
      holeinfo(1,nholes+1) = ndefsf+1
      holeinfo(2,nholes+1) = nbc

      if ( nextracrvs>0 ) then

!     --- nextracrvs>0, fill extra curves in array bcord from position nbndpt

         call mshfillextcrvs ( icurvsmsh, curvesmsh, bcordtmp(1,nbndpt+1), &
                               iinput, .false., icurvsmsh )

      end if  ! ( nextracrvs>0 )

1000  if ( debug ) then

!     --- Debug information

         call prinin1 ( boundary, numcurvboun, 2, 'boundary' )
         write(irefwr,1) 'nbndpt, nbc', nbndpt, nbc
         write(irefwr,*) 'doublecurvs', doublecurvs
         write(irefwr,*) 'End debug information from msh011'

      end if
      call erclos ( 'msh011' )
      end
