      subroutine msh013_10 ( iinput, iinp, ikkmsq, istar2, kmeshm, &
                             icurnr, kmeshq, ipointcurv )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-02-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    initialize kmesh part m
!    Special case icurve = 10, 11
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iinput(*), iinp, ikkmsq, istar2, &
                             icurnr, kmeshq(*), ipointcurv(*)
      integer, intent(inout) :: kmeshm(*)

!     icurnr         i    Curve sequence number
!     iinp           i    last position used in array iinput
!     iinput         i    integer input array with respect to the curves
!     ikkmsq         i    Indication if array kmesh part q has been filled (>0)
!                         or not ( = 0)
!     ipointcurv     i    Array containing the relative starting addresses
!                         of all curves in the part of iinput referring
!                         to curves
!     istar2         i    Starting address of KMESHM part 2 in KMESHM
!     kmeshm        i/o   KMESH part m, contains nodal point numbers of the
!                         curves
!     kmeshq         i    KMESH part q, contains the subcurve numbers of
!                         each curve
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: nsubcv, j, jcurve, ic, nput, nh, kstep, k, jstart, &
                 ifirst, ilast, n, jinp, icprec, iprec, istart, icurv
      logical :: spline, begend

!     begend         Indicates if begin and end point of curve are equal
!                    (true) or not (false)
!     ic             Relative subcurve number
!     icprec         Type of curve to be "copied"
!     icurv          Curve number with direction
!     ifirst         First curve (surface) number read
!     ilast          Last curve (surface) number read
!     iprec          curve number of surface to be copied etc.
!     istart         First position used at a part
!     j              Counting variable
!     jcurve         Curve number
!     jinp           last position used in array iinput with respect to
!                    iprec
!     jstart         starting position of curve i in kmeshm
!     k              Counting variable
!     kstep          Indicates the direction of the curve
!                    (1: positive, -1:negative)
!     n              number of basis functions for each component
!     nh             number of nodes along a subcurve
!     nput           number of user points that have been put in kmeshm
!     nsubcv         Number of subcurves of a curve
!     spline         Indicates if preceding curve is a spline (true) or not
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh013_10' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh013_10'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iprec = iinput(iinp+4)
      n = iinput(iinp+3)
      if ( ikkmsq/=0 ) then
         nsubcv = kmeshq(iprec+1)-kmeshq(iprec)
      else
         nsubcv = 0
      end if
      jstart = istar2+kmeshm(4+icurnr)-1
      nput = 4
      if ( nsubcv==0 ) then

!     --- nsubcv = 0, no subcurves
!         check if begin and end point of preceding curve iprec coincide

         istart = istar2 + kmeshm(4+iprec) - 1
         nh = kmeshm(5+iprec)-kmeshm(4+iprec)
         begend = kmeshm(istart+1)==kmeshm(istart+nh)

!        --- Check if preceding curve is a spline

         jinp = ipointcurv(iprec)
         icprec = iinput(jinp)
         spline = icprec==4 .or. icprec==12

         do ic = 1, nh
            j = kmeshm(istart+ic)
            if ( j<0 ) then

!           --- kmeshm(istart+ic)<0
!               next user point

               nput = nput+1
               if ( ic>1 .and. spline ) then

!              --- Next point of spline
!                  Since a spline contains only two user points
!                  connected to nodal points, this user point must
!                  be the last one

                  kmeshm(istar2+kmeshm(5+icurnr)-1) = &
                        -abs(iinput(iinp+n+4))

               else if ( nput==n+4 .and. &
                         iinput(iinp+nput)<0 ) then

!              --- Last user point, so this point gets the position
!                  of the last point in curve iprec

                  kmeshm(istar2+kmeshm(5+icurnr)-1) = &
                     iinput(iinp+nput)

               else

!              --- Standard situation

                  kmeshm(jstart+ic) = -iinput(iinp+nput)

               end if
               if ( ic==1 .and. begend ) kmeshm(jstart+nh) = &
                                        kmeshm(jstart+1)
               if ( nput==n+4 ) goto 1000

            end if

         end do

      else

!     --- nsubcv>0, nsubcv subcurves

         icurv = kmeshq( kmeshq(iprec) + 1 )
         ifirst = iinput(iinp+5)
         nh = 1
         do k  = 1, nsubcv
            icurv = abs( kmeshq( abs( kmeshq(iprec)+k ) ) )
            if ( k==1 ) ilast = kmeshm(istar2+kmeshm(icurv+4))
            nh = nh + kmeshm(5+icurv) - kmeshm(4+icurv) - 1
         end do
         if ( ilast==kmeshm(istar2+kmeshm(icurv+5)-1) ) &
              kmeshm(jstart+nh) = -ifirst
         kmeshm(jstart+1) = -ifirst
         nput = nput + 1
         if ( nput==n+4 ) goto 1000
         do k = 1, nsubcv
            icurv = kmeshq( kmeshq(iprec)+k )
            if ( icurv>0 ) then

!           --- icurv>0

               istart = istar2 + kmeshm(icurv+4) - 1
               nh = kmeshm(5+icurv) - kmeshm(4+icurv)
               kstep = 1

            else

!           --- icurv<0

               jcurve = - icurv
               istart = istar2 + kmeshm(jcurve+5)
               nh = kmeshm(5+jcurve) - kmeshm(4+jcurve)
               kstep = - 1

            end if
            do ic = 2, nh
               j = kmeshm(istart+kstep*ic)
               if ( j<0 ) then

!              --- kmeshm(istart+kstep*ic)<0
!                  next user point

                  nput = nput+1
                  if ( nput==n+4 .and. iinput(iinp+nput)<0 ) then

!                 --- Last user point, so this point gets the position
!                     of the last point in curve iprec

                     kmeshm(istar2+kmeshm(5+icurnr)-1) = &
                        iinput(iinp+nput)

                  else

!                 --- Standard situation

                     kmeshm(jstart+ic) = -iinput(iinp+nput)

                  end if

                  if ( nput==n+4 ) goto 1000

               end if

            end do
            jstart = jstart + nh - 1

         end do

      end if

1000  call erclos ( 'msh013_10' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh013_10'

      end if  ! ( debug )

      end subroutine msh013_10
