      subroutine msh017kmsm ( indkbndpt, iinput, ikkmsq, kmeshq, kmeshl, &
                              kmeshm, istar3, istype, kbndpt, indfor )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-03-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Adapt arrays kmeshm, kmeshl and kbndpt for external curves
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iinput(*), ikkmsq, kmeshq(*), istar3, &
                             istype, indfor
      integer, intent(out) :: indkbndpt
      integer, intent(inout) :: kmeshl(*), kmeshm(*), kbndpt(*)

!     iinput         i    integer input array
!     ikkmsq         i    Indication if array kmesh part q has been filled (>0)
!                         or not ( = 0)
!     indfor         i    First index of a closed curve
!     indkbndpt      o    Next position in kbndpt
!     istar3         i    Starting address of KMESHM part 3 in KMESH part M
!     istype         i    Type of surface generator used (istype=10:
!                         pipe surface)
!     kbndpt        i/o   array containing the nodal point numbers of the
!                         boundary
!     kmeshl        i/o   KMESH part l, contains nodal point numbers of the
!                         user points
!     kmeshm        i/o   KMESH part m, contains nodal point numbers of the
!                         curves
!     kmeshq         i    KMESH part q, contains the subcurve numbers of
!                         each curve
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ifirsp, ndefsf, ilastp, i, kcurv, lcurv, ic, nsubcv, &
                 icurv, jcurv, k1, k2, ih, idirec, iknpt, jknpt, k
      logical :: loopfound

!     i              Counting variable
!     ic             Relative subcurve number
!     icurv          Curve number with direction
!     idirec         Defines in which direction the curves must be
!                    considered
!                    idirec = 1, from low to high, idirec = -1: reversed
!                    order
!     ifirsp         Indicates first point of curve
!     ih             Help variable to store a constant temporarily
!     iknpt          Nodal point number
!     ilastp         Indicates last point of curve
!     jcurv          Absolute value of curve number
!     jknpt          Nodal point number
!     k              Counting variable
!     k1             First point of curve in KMESH part m
!     k2             Last point of curve in KMESH part m
!     kcurv          Curve number with direction
!     lcurv          |kcurv|
!     loopfound      If true end point found that is the same as initial point
!     ndefsf         Number of curves in surface
!     nsubcv         Number of subcurves of a curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     617   User point corresponds to two different nodal point numbers.
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh017kmsm' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh017kmsm'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      indkbndpt = 1
      ifirsp = 0
      ndefsf = mod(iinput(3),10000)
      ilastp = 0
      loopfound = .false.
      nsubcv = 0

      do i = 1, ndefsf

         kcurv = iinput(3+i)
         lcurv = abs(kcurv)
         if ( ikkmsq/=0 ) nsubcv = kmeshq(lcurv+1) - kmeshq(lcurv)

!        --- Loop over subcurves

         do ic = 1, max(1, nsubcv)
            if ( nsubcv==0 ) then
               icurv = kcurv
            else
               if ( kcurv>0 ) then
                  icurv = kmeshq( kmeshq(lcurv)+ic )
               else
                  icurv = - kmeshq( kmeshq(lcurv+1)-ic+1 )
               end if
            end if
            jcurv = abs(icurv)
            k1 = kmeshm(4+jcurv)
            k2 = kmeshm(5+jcurv)-1

            if ( icurv>0 ) then

!           --- icurve>0  copy forwards

               ih = indkbndpt
               idirec = 1

            else

!           --- icurve<0  copy backwards

              ih = indkbndpt+k2-k1
              idirec = -1

            end if
            if ( ifirsp==0 .or. loopfound ) then

!           --- ifirsp has not yet been filled

               if ( idirec==1 ) then

!              --- idirec = 1, standard sequence (ifirsp corresponds to k1)

                  ifirsp = kmeshm(istar3+k1)

               else

!              --- idirec =-1, reversed sequence (ifirsp corresponds to k2)

                  ifirsp = kmeshm(istar3+k2)

               end if
               if ( ifirsp<0 ) ifirsp = kmeshl(-ifirsp)
               if ( loopfound .and. ifirsp/=ilastp ) then
                  indkbndpt = indkbndpt+1
                  ih = ih + 1
               end if
               loopfound = .false.

            end if

            if ( idirec==1 ) then

!           --- idirec = 1, standard sequence (ilastp corresponds to k2)

               ilastp = kmeshm(istar3+k2)

            else

!           --- idirec =-1, reversed sequence (ilastp corresponds to k1)

               ilastp = kmeshm(istar3+k1)

            end if
            if ( ilastp<0 ) ilastp = kmeshl(-ilastp)

            do k = k1, k2
               iknpt = kbndpt(ih)
               ih = ih+idirec
               jknpt = kmeshm(istar3+k)
               if ( jknpt<0 ) then

!              --- jknpt<0  fill kmesh l and kmeshm

                  if ( kmeshl(-jknpt)==0 ) then
                     kmeshl(-jknpt) = iknpt
                  else if ( kmeshl(-jknpt)/= iknpt ) then
                     call errint( -jknpt, 1 )
                     call errint( iknpt, 2 )
                     call errint( kmeshl(-jknpt), 3 )
                     call errwar ( 617, 3, 0, 0 )
                  end if
                  kmeshm(istar3+k) = iknpt

               else if ( jknpt==0 ) then

!              --- jknpt = 0  fill kmeshm

                  kmeshm(istar3+k) = iknpt

               end if

            end do

!           --- Check if end of closed boundary is reached, if so raise
!               indkbndpt

            if ( istype==10 ) then

!           --- istype = 10, i.e. pipe surface, special treatment

              if ( ic==max(1,nsubcv) ) indkbndpt = indkbndpt+1

            else if ( (istype==3 .or. istype==19 .or. istype==22 &
                      .or. istype==99) .and. &
                      kbndpt(indkbndpt+k2-k1)==ifirsp ) then

!           --- starting point of new boundary is equal to first point, &
!               this means that end of closed boundary has been found
!               This situation can only occur for istype = 3 (triangle)

               loopfound = .true.
               if ( i==ndefsf .and. ic==max(1,nsubcv) ) then
                  kbndpt(indkbndpt+k2-k1) = -indfor
                  indkbndpt  = indkbndpt + 1
                  ifirsp = 0
               end if

            end if

            indkbndpt = indkbndpt+k2-k1

         end do

      end do

1000  call erclos ( 'msh017kmsm' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh017kmsm'

      end if  ! ( debug )

      end subroutine msh017kmsm
