      subroutine msh030 ( icurnr, ncurvs, iinput, iinp, rinput, icurve, &
                          nelm, curstr, rincor, ndim, nuspnt, edge, &
                          irelcurves, istartcur, istartrel )
! ======================================================================
!
!        programmer    Niek Praagman
!        version 10.1  date 10-02-2015 Extension with ccircle
!        version 10.0  date 11-12-2014 Extra parameters
!        version  9.3  date 08-12-2014 Extension of segmented lines
!
!   copyright (c) 1984-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     determine number of elements to be created along
!     curves with given coarseness
! **********************************************************************
!
!                       KEYWORDS
!
!     create_point
!     curve
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: icurnr, ncurvs, iinp, icurve, ndim, nuspnt
      integer, intent(out) :: nelm
      integer, intent(inout) :: istartcur, irelcurves(*), istartrel, &
                                iinput(*)
      double precision, intent(in) :: rinput(ndim,nuspnt), rincor(*), &
                                      edge
      double precision, intent(inout) :: curstr(*)

!     curstr        i/o   Contains real information of the curves
!                         See subroutine mshcrtcurv
!     edge           i    Unit length defined for coarseness
!     icurnr         i    curve number
!     icurve         i    indication of the type of curve to be considered
!                          5: straight line
!                          6: circle
!                          9: parameter fuction
!                         12: spline
!     iinp           i    starting position of curve input in array iinput
!     iinput        i/o   array containing the user input concerning the
!                         curve from position iinp
!     irelcurves    i/o   Contains integer information of the curves
!                         See subroutine mshcrtcurv
!     istartcur     i/o   next free position in curstr
!     istartrel     i/o   next free position in irelcurves
!     ncurvs         i    number of curves
!     nelm           o    number of elements on curve
!     rincor         i    Part of array rinput refering to coarseness
!     rinput         i    array containing on the first nuspnt*ndim positions
!                         coordinates, on position nuspnt+1 the unit edge length
!                         and on the next nuspnt positions the defined
!                         coarseness of the user points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer np, p1, p2
      double precision stotal, x1(ndim), x2(ndim), edgemn, sleng

!     edgemn         Mean length of edges
!     ndim           Dimension of the space.
!     np             Defines whether the number of points is odd (2) or
!                    even (3)
!     nuspnt         Number of user points
!     p1             user point number of first point
!     p2             user point number of second point
!     sleng          length of the curve
!     stotal         total length of all segments together
!     x1             coordinates of first point
!     x2             coordinates of second point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSH030CCIRCLE  determine number of elements to be created along
!                    circle with more than ndim points
!     MSH030CIRC     determine number of elements to be created along
!                    circle with given coarseness
!     MSH030ELLIPS   determine number of elements to be created along
!                    ellips with given coarseness
!     MSH030GEN      determine number of elements to be created along
!                    general part for some types of curves
!     MSH030LINESEGM Compute number of elements along a set of line segments
!                    defined by cline
!     MSH030PARM     determine number of elements to be created along
!                    parameter function with given coarseness
!     MSH030PROF     determine number of elements to be created along
!                    profile with given coarseness
!     MSH030SPLINE   determine number of elements to be created along
!                    spline with given coarseness
!     MSHCOMPNELM    Compute number of the number of elements given the length
!                    of the curve
!     MSHNFRAMECUR   determine number of elements to be created along
!                    polygon-track in case of framecurve
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     124   code for generation of curves (icurve) incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh030' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh030'
         write(irefwr,1) 'icurnr, icurve, ncurvs, iinp', &
                          icurnr, icurve, ncurvs, iinp
         write(irefwr,2) 'edge', edge
         write(irefwr,1) 'istartcur, istartrel', istartcur, istartrel
         call prinin ( iinput(iinp), 10, 'iinput' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      np     = 0
      if ( debug ) &
         write(irefwr,1) 'ndim, nuspnt', ndim, nuspnt

      if ( icurve==12 ) then

!     --- icurve = 12: spline

         call msh030spline ( rinput, nuspnt, iinput, rincor, nelm, &
                             ndim, icurnr, edge, stotal, iinp, np )
         curstr(istartcur) = stotal
         irelcurves(icurnr+ncurvs) = istartcur
         istartcur = istartcur+1

      else if ( icurve==21 ) then

!     --- icurve = 21: framecurve
!         call routine to determine number of elements (nelm) on curve
!         and total piecewise Euclidian length (stotal) of curve

         call mshnframecur ( rincor, nelm, icurnr, stotal )
         curstr(istartcur) = stotal
         irelcurves(icurnr+ncurvs) = istartcur
         istartcur = istartcur+1

      else if ( icurve==5 ) then

!     --- icurve = 5: cline
!         call routine to determine number of elements (nelm) on curve
!         and total piecewise Euclidian length (stotal) of curve

         call msh030linesegm ( rinput, iinput, iinp, nelm, rincor, np, &
                               icurnr, istartcur, curstr, irelcurves, &
                               istartrel )

      else if ( icurve==24 ) then

!     --- icurve = 24: ccircle
!         call routine to determine number of elements (nelm) on curve

         call msh030ccircle ( rinput, iinput, rincor, nelm, ndim, &
                              icurnr, edge, iinp, np, curstr(istartcur), &
                              istartcur, irelcurves, istartrel )

      else

!     --- icurve # 12

         if ( icurve==16 ) then
            p1 = iinput(iinp+5)
            p2 = iinput(iinp+6)
         else
            p1 = iinput(iinp+4)
            p2 = iinput(iinp+5)
         end if
         call msh030gen ( rinput, p1, p2, x1, x2, ndim, edge, &
                          edgemn, rincor, nuspnt, icurve, icurnr )

         select case ( icurve )

         case(6)

!        --- icurve = 6, arc

            call msh030circ ( rinput, nuspnt, iinput, ndim, iinp, np, &
                              x1, x2, sleng, curstr(istartcur), icurnr )

         case(9)

!        --- icurve = 9, parameter function

            call msh030parm ( rincor(3+ncurvs+nuspnt), iinput, iinp, &
                              icurnr, ndim, x1, x2, sleng, edgemn, np )

         case(16)

!        --- icurve = 16: cprofile

            call msh030prof ( rincor(3+ncurvs+nuspnt), iinput, iinp, &
                              icurnr, ndim, x1, x2, sleng, edgemn, np )

         case(23)

!        --- icurve = 23, cell_arc
!            Get standard parameters

            call msh030ellips ( iinput, iinp, np, rinput, nuspnt, &
                                x1, x2, curstr(istartcur), icurnr, &
                                sleng )

         case default

!        --- incorrect value, error

            call errint ( icurve, 1 )
            call errint ( 23, 2 )
            call errsub ( 124, 2, 0, 0 )
            go to 1000

         end select  ! case ( icurve )
         if ( ierror/=0 ) go to 1000

         curstr(istartcur) = sleng
         irelcurves(icurnr+ncurvs) = istartcur
         istartcur = istartcur+1

         if ( icurve==6 .and. ndim==3 ) then
            istartcur = istartcur+5
         else if ( icurve==6 .or. icurve==23 ) then
            istartcur = istartcur+4
         end if  ! ( icurve==6 .and. ndim==3 )

         call mshcompnelm ( p1, p2, rincor, edge, sleng, nelm )

      end if

      nelm = max ( nelm, 1 )
      if ( np==2 ) then

!     --- np = 2 , odd number of points

         if ( mod(nelm,2)/=0 ) nelm = nelm+1

      else if ( np==3 ) then

!     --- np = 3 , even number of points

         if ( mod(nelm,2)==0 ) nelm = nelm+1

      end if

!     --- Fill result in iinput

      iinput(iinp+2) = nelm

1000  call erclos ( 'msh030' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'nelm, ncurvs', nelm, ncurvs
         write(irefwr,1) 'istartcur, istartrel', istartcur, istartrel
         call prinin ( irelcurves, istartrel, 'irelcurves' )
         write(irefwr,*) 'End msh030'

      end if  ! ( debug )

      end
