      subroutine msh030parm ( rincor, iinput, iinp, icurnr, ndim, &
                              x1, x2, sleng, edgemn, np )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 31-07-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     determine number of elements to be created along
!     parameter function with given coarseness
! **********************************************************************
!
!                       KEYWORDS
!
!     create_point
!     curve
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iinp, icurnr, ndim, np
      double precision rincor(2,*), x1(ndim), x2(ndim), sleng, edgemn

!     edgemn         i    Mean length of edges
!     icurnr         i    Curve sequence number
!     iinp           i    starting position of curve input in array iinput
!     iinput         i    array containing the user input concerning the
!                         curve from position iinp
!     ndim           i    Dimension of the space.
!     np             o    Defines whether the number of points is odd (2) or
!                         even (3)
!     rincor         i    Part of array rinput refering to coarseness
!     sleng          o    length of the curve
!     x1             i    coordinates of first point
!     x2             i    coordinates of second point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ncheck, kmax, k
      double precision t0, t1, dt, x(3), xp(ndim), t, v(ndim)

!     dt             Step size of parameter t
!     k              Counting variable
!     kmax           Number of steps of parameter t
!     ncheck         Checking parameter
!     t              parameter t along curve
!     t0             Initial value of t
!     t1             End value of t
!     v              differnce between two vector
!     x              contains co-ordinates of point
!     xp             contains co-ordinates of prior point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     FUNCCV         User function to define the curve in parameter form
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     423   initial and end parameter for curve generation are the same
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh030parm' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh030parm'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      t0 = rincor(1,icurnr)
      t1 = rincor(2,icurnr)
      np = iinput(iinp+6)
      dt = 0.1d0*(t1-t0)
      x  = 0d0
      ncheck = 10
      kmax = 10

      if ( debug ) then
         write(irefwr,1) 'np', np
         write(irefwr,2) 't0, t1, dt', t0, t1, dt
      end if  ! ( debug )

!     --- Loop

100   dt = 0.1d0*dt
      kmax = kmax*10
      if ( abs(dt)<=0d0 ) then

!     --- Error dt<=0

         call errint ( icurnr, 1 )
         call errsub ( 423, 1, 0, 0 )

      else

!     --- Standard case

         xp = x1
         sleng = 0d0

         do k = 1, kmax-1

            t = t0+k*dt
            call funccv ( icurnr, t, x(1), x(2), x(3) )
            if ( ndim==1 ) then

!           --- ndim = 1

               sleng = sleng+abs(x(1)-xp(1))

            else

!           --- ndim = 2, 3

               v = x-xp
               sleng = sleng+sqrt(dot_product(v,v))

            end if  ! ( ndim==1 )

            xp(1:ndim) = x(1:ndim)

            if ( debug ) then
               write(irefwr,1) 'k', k
               write(irefwr,2) 'sleng', sleng
            end if  ! ( debug )

         end do  ! k = 1, kmax-1

         if ( ndim==1 ) then

!        --- ndim = 1

            sleng = sleng+abs(x2(1)-xp(1))

         else

!        --- ndim = 2, 3

            v = x2-xp
            sleng = sleng+sqrt(dot_product(v,v))

         end if  ! ( ndim==1 )

         if ( sleng/edgemn>ncheck) then

            ncheck = 10*ncheck
            goto 100

         end if

      end if  ! ( abs(dt)<=0d0 )

1000  call erclos ( 'msh030parm' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh030parm'

      end if  ! ( debug )

      end
