      subroutine msh031circsegm ( rinput, iinput, iinp, nelm, ishape, &
                                  icurnr, work, maxratio, tau, n, &
                                  irelcurves )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-02-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array tau for a circle with more than ndim user points
!     Also adapt irelcurves
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh_generation
!     curve
!     point_distribution
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iinp, iinput(*), icurnr, nelm, ishape
      integer, intent(inout) :: irelcurves(*)
      integer, intent(out) :: n
      double precision, intent(in) :: rinput(*), work(*), maxratio
      double precision, intent(out) :: tau(*)

!     icurnr         i    Curve sequence number
!     iinp           i    last position used in array iinput
!     iinput         i    array containing the user input concerning the
!                         curve from position iinp
!     irelcurves    i/o   Contains integer information of the curves
!                         See subroutine mshcrtcurv
!     ishape         i    Shape number of elements
!     maxratio       i    max allowed coarseness ratio
!     n              o    number of basis functions for each component
!     nelm           i    number of elements along curve
!     rinput         i    array containing on the first nuspnt*ndim positions
!                         coordinates, on position nuspnt+1 the unit edge length
!                         and on the next nuspnt positions the defined
!                         coarseness of the user points
!     tau            o    Work array of length n to store the relative distances
!                         between points
!     work           i    Work array containing the distribution of nodes
!                         along one side of an element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: numuserp, nuspnt, ndim, isegment, nelmloc, p1, p2, &
                 nelm1, nloc, istartcur, istartrel, ncurvs
      double precision :: tauloc(2*nelm)

!     isegment       sequence number of segment
!     istartcur      starting position curstr
!     istartrel      starting position irelcurves
!     ncurvs         Number of curves in mesh
!     ndim           Dimension of the space.
!     nelm1          Accumulated number of elements along this and prior
!                    segments
!     nelmloc        Number of elements along one segment
!     nloc           Number of nodes along one segment
!     numuserp       Number of user points along curve
!     nuspnt         Number of user points
!     p1             user point number of first point
!     p2             user point number of second point
!     tauloc         Array tau for one segment
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHLINE1SEGM   Fill array tau and compute sleng for a line with 1 segment
!     PRININ1        print 2d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh031circsegm' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh031circsegm'
         write(irefwr,1) 'nelm, iinp, icurnr', nelm, iinp, icurnr
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get some parameters

      ndim = iinput(1)
      nuspnt = iinput(2)
      ncurvs = iinput(3)

      if ( debug ) then
         write(irefwr,1) 'ndim, nuspnt, ncurvs', ndim, nuspnt, ncurvs
         call prinin1 ( irelcurves, 2, ncurvs, 'irelcurves' )
      end if  ! ( debug )

!     --- Set starting addresses

      istartrel = irelcurves(icurnr)
      istartcur = irelcurves(ncurvs+icurnr)

      if ( debug ) then
         write(irefwr,1) 'istartrel, istartcur', istartrel, istartcur
      end if  ! ( debug )

      n = 0

      numuserp = iinput(iinp+3)-1  ! number of user points along curve

      if ( debug ) then
         write(irefwr,1) 'ndim, nuspnt, numuserp', &
                          ndim, nuspnt, numuserp
      end if  ! ( debug )

      nelm1 = 0

      tau(1) = 0d0
      p2 = iinput(iinp+5)           ! point on circle

!     --- Loop over the segments

      do isegment = 2, numuserp

         p1 = p2  ! first user point
         p2 = iinput(iinp+4+isegment)

!        --- Fill x1, x2 and compute edge1, edge2 and edgemn

         nelmloc = irelcurves(istartrel+isegment-1) ! number of elements
         nelm1 = nelm1+nelmloc  ! accumulated

         if ( debug ) then
            write(irefwr,1) 'isegment, nelmloc, p1, p2', &
                             isegment, nelmloc, p1, p2
         end if  ! ( debug )

!        --- Fill array tauloc

         call mshline1segm ( icurnr, maxratio, rinput, nelmloc, ishape, &
                             nuspnt, nloc, ndim, tauloc, work, p1, p2 )

         if ( debug ) call prinrl ( tauloc, nloc, 'tauloc' )

!        --- Copy tauloc into tau

         tau(n+2:n+nloc) = tauloc(2:nloc)

         n = n+nloc-1
         irelcurves(istartrel+isegment-1) = n

      end do  ! isegment = 2, numuserp

!     --- Consider last part of the circle, from end point to initial point

      p1 = p2
      p2 = iinput(iinp+5)

      nelmloc = irelcurves(istartrel+numuserp) ! number of elements
      nelm1 = nelm1+nelmloc  ! accumulated

      if ( debug ) &
         write(irefwr,1) 'nelm1, nelm, nelmloc', nelm1, nelm, nelmloc

      if ( nelm1<nelm ) nelmloc = nelmloc+1

!     --- Fill array tauloc

      call mshline1segm ( icurnr, maxratio, rinput, nelmloc, ishape, &
                          nuspnt, nloc, ndim, tauloc, work, p1, p2 )

      if ( debug ) call prinrl ( tauloc, nloc, 'tauloc' )

!     --- Copy tauloc into tau

      tau(n+2:n+nloc) = tauloc(2:nloc)
      irelcurves(istartrel+numuserp) = n+nloc-1

1000  call erclos ( 'msh031circsegm' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'n, nelm', n, nelm
         call prinrl ( tau, n, 'tau' )
         write(irefwr,*) 'End msh031circsegm'

      end if  ! ( debug )

      end subroutine msh031circsegm
