      subroutine msh055surf7 ( ncurvs, isurnr, kmeshr, kmeshs, kmeshsp2, &
                               iprec, isurf, holes, kmeshm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 20-10-2008 Allow isurf = 24
!        version  1.0  date 08-02-2004
!
!   copyright (c) 2004-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Sort outer boundary of surfaces of surfaces in order to get a
!    complete continuous outer boundary (isurf = 7 or 11)
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     sort
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ncurvs, isurnr, kmeshr(*), kmeshsp2(8,*), &
              kmeshs(*), iprec, isurf, kmeshm(*)
      logical holes

!     holes         i/o   If true, a hole in the surface has been found
!     iprec          i    Surface number of surface to be copied etc.
!     isurf          i    Type of surface with respect to mesh generator
!     isurnr         i    Surface sequence number
!     kmeshm         i    KMESH part m, contains nodal point numbers of the
!                         curves
!     kmeshr         i    Kmesh part r, see the PG 13.1
!     kmeshs        i/o   Kmesh part s, contains information about the various
!                         surfaces. See the PG 13.1
!     kmeshsp2      i/o   Second part of kmeshs, referring to surfaces
!     ncurvs         i    Number of curves in mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nsubsf, krep, ninter, ioutercount, kstart, k, ksurnr, &
              istart, nc, ic, kcur, firstpnt, lastpnt, istar3, j, &
              iwork(ncurvs), iwork1(ncurvs), iwork2(ncurvs)

!     firstpnt       First point of a part
!     i              Counting variable
!     ic             Relative subcurve number
!     ioutercount    Counter for number of disjoint parts the outer boundary
!                    consists of
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     istart         First position used at a part
!     iwork          integer work array
!                    iwork(i) = 1, means that curve i is outer curve
!     iwork1         integer work array
!                    iwork1(i) Contains first node of curve i
!     iwork2         integer work array
!                    iwork2(i) Contains last node of curve i
!     j              Counting variable
!     k              Counting variable
!     kcur           curve sequence number
!     krep           Indicates starting position in loop in order to be
!                    able to distinguish between several closed outer
!                    boundaries
!                    In fact krep gives the first position in KMESHS to be
!                    filled
!     kstart         Start value of parameter
!     ksurnr         absolute value of surface sequence number
!     lastpnt        last point of a part
!     nc             number of points along the curves
!     ninter         Number of boundary curves that remains to be
!                    considered
!     nsubsf         Number of subsurfaces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     618   There is no curve in the outer boundary of a surface of surfaces
!           that starts with the same nodal point as the end point of this curve
!     619   The outer boundary of a surface of surfaces does not end with the
!           initial point
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh055surf7' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh055surf7'
         write(irefwr,1) 'isurnr, iprec, isurf', isurnr, iprec, isurf
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      istar3 = kmeshm(2)-1
      istart = kmeshs(isurnr)
      nsubsf = kmeshr(isurnr+1) - kmeshr(isurnr)
      if ( debug ) then
         write(irefwr,1) 'nsubsf', nsubsf
         call prinin ( kmeshr(kmeshr(isurnr)+1), nsubsf, 'subsurfaces' )
      end if  ! ( debug )

!     --- Surface contains subsurfaces, reorganize outer boundary
!         Store curves corresponding to outer boundary in iwork

      iwork = 0
      iwork1 = 0
      iwork2 = 0

!     --- Set start of closed curve loop at 1

      krep = 0
      ninter = kmeshsp2(3,iprec)
      if ( debug ) then
         write(irefwr,1) 'ninter', ninter
         call prinin ( kmeshsp2(1,iprec), 8, 'kmeshsp2' )
      end if  ! ( debug )
      ioutercount = 1
110   krep = krep + 1
      if ( ninter<krep ) goto 1000

!     --- If ninter = 0, there are no outer boundaries to be sorted

      if ( krep==1 ) then

!     --- krep = 1, i.e. the first loop
!         Fill information from KMESHS into iwork

         iwork(kmeshs(istart+1:istart+ninter)) = 1
         if ( debug ) &
            call prinin ( kmeshs(istart+1), ninter, 'internal curves' )

      end if

!     --- Find first subsurface of isurnr with an external boundary

      kstart = kmeshr(isurnr)
      do k = 1, nsubsf

         ksurnr = abs( kmeshr(kstart+k) )

!        --- find outer curves corresponding to ksurnr

         istart = kmeshs(ksurnr)
         nc = kmeshs(ksurnr+1) - istart

         do ic = 1, nc

            kcur = kmeshs(istart+ic)
            if ( iwork(abs(kcur))>0 ) goto 200

!           --- kcur is first curve number

         end do

      end do

!     --- Curve number kcur is first curve of outer boundary

200   istart = kmeshs(isurnr)
      kmeshs(istart+krep) = kcur
      if ( debug ) then

!     --- debug = true, perform debugging

         write(irefwr,1) 'kcur, krep', kcur, krep

      end if

!     --- Store initial and end points of curves in iwork
!         Pos. ncurvs + i   starting point of curve i
!         Pos. 2*ncurvs + i   end point of curve i

      do i = 1, ncurvs
         if ( iwork(i)>0 ) then
            iwork1(i) = kmeshm(istar3+kmeshm(4+i))
            iwork2(i) = kmeshm(istar3+kmeshm(5+i)-1)
         end if
      end do
      if ( debug ) then

!     --- debug = true, perform debugging

         call prinin ( iwork, ncurvs, 'iwork' )
         call prinin ( iwork1, ncurvs, 'iwork1' )
         call prinin ( iwork2, ncurvs, 'iwork2' )

      end if

!     --- First and last point of curve kcur

      if ( kcur>0 ) then
         firstpnt = iwork1(kcur)
         lastpnt = iwork2(kcur)
      else
         firstpnt = iwork2(-kcur )
         lastpnt = iwork1(-kcur )
      end if
      iwork( abs(kcur) ) = 0
      if ( debug ) then

!     --- debug = true, perform debugging

         write(irefwr,*) 'firstpnt, lastpnt ', firstpnt, lastpnt

      end if
      if ( firstpnt==lastpnt ) go to 110
      do i = krep+1, ninter

!     --- Find next point identical to lastpnt

         krep = i
         do j = 1, ncurvs
            if ( iwork(j)>0 ) then
               if ( lastpnt==iwork1(j) ) then

!              --- curve j has same first point as last point
!                  of preceding curve

                  iwork(j) = 0
                  lastpnt = iwork2(j)
                  kmeshs(istart+i) = j
                  if ( i<ninter .and. lastpnt==firstpnt ) then

!                 --- the boundary is closed and not the last curve has
!                     been reached, repeat process for next closed curve

                       if ( (isurf==7 .or. isurf==24) .and. &
                             ioutercount==1 ) kmeshsp2(5,iprec) = i
                       ioutercount = ioutercount+1
                       holes = .true.
                       go to 110

                  end if
                  goto 250

               else if ( lastpnt==iwork2(j) ) then

!              --- curve j has same last point as last point
!                  of preceding curve

                  iwork(j) = 0
                  lastpnt = iwork1(j)
                  kmeshs(istart+i) = - j

                  if ( i<ninter .and. lastpnt==firstpnt ) then

!                 --- the boundary is closed and not the last curve has
!                     been reached, repeat process for next closed curve

                       if ( (isurf==7 .or. isurf==24) .and. &
                            ioutercount==1 ) kmeshsp2(5,iprec) = i
                       ioutercount = ioutercount+1
                       holes = .true.
                       go to 110

                  end if
                  goto 250

               end if

            end if

         end do

!        --- curve i-1 does not end with the same starting point of a
!            next curve

         if ( isurf/=24 ) then

!        --- isurf = 7, hence give warning

            call errint ( abs(kcur), 1 )
            call errint ( abs(firstpnt), 2 )
            call errint ( abs(lastpnt), 3 )
            call errint ( isurnr, 4 )
            call errwar ( 618, 4, 0, 0 )

         end if  ! ( isurf/=24 )

250   end do

!     --- Check if first point of outer boundary coincides with last point

      kcur = kmeshs(istart+ninter)
      if ( kcur>0 ) then
         lastpnt = iwork2(kcur)
      else
         lastpnt = iwork1(-kcur)
      end if
      if ( firstpnt/=lastpnt .and. isurf/=24 ) then
         call errint ( abs(kcur), 1 )
         call errint ( abs(firstpnt), 2 )
         call errint ( abs(lastpnt), 3 )
         call errint ( isurnr, 4 )
         call errwar ( 619, 4, 0, 0 )
      end if
      kmeshsp2(6,isurnr) = ioutercount-1

1000  call erclos ( 'msh055surf7' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh055surf7'

      end if

      end
