      subroutine msh055surfs ( kmeshsp2, isurnr, kmeshm, kmeshs, &
                               isurfc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 12-12-2014 Remove warning
!        version  1.0  date 08-02-2004
!
!   copyright (c) 2004-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if the outer boundary of surface is closed
!     surfaces of surfaces have already been checked
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     sort
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshsp2(8,*), isurnr, kmeshm(*), kmeshs(*)

!     isurfc         i    Type of the surface to be copied
!     isurnr         i    Surface sequence number
!     kmeshm         i    KMESH part m, contains nodal point numbers of the
!                         curves
!     kmeshs        i/o   Kmesh part s, contains information about the various
!                         surfaces. See the PG 13.1
!     kmeshsp2      i/o   Second part of kmeshs, referring to surfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nc, kfirst, i, kcur, isurfc, firstpnt, istart, istar3, &
              lastpnt

!     firstpnt       First point of a curve
!     i              Counting variable
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     istart         First position used at a part
!     kcur           curve sequence number
!     kfirst         First user point of a curve
!     lastpnt        last point of a curve
!     nc             number of points along the curves
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1222   The boundary of the surface is not closed
!    1223   The first user point of the curve is different from the first
!           user point of the preceding curve
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh055surfs' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh055surfs'
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      istart = kmeshs(isurnr)
      istar3 = kmeshm(2)-1

      nc = kmeshs(isurnr+1) - istart
      kfirst = 0
      do i = 1, nc
         kcur = kmeshs(istart+i)
         if ( debug ) &
            write(irefwr,1) 'isurfc, i, kcur',isurfc, i, kcur

!        --- Find first user point of curve kcur

         if ( kcur>0 ) then

!        --- kcur>0, positive direction

            firstpnt = abs ( kmeshm(istar3+kmeshm(4+kcur)) )

         else

!        --- kcur<0, negative direction

            firstpnt = abs ( kmeshm(istar3+kmeshm(5-kcur)-1) )

         end if
         if ( debug ) &
            write(irefwr,1) 'firstpnt, kfirst',firstpnt, kfirst

         if ( kfirst==0 ) then

!        --- firstpnt = 0, First curve of closed curve

            kfirst = firstpnt

         else

!        --- Not the first curve, so check if first node is equal to
!            last node of preceding curve

            if ( debug ) &
               write(irefwr,1) 'lastpnt, kfirst',lastpnt, kfirst
            if ( firstpnt==0 .or. lastpnt==0 ) then

!           --- One of the nodes is 0, no problem

            else if ( lastpnt/=firstpnt ) then

!           --- Last point of preceding curve different from first point

               call errint ( isurnr, 1 )
               call errint ( kcur, 2 )
               call errint ( firstpnt, 3 )
               call errint ( lastpnt, 4 )
               call errsub ( 1222, 4, 0, 0 )

            end if

         end if

!        --- Find last user point of curve kcur

         if ( kcur>0 ) then

!        --- kcur>0, positive direction

            lastpnt = abs ( kmeshm(istar3+kmeshm(5+kcur)-1) )

         else

!        --- kcur<0, negative direction

            lastpnt = abs ( kmeshm(istar3+kmeshm(4-kcur)) )

         end if
         if ( debug ) write(irefwr,1) 'lastpnt', lastpnt

         if ( (isurfc==3 .or. isurfc==19 ) .and. &
               lastpnt==kfirst ) then

!        --- Boundary may consists of disjoint parts

            kfirst = 0
            if ( kmeshsp2(3,isurnr)==0 .and. i<nc ) &
               kmeshsp2(3,isurnr) = i
            if ( debug ) write(irefwr,1) 'kfirst', kfirst

         end if

      end do

      if ( kfirst>0 .and. lastpnt/=kfirst ) then

!     --- Last point of last curve different from first point of first
!         curve

         call errint ( isurnr, 1 )
         call errint ( nc, 2 )
         call errint ( kfirst, 3 )
         call errint ( lastpnt, 4 )
         call errsub ( 1223, 4, 0, 0 )

      end if

1000  call erclos ( 'msh055surfs' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh055surfs'

      end if

      end
