      subroutine msh0637 ( iinput, isurf, iinp, ncurvs, ndefsf, nsurfs, &
                           isurnr, kmeshs, iworkcopy, iwork1, &
                           kmeshscopy, lstart, maxncursur, istart )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 20-10-2008 Allow isurf = 24
!        version  1.0  date 08-10-2008
!
!   copyright (c) 2008-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill kmeshs part curves for surfaces of types 7 or 11
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), isurf, iinp, ncurvs, ndefsf, nsurfs, isurnr, &
              kmeshs(*), kmeshscopy(*), lstart, maxncursur, iwork1(*), &
              iworkcopy(*), istart

!     iinp           i    last position used in array iinput
!     iinput         i    integer input array containing information of the
!                         surfaces
!     istart         i    Position in kmeshscopy
!     isurf          i    Type of surface with respect to mesh generator
!     isurnr         i    Surface sequence number
!     iwork1         i    integer work array of length nsurfs containing
!                         the inverse of iseqsurfs
!     iworkcopy      i    Work array of length nsurfs+1 containing a copy
!                         of the first part of kmeshs in the sequence of
!                         iseqsurfs
!     kmeshs        i/o   Kmesh part s, contains information about the various
!                         surfaces. See the PG 13.1
!     kmeshscopy    i/o   Work array of the same length as kmeshs part 2
!                         containing a copy of kmeshs part 2 in the sequence
!                         of iseqsurfs
!     lstart         i    Position in kmeshs
!     maxncursur     i    Estimate of the length of kmeshscopy
!     ncurvs         i    Number of curves in mesh
!     ndefsf         i    Number of curves in surface
!     nsurfs         i    number of surfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iwork(ncurvs), jinp, k, isrnr, kstart, ncrvs, jsurf, i, &
              icrv, j, jcrv, kinp, nrow, ntel, nint1, istartsav

!     i              Counting variable
!     icrv           curve number
!     isrnr          surface number
!     istartsav      Help variable to save the value of istart
!     iwork          integer work array of length ncurvs
!                    iwork(i) = 0 :  Curve i is not used
!                    iwork(i) = 1 :  Curve i has been used once
!                    iwork(i)>1 :  Curve i has been used more than once
!     j              Counting variable
!     jcrv           curve number
!     jinp           last position used in array iinput
!     jsurf          type of surface
!     k              Counting variable
!     kinp           starting address in iinput
!     kstart         Start value of parameter
!     ncrvs          Number of curves
!     nint1          ?
!     nrow           Number of rows of surfaces in ordered surface of
!                    surfaces
!     ntel           ?
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
!    2097   Surface number out of range
!    2098   Surface corresponds to empty surface
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh0637' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh0637'
         write(irefwr,1) 'isurf, iinp, lstart, istart', &
                          isurf, iinp, lstart, istart
         istartsav = istart
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      jinp = iinp + 2

!     --- In order to delete double curves, array iwork of length ncurvs
!         is used
!         iwork(i) = 0 :  Curve i is not used
!         iwork(i) = 1 :  Curve i has been used once
!         iwork(i)>1 :  Curve i has been used more than once
!         Clear iwork

      iwork = 0

      if ( isurf==7 .or. isurf==24 ) then

!     --- surface consisting of subsurfaces, however, without any ordering
!         Loop over subsurfaces

         do k = 1, ndefsf
            isrnr = abs( iinput(jinp+k) )
            if ( isrnr<=0 .or. isrnr>nsurfs ) then

!           --- surface number out of range

               call errint ( isrnr, 1 )
               call errint ( nsurfs, 2 )
               call errint ( isurnr, 3 )
               call errsub ( 2097, 3, 0, 0 )

            else if ( kmeshs(nsurfs+2+(isrnr-1)*8)==0 ) then

!           --- surface empty

               call errint ( isrnr, 1 )
               call errint ( isurnr, 2 )
               call errsub ( 2098, 2, 0, 0 )

            end if
            kstart = iworkcopy(iwork1(isrnr))
            ncrvs = iworkcopy(iwork1(isrnr)+1) - kstart
            jsurf = kmeshs(nsurfs+1+(isrnr-1)*8+1)

            if ( debug ) &
               write(irefwr,1) 'k, isrnr, kstart, ncrvs, jsurf', &
                                k, isrnr, kstart, ncrvs, jsurf

            if ( jsurf==7 .or. jsurf==11 .or. jsurf==12 ) then

!           --- Subsurface itself is compound surface
!               Distinguish between external and internal boundaries

               nint1 = ncrvs
               ncrvs = kmeshs(nsurfs+1+(isrnr-1)*8+3)
               nint1 = nint1 - ncrvs

            else

!           --- Standard subsurface

               nint1 = 0

            end if
            if ( debug ) write(irefwr,1) 'nint1, ncrvs', nint1, ncrvs

!           --- Loop over curves

            do j = 1, ncrvs
               icrv = kmeshscopy(kstart+j)
               jcrv = abs(icrv)
               iwork(jcrv) = iwork(jcrv) + 1
            end do

            do j = ncrvs+1, ncrvs+nint1
               icrv = kmeshscopy(kstart+j)
               jcrv = abs(icrv)
               iwork(jcrv) = iwork(jcrv) + 2
            end do

         end do

      else

!     --- isurf = 11   Ordened surface
!         i.e. rectangle or coon's surface

         nrow = ndefsf
         kinp = jinp+nrow
         do i = 1, nrow
            ndefsf = iinput(jinp+i)
            do k = 1, ndefsf
               isrnr = abs( iinput(kinp+k) )
               if ( isrnr<=0 .or. isrnr>nsurfs ) then

!              --- surface number out of range

                  call errint ( isrnr, 1 )
                  call errint ( nsurfs, 2 )
                  call errint ( isurnr, 3 )
                  call errsub ( 2097, 3, 0, 0 )

               else if ( kmeshs(nsurfs+2+(isrnr-1)*8)==0 ) then

!              --- surface empty

                  call errint ( isrnr, 1 )
                  call errint ( isurnr, 2 )
                  call errsub ( 2098, 2, 0, 0 )

               end if
               kstart = iworkcopy(iwork1(isrnr))
               ncrvs = iworkcopy(iwork1(isrnr)+1) - kstart

               jsurf = kmeshs(nsurfs+1+(isrnr-1)*8+1)
               if ( jsurf==7 .or. jsurf==11 .or. jsurf==12 ) then

!              --- Subsurface itself is compound surface
!                  Distinguish between external and internal boundaries

                  nint1 = ncrvs
                  ncrvs = kmeshs(nsurfs+1+(isrnr-1)*8+3)
                  nint1 = nint1 - ncrvs

               else

!              --- Standard subsurface

                  nint1 = 0

               end if

!              --- Loop over curves

               do j = 1, ncrvs
                  icrv = kmeshscopy(kstart+j)
                  jcrv = abs(icrv)
                  iwork(jcrv) = iwork(jcrv) + 1
               end do
               do j = ncrvs+1, ncrvs+nint1
                  icrv = kmeshscopy(kstart+j)
                  jcrv = abs(icrv)
                  iwork(jcrv) = iwork(jcrv) + 2
               end do
            end do
            kinp = kinp + ndefsf

         end do

!        --- Check if last curve of first surface in row is a double
!            curve. If so the surface is closed and this curve must
!            be treated as an outer boundary curve. To that end it is
!            considered to be a single curve

      end if

!     --- Fill kmeshs; first the outer boundary is stored but not
!                      sorted
!                      Then the inner boundaries are stored with
!                      a negative value

      if ( debug ) then
         call prinin ( iwork, ncurvs, 'iwork' )
         write(irefwr,1) 'istart', istart
      end if  ! ( debug )

      ntel = 0
      do j = 1, ncurvs
         if ( iwork(j)==1 ) then
            istart = istart + 1
            kmeshscopy(istart) = j
            ntel = ntel + 1
         end if
      end do

      kmeshs(lstart+3) = ntel

      ntel = 0
      do j = 1, ncurvs
         if ( iwork(j)>1 ) then
            istart = istart + 1
            kmeshscopy(istart) = j
            ntel = ntel + 1
         end if
      end do
      kmeshs(lstart+4) = ntel

      if ( istart>maxncursur ) then

!     --- Estimation of length of kmeshscopy too small

         call errint ( istart, 1 )
         call errint ( maxncursur, 2 )
         call errchr ( 'kmeshscopy', 1 )
         call errsub ( 550, 2 , 0, 1 )
         go to 1000

      end if

1000  call erclos ( 'msh0637' )
      if ( debug ) then

!     --- Debug information

         call prinin ( kmeshs(lstart), 5, 'kmeshs' )
         call prinin ( kmeshscopy(istartsav+1), istart-istartsav, &
                       'kmeshscopy' )
         write(irefwr,*) 'End msh0637'

      end if  ! ( debug )

      end
