      subroutine mshcr07 ( coorol, coornw, usol, npoint, factor, &
                           iellist, anorm, work, omega, eps, maxiter, &
                           checkdir )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 06-01-2009 Debug information
!        version  1.1  date 06-03-2001 more comment, error messages, &
!                                      and growth is allowed
!        version  1.0  date 28-01-1997
!
!   copyright (c) 1997-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Compute the co-ordinates of the new boundary in msh047 in the case
!   iadapt = 7 i.e. x_new = x_old + factor (grad c.n) n
!   grad c must be given per element and the displacement of the mid points
!   of the linear elements is computed
!   The displacement of the vertices is computed by linear interpolation
!   between the displacement of the mid points
!   It concerns the improved method described in
!   Guus Segal, Kees Vuik and Fred Vermolen:
!   A conserving discretization for the free boundary in a two-dimensional
!   region
! **********************************************************************
!
!                       KEYWORDS
!
!     adapt_boundary
!     free_boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint, iellist(*), maxiter, checkdir
      double precision coorol(2,*), coornw(2,*), usol(2,*), factor, &
                       anorm(2,*), work(*), eps, omega

!     anorm      i    In this array the normal vector is stored
!                     anorm(j,i) contains the j th component of the normal
!                     vector in the i-th integration point
!     checkdir   i    Is used to check the direction of the displacement
!     coornw     o    Array containing the new co-ordinates along the boundary
!     coorol     i    Array containing the old co-ordinates along the boundary
!     eps        i    Accuracy required for the iteration process
!     factor     i    Multiplication factor
!     iellist    i    Array of length npoint-1 containing a
!                     list of elements that are adjacent to the boundary
!                     element iellist(i) contains both nodal points
!                     iwork(i) and iwork(i+1)
!     maxiter    i    Maximum number of iterations for the non-linear process
!     npoint     i    Number of nodal points along the boundary
!     omega      i    Relaxation parameter for the non-linear process
!     usol       i    Solution vector
!     work            Work array of length npoint
!                     The area of the displacement around the node i is
!                     stored in work(i)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ielem, iter
      double precision u, v, alpha, n1, n2, dx, dy, h, s, y1, y2, &
                       x1, x2, area, areaold, x1old, x2old, y1old, &
                       y2old, x, y, f2, f3, diff, xs, ys, d, &
                       artot
      logical convrgd

!     alpha          Multiplication factor factor u.n
!     area           Area of displacement times element length / 2
!     areaold        Area of previous element
!     artot          Total area of displacements
!     convrgd        If true convergence has been reached
!     d              Maximal difference between two succeeding iterations
!     diff           Difference between two succeeding iterations in one point
!     dx             displacement in x-direction of mid-point
!     dy             displacement in y-direction of mid-point
!     f2             help variable to compute the area
!     f3             help variable to compute the area
!     h              length of an element along the boundary
!     i              Counting variable
!     ielem          Element number
!     iter           Iteration sequence number
!     n1             x-component of normal with respect to boundary element
!     n2             y-component of normal with respect to boundary element
!     s              Factor by which the normal must be multiplied
!     u              First component of solution
!     v              Second component of solution
!     x              x co-ordinate of point
!     x1             x-coordinate of mid-point
!     x1old          x-coordinate of mid-point of previous element
!     x2             x-coordinate of mid-point
!     x2old          x-coordinate of mid-point of previous element
!     xs             x-coordinate of point
!     y              y co-ordinate of point
!     y1             y-coordinate of mid-point
!     y1old          y-coordinate of mid-point of previous element
!     y2             y-coordinate of mid-point
!     y2old          y-coordinate of mid-point of previous element
!     ys             y-coordinate of point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     991   Displacement in negative direction
!     992   Too few iterations for update free boundary
! **********************************************************************
!
!                       PSEUDO CODE
!
!    The co-ordinates along the boundary are updated according to the
!    formula:
!    x_new(i) = x_old(i) + factor grad(i) . n(i) n(i)
!    For all elements i along the boundary do
!       ielem := element number of adjacent element
!       u := usol(1,ielem)
!       v := usol(2,ielem)
!       compute n
!       alpha := factor u.n
!       dx = alpha * n
!       area = area region spanned by dx and element / 2
!       if ( i=1 ) then
!          unew(i) := uold(i) + dx
!       else
!          Compute n in vertex
!          Compute s such that displacement s n covers (area+areaold)/2
!          unew(i-1) := uold(i-1) + s n
!       dxold := dx
!       areaold := area
!    unew(npoint) := old(npoint) + dx
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcr07' )

      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcr07'
         write(irefwr,*) 'npoint', npoint
         call prinrl1 ( coorol, npoint, 2, 'coorol' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- artot is used in the termination criterion.

      artot = 0d0
      do i = 1, npoint-1
         ielem = iellist(i)
         u = usol(1,ielem)
         v = usol(2,ielem)

!        --- (n1,n2) is the unit outward pointing normal vector at the
!            mid-points.

         n1 = coorol(2,i+1)-coorol(2,i)
         n2 = coorol(1,i)-coorol(1,i+1)
         h = sqrt(n1**2+n2**2)
         n1 = n1/h
         n2 = n2/h

!        --- This is the adaptation of the boundary at the mid-points, &
!            without conservation of mass.

         alpha = factor*(u*n1+v*n2)
         dx = alpha*n1
         dy = alpha*n2
         x1 = 0.5d0*(coorol(1,i)+coorol(1,i+1))
         y1 = 0.5d0*(coorol(2,i)+coorol(2,i+1))
         x2 = x1+dx
         y2 = y1+dy

!        --- Below is the signed area used in the conservative approach.

         area = sign(0.5d0*sqrt(dx**2+dy**2)*h,alpha)
         work(i) = area
         x=coorol(1,i)
         y=coorol(2,i)
         n1 = anorm(1,i)
         n2 = anorm(2,i)
         artot = artot+2d0*area

!        --- First iteration of the conservative approach. The first
!            and the last point of the boundary are treated differently.
!            For the first point f2 is two times the signed area minus the
!            signed area of the triangle spanned by (x,y), (x1,y1), and
!            (x2,y2). Finally s*f3 is two times the signed area of the
!            triangle spanned by (x,y), (xs,ys), and (x2,y2).

         if ( i==1 ) then
            f2 = 2d0*area-(x1-x)*(y-y2)+(y1-y)*(x-x2)
            f3 = n1*(y2-y)+n2*(x-x2)
         else
            f2 = 2d0*(area+areaold)-(x-x2old)*(y1old-y)+ &
                 (y-y2old)*(x1old-x)-(x1-x)*(y-y2)+(y1-y)*(x-x2)
            f3 = n1*(y2-y2old)+n2*(x2old-x2)
         end if
         s = f2/f3
         if ( s*checkdir<0d0 ) then

!        --- Displacement with wrong sign

            call errsub ( 991, 0, 0, 0 )

         end if

         if ( debug ) then
            write(irefwr,1) 'i', i
            write(irefwr,2) 'u, v, x, y, s', u, v, x, y, s
            write(irefwr,2) 'n1, n2, f2, f3', n1, n2, f2, f3
            write(irefwr,2) 'dx, dy, area', dx, dy, area
         end if  ! ( debug )

         coornw(1,i) = coorol(1,i)+s*n1
         coornw(2,i) = coorol(2,i)+s*n2
         x1old = x1
         x2old = x2
         y1old = y1
         y2old = y2
         areaold = area
      end do
      i=npoint
      x=coorol(1,i)
      y=coorol(2,i)
      n1 = anorm(1,i)
      n2 = anorm(2,i)
      f2 = 2d0*area+(x1-x)*(y-y2)-(y1-y)*(x-x2)
      f3 = n1*(y-y2)+n2*(x2-x)
      s=f2/f3
      if ( s*checkdir<0d0 ) then

!     --- Displacement with wrong sign

         call errsub ( 991, 0, 0, 0 )

      end if
      coornw(1,i) = coorol(1,i)+s*n1
      coornw(2,i) = coorol(2,i)+s*n2
      convrgd = maxiter<=1
      iter = 1

!     --- This loop solves the nonlinear system of conservation
!         equations using a relaxed Jacobi method.

100   if ( .not. convrgd ) then
         iter=iter+1
         d = 0d0
         do i = 1, npoint-1
            area = work(i)
            x1 = 0.5d0*(coorol(1,i)+coorol(1,i+1))
            y1 = 0.5d0*(coorol(2,i)+coorol(2,i+1))
            x2 = 0.5d0*(coornw(1,i)+coornw(1,i+1))
            y2 = 0.5d0*(coornw(2,i)+coornw(2,i+1))
            x=coorol(1,i)
            y=coorol(2,i)
            n1 = anorm(1,i)
            n2 = anorm(2,i)
            if ( i==1 ) then
               f2 = 2d0*area-(x1-x)*(y-y2)+(y1-y)*(x-x2)
               f3 = n1*(y2-y)+n2*(x-x2)
            else
               f2 = 2d0*(area+areaold)-(x-x2old)*(y1old-y)+ &
                    (y-y2old)*(x1old-x)-(x1-x)*(y-y2)+(y1-y)*(x-x2)
               f3 = n1*(y2-y2old)+n2*(x2old-x2)
            end if
            s=f2/f3
            if ( s*checkdir<0d0 ) then

!           --- Displacement with wrong sign

               call errsub ( 991, 0, 0, 0 )
            end if
            xs=coorol(1,i)+s*n1
            ys=coorol(2,i)+s*n2
            diff =sqrt((xs-coornw(1,i))**2+(ys-coornw(2,i))**2)
            d=max(d,diff)
            coornw(1,i) = coornw(1,i)+omega*(xs-coornw(1,i))
            coornw(2,i) = coornw(2,i)+omega*(ys-coornw(2,i))
            x1old = x1
            x2old = x2
            y1old = y1
            y2old = y2
            areaold = area
         end do
         i=npoint
         x=coorol(1,i)
         y=coorol(2,i)
         n1 = anorm(1,i)
         n2 = anorm(2,i)
         f2 = 2d0*area+(x1-x)*(y-y2)-(y1-y)*(x-x2)
         f3 = n1*(y-y2)+n2*(x2-x)
         s=f2/f3
         if ( s*checkdir<0d0 ) then

!        --- Displacement with wrong sign

            call errsub ( 991, 0, 0, 0 )
         end if
         xs=coorol(1,i)+s*n1
         ys=coorol(2,i)+s*n2
         coornw(1,i) = coornw(1,i)+omega*(xs-coornw(1,i))
         coornw(2,i) = coornw(2,i)+omega*(ys-coornw(2,i))
         if ( iter>maxiter .or. d<=eps*abs(artot) ) then
            convrgd = .true.
         end if
         go to 100
      end if
      if ( .not. convrgd ) then

!     --- No convergence of iteration process

         call errint ( iter, 1 )
         call errint ( maxiter, 2 )
         call erreal ( eps, 1 )
         call erreal ( d/abs(artot), 2 )
         call errwar ( 992, 2, 2, 0 )
      end if

1000  call erclos ( 'mshcr07' )
      if ( debug ) then

!     --- Debug information

         call prinrl1 ( coornw, npoint, 2, 'coornw' )
         write(irefwr,*) 'End mshcr07'

      end if  ! ( debug )

      end
