      subroutine msher1 ( jnew, irand, kbndpt, &
                          bcord, n, m, coorlc, kmeshclc1, jdiag )
! ======================================================================
!
!        programmer    Guus Segal
!        version 12.3  date 29-05-2014 Extension of timebefore
!        version 12.2  date 27-06-2013 Debug statements
!        version 12.1  date 06-11-2010 Adjust length helparray tau
!
!   copyright (c) 1984-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create a grid for a region that can be mapped onto a rectangle
!     The number of elements at opposite sides must be equal
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     rectangle
!     structured
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical jnew
      integer :: kmeshclc1(*), kbndpt(*), irand, n, m, jdiag
      double precision coorlc(2,*), bcord(2,*)

!     bcord          i    array containing the coordinates of the boundaries
!     coorlc         o    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to
!                         the global numbering
!     irand          i    number of points on the side of an element - 1
!     jdiag          i    indication whether the direction of the diagonal in a
!                         quadrilateral must be computed (jdiag=0) or not
!                         (jdiag=1)
!     jnew           i    indication whether the mesh is new (true) or old
!     kbndpt         o    array containing the nodal point numbers of the
!                         boundary
!     kmeshclc1       o    kmesh part c, contains the nodal points corresponding
!                         to the elements
!     m              i    number of elements in the "second" direction
!     n              i    number of elements in the "first" direction
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numnp1, numnp2, ivert1, ivert2, ivert3, ivert4, jtimessav
      double precision eps, tau(2*irand*(n+m)+1)
      logical plotdebug
      double precision :: timebefore(2)

!     eps            Local accuracy, see ismoot
!     ivert1         Index number of first vertex
!     ivert2         Index number of second vertex
!     ivert3         Index number of third vertex
!     ivert4         Index number of fourth vertex
!     jtimessav      Help variable to save the value of jtimes
!     numnp1
!     numnp2
!     plotdebug      If true plots must be made for debugging
!     tau            array containing the relative arclengths along the
!                    boundary
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
!     ERSETTIME      Get the CPU time at start of subroutine
!     MSH2DQ         plot one surface (2D)
!     MSHD7          Places coorindates of centroids of 7-point triangles
!                    in array coor
!     MSHR11         Fill array tau for rectangular grid
!     MSHR12         Fill boundary in array coor for rectangular grid
!     MSHR13         Fill coordinates of vertices for rectangular grid
!     MSHR14         Fill coordinates of midside points for rectangular grid of
!                    quadratic elements
!     MSHR15         Fill KMESH part c for vertices for rectangular grid
!     MSHR16         Fill KMESH part c for quadratic elements for rectangular
!                    grid
!     MSHR18         Fill coordinates of points between for rectangular grid of
!                    higher order elements
!     MSHR19         Fill array kbndpt for rectangular grid
!     MSHSMH         Perform smoothing of coordinates for rectangular grid
!     PRINRL1        Print 2d real vector
!     PRINTTIME      Print the CPU time at end of subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1525   smoothing for quadratic elements has not yet been implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msher1' )
      debug = .false.
      plotdebug = .false.
      numnp1 = n*irand
      numnp2 = m*irand
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msher1'
         write(irefwr,*) 'jnew', jnew
         write(irefwr,1) 'inpelm, irand, mtrian, n, m', &
                          inpelm, irand, mtrian, n, m
         write(irefwr,1) 'jdiag, ismoot, isurnr ', jdiag, ismoot, isurnr
         call prinrl1 ( bcord, numnp1, 2, 'bcord1' )
         call prinrl1 ( bcord(1,numnp1), numnp2, 2, 'bcord2' )
         call prinrl1 ( bcord(1,numnp1+numnp2), numnp1, 2, 'bcord3' )
         call prinrl1 ( bcord(1,2*numnp1+numnp2), numnp2, 2, 'bcord4' )
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000
      if ( itime>0 ) call ersettime ( timebefore )
      ivert1 = numnp1+1
      ivert2 = ivert1+numnp2
      ivert3 = ivert2+numnp1
      ivert4 = ivert3+numnp2

!     --- Clear array coorlc

      coorlc(:,1:(n*irand+1)*(m*irand+1)) = 0d0

!     --- Compute relative distance of nodes per side and store in array tau

      call mshr11 ( bcord, tau, n, m, irand )

!     --- Copy boundary coordinates from bcord into array coorlc

      call mshr12 ( bcord, coorlc, ivert1, ivert3, ivert4, numnp1, numnp2, &
                    inpelm )

!     --- Fill points between by interpolation

      call mshr13 ( bcord, tau, coorlc, ivert1, ivert2, ivert3, ivert4, &
                    irand, numnp1, numnp2, inpelm )

      if ( ismoot>0 ) then

!     --- ismoot>0, smoothing

         if ( inpelm>4 ) then

!        --- quadratic elements have not yet been implemented

            call errint ( ismoot, 1 )
            call errint ( inpelm, 2 )
            call errint ( isurnr, 3 )
            call eropen ( 'msher1' )
            call errwar ( 1525, 3, 0, 0 )
            call erclos ( 'msher1' )

         else

!        --- linear case

            eps = min(ismoot,9)
            eps = 1d0/(1d1**eps)
            call mshsmh ( n, m, coorlc, eps )
         end if

      end if
      if (jnew) then

!     --- jnew = true  new mesh

         if ( mtrian==1 ) call mshr15 ( kmeshclc1, numnp1, numnp2, inpelm )
         if ( mtrian==2 ) call mshr16 ( kmeshclc1, numnp1, numnp2, &
            inpelm, coorlc, jdiag )
         call mshr19 ( kbndpt, numnp1, numnp2, inpelm )

      end if

!     --- Fill coordinates for all non-vertex points

      if ( irand==2 ) then

!     --- irand = 2, quadratic elements
!         Intermediate points are corrected if they are close to
!         the midpoints in order to get straight elements

         call mshr14 ( coorlc, numnp1, numnp2, kmeshclc1, inpelm )
         if ( inpelm==7 ) then

!        --- inpelm = 7, fill coordinates of centroid

            call mshd7 ( kmeshclc1, coorlc, numnp1*numnp2/2, 2 )

         end if

      else if ( irand==3 ) then

!     --- irand = 3, elements with 3 points along each edge

         call mshr18 ( coorlc, n, m, kmeshclc1 )

      end if
      if ( itime==1 ) call printtime('rectangle')

1000  call erclos ( 'msher1' )

      if ( plotdebug ) then

!     --- Debug information

         jtimessav = jtimes
         jtimes = 0
         call msh2dq ( bcord, coorlc, kmeshclc1, 15d0, 1d0, 0, 0 )
         jtimes = jtimessav

      end if

      if ( debug ) then

!     --- Debug information

         call prinrl1 ( coorlc, (n*irand+1)*(m*irand+1), 2, 'coorlc' )
         write(irefwr,*) 'End msher1'

      end if

      end
