      subroutine mshg21( kelem, nbspec, coor, ndim, factor )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  1.2  date 06-09-2003 Correction centroid quadrilateral
!        version  1.1  date 16-12-1994 Adjust centroid computation
!                                      and stretch the internal points
!        version  1.0  date 14-10-1994
!
!   copyright (c) 1994-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Adjust coordinates of internal points in case of general quadri- &
!     lateral spectral elements for Gauss-Legendre-Lobatto points
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     mesh
!     mesh_generation
!     spectral
!     quadratic
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision coor(*), factor(*)
      integer nbspec, kelem(*), ndim

!     coor        i,o     coordinates array
!     factor       i      array containing the one dimensional
!                         Gauss integration points for [-1,+1]
!     kelem        i      array containing the spectral elements
!     nbspec       i      number of points in boundary of spectral
!                         element (end points not taken into account)
!     ndim         i      dimension of space (2D or 3D)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, ikn, inp(8),inh(8), ih, jpl, jph, idim, ieven
      double precision dx1(3), dx2(3), dx3(3), dx4(3), xh, yh, zh, &
                       x1(9), x2(9), x3(9), ski, eta, xn(3)

!     dx1        helpvector for correction on first side
!     dx2        helpvector for correction on second side
!     dx3        helpvector for correction on third side
!     dx4        helpvector for correction on fourth side
!     eta        second coordinate in local system
!     i          loop variable
!     idim       loop variable
!     ieven      indicator whether nbspec is even or odd
!     ih         help for node points
!     ikn        node number
!     inh        array for local node points (even situation)
!     inp        array for local node points
!     j          loop variable
!     jph        position variable
!     jpl        position variable
!     ski        first coordinate in local system
!     x1         array for first coordinate basis points
!     x2         array for second coordinate basis points
!     x3         array for third coordinate basis points
!     xh         helpvariable for x-values
!     xn         temporary array for coordinates transformed
!                Gauss point
!     yh         helpvariable for y-values
!     zh         helpvariable for z-values
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Deconcatenate name of subroutine from string
!     EROPEN  Concatenate name to string of calling subroutines
!     MSHG22  Compute coordinates Gauss point in general quadrilateral
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Compute the coordinates of the (eight) basis points
!     Compute coordinates of the centroid
!     Compute the new internal points using a quadratic isoparametric
!     transformation
!     Do for all internal Gauss points of the standard element
!        Determine coordinates, call MSHG22
!        Compute differences on boundaries and adjust coordinates
!        accordingly
!        Place new determined coordinates in array COOR
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshg21' )

!     --- Run through all internal points and adjust the coordinates
!         utilizing an isoparametric projection for the corners and
!         the midpoints

      ih = 2 * int( nbspec / 2 )

      if ( ih.eq.nbspec ) then

         ieven = 2

      else

         ieven = 1

      end if

      if ( ieven.eq.1 ) then

!     --- Number of sidepoints is odd, simple situation

         ih = int( nbspec / 2 ) + 1

         ikn = 1
         inp(1) = kelem( ikn )
         ikn = 1 + ih
         inp(2) = kelem( ikn )
         ikn = 2 + nbspec
         inp(3) = kelem( ikn )
         ikn = ih * (2+nbspec) + 2+nbspec
         inp(4) = kelem( ikn )
         ikn = (2+nbspec)*(2+nbspec)
         inp(5) = kelem( ikn )
         ikn = (1+nbspec)*(2+nbspec) + 1 + ih
         inp(6) = kelem( ikn )
         ikn = (1+nbspec)*(2+nbspec) + 1
         inp(7) = kelem( ikn )
         ikn = ih * (2+nbspec) + 1
         inp(8) = kelem( ikn )

!        --- Now the coordinates of the basis points can be computed

         do i = 1, 8

             jpl   =  ndim * (inp(i)-1)

             x1(i) = coor( jpl + 1 )
             x2(i) = coor( jpl + 2 )

             if ( ndim.eq.3 ) then
                x3(i) = coor(jpl+3)
             else
                x3(i) = 0d0
             end if

         end do

      else

!     --- Number of sidepoints is even, difficult situation

         ih = int( nbspec / 2 )

         ikn = 1
         inp(1) = kelem( ikn )
         inh(1) = kelem( ikn )
         ikn = 1 + ih
         inp(2) = kelem( ikn )
         ikn = 2 + ih
         inh(2) = kelem( ikn )
         ikn = 2 + nbspec
         inp(3) = kelem( ikn )
         inh(3) = kelem( ikn )
         ikn = ih * ( 2 + nbspec ) + 2 + nbspec
         inp(4) = kelem( ikn )
         ikn = ikn + 2 + nbspec
         inh(4) = kelem( ikn )
         ikn = (2+nbspec)*(2+nbspec)
         inp(5) = kelem( ikn )
         inh(5) = kelem( ikn )
         ikn = (1+nbspec)*(2+nbspec) + 1 + ih
         inp(6) = kelem( ikn )
         ikn = ikn + 1
         inh(6) = kelem( ikn )
         ikn = (1+nbspec)*(2+nbspec) + 1
         inp(7) = kelem( ikn )
         inh(7) = kelem( ikn )
         ikn = ih * (2+nbspec) + 1
         inp(8) = kelem( ikn )
         ikn = ikn + 2 + nbspec
         inh(8) = kelem( ikn )

!        --- Now the coordinates of the basis points can be computed

         do i = 1, 8

             jpl   =  ndim * (inp(i)-1)
             jph   =  ndim * (inh(i)-1)

             x1(i) = (coor( jpl+1 ) + coor( jph+1 ))* 0.5d0
             x2(i) = (coor( jpl+2 ) + coor( jph+2 ))* 0.5d0

             if ( ndim.eq.3 ) then
                x3(i) = (coor(jpl+3) + coor( jph+3 ))* 0.5d0
             else
                x3(i) = 0d0
             end if

         end do

      end if

!     --- Finally the centroid of the quadrilateral, which is not
!         given a priori, is needed

      x1(9) = 0.5d0 *(x1(2)+x1(4)+x1(6)+x1(8))- &
              0.25d0*(x1(1)+x1(3)+x1(5)+x1(7))
      x2(9) = 0.5d0 *(x2(2)+x2(4)+x2(6)+x2(8))- &
              0.25d0*(x2(1)+x2(3)+x2(5)+x2(7))
      x3(9) = 0.5d0 *(x3(2)+x3(4)+x3(6)+x3(8))- &
              0.25d0*(x3(1)+x3(3)+x3(5)+x3(7))

!     --- Compute the new internal points

      do j = 2 , nbspec + 1

          eta = factor(j)

          do i = 2 , nbspec + 1

              ski = factor(i)

!             --- Determine displacements in order to stretch, &
!                 compute the corrections

!                 Point (i,1) i.e. (ski,-1d0)

              ikn = kelem( i )

              xh = coor( ndim*(ikn-1) + 1)
              yh = coor( ndim*(ikn-1) + 2)
              zh = 0d0

              if ( ndim.eq.3 ) zh = coor( ndim*(ikn-1) + 3)

              call mshg22( ski, -1d0, x1, x2, x3, ndim, xn )

              dx1(1) = xn(1) - xh
              dx1(2) = xn(2) - yh
              dx1(3) = xn(3) - zh

!             --- Point (nbspec+2,j) i.e. (1,eta)

              ikn = kelem( j * (nbspec+2) )

              xh = coor( ndim*(ikn-1) + 1)
              yh = coor( ndim*(ikn-1) + 2)

              if ( ndim.eq.3 ) zh = coor( ndim*(ikn-1) + 3)

              call mshg22( 1d0, eta, x1, x2, x3, ndim, xn )

              dx2(1) = xn(1) - xh
              dx2(2) = xn(2) - yh
              dx2(3) = xn(3) - zh

!             --- Point (i, nbspec+2) i.e. (ski,1d0)

              ikn = kelem( (nbspec+1)*(nbspec+2) + i )

              xh = coor( ndim*(ikn-1) + 1)
              yh = coor( ndim*(ikn-1) + 2)

              if ( ndim.eq.3 ) zh = coor( ndim*(ikn-1) + 3)

              call mshg22( ski, 1d0, x1, x2, x3, ndim, xn )

              dx3(1) = xn(1) - xh
              dx3(2) = xn(2) - yh
              dx3(3) = xn(3) - zh

!             --- Point (1,j) i.e. (-1d0,eta)

              ikn = kelem( (j-1)*(nbspec+2) + 1 )

              xh = coor( ndim*(ikn-1) + 1)
              yh = coor( ndim*(ikn-1) + 2)

              if ( ndim.eq.3 ) zh = coor( ndim*(ikn-1) + 3)

              call mshg22( -1d0, eta, x1, x2, x3, ndim, xn )

              dx4(1) = xn(1) - xh
              dx4(2) = xn(2) - yh
              dx4(3) = xn(3) - zh

!             --- Determine coordinates

              call mshg22( ski, eta, x1, x2, x3, ndim, xn )

!             --- Place new determined coordinates with corrections in
!                 array coor

              ikn = kelem( (j-1)*(nbspec+2) + i )

              do idim = 1 , ndim

                 coor( ndim*(ikn-1) + idim ) = xn(idim) - &
                        ( 1d0 - eta ) * dx1(idim)* 0.5d0 - &
                        ( 1d0 + ski ) * dx2(idim)* 0.5d0 - &
                        ( 1d0 + eta ) * dx3(idim)* 0.5d0 - &
                        ( 1d0 - ski ) * dx4(idim)* 0.5d0

              end do

          end do

      end do

      call erclos ( 'mshg21' )

      end
