      subroutine msho70 ( nbndpt, coor, npoint, nelem, kelem )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  1.3  date 24-07-2007 Clock and counterclockwise allowed
!        version  1.2  date 20-07-2007 Debug statements
!        version  1.1  date 04-10-1996 Replace writes by error messages
!
!   copyright (c) 1996-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Construct a triangular grid if only the nodes in a domain are given.
!     Use is made of a special TESSELATION property. It is assumed that
!     the first points form the boundary of the domain. In this version
!     use is made of the advancing boundary technique.
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     surface
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kelem(3,*), nbndpt, nelem, npoint
      double precision coor(2,*)

!     coor    i    array containing the 2d Euclidian coordinates
!     kelem   o    array containing the nodes of the triangles.
!                  length of this array 3*nelem.
!     nbndpt  i    the first nbndpt nodes are the boundary points.
!     nelem   o    number of triangles created.
!                  estimation : nelem=2*(npoint-1)-nbndpt.
!     npoint  i    total number of points in mesh.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ia, ib, icheck, ieen, ii, ip, itwe, j, jap, jj, k, ks, &
              kstap, ls, m, mh, mm, nelm, nelemest, icounter, &
              klaar(npoint), kstapl(2*npoint)
      double precision afij, afik, afkj, alva, beta, dela, det, dr, eps, &
              epsa, ex1, ex2, factor, gama, r, rh, totaal
      logical restrt

!     afij           local distance i to j
!     afik           local distance i to k
!     afkj           local distance k to j
!     alva           helpcoefficient
!     beta           helpcoefficient
!     dela           local distance
!     det            surface of triangle considered
!     dr             local distance
!     eps            accuracy
!     epsa           local accuracy
!     ex1            local sum variable
!     ex2            local sum variable
!     factor         help variable for disturbing coordinates
!     gama           local x-distance
!     i              loop-variable
!     ia             first node of line segment
!     ib             second node of line segment
!     icheck         indicator whether visibility should be checked or not
!     icounter       help variable for counterclockwise boundary registration
!     ieen           help variable for adjustment of local boundary array
!     ii             help variable for klaar array
!     ip             loop variable
!     itwe           help variable for adjustment of local boundary array
!     j              loop variable
!     jap            indicator for visibility checks
!     jj             help variable for klaar array
!     k              loop variable
!     klaar          help array for detecting that all work is done
!     ks             loop variable
!     kstap          number of line segments in actual boundary
!     kstapl         help array for actual boundary of region, &
!     ls             help variable to adjust array kstapl
!     m              nearest nodal point
!     mh             next nearest nodal point
!     mm             help variable for klaar array
!     nelemest       Estimate of the number of elements
!     nelm           number of triangles
!     r              radius of smallest innercircle
!     restrt         indicator whether computation is restarted (.true.) or
!                    not (.false.)
!     rh             radius of next smallest innercircle
!     totaal         local distance parameter
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     MSHO81         determines whether a line is permitted, i.e. whether a
!                    proposed line does not cross the boundary
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     979   No point could be found
!     980   Too many elements
!     981   Too few elements
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Initialise some arrays and parameters
!     While the boundary consists of more than one line segment do
!        Take first line segment of boundary
!        While restart do
!           Restart = false
!           Do for all points
!              If point is not passed by the boundary then
!                 If point forms with line segment a counter clockwise triangle
!                    Calculate dr?
!                    If dr less than rh then
!                       Check visibility
!                       Update some parameters if necessary
!                    Endif
!                 Endif
!              Endif
!           Enddo
!           If new point too close then restart = true
!           If not restart then
!              Check visbility of new point
!              If wrong then restart = true
!           Endif
!        Endwhile
!        Store new element
!        Adjust arrays
!     Endwhile
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen( 'msho70' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msho70'
         write(irefwr,1) 'npoint, nbndpt', npoint, nbndpt
         call prinrl1 ( coor, npoint, 2, 'coor_in' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nelemest = nelem    ! estimate of nelem

      eps = 10 * epsmac

!     --- Compute number of elements expected

      nelm = 2 * ( npoint - 1 ) - nbndpt

!     --- Clear boundary array kstapl

      do i = 1, 2 * npoint
         kstapl(i) = 0
      end do

!     --- Set array klaar

      do i = 1, npoint
         klaar(i) = 1
      end do

!     --- Disturb coordinates

      j = 0

      do i = 1, npoint

         j = j + 1
         if ( j>13 ) j = 0
         factor = 1.d0 + j * 10 * eps
         coor(1,i) = coor(1,i) * factor
         coor(2,i) = coor(2,i) * factor

      end do
      if ( debug ) call prinrl1 ( coor, npoint, 2, 'coor_dist' )

      nelem    = 0
        icounter = 0

100   if ( nelem==0 .and. icounter/=-1 ) then

!     --- Set boundary points in kstapl

         if ( icounter==0 ) then

              ! --- Use original direction:
            do i = 1, nbndpt
               kstapl(2*i-1) = i
               kstapl(2*i)   = i + 1
               klaar(i)      = -1
            end do

            kstapl(2*nbndpt) = 1

         else
            ! --- Try the other way around:
              do i = nbndpt, 1, -1
                 kstapl(2*i-1) = i
               kstapl(2*i)   = i - 1
                 klaar(i)      = -1
              end do

              kstapl(2) = nbndpt

              icounter = 1

           end if

         if ( debug ) call prinin1 ( kstapl, nbndpt, 2, 'kstapl/1' )

         kstap = nbndpt

500      if ( kstap/=0 ) then

!        --- Determine boundary points to be considered

            i = kstapl(1)
            j = kstapl(2)

            if ( debug ) write(irefwr,1) 'kstap, i, j', kstap, i, j

            icheck = 0

!           --- Create triangle for line i-j

            restrt = .true.

600         if ( restrt ) then

               restrt = .false.

               m  = 0
               mh = 0
               r  = 1.d30
               rh = r

               do k = 1, npoint

                  if ( debug ) write(irefwr,1) 'k', k

                  if ( klaar(k)/=0 .and. k/=i .and. k/=j ) then

!                 --- Suppose point is visible both from i and j

                     alva = coor(1,i) - coor(1,j)
                     beta = coor(2,i) - coor(2,j)
                     gama = coor(1,i) - coor(1,k)
                     dela = coor(2,i) - coor(2,k)

!                    --- Counterclockwise ?

                     det = alva * dela - beta * gama

                     if ( det>eps ) then

                        epsa= ( ( coor(1,i) - coor(1,k) ) * &
                                ( coor(1,k) - coor(1,j) ) + &
                                ( coor(2,i) - coor(2,k) ) * &
                                ( coor(2,k) - coor(2,j) ) ) /2d0

                        afij   = alva * alva + beta * beta
                        dr     = abs ( sqrt ( afij ) * epsa / det )
                        afik   = gama * gama + dela * dela
                        ex1    = alva - gama
                        ex2    = beta - dela
                        afkj   = ex1 * ex1 + ex2 * ex2
                        totaal = afij - afik - afkj

                        if ( totaal>0d0 ) dr = -dr

                     else

                        dr = 1d30

                     end if

                     if ( debug ) write(irefwr,2) 'dr, rh', dr, rh

                     if ( dr<rh ) then

!                    --- Check visibility if necessary

                        jap = 1

                        if ( debug ) write(irefwr,1 ) 'icheck', icheck

                        if ( icheck==1 ) then

                           call msho81 ( j, k, coor, nbndpt, jap )

                           if ( debug ) write(irefwr,1 ) 'jap', jap

                           if ( jap/=1 ) then

                              call msho81 ( i, k, coor, nbndpt, jap )

                           end if

                           if ( debug ) write(irefwr,1 ) 'jap', jap

                        end if

                        if ( icheck/=1 .or. jap/=1 ) then
                           if ( dr<r ) then
                              mh = m
                              rh = r
                              m  = k
                              r  = dr
                           else
                              mh = k
                              rh = dr
                           end if
                        end if

                     end if

                  end if

               end do

               if ( debug ) write(irefwr,1 ) 'New point m', m

!              --- New point m found

               if ( m==0 ) then

                  if ( icounter==0 ) then
                       icounter = 1
                       go to 100
                    end if
                  call errsub ( 979, 0, 0, 0 )
                  go to 1000

               end if

               if ( debug ) write(irefwr,2 ) 'r - rh', r - rh

               if ( abs ( r - rh )<10 * eps ) then

!              --- Disturb point mh and make a restart

                  coor(1 ,mh ) = coor(1 ,mh ) * ( 1.d0 + 100. * eps )
                  coor(2 ,mh ) = coor(2 ,mh ) * ( 1.d0 + 100. * eps )

!                 --- Restart

                  restrt = .true.

               end if

               if ( .not. restrt ) then

!              --- Check visibility of new point

                  if ( icheck==0 ) then

                     call msho81 ( j, m, coor, nbndpt, jap )

                     if ( jap==0 ) call msho81 ( i,m,coor,nbndpt,jap )

                     if ( jap==1 .and. mh/=0 ) m = mh

                     if ( m==mh ) then

!                    --- Check this new point

                        call msho81 ( j, m, coor, nbndpt, jap )

                        if ( jap==0 ) call msho81 ( i, m, coor, nbndpt, jap )

                     end if

                     if ( jap==1 ) then

!                    --- Restart

                        icheck = 1
                        restrt = .true.

                     end if

                     if ( debug ) &
                        write(irefwr,1) 'm, icheck, jap', m, icheck, jap

!                    --- new point m is visible both from i and j !

                  end if

               end if

               go to 600

            end if

!           --- Place new element in array kelem

            nelem = nelem + 1

            if ( nelem>nelemest ) then

               call errsub ( 980, 0, 0, 0 )
               go to 1000

            end if

            kelem( 1, nelem ) = i
            kelem( 2, nelem ) = j
            kelem( 3, nelem ) = m

            if ( debug ) call prinin ( kelem(1,nelem),3,'new element' )

!           --- Adjust array kstapl

            ieen = 1
            itwe = 1
            ls   = 0

            do ks = 2, kstap

               ia = kstapl(2*ks-1)
               ib = kstapl(2*ks)

               if ( ia==m .and. ib==i ) then
                  ieen = 0
               else if ( ia==j .and. ib==m ) then
                  itwe = 0
               else
                  ls = ls + 1
                  kstapl(2*ls-1) = ia
                  kstapl(2*ls)   = ib
               end if

            end do

            if ( ieen==1 ) then
               ls = ls + 1
               kstapl(2*ls-1) = i
               kstapl(2*ls)   = m
            end if

            if ( itwe==1 ) then
               ls = ls + 1
               kstapl(2*ls-1) = m
               kstapl(2*ls)   = j
            end if

            kstap = ls

            if ( debug ) call prinin1 ( kstapl, kstap, 2, 'kstapl/2' )

!           --- Adjust array klaar

            ii = 0
            jj = 0
            mm = 0

            do ip = 1, ls
               ia = kstapl(2*ip-1)
               ib = kstapl(2*ip  )
               if ( i==ia .or. i==ib ) ii = -1
               if ( j==ia .or. j==ib ) jj = -1
               if ( m==ia .or. m==ib ) mm = -1
            end do

            klaar(i) = ii
            klaar(j) = jj
            klaar(m) = mm

            go to 500

         end if

!        --- Return to first position

         if ( icounter==0 ) icounter = 1

           goto 100

        end if

!       --- Check

      if ( nelm/=nelem ) call errsub ( 981, 0, 0, 0 )

1000  call erclos ( 'msho70' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msho70'

      end if  ! ( debug )

      end
