      subroutine mshob3dcross1 ( xp, yp, zp, ipcell, jpcell, kpcell, &
                                 iobgrid, nx, ny, nz, eps, coor, &
                                 kmeshc, inpelm, nintersect, &
                                 leniobgrid, xi )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 07-01-2002 Extra parameter xi
!        version  2.0  date 14-11-2001 Extra parameters leniobgrid
!        version  1.0  date 29-06-2001
!
!   copyright (c) 2001-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute number of cross sections of the line y=yp,z=zp with the obstacle in
!    cell (i,j,k)
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     node
!     obstacle
!     crossing
!     cross_section
!     crossing_point
!     intersection
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision xp, yp, zp, coor(3,*), eps, xi(*)
      integer ipcell, jpcell, kpcell, nx, ny, nz, &
              iobgrid(0:nx-1,0:ny-1,0:nz-1,*), inpelm, &
              kmeshc(inpelm,*), nintersect, leniobgrid

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         obstacle
!     eps            i    Accuracy
!                         Points that are at a distance less than eps of the
!                         obstacle are defined as part of the obstacle
!     inpelm         i    number of points per element in obstacle
!     iobgrid        i    In R^3:
!                         array of length nx x ny x nz x 6 that is filled
!                         as follows:
!                         The region covered is defined by xmin, xmax, ymin, &
!                         ymax, zmin, zmax
!                         The cell size is dx x dy x dz
!                         Cell i,j,k refers to the coordinates:
!                         (xmin+(i-1)dx,ymin+(j-1)dy,zmin+(k-1)dz) x
!                         (xmin+idx,ymin+jdy,zmin+kdz)
!                         iobgrid(i,j,k,.) = 0: cell outside obstacle
!                         iobgrid(i,j,k,.) = l>0: boundary part l crosses cell
!                         iobgrid(i,j,k,.) = -1: cell completely in obstacle
!                         The last index refers to the various elements in a
!                         cell
!     ipcell         i    i-index of cell(i,j,k)
!     jpcell         i    j-index of cell(i,j,k)
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the obstacle elements
!     kpcell         i    k-index of cell(i,j,k)
!     leniobgrid     i    Declared length of last dimension of iobgrid
!     nintersect    i/o   Number of intersections found
!     nx             i    number of points in x direction
!     ny             i    number of points in y direction
!     nz             i    number of points in z direction
!     xi            i/o   Array containing the x-coordinates of the intersection
!                         points in the cell
!     xp             i    x-coordinate of point
!     yp             i    y-coordinate of point
!     zp             i    z-coordinate of point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielem, icell(leniobgrid), ncell, i, nxi, l, inode
      double precision x(leniobgrid), y(leniobgrid), z(leniobgrid), &
                       det, alpha1, beta1, xcross

!     alpha1         multiplication factor
!     beta1          multiplication factor
!     det            determinant
!     i              Counting variable
!     icell          Contains element numbers in cell
!     ielem          Element number
!     inode          Node sequence number
!     l              General loop variable
!     ncell          Number of elements in cell
!     nxi            Number of points that are intersected in x-direction
!     x              x co-ordinate of point
!     xcross         x co-ordinate of intersection point
!     y              y co-ordinate of point
!     z              z co-ordinate of point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
!    2254   inpelm # 3
! **********************************************************************
!
!                       PSEUDO CODE
!
!    See mshob3dcross
!    Only the intersections with y=yp, z=zp are computed
!    The number of intersections is used to raise nintersect
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshob3dcross1' )
      if ( inpelm.ne.3 ) then

!     --- Only triangles have been implemented

         call errsub ( 2254, 0, 0, 0 )
         go to 1000

      end if

!     --- Store triangles in cell into icell

      ncell = 1
      icell(1) = iobgrid(ipcell,jpcell,kpcell,1)
      do l = 2, leniobgrid
         if ( iobgrid(ipcell,jpcell,kpcell,l).eq.0 ) go to 100
         ncell = ncell+1
         icell(ncell) = iobgrid(ipcell,jpcell,kpcell,l)
      end do

!     --- For i := 1 (1) ncell do

100   nxi = nintersect
      do i = 1, ncell

!     --- Store coordinates in x, y and z

         ielem = icell(i)
         do l = 1, inpelm
            inode = kmeshc(l,ielem)
            x(l) = coor(1,inode)
            y(l) = coor(2,inode)
            z(l) = coor(3,inode)
         end do

!        --- Find intersection with the line y=yp and z=zp

         det = (y(2)-y(1))*(z(3)-z(1))-(y(3)-y(1))*(z(2)-z(1))
         if ( det.gt.eps*1d-4 ) then

!        --- determinant not zero, intersection possible

            det = 1d0/det
            alpha1 = det*((z(3)-z(1))*(yp-y(1))+(y(1)-y(3))*(zp-z(1)))
            beta1  = det*((z(1)-z(2))*(yp-y(1))+(y(2)-y(1))*(zp-z(1)))
            if ( alpha1.ge.-1d-6 .and. alpha1.le.1.000001d0 .and. &
                 beta1.ge.-1d-6 .and. beta1.le.1.000001d0 .and. &
                 alpha1+beta1.le.1.000001d0 ) then

!           --- intersection found

               xcross = x(1)+alpha1*(x(2)-x(1))+beta1*(x(3)-x(1))
               if ( xcross.lt.xp .and. abs(xp-xcross).gt.eps ) then

!              --- store x in xi if point not yet stored

                  do l = 1, nxi
                     if ( abs(xi(l)-xcross ).le.1d-5 ) go to 200
                  end do
                  nxi = nxi+1
                  if ( nxi.gt.leniobgrid ) then

!                 --- xi too small

                     call errint ( nxi, 1 )
                     call errint ( leniobgrid, 2 )
                     call errchr ( 'xi', 1 )
                     call errsub ( 550, 2, 0, 1 )
                     go to 1000

                  end if
                  xi(nxi) = xcross

               end if

            end if

         end if

200   end do

!     --- Raise nintersect

      nintersect = nxi

1000  call erclos ( 'mshob3dcross1' )
      end
