      subroutine mshob3dintlin1 ( x1, x2, xmin, ymin, zmin, nx, ny, nz, &
                                  iobgrid, icell, ncell, dx, dy, dz, &
                                  leniobgrid )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 14-11-2001 Extra parameter leniobgrid
!        version  1.0  date 29-06-2001
!
!   copyright (c) 2001-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array icell with indices of surface parts of the obstacle that are
!     possibily intersected by the line x1-x2
!     3d version
! **********************************************************************
!
!                       KEYWORDS
!
!     intersection
!     line
!     obstacle
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nx, ny, nz, iobgrid(0:nx-1,0:ny-1,0:nz-1,*), icell(*), &
              ncell, leniobgrid
      double precision x1(3), x2(3), xmin, ymin, zmin, dx, dy, dz

!     dx             i    length of vector in x-direction
!     dy             i    length of vector in y-direction
!     dz             i    length of vector in z-direction
!     icell          o    Array containing the element numbers of the surface
!                         that must be considered for computing the
!                         cross sections
!     iobgrid        i    In R^3:
!                         array of length nx x ny x nz x 6 that is filled
!                         as follows:
!                         The region covered is defined by xmin, xmax, ymin, &
!                         ymax, zmin, zmax
!                         The cell size is dx x dy x dz
!                         Cell i,j,k refers to the coordinates:
!                         (xmin+(i-1)dx,ymin+(j-1)dy,zmin+(k-1)dz) x
!                         (xmin+idx,ymin+jdy,zmin+kdz)
!                         iobgrid(i,j,k,.) = 0: cell outside obstacle
!                         iobgrid(i,j,k,.) = l>0: boundary part l crosses cell
!                         iobgrid(i,j,k,.) = -1: cell completely in obstacle
!                         The last index refers to the various elements in a
!                         cell
!     leniobgrid     i    Declared length of last dimension of iobgrid
!     ncell          o    Number of surfaces stored in icell
!     nx             i    number of points in x direction
!     ny             i    number of points in y direction
!     nz             i    number of points in z direction
!     x1             i    coordinates of first point
!     x2             i    coordinates of last point
!     xmin           i    Minimum x-value
!     ymin           i    Minimum y-value
!     zmin           i    Minimum z-value
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ielem, ipcell, jpcell, kpcell, ip1, ip2, jp1, jp2, &
              kp1, kp2, l
      double precision x, y, z, alpha1, beta1, gamma1

!     alpha1         multiplication factor
!     beta1          multiplication factor
!     gamma1         multiplication factor
!     i              Counting variable
!     ielem          Element number
!     ip1            i-index of cell(i,j,k)
!     ip2            i-index of cell(i,j,k)
!     ipcell         i-index of cell(i,j,k)
!     jp1            j-index of cell(i,j,k)
!     jp2            j-index of cell(i,j,k)
!     jpcell         j-index of cell(i,j,k)
!     kp1            k-index of cell(i,j,k)
!     kp2            k-index of cell(i,j,k)
!     kpcell         k-index of cell(i,j,k)
!     l              General loop variable
!     x              x co-ordinate of point
!     y              y co-ordinate of point
!     z              z co-ordinate of point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     ncell := 0
!     Compute cell index of x1, store in ip1, jp1, kp1
!     Compute cell index of x2, store in ip2, jp2, kp2
!     If ( indices not in correct range) then
!        give ip1 ... impossible values
!     else
!        restrict values to maximum range
!     For all possible values of ip,jp,kp
!        If ( there is a new surface in the cell ) then
!           raise ncell
!           Sore surcae in icell
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshob3dintlin1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshob3dintlin1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- ncell := 0

      ncell = 0

!     --- Compute cell index of x1, store in ip1, jp1, kp1

      x = x1(1)
      y = x1(2)
      z = x1(3)
      alpha1 = (x-xmin)/dx
      beta1  = (y-ymin)/dy
      gamma1 = (z-zmin)/dz
      ip1 = int(alpha1)
      jp1 = int(beta1)
      kp1 = int(gamma1)

!     --- Compute cell index of x2, store in ip2, jp2, kp2

      x = x2(1)
      y = x2(2)
      z = x2(3)
      alpha1 = (x-xmin)/dx
      beta1  = (y-ymin)/dy
      gamma1 = (z-zmin)/dz
      ip2 = int(alpha1)
      jp2 = int(beta1)
      kp2 = int(gamma1)
      if ( ip1<0 .and. ip2.lt.0 .or. ip1>=nx .and. ip2.ge.nx .or. &
           jp1<0 .and. jp2.lt.0 .or. jp1>=ny .and. jp2.ge.ny .or. &
           kp1<0 .and. kp2.lt.0 .or. kp1>=nz .and. kp2.ge.nz ) then

!     --- No intersection possible

         ip1 = -1
         ip2 = -2
         jp1 = -1
         jp2 = -2
         kp1 = -1
         kp2 = -2

      else

!     --- intersection possible, set ip1 etc

         ip1 = min(max(ip1,0),nx-1)
         ip2 = min(max(ip2,0),nx-1)
         jp1 = min(max(jp1,0),ny-1)
         jp2 = min(max(jp2,0),ny-1)
         kp1 = min(max(kp1,0),nz-1)
         kp2 = min(max(kp2,0),nz-1)
         if ( ip1>ip2 ) then
            ipcell = ip1
            ip1 = ip2
            ip2 = ipcell
         end if
         if ( jp1>jp2 ) then
            jpcell = jp1
            jp1 = jp2
            jp2 = jpcell
         end if
         if ( kp1>kp2 ) then
            kpcell = kp1
            kp1 = kp2
            kp2 = kpcell
         end if

      end if
      do ipcell = ip1, ip2

         do jpcell = jp1, jp2

            do kpcell = kp1, kp2

!           --- check if there is a surface in the cell

               if ( iobgrid(ipcell,jpcell,kpcell,1)>0 ) then

!              --- surface in cell has been found
!                  find all surfaces that are not yet in icell

                  do l = 1, leniobgrid
                     ielem = iobgrid(ipcell,jpcell,kpcell,l)
                     if ( ielem==0 ) go to 110
                     do i = 1, ncell
                        if ( ielem==icell(i) ) go to 100
                     end do
                     ncell = ncell+1
                     icell(ncell) = ielem
100               end do

               end if

110         end do

         end do

      end do
1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshob3dintlin1'

      end if  ! ( debug )
      call erclos ( 'mshob3dintlin1' )

      end subroutine mshob3dintlin1
