      subroutine mshobsfl31 ( nx, ny, nz, iobgrid, ielem, x, y, z, &
                              i1, i2, i3, inpelm, xmin, ymin, zmin, dx, &
                              dy, dz, eps, leniobgrid, leniobgridact )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 14-11-2001 Extra parameters leniobgrid,leniobgridact
!        version  1.0  date 15-06-2001 Layout
!
!   copyright (c) 2001-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array iobgrid for all edges of an obstacle element in case
!     the variation in all directions is unequal to zero
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nx, ny, nz, iobgrid(0:nx-1,0:ny-1,0:nz-1,*), ielem, &
              i1(*), i2(*), i3(*), inpelm, leniobgrid, leniobgridact
      double precision x(*), y(*), z(*), xmin, ymin, zmin, dx, dy, dz

!     dx             i    length of vector in x-direction
!     dy             i    length of vector in y-direction
!     dz             i    length of vector in z-direction
!     eps            i    Local accuracy
!     i1             i    Array containing the "i" values of the vertices
!     i2             i    Array containing the "j" values of the vertices
!     i3             i    Array containing the "k" values of the vertices
!     ielem          i    Element number
!     inpelm         i    number of points per element
!     iobgrid       i/o   In R^3:
!                         array of length nx x ny x nz x 6 that is filled
!                         as follows:
!                         The region covered is defined by xmin, xmax, ymin, &
!                         ymax, zmin, zmax
!                         The cell size is dx x dy x dz
!                         Cell i,j,k refers to the coordinates:
!                         (xmin+(i-1)dx,ymin+(j-1)dy,zmin+(k-1)dz) x
!                         (xmin+idx,ymin+jdy,zmin+kdz)
!                         iobgrid(i,j,k,.) = 0: cell outside obstacle
!                         iobgrid(i,j,k,.) = l>0: boundary part l crosses cell
!                         iobgrid(i,j,k,.) = -1: cell completely in obstacle
!                         The last index refers to the various elements in a
!                         cell
!     leniobgrid     i    Declared length of last dimension of iobgrid
!     leniobgridact i/o   Actual length of last dimension of iobgrid
!     nx             i    number of points in x direction
!     ny             i    number of points in y direction
!     nz             i    number of points in z direction
!     x              i    Contains the x-coordinates of the vertices
!     xmin           i    Minimum x-value
!     y              i    Contains the y-coordinates of the vertices
!     ymin           i    Minimum y-value
!     z              i    Contains the z-coordinates of the vertices
!     zmin           i    Minimum z-value
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision x1(3), x2(3), xt, yt, zt, alpha, eps, xmin1, &
                       xmax1, ymin1, ymax1, zmin1, zmax1
      logical finished, found
      integer i, isign, j, jsign, k, ksign, l

!     alpha          multiplication factor
!     finished       Indicates if the loop has been finished (true) or not
!                    (false)
!     found          If true a cross-section is found
!     i              Counting variable
!     isign          Indicates step in i-direction
!     j              Counting variable
!     jsign          Indicates step in j-direction
!     k              Counting variable
!     ksign          Indicates step in k-direction
!     l              General loop variable
!     x1             x-coordinate of first point
!     x2             x-coordinate of last point
!     xmax1          Maximum x-value of cell
!     xmin1          Minimum x-value of cell
!     xt             x-coordinate
!     ymax1          Maximum y-value of cell
!     ymin1          Minimum y-value of cell
!     yt             y-coordinate
!     zmax1          Maximum z-value of cell
!     zmin1          Minimum z-value of cell
!     zt             z-coordinate
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     MSHOBSFL1      Fills array iobsgrid for one cell
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2252   No cross-section found
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For all edges of element do
!        Store initial and end point in x1 and x2
!        Compute isign, jsign and ksign
!        Set i, j, k start
!        While ( isign#0 or jsign#0 or ksign#0 ) do
!           Compute new cell that is crossed by edge by following edge in cell
!           Adapt isign, jsign and ksign
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobsfl31' )

!     --- Loop over edges of element

      do l = 1, inpelm

!     --- Store initial and end point in x1 and x2

         x1(1) = x(l)
         x1(2) = y(l)
         x1(3) = z(l)
         x2(1) = x(l+1)
         x2(2) = y(l+1)
         x2(3) = z(l+1)

!        --- Compute isign, jsign and ksign

         if ( i1(l+1).gt.i1(l) ) then
            isign = 1
         else if ( i1(l+1).lt.i1(l) ) then
            isign = -1
         else
            isign = 0
         end if

         if ( i2(l+1).gt.i2(l) ) then
            jsign = 1
         else if ( i2(l+1).lt.i2(l) ) then
            jsign = -1
         else
            jsign = 0
         end if

         if ( i3(l+1).gt.i3(l) ) then
            ksign = 1
         else if ( i3(l+1).lt.i3(l) ) then
            ksign = -1
         else
            ksign = 0
         end if

!        --- Set i, j, k start

         i = i1(l)
         j = i2(l)
         k = i3(l)
         finished = isign.eq.0 .and. jsign.eq.0 .and. ksign.eq.0

!        --- Loop over all cells that are crossed

         do while ( .not. finished )

!        --- Compute point where cell is left

            found = .false.
            xmin1 = xmin+i*dx-eps
            xmax1 = xmin+(i+1)*dx+eps
            ymin1 = ymin+j*dy-eps
            ymax1 = ymin+(j+1)*dy+eps
            zmin1 = zmin+k*dz-eps
            zmax1 = zmin+(k+1)*dz+eps

            if ( isign.ne.0 ) then

!           --- Check i-direction

               xt = xmin+i*dx
               if ( isign.gt.0 ) xt = xt+dx
               alpha = (xt-x1(1)) / (x2(1)-x1(1))
               yt = x1(2)+alpha*(x2(2)-x1(2))
               zt = x1(3)+alpha*(x2(3)-x1(3))
               if ( yt.gt.ymin1 .and. yt.lt.ymax1 .and. &
                    zt.gt.zmin1 .and. zt.lt.zmax1 ) then

!              --- cross section found

                  i = i+isign
                  if ( i.eq.i1(l+1) ) isign = 0
                  found = .true.

               end if

            end if

            if ( jsign.ne.0 ) then

!           --- Check j-direction

               yt = ymin+j*dy
               if ( jsign.gt.0 ) yt = yt+dy
               alpha = (yt-x1(2)) / (x2(2)-x1(2))
               xt = x1(1)+alpha*(x2(1)-x1(1))
               zt = x1(3)+alpha*(x2(3)-x1(3))
               if ( xt.gt.xmin1 .and. xt.lt.xmax1 .and. &
                    zt.gt.zmin1 .and. zt.lt.zmax1 ) then

!              --- cross section found

                  j = j+jsign
                  if ( j.eq.i2(l+1) ) jsign = 0
                  found = .true.

               end if

            end if

            if ( ksign.ne.0 ) then

!           --- Check k-direction

               zt = zmin+k*dz
               if ( ksign.gt.0 ) zt = zt+dz
               alpha = (zt-x1(3)) / (x2(3)-x1(3))
               xt = x1(1)+alpha*(x2(1)-x1(1))
               yt = x1(2)+alpha*(x2(2)-x1(2))
               if ( xt.gt.xmin1 .and. xt.lt.xmax1 .and. &
                    yt.gt.ymin1 .and. yt.lt.ymax1 ) then

!              --- cross section found

                  k = k+ksign
                  if ( k.eq.i3(l+1) ) ksign = 0
                  found = .true.

               end if

            end if

            if ( found ) then

!           --- cross section found, store in iobgrid

               call mshobsfl1 ( nx, ny, nz, iobgrid, i, j, k, ielem, &
                                leniobgrid, leniobgridact )

            else if ( isign.ne.0 .and. jsign.ne.0 .and. ksign.ne.0 ) then

!           --- No cross- section found, must be a programming error

               call errsub ( 2252, 0, 0, 0 )
               go to 1000

            else

!           --- Nothing else to do

               finished = .true.

            end if

         end do

      end do
1000  call erclos ( 'mshobsfl31' )
      end
