      subroutine mshobsfl3d ( rmeshx, iobgrid3d, kmeshc, coor, nx, ny, nz, &
                              iobst, inpelm, nelem, leniobgrid )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 06-11-2012 Extra debug
!        version  2.0  date 14-11-2001 Extra parameter leniobgrid
!        version  1.0  date 16-06-2001
!
!   copyright (c) 2001-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array iobgrid3d with grid information of one obstacle
!     3d case only
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduledebug
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nx, ny, nz, inpelm, nelem, iobst, leniobgrid, &
              iobgrid3d(0:nx-1,0:ny-1,0:nz-1,leniobgrid), &
              kmeshc(inpelm,nelem)
      double precision rmeshx(9,*), coor(3,*)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     inpelm         i    number of points per element
!     iobgrid3d       i/o   In R^3:
!                         array of length nx x ny x nz x 6 that is filled
!                         as follows:
!                         The region covered is defined by xmin, xmax, ymin, &
!                         ymax, zmin, zmax
!                         The cell size is dx x dy x dz
!                         Cell i,j,k refers to the coordinates:
!                         (xmin+(i-1)dx,ymin+(j-1)dy,zmin+(k-1)dz) x
!                         (xmin+idx,ymin+jdy,zmin+kdz)
!                         iobgrid3d(i,j,k,.) = 0: cell outside obstacle
!                         iobgrid3d(i,j,k,.) = l>0: boundary part l crosses cell
!                         iobgrid3d(i,j,k,.) = -1: cell completely in obstacle
!                         The last index refers to the various elements in a
!                         cell
!     iobst          i    Obstacle sequence number
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     leniobgrid    i/o   Declared length of last dimension of iobgrid3d
!     nelem          i    Number of elements
!     nx             i    number of points in x direction
!     ny             i    number of points in y direction
!     nz             i    number of points in z direction
!     rmeshx         i    Real part of kmesh part x (i = obstacle number)
!                         rmeshx(1,i) = xmin
!                         rmeshx(2,i) = xmax
!                         rmeshx(3,i) = ymin
!                         rmeshx(4,i) = ymax
!                         rmeshx(5,i) = dx
!                         rmeshx(6,i) = dy
!                         In R^3:
!                         rmeshx(5,i) = zmin
!                         rmeshx(6,i) = zmax
!                         rmeshx(7,i) = dx
!                         rmeshx(8,i) = dy
!                         rmeshx(9,i) = dz
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision xmin, dx, dy, eps, dz, ymin, zmin

!     dx             length of vector in x-direction
!     dy             length of vector in y-direction
!     dz             length of vector in z-direction
!     eps            Local accuracy
!     xmin           Minimum x-value
!     ymin           Minimum y-value
!     zmin           Minimum y-value
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHOBSFL30     Fill array iobgrid3d for all cells that are crossed by an
!                    obstacle surface element
!     MSHOBSFL3IN    Fill array iobgrid3d with information about which cells
!                    are internal
!     PLOBSTACLEGR3D Plot all cells of the grid overlapping an obstacle using a
!                    colour to distinguish between inner and outside cells and
!                    boundary cells
!                    3d version, slices of constant z values are plotted
!     PRININ4_0      Print an integer four-dimensional array, starting in 0
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Initializations
!     Clear array iobgrid3d
!     Fill array iobgrid3d for all cells that are crossed by an
!     obstacle surface element
!     Fill array iobgrid3d with information about which cells are internal
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobsfl3d' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobsfl3d'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- initializations

      xmin = rmeshx(1,iobst)
      ymin = rmeshx(3,iobst)
      zmin = rmeshx(5,iobst)
      dx   = rmeshx(7,iobst)
      dy   = rmeshx(8,iobst)
      dz   = rmeshx(9,iobst)
      eps = 0.001d0*dx

      if ( debug ) then

!     --- debug is true, print info

         write(irefwr,*) 'Data with respect to obstacle'
         write(irefwr,2) 'xmin, ymin, zmin, dx, dy, dz', &
                          xmin, ymin, zmin, dx, dy, dz
         write(irefwr,1) 'nx, ny, nz', nx, ny, nz
         write(irefwr,1) 'iobst ',iobst

      end if

!     --- Clear array iobgrid3d

      iobgrid3d = 0

!     --- Fill array iobgrid3d for all cells that are crossed by an
!         obstacle surface element

      call mshobsfl30 ( nx, ny, nz, iobgrid3d, nelem, kmeshc, &
                        coor, inpelm, xmin, ymin, zmin, &
                        dx, dy, dz, eps, leniobgrid )

      if ( ierror/=0 ) go to 1000

!     --- Fill array iobgrid3d with information about which cells are internal

      call mshobsfl3in ( nx, ny, nz, iobgrid3d, kmeshc, coor, &
                         inpelm, xmin, ymin, zmin, dx, dy, dz, &
                         leniobgrid )

      if ( debug ) then

!     --- debug is true, print info

         call prinin4_0 ( iobgrid3d, nx, ny, nz, leniobgrid, 'iobgrid3d' )

      end if

      if ( debug .or. plotobst ) then

!     --- debug is true, plot info

         call plobstaclegr3d ( rmeshx, nx, ny, nz, iobgrid3d, iobst, &
                               kmeshc, coor, inpelm, nelem )

      end if
1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshobsfl3d'

      end if  ! ( debug )
      call erclos ( 'mshobsfl3d' )

      end
