      subroutine mshobsflinsx ( xlow, xmid, xhigh, x1, x2, y1, y2, eps, &
                                xmin12, xmax12, ymin12, ymax12, icount, &
                                storecurves, ymin, iobgrid, nx, ny, i, &
                                numcells, icurpart, iwork, &
                                leniwork, dy )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 02-10-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute intersection of part of obstacle with x = ....
!     Part not parallel to y-axis
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer icount, nx, ny, iobgrid(0:nx-1,0:ny-1), i, &
              numcells(2,*), icurpart, leniwork, &
              iwork(2,leniwork)
      double precision xlow, x1, x2, y1, y2, xmid, xhigh, eps, ymin, &
                       xmin12, xmax12, ymin12, ymax12, storecurves(*), &
                       dy

!     dy             i    step size of grid in y-direction
!     eps            i    Local accuracy
!     i              i    Column number of cells that is considered
!     icount        i/o   Next position in storecurves to be used
!     icurpart       i    Sequence number of curve part
!     iobgrid       i/o   position i,j contains:
!                         -1:  cell inside obstacle
!                          0:  cell outside obstacle
!                         >0:  cell contains part of the obstacle
!                         The value contains a starting address in iobgrid1
!     iwork         i/o   Contains the cell numbers (i,j) in the sequence
!                         of the curve parts
!                         Must be used in connection with numcells
!     leniwork       i    Declared length of array iwork
!     numcells      i/o   numcells(1,k) contains the number of cells curvepart
!                         k intersects
!                         numcells(1,k) contains the starting address of
!                         the cell numbers in iwork
!     nx             i    number of points in x direction in grid
!     ny             i    number of points in y direction in grid
!     storecurves   i/o   Contains the y-coordinates of the curve parts that
!                         cross one column in the grid (cross section point)
!     x1             i    x-coordinate of first point in curve part
!     x2             i    x-coordinate of last point in curve part
!     xhigh          i    x-coordinate of right-hand side of cell
!     xlow           i    x-coordinate of left-hand side of cell
!     xmax12         i    max (x1,x2)
!     xmid           i    x-coordinate of middle of cell
!     xmin12         i    min (x1,x2)
!     y1             i    y-coordinate of first point in curve part
!     y2             i    y-coordinate of last point in curve part
!     ymax12         i    max (y1,y2)
!     ymin           i    Minimum y-value
!     ymin12         i    min (y1,y2)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision x, alpha, ymid, yhigh, beta1, beta2, ylow

!     alpha          parameter alpha to define a line
!     beta1          parameter beta1 to define a line
!     beta2          parameter beta2 to define a line
!     x              x co-ordinate of point
!     yhigh          y-coordinate of right-hand side of cell
!     ylow           y-coordinate of left-hand side of cell
!     ymid           y-coordinate of middle of cell (cross section)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHOBSFLEDGE2  Compute intersection of part of obstacle with x = ....
!                    Part parallel to y-axis
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobsflinsx' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobsflinsx'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- The line is not parallel to the y-axis
!         Compute intersection with x = ....

      x = xlow
      alpha = (x-x1)/(x2-x1)
      ylow = y1+alpha*(y2-y1)
      x = xmid
      alpha = (x-x1)/(x2-x1)
      ymid = y1+alpha*(y2-y1)
      x = xhigh
      alpha = (x-x1)/(x2-x1)
      yhigh = y1+alpha*(y2-y1)
      if ( ymid>=ymin12-eps .and. ymid<=ymax12+eps .and. &
           xmid>=xmin12-eps .and. xmid<=xmax12+eps ) then

!     --- mid point is in interval

         icount = icount+1
         storecurves(icount) = ymid

      end if  ! ( ymid>=ymin12-eps .and. ymid<=ymax12+eps ...

      if ( ylow<ymin12 .or. ylow>ymax12 ) then

!     --- ylow not in interval hence boundary part does not
!         intersect x=xlow

         if ( yhigh<ymin12 .or. yhigh>ymax12) then

!        --- yhigh not in interval hence boundary part does not
!            intersect x=xhigh.
!            This means that the complete part is in the column

            beta1  = (y1-ymin)/dy
            beta2  = (y2-ymin)/dy
            call mshobsfledge2 ( iobgrid, nx, ny, i, numcells, &
                                 icurpart, iwork, beta1, beta2 )

         else if ( x1>xhigh ) then

!        --- yhigh in interval hence boundary part does intersect
!            x=xhigh but not x=xlow
!            x1 outside interval, hence x2 inside

            beta1  = (yhigh-ymin)/dy
            beta2  = (y2-ymin)/dy
            call mshobsfledge2 ( iobgrid, nx, ny, i, numcells, &
                                 icurpart, iwork, beta1, beta2 )

         else

!        --- x2 outside interval, hence x1 inside

            beta1  = (yhigh-ymin)/dy
            beta2  = (y1-ymin)/dy
            call mshobsfledge2 ( iobgrid, nx, ny, i, numcells, &
                                 icurpart, iwork, beta1, beta2 )

         end if

      else

!     --- ylow in interval hence boundary part intersects x=xlow

         if ( yhigh<ymin12 .or. yhigh>ymax12) then

!        --- yhigh not in interval hence boundary part does not
!            intersect x=xhigh.

            if ( x1<xlow ) then

!           --- x1 outside interval, hence x2 inside

               beta1  = (ylow-ymin)/dy
               beta2  = (y2-ymin)/dy
               call mshobsfledge2 ( iobgrid, nx, ny, i, numcells, &
                                    icurpart, iwork, beta1, beta2 )

            else

!           --- x2 outside interval, hence x1 inside

               beta1  = (ylow-ymin)/dy
               beta2  = (y1-ymin)/dy
               call mshobsfledge2 ( iobgrid, nx, ny, i, numcells, &
                                    icurpart, iwork, beta1, beta2 )

            end if  ! ( x1<xlow )

         else

!        --- yhigh in interval hence boundary part intersects both
!            sides

            beta1  = (ylow-ymin)/dy
            beta2  = (yhigh-ymin)/dy
            call mshobsfledge2 ( iobgrid, nx, ny, i, numcells, &
                                 icurpart, iwork, beta1, beta2 )

         end if  ! ( yhigh<ymin12 .or. yhigh>ymax12)

      end if  ! ( ylow<ymin12 .or. ylow>ymax12 )

1000  call erclos ( 'mshobsflinsx' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshobsflinsx'

      end if  ! ( debug )

      end
