      subroutine mshobsin33 ( iobst )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-11-2001
!
!   copyright (c) 2001-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill information in kmesh part x concerning one obstacle
!     3d case, obstacle must be a closed surface
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemesh
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iobst

!     iobst          i    Obstacle sequence number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, node, j, k
      double precision xmin, xmax, ymin, ymax, zmin, zmax, dx, dy, dz, &
                       x, y, z, unit, xc(3,3), dxdxsi(3,3), area
      logical closed

!     area           Area of element
!     closed         If true the boundary is closed
!     dx             length of vector in x-direction
!     dxdxsi         Work array of length 3x3 to store help variables to
!                    compute the area, see elp620
!     dy             length of vector in y-direction
!     dz             length of vector in z-direction
!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     node           Node number
!     unit           unit length
!     x              x co-ordinate of point
!     xc             array with the coordinates of the vertices of a triangle
!     xmax           Maximum x-value
!     xmin           Minimum x-value
!     y              y co-ordinate of point
!     ymax           Maximum y-value
!     ymin           Minimum y-value
!     z              z co-ordinate of point
!     zmax           Maximum z-value
!     zmin           Minimum z-value
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHOBSTCHSURF  Check if surface of obstacle is closed and if all edges
!                    contain exactly 2 obstacle surface elements
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1382   Value of NPELM is not yet implemented for this subroutine
! **********************************************************************
!
!                       PSEUDO CODE
!
!      Determine if kmeshx/rmeshx has already been filled
!      Compute xmin, xmax, ymin, ymax, zmin, zmax
!      Check if obstacle is closed
!      unit := minimum length in elsize
!      dx := xmax-xmin; dy := ymax-ymin; dz := zmax-zmin;
!      xmax := xmax+0.005*dx  xmin := xmin-0.005*dx
!      ymax := ymax+0.005*dy  ymin := ymin-0.005*dy
!      zmax := zmax+0.005*dz  zmin := zmin-0.005*dz
!      nx := max(5,4*(xmax-xmin)/unit+1)
!      ny := max(5,4*(ymax-ymin)/unit+1)
!      nz := max(5,4*(zmax-zmin)/unit+1)
!      dx := (xmax-xmin)/nx;  dy := (ymax-ymin)/ny;  dz := (zmax-zmin)/nz
!      Fill information in arrays kmeshx
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobsin33' )

!     --- Determine if kmeshx/rmeshx has already been filled

      xmin = rmeshx(1,iobst)
      xmax = rmeshx(2,iobst)
      ymin = rmeshx(3,iobst)
      ymax = rmeshx(4,iobst)
      zmin = rmeshx(5,iobst)
      zmax = rmeshx(6,iobst)
      if ( abs(xmax-xmin)+abs(ymax-ymin)+abs(zmax-zmin).gt.0d0 ) &
         go to 1000
      if ( inpelm.ne.3 ) then

!     --- inpelm # 3, not yet implemented

         call errint ( inpelm, 1 )
         call errsub ( 1382, 1, 0, 0 )
         go to 1000

      end if

!     --- unit := sqrt ( minimum area of triangles)

      unit = rinfin
      do i = 1, inelem
         do j = 1, inpelm
            node = kmeshcobst(j,i)
            do k = 1, 3
               xc(j,k) = coorobst(k,node)
            end do
         end do
         do j = 1, 3
            dxdxsi(j,1) = xc(2,j)-xc(1,j)
            dxdxsi(j,2) = xc(3,j)-xc(1,j)
         end do

!        --- fill jacobian in area

         dxdxsi(1,3) = dxdxsi(2,1)*dxdxsi(3,2)- &
                       dxdxsi(3,1)*dxdxsi(2,2)
         dxdxsi(2,3) = dxdxsi(3,1)*dxdxsi(1,2)- &
                       dxdxsi(1,1)*dxdxsi(3,2)
         dxdxsi(3,3) = dxdxsi(1,1)*dxdxsi(2,2)- &
                       dxdxsi(2,1)*dxdxsi(1,2)
         area = sqrt( dxdxsi(1,3)**2 + &
                      dxdxsi(2,3)**2 + &
                      dxdxsi(3,3)**2 )
         unit = min(unit,sqrt(0.5d0*area))

      end do

!     --- Compute xmin, xmax, ymin, ymax, zmin, zmax

      xmin = rinfin
      xmax =-rinfin
      ymin = rinfin
      ymax =-rinfin
      zmin = rinfin
      zmax =-rinfin

      do i = 1, npointlc
         x    = coorobst(1,i)
         y    = coorobst(2,i)
         z    = coorobst(3,i)
         xmin = min(xmin,x)
         xmax = max(xmax,x)
         ymin = min(ymin,y)
         ymax = max(ymax,y)
         zmin = min(zmin,z)
         zmax = max(zmax,z)

      end do

!     --- dx := xmax-xmin; dy := ymax-ymin; dz := zmax-zmin

      dx = xmax-xmin
      dy = ymax-ymin
      dz = zmax-zmin

!     --- Check if obstacle is closed

      call mshobstchsurf ( kmeshcobst, inelem, npointlc, inpelm,closed )
      if ( ierror.ne.0 ) go to 1000

!     --- xmax := xmax+0.005*dx  xmin := xmin-0.005*dx
!         ymax := ymax+0.005*dy  ymin := ymin-0.005*dy
!         zmax := zmax+0.005*dz  zmin := zmin-0.005*dz
!         nx := max(5,4*(xmax-xmin)/unit+1)
!         ny := max(5,4*(ymax-ymin)/unit+1)
!         nz := max(5,4*(zmax-zmin)/unit+1)
!         dx := (xmax-xmin)/nx; dy := (ymax-ymin)/ny; dz := (zmax-zmin)/nz

      xmax = xmax+0.005d0*dx
      xmin = xmin-0.005d0*dx
      ymax = ymax+0.005d0*dy
      ymin = ymin-0.005d0*dy
      zmax = zmax+0.005d0*dz
      zmin = zmin-0.005d0*dz
      dx   = xmax-xmin
      dy   = ymax-ymin
      dz   = zmax-zmin
      nx   = min(max(5,int(1.1d0*dx/unit+1d0)),100)
      ny   = min(max(5,int(1.1d0*dy/unit+1d0)),100)
      nz   = min(max(5,int(1.1d0*dz/unit+1d0)),100)
      dx   = dx/nx
      dy   = dy/ny
      dz   = dz/nz

!     --- Fill information in arrays kmeshx

      rmeshx(1,iobst) = xmin
      rmeshx(2,iobst) = xmax
      rmeshx(3,iobst) = ymin
      rmeshx(4,iobst) = ymax
      rmeshx(5,iobst) = zmin
      rmeshx(6,iobst) = zmax
      rmeshx(7,iobst) = dx
      rmeshx(8,iobst) = dy
      rmeshx(9,iobst) = dz
      if ( closed ) then
         kmeshx(1,iobst) = 2
      else
         kmeshx(1,iobst) = -2
      end if
      kmeshx(2,iobst) = isurnr
      kmeshx(3,iobst) = nx
      kmeshx(4,iobst) = ny
      kmeshx(5,iobst) = nz
      kmeshx(6,iobst) = 0
      kmeshx(7,iobst) = 0
      kmeshx(10,iobst) = inpelm
      kmeshx(11,iobst) = inelem
      kmeshx(12,iobst) = npointlc

!     --- The filling of kmeshxa has not yet been implemented

1000  call erclos ( 'mshobsin33' )
      end
