      subroutine mshobstcross ( xp, yp, ipcell, jpcell, onobstacle, &
                                rmeshx, iobgrid, nx, ny, curves, nnodes, &
                                ixcross, iycross, eps, xi, nxi, &
                                iobgrid1 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 02-10-2012 Extra parameter iobgrid1
!        version  2.0  date 07-01-2002 Extra parameters xi and nxi
!        version  1.1  date 31-01-2001 Adaptation for cross-sections at common
!                                      points of curve parts
!
!   copyright (c) 2000-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute number of cross sections of the line y=yp with the obstacle in
!    cell (i,j) left and right of x=xp and also of the line x=xp with the
!    obstacle in cell (i,j) above and below of y=yp
!    Store the results in ixcross and iycross
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     node
!     obstacle
!     crossing
!     cross_section
!     crossing_point
!     intersection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulegen
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical onobstacle
      double precision xp, yp, rmeshx(6), curves(2,*), eps, xi(*)
      integer ipcell, jpcell, nx, ny, iobgrid(0:nx-1,0:ny-1), nnodes, &
              ixcross(2), iycross(2), nxi, iobgrid1(*)

!     curves         i    array containing the coordinates of the curves
!                         In this particular case only the coordinates of the
!                         obstacle must be used, hence array curves must be
!                         called with starting address corresponding to the
!                         specific obstacle
!     eps            i    Accuracy
!                         Points that are at a distance less than eps of the
!                         obstacle are defined as part of the obstacle
!     iobgrid        i    array of length nx x ny that is filled as follows:
!                         The region covered is defined by xmin, xmax, ymin, &
!                         ymax
!                         The cell size is dx x dy
!                         Cell i,j refers to the coordinates:
!                         (xmin+(i-1)dx,ymin+(j-1)dy) x (xmin+idx,ymin+jdy)
!                         iobgrid(i,j) = 0: cell outside obstacle
!                         iobgrid(i,j) = k>0: boundary part k crosses cell
!                         iobgrid(i,j) = -1: cell completely in obstacle
!     iobgrid1       i    Must be used in connection with iobgrid
!                         iobgrid contains the starting addresses for each cell
!                         i,j
!                         From this starting address iobgrid1 contains
!                         the number of curve parts of the obstacle
!                         intersecting the cell followed by the sequence
!                         numbers of these curve parts
!     ipcell         i    i-index of cell(i,j)
!     ixcross        o    Contains the number of cross sections of the line x=xp
!                         with the obstacle within cell(i,j)
!                         ixcross(1) contains the number of cross sections
!                         below yp
!                         ixcross(2) contains the number of cross sections
!                         above yp
!     iycross        o    Contains the number of cross sections of the line y=yp
!                         with the obstacle within cell(i,j)
!                         iycross(1) contains the number of cross sections
!                         at the left-hand side of xp
!                         iycross(2) contains the number of cross sections
!                         at the right-hand side of xp
!     jpcell         i    j-index of cell(i,j)
!     nnodes         i    Number of nodes in the obstacle
!     nx             i    number of cells in x direction corresponding to
!                         obstacle
!     nxi            o    Number of points that are intersected in x-direction
!     ny             i    number of cells in y direction corresponding to
!                         obstacle
!     onobstacle     o    If true the point is positioned on the obstacle
!     rmeshx         i    Real part of kmesh part x (i = obstacle number)
!                         rmeshx(1,i) = xmin
!                         rmeshx(2,i) = xmax
!                         rmeshx(3,i) = ymin
!                         rmeshx(4,i) = ymax
!                         rmeshx(5,i) = dx
!                         rmeshx(6,i) = dy
!     xi             o    Array containing the x-coordinates of the intersection
!                         points in the cell
!     xp             i    x-coordinate of point
!     yp             i    y-coordinate of point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer maxcells
      parameter ( maxcells=10 )
      integer icell(3*maxcells), ncell, i, ind, ncurpart, istart
      double precision xmin, xmax, ymin, ymax, dx, dy, xlow(2), tang(2), &
                       alpha1, x, y, xprev, yprev
      logical doublepoint, check

!     alpha1         multiplication factor
!     check          if true intersections must be checked
!     doublepoint    if true an intersection at a common point of two
!                    curve parts has been found
!     dx             length of vector in x-direction
!     dy             length of vector in y-direction
!     i              Counting variable
!     icell          Array containing the curve parts that must be considered
!                    for computing the cross sections
!     ind            indicates index in curve part
!     istart         Starting address
!     maxcells       Declared length of array icell
!     ncell          Number of curve parts that must be used to compute
!                    cross sections
!     ncurpart       Number of elements obstacle grid consists of
!     tang           In this array the tangential vector is stored
!     x              x co-ordinate of point
!     xlow           Array containing the coordinates of the first point at
!                    a curve part
!     xmax           Maximum x-value
!     xmin           Minimum x-value
!     xprev          Previous x value
!     y              y co-ordinate of point
!     ymax           Maximum y-value
!     ymin           Minimum y-value
!     yprev          Previous y value
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     MSHFILLCELL    Fill array icell for one grid cell
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    onobstacle := false
!    ixcross := 0;  iycross := 0
!    Extract xmin,xmax,ymin,ymax of cell(i,j) from obstacle array
!    ind1 := iobgrid(i,j)
!    icell(1) := ind1-1; icell(2) := ind1; icell(3) := ind1+1; ncell := 3
!    For i := 1 (1) ncell do
!       xlow := coordinates first node of curve part icell(i)
!       tang := tangential vector along curve part icell(i)
!       if ( tang(1)#0 ) then
!          alpha := (xp-xlow(1))/tang(1)
!          y := xlow(2)-alpha*tang(2)
!          if ( ymin<=y<=ymax ) then
!             if ( y<yp ) then
!                ixcross(1) := ixcross(1)+1
!             else if ( y>yp ) then
!                ixcross(2) := ixcross(2)+1
!             else
!                onobstacle := true
!       if ( tang(2)#0 ) then
!          alpha := (yp-xlow(2))/tang(2)
!          x := xlow(2)-alpha*tang(2)
!          if ( xmin<=x<=xmax ) then
!             if ( x<xp ) then
!                iycross(1) := iycross(1)+1
!             else if ( x>xp ) then
!                iycross(2) := iycross(2)+1
!             else
!                onobstacle := true
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      debug = .false.
      call eropen ( 'mshobstcross' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobstcross'
         write(irefwr,1) 'ipcell, jpcell', ipcell, jpcell
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- onobstacle := false
!         ixcross := 0;  iycross := 0

      onobstacle = .false.
      ixcross(1) = 0
      ixcross(2) = 0
      iycross(1) = 0
      iycross(2) = 0
      nxi = 0

!     --- Extract xmin,xmax,ymin,ymax of cell(i,j) from obstacle array

      dx = rmeshx(5)
      dy = rmeshx(6)
      xmax = rmeshx(1)+(ipcell+1)*dx
      if ( ipcell==0 ) then
         xmin = rmeshx(1)-eps
      else
         xmin = xmax-dx
         if ( ipcell==nx-1 ) xmax = rmeshx(2)+eps
      end if
      ymax = rmeshx(3)+(jpcell+1)*dy
      if ( jpcell==0 ) then
         ymin = rmeshx(3)-eps
      else
         ymin = ymax-dy
         if ( jpcell==ny-1 ) ymax = rmeshx(4)+eps
      end if

      if ( debug ) &
         write(irefwr,2) 'dx, dy, xmin, xmax, ymin, ymax', &
                          dx, dy, xmin, xmax, ymin, ymax

!     --- ind1 := iobgrid(i,j)
!         icell(1) := ind1-1; icell(2) := ind1; icell(3) := ind1+1; ncell := 3

      ncell = 0
      istart = iobgrid(ipcell,jpcell)
      ncurpart = iobgrid1(istart)

      call inichkarlen ( ncurpart, maxcells, 'icell' )
      if ( ierror/=0 ) go to 1000

      call mshfillcell ( istart, iobgrid1, ncell, maxcells, &
                         icell, nnodes )

      if ( debug ) call prinin ( icell, ncell, 'icell' )

!     --- For i := 1 (1) ncell do

      doublepoint = .false.
      do i = 1, ncell

!     --- xlow := coordinates first node of curve part icell(i)
!         tang := tangential vector along curve part icell(i)

         ind = icell(i)
         xlow(1) = curves(1,ind)
         xlow(2) = curves(2,ind)
         tang(1) = curves(1,ind+1)-xlow(1)
         tang(2) = curves(2,ind+1)-xlow(2)
         check = .false.

         if ( abs(tang(1))>eps ) then

!        --- tang(1)#0
!            alpha := (xp-xlow(1))/tang(1)
!            y := xlow(2)-alpha*tang(2)

            alpha1 = (xp-xlow(1))/tang(1)
            y = xlow(2)+alpha1*tang(2)
            if ( doublepoint .and. &
                 ( alpha1>=-1d-6 .and. alpha1<=1d-6 .or. &
                   alpha1>=0.999999d0 .and. alpha1<=1.000001d0 ) ) then

!           --- Possiby double point, check y

               if ( abs(y-yprev)>1d-5 ) then

!              --- Points are different, set check

                  check = .true.
                  yprev = y
                  doublepoint = .false.

               end if

            else if ( alpha1>=-accuracy_obstacle .and. &
                      alpha1<=1d0+accuracy_obstacle .and. &
                      y>=ymin .and. y<=ymax ) then

!           --- ymin<=y<=ymax, set check

               check = .true.
               if ( alpha1>=-1d-6 .and. alpha1<=1d-6 .or. &
                    alpha1>=0.999999d0 .and. alpha1<=1.000001d0 ) then

!              --- Point at the end of a cell, mark as possibly double

                  doublepoint = .true.
                  yprev = y

               end if

            end if
            if ( check ) then

!           --- Check = true, check
!               if ( y<yp ) then
!                  ixcross(1) := ixcross(1)+1
!               else if ( y>yp ) then
!                  ixcross(2) := ixcross(2)+1
!               else
!                  onobstacle := true

               if ( y<yp-eps ) then
                  ixcross(1) = ixcross(1)+1
               else if ( y>yp+eps ) then
                  ixcross(2) = ixcross(2)+1
               else
                  if ( abs(tang(2))>eps ) then
                     onobstacle = .true.
                     go to 1000
                  else if ( tang(1)>0d0 ) then
                     if ( xp>=xlow(1)-eps .and. &
                          xp<=xlow(1)+tang(1)+eps ) then
                        onobstacle = .true.
                        go to 1000
                     end if
                  else
                     if ( xp>=xlow(1)-eps+tang(1) .and. &
                          xp<=xlow(1)+eps ) then
                        onobstacle = .true.
                        go to 1000
                     end if
                  end if
               end if

            end if

         end if

         if ( abs(tang(2))>eps ) then

!        --- tang(2)#0
!            alpha := (yp-xlow(2))/tang(2)
!            x := xlow(2)-alpha*tang(2)

            alpha1 = (yp-xlow(2))/tang(2)
            x = xlow(1)+alpha1*tang(1)
            if ( doublepoint .and. &
                 ( alpha1>=-1d-6 .and. alpha1<=1d-6 .or. &
                   alpha1>=0.999999d0 .and. alpha1<=1.000001d0 ) ) then

!           --- Possiby double point, check x

               if ( abs(x-xprev)>1d-5 ) then

!              --- Points are different, set check

                  check = .true.
                  xprev = x
                  doublepoint = .false.

               end if

            else if ( alpha1>=-accuracy_obstacle .and. &
                      alpha1<=1d0+accuracy_obstacle .and. &
                      x>=xmin .and. x<=xmax ) then

!           --- xmin<=x<=xmax, set check

               check = .true.
               if ( alpha1>=-1d-6 .and. alpha1<=1d-6 .or. &
                    alpha1>=0.999999d0 .and. alpha1<=1.000001d0 ) then

!              --- Point at the end of a cell, amrk as possibly double

                  doublepoint = .true.
                  xprev = x

               end if

            end if
            if ( check ) then

!           --- Check = true, check
!               if ( x<xp ) then
!                  iycross(1) := iycross(1)+1
!               else if ( x>xp ) then
!                  iycross(2) := iycross(2)+1
!               else
!                  onobstacle := true

               if ( x<xp-eps ) then
                  iycross(1) = iycross(1)+1
                  nxi = nxi+1
                  xi(nxi) = x
               else if ( x>xp+eps ) then
                  iycross(2) = iycross(2)+1
               else
                  if ( abs(tang(1))>eps ) then
                     onobstacle = .true.
                     go to 1000
                  else if ( tang(2)>0d0 ) then
                     if ( yp>=xlow(2)-eps .and. &
                          yp<=xlow(2)+tang(2)+eps ) then
                        onobstacle = .true.
                        go to 1000
                     end if
                  else
                     if ( yp>=xlow(2)-eps+tang(2) .and. &
                          yp<=xlow(2)+eps ) then
                        onobstacle = .true.
                        go to 1000
                     end if
                  end if

               end if

            end if

         end if

      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshobstcross'

      end if  ! ( debug )
      call erclos ( 'mshobstcross' )

      end
