      subroutine mshobstintline ( nvert, inodes, iobst, &
                                  nx, ny, iverts, rverts, &
                                  nintersect, curves, nnodes, iwork1 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.2  date 02-10-2012 New call to mshobstintlin1
!        version  2.1  date 01-12-2005 Set accuracy to sqreps
!        version  2.0  date 26-11-2002 mshcrosscurve added
!                                      extra parameters
!                                      enlarged icell
!                                      added error (RvL)
!
!   copyright (c) 2000-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find the intersection of all sides in the element with vertex nodes
!     stored in inodes with an obstacle
!     The number of intersections and the side numbers are returned
!     If a side crosses an obstacle twice a warning is given and only
!     the first intersection is stored
! **********************************************************************
!
!                       KEYWORDS
!
!     intersection
!     line
!     element
!     obstacle
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nvert, inodes(3,*), iobst, nx, ny, nnodes, iwork1(*), &
              iverts(*), nintersect
      double precision rverts(*), curves(2,*)

!     curves         i    array containing the coordinates of the curves
!     inodes         i    Array of length 3*nvert containing the node numbers
!                         of the edges as well as the edge number
!                         inodes(1,i) = first node of edge i
!                         inodes(2,i) = last node of edge i
!                         inodes(3,i) = edge number
!     iobst          i    Obstacle sequence number
!     iverts         o    Array of length nintersect containing the edge
!                         numbers of the edges that interect the obstacle
!     iwork1         i    Integer work array of length npoint
!                         iwork1(i) = 1, means that node i is on the
!     nintersect     o    Number of intersections found
!     nnodes         i    Number of nodes
!     nvert          i    Number of edges to be considered
!     nx             i    number of cells in x direction
!     ny             i    number of cells in y direction
!     rverts         o    Contains the factors corresponding to the intersection
!                         points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer maxcells
      parameter ( maxcells=60 )
      integer i, ncell, ivert, icell(maxcells), ind
      double precision x1(2), x2(2), xlow(2), xupp(2), xmin, ymin, &
                       dx, dy, eps, fact1, fact2, eps1, xobst(2,3)
      logical found, skip

!     dx             length of vector in x-direction
!     dy             length of vector in y-direction
!     eps            Local accuracy
!     eps1           Local accuracy
!     fact1          Defines the factor for the intersection point
!                    The intersection point on the face is defined by
!                    x1 + fact1 * (x2-x1)
!     fact2          Defines the factor for the intersection point with respect
!                    to the obstacle curve
!                    There is only an intersection if 0<=fact2<=1
!     found          If true already a possibility has been found
!     i              Counting variable
!     icell          Array containing the curve parts that must be considered
!                    for computing the cross sections
!     ind            indicates index in curve part
!     ivert          Loop variable
!     maxcells       Declared length of array icell
!     ncell          Number of curve parts that must be used to compute
!                    cross sections
!     skip           If true the intersection must be skipped
!     x1             contains coordinates of first point on face
!     x2             contains coordinates of last point on face
!     xlow           contains coordinates of first point on curve part
!     xmin           Minimum x-value
!     xobst          Contains the coordinats of the 3 points of a quadratic
!                    obstacle curve element
!     xupp           contains coordinates of last point on curve part
!     ymin           Minimum y-value
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     MSHCROSSCURVE  Determine intersection of straight edge with curved
!                    obstacle edge
!     MSHCROSSLINE   Computes the intersection of two lines
!     MSHOBSTINTLIN1 Fill array icell with indices of curve parts of the
!                    obstacle that is possibly intersected by the line x1-x2
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
!    2211   Obstacle has more than one intersection with a face of the mesh
! **********************************************************************
!
!                       PSEUDO CODE
!
!     nintersect := 0
!     for ivert := 1 (1) nvert do
!        x1 := coordinates node inodes(i)
!        x2 := coordinates node inodes(i+1)
!        ncell := 0
!        For both points x1 and x2 do
!           Compute corresponding index pairs ip,jp referring to the
!              obstacle cells
!           ind := the corresponding part of the obstacle curve
!           for i := ind-1 (1) ind+1 do
!              if i not in icell then
!                 ncell:= ncell+1
!                 icell(ncell) := i
!           if ( there is a second curve crossing the obstacle cell ) then
!              ind := the corresponding part of the second obstacle curve
!              for i := ind-1 (1) ind+1 do
!                 if i not in icell then
!                    ncell:= ncell+1
!                    icell(ncell) := i
!        found := false
!        For i := 1 (1) ncell do
!           xlow := first point of obstacle part icell(i)
!           xupp := last point of obstacle part icell(i)
!           if ( the line x1-x2 intersects the line xlow-xupp ) then
!              if ( found ) then
!                 warning
!              else ( not yet found )
!                 found := true
!                 nintersect := nintersect+1
!                 Store ivert and coordinates of intersection point in iverts
!                 and rverts
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobstintline' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobstintline'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- nintersect := 0

      nintersect = 0

!     --- Extract xmin,xmax,ymin,ymax from obstacle array

      xmin = rmeshx(1,iobst)
      ymin = rmeshx(3,iobst)
      dx   = rmeshx(5,iobst)
      dy   = rmeshx(6,iobst)
      eps  = 0.01d0
      eps1 = sqreps

!     --- for ivert := 1 (1) nvert do

      do ivert = 1, nvert

!     --- x1 := coordinates node inodes(i)
!         x2 := coordinates node inodes(i+1)
!         ncell := 0

         x1(1) = coor(1,inodes(1,ivert))
         x1(2) = coor(2,inodes(1,ivert))
         x2(1) = coor(1,inodes(2,ivert))
         x2(2) = coor(2,inodes(2,ivert))

!        --- Fill array icell with indices of curve parts of the obstacle that
!            is possibily intersected by the line x1-x2

         call mshobstintlin1 ( x1, x2, xmin, ymin, nx, ny, iobgrid2d, &
                               icell, ncell, dx, dy, nnodes, maxcells, &
                               iobgrid2d1 )

         if ( ncell>20 ) then

!        --- ncell too large

            call errint ( ncell, 1 )
            call errint ( 12, 1 )
            call errchr ( 'icell', 1 )
            call errsub ( 550, 2, 0 , 1 )
            go to 1000

         end if

!        --- found := false

         found = .false.

!        --- For i := 1 (1) ncell do

         do i = 1, ncell

!        --- xlow := first point of obstacle part icell(i)
!            xupp := last point of obstacle part icell(i)

            ind = icell(i)
            xlow(1) = curves(1,ind)
            xlow(2) = curves(2,ind)
            xupp(1) = curves(1,ind+1)
            xupp(2) = curves(2,ind+1)

!           --- Check if lines intersect and compute intersection

            call mshcrossline ( x1, x2, xlow, xupp, fact1, fact2, eps )

            if ( fact2>-eps .and. fact2<1d0+eps ) then

!           --- fact2 between 0 and 1

               if ( abs(kmeshx(1,iobst))==3 ) then

!              --- Quadratic element
!                  determine the three coordinates corresponding to
!                  the quadratic obstacle element

                  if (mod(ind,2)>0) then
                     xobst(1,1) = curves(1,ind)
                     xobst(2,1) = curves(2,ind)
                     xobst(1,2) = curves(1,ind+1)
                     xobst(2,2) = curves(2,ind+1)
                     xobst(1,3) = curves(1,ind+2)
                     xobst(2,3) = curves(2,ind+2)
                  else
                     xobst(1,1) = curves(1,ind-1)
                     xobst(2,1) = curves(2,ind-1)
                     xobst(1,2) = curves(1,ind)
                     xobst(2,2) = curves(2,ind)
                     xobst(1,3) = curves(1,ind+1)
                     xobst(2,3) = curves(2,ind+1)
                  end if

                  call mshcrosscurve (x1, x2, xobst, fact1)

               end if

!              --- the line x1-x2 intersects the line xlow-xupp

               skip = .false.
               if ( abs(fact1)<=eps1 ) then

!              --- first point may also be a node

                  if ( iwork1(inodes(1,ivert))/=0 ) skip = .true.

               end if
               if ( abs(fact1)>=1d0-eps1 ) then

!              --- second point may also be a node

                  if ( iwork1(inodes(2,ivert))/=0 ) skip = .true.

               end if
               if ( .not. skip .and. found ) then

!              --- warning already an intersection found before

                   call errint ( iobst, 1 )
                   call errwar ( 2211, 1, 0, 0 )

!                  --- Compute a mean value except when on of the nodes close
!                      to the end of the edge

                   if ( fact1<eps1 ) then

!                  --- fact1<eps1, point close to first point
!                      choose point that is at longest distance of end points

                      if ( rverts(nintersect)<1d0-eps1 ) then

!                     --- standard case

                         rverts(nintersect) = &
                            max(rverts(nintersect),fact1)

                      else

!                     --- first point close to begin and other point close to
!                         end

                         if ( rverts(nintersect)>1d0-eps1+fact1 ) &
                            rverts(nintersect) = fact1

                      end if

                   else if ( fact1>1d0-eps1 ) then

!                  --- fact1>1-eps1, point close to second point
!                      choose point that is at longest distance of end points

                      if ( rverts(nintersect)>eps1 ) then

!                     --- standard case

                         rverts(nintersect) = &
                            min(rverts(nintersect),fact1)

                      else

!                     --- first point close to begin and other point close to
!                         end

                         if ( rverts(nintersect)<1d0-fact1 ) &
                            rverts(nintersect) = fact1

                      end if

                   else if ( rverts(nintersect)<eps1 ) then

!                  --- rverts(nintersect)<eps1, point close to first point
!                      New point not, hence use new point

                      rverts(nintersect) = fact1

                   else if ( rverts(nintersect)>1d0-eps1 ) then

!                  --- rverts(nintersect)>1-eps1, point close to end point
!                      New point not, hence use new point

                      rverts(nintersect) = fact1

                   else

!                  --- None of these cases, use mean value

                      rverts(nintersect) = &
                         0.5d0*(fact1+rverts(nintersect))

                   end if

               else if ( .not. skip ) then

!              --- no intersection found before
!                  found := true
!                  nintersect := nintersect+1
!                  Store ivert and coordinates of intersection point in iverts
!                  and rverts

                  found = .true.
                  nintersect = nintersect+1
                  iverts(nintersect) = inodes(3,ivert)
                  rverts(nintersect) = fact1

               end if

            end if

         end do

      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshobstintline'

      end if  ! ( debug )
      call erclos ( 'mshobstintline' )

      end
