      subroutine mshobstms ( meshorig, meshobst, map1d, iinputusr, rinputusr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.1  date 29-05-2014 Extension of timebefore
!        version  4.0  date 05-11-2013 Extra parameter rinputusr
!        version  3.0  date 10-10-2011 Split body in parts
!
!   copyright (c) 2001-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create a new mesh from a fixed mesh with an obstacle crossing the mesh
!     Elements that are intersected are subdivided into several elements,
!     such that these new elements all belong to the obstacle or are
!     outside the obstacle
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     subdivision
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemain
      use sepmoduleplot
      use sepmoduledebug
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer meshorig, meshobst, iinputusr(*)
      double precision rinputusr(*)
      type (mapping) :: map1d

!     iinputusr      i    Integer input array defining which actions must
!                         be undertaken
!                         At this moment iinputusr is under construction
!                         Contents
!                         Pos. 1  Last entry that is filled by the calling
!                                 program
!                         Pos. 2  shft: Defines whether nodes must be
!                                       shifted to the obstacle (1) or not (<1)
!                                       if -1 do not call mshobstbounxx
!                                       Default value: 0
!                         Pos. 3  smth: Defines type of smoothing;
!                                       Laplacian smoothing (1), angle
!                                       optimization (2), both (3), none (0)
!                                       Only allowed if shft=1
!     map1d          o    contains information of the mapping
!     meshobst       i    Sequence number of obstacle mesh to be used
!     meshorig       i    Sequence number of original mesh to be used
!     rinputusr      i    Real array containing real user input
!                         Contents:
!                         1:  eps_move  defines when a node may be shifted
!                                       to the boundary of en obstacle
!                                       0<=eps_move<=0.5
!                                       If an edge is intersected by an
!                                       obstacle and the distance between one
!                                       of the nodes and the intersection
!                                       point is less than eps_move*the length
!                                       of the edge the node may be shifted
!                         2:  eps_min   defines when nodes may be shifted
!                                       when they have not been shifted
!                                       in the first step.
!                                       This step is meant to avoid very small
!                                       elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: shft, iseqkm1, iseqkm2, idum(1)
      integer, pointer :: kmeshnew
      logical debugloc
      double precision :: timebefore(2)

!     debugloc       If true local debug statements are carried out otherwise
!     idum           Dummy array
!     iseqkm1        Sequence number with respect to km of first mesh
!     iseqkm2        Sequence number with respect to km of second mesh
!     kmeshnew       New array kmesh
!     shft           Parameter that determines if nodes must be shifted (1) or
!                    not (0)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLMN         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     ERSETTIME      Get the CPU time at start of subroutine
!     MSHCHECKELMS   Check if two-dimensional mesh contains hanging nodes
!     MSHOBSTCHECK   Check consistency of nodes in array ipointobst and
!                    elements in array ielemobst
!     MSHOBSTMS08    Delete some parts of kmeshnew and copy parts l and m
!                    from kmesh
!     MSHOBSTMS1D    Create a new mesh from a fixed mesh with an obstacle
!                    crossing the mesh (1d)
!     MSHOBSTMS23D   Create a new mesh from a fixed mesh with an obstacle
!                    crossing the mesh (2d)
!     MSHOBSTMS2DSH  Create a new mesh from a fixed mesh with an obstacle
!                    crossing the mesh (2d), nodes may be shifted
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPPLOT3DOBST  Make three dimensional plot of obstacles
!     SEPPLOTM2      plot 2D mesh
!     SEPPRINKMESH   Print array KMESH
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2290   No obstacles in mesh
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Perform some tests
!     Activate arrays
!     Reserve temporary space
!     Compute starting addresses
!     Create new mesh
!     Make new arrays permanent
!     Compute some extra arrays by topol
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobstms' )
      if ( itime>0 ) call ersettime ( timebefore )
      debug = .false.
      debugloc = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobstms'
         write(irefwr,1) 'meshorig, meshobst', meshorig, meshobst
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- extract parameters from iinputusr

      shft = 0
      if ( iinputusr(1)>1 ) shft = iinputusr(2)
      kmesh => kmesh2d(meshorig)
      kmeshnew => kmesh2d(meshobst)

      call sepfillkmesh ( kmesh )
      iseqkm1 = iseqkm

      if ( debug ) &
         write(irefwr,1) 'iseqkm1, kmeshnew', iseqkm1, kmeshnew

!     --- Perform some tests

      if ( numobst==0 ) then

!     --- No obstacle

         call errsub ( 2290, 0, 0, 0 )
         go to 1000

      end if
      if ( debug ) &
         write(irefwr,1) 'numobst, ndim, shft', numobst, ndim, shft

!     --- Delete some parts of kmeshnew and copy part l from kmesh

      call mshobstms08 ( kmeshnew )
      iseqkm = iseqkm1

      if ( ndim==1 ) then

!     --- ndim = 1, simple 1 d case

         call mshobstms1d ( meshorig, meshobst, map1d )

      else if ( shft<=0 ) then

!     --- ndim = 2, 3, shft<=0:  2d or 3d case without shift

         call mshobstms23d ( meshorig, meshobst, map1d, shft, rinputusr )

      else

!     --- ndim = 2, 3, shft = 1:  2d or 3d case with shift
!         Only for special purposes

         call mshobstms2dsh ( meshorig, meshobst, map1d, iinputusr )

      end if  ! ( ndim==1 )

      kmesh =>  kmesh2d(meshobst)
      call sepfillkmesh ( kmesh )

      if ( checkobstconsistency ) then

!     --- Check consistency of obstacle

         call mshobstcheck

      end if  ! ( checkobstconsistency )

      call sepfillkmesh ( kmesh )

      if ( debug ) then

!     --- local debugging, plot and print mesh

         select case ( ndim )
         case(2)

!        --- ndim = 2

            jmark = 0
            jkader = -1
            call sepplotm2 ( 0, idum, 15d0, 1d0 )
            call sepplotm2 ( 0, idum, 15d0, 1d0 )

         case(3)

!        --- ndim = 3

            call sepplot3dobst

         end select  ! case ( ndim )

         if ( debugloc ) call sepprinkmesh ( 0 )

      end if

      if ( check_hanging_nodes ) then

!     --- check_hanging_nodes is true, check mesh (hanging nodes)

         call mshcheckelms

      end if

1000  if ( debug ) then

!     --- Debug information

         if ( .false. ) then
         write(irefwr,*) 'old mesh'
         kmesh => kmesh2d(meshorig)
         call sepprinkmesh ( 1 )
         call sepprinkmesh ( 38 )
         write(irefwr,*) 'new mesh'
         kmesh =>kmesh2d(meshobst)
         call sepprinkmesh ( 1 )
         call sepprinkmesh ( 38 )
         kmesh =>kmesh2d(meshobst)
         kmesh => kmesh2d(meshorig)
         end if  ! ( .false. )
         write(irefwr,*) 'End mshobstms'

      end if
      call erclmn ( 'mshobstms', timebefore )

      end subroutine mshobstms
