      subroutine mshr03 ( iinput, lengin, inpmax, interm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 28-03-2013 Extra parameter interm
!        version  3.3  date 19-11-2008 Allow constants
!        version  3.2  date 02-03-2001 Minor correction call intx00
!
!   copyright (c) 1990-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpret the information following the phrase intermediate points
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     intermediate
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpmax, iinput(inpmax), lengin
      logical interm

!     iinput        i/o  integer input array
!     inpmax         i   maximal space for iinput
!     interm        i/o  If true intermediate has been found
!     lengin        i/o  Length of array IINPUT. The actual length is 5 longer
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nlcode, ichois, n, niedge, isub, nisurf, nivolm
      parameter ( nlcode=5 )
      character (len=3) clcode(nlcode)

!     clcode     Table containing the various local codes to be recognized, &
!                See ICHOIS for the various possibilities
!     ichois     Choice parameter, indicating what type of word has been found
!                Possibilities:
!                1:  curves
!                2:  surfaces
!                3:  volumes
!                4:  subdivision
!                5:  sidepoints
!                6:  midpoints
!     isub       Type of subdivision:
!                0:  equidistant
!                1:  Gauss-Legendre
!     n          Number of groups that have been read
!     niedge     Number of internal (spectral) points in line
!     nisurf     Number of internal (spectral) points in surface
!     nivolm     Number of internal (spectral) points in volume
!     nlcode     Length of array clcode
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     READREALCONS   Check if last string read corresponds to constant
!     SEPCOPYCODE    Copy code2 and related arrays into code
! **********************************************************************
!
!                       I/O
!
!     The following type of input is permitted:
!     definition of points,  type of subdivision
!        with
!     definition of points may have one of the following shapes:
!        sidepoints = niedge  (all line elements and all edges of 2 and 3
!                              dimensional elements will get niedge internal
!                              points)
!        facepoints = nisurf  (all surface elements and all faces of 3d
!                              elements will get nisurf internal points)
!        vlmpoints  = nivolm  (all volume elements will get nivolm internal
!                              points)
!        midpoints  = m       (all line elements get m points, all surface
!                              elements m*m and all volume elements m*m*m)
!     type of subdivision may have one of the following shapes:
!        subdivision = equidistant
!        subdivision = Legendre
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     142   Internal error, too few space for integers
!     274   character could not be recognized
!     648   string could not be recognized
!    1366   no sidepoints found (sidepoints is obligatory)
!    1369   number of sidepoints found has wrong value
!    1370   number of sidepoints found has wrong value
!    1552   keyword found more than once
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data clcode / 'SID', 'FAC', 'VLM', 'SUB', 'MID' /
! ======================================================================
!
      call eropen ( 'mshr03' )

      if ( interm ) then

!     --- intermediate has been found more than once

         call errchr ( string(1:nchar), 1 )
         call errwar ( 1552, 0, 0, 1 )

      end if
      interm = .true.

      n          = 0
      iinput(14) = lengin + 6
      lengin     = lengin + 1

!     --- Read next record until main keyword is found

100   ityprd = 1
      call asknxt

!     --- Interpret item

      if ( ityprd/=4 ) then

!     --- ityprd # 4, not a string found

         call errsub ( 274, 0, 0, 0 )
         call errcrd
         go to 100

      else

!     --- If a main keyword has been found, return to calling subroutine

         call sepcopycode ( codemain(3), nncodemain-2, ncodemain(3), &
                            ncodmxmain(3) )
         ierpri = 0
         icode = sepsubkeyw()-1
         if ( icode>=0 ) then
            icode = icode + 2
            go to 1000
         end if

      end if

!     --- Intermediate record, set defaults

      n      = n + 1
      niedge = -1
      nisurf = 0
      nivolm = 0
      isub   = 0

!     --- Detect contents of record

200   if ( ityprd==1 ) then

!     --- End of record found

         if ( niedge<0 ) then

!        --- No edge points found in this record

            call errsub ( 1366, 0, 0, 0 )
            call errcrd

         end if

         if ( lengin+4>=inpmax ) then

!        --- lengin>=  inpmax

            call errint ( inpmax, 1 )
            call errint ( lengin+4, 2 )
            call errsub ( 142, 2, 0, 0 )
            go to 1000

         end if

         iinput(lengin+1) = niedge
         iinput(lengin+2) = nisurf
         iinput(lengin+3) = nivolm
         iinput(lengin+4) = isub
         lengin           = lengin + 4

         go to 100

      else if ( ityprd/=4 ) then

!     --- ityprd # 4, not a string found

          call errsub ( 274, 0, 0, 0 )
          call errcrd
          go to 100

      else

!     --- Text found, detect this text

         nncode = nlcode
         code(1:nncode) = clcode
         ierpri = 1
         ncodefix = 3
         ichois = sepkeyword3()

         if ( ichois==0 ) then

            go to 100

         else if ( ichois==1 ) then

!        --- ichois = 1, sidepoints found
!            Interprete rest of text

            call asknxt
            call readrealcons
            if ( ityprd==2 ) then

!           --- Number found

               niedge = nint( realrd )

               if ( niedge<0 ) then

!              --- niedge has wrong value

                  call errint ( niedge, 1 )
                  call errsub ( 1370, 1, 0, 0 )
                  call errcrd

               end if

            else

               call errsub ( 274, 0, 0, 0 )
               call errcrd
               go to 1000

            end if

            call asknxt

         else if ( ichois==2 ) then

!        --- ichois = 2, facepoints found
!            Interpret rest of text

            call asknxt
            call readrealcons
            if ( ityprd==2 ) then

!           --- Number found

               nisurf = nint( realrd )

               if ( nisurf<-1 ) then

!              --- nisurf has wrong value

                  call errint ( nisurf, 1 )
                  call errsub ( 1370, 1, 0, 0 )
                  call errcrd

               end if

            else

               call errsub ( 274, 0, 0, 0 )
               call errcrd
               go to 1000

            end if

            call asknxt

         else if ( ichois==3 ) then

!        --- ichois = 3, vlmpoints found
!            Interpret rest of text

            call asknxt
            call readrealcons
            if ( ityprd==2 ) then

!           --- Number found

               nivolm = nint( realrd )

               if ( nivolm<-1 ) then

!              --- nivolm has wrong value

                  call errint ( nivolm, 1 )
                  call errsub ( 1370, 1, 0, 0 )
                  call errcrd
                  go to 1000

               end if

            else

               call errsub ( 274, 0, 0, 0 )
               call errcrd

            end if

            call asknxt

         else if ( ichois==4 ) then

!        --- ichois = 4, sudivision found
!            Interpret rest of text

            call asknxt
            if ( ityprd/=4 ) then
               call errsub ( 274, 0, 0, 0 )
               call errcrd
            else if ( string(1:1)=='E' ) then

!           --- equidistant found

               isub = 0

            else if ( string(1:1)=='L' ) then

!           --- Legendre found

               isub = 1

            else

!           --- wrong string found

               call errchr ( string(1:nchar), 1 )
               call errsub ( 648, 0, 0, 1 )
               call errcrd

            end if

            call asknxt

         else if ( ichois==5 ) then

!        --- ichois = 5, midpoints found
!            Interprete rest of text

            call asknxt
            call readrealcons
            if ( ityprd==4 ) then

!           --- Text found, must be: FILLED

               if ( string(1:4)=='FILL' ) then

                  if ( nisurf==0 ) nisurf = -1
                  if ( nivolm==0 ) nivolm = -1

               else

!              --- wrong string found

                  call errchr ( string(1:nchar), 1 )
                  call errsub ( 648, 0, 0, 1 )
                  call errcrd

               end if

            else if ( ityprd==2 ) then

!           --- Number found

               niedge = nint( realrd )
               nisurf = -1
               nivolm = -1

               if ( niedge<1 ) then

!              --- niedge has wrong value

                  call errint ( niedge, 1 )
                  call errsub ( 1369, 1, 0, 0 )
                  call errcrd

               end if

            else

               call errsub ( 274, 0, 0, 0 )
               call errcrd

            end if

            call asknxt

         end if
         go to 200

      end if

1000  iinput ( iinput(14)-5 ) = n

      call erclos ( 'mshr03' )

      end subroutine mshr03
