      subroutine mshrd1rest ( maxnum, iinput, iinp, ndim, cunit, nuspnt, &
                              icurve, icurnr, rinput, rinpt1, rincor, &
                              coorusp, maxcrv )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 08-01-2015 Correction for ccircle
!        version  1.0  date 20-11-2014
!
!   copyright (c) 2014-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read rest of information for curve record
!
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     mesh
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmodulecomio
      use sepmoduleconsts
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: ndim, icurnr, nuspnt, maxcrv
      integer, intent(inout) :: iinput(*), iinp, icurve
      integer, intent(out) :: maxnum
      double precision, intent(in) :: cunit, coorusp(ndim,nuspnt), &
                                      rincor(*)
      double precision, intent(inout) :: rinput(*), rinpt1(2,maxcrv)

!     coorusp        i    contains coordinates of user points
!     cunit          i    Unit length for coarseness
!     icurnr         i    Curve sequence number
!     icurve        i/o   Type of curve
!     iinp          i/o   last position used in array iinput
!     iinput        i/o   integer input array
!     maxcrv         i    Maximal number of curves allowed
!     maxnum         o    Maximal number of numbers to be read
!     ndim           i    Dimension of the space.
!     nuspnt         i    Number of user points
!     rincor         i    Contains the coarseness in the user points
!     rinpt1        i/o   in this array the initial and end parameters t0 and t1
!                         for the curves are stored temporarily
!                         storage: t0 for curve i in rinpt1(1,i)
!                                  t1 for curve i in rinpt1(2,i)
!     rinput        i/o   real input array;  factors are filled from position 1, &
!                         hence rinput must be started from 8 + ndim * nuspnt
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, n, p1, p2, ifunc, iprofile, iratio, iseq, it1, it2, &
                 itype, j, ncheck, nelm, nodd
      double precision :: factor, t0, t1, alpha, radius, circumference, &
                          dx(ndim), coarseness
      logical :: coarsecurv, speccurv, paramcurv

!     alpha          Parameter alpha with respect to splines
!     circumference  circumference of circle
!     coarsecurv     If true coarse is used to create the distribution of nodes
!                    along the curve
!     coarseness     coarseness in a point
!     dx             difference vector
!     factor         Multiplication factor
!     i              Counting variable
!     ifunc          Parameter ifunc for own_curve
!     iprofile       Defines the type of profile
!                    Possible values:
!                    1: upper part of naca0012
!                    2: lower part of naca0012
!     iratio         Ratio as defined in input
!     iseq           Sequence number in array work to be used for finding other
!                    data than nodal point numbers
!     it1            Nodal point number corresponding to first tangent
!     it2            Nodal point number corresponding to second tangent
!     itype          Indicates the type of action corresponding to keywords
!                    in code
!     j              Counting variable
!     n              Number of nodes stored in array work
!     ncheck         Number of user points that must be checked
!     nelm           Number of elements given by the user
!     nodd           Parameter nodd as defined in input
!     p1             user point number of centroid of circle
!     p2             first user point on circle
!     paramcurv      If true a parameter curve is created
!     radius         Radius of sphere or circle
!     speccurv       If true a special curve (by contraction of copying)
!                    is created
!     t0             Initial t-value in parameter curve
!     t1             End t-value in parameter curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     MSHR00         Read information after the part Ci = LINE ( .. )
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     179   point number of user point is incorrect
!     534   The value of IRATIO is incorrect
!    1148   Not tangent given for begin and end point of SPLINE record of
!           type 4
!    1191   Tang has been read, whereas type # 4
!    1194   Value of nelm incorrect
!    1197   Parameter out of range
!    1198   t0>=t1
!    1286   Too few user points found
!    1483   Coinciding user points at one curve
!    1575   Useless parameter found in curve record
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrd1rest' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrd1rest'
         write(irefwr,1) 'icurve, iinp', icurve, iinp
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( work(1300) )

!     --- Detect rest of record

      call mshr00 ( nuspnt )

      n      = nint(work(1))
      nelm   = nint(work(2))
      iratio = nint(work(3))
      factor = work(4)
      nodd   = nint(work(5))
      t0     = work(6)
      t1     = work(7)
      itype  = nint(work(8))
      alpha  = work(12)
      iprofile = nint(work(13))
      ifunc  = nint(work(15))
      coarsecurv = icurve==5 .or. icurve==6 .or. icurve==9 .or. &
                   icurve==16 .or. icurve==23 .or. icurve==24
      speccurv = icurve==2 .or. icurve==7 .or. &
                 icurve>=10 .and. icurve<=13 .or. icurve==21
      paramcurv = icurve==8 .or. icurve==9 .or. icurve==15 .or. &
                  icurve==16

      if ( debug ) then
         write(irefwr,1) 'n, nelm, iratio, nodd, itype', &
                          n, nelm, iratio, nodd, itype
         write(irefwr,1) 'iprofile, ifunc', iprofile, ifunc
         write(irefwr,2) 'factor, alpha, t0, t1', &
                          factor, alpha, t0, t1
         write(irefwr,*) 'coarsecurv, speccurv, paramcurv ', &
                          coarsecurv, speccurv, paramcurv
      end if  ! ( debug )

      if ( nelm>0 .and. (coarsecurv .or. speccurv) ) then

!     --- Parameter nelm found where it does not make sense

         call errchr ( 'nelm', 1 )
         call errint ( icurnr, 1 )
         call errwar ( 1575, 1, 0, 1 )

      end if
      if ( iratio/=-100 .and. (coarsecurv .or. speccurv) ) then

!     --- Parameter ratio found where it does not make sense

         call errchr ( 'ratio', 1 )
         call errint ( icurnr, 1 )
         call errwar ( 1575, 1, 0, 1 )

      end if
      if ( abs(factor-1d0)>1d-5 .and. (coarsecurv .or. speccurv) ) then

!     --- Parameter factor found where it does not make sense

         call errchr ( 'factor', 1 )
         call errint ( icurnr, 1 )
         call errwar ( 1575, 1, 0, 1 )

      end if
      if ( nodd/=0 .and. .not. coarsecurv ) then

!     --- Parameter nodd found where it does not make sense

         call errchr ( 'nodd', 1 )
         call errint ( icurnr, 1 )
         call errwar ( 1575, 1, 0, 1 )

      end if
      if ( abs(t0)>1d-5 .and. .not. paramcurv ) then

!     --- Parameter init found where it does not make sense

         call errchr ( 'init', 1 )
         call errint ( icurnr, 1 )
         call errwar ( 1575, 1, 0, 1 )

      end if
      if ( abs(t1-1d0)>1d-5 .and. .not. paramcurv ) then

!     --- Parameter end found where it does not make sense

         call errchr ( 'end', 1 )
         call errint ( icurnr, 1 )
         call errwar ( 1575, 1, 0, 1 )

      end if
      if ( itype>=15 .and. itype<=16 ) then

!     --- profile

         if ( t0<0d0 .or. t0>1d0 ) then

!        --- t0 out of range

            call erreal ( t0, 1 )
            call errchr ( 't0', 1 )
            call errint ( iprofile, 1 )
            call errsub ( 1197, 1, 1, 1 )

         end if
         if ( t1<0d0 .or. t1>1d0 ) then

!        --- t1 out of range

            call erreal ( t1, 1 )
            call errchr ( 't1', 1 )
            call errint ( iprofile, 1 )
            call errsub ( 1197, 1, 1, 1 )

         end if
         if ( t0>=t1 ) then

!        --- t0>=t1

            call erreal ( t0, 1 )
            call erreal ( t1, 1 )
            call errint ( iprofile, 1 )
            call errsub ( 1198, 1, 2, 0 )

         end if

      end if
      if ( itype/=1 .and. icurve/=4 .and. icurve/=12 ) then

!     --- Parameter type found where it does not make sense

         call errchr ( 'type', 1 )
         call errint ( icurnr, 1 )
         call errwar ( 1575, 1, 0, 1 )

      end if

      if ( nelm==0 ) then

!     --- nelm = 0, replace line by cline etc.

         select case ( icurve )
            case(1)
               icurve = 5   ! cline
            case(3)
               icurve = 6   ! carc
            case(4)
               icurve =12   ! cspline
            case(8, 15, 20)
               icurve = icurve+1   ! cparam, cprofile, cell_arc
            case(17)
               icurve =24   ! ccircle
         end select  ! case

      end if  ! ( nelm==0 )

      iinput(iinp) = icurve

      if ( debug ) &
         write(irefwr,1) 'nelm, icurve, iinp', nelm, icurve, iinp

!     --- Distinguish between the various curves

      select case ( icurve )

         case(1)

!        --- icurve = 1, LINE found

            maxnum = 2
            ncheck = 2
            if ( n>=3 ) then

!           --- n>2, more than 2 numbers read

               iseq = 2
               if ( nelm==0 ) then
                  nelm = nint(work(25+iseq))
                  iseq = iseq+1
               end if
               if ( iratio==-100 .and. iseq<n ) then
                  iratio = nint(work(25+iseq))
                  iseq = iseq+1
               end if
               if ( abs(factor-1d0)< 10d0*epsmac .and. iseq<n ) then
                  factor = work(25+iseq)
                  iseq = iseq+1
               end if

            end if

         case(2)

!        --- icurve = 2, User defined curve found

            maxnum = n
            ncheck = n
            nelm = n-1

         case(3,22)

!        --- icurve = 3, ARC or ELL_ARC found

            maxnum = 3
            ncheck = 1
            if ( n>=4 ) then

!           --- n>3, more than 3 numbers read

               iseq = 3
               if ( nelm==0 ) then
                  nelm = nint(work(25+iseq))
                  iseq = iseq+1
               end if
               if ( iratio==-100 .and. iseq<n ) then
                  iratio = nint(work(25+iseq))
                  iseq = iseq+1
               end if
               if ( abs(factor-1d0)< 10d0*epsmac .and. iseq<n ) then
                  factor = work(25+iseq)
                  iseq = iseq+1
               end if

            end if

         case(4, 12)

!        --- icurve = 4 or 12, (C)Spline found

            maxnum = n
            ncheck = n

!           --- Check whether Tangent values have been given

            it1 = nint(work(9))
            it2 = nint(work(10))
            if ( itype==4 .and. it2==0 ) then

               call errsub ( 1148, 0, 0, 0 )

            else if ( itype/=4 .and. it1/=0 ) then

!           --- Tang read, whereas itype # 4

               call errsub ( 1191, 0, 0, 0 )

            end if
            if ( abs(alpha)<sqreps ) then
               itype = itype+10
            else if ( abs(alpha-0.5d0)<sqreps ) then
               itype = itype+20
            else if ( abs(alpha-1d0)>sqreps ) then
               itype = itype+30
            end if

         case(5)

!        --- icurve = 5, CLINE found

            maxnum = n
            ncheck = n

            if ( debug ) write(irefwr,1) 'n, nodd', n, nodd

         case(6, 23)

!        --- icurve = 6, 23 CARC or CELL_ARC found

            maxnum = 3
            ncheck = 1
            if ( n>=4 ) then

!           --- n>3, more than 3 numbers read

               if ( nodd==0 ) nodd = nint(work(28))

            end if

         case(8)

!        --- icurve = 8, PARAM found

            maxnum = 2
            ncheck = 1
            if ( n>=3 ) then

!           --- n>2, more than 2 numbers read

               iseq = 2
               if ( nelm==0 ) then
                  nelm = nint(work(25+iseq))
                  iseq = iseq+1
               end if
               if ( nelm==0 ) then
                  nelm = nint(work(25+iseq))
                  iseq = iseq+1
               end if
               if ( iratio==-100 .and. iseq<n ) then
                  iratio = nint(work(25+iseq))
                  iseq = iseq+1
               end if
               if ( abs(factor-1d0)< 10d0*epsmac .and. iseq<n ) then
                  factor = work(25+iseq)
                  iseq = iseq+1
               end if

            end if

         case(9)

!        --- icurve = 9, CPARAM found

            maxnum = 2
            ncheck = 1
            if ( n>=3 ) then

!           --- n>2, more than 2 numbers read

               if ( nodd==0 ) nodd = nint(work(27))

            end if

         case(15, 16, 20)

!        --- icurve = 15, 16, 20, PROFILE, CPROFILE, OWN_CURVE found

            maxnum = 2
            ncheck = 2

         case(17)

!        --- icurve = 17, CIRCLE found

            maxnum = ndim
            ncheck = ndim

         case(21)

!        --- icurve = 21, framecurve found

            maxnum = n
            ncheck = n

         case(24)

!        --- icurve = 24, CCIRCLE

            maxnum = max(n,ndim)
            ncheck = maxnum

      end select  ! case ( icurve )

!     --- Fill nodal points in iinput

      if ( iratio==-100 ) iratio = 0
      if ( iratio==3 .or. iratio==4 .or. &
           iratio==7 .or. iratio==8 ) then

!     --- iratio = 3, 4, 7, 8  special situation: iratio: = iratio-2;
!                        factor: = 1/factor

         iratio = iratio - 2
         if ( abs(factor)>epsmac ) factor = 1d0 / factor

      end if
      if ( iratio<=-4 ) then

!     --- or iratio = -4, -5:  iratio : =  iratio + 2; factor : =  1/factor

         iratio = iratio + 2
         if ( abs(factor)>epsmac ) factor = 1d0/factor

      end if

      if ( debug ) write(irefwr,1) 'iratio, iinp', iratio, iinp
      iinput(iinp+3) = iratio
      if ( icurve/=8 .and. iratio<0 ) then
         call errint ( iratio, 1 )
         call errint ( icurve, 2 )
         call errsub ( 534, 2, 0, 0 )
      end if

      if ( icurve==5 .or. icurve==24 ) iinput(iinp+3) = n
      rinput(icurnr)  = factor
      rinpt1(1,icurnr) = t0
      rinpt1(2,icurnr) = t1
      if ( nelm<0 .or. nelm>10000 ) then

!     --- nelm out of range

         call errint ( nelm, 1 )
         call errint ( 10000, 2 )
         call errsub ( 1194, 2, 0, 0 )

      end if  ! ( nelm<0 .or. nelm>10000 )

      if ( icurve<=4 .or. icurve==8 .or. icurve==15 .or. &
           icurve==17 .or. icurve==20 .or. icurve==22 ) &
         iinput(iinp+2) = nelm
      if ( icurve==4 .or. icurve==12 ) then

!     --- Splines

         if ( itype==4 ) then

!        --- itype = 4, store tangents

            work(25+maxnum) = it1
            work(26+maxnum) = it2
            maxnum = maxnum + 2
            n = n + 2

         end if

         iinput(iinp+4) = itype
         iinput(iinp+5) = n
         iinp = iinp + 2
         rinpt1(1,icurnr) = alpha

      else if ( icurve==15 .or. icurve==16 ) then

!     --- profile

         iinput(iinp+4) = iprofile
         iinp = iinp + 1

      else if ( icurve==21 ) then

!     --- framecurve

         iinput(iinp+4) = 1     ! itype (not yet used)
         iinput(iinp+5) = n     ! number of nodes
         iinp = iinp + 2        ! two extra positions

      end if
      if ( icurve==20 ) iinput(iinp+3) = ifunc

      if ( n<maxnum ) then

!     --- Too few user points found for curve

         call errint ( icurnr, 1 )
         call errint ( n, 2 )
         call errint ( maxnum, 3 )
         call errsub ( 1286, 3, 0, 0 )

      end if

      if ( debug ) write(irefwr,1) 'iinp', iinp

      do i = 1, maxnum
         iinput(iinp+i+3) = nint(work(24+i))
         if ( iinput(iinp+i+3)==0 .or. abs(iinput(iinp+i+3))> &
              nuspnt ) then

!        --- Error 179: User point number <1 or >number of user points

            call errint ( iinput(iinp+i+3), 1 )
            call errint ( nuspnt, 2 )
            call errint ( i, 3 )
            call errint ( icurnr, 4 )
            call errsub ( 179, 4, 0, 0 )

         end if

      end do

      if ( debug ) write(irefwr,1) 'iinp', iinp

      if ( icurve==24 .and. iinput(iinp+3)==ndim ) then

!     --- icurve = 24, hence ccircle, compute nelm and reset to icurve = 17

         p1 = iinput(iinp+4)           ! centre of circle
         p2 = iinput(iinp+5)           ! point on circle
         dx = coorusp(:,p2)-coorusp(:,p1)
         radius = sqrt(dot_product(dx,dx))
         circumference = 2d0*pi*radius
         coarseness = rincor(p2)
         if ( coarseness<=0d0 ) coarseness = 1d0
         coarseness = coarseness*cunit
         nelm = nint(circumference/coarseness)
         if ( nodd==2 .and. mod(nelm,2)==0 ) nelm = nelm+1  ! correction
         icurve = 17    ! reset icurve
         rinput(icurnr) = 1d0     ! factor
         iinput(iinp) = 17
         iinput(iinp+2) = nelm
         iinput(iinp+3) = 0       !iratio

      end if  ! ( icurve==24 )

!     --- Test for coinciding user points
!         The only exception is that for cline (icurve==5) the first
!         and last point may be the same as ong as there are more than
!         3 user points

      do i = 1, ncheck-1
         do j = i+1, ncheck
            if ( iinput(iinp+i+3)==iinput(iinp+j+3) .and. .not. &
                 (i==1 .and. j==ncheck .and. icurve==5 .and. ncheck>3) ) then

!           --- Error 1483: Coinciding user points

               call errint ( i, 1 )
               call errint ( icurnr, 2 )
               call errint ( iinput(iinp+i+3), 3 )
               call errint ( j, 4 )
               call errsub ( 1483, 4, 0, 0 )

            end if

         end do

      end do
      iinp = iinp + 4 + maxnum

      if ( debug ) &
         write(irefwr,1) 'iinp, icurve, nodd', iinp, icurve, nodd

      if ( icurve==5 .or. icurve==6 .or. icurve==9 .or. &
           icurve==12 .or. icurve==16 .or. icurve==21 .or. &
           icurve==23 .or. icurve==24 ) then

!     --- ICURVE = 5, 6, 9, 12, 16, 21, 24  C... found, Fill NODD

         iinput(iinp) = nodd
         iinp = iinp + 1

      end if
      deallocate ( work )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iinp', iinp
         call prinin ( iinput, iinp-1, 'iinput' )
         write(irefwr,*) 'End mshrd1rest'

      end if  ! ( debug )
      call erclos ( 'mshrd1rest' )

      end subroutine mshrd1rest
