      subroutine mshrd2rest ( iinp, iinput, maxsrf, isurf, isurnr, &
                              icopy, icurvs, ncurvs, nuspnt, rinput, &
                              rpin, part4, trian, nsurfs, ipoint, &
                              clcode, ndim, ishape, lenipoint, &
                              changeinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 21-11-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read rest of surface input
!
! **********************************************************************
!
!                       KEYWORDS
!
!     surface
!     mesh
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmoduleread
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: maxsrf, icopy, icurvs(*), ishape, &
                             ncurvs, nuspnt, ndim, lenipoint
      integer, intent(inout) :: iinp, iinput(*), isurf, nsurfs, &
                                ipoint(*), isurnr
      double precision, intent(out) :: rinput(4,maxsrf), rpin(4,ncurvs)
      logical, intent(out) :: part4
      logical, intent(inout) :: changeinp
      logical, intent(in) :: trian
      character (len=*) clcode(*)

!     changeinp     i/o   if true extra curves have been created
!     clcode         i    In this table the significant characters of the
!                         various types of surfaces are stored.
!     icopy          i    Number of surface to be copied
!     icurvs         i    Array indicating if curve i has been filled
!                         (icurvs(i)>0) or not (icurvs(i)=0)
!     iinp          i/o   last position used in array iinput
!     iinput        i/o   integer input array
!                         See subroutine mshrea
!     ipoint        i/o   Array of length nparts+1 containing the starting
!                         addresses in array inodes of the various parts
!                         the boundary consists of.
!                         ipoint(1) = 1, &
!                         number of nodes at part i = ipoint(i+1)-ipoint(i)
!     ishape         i    Shape number of elements
!     isurf         i/o   Type of surface with respect to mesh generator
!     isurnr        i/o   Surface sequence number
!     lenipoint      i    Length of array ipoint
!     maxsrf         i    Maximal number of surfaces allowed
!     ncurvs         i    Number of curves in mesh
!     ndim           i    Dimension of the space.
!     nsurfs         i    number of surfaces
!     nuspnt        i/o   Number of user points
!     part4          o    If true rinput part 4 is used
!     rinput         o    part of real input array, length 4*maxsrf
!     rpin           o    information conserning thickness and factor for paver
!     trian          i    Indication whether triangles (true) or quadrilaterals
!                         are created (false)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ncur, n, m, nelem, npoint, iblend, icurve, ismoot, &
                 idiag, mapping, icentre, itypesurf, nsur, iregular, &
                 nlayers, nicv, jinp, icurnr, i
      double precision :: umin, umax, vmin, vmax

!     i              Counting variable
!     iblend         Parameter for generator MSHZDE
!     icentre        User point number of centroid
!     icurnr         Curve sequence number
!     icurve         Parameter for generator MSHZDE
!     idiag          Consists of 2 parts idiag_orig and interpolation
!                    accordinag to idiag = idiag_orig + 10 * interpolation
!                    idiag_orig: defines diagonal direction:
!                       0: standard (counter clockwise)
!                       1: clockwise
!                       2: alternating
!                    interpol: defines the type of interpolation
!                       0: default
!                       1: hor_straight
!                       2: vert_straight
!     iregular       Defines the type of subdivision
!                    0: default
!                    1: regular
!     ismoot         Smoothing parameter, the iteration process is stopped
!                    if the difference between two grids is less than
!                    10^-ismoot
!     itypesurf      Defines type of surface to be created
!     jinp           last position used in array iinput with respect to
!                    iprec
!     m              Parameter m in surface record
!     mapping        defines the type of mapping to a 3d surface
!     n              Parameter n in surface record
!     ncur           Number of curves surrounding a surface
!     nelem          Number of elements
!     nicv           Number of data records to store per curve (1 in general
!                    3 for paver
!     nlayers        Maximum number of layers for application of paver
!                    algorithm
!                    The rest of the region is filled by triangle
!     npoint         Number of nodal points
!     nsur           Number of surfaces surrounding a volume
!     umax           largest value first coordinate parameter surface
!     umin           smallest value first coordinate parameter surface
!     vmax           largest value second coordinate parameter surface
!     vmin           smallest value second coordinate parameter surface
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCARD         Mark part in a record by underlining with ***
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHR01         Read information after the part Si = RECTANGLE ( ..
!     MSHRD2COONS    Fill information for coons or quadrilateral in array iinput
!     MSHRD2FRAME    Fill information for framesurf or internal points in array
!                    iinput
!     MSHRD2PAVER    Fill rpin in case of paver
!     MSHRD2RECT     Fill information for rectangle in array iinput
!     MSHRD2SPHERE   Fill information for sphere in array iinput
!     MSHRD2SQUARE   Fill information for square in array iinput
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     129   Incorrect curve number found
!     148   no curve numbers in surface record.
!     497   Record with Vi = PIPEj or Si = PIPESURFACEj contains incorrect
!           number of surface numbers
!     971   Mesh generator only available for fixed number of constituting
!           curves
!    1152   Surface number to be copied is larger than actual surface number
!    2087   Shape number not available
!    2088   Only ndim = 3 allowed for this type of surface
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrd2rest' )
      debug = .false. .and. ioutp>=0
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrd2rest'
         write(irefwr,1) 'isurf, ncurvs', isurf, ncurvs
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( work(1500) )

!     --- Detect rest of record

      call mshr01 ( icurvs, ncurvs, isurf, isurnr, maxsrf, &
                    nuspnt, lenipoint )

      ncur    = nint(work( 1))
      n       = nint(work( 2))
      m       = nint(work( 3))
      nelem   = nint(work( 4))
      npoint  = nint(work( 5))
      iblend  = nint(work( 6))
      icurve  = nint(work( 7))
      umin    = work( 8)
      umax    = work( 9)
      vmin    = work(10)
      vmax    = work(11)
      ismoot  = nint(work(12))
      idiag   = nint(work(13))+10*nint(work(19))
      mapping = nint(work(14))
      icentre = nint(work(15))
      itypesurf = nint(work(16))
      nsur    = nint(work(17))
      iregular = nint(work(18))
      nlayers = nint(work(24))

!     --- In case of triangle with internal points, make isurf 99

      if ( isurf==3 .and. (work(23)>0d0 .or. work(22)>0d0) ) then

!     --- internal points or curves found, set isurf to 99

         isurf = 99
         iinput(iinp) = isurf

      end if  ! ( isurf==3 .and. work(23)>0d0 )

!     --- Set parameter nicv

      if ( isurf==19 ) then

!     --- PAVER

         nicv = 5
         part4 = .true.

      else

         nicv = 1

      end if

!     --- Distinguish between the various surfaces

      if ( ncur==0 .and. isurf/=21 ) then

!     --- Error 148: No curves found in surface record

         call errint ( isurnr, 1 )
         call errsub ( 148, 1, 0, 0 )
         call errcrd
         go to 1000

      end if

!     --- Check and fill curves

      iinput(iinp+2) = ncur
      do i = 1, ncur
         icurnr = nint(work(24+i*nicv))
         iinput(iinp+2+i) = icurnr

!        --- Fill rpin in case of paver

         if ( nicv==5 ) call mshrd2paver ( i, work, rpin )

         if ( isurf/=23 .and. (icurnr==0 .or. abs(icurnr)>ncurvs) ) then

!        --- curve number out of range

            call errint ( icurnr, 1 )
            call errint ( ncurvs, 2 )
            call errint ( isurnr, 3 )
            call errsub ( 129, 3, 0, 0 )

         end if  ! ( isurf/=23 .and. (icurnr==0 .or. abs(icurnr)>ncurvs) )

      end do
      jinp = iinp
      iinp = iinp + 3 + ncur

      select case ( isurf )

      case ( 1 )

!     --- rectangle

         call mshrd2rect ( iinp, iinput, n, m, idiag, isurnr, ismoot, &
                           jinp, ncur )

      case ( 4, 13 )

!     --- quadrilateral or Coons

         call mshrd2coons ( iinp, iinput, idiag, isurnr, iblend, &
                            icurve, jinp, ncur, isurf )

      case ( 5 )

!     --- meshus

         iinput(iinp) = nelem
         iinput(iinp+1) = npoint
         iinp = iinp+2

      case ( 8, 9, 14, 16, 21 )

!     --- translate, rotate, reflect or similar

         iinput(iinp) = icopy
         iinp = iinp+1
         if ( icopy<=0 ) then
            call errint ( icopy, 1 )
            call errint ( isurnr, 2 )
            call errsub ( 1152, 2, 0, 0 )
            call errcrd
         end if
         if ( debug ) &
            write(irefwr,1) 'isurf, iinp, icopy', isurf, iinp, icopy

      case ( 10 )

!     --- pipe surface

         if ( ncur<3 .or. ncur>4 ) then
            call errint ( ncur, 1 )
            call errint ( isurnr, 2 )
            call errsub ( 497, 2, 0, 0 )
            call ercard ( kcolum, icolum )
         end if
         iinput(jinp+2) = iinput(jinp+2)+10000*idiag

      case ( 15 )

!     --- PARSURF

         if ( ncur/=4 ) then
            call errint ( 4, 1 )
            call errint ( ncur, 2 )
            call errint ( isurnr, 3 )
            call errchr ( 'PARSURF', 1 )
            call errsub ( 971, 3, 0, 1 )
            call ercard ( kcolum, icolum )
         end if
         iinput(iinp  ) = nelem
         iinput(iinp+1) = npoint
         iinput(iinp+2) = iblend
         iinput(iinp+3) = icurve
         iinp           = iinp + 4

      case ( 20 )

!     --- SPHERE

         call mshrd2sphere ( iinp, iinput, isurnr, nsur, nsurfs, &
                             ncur, icentre, ipoint, itypesurf )

      case ( 22, 99 )

!     --- isurf = 22, 99, framsesurf or triangle with internal points
!         store in iinput

         call mshrd2frame ( iinp, iinput )

      case ( 23 )

!     --- square

         call mshrd2square ( iinp, iinput )
         changeinp = .true.

      end select  ! case ( isurf )

      if ( isurf==13 .or. isurf==18 ) then

!     --- Coons or isopar store mapping

         if ( isurf==18 .and. ncur/=3 ) then
            call errint ( 3, 1 )
            call errint ( ncur, 2 )
            call errint ( isurnr, 3 )
            call errchr ( 'ISOPAR', 1 )
            call errsub ( 971, 3, 0, 1 )
            call ercard ( kcolum, icolum )
         end if
         iinput(iinp) = mapping
         iinput(iinp+1) = icentre
         iinp = iinp+2
         if ( isurf==18 ) then
            iinput(iinp) = iregular
            iinp = iinp+1
         end if

      end if
      if ( .not. trian .and. &
           (isurf==3 .or. isurf==20 .or. isurf==22 .or. &
            isurf==99) ) then

!     --- Quadrilateral elements not allowed for this type of surface generator

         call errint ( ishape, 1 )
         call errint ( isurnr, 2 )
         call errchr ( clcode(isurf), 1 )
         call errsub ( 2087, 2, 0, 1 )

      end if
      if ( ndim/=3 .and. (isurf==10 .or. isurf==20) ) then

!     --- Only R^3 allowed for this type of surface generator

         call errint ( ndim, 1 )
         call errint ( isurnr, 2 )
         call errchr ( clcode(isurf), 1 )
         call errsub ( 2088, 2, 0, 1 )

      end if

      if ( isurf==15 ) then

!     --- parsurf

         rinput( 1, isurnr ) = umin
         rinput( 2, isurnr ) = umax
         rinput( 3, isurnr ) = vmin
         rinput( 4, isurnr ) = vmax

      else if ( isurf==19 ) then

!     --- paver

         rinput( 1, isurnr ) = work(20)
         iinput(iinp) = nlayers
         iinp = iinp+1

      end if

      deallocate ( work )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrd2rest'

      end if  ! ( debug )
      call erclos ( 'mshrd2rest' )

      end subroutine mshrd2rest
