      subroutine mshrd3 ( iinput, nlines, inpmax, nelgrp, nintprop, &
                          nrealprop, intprop, realprop, iinputglob )
! ======================================================================
!
!        programmer    Guus Segal
!        version 14.0  date 21-11-2014 New parameter list
!        version 13.2  date 27-11-2013 Default value ishape
!        version 13.1  date 24-04-1998 Correction properties
!
!   copyright (c) 1984-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     read cards with line elements until a card with mesh or with
!     renumber, plot or end is found
!     fill array iinput with the information of the line elements
!     iinput must be filled from position 1
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nlines, inpmax, iinput(inpmax), nelgrp, iinputglob(*), &
              nintprop, nrealprop, intprop(10,*)
      double precision realprop(10,*)

!     iinput        i/o   integer input array of length iinpmax (local one)
!                         See subroutine mshrea
!     iinputglob     i    Global array iinput
!                         See subroutine mshrea
!     inpmax         i    Available space for array iinput
!     intprop       i/o   Integer array with properties
!     nelgrp         i    Number of element groups
!     nintprop      i/o   Number of integer properties per element group
!     nlines         i    number of parts where line elements are created
!     nrealprop     i/o   Number of real properties per element group
!     realprop      i/o   Real array with properties
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncodelc=8
      character (len=13) codelc(nncodelc)
      integer ncodelc(nncodelc), ncodmxlc(nncodelc), iprop, &
              ielgrp, ifirst, iinp, ilast, ishape, i, m, &
              ncurvs, nuspnt
      integer, allocatable :: iwork(:)
      double precision :: rprop
      logical ::  newelgrp

!     codelc         Table with keywords
!     i              General loop parameter
!     ielgrp         element group number
!     ifirst         First curve (surface) number read
!     iinp           last position used in array iinput
!     ilast          Last curve (surface) number read
!     iprop          Integer property
!     ishape         Shape number of elements
!     iwork          integer work array of length ncurvs
!                    iwork(i), i=1(1)nsurfs  is used to indicate if a
!                    curve has already been used for an element group (>0)
!                    or not (0)
!     m              number of items read for quantity
!     ncodelc        Table containing the number of significant characters in
!                    the keyword table codelc
!     ncodmxlc       Table containing the maximal number of  characters in
!                    the keyword table codelc
!     ncurvs         Number of curves in mesh
!     newelgrp       If true a new element group is found
!     nncodelc       Number of entries in codelc
!     nuspnt         Number of user points
!     rprop          Real property
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINRL         Print 1d real vector
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!    The subroutine reads input records of the shape:
!    LELM i = (shape = n, C1, C2 ), int_properity j = k, real_property l = r
!    LELM i = (shape = 0, P1, P2 ), int_properity j = k, real_property l = r
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      68   element group number not equal to preceding element group number or
!           preceding element group number plus one
!     142   Internal error, too few space for integers
!     151   first curve number has incorrect value
!     152   last curve number has incorrect value
!     424   first user point number (ifirst) has incorrect value
!     425   last point number (ilast) has incorrect value
!    1078   Integer property has different values for the same group
!    1082   Real property has different values for the same group
!    1544   Curves have been used more than once to create element groups
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data codelc / 'LELM', 'SHAPE', 'INT_PROPERTY', 'INT_PROPERTY', &
                    'REAL_PROPERTY', 'REAL_PROPERTY', 'C', 'P' /
      data ncodelc / 3,  3,  3,  3,  3,     3,  1,  1 /
      data ncodmxlc/ 4,  5, 12, 12, 13,    13,  1,  1 /

!     codelc     Table with keywords
!              The following types of texts are recognised:
!              1:   LELM
!              2:   SHAPE
!              3:   INT_PROPERTY
!              4:   INT_PROPERTY
!              5:   REAL_PROPERTY
!              6:   REAL_PROPERTY
!              7:   C
!              8:   P
! ======================================================================
!
      call eropen ( 'mshrd3' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrd3'
         write(irefwr,1) 'ncurvs', ncurvs
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call sepcopycode ( codelc, nncodelc, ncodelc, ncodmxlc )
      nmain = nncodemain

      itype(1:nncodelc) = 0
      itype(3) = 34
      itype(5) = 34
      itype(6) = 1
      itype(7:8) = 14
      tabmin(1:nncodelc) = -1d9
      tabmin(1) = 0d0
      tabmin(2) = -2d0
      tabmin(3) = 1d0
      tabmin(5) = 1d0
      tabmin(7:8) = 2d0
      tabmax(1:nncodelc) = 1d9
      tabmax(1) = 1d3
      tabmax(2:3) = 1d1
      tabmax(5) = 1d1
      tabmax(7:8) = 0d0

!     --- Get parameters

      nuspnt = iinputglob(2)
      ncurvs = iinputglob(3)

      allocate ( iwork(ncurvs), stat = error )
      if ( error/=0 ) call eralloc ( error, ncurvs, 'iwork' )
      if ( ierror/=0 ) go to 1000

      nlines = 0
      iinp = 0
      nelgrp = 0
      iwork = 0

!     --- read next record

      ityprd = 1
200   table(1:nncode) = 0d0

      tabmax(7) = ncurvs
      tabmax(8) = nuspnt
      call sepreadsetkeyw
      icode = iretur

      if ( debug ) then
         write(irefwr,1) 'icode', icode
         call prinrl ( table, nncodelc, 'table' )
      end if  ! ( debug )

      if ( icode>0 ) go to 1000

!     --- detect the line elements

      nlines = nlines+1
      ielgrp = nint(table(1))
      if ( ielgrp>nelgrp ) then
         nelgrp = nelgrp+1
         newelgrp = .true.
      else
         newelgrp = .false.
      end if
      if ( ielgrp<nelgrp ) then
         call errint ( ielgrp, 1 )
         call errint ( nelgrp-1, 2 )
         call errsub ( 68, 2, 0, 0 )
         call errcrd
      end if
      ishape = nint(table(2))

      if ( ishape==0 .and. table(7)>0d0 ) ishape = 1
      if ( ishape==0 ) then

!     --- user point found

         iposut = nint(table(8))
         m = nint(tabmax(8))
         ifirst = nint(tabout(iposut))
         if ( m==1 ) then
            ilast = ifirst
         else
            ilast = nint(tabout(iposut+1))
         end if
         if ( ifirst<=0 )  then

            call errint ( ifirst, 1 )
            call errint ( nuspnt, 2 )
            call errsub ( 424, 2, 0, 0 )
            call errcrd

         end if
         if ( ilast<ifirst )  then

            call errint ( ilast, 1 )
            call errint ( ifirst, 2 )
            call errint ( nuspnt, 3 )
            call errsub ( 425, 3, 0, 0 )
            call errcrd

         end if

      else

!     --- curve found

         iposut = nint(table(7))
         m = nint(tabmax(7))
         ifirst = nint(tabout(iposut))
         if ( m==1 ) then
            ilast = ifirst
         else
            ilast = nint(tabout(iposut+1))
         end if
         if ( ifirst<=0 )  then

            call errint ( ifirst, 1 )
            call errint ( ncurvs, 2 )
            call errsub ( 151, 2, 0, 0 )
            call errcrd

         end if
         if ( ilast<ifirst )  then

            call errint ( ilast, 1 )
            call errint ( ifirst, 2 )
            call errint ( ncurvs, 3 )
            call errsub ( 152, 3, 0, 0 )
            call errcrd

         end if

      end if
      if ( newelgrp ) then

!     --- new element group, set integer properties

         iposut = nint(table(3))
         if ( iposut>0 ) then

!        --- integer property found

            do i = 1, 10
               if ( tabout(iposut+i-1)>-1d30 ) then

!              --- integer property found

                  nintprop = max(i,nintprop)
                  intprop(i,ielgrp) = nint(tabout(iposut+i-1))

               else
                  intprop(i,ielgrp) = -1000000000
               end if

            end do

         end if

!        --- set real properties

         iposut = nint(table(5))
         if ( iposut>0 ) then

!        --- real property found

            do i = 1, 10
               if ( tabout(iposut+i-1)>-1d30 ) then

!              --- real property found

                  nrealprop = max(i,nrealprop)
                  realprop(i,ielgrp) = tabout(iposut+i-1)

               else
                  realprop(i,ielgrp) = -1d30
               end if

            end do

         end if

      else

!     --- old element group, check integer properties

         iposut = nint(table(3))
         if ( iposut>0 ) then

!        --- integer property found

            do i = 1, 10
               if ( tabout(iposut+i-1)>-1d30 ) then

!              --- integer property found

                  nintprop = max(i,nintprop)
                  iprop = nint(tabout(iposut+i-1))
                  if ( intprop(i,ielgrp)<=-1000000000 ) then
                     intprop(i,ielgrp) = iprop
                  else if ( intprop(i,ielgrp)/=iprop ) then

!                 --- Property is not the same as given before

                     call errint ( i, 1 )
                     call errint ( ielgrp, 2 )
                     call errint ( iprop, 3 )
                     call errint ( intprop(i,ielgrp), 4 )
                     call errsub ( 1078, 4, 0, 0 )

                  end if

               end if

            end do

         end if

         iposut = nint(table(5))
         if ( iposut>0 ) then

!        --- real property found

            do i = 1, 10
               if ( tabout(iposut+i-1)>-1d30 ) then

!              --- real property found

                  nrealprop = max(i,nrealprop)
                  rprop = tabout(iposut+i-1)
                  if ( realprop(i,ielgrp)<=-1d30 ) then
                     realprop(i,ielgrp) = rprop
                  else if ( abs(realprop(i,ielgrp)-rprop)> &
                            sqreps*rprop ) then

!                 --- Property is not the same as given before

                     call errint ( i, 1 )
                     call errint ( ielgrp, 2 )
                     call erreal ( rprop, 1 )
                     call erreal ( realprop(i,ielgrp), 2 )
                     call errsub ( 1082, 2, 2, 0 )

                  end if

               end if

            end do

         end if

      end if

      if ( iinp+4>inpmax ) then

!     --- iinp>inpmax

         call errint ( inpmax, 1 )
         call errint ( iinp, 2 )
         call errsub ( 142, 2, 0, 0 )
         call errcrd
         go to 1000

      end if

      if ( ishape/=0 ) then

         do i = ifirst, ilast

            if ( iwork(i)==0 ) then

!           --- curve has not yet been used

               iwork(i) = ielgrp

            else

!           --- error curve has been used before

               call errint ( i, 1 )
               call errint ( iwork(i), 2 )
               call errint ( ielgrp, 3 )
               call errsub ( 1544, 3, 0, 0 )

            end if

         end do

      end if

      iinput(iinp+1) = ishape
      iinput(iinp+2) = ielgrp
      iinput(iinp+3) = ifirst
      iinput(iinp+4) = ilast
      iinp = iinp+4
      goto 200

1000  icode = icode - 1

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrd3'

      end if  ! ( debug )
      call erclos ( 'mshrd3' )

      end
