      subroutine mshrd4 ( iinput, nsurel, iinputglob, inpmax, nelgrp, &
                          inpsur, nintprop, nrealprop, intprop, realprop, &
                          nvolms )
! ======================================================================
!
!        programmer    Guus Segal
!        version 18.0  date 21-11-2014 New parameter list
!        version 17.1  date 02-09-2013 Correction maximum number of warnings
!        version 17.0  date 13-09-2000 Extra parameter nvolms
!
!   copyright (c) 1982-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   read cards with surface elements until a card with mesh or with
!   renumber, plot or end is found
!   fill array iinput with the information of the surface elements
!   iinput must be filled from position 1
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpmax, iinput(inpmax), nsurel, &
              nelgrp, inpsur(*), nintprop, &
              nrealprop, intprop(10,*), nvolms, iinputglob(*)
      double precision realprop(10,*)

!     iinput        i/o   integer input array of length iinpmax
!     iinputglob     i    Global array iinput
!                         See subroutine mshrea
!     inpmax         i    maximal space for iinput
!     inpsur         i    array iinput containing information of the surfaces
!     intprop       i/o   Integer array with properties
!     nelgrp        i/o   number of element groups
!     nintprop      i/o   Number of integer properties per element group
!     nrealprop     i/o   Number of real properties per element group
!     nsurel         o    number of parts where surface elements are defined
!     nvolms         i    number of volumes
!     realprop      i/o   Real array with properties
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nsurfs
      integer, parameter :: nncodelc=6
      character *13 codelc(nncodelc)
      integer ncodelc(nncodelc), ncodmxlc(nncodelc), iprop, &
              iinp, i, isurf, ielgrp, ifirst, ilast, m, nwarnlc
      integer, allocatable :: iwork(:)
      double precision :: rprop
      logical newelgrp

!     codelc         Table with keywords
!     i              Counting variable
!     ielgrp         element group number
!     ifirst         First curve (surface) number read
!     iinp           last position used in array iinput
!     ilast          Last curve (surface) number read
!     iprop          Integer property
!     isurf          Type of surface with respect to mesh generator
!     iwork          integer work array of length 2*nsurfs
!                    iwork(i), i=1(1)nsurfs contains the shape numbers of
!                    the surfaces
!                    iwork(i+nsurfs), i=1(1)nsurfs is used to indicate if a
!                    surface has already been used for an element group
!                    (>0) or not (0)
!     m              number of items read
!     ncodelc        number of significant characters in codelc
!     ncodmxlc       maximum number of characters in codelc
!     newelgrp       If true a new element group is found
!     nncodelc       Nuber if items in codelc
!     nsurfs         number of surfaces
!     nwarnlc        Number of local warnings
!     rprop          Real property
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     INTX02         Interpret a record containing keywords followed by numbers
!     MSH049         Compute starting address of specific surface in array
!                    IINPUT
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!    The subroutine reads input records of the shape:
!    SELM i = (S1, S2 ), int_properity j = k, real_property l = r
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      68   element group number not equal to preceding element group number or
!           preceding element group number plus one
!     142   Internal error, too few space for integers
!     155   first surface number has incorrect value
!     156   last surface number has incorrect value
!    1078   Integer property has different values for the same group
!    1082   Real property has different values for the same group
!    1543   Surfaces have been used more than once to create element groups
!    1837   Present element group number is equal to preceding one, whereas a
!           new type of elements is introduced
!    2198   Surface not connected to element group
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data codelc / 'SELM', 'INT_PROPERTY', 'INT_PROPERTY', &
                  'REAL_PROPERTY', 'REAL_PROPERTY', 'S' /
      data ncodelc / 3,  3,  3,  3,  3,     1 /
      data ncodmxlc/ 4, 12, 12, 13, 13,     1 /

!     codelc     Table with keywords
!              The following types of texts are recognised:
!              1:   SELM
!              2:   INT_PROPERTY
!              3:   INT_PROPERTY
!              4:   REAL_PROPERTY
!              5:   REAL_PROPERTY
!              6:   S
! ======================================================================
!
!     --- fill array iwork with shape numbers of elements in surfaces

      call eropen ( 'mshrd4' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrd4'
         write(irefwr,1) 'icode', icode
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call sepcopycode ( codelc, nncodelc, ncodelc, ncodmxlc )
      nmain = nncodemain

      itype(1:nncodelc) = 0
      itype(2) = 34
      itype(4) = 34
      itype(5) = 1
      itype(6) = 14
      tabmin(1:nncodelc) = -1d9
      tabmin(1) = 0d0
      tabmin(2) = 1d0
      tabmin(4) = 1d0
      tabmin(6) = 2d0
      tabmax(1:nncodelc) = 1d9
      tabmax(1) = 1d3
      tabmax(2) = 1d1
      tabmax(4) = 1d1

!     --- Get parameters

      nsurfs = iinputglob(4)

      allocate ( iwork(2*nsurfs), stat = error )
      if ( error/=0 ) call eralloc ( error, 2*nsurfs, 'iwork' )
      if ( ierror/=0 ) go to 1000

      iinp = 1
      do i = 1, nsurfs
         isurf = inpsur(iinp)
         iwork(i) = inpsur(iinp+1)
         iwork(i+nsurfs) = 0
         call msh049 ( inpsur, iinp, isurf )
      end do
      nsurel = 0
      iinp = 0

!     --- read next record

      ityprd = 1
200   table(1:nncode) = -1d20
      table(1) = 0d0
      table(2) = 0d0
      table(4) = 0d0
      table(6) = 0d0
      tabmax(6) = nsurfs
      call sepreadsetkeyw
      icode = iretur
      if ( debug ) write(irefwr,1) 'icode', icode
      if ( icode>0 ) go to 900

!     --- detect the surface elements

      nsurel = nsurel+1
      ielgrp = nint(table(1))

      if ( debug ) &
         write(irefwr,1) 'iinp, ielgrp, nelgrp', iinp, ielgrp, nelgrp

      if ( iinp==0 ) then

!     --- No records with respect to surface elements read, so nelgrp must
!         be raised

         if ( ielgrp==nelgrp ) then

!        --- Error 1837: ielgrp = nelgrp

            call errint ( ielgrp, 1 )
            call errsub ( 1837, 1, 0, 0 )
            call errcrd

         end if
         nelgrp = nelgrp + 1
         newelgrp = .true.

      else if ( ielgrp>nelgrp ) then

!     --- Records have been read and ielgrp>nelgrp, raise nelgrp

         nelgrp = nelgrp+1
         newelgrp = .true.

      else

         newelgrp = .false.

      end if

      if ( ielgrp<nelgrp ) then
         call errint ( ielgrp, 1 )
         call errint ( nelgrp-1, 2 )
         call errsub ( 68, 2, 0, 0 )
         call errcrd
      end if

      iposut = nint(table(6))
      m = nint(tabmax(6))
      ifirst = nint(tabout(iposut))
      if ( m==1 ) then
         ilast = ifirst
      else
         ilast = nint(tabout(iposut+1))
      end if
      if ( ifirst<=0 ) then
         call errint ( ifirst, 1 )
         call errint ( nsurfs, 2 )
         call errsub ( 155, 2, 0, 0 )
         call errcrd
      end if
      if ( ilast<ifirst ) then
         call errint ( ilast, 1 )
         call errint ( ifirst, 2 )
         call errint ( nsurfs, 3 )
         call errsub ( 156, 3, 0, 0 )
         call errcrd
      end if
      if ( newelgrp ) then

!     --- new element group, set integer properties

         iposut = nint(table(2))
         if ( iposut>0 ) then

!        --- integer property found

            do i = 1, 10
               if ( tabout(iposut+i-1)>-1d30 ) then

!              --- integer property found

                  nintprop = max(i,nintprop)
                  intprop(i,ielgrp) = nint(tabout(iposut+i-1))

               else
                  intprop(i,ielgrp) = -1000000000
               end if

            end do

         end if

!        --- set real properties

         iposut = nint(table(4))
         if ( iposut>0 ) then

!        --- real property found

            do i = 1, 10
               if ( tabout(iposut+i-1)>-1d30 ) then

!              --- real property found

                  nrealprop = max(i,nrealprop)
                  realprop(i,ielgrp) = tabout(iposut+i-1)

               else
                  realprop(i,ielgrp) = -1d30
               end if

            end do

         end if

      else

!     --- old element group, check integer properties

         iposut = nint(table(2))
         if ( iposut>0 ) then

!        --- integer property found

            do i = 1, 10
               if ( tabout(iposut+i-1)>-1d30 ) then

!              --- integer property found

                  nintprop = max(i,nintprop)
                  iprop = nint(tabout(iposut+i-1))
                  if ( intprop(i,ielgrp)<=-1000000000 ) then
                     intprop(i,ielgrp) = iprop
                  else if ( intprop(i,ielgrp)/=iprop ) then

!                 --- Property is not the same as given before

                     call errint ( i, 1 )
                     call errint ( ielgrp, 2 )
                     call errint ( iprop, 3 )
                     call errint ( intprop(i,ielgrp), 4 )
                     call errsub ( 1078, 4, 0, 0 )

                  end if

               end if

            end do

         end if

         iposut = nint(table(4))
         if ( iposut>0 ) then

!        --- real property found

            do i = 1, 10
               if ( tabout(iposut+i-1)>-1d30 ) then

!              --- real property found

                  nrealprop = max(i,nrealprop)
                  rprop = tabout(iposut+i-1)
                  if ( realprop(i,ielgrp)<=-1d30 ) then
                     realprop(i,ielgrp) = rprop
                  else if ( abs(realprop(i,ielgrp)-rprop)> &
                            sqreps*rprop ) then

!                 --- Property is not the same as given before

                     call errint ( i, 1 )
                     call errint ( ielgrp, 2 )
                     call erreal ( rprop, 1 )
                     call erreal ( realprop(i,ielgrp), 2 )
                     call errsub ( 1082, 2, 2, 0 )

                  end if

               end if

            end do

         end if

      end if

      if ( iinp+4>inpmax ) then

!     --- iinp>inpmax

         call errint ( inpmax, 1 )
         call errint ( iinp, 2 )
         call errsub ( 142, 2, 0, 0 )
         call errcrd
         go to 1000

      end if

      do i = ifirst, ilast

         if ( iwork(i+nsurfs)==0 ) then

!        --- surface has not yet been used

            iwork(i+nsurfs) = ielgrp

         else

!        --- error surface has been used before

            call errint ( i, 1 )
            call errint ( iwork(i+nsurfs), 2 )
            call errint ( ielgrp, 3 )
            call errsub ( 1543, 3, 0, 0 )

         end if

      end do

      iinput(iinp+1) = iwork(ifirst)
      iinput(iinp+2) = ielgrp
      iinput(iinp+3) = ifirst
      iinput(iinp+4) = ilast
      iinp = iinp+4
      goto 200

900   icode = icode - 1

      if ( nvolms==0 ) then

!     --- nvolms = 0, check if all surfaces have been connected to element
!         groups

         nwarnlc = 0
         do i = 1, nsurfs

            if ( iwork(i+nsurfs)==0 ) then

!           --- surface i is not connected to an element group

               if ( nwarnlc<999 ) then
                  nwarnlc = nwarnlc+1
                  call errint ( i, nwarnlc )
               end if  ! ( nwarnlc<999 )

            end if

         end do
         if ( nwarnlc>0 ) call errwar ( 2198, nwarnlc, 0, 0 )

      end if

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrd4'

      end if  ! ( debug )
      call erclos ( 'mshrd4' )

      end
