      subroutine mshrendisjoin ( kmeshtopol, coor3d, inpelm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 12-06-2015 Use markelem
!        version  1.0  date 28-08-2013
!
!   copyright (c) 2013-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Renumber nodes in submesh in case the submesh consists of disjoint parts
!     Also renumber elements
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     renumbering
!     disjoint
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpelm, kmeshtopol(inpelm,nelem)
      double precision coor3d(ndim,npoint)

!     coor3d          i/o   array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     inpelm         i    number of points per element
!     kmeshtopol        i/o   kmesh part c, contains the nodal points
!                         corresponding to the elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer isum, ielem
      integer, allocatable :: irenum(:), isplit(:), markelem(:)

!     ielem          Element number
!     irenum         Array in which the new node numbers are stored
!     isplit         Contains the number of nodes per disjoint set
!     isum           Integer variable to compute a sum
!     markelem       Work array to mark elements that must be used
!                    0: element is skipped
!                    1: element is used
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INIACCUMULARR  Accumulate contents of array iarray
!     INIMAPINTARR   Map integer array onto new numbering using numbering
!                    stored in irenum
!     MSHFINDDISJ    Check if mesh consists of disjoint parts
!     MSHRENKMESHC   Renumber elements in case elements belong to disjoint sets
!     PR0012         Print accumulated array
!     PRININ1        print 2d integer array
!     PRINRL1        Print 2d real vector
!     TOORDR2D       Map 2d array coor3d to new numbering, where only the second
!                    parameter should be renumbered
!     TOPFILLKMESHE1 compute length of kmesh part e2
!     TOPFILLKMESHE2 compute nodal points connected with nodal points
!     TOPFILLKMSHD1  Fill part of kmshd1 for one element group
!     TOPFILLKMSHD2  Fill array knel with kmshd part 2 for one element group
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Fill array kmshd for submesh (elements connected to nodes)
!     Fill array kmshe for submesh (nodes connected to nodes)
!     Find number of disjoint sets and find node numbers per set
!     If ( there are disjoint sets ) then
!        Renumber nodes, coor3d and kmeshtopol
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrendisjoin' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrendisjoin'
         call prinin1 ( kmeshtopol, nelem, inpelm, 'kmeshtopol' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill array kmshd for submesh (elements connected to nodes)
!         First fill kmshd1 (see subroutine toplm1)

      allocate ( kmshd1(0:npoint), kmshe1(npoint+1), markelem(nelem), &
                 stat = error )
      if ( error/=0 ) call eralloc ( error, 2*npoint+1, 'kmshd1' )
      if ( ierror/=0 ) go to 1000

      markelem = 1

      kmshd1 = 0
      ielem = 0
      call topfillkmshd1 ( nelem, inpelm, kmeshtopol, kmshd1, ielem, &
                           markelem )
      call iniaccumularr ( kmshd1, npoint )
      isum = kmshd1(npoint)

      allocate ( kmshd2(isum), stat = error )
      if ( error/=0 ) call eralloc ( error, isum, 'kmshd2' )
      if ( ierror/=0 ) go to 1000

!     --- Next fill kmshd2 (see subroutine toplm2)
!         Use kmshe1 as work array

      kmshe1(1:npoint) = 1
      ielem = 0
      call topfillkmshd2 ( nelem, inpelm, kmeshtopol, kmshd1, &
                           kmshd2, kmshe1, ielem, markelem )

      if ( debug ) then
         write ( irefwr, '(/a/)') &
               'elements corresponding to nodal points'
         call pr0012 ( npoint, kmshd1, kmshd2 )
      end if  ! ( debug )

!     --- Fill array kmshe for submesh (nodes connected to nodes)
!         First compute length of part e1 (See subroutine toplnp0)

      isum = npoint
      call topfillkmeshe1 ( kmeshtopol, kmshd1, kmshd2, kmshe1, &
                            isum, inpelm, nelem, npoint )

      allocate ( kmshe2(isum), stat = error )
      if ( error/=0 ) call eralloc ( error, isum, 'kmshe2' )
      if ( ierror/=0 ) go to 1000

!     --- Next fill kmesh part e2 (See subroutine topnp)

      call topfillkmeshe2 ( kmeshtopol, kmshd1, kmshd2, kmshe1, &
                            kmshe2, isum, inpelm, nelem, &
                            npoint )

      if ( debug ) then
         write ( irefwr, '(/a/)') &
               'nodal point connected to nodal points'
         call pr0012 ( npoint, kmshe1, kmshe2 )
      end if  ! ( debug )

      deallocate ( kmshd1, kmshd2, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'kmshd1' )
      if ( ierror/=0 ) go to 1000

!     --- Find number of disjoint sets and find node numbers per set

      allocate ( irenum(npoint), isplit(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, 2*npoint, 'irenum' )
      if ( ierror/=0 ) go to 1000

      call mshfinddisj ( irenum, isplit )

      if ( nsplit>1 ) then

!     --- nsplit>1, there are disjoint sets
!         Renumber nodes, coor3d and kmeshtopol

         if ( debug ) then
            call prinrl1 ( coor3d, npoint, 3, 'coor3d' )
            call prinin1 ( kmeshtopol, nelem, inpelm, 'kmeshtopol' )
         end if  ! ( debug )

         call inimapintarr ( kmeshtopol, irenum, inpelm*nelem )
         call toordr2d ( irenum, coor3d, npoint, 3 )

         if ( debug ) then
            call prinrl1 ( coor3d, npoint, 3, 'coor3d' )
            call prinin1 ( kmeshtopol, nelem, inpelm, 'kmeshtopol' )
         end if  ! ( debug )

!        --- Renumber elements

         call mshrenkmeshc ( kmeshtopol, inpelm, nelem, isplit, nsplit, &
                             npoint )
         if ( debug ) then
            call prinin1 ( kmeshtopol, nelem, inpelm, 'kmeshtopol' )
         end if  ! ( debug )

      end if  ! ( nsplit>1 )

      deallocate ( kmshe1, kmshe2, irenum, isplit, markelem, &
                   stat = error )
      if ( error/=0 ) call erdealloc ( error, 'kmshe1' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrendisjoin'

      end if  ! ( debug )
      call erclos ( 'mshrendisjoin' )

      end
