      subroutine mshrot ( bcorol, bcornw, nbndpt, ndim, acoor, npoint, &
                          isurf )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  4.3  date 29-05-2014 Extension of timebefore
!        version  4.2  date 12-08-1999 Replace askclk by printtime
!        version  4.1  date 24-09-1992 warning if submitted curves can not
!                                      be found as translation/rotation of
!                                      other curves)
!
!   copyright (c) 1987-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!        Compute rotated surface
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     rotation
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nbndpt, ndim, npoint, isurf
      double precision bcorol(ndim,*), bcornw(ndim,*), acoor(ndim,*)

!     acoor  i/o  at input acoor contains the coordinates of the surface
!                 to be copied
!                 at output acoor contains the coordinates of the new surface
!     bcornw  i   array containing the coordinates of the new boundary
!     bcorol  i   array containing the co-ordinates of boundary of surface
!                 to be copied
!     isurf   i   surface number
!     nbndpt  i   number of points in bcornw
!     ndim    i   dimension of space
!     npoint  i   number of points in coor
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, itwo, k
      double precision alpha, angle, beta, d1, d2, d3, e1, eps, &
               gamma, sum, sum1, sum2, t1(3), t2(3), t3(3), &
               t1a(3), t2a(3), t3a(3), xo(3), x1, x2, x3
      double precision :: timebefore(2)

!     alpha          first coordinate in local basis
!     angle          helpvariable for computing angle between two vectors
!     beta           second coordinate in local basis
!     d1             help variable for orthogonalization
!     d2             help variable for orthogonalization
!     d3             help variable for orthogonalization
!     e1             help variable to store the local magnitude
!     eps            accuracy parameter
!     gamma          third coordinate in local basis
!     i              loop variable
!     itwo           number of point for second basis vector
!     k              loop variable
!     sum            summation parameter
!     sum1           summation parameter
!     sum2           summation parameter
!     t1             first basis vector in old local system
!     t1a            first basis vector in new local system
!     t2             second basis vector in old local system
!     t2a            second basis vector in new local system
!     t3             third basis vector in old local system
!     t3a            third basis vector in new local system
!     x1             x-coordinate in global system
!     x2             y-coordinate in global system
!     x3             z-coordinate in global system
!     xo             local origin ( old and later new)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Places real value in error message array
!     ERRINT         Places integer value in error message array
!     ERRSUB         Produces error message
!     ERRWAR         Produces warning
!     ERSETTIME      Get the CPU time at start of subroutine
!     INSTOP         Stops program execution and closes all files
!     PRINTTIME      Print the CPU time at end of subroutine
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   914    ndim has wrong value
!   915    nbndpt is too small
!   916    all boundary points on a line
!   917    points are too close to each other
!   948    bcornw not correct translation/rotation of bcorol
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Do some checks on input
!     Compute orthogonal basis T1, T2 and T3 of old object
!     Compute based on the same points basis TA1, TA2 and TA3, for
!     rotated object
!     Compute local coordinates of original points in old system and
!     next the carthesian coordinates of the rotated points
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrot' )
      if ( itime>0 ) call ersettime ( timebefore )

      eps = 10d3*epsmac

      if ( ndim/=2 .and. ndim/=3 ) then
           call errint ( ndim, 1 )
           call errsub ( 914, 1, 0, 0 )
           call instop
      end if

      if ( nbndpt<3 ) then
           call errint ( isurf, 1 )
           call errsub ( 915, 1, 0, 0 )
           call instop
      end if

!     --- Check whether mutual distances in bcorol and bcornw are the same

      d3 = 0d0
      do 10 i = 2, nbndpt

          d1 = bcorol(1,i) - bcorol(1,1)
          d2 = bcorol(2,i) - bcorol(2,1)
          if ( ndim==3 ) d3 = bcorol(3,i) - bcorol(3,1)

          sum1 = sqrt( d1*d1 + d2*d2 + d3*d3 )

          d1 = bcornw(1,i) - bcornw(1,1)
          d2 = bcornw(2,i) - bcornw(2,1)
          if ( ndim==3 ) d3 = bcornw(3,i) - bcornw(3,1)

          sum2 = sqrt( d1*d1 + d2*d2 + d3*d3 )

          e1   = sqreps * ( abs(sum1) + abs(sum2) )

          if ( e1<epsmac ) e1 = sqreps

          if ( abs(sum1-sum2)>e1 ) then

!         --- Give warning

               call errint( isurf, 1 )
               call errint(  i   , 2 )
               call errwar(  948 , 2 , 0 , 0 )

          end if

10    continue

!     --- Determine basis T1, T2 and T3 of old object

      t1(3) = 0d0

      do 40 i = 1, ndim
          t1(i) = bcorol(i,2)-bcorol(i,1)
40    continue

!     --- Normalize

      sum = sqrt(t1(1)*t1(1)+t1(2)*t1(2)+t1(3)*t1(3))
      sum1 = sum

      do 50 i = 1, ndim
          t1(i) = t1(i)/sum
50    continue

!     --- Determine appropriate second vector

      do 100 i = 3, nbndpt

          itwo = i
          t2(3) = 0d0

          do 110 k = 1, ndim
              t2(k) = bcorol(k,i)-bcorol(k,i-1)
110       continue

!         --- Normalize

          sum = sqrt(t2(1)*t2(1)+t2(2)*t2(2)+t2(3)*t2(3))
          sum2 = sum

          do 120 k = 1, ndim
              t2(k) = t2(k)/sum
120       continue

          angle = abs(t1(1)*t2(1)+t1(2)*t2(2)+t1(3)*t2(3))

          if ( angle<0.9d0 ) goto 200

100   continue

!     --- Angle never large enough : Error

      call errint( isurf, 1 )
      call errsub ( 916, 1, 0, 0 )
      call instop

200   continue

!     --- Make first two basis vectors orthogonal

      d1=t1(1)*(t1(2)*t2(2)+t1(3)*t2(3))-t2(1)*(t1(2)*t1(2)+t1(3)*t1(3))
      d2=t1(2)*(t1(1)*t2(1)+t1(3)*t2(3))-t2(2)*(t1(1)*t1(1)+t1(3)*t1(3))
      d3=t1(3)*(t1(1)*t2(1)+t1(2)*t2(2))-t2(3)*(t1(1)*t1(1)+t1(2)*t1(2))

      sum = sqrt( d1*d1 + d2*d2 + d3*d3 )

      t2(1) = d1 / sum
      t2(2) = d2 / sum
      t2(3) = d3 / sum

!     --- Compute third basis vector

      t3(1) = t1(2)*t2(3) - t2(2)*t1(3)
      t3(2) = t1(3)*t2(1) - t2(3)*t1(1)
      t3(3) = t1(1)*t2(2) - t2(1)*t1(2)

!     --- Compute based on the same vectors basis for rotated object

      xo(3) = 0d0

      do 210 i = 1, ndim
          xo(i) = bcornw(i,1)
210   continue

!     --- Determine basevectors TA1, TA2 and TA3

      t1a(3) = 0d0
      do 220 i = 1, ndim
          t1a(i) = bcornw(i,2)-xo(i)
220   continue

!     --- Normalize

      sum = sqrt(t1a(1)*t1a(1)+t1a(2)*t1a(2)+t1a(3)*t1a(3))

      if ( abs(sum1-sum)>eps*( abs(sum)+abs(sum1) ) ) then

           call errint( isurf, 1 )
           call erreal( sum  , 1 )
           call erreal( sum1 , 2 )
           call erreal( eps  , 3 )
           call errsub ( 917, 1, 3, 0 )
           call instop

      end if

      do 230 i = 1, ndim
          t1a(i) = t1a(i)/sum
230   continue

!     --- Next vector is based on itwo

      t2a(3) = 0d0

      do 240 i = 1, ndim
          t2a(i) = bcornw(i,itwo)-bcornw(i,itwo-1)
240   continue

!     --- Normalize

      sum = sqrt(t2a(1)*t2a(1)+t2a(2)*t2a(2)+t2a(3)*t2a(3))
      if ( abs(sum2-sum)>eps*(abs(sum)+abs(sum2)) ) then
           call errint( isurf, 1 )
           call erreal( sum, 1 )
           call erreal( sum2, 2 )
           call erreal( eps, 3 )
           call errsub ( 917, 1, 3, 0 )
           call instop
      end if

      do 250 i = 1, ndim
          t2a(i) = t2a(i)/sum
250   continue

!     --- Make basis orthogonal

      d1 = t1a(1)*(t1a(2)*t2a(2)+t1a(3)*t2a(3)) &
          -t2a(1)*(t1a(2)*t1a(2)+t1a(3)*t1a(3))

      d2 = t1a(2)*(t1a(1)*t2a(1)+t1a(3)*t2a(3)) &
          -t2a(2)*(t1a(1)*t1a(1)+t1a(3)*t1a(3))

      d3 = t1a(3)*(t1a(1)*t2a(1)+t1a(2)*t2a(2)) &
          -t2a(3)*(t1a(1)*t1a(1)+t1a(2)*t1a(2))

      sum = sqrt( d1*d1 + d2*d2 + d3*d3 )

      t2a(1) = d1 / sum
      t2a(2) = d2 / sum
      t2a(3) = d3 / sum

!     --- Compute third basis vector

      t3a(1) = t1a(2)*t2a(3) - t2a(2)*t1a(3)
      t3a(2) = t1a(3)*t2a(1) - t2a(3)*t1a(1)
      t3a(3) = t1a(1)*t2a(2) - t2a(1)*t1a(2)

!     --- Compute coordinates of rotated points

      do 300 i = 1 , npoint

          x1 = acoor(1,i)-bcorol(1,1)
          x2 = acoor(2,i)-bcorol(2,1)

          if ( ndim==3 ) then
               x3 = acoor(3,i)-bcorol(3,1)
          else
               x3 = 0d0
          end if

!         --- Compute coordinates in old system

          alpha = x1*t1(1) + x2*t1(2) + x3*t1(3)
          beta  = x1*t2(1) + x2*t2(2) + x3*t2(3)
          gamma = x1*t3(1) + x2*t3(2) + x3*t3(3)

!         --- Compute new coordinates

          do 310 k = 1, ndim

              acoor( k,i ) =  xo(k) + alpha * t1a(k) + &
                              beta * t2a(k) + gamma * t3a(k)

310       continue

300   continue

      if ( itime==1 ) call printtime('ROTATE')

      call erclos ( 'mshrot' )

      end
