      subroutine mshseqsurfs ( iinput, nsurfs, iseqsurfs, ipointsurf )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 08-09-2010 New parameter list
!        version  1.2  date 20-10-2008 Allow isurf = 24
!        version  1.1  date 07-11-2002 Correction for ordered surfaces
!
!   copyright (c) 1999-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Make a reordening of surfaces such that surfaces of surfaces or surfaces
!   to be copied are created once the subsurfaces are created
! **********************************************************************
!
!                       KEYWORDS
!
!     surface
!     mesh
!     reordering
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), nsurfs, iseqsurfs(*), ipointsurf(*)

!     iinput         i    integer input array
!     ipointsurf     o    Array containing the relative starting addresses
!                         of all surfaces in the part of iinput referring
!                         to surfaces
!     iseqsurfs      o    Contains new sequence of surface numbers
!     nsurfs         i    Number of surfaces in mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iinp, iseq, isurnr, isurf, ih, ndefsf, minloop, istart, &
              i, jsurnr, iprec, j, icount, jstart, nrow, iwork(nsurfs), &
              iwork1(2,nsurfs), isubsurfs(40*nsurfs)
      logical check

!     check          if true a surface has got a new sequence number
!     i              Counting variable
!     icount         Counter
!     ih             Help variable to store a constant temporarily
!     iinp           last position used in array iinput
!     iprec          Surface number of surface to be copied etc.
!     iseq           Sequence number
!     istart         First position used at a part
!     isubsurfs      integer work array of length 40*nsurfs in which
!                    the subsurfaces of the surfaces are stored
!     isurf          Type of surface with respect to mesh generator
!     isurnr         Surface sequence number
!     iwork          integer work array of length nsurfs
!                    In this array the new sequence numbers of the
!                    surfaces are stored
!     iwork1         integer work array of length 2*nsurfs
!                    iwork1(1,i) contains the starting address of surface
!                    i in isubsurfs
!                    iwork1(2,i) contains the number of subsurfaces of
!                    surface i
!     j              Counting variable
!     jstart         starting position of curve i in kmeshm
!     jsurnr         Surface sequence number
!     minloop        Minimum bound for loop
!     ndefsf         Number of surfaces in surface
!     nrow           Number of rows in an ordered surface of surfaces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSH049         Compute starting address of specific surface in array
!                    IINPUT
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
!    2094   Surface numbers can not be ordered because of unsolvable recursion
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear work arrays
!     iseq := 0
!     for isurnr := 1 (1) nsurfs do
!        isurf := type of surface
!        if ( isurf is composite surface ) then
!           Check if all subsurfaces have lower sequence number
!           If ( there are subsurface with seq. number higher than isurnr ) then
!              Store these sequence numbers into isubsurfs and adapt iwork1
!           else ( all subsurface sequence numbers<isurnr )
!              iseq := iseq+1
!              iwork(isurnr) := iseq
!        else if ( isurf is copy of surface ) then
!           If ( original surfaces has lower sequence number ) then
!              iseq := iseq+1
!              iwork(isurnr) := iseq
!           else ( original surfaces has higher sequence number )
!              Store sequence number into isubsurfs and adapt iwork1
!        else ( standard surface )
!           iseq := iseq+1
!           iwork(isurnr) := iseq
!           If ( there are any of the previous numbers that can get a sequence
!                number ) then
!              Raise iseq
!              Store these surfaces into iwork
!              Adapt iwork1 and isubsurfs
!     Check if all surfaces have got a number
!     Store inverse of iwork into iseqsurfs
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshseqsurfs' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshseqsurfs'

      end if

!     --- Clear work arrays

      do isurnr = 1, nsurfs
         iwork(isurnr) = 0
         iwork1(1,isurnr) = 0
         iwork1(2,isurnr) = 0
      end do
      iinp = 1
      iseq = 0
      minloop = nsurfs+1
      istart = 0

!     --- Loop over surfaces

      do isurnr = 1, nsurfs
         isurf = iinput(iinp)
         if ( isurf==24 ) isurf = 7    ! compound surface
         check = .false.
         ipointsurf(isurnr) = iinp
         if ( debug ) write(irefwr,*) 'isurnr, isurf ', isurnr, isurf
         if ( isurf==7 .or. isurf==11 .or. isurf==12 ) then

!        --- Surface of surfaces, check if there are surfaces with higher
!            number

            if ( isurf==7 ) then

!           --- isurf = 7, surface of surfaces

               ndefsf = iinput(iinp+2)
               jstart = iinp+2

            else

!           --- isurf = 11, 12, ordered surface of surfaces

               nrow = iinput(iinp+2)
               ndefsf = 0
               do i = 1, nrow
                  ndefsf = ndefsf+iinput(iinp+2+i)
               end do
               jstart = iinp+2+nrow

            end if
            icount = 0
            do i = 1, ndefsf
               jsurnr = abs(iinput(jstart+i))
               if ( debug ) &
                  write(irefwr,*) ' i, jsurnr, iwork(jsurnr) ', &
                                    i, jsurnr, iwork(jsurnr)
               if ( jsurnr>isurnr .or. iwork(jsurnr)<=0 ) then
                  minloop = min(minloop,isurnr)
                  icount = icount+1
                  isubsurfs(istart+icount) = jsurnr
               end if
            end do
            if ( debug ) write(irefwr,*) 'icount ', icount

            if ( icount>0 ) then

!           --- There are surfaces with sequence number higer than isurnr

               iwork1(1,isurnr) = istart+1
               iwork1(2,isurnr) = icount
               istart = istart+icount

            else

!           --- Standard case, store next sequence number into iwork

               iseq = iseq + 1
               iwork(isurnr) = iseq
               check = .true.

            end if

         else if ( isurf==8 .or. isurf==9 .or. isurf==14 .or. &
                   isurf==16 ) then

!        --- Surface is a copy of a previous surface

            ndefsf = iinput(iinp+2)
            iprec = iinput(iinp+2+ndefsf+1)
            if ( iprec>isurnr .or. iwork(iprec)<=0 ) then

!           --- surface number of surface to be copied has a higher number
!               or has not been created yet

               minloop = min(minloop,isurnr)
               istart = istart+1
               iwork1(1,isurnr) = istart
               iwork1(2,isurnr) = 1
               isubsurfs(istart) = iprec

            else

!           --- Standard case

               iseq = iseq + 1
               iwork(isurnr) = iseq
               check = .true.

            end if

         else

!        --- Standard surface

            iseq = iseq + 1
            iwork(isurnr) = iseq
            check = .true.

         end if
100      if ( check .and. minloop<=isurnr ) then

!        --- Check if there are any surfaces that can be numbered now

            ih = nsurfs+1
            check = .false.
            do i = minloop, isurnr
               if ( iwork(i)<=0 ) then

!              --- Surface i has not been numbered yet

                  icount = 0
                  jstart = iwork1(1,i)-1
                  do j = 1, iwork1(2,i)
                     if ( iwork(isubsurfs(jstart+j))<=0 ) then
                        icount = icount+1
                        isubsurfs(jstart+icount) = isubsurfs(jstart+j)
                     end if
                  end do
                  iwork1(2,i) = icount
                  if ( icount==0 ) then
                     iseq = iseq + 1
                     iwork(i) = iseq
                     check = .true.
                  else
                     ih = min(ih,i)
                  end if
               end if
            end do
            if ( ih<=nsurfs ) minloop = ih
            go to 100
         end if
         if ( istart>40*nsurfs ) then

!        --- Error: estimated length of isubsurfs too small

            call errchr ( 'isubsurfs', 1 )
            call errint ( istart, 1 )
            call errint ( 40*nsurfs, 2 )
            call errsub ( 550, 2, 0, 1 )
            go to 1000

         end if
         call msh049 ( iinput, iinp, isurf )

      end do

!     --- Check if all surfaces have got a number

      do isurnr = 1, nsurfs
         if ( iwork(isurnr)<=0 ) then

!        --- surface has not yet been numbered

            call errint ( isurnr, 1 )
            call errsub ( 2094, 1, 0, 0 )

         end if

      end do

!     --- Store inverse of iwork into iseqsurfs

      do isurnr = 1, nsurfs
         iseqsurfs(iwork(isurnr)) = isurnr
      end do
      if ( debug ) then

!     --- Debug information

         call prinin ( iwork, nsurfs, 'iwork' )
         call prinin ( iseqsurfs, nsurfs, 'iseqsurfs' )

      end if

1000  if ( debug ) write(irefwr,*) 'End mshseqsurfs'
      call erclos ( 'mshseqsurfs' )

      end subroutine mshseqsurfs
