      subroutine sepda2sep ( iprmda, rprmda, ifile, ivectr, &
                             iproblc, ielhlp )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  1.0  date 02-06-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the function values in a SEPRAN vector of special structure
!     when a number of data points (structured or unstructured) are given
!     The coordinates of the measured points should not be too far outside
!     the region as specified by the mesh in KMESH. Use either TESSALATION
!     (if the data points are not structured) or special division techniques
!     (if the data points are structured) to obtain a local structure for
!     the data points. Next use interpolation and eventually extrapolation
!     to compute the function values in the original mesh as given in KMESH.
! **********************************************************************
!
!                       KEYWORDS
!
!     data
!     interpolation
!     read
!     tesselation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulemain
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iprmda(*), ifile, ivectr, iproblc, ielhlp(*)
      double precision rprmda(*)

!     ielhlp  i  integer array (reserved for later use)
!     ifile   i  file where coordinates and function information for
!                DA2SEP is stored
!     iprmda  i  integer array with parameters for DA2SEP
!                iprmda(1) = number of significant elements of iprmda
!                iprmda(2) = ichoic (default = 0)
!                ichoic = 0 or 1 standard case; vector with only 1 variable
!                    all measured data are given sequentially; on the first
!                    record the values of NPOINT and NBNDPT are expected.
!                    Furthermore it is assumed that the first NBNDPT points
!                    points constitute the boundary. In total NPOINT
!                    coordinates and function values should be given.
!                ichoic = 2 rectangular case;
!                    on the first record npx and npy indicating the number
!                    of points in x- and y-direction are expected. It is
!                    assumed that there is a simple rectangular structure in
!                    the points
!                ichoic = 3 circular case;
!                    on the first record npc and npr indicating the number
!                    of points along one "circle" and the number of
!                    "concentric" circles to be considered are expected.
!                    On the next records in total npc * npr + 1 values for
!                    coordinates and function. The last record contains the
!                    values for the "origin".
!     iproblc i    Actual problem number
!     ivectr  o    standard sepran array denoting usol or vector of special
!                  structure defined in nodal points in which the grid function
!                  is stored
!     rprmda  i  real array with parameters for DA2SEP; not yet used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i,      lengco, lengke, lengfk, &
              lengve, nelemh, number, nbndpt, idum, &
              ivec , kpivec, ndegfd, nvec , npx  , npy  , npc  , npr  , &
              method, idegfd
      integer, allocatable :: kelemh(:)
      double precision, allocatable :: coorh(:), funkh(:), tempvec(:)
      double precision :: timebefore(2)

!     coorh          coordinate array to be used for TESSELATION mesh
!     funkh          array with function values in all nodes of mesh
!     i              counting variable
!     idegfd         degree of freedom
!     idum           dummy
!     ivec           sequence number of array of special structure
!     kelemh         element array of TESSELATION mesh
!     kpivec         pointer to number of degrees of freedom for sepran vector
!     lengco         length of coordinate array
!     lengfk         length of function array
!     lengke         length of element array
!     lengve         length of new vector to be created
!     method         indicator which method should be used (1, 2 or 3)
!     nbndpt         number of boundary points in measurement
!     ndegfd         number of degrees of freedom of vector in mesh
!     nelemh         number of elements in helparray
!     npc            number of points in circular direction
!     npr            number of points in radial direction
!     npx            number of points in first direction in case of a structured
!                    mesh
!     npy            number of points in second direction in case of a
!                    structured mesh
!     number         number of measured values to be read by DA2SEP
!     nvec           number of vectors of special structure
!     tempvec        temporary vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLMN         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERSETTIME      Get the CPU time at start of subroutine
!     INSTOP         Stop the program
!     MSH465         Read measurements, create new mesh and fill function values
!     MSH466         Use new mesh of MSH465 and interpolate for all nodes of old
!                    mesh in order to fill special array
!     MSH471         Read measurements, create new mesh and fill function values
!                    in case of a rectangular structure
!     MSH472         Read measurements, create new mesh and fill function values
!                    in case of a circular structure
!     SEPCREASOL110  Activate or create arrays of type 110
!     SEPCREASOLBF1  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
!     TO0082         Map one vector onto another (real case)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     206   No vector of special structure at all
!    1732   No appropriate sepran vector available
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Determine which method has to be used
!     If method Tessa ( method = 1 ) then
!        Read information from file (number of points and number of points
!        on outer boundary of domain considered)
!        Make reservations
!        Use TESSELATION routine to make structure in the data points
!     else if method Rectangle ( method = 2 ) then
!        Read information from file (number of points in respectively
!        x- and y-direction, coordinates and function values)
!        Make reservations
!        Use simple triangles based on the structure of the data points, &
!        to get an overall structure
!     else if method Circular ( method = 3 ) then
!        Read information from file (number of points in circular direction
!        and number of points in radial direction)
!        Make reservations
!        Use simple triangles based on the special circular structure of
!        the data points to get an overall structure
!     end if
!     Run through all nodal points of the original mesh and determine
!     function values by interpolation based on the values as given
!     in the (see above) constructed mesh
!     Fill solution vector or vector of special structure
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen( 'sepda2sep' )
      if ( itime>0 ) call ersettime ( timebefore )

      iprob = iproblc

      if ( iprmda(1)<2 ) then

!     --- Use default Method

         method = 1

      else

!     --- User specified the method

         method = iprmda(2)

      end if
      if ( method==1 ) then

!     --- Classical Method, number points and the first nbndpt
!         points are the boundary; it is required that all other points
!         (hence all points with sequence number greater than nbndpt)
!         are inside that boundary.
!         Read information from measu.dat file (as indicated)

         read ( ifile,*) number, nbndpt
         lengco = number * ndim
         lengke = 3 * ( 2 * ( number - 1 ) - nbndpt )

!        --- Make reservation for helparrays in order to make structure

         allocate ( coorh(lengco), kelemh(lengke), funkh(number), &
                    tempvec(npoint), stat = error )
         if ( error/=0 ) call eralloc ( error, lengco+lengke, 'coorh' )
         if ( ierror/=0 ) go to 1000

!        --- Make structure

         call msh465( ifile, coorh, number, nbndpt, &
                      kelemh, nelemh, funkh )

      else if ( method==2 ) then

!     --- "Rectangular" mesh, all points are given on rows. (npx * npy)

         read( ifile, * ) npx, npy

!        --- Make temporary reservation for coorh

         lengco = npx * npy * ndim

         number = npx * npy

!        --- Make temporary reservation for kelemh (always triangles)

         lengke = 6 * ( npx-1 ) * ( npy-1 )

         allocate ( coorh(lengco), kelemh(lengke), funkh(number), &
                    tempvec(number), stat = error )
         if ( error/=0 ) call eralloc ( error, lengco+lengke, 'coorh' )
         if ( ierror/=0 ) go to 1000

         call msh471( ifile, coorh, npx, npy, kelemh, &
                      nelemh, 3, funkh )

      else if ( method==3 ) then

!     --- "Circular" mesh, all points are given on concentric circles
!         there are npr circles and on each are npc points

         read( ifile, * ) npc, npr

!        --- Make temporary reservation for coorh

         lengco = ( npr * npc + 1 ) * ndim

!        --- Make temporary reservation for kelemh (always triangles)

         lengke = 3 * ( 2 * npc * (npr-1) + npc )
         number = npc * npr + 1

         allocate ( coorh(lengco), kelemh(lengke), funkh(number), &
                    tempvec(npoint), stat = error )
         if ( error/=0 ) call eralloc ( error, lengco+lengke, 'coorh' )
         if ( ierror/=0 ) go to 1000

         call msh472( ifile, coorh, npc, npr, kelemh, &
                      nelemh, funkh )


      else

!     --- Method is not yet available

         write(irefwr,*) ' In routine DA2SEP the value for the method '
         write(irefwr,*) ' is : ',method
         write(irefwr,*) ' This method is not available ! '
         write(irefwr,*) ' Execution stopped adjust the method value '

         call instop

      end if

!     --- Make interpolation for old points on TESSELATION mesh

      call msh466( coorh, number, kelemh, &
                   funkh, nelemh, coor, &
                   ndim, npoint, tempvec )

!     --- Make reservation for vector permanent,
!         find either solution vector or special structure vector

      iseqkp = kprob
      nprob = kp(iseqkp)%nprob
      iprob = nprob
      kprobpart => kp(iseqkp)%kprobsub(iprob)
      kprobloc => kprobpart%kprobloc

!     --- first check  itypvc = 110     array of structure usol

      nunkp   = kprobloc(4)
      ndegfd  = 0

      if ( nunkp==1 .and. kprobloc(19)<=0 ) then

!     --- number of degrees of freedom is constant and equal to 1

         call sepcreasol110 ( ivectr, 'ivectr', 0 )
         vectout => usoltemp
         kprobpart => kp(iseqkp)%kprobsub(iprob)
         kprobloc => kprobpart%kprobloc

         ndegfd      = kprobloc(5 )

!        --- restriction: nphys=1, indprf=0

         idegfd = 0
         indprfi = 0
         nphysi = 1
         nphyso = 1
         call to0082( tempvec, vectout, idegfd, 0, idum )

      else

!     --- Try itypvc = 115 : vector of special structure

         nvec = kprobloc(6 )

         if ( nvec<=0 ) then

!        --- error : no vector with sequence number ivec>0 defined

            call errint( ivec, 1 )
            call errint( nvec, 2 )
            call errsub ( 206, 2, 0, 0 )
            call instop

         end if

         nunkp  =  0

         do ivec = 1, nvec

            kpivec =  kprobloc(18 ) - 1 + &
                      kelgrp * kprobloc(6 ) + ivec

            nunkp  =  kprobloc(kpivec )

            if ( nunkp==-1 ) then

!           --- nunkp<0   constant number of degrees of freedom

               nunkp   = -nunkp
               ndegfd  =  nunkp * npoint

               call sepcreasolbf1 ( ivectr, 115, ivec, 'ivectr', 0 )

            end if

         end do

      end if

      if ( ndegfd==0 ) then

!     --- failed to find appropriate sepran vector

         call errsub ( 1732, 0, 0, 0)
         call instop

      end if
      deallocate ( coorh, kelemh, funkh, tempvec, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'coorh' )

1000  call erclmn( 'sepda2sep', timebefore )

      end subroutine sepda2sep
