      subroutine sepmsh059 ( iref, ihelp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     write information of the mesh to the file indicated by iref
!     If iref>=0, then formatted output is used with at most 80 columns
!     If iref<0 unformatted output
!     This subroutine performs the tasks of the previous subroutine meshwr
!     Old subroutine msh059
! **********************************************************************
!
!                       KEYWORDS
!
!     write_mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecpack
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iref, ihelp(9)

!     ihelp      i    In this array the user must put information to be written
!                     in the second record after nvolms
!     iref       i    dataset reference number for the output
!                     when iref = 0  the standard output file is assumed
!                     when iref>0, the output is written to the file with
!                     reference number iref with formatted output
!                     When iref<0, the output is written to the file with
!                     reference number -iref with unformatted output
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ielgrp, inboun, irefc, ivers, lenkmsv, istart, &
              j, lenrl, index1, index2, iobst, leniwork, &
              lenintarr, lenrealarr
      character *80 text

!     i              Counting variable
!     ielgrp         element group number
!     inboun
!     index1         Memory management sequence number with respect to kmeshc
!                    for obstacle
!     index2         Memory management sequence number with respect to coor
!                    for obstacle
!     iobst          Obstacle sequence number
!     irefc          dataset reference number for reading (>0 formatted, &
!                    <0 unformatted)
!     istart         First position used at a part
!     ivers          Version sequence number
!     j              Counting variable
!     lenintarr      Length of array intarr
!     leniwork       Length of array iwork
!     lenkmsv        Length of array KMESH part v
!     lenrealarr     Length of array realarr
!     lenrl          Number of reals in one row of rmeshx
!     text           String variable containing some text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHWRITEINT    Write integer array to file irefc
!     MSHWRITEREAL   Write real array to file irefc
!     SEPFILLKMESHAA Fill first part of array KMESH part aa
!     SEPFILLNMESHA  Fill array nmesha
!     SEPFILLNMESHC  Fill array nmeshc
!     SEPMSH026      write mesh arrays to file irefc
!     SEPMSH077      Write arrays KMESHV, INTPROP and REALPROP to file irefc
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepmsh059' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepmsh059'
         write(irefwr,1) 'nelgrp', nelgrp
         write(irefwr,*) 'parallel ', parallel
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      irefc = iref
      if ( iref==0 ) irefc = irefwr
      ishape = 0
      nboun  = npelm
      ivers  = 8

      if ( nelgrp==1 ) then

!     --- nelgrp  =  1

         ishape = kmeshf(1)
         if ( kelmb/=0) nboun = kmeshb(1)

         if ( debug ) write(irefwr,1) 'ishape, nboun', ishape, nboun

      end if

      ihelp = 0
      if ( .not. parallel ) then

!     --- These parts are only written in case of serial computation

         ihelp(3) = kelmk
         ihelp(4) = kelmu
         ihelp(5) = kelmj

      end if  ! ( .not. parallel )

      ihelp(1) = kelmt
      ihelp(2) = nextpnt
      ihelp(6) = kelmm
      ihelp(7) = kelmn
      if ( ispecial==1 .or. itrans==1 .or. irefin==1 .or. kelmu>0 ) then
         ihelp(8) = 0
      else
         ihelp(8) = 1
      end if
      ihelp(9) = ivers
      if ( irefc>=0 ) then
         write(irefc,100) -ndim, npelm, nboun, ishape, nelgrp, npoint, &
               nelem, nuspnt, ncurvs, nsurfs
         write(irefc,100) nvolms, (ihelp(i),i=1,9)
100      format(10i10)
         write(irefc,110) '//First 2 records: dimension parameters:'
         write(irefc,110) '// -ndim,  npelm,  nboun,  ishape, nelgrp,', &
                          ' npoint,  nelem,  nuspnt, ncurvs,nsur'
         write(irefc,110) '// nvolms,  kelmt,  nextpnt, kelmk, kelmu,', &
                          '  kelmj,   kelmm,  kelmn, imap, ivers'
110      format (4a)
      else
         write(-irefc) -ndim, npelm, nboun, ishape, nelgrp, npoint, &
               nelem, nuspnt, ncurvs, nsurfs
         write(-irefc) nvolms, (ihelp(i),i=1,9)
         text = '//First 2 records: dimension parameters:'
         write(-irefc) text
         text = '//-ndim, npelm, nboun, ishape, nelgrp, npoint, nelem'// &
                ', nuspnt, ncurvs, nsurfs'
         write(-irefc) text
         text = '//nvolms, kelmt, nextpnt, kelmk, kelmu, kelmj, kelmm'// &
                ', kelmn, imap, ivers'
         write(-irefc) text
      end if

      if ( nelgrp>1 ) then

!     --- nelgrp>1

         inpelm = npelm
         if ( irefc>=0 ) then
            write(irefc,110) '//Element_Group_Information'
         else
            text = '//Element_Group_Information'
            write(-irefc) text
         end if
         do ielgrp = 1, nelgrp

            inpelm = Topgrp(ielgrp)%inpelm
            inboun = sepgetbounelm(ielgrp)
            inelem = Topgrp(ielgrp)%inelem
            ishape = kmeshf(ielgrp)
            if ( inelem>0 ) then

!           --- Standard situation

               if ( irefc>=0 ) then
                  write(irefc,100) inpelm, ishape, inelem, inboun
               else
                  write(-irefc) inpelm, ishape, inelem, inboun
               end if

            else

!           --- element group must be skipped

               if ( irefc>=0 ) then
                  write(irefc,100) inpelm, ishape, inelem, inboun
               else
                  write(-irefc) inpelm, ishape, inelem, inboun
               end if

            end if

         end do

      end if

      leniwork = max(10,npoint)*2
      leniwork = max(leniwork,ncurvs)

      if ( debug ) write(irefwr,*) 'before sepmsh026'
      call sepmsh026 ( irefc, leniwork, ihelp(8) )

!     --- Extension for version 5
!         Kmesh parts v and w

      do i = 1, 9
         ihelp(i) = 0
      end do
      ihelp(1) = kelmv
      ihelp(2) = kelmw
      if ( numobst>0 ) then
         ihelp(3) = numobst
         ihelp(6) = size(kmeshx)
      end if
      ihelp(7) = mmouter
      ihelp(8) = mmnormal
      ihelp(9) = kelmag
      if ( iwritfaces>0 ) then

!     --- faces/edges must be written
!         First they must be activated or created

         if ( kelmaa==0 ) call sepfillkmeshaa
         if ( nmesh(31)==0 .or. nmesh(32)==0 ) call sepfillnmesha
         if ( nmesh(33)==0 ) call sepfillnmeshc
         ihelp(4) = nedges
         ihelp(5) = nfaces

      end if
      if ( irefc>=0 ) then
         write(irefc,110) '//Second_Dimension_Group'
         write(irefc,100) (ihelp(i),i=1,9), kelmak
      else
         text = '//Second_Dimension_Group'
         write(-irefc) text
         write(-irefc) (ihelp(i),i=1,9), kelmak
      end if

      if ( kelmv>0 ) then

!     --- kelmv>0, i.e. kmesh part v exists

         lenkmsv = size(kmeshv)

      else

!     --- kelmv = 0, i.e. kmesh part v does not exist

         lenkmsv = 0

      end if

      if ( debug ) write(irefwr,*) 'before sepmsh077'
      call sepmsh077 ( irefc )

      if ( numobst>0 ) then

!     --- numobst>0, i.e. kmesh part x exists

         if ( irefc>=0 ) then
            write(irefc,110) '//Kmeshx'
            write(irefc,100) kmeshx
            write(irefc, 120) rmeshx
120         format(1x, 3e19.10)
         else
            text = '//Kmeshx'
            write(-irefc) text
            write(-irefc) kmeshx
            write(-irefc) rmeshx
         end if
         if ( ndim==3 ) then

!        --- ndim = 3, also write coordinates and topology of the obstacles

            numobst = numobst
            do iobst = 1, numobst
               index1 = kmeshx(8,iobst)
               index2 = kmeshx(9,iobst)
               inpelm = kmeshx(10,iobst)
               nelemlc  = kmeshx(11,iobst)
               npointlc = kmeshx(12,iobst)
               kmeshcobst => kmobst(iobst)%kmeshcobst
               coorobst => kmobst(iobst)%coorobst
               if ( irefc>=0 ) then
                  write(irefc,100) kmeshcobst
                  write(irefc, 120) coorobst
               else
                  write(-irefc) kmeshcobst
                  write(-irefc) coorobst
               end if

            end do

         end if

      end if

      if ( debug ) write(irefwr,*) 'before volume_surfs'
      if ( kelmag>0 ) then

!     --- kelmag >> 0, write volume_surfs

         text = '//Volume_surfs'
         lenintarr = size(volume_surfs)
         call mshwriteint ( irefc, text, volume_surfs, lenintarr )

      end if

      if ( debug ) write(irefwr,*) 'before outer boundary'
      if ( mmouter>0 ) then

!     --- mmouter >> 0, write outer boundary

         text = '//Outercurves'
         lenintarr = size(outerboun)
         call mshwriteint ( irefc, text, outerboun, lenintarr )

      end if

      if ( debug ) write(irefwr,*) 'before normals'
      if ( mmnormal>0 ) then

!     --- mmnormal >> 0, write normals

         text = '//Normals'
         lenrealarr = size(normal)
         call mshwritereal ( irefc, text, normal, lenrealarr )

      end if

      if ( debug ) write(irefwr,*) 'before curveusrp'
      if ( kelmak>0 ) then

!     --- kelmak >> 0, write curveusrp

         text = '//Curveusp'
         lenintarr = size(curveusrp)
         call mshwriteint ( irefc, text, curveusrp, lenintarr )

      end if

      if ( ihelp(4)>0 ) then

!     --- The edges are constructed

         if ( irefc>=0 ) then
            write(irefc,110) '//Edges'
            write(irefc,100) nedges, nfaces
            write(irefc,100) nmeshb(1:nfaces)
            istart = 1+nfaces
            do i = 1, nfaces
               length = nmeshb(istart)
               write(irefc,100) (nmeshb(istart+j),j=0,length)
               istart = istart+length+1
            end do
         else
            text = '//Edges'
            write(-irefc) text
            write(-irefc) nedges, nfaces
            write(-irefc) nmeshb(1:nfaces)
            write(-irefc) nmeshb(nfaces+1:size(nmeshb))
         end if

      end if
      if ( ihelp(5)>0.and.nvolms>0 ) then

!     --- The faces are constructed

         if ( irefc>=0 ) then
            write(irefc,110) '//Faces'
            write(irefc,100) nelem, nfaces
            write(irefc,100) nmesha(1:nelem)
            istart = 1+nelem
            do i = 1, nelem
               length = nmesha(istart)
               write(irefc,100) (nmesha(istart+j),j=0,length)
               istart = istart+length+1
            end do
         else
            text = '//Faces'
            write(-irefc) text
            write(-irefc) nelem, nfaces
            write(-irefc) nmesha(1:nelem)
            write(-irefc) nmesha(nelem+1:size(nmesha))
         end if

      end if
      if ( ihelp(4)>0 ) then

!     --- The points per edge are written

         if ( irefc>=0 ) then
            write(irefc,110) '//Nodes_per_edge'
            write(irefc,100) nedges
            write(irefc,100) nmeshc(1:nedges)
            istart = 1+nedges
            do i = 1, nedges
               length = nmeshc(istart)
               write(irefc,100) (nmeshc(istart+j),j=0,length)
               istart = istart+length+1
            end do
         else
            text = '//Nodes_per_edge'
            write(-irefc) text
            write(-irefc) nedges
            write(-irefc) nmeshc(1:nedges)
            write(-irefc) nmeshc(nedges+1:size(nmeshc))
         end if

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepmsh059'

      end if  ! ( debug )
      call erclos ( 'sepmsh059' )

      end subroutine sepmsh059
