      subroutine sepmshnorc0
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute normals on boundary in R^2 and store in array normal
!     Old subroutine mshnorc0
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     boundary
!     outer_boundary
!     normal_vector
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iwork(ncurvs), istart, istar3, ifirstsub, ilastsub, &
              ifirst, ilast, icurnr, k1, k2, inext, iprev, inode, isub, &
              isubcv, nnodes, jstart, kstep, nsubcv, k, iprevlast
      double precision h, any, anx
      logical closed

!     anx            x-component of normal vector
!     any            y-component of normal vector
!     closed         If true the boundary is closed
!     h              Length of curve part
!     i              Counting variable
!     icurnr         Curve sequence number
!     ifirst         First point in a curve
!     ifirstsub      First point in a subcurve
!     ilast          Last point in a curve
!     ilastsub       Last point in a subcurve
!     inext          Next point in curve
!     inode          Node sequence number
!     iprev          Previous point in curve
!     iprevlast      Previous last point in subcurve
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     istart         First position used at a part
!     isub           Loop variable
!     isubcv         Subcurve sequence number with direction
!     iwork          integer work array that is used to mark curves
!                    iwork(i) = -1, curve is composite curve
!                    iwork(i) = 0, curve has not been treated yet
!                    iwork(i) = 1, curve has been treated
!     jstart         starting position of first curve of composite one
!     k              Counting variable
!     k1             First point of curve in KMESH part m
!     k2             Last point of curve in KMESH part m
!     kstep          Step in loop
!     nnodes         Number of nodes
!     nsubcv         Number of subcurves of a curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PR0008         Print information of curves
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     339   Length of line between two points is zero
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array iwork
!     Mark all composite curves with a -1
!     For all single curves do
!        If ( curve not treated yet ) then
!           compute normal per curve
!     For all composite curves do
!        Copy normal per subcurve, but use mean value in common points
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepmshnorc0' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepmshnorc0'

      end if

!     --- Clear array iwork

      do i = 1, ncurvs
         iwork(i) = 0
      end do

!     --- Mark all composite curves with a -1

      if ( kelmq>0 ) then

!     --- kelmq>0, there are composite curves

         do i = 1, ncurvs
            if ( kmeshq(i+1)-kmeshq(i)>0 ) iwork(i) = -1
         end do

      end if
      istar3 = kmeshm(2)-1

!     --- For all single curves do
!           compute normal per curve

      do icurnr = 1, ncurvs

         if ( iwork(icurnr)==0 ) then

!        --- Curve is a single curve

            k1 = kmeshm(4+icurnr)
            k2 = kmeshm(5+icurnr)-1
            closed = kmeshm(istar3+k1)==kmeshm(istar3+k2)
            if ( debug ) write(irefwr,*) ' icurnr, k1, k2, closed ' , &
               icurnr, k1, k2, closed
            do k = k1, k2

               inode = kmeshm(istar3+k)
               if ( inode<=0 ) then

!              --- Node has not been filled

                  normal(1,k) = 0d0
                  normal(2,k) = 0d0

               else

!              --- Node has been filled, store previous and next one

                  if ( k>k1 ) then
                     iprev = kmeshm(istar3+k-1)
                     if ( iprev<=0 ) iprev = inode
                  else if ( closed ) then
                     iprev = kmeshm(istar3+k2-1)
                     if ( iprev<=0 ) iprev = inode
                  else
                     iprev = inode
                  end if
                  if ( k<k2 ) then
                     inext = kmeshm(istar3+k+1)
                     if ( inext<=0 ) inext = inode
                  else if ( closed ) then
                     inext = kmeshm(istar3+k1+1)
                     if ( inext<=0 ) inext = inode
                  else
                     inext = inode
                  end if
                  if ( iprev==inext ) then

!                 --- Node is single, normal can not be computed

                     normal(1,k) = 0d0
                     normal(2,k) = 0d0

                  else

!                 --- compute normal

                     any = coor(1,inext)-coor(1,iprev)
                     anx = coor(2,inext)-coor(2,iprev)
                     h = sqrt(any*any+anx*anx)
                     if ( h<=0 ) then
                        call errint ( inode, 1 )
                        call errint ( iprev, 2 )
                        call errint ( inext, 3 )
                        call errint ( icurnr, 4 )
                        call errint ( icurnr, 5 )
                        call errint ( k-k1+1, 6 )
                        call errsub ( 339, 6, 0, 0 )
                        go to 1000
                     end if
                     h = 1d0/h
                     normal(1,k) = anx*h
                     normal(2,k) =-any*h

                  end if

               end if

            end do
            iwork(icurnr) = 2

         end if

      end do

!     --- Fill composite curves from single curves

      if ( kelmq>0 ) then

!     --- kelmq>0, there are composite curves

         do icurnr = 1, ncurvs

            nsubcv = kmeshq(icurnr+1) - kmeshq(icurnr)
            if ( nsubcv>0 ) then

!           --- nsubcv > 0, compute first and last node number

               istart = kmeshm(4+icurnr)
               isubcv = kmeshq(kmeshq(icurnr)+1)
               if ( isubcv>0 ) then

!              --- isubcv > 0

                  ifirst = kmeshm(istar3+kmeshm(4+isubcv))

               else

!              --- isubcv < 0

                  ifirst = kmeshm(istar3+kmeshm(5-isubcv)-1)

               end if

               isubcv = kmeshq(kmeshq(icurnr)+nsubcv)
               if ( isubcv>0 ) then

!              --- isubcv > 0

                  ilast = kmeshm(istar3+kmeshm(5+isubcv)-1)

               else

!              --- isubcv < 0

                  ilast = kmeshm(istar3+kmeshm(4-isubcv))

               end if
               if ( debug )  write(irefwr,*) &
                  'icurnr, ifirst, ilast, istart ', &
                   icurnr, ifirst, ilast, istart

               normal(1,istart) = 0d0
               normal(2,istart) = 0d0
               jstart = istart
               do isub = 1, nsubcv
                  isubcv = kmeshq(kmeshq(icurnr)+isub)
                  if ( isubcv>0 ) then

!                 --- isubcv > 0

                      nnodes = kmeshm(isubcv+5)-kmeshm(isubcv+4)
                      k1 = kmeshm(isubcv+4)
                      ifirstsub = kmeshm(istar3+k1)
                      ilastsub = kmeshm(istar3+kmeshm(isubcv+5)-1)
                      kstep = 1
                      iprevlast = 0

                  else

!                 --- isubcv < 0

                      nnodes = kmeshm(-isubcv+5)-kmeshm(-isubcv+4)
                      k1 = kmeshm(-isubcv+5)-1
                      kstep = -1
                      ilastsub = kmeshm(istar3+k1)
                      ifirstsub = kmeshm(istar3+kmeshm(-isubcv+4))

                  end if
                  if ( debug ) then

!                 --- Debug information

                     write(irefwr,*) 'isub, isubcv, nnodes, istart ', &
                                      isub, isubcv, nnodes, istart
                     write(irefwr,*) 'ifirstsub, ilastsub ', &
                                      ifirstsub, ilastsub
                  end if

                  do k = 1, nnodes
                     normal(1,istart+k-1) = normal(1,k1+(k-1)*kstep)
                     normal(2,istart+k-1) = normal(2,k1+(k-1)*kstep)
                  end do

                  if ( ifirstsub==iprevlast ) then

!                 --- First node is equal to previous last node
!                     Compute mean value of normals

                     normal(1,istart) = 0.5d0* &
                                     (normal(1,istart-1)+normal(1,k1))
                     normal(2,istart) = 0.5d0* &
                                     (normal(2,istart-1)+normal(2,k1))

                  end if

                  istart = istart+nnodes
                  if ( debug )  write(irefwr,*) ' istart ', istart
                  iprevlast = ilastsub

               end do
               if ( ifirst>0 .and. ifirst==ilast ) then

!              --- curve is closed, compute mean value in initial and end point

                  normal(1,istart-1) = 0.5d0* &
                                (normal(1,istart-1)+normal(1,jstart))
                  normal(2,istart-1) = 0.5d0* &
                                (normal(2,istart-1)+normal(2,jstart))

               end if

            end if

         end do

      end if

      if ( debug ) then

!     --- Debug information

         call pr0008 ( kmeshm, ncurvs )
         istart = kmeshm(ncurvs+5)-kmeshm(5)
         call prinrl1 ( normal, istart, 2, 'normals' )

      end if

1000  call erclos ( 'sepmshnorc0' )
      end
