      subroutine sepobstmarkpt ( iobst, nx, ny, iobgrid, ipointobstlc, &
                                 iwork, lencurvs, iobgrid1 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Mark all nodal points in the mesh with information if they are
!     within an obstacle or outside an obstacle
!     Store this information in array ipointobst
!     Remark:  if this subroutine is changed also change MSHOBSTPOINT
!     Old subroutine mshobstmarkpt
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     node
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulegen
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iobst, nx, ny, iobgrid(0:nx-1,0:ny-1), &
              iwork(*), lencurvs, iobgrid1(*), ipointobstlc(*)

!     iobgrid        i    array of length nx x ny that is filled as follows:
!                         The region covered is defined by xmin, xmax, ymin, &
!                         ymax
!                         The cell size is dx x dy
!                         Cell i,j refers to the coordinates:
!                         (xmin+(i-1)dx,ymin+(j-1)dy) x (xmin+idx,ymin+jdy)
!                         iobgrid(i,j) = 0: cell outside obstacle
!                         iobgrid(i,j) = k>0: boundary part k crosses cell
!                         iobgrid(i,j) = -1: cell completely in obstacle
!     iobgrid1       i    Must be used in connection with iobgrid
!                         iobgrid contains the starting addresses for each cell
!                         i,j
!                         From this starting address iobgrid1 contains
!                         the number of curve parts of the obstacle
!                         intersecting the cell followed by the sequence
!                         numbers of these curve parts
!     iobst          i    Obstacle sequence number
!     ipointobstlc   o    Array containing information about the nodes in an
!                         obstacle
!                         In this subroutine it contains information for one
!                         obstacle only
!                         See PG Section 24.2 for a description
!     iwork               Work array of length npoint to store the boundary
!                         nodes temporarily
!     lencurvs       i    Overestimate of the number of points along the new
!                         curves
!     nx             i    number of cells in x direction corresponding to
!                         obstacle
!     ny             i    number of cells in y direction corresponding to
!                         obstacle
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical onobstacle, found, closed
      double precision xmin, xmax, ymin, ymax, x, y, dx, dy, alpha1, &
                       beta1, eps, xminorig, yminorig, xi(6), &
                       curvescp(2,lencurvs)
      integer ipcell, jpcell, nnodes, ixcross(2), iycross(2), i, &
              icurnrobs, inode, nintersect, &
              numpntobst, numbndobst, nxi

!     alpha1         multiplication factor
!     beta1          multiplication factor
!     closed         If true the boundary is closed
!     curvescp       Array to store all coordinates of one curve
!     dx             step in x-direction
!     dy             step in y-direction
!     eps            Accuracy
!                    Points that are at a distance less than eps of the
!                    obstacle are defined as part of the obstacle
!     found          If true already the point has been determined
!     i              Counter
!     icurnrobs      Curve sequence number of obstacle
!     inode          Node sequence number
!     ipcell         i-index of cell(i,j)
!     ixcross        Contains the number of cross sections of the line x=xp
!                    with the obstacle within cell(i,j)
!                    ixcross(1) contains the number of cross sections
!                    below yp
!                    ixcross(2) contains the number of cross sections
!                    above yp
!     iycross        Contains the number of cross sections of the line y=yp
!                    with the obstacle within cell(i,j)
!                    iycross(1) contains the number of cross sections
!                    at the left-hand side of xp
!                    iycross(2) contains the number of cross sections
!                    at the right-hand side of xp
!     jpcell         j-index of cell(i,j)
!     nintersect     Number of intersections found
!     nnodes         Number of nodes
!     numbndobst     Number of nodal points on the boundary of the obstacle
!     numpntobst     Number of nodal points inside the obstacle
!     nxi            Number of points that are intersected in x-direction
!     onobstacle     If true the point is positioned on the obstacle
!     x              x co-ordinate of point
!     xi             Array containing the x-coordinates of the intersection
!                    points in the cell
!     xmax           Maximum x-value
!     xmin           Minimum x-value
!     xminorig       Minimum x-value to compute ipcell, jpcell
!     y              y co-ordinate of point
!     ymax           Maximum y-value
!     ymin           Minimum y-value
!     yminorig       Minimum y-value to compute ipcell, jpcell
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHOBSTCROSS   Compute cross section of obstacle with lines x=xp and y=yp
!                    in one cell
!     MSHOBSTCROSS1  Compute cross section of obstacle with line y=yp
!                    in one cell
!     SEPCOPCURVCOOR Copy coordinates of curve icurnr from array curves into
!                    array curvescp
!     SEPOBSTMARKPT1 Reduced version of subroutine mshobstmarkpt in case of open
!                    obstacles lying in x, y or z direction
!     SEPPRINIPOBST  Print the contents of array ipointobst
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Extract xmin,xmax,ymin,ymax from obstacle array
!    numpntobst := 0
!    numbndobst := 0
!    For inode := 1 (1) npoint do
!       (x,y) := coordinates of node
!       If ( (x,y) inside (xmin,xmax;ymin,ymax) )
!          icell := (i,j) index of obstacle cell in which (x,y) is lying
!          if ( iobgrid(i,j) = -1 ) then
!             numpntobst := numpntobst+1
!             ipointobst(numpntobst) := inode
!          else if ( iobgrid(i,j)>0 ) then
!             compute number of cross sections of line x = xp left and right
!             of xp in rectangle as well as number of cross sections of
!             line y = yp below and above yp
!             found := false
!             if ( no cross sections left of x = xp ) then
!                if ( cell at left-hand side is in obstacle ) then
!                   numpntobst := numpntobst+1
!                   ipointobst(numpntobst) := inode
!                   found := true
!                else if ( cell at left-hand side is outside obstacle ) then
!                   found := true
!             else if ( no cross sections right of x = xp ) then
!                if ( cell at right-hand side is in obstacle ) then
!                   numpntobst := numpntobst+1
!                   ipointobst(numpntobst) := inode
!                   found := true
!                else if ( cell at right-hand side is outside obstacle ) then
!                   found := true
!             if ( no cross sections below y = yp ) then
!                if ( cell below is in obstacle ) then
!                   numpntobst := numpntobst+1
!                   ipointobst(numpntobst) := inode
!                   found := true
!                else if ( cell below is outside obstacle ) then
!                   found := true
!             else if ( no cross sections above y = yp ) then
!                if ( cell above is in obstacle ) then
!                   ipointobst(inode) := iobst
!                   found := true
!                else if ( cell above is outside obstacle ) then
!                   found := true
!             else
!                use line x=xp to detect the number of cross sections at
!                the left-hand side
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepobstmarkpt' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepobstmarkpt'

      end if
      if ( ierror/=0 ) go to 1000

!     --- initialize parameters

      icurnrobs = kmeshx(2,iobst)
      numpntobst = 0
      numbndobst = 0
      nxi = 0

!     --- Copy coordinates of curve into curvescp

      call sepcopcurvcoor ( icurnrobs, curvescp, nnodes )

!     --- Extract xmin,xmax,ymin,ymax from obstacle array

      xminorig = rmeshx(1,iobst)
      xmax     = rmeshx(2,iobst)
      yminorig = rmeshx(3,iobst)
      ymax     = rmeshx(4,iobst)
      dx   = rmeshx(5,iobst)
      dy   = rmeshx(6,iobst)
      closed = kmeshx(1,iobst)>0
      eps = rinfin
      do i = 1, ncurvs
         eps = min(eps,elsize(1,i),elsize(2,i))
      end do
      eps = eps*accuracy_obstacle
      xmin = xminorig-eps
      xmax = xmax    +eps
      ymin = yminorig-eps
      ymax = ymax    +eps

      if ( debug ) then

!     --- debug is true, print info

         write(irefwr,*) 'Data with respect to obstacle'
         write(irefwr,*) 'xmin, xmax, ymin, ymax ', &
                          xmin, xmax, ymin, ymax
         write(irefwr,*) 'accuracy ', eps
         write(irefwr,*) 'nx, ny, dx, dy ', nx, ny, dx, dy

      end if

!     --- For inode := 1 (1) npoint do

      do inode = 1, npoint

!     --- (x,y) := coordinates of node

         x = coor(1,inode)
         y = coor(2,inode)
         if ( x>=xmin .and. x<=xmax .and. &
              y>=ymin .and. y<=ymax ) then

!        --- (x,y) inside (xmin,xmax;ymin,ymax)
!            icell := (i,j) index of obstacle cell in which (x,y) is lying

            alpha1 = (x-xminorig)/dx
            beta1  = (y-yminorig)/dy
            ipcell = int(alpha1)
            jpcell = int(beta1)
            ipcell = min(max(ipcell,0),nx-1)
            jpcell = min(max(jpcell,0),ny-1)

            if ( debug ) then

!           --- debug is true, print info

               write(irefwr,*) 'Node ', inode, &
                               ' is in rectangle around obstacle'
               write(irefwr,*) 'x, y ', x, y
               write(irefwr,*) 'alpha1, beta1 ',alpha1, beta1
               write(irefwr,*) 'ipcell, jpcell ',ipcell, jpcell
               write(irefwr,*) 'iobgrid info for cell ', &
                                iobgrid(ipcell,jpcell)

            end if

            if ( iobgrid(ipcell,jpcell)==-1 ) then

!           --- iobgrid(i,j) = iobst

               numpntobst = numpntobst+1
               ipointobstlc(numpntobst+4) = inode

            else if ( iobgrid(ipcell,jpcell)>0 ) then

!           --- iobgrid(i,j)>0
!               compute number of cross sections of line x = xp left and right
!               of xp in rectangle as well as number of cross sections of
!               line y = yp below and above yp

               call mshobstcross ( x, y, ipcell, jpcell, onobstacle, &
                                   rmeshx(1,iobst), iobgrid, nx, ny, &
                                   curvescp, nnodes, ixcross, iycross, &
                                   eps, xi, nxi, iobgrid1 )

               if ( onobstacle ) then

!              --- Point on obstacle

                  numbndobst = numbndobst+1
                  iwork(numbndobst) = inode
                  go to 100

               end if

               found = .false.

               if ( ixcross(1)==0 ) then

!              --- no cross sections below y = yp

                  if ( jpcell<=0 ) then

!                 --- Below there is no obstacle

                     found = .true.

                  else if ( iobgrid(ipcell,jpcell-1)==-1 ) then

!                 --- cell below is in obstacle
!                     ipointobst(inode) := iobst
!                     found := true

                     numpntobst = numpntobst+1
                     ipointobstlc(numpntobst+4) = inode
                     found = .true.

                  else if ( iobgrid(ipcell,jpcell-1)==0 ) then

!                 --- cell below is outside obstacle
!                     ipointobst(inode) := 0
!                     found := true

                     found = .true.

                  end if

               else if ( ixcross(1)==1 ) then

                  if ( jpcell==0 ) then

!                 --- Below there is no obstacle

                     numpntobst = numpntobst+1
                     ipointobstlc(numpntobst+4) = inode
                     found = .true.

                  else if ( iobgrid(ipcell,jpcell-1)==-1 ) then

!                 --- cell below is inside obstacle
!                     ipointobst(inode) := 0
!                     found := true

                     found = .true.

                  else if ( iobgrid(ipcell,jpcell-1)==0 ) then

!                 --- cell below is outside obstacle
!                     ipointobstlc(inode) := 1
!                     found := true

                     numpntobst = numpntobst+1
                     ipointobstlc(numpntobst+4) = inode
                     found = .true.

                  end if

               end if

               if ( .not. found .and. ixcross(2)==0 ) then

!              --- no cross sections above y = yp

                  if ( jpcell>=ny-1 ) then

!                 --- Above there is no obstacle

                     found = .true.

                  else if ( iobgrid(ipcell,jpcell+1)==-1 ) then

!                 --- cell above is in obstacle
!                     ipointobst(inode) := iobst
!                     found := true

                     numpntobst = numpntobst+1
                     ipointobstlc(numpntobst+4) = inode
                     found = .true.

                  else if ( iobgrid(ipcell,jpcell+1)==0 ) then

!                 --- cell above is outside obstacle
!                     ipointobst(inode) := 0
!                     found := true

                     found = .true.

                  end if

               else if ( .not. found .and. ixcross(2)==1 ) then

!              --- One cross section above

                  if ( jpcell>=ny-1 ) then

!                 --- Above there is no obstacle

                     numpntobst = numpntobst+1
                     ipointobstlc(numpntobst+4) = inode
                     found = .true.

                  else if ( iobgrid(ipcell,jpcell+1)==-1 ) then

!                 --- cell above is in obstacle
!                     ipointobst(inode) := 0
!                     found := true

                     found = .true.

                  else if ( iobgrid(ipcell,jpcell+1)==0 ) then

!                 --- cell above is outside obstacle
!                     ipointobst(inode) := 1
!                     found := true

                     numpntobst = numpntobst+1
                     ipointobstlc(numpntobst+4) = inode
                     found = .true.

                  end if

               end if

               if ( .not. found .and. iycross(1)==0 ) then

!              --- no cross sections left of x = xp

                  if ( ipcell<=0 ) then

!                 --- At the left-hand side there is no obstacle

                     found = .true.

                  else if ( iobgrid(ipcell-1,jpcell)==-1 ) then

!                 --- cell at left-hand side is in obstacle
!                     ipointobst(inode) := iobst
!                     found := true

                     numpntobst = numpntobst+1
                     ipointobstlc(numpntobst+4) = inode
                     found = .true.

                  else if ( iobgrid(ipcell-1,jpcell)==0 ) then

!                 --- cell at left-hand side is outside obstacle
!                     ipointobst(inode) := 0
!                     found := true

                     found = .true.

                  end if

               else if ( .not. found .and.  iycross(1)==1 ) then

!              --- one cross section at the left-hand side

                  if ( ipcell==0 ) then

!                 --- At the left-hand side there is no obstacle

                     numpntobst = numpntobst+1
                     ipointobstlc(numpntobst+4) = inode
                     found = .true.

                  else if ( iobgrid(ipcell-1,jpcell)==-1 ) then

!                 --- cell at left-hand side is in obstacle
!                     ipointobst(inode) := 0
!                     found := true

                     found = .true.

                  else if ( iobgrid(ipcell-1,jpcell)==0 ) then

!                 --- cell at left-hand side is outside obstacle
!                     ipointobst(inode) := 1
!                     found := true

                     numpntobst = numpntobst+1
                     ipointobstlc(numpntobst+4) = inode
                     found = .true.

                  end if

               end if

               if ( .not. found .and. iycross(2)==0 ) then

!              --- no cross sections right of x = xp

                  if ( ipcell>=nx-1 ) then

!                 --- At the right-hand side there is no obstacle

                     found = .true.

                  else if ( iobgrid(ipcell+1,jpcell)==-1 ) then

!                 --- cell at right-hand side is in obstacle
!                     ipointobst(inode) := iobst
!                     found := true

                     numpntobst = numpntobst+1
                     ipointobstlc(numpntobst+4) = inode
                     found = .true.

                  else if ( iobgrid(ipcell+1,jpcell)==0 ) then

!                 --- cell at right-hand side is outside obstacle
!                     ipointobst(inode) := 0
!                     found := true

                     found = .true.

                  end if

               else if ( .not. found .and. iycross(2)==1 ) then

!              --- one cross section at right-hand side

                  if ( ipcell>=nx-1 ) then

!                 --- At the right-hand side there is no obstacle

                     numpntobst = numpntobst+1
                     ipointobstlc(numpntobst+4) = inode
                     found = .true.

                  else if ( iobgrid(ipcell+1,jpcell)==-1 ) then

!                 --- cell at right-hand side is in obstacle
!                     ipointobst(inode) := 0
!                     found := true

                     found = .true.

                  else if ( iobgrid(ipcell+1,jpcell)==0 ) then

!                 --- cell at right-hand side is outside obstacle
!                     ipointobst(inode) := 1
!                     found := true

                     numpntobst = numpntobst+1
                     ipointobstlc(numpntobst+4) = inode
                     found = .true.

                  end if

               end if

               if ( .not. found ) then

!              --- Complicated situation, type not yet detected
!                  use line y=yp to detect the number of cross sections at
!                  the left-hand side

                  nintersect = 0
                  do i = 1, nxi

                     if ( xi(i)<x ) then

!                    --- Cross section left of x

                        nintersect = nintersect+1
                        xi(nintersect) = xi(i)

                    end if

                  end do
                  do while ( .not. found )

!                 --- Find number of intersections in cell at left-hand side

                     ipcell = ipcell-1
                     call mshobstcross1 ( y, ipcell, jpcell, &
                                          rmeshx(1,iobst), nintersect, &
                                          iobgrid, nx, ny, curvescp, &
                                          nnodes, xi, iobgrid1 )

                     if ( ipcell<=0 ) then

!                    --- At the left-hand side there is no obstacle

                        if ( mod(nintersect,2)/=0 ) then

!                       --- odd number of cross sections

                           numpntobst = numpntobst+1
                           ipointobstlc(numpntobst+4) = inode

                        end if
                        found = .true.

                     else if ( iobgrid(ipcell-1,jpcell)==-1 ) then

!                    --- cell at left-hand side is in obstacle
!                        ipointobst(inode) := iobst
!                        found := true

                        if ( mod(nintersect,2)==0 ) then

!                       --- even number of cross sections

                           numpntobst = numpntobst+1
                           ipointobstlc(numpntobst+4) = inode

                        end if
                        found = .true.

                     else if ( iobgrid(ipcell-1,jpcell)==0 ) then

!                    --- cell at left-hand side is outside obstacle
!                        ipointobst(inode) := 0
!                        found := true

                        if ( mod(nintersect,2)/=0 ) then

!                       --- odd number of cross sections

                           numpntobst = numpntobst+1
                           ipointobstlc(numpntobst+4) = inode

                        end if
                        found = .true.

                     end if

                  end do

               end if

            end if

         end if

100   end do

!     --- Fill four positions of ipointobst
!         At this moment position 2 is set to zero, it is filled in subroutine
!         mshobstmarkel
!         In that subroutine points are moved from part 1 to part 2
!         Position 4 is also set to zero
!         In mshobstmarkbt we fill position 4 and part 4

      ipointobstlc(1) = numpntobst
      ipointobstlc(2) = 0
      ipointobstlc(3) = numbndobst
      ipointobstlc(4) = 0
      do i = 1, numbndobst
         ipointobstlc(numpntobst+4+i) = iwork(i)
      end do

      if ( .not. closed .and. numpntobst==0 .and. &
           numbndobst==0 .and. iobst==1 ) then

!     --- Open obstacle, carry out the special reduced version

         call sepobstmarkpt1 ( iobst, iwork )
      end if

1000  if ( debug ) then

!     --- Debug information

         call sepprinipobst ( ipointobst )
         write(irefwr,*) 'End sepobstmarkpt'

      end if
      call erclos ( 'sepobstmarkpt' )

      end
