      subroutine spectr ( iinput, rinput )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  1.12 date 29-05-2014 Extension of timebefore
!        version  1.11 date 25-11-2012 Long integers
!        version  1.10 date 15-11-2012 Long integers
!
!   copyright (c) 1999-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Transform an existing Sepran mesh into a mesh with spectral elements
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     transformation
!     spectral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduledebug
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*)
      double precision rinput(*)

!     iinput  i   Output array of subroutine MESH
!     rinput  i   Output array of subroutine MESH
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lncoor, lnkmsc, lnkmsm, lnkmsn, ipiinp, &
              ibrp, npointnw, lncurves
      integer, allocatable :: iwork(:), kmeshmnew(:), kmeshnnew(:)
      integer, allocatable :: kmeshcnew(:)
      double precision, allocatable :: coornew(:,:)
      double precision :: timebefore(2)

!     coornew        Array containing the new coordinates of the nodes
!     ibrp           Number of neighbor nodes expected
!     ipiinp         Pointer in input array to spectral part
!     iwork          integer work array with information of the curves
!     kmeshcnew      New kmesh part c
!     kmeshmnew      New array kmeshm
!     kmeshnnew      New array kmeshn
!     lncoor         declared length for coor, resp coorh
!     lncurves       Length of array CURVES
!     lnkmsc         Length of array KMESH part c
!     lnkmsm         Length of array KMESH part m
!     lnkmsn         Length of array KMESH part n
!     npointnw       Estimate of new number of nodes in mesh
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLMN         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     ERSETTIME      Get the CPU time at start of subroutine
!     MSH414         Determine whether curves are filled in
!     MSHSPADAPCOOR  Adapt the coordinates of midside points of triangles in
!                    case the boundary is curved and spectral points are placed
!     MSHSPC         Place all extra spectral nodes
!     MSHSPECREORDER Reorder elements of shape 5 and 13 in case of spectral
!                    elements without fill-in
!     MSHSPECREORKN  Reorder elements of shape 5 in kmesh part n in case of
!                    spectral elements without fill-in
!     SEPFILLKMESHC  Make relation between kmeshc and Topgr
!     SEPPOINTKMESHC Fill array pointkmsc with starting addresses of element
!                    groups in kmeshc (minus 1)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     1277   Array IINPUT has not been filled correctly
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Do some general checks
!     Determine whether there are surfaces and/or volumes
!     Determine the coordinates     ( kmesh part i )
!     Determine the elements        ( kmesh part c )
!     Determine the curves          ( kmesh part m )
!     Determine the surfaces        ( kmesh part n )
!     Determine startpositions for new arrays i,c,m,n
!     Make guess of number of neighbors
!     Make reservation for neighbor arrays
!     Set controle integers in BUFFER array
!     Set pointers of old arrays
!     Fill array iwork with information of the curves
!     Place the extra spectral nodes
!     Check control integers
!     Refill the parts of KMESH
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'spectr' )
      if ( itime>0 ) call ersettime ( timebefore )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from spectr'

      end if
      if ( ierror/=0 ) go to 1000

      if ( idebug>0 ) then
         ifirws = 0
      end if

!     --- Check whether array IINPUT has to be checked and if so check

      if ( kelmh/=0 ) then

         if ( iinput(1)<1 .or. iinput(1)>3  ) &
              call errsub ( 1277, 0, 0, 0 )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Check array iinput

      if ( iinput(14)==0 ) goto 1000

!     --- User has given command INTERMEDIATE points.
!         Determine the values of nodes on the edges

      ipiinp = iinput(14) - 4  ! actual value is 5 larger

      niedge = iinput( ipiinp     )
      nisurf = iinput( ipiinp + 1 )
      nivolm = iinput( ipiinp + 2 )
      if ( debug ) &
         write(irefwr,*) 'niedge,nisurf,nivolm ', niedge, nisurf, nivolm

!     --- Check whether points have to be added

      if ( niedge<=0 .and. nisurf<=0 .and. nivolm<=0 ) then

!     --- No points have to be added, reorganize kmeshc for elements of shape 5
!         and 13

         call mshspecreorder ( kmeshcnew )
         if ( kelmn/=0 ) then

!        --- kelmn/=0, i.e. kmesh part n has been filled
!            reorder

            call mshspecreorkn

         end if  ! ( kelmn/=0 )

      else

!     --- Points have to be added
!         To place points coordinates and elements are needed
!        --- Determine the elements ( kmesh part c )

         lnkmsc = size ( kmeshc )

!        --- Determine the curves

         if ( kelmm/=0 ) then

            lnkmsm = size ( kmeshm )

         else

            lnkmsm = 0

         end if

!        --- Determine the surfaces and the length of the arrays

         if ( kelmn/=0 ) then

            lnkmsn = size (kmeshn) - 2 * nsurfs

         else

            lnkmsn   = 0

         end if

!        --- Determine part h
!            Determine start positions for new arrays
!            Reserve space for new arrays:
!            array coornew     length =  8 or 24 * npoint real positions
!                                                  positions
!            array kmeshmnew   length =  2 * length kmeshm integer positions
!            array kmeshnnew   length =  4 * length kmeshn integer positions
!            array iwork       length =  ncurvs integer positions
!            array curves      length = (2_niedge)*old length

         npointnw = (2+niedge)*(2+niedge)*npoint
         lncoor = npointnw*ndim
         if ( ndim==3 ) lncoor = ( 2+niedge ) * lncoor

         lnkmsc = ( 1+niedge ) * ( 1+niedge ) * lnkmsc

         if ( ndim==3 ) lnkmsc = ( 1+niedge ) * lnkmsc

         lnkmsm = ( 2+niedge ) * lnkmsm

         if ( kelmn/=0 ) then

            lnkmsn = ( 1+niedge ) * ( 1+niedge ) * lnkmsn + 2*nsurfs

         end if

         if ( ndim==2 ) then
              ibrp = (2+niedge)*(2+niedge)*6
         else
              ibrp = 40
         end if
         lncurves = size(curvesmsh)*(2+niedge)

         allocate ( iwork(ncurvs), &
                    kmeshmnew(lnkmsm), kmeshnnew(lnkmsn), &
                    kmeshcnew(lnkmsc), coornew(ndim,lncoor/ndim), stat = error )
         if ( error/=0 ) &
            call eralloc ( error, ncurvs+(npoint+1)*ibrp+lnkmsm+lnkmsn, &
                          'iwork' )
         if ( ierror/=0 ) go to 1000

!        --- Fill array iwork with information of the curves
!            when curve i has been filled then iwork(i) = 0 else iwork(i) = 1

         if ( lnkmsm>0 ) call msh414 ( kmeshm, ncurvs, iwork )

         if ( debug ) write(irefwr,*) 'before mshspc'
         call mshspc ( coornew, kmeshcnew, lncoor, lnkmsc, &
                       kmeshmnew, lnkmsm, iwork, &
                       kmeshnnew, lnkmsn, ibrp, &
                       iinput(ipiinp), iinput, rinput(3), lncurves )

!        --- Adapt internal points to curved boundary in case of 2-dimensional
!            triangles

         if ( debug ) write(irefwr,*) 'before mshspadapcoor'
         call mshspadapcoor ( kmeshcnew, kmeshmnew, &
                              coornew, iinput(ipiinp))

!        --- Refill the parts of KMESH

         if ( debug ) write(irefwr,*) 'before deallocate'
         deallocate ( km(iseqkm)%coor, km(iseqkm)%kmeshc )
         allocate ( km(iseqkm)%coor(ndim,lncoor/ndim),  km(iseqkm)%kmeshc(lnkmsc) )
         coor => km(iseqkm)%coor
         kmeshc => km(iseqkm)%kmeshc
         coor = coornew(:,1:lncoor/ndim)
         kelmi = lncoor
         kmeshc = kmeshcnew(1:lnkmsc)
         call sepfillkmeshc   ! Make relation between kmeshc and Topgrp
         call seppointkmeshc  ! Fill array pointkmsc

         if ( kelmm/=0 ) then
            deallocate ( km(iseqkm)%kmeshm )
            allocate ( km(iseqkm)%kmeshm(lnkmsm) )
            kmeshm => km(iseqkm)%kmeshm
            kmeshm = kmeshmnew(1:lnkmsm)
            km(iseqkm)%kmesh(27) = lnkmsm
         end if  ! ( kelmm/=0 )
         if ( kelmn/=0 ) then
            deallocate ( km(iseqkm)%kmeshn )
            allocate ( km(iseqkm)%kmeshn(lnkmsn) )
            kmeshn => km(iseqkm)%kmeshn
            kmeshn = kmeshnnew(1:lnkmsn)
            km(iseqkm)%kmesh(28) = lnkmsn
        end if  ! ( kelmn/=0 )

      end if

      deallocate ( iwork, kmeshcnew, kmeshmnew, kmeshnnew, &
                   coornew, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End spectr'

      end if
      call erclmn ( 'spectr', timebefore )
      end subroutine spectr
