      subroutine msh048 ( inpada, iinput, rinput, &
                          curstr, lengcv, icurvar, ispinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version 15.1  date 11-12-2014 Adaptation due to new curve routines
!        version 15.0  date 02-09-2010 Remove intarmsh
!        version 14.0  date 07-11-2009 Remove some parameters
!
!   copyright (c) 1986-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check which user points are part of the curves stored in icurvar
!     and hence have new co-ordinates
!     This subroutine is called by subroutine chanbn, after the call of
!     msh047.  In msh047 the co-ordinates of the curves in icurvar
!     are changed, in this subroutine it is checked which user points
!     are part of the curves in icurvar and hence have new co-ordinates.
!     The co-ordinates of all curves containing these user points (except
!     the curves in icurvar ) are changed also.
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     mesh
!     mesh_generation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iinput(*), icurvar(*), inpada(*), &
                 ispinp(*), lengcv
      double precision rinput(*), curstr(*)

!     curstr         i    array of length 10 * ncurvs to store some
!                         geometrical information of the curves
!     icurvar             integer work array of length ncurvs
!                         icurvar(i) = 1 means curve i is in the boundary
!                         icurvar(i) = 0 means curve i is not in the boundary
!     iinput         i    integer input array; result of a preceding call of
!                         subroutine mesh
!     inpada         i    Integer array to be filled by the user, indicating how
!                         the boundary must be adapted.
!                         INPADA is only used if iread = -1
!                         INPADA must be filled as follows:
!                         Pos. 1   Position of last entry that has been filled
!                                  by the user. If inpada(1) = 0 or 1, only
!                                  defaults are used.
!                         Pos. 2   ncurves, indicates the number of curves
!                                  in array icurves that must be used for the
!                                  adaptation of the mesh
!                                  Default value: 1
!                         Pos. 3   Not yet used. Must be set equal to 0
!                                  Default value: 0
!                         Pos. 4   iadapt Indicates how the boundary must be
!                                  adapted
!                                  Possible values:
!                                  1.  The curves are adapted by:
!                                      xnew(i) = xold(i) + alpha(i) n(i)
!                                      with xold(i) the old co-ordinates of the
!                                      i-th node and xnew(i) the new ones.
!                                      n(i) is the normal at the boundary of the
!                                      i-th node and alpha(i) the number-th
!                                      degree of freedom in this node
!                                      The normal is computed assuming a counter
!                                      clockwise direction of the boundary
!                                  2.  The curves are adapted by the user
!                                      provided function subroutine FUNCCR
!                                  3.  See 1, however, now alpha is computed
!                                      according to the 'film method'
!                                      See: Caswell and Viriyayuthakorn, &
!                                      Finite element simulation of die swell
!                                      for a Maxwell fluid, J. of Non-Newt.
!                                      Fluid Mech. Vol 12, 13-29(1983).
!                                      alpha(i) = factor*(psi(i)-psi(0))/|vt(i)|
!                                                       xi
!                                      psi(i)-psi(0) = /  v.n dx
!                                                     x0
!                                      The vector isol must be the velocity
!                                      vector with degrees of freedom number
!                                      and number+1 from isol.
!                                      Cartesian coordinates.
!                                      Starting flow rate integration along free
!                                      boundary with flow rate zero in first
!                                      point of curves.
!                                  4.  Other method according to:
!                                      xnew(i) = xold(i) + v(i) * factor
!                                      Time-dependent case if imult=1
!                                      see inpada(11)
!                                      In the axisymmetric case we use:
!                                      (r,z)new(i) = (r,z)old(i) + &
!                                                     (vr,vz)(i) dt /(2 pi r(i))
!                                  5.  See 1. Now alpha is computed according to
!                                      alpha=v.n * factor
!                                  Default value: 1
!                         Pos. 5   idirec Indicates the direction of the
!                                  computation
!                                  If idirec = 1 the computation is performed
!                                  from first to last point, &
!                                  if idirec = -1 from last to first point
!                                  It is sometimes required to alternate between
!                                  1 and -1 during iteration.
!                                  This option is only used for iadapt = 3
!                                  Default value: 1
!                         Pos. 6   number, the number-th degree of freedom of
!                                  isol (and higher) are used for the
!                                  computation of alpha. See iadapt.
!                                  Default value: 1
!                         Pos. 7   linear. Indicates if the elements along the
!                                  boundary must be treated as linear or as
!                                  quadratic elements.
!                                  Possible values:
!                                  1: linear
!                                  2: quadratic
!                                  Default value: to be computed by the
!                                  subroutine in the following way:
!                                  If all internal elements are quadratic, the
!                                  boundary elements are assumed quadratic, &
!                                  otherwise linear
!                         Pos. 8   jcart defines the type of co-ordinate system
!                                  to be used. This is of importance for
!                                  iadapt = 3, 4 and 5
!                                  Possible values:
!                                  1:  Cartesian co-ordinates
!                                  2:  Axi-symmetric co-ordinates
!                                  Default value: 1
!                         Pos. 9   iexlude_1: indicates which of the
!                                  co-ordinates of the first point of the first
!                                  curve must be adapted.
!                                  Possible values:
!                                  0  Both co-ordinates must be adapted
!                                  1  Only the second co-ordinate must be
!                                     adapted, the first one remains unchanged
!                                  2  Only the first co-ordinate must be adapted
!                                     the second one remains unchanged
!                                  3  Both co-ordinates remain unchanged
!                                  Default value: 0
!                         Pos. 10  iexlude_2: indicates which of the coordinates
!                                  of the last point of the last curve must be
!                                  adapted.
!                                  Possible values:
!                                  0  Both co-ordinates must be adapted
!                                  1  Only the second co-ordinate must be
!                                     adapted, the first one remains unchanged
!                                  2  Only the first co-ordinate must be
!                                     adapted, the second one remains unchanged
!                                  3  Both co-ordinates remain unchanged
!                                  Default value: 0
!                         Pos. 11  imult: indicates how the value of factor must
!                                  be computed. (iadapt>2)
!                                  Possible values:
!                                  0  Multiplication factor is 1
!                                  1  Multiplication factor is dt from common
!                                     ctime
!                                  2  Multiplication factor is stored in
!                                     rinada(1)
!                                  3  Multiplication factor is dt * rinada(1)
!                                  Default value: 0
!                         Pos. 12  ithres: indicates if a threshold value is
!                                  used in Caswell's method and where to find it
!                                  Possible values:
!                                  0  Threshold value = 0
!                                  1  Threshold value is stored in rinada(2)
!                                  Default value: 0
!                         Pos. 13  iangle_1: Indicates if angle of freesurface
!                                  at first point is prescribed or not
!                                  Possible values:
!                                  0  angle is not given
!                                  1  angle is stored in rinada(3)
!                                  Default value: 0
!                         Pos. 14  iangle_2: Indicates if angle of freesurface
!                                  at last point is prescribed or not
!                                  Possible values:
!                                  0  angle is not given
!                                  1  angle is stored in rinada(4)
!                                  Default value: 0
!                         Pos. 15  redistribute nodes
!                                  Consists of two parts
!                                  iredist_type and ncurves_redist according to:
!                                  iredist = iredist_type + 100*ncurves_redist
!                                  Possible values:
!                                  iredist_type:
!                                  0  no node redistribution
!                                  1  nodes are redistributed, nelm constant
!                                  2  nodes are redistributed, nelm recomputed
!                                     according to coarseness
!                                  ncurves_redist: number of curves that
!                                  must be redistributed
!                                  The curve numbers are stored in icurvs
!                                  positions ncurves+1 to
!                                  ncurves+ncurves_redist
!                                  This option is only available in combination
!                                  with iredist = 1
!                                  Default value: 0
!                         Pos. 16  Plot_boun
!                                  Possible values:
!                                  0  do not plot new boundary
!                                  1  plot new boundary
!                                  Default value: 0
!     ispinp              Work array to be used by subroutine mshcursplin as
!                         input array iinput
!                         Length n+2, where n is the number of nodes along
!                         the curve
!     lengcv         i    Overestimate of the number of points along the curves
!     rinput         i    real input array; result of subroutine msh047
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iposc, irnp, iinp, icurve, j, jinp, nelm, &
              iredist, istartrel, n
      logical chancurv
      integer, allocatable :: irelcurves(:), ihelp(:), iusrpn(:)
      double precision, allocatable :: tau(:)

!     chancurv       If true at least one of the user points of the curve
!                    has new co-ordinates
!     i              Counting variable
!     icurve         Type of curve
!     ihelp               integer work array of length ncurvs
!     iinp           last position used in array iinput
!     iposc          last position used in array curves
!     iredist        Defines type of redistribution of nodes
!                    Possible values:
!                    0: no redistribution
!                    1: redistribution of nodes, the number of nodes is kept
!                    2: redistribution of nodes, the number of nodes may be
!                       changed
!     irelcurves     dummy
!     irnp           position in array rinput
!     istartrel      dummy
!     iusrpn         work array of length nuspnt in which it is indicated
!                    whether a user point is changed (1) or not (0)
!     j              Counting variable
!     jinp           Copy of iinp
!     n              number of points along curve
!     nelm           number of elements along a curve
!     tau            work array of length at most length curves / 2 to
!                    store relative arc lengths along curves
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSH046         Raise iinp as function of icurve
!     MSHCURADAPEND  Change coordinates of end points of curve by the
!                    coordinates as stored in array rinput
!     MSHCURCR       Create curve
!     MSHCURSPLIN    Redistribute the coordinates along a curve without changing
!                    the number of nodes along that curve
!     PRINCURV       Print coordinates of curves
!     PRININ         print 1d integer array
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3038   Array curstr has not been filled
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- find the user points in all curves

      call eropen ( 'msh048' )
      debug = .false.

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information in msh048'

      end if

      istartrel = 2*ncurvs+1

      allocate ( irelcurves(10*ncurvs), ihelp(ncurvs), iusrpn(nuspnt), &
                 tau(10*ncurvs), stat = error )
      if ( error/=0 ) call eralloc ( error, 10*ncurvs+nuspnt, 'irelcurves' )
      if ( ierror/=0 ) go to 1000

      irelcurves = 0

      if ( debug ) then
         call princurv ( 0, icurvsmsh, curvesmsh, ncurvs, ndim, curveusrp, &
                         'curves at start msh048' )
      end if

!     --- find the user points in the boundary and store in array iusrpn
!         in the following way:
!         iusrpn(i) = 0:  user point i not at the boundary
!         iusrpn(i) = 1:  user point i at the boundary
!         reset iusrpn

      iusrpn = 0

      do icurnr = 1, ncurvs

         if ( icurvar(icurnr)==1 ) then
            do j = curveusrp(icurnr)+1, curveusrp(icurnr+1)
               iusrpn(abs(curveusrp(j))) = 1
            end do
         end if

      end do

      if ( debug ) then

!     --- Debug information

         call prinin ( icurvar, ncurvs, 'icurvar' )
         call prinin ( iusrpn, nuspnt, 'iusrpn' )

      end if

!     --- end of filling of iusrpn
!         check which curves outside icurves contain user points
!         from iusrpn
!         change the co-ordinates of these curves

      jcoars = iinput(11)
      iposc = 1
      irnp = (ndim+jcoars)*nuspnt+jcoars
      iinp = 16
      irnp1 = 1
      irnp2 = (ndim+jcoars)*nuspnt+2*jcoars+irnp1
      irnp3 = irnp2+3*ncurvs-1+4*nsurfs
      iredist = mod(inpada(15),100)

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'iredist, ncurvs ',iredist, ncurvs
         call prinin ( icurvsmsh, ncurvs, 'icurvs-before' )
         call prinrl1 ( curvesmsh, icurvsmsh(ncurvs), 2, 'curves-before' )

      end if

      do icurnr = 1, ncurvs
         ihelp(icurnr) = iinp
         icurve = abs(iinput(iinp))
         if ( debug ) write(irefwr,*) 'icurnr, icurve ', icurnr, icurve
         if ( icurve==0 ) then

!        --- Empty curve found: skip

            n = 0
            go to 500

         end if

!        --- curves that are used to build spline curves must be skipped

         if ( icurvsmsh(ncurvs+icurnr)==2 ) go to 400
         chancurv = .false.
         if ( icurvar(icurnr)==0 ) then

!        --- curve icurnr is outside the boundary, check user points

            do j = curveusrp(icurnr)+1, curveusrp(icurnr+1)

               if ( iusrpn(abs(curveusrp(j)))/=0 ) then
                  chancurv = .true.
                  go to 300
               end if

            end do

         else

!        --- curve is changed, set kmeshz

            if ( kmeshz(icurnr)==-1 ) then
               kmeshz(icurnr) = 2
            else if ( kmeshz(icurnr)==0 ) then
               kmeshz(icurnr) = 1
            end if

         end if

300      if ( icurve/=7 .and. chancurv ) then

!        --- at least one of the user points is changed, recompute the curve

            if ( debug ) write(irefwr,*) 'chancurv ', chancurv
            if ( icurve/=7 .and. kmeshz(icurnr)==0 ) &
               kmeshz(icurnr) = 1

            if ( iredist==2 ) then

!           --- mark free boundary

               iinput(iinp) = -icurve

!              --- move the boundary points connected with user points

               ishape   = iinput(iinp+1)
               nelm     = iinput(iinp+2)
               n = nelm*ishape+1

               if ( icurve==8 .or. icurve==9 ) then

!              --- Parameter curve, we can not recreate the curve
!                  The curve is approximated by a spline and
!                  then to recompute the spline with the new end points
!                  The end points must be moved along the spline or projected
!                  onto it

                  call mshcuradapend ( curvesmsh(1,iposc), iinput(iinp), &
                                       ndim, rinput, n )
                  call mshcursplin ( iinput, iinp, irnp, icurnr, rinput, &
                                     icurve, ndim, nuspnt, jcoars, &
                                     curveusrp, tau, ispinp, iposc, &
                                     curvesmsh, 0.5d0 )

               else

!              --- since the end points are moved, the curve is recomputed

                  jinp = iinp
                  if ( icurve==9 .or. icurve==16 .or. icurve==23 ) then

!                 --- Array curstr has not been filled

                     call errint ( icurve, 1 )
                     call errsub ( 3038, 1, 0, 0 )
                     go to 1000

                  end if  ! ( icurve==9 .or. icurve==16 .or. icurve==23 )

                  call mshcurcr ( iinput, rinput, curstr,icurve, jinp, &
                                  irnp, iposc, icurnr, n, jcoars, &
                                  ihelp, irelcurves, irnp1, istartrel )
               end if

            else if ( icurve==10 .or. icurve==11 ) then

!           --- translate or rotate, curve is approximated by a spline

               ishape   = iinput(iinp+1)
               nelm     = iinput(iinp+2)
               n = nelm*ishape+1
               call mshcuradapend ( curvesmsh(1,iposc), iinput(iinp), &
                                    ndim, rinput, n )
               call mshcursplin ( iinput, iinp, irnp, icurnr, rinput, &
                                  icurve, ndim, nuspnt, jcoars, &
                                  curveusrp, tau, ispinp, iposc, &
                                  curvesmsh, 0.5d0 )

            else

!           --- standard curve

               jinp = iinp
               call mshcurcr ( iinput, rinput, curstr, icurve, jinp, &
                               irnp, iposc, icurnr, n, jcoars, &
                               ihelp, irelcurves, irnp1, istartrel )
            end if

         end if

400      n = kmeshm(5+icurnr)-kmeshm(4+icurnr)
500      call msh046 ( iinput, iinp, icurve )
         iposc = iposc+n

!        --- end changing of curves

      end do

      deallocate ( irelcurves, ihelp, iusrpn, tau, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'irelcurves' )

1000  if ( debug ) then

!     --- Debug information

         call princurv ( 0, icurvsmsh, curvesmsh, ncurvs, ndim, curveusrp, &
                         'curves at end msh048' )

      end if

      call erclos ( 'msh048' )

      end
