      subroutine msh302 ( coor, i1, i2, i3, i4, n, m )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 27-11-2012 Long integers
!        version  2.0  date 27-08-2010 Make tau local
!        version  1.0  date 03-12-1984
!
!   copyright (c) 1984-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     determine coordinates of the internal points of a surface
!
! **********************************************************************
!
!                       KEYWORDS
!
!     surface
!     mesh
!     coordinate
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer i1(*), i2(*), i3(*), i4(*), m, n
      double precision coor(*)

!     coor          i/o   array containig the 3*nx*ny*nz coordinates
!                         of the nodal points
!                         at input only the coordinates of the points on the
!                         four sides must have been stored in coor.
!                         at output the coordinates of the internal points of
!                         the surface are filled in in coor.
!     i1             i    the n nodal point numbers on side 1-2
!     i2             i    the m nodal point numbers on side 1-4
!     i3             i    the m nodal point numbers on side 2-3
!     i4             i    the n nodal point numbers on side 4-3
!     m              i    number of points along 1-2 and 4-3
!     n              i    number of points along 1-4 and 2-3
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ih, ihelp, ii, iunder, iupper, j, m1, n1
      double precision tau(2*(m+n)), sigma1, sigma2, sigmax, tau1, &
                       tau2, taux, x, x1j, xi1, xim, xnj, &
                       y, y1j, yi1, yim, ynj, &
                       z, z1j, zi1, zim, znj

!     i              Counting variable
!     ih             Help variable to store a constant temporarily
!     ihelp          ?
!     ii             Counting variable
!     iunder         ?
!     iupper         ?
!     j              Counting variable
!     m1             ?
!     n1             ?
!     sigma1         ?
!     sigma2         ?
!     sigmax         ?
!     tau            Work array
!     tau1           ?
!     tau2           ?
!     taux           ?
!     x              x co-ordinate of point
!     x1j            ?
!     xi1            ?
!     xim            ?
!     xnj            ?
!     y              y co-ordinate of point
!     y1j            ?
!     yi1            ?
!     yim            ?
!     ynj            ?
!     z              z co-ordinate of point
!     z1j            ?
!     zi1            ?
!     zim            ?
!     znj            ?
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSH305         Determine tau values along line
!     MSH306         Determine coordinate in surface using the first part of
!                    coon's formula
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!                                 i1
!     1 - 2 - 3 - 4          1 *-------* 2
!                              r       r
!                           i2 r       r i3
!                              r       r
!                            4 *-------* 3
!                                 i4
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh302' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh302'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- determine values of " tau "
!         side 1-2

      call msh305(coor,i1,n,tau(1))

!     --- side 1-4

      ih=n+1
      call msh305(coor,i2,m,tau(ih))

!     --- side 2-3

      ih=n+m+1
      call msh305(coor,i3,m,tau(ih))

!     --- side 4-3

      ih=n+2*m+1
      call msh305(coor,i4,n,tau(ih))

!     --- determine sigma and tau of all internal nodes
!         start with coordinates of the four corners

      n1=n-1
      m1=m-1
      do i=2,n1
         do j=2,m1
          tau1=tau(i)
          sigma1=tau(n+j)
          sigma2=tau(n+m+j)
          tau2=tau(n+2*m+i)
          sigmax=tau1*(sigma2-sigma1)+sigma1
          taux=1d0+(tau1-tau2)*(sigma2-sigma1)
          sigmax=sigmax/taux
          taux=sigmax*tau2+(1d0-sigmax)*tau1

!         --- determine nodal point number

          iupper=i1(i)
          iunder=i4(i)
          ihelp=(iupper-iunder)/(m1)
          ii=iupper-j*ihelp+ihelp

!         --- coordinates

          x1j=coor(3*i2(j)-2)
          y1j=coor(3*i2(j)-1)
          z1j=coor(3*i2(j))
          xnj=coor(3*i3(j)-2)
          ynj=coor(3*i3(j)-1)
          znj=coor(3*i3(j))
          xi1=coor(3*i1(i)-2)
          yi1=coor(3*i1(i)-1)
          zi1=coor(3*i1(i))
          xim=coor(3*i4(i)-2)
          yim=coor(3*i4(i)-1)
          zim=coor(3*i4(i))

!         --- formula for the computation

          call msh306(x,x1j,xnj,xi1,xim, taux,sigmax )
          call msh306(y,y1j,ynj,yi1,yim, taux,sigmax )
          call msh306(z,z1j,znj,zi1,zim, taux,sigmax )

!         --- fill coor

          coor(3*ii-2)=x
          coor(3*ii-1)=y
          coor(3*ii)=z

         end do !  j=2,m1

      end do !  i=2,n1

1000  call erclos ( 'msh302' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh302'

      end if  ! ( debug )

      end
