      subroutine msh350 ( m, n, npobot, bcord1, bcord2, bcord3, &
                          coor, linear, trian, kmscbt, nelbot, kbndbt, &
                          npoint, found, klp, nlp, channel, inpelmbot )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  8.1  date 22-05-2014 Extension with shape 22
!        version  8.0  date 24-08-2010 Removal of iwork
!        version  7.0  date 10-12-2002 extra parameter beacause of prisms
!
!   copyright (c) 1987-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill co-ordinates in array coor for mesh generator PIPE
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     volume
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical trian, channel
      integer kmscbt(*), kbndbt(*), &
              m, n, npobot, nelbot, npoint, linear, found, &
              klp(2,*), nlp, inpelmbot
      double precision coor(*),bcord1(3,*),bcord2(3,*),bcord3(*)

!     bcord1         i    Co-ordinates of nodal points in bottom surface
!     bcord2         i    Co-ordinates of nodal points in top surface
!     bcord3         i    Co-ordinates of nodal points in envelopping pipe
!                         surface
!     channel        i    If true the pipe is treated like a channel
!                         This means that the upper and lower face must be
!                         topological equivalent and that all other points
!                         are placed on straight lines from upper to lower face.
!                         The faces itself may be completely curved
!     coor           o    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to
!                         the global numbering
!     found          i    Indicates wether wedges can be split into 3
!                         tetrahedrons or if conventional split into 8
!                         tetrahedrons has to be done
!     ibuurp              Help array for neighbours
!     inpelmbot      i    Number of nodes per element of bottom surface and
!                         top surface of pipe
!     iwork               Help array for start positions neighbour points
!     kbndbt         i    Array KBNDPT for bottom surface
!                         Contains the local nodal point numbers of the
!                         boundary of the bottom surface
!     klp            i    In this array the relative node numbers with respect
!                         to array kbndbt of first and last point of a
!                         consecutive part are stored.
!                         The number of parts the boundary consists of is nlp
!                         Hence klp(1,1) = 1, klp(2,1) denotes the end point of
!                         the first part of the boundary
!                         klp(1,2) contains the node number of the first point
!                         of the second part. This must be klp(2,1)+1
!                         klp(2,nlp) contains the length of kbndbt
!     kmscbt         i    Kmesh part c corresponding to bottom surface
!                         Contains the local nodal point numbers of the bottom
!                         surface
!     linear         i    Indicates whether the element is a linear simplex
!                         (true) or not (false)
!     m              i    Number of elements in boundary of bottom surface
!     n              i    Number of line elements in vertical direction
!     nelbot         i    Number of elements in bottom surface
!     nlp            i    Number of disjoint parts the boundary of the bottom
!                         surface consists of
!     npobot         i    Number of points of bottom surface
!     npoint         o    Number of nodal points
!     trian          i    Indication whether triangles (true) or quadrilaterals
!                         are created (false)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ncorpl, npocrs, m2, istep3, nstep, ipcoor, ipcros, lengnb, &
              ih, ih1, ih2, ih3, i, j, k, iplane, ikelm, ielem, &
              iknpt, ihk, ihk1, imax(3), ii, ilp, n2, iwork(npobot), &
              ibuurp(9*npobot)
      double precision dx(3), plane(4), dz, alpha, eps, ds, &
                       xsieta(2,npobot,2)
      logical :: repos, first

!     alpha          multiplication factor to be used between under surface
!                    and upper surface
!                    The ksi-eta co-ordinates in a coross section are computed
!                    by xsieta = (1-alpha)*xsieta_bot _ alpha*xsieta_top
!     ds             relative length of vector along the boundary
!     dx             length of vector in x-direction
!     dz             length of vector in z-direction
!     eps            Smallest distance between two boundary points at the pipe
!                    surface
!                    eps/3 indicates the accuracy for which it is decided if
!                    repositioning should be applied.
!                    If || x_boundary - x_boundary(computed) ||>eps
!                    repositioning must be applied
!     first          If true array ibuurp has not been filled yet
!     i              Counting variable
!     ielem          Element number
!     ih             Help variable to store a constant temporarily
!     ih1            Help variable to store a constant temporarily
!     ih2            Help variable to store a constant temporarily
!     ih3            Help variable to store a constant temporarily
!     ihk            Help variable to store a constant temporarily
!     ihk1           Help variable to store a constant temporarily
!     ii             Counting variable
!     ikelm
!     iknpt          Nodal point number
!     ilp            Counting variable
!     imax           In this array the indices with respect to boundary points
!                    in the surfaces are stored
!                    imax(1) stores the node number of the first point
!                    (usually 1)
!                    imax(2) stores the node number with respect to the xsi dir.
!                            The xsi direction is defined as P2-P1, where P2 is
!                            given by bcord(kbndbt(imax(2))
!                    imax(3) stores the node number with respect to the first
!                            eta dir.
!                            The eta direction is defined as P3-P1, where P3 is
!                            given by bcord(kbndbt(imax(3))
!     ipcoor         starting address of co-ordinates array, see array ipoint
!     ipcros         starting address of cross section in array bcord3
!     iplane         Sequence number of cross-section
!     istep3         number of co-ordinates in the circumference in a
!                    cross-section
!                    including points in section between two vertex planes
!     j              Counting variable
!     k              Counting variable
!     lengnb         length of neighbour array
!     m2             number of points in circumference in each cross-section
!     n2             number of points in "pipe direction"
!     ncorpl         number of co-ordinates in a cross-section
!     npocrs         number of co-ordinates in a block cross-section, i.e.
!                    number of co-ordinates from one cross-section to the other
!     nstep          number of co-ordinates from even plane to odd plane
!                    (quadratic elements)
!     plane          Array of length 4 containing the parameters a, b, c and d
!                    describing the plane: ax=by+cz+d = 0
!                    plane(1) = a; plane(2) = b; plane(3) = c; plane(4) = d;
!     repos          If true, repositioning must be applied
!     xsieta         Work array to store the ksi-eta co-ordinates of the
!                    transformation. length: 2*npoint
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     DCOPY     Copy double precision dx to double precision dy
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     MSH356    Fill array ibuurp with neighbours of nodes
!     MSH357    Perform repositioning
!     MSHDS1    Compute the average distance between two surfaces
!     MSHDS2    Compute the factor alpha defining the average distance between
!               cross section and under surface in the points 1 and imax at the
!               boundary,divided by the distance between upper and under surface
!     MSHPLN    Approximate a set of co-ordinates by a plane
!     MSHPR1    Project co-ordinates stored in bcord onto plane
!     MSHPRO    Project co-ordinates stored in coor onto plane
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Approximate under surface by plane
!    Project under surface onto this plane
!    Define axis into this plane (ksi-eta axis)
!    Compute ksi-eta co-ordinates of under surface
!    Approximate upper surface by plane
!    Project upper surface onto this plane
!    Define axis into this plane (ksi-eta axis)
!    Compute ksi-eta co-ordinates of upper surface
!    Copy under surface into array coor
!    For each cross-section
!      Approximate surface by a plane
!      Project boundary onto this plane
!      Define axis into this plane (ksi-eta axis)
!      Compute relative distance of cross section to upper and under plane
!      Compute ksi-eta co-ordinates of cross-section by weigthing between
!      upper and under plane
!      Map cross section into array coor
!    Copy upper surface into array coor
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh350' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh350'
         write(irefwr,*) 'channel, trian', channel, trian
         write(irefwr,1) 'npobot, linear, found', npobot, linear, found
         write(irefwr,1) 'm, n', m, n
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Compute some constants

      ncorpl = 3 * npobot
      npocrs = ncorpl * linear

      if ( trian .and. found==0 ) &
         npocrs = npocrs + 3*nelbot*(1+6*(linear-1))

      m2     = m * linear + 1
      istep3 = 3 * m2 * linear
      nstep  = ncorpl
      if ( trian ) nstep = nstep + 9 * nelbot
      first = .true.

      if ( debug ) &
         write(irefwr,1) 'ncorpl, npocrs, m2, nstep', &
                          ncorpl, npocrs, m2, nstep

      if ( .not. channel ) then

!     --- Approximate under surface by plane

         call mshpln ( bcord1, npobot, plane )

!        --- Project under surface onto this plane
!            Define axis into this plane (ksi-eta axis)
!            Compute ksi-eta co-ordinates of under surface

         imax(1) = 1
         imax(2) = 0
         imax(3) = 0
         call mshpro ( bcord1, npobot, plane, xsieta(1,1,1), imax, &
                       kbndbt, klp, nlp )

!        --- Approximate upper surface by plane

         call mshpln ( bcord2, npobot, plane )

!        --- Project upper surface onto this plane
!            Define axis into this plane (ksi-eta axis)
!            Compute ksi-eta co-ordinates of upper surface

         call mshpro ( bcord2, npobot, plane, xsieta(1,1,2), imax, &
                       kbndbt, klp, nlp )

      end if

!     --- Copy under surface into array coor

      call dcopy ( ncorpl, bcord1, 1, coor, 1 )

      if ( channel ) then

!     --- Channel

         dz = sqrt ( (bcord2(1,1)-bcord1(1,1))**2 + &
                     (bcord2(2,1)-bcord1(2,1))**2 + &
                     (bcord2(3,1)-bcord1(3,1))**2 )
         n2 = n*linear+1
         ipcoor = 0
         do iplane = 2, n2-1

            i = (iplane-1)*m2*3
            ds = sqrt((bcord3(1+i)-bcord3(1))**2+ &
                      (bcord3(2+i)-bcord3(2))**2+ &
                      (bcord3(3+i)-bcord3(3))**2)/dz
            ipcoor = ipcoor+3*npobot

            if ( trian .and. found==0 ) then

!           --- There are extra points between the surfaces, raise ipcoor

               if ( linear==1 ) then

!              --- linear triangles, nelbot extra points

                  ipcoor = ipcoor+3*nelbot

               else if ( mod(iplane,2)==0 ) then

!              --- Even plane in quadratic triangles, hence mid plane
!                  Below are 3*nelbot extra points

                  ipcoor = ipcoor+3*nelbot*3

               else

!              --- Odd plane in quadratic triangles
!                  Below are 4*nelbot extra points

                  ipcoor = ipcoor+3*nelbot*4

               end if

            end if

            do k = 1, npobot

               coor(ipcoor+3*k-2) = (1d0-ds)*bcord1(1,k) +ds*bcord2(1,k)
               coor(ipcoor+3*k-1) = (1d0-ds)*bcord1(2,k) +ds*bcord2(2,k)
               coor(ipcoor+3*k) = (1d0-ds)*bcord1(3,k) + ds*bcord2(3,k)

            end do

         end do

      else

!     --- Pipe
!         Compute distance between upper and under surface

         call mshds1 ( bcord1, bcord2, imax, kbndbt, dz )

!        --- Loop over cross-sections
!            Only the vertex cross-sections are filled

         do iplane = 1, n-1

!        --- Approximate surface by a plane

            ipcros =  iplane * istep3 + 1
            call mshpln ( bcord3(ipcros), m2-1, plane )

!           --- Project boundary onto this plane
!               Define axis into this plane (ksi-eta axis)
!               Compute relative distance of cross section to upper and under
!               plane
!               Compute ksi-eta co-ordinates of cross-section by weigthing
!               between upper and under plane
!               Map cross section into array coor

            call mshds2 ( bcord1, bcord3(ipcros), imax, kbndbt, dz, &
                          alpha )
            ipcoor =  iplane*npocrs + 1
            call mshpr1 ( coor(ipcoor), npobot, plane, xsieta, imax, &
                          bcord3(ipcros), kbndbt, alpha, eps, repos, &
                          klp, nlp )
            if ( repos ) then

!           --- Compute new points in vertex planes by repositioning

               if ( first ) then

!              --- First time that repositioning must be applied
!                  Fill array of neighbour nodes for repositioning

                  lengnb = 9 * npobot
                  call msh356 ( kmscbt, nelbot, npobot, inpelmbot, &
                                iwork, ibuurp, lengnb, linear, kbndbt, &
                                klp, nlp )
                  first = .false.

               end if
               call msh357 ( bcord3(ipcros), coor(ipcoor), npobot, &
                             ibuurp, iwork, inpelmbot, kmscbt, nelbot, &
                             kbndbt, eps, klp, nlp )

            end if

         end do

      end if

!     --- Copy top points in COOR
!           ( compute starting position in coor )

      call dcopy ( ncorpl, bcord2, 1, coor(n*npocrs+1), 1 )

!     --- Fill co-ordinate of odd planes for quadratic elements

      if ( .not. channel .and. linear==2 ) then

!     --- n extra planes

         do iplane = 1, n

!        --- Compute starting position in "upper" plane

            ih = iplane * npocrs - 3

!           --- Run through plane and place points between lower and upper
!               vertex plane

            do i = 1 , npobot

               ih2  = ih  + 3 * i
               ih1  = ih2 - npocrs
               ih3  = ih1 + nstep

               do k = 1 , 3

                  coor(ih3+k)=0.5d0*(coor(ih1+k)+coor(ih2+k))

               end do

            end do

!           --- Copy for this plane boundary from bcord3 into coor

            ih3 = (iplane-1) * istep3 - 3 + 3 * m2
            ii = 0

            do ilp = 1, nlp

               do i = klp(1,ilp), klp(2,ilp)
                  j = kbndbt(i)
                  ii = ii+1

                  ih2 = ih  - npocrs + nstep + 3*j
                  ih1 = ih3 + 3*ii

                  do k = 1 , 3
                     coor(ih2+k) = bcord3(ih1+k)
                  end do
              end do
           end do

         end do

      end if

!     --- Total number of points created in regular planes

      npoint = n * npobot * linear + npobot

!     --- Next fill co-ordinates of extra points for tetrahedral elements

      if ( trian .and. found==0 ) then

!     --- In the case of linear elements these co-ordinates are positioned
!         between the standard cross-sections.
!         For quadratic elements the sequence of the points in a set of two
!         vertex planes is as follows:
!            First the bottom plane is numbered in the sequence of bcord1
!            Then 3*nelbot points corresponding to the midpoints of the sides
!            that originate from the centroid points between two vertex planes
!            and ending in the vertices in the bottom plane are numbered.
!            Next the odd plane between the bottom and top plane is numbered
!            in the sequence of bcord1
!            This plane is followed by nelbot centroid points
!            Then 3*nelbot points corresponding to the midpoints of the sides
!            that originate from the centroid points between two vertex planes
!            and ending in the vertices in the top plane
!            Finally the top plane is numbered in the sequence of bcord1
!            The extra points are numbered in the same sequence as the elements
!            in the bottom plane and top plane are numbered

         if ( linear==1 ) then

!        --- Linear = 1 :  linear elements, only centroids
!                  ( There are n planes, with each nelbot points )

            do iplane = 1, n

               ih = (iplane-1) * npocrs - 3
               ikelm = 0

               do ielem = 1 , nelbot

!              --- Compute mean value in element ielem

                  do k = 1 , 3
                     dx(k) = 0d0
                  end do

                  do j = 1 , 3
                     iknpt=kmscbt(ikelm+j)
                     ih1=ih+3*iknpt
                     ih2=ih1+npocrs
                     do k = 1 , 3
                        dx(k) = dx(k) + coor(ih1+k) + coor(ih2+k)
                     end do
                  end do

                  ih1 = ih + 3*npobot + 3*ielem

                  do k = 1 , 3
                     coor(ih1+k) = dx(k)/6d0
                  end do

                  ikelm=ikelm+3

               end do
            end do

!           --- Total number of points

            npoint = npoint + n * nelbot

         else

!        --- Quadratic elements

            do iplane = 1 , n

               ih = (iplane-1) * npocrs - 3
               ikelm=0

               do ielem = 1 , nelbot

!              --- Compute mean value in element ielem

                  do k = 1, 3
                     dx(k)=0d0
                  end do

                  do j = 1, 5, 2

                     iknpt=kmscbt(ikelm+j)

                     ih1=ih+3*iknpt
                     ih2=ih1+npocrs

                     do k = 1, 3
                        dx(k)=dx(k)+coor(ih1+k)+coor(ih2+k)
                     end do
                  end do

                  do k = 1, 3
                     dx(k)=dx(k)/6d0
                  end do

!                 --- Centroid point

                  ih1=ih+2*ncorpl+3*ielem+9*nelbot
                  do k=1,3
                     coor(ih1+k)=dx(k)
                  end do

!                 --- Other points

                  ihk = ih  + ncorpl +  9 * (ielem-1)
                  ihk1= ihk + ncorpl + 12 *  nelbot

                  do j = 1 , 3

                     iknpt = kmscbt(ikelm+2*j-1)

!                    --- "lower vertices"

                     ih1   = ih  + 3*iknpt
                     ih2   = ihk + 3*j
                     do k = 1 , 3
                        coor(ih2+k)=0.5d0*(coor(ih1+k)+dx(k))
                     end do

!                    --- "upper vertices"

                     ih1   = ih1  +  npocrs
                     ih2   = ihk1 + 3*j

                     do k = 1 , 3
                        coor(ih2+k)=0.5d0*(coor(ih1+k)+dx(k))
                     end do

                  end do

                  ikelm = ikelm + 6

               end do

            end do

!           --- Extra points in this case

            npoint = npoint + 7 * nelbot * n

         end if

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh350'

      end if  ! ( debug )
      call erclos ( 'msh350' )

      end
