      subroutine msh357 ( bcord, coor, npoint, ibuur, istart, &
                          inpelm, kmscbt, nelem, kbndpt, eps, &
                          klp, nlp )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  3.1  date 07-12-2002 New treatment of centroids
!        version  3.0  date 29-04-1999 Extra param's for hollow pipes (BvR)
!        version  2.0  date 19-11-1994 Extra parameter eps
!        version  1.1  date 19-10-1992 do not recompute internal
!                                      boundary points
!        version  1.0  date 15-05-1990
!
!   copyright (c) 1990-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill array COOR with the coordinates by using repositioning
!     in a 3-D plane.
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     repositioning
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision bcord(3,*), coor(3,*), eps
      integer npoint, inpelm, ibuur(*), istart(*), kmscbt(inpelm,*), &
              nelem, kbndpt(*), klp(2,*), nlp

!     bcord   i     coordinates in boundary
!     coor    o     coordinate array
!     eps     i     Accuracy parameter
!                   If zero the repositioning is carried out in the standard
!                   way, which means that coor starts form scratch and
!                   that the process is stopped if the difference between
!                   two succeeding iterations is less than 10^(-4) times
!                   the minimal distance between points at the boundary
!                   if eps > 0, array coor has been initialized and the
!                   repositioning is halted if the difference between two
!                   iterations is less than 0.1*eps
!     ibuur   i     array with neighbour nodal points
!     inpelm  i     indicator for quadratic triangles (6) or
!                   quadrilaterals (9)
!     istart  i     array containing pointers for array ibuur
!     kbndpt  i     array with node numbers of boundary points
!     klp     i    In this array the relative node numbers with respect to
!                  array kbndbt of first and last point of a consecutive part
!                  are stored. The number of parts the boundary consists of
!                  is nlp
!                  Hence klp(1,1) = 1, klp(2,1) denotes the end point of the
!                  first part of the boundary
!                  klp(1,2) contains the node number of the first point of
!                  the second part. This must be klp(2,1)+1
!                  klp(2,nlp) contains the length of kbndbt
!     kmscbt  i     element array
!     nelem   i     number of elements
!     nlp     i    Number of disjoint parts the boundary of the bottom
!                  surface consists of
!     npoint  i     number of points in mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, ikn, jst, jen, jrep, i1, i2, i3, i4, i5, i6, i7, &
              i8, i9, k, ilp, ii
      double precision xz, yz, zz, dist, ds, dx, dy, dz, af

!     af     distance
!     dist   distance
!     ds     distance
!     dx     x-distance
!     dy     y-distance
!     dz     z-distance
!     i      loop variable
!     i1
!     i2
!     i3
!     i4
!     i5     node numbers
!     i6
!     i7
!     i8
!     i9
!     ii     Counting variable
!     ikn    node number
!     ilp    Counting variable
!     j      loop variable
!     jen    end of loop
!     jrep   reposition loop variable
!     jst    start of loop
!     k      loop variable
!     xz     x-coordinate of barycentre
!     yz     y-coordinate of barycentre
!     zz     z-coordinate of barycentre
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHD7          Put coordinates of extra centre point in quadratic triangle
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen( 'msh357' )
      if ( eps.le.0d0 ) then

!     --- Standard situation, coor has not yet been filled
!         Clean array COOR

          do i = 1, npoint
             do k = 1, 3
                coor(k,i) = 0
             end do
          end do

!         --- Place all boundary points in COOR

          do i = klp(1,1), klp(2,nlp)
             j = kbndpt(i)
             do k = 1, 3
                 coor(k,j) = bcord(k,i)
             end do
         end do

!        --- Compute discretization value

         i = 0
         dist = rinfin
         do ilp=1,nlp
            do ii = klp(1,ilp), klp(2,ilp)-1
                i = i+1
                dx = bcord(1,i+1) - bcord(1,i)
                dy = bcord(2,i+1) - bcord(2,i)
                dz = bcord(3,i+1) - bcord(3,i)
                ds = sqrt ( dx*dx + dy*dy + dz*dz )
                if ( ds.lt.dist ) dist = ds
            end do
         end do
         dist = 0.0001 * dist
      else

!     --- eps has been given a value before

         dist = 0.1d0 * eps
      end if

!     --- Reposition internal points

      do jrep = 1, 50
         af  = 0d0
         jen = 0
         do i = 1, npoint
            jst = jen + 1
            jen = istart(i  )
            if ( jen.le.jst ) goto 200
            do k = klp(1,1), klp(2,nlp)
               if ( i.eq.kbndpt(k) ) goto 200
            end do
            xz = 0d0
            yz = 0d0
            zz = 0d0
            do j = jst, jen
               ikn = ibuur(j)
               xz  = xz + coor(1,ikn)
               yz  = yz + coor(2,ikn)
               zz  = zz + coor(3,ikn)
            end do
            xz  =  xz / ( jen - jst + 1d0 )
            yz  =  yz / ( jen - jst + 1d0 )
            zz  =  zz / ( jen - jst + 1d0 )
            dx = coor(1,i) - xz
            dy = coor(2,i) - yz
            dz = coor(3,i) - zz
            ds = sqrt ( dx*dx + dy*dy + dz*dz )
            if ( ds.gt.af ) af = ds
            coor(1,i) = xz
            coor(2,i) = yz
            coor(3,i) = zz
200      end do
         if ( af.lt.dist .and. jrep.gt.3 ) goto 400
      end do

!     --- Place coordinates in midpoints for quadratic elements

400   if ( inpelm.gt.5 ) then
         do i = 1, nelem
            i1 = kmscbt(1,i)
            i2 = kmscbt(2,i)
            i3 = kmscbt(3,i)
            i4 = kmscbt(4,i)
            i5 = kmscbt(5,i)
            i6 = kmscbt(6,i)
            if ( inpelm.eq.9 ) then
               i7 = kmscbt(7,i)
               i8 = kmscbt(8,i)
               i9 = kmscbt(9,i)
            end if
            do k = 1, 3
               coor(k,i2) = ( coor(k,i1)+coor(k,i3) ) / 2d0
               coor(k,i4) = ( coor(k,i3)+coor(k,i5) ) / 2d0
               if ( inpelm.eq.6 ) then
                  coor(k,i6) = ( coor(k,i5)+coor(k,i1) ) / 2d0
               else if ( inpelm.eq.9 ) then
                  coor(k,i6) = ( coor(k,i5)+coor(k,i7) ) / 2d0
                  coor(k,i8) = ( coor(k,i7)+coor(k,i1) ) / 2d0
                  coor(k,i9) = ( coor(k,i2)+coor(k,i6) ) / 2d0
               end if
            end do
         end do
      end if
      if ( inpelm.eq.7 ) then

!     --- inpelm = 7, compute centroid

         call mshd7 ( kmscbt, coor, nelem, 3 )

      end if

!     --- Replace boundary points

      i = 0
      do ilp = 1, nlp
         do ii = klp(1,ilp), klp(2,ilp)
            i = i + 1
            j = kbndpt(ii)
            do k = 1, 3
               coor(k,j) = bcord(k,i)
            end do
         end do
      end do
      call erclos( 'msh357' )
      end
