      subroutine msh3dquadboun ( npoint, kbndpt, nelboun, nedges, &
                                 nfaces, nmeshb, nmeshc, nelem, ishape, &
                                 nmesha )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 29-02-2012 abs(iface)
!        version  1.0  date 15-05-2002
!
!   copyright (c) 2002-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array kbndpt for a pipe consisting of tetrahedrons of shape 15 or 16
!     It is supposed that kbndpt has already been filled for linear tetrahedrons
!     kbndpt contains the triangles on the boundary of the pipe in the
!     specific pipe sequence
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     volume
!     pipe
!     tetrahedron
!     boundary
!     quadratic
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint, kbndpt(*), nelboun, nedges, nfaces, nmeshb(*), &
              nmeshc(*), nelem, ishape, nmesha(*)

!     ishape         i    Shape number of volume elements
!                         In this case ishape = 15 or 16
!     kbndpt        i/o   array containing the nodal point numbers of the
!                         boundary stored per triangle
!                         At input the linear triangles with corresponding
!                         node numbers
!                         At output the 7 point triangles with the new
!                         numbering
!                         Storage per element as in kmeshn
!     nedges         i    number of edges of volume
!     nelboun        i    Number of elements in the boundary, i.e. in kbndpt
!     nelem          i    Number of elements
!     nfaces         i    Number of faces for volume
!     nmesha         i    nmesh part a
!     nmeshb         i    nmesh part b for volume
!     nmeshc         i    nmesh part c for volume
!     npoint         i    Number of nodal points for volume
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iwork(npoint), i, iedgenodes(npoint+5*nelboun), length, &
              iworkedge(nedges), ifacedges(2,nedges), j, iface, &
              iworkfaces(nfaces), istart

!     i              Counting variable
!     iedgenodes     Work array in which all the boundary edges corresponding to
!                    the boundary nodes are stored in the following sequence
!                    Pos. 1 to npoint: starting positions of all nodes i
!                    in iedgenodes
!                    iedgenode(1) = npoint+1
!                    Number of edges coupled to node i:
!                    iedgenode(i+1)-iedgenode(i)
!                    edges coupled to node i are stored in iedgenode
!                    from position iedgenode(i)+1
!     iface          Face sequence number
!     ifacedges      Work array in which all the faces corresponding to
!                    the boundary edges i are stored in
!                    ifaceedges(1,i) and ifaceedges(2,i)
!                    if i is not a boundary edge ifaceedges(.,i) = 0
!     istart         First position used at a part
!     iwork          integer work array of length npoint
!                    iwork(i) = 0, means point i is an internal point
!                    iwork(i) = 1, means point i is on the boundary
!     iworkedge      work array of length nedges
!                    iwork(i) = 0, means edge i is an internal edge
!                    iwork(i) = 1, means edge i is on the boundary
!     iworkfaces     Integer work array of size nfaces indicating the number
!                    of elements connected to the faces
!     j              Counting variable
!     length         Help parameter to indicate the length of arrays
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSH3DQUADBOUN1 Fill midside points and centroids of the boundary elements
!     MSHEDGNODES    Find all boundary edges corresponding to boundary nodes
!     MSHFACEEDGES   Find all boundary faces corresponding to edges
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Mark iwork for all boundary nodes
!     Find all boundary edges corresponding to boundary nodes
!     Find all boundary faces corresponding to edges
!     Fill all midside points of the elements
!     Fill all centroid points of the elements
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh3dquadboun' )

!     --- Mark iwork for all boundary nodes

      do i = 1, npoint
         iwork(i) = 0
      end do
      do i = 1, nelboun*3
         iwork(kbndpt(i)) = 1
      end do

!     --- Mark iworkfaces for all boundary faces

      do i = 1, nfaces
         iworkfaces(i) = 0
      end do
      do i = 1, nelem
         istart = nmesha(i)
         do j = 1, 4
            iface = abs(nmesha(istart+j))
            iworkfaces(iface) = iworkfaces(iface)+1
         end do
      end do

!     --- Find all boundary edges corresponding to boundary nodes

      length = npoint+5*nelboun
      call mshedgnodes ( iwork, npoint, nmeshc, nedges, &
                         iedgenodes, length, iworkedge )

!     --- Find all boundary faces corresponding to edges

      call mshfaceedges ( nmeshb, nfaces, nedges, ifacedges, &
                          iworkfaces )

!     --- Fill all midside points of the elements
!         Fill all centroid points of the elements

      call msh3dquadboun1 ( kbndpt, nelboun, iedgenodes, nmeshc, &
                            ifacedges, npoint, nedges, nfaces, nelem, &
                            ishape )

      call erclos ( 'msh3dquadboun' )
      end
