      subroutine msh440 ( coornew, kmeshcnew, &
                          kmeshmnew, kmeshnnew, lencoor, lkmeshc, &
                          lenkmeshm, lenkmeshn, ibrp  )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  3.0  date 14-10-2002 Extra parameter kmeshq
!        version  2.2  date 28-02-1999 Include commons
!        version  2.1  date 27-04-1995 New calls due to spectral
!                                      incorporation
!
!   copyright (c) 1992-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Transform an existing mesh of linear triangles (ishape=3) or
!     linear tetrahedrons (ishape=11) into a mesh of linear
!     quadrilaterals (ishape=5) or linear hexahedrons (ishape=13)
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     3d
!     element
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision :: coornew(ndim,*)
      integer :: kmeshcnew(*), kmeshmnew(*), kmeshnnew(*), &
                 lencoor, lkmeshc, lenkmeshm, lenkmeshn, ibrp

!     coornew        o    Array containing the new coordinates of the nodes
!     ibrp           i    Estimated number of neighbours
!     kmeshcnew      o    New kmesh part c
!     kmeshmnew      o    New array kmeshm
!     kmeshnnew      i    New array kmeshn
!     lencoor       i/o   Length of array COORNEW
!                         at input : declared length
!                         at output : used length
!     lenkmeshm     i/o   Length of array kmeshmnew
!                         at input : declared length
!                         at output : used length
!     lenkmeshn     i/o   Length of array kmeshnnew
!                         at input : declared length
!                         at output : used length
!     lkmeshc       i/o   Length of array kmeshcnew
!                         at input : declared length
!                         at output : used length
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iklemh, ikmeshc, nelhel, neltal, npelol, inpelo, &
              ielgrp, nextra, inpelm, ipnt, nelmlc, inod, leused, ipos, &
              is, nel, i1, i2, i3, i4, i5, &
              i6, i7, jstart, &
              jstend, ia, ib, ic, npnold, kcoor, ih, npt
      integer, allocatable :: ibrpnt(:), istart(:)

!     i        loop variable
!     i1       node
!     i2       node
!     i3       node
!     i4       node
!     i5       node
!     i6       node
!     i7       node
!     ia       first node on line or in triangle
!     ib       second node on line or in triangle
!     ibrpnt         help array for neighbor array and new points
!     ic       third (ordered) node in triangle
!     ielgrp   loop variable for groups
!     ih       help node number
!     iklemh   start position kmeshcnew array in buffer
!     ikmeshc   start position kmeshc array in buffer
!     inod     start position of node values
!     inpelm   number of nodes in element
!     inpelo   number of points in element
!     ipnt     node number
!     ipos     starting position
!     is       temporary node number
!     istart         help array for starting positions in ibrpnt
!     jstart   first position to be considered
!     jstend   last position to be considered
!     kcoor    number of positions in new node array
!     leused   length used to store surface array (KMESH part n)
!     nel      local number of elements
!     nelhel   variable to store actual number of elements
!     nelmlc   number of elements in local element group
!     neltal   variable to store number of elements
!     nextra   number of extra points created in centroids
!     npelol   number of old elements in group
!     npnold   number of points in old mesh
!     npt      new node number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSH401         Run through group of elements to determine neighbours
!     MSH403         Determine number refinement point in line piece
!     MSH405         Store nodes of quadrilateral in element array
!     MSH443         Place new points in all faces of tetrahedrons
!     MSH444         Place all triangles in neighbour array
!     MSH446         Create internal points and new linear hexahedrons
!     MSH461         Order three node numbers
!     MSH462         Determine new nodes
!     MSHTRAKMESHM   Fill kmeshm after transformation
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      940   Not enough positions reserved
!      972   Meshtype not allowed as start in TRANSF
!     1800   No triangles or tetrahedrons in the mesh
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Determine the important parameters
!     Copy array coor to coornew
!     Set variables for determination points in triangles and tetrahedrons
!     Clear array istart and array ibrpnt
!     Run through group of elements, check each side and make reservation
!     if not yet done in arrays ibrpnt and istart
!     Rearrange istart and ibrpnt
!     Place all neighbours not yet placed
!     Check declared length of array ibrpnt
!     Set number of points for copiing later
!     Place new nodal-point numbers in array ibrpnt
!     Fill array coornew
!     Make extra loop in case ndim = 3 and tetrahedrons to place also
!     points in the faces of the volumes
!     Place internal points and create new elements
!     mesh has been changed (fill new values in kmesh)
!     Fill kmesh part m and if necessary fill kmesh part n
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh440' )

!     --- Set variables to be used for the surfaces

      iklemh  =   1
      ikmeshc  =   1

!     --- Run through group of elements, check each side and make reservation
!         if not yet done

      neltal = 0
      kelgrp = 0
      inpelm = 0

      allocate ( ibrpnt(ibrp*(npoint+1)), istart(npoint+1), stat = error )
      if ( error/=0 ) call eralloc ( error, ibrp*(npoint+1), 'ibrpnt' )
      if ( ierror/=0 ) go to 1000

!     --- Clear array istart (array for the starting addresses)

      istart = 0

!     --- Clear array ibrpnt (array for the neighbours)

      ibrpnt = 0

      kmeshcnew(1) = 0

      if ( kelma==0 ) then

!     --- Only one type of standard element

           ishape = kmeshf(1)

!          --- Check type, only linear triangles or tetrahedrons
!              are allowed

           if ( ishape==3 ) then

                niedge = 1
                nisurf = 1
                nivolm = 0

           else if ( ishape==11 ) then

                niedge = 1
                nisurf = 1
                nivolm = 1

           else

                call errint( ishape, 1 )
                call errsub ( 972, 1, 0, 0 )
                go to 1000

           end if

!          --- First all lines are stored in ibrpnt

           call msh401 ( npoint, kmeshc, nelem, istart, ibrp, ibrpnt, &
                         kmeshcnew, ishape, npelm, niedge, nisurf, &
                         nivolm )

!          --- Next all internal points

           call msh444 ( kmeshc , nelem , istart, ibrp, ibrpnt, &
                         kmeshcnew, ishape, npoint )

      else

!     --- Several standard elements
!         Start address in kmeshc

           ikmeshc = 1

           do ielgrp = 1, nelgrp

               inpelm = kmesha(ielgrp+1) - kmesha(ielgrp)
               ishape = kmeshf(ielgrp)

!              --- Check type of elements, only linear triangles and
!                  tetrahedrons are allowed

               if ( ishape==3 ) then

                    niedge = 1
                    nisurf = 1
                    nivolm = 0

               else if ( ishape==11 ) then

                    niedge = 1
                    nisurf = 1
                    nivolm = 1

               else

                    call errint( ishape, 1 )
                    call errsub ( 972, 1, 0, 0 )
                    go to 1000

               end if

!              --- Number of elements of this type

               inelem = kmeshg(ielgrp)

!              --- First all lines of the elements are stored in ibrpnt

               call msh401 ( npoint, kmeshc( ikmeshc ), inelem, istart, &
                             ibrp  , ibrpnt, kmeshcnew, ishape, inpelm, &
                             niedge, nisurf, nivolm )

!              --- Next all internal points of the elements are stored

               call msh444 ( kmeshc, nelem, istart, ibrp, ibrpnt, &
                             kmeshcnew, ishape, npoint )

!              --- Increase ikmeshc

               ikmeshc = ikmeshc + inpelm * inelem

           end do

      end if

      npnold = npoint

!     --- All neighbours have been placed, rearrange arrays istart and
!         ibrpnt. Determine node numbers of new neighbours and place
!         new points

      call msh462 ( ibrp, ibrpnt, istart, npoint, kmeshcnew, niedge, &
                    nisurf, nivolm )

!     --- Save new number of nodes

      kcoor = ndim * npoint

      if (  kcoor>lencoor ) then

            call errint ( kcoor, 1 )
            call errint ( lencoor, 2 )
            call errsub ( 940, 2, 0, 0 )
            go to 1000

      end if

!     --- Fill coornew (for barycenters and quadratic cases only
!                     a first estimate)

      do i = 1, npnold

          jstart = istart(i  )
          jstend = istart(i+1) - 2

!         --- Copy old coordinates to new array

          coornew(:,i) = coor(:,i)

          do ipnt = jstart, jstend, 2

              ih  = ibrpnt( ipnt   )
              npt = ibrpnt( ipnt+1 )

!             --- Check value of ih

              if ( ih>0 .and. ih<npnold ) then

                 coornew(:,npt) = (coor(:,ih)+coor(:,i))*0.5d0

              else

!             --- Point in barycenter, use npnold and change sign

                 ih = abs(ih)

                 i1 = i
                 i2 = int( ih / npnold )
                 i3 = ih - ( npnold * i2 )

                 coornew(:,npt) = ( coor(:,i1) + &
                     coor(:,i2) + coor(:,i3) ) / 3d0

              end if

          end do

      end do

!     --- Next loop place internal points and create new elements

      nextra = 0
      nelhel = 0
      ikmeshc = 1

      do ielgrp = 1 , nelgrp

!     --- determine meshtype

          ishape = kmeshf(ielgrp)

!         --- number of elements of this type

          if ( kelmg==0 ) then
               inelem = nelem
          else
               inelem = kmeshg(ielgrp)
          end if

          npelol = inelem

          if ( ishape==3 ) then

               inpelo = 3
               inpelm = 4
               nelhel = nelhel + inelem

!              --- Create new elements for triangles

               call msh443( kmeshc(ikmeshc), inelem, kmeshcnew(iklemh), &
                            npnold, ibrpnt, istart )

               inelem = 3 * inelem
               ishape = 5

          else if ( ishape==11 ) then

               inpelo = 4
               inpelm = 8

!              --- linear tetrahedrons : create new point in center of
!                  tetrahedron and new hexahedronal elements

               call msh446( kmeshc(ikmeshc), inelem, kmeshcnew(iklemh), &
                            coornew, npoint, ibrpnt, istart, npnold, &
                            nextra )

               nextra = nextra + inelem
               inelem = 4 * inelem
               ishape = 13

          end if

!         --- Increase ikmeshc and iklemh

          ikmeshc = ikmeshc + npelol * inpelo
          iklemh = iklemh + inelem * inpelm

          kmeshf(ielgrp) = ishape

          if ( kelmg/=0 ) kmeshg(ielgrp) = inelem

          neltal = neltal + inelem
          kelgrp = kelgrp + inpelm

      end do

!     --- mesh has been changed (fill new values in kmesh)

      npoint = npoint + nextra
      npelm = inpelm

      nelem  = neltal

      if ( (ndim * npoint)>lencoor ) then

!     --- Not enough positions reserved for coordinates of points

           call errint ( ndim * npoint, 1 )
           call errint ( lencoor, 2 )
           call errsub ( 940, 2, 0, 0)
           go to 1000

      end if

      lencoor = ndim * npoint

      if ( ndim==2 .and. 4 * nelem>lkmeshc ) then

!     --- Not enough positions reserved for elements

           call errint ( 4 * nelem, 1 )
           call errint ( lkmeshc, 2 )
           call errsub ( 940, 2, 0, 0)
           go to 1000

      else if ( ndim==3 .and. 8 * nelem>lkmeshc ) then

!     --- Not enough positions reserved for elements

           call errint ( 8 * nelem, 1 )
           call errint ( lkmeshc, 2 )
           call errsub ( 940, 2, 0, 0)
           go to 1000

      end if

!     --- Make final reservation

      if ( ndim==2 ) then

         lkmeshc = 4 * nelem

      else if ( ndim==3 ) then

         lkmeshc = 8 * nelem

      end if

!     --- Fill kmeshmnew

      call mshtrakmeshm ( kmeshmnew, istart, ibrpnt, npnold, lenkmeshm )

      if ( ndim==3 .and. kelmn>0 ) then

!     --- Fill kmeshnnew (kmesh part n)

           inod   = 2 * nsurfs
           ipos   = inod + 1
           nelhel = 0

           do is = 1 , nsurfs

!          --- Value of npelm was 3 store new value 4

               kmeshnnew( is ) = 4

!              --- determine number of elements

               nelmlc = kmeshn( is + nsurfs )

!              --- Store new value

               kmeshnnew( is + nsurfs ) = 3 * nelmlc

!              --- Run through all triangles and make quadrilaterals

               do nel = 1 , nelmlc

                   i1   = kmeshn(inod+1)
                   i2   = kmeshn(inod+2)
                   i3   = kmeshn(inod+3)

                   inod = inod + 3

!                  --- determine the other four points

                   call msh403 ( npnold, i1, i2, ibrpnt, istart, i4 )
                   call msh403 ( npnold, i2, i3, ibrpnt, istart, i5 )
                   call msh403 ( npnold, i3, i1, ibrpnt, istart, i6 )

                   call msh461( i1, i2, i3, ia, ib, ic )

                   ih = - (ib + npnold * ic)

                   call msh403( npnold, ia, ih, ibrpnt, istart, i7 )

                   nelhel = nelhel + 1
                   call msh405( kmeshnnew(ipos), nelhel, i1, i4, i7, i6)
                   nelhel = nelhel + 1
                   call msh405( kmeshnnew(ipos), nelhel, i4, i2, i5, i7)
                   nelhel = nelhel + 1
                   call msh405( kmeshnnew(ipos), nelhel, i7, i5, i3, i6)

               end do

           end do

           leused = ipos + 4 * nelhel - 1
           lenkmeshn  = leused

      end if

      if ( inpelm==0 ) call errsub ( 1800, 0, 0, 0 )

      deallocate ( ibrpnt, istart, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'ibrpnt' )

1000  call erclos ( 'msh440' )

      end subroutine msh440
