      subroutine mshcurves1 ( kmeshmtmp, ncurvsnew, lenkmeshmtmp, &
                              curvstosurfs, ipoint, lenipoint, lncurvstosurfs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.3  date 29-04-2005 Debug ststements
!        version  3.2  date 08-04-2005 Correction for ncurvs>nsurfs
!        version  3.1  date 21-03-2005 Fortran 90
!
!   copyright (c) 2004-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create new curves as intersection of surfaces, provided no curve exists
!     already as intersection
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshmtmp(*), ncurvsnew, lenkmeshmtmp, &
              curvstosurfs(nsurfs,*), lncurvstosurfs, ipoint(*), lenipoint

!     curvstosurfs   o    Array of size nsurfs x nsurfs
!                         curvstosurfs(i,j) contains the j-th curve
!                         corresponding to surface i
!                         if 0 no more curves can be found
!     ipoint         o    Array of length ncurvsnew containing the starting
!                         address of the next curve in kmeshmtmp
!                         So in ipoint(1) the number of nodes for the first new
!                         curve in kmeshmtmp plus one is stored etc.
!     kmeshmtmp      o    Temporary array kmeshm in which the nodes of the new
!                         curves are stored
!     lenipoint     i/o   Length of array ipoint
!                         At input: declared length
!                         At output: computed length
!     lenkmeshmtmp  i/o   Length of array kmeshmtmp
!                         At input: declared length
!                         At output: computed length
!     lncurvstosurfs i    Declared length of second dimension of array
!                         curvstosurfs
!     ncurvsnew      o    Number of new curves
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, isurnr, jsurnr, istart, inode, jstart, icurnr, j, &
              istartm , numpoints, maxlen, &
              iworksurfs(nsurfs), ipstart, iielem, &
              jnelem, nedge_face, iedge, iface, numedges
      logical single
      integer, allocatable :: iworkedges(:), istoredges(:), istore(:), &
                              iworkj(:), iworkmarkedgs(:), iworkmarknods(:)

!     i              Counting variable
!     icurnr         Curve sequence number
!     iedge          edge sequence number
!     iface          Face sequence number
!     iielem         Relative element number in element group
!     inode          Node sequence number
!     ipstart        Starting address in array
!     istart         First position used at a part
!     istartm        Starting address in array kmeshmtmp
!     istore         Work array of length npoint
!                    In this array the common points of two surfaces are
!                    stored
!     istoredges     Work array of length nedges
!                    Is used to store a set of edges
!     isurnr         Surface sequence number
!     iworkedges     Work array of length nedges
!                    iworkedge(i) = 0: edge is not in isurnr
!                    iworkedge(i) = 1: edge is in isurnr but nor jsurnr
!                    iworkedge(i) = 2: edge is in isurnr and jsurnr
!                    iworkedge(i) = 3: edge has been stored
!     iworkj         integer work array of length npoint
!                    if iworkj(i)  > 0, point i is in surface jsurnr and
!                    in surface isurnr
!     iworkmarkedgs  integer work array of length nedges
!                    if iworkmarkedgs(i) > 0, i is stored in istoreedges
!     iworkmarknods  integer work array of length npoint
!                    Is used by subroutines called by this subroutine
!     iworksurfs     Work array of length nsurfs to store the number of curves
!                    that have been filled in curvstosurfs for each surface
!     j              Counting variable
!     jnelem         number of elements in an element group for one super
!                    element
!     jstart         starting position
!     jsurnr         Surface sequence number
!     maxlen         Maximum number of curves that correspond to one surface
!     nedge_face     Number of edges in face
!     numedges       number of edges
!     numpoints      number of points
!     single         if true the surface is not composite
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     MSHCURVES2     Sort nodes in curve in order to get a SEPRAN curve
!     MSHCURVES3     Check if new curve stored in istore is not already stored
!                    in kmeshm
!                    If new curve already in kmeshm, set numpoints = 0
!     PRININ         print 1d integer array
!     PRININ2        Print an integer two-dimensional array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For isurnr := 1 (1) nsurfs do
!        For jsurnr := isurnr+1 (1) nsurfs do
!           if ( both isurnr and jsurnr are simple surfaces ) then
!              numpoints := number of common points of isurnr and jsurnr
!              if ( numpoints>0 and common points do not form a curve ) then
!                 Store common points to create a new curve
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcurves1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcurves1'
100      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      ncurvsnew = 0
      icurnr = 0
      maxlen = 0

      allocate ( iworkedges(nedges), istoredges(nedges), &
                 istore(npoint), iworkj(npoint),iworkmarkedgs(nedges), &
                 iworkmarknods(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, nedges+npoint, 'iworkedges' )
      if ( ierror/=0 ) go to 1000

!     --- Clear arrays

      iworkedges = 0
      iworkj = 0

!     --- Clear array iworksurfs(1:nsurfs) and curvstosurfs

      do i = 1, nsurfs
         iworksurfs(i) = 0
         do j = 1, lncurvstosurfs
            curvstosurfs(i,j) = 0
         end do
      end do

!     --- For isurnr := 1 (1) nsurfs do
!            For jsurnr := isurnr+1 (1) nsurfs do

      istartm = 1
      istart = 0
      do isurnr = 1, nsurfs

         if ( debug ) write(irefwr,100) 'isurnr', isurnr
         if ( kelmr>0 ) then

!        --- There are composite surfaces

            single = kmeshr(isurnr+1)-kmeshr(isurnr)<=1

         else

!        --- No composite surfaces

            single = .true.

         end if
         inelem = kmeshn(isurnr+nsurfs)
         if ( debug ) write(irefwr,*) 'single ', single

         if ( single ) then

!        --- Surface isurnr is single surface

            do jsurnr = isurnr+1, nsurfs

!           --- Mark all edges in the faces of the surface by a 1 in array
!               iworkedges

               do iielem = 1, inelem

                  iface = nmeshj(istart+iielem) ! face number of surf element
                  ipstart = nmeshb(iface)          ! starting address of edges
                  nedge_face = nmeshb(ipstart)     ! number of edges
                  do i = 1, nedge_face
                     iedge = abs(nmeshb(ipstart+i))! edge in face
                     iworkedges(iedge) = 1
                  end do  ! i = 1, nedge_face

               end do  ! iielem = 1, inelem
               if ( kelmr>0 ) then

!              --- There are composite surfaces

                  single = kmeshr(jsurnr+1)-kmeshr(jsurnr)<=1

               else

!              --- No composite surfaces

                  single = .true.

               end if
               if ( debug ) &
                  write(irefwr,*) 'jsurnr, single ', jsurnr, single

               numedges = 0      ! number of common edges
               if ( single ) then

!              --- both isurnr and jsurnr are simple surfaces
!                  Find common edges and store in array istoreedge

                  jstart = 0        ! address in nmeshj
                  do i = 1, jsurnr-1
                     jstart = jstart+kmeshn(i+nsurfs)
                  end do
                  jnelem = kmeshn(jsurnr+nsurfs)

                  do iielem = 1, jnelem

                     iface = nmeshj(jstart+iielem)   ! face number of surf el
                     ipstart = nmeshb(iface)            ! starting address edges
                     nedge_face = nmeshb(ipstart)       ! number of edges
                     do i = 1, nedge_face

                        iedge = abs(nmeshb(ipstart+i))  ! edge in face
                        if ( iworkedges(iedge)==1 ) then

!                       --- Common edge found, store in istoredges

                           numedges = numedges+1
                           iworkedges(iedge) = 2
                           istoredges(numedges) = iedge

                        end if  ! ( iworkedges(iedge)==1 )

                     end do  ! i = 1, nedge_face

                  end do  ! iielem = 1, inelem

               end if  ! ( single )

               if ( numedges>0 ) then

!              --- Surfaces contain common edges

                  if ( debug ) then
                     write(irefwr,100) 'isurnr, jsurnr', isurnr, jsurnr
                     write(irefwr,100) 'number of common edges', &
                                        numedges
                  end if  ! ( debug )

!                 --- Store nodes in istore

                  numpoints = 0
                  do i = 1, numedges
                     iedge = istoredges(i)
                     ipstart = nmeshc(iedge)

                     do j = 1, nmeshc(ipstart)

                        inode = nmeshc(ipstart+j)
                        if ( iworkj(inode)==0 ) then
                           iworkj(inode) = 1
                           numpoints = numpoints + 1
                           istore(numpoints) = inode
                        end if

                     end do  ! j = 1, nmeshc(ipstart)

                  end do  ! i = 1, numedges

                  if ( debug ) then
                     write(irefwr,100) 'number of common points', &
                                        numpoints
                     call prinin ( istore, numpoints, 'istore' )
                  end if  ! ( debug )

!                 --- Surfaces isurnr and jsurnr have common points
!                     Check if they do not already form a curve
!                     If so set numpoints = 0

                  call mshcurves3 ( iworkmarknods, iworkj, numpoints, icurnr )
                  if ( ierror/=0 ) go to 1000
                  if ( debug ) &
                     write(irefwr,100) 'icurnr, numpoints', &
                                        icurnr, numpoints

                  if ( numpoints==0 ) then

!                 --- numpoints = 0, icurnr corresponds to both surfaces

                     iworksurfs(isurnr) = iworksurfs(isurnr)+1

                     call inichkarlen ( iworksurfs(isurnr), &
                                        lncurvstosurfs, &
                                        'curvstosurfs' )
                     maxlen = max(maxlen,iworksurfs(isurnr))
                     if ( ierror/=0 ) go to 1000

                     if ( debug ) &
                        write(irefwr,100) 'isurnr, jsurnr, icurnr', &
                                           isurnr, jsurnr, icurnr
                     curvstosurfs(isurnr,iworksurfs(isurnr)) = icurnr
                     iworksurfs(jsurnr) = iworksurfs(jsurnr)+1

                     call inichkarlen ( iworksurfs(jsurnr), &
                                        lncurvstosurfs, &
                                        'curvstosurfs' )
                     maxlen = max(maxlen,iworksurfs(jsurnr))
                     if ( ierror/=0 ) go to 1000

                     curvstosurfs(jsurnr,iworksurfs(jsurnr)) = icurnr

                  else if ( numpoints>0 ) then

!                 --- Surfaces isurnr and jsurnr have common points
!                     Sort nodes in istore to form a curve

                     call mshcurves2 ( istore, numpoints, &
                                       kmeshmtmp(istartm), &
                                       iworkmarknods,  isurnr, jsurnr, &
                                       numedges, istoredges, iworkedges, &
                                       iworkmarkedgs, istartm, &
                                       ncurvsnew, ipoint, iworksurfs, &
                                       curvstosurfs, &
                                       lenkmeshmtmp, lncurvstosurfs, &
                                       maxlen )
                     if ( ierror/=0 ) go to 1000
                     if ( debug ) &
                        write(irefwr,100) 'ncurvsnew', ncurvsnew

                  end if  ! ( numpoints==0 )

!                 --- Clear iworkj

                  do i = 1, numedges
                     iedge = istoredges(i)
                     ipstart = nmeshc(iedge)
                     do j = 1, nmeshc(ipstart)
                        inode = nmeshc(ipstart+j)
                        iworkj(inode) = 0
                     end do  ! j = 1, nmeshc(ipstart)
                  end do  ! i = 1, numedges

               end if  ! ( numedges>0 )

!              --- Reset

            end do  ! jsurnr = isurnr+1, nsurfs

         end if  ! ( single )

         call inichkarlen ( istartm-1, lenkmeshmtmp, 'kmeshmtmp' )
         call inichkarlen ( ncurvsnew, lenipoint, 'ipoint' )
         if (ierror/=0 ) go to 1000

!        --- Clear array iworkedges

         do iielem = 1, inelem

            iface = nmeshj(istart+iielem)   ! face number of surf element
            ipstart = nmeshb(iface)            ! starting address of edges
            nedge_face = nmeshb(ipstart)       ! number of edges
            do i = 1, nedge_face
               iedge = abs(nmeshb(ipstart+i))  ! edge in face
               iworkedges(iedge) = 0
            end do  ! i = 1, nedge_face

         end do  ! iielem = 1, inelem

!        --- Raise istart

         istart = istart+inelem

      end do  ! isurnr = 1, nsurfs
      lenkmeshmtmp = istartm-1

!     --- Extend the set of curves with one empty curve
!         This is done to simplify the creation of empty curves

      ncurvsnew = ncurvsnew+1
      call inichkarlen ( ncurvsnew, lenipoint, 'ipoint' )
      ipoint(ncurvsnew) = istartm

      deallocate ( iworkedges, istoredges, istore, iworkj, &
                   iworkmarkedgs, iworkmarknods, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iworkedges' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshcurves1'
         write(irefwr,100) 'ncurvsnew', ncurvsnew
         call prinin2 ( curvstosurfs, nsurfs, maxlen, 'curvstosurfs' )
         call prinin ( ipoint, ncurvsnew, 'ipoint' )

      end if
      call erclos ( 'mshcurves1' )

      end
