      subroutine mshcurvsurfs ( curve_surfs, lencurvsur, &
                                surface_curvs, ncurvs, nsurfs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-03-2002
!
!   copyright (c) 2002-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Store all outer surfaces coupled to a curve in array curv_surfaces
!     Only single curves in a surface are considered
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     surface
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer curve_surfs(*), lencurvsur, surface_curvs(*), ncurvs, &
              nsurfs

!     curve_surfs    o    Contains all outer surfaces to the "single" curves
!                         provided the curve contains exactly 2 outer surfaces
!                         Pos. 1 to ncurvs: starting addresses in part 2
!                         Contents of each curve in part 2
!                         Pos. 1 number of outer surfaces coupled to curve
!                         Followed by surface numbers
!     lencurvsur     i    Declared length of array curve_surfs
!     ncurvs         i    Number of curves in mesh
!     nsurfs         i    number of surfaces
!     surface_curvs  i    Contains the number of "single" curves coupled
!                         to all outer surfaces in the following way
!                         Pos. 1 to nsurfs: starting addresses for second part
!                         For each surface in second part:
!                         Pos. 1: number of single curves
!                         Followed by curve numbers (all positive)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iwork(ncurvs), i, isurnr, icurnr, ipstart, istart, &
              nouterc, numsurfs

!     i              Counting variable
!     icurnr         Curve sequence number
!     ipstart        Starting address in array
!     istart         First position used at a part
!     isurnr         Surface sequence number
!     iwork          integer work array
!     nouterc        Number of outer curves
!     numsurfs       number of surfaces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
!    2415   More than 2 outer surfaces coupled to curve
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Fill number of surfaces per curve:
!        Clear array iwork(1:ncurvs)
!        For all surface isurnr do
!           nouterc := number of "single" curves corresponding to outer surface
!           For all these curves do
!              icurnr := curve number
!              iwork(icurnr) := iwork(icurnr)+1
!     Fill first part of array curve_surfs:
!        istart := ncurvs+1
!        For all curves icurnr do
!           curve_surfs(icurnr) := istart
!           if ( iwork(icurnr)>2 ) then
!              error not yet implemented
!           else if ( iwork(icurnr)=2 ) then
!              curve_surfs(istart) := 2
!              istart := istart+3
!           else if ( iwork(icurnr)<2 )
!              curve_surfs(istart) := 0
!              istart := istart+1
!     Fill last part of array curve_surfs:
!        Clear array iwork(1:ncurvs)
!        For all surface isurnr do
!           nouterc := number of "single" curves corresponding to outer surface
!           For all these curves do
!              icurnr := curve number
!              istart := curve_surfs(icurnr)
!              numsurfs := curve_surfs(istart)
!              if ( numsurfs=2 ) then
!                 iwork(icurnr) := iwork(icurnr)+1
!                 curve_surfs(istart+iwork(icurnr)) := isurnr
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcurvsurfs' )

!     --- Fill number of surfaces per curve:
!         Clear array iwork(1:ncurvs)

      do i = 1, ncurvs
         iwork(i) = 0
      end do

!     --- For all surface isurnr do

      do isurnr = 1, nsurfs

!     --- nouterc := number of "single" curves corresponding to outer surface

         ipstart = surface_curvs(isurnr)
         nouterc = surface_curvs(ipstart)

!        --- For all these curves do
!              icurnr := curve number
!              iwork(icurnr) := iwork(icurnr)+1

         do i = 1, nouterc
            icurnr = surface_curvs(ipstart+i)
            iwork(icurnr) = iwork(icurnr)+1
         end do

      end do

!     --- Fill first part of array curve_surfs:
!         istart := ncurvs+1
!         For all curves icurnr do

      istart = ncurvs+1
      do icurnr = 1, ncurvs

!     --- curve_surfs(icurnr) := istart

         curve_surfs(icurnr) = istart
         if ( iwork(icurnr).gt.2 ) then

!        --- iwork(icurnr)>
!            error not yet implemented

            call errint ( icurnr, 1 )
            call errint ( iwork(icurnr), 2 )
            call errwar ( 2415, 2, 0, 0 )

         end if
         if ( iwork(icurnr).ge.2 ) then

!        --- iwork(icurnr)>=2
!            curve_surfs(istart) := iwork(icurnr)
!            istart := istart+iwork(icurnr)+1

            curve_surfs(istart) = iwork(icurnr)
            istart = istart+iwork(icurnr)+1

         else

!        --- iwork(icurnr)<2, not interesting case
!            curve_surfs(istart) := 0
!            istart := istart+1

            curve_surfs(istart) = 0
            istart = istart+1

         end if

      end do
      if ( istart-1.gt.lencurvsur ) then

!     --- estimated length of array surface_curvs too small

         call errint ( istart-1, 1 )
         call errint ( lencurvsur, 2 )
         call errchr ( 'curve_surfs', 1 )
         call errsub ( 550, 2, 0, 1 )
         go to 1000

      end if

!     --- Fill last part of array curve_surfs:
!         Clear array iwork(1:ncurvs)

      do i = 1, ncurvs
         iwork(i) = 0
      end do

!     --- For all surface isurnr do

      do isurnr = 1, nsurfs

!     --- nouterc := number of "single" curves corresponding to outer surface

         ipstart = surface_curvs(isurnr)
         nouterc = surface_curvs(ipstart)

!        --- For all these curves do
!              icurnr := curve number
!              istart := curve_surfs(icurnr)
!              numsurfs := curve_surfs(istart)

         do i = 1, nouterc

            icurnr = surface_curvs(ipstart+i)
            istart = curve_surfs(icurnr)
            numsurfs = curve_surfs(istart)
            if ( numsurfs.ge.2 ) then

!           --- numsurfs>=2
!               iwork(icurnr) := iwork(icurnr)+1
!               curve_surfs(istart+iwork(icurnr)) := isurnr

               iwork(icurnr) = iwork(icurnr)+1
               curve_surfs(istart+iwork(icurnr)) = isurnr

            end if

         end do

      end do

!     --- For all these curves do
!            icurnr := curve number
!            if ( numsurfs=2 ) then
!               iwork(icurnr) := iwork(icurnr)+1
!               curve_surfs(istart+iwork(icurnr)) := isurnr

1000  call erclos ( 'mshcurvsurfs' )

      end
