      subroutine mshfil1112 ( iinput, icurvs, kbndpt, iwork, iposi, &
                              leniwork, nptot, neltot )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-11-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill array KBNDPT for a composite surface
!     and write to file 3
!     Type surface: 11, 12 (ordered)
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      use sepmodulefiles
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kbndpt(*), icurvs(*), iwork(*), iinput(*), &
              iposi, leniwork, nptot, neltot

!     icurvs       i/o    Array of length ncurvs containing information of the
!                         curves with respect to kbndpt
!                         icurvs is filled as follows:
!                         icurvs(i) = 0    Curve i is not present
!                         icurvs(i)>0    Curve i has not been used before
!                                          The value gives the starting position
!                                          in array kbndpt
!                         icurvs(i)<0    Curve i has been used before
!                                          Minus the value gives the starting
!                                          position in array kbndpt
!     iinput        i     Array containing input for the composite surface
!     iposi         i     next free position of iinput
!     iwork               Work array of length 2*(npoloc+nelloc)+nsurfs
!                         npoloc = number of points in surface
!                         nelloc = number of elements in surface
!                         With respect to mshfl0, iwork is extended with ncurvs
!                         positions
!     kbndpt         o    Array containing the boundary points of the
!                         composite surface;curves are counted for only once
!     leniwork       i    Computed length of array iwork
!     neltot              Total number of elements in composite surface
!     nptot               Total number of nodes in composite surface,
!                         centroids excluded
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, idirec, index, &
              ipoint, iprec, irow, ishap1, issub, issubf, &
              istart, isub, isubsf, jstarr, &
              jsubsf, jsurf, kstart, leng3, m1, &
              mtot, n1, ndefsf, nelloc, &
              nelslc, ngboun, nloc, npoloc, &
              nposlc, nrel, nrow, ns, nsubsf, ntot, nwork, &
              jssubf
      integer, allocatable :: kbndlc(:)

!     i              Counting variable
!     idirec         Defines in which direction the curves must be considered
!                    idirec = 1, from low to high, idirec = -1: reversed order
!     index          Index array to be used if 1000<=ind(number)<2000 and
!                    ichois = 2
!     ipoint
!     iprec          Surface number of surface to be copied etc.
!     irow           Row number in rows of surfaces in ordered surface of
!                    surfaces
!     ishap1
!     issub          Loop variable (local subsurface)
!     issubf         Subsubsurface sequence number
!     istart         Starting address in array VECOLD
!     isub           Loop variable
!     isubsf         Subsurface sequence number
!     jssubf         Subsurface sequence number
!     jstarr
!     jsubsf         Sequence number of subsurface
!     jsurf
!     kbndlc         Local array KBNDPT corresponding to surface
!     kstart
!     leng3
!     m1             Number of elements in vertical direction per row of
!                    surfaces
!     mtot
!     n1             Number of elements in horizontal direction per row of
!                    surfaces
!     ndefsf         Number of subsurfaces in surface
!     nelloc         Number of elements in local subsurface
!     nelslc         Number of elements in local subsubsurface
!     ngboun         Number of curves the boundary of the surface of surface
!                    consists of
!     nloc
!     npoloc         Number of points in local subsurface
!     nposlc         Number of points in local subsubsurface
!     nrel
!     nrow           Number of rows of surfaces in ordered surface of surfaces
!     ns
!     nsubsf         Number of subsurfaces of subsurface
!     ntot
!     nwork          Last position used by first part of IWORK
! **********************************************************************
!
!                       SUBROUTINES CALLED
!

!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHFL3         Fill array KBNDPT for an ordered surface
!     MSHFL4         Fill array KBNDPT for an ordered surface of type
!                    pipe-surface
!     PRININ         print 1d integer array
!     READFL         Read integer array from a direct access SEPRAN file
!     WRITFL         Write array to SEPRAN direct access file
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
!     633   Subsurface of ordered surface is not of type RECTANGLE
!           or PIPESURFACE
!     634   Number of nodes in second direction of ordered surface of a row
!           is not constant.
!     635   Elements in ordered surface of surfaces have not all the same shape.
!     636   Number of nodes in first direction of ordered surface not constant.
!    1514   Number of points at opposite sides not equal
!    2374   A subsubsurface of a subsurface has a negative value
!    2468   nelmax too small
!    2469   npomxs too small
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshfil1112' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshfil1112'
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      ipoint = npomxs + nelmax
      nwork = ipoint+nsurfs
      nrow = iinput(iposi+2)
      mtot = 0
      jstarr = kmeshr(nsurfs+1) - nsurfs-1 + kmeshr(isurnr)

!     --- Loop over rows of subsurfaces

      kstart = iposi + nrow + 2
      do irow = 1, nrow
         ndefsf = mod(iinput(iposi+irow+2),10000)

!        --- Loop over subsurfaces in one row

         do isub = 1, ndefsf
            isubsf = abs( iinput(kstart+isub) )

!           --- Compute m and n for isubsf

            ns = nsurfs + 2 + (isubsf-1) * 8
100         jsurf  = kmeshs(ns)
            ishape = kmeshs(ns+1)
            if ( debug ) &
                write(irefwr,1) 'isub, isubsf, jsurf, ishape', &
                                 isub, isubsf, jsurf, ishape

            if ( jsurf==1 .or. jsurf==17 ) then

!           --- jsurf = 1, 17   rectangle

               nmes = kmeshs(ns+2)
               mmes = kmeshs(ns+3)

            else if ( jsurf==11 ) then

!           --- jsurf = 11  ordered surface of surfaces

               nmes = kmeshs(ns+4)
               mmes = kmeshs(ns+5)

            else if ( jsurf==13 .or. jsurf==15 ) then

!           --- jsurf = 13 or 15  coons or parsurf

               nmes = kmeshs(ns+2)
               mmes = kmeshs(ns+3)
               if ( nmes/=kmeshs(ns+4) .or. mmes/=kmeshs(ns+5) ) then
                  if ( jsurf==13 ) then
                     call errchr ( 'coons', 1 )
                  else
                     call errchr ( 'parsurf', 1 )
                  end if
                  call errint ( isurnr, 1 )
                  call errint ( nmes, 2 )
                  call errint ( kmeshs(ns+4), 3 )
                  call errint ( mmes, 4 )
                  call errint ( kmeshs(ns+5), 5 )
                  call errsub ( 1514, 5, 0, 1 )
               end if

            else if ( jsurf==8 .or. jsurf==9 .or. jsurf==14 &
                      .or. jsurf==16 ) then

!           --- jsurf = 8,9,14,16   translate, rotate, similar, reflect

               iprec = kmeshs(ns+2)
               ns = nsurfs + 2 + (iprec-1) * 8
               goto 100

            else if ( jsurf==10 .or. jsurf==12 ) then

!           --- jsurf = 10   pipe surface

               nmes = kmeshs(ns+6)
               mmes = kmeshs(ns+7)

            else

!           --- jsurf # 1, 7, 8, or 10
!               Ordered surface of surfaces not possible

              call errint ( isurnr,1 )
              call errint ( jsubsf,2 )
              call errint ( jsurf, 3 )
              call errsub ( 633, 3, 0, 0 )
              go to 1000

            end if
            if ( isub==1 ) then
               m1 = mmes
               n1 = nmes
               if ( irow==1 ) ishap1 = ishape
            else
               if ( m1/=mmes ) then
                  call errint ( iinput(kstart+1), 1 )
                  call errint ( m1, 2 )
                  call errint ( iinput(kstart+isub), 3 )
                  call errint ( mmes, 4 )
                  call errint ( isurnr, 5 )
                  call errsub ( 634, 5, 0, 0 )
               end if
               n1 = n1 + nmes
            end if
            if ( ishape/= ishap1 ) then
               call errint ( ishape, 1 )
               call errint ( ishap1, 2 )
               call errint ( isurnr, 3 )
               call errsub ( 635, 3, 0, 0 )
            end if
            iwork(ipoint+isub) = nmes
            if ( debug ) write(irefwr,1) 'n, m, n1, m1', nmes, mmes, n1,m1
         end do
         if ( irow==1 ) then
            ntot = n1
         else
            if ( n1/=ntot ) then
               call errint ( n1, 1 )
               call errint ( ntot, 2 )
               call errint ( isurnr, 3 )
               call errsub ( 636, 3, 0, 0 )
            end if
         end if
         nrel = 0
         mtot = mtot + m1
         if ( debug )  write(irefwr,1) 'mtot', mtot
         do isub = 1, ndefsf
            jsubsf = iinput(kstart+isub)
            isubsf = abs(jsubsf)
            if ( jsubsf>0 ) then
               idirec = 1
            else
               idirec = -1
            end if
            if ( debug ) write(irefwr,1) 'isub, jsubsf, idirec', &
                                          isub, jsubsf, idirec
            npoloc = abs(kmeshh(1,isubsf+1))
            nelloc = kmeshh(2,isubsf+1)
            npomxs = max(npomxs,npoloc)
            nelmax = max(nelmax,nelloc)

            leng3  = kmeshh(4,isubsf+1)+1

            allocate ( kbndlc(leng3), stat = error )
            if ( error/=0 ) call eralloc ( error, leng3, 'kbndlc' )
            if ( ierror/=0 ) go to 1000

            kbndlc(1:leng3) = kmhsur(isubsf)%kbndptsur(1:leng3)

!           --- Fill array kbndpt for surface isubsf

            istart = kmeshs(isubsf)
            nboun = kmeshs(isubsf+1) - istart
            nloc = iwork(ipoint+isub)

!           --- Fill array iwork with the signs of the curve numbers as
!               present in array kmeshs

            ngboun = kmeshs(isurnr+1) - kmeshs(isurnr)
            index = kmeshs(isurnr)
            do i = 1, ngboun
               iwork(nwork+abs(kmeshs(index+i))) = &
                     kmeshs(index+i)
            end do

            if ( isurf==11 ) then

               call mshfl3 ( kbndlc, kbndpt, icurvs, iwork, nloc, m1, &
                             n1, kmeshs(istart+1), nrel, &
                             npoloc, idirec, iwork(nwork+1) )

            else

               call mshfl4 ( kbndlc, kbndpt, icurvs, iwork, nloc, m1, &
                             n1, kmeshs(istart+1), nrel, &
                             npoloc, idirec, isubsf, irow, nrow, &
                             isub, ndefsf, iwork(nwork+1), &
                             neltot, nptot )

            end if
            nsubsf = kmeshr(isubsf+1)-kmeshr(isubsf)
            if ( debug ) then

!           --- debug information

               write(irefwr,1) 'isubsf, nsubsf,npoloc,nelloc', &
                                isubsf, nsubsf,npoloc,nelloc
               call prinin ( iwork, npoloc, 'iwork_points' )
               call prinin ( iwork(npoloc+1), nelloc, &
                             'iwork_elements' )
            end if
            if ( nsubsf<=1 ) then

!           --- Standard situation subsurface is a standard surface

               index = ifree3
               call writfl ( index, 1, npoloc+nelloc, 1, 3, iwork )
               jstarr = jstarr+1
               kmeshr(jstarr) = index

            else

!           --- subsurface consists itself of subsurfaces
!               Array iwork must be mapped on each subsurface

               do issub = 1, nsubsf

                  jssubf = kmeshr(kmeshr(isubsf)+issub)
                  if ( jssubf<0 ) then

!                 --- negative sub surface number, may be wrong

                     call errint ( isubsf, 1 )
                     call errint ( jssubf, 2 )
                     call errsub ( 2374, 2, 0, 0 )
                     go to 1000

                  end if
                  issubf = abs(jssubf)
                  nelslc = kmeshh(2,issubf+1)
                  nposlc = abs(kmeshh(1,issubf+1))
                  index = kmeshr(kmeshr(nsurfs+1)-nsurfs-1+ &
                                 kmeshr(isubsf)+issub)

!                 --- Read iwork of subsubsurface

                  if ( nelmax<nelloc ) then

!                 --- nelmax too small

                     call errint ( nelloc, 1 )
                     call errint ( nelmax, 2 )
                     call errsub ( 2468, 2, 0, 0 )
                     go to 1000

                  end if
                  if ( npomxs<npoloc ) then

!                 --- npomxs too small

                     call errint ( npoloc, 1 )
                     call errint ( npomxs, 2 )
                     call errsub ( 2469, 2, 0, 0 )
                     go to 1000

                  end if
                  if ( nwork+nposlc+nelslc>leniwork ) then

!                 --- Array iwork too small

                     call errint ( nwork+nposlc+nelslc, 1 )
                     call errint ( leniwork, 2 )
                     call errchr ( 'iwork', 1 )
                     call errsub ( 550, 2, 0, 1 )
                     go to 1000

                  end if
                  call readfl ( index, 1, nposlc+nelslc, 1, 3, &
                                iwork(nwork+1) )

!                 --- Transform to new surface

                  do i = 1, nposlc
                     iwork(nwork+i) = iwork(iwork(nwork+i))
                  end do
                  do i = 1, nelslc
                     iwork(nwork+i+nposlc) = &
                        iwork(npoloc+iwork(nwork+i+nposlc))
                  end do
                  index = ifree3
                  call writfl ( index, 1, nposlc+nelslc, 1, 3, &
                                iwork(nwork+1) )
                  jstarr = jstarr+1
                  kmeshr(jstarr) = index
                  if ( debug ) then

!                 --- debug information

                     write(irefwr,1) 'isubsf, nsubsf,issub', &
                                      isubsf, nsubsf,issub
                     write(irefwr,1) 'nposlc,nelslc', nposlc,nelslc
                     write(irefwr,1) 'npoloc, nwork', npoloc, nwork
                     call prinin ( iwork(nwork+1), nposlc, &
                                   'iwork_points' )
                     call prinin ( iwork(nwork+1+nposlc), nelslc, &
                                   'iwork_elements' )
                  end if
               end do

            end if

            nrel = nrel + nloc

            deallocate ( kbndlc, stat = error )
            if ( error/=0 ) call erdealloc ( error, 'kbndlc' )
            if ( ierror/=0 ) go to 1000

         end do
         kstart = kstart + ndefsf
         if ( ishape==3 ) then
            npointlc = npointlc + (n1+1) * m1
            nelemlc  = nelemlc  + 2 * n1 * m1
         else if ( ishape==4 .or. ishape==7 ) then
            npointlc = npointlc + (2*n1+1) * 2 * m1
            nelemlc  = nelemlc  + 2 * n1 * m1
            if ( ishape==7 ) npointlc = npointlc + 2*n1*m1
         else if ( ishape==5 ) then
            npointlc = npointlc + (n1+1) * m1
            nelemlc  = nelemlc  +  n1 * m1
         else if ( ishape==6 ) then
            npointlc = npointlc + (2*n1+1) * 2 * m1
            nelemlc  = nelemlc  +  n1 * m1
         end if
         if ( isurf==12 .and. icirc==1 ) npointlc = npointlc - m1

      end do
      if ( ishape==3 .or. ishape==5 ) then
         npointlc = npointlc+ n1 + 1
      else
         npointlc = npointlc + 2*n1 + 1
      end if
      ns = nsurfs + 2 + (isurnr-1) * 8
      kmeshs(ns+1) = ishape
      if ( isurf==11 ) then
         kmeshs(ns+4) = n1
         kmeshs(ns+5) = mtot
      else
         kmeshs(ns+6) = n1
         kmeshs(ns+7) = mtot
      end if
      if ( debug ) write(irefwr,1) 'isurnr, isurf, ns, n1, mtot', &
                                    isurnr, isurf, ns, n1, mtot

1000  if ( debug ) then

         write(irefwr,*) 'End mshfil1112'

      end if  ! ( debug )
      call erclos ( 'mshfil1112' )
      end subroutine mshfil1112
