      subroutine mshiorcr ( iorien, locsur, nx, ny, nz, bcord, ishape )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 23-08-2010 Remove array work from parameter list
!        version  2.1  date 25-06-2004 Correction for shape 7
!        version  2.0  date 19-08-2002 Extra parameter ishape
!                                      Extension for shape 7
!        version  1.0  date 07-03-1997
!
!   copyright (c) 1997-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Rearrange array bcord such that the new orientation of the
!     surface is taken into account
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     volume
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iorien, locsur, nx, ny, nz, ishape
      double precision bcord(3,*)

!     bcord         i/o   array containing the coordinates of the boundaries
!                         bcord refers to one boundary only
!     iorien         i    Orientation of axis
!     ishape         i    Shape number of elements
!     locsur         i    Local surface number with respect to brick
!     nx             i    number of points in x direction
!     ny             i    number of points in y direction
!     nz             i    number of points in z direction
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer n1, n2, idim, i, j, ishape2d, m1 ,m2
      double precision, allocatable :: work(:,:)

!     error          Return value of allocate or deallocate
!     i              Counting variable
!     idim           dimension of the space
!     ishape2d       Shape number corresponding to 2d elements
!                    Only shape = 7 is used, &
!                    in all other cases shape is set to 0
!     j              Counting variable
!     m1             Number of elements in the first direction of the surface
!     m2             Number of elements in the second direction of the surface
!     n1             Number of nodes in the first direction of the surface
!     n2             Number of nodes in the second direction of the surface
!     work           Work array of the same length as bcord
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHCOORCENTR7  Fill coordinates of centroid of 7 point triangle when the
!                    coordinates of the vertices and midside points are given
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2910   Not enough space
!    2911   Not possible to deallocate
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The sequence corresponding to iorien is as follows:
!                                             i               j
!     .-------.       .-------.         .-------X          X-------.
!     |       |       |       |         |       |          |       |
!  j  |       |       |       | i       |       | j      i |       |
!     |       |       |       |         |       |          |       |
!     X-------.       .-------X         .-------.          .-------.
!         i               j
!     iorien=0         iorien=1         iorien=2           iorien=3
!     standard
!                                             j               i
!     .-------.       .-------.         .-------X          X-------.
!     |       |       |       |         |       |          |       |
!  i  |       |       |       | j       |       | i      j |       |
!     |       |       |       |         |       |          |       |
!     X-------.       .-------X         .-------.          .-------.
!         j               i
!     iorien=4         iorien=5         iorien=6           iorien=7
!     Set n1 and n2 depending on locsur
!     Copy array bcord into work
!     if ( iorien=1 ) then
!        bcord_new(n1-j+1,i) = bcord_old(i,j)  (j=1(1)n1,i=1(1)n2)
!     else if ( iorien=2 ) then
!        bcord_new(n1-i+1,n2-j+1) = bcord_old(i,j)  (i=1(1)n1,j=1(1)n2)
!     else if ( iorien=3 ) then
!        bcord_new(j,n2-i+1) = bcord_old(i,j)  (j=1(1)n1,i=1(1)n2)
!     else if ( iorien=4 ) then
!        bcord_new(j,i) = bcord_old(i,j)  (j=1(1)n1,i=1(1)n2)
!     else if ( iorien=5 ) then
!        bcord_new(n1-i+1,j) = bcord_old(i,j)  (i=1(1)n1,j=1(1)n2)
!     else if ( iorien=6 ) then
!        bcord_new(n1-j+1,n2-i+1) = bcord_old(i,j)  (j=1(1)n1,i=1(1)n2)
!     else if ( iorien=7 ) then
!        bcord_new(i,n2-j+1) = bcord_old(i,j)  (i=1(1)n1,j=1(1)n2)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshiorcr' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshiorcr'

      end if
      if ( ierror/=0 ) go to 1000

!     --- Set n1 and n2 depending on locsur

      if ( locsur==1 .or. locsur==6 ) then
         n1 = nx
         n2 = ny
      else if ( locsur==2 .or. locsur==4 ) then
         n1 = nx
         n2 = nz
      else
         n1 = ny
         n2 = nz
      end if

      if ( ishape==15 .or. ishape==16 ) then

!     --- Special case, 7 point triangles on surface

         ishape2d = 7
         m1 = n1/2
         m2 = n2/2

      else

!     --- Standard case, ishape2d is not really used, so a arbitrary value
!         is set

         ishape2d = 0
         m1 = 0
         m2 = 0

      end if

      if ( debug ) then

!     --- debug information

         write(irefwr,*) 'nx,ny,nz,n1,n2,iorien,locsur,ishape ', &
                          nx,ny,nz,n1,n2,iorien,locsur,ishape
         call prinrl1(bcord,n1*n2+m1*m2*2,3,'bcord-in')

      end if

      if ( ishape2d==0 ) then

!     --- Standard case
!         Copy array bcord into work

         allocate ( work(3,n1*n2), stat = error )
         if ( error/=0 ) then

!        --- error#0, so not enough space

            call errint ( 3*n1*n2, 1 )
            call errint ( error, 2 )
            call errchr ( 'work', 1 )
            call errsub ( 2910, 2, 0, 1 )
            go to 1000

         end if  ! ( error/= 0 )

         work(1:3,1:n1*n2) = bcord(1:3,1:n1*n2)

         if ( iorien==1 ) then

!        --- iorien=1
!            bcord_new(n1-j+1,i) = bcord_old(i,j)  (j=1(1)n1,i=1(1)n2)

            do idim = 1, 3
               do j = 1, n1
                  do i = 1, n2
                     bcord(idim,n1-j+1+(i-1)*n1) = work(idim,i+(j-1)*n2)
                  end do
               end do
            end do

         else if ( iorien==2 ) then

!        --- iorien=2
!            bcord_new(n1-i+1,n2-j+1) = bcord_old(i,j)  (i=1(1)n1,j=1(1)n2)

            do idim = 1, 3
               do j = 1, n2
                  do i = 1, n1
                     bcord(idim,n1-i+1+(n2-j)*n1) = &
                        work(idim,i+(j-1)*n1)
                  end do
               end do
            end do

         else if ( iorien==3 ) then

!        --- iorien=3
!            bcord_new(j,n2-i+1) = bcord_old(i,j)  (j=1(1)n1,i=1(1)n2)

            do idim = 1, 3
               do j = 1, n1
                  do i = 1, n2
                     bcord(idim,j+(n2-i)*n1) = work(idim,i+(j-1)*n2)
                  end do
               end do
            end do

         else if ( iorien==4 ) then

!        --- iorien=4
!            bcord_new(j,i) = bcord_old(i,j)  (j=1(1)n1,i=1(1)n2)

            do idim = 1, 3
               do j = 1, n1
                  do i = 1, n2
                     bcord(idim,j+(i-1)*n1) = work(idim,i+(j-1)*n2)
                  end do
               end do
            end do

         else if ( iorien==5 ) then

!        --- iorien=5
!            bcord_new(n1-i+1,j) = bcord_old(i,j)  (i=1(1)n1,j=1(1)n2)

            do idim = 1, 3
               do j = 1, n2
                  do i = 1, n1
                     bcord(idim,n1-i+1+(j-1)*n1) = work(idim,i+(j-1)*n1)
                  end do
               end do
            end do

         else if ( iorien==6 ) then

!        --- iorien=6
!            bcord_new(n1-j+1,n2-i+1) = bcord_old(i,j)  (j=1(1)n1,i=1(1)n2)

            do idim = 1, 3
               do j = 1, n1
                  do i = 1, n2
                     bcord(idim,n1-j+1+(n2-i)*n1) = &
                        work(idim,i+(j-1)*n2)
                  end do
               end do
            end do

         else if ( iorien==7 ) then

!        --- iorien=7
!            bcord_new(i,n2-j+1) = bcord_old(i,j)  (i=1(1)n1,j=1(1)n2)

            do idim = 1, 3
               do j = 1, n2
                  do i = 1, n1
                     bcord(idim,i+(n2-j)*n1) = work(idim,i+(j-1)*n1)
                  end do
               end do
            end do

         end if

      else

!     --- Special case of 7 point triangles
!         Copy array bcord into work

         allocate ( work(3,n1*n2+2*m1*m2), stat = error )
         if ( error/=0 ) then

!        --- error#0, so not enough space

            call errint ( 3*n1*n2+2*m1*m2, 1 )
            call errint ( error, 2 )
            call errchr ( 'work', 1 )
            call errsub ( 2910, 2, 0, 1 )
            go to 1000

         end if  ! ( error/= 0 )

         work(1:3,1:n1*n2+2*m1*m2) = bcord(1:3,1:n1*n2+2*m1*m2)

         if ( iorien==1 ) then

!        --- iorien=1
!            first vertices and mid points

            do idim = 1, 3
               do j = 1, n1
                  do i = 1, n2
                     bcord(idim,n1+(i-1)*(n1+m1)-(j-1)) = &
                        work(idim,i+(j-1)*(n2+m2))
                  end do
               end do
            end do

!           --- Next centroids of triangles

            call mshcoorcentr7 ( bcord, m1, m2 )

         else if ( iorien==2 ) then

!        --- iorien=2
!            first vertices and mid points

            do idim = 1, 3
               do i = 1, n1
                  do j = 1, n2
                     bcord(idim,n1*n2+(n2-1)*m1-(j-1)*(n1+m1)-(i-1)) = &
                        work(idim,i+(j-1)*(n1+m1))
                  end do
               end do
            end do

!           --- Next centroids of triangles

            call mshcoorcentr7 ( bcord, m1, m2 )

         else if ( iorien==3 ) then

!        --- iorien=3
!            first vertices and mid points

            do idim = 1, 3
               do j = 1, n1
                  do i = 1, n2
                     bcord(idim,1+(n2-1)*(n1+m1)-(i-1)*(n1+m1)+(j-1)) = &
                        work(idim,i+(j-1)*(n2+m2))
                  end do
               end do
            end do

!           --- Next centroids of triangles

            call mshcoorcentr7 ( bcord, m1, m2 )

         else if ( iorien==4 ) then

!        --- iorien=4
!            first vertices and mid points

            do idim = 1, 3
               do j = 1, n1
                  do i = 1, n2
                     bcord(idim,1+(i-1)*(n1+m1)+(j-1)) = &
                        work(idim,i+(j-1)*(n2+m2))
                  end do
               end do
            end do

!           --- Next centroids of triangles

            call mshcoorcentr7 ( bcord, m1, m2 )

         else if ( iorien==5 ) then

!        --- iorien=5
!            first vertices and mid points

            do idim = 1, 3
               do i = 1, n1
                  do j = 1, n2
                     bcord(idim,n1+(j-1)*(n1+m1)-(i-1)) = &
                        work(idim,i+(j-1)*(n1+m1))
                  end do
               end do
            end do

!           --- Next centroids of triangles

            call mshcoorcentr7 ( bcord, m1, m2 )

         else if ( iorien==6 ) then

!        --- iorien=6
!            first vertices and mid points

            do idim = 1, 3
               do j = 1, n1
                  do i = 1, n2
                     bcord(idim,n2*n1+(n2-1)*m1-(i-1)*(n1+m1)-(j-1)) = &
                        work(idim,i+(j-1)*(n2+m2))
                  end do
               end do
            end do

!           --- Next centroids of triangles

            call mshcoorcentr7 ( bcord, m1, m2 )

         else if ( iorien==7 ) then

!        --- iorien=7
!            first vertices and mid points

            do idim = 1, 3
               do i = 1, n1
                  do j = 1, n2
                     bcord(idim,1+(n2-1)*(n1+m1)-(j-1)*(n1+m1)+(i-1)) = &
                        work(idim,i+(j-1)*(n1+m1))
                  end do
               end do
            end do

!           --- Next centroids of triangles

            call mshcoorcentr7 ( bcord, m1, m2 )

         end if

      end if

      if ( debug ) then

!     --- debug information

         call prinrl1(bcord,n1*n2+2*m1*m2,3,'bcord-out')

      end if

1000  deallocate ( work, stat = error )
      if ( error/= 0 ) then

!     --- error#0, problem with deallocate

         call errint ( error, 1 )
         call errchr ( 'work', 1 )
         call errsub ( 2911, 1, 0, 1 )

      end if  ! ( error/= 0 )

      call erclos ( 'mshiorcr' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshiorcr'

      end if

      end
