      subroutine mshobstboun3d ( maploc, voltot )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.10 date 27-11-2012 Long integers
!        version  1.9  date 22-11-2011 New call to mshobstsurf
!        version  1.8  date 20-07-2011 New call to plotobstacle
!
!   copyright (c) 2002-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if there are elements that have all nodes on the obstacle or on the
!     obstacle and an outer boundary
!     If this is the case subdivide these elements into three elements
!     by adding an extra point in the centroid
!     This subroutine can only be applied for linear tetrahedrons
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     3d
!     surface
!     element
!     obstacle
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      use sepmodulemesh
      use sepmoduledebug
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision voltot
      type (mapping) :: maploc

!     maploc        i/o   Mapping array
!     voltot         i    Total volume of region
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer idimedges, i, lenkmeshc, mmipointobst, mmnmeshc, mmnmeshd, &
              npointnew, numbounelms, lenkmeshn, npointsav, &
              krenum(1), lenielemobst, nmap, npointnewsav, &
              mmioutersurfs, mmnmeshk, mmnmeshl, idummy
      double precision voltmp, volumes(3)
      integer, allocatable :: ielemobstlc(:), iworkedge(:), &
                              kmeshcnew(:), iwork(:), ibounelems(:), &
                              kmeshnnew(:)
      double precision, allocatable :: coornew(:,:)

!     coornew        Array containing the new coordinates of the nodes
!     i              Counting variable
!     ibounelems     Array of length numbounelms (<nelem) containing
!                    the element numbers of the elements that have all
!                    points on the boundary or on the obstacle
!     idimedges      Length of first dimension of array iedges
!                    (linear:1, quadratic: 3)
!     idummy         Dummy parameter
!     ielemobstlc      Array containing information about the elements in an
!                    obstacle
!                    The first numobst positions contain the starting
!                    positions of each obstacle sequentially
!                    Per obstacle the first three positions contain
!                    1. The number of elements that are completely inside
!                       the obstacle
!                    2. The number of elements that are not completely
!                       inside the element but with all vertices inside
!                       the element
!                    3. The number of elements that are partly inside
!                       the obstacle
!                       elements that contain only boundary points of the
!                       obstacle are not considered
!                    These positions are followed by the element numbers
!                    in the sequence defined by the first three positions
!     iwork          integer work array of length npoint
!                    iwork(i) is filled as follows (see pseudo code)
!                     0 for all points outside the obstacle and
!                       the boundaries
!                    -1 for all points inside the obstacle
!                     i for all points on the boundary of the obstacle i
!                    1001 for all points on the outer surfaces
!                         but not on an obstacle
!                    1002 for all points on the outer surfaces
!                         and on an obstacle
!     iworkedge      work array of length nedges to be used to mark
!                    edges
!                     0:  not marked yet
!                    -1:  boundary edge that has been used already
!                    -2:  diagonal of quadrilateral that may be removed
!     kmeshcnew      New kmesh part c
!     kmeshnnew      New kmesh part n
!     krenum         Parameter jrenum for TOPOL
!     lenielemobst   Computed length of array ielemobst
!     lenkmeshc      length of array kmeshc (in first instance estimated)
!     lenkmeshn      Length of array kmeshn (in first instance estimated)
!     mmioutersurfs  memory management sequence number of array ioutersurfs
!     mmipointobst   memory management sequence number of array ipointobst
!     mmnmeshc       memory management sequence number of array nmeshc
!     mmnmeshd       memory management sequence number of array nmeshd
!     mmnmeshk       memory management sequence number of array nmeshk
!     mmnmeshl       memory management sequence number of array nmeshl
!     nmap           length of second parameter of imap and rmap
!     npointnew      Sequence number of last generated node
!     npointnewsav   Help variable to store the old value of npointnew
!     npointsav      Help variable to store the old value of npoint
!     numbounelms    Number of elements that have all points on the
!                    boundary of the obstacle or a set of curves
!     voltmp         Temporary computed value of volume of region
!     volumes        Array of length 3 containing the volume of the
!                    following subparts:
!                    1: all elements completely inside the obstacle
!                    2: all elements partly inside the obstacle
!                    3: all elements outside the obstacle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHDELSEL      Delete a selected set of subarrays of kmesh
!     MSHOBSTBOUN3D1 Find all elements that have all nodes on the obstacle or
!                    on the obstacle and an outer boundary
!     MSHOBSTBOUN3D2 Create new points in mesh by subdividing some edges of
!                    elements that have all points on the boundary of the mesh
!                    and/or the obstacle
!     MSHOBSTBOUN3D3 create new elements in case new nodes have been made
!                    in combination with elements with all nodes on boundary
!                    and obstacle
!     MSHOBSTMARKELN Mark all elements in the mesh with information if they are
!                    within an obstacle or outside an obstacle
!     MSHOBSTSURF    Create new array kmeshn in the case that extra points at
!                    the surface are created when a new mesh is created due to
!                    intersection with an obstacle (3D only)
!     MSHOBSTVOL3D   Compute volume of part of region inside, outside and
!                    near obstacle
!     PLOTOBSTACLE   Print and plot information about the obstacle
!     PRINMAP1       Actual printing of array imap
!     SEPFILLKMESHC  Make relation between kmeshc and Topgr
!     SEPFILLNMESHC  Fill array nmeshc
!     SEPFILLNMESHK  Fill array nmesh part k
!     SEPFILLNMESHL  Fill array nmesh part l
!     SEPFILLOUTERB  Fill outer boundary of mesh
!     SEPFLPOINTOBST Fill array kmesh ad  part 1: ipointobst
!     SEPRMNMESH     Remove subarrays of nmesh
!     TOPOL          Fill integer information corresponding to the mesh in KMESH
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2292   More than one obstacle
!    2338   Subdivision incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
!     numbounelms := number of elements that have all nodes on the boundary
!     if ( numbounelms>0 ) then
!         Create new points and new elements
!         Adapt arrays ipointobst and ielemobst
!         Adapt arrays imap and rmap
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobstboun3d' )
      debug = debug_mshobstmesh

!     --- numbounelms := number of elements that have all nodes on the boundary

      idimedges = 1
      if ( numobst>1000 ) then

!     --- numobst>1000

         call errint ( numobst, 1 )
         call errsub ( 2292, 1, 0, 0 )
         go to 1000

      end if

!     --- Create or activate array ipointobst

      call sepflpointobst

!     --- Create or activate array outersurfs containing the outer curves

      if ( mmouter==0 ) call sepfillouterb

!     --- Next create the semi-permanent arrays iwork and ibounelems
!         these are deleted later on

      allocate ( iwork(npoint), ibounelems(3+nelem), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint+3+nelem, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Find all elements that have all nodes on the obstacle or on the
!         obstacle and an outer boundary

      call mshobstboun3d1 ( iwork, numbounelms, ibounelems )
      if ( ierror/=0 ) go to 1000

      if ( numbounelms>0 ) then

!     --- numbounelms>0
!         Create new points if necessary
!         First activate arrays

         if ( nmesh(33)==0 ) call sepfillnmeshc
         if ( nmesh(41)==0 ) call sepfillnmeshk
         if ( nmesh(42)==0 ) call sepfillnmeshl

!        --- Create space for the coordinates array
!            Also for new arrays imap, rmap, iworkedge

         npointnew = npoint+nedges
         npointnewsav = npointnew
         nmap = 4

         allocate ( coornew(3,npointnew), rmap2(npointnew,nmap), &
                    imap2(npointnew,nmap+1), iworkedge(nedges), &
                    stat = error )
         if ( error/=0 ) &
            call eralloc ( error, (3+nmap)*npointnew+nelem+nedges, 'coornew' )
         if ( ierror/=0 ) go to 1000

!        --- Clear arrays imap2 and rmap2

         imap2 = 0
         rmap2 = 0d0
         imap => maploc%imap
         rmap => maploc%rmap

         call mshobstboun3d2 ( ibounelems, iworkedge, npointnew, coornew, &
                               iwork, nmap )
         if ( ierror/=0 ) go to 1000

         if ( npointnew>npoint ) then

!        --- npointnew>npoint, create new elements
!            Adapt arrays kmeshc and ielemobst
!            Adapt arrays imap and rmap
!            First copy coor to coornew

            coornew(:,1:npoint) = coor(:,1:npoint)

!           --- Make new arrays imap and rmap permanent

            deallocate ( maploc%imap, maploc%rmap )
            allocate ( maploc%imap(npointnew,nmap+1), &
                       maploc%rmap(npointnew,nmap) )
            imap => maploc%imap
            rmap => maploc%rmap
            imap = imap2(1:npointnew,1:nmap+1)
            rmap = rmap2(1:npointnew,1:nmap)

!           --- Next replace coor by coornew and delete coornew

            if ( km(iseqkm)%kmesh(23)>0 ) deallocate(km(iseqkm)%coor)
            allocate(km(iseqkm)%coor(3,npointnew))
            coor => km(iseqkm)%coor
            coor = coornew(:,1:npointnew)
            km(iseqkm)%kmesh(23) = npointnew
            km(iseqkm)%kmesh(8) = npointnew

!           --- Create new kmeshc and kmeshn at the end of idummy

            lenkmeshc = size ( kmeshc )+(npointnew-npoint)*48
            lenkmeshn = size ( kmeshn )*4
            npointsav = npoint
            npoint = npointnew

            allocate ( kmeshcnew(lenkmeshc), kmeshnnew(lenkmeshn), &
                       stat = error )
            if ( error/=0 ) &
               call eralloc ( error, lenkmeshc+lenkmeshn, 'kmeshcnew' )
            if ( ierror/=0 ) go to 1000

!           --- Set starting addresses

            if ( ierror/=0 ) go to 1000

!           --- create new elements

            call mshobstboun3d3 ( kmeshcnew, iworkedge, lenkmeshc )
            km(iseqkm)%kmesh(9) = nelem
            if ( ierror/=0 ) go to 1000

!           --- Adapt kmeshn

            call mshobstsurf ( kmeshnnew, lenkmeshn, iworkedge, &
                               idummy, idimedges, 3 )

!           --- Next replace kmeshc by kmeshcnew and kmeshn by kmeshnnew

            if ( km(iseqkm)%kmesh(17)>0 ) deallocate(km(iseqkm)%kmeshc)
            allocate(km(iseqkm)%kmeshc(lenkmeshc))
            kmeshc => km(iseqkm)%kmeshc
            kmeshc = kmeshcnew(1:lenkmeshc)
            km(iseqkm)%kmesh(17) = lenkmeshc
            call sepfillkmeshc

            if ( km(iseqkm)%kmesh(28)>0 ) deallocate(km(iseqkm)%kmeshn)
            allocate(km(iseqkm)%kmeshn(lenkmeshn))
            kmeshn => km(iseqkm)%kmeshn
            kmeshn = kmeshnnew(1:lenkmeshn)
            km(iseqkm)%kmesh(28) = lenkmeshn

            deallocate ( kmeshcnew, kmeshnnew, stat = error )
            if ( error/=0 ) call erdealloc ( error, 'kmeshcnew' )
            if ( ierror/=0 ) go to 1000

!           --- Recompute topology after removing some subarrays

            call seprmnmesh

!           --- Mark elements that are on the obstacle

            lenielemobst = 6*km(iseqkm)%kmesh(42)+nelem

            if ( associated(ielemobst) ) nullify(ielemobst)
            allocate ( ielemobst(lenielemobst), stat = error )
            if ( error/=0 ) call eralloc ( error, lenielemobst, 'ielemobst' )
            if ( ierror/=0 ) go to 1000

!           --- Fill array

            call mshobstmarkeln ( ipointobst, &
                                  ielemobst, kmeshc, lenielemobst )
            npoint = npointsav
            km(iseqkm)%kmesh(8) = npoint

!           --- Make array permanent

            kelmad = km(iseqkm)%kmesh(52)
            if ( associated(km(iseqkm)%ielemobst) ) &
               deallocate(km(iseqkm)%ielemobst)
            allocate ( km(iseqkm)%ielemobst(lenielemobst) )
            km(iseqkm)%ielemobst = ielemobst(1:lenielemobst)

!           --- Delete some parts of kmesh, because the mesh has been changed

            call mshdelsel ( kmesh )
            krenum(1) = 9
            call topol ( krenum(1) )

            if ( debug .or. plotobst .or. printobst ) then

!           --- debug = true, plot nodes and elements with a colour
!               First print arrays

               call plotobstacle ( debug_mshobstmesh, 0 )

               if ( debug ) then

!              --- debug true, check volume

                  call prinmap1 ( npointnew )
                  call mshobstvol3d ( volumes )
                  voltmp = volumes(1)+volumes(2)+volumes(3)
                  if ( abs(voltot-voltmp)>1d-13*abs(voltot) ) then

!                 --- Error volume after subsdivision incorrect

                     call erreal ( voltot, 1 )
                     call erreal ( voltmp, 2 )
                     call errsub ( 2338, 0, 2, 0 )

                  end if
                  write(irefwr,200) volumes, voltot
200               format ( 'After second subdivision:'/ &
                           'Volume of elements inside obstacle  ',d17.8/ &
                           'Volume of elements near obstacle    ',d17.8/ &
                           'Volume of elements outside obstacle ',d17.8/ &
                           'Volume of complete region           ',d17.8)

               end if

            end if

         end if ! npointnew>npoint

         deallocate ( coornew, rmap2, imap2, iworkedge, &
                      stat = error )
         if ( error/=0 ) call erdealloc ( error, 'coornew' )
         if ( ierror/=0 ) go to 1000

      end if ! numbounelms>0

!     --- Delete semi-permanent arrays

      deallocate ( iwork, ibounelems, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )
      if ( ierror/=0 ) go to 1000

1000  call erclos ( 'mshobstboun3d' )

      end subroutine mshobstboun3d
