      subroutine mshobstms3d03 ( tetnode, tetnodenew, iworkpoint, &
                                 npointnew, ielemnew, &
                                 coornew, iwork, iedges, nmeshc, &
                                 icrossobst, rcrossobst, ielem, &
                                 ikelmcnew, nmap, inpelm, nelem, edgemoved )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.0  date 26-02-2012 Extra parameter edgemoved
!        version  4.4  date 19-05-2005 New calls to subroutines
!        version  4.3  date 05-04-2005 Correction interpolation
!
!   copyright (c) 2001-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subdivide a tetrahedron if necessary into subtetareders and
!     indicate which of these subtetareders is in or out the obstacle
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     linear
!     tetrahedron
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemap
      use sepmodulecomio
      use sepmodulegen
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpelm, tetnode(inpelm), tetnodenew(8*inpelm), npointnew, &
              ielemnew, nelem, iwork(nelem,*),iedges(3,*), nmeshc(*), &
              icrossobst(*), ielem, ikelmcnew, nmap, &
              iworkpoint(nmap,*), edgemoved(*)

      double precision coornew(3,*), rcrossobst(*)

!     coornew       i/o   Array containing the new coordinates of the nodes
!     edgemoved      i    Array of size nedges indicating if nodes on an edge
!                         have been moved (1) or not (0)
!     icrossobst     i    array containing information about the cross section
!                         of obstacles with edges
!                         The storage of icrossobst is as follows
!                         Pos. 1 to numobst (number of obstacles):
!                               icrossobst(iobst) = istartobst (starting address
!                               of obstacle iobst in array
!                         Per obstacle from position istartobst:
!                         Number of cross sections, followed by 2 positions for
!                         each cross section consisting of the edge sequence
!                         number and a reference to rcrossobst
!                         This part is followed by the number of elements
!                         that is intersected by the obstacle followed by
!                         for each element: element number, number of
!                         intersections and sequence numbers of intersections
!     iedges        i/o   Work array of size nedges
!                         If iedges(i) = 0, no point has been created in edge i
!                         If iedges(i)>0, a point has been created in edge i, &
!                         with sequence number iedges(i)
!     ielem          i    Element number
!     ielemnew      i/o   Sequence number of element in new mesh
!     ikelmcnew     i/o   Starting address of next element in kmeshcnew
!     iwork          i    integer work array of length nelem*2
!                         iwork(ielem,1) contains the starting address of
!                         element ielem in array icrossobst
!                         If 0, the element is not present in icrossobst
!                         iwork(ielem,2) contains the corresponding obstacle
!                         sequence number iobst
!     iworkpoint    i/o   Work array of length npointnew*2
!                         This array is filled as follows:
!                         iworkpoint(i,1) = 0 if node i outside obstacle
!                         iworkpoint(i,1) = 1 if node i inside obstacle
!                         iworkpoint(i,1) = 2 if node i inside obstacle (near
!                                             boundary)
!                         iworkpoint(i,1) = 3 if node i on obstacle
!                         iworkpoint(i,2) contains the corresponding obstacle
!                         sequence number iobst
!     nelem          i    Number of elements
!     nmap           i    length of first parameter of imap and rmap
!     nmeshc         i    nmesh part c
!     npointnew     i/o   Sequence number of last generated node
!     rcrossobst     i    Real array corresponding to icrossobst
!                         For each cross section a real factor f is stored
!                         defining the intersection of the edge with the
!                         obstacle in the following way
!                         x_intersect = x1 + f * (x2-x1)
!                         where x1 and x2 are the first and last point of the
!                         edge respectively
!     tetnode        i    Array of nodal point numbers of the corners of
!                         the original tetrahedron.
!     tetnodenew     o    Array of nodal point numbers of the corners of
!                         the new tetrahedrons.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer edgenode(3,6), istart, icross, nodebegin, nodeend, &
              ipstart, i, iedge, nextranodes, ireledge, &
              nelemnew, istarticross, iobst
      double precision eps, fact

!     edgenode       Array with all nodes on the crossed edge
!                    edgenode(1,1) first node on crossed edge
!                    edgenode(2,1) last node on crossed edge
!                    edgenode(3,1) new node on crossed edge
!     eps            Local accuracy
!     fact           Interpolation factor
!     i              Counting variable
!     icross         Position of iedge in array icrossobst
!     iedge          Edge sequence number
!     inpelm         Number of nodes per element
!     iobst          Obstacle sequence number
!     ipstart        Starting address in array
!     ireledge       relative edge sequence number
!     istart         First position used at a part
!     istarticross   Starting address of obstacle in array icrossobst
!     nelemnew       Number of elements in new mesh
!     nextranodes    Number of extra nodes in element
!     nodebegin      node at the begin of a crossed edge
!     nodeend        node at the end of a crossed edge
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHOBSTMS3D01  Subdivide each tetrahedron into the right number of
!                    subtetrahedrons
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     eps := 0.2
!     nextranodes = 0
!     do i = 1,number of intersections
!        iedge := i-th edge which is intersected
!        fact := interpolation factor
!        if ( cross section with edge close to end point of edge ) then
!           mark corresponding node as on boundary
!        else ( i-th edge is intersected )
!           nextranodes := nextranodes+1
!           edgenode(1,i) := first point of edge
!           edgenode(2,i) := last point on edge
!           if ( first time edge is intersected) then
!              mark edge as intersected
!              introduce the intersection node
!              calculate its coordinates
!              mark it as boundary node
!              store node interpolation data
!           end if
!           edgenode(3,i) := intersection node
!        end if
!     end do
!     if ( nextranodes/=0 ) then
!        count(i) := number of intersected edges which contain tetnode(i)
!        ledgenode(1:2,j) are the local numbers of the corner nodes which lay
!                        on the same edge as intersection node j.
!        isort := the sort index of count, in other words
!                 if ( isort(i)>isort(j) then count(i)>count(j)
!     end if
!     subdivide tetrahedron into the right number of subtetrahedrons, &
!     if ( checksubtetrahedrons) then
!        check volumes of subtetrahedrons
!     end if
!     update pointers ikelmcnew and ielemnew
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobstms3d03' )
      debug = .false.

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobstms3d03'
         write(irefwr,1) 'ielem, ielemnew', ielem, ielemnew
  1      format ( a, 1x, (10i6) )
  4      format ( a, 1x, 2i6, d12.3)
         call prinin ( tetnode, 4, 'tetnode' )
         write(irefwr,1) 'iworkpoint', iworkpoint(tetnode(1:4),1)

      end if
      if ( ierror/=0 ) go to 1000

!     --- set local accuracy eps

      eps = 0.666666667d0*accuracy_obstacle

!     --- nextranodes := 0

      nextranodes = 0

!     --- loop over the cut edges which
!         1. checks whether the crossing point is far enough from the
!            tetrahedron nodes
!         2. If so, introduces a new node
!         3. Set/adjust node properties

      istart = iwork(ielem,1)
      iobst = iwork(ielem,2)
      istarticross = icrossobst(iobst)

      if ( debug ) write(irefwr,1) 'nintersect', icrossobst(istart)

!     --- do i = 1,nintersect

      do i = 1, icrossobst(istart)
         ireledge = icrossobst(istart+i)
         icross  = istarticross+1+2*(ireledge-1)

!        --- iedge := edge which is intersected

         iedge = icrossobst(icross)

         if ( debug ) then
            write(irefwr,1) 'i, ireledge, iedge, edgemoved(iedge)', &
                             i, ireledge, iedge, edgemoved(iedge)
            ipstart = nmeshc(iedge)
            call prinin ( nmeshc(ipstart+1), 2, 'nodes' )
         end if  ! ( debug )

         if ( edgemoved(iedge)==0 ) then

!        --- nodes on edge have not been moved yet

            fact  = rcrossobst(icrossobst(icross+1))
            ipstart = nmeshc(iedge)
            if ( debug ) &
               write(irefwr,4) 'i, iedge, fact', i, iedge, fact
            if ( fact>1d0-eps ) then

!           --- i-th cross section close to begin point of edge
!               mark node corresponding to begin point as on boundary

               nodebegin = nmeshc(ipstart+2)
               iworkpoint(nodebegin,1) = 3
               iworkpoint(nodebegin,2) = iobst
               if ( debug ) write(irefwr,1) 'nodebegin, other node', &
                   nodebegin, nmeshc(ipstart+1)

            else if ( fact<eps ) then

!           --- i-th cross section close to end point of edge
!               mark corresponding node as on boundary

               nodeend = nmeshc(ipstart+1)
               iworkpoint(nodeend,1) = 3
               iworkpoint(nodeend,2) = iobst
               if ( debug ) write(irefwr,1) 'nodeend, other node', &
                  nodeend, nmeshc(ipstart+2)

            else

!           --- i-th edge is intersected
!               increment nextranodes
!               edgenode(1,nextranodes) := first point of edge
!               edgenode(2,nextranodes) := last point on edge

               nextranodes = nextranodes+1
               edgenode(1,nextranodes) = nmeshc(ipstart+1)
               edgenode(2,nextranodes) = nmeshc(ipstart+2)

               if ( iedges(1,iedge)==0 ) then

!              --- New point
!                  npointnew := npointnew+1
!                    Fill coordinates of cross section in coornew(npointnew)
!                    mark node as boundary node

                  npointnew = npointnew+1
                  iedges(1,iedge) = npointnew
                  coornew(1,npointnew) = (1d0-fact)* &
                                      coornew(1,edgenode(1,nextranodes)) &
                                +fact*coornew(1,edgenode(2,nextranodes))
                  coornew(2,npointnew) = (1d0-fact)* &
                                      coornew(2,edgenode(1,nextranodes)) &
                                +fact*coornew(2,edgenode(2,nextranodes))
                  coornew(3,npointnew) = (1d0-fact)* &
                                      coornew(3,edgenode(1,nextranodes)) &
                                +fact*coornew(3,edgenode(2,nextranodes))
                  iworkpoint(npointnew,1) = 3
                  iworkpoint(npointnew,2) = iobst
                  imap(npointnew,1) = 2
                  imap(npointnew,2) = edgenode(1,nextranodes)
                  imap(npointnew,3) = edgenode(2,nextranodes)
                  rmap(npointnew,1) = 1d0-fact
                  rmap(npointnew,2) = fact
                  if ( debug ) &
                     write(irefwr,1) 'npointnew, node1, node2', &
                         npointnew, nmeshc(ipstart+1), nmeshc(ipstart+2)

               end if  ! ( iedges(1,iedge)==0 )

!              --- edgenode(3,nextranodes) := cross section node

               edgenode(3,nextranodes) = iedges(1,iedge)
               if ( debug ) then

!              --- Debug information

                  write(irefwr,1) 'iedge', iedge
                  call prinin ( edgenode(1,nextranodes), 3, 'nodes' )

               end if  ! ( debug )

            end if  ! ( fact>1d0-eps )

         end if  ! ( edgemoved(iedge)==0 )

      end do  ! i = 1, icrossobst(istart)

!     --- Create new elements

      call mshobstms3d01 ( tetnode, tetnodenew, coornew, &
                           nextranodes, nelemnew, edgenode, &
                           inpelm )
      if ( debug ) call prinin1 ( tetnodenew, nelemnew, 4, 'tetnodenew')

!     --- update pointers ielemnew and ikelmcnew

      ielemnew = ielemnew+nelemnew-1
      ikelmcnew = ikelmcnew+inpelm*nelemnew

1000  if ( debug ) then

!     --- Debug information

         if ( debug ) write(irefwr,1) 'ielemnew', ielemnew
         write(irefwr,*) 'End mshobstms3d03'

      end if
      call erclos ( 'mshobstms3d03' )

      end
