      subroutine mshouterboun3d
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.10 date 27-11-2012 Long integers
!        version  1.9  date 27-11-2011 Extra debug
!        version  1.8  date 09-01-2008 Call to mshcompsurf3d
!
!   copyright (c) 2002-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the outer boundary surfaces of a region in R^3 and sort this
!     boundary such that the normal is directed outwards
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     boundary
!     outer_boundary
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lenoutersurfs, i, &
              lensurfcur, lencurvsur, nparts, nelmsurfs
      integer, allocatable :: listgrp(:)
      logical :: created

!     created        If true an outer boundary has been created
!     i              Loop variable
!     lencurvsur     Declared length of array curve_surfs
!     lenoutersurfs  Length of arrayouterboun
!     lensurfcur     Declared length of array surface_curvs
!     listgrp        Array of size nelgrp indicating if an element
!                    group must be used (1) or not (0)
!     nelmsurfs      Overestimation of the number of surface elements
!     nparts         number of parts the region consists of
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MSHMAKELISTG   Fill array listgroup with a 1 when an element is a volume
!                    element and a zero otherwise
!     MSHOUTERFIND3D Find the outer boundary of a three-dimensional region and
!                    store the surface numbers in array ioutersurfs
!     MSHOUTERORD3D  Order array ioutersurfs containing the outer surfaces of
!                    the region such that all surfaces are ordered counter
!                    clockwise
!     MSHOUTERSURF   Make alist of all outer surfaces of the mesh under the
!                    condition that array kmesh part n has been filled but
!                    kmesh part s not
!     PRINIOUTERCS   Print array ioutercurvs or ioutersurfs,
!                    depending on the dimension
!     SEPFILLKMESHAA Fill first part of array KMESH part aa
!     SEPFILLNMESHA  Fill array nmesha
!     SEPFILLNMESHF  Fill array nmeshf
!     SEPFILLNMESHG  Fill array nmeshg
!     SEPFILLNMESHK  Fill array nmesh part k
!     SEPPRINKMESH   Print array KMESH
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Compute all outer surfaces of the region
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshouterboun3d' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshouterboun3d'

      end if
      if ( ierror/=0 ) go to 1000

!     --- Compute all the outer surfaces of the region

      if ( debug ) &
        write(irefwr,*) 'ncurvs, nsurfs, nvolms ',ncurvs, nsurfs, nvolms

      created = .false.

      if ( kelms/=0 .and. kelmag>0 ) then

!     --- kmeshs exists
!         Reserve temporary space

         lenoutersurfs = 4*nsurfs
         allocate ( outerboun(lenoutersurfs), stat = error )
         if ( error/=0 ) call eralloc ( error, lenoutersurfs, 'outerboun' )
         if ( ierror/=0 ) go to 1000

!        --- Compute length of arrays surface_curvs and curve_surfs

         lensurfcur = kmeshs(1+nsurfs)-kmeshs(1)+2*nsurfs
         lencurvsur = 6*ncurvs

!        --- Actual computation

         call mshouterfind3d ( lenoutersurfs, lensurfcur, lencurvsur )
         if ( ierror/=0 ) go to 1000

         if ( lenoutersurfs>0 ) then

!           --- Overestimate nelemsurfs

            nelmsurfs = 0
            do i = 1, nsurfs
               nelmsurfs = nelmsurfs+kmeshn(nsurfs+i)
            end do

!           --- Make the direction of these boundaries such that they are always
!               counter clockwise

            nparts = outerboun(1)
            call mshouterord3d ( nparts, nelmsurfs )
            created = .true.
            if ( ierror/=0 ) go to 1000

         end if

      else

!     --- kmeshs does not exist, difficult way to create outer boundary

         if ( kelmn/=0 ) then

!        --- kmesh part n exists, use mshoutersurf to create outersurfaces
!            First create array listgrp at the end of ibuffr

            allocate ( listgrp(nelgrp), stat = error )
            if ( error/=0 ) call eralloc ( error, nelgrp, 'listgrp' )
            if ( ierror/=0 ) go to 1000

!           --- Fill array listgrp with a 1 when it concerns a volume element
!               and a zero if it is a surface element

            call mshmakelistg ( listgrp )

!           --- Activate arrays

            if ( kelmaa==0 ) call sepfillkmeshaa
            if ( nmesh(31)==0 ) call sepfillnmesha
            if ( nmesh(36)==0 ) call sepfillnmeshf
            if ( nmesh(37)==0 ) call sepfillnmeshg
            if ( nmesh(41)==0 ) call sepfillnmeshk
            if ( debug ) then
               call sepprinkmesh ( 44 )
               call sepprinkmesh ( 46 )
               call sepprinkmesh ( 48 )
            end if

!           --- Reserve temporary space

            lenoutersurfs = 4*nsurfs

            allocate ( outerboun(lenoutersurfs), stat = error )
            if ( error/=0 ) call eralloc ( error, lenoutersurfs, 'outerboun' )
            if ( ierror/=0 ) go to 1000

            call mshoutersurf ( lenoutersurfs, listgrp )
            created = .true.

            deallocate ( listgrp, stat = error )
            if ( error/=0 ) call erdealloc ( error, 'listgrp' )

         end if  ! ( kelmn/=0 )

      end if  ! ( kelms/=0 )

      if ( created ) then

!     --- Make array outerboun permanent

         allocate ( km(iseqkm)%outerboun(lenoutersurfs) )
         km(iseqkm)%outerboun = outerboun(1:lenoutersurfs)

         deallocate ( outerboun, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'outerboun' )

         outerboun => km(iseqkm)%outerboun
         mmouter = lenoutersurfs

      else

!        --- Not possible to create outer boundary

         if ( mmouter>0 ) deallocate(km(iseqkm)%outerboun)
         mmouter = 0

         deallocate ( outerboun, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'outerboun' )

      end if  ! ( mmouter>0 )

1000  if ( debug ) then

!     --- Debug information

         if ( mmouter>0 ) call prinioutercs
         write(irefwr,*) 'End mshouterboun3d'

      end if
      call erclos ( 'mshouterboun3d' )

      end
