      subroutine mshouterord3d ( nparts, nelmsurfs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 09-01-2008 Debug information
!        version  2.0  date 17-08-2004 Extra parameters
!        version  1.0  date 04-02-2002
!
!   copyright (c) 2002-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Order array outerboun containing the outer surfaces of the region such
!    that all surfaces are ordered counter clockwise
!    In this way the outer normal is uniquely defined
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     boundary
!     outer_boundary
!     sort
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nparts, nelmsurfs

!     nelmsurfs      i    Overestimation of the number of surface elements
!     nparts         i    number of parts the region consists of
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer imarkpart(nparts), ipart, istart, nouter, i, isurnr, &
              j, nodp, inpelm, ipstart, &
              icount, ivert(4), jsurnr, &
              itria(3), iielem, iouter, maxcount, nvertices, nside
      double precision z, zmin, normal1(3)
      logical finished, downwards, parallel_elm
      integer :: istore(2,nelmsurfs), isequence(nelmsurfs)
      double precision zstore(nelmsurfs)

!     downwards      If true the boundary part is directed "down wards"
!     finished       Indicates if the loop has been finished (true) or not
!                    (false)
!     i              Counting variable
!     icount         Counter
!     iielem         Relative element number in element group
!     imarkpart      Array to mark boundary parts
!                    imarkpart(i) =-1, part has been in negative direction
!                    imarkpart(i) = 0, part has not been marked yet
!                    imarkpart(i) = 1, part has been in positive direction
!     inpelm         number of points per element
!     iouter         Sequence number of outer surface
!     ipart          Sequence number of part
!     ipstart        Starting address in array
!     isequence      Work array of length nelmsurfs to store the surfaces
!                    temporarily
!                    Is also used as help array for sorting
!     istart         First position used at a part
!     istore         Contains the element numbers and the outer surface
!                    numbers corresponding to zstore
!     isurnr         Surface sequence number
!     itria          Contains the node numbers of a triangle
!     ivert          Contains the vertex numbers of an element
!     j              Counting variable
!     jsurnr         Surface sequence number
!     maxcount       Maximum value of icount
!     nodp           Nodal point sequence number
!     normal1        First normal vector
!     nouter         Number of points outside the obstacle
!     nside          number of points along side of spectral element
!     nvertices      Number of vertices
!     parallel_elm   If true the element is parallel to the line
!     z              z co-ordinate of point
!     zmin           Minimum z-value
!     zstore         Contains the minimum z-values of the surface elements
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     MSHFILLVERT2D  Fill the node numbers of the vertices of a 2d element
!                    in array ivert
!     MSHFILLVERTSP2 Fill the node numbers of the vertices of a 2d element
!                    in array ivert for spectral elements
!     MSHNORMALDIR   Compute the normal on a triangle and check whether the
!                    normal is perpendicular to the z-axis, upwards or
!                    downwards
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
!     PRINOUTCRVS    Print the contents of array ioutercurvs
!     PRINRL         Print 1d real vector
!     PRSORT         Sort real array for increasing sequence
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2420   All elements perpendicular to z-axis
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Mark all boundary parts as not oriented
!     For all boundary parts  do
!        For all elements in boundary do
!           Store minimum z-values
!        Sort all elements in increasing z-value
!        finished := false
!        while ( not finished ) do
!           take next elements with minimum z-value
!           compute normal on this element
!           if ( normal not perpendicular to z-axis ) then
!              downwards := normal(3)<0
!              finished := true
!              Mark boundary
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshouterord3d' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshouterord3d'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( debug ) then

!     --- debugging information

         write(irefwr,*) 'start mshouterord3d'
         call prinoutcrvs ( outerboun )

      end if

!     --- Mark all boundary parts as not oriented

      do ipart = 1, nparts
         imarkpart(ipart) = 0
      end do

!     --- For all parts do

      do ipart = 1, nparts

!     --- For all elements in boundary do

         ipstart = outerboun(ipart+1)
         nouter = outerboun(ipstart)
         icount = 0
         do i = 1, nouter

            jsurnr = outerboun(ipstart+i)
            isurnr = abs(jsurnr)
            istart = 2*nsurfs
            do j = 1, isurnr-1
               istart = istart+kmeshn(j)*kmeshn(j+nsurfs)
            end do
            inpelm = kmeshn(isurnr)
            inelem = kmeshn(isurnr+nsurfs)

!           --- Store minimum z-values

            do iielem = 1, inelem
               zmin = rinfin
               do j = 1, inpelm
                  nodp = kmeshn(istart+j)
                  if ( nodp>0 ) then

!                 --- Nodal point on boundary

                     z = coor(3,nodp)
                     zmin = min(z,zmin)

                  end if

               end do

               icount = icount+1
               zstore(icount) = z
               istore(1,icount) = i
               istore(2,icount) = iielem
               istart = istart+inpelm

            end do

         end do ! i = 1, nouter

!        --- Sort all elements in increasing z-value

         call prsort ( zstore, isequence, icount )

         if ( debug ) then

!        --- Debugging information

            write(irefwr,*) 'ipart ', ipart
            call prinrl ( zstore, icount, 'min. z-values' )
            call prinin1 ( istore, icount, 2, 'istore' )
            call prinin ( isequence, icount, 'isequence' )

         end if

!        --- finished := false

         finished = .false.

!        --- while ( not finished ) do

         maxcount = icount
         icount = 0
         do while ( .not. finished )

            icount = icount+1
            if ( icount>maxcount ) then

!           --- internal error

               call errsub ( 2420, 0, 0, 0 )
               go to 1000

            end if

!           --- take next elements with minimum z-value

            iouter = istore(1,isequence(icount))
            iielem  = istore(2,isequence(icount))
            jsurnr = outerboun(ipstart+iouter)
            isurnr = abs(jsurnr)
            istart = 2*nsurfs
            do j = 1, isurnr-1
               istart = istart+kmeshn(j)*kmeshn(j+nsurfs)
            end do
            inpelm = kmeshn(isurnr)
            inelem = kmeshn(isurnr+nsurfs)
            ishape = kmeshs2(2,isurnr)
            istart = istart+(iielem-1)*inpelm

            if ( debug ) then

!           --- Debugging information

               write(irefwr,*) 'icount,iouter,iielem,jsurnr,inpelm ', &
                                icount,iouter,iielem,jsurnr,inpelm
               write(irefwr,*) 'inelem,ishape,istart ', &
                                inelem,ishape,istart

            end if

!           --- Fill vertices in ivert
!               nvertices := number of vertices

            if ( kelmu==0 ) then

!           --- kelmu = 0, standard elements

               call mshfillvert2d ( kmeshn(istart+1), ishape, nvertices, &
                                    ivert )

            else

!           --- kelmu>0, spectral elements

               nside = 2+kmeshu(2)
               call mshfillvertsp2 ( kmeshn(istart+1), ishape, &
                                     nvertices, ivert, nside )

            end if

            if ( debug ) then

!           --- Debugging information

               write(irefwr,*) 'nvertices, ivert ',nvertices, ivert

            end if

!           --- compute normal on this element
!               if ( normal not perpendicular to z-axis ) then
!                  downwards := normal(3)<0
!                  finished := true
!                  Mark boundary

            if ( nvertices==3 ) then

!           --- nvertices = 3, triangle

               call mshnormaldir ( ivert, coor, parallel_elm, &
                                   normal1 )

               if ( debug ) then

!              --- Debugging information

                  call prinrl ( normal1, 3, 'normal' )

               end if

               if ( .not. parallel_elm ) then

!              --- normal not perpendicular to z-axis

                  downwards = normal1(3)<0
                  finished = .true.

               end if

            else

!           --- nvertices = 4, quadrilateral
!               Compute intersection for both of the triangles

               itria(1) = ivert(1)
               itria(2) = ivert(2)
               itria(3) = ivert(3)

               call mshnormaldir ( itria, coor, parallel_elm, &
                                   normal1 )

               if ( debug ) then

!              --- Debugging information

                  call prinrl ( normal1, 3, 'normal' )

               end if

               if ( .not. parallel_elm ) then

!              --- normal not perpendicular to z-axis

                  downwards = normal1(3)<0
                  finished = .true.

               else

!              --- normal perpendicular to z-axis, try other triangle

                  itria(1) = ivert(1)
                  itria(2) = ivert(3)
                  itria(3) = ivert(4)

                  call mshnormaldir ( itria, coor, parallel_elm, &
                                      normal1 )

                  if ( debug ) then

!                 --- Debugging information

                     call prinrl ( normal1, 3, 'normal' )

                  end if

                  if ( .not. parallel_elm ) then

!                 --- normal not perpendicular to z-axis

                     downwards = normal1(3)<0
                     finished = .true.

                  end if

               end if

            end if

         end do ! while ( .not. finished )

         if ( downwards ) then

!        --- normal is oriented outwards

            imarkpart(ipart) = 1

         else

!        --- normal is oriented inwards

            imarkpart(ipart) = -1

         end if
         if ( jsurnr<0 ) imarkpart(ipart) = -imarkpart(ipart)

      end do ! ipart = 1, nparts

!     --- Reorder sequence if necessary

      do ipart = 1, nparts

         if ( imarkpart(ipart)<0 ) then

!        --- imarkipart(ipart)<0, reverse direction

            istart = outerboun(ipart+1)
            nouter = outerboun(istart)
            do i = 1, nouter
               isequence(i) = outerboun(istart+i)
            end do
            do i = 1, nouter
               outerboun(istart+i) = -isequence(nouter+1-i)
            end do
         end if

      end do

      if ( debug ) then

!     --- debugging information

         call prinin ( imarkpart, nparts, 'imarkpart' )
         call prinoutcrvs ( outerboun )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshouterord3d'

      end if  ! ( debug )
      call erclos ( 'mshouterord3d' )

      end
