      subroutine mshoutersurf ( lenoutersurfs, listgrp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.1  date 08-09-2005 New call to mshoutersurf1
!        version  4.0  date 28-05-2004 Extra parameters
!        version  3.0  date 30-03-2004 Extra parameters
!
!   copyright (c) 2004-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Make a list of all outer surfaces of the mesh under the condition that
!     array kmesh part n has been filled but kmesh part s not
!     Make the direction of these surfaces such that the outer normal
!     is pointing outwards
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     boundary
!     outer_boundary
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: lenoutersurfs, listgrp(*)

!     lenoutersurfs  o    Length of array outerboun
!     listgrp        i    Array of size nelgrp indicating if an element
!                         group must be used (1) or not (0)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istart, inode, ipstart, i, k, numfaces, numelems, &
              ielgrp, ielem, numvolelms, isign, &
              istore(nsurfs), iface, nouter, jface, icount, &
              inode1, inode2, jpstart
      logical single
      integer :: iworkelem(nelem)
      integer, allocatable :: iwork(:)

!     i              Counting variable
!     icount         Counter
!     ielem          Element number
!     ielgrp         element group number
!     iface          Face sequence number
!     inode          Node sequence number
!     inode1         First node in surface element
!     inode2         Second node in surface element
!     ipstart        Starting address in array
!     isign          sign of surface element compared to face (1 or -1)
!     istart         First position used at a part
!     istore         Contains the last level set
!     iwork          integer work array of length nfaces to store the
!                    number of nodes of an element corresponding to a face
!                    In mshoutersurf1 it is used as work array of length
!                    npoint
!     iworkelem      integer work array of length nelem to store the number
!                    of volume elements corresponding to a face
!     jface          Face sequence number
!     jpstart        Starting address in array
!     k              Counting variable
!     nouter         Number of surfaces
!     numelems       Number of elements
!     numfaces       number of faces
!     numvolelms     number of volume elements corresponding to face
!     single         If true the surface is not a compound surface
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     MSHOUTERSURF1  Organize the surfaces stored in array istore such that
!                    connected surfaces are stored in parts
!     PRININ         print 1d integer array
!     PRINIOUTERCS   Print array ioutercurvs or outerboun, &
!                    depending on the dimension
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2484   No outer surface found for this mesh
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array iworkelem
!     Make iworkelem(i) = 1 if i is volume element
!     For all surfaces isurnr do
!        find face number iface corresponding to first surface element in isurnr
!        find the element numbers corresponding to iface
!        if ( number of volume elements connected to iface = 1 ) then
!           surface isurnr is outer surface
!           find direction of normal on face
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshoutersurf' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshoutersurf'
         write(irefwr,*) 'nsurfs, nfaces, nelem ', nsurfs, nfaces, nelem

      end if
      if ( ierror/=0 ) go to 1000

      allocate ( iwork(max(nfaces,npoint)), stat = error )
      if ( error/=0 ) call eralloc ( error, max(nfaces,npoint), 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Clear array iwork

      iwork(1:nfaces) = 0

!     --- Clear array iworkelem

      iworkelem(1:nelem) = 0

!     --- Make iworkelem(i) = 1 if i corresponds to the correct element group

      ielem = 1

      do ielgrp = 1, nelgrp

         inelem = Topgrp(ielgrp)%inelem
         if ( listgrp(ielgrp)>0 ) then

!        --- listgrp(ielgrp)>0, hence volume element of specific domain

            do i = ielem, ielem+inelem-1
               iworkelem(i) = 1
            end do  ! i = ielem, ielem+inelem-1

         end if  ! ( ishape>10 )
         ielem = ielem+inelem

      end do  ! ielgrp = 1, nelgrp

!     --- For all surfaces isurnr do

      istart = 2*nsurfs
      nouter = 0
      do isurnr = 1, nsurfs

!     --- find face number iface corresponding to first surface element in
!         isurnr

         inpelm = kmeshn(isurnr)
         inelem = kmeshn(isurnr+nsurfs)

         if ( kelmr>0 ) then

!        --- There are composite surfaces

            single = kmeshr(isurnr+1)-kmeshr(isurnr)<=1

         else

!        --- No composite surfaces

            single = .true.

         end if  ! ( kelmr>0 )

         if ( .not. single ) go to 200  ! jump in case of compound surface
         if ( debug ) then
            write(irefwr,*) 'isurnr, inpelm, inelem ', &
                             isurnr, inpelm, inelem
            call prinin ( kmeshn(istart+1), inpelm, 'face' )
         end if

         do i = 1, inpelm
            inode = kmeshn(istart+i)

!           --- Jump if node<=0, because surface not connected to mesh

            if ( inode<=0 ) then

!           --- Surface has not been filled

               if ( debug ) write(irefwr,*) 'Surface ', isurnr, &
                  ' is not connected '
               go to 200  ! jump, not connected

            end if  ! ( inode<=0 )

            ipstart = nmeshk(inode)
            numfaces = nmeshk(ipstart)
            if ( debug ) then
               write(irefwr,*) 'i, inode, numfaces ', &
                                i, inode, numfaces
               call prinin ( nmeshk(ipstart+1), numfaces, 'facenrs' )
            end if

            do k = 1, numfaces
               iface = nmeshk(ipstart+k)
               iwork(iface) = iwork(iface)+1

!              --- As soon as iwork(iface) = inpelm, we have found the face
!                  number and we leave the loop

               if ( iwork(iface)==inpelm ) go to 150

            end do  ! k = 1, numfaces

         end do  ! i = 1, inpelm

!        --- if the routine comes here, no face has been found
!            we skip this surface
!            This is for example the case if the surface is not coupled
!            to volume elements and the mesh
!            An example is the obstacle surface

         if ( debug ) write(irefwr,*) 'Surface ', isurnr, &
            ' is not connected '
         go to 200

!        --- Clear array iwork

150      if ( debug ) write(irefwr,*) 'iface ', iface
         do i = 1, inpelm

            inode = kmeshn(istart+i)
            ipstart = nmeshk(inode)
            numfaces = nmeshk(ipstart)
            do k = 1, numfaces
               jface = nmeshk(ipstart+k)
               iwork(jface) = 0
            end do  ! k = 1, numfaces

         end do  ! i = 1, inpelm
         if ( debug ) write(irefwr,*) 'iface ', iface

!        --- find the element numbers corresponding to iface

         ipstart = nmeshf(iface)
         numelems = nmeshf(ipstart)
         numvolelms = 0   ! number of volume elements corresponding to iface
         do i = 1, numelems

            if ( iworkelem(nmeshf(ipstart+i))>0 ) then

!           --- element is volume element

               ielem = nmeshf(ipstart+i) ! volume element corresponding to iface
               numvolelms = numvolelms+1

            end if

         end do  ! i = 1, numelems
         if ( debug ) then
            write(irefwr,*) 'numvolelms ', numvolelms
            call prinin ( nmeshf(ipstart+1), numelems, 'elements' )
         end if

         if ( numvolelms==1 ) then

!        --- number of volume elements connected to iface = 1
!            surface isurnr is outer surface
!            find direction of normal on face
!            first find direction of element in surface with respect to
!            element in face

            inode1 = kmeshn(istart+1)   ! first node in surface element
            inode2 = kmeshn(istart+2)   ! second node in surface element
            jpstart = nmeshg(iface)     ! starting address in nmeshg

!           --- Find position of inode1 in face

            icount = 1
            do while ( nmeshg(jpstart+icount)/=inode1 .and. &
                       icount<inpelm )
               icount = icount+1
            end do  ! while ( nmeshg(jpstart+icount)/=inode1 ... )

!           --- node stored at position icount in surface element

            if ( icount<inpelm ) then

!           --- Not the last node, find node2

               if ( nmeshg(jpstart+icount+1)==inode2 ) then

!              --- Next node is inode2

                  isign = 1

               else

!              --- Previous node is inode2

                  isign = -1

               end if

            else

!           --- Last node, find node2

               if ( nmeshg(jpstart+1)==inode2 ) then

!              --- Next node is inode2

                  isign = 1

               else

!              --- Previous node is inode2

                  isign = -1

               end if

            end if  ! ( icount<inpelm )

            ipstart = nmesha(ielem)
            numfaces = nmesha(ipstart)/2 !number of faces corresponding to ielem
            do i = 1, numfaces

               if ( abs(nmesha(ipstart+i))==iface ) then

!              --- face found

                  nouter = nouter+1
                  if ( nmesha(ipstart+i)>0 ) then

!                 --- direction is positive

                     istore(nouter) = isign*isurnr

                  else

!                 --- direction is negative

                     istore(nouter) = -isign*isurnr

                  end if

               end if  ! ( abs(nmesha(ipstart+i))==iface )

            end do  ! i = 1, numfaces

         end if  ! ( numvolelms==1 )

200      istart = istart+inelem*inpelm

      end do  ! isurnr = 1, nsurfs
      if ( debug ) call prinin ( istore, nouter, 'istore' )

      if ( nouter==1 ) then

!     --- simple case, only one outer surface

          outerboun(1) = 1           ! nparts
          outerboun(2) = 3           ! starting address
          outerboun(3) = 1           ! nouter
          outerboun(4) = istore(1)   ! surface number with sign
          lenoutersurfs = 4

      else if ( nouter>1 ) then

!     --- complex case, more than one outer surface
!         Now we have to check which surfaces form a part

         call mshoutersurf1 ( istore, nouter, nsurfs, outerboun, &
                              lenoutersurfs, npoint, kmeshn, iwork )

      else

!     --- error: no outer surfaces found

         call errsub ( 2484, 0, 0, 0 )

      end if  ! ( nouter==1 )

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'lenoutersurfs ', lenoutersurfs
         call prinioutercs
         write(irefwr,*) 'End mshoutersurf'

      end if
      call erclos ( 'mshoutersurf' )

      end
