      subroutine mshoutersurf1 ( istore, nouter, nsurfs, ioutersurfs, &
                                 lenoutersurfs, npoint, kmeshn, nodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 08-09-2005 Extra parameter nodes
!        version  1.0  date 14-01-2004
!
!   copyright (c) 2004-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Organize the surfaces stored in array istore such that connected
!     surfaces are stored in parts
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     boundary
!     outer_boundary
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istore(*), nouter, nsurfs, ioutersurfs(*), lenoutersurfs, &
              npoint, kmeshn(*), nodes(npoint)

!     ioutersurfs    o    Contains information about the surfaces in the outer
!                         boundary (3D) in the following way
!                         Pos. 1  nparts  Number of parts the outer boundary
!                                         consists of
!                         Pos. 2-1+nparts Starting addresses of each part
!                         Per part: (first address: ioutersurfs(1+i)
!                         Pos. 1: nouter  Number of surfaces in part
!                         followed by the surface numbers (provided with a sign)
!                         The part itself is closed
!     istore         i    Contains the set of outer surfaces
!     kmeshn         i    KMESH part n, contains nodal point numbers of the
!                         surfaces
!     lenoutersurfs  o    Length of array ioutersurfs
!     nodes               Work array of length npoint to mark all nodes in a
!                         surface or a set of surfaces
!     nouter         i    Number of outer surfaces
!     npoint         i    Number of nodal points
!     nsurfs         i    number of surfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iwork(nsurfs), icopy(nouter), i, nleft, &
              nparts, icount, isurnr, j, istart, ipstart, numsurfs, &
              numstore(nouter)
      logical found

!     found          If true already a possibility has been found
!     i              Counting variable
!     icopy          Copy of array istore ordened for the final sequence
!     icount         Counter
!     ipstart        Starting address in array
!     istart         First position used at a part
!     isurnr         Surface sequence number
!     iwork          integer work array of length nsurfs
!                    if iwork(i) = 0 surface i is not longer in the
!                    list of free outer surfaces
!     j              Counting variable
!     nleft          Number of free surfaces left
!     nparts         number of parts the boundary consists of
!     numstore       work array to store the number of surfaces per part
!     numsurfs       number of surfaces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array iwork (1:nsurfs)
!     Clear array nodes ( 1:npoint)
!     Set iwork(i) # 0, if surface i in istore
!     nleft := nouter
!     nparts := 0; icount := 0
!     while ( nleft>0 ) do
!        isurnr := first surface with iwork(isurnr) # 0
!        nparts := nparts+1;  numsurfs := 1; icount := icount+1
!        icopy(icount) := iwork(isurnr); iwork(isurnr) := 0; nleft := nleft-1
!        Set nodes(i) = 1 for all nodes in surface
!        isurnr := 1
!        while ( nleft>0 and isurnr>nsurfs ) do
!           Find surface isurnr with iwork(isurnr)>0
!           if ( at least one node of isurnr has nodes(node)=1 ) then
!              Set nodes(i) = 1 for all nodes in surface isurnr
!              numsurfs := numsurfs+1; icount := icount+1
!              icopy(icount) := iwork(isurnr); iwork(isurnr) := 0;
!              nleft := nleft-1
!        numstore(nparts) := numsurfs
!        if ( nleft>0 ) then
!           Clear array nodes
!     ioutersurfs(1) := nparts; ipstart := nparts+2; icount:= 0
!     for i := 1 (1) nparts do
!        numsurfs := numstore(i); ioutersurfs(i+1) := ipstart;
!        ioutersurfs(ipstart) := numsurfs
!        for j := 1 (1) numsurfs do
!           ioutersurfs(ipstart+j) := icopy(icount+j)
!        icount := icount+numsurfs; ipstart := ipstart+numsurfs+1
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshoutersurf1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshoutersurf1'
         call prinin ( istore, nouter, 'istore' )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Clear array iwork (1:nsurfs)

      do i = 1, nsurfs
         iwork(i) = 0
      end do

!     --- Clear array nodes ( 1:npoint)

      do i = 1, npoint
         nodes(i) = 0
      end do

!     --- Set iwork(i) # 0, if surface i in istore

      do i = 1, nouter
         iwork(abs(istore(i))) = istore(i)   ! Use istore to get the correct
                                             ! sign
      end do
      if ( debug ) call prinin ( iwork, nsurfs, 'iwork' )

!     --- nleft := nouter
!         nparts := 0; icount := 0

      nleft = nouter
      nparts = 0  ! number of parts
      icount = 0  ! number of surfaces in icopy

!     --- while ( nleft>0 ) do

      do while ( nleft>0 )

!     --- isurnr := first surface with iwork(isurnr) = 1

         isurnr = 1
         do while ( iwork(isurnr)==0 )
            isurnr = isurnr+1
         end do
         if ( debug ) write(irefwr,*) 'isurnr, nleft ', isurnr, nleft

!        --- nparts := nparts+1;  numsurfs := 1; icount := icount+1
!            icopy(icount) := iwork(isurnr); iwork(isurnr) := 0
!            nleft := nleft-1

         nparts = nparts+1
         numsurfs = 1                   ! number of surfaces for this part
         icount = icount+1              ! next position in icopy
         icopy(icount) = iwork(isurnr)  ! surface number with sign
         iwork(isurnr) = 0
         nleft = nleft-1
         if ( debug ) write(irefwr,*) 'nparts, icount, nleft ', &
            nparts, icount, nleft

!        --- Set nodes(i) = 1 for all nodes in surface

         istart = 2*nsurfs
         do i = 1, isurnr-1
            istart = istart+kmeshn(i)*kmeshn(nsurfs+i)
         end do
         do i = istart+1, istart+kmeshn(isurnr)*kmeshn(isurnr+nsurfs)
            nodes(kmeshn(i)) = 1
         end do

         isurnr = 1

!        --- while ( nleft>0 and isurnr>nsurfs ) do

         do while ( nleft>0 .and. isurnr<=nsurfs )

!        --- nleft>0 and isurnr>nsurfs
!            Find surface isurnr with iwork(isurnr)>0

            do while ( iwork(isurnr)==0 .and. isurnr<=nsurfs )
               isurnr = isurnr+1
            end do
            if ( isurnr>nsurfs ) go to 200
            if ( debug ) write(irefwr,*) 'isurnr, nleft ', isurnr, nleft

            istart = 2*nsurfs
            do i = 1, isurnr-1
               istart = istart+kmeshn(i)*kmeshn(nsurfs+i)
            end do
            found = .false.
            if ( kmeshn(istart+1)>0 ) then

!           --- Only surfaces that have been filled may be used

               do i = istart+1, &
                      istart+kmeshn(isurnr)*kmeshn(isurnr+nsurfs)
                  if ( nodes(kmeshn(i))>0 ) then

!                 --- Node with nodes(inode) = 1 found

                     found = .true.
                     go to 100

                  end if

               end do

            end if  ! ( kmeshn(istart+1)>0 )

100         if ( found ) then

!           --- at least one node of isurnr has nodes(node)=1
!               Set nodes(i) = 1 for all nodes in surface isurnr

               do i = istart+1, &
                      istart+kmeshn(isurnr)*kmeshn(isurnr+nsurfs)
                  nodes(kmeshn(i)) = 1
               end do

!              --- numsurfs := numsurfs+1; icount := icount+1
!                  icopy(icount) := iwork(isurnr); iwork(isurnr) := 0;
!                  nleft := nleft-1

               numsurfs = numsurfs+1
               icount = icount+1
               icopy(icount) = iwork(isurnr)
               iwork(isurnr) = 0
               nleft = nleft-1
               if ( debug ) &
                  write(irefwr,*) 'numsurfs, isurnr, icount, nleft ', &
                                   numsurfs, isurnr, icount, nleft
               isurnr = 0

            end if  ! ( found )
            isurnr = isurnr+1
            if ( debug ) write(irefwr,*) 'found, isurnr ', found, isurnr

200      end do  ! while ( nleft>0 .and. isurnr<=nsurfs )

!        --- numstore(nparts) := numsurfs

         numstore(nparts) = numsurfs

         if ( nleft>0 ) then

!        --- nleft>0
!            Clear array nodes

            do j = 0, numsurfs-1
               isurnr = abs(icopy(icount-j))
               istart = 2*nsurfs
               do i = 1, isurnr-1
                  istart = istart+kmeshn(i)*kmeshn(nsurfs+i)
               end do
               do i = istart+1, &
                      istart+kmeshn(isurnr)*kmeshn(isurnr+nsurfs)
                  nodes(kmeshn(i)) = 0
               end do
            end do  ! i = 0, numsurfs-1

         end if  ! ( nleft>0 )

      end do  ! while ( nleft>0 )

      if ( debug ) then
         write(irefwr,*) 'nparts, icount ', nparts, icount
         call prinin ( icopy, icount, 'icopy' )
         call prinin ( numstore, nparts, 'numstore' )
      end if

!     --- ioutersurfs(1) := nparts; ipstart := nparts+2; icount:= 0

      ioutersurfs(1) = nparts
      ipstart = nparts+2
      icount= 0

!     --- for i := 1 (1) nparts do

      do i = 1, nparts

!     --- numsurfs := numstore(i); ioutersurfs(i+1) := ipstart;
!         ioutersurfs(ipstart) := numsurfs

         numsurfs = numstore(i)
         ioutersurfs(i+1) = ipstart
         ioutersurfs(ipstart) = numsurfs

!        --- for j := 1 (1) numsurfs do
!               ioutersurfs(ipstart+j) := icopy(icount+j)

         do j = 1, numsurfs
            ioutersurfs(ipstart+j) = icopy(icount+j)
         end do  ! j = 1, numsurfs

!        --- icount := icount+numsurfs; ipstart := ipstart+numsurfs+1

         icount = icount+numsurfs
         ipstart = ipstart+numsurfs+1

      end do  ! i = 1, nparts

      lenoutersurfs = ipstart-1

1000  call erclos ( 'mshoutersurf1' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshoutersurf1'

      end if

      end
