      subroutine mshparelblocks ( nprocessors, nelements )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-01-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill arrays elements and nelements in order to get a subdivision of
!     the region in blocks for parallel computing
!     The number of processors must be a power of 2
! **********************************************************************
!
!                       KEYWORDS
!
!     parallel
!     subdivision
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nprocessors, nelements(*)

!     nelements      o    Array of size nprocessors containing for each block
!                         the end pointer in array elements
!                         Hence the elements stored in array elements
!                         pos. 1 to nelements(1) belong to block 1, &
!                         the elements from pos.nelements(1)+1 to nelements(2)
!                         belong to block 2 etc
!     nprocessors    i    Number of processors that is to be used for
!                         parallel processing
!                         Defines also the number of submeshes
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nlevelsub, ilevel, i, numelms, nodes(npoint), nnodes, &
              elemstold(2,nprocessors), elemstnew(2,nprocessors), j, &
              ksetnp(npoint), mset, &
              ifirst, nhalf, istart, &
              numelms1
      double precision a
      integer, allocatable :: elements(:)

!     a              Help variable to store quantity
!     elements       Array of size nelemloc containing the element
!                    numbers corresponding to the local mesh
!     elemstnew      Array of size 2 x nprocessors that is used to store
!                    the starting and end addresses of a set of elements
!                    in array elements
!                    Pos. (1,i) contains the starting address of part i and
!                    pos. (2,i) the end address
!                    This is used during the creation of the elements
!                    and corresponds to the present level
!     elemstold      See elemstnew, but now for the previous level
!     i              Counting variable
!     ifirst         Refers to first node in a set
!     ilevel         Sequene number of present level in subdivision
!     istart         First position used at a part
!     j              Counting variable
!     ksetnp         array containing a set of nodal points
!     mset           number of nodal points in ksetnp
!     nhalf          Approximation of number of nodes in set subdivided by 2
!                    and increased a little bit since an element belongs to
!                    a set only if all nodes belong to that set
!     nlevelsub      Number of levels for the subdivision (2lognproessors)
!     nnodes         Number of nodes in nnodes
!     nodes          Contains the node numbers of a set
!     numelms        Number of elements in a set
!     numelms1       Number of elements that correspond to first "half"
!                    of a set
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     CHTMC2         The starting level for the renumbering is found
!     CHTMC4         The rest of the levels is computed according to the
!                    Cuthill-Mckee algorithm
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIELEMNODES   subdivide a set of elements into a set of which all nodes
!                    are part of a set of nodes and the rest
!     ININODEELMS    Fill all nodes of the elements stored in array elements
!                    into array nodes
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2497   Number of processors not a power of 2
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The idea is as follows:
!        we start with the complete set of (possibly renumbered nodes)
!        this set is subdivided into 2 parts of almost equal size
!        Because we want the number of elements to be approximately equal
!        in each set the first set of nodes is slightly enlarged
!        After the subdivision each set is renumbered and the process is
!        repeated for each subset separately
!        This process is repeated recursively until we have nprocessor blocks
!
!     Fill starting addresses of all elements in kmeshc
!     Initialize array elements
!     nlevelsub := 2log nprocessors
!     For ilevel := 1 (1) nlevelsub do
!        For i := 1 (1) 2^(ilevel-1) do
!           if ( ilevel > 1 ) then
!              Find nodes in elements elemstold(1,i) .. (2,i)
!              Store in ksetnp
!              Renumber set of nodes in ksetnp
!           else ( ilevel = 1 )
!              Copy ( renumbered ) nodes in ksetnp
!           Subdivide region in 2 parts
!           Fill elemstnew(.,2*i-1) and (.,2*i)
!        Copy elemstnew onto elemstold
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshparelblocks' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshparelblocks'

      end if

      allocate ( elements(nelem), marknodes(npoint) )

!     --- Initialize array elements and elemstold

      do i = 1, nelem
         elements(i) = i
      end do
      elemstold(1,1) = 1
      elemstold(2,1) = nelem

!     --- nlevelsub := 2log nprocessors

      a = nprocessors
      nlevelsub = nint(log(a)/log(2d0))
      if ( 2**nlevelsub/=nprocessors ) then

!     --- Number of processors nnot a power of 2

         call errint ( nprocessors, 1 )
         call errint ( nlevelsub, 2 )
         call errint ( 2**nlevelsub, 3 )
         call errsub ( 2497, 3, 0, 0 )
         go to 1000

      end if

!     --- For ilevel := 1 (1) nlevelsub do

      do ilevel = 1, nlevelsub

!     --- For i := 1 (1) 2^(ilevel-1) do

         if ( debug ) write(irefwr,*) 'ilevel ', ilevel
         do i = 1, 2**(ilevel-1)

            if ( debug ) write(irefwr,*) 'isublevel ', i
            if ( ilevel>1 ) then

!           --- ilevel > 1
!               Find nodes in elements elemstold(1,i) .. (2,i)
!               Store in nodes

               numelms = elemstold(2,i)-elemstold(1,i)+1
               if ( debug ) write(irefwr,*) &
                  'numelms, elemstold(2,i), elemstold(1,i) ', &
                   numelms, elemstold(2,i), elemstold(1,i)
               call ininodeelms ( elements(elemstold(1,i)), numelms, &
                                  nodes, nnodes )
               if ( debug ) then

!              --- Debug information

                  write(irefwr,*) 'elemstold(1,i), numelms, nnodes ', &
                                   elemstold(1,i), numelms, nnodes
                  call prinin ( nodes, nnodes, 'nodes' )

               end if

!              --- Set marknodes equal to -10

               do j = 1, npoint
                  marknodes(j) = -10
               end do

!              --- Set marknodes 0 for all nodes in nodes

               do j = 1, nnodes
                  marknodes(nodes(j)) = 0
               end do

!              --- Renumber set of nodes in ksetnp
!                  First the starting level is found

               ifirst = 1
               call chtmc2 ( nodes, ksetnp, mset, nnodes, 2, .true., &
                             ifirst )

!              --- Next the Cuthill-McKee numbering is applied

               call chtmc4 ( ksetnp, mset, 1 )

            else

!           --- ilevel = 1
!               Copy ( renumbered ) nodes in ksetnp

               if ( kelmj==0 ) then

!              --- No renumbering

                  do j = 1, npoint
                     ksetnp(j) = j
                  end do

               else

!              --- Renumbering

                  do j = 1, npoint
                     ksetnp(j) = kmeshj(j)
                  end do

               end if
               mset = npoint

            end if  ! ( ilevel>1 )

!           --- Subdivide region in 2 parts

            a = mset
            nhalf = mset/2+nint(sqrt(a))

!           --- Fill elemstnew(.,2*i-1) and (.,2*i)
!               First detect starting address in elements and number of elements
!               of the set

            if ( i==1 ) then

!           --- i = 1, first set of elements

               istart = 1
               numelms = nelem

            else

!           --- i > 1, first set of elements

               istart = elemstold(1,i)
               numelms = elemstold(2,i)-istart+1

            end if
            if ( debug ) &
               write(irefwr,*) 'istart, numelms ',istart, numelms

!           --- Next reorder this part of elements, such that the elements
!               correspond to either set

            call inielemnodes ( ksetnp, nhalf, elements(istart), &
                                numelms, numelms1 )
            if ( debug ) then

!           --- Debug information

               write(irefwr,*) 'istart, numelms, numelms1 ', &
                                istart, numelms, numelms1
               call prinin ( elements(istart), numelms, &
                             'elements' )

            end if

!           --- Finally fill elemstnew(.,2*i-1) and (.,2*i)

            elemstnew(1,2*i-1) = istart
            elemstnew(2,2*i-1) = istart+numelms1-1
            elemstnew(1,2*i) = istart+numelms1
            elemstnew(2,2*i) = elemstold(2,i)
            if ( debug ) &
               call prinin1 ( elemstnew(1,2*i-1), 2, 2, 'elemstnew' )

         end do  ! i = 1, 2**(nlevelsub-1)

!        --- Copy elemstnew onto elemstold

         do i = 1, 2**ilevel
            elemstold(1,i) = elemstnew(1,i)
            elemstold(2,i) = elemstnew(2,i)
         end do

      end do  ! ilevel = 1, nlevelsub

!     --- Fill array nelements

      do i = 1, nprocessors
         nelements(i) = elemstold(2,i)
      end do

1000  if ( debug ) then

!     --- Debug information

         call prinin ( nelements, nprocessors, 'nelements' )
         call prinin ( elements, nelem, 'elements' )
         write(irefwr,*) 'End mshparelblocks'

      end if
      call erclos ( 'mshparelblocks' )

      deallocate ( marknodes, elements )

      end
