      subroutine mshparelemsmet ( nprocessors )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-03-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subdivide the region into nprocessors blocks using
!     the metis package
! **********************************************************************
!
!                       KEYWORDS
!
!     parallel
!     mesh
!     metis
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemesh
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: nprocessors

!     nprocessors    i    Number of processors that is to be used for
!                         parallel processing
!                         Defines also the number of submeshes
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, allocatable :: elperseg(:), iwork(:)
      integer :: iprocessor, numelms, numnodes
      integer, allocatable :: elements(:), points(:)

!     elements       Array of size nelemloc containing the element
!                    numbers corresponding to the local mesh
!     elperseg       Array of size nelem containing the sequence
!                    number of the processor to which it is added
!     iprocessor     Processor sequence number
!     iwork          integer work array of size npoint
!     numelms        Number of elements
!     numnodes       Number of nodal points in array nodes
!     points         Array of size npointloc containing the node
!                    numbers corresponding to the local mesh
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MSHMETIS       Subdivide a region into blocks using the metis subroutines
!     MSHMETIS3      Fill array elements with the elements corresponding to one
!                    block and also the corresponding array points
!                    with the point numbers
!                    The elements are subdivided by metis
!     MSHPARELEMS3   Store arrays elements and points into ibuffr and store the
!                    corresponding memory management sequence numbers into
!                    elempointers and nodepointers
!     MSHPARRENUMMET Renumber nodes per cpu created by metis
!                    for a parallel subdivision
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshparelemsmet' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshparelemsmet'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( elperseg(nelem), iwork(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, nelem, 'elperseg' )
      if ( ierror/=0 ) go to 1000
      allocate ( elements(nelem), points(npoint) )

!     --- Subdivision by metis

      call mshmetis ( nprocessors, elperseg )
      if ( ierror/=0 ) go to 1000

!     --- Loop over the processors in reversed order

      do iprocessor = nprocessors, 1, -1

!     --- subdivide region in blocks using metis

         call mshmetis3 ( iprocessor, nprocessors, elements, points, &
                          numelms, numnodes, elperseg )
         if ( ierror/=0 ) go to 1000

         if ( debug ) then
            call prinin ( elements, numelms, 'elements' )
            call prinin ( points, numnodes, 'points' )
         end if  ! ( debug )

!        --- reorder nodes
!            This ensures that disjoint parts have node numbers in increasing
!            sequence

         call mshparrenummet ( elements, points, numelms, numnodes )

!        --- Copy arrays elements and ipoints in ibuffr

         call mshparelems3 ( elements, iprocessor, numelms, points, &
                             numnodes )
         if ( ierror/=0 ) go to 1000

      end do  ! nprocessors, 1, -1

      deallocate ( elperseg, iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'elperseg' )

      deallocate ( elements, points )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshparelemsmet'

      end if  ! ( debug )
      call erclos ( 'mshparelemsmet' )

      end subroutine mshparelemsmet
