      subroutine mshparrenummet ( elements, points, numelms, numnodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-06-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Renumber nodes per cpu created by metis for a parallel subdivision
!     The nodes are renumbered such that disjoint parts contain nodes
!     in increasing sequence
!     Cuthill McKee is used
! **********************************************************************
!
!                       KEYWORDS
!
!     parallel
!     mesh
!     element
!     node
!     subdivision
!     renumbering
!     reordering
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: numelms, elements(numelms), numnodes
      integer, intent(inout) :: points(numnodes)

!     elements       i    Array of size numelms containing the element
!                         numbers corresponding to the local mesh
!     numelms        i    Number of elements
!     numnodes       i    Number of nodal points in array points
!     points        i/o   Array of size numnodes containing the node
!                         numbers corresponding to the local mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, pointer :: markelem(:), kmeshd1(:), kmeshd2(:), &
                          kmeshe1(:), kmeshe2(:)

!     kmeshd1        first part of kmesh part d, contains the number of
!                    elements corresponding to each nodal point accumulated
!                    kmeshd1(1) = 0.
!                    Length: npoint+1
!     kmeshd2        second part of kmesh part d
!                    Contains all the element numbers the nodal points are
!                    lying in sequentially
!     kmeshe1        Kmesh part e1, contains number of neighbor nodes
!                    of a node accumulated
!     kmeshe2        Kmesh part e2, contains neighbor nodes of a node
!     markelem       Work array to mark elements that must be used
!                    0: element is skipped
!                    1: element is used
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PR0012         Print accumulated array
!     PRININ         print 1d integer array
!     SEPCUTHILL     Renumber a set of (possibly disjoint) nodes according to
!                    Cuthill McKee
!     SEPFILLKMESHD1 Compute length of KMESH part D2, fill KMESH part D1
!     SEPFILLKMESHD2 Fill KMESH part D2
!     SEPFILLKMESHE1 Compute length of KMESH part E2
!     SEPFILLKMESHE2 Fill KMESH part E2
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     markelem(1:nelem) := 0
!     Mark active elements in markelem
!     Fill array kmeshe with nodes coupled to nodes
!     Apply Cuthill McKee to get new numbering
!     Refill array points
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshparrenummet' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshparrenummet'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( markelem(nelem), kmeshd1(npoint+1), kmeshe1(npoint+1), &
                 levelstruc(npoint), pointlevel(npoint+1), &
                 marknodes(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, nelem+2*npoint, 'markelem' )
      if ( ierror/=0 ) go to 1000

!     --- markelem(1:nelem) := 0
!         Mark active elements in markelem

      markelem = 0
      markelem(elements) = 1
      marknodes = 0

      if ( debug ) call prinin ( markelem, nelem, 'markelem' )

!     --- Fill array kmeshe with nodes coupled to nodes
!         First we fill array kmeshd1 with number of elements corresponding
!         to nodes (accumulated)

      kmshd1 => kmeshd1
      call sepfillkmeshd1 ( markelem )

!     --- Next kmeshd2 is filled with element numbers

      length = kmeshd1(npoint+1)
      allocate ( kmeshd2(length), stat = error )
      if ( error/=0 ) call eralloc ( error, length, 'kmeshd2' )
      if ( ierror/=0 ) go to 1000

      kmshd2 => kmeshd2
      call sepfillkmeshd2 ( markelem )

!     --- Fill kmesh part e1 and compute length of part e2

      kmshe1 => kmeshe1
      call sepfillkmeshe1

      if ( debug) then
         write(irefwr,*) 'elements coupled to nodes'
         call pr0012 ( npoint, kmeshd1, kmeshd2 )
      end if  ! ( debug)

!     --- Next kmeshe2 is filled with node numbers

      length = kmeshe1(npoint+1)
      allocate ( kmeshe2(length), stat = error )
      if ( error/=0 ) call eralloc ( error, length, 'kmeshe2' )
      if ( ierror/=0 ) go to 1000

      kmshe2 => kmeshe2
      call sepfillkmeshe2

      if ( debug) then
         write(irefwr,*) 'nodes coupled to nodes'
         call pr0012 ( npoint, kmeshe1, kmeshe2 )
      end if  ! ( debug)

!     --- Apply Cuthill McKee to get new numbering
!         Refill array points

      call sepcuthill ( numnodes, points )

      deallocate ( markelem, kmeshd1, kmeshd2, kmeshe1, kmeshe2, &
                   levelstruc, pointlevel, marknodes, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'markelem' )

      kmshd1 => kmeshd(1:npoint+1)
      kmshd2 => kmeshd(npoint+2:size(kmeshd))
      kmshe1 => kmeshe(1:npoint+1)
      kmshe2 => kmeshe(npoint+2:size(kmeshe))

1000  if ( debug ) then

!     --- Debug information

         call prinin ( points, numnodes, 'points' )
         write(irefwr,*) 'End mshparrenummet'

      end if  ! ( debug )
      call erclos ( 'mshparrenummet' )

      end subroutine mshparrenummet
