      subroutine mshpav2setzero ( maxbounds, nowbnd, nloop, crd, layer )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 30-01-2006 Jump in case of loop without nodes
!        version  1.1  date 04-06-2004 Put thickness in crd(4,.)
!        version  1.0  date 28-11-2003
!
!   copyright (c) 2003-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Set thickness equal to zero for nodes in edges where one of the
!     two nodes has already thickness zero
!     This is to avoid too many edges ending in one point
!     The number of layers may increase only slowly
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     paver
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer maxbounds, nowbnd(maxbounds,*), nloop, layer
      double precision crd(4,*)

!     crd           i/o   array containing the coordinates of the nodes of
!                         the boundary as well as the thickness and factor
!                         See subroutine mshpav
!     layer          i    Present layer sequence number
!     maxbounds      i    Max number of loops in nowbnd
!     nloop          i    number of boundaries in nowbnd
!     nowbnd         i    array containing the boundary nodes for each closed
!                         part
!                         This array is two-dimensional of size (imxlps,*), &
!                         where implxs defines the maximum number of loops
!                         and * is at most the number of nodes in the boundary+1
!                         Pos. (i,1) contains the number of nodes in the closed
!                            boundary part i
!                         Pos. (i,j+1) contains the j-th node number along this
!                            boundary
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ibndnr, ind, inode, numnodes, inodenext
      double precision thickness, thickprev, thicknext

!     ibndnr         i'th boundary in nowbnd
!     ind            Relative sequence number of node along boundary
!     inode          Node sequence number
!     inodenext      Next node along boundary
!     numnodes       Number of nodal points
!     thickness      Thickness in node
!     thicknext      Thickness in next node
!     thickprev      Thickness in previous node
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For ibndnr := 1 (1) nloop do
!        numnodes := number of nodes in ibndnr
!        inode := last node of ibndnr; thickprev := thickness in inode
!        for ind := 1 (1) numnodes do
!           inode := ind-th node in ibndnr; thickness := thickness in inode
!           inodenext := next node; thicknext := thickness in inodenext
!           if ( thickprev = 0 or thicknext = 0 ) then
!              set thickness in inode to 0
!           thickprev := thickness
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpav2setzero' )
      debug = .false.
      if ( layer>=10000 ) debug = .true.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpav2setzero'
         write(irefwr,*) 'layer, nloop ', layer, nloop

      end if
      if ( ierror/=0 ) go to 1000

!     --- For ibndnr := 1 (1) nloop do

      do ibndnr = 1, nloop

!     --- numnodes := number of nodes in ibndnr
!         inode := last node of ibndnr; thickprev := thickness in inode

         numnodes = nowbnd(ibndnr,1)
         if ( numnodes>0 ) then

!        --- There are nodes in this loop
!            Set thickprev

            inode = nowbnd(ibndnr,numnodes+1)
            thickprev = crd(3,inode)

         end if  ! ( numnodes>0 )

!        --- for ind := 1 (1) numnodes do

         do ind = 1, numnodes

!        --- inode := ind-th node in ibndnr; thickness := thickness in inode
!            inodenext := next node; thicknext := thickness in inodenext

            inode = nowbnd(ibndnr,ind+1)
            thickness = crd(3,inode)
            if ( ind<numnodes ) then
               inodenext = nowbnd(ibndnr,ind+2)
            else
               inodenext = nowbnd(ibndnr,2)
            end if
            thicknext = crd(3,inodenext)
            if ( (thickprev<=0d0 .or. thicknext<=0d0) .and. &
                 thickness>0d0 ) then

!           --- thickprev = 0 or thicknext = 0, &
!               i.e. thickness in first node of edge or last node of edge is 0
!               set thickness in inode to 0

               crd(3,inode) = 0d0
               crd(4,inode) = thickness
               if ( debug ) write(irefwr,*) 'ibndnr, ind, inode ', &
                  ibndnr, ind, inode

            end if  ! ( thickprev<=0d0 )

!           --- thickprev := thickness

            thickprev = thickness

         end do  ! ind = 1, numnodes

      end do  ! ibndnr = 1, nloop

1000  call erclos ( 'mshpav2setzero' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshpav2setzero'

      end if

      end
