      subroutine mshpav2trian ( inode1, inode2, inode3, inode4, &
                                crd, nel, kmeshc, icount )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-07-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create 2 triangles from the four nodes inode1-4 which form the
!     closed boundary in that sequence
!     The diagonal is chosen such that the largest angle is subdivided
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     paver
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision crd(4,*)
      integer inode1, inode2, inode3, inode4, nel, kmeshc(3,*), icount

!     crd            i    array containing the coordinates of the nodes of
!                         the boundary as well as the thickness and factor
!                         See subroutine mshpav
!     icount        i/o   Defines the direction of the diagonal in case of
!                         a rectangular cell (0 or 1)
!                         This direction is changed subsequently
!     inode1         i    First node in boundary
!     inode2         i    Second node in boundary
!     inode3         i    Third node in boundary
!     inode4         i    Fourth node in boundary
!     kmeshc        i/o   kmesh part c, contains the nodal points
!                         corresponding to the elements
!     nel           i/o   number of elements in kmeshc
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision x(6), y(6)
      integer idirec
      logical different

!     different      If true all nodes are different
!     idirec         Defines in which direction the diagonal must be made
!                    0: from 1 to 3
!                    1: from 2 to 4
!                    2: all angles are 90 degrees
!     x              x co-ordinates of vertices of quadrilateral in
!                    positions 2 to 5
!                    The positions 1 and 6 are used as work space
!     y              y-coordinates of vertices of quadrilateral in
!                    positions 2 to 5
!                    The positions 1 and 6 are used as work space
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHR17         detect direction of diagonal for triangular elements
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpav2trian' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpav2trian'

      end if
      different = .true.
      if ( inode1.eq.inode2 .or. inode1.eq.inode3 .or. &
           inode1.eq.inode4 ) then

!     --- node1 is equal to one of the other nodes, check rest

         different = .false.
         if ( inode2.ne.inode3 .and. inode2.ne.inode4 .and. &
              inode3.ne.inode4 ) then
            nel = nel + 1
            kmeshc(1,nel) = inode2
            kmeshc(2,nel) = inode3
            kmeshc(3,nel) = inode4
         end if

      else if ( inode2.eq.inode3 .or. inode2.eq.inode4 ) then

!     --- node2 is equal to one of the other nodes, check rest

         different = .false.
         if ( inode3.ne.inode4 ) then
            nel = nel + 1
            kmeshc(1,nel) = inode1
            kmeshc(2,nel) = inode3
            kmeshc(3,nel) = inode4
         end if

      else if ( inode3.eq.inode4 ) then

!     --- node3 is equal to node4

         different = .false.
         nel = nel + 1
         kmeshc(1,nel) = inode1
         kmeshc(2,nel) = inode2
         kmeshc(3,nel) = inode3

      end if

      if ( different ) then

!     --- Fill 4 coordinates in x(2-5), y(2-5)

         x(2) = crd(1,inode1)
         x(3) = crd(1,inode2)
         x(4) = crd(1,inode3)
         x(5) = crd(1,inode4)

         y(2) = crd(2,inode1)
         y(3) = crd(2,inode2)
         y(4) = crd(2,inode3)
         y(5) = crd(2,inode4)

         call mshr17 ( x, y, idirec )

         if ( idirec.eq.0 .or. idirec.eq.2 .and. icount.eq.0 ) then

!        --- diagonal from present node to last node created

            nel = nel + 1
            kmeshc(1,nel) = inode1
            kmeshc(2,nel) = inode2
            kmeshc(3,nel) = inode4

            nel = nel + 1
            kmeshc(1,nel) = inode3
            kmeshc(2,nel) = inode4
            kmeshc(3,nel) = inode2

            icount = 1

         else

!        --- the diagonal goes from next node to previous node created

            nel = nel + 1
            kmeshc(1,nel) = inode2
            kmeshc(2,nel) = inode3
            kmeshc(3,nel) = inode1

            nel = nel + 1
            kmeshc(1,nel) = inode4
            kmeshc(2,nel) = inode1
            kmeshc(3,nel) = inode3

            icount = 0

         end if  ! ( idirec.eq.0 .or. idirec.eq.2 .and. icount.eq.0 )

      end if  ! ( different )

      call erclos ( 'mshpav2trian' )
      if ( debug ) write(irefwr,*) 'End mshpav2trian'

      end
