      subroutine mshpavbndcheck ( numnodes, bndcoornew, nodenrs, &
                                  bndcoorold, layer, ibndnr, nowbnd, &
                                  maxbounds, imark )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-10-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if old and new boundary do not intersect and also if
!     lines from old to new boundary do not introduce intersections
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     paver
!     triangle
!     check
!     boundary
!     intersection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numnodes, nodenrs(*), layer, ibndnr, maxbounds, &
              nowbnd(maxbounds,*), imark(*)
      double precision bndcoornew(4,*), bndcoorold(2,*)

!     bndcoornew    i/o   Contains the coordinates of the new boundary created
!                         by making one layer during the paver algorithm
!     bndcoorold    i/o   Contains the coordinates of the previous (old)
!                         boundary during the paver process
!     ibndnr         i    i'th boundary in nowbnd
!     imark         i/o   Array indicating for each point in a boundary loop
!                         how it should be treated in the following way:
!                          -1: new node is skipped  (sharp corner)
!                           0: thickness = 0: reuse old node
!                           1: standard case: a new node is created
!                         n>1: n new node are created (concave case)
!     layer          i    Present layer sequence number
!     maxbounds      i    maximum number of loops in nowbnd in paver
!     nodenrs       i/o   Array containing for each point in a boundary loop, &
!                         the new node number after paving, if it has one
!     nowbnd         i    array containing the boundary nodes for each closed
!                         part
!                         This array is two-dimensional of size (maxbounds,*), &
!                         where implxs defines the maximum number of loops
!                         and * is at most the number of nodes in the boundary+1
!                         Pos. (i,1) contains the number of nodes in the closed
!                            boundary part i
!                         Pos. (i,j+1) contains the j-th node number along this
!                            boundary
!     numnodes       i    Number of nodal points in array nodes
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iedge, jedge, inode1, inode2, inode3, inode4, inode5, &
              inode6, i, j, jedge1
      double precision x1(2), x2(2), x3(2), x4(2), xmin, xmax, ymin, &
                       ymax, xminnew, xmaxnew, yminnew, ymaxnew, fact1, &
                       fact2, x5(2), x6(2), x7(2)
      logical outbox, changed

!     changed        If true a boundary has been changed
!     fact1          multiplication factor
!     fact2          multiplication factor
!     i              Counting variable
!     iedge          edge sequence number
!     inode1         First node in edge
!     inode2         Second node in edge
!     inode3         First node in second edge
!     inode4         Second node in second edge
!     inode5         First node in third edge
!     inode6         Second node in third edge
!     j              Counting variable
!     jedge          edge sequence number
!     jedge1         last node of edge jedge
!     outbox         if true the edge lies outside the rectangle
!     x1             coordinates of inode1
!     x2             coordinates of inode2
!     x3             coordinates of inode3
!     x4             coordinates of inode4
!     x5             coordinates of inode5
!     x6             coordinates of inode6
!     x7             help array to store coordinates
!     xmax           Maximum x-value
!     xmaxnew        Maximum x-value of edge
!     xmin           Minimum x-value
!     xminnew        Minimum x-value of edge
!     ymax           Maximum y-value
!     ymaxnew        Maximum y-value of edge
!     ymin           Minimum y-value
!     yminnew        Minimum y-value of edge
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHCROSSLINE   Computes the intersection of two lines
!     PRININ         print 1d integer array
!     PRINNOWBND     Print array nowbnd which is used in mshpav
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For all edges on the boundary do
!        [xmin, xmax, ymin, ymax] := rectangle around old edge
!        For all edges on new boundary do
!           if ( edge in rectangle ) then
!              compute intersection of edges
!              if ( edges intersect ) then
!                 adapt boundary nodes
!     For all nodes on the boundary do
!        line := line from old to new boundary
!        if ( line intersects previous line ) then
!           adapt boundary nodes
!     For all nodes on the boundary do
!        if ( line intersects new boundary ) then
!           adapt boundary nodes
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavbndcheck' )
      debug = .false.
      if ( layer.ge.10000 ) debug = .true.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavbndcheck'
         call prinin ( nodenrs, numnodes, 'nodenrs' )
         call prinnowbnd ( nowbnd, ibndnr, maxbounds, layer )

      end if
      if ( ierror.ne.0 ) go to 1000
      changed = .false.

!     --- Copy first node behind the arrays bndcr, bndcoornew and nodenrs
!         for ease of programming

      nodenrs(numnodes+1) = nodenrs(1)
      bndcoorold(1,numnodes+1) = bndcoorold(1,1)
      bndcoorold(2,numnodes+1) = bndcoorold(2,1)
      bndcoornew(1,numnodes+1) = bndcoornew(1,1)
      bndcoornew(2,numnodes+1) = bndcoornew(2,1)

!     --- For all edges on the boundary do

      do iedge = 1, numnodes

!     --- [xmin, xmax, ymin, ymax] := rectangle around old edge

         x1(1) = bndcoorold(1,iedge)
         x1(2) = bndcoorold(2,iedge)
         x2(1) = bndcoorold(1,iedge+1)
         x2(2) = bndcoorold(2,iedge+1)
         xmin = min(x1(1),x2(1))
         xmax = max(x1(1),x2(1))
         ymin = min(x1(2),x2(2))
         ymax = max(x1(2),x2(2))
         inode1 = nowbnd(ibndnr,iedge+1)
         inode2 = nowbnd(ibndnr,iedge+2)

!        --- For all edges on new boundary do

         do jedge = 1, numnodes

            jedge1 = jedge+1
            if ( jedge1.gt.numnodes ) jedge1 = 1
            inode3 = nodenrs(jedge)
            inode4 = nodenrs(jedge1)
            if ( inode1.ne.inode3 .and. inode1.ne.inode4 .and. &
                 inode2.ne.inode3 .and. inode2.ne.inode4 .and. &
                 inode3.ne.inode4 ) then

!           --- Nodes do not coincide

               x3(1) = bndcoornew(1,jedge)
               x3(2) = bndcoornew(2,jedge)
               x4(1) = bndcoornew(1,jedge1)
               x4(2) = bndcoornew(2,jedge1)
               xminnew = min(x3(1),x4(1))
               xmaxnew = max(x3(1),x4(1))
               yminnew = min(x3(2),x4(2))
               ymaxnew = max(x3(2),x4(2))

               outbox = xmaxnew.lt.xmin .or. xminnew.gt.xmax .or. &
                        ymaxnew.lt.ymin .or. yminnew.gt.ymax

               if ( .not. outbox ) then

!              --- edge in rectangle
!                  compute intersection of edges

                  call mshcrossline ( x1, x2, x3, x4, fact1, fact2, &
                                      sqreps )
                  if ( fact2.ge.-sqreps .and. fact2.le.1d0+sqreps ) then

!                 --- edges intersect
!                     Reset this boundary part to old boundary

                     nodenrs(jedge) = nowbnd(ibndnr,jedge1)
                     nodenrs(jedge1) = nowbnd(ibndnr,jedge+2)
                     bndcoornew(1,jedge) = bndcoorold(1,jedge)
                     bndcoornew(2,jedge) = bndcoorold(2,jedge)
                     bndcoornew(3,jedge) = 0d0
                     bndcoornew(1,jedge1) = bndcoorold(1,jedge1)
                     bndcoornew(2,jedge1) = bndcoorold(2,jedge1)
                     bndcoornew(3,jedge1) = 0d0
                     imark(jedge) = 0
                     imark(jedge1) = 0
                     changed = .true.

                  end if  ! ( fact2.ge.-sqreps .and. fact2.le.1d0+sqreps )

               end if  ! ( .not. outbox )

            end if  ! ( inode1.ne.inode3 ... )

         end do  ! jedge = 1, numnodes

      end do  ! iedge = 1, numnodes

!     --- Find last node on new boundary

      inode3 = nowbnd(ibndnr,numnodes+1)
      inode4 = nodenrs(numnodes)
      x3(1) = bndcoorold(1,numnodes)
      x3(2) = bndcoorold(2,numnodes)
      x4(1) = bndcoornew(1,numnodes)
      x4(2) = bndcoornew(2,numnodes)

!     --- For all nodes on the boundary do

      do i = 1, numnodes

!     --- line := line from old to new boundary

         inode1 = nowbnd(ibndnr,i+1)
         inode2 = nodenrs(i)
         x1(1) = bndcoorold(1,i)
         x1(2) = bndcoorold(2,i)
         x2(1) = bndcoornew(1,i)
         x2(2) = bndcoornew(2,i)

!        --- We do not want a node ending close to another boundary
!            Therefore the line is increased by a factor 2 (i.e. +h)

         x7(1) = x2(1)+2d0*(x2(1)-x1(1))
         x7(2) = x2(2)+2d0*(x2(2)-x1(2))
         if ( inode1.ne.inode2 ) then

!        --- Define box along nodes

            xmin = min(x1(1),x2(1))
            xmax = max(x1(1),x2(1))
            ymin = min(x1(2),x2(2))
            ymax = max(x1(2),x2(2))

            if ( inode2.ne.inode4 .and. inode3.ne.inode4 ) then

!           --- Line from old to new boundary found
!               Compute intersection of x1-x2; x3-x4

               call mshcrossline ( x1, x2, x3, x4, fact1, fact2, &
                                   sqreps )

               if ( fact2.ge.-sqreps .and. fact2.le.1d0+sqreps ) then

!              --- line intersects previous line
!                  adapt boundary nodes

                  nodenrs(i) = inode4
                  if ( debug ) write(irefwr,*) 'i, inode1...4 ', i, &
                     inode1, inode2, inode3, inode4

               end if  ! ( fact2.ge.-sqreps .and. fact2.le.1d0+sqreps )

            end if  ! ( inode2.ne.inode4 .and. inode3.ne.inode4 )
            x4(1) = x2(1)
            x4(2) = x2(2)
            inode4 = inode2

!           --- if ( line intersects new boundary ) then
!               adapt boundary nodes
!               For all edges on new boundary do

            do jedge = 1, numnodes

               inode5 = nodenrs(jedge)
               jedge1 = jedge+1
               if ( jedge1.gt.numnodes ) jedge1 = 1
               inode6 = nodenrs(jedge1)
               if ( inode1.ne.inode5 .and. inode1.ne.inode6 .and. &
                    inode2.ne.inode5 .and. inode2.ne.inode6 .and. &
                    inode5.ne.inode6 ) then

!              --- Nodes do not coincide

                  x5(1) = bndcoornew(1,jedge)
                  x5(2) = bndcoornew(2,jedge)
                  x6(1) = bndcoornew(1,jedge1)
                  x6(2) = bndcoornew(2,jedge1)
                  xminnew = min(x5(1),x6(1))
                  xmaxnew = max(x5(1),x6(1))
                  yminnew = min(x5(2),x6(2))
                  ymaxnew = max(x5(2),x6(2))

                  outbox = xmaxnew.lt.xmin .or. xminnew.gt.xmax .or. &
                           ymaxnew.lt.ymin .or. yminnew.gt.ymax

                  if ( .not. outbox ) then

!                 --- edge in rectangle
!                     compute intersection of edges

                     call mshcrossline ( x1, x7, x5, x6, fact1, &
                                         fact2, sqreps )
                     if ( fact2.ge.-sqreps .and. &
                          fact2.le.1d0+sqreps ) then

!                    --- line from old boundary to new boundary intersects
!                        new boundary
!                        adapt boundary nodes by replacing inode2 by
!                        inode5 or inode6

                        nodenrs(jedge) = nowbnd(ibndnr,jedge1)
                        nodenrs(jedge1) = nowbnd(ibndnr,jedge+2)
                        bndcoornew(1,jedge) = bndcoorold(1,jedge)
                        bndcoornew(2,jedge) = bndcoorold(2,jedge)
                        bndcoornew(3,jedge) = 0d0
                        bndcoornew(1,jedge1) = bndcoorold(1,jedge1)
                        bndcoornew(2,jedge1) = bndcoorold(2,jedge1)
                        bndcoornew(3,jedge1) = 0d0
                        nodenrs(i) = nowbnd(ibndnr,i+1)
                        bndcoornew(1,i) = bndcoorold(1,i)
                        bndcoornew(2,i) = bndcoorold(2,i)
                        imark(jedge) = 0
                        imark(jedge1) = 0
                        imark(i) = 0
                        changed = .true.
                        do j = 1, numnodes

                           if ( nodenrs(j).eq.inode2 ) then

!                          --- node2 found

                              nodenrs(j) = nowbnd(ibndnr,j+1)
                              bndcoornew(1,j) = bndcoorold(1,j)
                              bndcoornew(2,j) = bndcoorold(2,j)
                              imark(j) = 0

                           end if  ! ( nodenrs(j).eq.inode2 )

                        end do  ! j = 1, numnodes

                     end if  ! ( fact2.ge.-sqreps .and. fact2.le.1d0+sqreps )

                  end if  ! ( .not. outbox )

               end if  ! ( inode1.ne.inode3 ... )

            end do  ! jedge = 1, numnodes

         else

!        --- inode1 = inode2

            x4(1) = x2(1)
            x4(2) = x2(2)
            inode4 = inode2

         end if  ! ( inode1.ne.inode2 )
         x3(1) = x1(1)
         x3(2) = x1(2)
         inode3 = inode1

      end do  ! i = 1, numnodes

      if ( changed ) then

!     --- The boundary has been changed, remove all entries of imark > 1

         do i = 1, numnodes
            imark(i) = min(1,imark(i))
         end do

      end if

1000  call erclos ( 'mshpavbndcheck' )

      if ( debug ) then

!     --- Debug information

         call prinin ( nodenrs, numnodes, 'nodenrs' )
         write(irefwr,*) 'End mshpavbndcheck'

      end if

      end
