      subroutine mshpavmnsmoot6 ( layer, nowbnd, maxbounds, crd, newbnd, &
                                  kmeshc, nel, ibndnr, nloop )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 10-03-2005 Correction nodeprev = 0
!        version  1.0  date 22-11-2003
!
!   copyright (c) 2003-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Smooth mesh by adding extra elements in sharp corners
!     If layer >= 4 sharp means 90 degrees
!     If layer = 3 sharp means 87 degrees
!     If layer = 2 sharp means 67 degrees
!     If layer = 1 sharp means 50 degrees
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     paver
!     triangle
!     smoothing
!     boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshc(3,*), maxbounds, nowbnd(maxbounds,*), newbnd(*), &
              nel, nloop, ibndnr, layer
      double precision crd(4,*)

!     crd            i    array containing the coordinates of the nodes of
!                         the boundary as well as the thickness and factor
!                         See subroutine mshpav
!     ibndnr         i    i'th boundary in nowbnd
!     kmeshc        i/o   kmesh part c, contains the nodal points
!                         corresponding to the elements
!     layer          i    Present layer sequence number
!     maxbounds      i    Max number of loops in nowbnd
!     nel           i/o   number of elements in the topology
!     newbnd        i/o   temp array containing the boundary nodes for one loop
!     nloop          i    number of boundaries in nowbnd
!     nowbnd         i    array containing the boundary nodes for each closed
!                         part
!                         This array is two-dimensional of size (imxlps,*), &
!                         where implxs defines the maximum number of loops
!                         and * is at most the number of nodes in the boundary+1
!                         Pos. (i,1) contains the number of nodes in the closed
!                            boundary part i
!                         Pos. (i,j+1) contains the j-th node number along this
!                            boundary
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inodeprev, numnodes, inode, inodenext, icount, i
      double precision minangle, x1(2), x2(2), x3(2), angle, h, t1(2), &
                       t2(2), norm, inner
      logical repeat, allowed

!     allowed        If true an element may be created
!     angle          angle between two edges
!     h              width of layer in point
!     i              Counting variable
!     icount         Counter
!     inner          inner product
!     inode          Node sequence number
!     inodenext      Next node along boundary
!     inodeprev      Previous node along boundary
!     minangle       maximum angle in boundary for which an element may be
!                    created
!     norm           norm of vector
!     numnodes       Number of nodal points in array newbnd
!     repeat         If true the process has not yet been finished
!     t1             First tangential vector
!     t2             Second tangential vector
!     x1             coordinates of first point
!     x2             coordinates of second point
!     x3             coordinates of third point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHANGLE       Compute the cosine of the angle between the nodes with
!                    coordinates x1, x2 and x3, where the angle is defined
!                    as x3-x1-x2
!     MSHPAV1TRIAN   Create 1 triangle from the three nodes inode1-3 which form
!                    the closed boundary in that sequence
!     MSHPAVMNCHELM  Check if the line x1-x2 does not intersect one of the
!                    boundaries of the mesh or is too close to the boundaries
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     minangle := minimum angle corresponding to layer
!     repeat := true
!     while ( repeat ) do
!        repeat := false
!        For all edges along boundary do
!           if ( angle<minangle ) then
!              if ( element may be created by connecting nodes on boundary) then
!                 create element
!                 repeat := true
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavmnsmoot6' )
      debug = .false.
      if ( layer>=10000 ) debug = .true.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavmnsmoot6'
         write(irefwr,*) 'layer, ibndnr, nloop, nel ', &
                          layer, ibndnr, nloop, nel
         call prinin ( newbnd(2), newbnd(1), 'newbnd/start' )

      end if
      if ( ierror/=0 ) go to 1000

!     --- minangle := minimum angle corresponding to layer

      if ( layer<=1 ) then
         minangle = 0.7d0               ! cos(50) approximately
      else if ( layer==2 ) then
         minangle = 0.5d0               ! cos(67)
      else if ( layer==3 ) then
         minangle = 0.2d0               ! cos(87)
      else  ! ( layer>3 )
         minangle = -0.0001d0       ! cos(90)
      end if  ! ( layer<=3 )
      if ( debug ) write(irefwr,*) 'minangle ', minangle

!     --- repeat := true

      repeat = .true.

!     --- while ( repeat ) do

      do while ( repeat )

!     --- repeat := false

         repeat = .false.
         numnodes = newbnd(1)              ! number of nodes along boundary
         inodeprev = newbnd(numnodes+1)    ! previous node on boundary
         icount = 0                        ! number of nodes that are removed

!        --- For all edges along boundary do

         do i = 1, numnodes
            inode = newbnd(i+1)            ! present node
            if ( i<numnodes ) then
               inodenext = newbnd(i+2)     ! next node
            else
               inodenext = newbnd(2)
               if ( inodenext==0 ) go to 100 ! first node has been removed
            end if
            if ( inodeprev==0 ) go to 100  ! previous node has been removed

            x1(1) = crd(1,inode)           ! coordinates of inode
            x1(2) = crd(2,inode)
            x3(1) = crd(1,inodeprev)       ! coordinates of inodeprev
            x3(2) = crd(2,inodeprev)
            x2(1) = crd(1,inodenext)       ! coordinates of inodenext
            x2(2) = crd(2,inodenext)

!           --- Compute cosine of angle
!               First we have to check if the angle is ess than pi
!               This is done by comparing the normal vector (-t1(2),t1(1))
!               on the first edge with the tangential vector t2 of the second
!               edge
!               If the inner product is less than 0, the angle is more than pi

            t1(1) = x1(1)-x3(1)
            t1(2) = x1(2)-x3(2)
            norm = 1d0/sqrt(t1(1)**2+t1(2)**2)
            t1(1) = norm*t1(1)
            t1(2) = norm*t1(2)
            t2(1) = x2(1)-x1(1)
            t2(2) = x2(2)-x1(2)
            norm = 1d0/sqrt(t2(1)**2+t2(2)**2)
            t2(1) = norm*t2(1)
            t2(2) = norm*t2(2)
            inner = -t1(2)*t2(1)+t1(1)*t2(2)
            if ( inner<sqreps ) then
               angle = -1d0
            else
               call mshangle ( x1, x2, x3, angle, 2 )
            end if
            allowed = angle>=minangle      ! angle>=minangle
            if ( .not. allowed .and. angle>0d0 ) then

!           --- angle is less than 90 degrees, but layer<4
!               we may remove this angle if the new diagonal is less than 1.5h

               h = crd(3,inode)*crd(4,inode)
               if ( sqrt((x3(1)-x2(1))**2+(x3(2)-x2(2))**2)<=1.5d0*h ) &
                  allowed = .true.

            end if

            if ( allowed ) then

!           --- cos(angle)>=cos(minangle), i.e angle<=minangle
!               Here we have a candidate for an element
!               Check if the line x2 - x3 does not intersect one of the
!               other boundaries or is too close to such boundary

               call mshpavmnchelm ( allowed, nowbnd, maxbounds, crd, &
                                    newbnd, ibndnr, inodeprev, &
                                    inodenext, inode, x2, x3, nloop )
               if ( debug ) &
                  write(irefwr,*) 'i, inode, inodeprev, inodenext,', &
                                  ' angle, allowed ', &
                                   i, inode, inodeprev, inodenext, &
                                   angle, allowed
               if ( allowed ) then

!              --- element may be created by connecting nodes on boundary
!                  create element
!                  repeat := true

                  call mshpav1trian ( inodeprev, inode, inodenext, &
                                      nel, kmeshc )
                  repeat = .true.

!                 --- Raise icount and remove node from newbnd

                  icount = icount+1
                  newbnd(i+1) = 0
                  inode = 0     !trick to make inodeprev equal to 0

               end if  ! ( allowed )

            end if  ! ( angle>=minangle )

100         inodeprev = inode

         end do  ! i = 1, numnodes

         if ( icount>0 ) then

!        --- At least one of the nodes of newbnd is removed, update the boundary

            icount = 0
            do i = 1, numnodes

               inode = newbnd(i+1)
               if ( inode>0 ) then

!              --- New node found, add

                  icount = icount+1
                  newbnd(icount+1) = inode

               end if

            end do  ! i = 1, numnodes

            newbnd(1) = icount

         end if  ! ( icount>0 )

         if ( icount==3 ) then

!        --- The boundary consists of 3 nodes, make one element

            call mshpav1trian ( newbnd(2), newbnd(3), newbnd(4), &
                                nel, kmeshc )

         end if  ! ( icount==3 )

         if ( icount>0 .and. icount<=3 ) then

!        --- Boundary contains less than 3 nodes, remove

            newbnd(1) = 0
            repeat = .false.
            icount = 0

         end if

      end do  ! while ( repeat )

1000  call erclos ( 'mshpavmnsmoot6' )

      if ( debug ) then

!     --- Debug information

         call prinin ( newbnd(2), newbnd(1), 'newbnd/end' )
         write(irefwr,*) 'End mshpavmnsmoot6'

      end if

      end
