      subroutine mshpavmnsmooth ( crd, layer, nowbnd, maxbounds, ibndnr, &
                                  nel, kmeshc, ncrd, newbnd, nelprev, &
                                  nbndpt, nloop )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 07-07-2004 new call to plotmeshinter
!        version  1.0  date 05-11-2003
!
!   copyright (c) 2003-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Smooth the layer of elements created by removing small elements
!     if possible
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     paver
!     triangle
!     smoothing
!     boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numnodes, layer, maxbounds, nowbnd(maxbounds,*), ibndnr, &
              nel, kmeshc(3,*), ncrd, newbnd(*), nloop, &
              nelprev, nbndpt
      double precision crd(4,*)

!     crd            i    array containing the coordinates of the nodes of
!                         the boundary as well as the thickness and factor
!                         See subroutine mshpav
!     ibndnr         i    i'th boundary in nowbnd
!     kmeshc        i/o   kmesh part c, contains the nodal points
!                         corresponding to the elements
!     layer          i    Present layer sequence number
!     maxbounds      i    Max number of loops in nowbnd
!     nbndpt         i    Number of nodal points in boundary, length of array
!                         kbndpt
!     ncrd           i    number of nodes in crd
!     nel           i/o   number of elements in the topology
!     nelprev        i    Number of elements stored in kmeshc at the start
!                         of the subroutine
!     newbnd        i/o   temp array containing the boundary nodes for one loop
!     nloop          i    number of boundaries in nowbnd
!     nowbnd         i    array containing the boundary nodes for each closed
!                         part
!                         This array is two-dimensional of size (imxlps,*), &
!                         where implxs defines the maximum number of loops
!                         and * is at most the number of nodes in the boundary+1
!                         Pos. (i,1) contains the number of nodes in the closed
!                            boundary part i
!                         Pos. (i,j+1) contains the j-th node number along this
!                            boundary
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielem, imark(0:ncrd+1), i, inode, icount
      logical deleted, plotdebug

!     deleted        if true an element has been deleted
!     i              Counting variable
!     icount         Counter
!     ielem          Element number
!     imark          Work array of size ncrd indicating if a node is
!                    on new boundary or old boundary
!                    0: not on boundary or on outer boundary
!                    -1: on old boundary
!                    >0: on new boundary
!     inode          Node sequence number
!     numnodes       Number of nodal points in array newbnd
!     plotdebug      If true plots must be made for debugging
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCONTROL     Check control integers around integer array
!     MSHPAVMNSMOOT1 Smooth the layer of elements created by removing small
!                    elements if possible
!     MSHPAVMNSMOOT2 Try to remove all edges on the new boundary that have
!                    length less than 0.5 of both its neighbours, provided
!                    no strange elements are created
!     MSHPAVMNSMOOT6 Smooth mesh by adding extra elements in sharp corners
!     PLOTMESHINTER  Plot actual mesh during the process of creation of a mesh
!     PLOTNOWBND     Plot the actual boundary during mesh generation which is
!                    defined in array nowbnd, used in mshpav
!     PRININ         print 1d integer array
!     PRININ1PART    print 2d integer array partly
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Mark all nodes on new boundary that are not on the old boundary
!     For all new elements do
!        Compute angles of triangle
!        if ( cos(angle) > 0.9 ) then
!           if ( element can be removed, without effecting neighbour elements )
!              remove element
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavmnsmooth' )
      debug = .false.
      plotdebug = .false.
      if ( layer.ge.10000 ) debug = .true.
      if ( layer.ge.10000 ) plotdebug = .true.
      numnodes = newbnd(1)
      if (  numnodes.le.0 ) go to 1000
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavmnsmooth'
         write(irefwr,*) 'layer, ibndnr, numnodes, nbndpt ', &
                          layer, ibndnr, numnodes, nbndpt
         write(irefwr,*) 'nelprev, nel ', nelprev, nel
         call prinin ( newbnd(2), newbnd(1), 'newbnd_start' )

      end if
      if ( plotdebug ) then
         jtimes = 1
         call plotnowbnd ( nowbnd, nloop, maxbounds, layer, crd, &
                           'mshpavmnsmooth/1' )
         jtimes = 3
         call plotmeshinter ( nel, kmeshc, crd, layer, 4, nbndpt, &
                              'mshpavmnsmooth/1' )
         jtimes = 0
      end if
      if ( ierror.ne.0 ) go to 1000

!     --- Set control integers

      imark(0) = 1008
      imark(ncrd+1) = 1008

!     --- Mark all nodes on new boundary that are not on the old boundary
!         0: not on boundary or on outer boundary
!         -1: on old boundary
!         >0: on new boundary

      do i = 1, ncrd
         imark(i) = 0
      end do
      do i = 1, nowbnd(ibndnr,1)
         imark(nowbnd(ibndnr,i+1)) = -1
      end do
      do i = 1, newbnd(1)
         inode = newbnd(i+1)
         if ( inode.ge.nbndpt .and. imark(inode).eq.0 ) imark(inode) = i
      end do
      deleted = .false.

!     --- First we remove all elements with small angles

      call mshpavmnsmoot1 ( numnodes, crd, layer, ibndnr, nel, &
                            kmeshc, newbnd, nelprev, nbndpt, &
                            deleted, imark(1) )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'After mshpavmnsmoot1: nel = ',nel
         call prinin ( newbnd(2), newbnd(1), 'newbnd/1' )

      end if
      if ( plotdebug ) then
         jtimes = 1
         call plotnowbnd ( nowbnd, nloop, maxbounds, layer, crd, &
                           'mshpavmnsmooth/1' )
         jtimes = 3
         call plotmeshinter ( nel, kmeshc, crd, layer, 4, nbndpt, &
                              'mshpavmnsmooth/1' )
         jtimes = 0
      end if

      if ( layer.gt.1 ) then

!     --- layer > 1
!         Next we try to remove edges on the new boundary that have length less
!         than 1/2 of each of its neighbours, provided newbnd has more than 4
!         points and layer >= 2

         call mshpavmnsmoot2 ( crd, newbnd, numnodes, layer, imark(1), &
                               kmeshc, nelprev, nel, deleted, &
                               ibndnr )

         if ( debug ) then

!        --- Debug information

            write(irefwr,*) 'After mshpavmnsmoot2: nel = ',nel
            call prinin ( newbnd(2), newbnd(1), 'newbnd/2' )

         end if
         if ( plotdebug ) then
            jtimes = 1
            call plotnowbnd ( nowbnd, nloop, maxbounds, layer, crd, &
                              'mshpavmnsmooth/2' )
            jtimes = 3
            call plotmeshinter ( nel, kmeshc, crd, layer, 4, nbndpt, &
                                 'mshpavmnsmooth/2' )
            jtimes = 0
         end if

      end if  ! ( layer.gt.1 )

!     --- Remove sharp angles from the new boundary by creating extra
!         elements

      call mshpavmnsmoot6 ( layer, nowbnd, maxbounds, crd, newbnd, &
                            kmeshc, nel, ibndnr, nloop )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'After mshpavmnsmoot6: nel = ',nel
         call prinin ( newbnd(2), newbnd(1), 'newbnd/3' )

      end if

      if ( deleted ) then

!     --- Some elements have been removed, adapt kmeshc

         icount = nelprev
         do ielem = nelprev+1, nel
            if ( kmeshc(1,ielem).ne.0 ) then
               icount = icount+1
               kmeshc(1,icount) = kmeshc(1,ielem)
               kmeshc(2,icount) = kmeshc(2,ielem)
               kmeshc(3,icount) = kmeshc(3,ielem)
            end if
         end do
         nel = icount

      end if
      if ( plotdebug ) then
         jtimes = 1
         call plotnowbnd ( nowbnd, nloop, maxbounds, layer, crd, &
                           'mshpavmnsmooth/3' )
         jtimes = 3
         call plotmeshinter ( nel, kmeshc, crd, layer, 4, nbndpt, &
                              'mshpavmnsmooth/3' )
         jtimes = 0
      end if

!     --- Check control integers

      call errcontrol ( imark(0), imark(ncrd+1), 1008, 'imark' )

!     --- Finally we try to get rid of sharp corners by adding extra elements

1000  call erclos ( 'mshpavmnsmooth' )

      if ( debug ) then

!     --- Debug information

         call prinin1part ( kmeshc, nel, 3, nelprev+1, 'kmeshc/2' )
         write(irefwr,*) 'End mshpavmnsmooth'

      end if

      end
