      subroutine mshpavtrloop ( nowbnd, nloop, nel, crd, &
                                maxbounds, lnnowbnd, rgridinfo, &
                                grid, gridinfo, lengridinfo, gridfilled, &
                                lengridfilled, numgridfilled, maxlen, &
                                ncrd, maxloop, maxnpoint, maxnelem )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 02-09-2010 Remove intarmsh
!        version  2.0  date 29-01-2010 Extra parameter intarmsh
!        version  1.1  date 07-07-2004 new call to plotmeshinter
!        version  1.0  date 22-11-2003
!
!   copyright (c) 2003-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create paver mesh
!     Along all boundaries a layer of elements must be created
!     Once the number of layers exceeds maxloop triangle is called
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     paver
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemesh
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision crd(4,*), maxlen, rgridinfo(*)
      integer maxbounds, &
              nowbnd(maxbounds,*), lnnowbnd, grid(*), &
              lengridfilled, ncrd, maxnpoint, maxnelem, &
              gridinfo(*), lengridinfo, gridfilled(2,*), nel, nloop, &
              numgridfilled, maxloop

!     crd            o    array containing the coordinates of the nodes of
!                         the boundary as well as the thickness and factor
!                         See subroutine mshpav
!     grid           i    Work array of size at most 101 x 101 to store
!                         information about the boundary in relation with
!                         arrays gridinfo and rgridinfo
!                         The actual size is (0:nx, 0:ny) and (ix,iy) refers
!                         to the cell ( xmin+ix*dx, ymin+iy*dy) x
!                         ( xmin+(ix+1)*dx, ymin+(iy+1)*dy)
!                         if grid(ix,iy) = 0, no boundary node is stored in cell
!                         (ix,iy), if>0 information about the boundary cells
!                         is stored in array grdinfo from position
!                         grid(ix,iy)
!     gridfilled     i    Work array of size 2 x lengridfilled, indicating which
!                         cells in array grid have value larger than zero
!                         gridfilled(1,i) contains ix and
!                         gridfilled(2,i) contains iy for a cell with
!                         grid(ix,iy)>0
!     gridinfo       i    In this array the boundary points corresponding to a
!                         specific grid cell in grid are stored
!                         istart = grid(ix,iy)
!                         In gridinfo(istart) the number of boundary points of
!                         cell (ix,iy) are stored (if>0). This is followed
!                         by 2 positions for each boundary point
!                         The pair consists of the loop sequence number in the
!                         boundary where loop refers to a closed loop and
!                         a sequence number in the loop
!     lengridfilled  i    Contains the declared length of array gridfilled
!     lengridinfo    i    Contains the declared length of array gridinfo
!     lnnowbnd       i    Declared length of second index of array nowbnd
!     maxbounds      i    Max number of loops in nowbnd
!     maxlen         i    Maximum amount of contiguous space in IBUFFR after a
!                         starting address
!     maxloop        i    maximum number of layers that may be constructed
!                         before triangle is applied
!     maxnelem       i    Maximum number of elements allowed
!     maxnpoint      i    Maximum number of nodal points allowed
!     ncrd           o    number of nodes in crd
!     nel            o    number of elements in the topology
!     nloop          i    number of boundaries in nowbnd
!     nowbnd         i    array containing the boundary nodes for each closed
!                         part
!                         This array is two-dimensional of size (imxlps,*), &
!                         where implxs defines the maximum number of loops
!                         and * is at most the number of nodes in the boundary+1
!                         Pos. (i,1) contains the number of nodes in the closed
!                            boundary part i
!                         Pos. (i,j+1) contains the j-th node number along this
!                            boundary
!     numgridfilled  i    Contains the number of gridpoints filled in array
!                         gridfilled
!     rgridinfo      i    Array of size 4 containing the following information
!                         about array grid
!                         1:  xmin
!                         2:  ymin
!                         3:  dx
!                         4:  dy
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer layer, i, markboun(0:lnnowbnd*nloop*3+1), lniwork, npoint
      integer, allocatable :: iwork(:)
      double precision bndangle(0:lnnowbnd*nloop+1), &
                       bndcoornew(0:4*lnnowbnd*nloop+1), &
                       bndcoorold(0:2*lnnowbnd*nloop+1), work(2*maxnpoint)
      logical plotdebug, finished, allmarked, zero(nloop)

!     allmarked      If true all nodes on the actual boundary are marked
!                    and hence there are no points left to perform the
!                    paver algorithm
!     bndangle       Array containing the angles corresponding to all nodes
!                    of a boundary loop
!     bndcoornew     Contains the coordinates of the new boundary created
!                    by making one layer during the paver algorithm
!     bndcoorold     Contains the coordinates of the previous (old)
!                    boundary during the paver process
!     finished       Indicates if the loop has finished (true) or not
!                    (false)
!     i              Counting variable
!     iwork          integer work array to be used by several subroutines
!                    Its length is checked during the computations
!     layer          Present layer sequence number
!     lniwork        Declared length of array iwork
!     markboun       Three diemsional array of size lnnowbnd x nloop x 3
!                    with the following contents
!                    markboun(i,j,k) refers to relative node i along the
!                    j-th boundary in nowbnd
!                    k has the following meaning.
!                    if Pos. 1 = 0 then the node is free otherwise (1)
!                    it is clcoe to another node.
!                    In that case position 2 contains the boundary number
!                    of that other node and position 3 the relative
!                    position  of the node along that boundary
!     npoint         Number of nodal points
!     plotdebug      If true plots must be made for debugging
!     work           Work array array to be used by several subroutines
!                    Its length is checked during the computations
!     zero           If true all nodes along the boundary have thickness
!                    zero
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCONTREAL    Check control integer around a real work array
!     ERRCONTROL     Check control integers around integer array
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     MSHPAV1CLOSE   Mark all nodes on the boundary of which the distance to the
!                    opposite boundary is less than 4h, with h the average
!                    distance on both boundaries
!     MSHPAV2FILLBND Fill arrays bndcoorold and bndcoornew with old and new
!                    coordinates of all boundaries
!     MSHPAV2LAYERS  The paving algorithm
!     MSHPAVTRIAN    Fill remaining parts of mesh after application of the paver
!                    algorithm by submesh generator triangle
!     PLOTMESHINTER  Plot actual mesh during the process of creation of a mesh
!     PLOTNOWBND     Plot the actual boundary during mesh generation which is
!                    defined in array nowbnd, used in mshpav
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     layer := 0; finished := false
!     Clear array markboun
!     while ( not finished and layer<maxloop ) do
!        layer := layer+1; finished := true
!        Fill the arrays bndcoorold and bndcoornew for both boundaries
!        if ( layer>1 ) then
!           Mark all nodes on the boundary of which the distance to the opposite
!           boundary is less than 4h, with h the average distance on
!           both boundaries
!        if ( there are still unmarked points on the boundary ) do
!           pave a set of elements along the unmarked parts of the boundary
!           finished := false
!     Create elements in gap between two boundaries
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavtrloop' )
      debug = .false.
      plotdebug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavtrloop'
         write(irefwr,1) 'lnnowbnd, nloop', lnnowbnd, nloop
  1      format ( a, 1x, (10i6) )
      end if
      if ( ierror/=0 ) go to 1000

      lniwork = maxnpoint*25
      allocate ( iwork(lniwork), stat = error )
      if ( error/=0 ) call eralloc ( error, lniwork, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Set control integers

      markboun(0) = 1003
      markboun(lnnowbnd*nloop*3+1) = 1003
      bndangle(0) = 1001d0
      bndangle(lnnowbnd*nloop+1) = 1001d0
      bndcoornew(0) = 1002d0
      bndcoornew(4*lnnowbnd*nloop+1) = 1002d0
      bndcoorold(0) = 1003d0
      bndcoorold(2*lnnowbnd*nloop+1) = 1003d0

!     --- layer := 0; finished := false

      layer = 0
      finished = .false.
      allmarked = .false.

!     --- while ( not finished ) do

      do while ( .not. finished .and. layer<maxloop )

!     --- layer := layer+1; finished := true

         layer = layer+1
         finished = .true.

!        --- Clear array markboun

         markboun(1:lnnowbnd*nloop*3) = 0

!        --- activate debugging

         if ( layer>=10000 ) debug = .false.
         if ( layer>=10000 ) plotdebug = .false.
         if ( debug ) then
            write(irefwr,1) 'layer, nloop, nbndpt, maxbounds', &
                             layer, nloop, nbndpt, maxbounds
            write(irefwr,1) 'ncrd', ncrd
            write(irefwr,1) 'lnnowbnd, nloop', lnnowbnd, nloop
         end if

!        --- Fill the arrays bndcoorold and bndcoornew for all boundaries

         call mshpav2fillbnd ( nowbnd, maxbounds, bndcoorold(1), &
                               bndcoornew(1), crd, nloop, bndangle(1), &
                               lnnowbnd, layer, maxlen, zero )
         if ( ierror/=0 ) go to 1000

         if ( layer>1 ) then

!        --- layer>1
!            Mark all nodes on the boundary of which the distance to the
!            opposite boundary is less than 4h, with h the average distance on
!            both boundaries
!            Array markboun is used

            call mshpav1close ( rgridinfo, nx, ny, grid, nowbnd, &
                                maxbounds, nloop, gridinfo, crd, layer, &
                                lengridinfo, gridfilled, lengridfilled, &
                                numgridfilled, allmarked, markboun(1), &
                                lnnowbnd, bndcoornew(1), maxlen, nel )
            if ( ierror/=0 ) go to 1000

         end if  ! ( layer>1 )
         if ( debug ) then
            write(irefwr,1) 'nel, ncrd/1', nel, ncrd
            write(irefwr,*) 'allmarked ', allmarked
         end if  ! ( debug )

         if ( .not. allmarked ) then

!        --- there are still unmarked points on the boundary
!            pave a set of elements along the unmarked parts of the boundary
!            finished := false

            call mshpav2layers ( crd, ncrd, kmeshclc, nel, nowbnd, nloop, &
                                 nbndpt, maxbounds, layer, lnnowbnd, &
                                 maxlen, markboun(1), bndcoornew(1), &
                                 bndangle(1), bndcoorold(1), zero )
            if ( ierror/=0 ) go to 1000
            if ( debug ) write(irefwr,1) 'nel, ncrd/2', nel, ncrd
            call inichkarlen ( ncrd, maxnpoint, 'crd' )
            call inichkarlen ( nel, maxnelem, 'kmeshclc' )
            if ( ierror/=0 ) go to 1000
            do i = 1, nloop
               if ( .not. zero(i) ) finished = .false.
            end do
            if ( plotdebug )  then

!           --- Debug information to be plotted

               jtimes = 1
               call plotnowbnd ( nowbnd, nloop, maxbounds, layer, crd, &
                                 'mshpavtrloop/1' )
               jtimes = 3
               call plotmeshinter ( nel, kmeshclc, crd, layer, 4, ncrd, &
                                    'mshpavtrloop/1' )
               jtimes = 0

            end if

         end if  ! ( .not. allmarked )

      end do  ! while ( .not. finished )

!     --- Create elements in gap between boundaries

      npoint = 6*ncrd

      call mshpavtrian ( maxbounds, nowbnd, nloop, crd, ncrd, kmeshclc, &
                         nel, npoint, work, iwork, lniwork, maxnpoint, &
                         maxnelem, isurnr )
      call inichkarlen ( npoint, maxnpoint, 'crd' )
      call inichkarlen ( nel, maxnelem, 'kmeshclc' )
      if ( ierror/=0 ) go to 1000

      if ( plotdebug )  then

!     --- Debug information to be plotted

         jtimes = 1
         call plotnowbnd ( nowbnd, nloop, maxbounds, layer, crd, &
                           'mshpavtrloop/2' )
         jtimes = 3
         call plotmeshinter ( nel, kmeshclc, crd, layer, 4, npoint, &
                              'mshpavtrloop/2' )
         jtimes = 0

      end if

!     --- Check control integers

      call errcontrol ( markboun(0), markboun(lnnowbnd*nloop*3+1), 1003, &
                        'markboun' )
      call errcontreal ( bndangle(0), bndangle(lnnowbnd*nloop+1), &
                         1001d0, 'bndangle' )
      call errcontreal ( bndcoornew(0), bndcoornew(4*lnnowbnd*nloop+1), &
                         1002d0, 'bndcoornew' )
      call errcontreal ( bndcoorold(0), bndcoorold(2*lnnowbnd*nloop+1), &
                         1003d0, 'bndcoorold' )

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshpavtrloop'

      end if
      call erclos ( 'mshpavtrloop' )

      end subroutine mshpavtrloop
