      subroutine mshpip ( nsurbt, nelz, npobot, bcord1, bcord2, &
                          bcord3, coor, kmscbt, nelbot, kbndbt, &
                          npoint, nelem, kmeshc, ivolnr, kbndpt, &
                          nbndpt, kmscps, ivol, ishape )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  6.2  date 29-05-2014 Extension of timebefore
!        version  6.1  date 22-05-2014 Extra debug
!        version  6.0  date 24-08-2010 Removal of iwork1
!
!   copyright (c) 1987-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Generate volume elements with mesh generator PIPE
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     volume
!     pipe
!     tetrahedron
!     hexahedron
!     coordinate
!     element
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmscbt(*), kbndbt(*), kmeshc(*), kbndpt(*), &
              kmscps(*), nsurbt, nelz, npobot, nelbot, &
              npoint, nelem, ivolnr, nbndpt, ivol, ishape
      double precision coor(*), bcord1(*), bcord2(*), bcord3(*)

!     bcord1         i    Co-ordinates of nodal points in bottom surface
!     bcord2         i    Co-ordinates of nodal points in top surface
!     bcord3         i    Co-ordinates of nodal points in enveloping pipe
!                         surface
!     coor           o    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     ishape         i    Shape number of elements
!     ivol           i    Type of volume:
!                         4:  pipe
!                         9:  channel
!     ivolnr         i    Volume sequence number
!     kbndbt         i    Array KBNDPT for bottom surface
!                         Contains the local nodal point numbers of the
!                         boundary of the bottom surface
!     kbndpt         o    array containing the nodal point numbers of the
!                         boundary
!     kmeshc         o    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     kmscbt         i    Kmesh part c corresponding to bottom surface
!                         Contains the local nodal point numbers of the bottom
!                         surface
!     kmscps         i    Kmesh part c corresponding to pipe surface
!                         This array is required to check the direction of
!                         diagonals in the pipe surface
!     nbndpt         o    Number of nodal points in boundary, length of array
!                         kbndpt
!     nelbot         i    Number of elements in bottom surface
!     nelem          o    Number of elements
!     nelz           i    Number of line elements in vertical direction
!     npobot         i    Number of points of bottom surface
!     npoint         o    Number of nodal points
!     nsurbt         i    Number of elements in boundary of bottom surface
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical trian
      integer m, n, linear, found, i, ilpst, inpelm, &
              klp(2,20), nlp, inpelmbot, inpelmsur, ishapebot, &
              weddir(7,nelbot)
      logical sameorien, prism
      double precision :: timebefore(2)

!     found          Indicates wether wedges can be split into 3 tetraheders (1)
!                    or if conventional split into 8 tetraheders has to be done
!                    (0)
!     i              Counting variable
!     ilpst          Relative last point of a part of kbndbt
!     inpelm         number of points per element
!     inpelmbot      Number of nodes per element of bottom surface and
!                    top surface of pipe
!     inpelmsur      Number of nodes per element of pipe surface
!     ishapebot      Shape number of elements in bottom and top surface
!                    of pipe
!     klp            In this array the relative node numbers with respect
!                    to array kbndbt of first and last point of a
!                    consecutive part are stored. The number
!                    of parts the boundary consists of is nlp
!                    Hence klp(1,1) = 1, klp(2,1) denotes the end point of
!                    the first part of the boundary
!                    klp(1,2) contains the node number of the first point
!                    of the second part. This must be klp(2,1)+1
!                    klp(2,nlp) contains the length of kbndbt
!     linear         Indicates whether the element is a linear simplex
!                    (true) or not (false)
!     m              number of points in circumference
!     n              number of intermediate circles
!     nlp            Number of disjoint parts the boundary of the bottom
!                    surface consists of
!     prism          If true prismatic elements are used
!     sameorien      If true all elements in the bottom surface have the same
!                    orientation (clockwise or counterclockwise) if false
!                    they have not
!     trian          Indication whether triangles (true) or quadrilaterals
!                    are created (false)
!     weddir         Direction of the diagonals per wedge
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     ERSETTIME      Get the CPU time at start of subroutine
!     MSH300         Fill centroid for 9 point hexahedron
!     MSH350         Fill co-ordinates for pipe
!     MSH351         Check co-ordinates of boundary for pipe
!     MSH352         Fill elements for pipe
!     MSH353         Fill array kbndt for pipe
!     MSH354         Fill the array distri with the directions of the diagonals
!                    on the sides of the wedges resulting from the translation
!                    of a triangle in pipe surface
!     MSH358         Check if all elements in the bottom surface have the same
!                    orientation
!     MSHPIPPARMS    Compute some parameters for a pipe as function of the shape
!                    number of the elements
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
!     PRINRL1        Print 2d real vector
!     PRINTTIME      Print the CPU time at end of subroutine
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
!    2294   sameorien is false
! **********************************************************************
!
!                       PSEUDO CODE
!
!     compute some parameters as function of ishape
!     Check co-ordinates of boundary
!     fill klp with nlp loops of info on first and last entry in kbndpt
!     Check if all elements in the bottom surface have the same orientation
!     For tetraheders debug wether special mesh can
!     be made with 3 instead of 8 tets per wedge
!     Fill co-ordinates in coor
!     Fill elements in kmeshc
!     Fill kbndpt
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen( 'mshpip' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpip'
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000
      if ( itime>0 ) call ersettime ( timebefore )

!     --- compute some parameters as function of ishape

      call mshpipparms ( ishape, inpelmbot, inpelmsur, trian, &
                         prism, inpelm, linear, ishapebot )

!     --- Check co-ordinates of boundary

      m = nsurbt*linear
      n = nelz*linear-1

      if ( debug ) then

!     --- Check = true, print some information

         write(irefwr,1) 'nsurbt, linear, nelz, nelbot, inpelmbot', &
                          nsurbt, linear, nelz, nelbot, inpelmbot
         write(irefwr,1) 'm, n, inpelmsur,nsurbt', &
                          m, n, inpelmsur,nsurbt
         write(irefwr,*) 'trian', trian
         call prinin1(kmscbt,nelbot,inpelmbot,'bottom surface')
         if ( trian ) then
            call prinin1(kmscps,nsurbt*nelz*2,inpelmsur,'pipe surface')
         else
            call prinin1(kmscps,nsurbt*nelz,inpelmsur,'pipe surface')
         end if
         call prinrl1(bcord1,npobot,3,'coor bottom')
         call prinrl1(bcord2,npobot,3,'coor top')
         call prinrl1(bcord3,(n+2)*(m+1),3,'coor pipe')
         call prinin(kbndbt,m+1,'boundary bottom surface')

      end if

      if ( m>2000 ) then

!     --- temp array too small

         call errint ( m, 1 )
         call errint ( 2000, 2 )
         call errchr ( 'temp', 1 )
         call errsub ( 550, 2, 0, 1 )
         go to 1000

      end if

!     --- fill klp with nlp loops of info on first and last entry in kbndpt

      ilpst = kbndbt(1)
      klp(1,1) = 1
      nlp = 1
      do i = 2, m
         if (kbndbt(i+nlp-1)==ilpst) then
            klp(2,nlp) = i+nlp-2
            klp(1,nlp+1) = i+nlp
            nlp = nlp + 1
            ilpst = kbndbt(i+nlp-1)
         end if
      end do
      klp(2,nlp) = m+nlp-1

!     --- Check coordinates of three surfaces

      call msh351 ( bcord1, bcord2, bcord3, m, n, ivolnr, kbndbt, &
                    klp, nlp )

!     --- Check if all elements in the bottom surface have the same orientation

      sameorien = .true.
      if ( linear==1 .and. trian ) then

         call msh358 ( nelbot, bcord1, kmscbt, inpelmbot, sameorien )
         if ( .not. sameorien ) then

!        --- sameorien = false, should not occur

            call errint ( ivolnr, 1 )
            call errwar ( 2294, 1, 0, 0 )

         end if

      end if

!     --- For tetraheders debug wether special mesh can
!         be made with 3 instead of 8 tets per wedge

      found = 0
      if ( linear==1 .and. trian .and. .not. prism .and. &
           (sameorien .or. ivol/=4) ) then

         call msh354 ( kmscbt, kmscps, kbndbt, nelbot, weddir, &
                       found, klp, nlp )

      end if

!     --- Fill co-ordinates in coor

      call msh350 ( nsurbt, nelz, npobot, bcord1, bcord2, bcord3, &
                    coor, linear, trian, kmscbt, nelbot, kbndbt, npoint, &
                    found, klp, nlp, ivol==9, inpelmbot )

!     --- Fill elements in kmeshc

      call msh352 ( kmeshc, inpelm, nelz, npobot, nelbot, &
                    kmscbt, nelem, kbndbt, kmscps, weddir, &
                    found, klp, nlp, ishape, inpelmbot )

      if ( inpelm==9 .or. inpelm==5 ) then

!     --- inpelm = 9, fill centroids

         call msh300 ( kmeshc, coor, nelem, npoint, inpelm )

      end if

!     --- Fill kbndpt

      call msh353 ( kbndpt, linear, trian, kmscbt, kbndbt, npobot, nelz, &
                    nelbot, inpelmbot, nsurbt, nbndpt, kmscps, found, &
                    klp, nlp, inpelmsur )

      if ( itime==1 ) call printtime ( 'pipe')
1000  if ( debug ) then

!     --- Check = true, print some information

         call prinin1( kmeshc, nelem, inpelm, 'kmeshc' )
         call prinin1( kbndpt, nelbot, inpelmbot, 'bottom surface' )
         call prinin1( kbndpt(1+inpelmbot*nelbot), nelbot, inpelmbot, &
                       'top surface' )
         if ( trian ) then
            call prinin1( kbndpt(1+inpelmbot*nelbot*2), nsurbt*nelz*2, &
                          inpelmsur, 'pipe surface' )

         else
            call prinin1( kbndpt(1+2*inpelmbot*nelbot), nsurbt*nelz, &
                          inpelmsur, 'pipe surface' )
         end if
         call prinrl1 ( coor, npoint, 3 ,'coor-pipe' )
         write(irefwr,*) 'End mshpip'

      end if
      call erclos( 'mshpip' )

      end
