      subroutine mshplcompsurf ( rinput )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 15-11-2012 Long integers
!        version  2.0  date 02-09-2010 Remove intarmsh
!        version  1.3  date 11-08-2008 minor change
!
!   copyright (c) 2002-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Plot compound surface
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     compound_surface
!     plot
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleconsts
      use sepmoduleplot
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision rinput(*)

!     rinput         i    Real input array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istart, jstart, isubsf, isur, index, &
              npoloc, leng2, nelloc, iboun2, &
              ikelmc, k, i, jsur, ih1, nodp, nsubsf, iscolr, &
              kmeshcloc(nelemlc*inpelm), iwork(nelemlc*(inpelm+1)+npointlc)
      double precision coorlc1(ndim,npointlc), work(ndim,npointlc), &
                       plotfm, yfact
      character (len=11) charac

!     charac         String variable to store a text
!     coorlc1           array of length ndim x npointlc containing the
!                    co-ordinates of the nodal points with respect to the
!                    global numbering
!     i              Counting variable
!     iboun2         Position in array iwork
!     ih1            Help variable to store a constant temporarily
!     ikelmc         position in kmesh part c
!     index          Sequence number of record to be read
!     iscolr         Help variable to store the value of dcolor temporarily
!     istart         First position used at a part
!     isubsf         Sequence number of subsurface
!     isur           surface number
!     iwork          integer work array of length nelemlc*inpelm+npointlc
!                    It is used to store the local node numbers as
!                    well as the local array kmeshcloc
!     jstart         starting position of curve i in kmeshm
!     jsur           surface number
!     k              Counting variable
!     kmeshcloc         kmesh part c, contains the nodal points
!                    corresponding to the elements
!     leng2          Length
!     nelloc         Local number of elements
!     nodp           Nodal point sequence number
!     npoloc         Number of nodes in subsurface
!     nsubsf         Number of subsurfaces
!     plotfm         Plotting factor. When PLOTFM<0, then the length of
!                    the plot is computed by - (xmax-xmin) * PLOTFM
!                    and the width by - (ymax-ymin) * YFACT * PLOTFM
!                    else the length of the plot is equal to PLOTFM cm
!                    When by some mistake the user has given | PLOTFM | a
!                    value smaller than 1 cm, then the length is chosen
!                    equal to the default length stored in common block
!                    consta
!     work           Work array of length ndim*npointlc
!                    It is used to store the local coordinates
!     yfact          Multiplication for the y-co-ordinates. With this
!                    parameter the user can influence the scale in
!                    y-direction
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     DCOPY          Copy double precision dx to double precision dy
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSH3DQ         plot one surface in 3-D (no hidden lines)
!     PLAFEP         General SEPRAN low level plotting subroutine
!     PLAFP7         Plot text using absolute co-ordinates
!     PLSCAL         Plot scales
!     PRININ         print 1d integer array
!     PRINSURF       Print surfaces
!     READFL         Read integer array from a direct access SEPRAN file
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshplcompsurf' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshplcompsurf'
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      istart = kmeshr(isurnr)
      jstart = kmeshr(nsurfs+1) - nsurfs-1 + istart
      nsubsf = kmeshr(isurnr+1) - kmeshr(isurnr)
      do isubsf = 1, nsubsf
         jsur = kmeshr(istart+isubsf)
         isur = abs ( jsur )
         if ( debug ) then

!        --- Debug statements

            write(irefwr,*) 'Subsurface ', isubsf, ' is ', isur

         end if
         index = kmeshr(jstart+isubsf)
         npoloc = kmeshh(1,isur+1)
         nelloc = kmeshh(2,isur+1)
         leng2 = inpelm*nelloc
         if ( debug ) then
            write(irefwr,1) 'jstart, index, leng2', &
                             jstart, index, leng2
            write(irefwr,1) 'npoloc, nelloc', npoloc, nelloc
         end if  ! ( debug )

!        --- Read local numbering and store in array iwork of length
!            npoloc

         call readfl ( index, 1, npoloc+nelloc, 1, 3, iwork )
         if ( debug ) then
            write(irefwr,1) 'npoloc, nelloc', npoloc, nelloc
            call prinin ( iwork, npoloc, 'iwork_nodes' )
            call prinin ( iwork(npoloc+1), nelloc, 'iwork_elements' )
         end if  ! ( debug )

!        --- read local array coorlc1 and store in array work
!            Length: npoloc * ndim

         call dcopy ( ndim*npoloc, kmhsur(isur)%coorsur, 1, work, 1 )

!        --- Put co-ordinates from local array coorlc1 into global coor

         do i = 1, npoloc
            nodp = iwork(i)
            if ( nodp<0 ) then
               nodp=-nodp
            end if
            do k = 1, ndim
               coorlc1(k,nodp) = work(k,i)
            end do
         end do

!        --- Read local array kmeshcloc and store in kmeshcloc

         iboun2 = nelloc+npoloc+1
         iwork(iboun2:iboun2+leng2-1) = kmhsur(isur)%kmeshcsur
         if ( debug ) call prinin ( iwork(iboun2), leng2, 'iwork_2' )

!        --- Put local array kmeshcloc into global kmeshcloc

         do i = 1, nelloc

            ikelmc = (iwork(npoloc+i)-1 ) * inpelm
            ih1 = ( i-1 ) * inpelm + iboun2 - 1
            if ( jsur>0 ) then

!           --- jsur>0, standard case, positive direction

               do k  = 1, inpelm
                  kmeshcloc(ikelmc+k) = iwork(iwork(ih1+k))
               end do

            else

!           --- jsur<0, special case, reversed direction

               if ( ishape<=5 ) then

!              --- ishape = 3, 4, 5 all points must be reversed

                  kmeshcloc(ikelmc+1) = iwork(iwork(ih1+1))
                  do k  = 2, inpelm
                     kmeshcloc(ikelmc+inpelm+2-k) = iwork(iwork(ih1+k))
                  end do

               else

!              --- ishape>5 last point must be kept

                  kmeshcloc(ikelmc+1) = iwork(iwork(ih1+1))
                  do k  = 2, inpelm-1
                     kmeshcloc(ikelmc+inpelm+1-k) = iwork(iwork(ih1+k))
                  end do
                  k = inpelm
                  kmeshcloc(ikelmc+k) = iwork(iwork(ih1+k))

               end if

            end if

         end do

      end do
      plotfm = rinput(1)
      yfact = rinput(2)
      if ( abs(plotfm)<=0.1d0 ) plotfm = defplo
      if ( abs(yfact)<=0d0 ) yfact = 1d0
      iscolr = dcolor
      npointlc = abs(kmeshh(1,isurnr+1))
      nelemlc = kmeshh(2,isurnr+1)
      if ( debug ) then

!     --- debug, i.e. print surface

         call prinsurf ( isurnr, npointlc, nelemlc, ndim, inpelm, &
                         coorlc1, kmeshcloc, 'surfaces' )

      end if

      if ( (debug .or. iplot/=0) .and. &
           isurnr>numsub .and. ndim==3 ) then

!     --- ndim = 3, plot surface

         call msh3dq ( isurnr, coorlc1, kmeshcloc, npointlc, plotfm, yfact, &
                       inpelm, nelemlc )
         if ( jtimes==2 ) then

!        --- jtimes = 2, plot the text "submesh ..." and close the plot

            call plscal
            charac(1:8) = 'SURFACE '
            write ( charac(9:11), 210 ) isurnr
210         format(i3)
            call plafp7 ( ale*0.3d0, 0.01d0, charac, 0d0, &
                          0.5d0, 11 )
            jtimes = 3
            call plafep ( 2, 0d0, 0d0, 0d0, 0d0, 0 )
            jtimes = 1

         end if

      end if
      dcolor = iscolr

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshplcompsurf'

      end if
      call erclos ( 'mshplcompsurf' )

      end subroutine mshplcompsurf
