      subroutine mshpro ( coor, npoint, plane, xsieta, imax, kbndbt, &
                          klp, nlp)
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 26-08-1999 Check on distance h
!        version  2.0  date 29-04-1999 Extra param's for hollow pipes (BvR)
!        version  1.1  date 22-09-1995 New definition of imax
!        version  1.0  date 12-11-1994
!
!   copyright (c) 1994-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Project co-ordinates stored in coor onto plane defined by array plane
!   Map the result into the array xsieta using local co-ordinates
! **********************************************************************
!
!                       KEYWORDS
!
!     plane
!     projection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint, imax(3), kbndbt(*), klp(2,*), nlp
      double precision coor(3,npoint), plane(4), xsieta(2,npoint)

!     coor    i    array of length ndim x npoint containing the co-ordinates
!                  of the nodal points with respect to the global numbering
!     imax   i/o   In this array the indices with respect to boundary points
!                  in the surfaces are stored
!                  imax(1) stores the node number of the first point (usually 1)
!                  imax(2) stores the node number with respect to the xsi dir.
!                          The xsi direction is defined as P2-P1, where P2 is
!                          given by bcord(kbndbt(imax(2))
!                  imax(3) stores the node number with respect to the first eta
!                          dir.
!                          The eta direction is defined as P3-P1, where P3 is
!                          given by bcord(kbndbt(imax(3))
!                  imax must be filled if imax=0
!     kbndbt  i    Array KBNDPT for bottom surface
!                  Contains the local nodal point numbers of the boundary of
!                  the bottom surface
!     klp     i    In this array the relative node numbers with respect to
!                  array kbndbt of first and last point of a consecutive part
!                  are stored. The number of parts the boundary consists of
!                  is nlp
!                  Hence klp(1,1) = 1, klp(2,1) denotes the end point of the
!                  first part of the boundary
!                  klp(1,2) contains the node number of the first point of
!                  the second part. This must be klp(2,1)+1
!                  klp(2,nlp) contains the length of kbndbt
!     nlp     i    Number of disjoint parts the boundary of the bottom
!                  surface consists of
!     npoint  i    Number of nodal points
!     plane   i    Array of length 4 containing the parameters a, b, c and d
!                  describing the plane: ax=by+cz+d = 0
!                  plane(1) = a; plane(2) = b; plane(3) = c; plane(4) = d;
!     xsieta  o    Array containing the ksi-eta co-ordinates of the mapping
!                  xsieta(1,i) contains the ksi co-ordinate of point i
!                  xsieta(2,i) contains the eta co-ordinate of point i
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision x1(3), x2(3), alam, dist, h, t1(3), t2(3), eps
      integer i, j, k, ilp

!     alam    Factor lambda in method
!     dist    Help variable to compute the distance between two points
!     eps     Local accuracy
!     h       Help variable to store a constant temporarily
!     i       Counting variable
!     ilp     Counting variable
!     j       Counting variable
!     k       Counting variable
!     t1      First tangential vector
!     t2      Second tangential vector
!     x1      array to store the co-ordinates of the first point
!     x2      array to store the co-ordinates of the second point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELOUTR  Compute outer product of two vectors
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    none
! **********************************************************************
!
!                       PSEUDO CODE
!
!    method:
!    Consider the point P = (x1,y1,z1) and the plane n1 x + n2 y + n3 z + d = 0
!    The line through P with direction (n1,n2,n3) can be written as
!    x = x1 + lambda n
!    Hence for the intersection point we have:
!    n1 (x1+lambda n1) + n2 (x2+lambda n2) + n3 (x3+lambda n3) + d = 0
!    Because ||n||=1, this means
!    lambda = -d-(n1 x1 + n2 x2 + n3 x3 )
!    To define the local co-ordinates we define the first point in the
!    projection as origin O
!    The ksi direction is determined by taking the line through O and the
!    projected point P in the plane with the largest distance to O
!    Define t1 as the line OP, and normalize the length to 1
!    This defines the ksi-axis
!    In order to normalize the normal direction we proceed as follows
!    In the first call we have imax(3) = 0
!    In that case we find the boundary point in the projected plane such
!    that the angle between the xsi-axis and this point is maximal
!    imax is the corresponding index in kbndbt
!    n2 = t1 x (x(1)-x(imax))/||x(1)-x(imax)||
!    if ( n, n2 ) < 0, reverse sign of plane
!    The eta axis is defined by the vector t2 = n x OP / || n x OP ||
!    The mapping is defined as:  x-xo = ksi t1 + eta t2
!          |x-xo|                      |ksi|
!    or    |y-yo|   = |  t1  t2  n  |  |eta|
!          |z-zo|                      | 0 |
!    The inverse transformation is given by:
!       |ksi|                      -1   |x-xo|
!       |eta|   =   |  t1  t2  n  |     |y-yo|
!       | 0 |                           |z-zo|
!    Inverse of |t1 t2 n|:  |det( t1 t2 n )| = 1
!      -1   | t22 n3 - t23 n2    t23 n1 - t21 n3    t21 n2 - t22 n1 |
!     T     | n2 t13 - n3 t12    n3 t11 - n1 t13    n1 t12 - n2 t11 |
!           | t12 t23- t13 t22   t13 t21- t11 t23   t11 t22- t12 t21|
!    hence:
!   ksi = (t22 n3 - t23 n2) (x-xo) + (t23 n1 - t21 n3) (y-yo) + &
!         (t21 n2 - t22 n1) (z-zo)
!   eta = (t13 n2 - t12 n3) (x-xo) + (t11 n3 - t13 n1) (y-yo) + &
!         (t12 n1 - t11 n2) (z-zo)
!    pseudo code:
!    lambda = -d-(n1 x1 + n2 x2 + n3 x3 )
!    P1 := first point at boundary
!    P2 := second point at boundary
!    t1 := P2-P1/||P2-P1||
!    if ( imax=0 ) then
!       imax := index of point in boundary with largest angle w.r.t t1
!    t2 := Pimax - P1 / ||Pimax - P1||
!    n2 := t1 x t2 / ||t1xt2||
!    if ( (n,n2) < 0 ) then
!       n := -n
!    t2 := n x OP / || n x OP ||
!    For i := 1 (1) npoint do
!       (x,y,z) = coor(1-3,i)
!       ksi = (t22 n3 - t23 n2) xd+ (t23 n1 - t21 n3) yd+ (t21 n2 - t22 n1) zd
!       eta = (t13 n2 - t12 n3) xd+ (t11 n3 - t13 n1) yd+ (t12 n1 - t11 n2) zd
!       with xd = x-xo
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- lambda = -d-(n1 x1 + n2 x2 + n3 x3 )

      j = kbndbt(imax(1))
      alam = -plane(4)-plane(1)*coor(1,j)-plane(2)*coor(2,j)- &
              plane(3)*coor(3,j)

!     --- P1 = first point at boundary

       do i = 1, 3
         x1(i) = coor(i,j) + plane(i)*alam
      end do
      j = kbndbt(klp(1,1)+1)
      do i = 1, 3
         x2(i) = coor(i,j) + plane(i)*alam
      end do
      eps = sqreps * sqrt((x2(1)-x1(1))**2+(x2(2)-x1(2))**2+ &
                          (x2(3)-x1(3))**2)
      if ( imax(2).eq.0 ) then

!     --- Find point P2 at boundary with largest distance to P1

         dist = 0d0
         do ilp = 1, nlp
            do k = klp(1,ilp), klp(2,ilp)
               j = kbndbt(k)
               do i = 1, 3
                  x2(i) = coor(i,j) + plane(i)*alam
               end do
               h = sqrt((x2(1)-x1(1))**2+(x2(2)-x1(2))**2+ &
                        (x2(3)-x1(3))**2)
               if ( h.gt.dist ) then
                  dist = h
                  imax(2) = k
               end if
            end do
         end do
      end if
      do i = 1, 3
         x2(i) = coor(i,kbndbt(imax(2))) + plane(i)*alam
      end do

!     --- t1 = P2-P1/||P2-P1||

      h = 1d0/sqrt((x2(1)-x1(1))**2+(x2(2)-x1(2))**2+(x2(3)-x1(3))**2)
      do i = 1, 3
         t1(i) = (x2(i)-x1(i))*h
      end do
      if ( imax(3).eq.0 ) then

!     --- Find point at boundary with largest angle with respect to xsi-axis

         dist = 1d2
         do ilp = 1, nlp
            do k = klp(1,ilp), klp(2,ilp)
               j = kbndbt(k)
               do i = 1, 3
                  x2(i) = coor(i,j) + plane(i)*alam
                  t2(i) = x2(i)-x1(i)
               end do
               h = sqrt(t2(1)**2+t2(2)**2+t2(3)**2)
               if ( h.gt.eps ) then
                  h = abs(t1(1)*t2(1)+t1(2)*t2(2)+t1(3)*t2(3))/h
                  if ( h.lt.dist ) then
                     dist = h
                     imax(3) = k
                  end if
               end if
           end do
         end do
      end if
      do i = 1, 3
         t2(i) = coor(i,kbndbt(imax(3)))+plane(i)*alam-x1(i)
      end do
      h = 1d0/sqrt(t2(1)**2+t2(2)**2+t2(3)**2)
      do i = 1, 3
         t2(i) = h*t2(i)
      end do

!     --- n2 = t1 x t2

      call eloutr ( t1, t2, x2 )
      if ( plane(1)*x2(1)+plane(2)*x2(2)+plane(3)*x2(3).lt.0d0 ) then

!     --- Reverse sign of plane

         do i = 1, 4
            plane(i) = -plane(i)
         end do
      end if

!     --- t2 = n x OP / || n x OP ||

      call eloutr ( plane, t1, t2 )
      h = 1d0/sqrt(t2(1)**2+t2(2)**2+t2(3)**2)
      do i = 1, 3
         t2(i) = h*t2(i)
      end do

!     --- Map onto xsi-eta plane

      j = kbndbt(imax(1))
      x1(1) = coor(1,j)
      x1(2) = coor(2,j)
      x1(3) = coor(3,j)
      do i = 1, npoint
         xsieta(1,i)=(t2(2)*plane(3)-t2(3)*plane(2))*(coor(1,i)-x1(1))+ &
                     (t2(3)*plane(1)-t2(1)*plane(3))*(coor(2,i)-x1(2))+ &
                     (t2(1)*plane(2)-t2(2)*plane(1))*(coor(3,i)-x1(3))
         xsieta(2,i)=(t1(3)*plane(2)-t1(2)*plane(3))*(coor(1,i)-x1(1))+ &
                     (t1(1)*plane(3)-t1(3)*plane(1))*(coor(2,i)-x1(2))+ &
                     (t1(2)*plane(1)-t1(1)*plane(2))*(coor(3,i)-x1(3))
      end do
      end
