      subroutine mshr3d ( jnew, bcord, coor, kmeshc, kbndpt, &
                          ichois, nx, ny, nz, inpelm, npoint, nelem, &
                          ivolnr, iinput, nsurfs, nbndpt )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  8.1  date 29-05-2014 Extension of timebefore
!        version  8.0  date 24-08-2010 Removal of work arrays
!        version  7.6  date 15-10-2003 Extra debug statements
!
!   copyright (c) 1984-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute all internal mesh points of a 3-D block domain and fill
!     part c (the elements) of KMESH
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh_generation
!     3d
!     regular
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision bcord(3,*), coor(3,*)
      integer kmeshc(*), nbndpt, kbndpt(nbndpt), ichois, nx, ny, nz, &
              inpelm, npoint, nelem, ivolnr, iinput(*), nsurfs
      logical jnew

!     bcord          i    array in which all the coordinates of the
!                         six boundary surfaces of the block are given
!                          surf. 1-2-3-4   pos 1-3*nx*ny
!                          surf. 1-2-6-5   next 3*nx*nz positions
!                          surf. 2-3-7-6   next 3*ny*nz positions
!                          surf. 4-3-7-8   next 3*nx*nz positions
!                          surf. 1-4-8-5   next 3*ny*nz positions
!                          surf. 5-6-7-8   next 3*nx*ny positions
!     coor           o    array to be filled with the coordinates
!                         of all points
!     ichois         i    Choice parameter to indicate in which way
!                         the internal points have to be determined
!                         1       in horizontal planes
!                         2       in vertical planes
!                         3       in lateral planes
!                         4       no preference , a mean value
!     iinput         i    integer input array for the mesh generator
!                         only the part referred to the surface numbers
!                         corresponding to the brick is used
!     inpelm         i    number of points per element
!     ivolnr         i    Volume sequence number
!     jnew           i    logical variable to indicate whether a
!                         new mesh is created ('true') or an old
!                         one is repositioned ('false') (option not used !!!)
!     kbndpt        i/o   array containing the nodal point numbers of the
!                         boundary
!     kmeshc         o    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     nbndpt         i    Number of nodal points in boundary, length of array
!                         kbndpt
!     nelem          o    Number of elements
!     npoint         o    Number of nodal points
!     nsurfs         i    number of surfaces
!     nx             i    number of points in x direction
!     ny             i    number of points in y direction
!     nz             i    number of points in z direction
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision xc, x(18), yc, y(18), zc, z(18), tau, sig, vuu
      integer i, ihelp, ii, j, k, nh, nhx, nhxy, nmax, &
              no(18), np, nx1, ny1, nz1
      integer, allocatable :: iwork(:), i1(:), i2(:), i3(:), i4(:)
      double precision :: timebefore(2)

!     error          Return value of allocate or deallocate
!     i              loop variable
!     i1             workarray of max( nx, ny, nz ) positions
!     i2             as i1
!     i3             as i1
!     i4             as i1
!     ihelp          helpvariable for number of points in one plane
!     ii             node number to be considered
!     iwork          integer work array of the same length as kbndpt
!     j              loop variable
!     k              loop variable
!     nh             number of nodes
!     nhx            number of nodes along x-axis direction
!     nhxy           number of nodes in x-y plane
!     nmax           maximum of nx, ny and nz
!     no             node number array for nodes in boundaries
!     np             local node number
!     nx1            number of nodes - 1 in x - direction
!     ny1            number of nodes - 1 in y - direction
!     nz1            number of nodes - 1 in z - direction
!     sig            local variable in second direction
!     tau            local variable in first direction
!     vuu            local variable in third direction
!     x              array for first coordinates
!     xc             first coordinate of point to be computed
!     y              array for second coordinates
!     yc             second coordinate of point to be computed
!     z              array for third coordinates
!     zc             third coordinate of point to be considered
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERSETTIME      Get the CPU time at start of subroutine
!     MSH300         Fill centroid for 9 point hexahedron
!     MSH302         Compute coordinates in parallel planes
!     MSH303         Fill array kmesh_part_c
!     MSH308         Check whether BCORD is consistent in itself
!     MSH309         Fill array KBNDPT if inpelm =  4
!     MSH310         Fill array KBNDPT if inpelm =  8
!     MSH311         Fill array KBNDPT if inpelm = 12
!     MSH312         Fill array KBNDPT if inpelm = 27
!     MSH315         Special routine for INPELM = 14 or 15
!     MSH320         Place all boundary points in COOR
!     PRININ1        print 2d integer array
!     PRINRL1        Print 2d real vector
!     PRINTTIME      Print the CPU time at end of subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2910   Not enough space
!    2911   Not possible to deallocate
! **********************************************************************
!
!                       PSEUDO CODE
!
!     determine number of points in block
!     if inpelm = 14 or 15
!        use special routine (MSH315)
!     else
!        check whether bcord is consistent in itself  (MSH308)
!        place all given boundary coordinates in coor (MSH320)
!        determine all internal points, &
!        dependent on the value of ichois
!        ichois = 1 , coordinates are computed in planes
!                     parallel to 1-2-3-4
!        ichois = 2 , parallel to 1-2-5-6
!        ichois = 3 , coordinates are computed in planes parallel
!                     to 1-4-5-8
!        ichois = 4 , no preference , apply the COONS method to the three
!                     possible planes and use a mean value
!                     compute the local variables tau, sigma and vu
!                     compute intermediate values using a combination of
!                     three COONS formulae for three planes
!        check whether elements are of correct shape (MSH318)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshr3d' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshr3d'
         write(irefwr,*) 'inpelm, nx, ny, nz, nelem, npoint ', &
                          inpelm, nx, ny, nz, nelem, npoint
         call prinrl1 ( bcord, 2*(nx*ny+ny*nz+nz*nx), 3, 'bcord' )

      end if

      if ( itime>0 ) call ersettime ( timebefore )

!     --- determine number of points

      npoint=nx*ny*nz
      nmax = max(nx, ny, nz)

      allocate ( iwork(nbndpt), i1(nmax), i2(nmax), i3(nmax), i4(nmax), &
                 stat = error )
      if ( error/=0 ) then

!     --- error#0, so not enough space

         call errint ( 4*nmax+nbndpt, 1 )
         call errint ( error, 2 )
         call errchr ( 'iwork', 1 )
         call errsub ( 2910, 2, 0, 1 )
         go to 1000

      end if  ! ( error/= 0 )

!     --- Clear array coor

      coor( 1:3, 1:npoint ) = 0d0

!     --- If inpelm = 14 or 15 a special routine has to be used

      if ( inpelm==14 .or. inpelm==15 ) then

          npoint = ( 5*nx*ny - 2*nx - 2*ny + 1 ) * (nz-1) / 2 + &
                       nx*ny + (nx-1) * (ny-1) / 2

          call msh315 ( jnew, bcord, nx, ny, nz, nelem, coor, &
                        kmeshc, kbndpt, inpelm, npoint, ivolnr, iinput, &
                        iwork, nsurfs )
          if ( debug ) then

!         --- debug information

             write(irefwr,*) 'In mshr3d: inpelm = ', inpelm
             write(irefwr,*) 'nx, ny, nz, nelem, npoint ', &
                              nx, ny, nz, nelem, npoint
             call prinrl1 ( coor, npoint, 3, 'coor' )
             call prinin1 ( kmeshc, nelem, inpelm,  'kmeshc' )

          end if

          goto 900

      end if

!     --- Check whether bcord is consistent in itself

      call msh308 ( bcord, nx, ny, nz, ivolnr, inpelm, kbndpt )

!     --- Place all given boundary coordinates in coor

      call msh320 ( bcord, coor, nx, ny, nz )
      if ( debug ) then

!     --- debug information

         write(irefwr,*) 'In mshr3d: inpelm = ', inpelm
         write(irefwr,*) 'nx, ny, nz, nelem, npoint ', &
                          nx, ny, nz, nelem, npoint
         call prinrl1 ( coor, npoint, 3, 'coor' )

      end if

      if ( jnew ) then

!     --- Determine all internal points, dependent on the value of ichois

         goto ( 100, 200, 300, 400 ), ichois

!        --- ichois = 1 , coordinates are computed in planes
!                         parallel to 1-2-3-4
!            Number of planes

100      np = nz - 2

         do nh = 1, np

            nhxy = nh * nx * ny

            do j = 1, nx

                i1(j) = j +             nhxy
                i4(j) = j + (ny-1)*nx + nhxy

            end do

            do j = 1, ny

                i2(j) = 1 + (j-1) * nx + nhxy
                i3(j) = j * nx + nhxy

            end do

            call msh302 ( coor, i1, i2, i3, i4, nx, ny )

         end do

         goto 500

!        --- ichois = 2 , parallel to 1-2-5-6
!            number of planes

200      np  =  ny - 2

         do nh = 1, np

            nhx=nh*nx

            do j = 1, nx

                i1(j) = j +                    nhx
                i4(j) = j + (nz-1) * nx * ny + nhx

            end do

            do j = 1, nz

                i2(j) = 1  + (j-1) * nx * ny + nhx
                i3(j) = nx + (j-1) * nx * ny + nhx

            end do

            call msh302 ( coor, i1, i2, i3, i4, nx, nz )

         end do

         goto 500

!        --- ichois = 3 , coordinates are computed in planes parallel
!                         to 1-4-5-8
!            Number of planes

300      np = nx - 2

         do nh = 1, np

            do j = 1, ny

                i1(j) = 1 + (j-1) * nx +                    nh
                i4(j) = 1 + (j-1) * nx + (nz-1) * nx * ny + nh

            end do

            do j = 1, nz

                i2(j) =  1 +               (j-1) * nx * ny + nh
                i3(j) =  1 + (ny-1) * nx + (j-1) * nx * ny + nh

            end do

            call msh302 ( coor, i1, i2, i3, i4, ny, nz )

         end do

         goto 500

!        --- ichois = 4 , no preference , apply the COONS method to the three
!                         possible planes and use a mean value

400      ihelp = nx * ny
         nx1   = nx - 1
         ny1   = ny - 1
         nz1   = nz - 1

         do i =  2 , nx1

!        --- Compute the value of tau

            tau  = ( i-1d0 ) / ( nx - 1d0 )

            no( 7)  =  i
            no( 8)  =  i    +          ihelp - nx
            no(11)  = no( 7) + (nz-1) * ihelp
            no(12)  = no( 8) + (nz-1) * ihelp

            x( 7)  = coor( 1, no( 7) )
            x( 8)  = coor( 1, no( 8) )
            x(11)  = coor( 1, no(11) )
            x(12)  = coor( 1, no(12) )

            y( 7)  = coor( 2, no( 7) )
            y( 8)  = coor( 2, no( 8) )
            y(11)  = coor( 2, no(11) )
            y(12)  = coor( 2, no(12) )

            z( 7)  = coor( 3, no( 7) )
            z( 8)  = coor( 3, no( 8) )
            z(11)  = coor( 3, no(11) )
            z(12)  = coor( 3, no(12) )

            do j =  2 , ny1

!           --- Compute the value of sigma

               sig  =  ( j - 1d0 ) / ( ny - 1d0 )

               no( 9)  =  1  + (j-1) * nx
               no(10)  =        j*nx
               no(13)  = no( 9) + (nz-1) * ihelp
               no(14)  = no(10) + (nz-1)*ihelp

               x( 9)  = coor( 1, no( 9) )
               x(10)  = coor( 1, no(10) )
               x(13)  = coor( 1, no(13) )
               x(14)  = coor( 1, no(14) )

               y( 9)  = coor( 2, no( 9) )
               y(10)  = coor( 2, no(10) )
               y(13)  = coor( 2, no(13) )
               y(14)  = coor( 2, no(14) )

               z( 9)  = coor( 3, no( 9) )
               z(10)  = coor( 3, no(10) )
               z(13)  = coor( 3, no(13) )
               z(14)  = coor( 3, no(14) )

               do k = 2, nz1

!              --- Compute the value of nu

                  vuu  =  ( k - 1d0 ) / ( nz - 1d0 )

                  no( 1)  =  i + (k-1) * ihelp
                  no( 2)  =  i +  k    * ihelp - nx
                  no( 3)  =  1 + (j-1) * nx + (k-1) * ihelp
                  no( 4)  =       j    * nx + (k-1) * ihelp
                  no( 5)  =  i + (j-1) * nx
                  no( 6)  =  i + (j-1) * nx + (nz-1) * ihelp

                  no(15)  =  1 + (k-1) * ihelp
                  no(16)  = nx + (k-1) * ihelp
                  no(17)  =  1 +  k    * ihelp - nx
                  no(18)  =       k    * ihelp

                  x(15)  = coor( 1, no(15) )
                  x(16)  = coor( 1, no(16) )
                  x(17)  = coor( 1, no(17) )
                  x(18)  = coor( 1, no(18) )

                  y(15)  = coor( 2, no(15) )
                  y(16)  = coor( 2, no(16) )
                  y(17)  = coor( 2, no(17) )
                  y(18)  = coor( 2, no(18) )

                  z(15)  = coor( 3, no(15) )
                  z(16)  = coor( 3, no(16) )
                  z(17)  = coor( 3, no(17) )
                  z(18)  = coor( 3, no(18) )

!                 --- nodal point number of internal point to be considered

                  ii = i + (j-1) * nx + (k-1) * ihelp

!                 --- x - coordinate

                  x( 1) =  coor(1, no( 1) )
                  x( 2) =  coor(1, no( 2) )
                  x( 3) =  coor(1, no( 3) )
                  x( 4) =  coor(1, no( 4) )
                  x( 5) =  coor(1, no( 5) )
                  x( 6) =  coor(1, no( 6) )

!                 --- Compute first part of expression

                  xc = ( (1d0-sig) * x(1) +  sig * x(2) + &
                         (1d0-tau) * x(3) +  tau * x(4) + &
                         (1d0-vuu) * x(5) +  vuu * x(6) ) * 2d0

!                 --- Subtract second part of expression

                  xc = xc - &
                     ( (1d0-sig)*(1d0-vuu)*x( 7) + sig*(1d0-vuu)*x( 8) + &
                       (1d0-tau)*(1d0-vuu)*x( 9) + tau*(1d0-vuu)*x(10) + &
                       (1d0-sig)*     vuu *x(11) + sig*     vuu *x(12) + &
                       (1d0-tau)*     vuu *x(13) + tau*     vuu *x(14) + &
                       (1d0-sig)*(1d0-tau)*x(15) + tau*(1d0-sig)*x(16) + &
                       (1d0-tau)*     sig *x(17) + sig*     tau *x(18) )

!                 --- Divide by 3 and place in coor

                  coor( 1, ii ) = xc / 3d0

!                 --- y - coordinate

                  y(1)  =  coor( 2, no( 1) )
                  y(2)  =  coor( 2, no( 2) )
                  y(3)  =  coor( 2, no( 3) )
                  y(4)  =  coor( 2, no( 4) )
                  y(5)  =  coor( 2, no( 5) )
                  y(6)  =  coor( 2, no( 6) )

!                 --- Compute first part of expression

                  yc = ( (1d0-sig) * y(1) +  sig * y(2) + &
                         (1d0-tau) * y(3) +  tau * y(4) + &
                         (1d0-vuu) * y(5) +  vuu * y(6) ) * 2d0

!                 --- Subtract second part of expression

                  yc = yc - &
                     ( (1d0-sig)*(1d0-vuu)*y( 7) + sig*(1d0-vuu)*y( 8) + &
                       (1d0-tau)*(1d0-vuu)*y( 9) + tau*(1d0-vuu)*y(10) + &
                       (1d0-sig)*     vuu *y(11) + sig*     vuu *y(12) + &
                       (1d0-tau)*     vuu *y(13) + tau*     vuu *y(14) + &
                       (1d0-sig)*(1d0-tau)*y(15) + tau*(1d0-sig)*y(16) + &
                       (1d0-tau)*     sig *y(17) + sig*     tau *y(18) )

!                 --- Divide by 3

                  coor( 2, ii ) = yc / 3d0

!                 --- z - coordinate

                  z(1)  =  coor( 3, no( 1) )
                  z(2)  =  coor( 3, no( 2) )
                  z(3)  =  coor( 3, no( 3) )
                  z(4)  =  coor( 3, no( 4) )
                  z(5)  =  coor( 3, no( 5) )
                  z(6)  =  coor( 3, no( 6) )

!                 --- Compute first part of expression

                  zc = ( (1d0-sig) * z(1) +  sig * z(2) + &
                         (1d0-tau) * z(3) +  tau * z(4) + &
                         (1d0-vuu) * z(5) +  vuu * z(6) ) * 2d0

!                 --- Subtract second part of expression

                  zc = zc - &
                     ( (1d0-sig)*(1d0-vuu)*z( 7) + sig*(1d0-vuu)*z( 8) + &
                       (1d0-tau)*(1d0-vuu)*z( 9) + tau*(1d0-vuu)*z(10) + &
                       (1d0-sig)*     vuu *z(11) + sig*     vuu *z(12) + &
                       (1d0-tau)*     vuu *z(13) + tau*     vuu *z(14) + &
                       (1d0-sig)*(1d0-tau)*z(15) + tau*(1d0-sig)*z(16) + &
                       (1d0-tau)*     sig *z(17) + sig*     tau *z(18) )

!                 --- Divide by 3

                  coor( 3, ii ) = zc / 3d0

               end do

            end do

         end do

!        --- Fill array kmesh_part_c

500      call msh303 ( inpelm, kmeshc, nx, ny, nz, nelem )

         if ( debug ) then

!        --- Debug information

             call prinin1 ( kmeshc, nelem, inpelm,  'kmeshc' )

         end if

      end if

!     --- Fill array kbndpt

      if ( inpelm<=5 ) then
         call msh309 ( iinput, nx, ny, nz, kbndpt, iwork, nsurfs )
      else if ( inpelm==8 .or. inpelm==9 ) then
         call msh310 ( iinput, nx, ny, nz, kbndpt, iwork, nsurfs )
      else if ( inpelm==10 ) then
         call msh311 ( iinput, nx, ny, nz, kbndpt, iwork, nsurfs )
      else
         call msh312 ( iinput, nx, ny, nz, kbndpt, iwork, nsurfs )
      end if

900   if ( inpelm==9 .or. inpelm==5 ) then

!     --- inpelm = 9, fill centroids

         call msh300 ( kmeshc, coor, nelem, npoint, inpelm )

      end if

      if ( itime==1 ) call printtime( 'brick')

1000  deallocate ( iwork, i1, i2, i3, i4, stat = error )
      if ( error/= 0 ) then

!     --- error#0, problem with deallocate

         call errint ( error, 1 )
         call errchr ( 'iwork', 1 )
         call errsub ( 2911, 1, 0, 1 )

      end if  ! ( error/= 0 )

      call erclos ( 'mshr3d' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshr3d'

      end if

      end
