      subroutine mshrefcoorfc ( ielem, nodenumbers, fillmap, nmapout )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-06-2014 extra parameters
!        version  1.0  date 26-09-2003
!
!   copyright (c) 2003-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the coordinates for the "face" points in for all faces
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     coordinate
!     node
!     refine
!     face
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemap
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ielem, nodenumbers(*), nmapout
      logical :: fillmap

!     fillmap        o    If true array imap must be filled
!     ielem          i    Element number
!     nmapout        o    Size of last dimension of rmap
!     nodenumbers    i    Array containing the node numbers of the newly created
!                         nodes in the following way
!                         Pos. 1 to nedges node number of first node on edge
!                         Next nfaces positions node number of first node on
!                         face not stored before
!                         Next nelem positions node numbers of first extra node
!                         in element not stored before
!                         if elddimmax = 1 only the third part is used
!                         if elddimmax = 2 the second part is not used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iface, ipstart, numedges, numnodes, first_node

!     first_node     Node number of next node to be created
!     iface          Face sequence number
!     ipstart        Starting address in array
!     numedges       number of edges
!     numnodes       Number of nodal points in array nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHREFCOOR10   Fill the coordinates for the "face" points in case of
!                    extended linear triangles of shape 10
!     MSHREFCOOR4    Fill the coordinates for the "face" points in case of
!                    quadratic triangle of shape 4
!     MSHREFCOOR5    Fill the coordinates for the "face" points in case of
!                    bilinear quadrilaterals of shape 5
!     MSHREFCOOR6    Fill the coordinates for the "face" points in case of
!                    bi-quadratic quadrilaterals of shape 6
!     MSHREFCOOR7    Fill the coordinates for the "face" points in case of
!                    extended quadratic triangle of shape 7
!     MSHREFCOOR9    Fill the coordinates for the "face" points in case of
!                    extended bilinear quadrilaterals of shape 9
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For all faces iface do
!        numedges := number of edges in iface
!        numnodes := number of nodes in iface
!        ielem := ielem+1
!        if ( numnodes=6 ) then
!           compute coordinates of 3 face nodes
!        else if ( numnodes=4 and numedges=4 ) then
!           compute coordinates of 1 face node
!        else if ( numnodes=9 ) then
!           compute coordinates of 8 face nodes
!        else if ( numnodes=7 ) then
!           compute coordinates of 6 face nodes
!        else if ( numnodes=10 ) then
!           error
!        else if ( numnodes=5 ) then
!           compute coordinates of 4 face nodes
!        else if ( numnodes=4 ) then
!           compute coordinates of 3 face nodes
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefcoorfc' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefcoorfc'
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- For all faces iface do

      do iface = 1, nfaces

!     --- numedges := number of edges in iface
!         numnodes := number of nodes in iface
!         ielem := ielem+1

         ipstart = nmeshb(iface)
         numedges = nmeshb(ipstart)
         ipstart = nmeshg(iface)
         numnodes = nmeshg(ipstart)
         ielem = ielem+1
         first_node = nodenumbers(ielem)

         if ( debug ) then
            write(irefwr,1) 'iface, ielem, numnodes, numedges', &
                             iface, ielem, numnodes, numedges
            write(irefwr,1) 'first_node', first_node
            call prinin ( nmeshg(ipstart+1), numnodes, 'nodes' )
         end if  ! ( debug )

         if ( numnodes==6 ) then

!        --- numnodes=6, quadratic triangle, 3 face nodes

            call mshrefcoor4 ( nmeshg(ipstart+1), &
                               first_node, fillmap )

         else if ( numnodes==4 .and. numedges==4 ) then

!        --- numnodes=4, numedges=4, quadrilateral, 1 face node

            call mshrefcoor5 ( nmeshg(ipstart+1), &
                               first_node, fillmap )
            nmapout = max(nmapout,4)

         else if ( numnodes==9 ) then

!        --- numnodes=9, bi-quadratic quadrilateral, 8 face nodes

            call mshrefcoor6 ( nmeshg(ipstart+1), &
                               first_node, fillmap )
            nmapout = max(nmapout,4)

         else if ( numnodes==7 ) then

!        --- numnodes=7, extended quadratic triangle, 3 face nodes

            call mshrefcoor7 ( nmeshg(ipstart+1), &
                               first_node, fillmap )
            nmapout = max(nmapout,3)

         else if ( numnodes==10 ) then

!        --- numnodes=10, not yet implemented

            call errint ( 8, 1 )
            call errsub ( 800, 1, 0, 0 )
            go to 1000

         else if ( numnodes==5 ) then

!        --- numnodes=5, extended quadrilateral, 4 face nodes

            call mshrefcoor9 ( nmeshg(ipstart+1), &
                               first_node, fillmap )
            nmapout = max(nmapout,4)

         else if ( numnodes==4 ) then

!        --- numnodes=4, extended triangle, 3 face nodes

            call mshrefcoor10 ( nmeshg(ipstart+1), &
                                first_node, fillmap )
            nmapout = max(nmapout,3)

         end if  ! ( numnodes==6 )

      end do  ! iface = 1, nfaces

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrefcoorfc'

      end if
      call erclos ( 'mshrefcoorfc' )

      end
