      subroutine mshrefkmeshc ( kmeshcnew, eldimmax, nodenumbers, lenkmeshc, &
                                fillmap )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 18-06-2014 Extra parameters
!        version  1.2  date 10-02-2012 use getnpelmelgrp
!        version  1.1  date 27-11-2011 use function getshapeelgrp
!
!   copyright (c) 2003-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the new array kmeshc in the case that the complete mesh is refined
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     refine
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemap
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshcnew(*), eldimmax, &
              nodenumbers(*), lenkmeshc
      logical fillmap

!     eldimmax       i    Maximum dimension of the elements, i.e.
!                         3 for volume elements
!                         2 for surface elements
!                         1 for line elements
!     fillmap        i    If true array imap must be filled
!     imap           i    Mapping array
!                         Array imap is filled as follows:
!                         imap(i,1) = 0 means node i has not been interpolated
!                         yet
!                         imap(i,1) = n (>0) means node i must be interpolated
!                                     with n points
!                         imap(i,2-n+1) contain the nodes in the old mesh
!                                     required for the interpolation
!     kmeshcnew      o    New kmesh part c
!     lenkmeshc     i/o   length of array kmeshc
!                         at input estimated length
!                         at output actual length
!     nodenumbers    i    Array containing the node numbers of the newly created
!                         nodes in the following way
!                         Pos. 1 to nedges node number of first node on edge
!                         Next nfaces positions node number of first node on
!                         face not stored before
!                         Next nelem positions node numbers of first extra node
!                         in element not stored before
!                         if elddimmax = 1 only the third part is used
!                         if elddimmax = 2 the second part is not used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielgrp, ikelmc, ikelmcnew, &
              inpelm , nelemnew, nelprev, nsub

!     ielgrp         element group number
!     ikelmc         position in kmesh part c
!     ikelmcnew      Starting address of next element in kmeshcnew
!     inpelm         number of points per element
!     nelemnew       Number of elements in new mesh
!     nelprev        Number of elements stored in kmeshc at the start
!                    of the subroutine
!     nsub           Number of elements an element is subdivided in
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHREFKMESHC1  Fill the new array kmeshc for one element group in case
!                    that the complete mesh is refined
!                    Elements of shape 1, i.e. linear line elements
!     MSHREFKMESHC2  Fill the new array kmeshc for one element group in case
!                    that the complete mesh is refined
!                    Elements of shape 2, i.e. quadratic line elements
!     MSHREFKMESHCS  Fill the new array kmeshc for one element group in case
!                    that the complete mesh is refined
!                    surface elements, line elements but no volume elements
!     MSHREFKMESHCV  Fill the new array kmeshc for one element group in case
!                    that the complete mesh is refined
!                    volume elements, surface elements, line elements
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
!     if ( eldimmax = 1 ) then
!        For all element groups do
!           ishape := shape number of element
!           for all elements in group do
!              fill kmeshcnew
!     else if ( eldimmax = 2 ) then
!        fill elements
!     else ( eldimmax = 3 )
!        fill elements
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefkmeshc' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefkmeshc'

      end if
      if ( ierror/=0 ) go to 1000

!     --- initializations

      nelemnew = 0
      ikelmcnew = 1

      if ( eldimmax==1 ) then

!     --- eldimmax = 1, only 1d elements
!         For all element groups do

         ikelmc = 1
         nelprev = 0
         do ielgrp = 1, nelgrp

!        --- ishape := shape number of element

            ishape = kmeshf(ielgrp)
            inelem = Topgrp(ielgrp)%inelem
            inpelm = Topgrp(ielgrp)%inpelm

!           --- for all elements in group do
!               fill kmeshcnew

            if ( ishape==1 ) then

!           --- ishape = 1, linear line elements

               call mshrefkmeshc1 ( kmeshc(ikelmc), &
                                    kmeshcnew(ikelmcnew), inelem, &
                                    nodenumbers, nelprev )
               nsub = 2

            else if ( ishape==2 ) then

!           --- ishape = 2, quadratic line elements

               call mshrefkmeshc2 ( kmeshc(ikelmc), &
                                    kmeshcnew(ikelmcnew), inelem, &
                                    nodenumbers, nelprev )
               nsub = 2

            else if ( ishape==-10001 ) then

!           --- connection elements

               nsub = 0

            else

!           --- ishape #1, 2 or -10001, shape not implemented

               call errint ( ishape, 1 )
               call errsub ( 800, 1, 0, 0 )
               go to 1000

            end if  ! ( ishape==1 )
            if ( ierror/=0 ) go to 1000
            ikelmc = ikelmc+inelem*inpelm
            ikelmcnew = ikelmcnew+inelem*nsub*inpelm
            nelemnew = nelemnew+nsub*inelem
            if ( nelgrp>1 ) kmeshg(ielgrp) = nsub*inelem
            nelprev = nelprev+inelem
            if ( ikelmcnew-1>lenkmeshc ) then

!           --- Array too short

               call errint ( ikelmcnew-1, 1 )
               call errint ( lenkmeshc, 1 )
               call errchr ( 'kmeshc_new', 1 )
               call errsub ( 550, 2, 0, 1 )
               go to 1000

            end if

         end do  ! ielgrp = 1, nelgrp
         lenkmeshc = ikelmcnew-1

      else if ( eldimmax==2 ) then

!     --- There are also surface elements

         call mshrefkmeshcs ( kmeshcnew, &
                              nodenumbers, lenkmeshc, nelemnew )
         if ( ierror/=0 ) go to 1000

      else  ! ( eldimmax==3 )

!     --- There are also volume elements

         call mshrefkmeshcv ( kmeshcnew, nodenumbers, lenkmeshc, nelemnew, &
                              fillmap )
         if ( ierror/=0 ) go to 1000

      end if  ! ( eldimmax==1 )
      nelem = nelemnew

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrefkmeshc'

      end if
      call erclos ( 'mshrefkmeshc' )

      end
