      subroutine mshshiftnodes ( iworkpoint, coornew, iwork, &
                                 istarticross, skipnod,  eps, &
                                 map, smth, fluidnd, inelem1, shifted )
! ======================================================================
!
!        programmer    Raoul van Loon
!        version  4.1  date 22-11-2011 Debug information
!        version  4.0  date 23-06-2004 Extra parameters
!        version  3.0  date 23-04-2003 Extra parameters
!
!   copyright (c) 2002-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Shift node of intersecting curve onto boundary if
!     node is close (depending on eps) to boundary
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     triangle
!     subdivision
!     adaptation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(*), iworkpoint(*), istarticross, &
              skipnod(*), map(*), inelem1, &
              smth, fluidnd(*), shifted(*)
      double precision coornew(ndim,*), eps

!     coornew        i    Array containing the new coordinates of the nodes
!     eps            i    Local accuracy
!     fluidnd        o    Array that contains the nodal numbers of the coupling
!                         elements in case of fluid-structure computations
!     inelem1        o    Number of coupling elements in case of fluid-structure
!                         computations
!     istarticross   i    Starting address of obstacle in array icrossobst
!     iwork          i    integer work array
!     iworkpoint     i    Work array of length npointnew
!                         This array is filled as follows:
!                         iworkpoint(i) = 0 if node i outside obstacle
!                         iworkpoint(i) = 1 if node i inside obstacle
!                         iworkpoint(i) = 2 if node i inside obstacle (near
!                                           boundary)
!                         iworkpoint(i) = 3 if node i on obstacle
!     map           i/o   Mapping array that is filled if ichoice = 2 or must
!                         be filled if ichoice = 3
!                         map is an integer array of length 5 with the following
!                         contents:
!                         1: mmimap  memory management reference to array imap
!                         2: 130 to indicate the type of array
!                         3: mmrmap  memory management reference to array rmap
!                         4: mmcoor1  memory management reference to array coor1
!                         5: mmcoor2  memory management reference to array
!                                     coor2
!     shifted        i    Array of npoint long in which the nodes are marked
!                         which are shifted onto the obstacle
!     skipnod        o    Array of npoint long in which the nodes are marked
!                         which are not allowed to be moved
!                         skipnod=1 a node is positioned on a boundary
!                         skipnod=2 overcome problem that all vertices are
!                         on the obstacle
!                         skipnod=3 don't use the midpoint nodes an centroid
!                        in smoothing
!     smth           i    Parameter for Laplacian smoothing (1), angle
!                         optimization (2), both (3) or none (0)
!                         Is only allowed if shft=1
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer  i, marknod(npoint), bndnod, indelm(nelem)

!     bndnod         Node that lies on boundary of the fluid domain and
!                    on the obstacle boundary
!     i              Counting variable
!     indelm         Array of length nelem in which elements are marked
!                    of which at least one node is shifted
!     marknod        Mark nodes that will be shifted
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHSHIFTNOD1   Check if there are nodes on the boundary and mark them.
!     MSHSHIFTNOD2   Shift node of intersecting curve onto boundary if
!                    node is close (depending on eps) to boundary
!     MSHSHIFTNOD3   Check if all nodes of an element are marked to be shifted
!                    onto the obstacle
!     MSHSHIFTNOD4   Shift node of intersecting curve onto boundary if
!                    node is close (depending on eps) to boundary
!     MSHSHIFTNOD5   Shift midside nodes between vertice position
!     MSHSHIFTNOD6   Shift midside nodes onto the obstacle boundary
!     MSHSMOOTHING   Apply smoothing around the obstacle
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshshiftnodes' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshshiftnodes'
         write(irefwr,1) 'smth', smth
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do i=1,npoint
         marknod(i)=0
         shifted(i)=0
         skipnod(i)=0
         if (iworkpoint(i)==3) shifted(i)=1
      end do

      do i=1, nelem
         indelm(i)=0
      end do

      inelem1=0
      bndnod=0

!     --- skip nodes on boundary when shifting

      call mshshiftnod1 ( skipnod, bndnod, iworkpoint )

      if ( debug ) then
         write(irefwr,1) 'bndnod', bndnod
         call prinin ( skipnod, npoint, 'skipnod' )
      end if  ! ( debug )

!     --- Mark nodes that will be shifted from both sides of the obstacle

      call mshshiftnod2 ( iwork, istarticross, marknod, eps, shifted)

      if ( debug ) then
         call prinin ( marknod, npoint, 'marknod' )
      end if  ! ( debug )

      if (smth>0) then

!     --- check if all nodes of an element will be shifted onto the
!         boundary and provide it

         call mshshiftnod3 ( iwork, istarticross, skipnod, marknod )

      end if

!     --- nodes are shifted from both sides of the obstacle

      call mshshiftnod4 ( iworkpoint, coornew, iwork, &
                          istarticross, skipnod, eps, shifted, &
                          indelm, map, fluidnd, inelem1 )

      if (smth>0) then

         do i=1,npoint
            if (skipnod(i)==3) skipnod(i)=1
         end do

         if (ndim==2) length=6*3
         if (ndim==3) length=24*12

!        --- laplacian smoothing combined with an angle
!            optimizing smoothing algorithm

         call mshsmoothing ( skipnod, coornew, shifted, indelm, &
                             map, smth )

!        --- position the midside nodes and centroid

         call mshshiftnod5 ( coornew, indelm, map )

!        --- shift midside nodes on the curved obstacle

         call mshshiftnod6 ( coornew, kmeshd, nmeshe, npoint, kmeshx, &
                             map, icurvsmsh, curvesmsh, shifted, kmeshe, &
                             bndnod, fluidnd, inelem1)

!        --- compute the number of coupling elements

      inelem1=(inelem1-1)/2

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshshiftnodes'

      end if  ! ( debug )
      call erclos ( 'mshshiftnodes' )

      end
