      subroutine mshuspvolms ( volusp, lenvolusp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 07-11-2012 Correction negative user points
!        version  1.1  date 13-12-2009 Check on user points
!        version  1.0  date 13-11-2002
!
!   copyright (c) 2002-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find all user points connected to the volumes and store in array
!     volusp
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     volume
!     user_point
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer volusp(*), lenvolusp

!     lenvolusp     i/o   Length of array volusp
!                         At input: estimated length
!                         At output: computed length
!     volusp         o    array containing the user points corresponding to
!                         the volumes in the following way:
!                         volusp(1)=nvolms+1
!                         number of user points in volume i:
!                         volusp(i+1)-volusp(i)
!                         user points in volume i are stored in volusp from
!                         position volusp(i)+1 to volusp(i+1)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iwork(nuspnt), istart, ivolnr, ipstart, numsurf, &
              isurnr, numcrv, i, j, icurnr, k, numusr, iuserp

!     i              Counting variable
!     icurnr         Curve sequence number
!     ipstart        Starting address in array
!     istart         First position used at a part
!     isurnr         Surface sequence number
!     iuserp         User point sequence number
!     ivolnr         Volume sequence number
!     iwork          integer work array
!     j              Counting variable
!     k              Counting variable
!     numcrv         number of parts where curves are given for generation
!                    of boundary elements
!     numsurf        Number of surfaces
!     numusr         Number of user points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
!    3036   user point number has wrong value
! **********************************************************************
!
!                       PSEUDO CODE
!
!     volusp(1) = nvolms+1
!     For all volumes ivolnr do
!        Clear array iwork (1:nuspnt)
!        For all surfaces isurnr corresponding to ivolnr do
!           For all curves icurnr corresponding to isurnr do
!              Store user points in iwork
!        Raise volusp(ivolnr+1) and store user points
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshuspvolms' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshuspvolms'

      end if

!     --- volusp(1) = nvolms+1

      istart = nvolms+1
      volusp(1) = istart

!     --- Clear array iwork (1:nuspnt)

      iwork = 0

!     --- For all volumes ivolnr do

      do ivolnr = 1, nvolms

         ipstart = volume_surfs(ivolnr)
         numsurf = volume_surfs(ipstart)
         if ( debug ) write(irefwr,*) 'ivolnr, ipstart, numsurf ', &
            ivolnr, ipstart, numsurf

         if ( numsurf>0 ) then

!        --- Volume contains surfaces
!            For all surfaces isurnr corresponding to ivolnr do

            do i = 1, numsurf
               isurnr = abs(volume_surfs(ipstart+i))

!              --- For all curves icurnr corresponding to isurnr do
!                  Store user points in iwork

               numcrv = kmeshs(isurnr+1)-kmeshs(isurnr)
               if ( debug ) write(irefwr,*) 'i, isurnr, numcrv ', &
                  i, isurnr, numcrv
               do j = 1, numcrv
                  icurnr = abs(kmeshs(kmeshs(isurnr)+j))
                  numusr = curveusrp(icurnr+1)-curveusrp(icurnr)

                  if ( debug ) then

!                 --- Debug information

                     write(irefwr,*) 'j, icurnr, numusr ', &
                                      j, icurnr, numusr
                     call prinin ( curveusrp(curveusrp(icurnr)+1), numusr, &
                                   'user_points' )
                  end if

                  do k = 1, numusr
                     iuserp = abs(curveusrp(curveusrp(icurnr)+k))
                     if ( iuserp<1 .or. iuserp>nuspnt ) then
                        call errint ( iuserp, 1 )
                        call errint ( nuspnt, 2 )
                        call errsub ( 3036, 2, 0, 0 )
                        go to 1000
                     end if  ! ( iuserp<0 )
                     iwork(iuserp) = 1
                  end do
               end do

            end do

!           --- store user points

            do j = 1, nuspnt

               if ( iwork(j)==1 ) then

!              --- New user point found

                  istart = istart+1
                  volusp(istart) = j
                  iwork(j) = 0

               end if

            end do

         end if

!        --- Raise volusp(ivolnr+1)

         volusp(ivolnr+1) = istart
         if ( istart>lenvolusp ) then

!        --- Length of array volusp too large

            call errint ( istart, 1 )
            call errint ( lenvolusp, 2 )
            call errchr ( 'volusp', 1 )
            call errsub ( 550, 2, 0, 1 )
            go to 1000

         end if

      end do
      lenvolusp = istart
      if ( debug ) call prinin ( volusp, istart, 'volusp' )

1000  if ( debug ) write(irefwr,*) 'End mshuspvolms'
      call erclos ( 'mshuspvolms' )

      end
