      subroutine mshvlm ( iinput, iposi, jnew, bcord, rinput )
! ======================================================================
!
!        programmer    Guus Segal
!        version 12.2  date 22-05-2014 Extra debug
!        version 12.1  date 15-11-2012 Long integers
!        version 12.0  date 24-08-2010 Removal of work arrays
!
!   copyright (c) 1984-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     create the volumes and write information to file 3
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     volume
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduledebug
      use sepmodulemesh
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical jnew
      integer iinput(*), iposi
      double precision bcord(*), rinput(*)

!     bcord               local array containing the coordinates of the nodal
!                         points of the boundary of the submesh
!     iinput         i    input array    see subroutine mesh
!                         iinput contains only the part with respect to the
!                         definition of the volumes
!     iposi         i/o   positions in array iinput
!     jnew           i    indication whether a new mesh must be created (true)
!                         or not
!     rinput         i    Real input array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ndefsf, nsubsf, ih, leng1, idummy, &
              leng2, jposi, ichois, npoh, nelh, ivlold, &
              ichnge, iposiinp
      double precision jac
      integer, allocatable :: ibottopol(:), ibotboun(:), ipipetopol(:)
      double precision, allocatable :: fix_pnt(:), coorcurvs(:), &
                                       bcorol(:,:)

!     bcorol         Contains the coordinates of the boundary of the surface
!                    to be copied
!     coorcurvs      Contains the coordinates of the extra curves
!     fix_pnt        Real array of size 4 x nfixpnt containing the
!                    coordinates and coarseness of the fixed points
!                    in the mesh
!     ibotboun       Contains node numbers of boundary of bottom surface
!     ibottopol      Contains topology of bottom surface
!     ichnge         indicator how many elements were changed
!     ichois         Choice parameter
!     idummy         Dummy parameter
!     ih             Help variable to store a constant temporarily
!     ipipetopol     Contains topology of pipe surface
!     iposiinp       Starting address in array iinput with respect to the
!                    extra user points (internal points)
!     ivlold         sequence number of volume to be "copied"
!     jac            jacobian of element
!     jposi          Help variable to store the value of iposi
!     leng1          length of array
!     leng2          length indicator elements
!     ndefsf         Number of curves in surface
!     nelh           help variable to store number of elements
!     npoh           estimate of number of nodes
!     nsubsf         Number of subsurfaces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     CHKSBM         Check volume of submesh
!     DCOPY          Copy double precision dx to double precision dy
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     MESH3D         Subroutine to create volume elements for given "general"
!                    shaped surface
!     MSHCHACR       Change coordinates in array coor by calls to FUNCCOOR
!     MSHFILLEXTCRVS Fill coordinates of extra curves in bcord
!     MSHORI3D       Make orientation of element such that Jacobian>0
!     MSHPIPBOUN     Creates and checkes boundary for pipe volume elements
!     MSHPIPBOUNI    Reorders array KBNDPT in case of negative subsurfaces
!     MSHPIPE        Subroutine to create pipe volume elements
!     MSHR3D         Subroutine to create volume elements for "brick-shaped"
!                    volume
!     MSHRFL         Submesh generator REFLECT
!     MSHROT         Submesh generator ROTATE
!     MSHTRA         Fill array coor by translation with respect to preceding
!                    array
!     MSHVLMCHKSRF   Check whether elements of surface do not intersect each
!                    other
!     MSHVLMTETRACP  Copy coordinates of user points from rinput into array
!                    fix_pnt
!     MSHVOLBCORD    Fill coordinates of boundary of volume into array bcord
!     MSHVOLBOUN     Creates boundary for volumes
!     MSHVOLBOUNI    Reorders array KBNDPT in case of negative subsurfaces
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     298   code for generation of volumes incorrect
!     360    Number of nodes larger than estimated
!     361    Number of elements larger than estimated
!    2424    Not all volume elements have the same sign
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- fill local array bcord

      call eropen ( 'mshvlm' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshvlm'
         write(irefwr,*) 'jnew ', jnew
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      iposiinp   = max(1,ncoarstart) ! address in iinput
      ncurvstart = max(1,ncurvstart) ! Starting address of extra curves
      nsurfstart = max(1,nsurfstart) ! Starting address of extra surfs

!     --- surfaces in iinput

      ivol   = iinput(iposi)

      npoh   = npointlc  ! estimated value of npointlc
      nelh   = nelemlc   ! estimated value of nelemlc

      if ( ivol<0 .or. ivol>11 ) then
         call errint ( ivol, 1 )
         call errsub ( 298, 1, 0, 0 )
         go to 1000
      end if

      if ( ivol==0 ) then
         iposi = iposi+1
         go to 1000
      end if

      ndefsf = iinput(iposi+2)
      nsubsf = 0

      if ( debug ) then

!     --- Debug statements

         write(irefwr,1) 'Volume number', ivolnr
         write(irefwr,1) 'Type of volume generator', ivol
         write(irefwr,1) 'Number of internal points', nextrapnts
         write(irefwr,1) 'starting address in iinput', iposiinp
         write(irefwr,1) 'nsurfstart', nsurfstart
         write(irefwr,1) 'ncurvstart', ncurvstart
         write(irefwr,1) 'inplmt', inplmt

      end if

!     --- Fill coordinates of boundary of volume into array bcord

      call mshvolbcord ( bcord, iinput, iposi )

      if ( ivol==4 .or. ivol==9 ) then

!     --- ivol = 4;  Read kmeshc bottom and boundary of bottom surface

         isurnr = mod(iinput(iposi+3)-1,nsurfs)+1  ! top surface
         length = kmeshh(2,isurnr+1)*kmeshh(6,isurnr+1)
         leng1 = kmeshh(4,isurnr+1)+1
         isurnr = mod(iinput(iposi+5)-1,nsurfs)+1  ! pipe surface
         leng2 = kmeshh(2,isurnr+1)*kmeshh(6,isurnr+1)
         allocate ( ibottopol(length), ibotboun(leng1), &
                    ipipetopol(leng2), stat = error )
         if ( error/=0 ) then

!        --- error#0, so not enough space

            call eralloc ( error, length+leng1+leng2, 'ibottopol' )
            go to 1000

         end if  ! ( error/= 0 )

         call mshpipboun ( iinput, iposi, nsubsf, ipipetopol, &
                           ibottopol, ibotboun )
         if ( ierror/=0 ) go to 1000

      else if ( ivol==1 .or. ivol==5 .or. ivol>=6 .and. &
                ivol<=8 .or. ivol>=10 ) then

!     --- Brick, General3d, Translate, Rotate or Reflect
!         (Read kmeshc of boundary)

         call mshvolboun ( iinput, iposi, nsubsf, &
                           ndefsf, coor3d, ivlold, iinput(nsurfstart) )
         if ( ierror/=0 ) go to 1000

         if ( ivol>=6 .and. ivol<=8 ) then

!        --- Translate, Rotate or Reflect
!            Copy first part of coor into bcorol

            allocate ( bcorol(3,1:nbcrd), stat = error )
            if ( error/=0 ) then

!           --- error#0, so not enough space

               call eralloc ( error, 3*nbcrd, 'bcorol' )
               go to 1000

            end if  ! ( error/= 0 )

            bcorol(1:3,1:nbcrd) = coor3d(1:3,1:nbcrd)

         end if  ! ( ivol>=6 .and. ivol<=8 ) then
         if ( debug ) &
            call prinin1 ( kbndptlc, nbndpt/inplmt, inplmt, 'kbndptlc-in' )

      end if

      if ( debug ) write(irefwr,1) 'ivol, nsubsf', ivol, nsubsf

      jposi = iposi
      iposi = iposi+3+ndefsf

      if ( .not. jnew .or. ivol>=6 .and. ivol<=8 ) then

!     --- old mesh, read from file3, or copy old mesh

         if ( ivol>5 ) then
            ih = ivlold+nsurfs
         else
            ih = ivolnr+nsurfs
         end if
         npointlc = kmeshh(1,ih+1)
         nelemlc = kmeshh(2,ih+1)
         leng2 = inpelm*nelemlc
         call dcopy ( npointlc*ndim, kmhsur(ih)%coorsur, 1, coor3d, 1 )
         kmeshc(1:leng2) = kmhsur(ih)%kmeshcsur

      end if

      if ( debug ) then

!     --- debug = true, print coordinates of boundary

         write(irefwr,*) 'Volume number ', ivolnr
         call prinrl1 ( bcord, nbcrd, 3, 'bcord' )

      end if

!     --- volume generation

      if ( ivol==1 ) then

!     --- ivol = 1   mshr3d

         iposi = iposi+3
         npointlc = nx*ny*nz
         ichois = 4

         call mshr3d ( jnew, bcord, coor3d, kmeshc, &
                       kbndptlc, ichois, nx, ny, nz, inpelm, &
                       npointlc, nelemlc, ivolnr, &
                       iinput(iposi-3-ndefsf), nsurfs, nbndpt )

      else if ( ivol==4 .or. ivol==9 ) then

!     --- ivol = 4  pipe surface

         call mshpipe ( nc, nz, nx, bcord, ipipetopol, ny, &
                        kbndptlc, ibottopol, ibotboun )

         deallocate ( ibottopol, ibotboun, ipipetopol, stat = error )
         if ( error/= 0 ) then

!        --- error#0, problem with deallocate

            call erdealloc ( error, 'ibottopol' )
            go to 1000

         end if  ! ( error/= 0 )

      else if ( ivol==5 .or. ivol>=10 ) then

!     --- ivol = 5  General

         npointlc = nbcrd
         ncurvstart = max(1,ncurvstart)   ! Starting address of extra
         if ( debug ) then
            write(irefwr,1) 'nextrapnts, nextracrvs, nextrasrfs', &
                             nextrapnts, nextracrvs, nextrasrfs
            write(irefwr,1) 'ncurvstart, nsubsf', ncurvstart, nsubsf
         end if  ! ( debug )

         if ( ishape/=13 .and. ishape/=14 .and. ishape/=17 ) then

!        --- Check whether elements of surface do not intersect each other

            call mshvlmchksrf ( iinput(jposi+3), coor3d, kbndptlc )

         end if

         lenextracrvs = max(1,lenextracrvs)

         allocate ( fix_pnt(nextrapnts*4), coorcurvs(3*lenextracrvs), &
                    stat = error )
         if ( error/=0 ) then

!        --- error#0, so not enough space

            call eralloc ( error, nextrapnts*4+3*lenextracrvs, &
                           'fix_pnt' )
            go to 1000

         end if  ! ( error/= 0 )

         if ( nextrapnts>0 ) then

!        --- nextrapnts>0, so there are extra points, fill coordinates in
!            fix_pnt

            if ( debug ) write(irefwr,1) 'irnp1', irnp1
            call mshvlmtetracp ( rinput(irnp1), fix_pnt, &
                                 iinput(iposiinp), &
                                 rinput(irnp1+ndim*nuspnt) )

         end if  ! ( nextrapnts>0 )

         if ( nextracrvs>0 ) then

!        --- nextracrvs>0, so there are extra curves,
!            fill coordinates in fix_pnt from position nextrapnts*4+1

            ndim = 3
            ibcrds = 3
            call mshfillextcrvs ( icurvsmsh, curvesmsh, coorcurvs, &
                                  iinput, .true., numnodextcurvs )

         end if  ! ( nextracrvs>0 ) then

         call mesh3d ( ny, jnew, nmes, nx, fix_pnt, &
                       iinput(ncurvstart), coorcurvs, iinput(nsurfstart) )

         deallocate ( fix_pnt, coorcurvs, stat = error )
         if ( error/= 0 ) then

!        --- error#0, problem with deallocate

            call erdealloc ( error, 'fix_pnt' )

         end if  ! ( error/= 0 )

!        --- raise iposi in case of internal points, curves or surfaces

         if ( ivol==10 ) &
            iposi = iposi+3+nextrapnts+nextracrvs+nextrasrfs

      else if ( ivol==6 ) then

!     --- Translate

         call mshtra ( bcorol, bcord, nbcrd, ndim, coor3d, npointlc, &
                       ivolnr )

      else if ( ivol==7 ) then

!     --- Rotate

         call mshrot ( bcorol, bcord, nbcrd, ndim, coor3d, npointlc, &
                       ivolnr )

      else if ( ivol==8 ) then

!     --- Reflect

         call mshrfl ( bcorol, bcord, nbcrd, ndim, coor3d, npointlc, &
                       ivolnr )

      end if
      if ( ierror/=0 ) go to 1000

      if ( ivol>=6 .and. ivol<=8 ) then

!     --- Translate, Rotate or Reflect

         deallocate ( bcorol, stat = error )
         if ( error/= 0 ) then

!        --- error#0, problem with deallocate

            call erdealloc ( error, 'bcorol' )

         end if  ! ( error/= 0 )

      end if  ! ( ivol>=6 .and. ivol<=8 )

!     --- Check new values of npointlc and nelemlc

      if ( npoh<npointlc ) then
         call errint ( npoh, 1 )
         call errint ( npointlc, 2 )
         call errsub ( 360, 2, 0, 0)
      end if
      if ( nelh<nelemlc  ) then
         call errint ( nelh, 1 )
         call errint ( nelemlc, 2 )
         call errsub ( 361, 2, 0, 0)
      end if

!     --- check if all elements have positive volume
!         if not rotate

      call mshori3d ( kmeshc, inpelm, nelemlc, coor3d, ishape, ichnge, jac, &
                      ivol )
      if ( ichnge>0 .and. ichnge<nelemlc ) then

!     --- Number of rotations incorrect

         call errint ( ivolnr, 1 )
         call errint ( ichnge, 2 )
         call errint ( nelemlc, 3 )
         call errwar ( 2424, 3, 0, 0 )

      end if
      if ( ierror/=0 ) go to 1000

      if ( kelmr>0 ) then

!     --- kelmr>0, there are subsurfaces
!         Reorder kbndptlc for subsurfaces with negative sign

         if ( ivol==4 .or. ivol==9 ) then

!        --- ivol = 4;  pipe

            call mshpipbouni ( iinput, jposi, kbndptlc, kmeshs(nsurfs+2) )

         else if ( ivol==1 .or. ivol==5 .or. ivol==6 .or. &
                   ivol==7 .or. ivol==8 .or. ivol==10 .or. ivol==11 ) &
            then

!        --- Brick, General3d, Translate, Rotate or Reflect

            call mshvolbouni ( iinput, jposi, kbndptlc, ndefsf, &
                               kmeshs(nsurfs+2) )

         end if

      end if
      if ( ierror/=0 ) go to 1000

      npomax = max(npomax,npointlc)
      if ( iinp13/=0 ) then

!     --- iinp13 # 0   change coordinates of surface

         isurnr = ivolnr
         call mshchacr ( iinput(iinp13), coor3d, kmeshc, idummy, 4 )

      end if
      if ( icheck>0 ) then
         call chksbm ( kmeshc, nelemlc, ishape, inpelm, ivolnr, coor3d, &
                       ndim, icheck, npointlc )
      end if
      nbndpt = nbndpt

1000  if ( debug ) then

!     --- Debug statements

         write(irefwr,*) 'Volume number ', ivolnr
         write(irefwr,*) 'npointlc, nelemlc ', npointlc, nelemlc
         call prinrl1 ( coor3d, npointlc, 3, 'coor3d' )
         call prinin1 ( kmeshc, nelemlc, inpelm, 'kmeshc')
         if ( ivol/=4 .and. ivol/=9 ) &
            call prinin1 ( kbndptlc, nbndpt/inplmt, inplmt, 'kbndptlc-out' )
         if ( nextrapnts>0 ) call prinin ( kbndptlc(nbndpt+1), &
            nextrapnts, 'node numbers of extra points' )
         if ( nextracrvs>0 ) &
            call prinin ( kbndptlc(nbndpt+1+nextrapnts), &
                          lenextracrvs, 'kbndptlc extra curves' )
         write(irefwr,*) 'End mshvlm'

      end if
      call erclos ( 'mshvlm' )

      end subroutine mshvlm
