      subroutine sepob3dpnt ( nx, ny, nz, inpelm, leniobgrid, &
                              x, y, z, inside, onobstacle, xminorig, &
                              yminorig, zminorig, dx, dy, dz, eps )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if point with coordinates x,y,z is inside or outside the obstacle
!     Points on the boundary are considered as inside the obstacle
!     3d version
!     Old subroutine mshob3dpnt
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     node
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nx, ny, nz, leniobgrid,  inpelm
      double precision x, y, z, &
                       xminorig, yminorig, zminorig, dx, dy, dz, eps
      logical inside, onobstacle

!     dx             i    step in x-direction corresponding to xminorig
!     dy             i    step in y-direction corresponding to yminorig
!     dz             i    step in z-direction corresponding to zminorig
!     eps            i    Local accuracy
!     inpelm         i    number of points per element
!     inside         o    if true the node is inside the obstacle
!     leniobgrid     i    Declared length of last dimension of iobgrid3d
!     nx             i    number of points in x direction
!     ny             i    number of points in y direction
!     nz             i    number of points in z direction
!     onobstacle     o    If true the point is positioned on the obstacle
!     x              i    x co-ordinate of point
!     xminorig       i    Minimum z-value to compute ipcell, jpcell, kpcell
!     y              i    y co-ordinate of point
!     yminorig       i    Minimum y-value to compute ipcell, jpcell
!     z              i    z co-ordinate of point
!     zminorig       i    Minimum z-value to compute ipcell, jpcell, kpcell
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision alpha1, beta1, gamma1, xi(leniobgrid)
      integer ipcell, jpcell, kpcell, ixcross(2), iycross(2), &
              izcross(2), i, k, nxi, nintersect
      logical found

!     alpha1         multiplication factor
!     beta1          multiplication factor
!     found          If true already a possibility has been found
!     gamma1         multiplication factor
!     i              Counting variable
!     ipcell         i-index of cell(i,j,k)
!     ixcross        Contains the number of cross sections of the line
!                    y=xp,z=zp with the obstacle within cell(i,j.k)
!                    ixcross(1) contains the number of cross sections
!                    below xp
!                    ixcross(2) contains the number of cross sections
!                    above xp
!     iycross        Contains the number of cross sections of the line
!                    x=xp,z=zp with the obstacle within cell(i,j,k)
!                    iycross(1) contains the number of cross sections
!                    at the left-hand side of yp
!                    iycross(2) contains the number of cross sections
!                    at the right-hand side of yp
!     izcross        Contains the number of cross sections of the line
!                    x=xp,y=yp with the obstacle within cell(i,j,k)
!                    izcross(1) contains the number of cross sections
!                    at the left-hand side of zp
!                    izcross(2) contains the number of cross sections
!                    at the right-hand side of zp
!     jpcell         j-index of cell(i,j,k)
!     k              Counting variable
!     kpcell         k-index of cell(i,j,k)
!     nintersect     Number of intersections found
!     nxi            Number of points that are intersected in x-direction
!     xi             Array containing the x-coordinates of the intersection
!                    points in the cell
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHOB3DCROSS   Compute cross section of obstacle with lines x=xp,y=yp and
!                    x=xp, z=zp and y=yp, z=zp in one cell
!     MSHOB3DCROSS1  Compute cross section of obstacle with line y=yp, z=zp
!                    in one cell
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    inside := false
!    onobstacle  := false
!    icell := (i,j,k) index of obstacle cell in which (x,y,z) is lying
!    if ( iobgrid3d(i,j,k,1) = -1 ) then
!       inside = true
!    else if ( iobgrid3d(i,j,k,1)>0 ) then
!       compute number of cross sections of line y=yp, z=zp left and right
!       of xp in rectangle as well as number of cross sections of
!       line x=xp,z=zp below and above yp and
!       line x=xp,y=yp below and above zp
!       found := false
!       if ( no cross sections left of x = xp ) then
!          if ( cell atinside = true left-hand side is in obstacle ) then
!             inside = true
!             found := true
!          else if ( cell at left-hand side is outside obstacle ) then
!             found := true
!       else if ( no cross sections right of x = xp ) then
!          if ( cell at right-hand side is in obstacle ) then
!             inside = true
!             found := true
!          else if ( cell at right-hand side is outside obstacle ) then
!             found := true
!       if ( no cross sections below y = yp ) then
!          if ( cell below is in obstacle ) then
!             inside = true
!             found := true
!          else if ( cell below is outside obstacle ) then
!             found := true
!       else if ( no cross sections above y = yp ) then
!          if ( cell above is in obstacle ) then
!             inside = true
!             found := true
!          else if ( cell above is outside obstacle ) then
!             found := true
!       if ( no cross sections below z = zp ) then
!          if ( cell below is in obstacle ) then
!             inside = true
!             found := true
!          else if ( cell below is outside obstacle ) then
!             found := true
!       else if ( no cross sections above z = zp ) then
!          if ( cell above is in obstacle ) then
!             inside = true
!             found := true
!          else if ( cell above is outside obstacle ) then
!             found := true
!       else
!          use line y=yp, z=zp to detect the number of cross sections at
!          the left-hand side
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepob3dpnt' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepob3dpnt'

      end if
      if ( ierror/=0 ) go to 1000

      inside = .false.
      onobstacle = .false.

!     --- (x,y,z) inside (xmin,xmax;ymin,ymax;zmin,zmax)
!         icell := (i,j,k) index of obstacle cell in which (x,y,z) is lying

      alpha1 = (x-xminorig)/dx
      beta1  = (y-yminorig)/dy
      gamma1  = (z-zminorig)/dz
      ipcell = int(alpha1)
      jpcell = int(beta1)
      kpcell = int(gamma1)
      ipcell = min(max(ipcell,0),nx-1)
      jpcell = min(max(jpcell,0),ny-1)
      kpcell = min(max(kpcell,0),nz-1)

      if ( debug ) then

!     --- debug is true, print info

         write(irefwr,*) 'x, y, z ', x, y, z
         write(irefwr,*) 'alpha1, beta1, gamma1 ', &
                          alpha1, beta1, gamma1
         write(irefwr,*) 'ipcell, jpcell, kpcell ', &
                          ipcell, jpcell, kpcell
         write(irefwr,*) 'iobgrid3d info for cell ', &
           (iobgrid3d(ipcell,jpcell, kpcell,k),k=1,leniobgrid)

      end if

      if ( iobgrid3d(ipcell,jpcell,kpcell,1)==-1 ) then

!     --- iobgrid3d(i,j,k,1) = iobst

         inside = .true.

      else if ( iobgrid3d(ipcell,jpcell,kpcell,1)>0 ) then

!     --- iobgrid3d(i,j,k,1)>0
!         compute number of cross sections of line x = xp left and right
!         of xp in rectangle as well as number of cross sections of
!         line y = yp below and above yp

         call mshob3dcross ( x, y, z, ipcell, jpcell, kpcell, &
                             onobstacle, iobgrid3d, nx, ny, nz, &
                             ixcross, iycross, izcross, eps, &
                             coorobst, kmeshcobst, inpelm, &
                             leniobgrid, xi, nxi )
         if ( debug ) then

!        --- debug is true, print info

            write(irefwr,*) 'onobstacle ', onobstacle
            write(irefwr,100)  'ixcross, iycross, izcross ', &
                                ixcross, iycross, izcross
100         format ( a, 10i6 )

         end if  ! ( debug )

         if ( onobstacle ) then

!        --- Point on obstacle

            inside = .true.
            go to 1000

         end if

         found = .false.

         if ( ixcross(1)==0 ) then

!        --- no cross sections below x = xp

            if ( ipcell<=0 ) then

!           --- Below there is no obstacle

               found = .true.

            else if ( iobgrid3d(ipcell-1,jpcell,kpcell,1)==-1 ) &
               then

!           --- cell below is in obstacle
!               found := true

               inside = .true.
               found = .true.

            else if ( iobgrid3d(ipcell-1,jpcell,kpcell,1)==0 ) &
               then

!           --- cell below is outside obstacle
!               found := true

               found = .true.

            end if

         else if ( ixcross(1)==1 ) then

            if ( ipcell==0 ) then

!           --- Below there is no obstacle

               inside = .true.
               found = .true.

            else if ( iobgrid3d(ipcell-1,jpcell,kpcell,1)==-1 ) &
               then

!           --- cell below is inside obstacle
!               found := true

               found = .true.

            else if ( iobgrid3d(ipcell-1,jpcell,kpcell,1)==0 ) &
               then

!           --- cell below is outside obstacle
!               found := true

               inside = .true.
               found = .true.

            end if

         end if

         if ( .not. found .and. ixcross(2)==0 ) then

!        --- no cross sections above x = xp

            if ( ipcell>=nx-1 ) then

!           --- Above there is no obstacle

               found = .true.

            else if ( iobgrid3d(ipcell+1,jpcell,kpcell,1)==-1 ) &
               then

!           --- cell above is in obstacle
!               found := true

               inside = .true.
               found = .true.

            else if ( iobgrid3d(ipcell+1,jpcell,kpcell,1)==0 ) &
               then

!           --- cell above is outside obstacle
!               found := true

               found = .true.

            end if

         else if ( .not. found .and. ixcross(2)==1 ) then

!        --- One cross section above

            if ( ipcell>=nx-1 ) then

!           --- Above there is no obstacle

               inside = .true.
               found = .true.

            else if ( iobgrid3d(ipcell+1,jpcell,kpcell,1)==-1 ) &
               then

!           --- cell above is in obstacle
!               found := true

               found = .true.

            else if ( iobgrid3d(ipcell+1,jpcell,kpcell,1)==0 ) &
               then

!           --- cell above is outside obstacle
!               found := true

               inside = .true.
               found = .true.

            end if

         end if

         if ( .not. found .and. iycross(1)==0 ) then

!        --- no cross sections below y = yp

            if ( jpcell<=0 ) then

!           --- Below there is no obstacle

               found = .true.

            else if ( iobgrid3d(ipcell,jpcell-1,kpcell,1)==-1 ) &
               then

!           --- cell below is in obstacle
!               ipointobst(inode) := iobst
!               found := true

               inside = .true.
               found = .true.

            else if ( iobgrid3d(ipcell,jpcell-1,kpcell,1)==0 ) &
               then

!           --- cell below is outside obstacle
!               ipointobst(inode) := 0
!               found := true

               found = .true.

            end if

         else if ( .not. found .and. iycross(1)==1 ) then

            if ( jpcell==0 ) then

!           --- Below there is no obstacle

               inside = .true.
               found = .true.

            else if ( iobgrid3d(ipcell,jpcell-1,kpcell,1)==-1 ) &
               then

!           --- cell below is inside obstacle
!               found := true

               found = .true.

            else if ( iobgrid3d(ipcell,jpcell-1,kpcell,1)==0 ) &
               then

!           --- cell below is outside obstacle
!               found := true

               inside = .true.
               found = .true.

            end if

         end if

         if ( .not. found .and. iycross(2)==0 ) then

!        --- no cross sections above y = yp

            if ( jpcell>=ny-1 ) then

!           --- Above there is no obstacle

               found = .true.

            else if ( iobgrid3d(ipcell,jpcell+1,kpcell,1)==-1 ) &
               then

!           --- cell above is in obstacle
!               found := true

               inside = .true.
               found = .true.

            else if ( iobgrid3d(ipcell,jpcell+1,kpcell,1)==0 ) &
               then

!           --- cell above is outside obstacle
!               ipointobst(inode) := 0
!               found := true

               found = .true.

            end if

         else if ( .not. found .and. iycross(2)==1 ) then

!        --- One cross section above

            if ( jpcell>=ny-1 ) then

!           --- Above there is no obstacle

               inside = .true.
               found = .true.

            else if ( iobgrid3d(ipcell,jpcell+1,kpcell,1)==-1 ) &
               then

!           --- cell above is in obstacle
!               found := true

               found = .true.

            else if ( iobgrid3d(ipcell,jpcell+1,kpcell,1)==0 ) &
               then

!           --- cell above is outside obstacle
!               ipointobst(inode) := 1
!               found := true

               inside = .true.
               found = .true.

            end if

         end if

         if ( .not. found .and. izcross(1)==0 ) then

!        --- no cross sections below z = zp

            if ( kpcell<=0 ) then

!           --- Below there is no obstacle

               found = .true.

            else if ( iobgrid3d(ipcell,jpcell,kpcell-1,1)==-1 ) &
               then

!           --- cell below is in obstacle
!               found := true

               inside = .true.
               found = .true.

            else if ( iobgrid3d(ipcell,jpcell,kpcell-1,1)==0 ) &
               then

!           --- cell below is outside obstacle
!               ipointobst(inode) := 0
!               found := true

               found = .true.

            end if

         else if ( .not. found .and. izcross(1)==1 ) then

            if ( kpcell==0 ) then

!           --- Below there is no obstacle

               inside = .true.
               found = .true.

            else if ( iobgrid3d(ipcell,jpcell,kpcell-1,1)==-1 ) &
               then

!           --- cell below is inside obstacle
!               ipointobst(inode) := 0
!               found := true

               found = .true.

            else if ( iobgrid3d(ipcell,jpcell,kpcell-1,1)==0 ) &
               then

!           --- cell below is outside obstacle
!               ipointobst(inode) := 1
!               found := true

               inside = .true.
               found = .true.

            end if

         end if

         if ( .not. found .and. izcross(2)==0 ) then

!        --- no cross sections above z = zp

            if ( kpcell>=nz-1 ) then

!           --- Above there is no obstacle

               found = .true.

            else if ( iobgrid3d(ipcell,jpcell,kpcell+1,1)==-1 ) &
               then

!           --- cell above is in obstacle
!               ipointobst(inode) := iobst
!               found := true

               inside = .true.
               found = .true.

            else if ( iobgrid3d(ipcell,jpcell,kpcell+1,1)==0 ) &
               then

!           --- cell above is outside obstacle
!               ipointobst(inode) := 0
!               found := true

               found = .true.

            end if

         else if ( .not. found .and. izcross(2)==1 ) then

!        --- One cross section above

            if ( kpcell>=nz-1 ) then

!           --- Above there is no obstacle

               inside = .true.
               found = .true.

            else if ( iobgrid3d(ipcell,jpcell,kpcell+1,1)==-1 ) &
               then

!           --- cell above is in obstacle
!               ipointobst(inode) := 0
!               found := true

               found = .true.

            else if ( iobgrid3d(ipcell,jpcell,kpcell+1,1)==0 ) &
               then

!           --- cell above is outside obstacle
!               ipointobst(inode) := 1
!               found := true

               inside = .true.
               found = .true.

            end if

         end if

         if ( .not. found ) then

!        --- Complicated situation, type not yet detected
!            use line y=yp, z=zp to detect the number of cross sections at
!            the left-hand side
!            First store the intersections left of x into xi

            nintersect = 0
            do i = 1, nxi

               if ( xi(i)<x ) then

!              --- Cross section left of x

                  nintersect = nintersect+1
                  xi(nintersect) = xi(i)

              end if

            end do

            do while ( .not. found )

!           --- Find number of intersections in cell at left-hand side

               ipcell = ipcell-1
               call mshob3dcross1 ( x, y, z, ipcell, jpcell, &
                                    kpcell, iobgrid3d, nx, ny, nz, &
                                    eps, coorobst, kmeshcobst, &
                                    inpelm, nintersect, &
                                    leniobgrid, xi )

               if ( ipcell<=0 ) then

!              --- At the left-hand side there is no obstacle

                  if ( mod(nintersect,2)/=0 ) then

!                 --- odd number of cross sections

                     inside = .true.

                  end if
                  found = .true.

               else if ( iobgrid3d(ipcell-1,jpcell,kpcell,1)==-1 ) &
                  then

!              --- cell at left-hand side is in obstacle
!                  ipointobst(inode) := iobst
!                  found := true

                  if ( mod(nintersect,2)==0 ) then

!                 --- even number of cross sections

                     inside = .true.

                  end if
                  found = .true.

               else if ( iobgrid3d(ipcell-1,jpcell,kpcell,1)==0 ) &
                  then

!              --- cell at left-hand side is outside obstacle
!                  ipointobst(inode) := 0
!                  found := true

                  if ( mod(nintersect,2)/=0 ) then

!                 --- odd number of cross sections

                     inside = .true.

                  end if
                  found = .true.

               end if

            end do

         end if

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepob3dpnt'

      end if
      call erclos ( 'sepob3dpnt' )

      end
