      subroutine mshadcurvs ( usol, iwork, n, indprf, kprobf, &
                              nunkp, iadapt, rinada, coorol, &
                              coornw, anorm, inpada, closed, maxphys, &
                              numvecs, uold, iellist )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 09-03-2013 Correction declaration coornw
!        version  1.0  date 05-01-2009
!
!   copyright (c) 2009-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Adapt coordinates at boundary (2d)
!
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletime
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iadapt, iwork(*), n, indprf, kprobf(*), &
              nunkp, inpada(*), maxphys, numvecs, iellist(*)
      double precision  usol(*), rinada(*), coorol(2,n), coornw(2,*), &
                        anorm(2,n), uold(*)
      logical closed

!     anorm          i    In this array the normal vector is stored
!                         anorm(j,i) contains the j th component of the normal
!                         vector in the i-th integration point
!     closed         i    If true the boundary is closed
!     coornw        i/o   Array containing the new co-ordinates along the
!                         boundary
!     coorol         i    Array containing the old co-ordinates along the
!                         boundary
!     iadapt         i    Indicates how the boundary must be adapted, see INPADA
!     iellist        i    Array of length npoint-1 containing a
!                         list of elements that are adjacent to the boundary
!                         element iellist(i) contains both nodal points
!                         iwork(i) and iwork(i+1)
!     indprf         i    indication if kprob part f has been filled (>0)
!                         or not (0)
!     inpada         i    User input array for subroutine ADAPBOUN
!     iwork          i    integer work array containing the nodal points of the
!                         curves
!     kprobf         i    KPROB part f: contains the number of unknowns per
!                         point accumulated
!     maxphys        i    Maximum number of unknowns per point
!     n              i    number of nodes in boundary
!     numvecs        i    Number of vectors in iseqvecs
!     nunkp          i    maximal number of degrees of freedom in nodal points
!     rinada         i    User input array for subroutine ADAPBOUN
!     uold           i    Array containing the old solutions
!     usol           i    Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer imult, ncomp, number
      double precision amult, uloc(n,2), work(n)

!     amult          Multiplication factor
!     imult          Parameter imult as described in inpada(11)
!     ncomp          Number of components of the solution vector that are used
!     number         The number-th unknown in each nodal point of the array
!                    is considered.
!     uloc           Array of size n x ncomp in which the required
!                    degrees of freedom along the boundary are stored
!     work           Work array of length nunkp
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERSOLSELECT    Extract ncomp components form array usol in each point
!                    stored in iwork and put these components in array uloc
!     FUNCSOLCR      User subroutine to compute the co-ordinates of the new
!                    boundary if iadapt = 9
!     MSHCR01        Compute the co-ordinates of the new boundary if iadapt = 1
!     MSHCR02        Compute the co-ordinates of the new boundary if iadapt = 2
!     MSHCR03        Compute the co-ordinates of the new boundary if iadapt = 3
!     MSHCR04        Compute the co-ordinates of the new boundary if iadapt = 4
!     MSHCR05        Compute the co-ordinates of the new boundary if iadapt = 5
!     MSHCR06        Compute the co-ordinates of the new boundary if iadapt = 6
!     MSHCR07        Compute the co-ordinates of the new boundary if iadapt = 7
!     MSHCR08        Compute the co-ordinates of the new boundary if iadapt = 8
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2990   Value of iadapt not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshadcurvs' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshadcurvs'
         write(irefwr,1) 'iadapt, n, nunkp', iadapt, n, nunkp
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      imult  = inpada(11)
      number = inpada(6)

      if ( imult==0 ) then
         amult = 1d0
      else if ( imult==1 ) then
         amult = tstep
      else if ( imult==2 ) then
         amult = rinada(1)
      else
         amult = rinada(1) * tstep
      end if
      if ( iadapt==1 ) then

!     --- iadapt = 1

         ncomp = 1

      else

!     --- iadapt = 1

         ncomp = 2

      end if

      if ( debug ) then
         write(irefwr,1) 'imult, number, ncomp', imult, number, ncomp
         write(irefwr,2) 'amult', amult
      end if  ! ( debug )

      if ( iadapt==1 .or. iadapt>2 .and. iadapt<=5 ) then

!     --- Fill array uloc if necessary

         call ersolselect ( usol, iwork, n, indprf, kprobf, &
                            number, ncomp, uloc, nunkp )

      end if

!     --- compute new co-ordinates and store in array coornw

      select case (iadapt)

      case (1)

!     --- iadapt = 1, x_new = x_old + u n

         call mshcr01 ( coorol, coornw, uloc, anorm, n )

      case (2)

!     --- iadapt = 2, x_new = computed by funccr

         call mshcr02 ( coorol, coornw, usol, anorm, n, indprf, kprobf, &
                        number, iwork, nunkp, work )

      case (3)

!     --- iadapt = 3, x_new = computed by film method

         if ( debug ) call prinrl1 ( uloc, n, 2, 'uloc' )
         call mshcr03 ( coorol, coornw, uloc, anorm, n, inpada, rinada )

      case (4)

!     --- iadapt = 4, x_new = x_old + factor * u

         call mshcr04 ( coorol, coornw, uloc, n, amult )

      case (5)

!     --- iadapt = 5, x_new = x_old + f u.n n  (normal velocity)

         call mshcr05 ( coorol, coornw, uloc, anorm, n, amult )

      case (6)

!     --- iadapt = 6, x_new = x_old + f gradc.n n

         call mshcr06 ( coorol, coornw, usol, n, amult, iellist )

      case (7)

!     --- iadapt = 6, x_new = x_old + f gradc.n n

         call mshcr07 ( coorol, coornw, usol, n, amult, iellist, anorm, &
                        work, rinada(7), rinada(6), inpada(18), &
                        inpada(20) )

      case (8)

!     --- iadapt = 8, x_new = x_old + factor * uconst

         call mshcr08 ( coorol, coornw, rinada(15), n, amult )

      case (9)

!     --- iadapt = 9, x_new = computed by funcsolcr

         call funcsolcr ( coorol, coornw, uold, anorm, n, maxphys, &
                          numvecs )

      case default

         call errint ( iadapt, 1 )
         call errsub ( 2990, 1, 0, 0 )
         go to 1000

      end select  ! (case)

      if ( closed ) then

!     --- Curve is closed, copy first point to last one

         if ( iadapt/=6 ) then
            n = n+1
            coornw(1:2,n) = coornw(1:2,1)
         else
            coornw(1:2,n) = 0.5d0*(coornw(1:2,1)+coornw(1:2,n))
         end if

      end if

      if ( debug ) then

!     --- Debug information

         call  prinrl1(coorol, n, 2, 'coorol')
         call  prinrl1(coornw, n, 2, 'coornw')

      end if

1000  call erclos ( 'mshadcurvs' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshadcurvs'

      end if  ! ( debug )

      end
