      subroutine mshcentrecirc ( normal, coor, curvenodes, numpoints, &
                                 origin, radius, isurnr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 19-01-2007 Eps also via input file
!        version  1.1  date 06-04-2005 Extra debug information
!        version  1.0  date 15-10-2004
!
!   copyright (c) 2004-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute centroid of (part of a) circle as well as radius
!     Check if all nodes are positioned on circle
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     plane
!     surface
!     centre
!     radius
!     check
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision normal(3), coor(3,*), origin(3), radius
      integer curvenodes(*), numpoints, isurnr

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     curvenodes     i    Contains the nodes of the boundary curve
!     isurnr         i    Surface sequence number
!     normal         i    normal on plane
!     numpoints      i    Number of points in icurnr
!     origin         o    Contains the coordinates of the origin of the circle
!     radius         o    Radius of sphere or circle
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision angles(numpoints), eps
      integer numstraight, startpoints(2), endpoints(2), indcol(3)

!     angles         Array containing the angles along the closed curve in the
!                    sequence defined by curvenodes
!     endpoints      Array containing the relative end points of the straight
!                    lines on the part of the circle (if any)
!                    relative with respect to array curvenodes
!     eps            Local accuracy
!     indcol         Help array for print subroutine to store column numbers
!     numstraight    Number of straight lines on the part of the circle
!     startpoints    Array containing the relative start points of the straight
!                    lines on the part of the circle (if any)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHCENTRCIRCLE Compute centroid of circle as well as radius
!     MSHCENTREHALFC Compute centroid of a half circle as well as radius
!     MSHCENTREPARTC Compute centroid of part of a circle as well as radius
!     MSHNUMSTRAIGHT Find the number of straight lines as well as the start
!                    and end points
!     MSHSTOREANGLE  Store the angles corresponding to the closed curve with
!                    nodes stored in curvenodes, into array angles
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRINRL1_IND    Print parts of a full element matrix by indicating rows and
!                    columns
!     READCONSDEF    Check if real constant can be found in part constants of
!                    input file
!                    If so, set constant otherwise use default value
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Check if there are sets of consecutive nodes (at least 3) that are on
!     a straight line
!     t1 := first tangential vector
!     Find first tangential vector t2 along curve such that t1 . t2<0
!     Use last vector t2
!     C := intersection n x t1 and n x t2
!     R := distance from points on circle to C
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcentrecirc' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcentrecirc'
         call prinin ( curvenodes, numpoints, 'curvenodes' )
         indcol(1) = 1
         indcol(2) = 2
         indcol(3) = 3
         call prinrl1_ind ( coor, 3, numpoints, curvenodes, indcol, &
                            numpoints, 3, 'coordinates' )

      end if
      if ( ierror/=0 ) go to 1000

      eps = 1d-6

!     --- Store all angles in array angles

      call mshstoreangle ( coor, numpoints, curvenodes, 3, angles )
      if ( ierror/=0 ) go to 1000
      if ( debug ) call prinrl ( angles, numpoints, 'angles' )

!     --- Find the number of straight lines as well as the start and end points
!         of these lines

      call mshnumstraight ( numpoints, angles, numstraight, &
                            startpoints, endpoints, isurnr )
      if ( ierror/=0 ) go to 1000

      if ( numstraight==0 ) then

!     --- numstraight = 0, complete circle

         call mshcentrcircle ( normal, coor, curvenodes, numpoints, &
                               origin, radius, isurnr, eps )

      else if  ( numstraight==1 ) then

!     --- numstraight = 1, half circle

         call mshcentrehalfc ( coor, curvenodes, numpoints, origin, &
                               radius, isurnr, startpoints, &
                               endpoints, eps )

      else

!     --- numstraight = 2, part of circle

         call mshcentrepartc ( coor, curvenodes, numpoints, origin, &
                               radius, isurnr, startpoints, &
                               endpoints, eps )

      end if  ! ( numstraight==0 )
      if ( ierror/=0 ) go to 1000

1000  call erclos ( 'mshcentrecirc' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshcentrecirc'

      end if

      end
