      subroutine mshchsurf03 ( psi, nnodes, velocity, anorm, rinada, &
                               coornw, coorol, inpada )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 21-01-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the co-ordinates of the new 3d boundary in the case
!     iadapt = 3 i.e. x_new is computed by the so-called film method
! **********************************************************************
!
!                       KEYWORDS
!
!     adapt_boundary
!     free_boundary
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletime
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nnodes, inpada(*)
      double precision psi(nnodes,4), velocity(nnodes,3), rinada(*), &
                       anorm(3,nnodes), coornw(3,nnodes), &
                       coorol(3,nnodes)

!     anorm          i    In this array the normal vector is stored
!                         anorm(j,i) contains the j th component of the normal
!                         vector in the i-th boundary point
!     coornw        i/o   Array containing the new co-ordinates along the
!                         boundary
!     coorol         i    Array containing the old co-ordinates along the
!                         boundary
!     inpada         i    User input array for subroutine ADAPBOUN
!     nnodes         i    Number of nodes
!     psi            i    Contains the "pseudo" stream function at the
!                         boundary
!     rinada         i    User input array for subroutine ADAPBOUN
!     velocity       i    Contains the velocity vector in the boundary points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision un, ut(3), normut, thresh, amult, alpha
      integer ithres, imult, i

!     alpha          multiplication factor
!     amult          multiplication factor stored in rinada
!     i              Counting variable
!     imult          Defines mulitplication factor, see inpada
!     ithres         Defines how threshold value must be computed
!     normut         Norm of tangential vector
!     thresh         threshold value
!     un             normal component of velocity
!     ut             tangential vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    The co-ordinates along the boundary are updated according to the
!    formula:
!    x_new(i) = x_old(i) + alpha(i) u(i) * n(i)
!    alpha(i) = factor (psi(i)-psi(0)) / |u(i).t(i)|
!                       x(i)
!    psi(i) - psi(0) = /     u.n dx
!                      x(0)
!    Since in 3D the direction of the tangential vector is not so clear
!    the computation of the "stream function" psi must be done in a
!    different way.
!
!    We start with a line, where psi must be zero
!    Next psi is computed in the set of elements connected to the start
!    This process is repeated until all elements along the boundary are filled
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshchsurf03' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshchsurf03'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      imult  = inpada(11)
      ithres = inpada(12)

      if ( ithres==0 ) then
         thresh = epsmac
      else
         thresh = max ( epsmac, rinada(2) )
      end if

      if ( imult==0 ) then
         amult = 1d0
      else if ( imult==1 ) then
         amult = tstep
      else
         amult = rinada(1)
      end if

!     --- For all nodes along surface do

      do i = 1, nnodes

         un = velocity(i,1)*anorm(1,i) + &
              velocity(i,2)*anorm(2,i) + &
              velocity(i,3)*anorm(3,i)
         ut = velocity(i,1:3) - un * anorm(1:3,i)
         normut = sqrt(ut(1)**2+ut(2)**2+ut(3)**2)

         if ( normut>thresh ) then

!        --- ||u_t||>thresh, hence update

            alpha = psi(i,1)/normut
            if ( imult>0 ) alpha = amult*alpha
            coornw(1:3,i) = coorol(1:3,i)+alpha*anorm(1:3,i)

         else

!        --- ||u_t|| small use old coordinates

            coornw(1:3,i) = coorol(1:3,i)

         end if  ! ( normut>thresh )

      end do  ! i = 1, nnodes

1000  call erclos ( 'mshchsurf03' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshchsurf03'

      end if  ! ( debug )

      end
