      subroutine mshcompdispl ( iseqdisp, ireadadap, iseqvecs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 20-08-2016 use sepmoduleinput
!        version  1.0  date 30-12-2008
!
!   copyright (c) 2008-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute displacement vector for boundary due to change of
!     coordinates
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulemain
      use sepmoduleinput
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ireadadap, iseqvecs(*), iseqdisp

!     ireadadap      i    Sequence number for input adap boundary
!     iseqdisp       i    sequence number of displacement vector in ISOL
!     iseqvecs       i    Contains sequence numbers of vectors with respect to
!                         isol
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ivec, ipos, maxcall, iprobsav
      integer, pointer :: inpada(:)
      double precision, pointer :: coornw(:,:), rinada(:)

!     coornw         Array containing the new co-ordinates
!                    they are not
!     inpada         User input array for subroutine ADAPBOUN
!     ipos           Position of input arrays in sepmoduleinput
!     iprobsav       Help parameter to save the value of iprob
!     ivec           Sequence number of vector of special structure
!     maxcall        Maximum number of coefficient inputs read
!     rinada         User input array for subroutine ADAPBOUN
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INISUBTRACT    Subtract two vectors and store the result in a third vector
!     MSHCOMPDISPL1  Compute displacement vector as difference of coordinates
!     MSHCOMPDISPL2D Compute new coordinates of boundary (2d)
!     MSHCOMPDISPL3D Compute new coordinates of boundary (3d)
!     SEPCREASOL110  Activate or create arrays of type 110
!     SEPFILLKPROB   Fill parameters in module sepmodulekprob
!     SEPGETINPFINF  Find maxcall and ipos from sepmoduleinput as function of
!                    isubr and iseqnr
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     419   NDIM incorrect
!    1948   No input with the main keyword has been read
!    2910   Not enough space
!    2911   Not possible to deallocate
! **********************************************************************
!
!                       PSEUDO CODE
!
!     coornw := coorol
!     if ( ndim=2 ) then
!        update curves
!        copy new coordinates of curves into coornw
!     else ( ndim=3 )
!        update surfaces
!        copy new coordinates of curves into coornw
!     displacement = coornw - coorol
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcompdispl' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcompdispl'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate( coornw(ndim,npoint), stat = error)
      if ( error/= 0 ) go to 900

!     --- Fill arrays inpada and rinada

      call sepgetinpfinf ( 16, ireadadap, ipos, maxcall )
      if ( maxcall==0 ) then
         call errchr ( 'ADAPT_BOUNDARY', 1 )
         call errsub ( 1948, 0, 0, 1 )
         go to 1000
      end if

      iprobsav = iprob

      inpada => in(ipos)%iinput
      rinada => re(ipos)%rinput

      if ( ndim==2 ) then

!     --- ndim = 2, update curves
!         copy new coordinates of curves into coornw

         call  mshcompdispl2d ( coornw, inpada, rinada, iseqvecs )

      else if ( ndim==3 ) then

!     --- ndim = 3, update surfaces
!         copy new coordinates of curves into coornw

         call mshcompdispl3d ( coornw, inpada, rinada, iseqvecs )

      else

!     --- ndim # 2, 3, error

         call errint ( ndim, 1 )
         call errint ( 2, 2 )
         call errint ( 3, 3 )
         call errsub ( 419, 3, 0, 0 )
         go to 1000

      end if  ! ( ndim==2 )

!     --- Create new solution array corresponding to displacement

      iprob = iprobsav
      iseqkp = kprob
      kprobpart => kp(iseqkp)%kprobsub(iprob)
      kprobloc => kprobpart%kprobloc
      indprf = kprobloc(19)
      kprobf => kprobpart%kprobf
      call sepfillkprob

      call sepcreasol110 ( isolut2d(iseqdisp), 'displacement', 0 )
      displacement => usoltemp

!     --- displacement = coornw - coorol

      if ( indprf==0 ) then

!     --- indprf = 0, constant number of degrees of freedom

         call inisubtract ( coornw, coor, displacement, npoint*ndim )

      else

!     --- indprf # 0, non-constant number of degrees of freedom

         call  mshcompdispl1 ( npoint, coor, coornw, &
                               displacement, kprobf, ndim )

      end if  ! ( indprf==0 )
      go to 1000

900   if ( error/= 0 ) then

!     --- error#0, so not enough space

         call errint ( npoint*ndim, 1 )
         call errint ( error, 2 )
         call errchr ( 'coornw', 1 )
         call errsub ( 2910, 2, 0, 1 )
         go to 1000

      end if  ! ( error/= 0 )

1000  deallocate ( coornw, stat = error )
      if ( error/= 0 ) then

!     --- error#0, problem with deallocate

         call errint ( error, 1 )
         call errchr ( 'coornw', 1 )
         call errsub ( 2911, 1, 0, 1 )

      end if  ! ( error/= 0 )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshcompdispl'

      end if  ! ( debug )
      call erclos ( 'mshcompdispl' )

      end
