      subroutine mshcompdispl2d ( coornw, inpada, rinada, iseqvecs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 31-12-2008
!
!   copyright (c) 2008-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute new coordinates of boundary (2d)
!
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     mesh
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      use sepmodulekprob
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpada(*),  iseqvecs(*)
      double precision coornw(2,*), rinada(*)

!     coornw         o    Array containing the new co-ordinates
!     inpada         i    User input array for subroutine ADAPBOUN
!     iseqvecs       i    Contains sequence numbers of vectors with respect to
!                         isolut2d
!     rinada         i    User input array for subroutine ADAPBOUN
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ivectr, maxphys, numvecs
      integer, allocatable :: markcurv(:), marknodes(:)

!     ivectr         sequence number of velocity vector in isolut2d
!     markcurv       Array of size ncurvs that marks all curves that
!                    have been adapted in the moving boundary
!                    markcurv(icurnr) = 0: curve has not been moved
!                    markcurv(icurnr) = 1: curve has been moved
!     marknodes      Array of size npoint that marks all nodes that
!                    have been adapted in the moving boundary
!                    marknodes(node) = 0: node has not been moved
!                    marknodes(node) = 1: node has been moved
!     maxphys        Maximum number of unknowns per point
!     numvecs        Number of vectors in iseqvecs (at present 1)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHCOMPDISPL20 Compute new coordinates of boundary (2d)
!     MSHCOMPDISPL21 Adapt coordinates of curves of which at least one point has
!                    been moved in a changing boundary
!     SEPACTSOLBF2   compute pointers to degrees of freedom of vectors
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1393   Not constant number of degrees of freedom not available in intmsh.
!    2910   Not enough space
!    2911   Not possible to deallocate
! **********************************************************************
!
!                       PSEUDO CODE
!
!     coornew := coorold
!     Store nodes of curves to be changed in iwork and coordinates in curves
!     Compute new coordinates of curves
!     Adapt coornew
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcompdispl2d' )

      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcompdispl2d'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get parameters

      ivectr = iseqvecs(1)      ! sequence number of velocity vector

!     --- allocate arrays

      allocate ( markcurv(ncurvs), marknodes(npoint), stat = error)
      if ( error/= 0 ) then

!     --- error#0, so not enough space

         call errint ( npoint+ncurvs, 1 )
         call errint ( error, 2 )
         call errchr ( 'markcurv', 1 )
         call errsub ( 2910, 2, 0, 1 )
         go to 900

      end if  ! ( error/= 0 )

      markcurv = 0
      marknodes = 0

!     --- Activate arrays

      call sepactsolbf2 ( isolut2d(ivectr) )
      if ( ks(isolut2d(ivectr))%typevec==115 .and. nphyslc>0 ) then
         nunkpi=nphysi
      else if ( ks(isolut2d(ivectr))%typevec==116 ) then
         nunkpi = 2
      else if (ks(isolut2d(ivectr))%typevec/=110) then
         call errint ( ks(isolut2d(ivectr))%typevec, 1 )
         call errchr ( 'mshcompdispl2d', 1 )
         call errsub ( 1393, 1, 0, 1 )
         nunkpi = 0
      end if

!     --- Compute starting addresses

      usoltemp => ks(isolut2d(ivectr))%sol

!     --- Compute new coordinates

      maxphys = nphysi
      numvecs = 1
      call mshcompdispl20 ( coornw, inpada, rinada, nunkpi, &
                            maxphys, usoltemp, usoltemp, numvecs, indprfi, &
                            kprobfi, markcurv, marknodes )

!     --- Adapt curves of which at least one point has been changed

      call mshcompdispl21 ( markcurv, marknodes, coornw )

900   deallocate ( markcurv, marknodes, stat = error )
      if ( error/= 0 ) then

!     --- error#0, problem with deallocate

         call errint ( error, 1 )
         call errchr ( 'markcurv', 1 )
         call errsub ( 2911, 1, 0, 1 )

      end if  ! ( error/= 0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshcompdispl2d'

      end if  ! ( debug )
      call erclos ( 'mshcompdispl2d' )

      end subroutine mshcompdispl2d
