      subroutine mshcurspline ( nnodes, nodes, coorold, ndim, &
                                tau, icurnr, numnodes, coornew )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-01-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create a spline through an existing curve
!     In fact nodes are redistributed
! **********************************************************************
!
!                       KEYWORDS
!
!     approximation
!     curve
!     free_boundary
!     free_surface
!     adapt_boundary
!     spline
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nnodes, nodes(*), ndim, icurnr, numnodes
      double precision coorold(ndim,*), tau(nnodes), coornew(ndim,*)

!     coornew       i/o   Array containing the new coordinates of the nodes
!     coorold        i    Array containing the old coordinates of the nodes
!     icurnr         i    Curve sequence number
!     ndim           i    Dimension of the space.
!     nnodes         i    Number of nodes in curve
!     nodes          i    Contains the node numbers of curve icurnr
!     numnodes       i    Number of nodal points that can be used to define the
!                         spline
!     tau            i    Array of length n with the relative distances
!                         between points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ispinp(2+nnodes), i, nodp
      double precision curves(ndim,nnodes), alpha

!     alpha          factor for spline construction
!     curves         array containing the coordinates of the curve
!     i              Counting variable
!     ispinp         Integer input array of subroutine mshc01
!     nodp           Nodal point sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHC01         Fill array curves with splines
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcurspline' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcurspline'
         call prinrl ( tau, nnodes, 'tau' )
         call prinin ( nodes, nnodes, 'nodes' )
         write(irefwr,1) 'nnodes, numnodes', nnodes, numnodes
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill integer input array for mshc01

      ispinp(1) = 32
      ispinp(2) = numnodes
      do i = 1, numnodes
         ispinp(2+i) = i
      end do  ! i = 1, nnodes

!     --- Fill array curves

      if ( icurnr>0 ) then

!     --- icurnr>0, positive direction

         do i = 2+nnodes-numnodes, nnodes

            nodp = nodes(i)
            curves(1:ndim,i) = coorold(1:ndim,nodp)

         end do  ! i = 1, nnodes
         curves(1:ndim,1) = coornew(1:ndim,nodes(1))

      else

!     --- icurnr<0, negative direction

         do i = 1, numnodes-1

            nodp = nodes(numnodes-i)
            curves(1:ndim,i+1) = coorold(1:ndim,nodp)

         end do  ! i = 1, nnodes
         curves(1:ndim,1) = coornew(1:ndim,nodes(nnodes))

      end if  ! ( icurnr>0 )

      if ( debug ) call prinrl1 ( curves, numnodes, 2, 'curves/1' )

!     --- Compute spline, result in array curves

      alpha = 0.5d0
      call mshc01 ( ispinp, curves, curves, &
                    ndim, nnodes, tau, alpha, icurnr )
      if ( debug ) call prinrl1 ( curves, nnodes, 2, 'curves/2' )

!     --- Put interpolation into array coornew

      if ( icurnr>0 ) then

!     --- icurnr>0, positive direction

         do i = 2, nnodes

            nodp = nodes(i)
            coornew(1:ndim,nodp) = curves(1:ndim,i)

         end do  ! i = 1, nnodes

      else

!     --- icurnr<0, negative direction

         do i = 1, nnodes-1

            nodp = nodes(nnodes-i)
            coornew(1:ndim,nodp) = curves(1:ndim,i+1)

         end do  ! i = 1, nnodes

      end if  ! ( icurnr>0 )

1000  call erclos ( 'mshcurspline' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshcurspline'

      end if  ! ( debug )

      end
