      function mshelllength ( a, b, t0, phi, n )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-08-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Return with approximate length of ellips defined by parameters
!
! **********************************************************************
!
!                       KEYWORDS
!
!     length
!     curve
!     ellips
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer n
      double precision mshelllength, a, b, t0, phi

!     a              i    Parameter defining the ellips
!                         (length of first direction)
!     b              i    Parameter defining the ellips
!                         (length of second direction)
!     mshelllength   o    Approximate length of element
!     n              i    number of points used in numerical integration
!     phi            i    Difference in parameter t from begin to end
!     t0             i    Value of t corresponding to x1
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      double precision dphi, xs, ys, slength, t, xn, yn

!     dphi           phi/n
!     i              Counting variable
!     slength        Parameter to compute the length
!     t              parameter allong ellips
!     xn             x coordinate of first point in element
!     xs             x coordinate of last point in element
!     yn             y coordinate of first point in element
!     ys             y coordinate of last point in element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The ellips is defined as x = a cos(t), y = b sin(t)
!     From t0 to t1
!     To compute the length we sun the length of n intervals
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshelllength' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshelllength'
         write(irefwr,2) 'a, b, t0, phi', a, b, t0, phi
         write(irefwr,1) 'n', n
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- The ellips is defined as x = a cos(t), y = b sin(t)
!         From t0 to t1
!         To compute the length we sun the length of n intervals

      dphi = phi/n
      xs = a*cos(t0)
      ys = b*sin(t0)
      slength = 0d0
      do i = 2, n
         t = t0 + (i-1)*dphi
         xn = a*cos(t)
         yn = b*sin(t)
         slength = slength+sqrt((xn-xs)**2+(yn-ys)**2)
         xs = xn
         ys = yn
      end do  ! i = 2, n

      mshelllength = slength

1000  call erclos ( 'mshelllength' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,2) 'mshelllength', slength
         write(irefwr,*) 'End mshelllength'

      end if  ! ( debug )

      end
