      subroutine mshellparms ( x1, y1, x2, y2, xt, yt, a, b, t0, phi, &
                               iuserp3 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 19-02-2011  Extra parameter iuserp3
!        version  1.1  date 19-01-2011  Correction angle
!        version  1.0  date 15-08-2009
!
!   copyright (c) 2009-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute defining parameters for ellips
!     Two nodes and the centre of the ellips are given
! **********************************************************************
!
!                       KEYWORDS
!
!     ellips
!     parameter
!     centre
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleconsts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iuserp3
      double precision x1, y1, x2, y2, xt, yt, a, b, t0, phi

!     a              o    Parameter defining the ellips
!                         (length of first direction)
!     b              o    Parameter defining the ellips
!                         (length of second direction)
!     iuserp3        i    third user point
!     phi            o    Difference in parameter t from begin to end
!     t0             o    Value of t corresponding to x1
!     x1             i    x-coordinate of first point
!     x2             i    x-coordinate of last point
!     xt             i    x-coordinate of centre
!     y1             i    y-coordinate of first point
!     y2             i    y-coordinate of last point
!     yt             i    y-coordinate of centre
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision delta, t1, eps

!     delta          help parameter: -(y1^2-y2^2)/(x1^2-x2^2)
!     eps            local accuracy
!     t1             Value of t corresponding to x2
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Compute the ellips parameters a and b that define the ellips
!     according to (x-xo)^2/a^2 + (y-yo)^2/b^2 = 1
!     This is done by substituting x1 and x2 for x and solving the
!     2 systems of equations
!     Mark that the origin has been subtracted from x1 and x2
!     Subtracting equation with x2 from one with x1 gives
!     b^2/a^2 = -(y1^2-y2^2)/(x1^2-x2^2) = delta
!     Substitue b^2 = delta a^2 into equation 1:
!     a^2 = x1^2+y1^2/delta
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshellparms' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshellparms'
         write(irefwr,2) 'x1, y1, x2, y2', x1, y1, x2, y2
         write(irefwr,2) 'xt, yt', xt, yt
  2      format ( a, 1x, (6d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      eps = 1d-8*((x1-xt)**2+(y1-yt)**2)

      if ( (x1-x2)**2+(y1-y2)**2<eps ) then

!     --- initial and end point coincide, complete ellips

         phi = 2d0*pi
         t0 = 0d0
         t1 = 0d0
         go to 1000

      end if  ! ( (x1-x2)**2+(y1-y2)**2)<eps )

!     --- Subtract origin from x1 and x2

      x1 = x1-xt
      y1 = y1-yt
      x2 = x2-xt
      y2 = y2-yt
      if ( debug ) &
         write(irefwr,2) 'x1, y1, x2, y2', x1, y1, x2, y2

!     --- Compute the ellips parameters a and b that define the ellips
!         according to (x-xo)^2/a^2 + (y-yo)^2/b^2 = 1
!         This is done by substituting x1 and x2 for x and solving the
!         2 systems of equations
!         Mark that the origin has been subtracted from x1 and x2
!         Subtracting equation with x2 from one with x1 gives
!         b^2/a^2 = -(y1^2-y2^2)/(x1^2-x2^2) = delta
!         Substitute b^2 = delta a^2 into equation 1:
!         a^2 = x1^2+y1^2/delta

      delta = -(y1**2-y2**2)/(x1**2-x2**2)
      if ( debug ) write(irefwr,2) 'delta', delta
      a = sqrt(x1**2+y1**2/delta)
      b = a*sqrt(delta)
      if ( debug ) write(irefwr,2) 'a, b', a, b

!     --- Compute parameters t0 and t1 from the parameter representation
!         of the ellips:  x(t) = a cos(t), y(t) = b sin(t)
!         So t0 = arccos(x1/a), t1 = arccos(x2/a)

      t0 = acos(x1/a)
      if ( debug ) write(irefwr,2) 't0, y1', t0, y1
      if ( y1<0 ) t0 = 2d0*pi-t0
      t1 = acos(x2/a)
      if ( debug ) write(irefwr,2) 't0, t1, y2', t0, t1, y2
      if ( y2<0 ) t1 = 2d0*pi-t1
      if ( debug ) write(irefwr,2) 't1', t1
      if ( iuserp3>0 ) then

!     --- iuserp3 > 0, counterclockwise

         if ( t1<t0 ) t1 = t1+2d0*pi

      else

!     --- iuserp3 < 0, clockwise

         if ( t1>t0 ) t0 = t0+2d0*pi

      end if  ! ( iuserp3>0 ) then
      phi = t1 - t0     ! Total angle

1000  call erclos ( 'mshellparms' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,2) 't0, t1, phi', t0, t1, phi
         write(irefwr,*) 'End mshellparms'

      end if  ! ( debug )

      end
