      subroutine mshfillkmeshq
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 24-11-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if surfaces have more than one intersecting curve with other
!     surfaces. If so create compound curves by combination of such curves
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     surface
!     curve
!     compound_curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nextracurvs, lensubcurvs, lenkmeshm, lenkmeshmext
      logical check
      integer, allocatable :: kmeshmold(:), subcurvs(:), ipointer(:)

!     check          Indicates if there are surfaces with at least 2
!                    common curves
!     ipointer       pointer array of length nextracurvs
!                    contains starting addresses in array subcurvs
!     kmeshmold      Old array kmeshm
!     lenkmeshm      Length of array kmeshm
!     lenkmeshmext   Length of new array kmeshm
!     lensubcurvs    Extra length required for compound curves
!     nextracurvs    Number of extra curves to be created
!     subcurvs      Contains subcurve numbers of compound curves
!                    sequentially
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     ERTRACE        Prints trace back of subroutines
!     MSHCHECKCOMPCR Check if surfaces have more than one intersecting curve
!                    with other surfaces.
!     MSHFILLCOMPCUR Create set of compound curves by combining curves in the
!                    intersection of surfaces
!     MSHFILLKMESHME Extend array kmesh part m with new (compound) curves
!     MSHFILLKMESHQ1 Fill array kmeshq with information stored in the arrays
!                    ipointer and subcurves
!     PR0008         Print information of curves
!     PR0012         Print accumulated array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2744   The situation that kmesh part Q has already been filled has not yet
!           been implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
!     check := there are surfaces with more than one common curve
!     if ( check ) then
!        create set of compound curves
!        create or extend array kmeshq
!        adapt array kmeshm
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshfillkmeshq' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshfillkmeshq'
         call ertrace
         call pr0008 ( kmeshm, ncurvs )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- check := there are surfaces with more than one common curve

      call mshcheckcompcr ( check, nextracurvs, lensubcurvs )
      if ( ierror/=0 ) go to 1000
      if ( check ) then

!     --- check = true, hence surfaces with more than one common curve
!         create set of compound curves

         allocate ( subcurvs(lensubcurvs), ipointer(nextracurvs), &
                    stat = error )
         if ( error/=0 ) &
            call eralloc ( error, nextracurvs+lensubcurvs, 'subcurvs' )
         if ( ierror/=0 ) go to 1000

!        --- Set starting addresses

         call mshfillcompcur ( ipointer, subcurvs, lensubcurvs )
         if ( ierror/=0 ) go to 1000

!        --- create or extend array kmeshq

         if ( kelmq/=0 ) then

!        --- kelmq > 0, i.e. kmeshq has been filled
!            This situation has not yet been implemented

            call errsub ( 2744, 0, 0, 0 )
            go to 1000

         else

!        --- kelmq = 0, fill kmeshq

            length = nextracurvs+ncurvs+1+lensubcurvs

!           --- Create array

            if ( km(iseqkm)%kmesh(31)>0 ) deallocate(km(iseqkm)%kmeshq)
            allocate(km(iseqkm)%kmeshq(length))
            kmeshq => km(iseqkm)%kmeshq

!           --- Actual filling

            call mshfillkmeshq1 ( ncurvs, nextracurvs, kmeshq, &
                                  ipointer, subcurvs, kmeshm, &
                                  lenkmeshmext )
            if ( ierror/=0 ) go to 1000

         end if  ! ( kelmq/=0 )

!        --- adapt array kmeshm

         length = size ( kmeshm )
         lenkmeshm = length*2

!        --- Copy old kmeshm to end of ibuffr

         allocate ( kmeshmold(length), stat = error )
         if ( error/=0 ) call eralloc ( error, length, 'kmeshmold' )
         if ( ierror/=0 ) go to 1000

         kmeshmold(1:length) = kmeshm

!        --- Create new array kmeshm

         if ( km(iseqkm)%kmesh(27)>0 ) deallocate(km(iseqkm)%kmeshm)
         allocate(km(iseqkm)%kmeshm(lenkmeshm))
         kmeshm => km(iseqkm)%kmeshm
         kmeshmglob => km(iseqkm)%kmeshm
         call mshfillkmeshme ( ncurvs, nextracurvs, ipointer, &
                               subcurvs, kmeshmold, kmeshm )

!        --- Set new number of curves

         ncurvs = ncurvs+nextracurvs
         if ( debug ) write(irefwr,1) 'ncurvs', ncurvs

         deallocate ( kmeshmold, subcurvs, ipointer, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'kmeshmold' )

      end if  ! ( check )
      km(iseqkm)%kmesh(11) = ncurvs

1000  if ( debug ) then

!     --- Debug information

         if ( check ) then
            call pr0008 ( kmeshm, ncurvs )
            write ( irefwr,'(//'' Subcurve numbers of curves''/)')
            call pr0012 ( ncurvs, kmeshq, kmeshq )
         end if  ! ( check )
         write(irefwr,*) 'End mshfillkmeshq'

      end if
      call erclos ( 'mshfillkmeshq' )

      end
