      subroutine mshframecur ( iinput, coor, curves, ndim, n, tau, &
                               rinput, cunit, ishape, icurnr, &
                               irelnodes )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  1.1  date 20-06-2013 New definition file ref numbers
!        version  1.0  date 06-05-2009
!
!   copyright (c) 2009-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Driving subroutine for framecurve generation
!     Compute position of nodes on submitted framecurve in accordance
!     with the given coarsenessvalues and place the coordinates in array
!     curves. Fill array irelnodes.
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh_generation
!     framecurve
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), ndim, n, icurnr, &
              irelnodes(*), ishape
      double precision coor(ndim,*), curves(ndim,*), tau(*), &
                       rinput(*), cunit

!     coor     i/o   standard coordinate array
!     cunit     i    unit of global coarseness as given by user
!     curves    o    coordinates of nodes of curve array
!     icurnr    i    curve sequence number
!     iinput    i    array containing the basis nodal point numbers
!     irelnodes o    array with the same length as the first part of
!                    iinput (until surfaces)
!                    Relative node numbers of the user points in
!                    framecurv are filled at the same positions as
!                    the user points in iinput
!     ishape    i    elementstype indicator
!                    1  :  linear elements
!                    2  :  quadratic elements
!                    3  :  cubic elements
!     n         i    number of points to be created
!     ndim      i    Euclidian dimension
!     rinput    i    Part of general input array with respect to mesh generation
!                    The first position must refer to alpha
!     tau       i    distribution of points array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lenname, ios, iref
      character (len=13) filename
      integer, allocatable          :: helpusp(:)
      double precision, allocatable :: helppoints(:,:)
      integer i, ip, ipamat, iparc, ipsolv, iprhs1, iprhs2, iprhs3, &
              itau, itype, iusp, j, lnp, lenirelnodes, &
              ndef, ndegree, nh, nhelm, nnodes, ntype
      double precision alpha, coarse, curvelen, diff, dist, dtau, &
              fact, srest, tauh, x1, y1, z1, dx, dy, dz, h
      logical check
      double precision, allocatable :: work(:)

!     alpha          spline parameter
!     check          Indicates if the file exist (true) or not
!     coarse         helpvariable for local coarseness
!     curvelen       length of curve
!     diff           help parameter to compute the difference of two parameters
!     dist           Euclidian distance
!     dtau           step in tau values
!     dx             length of vector in x-direction
!     dy             length of vector in y-direction
!     dz             length of vector in z-direction
!     fact           factor for computing coordinates along linepiece
!     filename       Name of file
!     h              Help parameter to store constant temporarily
!     helppoints     ?
!     helpusp        ?
!     i              counting variable
!     ios            IOSTAT variable, returns from reading or writing
!     ip             pointer
!     ipamat         starting position of matrix
!     iparc          starting position of arclength array
!     iprhs1         starting position of x-rhside
!     iprhs2         starting position of y-rhside
!     iprhs3         starting position of z-rhside
!     ipsolv         starting position solution
!     iref           File reference number
!     itau           counting variable
!     itype          spline type
!     iusp           helpvariable to store usepoint number
!     j              counting variable
!     lenirelnodes   length of array irelnodes
!     lenname        Length of name
!     lnp            counting variable
!     ndef           number of (spline) defining points
!     ndegree        degree of spline
!     nh             local number of elements to be created
!     nhelm          total number of elements to be created
!     nnodes         Number of nodes
!     ntype          type of spline curve
!     srest          rest distance of previous part
!     tauh           element length to be used to end up in userpoint
!     work           work array to store the arrays
!                    SOLVE ( length: 4n * ndim )
!                    ARC   ( length:  n        )
!                    AMAT  ( length: 3n        )
!                    RHS   ( length:  n * ndim )
!     x1             x-coordinate
!     y1             y-coordinate
!     z1             z-coordinate
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inisetnewref

!     ERCHKINT       Check if integer is between two values
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     INIRMREF       Remove entry iref from the set of reference numbers used
!     INISETNEWREF   Create new reference number
!     MSHC02         Generate "open" spline
!     MSHC03         Determine coordinates points on generated spline
!     MSHC04         Generate "closed" spline
!     PRININ         print 1d integer array
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     139   curve number has incorrect value
!    1669   file does not exist
!    2441   Array irelnodes position 1 is not 1
!    2706   error in opening file
!    2910   Not enough space
!    2911   Not possible to deallocate
!    3011   irelnodes(lenirelnodes)) # n
!    3012   irelnodes(i+1)<=irelnodes(i)
!    3013   number of elements incorrect
!    3014   sequence number found more than once
!    3015   sequence number out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Determine type of spline and make reservations in work array
!     Dependent on type of spline call generating routine, &
!     if open spline call MSHC02
!     else if closed spline call MSHC04
!     Determine points on the spline generated (MSHC03)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      debug = .false.
      call eropen ( 'mshframecur' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshframecur'
         write(irefwr,1) 'icurnr', icurnr
         call prinin ( iinput, iinput(2)+3, 'iinput' )
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Create file name and open file

      filename = 'framecurve_'
      if ( icurnr<=0 .or. icurnr>=1000000 ) then

!     --- Curve number smaller than 1, error

         call errint ( icurnr, 1 )
         call errint ( 1000000, 2 )
         call errsub ( 139, 2, 0, 0 )
         go to 1000

      end if  ! ( icurnr<=0  )
      lenirelnodes = iinput(2)  ! length of array irelnodes

      h = icurnr+1e-6     ! h = icurnr + eps
      i = int(log10(h))+1 ! i defines number of digits
      lenname = 10+i      ! length of name
      select case (i)
      case (1)

!     --- Curve number between 1 and 9

         write(filename(11:lenname),100) icurnr
100      format(i1)

      case(2)

!     --- Curve number between 10 and 99

         write(filename(11:lenname),110) icurnr
110      format(i2)

      case(3)

!     --- Curve number between 100 and 999

         write(filename(11:lenname),120) icurnr
120      format(i3)

      case(4)

!     --- Curve number between 1000 and 9999

         write(filename(11:lenname),130) icurnr
130      format(i4)

      case(5)

!     --- Curve number between 10000 and 99999

         write(filename(11:lenname),140) icurnr
140      format(i5)

      case(6)

!     --- Curve number between 100000 and 999999

         write(filename(11:lenname),150) icurnr
150      format(i6)

      end select  ! case (i)

!     --- Check if file exists

      inquire ( file=filename, exist = check )
      if ( .not. check ) then

!     --- Error: file does not exist

         call errchr ( filename, 1 )
         call errsub ( 1669, 0, 0, 1 )
         go to 1000

      end if  ! ( .not. fileexist )
      iref = inisetnewref()  ! reference number of file
      open ( unit = iref, file = filename, iostat = ios )
      if ( ios/=0 ) then

!     --- error in opening file

         call errint ( ios, 1 )
         call errchr ( filename, 1 )
         call errsub ( 2706, 1, 0, 1 )
         go to 1000

      end if

!     --- Read number of nodes, type of curve and degree of helpspline
!         and check whether values are ok

      read(iref,*) nnodes, ntype, ndegree

!     --- Check usersubmitted values

      call erchkint ( nnodes,  1, 10000, 'number of nodes' )
      call erchkint ( ntype,   0,     1, 'type of curve'   )
      call erchkint ( ndegree, 0,     0, 'degree of fitting curve')
      if ( ierror/=0 ) go to 1000

!     --- Second step: determine all helpvalues to be used for the construction
!         of the curve

      allocate ( helpusp(nnodes),  helppoints(4,nnodes), stat = error )

      if ( error/=0 ) then

!     --- error#0, so not enough space

         call errint ( 4*nnodes, 1 )
         call errint ( error, 2 )
         call errchr ( 'helppoints', 1 )
         call errsub ( 2910, 2, 0, 1 )
         goto 1000

      end if  ! ( error/= 0 )

!     --- Run through file and read all help nodes of framecurve

      helpusp = 0
      do i = 1, nnodes

         read(iref,*) j, x1, y1, z1, iusp, coarse

         if ( j<1 .or. j>nnodes ) then

!        --- node number out of range

            call errint ( icurnr, 1 )
            call errint ( i, 2 )
            call errint ( j, 3 )
            call errint ( nnodes, 4 )
            call errchr ( filename, 1 )
            call errsub ( 3015, 4, 0, 1 )
            j = i  ! to prevent errors in next line

         end if  ! ( j<1 .or. j>nnodes ) then

         if ( helpusp(j)/=0 ) then

!        --- node number found more than once

            call errint ( icurnr, 1 )
            call errint ( i, 2 )
            call errint ( j, 3 )
            call errchr ( filename, 1 )
            call errsub ( 3014, 3, 0, 1 )

         end if

         helppoints(1,j) = x1
         helppoints(2,j) = y1
         helppoints(3,j) = z1

         helppoints(4,j) = coarse

         helpusp(j) = iusp

      end do
      if ( error/=0 ) go to 1000

      if ( debug ) then
         do i = 1 , nnodes
            write(irefwr,*) (helppoints(j,i),j=1,4),helpusp(i)
         end do
      end if

!     --- Determine coordinates of nodes and fill array curves:

      srest  = 0d0

!     --- First node is starting node:

      curves(1,1) = helppoints(1,1)
      curves(2,1) = helppoints(2,1)
      curves(3,1) = helppoints(3,1)

      itau = 1

!     --- Find number of elements (initialize)

      nhelm = 0

!     --- Fill array irelnodes simultaneously:

      lnp            = 1
      irelnodes(lnp) = 1

      do i = 1 , nnodes - 1

         x1 = helppoints(1,i)
         y1 = helppoints(2,i)
         z1 = helppoints(3,i)

         dx = helppoints(1,i+1) - x1
         dy = helppoints(2,i+1) - y1
         dz = helppoints(3,i+1) - z1

         dist = sqrt( dx*dx + dy*dy +dz*dz )

!        --- Use avarage coarseness value:

         coarse = cunit * (helppoints(4,i) + helppoints(4,i+1))/2d0

!        --- Add and check number of elements on this part:

         if ( helpusp(i+1)==0 ) then

!        --- Local node is only helpnode, place nodes between:

             nh    = int((srest+dist)/coarse)
             nhelm = nhelm + nh

!            --- Place nodes:

             do j = 1, nh

                itau = itau + ishape

                fact = (j * coarse - srest)/dist

                curves(1,itau) = x1 + fact*dx
                curves(2,itau) = y1 + fact*dy
                curves(3,itau) = z1 + fact*dz

                if ( debug ) write(irefwr,'(i6,3f14.6)') &
                   'pnt',itau, (curves(ip,itau),ip=1,3)

             end do

!            --- Compute srest (i.e. remaining Euclidian distance:

             srest = srest + dist - nh * coarse

         else if ( helpusp(i+1)/= 0 ) then

!        --- End node is userpoint, hence point has to be used:

            nh = int((srest+dist)/coarse+0.5d0)
            nhelm = nhelm + nh

!           --- Determine tau-step ("adjust coarse locally")

            tauh = (srest+dist)/nh

            do j = 1, nh

               fact = (j*tauh - srest)/dist

               itau = itau + ishape

               curves(1,itau) = x1 + fact * dx
               curves(2,itau) = y1 + fact * dy
               curves(3,itau) = z1 + fact * dz

               if ( debug ) write(irefwr,'(i6,3f14.6)') &
                  'crd point',itau,(curves(ip,itau),ip=1,3)

            end do

            srest = 0

            lnp  = lnp + 1
            irelnodes(lnp) = itau

         end if

      end do

!     --- Check array length of irelnodes

      call inichkarlen ( lnp, lenirelnodes, 'irelnodes' )
      if ( ierror/=0 ) go to 1000

!     --- Check the contents of array irelnodes

      if ( irelnodes(1)/=1 ) then

!     --- Position 1 is not 1

         call errint ( icurnr , 1 )
         call errint ( irelnodes(1), 2 )
         call errchr ( filename, 1 )
         call errsub ( 2441, 2, 0, 1 )

      end if  ! ( irelnodes(1)/=1 )

      do i = 2, lenirelnodes

         if ( irelnodes(i)<=irelnodes(i-1) ) then

!        --- Contents of irelnodes not increasing

            call errint ( icurnr , 1 )
            call errint ( i, 2 )
            call errint ( irelnodes(i), 3 )
            call errint ( i-1, 4 )
            call errint ( irelnodes(i-1), 5 )
            call errchr ( filename, 1 )
            call errsub ( 3012, 5, 0, 1 )

         end if  ! ( irelnodes(i)<=irelnodes(i-1) )

      end do  ! i = 2, lenirelnodes

      if ( irelnodes(lenirelnodes)/=n ) then

!     --- Last position is not n

         call errint ( icurnr, 1 )
         call errint ( lenirelnodes, 2 )
         call errint ( irelnodes(lenirelnodes), 3 )
         call errint ( n, 4 )
         call errchr ( filename, 1 )
         call errsub ( 3011, 4, 0, 1 )

      end if  ! ( irelnodes(lenirelnodes)/=n )

!     --- Check number of elements

      if ( ishape/=((n - 1 ) / nhelm) ) then

!     --- Wrong number of elements

         call errint ( icurnr, 1 )
         call errint ( ishape, 2 )
         call errint ( n, 3 )
         call errint ( nhelm, 4 )
         call errsub ( 3013, 4, 0, 0 )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Arrays curves and irelnodes have been filled
!         Check whether "mid"points have to be added

      if ( ishape>1 ) then

!     --- ishape>1, Fill points in between according to isubd
!         The relative division is stored in work

         dtau = 1d0 / (ishape * 1d0)

         do i = 1 , n-1 , ishape

            dx = curves(1,i+ishape) - curves(1,i)
            dy = curves(2,i+ishape) - curves(2,i)
            dz = curves(3,i+ishape) - curves(3,i)

            do j = 1, ishape-1
               curves(1,i+j) = curves(1,i) + dtau * dx
               curves(2,i+j) = curves(2,i) + dtau * dy
               curves(3,i+j) = curves(3,i) + dtau * dz
            end do

         end do

      end if

!     --- Close file and deallocate helparrays:

      if ( debug ) then
         write(irefwr,*) 'Coordinates nodes curve',icurnr

         do i=1, n
            write(irefwr,'(i6,3f14.5)') i,(curves(ip,i),ip=1,3)
         end do
         write(irefwr,*) 'Check results of curve',icurnr

      end if

      close ( unit = iref )
      call inirmref ( iref )
      if ( error==0 ) deallocate ( helpusp,    stat = error )
      if ( error==0 ) deallocate ( helppoints, stat = error )

      if ( error/=0 ) then

!     --- error#0, errors in closing and/or deallocating

         call errint ( error, 1 )
         call errchr ( 'helpusp', 1 )
         call errsub ( 2911, 1, 0, 1 )
         go to 1000

      end if  ! ( error/= 0 )

!     --- Next part is for the future:
!              if splines with higher degree are needed:

      if ( ndegree<0 ) then

!     --- Read type of Spline and number of defining points

      itype = iinput(1)
      ndef  = lenirelnodes

      allocate ( work((5+5*ndim)*max(n,ndef)), stat = error )
      if ( error/=0 ) then

!     --- error#0, so not enough space

         call errint ( (5+5*ndim)*max(n,ndef), 1 )
         call errint ( error, 2 )
         call errchr ( 'work', 1 )
         call errsub ( 2910, 2, 0, 1 )
         !go to 900

      end if  ! ( error/= 0 )

      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshframecur'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( itype<9 ) then
         alpha = 1d0
      else if ( itype<19 ) then
         alpha = 0d0
         itype = itype - 10
      else if ( itype<29 ) then
         alpha = 0.5d0
         itype = itype - 20
      else
         alpha = rinput(1)
         itype = itype - 30
      end if

!     --- Make reservations in WORK for the arrays
!         Array SOLVE , length : ndim * 4 * n real positions

      ipsolv = 1

!     --- Array ARC , length : n positions

      iparc = 1 +  4 * ndim * ndef

!     --- Array AMAT , length 3 * n positions

      ipamat = iparc + ndef

!     --- Array RHS , each ndim * n positions

      iprhs1 = ipamat + 3 * ndef
      iprhs2 = iprhs1 + ndef
      iprhs3 = iprhs2 + ndef

!     --- Compute spline coefficients

      if ( itype/=3 ) then
        call mshc02 ( work(ipsolv), ndef, ndim, work(iparc), &
                      work(ipamat), work(iprhs1), work(iprhs2), &
                      work(iprhs3), iinput, coor, alpha, icurnr )
      else
        call mshc04 ( work(ipsolv), ndef, ndim, work(iparc), &
                      work(ipamat), work(iprhs1), work(iprhs2), &
                      work(iprhs3), iinput, coor, alpha )
      end if

!     --- Determine points on curve

      call mshc03( work(ipsolv), ndef, ndim, work(iparc), tau, &
                   curves, n, alpha )
      if ( n==ndef ) then
         curvelen = 0d0
         diff = 0d0
         if ( ndim==1 ) then
            do i = 2, n
               j = iinput(2+i)
               curvelen = abs(curves(1,i) - curves(1,i-1))+curvelen
               diff = max(diff,abs(curves(1,i)-coor(1,j)))
            end do
         else if ( ndim==2 ) then
            do i = 2, n
               j = iinput(2+i)
               curvelen = sqrt((curves(1,i) - curves(1,i-1))**2 + &
                               (curves(2,i) - curves(2,i-1))**2)+ &
                          curvelen
               diff = max(sqrt((curves(1,i) - coor(1,j))**2 + &
                               (curves(2,i) - coor(2,j))**2),diff)
            end do
         else
            do i = 2, n
               j = iinput(2+i)
               curvelen = sqrt((curves(1,i) - curves(1,i-1))**2 + &
                               (curves(2,i) - curves(2,i-1))**2 + &
                               (curves(3,i) - curves(3,i-1))**2)+ &
                          curvelen
               diff = max(sqrt((curves(1,i) - coor(1,j))**2 + &
                               (curves(2,i) - coor(2,j))**2 + &
                               (curves(3,i) - coor(3,j))**2),diff)
            end do
         end if
         if ( diff<=curvelen*1d-4 ) then

!        --- Not much difference between original and new co-ordinates
!            Reset curve

            if ( ndim==1 ) then
               do i = 2, n
                  j = iinput(2+i)
                  curves(1,i)=coor(1,j)
               end do
            else if ( ndim==2 ) then
               do i = 2, n
                  j = iinput(2+i)
                  curves(1,i)=coor(1,j)
                  curves(2,i)=coor(2,j)
               end do
            else
               do i = 2, n
                  j = iinput(2+i)
                  curves(1,i)=coor(1,j)
                  curves(2,i)=coor(2,j)
                  curves(3,i)=coor(3,j)
               end do
            end if
         end if

      end if

!     --- Free space of work array:

      deallocate ( work, stat = error )
      if ( error/=0 ) then

!     --- error#0, problem with deallocate

         call errint ( error, 1 )
         call errchr ( 'work', 1 )
         call errsub ( 2911, 1, 0, 1 )

      end if  ! ( error/=0 )

      end if  ! end of future part

1000  if ( debug ) then

!     --- Debug information

         call prinin ( iinput, n, 'iinput' )
         call prinrl1 ( curves, n, ndim, 'curves' )
         write(irefwr,*) 'End mshframecur'

      end if  ! ( debug )
      call erclos ( 'mshframecur' )

      end
