      subroutine mshlevelint2d ( nvert, inodes, iverts, rverts, &
                                 nintersect, iwork1, levelset )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 26-09-2012 Extra debug
!        version  1.1  date 20-07-2011 Fortran 90
!        version  1.0  date 08-04-2004
!
!   copyright (c) 2004-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find the intersection of all sides in the element with
!     the level set function value 0
! **********************************************************************
!
!                       KEYWORDS
!
!     intersection
!     line
!     element
!     level_set
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nvert, inodes(3,*), iwork1(*), iverts(*), nintersect
      double precision rverts(*), levelset(*)

!     inodes         i    Array of length 3*nvert containing the node numbers
!                         of the edges as well as the edge number
!                         inodes(1,i) = first node of edge i
!                         inodes(2,i) = last node of edge i
!                         inodes(3,i) = edge number
!     iverts         o    Array of length nintersect containing the edge
!                         numbers of the edges that interect the obstacle
!     iwork1         i    Integer work array of length npoint
!                         iwork1(i) = 1, means that node i is on the
!     levelset       i    Array of length npoint containing the value of the
!                         level set function in each node
!     nintersect     o    Number of intersections found
!     nvert          i    Number of edges to be considered
!     rverts         o    Contains the factors corresponding to the intersection
!                         points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inode1, inode2, ivert
      double precision fact1

!     fact1          Defines the factor for the intersection point
!                    The intersection point on the face is defined by
!                    x1 + fact1 * (x2-x1)
!     inode1         First node in edge
!     inode2         Second node in edge
!     ivert          Loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshlevelint2d' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshlevelint2d'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- nintersect := 0

      nintersect = 0

!     --- for ivert := 1 (1) nvert do

      do ivert = 1, nvert

         inode1 = inodes(1,ivert)  ! first node on vertex
         inode2 = inodes(2,ivert)  ! second node on vertex
         if ( debug ) then
            write(irefwr,1) 'inode1, inode2', inode1, inode2
            write(irefwr,1) 'iwork1(inode1/2)', iwork1(inode1), &
                             iwork1(inode2)
         end if  ! ( debug )
         if ( iwork1(inode1)==0 .and. iwork1(inode2)==0 ) then

!        --- Both nodes are not marked as level set equal 0

            if ( levelset(inode1)*levelset(inode2)<0d0 ) then

!           --- intersection found, because level set functions
!               in end points has different signs
!               Compute the factor f for which level set becomes zero
!               with the formula x = x_1 + f (x_2-x_1)
!               f = phi_1 / ( phi_1-phi_2 )

               fact1 = levelset(inode1) / &
                  (levelset(inode1)-levelset(inode2))
               nintersect = nintersect+1
               iverts(nintersect) = inodes(3,ivert)
               rverts(nintersect) = fact1

               if ( debug ) then
                  write(irefwr,2) 'fact1', fact1
                  write(irefwr,2) 'levelset(inode1/2)', &
                                   levelset(inode1), levelset(inode2)
               end if  ! ( debug )


            end if  ! ( levelset(inode1)*levelset(inode2)<0d0 )

         end if  ! ( iwork1(inode)==0 .and. iwork1(inode2)==0 )

      end do  ! ivert = 1, nvert

1000  call erclos ( 'mshlevelint2d' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshlevelint2d'

      end if

      end
