      subroutine mshmaprect ( isol, isolloc, rinput, iinput, icurvs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 18-04-2005 Extra parameters iinput, icurvs
!        version  1.0  date 02-08-2004
!
!   copyright (c) 2004-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Map the solution defined on an arbitrary mesh onto a rectangular grid
!     A new rectangular mesh and problem definition are created
! **********************************************************************
!
!                       KEYWORDS
!
!     map
!     solution_vector
!     rectangle
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemnsub
      use sepmodulemain
      use sepmodulekmesh
      use sepmoduleplot
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isol, isolloc, iinput(*), icurvs(*)
      double precision rinput(*)

!     icurvs         i    Integer array containing the user points, curve or
!                         surface numbers that must be printed.
!                         Pos. 1 indicates how array icurvs is filled.
!                         Possible values:
!                          0   only one curve is used. The curve number is
!                              stored in icurvs(2)
!                         -1   The solution is printed along the curves
!                              icrv1 to icrv2. icrv1 is stored in icurvs(2) and
!                              icrv2 in icurvs(3).
!                         n>0  The curve numbers numbers are stored in
!                              icurvs(2) to icurvs(n+1). That sequence is used.
!     iinput         i    Integer input array with the following contents
!                         1:  idim
!     isol           i    Standard SEPRAN array containing information of the
!                         solution array
!     isolloc        i    ?
!     rinput         i    Real input array with the following contents
!                         1:  dx
!                         2:  dy
!                         3:  dz
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nx(3), i, iseqkm1, iseqkm2
      double precision dx(3), origin(3)
      type (mapping) :: maploc

!     dx             length of vector in x-direction
!     i              Counting variable
!     iseqkm1        Sequence number with respect to km of first mesh
!     iseqkm2        Sequence number with respect to km of second mesh
!     maploc         contains information of the mapping
!     nx             number of points in x direction
!     origin         Contains the coordinates of the origin of the circle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHBACKGRID    Create mesh for background rectangular grid
!     PRININ         print 1d integer array
!     PROBACKGRID    Fill array kprob for rectangular grid with constant number
!                    of unknowns per point
!     SEPCREAIBUF    Create permanent space
!     SEPCRMESHINFO  Create permanent space for mesh
!     SEPINTPOLMS    Interpolate solution from one mesh to another
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1393   Not constant number of degrees of freedom not available in intmsh.
! **********************************************************************
!
!                       PSEUDO CODE
!
!     First initialize kmesh and kprob
!     Create rectangular mesh
!     Fill kprob corresponding to rectangular mesh
!     Fill isolloc as far as possible
!     Copy solution from old mesh onto new mesh
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshmaprect' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshmaprect'
         call prinin ( isol, 5, 'isol' )
         call prinin ( kprob, 60, 'kprob' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseqkm1 = iseqkm
      kmesh => kmesh2d(maxmesh+1)
      kprob => kprob2d(maxmesh+1)
      call sepclear ( maploc )

!     --- First initialize  and kprob

      call sepcrmeshinfo ( kmesh )
      iseqkm2 = iseqkm
      call sepcrprobinfo ( kprob, 1 )
      kprobpart => kp(iseqkp)%kprobsub(iprob)
      kprobloc => kprobpart%kprobloc

!     --- Create rectangular mesh

      dx(1) = rinput(1)
      dx(2) = dx(1)
      dx(3) = dx(1)

      if ( ndim>1 ) dx(2) = rinput(2)
      if ( ndim>2 ) dx(3) = rinput(3)

      nx(1) = max(nint((xmax-xmin)/dx(1)),0)+1
      nx(2) = max(nint((ymax-ymin)/dx(2)),0)+1
      nx(3) = max(nint((zmax-zmin)/dx(3)),0)+1

      origin(1) = xmin
      origin(2) = ymin
      origin(3) = zmin

      call mshbackgrid ( nx, origin, dx, ndim )

!     --- Fill kprob corresponding to rectangular mesh
!         First we try to depict the number of unknowns per point

      nusol = ks(isol)%nusol
      nunkp = nusol/npoint
      if ( nusol/=nunkp*npoint ) then

!     --- Number of degrees of freedom per point not constant has not yet
!         been implemented

         call errint ( ks(isol)%typevec, 1 )
         call errchr ( 'mshmaprect', 1 )
         call errsub ( 1393, 1, 0, 1 )
         go to 1000

      else

!     --- Number of degrees of freedom per point constant

         nusol = nunkp * km(iseqkm2)%kmesh(8)

      end if  ! ( nusol/=nunkp*/npoint )

      call probackgrid

!     --- Fill isolloc as far as possible

      isolloc = 0

!     --- Copy solution from old mesh onto new mesh

      iinmap(1) = 5   ! number of entries filled
      iinmap(2) = 0   ! infmap (no mapping given_
      iinmap(3) = 0   ! minmax (free interpolation)
      iinmap(4) = 11  ! ichoice: new solution is of type 110
                      ! points outside the region are not interpolated
      if ( iinput(1)==3 ) then

!     --- idim = 3, set iinmap(5) = 1

         iinmap(5) = 1  ! idim for intpolmbf

      else

!     --- idim # 3, set iinmap(5) = 0

         iinmap(5) = 0  ! idim for intpolmbf

      end if  ! ( input(67)==3 )
      call sepintpolms ( meshnr, maxmesh+1, iinmap, &
                         isol, isolloc, maploc, icurvs )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshmaprect'

      end if
      call erclos ( 'mshmaprect' )

      end
