      subroutine mshobstcheck
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 06-12-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check consistency of nodes in array ipointobst and elements in array
!     ielemobst
! **********************************************************************
!
!                       KEYWORDS
!
!     obstacle
!     check
!     node
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iobst
      integer, allocatable :: imarkelem(:), imarknode(:)

!     imarkelem      Marker array of length nelem
!                    imarkelem(i) = 0: element i not inside obstacle
!                    imarkelem(i) = 1: element i inside obstacle
!     imarknode      Marker array of length npoint
!                    imarknode must have been initialized by 0
!                    imarknode(i) = 0: node i not inside obstacle
!                    imarknode(i) = 1: node i inside obstacle
!     iobst          Obstacle sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MSHOBSTCHECK1  Check consistency of nodes in array ipointobst and
!                    elements in array ielemobst
!     PRININNONZERO  Print the non-zero contents of an integer array
!     PRMARKCLEARELM Clear array imark for all elements that are completely
!                    inside the obstacle
!     PRMARKSKIPELM  Mark array imark for all elements that are completely
!                    inside the obstacle
!     PRMARKSKIPNODS Mark array imarknode for all nodes that are completely
!                    inside the obstacle
!     SEPFILLKMESHD  Fill kmesh part d
!     SEPPRINIPOBST  Print the contents of array ipointobst
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For all obstacles iobst do
!        Find set of elements that are completely inside the obstacle
!        Find set of all internal nodes in this set
!        Compare the set with the contents of array ipointobst
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobstcheck' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobstcheck'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- activate arrays

      if ( kelmd==0 ) call sepfillkmeshd
      if ( ierror/=0 ) go to 1000
      if ( debug ) call sepprinipobst ( ipointobst )

!     --- Create work arrays
!         imarkelms(1:nelem)
!         imarknodes(1:npoint)

      allocate ( imarkelem(nelem), imarknode(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint+nelem, 'imarkelem' )
      if ( ierror/=0 ) go to 1000

!     --- Clear array imark (is used to mark elements and nodes
!                            inside the obstacle)

      imarkelem = 0
      imarknode = 0
      if ( ierror/=0 ) go to 1000

!     --- For all obstacles iobst do

      do iobst = 1, numobst

!     --- Find set of elements that are completely inside the obstacle
!         This is doen by setting iwork(ielem) = 1, for these elements

         call prmarkskipelm ( imarkelem, ielemobst, iobst )
         if ( ierror/=0 ) go to 1000

!        --- Find the set of all internal nodes in this set

         call prmarkskipnods ( imarknode, imarkelem )
         if ( ierror/=0 ) go to 1000
         if ( debug ) then

!        --- Debug information

            write(irefwr,100) 'iobst ', iobst
100         format ( a, 10i6 )
            call prininnonzero ( imarknode, npoint, 'internal nodes' )

         end if  ! ( debug )

!        --- Clear array imarkelem

         call prmarkclearelm ( imarkelem, ielemobst, iobst )
         if ( ierror/=0 ) go to 1000

!        --- Compare the set with the contents of array ipointobst

         call mshobstcheck1 ( imarknode, iobst, ipointobst, npoint )
         if ( ierror/=0 ) go to 1000

      end do  ! iobst = 1, numobst

      deallocate ( imarkelem, imarknode, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'imarkelem' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshobstcheck'

      end if
      call erclos ( 'mshobstcheck' )

      end subroutine mshobstcheck
