      subroutine mshobstms1d01 ( kmeshcnew, coornew, meshobst, &
                                 ipointobstnew, ielemobstnew, &
                                 iinput, nmap )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 22-11-2011 Use mshfindlowsur
!        version  1.0  date 11-10-2011
!
!   copyright (c) 2011-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create a new mesh from a fixed mesh with an obstacle
!     crossing the mesh (1d)
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     subdivision
!     1d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemain
      use sepmodulegen
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshcnew(*), nmap, ipointobstnew(*), ielemobstnew(*), &
              meshobst, iinput(*)
      double precision coornew(*)

!     coornew        o    Array containing the new coordinates of the nodes
!     ielemobstnew   o    New array ielemobst
!     iinput         o    Array to pass some constants to mshobstms
!                         Contents
!                         1:  length of new array kmeshc
!                         2:  length of new array ipointobst
!                         3:  length of new array ielemobst
!     ipointobstnew  o    New array ipointobst
!     kmeshcnew      o    New kmesh part c
!     meshobst      i/o   sequence number of new array kmesh
!     nmap           i    length of first parameter of imap and rmap
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielem, ielgrp, inpelm, inode, &
              irelem, ielemnew, npointnew, i, ikelmc, ikelmcnew, &
              ilow, ielemtop, ielembot, icurnr, iseqkm1, iseqkm2, &
              inelem2, node1, node2, nodebegin, nodeend
      double precision fact
      integer, allocatable :: iwork(:,:), iworkpoint(:,:)
      integer, pointer :: kmeshobst, kmeshtobst(:,:)

!     fact           multiplication factor
!     i              Counting variable
!     icurnr         Curve sequence number
!     ielem          Element number
!     ielembot       Element sequence number of element with lowest number in
!                    curve icurnr
!     ielemnew       Sequence number of element in new mesh
!     ielemtop       Element sequence number of element with largest number in
!                    curve icurnr
!     ielgrp         element group number
!     ikelmc         position in kmesh part c
!     ikelmcnew      Starting address of next element in kmeshcnew
!     ilow           help variable to compute smallest element number
!     inelem2        Help variable for determination of number of elements
!     inode          Node sequence number
!     inpelm         number of points per element
!     irelem         Relative element sequence number with respect to
!                    element group
!     iseqkm1        Sequence number with respect to km of first mesh
!     iseqkm2        Sequence number with respect to km of second mesh
!     iwork          integer work array of length nelem*2
!                    iwork(ielem,1) contains the starting address of
!                    element ielem in array icrossobst
!                    If 0, the element is not present in icrossobst
!                    iwork(ielem,2) contains the corresponding obstacle
!                    sequence number iobst
!     iworkpoint     Work array of length npointnew*2
!                    This array is filled as follows:
!                    iworkpoint(i,1) = 0 if node i outside obstacle
!                    iworkpoint(i,1) = 1 if node i inside obstacle
!                    iworkpoint(i,1) = 2 if node i inside obstacle (near
!                                        boundary)
!                    iworkpoint(i,1) = 3 if node i on obstacle
!                    iworkpoint(i,2) contains the corresponding obstacle
!                    sequence number iobst
!     kmeshobst      kmesh with respect to obstacle
!     kmeshtobst     kmesht with respect to obstacle
!     node1          Node of face
!     node2          Node of face
!     nodebegin      Node at the begin of a crossed edge
!     nodeend        Node at the end of a crossed edge
!     npointnew      Sequence number of last generated node
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERTRACE        Prints trace back of subroutines
!     INICOPINT2D    Copy two-dimensional integer array
!     INICOPREAL2D   Copy two-dimensional real array
!     MSHOBSTMS022   Copy part of kmesh to kmeshnew n case of obstacles
!     MSHOBSTMS023   Fill iworkpoint with information about nodal points in and
!                    outside obstacle and corresponding obstacle sequence
!                    numbers
!     MSHOBSTMS024   Adapt the arrays ipointobstnew and ielemobstnew for new
!                    mesh
!     MSHOBSTMS1D02  Clear array iwork (1:nelem) and set
!                    iwork(i) = starting address in
!                    iobstcross if element i intersects obstacle iobst
!     PR0003         Print KMESH parts c and g
!     PRINIELOBST    Print the contents of array ielemobst
!     PRININ2        Print an integer two-dimensional array
!     PRINRL2        Print a real two-dimensional array
!     SEPFINDLOWSUR  Find surface (curve) with lowest non-zero entry in kmesht
!     SEPPRINIPOBST  Print the contents of array ipointobst
!     SEPPRINKMESHC  print kmesh part c
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Initialize some parameters
!     Copy part of kmesh to meshobst
!     Copy coordinates from coor to coornew
!     Clear array iwork
!     Set iwork to 1 for elements stored in icrossobst
!     Fill iworkpoint with information about nodal points in and outside obst
!     Fill iworkelem with information about elements in and outside obstacle
!     npointnew := npoint
!     ielem := 0
!     ielemnew := 0
!     For ielgrp := 1 (1) nelgrp do
!        inelem := number of elements in group ielgrp
!        inpelm := number of nodes per element in group ielgrp
!        For irelem := 1 (1) inelem do
!           ielem := ielem+1
!           ielemnew := ielemnew+1
!           if ( ielem completely in obstacle or completely outside obstacle )
!              then
!              copy element from kmeshc to kmeshcnew
!           else ( ielem is intersected by obstacle )
!              create new elements
!              copy these elements to kmeshcnew
!              Raise ielemnew
!     adapt ipointobstnew and ielemobstnew
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobstms1d01' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobstms1d01'
         call ertrace
         write(irefwr,1) 'nmap', nmap
         write(irefwr,2) 'accuracy_obstacle', accuracy_obstacle
         call sepprinkmeshc
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )
  4      format ( a, 1x, 2i6, d12.3)

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseqkm1 = iseqkm

      do i = 1, npoint
         imap(i,2) = i
      end do
      imap(1:npoint,1) = 1
      imap(1:npoint,3) = 0
      rmap(1:npoint,1) = 1d0
      rmap(1:npoint,2) = 0d0

      kmeshobst => kmesh2d(meshobst)

!     --- allocate work arrays

      allocate ( iwork(nelem,2), iworkpoint(nmap,2), stat = error )
      if ( error/=0 ) call eralloc ( error, inelem, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Copy part of kmesh to kmeshobst
!         Copy coordinates from coor to coornew

      call mshobstms022 ( kmeshobst, iseqkm2, coornew )
      inelem2 = 0

      if ( debug ) &
         write(irefwr,1) 'iseqkm2, kmeshobst', iseqkm2, kmeshobst

!     --- Clear array iwork
!         Set iwork to 1 for elements stored in icrossobst

      call mshobstms1d02 ( iwork, nelem, numobst, icrossobst )

!     --- Fill iworkpoint with information about nodal points in and outside
!         obstacle and corresponding obstacle sequence numbers

      call mshobstms023 ( npoint, iworkpoint, nmap, ipointobst, &
                          numobst )

!     --- npointnew := npoint
!         ielem := 0
!         ielemnew := 0

      npointnew = npoint
      ielem = 0
      ielemnew = 0

!     --- For ielgrp := 1 (1) nelgrp do

      ikelmc = 0
      ikelmcnew = 0

!     --- Find curve with lowest positive element number
!         This is necessary to fill kmesh part t

      call sepfindlowsur ( ncurvs, ielembot, ielemtop, icurnr )

      kmeshtobst => km(iseqkm2)%kmesht

      do ielgrp = 1, nelgrp

!     --- inelem := number of elements in group ielgrp
!         inpelm := number of nodes per element in group ielgrp

         inpelm = Topgrp(ielgrp)%inpelm
         inelem = Topgrp(ielgrp)%inelem
         ishape = kmeshf(ielgrp)
         if ( debug ) &
             write(irefwr,1) 'ielgrp, inpelm, ishape, inelem', &
                              ielgrp, inpelm, ishape, inelem

!        --- For irelem := 1 (1) inelem do

         do irelem = 1, inelem

!        --- ielem := ielem+1
!            ielemnew := ielemnew+1

            ielem = ielem+1
            ielemnew = ielemnew+1
            if ( ielem==ielembot ) then

!           --- Lowest element of curve, refill kmesht

               kmeshtobst(1,icurnr) = ielemnew

            end if
            if ( debug ) then
               write(irefwr,1) 'ielem, ielemnew', ielem, ielemnew
               write(irefwr,1) 'iwork(ielem,1)', iwork(ielem,1)
               write(irefwr,1) 'ikelmc, ikelmcnew', ikelmc, ikelmcnew
            end if  ! ( debug )

            if ( iwork(ielem,1)==0 .or. ishape<1 ) then

!           --- ielem completely in obstacle or completely outside obstacle
!               copy element from kmeshc to kmeshcnew

               kmeshcnew(ikelmcnew+1:ikelmcnew+inpelm) = &
                  kmeshc(ikelmc+1:ikelmc+inpelm)
               ikelmc = ikelmc+inpelm
               ikelmcnew = ikelmcnew+inpelm

            else

!           --- ielem is intersected by obstacle
!               create new elements
!               copy these elements to kmeshcnew
!               Raise ielemnew

               if ( ishape/=1 ) then

!              --- shape number not 1, not yet implemented

                  call errint ( ishape, 1 )
                  call errsub ( 800, 1, 0, 0 )
                  go to 1000

               end if

               node1 = kmeshc(ikelmc+1)
               node2 = kmeshc(ikelmc+2)
               fact = rcrossobst(iwork(ielem,2))

               if ( debug ) then
                  write(irefwr,1) 'node1, node2', node1, node2
                  write(irefwr,2) 'fact', fact
               end if  ! ( debug )
               if ( fact>=1d0-accuracy_obstacle ) then

!              --- i-th cross section close to begin point of edge
!                  mark node corresponding to begin point as on boundary

                  if ( debug ) &
                     write(irefwr,*) 'fact>1d0-accuracy_obstacle'
                  nodebegin = node2
                  inode = node2
                  iworkpoint(nodebegin,1) = 3
                  iworkpoint(nodebegin,2) = iwork(ielem,2)  ! iobst
                  coornew(nodebegin) = (1d0-fact)*coornew(node1) &
                                       +fact*coornew(node2)
                  if ( debug ) write(irefwr,4) 'nodebegin', nodebegin
                  kmeshcnew(ikelmcnew+1:ikelmcnew+inpelm) = &
                     kmeshc(ikelmc+1:ikelmc+inpelm)
                  ikelmc = ikelmc+inpelm
                  ikelmcnew = ikelmcnew+inpelm

               else if ( fact<=accuracy_obstacle ) then

!              --- i-th cross section close to end point of edge
!                  mark corresponding node as on boundary

                  if ( debug ) write(irefwr,*) 'fact<accuracy_obstacle'
                  nodeend = node1
                  inode = node1
                  iworkpoint(nodeend,1) = 3
                  iworkpoint(nodeend,2) = iwork(ielem,2)  ! iobst
                  coornew(nodeend) = (1d0-fact)*coornew(node1) &
                                       +fact*coornew(node2)
                  if ( debug ) write(irefwr,4) 'nodeend', nodeend
                  kmeshcnew(ikelmcnew+1:ikelmcnew+inpelm) = &
                     kmeshc(ikelmc+1:ikelmc+inpelm)
                  ikelmc = ikelmc+inpelm
                  ikelmcnew = ikelmcnew+inpelm

               else

!              --- i-th element is intersected
!                  Store two elements in kmeshcnew
!                  node1 - npointnew - node2

                  npointnew = npointnew+1
                  inode = npointnew
                  kmeshcnew(ikelmcnew+1) = node1
                  kmeshcnew(ikelmcnew+2) = npointnew

                  kmeshcnew(ikelmcnew+3) = npointnew
                  kmeshcnew(ikelmcnew+4) = node2
                  ikelmcnew = ikelmcnew+4
                  coornew(npointnew) = (1d0-fact)*coornew(node1) &
                                         +fact*coornew(node2)
                  ielemnew = ielemnew+1

                  ikelmc = ikelmc+inpelm
                  if ( debug ) then
                      write(irefwr,*) 'ielem, irelem, npointnew ', &
                                       ielem, irelem, npointnew
                  end if  ! ( debug )
                  iworkpoint(npointnew,1) = 3
                  iworkpoint(npointnew,2) = iwork(ielem,2)  ! iobst

               end if  ! ( fact>1d0-eps )
               imap(inode,1) = 2
               imap(inode,2) = node1
               imap(inode,3) = node2
               rmap(inode,1) = 1d0-fact
               rmap(inode,2) = fact

            end if
            if ( ielem==ielemtop ) then

!           --- Top element of curve, refill kmesht

               kmeshtobst(2,icurnr) = ielemnew

!              --- Raise ielembot and ielemtop and recompute icurnr

               ilow = nelem
               do i = 1, ncurvs
                  if ( kmesht(1,i)<ilow .and. &
                       kmesht(1,i)>ielemtop ) then

!                 --- new curve found

                     icurnr = i
                     ilow = kmeshtobst(1,i)

                  end if
               end do
               ielembot = kmeshtobst(1,icurnr)
               ielemtop = kmeshtobst(2,icurnr)

            end if  ! ( iwork(ielem,1)==0 .or. ishape<1 )

         end do
         if ( kelmg>0 ) then

!        --- Raise kmesh part g

            km(iseqkm2)%kmeshg(ielgrp) = ielemnew-inelem2
            inelem2=ielemnew

         end if  ! ( kelmg>0 )

      end do ! end loop over element groups
      if ( debug ) then
         write(irefwr,1) 'kelmt, ikelmcnew', kelmt, ikelmcnew
         call pr0003 ( kmeshobst, kmeshcnew )
      end if  ! ( debug )

!     --- adapt ipointobstnew and ielemobstnew

      call mshobstms024 ( nmap, iworkpoint, &
                          ipointobstnew, npointnew, kmeshobst, &
                          ielemnew, kmeshcnew, iinput, ielemobstnew )

!     --- Fill iinput

      iinput(1) = ikelmcnew

      deallocate ( iwork, iworkpoint, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'ipointobst_new'
         iseqkm = iseqkm2
         call sepprinipobst ( ipointobstnew )
         call sepprinielobst ( ielemobstnew )
         call prinin2 ( imap, npointnew, 3, 'imap' )
         call prinrl2 ( rmap, npointnew, 2, 'rmap' )
         write(irefwr,*) 'End mshobstms1d01'

      end if  ! ( debug )
      call erclos ( 'mshobstms1d01' )

      end
