      subroutine mshobstms3dq6a ( tetnode, tetnodenew, edgenode, &
                                  isort, ledgenode, nelemnew, centroid, &
                                  midfacenodes, facenode, fluidnd )
! ======================================================================
!
!        programmer    Raoul van Loon
!        version  1.0  date 06-04-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subdivide a tetrahedron with three crossed edges which have one
!     single node in common into four tetrahedrons. Quadratic case
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     quadratic
!     tetrahedron
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer tetnode(15),tetnodenew(15,4), edgenode(5,6), isort(4), &
              ledgenode(2,3), nelemnew, midfacenodes(4), centroid(*), &
              facenode(4,4), fluidnd(*)

!     centroid       i    ?
!     edgenode       i    Array with all nodes on the crossed edge
!                         edgenode(1,1) first node on crossed edge
!                         edgenode(2,1) last node on crossed edge
!                         edgenode(3,1) new node on crossed edge
!     facenode       i    ?
!     fluidnd        i    Array containing the fluidnodes that lie on the
!                         obstacle
!     isort          i    if isort(i)>isort(j) then tetnode(i) is at least on
!                         the same number of crossed edges as tetnode(j)
!     ledgenode      i    for i=1,2 and 1<j< number of crossed edges
!                         ledgenode(i,j) is the local tetnode of edgenode(i,j), &
!                         in other words:
!                         edgnode(i,j) = tetnode(ledgenode(i,j))
!     midfacenodes   i    ?
!     nelemnew       o    Number of elements in new mesh
!     tetnode        i    Array of nodal point numbers of the corners of
!                         the original tetrahedron.
!     tetnodenew     o    Array of nodal point numbers of the corners of
!                         the new tetrahedrons.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, transtot(15,12), itran, facenodet(4,4), &
              trans(4), transi(4),transtemp, transposj, transposmax, &
              tetnodet(15), edgenodet(2,6), facetrans(4), &
              edgenodemin, edgenodemax, nodetwo
      integer pedgenodet(4,4)
      logical oddpairintchan, debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     edgenodemax    edge node with lowest node number
!     edgenodemin    edge node with largest node number
!     edgenodet      Array of global nodal point numbers of X-point on edge i
!                    after transformation
!     facenodet      ?
!     facetrans      ?
!     i              Counting variable
!     itran          ?
!     j              Counting variable
!     nodetwo        local tet number which should be transformed to tetnode 2
!     oddpairintchan if true the transformation is an odd number of
!                    pair interchanges
!     pedgenodet     the local edge number = local edgenode number
!                    for the edge [local tetnode i, local tetnode j]
!     tetnodet       Array of global nodal point numbers of the corners of
!                    the transformed original tetrahedron
!     trans          Transformation array. trans(i) is the transformed
!                    local position of the i-th corner of the original
!                    tetrahedron.
!     transi         Inverse of trans
!     transposj      The transformed local edge node number of edge node j
!     transposmax    The transformed local edge node number of edgenode
!                    edgenodemax
!     transtemp      temporary storage
!     transtot       ?
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2270   integer incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Find intersection point with lowest and highest node number.
!
!     Define transformation(= rotation) to move node isort(4) to node 1 and
!     the intersection point with lowest node number to edge position 1.
!
!     Transform tetrahedron tetnode to tetnodet and the position of the
!     intersection points edgenode(3,.) to edgenodet(.).
!
!     Split tetrahedron into four subtetrahedrons, taking into account the
!     position of the intersection point with the highest node number.
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
!     pedgenodet(i,j) = the local edge number = local edgenode number
!                   for the edge [local tetnode i, local tetnode j]
      data pedgenodet /0,1,3,4, &
                       1,0,2,5, &
                       3,2,0,6, &
                       4,5,6,0/

!     trans: transtot
!      1234: 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15
!      2314: 3, 4, 5, 6, 1, 2, 8, 9, 7, 10, 11, 13, 14, 12, 15
!      3124: 5, 6, 1, 2, 3, 4, 9, 7, 8, 10, 11, 14, 12, 13, 15
!      1423: 1, 7, 10, 8, 3, 2, 6, 9, 4, 5, 12, 14, 13, 11, 15
!      2143: 3, 2, 1, 7, 10, 8, 4, 6, 9, 5, 12, 11, 14, 13, 15
!      4213: 10, 8, 3, 2, 1, 7, 9, 4, 6, 5, 12, 13, 11, 14, 15
!      1342: 1, 6, 5, 9, 10, 7, 2, 4, 8, 3, 14, 11, 13, 12, 15
!      3412: 5, 9, 10, 7, 1, 6, 4, 8, 2, 3, 14, 13, 12, 11, 15
!      4132: 10, 7, 1, 6, 5, 9, 8, 2, 4, 3, 14, 12, 11, 13, 15
!      2431: 3, 8, 10, 9, 5, 4, 2, 7, 6, 1, 13, 12, 14, 11, 15
!      3241: 5, 4, 3, 8, 10, 9, 6, 2, 7, 1, 13, 11, 12, 14, 15
!      4321: 10, 9, 5, 4, 3, 8, 7, 6, 2, 1, 13, 14, 11, 12, 15
      data transtot / 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, &
                      3, 4, 5, 6, 1, 2, 8, 9, 7, 10, 11, 13, 14, 12, 15, &
                      5, 6, 1, 2, 3, 4, 9, 7, 8, 10, 11, 14, 12, 13, 15, &
                      1, 7, 10, 8, 3, 2, 6, 9, 4, 5, 12, 14, 13, 11, 15, &
                      3, 2, 1, 7, 10, 8, 4, 6, 9, 5, 12, 11, 14, 13, 15, &
                      10, 8, 3, 2, 1, 7, 9, 4, 6, 5, 12, 13, 11, 14, 15, &
                      1, 6, 5, 9, 10, 7, 2, 4, 8, 3, 14, 11, 13, 12, 15, &
                      5, 9, 10, 7, 1, 6, 4, 8, 2, 3, 14, 13, 12, 11, 15, &
                      10, 7, 1, 6, 5, 9, 8, 2, 4, 3, 14, 12, 11, 13, 15, &
                      3, 8, 10, 9, 5, 4, 2, 7, 6, 1, 13, 12, 14, 11, 15, &
                      5, 4, 3, 8, 10, 9, 6, 2, 7, 1, 13, 11, 12, 14, 15, &
                      10, 9, 5, 4, 3, 8, 7, 6, 2, 1, 13, 14, 11, 12, 15/
! ======================================================================
!
      call eropen ( 'mshobstms3dq6a' )

      debug=.true.
      if (debug) write(*,*)'mshobstms3dq6a'

!     --- Find intersection point with lowest and highest node number

      if( edgenode(4,1).lt.edgenode(4,2) ) then
         edgenodemin = 1
         edgenodemax = 2
      else
         edgenodemin = 2
         edgenodemax = 1
      end if

      if( edgenode(4,3).gt.edgenode(4,edgenodemax) ) then
         edgenodemax = 3
      else if ( edgenode(4,3).lt.edgenode(4,edgenodemin) ) then
         edgenodemin = 3
      end if

!     --- Determine which of the edgenodemin segment end nodes is the shared
!         node isort(4), which will be transformed to node 1.
!         The other one will be transformed to node 2.

      if( ledgenode(1,edgenodemin).eq.isort(4) ) then
         nodetwo = ledgenode(2,edgenodemin)

      else
         nodetwo = ledgenode(1,edgenodemin)

      end if

!     --- Define transformation to move node isort(4) to node 1
!                              and node nodetwo to node 2
!         -- Initialize

      oddpairintchan = .false.
      do i = 1,4
         trans(i) = i
      end do

!     --- move node isort(4) to node 1

      if( isort(4).ne.trans(1) ) then
         transtemp = trans(1)
         trans(1) = isort(4)
         trans(isort(4)) = transtemp
         oddpairintchan = .not.oddpairintchan

       end if

!      --- move node nodetwo to node 2

      if (nodetwo.ne.trans(2) ) then
         transtemp = trans(2)
         trans(2) = nodetwo
         trans(trans(nodetwo)) = transtemp
         oddpairintchan = .not.oddpairintchan

      end if

!     --- make pairinterchanges even

      if( oddpairintchan ) then
         transtemp = trans(3)
         trans(3) = trans(4)
         trans(4) = transtemp
      end if

!     --- transi := inverse of trans

      do i = 1,4
         transi(trans(i)) = i
      end do

      if (trans(4).eq.4) then
         if (trans(1).eq.1) itran=1
         if (trans(1).eq.2) itran=2
         if (trans(1).eq.3) itran=3
      else if (trans(4).eq.3) then
         if (trans(1).eq.1) itran=4
         if (trans(1).eq.2) itran=5
         if (trans(1).eq.4) itran=6
      else if (trans(4).eq.2) then
         if (trans(1).eq.1) itran=7
         if (trans(1).eq.3) itran=8
         if (trans(1).eq.4) itran=9
      else if (trans(4).eq.1) then
         if (trans(1).eq.2) itran=10
         if (trans(1).eq.3) itran=11
         if (trans(1).eq.4) itran=12
      end if

!     --- Transform quadratic tetrahedron tetnode to tetnodet

      do i = 1,15
         tetnodet(i) = tetnode(transtot(i,itran))
         if (debug) write(*,*)'nodes of element', tetnodet(i)
      end do

!     --- Transform the intersection points to their transformed positions

      do j = 1,3
         transposj = pedgenodet(transi(ledgenode(1,j)), &
                                transi(ledgenode(2,j)))
         if (edgenode(1,j).eq.tetnodet(1)) then
            edgenodet(1,transposj) = edgenode(3,j)
            edgenodet(2,transposj) = edgenode(5,j)
         else if (edgenode(2,j).eq.tetnodet(1)) then
            edgenodet(1,transposj) = edgenode(5,j)
            edgenodet(2,transposj) = edgenode(3,j)
         end if
         if (debug) write(*,*)'edgenodet', transposj, &
                                edgenodet(1,transposj), &
                                edgenodet(2,transposj)
      end do
      transposmax = pedgenodet(transi(ledgenode(1,edgenodemax)), &
                               transi(ledgenode(2,edgenodemax)))

!     --- face1 = bottom face
!         face2 = front face
!         face3 = back face
!         face3 = right face

      do j=1,4
         do i=1,4
            if (midfacenodes(i).eq.tetnodet(10+j)) facetrans(j)=i
         end do
      end do

!     --- Transform faces

      do j=1,4
         do i=1,4
            facenodet(i,j)=facenode(i,facetrans(j))
            if (debug) write(*,*)'facenodet', i, j, facenodet(i,j)
         end do
      end do

!     --- Split tetrahedron

      tetnodenew(1,1) = tetnodet(1)
      tetnodenew(2,1) = edgenodet(1,1)
      tetnodenew(3,1) = tetnodet(2)
      tetnodenew(4,1) = tetnodet(11)
      tetnodenew(5,1) = tetnodet(6)
      tetnodenew(6,1) = edgenodet(1,3)
      tetnodenew(7,1) = edgenodet(1,4)
      tetnodenew(8,1) = tetnodet(12)
      tetnodenew(9,1) = tetnodet(14)
      tetnodenew(10,1) = tetnodet(7)
      tetnodenew(11,1) = facenodet(1,1)
      tetnodenew(12,1) = facenodet(1,2)
      tetnodenew(13,1) = tetnodet(15)
      tetnodenew(14,1) = facenodet(1,4)
      tetnodenew(15,1) = centroid(1)

      tetnodenew(1,2) = tetnodet(2)
      tetnodenew(2,2) = tetnodet(11)
      tetnodenew(3,2) = tetnodet(6)
      tetnodenew(4,2) = tetnodet(14)
      tetnodenew(5,2) = tetnodet(7)
      tetnodenew(6,2) = tetnodet(12)

      if( transposmax.eq.4 ) then

         tetnodenew(7,2) = facenodet(4,2)
         tetnodenew(8,2) = facenodet(4,4)
         tetnodenew(9,2) = edgenodet(2,4)
         tetnodenew(10,2) = tetnodet(10)
         tetnodenew(11,2) = tetnodet(15)
         tetnodenew(12,2) = centroid(5)
         tetnodenew(13,2) = facenodet(2,4)
         tetnodenew(14,2) = facenodet(2,2)
         tetnodenew(15,2) = centroid(2)

         tetnodenew(1,3) = tetnodet(6)
         tetnodenew(2,3) = tetnodet(11)
         tetnodenew(3,3) = tetnodet(2)
         tetnodenew(4,3) = facenodet(4,1)
         tetnodenew(5,3) = tetnodet(5)
         tetnodenew(6,3) = edgenodet(2,3)
         tetnodenew(7,3) = facenodet(4,4)
         tetnodenew(8,3) = facenodet(4,2)
         tetnodenew(9,3) = tetnodet(9)
         tetnodenew(10,3) = tetnodet(10)
         tetnodenew(11,3) = facenodet(2,1)
         tetnodenew(12,3) = centroid(5)
         tetnodenew(13,3) = centroid(6)
         tetnodenew(14,3) = facenodet(3,4)
         tetnodenew(15,3) = centroid(3)

      else if( transposmax.eq.3 ) then

         tetnodenew(7,2) = facenodet(4,1)
         tetnodenew(8,2) = edgenodet(2,3)
         tetnodenew(9,2) = facenodet(4,4)
         tetnodenew(10,2) = tetnodet(5)
         tetnodenew(11,2) = tetnodet(15)
         tetnodenew(12,2) = facenodet(2,1)
         tetnodenew(13,2) = facenodet(2,4)
         tetnodenew(14,2) = centroid(5)
         tetnodenew(15,2) = centroid(2)

         tetnodenew(1,3) = tetnodet(7)
         tetnodenew(2,3) = tetnodet(12)
         tetnodenew(3,3) = tetnodet(2)
         tetnodenew(4,3) = facenodet(4,1)
         tetnodenew(5,3) = tetnodet(5)
         tetnodenew(6,3) = facenodet(4,4)
         tetnodenew(7,3) = edgenodet(2,4)
         tetnodenew(8,3) = facenodet(4,2)
         tetnodenew(9,3) = tetnodet(9)
         tetnodenew(10,3) = tetnodet(10)
         tetnodenew(11,3) = centroid(5)
         tetnodenew(12,3) = facenodet(2,2)
         tetnodenew(13,3) = centroid(6)
         tetnodenew(14,3) = facenodet(3,4)
         tetnodenew(15,3) = centroid(3)

      else

!     --- transposmax has wrong value

         call errint ( transposmax, 1 )
         call errchr ( 'transposmax', 1 )
         call errsub ( 2270, 1, 0, 1 )
         go to 1000

      end if

      tetnodenew(1,4) = tetnodet(2)
      tetnodenew(2,4) = facenodet(4,1)
      tetnodenew(3,4) = tetnodet(5)
      tetnodenew(4,4) = tetnodet(9)
      tetnodenew(5,4) = tetnodet(10)
      tetnodenew(6,4) = facenodet(4,2)
      tetnodenew(7,4) = edgenodet(2,1)
      tetnodenew(8,4) = tetnodet(4)
      tetnodenew(9,4) = tetnodet(8)
      tetnodenew(10,4) = tetnodet(3)
      tetnodenew(11,4) = centroid(6)
      tetnodenew(12,4) = facenodet(3,1)
      tetnodenew(13,4) = tetnodet(13)
      tetnodenew(14,4) = facenodet(3,2)
      tetnodenew(15,4) = centroid(4)

      nelemnew = 4

      fluidnd(1)=tetnodet(2)
      fluidnd(2)=tetnodet(11)
      fluidnd(3)=tetnodet(6)
      fluidnd(4)=tetnodet(14)
      fluidnd(5)=tetnodet(7)
      fluidnd(6)=tetnodet(12)
      fluidnd(7)=tetnodet(15)

1000  call erclos ( 'mshobstms3dq6a' )

      end
