      subroutine mshpavquadinf ( itrian1, itrian2, iedge5, iedge1, &
                                 iedge3, iedge2, iedge4, nmeshd, &
                                 markedges, nmeshc, thickness, coor, &
                                 h1, h2, dist )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 10-03-2005 Fortran 90
!        version  1.0  date 09-07-2004
!
!   copyright (c) 2004-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Extract information about a quadrilateral consisting of 2 triangles
!     for mesh generator paver
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     paver
!     mesh_generation
!     edge
!     quadrilateral
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer itrian1, itrian2, iedge5, iedge1, iedge3, nmeshd(*), &
              markedges(*), nmeshc(*), iedge2, iedge4
      double precision thickness(*), coor(2,*), h1, h2, dist

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     dist           o    Distance between two points
!     h1             o    Average thickness in edge1
!     h2             o    Average thickness in edge3
!     iedge1         o    edge sequence number of first boundary edge
!     iedge2         o    edge sequence number of first connecting edge
!     iedge3         o    edge sequence number of second boundary edge
!     iedge4         o    edge sequence number of second connecting edge
!     iedge5         o    edge sequence number diagonal of quadrilateral
!     itrian1        i    Sequence number of first triangle
!     itrian2        i    Sequence number of second triangle
!     markedges      i    array of length nedges indicating whether a
!                         edge is internal or on the boundary
!                         -2:  internal edge between 2 boundary nodes
!                         -1:  boundary edge
!                          0:  standard internal edge
!     nmeshc         i    nmesh part c, contains node numbers corresponding to
!                         edges
!     nmeshd         i    nmesh part d, contains edge numbers of elements
!     thickness      i    Array containing the thickness in the nodes in exactly
!                         the same sequence as in coor
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ipstart, jpstart, iseq, jseq, iplus(4), &
              inode1, inode2, inode3, inode4
      double precision x1(2), x2(2), x3(2), x4(2)

!     inode1         First node in edge
!     inode2         Second node in edge
!     inode3         First node in second edge
!     inode4         Second node in second edge
!     iplus          Array containing i+1 modulo 3
!     ipstart        Starting address in array
!     iseq           Sequence number
!     jpstart        Starting address in array
!     jseq           Sequence number
!     x1             coordinates of first point of first boundary edge
!     x2             coordinates of last point of first boundary edge
!     x3             coordinates of first point of second boundary edge
!     x4             coordinates of last point of second boundary edge
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDISTLINES2D  Compute the distance between two interval in 2d by
!                    computing the distance of the line perpendicular to
!                    the mid point of the first interval and the line through
!                    the other interval
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!      4   e3   3
!      x--------x
!      |\       |
!      | \      |
!      |  \     |
!    e4|  e5    |e2
!      |    \   |
!      |     \  |
!      |      \ |
!      x--------x
!      1   e1   2
!
!     e5 := common edge of trian1 and trian2
!     e1 := boundary edge of trian1
!     e3 := boundary edge of trian2
!     e2 := remaining edge of trian2
!     e4 := remaining edge of trian1
!     node1, node2 := nodes e1
!     node3, node4 := nodes e3
!     h1 := mean thickness e1
!     h2 := mean thickness e3
!     d := distance e1 e3
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data iplus / 2, 3, 1, 2 /
! ======================================================================
!
      call eropen ( 'mshpavquadinf' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavquadinf'

      end if
      if ( ierror/=0 ) go to 1000

!     --- e5 := common edge of trian1 and trian2
!         e1 := boundary edge of trian1
!         e3 := boundary edge of trian2

      ipstart = nmeshd(itrian1)
      jpstart = nmeshd(itrian2)
      iedge1 = abs(nmeshd(ipstart+1))
      iseq = 1
      if ( markedges(iedge1)/=-1 ) then

!     --- iedge1 is not boundary edge, try next one

         iedge1 = abs(nmeshd(ipstart+2))
         if ( markedges(iedge1)/=-1 ) then

!        --- iedge1 is not boundary edge, it must be the last one

            iedge1 = abs(nmeshd(ipstart+3))
            iseq = 3

         else

!        --- iedge1 is boundary edge, so second edge

            iseq = 2

         end if  ! ( markedges(iedge1)/=-1 )

      end if  ! ( markedges(iedge1)/=-1 )
      if ( debug ) write(irefwr,*) 'iedge1, iseq ', iedge1, iseq

      iedge3 = abs(nmeshd(jpstart+1))
      jseq = 1
      if ( markedges(iedge3)/=-1 ) then

!     --- iedge3 is not boundary edge, try next one

         iedge3 = abs(nmeshd(jpstart+2))
         if ( markedges(iedge3)/=-1 ) then

!        --- iedge3 is not boundary edge, it must be the last one

            iedge3 = abs(nmeshd(jpstart+3))
            jseq = 3

         else

!        --- iedge3 is boundary edge, so second edge

            jseq = 2

         end if  ! ( markedges(iedge3)/=-1 )

      end if  ! ( markedges(iedge3)/=-1 )
      if ( debug ) write(irefwr,*) 'iedge3, jseq ', iedge3, jseq

      iedge5 = abs(nmeshd(ipstart+iplus(iseq)))
      if ( iedge5==abs(nmeshd(jpstart+iplus(jseq))) ) then

!     --- iedge5 is common edge

         iedge4 = abs(nmeshd(ipstart+iplus(iseq+1)))
         iedge2 = abs(nmeshd(jpstart+iplus(jseq+1)))

      else if ( iedge5==abs(nmeshd(jpstart+iplus(jseq+1))) ) then

!     --- iedge5 is common edge

         iedge4 = abs(nmeshd(ipstart+iplus(iseq+1)))
         iedge2 = abs(nmeshd(jpstart+iplus(jseq)))

      else

!     --- iedge5 is not common edge

         iedge4 = iedge5
         iedge5 = abs(nmeshd(ipstart+iplus(iseq+1)))
         if ( iedge5==abs(nmeshd(jpstart+iplus(jseq))) ) then
            iedge2 = abs(nmeshd(jpstart+iplus(jseq+1)))
         else
            iedge2 = abs(nmeshd(jpstart+iplus(jseq)))
         end if  ! ( iedge5==nmeshd(jpstart+iplus(jseq)) )

      end if  ! ( iedge5==nmeshd(ipstart+iplus(jseq)) )
      if ( debug ) &
         write(irefwr,*) 'iedge2, iedge4, iedge5 ', &
                          iedge2, iedge4, iedge5

!     --- node1, node2 := nodes e1

      ipstart = nmeshc(iedge1)
      inode1  = nmeshc(ipstart+1)
      inode2  = nmeshc(ipstart+2)

!     --- node3, node4 := nodes e3

      ipstart = nmeshc(iedge3)
      inode3  = nmeshc(ipstart+1)
      inode4  = nmeshc(ipstart+2)

!     --- h1 := mean thickness e1
!         h2 := mean thickness e3

      h1 = 0.5d0*(thickness(inode1)+thickness(inode2))
      h2 = 0.5d0*(thickness(inode3)+thickness(inode4))

!     --- d := distance e1 e3

      x1(1) = coor(1,inode1)
      x1(2) = coor(2,inode1)
      x2(1) = coor(1,inode2)
      x2(2) = coor(2,inode2)
      x3(1) = coor(1,inode3)
      x3(2) = coor(2,inode3)
      x4(1) = coor(1,inode4)
      x4(2) = coor(2,inode4)
      call erdistlines2d ( x1, x2, x3, x4, dist )

1000  call erclos ( 'mshpavquadinf' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshpavquadinf'

      end if

      end
