      subroutine mshpavsub2qua ( nmeshc, nmeshd, iedge1, iedge4, &
                                 iedge5, itrian1, itrian2, &
                                 node1extra, node2extra, coor, &
                                 kmeshc, nel )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 10-03-2005 Fortran 90
!        version  1.0  date 12-07-2004
!
!   copyright (c) 2004-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subdivide a quadrilateral consisting of 2 triangles into 2 quadrilaterals
!     each of which consist of 2 triangles
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     paver
!     triangle
!     quadrilateral
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nmeshc(*), nmeshd(*), iedge1, iedge5, itrian1, &
              node1extra, node2extra, itrian2, kmeshc(3,*), nel, iedge4
      double precision coor(2,*)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     iedge1         i    edge sequence number of first edge (not to be
!                         subdivided)
!     iedge4         i    edge sequence number of foruth edge
!                         (must be subdivided)
!     iedge5         i    edge sequence number of "diagonal"
!     itrian1        i    Sequence number of first triangle
!     itrian2        i    Sequence number of second triangle
!     kmeshc        i/o   kmesh part c, contains the nodal points
!                         corresponding to the elements
!     nel           i/o   number of elements in the topology
!     nmeshc         i    nmesh part c, contains node numbers corresponding to
!                         edges
!     nmeshd         i    nmesh part d, contains edge numbers of elements
!     node1extra     i    Node number of extra node on edge 2
!     node2extra     i    Node number of extra node on edge 4
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inode1, inode2, inode3, inode4, idir, iedge, i, inode, &
              ipstart, jpstart
      double precision x(6), y(6)

!     i              Counting variable
!     idir           If 1 the diagonal is from left to right, otherwise (-1)
!                    from right to left
!     iedge          edge sequence number
!     inode          Node sequence number
!     inode1         First node in quadrilateral
!     inode2         Second node in quadrilateral
!     inode3         Third node in quadrilateral
!     inode4         Fourth node in quadrilateral
!     ipstart        Starting address in array
!     jpstart        Starting address in array
!     x              Array containing the x coordinates of the 4 vertices
!                    plus 2 positions work space
!     y              Array containing the y coordinates of the 4 vertices
!                    plus 2 positions work space
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHR17         detect direction of diagonal for triangular elements
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     inode1 := common node of edge1 and edge5
!     inode2 := next node of edge1
!     inode3 := common node of edge3 and edge5
!     inode2 := next node of edge2
!     idir := direction inode1 ... inode2
!     Situation we have is one of the next:
!
!     4         e3      3             3         e3      4
!     x-----------------x             x-----------------x
!     x                 x             x                 x
! ne2 x       /e5    e2 x ne1     ne1 x       \e5    e4 x ne2
!     x e4              x             x e2              x
!     x-----------------x             x-----------------x
!     1      e1          2            2      e1          1
!
!     The direction of the diagonals in the quads is computed such that
!     the largest angle is subdivided
!
!     The quads are:
!
!     1, 2, ne1, ne2   and   ne2, ne1, 3, 4 if idir = 1
!     and
!     2, 1, ne2, ne1   and   ne1, ne2, 4, 3 if idir = -1
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavsub2qua' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavsub2qua'
         write(irefwr,*) 'itrian1, itrian2, nel ', &
                          itrian1, itrian2, nel
         write(irefwr,*) 'iedge1, iedge4, iedge5 ', &
                          iedge1, iedge4, iedge5
         call prinin ( kmeshc(1,itrian1), 3, 'element 1 before' )
         call prinin ( kmeshc(1,itrian2), 3, 'element 2 before' )

      end if
      if ( ierror/=0 ) go to 1000

!     --- inode1 := common node of edge1 and edge5
!         inode2 := next node of edge1
!         inode3 := common node of edge3 and edge5
!         inode4 := next node of edge2

      ipstart = nmeshc(iedge1)     ! start of edge 1
      jpstart = nmeshc(iedge5)     ! start of edge 5
      inode = nmeshc(ipstart+1)    ! first node of edge1
      if ( inode==nmeshc(jpstart+1) ) then

!     --- inode is common node

         inode1 = inode                ! inode1 is first node of edge1
         inode2 = nmeshc(ipstart+2)    ! inode2 is last node of edge1
         inode3 = nmeshc(jpstart+2)    ! inode3 is last node of edge5
         idir = 1                      ! positive direction

      else if ( inode==nmeshc(jpstart+2) ) then

!     --- inode is common node

         inode1 = inode                ! inode1 is first node of edge1
         inode2 = nmeshc(ipstart+2)    ! inode2 is last node of edge1
         inode3 = nmeshc(jpstart+1)    ! inode3 is first node of edge5
         idir = 1                      ! positive direction

      else

!     --- inode is not common node

         inode1 =  nmeshc(ipstart+2)    ! inode1 is last node of edge1
         inode2 = inode                 ! inode2 is first node of edge1
         if  ( inode1==nmeshc(jpstart+1) ) then
            inode3 = nmeshc(jpstart+2)    ! inode3 is last node of edge5
         else
            inode3 = nmeshc(jpstart+1)    ! inode3 is first node of edge5
         end if  ! ( inode==nmeshc(jpstart+1) )
         idir = -1                      ! negative direction

      end if  ! ( inode==nmeshc(jpstart+1) )

      ipstart = nmeshc(iedge4)          ! start of edge4

      if ( inode2==nmeshc(ipstart+1) .or. &
           inode1==nmeshc(ipstart+1) ) then
         inode4 = nmeshc(ipstart+2)     ! inode4 is last node of edge4
      else
         inode4 = nmeshc(ipstart+1)     ! inode4 is first node of edge4
      end if  ! ( inode3==nmeshc(ipstart+1) )

      if ( debug ) &
         write(irefwr,*) 'inode1, inode2, inode3, inode4, idir ', &
                          inode1, inode2, inode3, inode4, idir

!     --- idir := direction inode1 ... inode2

      ipstart = nmeshd(itrian1)          ! start of triangle 1
      do i = 1, 3

         iedge = nmeshd(ipstart+i)           ! edge in triangle 1
         if ( abs(iedge)==iedge1 ) then

!        --- edge found

            if ( iedge<0 ) idir = -idir   ! change direction in case of
                                             ! negative edge number
            go to 100

         end if  ! ( abs(iedge)==iedge1 )

      end do  ! i = 1, 3

100   if ( debug ) write(irefwr,*) 'Direction ', idir

!     --- The direction of the diagonals in the quads is computed such that
!         the largest angle is subdivided

      if ( idir==1 ) then

!     --- Positive direction (counter clockwise)
!         The quad is:
!         1, 2, ne1, ne2   and   ne2, ne1, 3, 4

         x(2) = coor(1,inode1)
         x(3) = coor(1,inode2)
         x(4) = coor(1,node1extra)
         x(5) = coor(1,node2extra)

         y(2) = coor(2,inode1)
         y(3) = coor(2,inode2)
         y(4) = coor(2,node1extra)
         y(5) = coor(2,node2extra)

!        --- detect the "best" diagonal direction

         call mshr17 ( x, y, idir )
         if ( idir==0 ) then

!        --- The diagonal must be from 1 to ne1

            kmeshc(1,itrian1) = inode1
            kmeshc(2,itrian1) = inode2
            kmeshc(3,itrian1) = node1extra

            kmeshc(1,itrian2) = node1extra
            kmeshc(2,itrian2) = node2extra
            kmeshc(3,itrian2) = inode1

         else

!        --- The diagonal must be from 2 to ne2

            kmeshc(1,itrian1) = inode1
            kmeshc(2,itrian1) = inode2
            kmeshc(3,itrian1) = node2extra

            kmeshc(1,itrian2) = node1extra
            kmeshc(2,itrian2) = node2extra
            kmeshc(3,itrian2) = inode2

         end if  ! ( idir==0 )

         x(2) = coor(1,node2extra)
         x(3) = coor(1,node1extra)
         x(4) = coor(1,inode3)
         x(5) = coor(1,inode4)

         y(2) = coor(2,node2extra)
         y(3) = coor(2,node1extra)
         y(4) = coor(2,inode3)
         y(5) = coor(2,inode4)

!        --- detect the "best" diagonal direction

         call mshr17 ( x, y, idir )
         if ( idir==0 ) then

!        --- The diagonal must be from ne2 to 3

            nel = nel+1
            kmeshc(1,nel) = node2extra
            kmeshc(2,nel) = node1extra
            kmeshc(3,nel) = inode3

            nel = nel+1
            kmeshc(1,nel) = inode3
            kmeshc(2,nel) = inode4
            kmeshc(3,nel) = node2extra

         else

!        --- The diagonal must be from ne1 to 4

            nel = nel+1
            kmeshc(1,nel) = node2extra
            kmeshc(2,nel) = node1extra
            kmeshc(3,nel) = inode4

            nel = nel+1
            kmeshc(1,nel) = inode3
            kmeshc(2,nel) = inode4
            kmeshc(3,nel) = node1extra

         end if  ! ( idir==0 )

      else

!     --- Negative direction (clockwise)
!         The quad is:
!         2, 1, ne2, ne1   and   ne1, ne2, 4, 3 if idir = -1

         x(2) = coor(1,inode2)
         x(3) = coor(1,inode1)
         x(4) = coor(1,node2extra)
         x(5) = coor(1,node1extra)

         y(2) = coor(2,inode2)
         y(3) = coor(2,inode1)
         y(4) = coor(2,node2extra)
         y(5) = coor(2,node1extra)

!        --- detect the "best" diagonal direction

         call mshr17 ( x, y, idir )
         if ( idir==0 ) then

!        --- The diagonal must be from 2 to ne2

            kmeshc(1,itrian1) = inode2
            kmeshc(2,itrian1) = inode1
            kmeshc(3,itrian1) = node2extra

            kmeshc(1,itrian2) = node2extra
            kmeshc(2,itrian2) = node1extra
            kmeshc(3,itrian2) = inode2

         else

!        --- The diagonal must be from 1 to ne1

            kmeshc(1,itrian1) = inode2
            kmeshc(2,itrian1) = inode1
            kmeshc(3,itrian1) = node1extra

            kmeshc(1,itrian2) = node2extra
            kmeshc(2,itrian2) = node1extra
            kmeshc(3,itrian2) = inode1

         end if  ! ( idir==0 )

         x(2) = coor(1,node1extra)
         x(3) = coor(1,node2extra)
         x(4) = coor(1,inode4)
         x(5) = coor(1,inode3)

         y(2) = coor(2,node1extra)
         y(3) = coor(2,node2extra)
         y(4) = coor(2,inode4)
         y(5) = coor(2,inode3)

!        --- detect the "best" diagonal direction

         call mshr17 ( x, y, idir )
         if ( idir==0 ) then

!        --- The diagonal must be from ne1 to 4

            nel = nel+1
            kmeshc(1,nel) = node1extra
            kmeshc(2,nel) = node2extra
            kmeshc(3,nel) = inode4

            nel = nel+1
            kmeshc(1,nel) = inode4
            kmeshc(2,nel) = inode3
            kmeshc(3,nel) = node1extra

         else

!        --- The diagonal must be from ne2 to 3

            nel = nel+1
            kmeshc(1,nel) = node1extra
            kmeshc(2,nel) = node2extra
            kmeshc(3,nel) = inode3

            nel = nel+1
            kmeshc(1,nel) = inode4
            kmeshc(2,nel) = inode3
            kmeshc(3,nel) = node2extra

         end if  ! ( idir==0 )

      end if  ! ( idir==1 )

1000  call erclos ( 'mshpavsub2qua' )

      if ( debug ) then

!     --- Debug information

         call prinin ( kmeshc(1,itrian1), 3, 'element 1 after' )
         call prinin ( kmeshc(1,itrian2), 3, 'element 2 after' )
         call prinin ( kmeshc(1,nel-1), 3, 'element 3 after' )
         call prinin ( kmeshc(1,nel), 3, 'element 4 after' )
         write(irefwr,*) 'End mshpavsub2qua'

      end if

      end
