      subroutine mshpavsub2tria ( nmeshd, itrian1, itrian2, nel, kmeshc, &
                                  nmeshc, coor, iworkedge )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-07-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subdivide a set of 2 triangles that form a quadrilateral if necessary
!     into more triangles
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     paver
!     triangle
!     quadrilateral
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nmeshd(*), itrian1, itrian2, nel, kmeshc(3,*), nmeshc(*), &
              iworkedge(*)
      double precision coor(2,*)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     itrian1        i    Sequence number of first triangle
!     itrian2        i    Sequence number of second triangle
!     iworkedge      i    work array of length nedges to be used to store
!                         new node numbers on edges
!                          0:  not an extra node
!                         >0:  node number of extra node
!     kmeshc        i/o   kmesh part c, contains the nodal points
!                         corresponding to the elements
!     nel           i/o   number of elements in the topology
!     nmeshc         i    nmesh part c, contains node numbers corresponding to
!                         edges
!     nmeshd         i    nmesh part d, contains edge numbers of elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ipstart, iedge, iedge1, iedge2, iedge4, iedge5, i, &
              node1extra, nextra, node2extra
      double precision area, areabefore, areaend
      logical checkarea

!     area           Area of element
!     areabefore     Area of region before subdivision
!     areaend        Area of region after subdivision
!     checkarea      If true the are must be checked
!     i              Counting variable
!     iedge          edge sequence number
!     iedge1         edge sequence number (first edge in quadrilateral, must
!                    not be subdivided)
!     iedge2         edge sequence number (second edge in quadrilateral, must
!                    be subdivided)
!     iedge4         edge sequence number (fourth edge in quadrilateral, must
!                    be subdivided)
!     iedge5         edge sequence number (diagonal of quad)
!     ipstart        Starting address in array
!     nextra         number of edges to be subdivided
!     node1extra     Node number of extra node on edge 2
!     node2extra     Node number of extra node on edge 4
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRAREA        Print error message if area is negative
!     MSHAREA        Compute the area of the triangle spanned by x1-x2-x3
!                    The sign of the are defines the direction
!     MSHPAVSUB1TRIA Subdivide one triangle into 2, because one of the edges is
!                    splitted, into 2 parts
!     MSHPAVSUB2QUA  Subdivide a quadrilateral consisting of 2 triangles into 2
!                    quadrilaterals each of which consist of 2 triangles
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     nextra := number of edges to be subdivided
!     if ( nextra = 1 ) then
!        subdivide one triangle
!     else if ( nextra = 2 ) then
!        replace both triangles by 4 new ones
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavsub2tria' )
      debug = .false.
      checkarea = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavsub2tria'
         write(irefwr,*) 'itrian1, itrian2 ', itrian1, itrian2
         call prinin ( kmeshc(1,itrian1), 3, 'triangle 1' )
         call prinin ( kmeshc(1,itrian2), 3, 'triangle 2' )

      end if  ! ( debug )
      if ( checkarea ) then

!     --- Check area

         call msharea ( coor(1,kmeshc(1,itrian1)), &
                        coor(1,kmeshc(2,itrian1)), &
                        coor(1,kmeshc(3,itrian1)), areabefore )
         if ( areabefore.le.0d0 ) call  errarea ( areabefore, itrian1 )
         call msharea ( coor(1,kmeshc(1,itrian2)), &
                        coor(1,kmeshc(2,itrian2)), &
                        coor(1,kmeshc(3,itrian2)), area )
         if ( area.le.0d0 ) call errarea ( area, itrian2 )
         areabefore = areabefore+area

      end if
      if ( ierror.ne.0 ) go to 1000

!     --- nextra := number of edges to be subdivided

      ipstart = nmeshd(itrian1)    ! first triangle
      nextra = 0
      node1extra = 0
      node2extra = 0
      iedge5 = 0
      do i = 1, 3

         iedge = abs(nmeshd(ipstart+i))         ! edge in first triangle
         if ( iworkedge(iedge).eq.-2 ) then
            iedge5 = iedge                      ! diagonal of quadrilateral
         else if ( iworkedge(iedge).gt.1 ) then
            iedge2 = iedge                      ! edge to be subdivided
            node1extra = iworkedge(iedge)       ! node on this edge
            nextra = 1
         else
            iedge1 = iedge                      ! edge not to be subdivided
         end if  ! ( iworkedge(iedge).eq.-2 )

      end do  ! i = 1, 3

      ipstart = nmeshd(itrian2)    ! second triangle
      do i = 1, 3

         iedge = abs(nmeshd(ipstart+i))         ! edge in second triangle
         if ( iworkedge(iedge).gt.1 ) then
            iedge4 = iedge                      ! edge to be subdivided
            node2extra = iworkedge(iedge)       ! node on this edge
            nextra = nextra+1
         end if  ! ( iworkedge(iedge).eq.-2 )

      end do  ! i = 1, 3

      if ( nextra.eq.1 ) then

!     --- nextra = 1
!         subdivide one triangle

         if ( node1extra.gt.0 ) then

!        --- first triangle must be subdivided

            call mshpavsub1tria ( iedge2, nmeshc, itrian1, kmeshc, nel, &
                                  node1extra )
            if ( checkarea ) then

!           --- Debug information

               call msharea ( coor(1,kmeshc(1,itrian1)), &
                              coor(1,kmeshc(2,itrian1)), &
                              coor(1,kmeshc(3,itrian1)), areaend )
               if ( areaend.le.0d0 ) call errarea ( areaend, itrian1 )
               call msharea ( coor(1,kmeshc(1,itrian2)), &
                              coor(1,kmeshc(2,itrian2)), &
                              coor(1,kmeshc(3,itrian2)), area )
               if ( area.le.0d0 ) call errarea ( area, itrian2 )
               areaend = areaend+area
               call msharea ( coor(1,kmeshc(1,nel)), &
                              coor(1,kmeshc(2,nel)), &
                              coor(1,kmeshc(3,nel)), area )
               if ( area.le.0d0 ) call errarea ( area, nel )
               areaend = areaend+area
               if ( abs(areaend-areabefore).gt.sqreps*areaend ) then

!              --- Error area before not same as area after

                  write(irefwr,*) 'node1extra ', node1extra
                  write(irefwr,*) 'areabefore, areaend ', &
                                   areabefore, areaend
                  call prinin ( kmeshc(1,itrian1), 3, 'triangle 1' )
                  call prinin ( kmeshc(1,itrian2), 3, 'triangle 2' )
                  call prinin ( kmeshc(1,nel), 3, 'triangle 3' )

               end if  ! ( abs(areaend-areabefore).gt.sqreps*areaend )

            end if  ! ( debug )

         else

!        --- second triangle must be subdivided

            call mshpavsub1tria ( iedge4, nmeshc, itrian2, kmeshc, nel, &
                                  node2extra )
            if ( checkarea ) then

!           --- Debug information

               call msharea ( coor(1,kmeshc(1,itrian1)), &
                              coor(1,kmeshc(2,itrian1)), &
                              coor(1,kmeshc(3,itrian1)), areaend )
               if ( areaend.le.0d0 ) call errarea ( areaend, itrian1 )
               call msharea ( coor(1,kmeshc(1,itrian2)), &
                              coor(1,kmeshc(2,itrian2)), &
                              coor(1,kmeshc(3,itrian2)), area )
               if ( area.le.0d0 ) call errarea ( area, itrian2 )
               areaend = areaend+area
               call msharea ( coor(1,kmeshc(1,nel)), &
                              coor(1,kmeshc(2,nel)), &
                              coor(1,kmeshc(3,nel)), area )
               if ( area.le.0d0 ) call errarea ( area, nel )
               areaend = areaend+area
               if ( abs(areaend-areabefore).gt.sqreps*areaend ) then

!              --- Error area before not same as area after

                  write(irefwr,*) 'node2extra ', node2extra
                  write(irefwr,*) 'areabefore, areaend ', &
                                   areabefore, areaend
                  call prinin ( kmeshc(1,itrian1), 3, 'triangle 1' )
                  call prinin ( kmeshc(1,itrian2), 3, 'triangle 2' )
                  call prinin ( kmeshc(1,nel), 3, 'triangle 3' )

               end if  ! ( abs(areaend-areabefore).gt.sqreps*areaend )

            end if  ! ( debug )

         end if  ! ( node1extra.gt.0 )

      else if ( nextra.eq.2 ) then

!     --- nextra = 2
!         replace both triangles by 4 new ones

         if ( iedge5.gt.0 ) then

!        --- edge5 > 0, remove diagonal

            call mshpavsub2qua ( nmeshc, nmeshd, iedge1, iedge4, &
                                 iedge5, itrian1, itrian2, &
                                 node1extra, node2extra, coor, &
                                 kmeshc, nel )

         else

!        --- iedge5 = 0, treat as separate triangles

            call mshpavsub1tria ( iedge2, nmeshc, itrian1, kmeshc, nel, &
                                  node1extra )

            call mshpavsub1tria ( iedge4, nmeshc, itrian2, kmeshc, nel, &
                                  node2extra )

         end if  ! ( iedge5.gt.0 )

         if ( checkarea ) then

!        --- Debug information

            call msharea ( coor(1,kmeshc(1,itrian1)), &
                           coor(1,kmeshc(2,itrian1)), &
                           coor(1,kmeshc(3,itrian1)), areaend )
            if ( areaend.le.0d0 ) call errarea ( areaend, itrian1 )
            call msharea ( coor(1,kmeshc(1,itrian2)), &
                           coor(1,kmeshc(2,itrian2)), &
                           coor(1,kmeshc(3,itrian2)), area )
            if ( area.le.0d0 ) call errarea ( area, itrian2 )
            areaend = areaend+area
            nel = nel-1
            call msharea ( coor(1,kmeshc(1,nel)), &
                           coor(1,kmeshc(2,nel)), &
                           coor(1,kmeshc(3,nel)), area )
            if ( area.le.0d0 ) call errarea ( area, nel )
            areaend = areaend+area
            nel = nel+1
            call msharea ( coor(1,kmeshc(1,nel)), &
                           coor(1,kmeshc(2,nel)), &
                           coor(1,kmeshc(3,nel)), area )
            if ( area.le.0d0 ) call errarea ( area, nel )
            areaend = areaend+area
            if ( abs(areaend-areabefore).gt.sqreps*areaend ) then

!           --- Error area before not same as area after

               write(irefwr,*) 'node1extra, node2extra ', &
                                node1extra, node2extra
               write(irefwr,*) 'areabefore, areaend ', &
                                areabefore, areaend
               call prinin ( kmeshc(1,itrian1), 3, 'triangle 1' )
               call prinin ( kmeshc(1,itrian2), 3, 'triangle 2' )
               nel = nel-1
               call prinin ( kmeshc(1,nel), 3, 'triangle 3' )
               nel = nel+1
               call prinin ( kmeshc(1,nel), 3, 'triangle 4' )

            end if  ! ( abs(areaend-areabefore).gt.sqreps*areaend )

         end if  ! ( debug )

      end if  ! ( nextra.eq.1 )

1000  call erclos ( 'mshpavsub2tria' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshpavsub2tria'

      end if

      end
