      subroutine mshrefpar3 ( iseqkm2, coorglobnew, &
                              kmeshmnew, lenkmeshm, lenkmeshn, &
                              coorlocnew, newmapping, &
                              kmeshmglobnew, lenkmeshmglob, &
                              kmeshcglobnew, kmeshcnew, lenkmeshcglnw )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 29-04-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Receive local arrays newmapping, coor, kmeshm and kmeshn from
!     other nodes
!     Merge results with global arrays
! **********************************************************************
!
!                       KEYWORDS
!
!     refine
!     parallel
!     coordinate
!     node
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulexxx
      use sepmodulekmesh
      use sepmodulecpack
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iseqkm2, kmeshmnew(*), &
              lenkmeshm, lenkmeshn, newmapping(*), &
              kmeshmglobnew(*), lenkmeshmglob, &
              kmeshcglobnew(*), kmeshcnew(*), lenkmeshcglnw
      double precision coorglobnew(ndim,*), coorlocnew(ndim,*)

!     coorglobnew    o    Contains coordinates of new global mesh
!     coorlocnew     i    Contains coordinates of new local mesh
!     iseqkm2        i    Sequence number with respect to km of second mesh
!     kmeshcglobnew  o    Array kmeshcglob corresponding to new mesh
!     kmeshcnew      i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     kmeshmglobnew  o    Array kmeshmglob corresponding to new mesh
!     kmeshmnew      i    New array kmeshm
!     lenkmeshcglnw i/o   Length of new array kmeshcglob
!                         At input: declared length
!                         At output: length
!     lenkmeshm      i    Length of array kmeshm
!     lenkmeshmglob i/o   Length of array kmeshmglob
!                         At input: declared length
!                         At output: length
!     lenkmeshn      i    Length of array kmeshn
!     newmapping     i    contains global node numbers for refined mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer npointglobold, lenbufarray, icontext, i, &
              iblock, npointglobnew, npointlocnew, &
              npointlocold, lenbuf, lenkmeshc, ielgrp, inpelm
      integer, allocatable :: ibufarray(:), imapping(:), &
                              kmeshmcopy(:,:), kmeshccopy(:,:), &
                              kmeshgcopy(:,:)

!     i              Counting variable
!     iblock         Block sequence number
!     ibufarray      Buffer array to store information to be sent or received
!     icontext       Tag for sending and receiving, is used as check
!     ielgrp         element group number
!     imapping       work array to store the global node numbers of the
!                    newly created points
!     inpelm         number of points per element
!     kmeshccopy     Array of size iacnodes x lenkmeshcglob containing
!                    the arrays kmeshc of each block
!                    The global numbering is used
!     kmeshgcopy     Array of size iacnodes x nelgrp containing
!                    the arrays kmeshg of each block
!     kmeshmcopy     Work array of length (iacnodes,lenkmeshm) to store
!                    all curves that have new nodal point numbers per
!                    parallel node
!                    (.,1) contains the used length for node .
!                    The next positions contain for each curve stored:
!                    Pos. i: curve number
!                         i+1: number of nodal points in curve
!                         followed by nodal point numbers
!     lenbuf         Local length of ibufarray
!     lenbufarray    Length of ibufarray
!     lenkmeshc      length of array kmeshc
!     npointglobnew  Number of global points in new mesh
!     npointglobold  Number of global points in old mesh
!     npointlocnew   Number of local points in new mesh
!     npointlocold   Number of local points in old mesh
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inilen, getnpelmelgrp

!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     GETNPELMELGRP  Fetch number of points per element in element group ielgrp
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     INILEN         Compute length of array in IBUFFR
!     INIRECEIVEMPI  Receive information from other node (mpi)
!     MSHREFPAR11    Create a new kmeshcglob for the parallel case if refinement
!                    is applied
!     MSHREFPAR4     Copy coordinates from local coordinates array into global
!                    coordinates array using array imapping
!     MSHREFPAR5     Create new array kmeshmglob from old one as well as
!                    information read from all nodes
!     MSHREFPAR7     Copy local coordinates of submesh in coorglobnew
!     MSHREFPAR8     Find single curves that are changed by refinement
!     PR0008         Print information of curves
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefpar3' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefpar3'
         write(irefwr,1) 'lenkmeshm, lenkmeshn', lenkmeshm, lenkmeshn
         call pr0008 ( kmeshmnew, km(iseqkm2)%kmesh(11) )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      npointglobold = kmeshpar(1)
      npointglobnew = km(iseqkm2)%kmeshpar(1)
      npointlocnew = km(iseqkm2)%kmesh(8)
      npointlocold = npoint
      lenbufarray = npointglobnew*ndim*intlen+2+lenkmeshm*2+ &
                    lenkmeshn*4+npointglobnew+nelgrp+lenkmeshcglnw

      if ( debug ) then
         write(irefwr,1) 'ndim, npointglobold, ncurvs, npointglobnew', &
                          ndim, npointglobold, ncurvs, npointglobnew
         write(irefwr,1) 'lenbufarray, npointlocnew', &
                          lenbufarray, npointlocnew
         write(irefwr,1) 'nelgrp, iacnodes, lenkmeshcglnw', &
                          nelgrp, iacnodes, lenkmeshcglnw
      end if  ! ( debug )

      allocate ( ibufarray(lenbufarray), imapping(0:npointglobnew), &
                 kmeshmcopy(iacnodes,lenkmeshm), &
                 kmeshccopy(iacnodes,lenkmeshcglnw), &
                 kmeshgcopy(iacnodes,nelgrp), stat = error )
      length = lenbufarray+1+npointglobnew+iacnodes*lenkmeshm+ &
               iacnodes*lenkmeshcglnw
      if ( error/=0 ) call eralloc ( error, length, 'ibufarray' )
      if ( ierror/=0 ) go to 1000

      kmeshmcopy = 0

!     --- Copy old global coordinates array in new one

      coorglobnew(:,1:npointglobold) = &
         coorglob(:,1:npointglobold)

!     --- Copy local coordinates in coorglobnew

      if ( debug ) write(irefwr,*) 'before mshrefpar7'
      call mshrefpar7 ( coorglobnew, coorlocnew, newmapping, &
                        npointlocold, npointlocnew, ndim )
      if ( ierror/=0 ) go to 1000
      if ( debug ) write(irefwr,*) 'after mshrefpar7'

!     --- Store information of local array kmeshc into kmeshccopy
!         using the global numbering

      lenkmeshc = size(km(iseqkm2)%kmeshc)
      call inichkarlen ( lenkmeshc, lenkmeshcglnw, 'kmeshccopy' )
      if ( ierror/=0 ) go to 1000

      if ( debug ) then
         write(irefwr,1) 'lenkmeshc', lenkmeshc
      end if  ! ( debug )

      kmeshccopy(1,1:lenkmeshc) = newmapping(kmeshcnew(1:lenkmeshc))

      if ( nelgrp==1 ) then

!     --- nelgrp = 1, hence kmeshg is not used

         kmeshgcopy(1,1) = km(iseqkm2)%kmesh(9)  ! nelem

      else

!     --- nelgrp>1, hence kmeshg contains number of elements per el group

         kmeshgcopy(1,1:nelgrp) = km(iseqkm2)%kmeshg(1:nelgrp) ! kmesh part g

      end if  ! ( nelgrp==1 )

!     --- Store information of single curves that have been changed in node 1
!         into kmeshmcopy

      if ( debug ) write(irefwr,*) 'before mshrefpar8'
      call mshrefpar8 ( kmeshmnew, newmapping, &
                        iseqkm, lenbuf, ibufarray )
      if ( debug ) write(irefwr,*) 'after mshrefpar8'
      kmeshmcopy(1,1:lenbuf) = ibufarray(1:lenbuf)
      if ( ierror/=0 ) go to 1000

      if ( debug ) then
         write(irefwr,1) 'lenbuf', lenbuf
         call prinin ( kmeshmcopy(1,1), lenbuf, 'kmeshmcopy/1' )
      end if  ! ( debug )

!     --- Loop over all other nodes

      do i = 2, iacnodes

!     --- Receive information of block iblock

         icontext = 25
         if ( debug ) write(irefwr,1) 'i, icontext', i, icontext
         call inireceivempi ( ibufarray, lenbufarray, SPINT, iblock, &
                              icontext )
         npoint = ibufarray(2)  ! number of new nodes in iblock
         if ( ierror/=0 ) go to 1000

         if ( debug ) &
            write(irefwr,1) 'i, iblock, lenbufarray', &
                             i, iblock, lenbufarray

!        --- Starting address of coordinates array is 3
!            Set lenbufarray

         lenbufarray = npoint*ndim*intlen+2
         if ( debug ) write(irefwr,1) 'lenbufarray/1', lenbufarray

!        --- Get mapping array (only new nodes)

         imapping(0) = 0
         imapping(1:npoint) = &
            ibufarray(lenbufarray+1:lenbufarray+npoint)
         lenbufarray = lenbufarray+npoint
         if ( debug ) write(irefwr,1) 'lenbufarray/2', lenbufarray

!        --- Store new coordinates read into array coorglobnew

         if ( debug ) write(irefwr,*) 'before mshrefpar4'
         call mshrefpar4 ( coorglobnew, ibufarray(3), imapping(1), &
                           npoint, ndim )
         if ( debug ) write(irefwr,*) 'after mshrefpar4'
         if ( ierror/=0 ) go to 1000

!        --- Get kmesh part g and kmesh part c from neighbor

         kmeshgcopy(iblock,1:nelgrp) = &
            ibufarray(lenbufarray+1:lenbufarray+nelgrp)

         lenkmeshc = 0
         do ielgrp = 1, nelgrp
            inpelm = Topgrp(ielgrp)%inpelm
            lenkmeshc = lenkmeshc+inpelm*kmeshgcopy(iblock,ielgrp)
         end do  ! ielgrp = 1, nelgrp

         lenbufarray = lenbufarray+nelgrp
         kmeshccopy(iblock,1:lenkmeshc) = &
            ibufarray(lenbufarray+1:lenbufarray+lenkmeshc)
         lenbufarray = lenbufarray+lenkmeshc

!        --- Store contents of kmeshm from other nodes into array kmeshmcopy

         lenbuf = ibufarray(lenbufarray+1)
         kmeshmcopy(iblock,1:lenbuf) = &
            ibufarray(lenbufarray+1:lenbufarray+lenbuf)

         if ( debug ) then
            write(irefwr,1) 'iblock, lenbuf, lenbufarray', &
                             iblock, lenbuf, lenbufarray
            call prinin ( ibufarray(lenbufarray+1), lenbuf, &
                          'ibufarray' )
         end if  ! ( debug )

         lenbufarray = lenbufarray+lenbuf+1

      end do  ! i = 2, iacnodes

      if ( debug ) then
         call prinin1 ( kmeshgcopy, nelgrp, iacnodes, &
                        'kmeshgcopy' )
         call prinin1 ( kmeshccopy, lenkmeshcglnw, iacnodes, &
                        'kmeshccopy' )
      end if  ! ( debug )

!     --- Make a new array kmeshmglob from old kmeshmglob and information
!         received from all nodes

      if ( debug ) write(irefwr,*) 'before mshrefpar5'
      call mshrefpar5 ( kmeshmglob, kmeshm, &
                        kmeshmcopy, kmeshmglobnew, lenkmeshmglob )
      if ( debug ) write(irefwr,*) 'after mshrefpar5'
      if ( ierror/=0 ) go to 1000

      if ( lenkmeshn>0 ) then

!     --- lenkmeshn>0 copy kmeshn in ibufarray

         write(irefwr,*) '3d has not yet been implemented in mshrefpar3'
         write(irefwr,*) 'Please contact SEPRA'
         ierror = 1
         go to 1000

      end if  ! ( lenkmeshn>0 )

!     --- Make a new array kmeshcglob from information
!         received from all nodes

      if ( debug ) write(irefwr,*) 'before mshrefpar11'
      call mshrefpar11 ( iseqkm2, kmeshcglobnew, kmeshccopy, &
                         kmeshgcopy, lenkmeshcglnw )
      if ( debug ) write(irefwr,*) 'after mshrefpar11'
      if ( ierror/=0 ) go to 1000

      deallocate ( ibufarray, imapping, kmeshmcopy, kmeshccopy, &
                   kmeshgcopy, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'ibufarray' )

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( coorglobnew, npointglobnew, ndim, &
                        'coorglobnew' )
         write(irefwr,*) 'End mshrefpar3'

      end if  ! ( debug )
      call erclos ( 'mshrefpar3' )

      end subroutine mshrefpar3
