      subroutine mshrefpar5 ( kmeshmglobold, kmeshmold, &
                              kmeshmcopy, kmeshmglobnew, &
                              lenkmeshmglob )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 29-04-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create new array kmeshmglob from old one as well as information read
!     from all nodes
! **********************************************************************
!
!                       KEYWORDS
!
!     refine
!     parallel
!     curve
!     node
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulecpack
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshmglobold(*), kmeshmold(*), &
              kmeshmcopy(iacnodes,*), kmeshmglobnew(*), &
              lenkmeshmglob

!     kmeshmcopy     o    Work array of length (iacnodes,lenkmeshm) to store
!                         all curves that have new nodal point numbers per
!                         parallel node
!                         (.,1) contains the used length for node .
!                         The next positions contain for each curve stored:
!                         Pos. i: curve number
!                              i+1: number of nodal points in curve
!                              followed by nodal point numbers
!     kmeshmglobnew  o    Array kmeshmglob corresponding to new mesh
!     kmeshmglobold  i    Array kmeshmglob corresponding to old mesh
!     kmeshmold      i    Old array kmeshm
!     lenkmeshmglob i/o   Length of array kmeshmglob
!                         At input: declared length
!                         At output: length
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istartnode(iacnodes), ipstarttmp, istartcur(ncurvs+1), &
              istarold3, ipstartold, icurnr, nsubcv, &
              lenkmeshmtmp, lencurvenew, lencurveold, npointold
      integer, allocatable :: kmeshmtmp(:)

!     icurnr         Curve sequence number
!     ipstartold     Starting address in kmeshmold
!     ipstarttmp     Starting address in kmeshmtmp
!     istarold3      Starting address in kmeshmglobold part 3
!     istartcur      starting position of curves in kmeshmtmp
!     istartnode     Array of size iacnodes containing the next position
!                    in kmeshmcopy to be used for each parallel block
!     kmeshmtmp      Temporary array kmeshm in which the nodes of the new
!                    curves are stored
!     lencurvenew    Number of nodes in new curve
!     lencurveold    Number of nodes in old curve
!     lenkmeshmtmp   Length of array kmeshmtmp
!     npointold      Number of nodes in old mesh
!     nsubcv         Number of subcurves of a curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MSHREFPAR10    Fill array kmeshmglob in case of refinement
!     MSHREFPAR6     Make a new curve from the old global curve and all changes
!                    that have been read from the various nodes
!     PR0008         Print information of curves
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefpar5' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefpar5'
         write(irefwr,*) 'old global kmesh part m'
         call pr0008 ( kmeshmglobold, ncurvs )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      npointold = npointglob

      istartnode = 2  ! starting address in kmeshmcopy
      ipstarttmp = 1              ! starting address in kmeshmtmp
      istartcur(1) = ipstarttmp
      istarold3 = kmeshmold(2)    ! starting address of kmeshm part 3
      ipstartold = istarold3

      lenkmeshmtmp = (kmeshmold(3)+kmeshmold(1))*2

      if ( debug ) &
         write(irefwr,1) 'npointold, kelmq, lenkmeshmtmp', &
                          npointold, kelmq, lenkmeshmtmp

      allocate ( kmeshmtmp(lenkmeshmtmp), stat = error )
      if ( error/=0 ) call eralloc ( error, lenkmeshmtmp, 'kmeshmtmp' )
      if ( ierror/=0 ) go to 1000

!     --- First we fill single curves in temporary array kmeshmtmp
!         Loop over all curves

      do icurnr = 1, ncurvs

         if ( kelmq/=0 ) then

!        --- kelmq/=0, there are subcurves

            nsubcv = kmeshq(icurnr+1)-kmeshq(icurnr)

         else

!        --- kelmq = 0, there are no subcurves

            nsubcv = 0

         end if  ! ( kelmq/=0 )

         if ( debug ) write(irefwr,1) 'icurnr, nsubcv', icurnr, nsubcv

         if ( nsubcv==0 ) then

!        --- there are no subcurves, hence information must be stored

            call mshrefpar6 ( kmeshmold, icurnr, istartnode, &
                              kmeshmcopy, kmeshmtmp(ipstarttmp), &
                              kmeshmglobold(ipstartold), &
                              npointold, lencurvenew )
            ipstarttmp = ipstarttmp+lencurvenew

         else

!        --- Curve contains subcurves and must be skipped

            lencurvenew = 0

         end if  ! ( nsubcv==0 )
         lencurveold = kmeshmold(5+icurnr)-kmeshmold(4+icurnr)
         ipstartold = ipstartold+lencurveold
         istartcur(icurnr+1) = ipstarttmp

      end do  ! icurnr = 1, ncurvs

      if ( debug ) then
         call prinin ( kmeshmtmp, ipstarttmp-1, 'kmeshmtmp' )
         call prinin ( istartcur, ncurvs, 'istartcur' )
      end if  ! ( debug )

!     --- Fill array kmeshmglobnew

      call mshrefpar10 ( kmeshmglobnew, kmeshmglobold, &
                         kmeshmtmp, istartcur, lenkmeshmglob )

      deallocate ( kmeshmtmp, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'kmeshmtmp' )

1000  if ( debug ) then

!     --- Debug information

         call pr0008 ( kmeshmglobnew, ncurvs )
         write(irefwr,*) 'End mshrefpar5'

      end if  ! ( debug )
      call erclos ( 'mshrefpar5' )

      end
