      subroutine mshrefupdate ( reflvl, maxreflvl, basis_mesh, &
                                nodeextrael, meshnr, mesh_new )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 19-09-2013 new call to subroutine
!        version  2.0  date 22-11-2011 Major revision
!        version  1.2  date 20-09-2006 Fill kmeshm also in 3d
!
!   copyright (c) 2005-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Update the mesh as determined based on the previous solution
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     refine
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulemain
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer maxreflvl, reflvl, &
              nodeextrael(0:maxreflvl), &
              basis_mesh, meshnr

!     basis_mesh     i    Mesh sequence number of the basis mesh
!     maxreflvl      i    Maximum value of number of refinements
!     mesh_new       i    Mesh sequence number of the new mesh
!     meshnr         i    Mesh sequence number
!     nodeextrael    i    Array of size maxreflvl that contains the number
!                         of extra nodes per element defined as function
!                         of the refinement level
!     reflvl         i    Level of refinement
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lnintnodes, ielmfaces(1), lnparentnodes, idum(1), &
              nelem_basis, mesh_old, idimedges, &
              ireflvl, lencoornw, iinput(20), &
              lenkmscnew, lenkmsnnew, linear, &
              mesh_new, mmnmeshc, mmnmeshd, &
              nelemnew, npointnew, iseqkm1, iseqkm2, &
              mmrefmapnew, npoint_basis, lenkmsmnewsv, &
              lnenthwork, mmnmeshk, mmnmeshl, lenkmsmnew, lenkmscnewsv, &
              lenkmsnnewsv
      integer, allocatable :: kmeshcnew(:), kmeshnnew(:), &
                              iedges(:), enthwork(:), &
                              kmeshmnew(:), elements(:), refmapprev(:)
      double precision, allocatable :: coornw(:)
      logical :: extradebug

!     coornw         Array containing the new coordinates of the nodes
!     elements       Array of length levels(nlevels) (<=nelem)
!                    containing the reordered element sequence numbers
!                    So elements(1) contains the first element number to
!                    be used, elements(2) the second one and so on
!     extradebug     If true extra debugging statements are carried out
!     idimedges      Length of first dimension of array iedges
!                    (linear:1, quadratic: 3)
!     idum           General dummy variable
!     iedges         Work array of size nedges
!     ielmfaces      Contains faces of elements
!     iinput         integer input array
!                    See subroutine mshobstms23d
!     ireflvl        current level of refinement
!     iseqkm1        Sequence number with respect to km of first mesh
!     iseqkm2        Sequence number with respect to km of second mesh
!     kmeshcnew      New kmesh part c
!     kmeshmnew      New array kmeshm
!     kmeshnnew      New array kmeshn
!     lencoornw      Estimated length of array coornw
!     lenkmscnew     Length of array kmshcnew
!     lenkmscnewsv   Help parameter to store original value of lenkmsmcew
!     lenkmsmnew     Length of array kmshmnew
!     lenkmsmnewsv   Help parameter to store original value of lenkmsmnew
!     lenkmsnnew     Length of array kmshnnew
!     lenkmsnnewsv   Help parameter to store original value of kmshnnew
!     linear         Indicates whether the element is a linear simplex
!                    (true) or not (false)
!     lnenthwork     Maximal length of array enthwork
!     lnintnodes     Maximal length of array intnodes
!     lnparentnodes  Maximal length of array parentnodes
!     mesh_new       Mesh number of the new mesh
!     mesh_old       Mesh number of the old mesh
!     mmnmeshc       memory management sequence number of array nmeshc
!     mmnmeshd       memory management sequence number of array nmeshd
!     mmnmeshk       memory management sequence number of array nmeshk
!     mmnmeshl       memory management sequence number of array nmeshl
!     mmrefmapnew    memory management sequence number of array refmapnew
!     nelem_basis    Number of elements of the basis mesh
!     nelemnew       Number of elements in new mesh
!     npoint_basis   npoint of the basis mesh
!     npointnew      Sequence number of last generated node
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INICOPARR      Copy one sepran array into the other, which may already
!                    have been created before.
!     MSHCOPRED      Copy the essential parts of array kmesh1 into kmesh2
!     MSHOBSTMS05    Update kmeshm and kmeshn if necessary
!     MSHOBSTMS07    Make new arrays permanent for subroutines mshobstms
!                    and refineloc
!     MSHREFESTIMATE Compute some parameters to estimate array length in case of
!                    local refinement
!     MSHREFINENEW   Fill array refinenew
!     MSHREFINIT2    Initialize some vectors needed for adaptive local mesh
!                    refinement
!     MSHREFINTNODPO Set the pointers in array intnodes
!     MSHREFLOCALENT Refine an existing mesh locally and store the result in a
!     MSHREFLOCCHK   Put control integers around the temporary arrays
!                    in refineloc
!     MSHREFLOCCHK1  Check control integers around the temporary arrays
!                    in refineloc
!     MSHREFUPDATELN Estimate length of some arrays used in mshrefupdate
!     MSHSETMESHNR   Set the mesh number to the next available number
!     PR0003         Print KMESH parts c and g
!     PRININ         print 1d integer array
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPFILLKMESHAA Fill first part of array KMESH part aa
!     SEPFILLNMESHC  Fill array nmeshc
!     SEPFILLNMESHK  Fill array nmesh part k
!     SEPFILLNMESHL  Fill array nmesh part l
!     SEPRMMESHINFO  Remove mesh information
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefupdate' )
      debug = .false.
      extradebug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefupdate'
         write(irefwr,1) 'basis_mesh', basis_mesh
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Set mesh_old
!         To obtain a refined mesh we always start with the basis mesh

      mesh_old = basis_mesh
      kmesh => kmesh2d(basis_mesh)
      call sepfillkmesh(kmesh)
      nelem_basis = nelem
      npoint_basis = npoint
      iseqkm1 = iseqkm

      if ( debug ) &
         write(irefwr,1)  'iseqkm1, nelem_basis, npoint_basis', &
                           iseqkm1, nelem_basis, npoint_basis

!     --- Compute the length of some arrays to be created

      call mshrefupdateln ( nelem_basis, refinenew, &
                            nodeextrael, lnintnodes, lnparentnodes, &
                            lnenthwork, reflvl )

!     --- Create semi-permanent array parentnodes and initialize

      allocate ( refmapprev(2*nelem_basis), enthwork(lnenthwork), &
                 intnodesnew(lnintnodes), parentnodes(lnparentnodes) )

!     --- initialize enthwork, parentnodes and refmapprev

      enthwork = 0
      parentnodes = 0
      call mshrefinit2(refmapprev, nelem_basis)

!     --- Fill pointers in array intnodes

      call mshrefintnodpo ( intnodesnew, refinenew, &
                            nodeextrael, nelem_basis, reflvl )

!     --- Make local copy of refinenew (which will grow/shrink throughout
!         the loop over ireflvl)

      allocate ( refinenewloc(size(refinenew)) )
      refinenewloc = refinenew

      if ( extradebug ) &
            call prinin( intnodesnew, nelem_basis, &
                         'intnodesnew pointers' )

!     --- Loop over refinements

      if ( debug ) write(irefwr,1) 'reflvl', reflvl

      do ireflvl = 1, reflvl
         if (debug) &
            write(irefwr,*) '-----------------------------------------'
         call mshsetmeshnr ( mesh_new )

!        --- Copy old kmesh partly to new kmesh

         if (debug) then
            write(irefwr,1) 'mesh_old, mesh_new', mesh_old, mesh_new
            write(irefwr,1) 'ireflvl', ireflvl
         end if  ! (debug)
         call mshcopred ( kmesh, kmesh2d(mesh_new) )     ! copy part of mesh
         iseqkm2 = kmesh2d(mesh_new)
         iseqkm = iseqkm1
         if (debug) then
            write(irefwr,1) 'mesh_old, mesh_new', mesh_old, mesh_new
         end if  ! (debug)

!        --- we call an adapted version of mshreflocal in which we fill vectors
!            map and int_nodes. Array map contains per element of the finest
!            mesh
!            the corresponding basis element number and the required refine-
!            ment level for this basis mesh element.
!            In addition an array int_nodes is filled which contains per
!            element of the basis mesh which internal nodes are added per
!            refinement level.

!               --- activate arrays


!        --- activate and/or create arrays

         if ( debug ) write(irefwr,1) 'kelmaa', kelmaa
         if ( kelmaa==0 ) call sepfillkmeshaa
         if ( debug ) write(irefwr,1) 'nmesh(33)', nmesh(33)
         if ( nmesh(33)==0 ) call sepfillnmeshc

!        --- Reserve temporary space

         call mshrefestimate ( nelemnew, npointnew, lenkmscnew, linear, &
                               idimedges, lenkmsnnew, lencoornw, &
                               lenkmsmnew )

         length = lenkmscnew+4*nedges+nelem+lenkmsnnew+ &
                  lenkmsmnew +lencoornw+7*2
                                             ! control integers
         if ( debug ) then
            write(irefwr,1) 'nelem, nelemnew', nelem, nelemnew
            write(irefwr,1) 'length', length
         end if  ! ( debug )

!        --- Reserve space for temporary arrays

         allocate ( coornw(0:lencoornw+1), kmeshcnew(0:lenkmscnew+1), &
                    kmeshnnew(0:lenkmsnnew+1), iedges(nedges), &
                    kmeshmnew(0:lenkmsmnew), elements(0:nelem+1), &
                    stat = error )
         if ( error/=0 ) call eralloc ( error, length, 'coornw' )
         if ( ierror/=0 ) go to 1000

!        --- Compute starting addresses of temporary arrays
!            and provide with control integers

         call mshreflocchk ( kmeshcnew, kmeshnnew, &
                             coornw, kmeshmnew, lencoornw, &
                             lenkmscnew,  lenkmsnnew, lenkmsmnew )
         lenkmscnewsv = lenkmscnew
         lenkmsmnewsv = lenkmsmnew
         lenkmsnnewsv = lenkmsnnew

!        --- create temporary refmapnew

         allocate ( refmapnew(2*nelemnew) )

         if ( debug ) write(irefwr,*) 'before mshreflocalent'

         call mshreflocalent ( kmeshcnew(1), coornw(1), &
                               iedges, iseqkm2, elements, &
                               refinenewloc, lenkmscnew, &
                               intnodesnew, parentnodes, &
                               refmapprev, refmapnew, &
                               ireflvl, nodeextrael, npoint_basis, &
                               enthwork, lnintnodes, &
                               kmesh2d(mesh_new) )
         if ( debug ) write(irefwr,*) 'after mshreflocalent'

         if (extradebug) then
            call prinin(refmapprev, &
                         2*km(iseqkm1)%kmesh(9),'refmapprev')
            call prinin(refmapnew, &
                         2*km(iseqkm2)%kmesh(9),'refmapnew')
            call pr0003 ( kmesh2d(mesh_new), kmeshcnew(1) )
         end if

         if ( ndim==3 ) then

!        --- ndim = 3, kmeshn and nmeshk and nmeshl

            if ( nmesh(41)==0) call sepfillnmeshk

         end if
         if ( nmesh(42)==0) call sepfillnmeshl

         if (ndim>1) then

!        --- Update kmeshm and kmeshn if necessary
!            Copy contents of imap and rmap to final positions

            call mshobstms05 ( kmeshnnew(1), &
                               lenkmsnnew, iedges, linear, &
                               ielmfaces, npointnew, kmesh2d(mesh_new), &
                               idimedges, kmeshmnew(1), lenkmsmnew )

         end if

!        --- check control integers

         call mshreflocchk1 ( kmeshcnew, kmeshnnew, &
                              coornw, kmeshmnew, lencoornw, &
                              lenkmscnewsv,  lenkmsnnewsv,lenkmsmnewsv )

!        --- Make new arrays permanent

         nelemnew = km(iseqkm2)%kmesh(9)
         iinput(4) = npointnew
         iinput(7) = lenkmsmnew
         iinput(8) = lenkmsnnew
         iinput(12) = lenkmscnew
         call mshobstms07 ( kmesh2d(mesh_new), coornw(1), &
                            kmeshcnew(1), kmeshnnew(1), &
                            kmeshmnew(1), iinput )

!        --- Remove intermediate mesh

         if ( mesh_old/=meshnr .and. mesh_old/=basis_mesh ) then
            if (debug) &
                write(irefwr,*)'removing mesh, mesh_old = ', mesh_old

!           --- mesh_old # meshnr, hence we have an intermediate mesh
!               Remove this mesh

            call seprmmeshinfo ( kmesh2d(mesh_old) )

         end if  ! ( mesh_old/=meshnr )

!        --- Set mesh_old equal to mesh_new

         mesh_old = mesh_new

!        --- Remove old refinenewloc

         deallocate ( refinenewloc )

!        --- Create new refinenewloc
!            No need to set mmrefinenewloc = 0, it is saved throughout loop

         allocate ( refinenewloc(nelemnew) )

!        --- Fill new refinenewloc using refmapnew

         call mshrefinenew ( refinenewloc, refmapnew, nelemnew)

!        --- Remove old refmapprev

         deallocate ( refmapprev )

!        --- Create new refmapprev by copying refmapnew

         allocate ( refmapprev(size(refmapnew)) )
         refmapprev = refmapnew

!        --- Remove temporary arrays

         deallocate ( kmeshcnew, kmeshnnew, elements, &
                      iedges, coornw, kmeshmnew, &
                      stat = error )
         if ( error/=0 ) call erdealloc ( error, 'coornw' )

!        --- Remove old refmapnew

         deallocate ( refmapnew )

         if (debug) &
            write(irefwr,*) '-----------------------------------------'

         kmesh => kmesh2d(mesh_old)
         call sepfillkmesh(kmesh)
         iseqkm1 = iseqkm

      end do

!     --- Copy refmapprev to refmapold and remove refmapprev

      deallocate ( refmapold )

      allocate ( refmapold(size(refmapprev)) )
      refmapold = refmapprev

!     --- Remove local permanent arrays

      deallocate ( refinenewloc, refmapprev, enthwork )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrefupdate'

      end if
      call erclos ( 'mshrefupdate' )

      end subroutine mshrefupdate
