      subroutine mshshift2d5 ( skipnod, iworkpoint, coornew, nmap, eps, &
                               iwork, edgemoved )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.1  date 14-11-2013 Extra debug
!        version  3.0  date 22-05-2013 Extra parameter markbnedges
!        version  2.1  date 21-11-2012 Correction nodes near curve
!
!   copyright (c) 2011-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Shift node of intersecting curve onto boundary if
!     node is close (depending on eps) to boundary
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     triangle
!     subdivision
!     adaptation
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer skipnod(*), iworkpoint(*), nmap, &
              iwork(nelem,*), edgemoved(*)
      double precision coornew(ndim,*), eps

!     coornew       i/o   Array containing the new coordinates of the nodes
!     edgemoved     i/o   Array of size nedges indicating if nodes on an edge
!                         have been moved (1) or not (0)
!     eps            i    Defines when the intersection is close to a node
!     iwork          i    integer work array
!                         See subroutine mshobstms2d
!     iworkpoint    i/o   Work array of length npointnew
!                         This array is filled as follows:
!                         iworkpoint(i) = 0 if node i outside obstacle
!                         iworkpoint(i) = 1 if node i inside obstacle
!                         iworkpoint(i) = 2 if node i inside obstacle (near
!                                           boundary)
!                         iworkpoint(i) = 3 if node i on obstacle
!     nmap           i    length of first parameter of imap and rmap
!     skipnod       i/o   Array of npoint long indicating if a node has been
!                         moved or not
!                         See subroutine mshshift2d
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, numnodesb
      integer, allocatable :: nodesb(:), nodesmv(:,:)
      double precision, allocatable :: nodesfac(:)

!     i              Counting variable
!     nodesb         Set of nodes B of nodes on edges that are intersected
!                    where the node closest to the boundary is used
!     nodesfac       Contains factor for moving along edge, measured from
!                    first node
!     nodesmv        Contains set of nodes to be moved
!                    (1,.) node to be moved
!                    (2,.) second node on edge
!     numnodesb      Number of nodes that are to be moved
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MSHSHIFT2D3    Mark edges of which one of the nodes has been moved
!     MSHSHIFT2DALLW Create the set of nodes that may be moved
!     MSHSHIFT2DEL   Fill array nodesmv with nodes that must be moved and
!                    corresponding node at other side of the edge
!                    Fill array nodesfac with factors for moving
!     MSHSHIFT2DNOD  Move nodes along edges or curves depending on skipnod
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshshift2d5' )
      debug = .false. .and. ioutp>=0

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshshift2d5'
         write(irefwr,2) 'eps', eps
         call prinin ( skipnod, npoint, 'skipnod' )
  2      format ( a, 1x, (5d12.4) )
  4      format ( i6, 2x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- create temporary arrays

      allocate ( nodesb(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'nodesb' )
      if ( ierror/=0 ) go to 1000

!     --- Create the set of nodes that may be moved

      call mshshift2dallw ( iwork, nodesb, numnodesb )

      if ( debug ) call prinin ( nodesb, numnodesb, 'nodesb' )

!     --- Fill array nodesmv with nodes that must be moved and corresponding
!         node at other side of the edge
!         Fill array nodesfac with factors for moving

      allocate ( nodesmv(2,numnodesb), nodesfac(numnodesb), &
                 stat = error )
      if ( error/=0 ) call eralloc ( error, 3*numnodesb, 'nodesmv' )
      if ( ierror/=0 ) go to 1000

      call mshshift2del ( iwork, nodesb, &
                          numnodesb, eps, nodesmv, nodesfac, skipnod )

      if ( debug ) then
         call prinin1 ( nodesmv, numnodesb, 2, 'nodesmv' )
         call prinrl ( nodesfac, numnodesb, 'nodesfac' )
      end if  ! ( debug )

!     --- Move nodes along edges or curves depending on skipnod

      call mshshift2dnod ( numnodesb, nodesmv, nodesfac, coornew, &
                           skipnod, nmap, ndim, &
                           iworkpoint )

!     --- Mark edges that are connected to nodes that have been moved

      call mshshift2d3 ( skipnod, iwork, edgemoved )

!     --- Remove temporary arrays

      deallocate ( nodesb, nodesmv, nodesfac, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'nodesb' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Nodes that are moved'
         do i = 1, npoint
            if ( skipnod(i)==3 ) then
               write(irefwr,4) i, coornew(:,i)
            end if  ! ( skipnodes(i)==3 )
         end do  ! i = 1, npoint

         write(irefwr,*) 'End mshshift2d5'

      end if  ! ( debug )
      call erclos ( 'mshshift2d5' )

      end
