      subroutine mshshift2dallw ( iwork, nodesb, numnodesb )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 17-09-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create the set of nodes that may be moved
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     triangle
!     adaptation
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(*), nodesb(*), numnodesb

!     iwork          i    integer work array
!                         See subroutine mshobstms2d
!     nodesb         o    Set of nodes B of nodes on edges that are intersected
!                         where the node closest to the boundary is used
!     numnodesb      o    Number of nodal points in array nodesb
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, allocatable :: imarknodes(:), imarkelem(:)

!     imarkelem      Array of length nelem to mark all elements as being
!                    inside or outside the obstacle
!                    0: element is either inside the obstacle or
!                       contains at least one node of the boundary
!                    1: element is completely outside the obstacle
!                    2: element is completely outside the obstacle
!                       or all nodes belong to set B
!                    3: all nodes of element belong to B
!     imarknodes     Work array of length npoint to mark nodes
!                    -1  node inside obstacle
!                     0  node outside obstacle
!                     1  node belongs to set B
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MSHSHIFT2DCOR  Remove those elements from set B where all nodes are
!                    inside the obstacle
!     MSHSHIFT2DSET  Fill the set B with the nodes on the intersected edges
!                    that are closest to the interface
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshshift2dallw' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshshift2dallw'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( imarknodes(npoint), imarkelem(nelem), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint+nelem, 'imarknodes' )
      if ( ierror/=0 ) go to 1000

!     --- Select the nodes of the edges that cross the interface with minimum
!         distance to the interface
!         Mark all elements completely outside the obstacle (1)

      call mshshift2dset ( iwork, nodesb, &
                           numnodesb, imarknodes, imarkelem )
      if ( debug ) call prinin ( nodesb, numnodesb, 'nodesb/1' )

!     --- Add those elements to the set of outside elements
!         that satisfy that all points are either outside the obstacle
!         or in the set B. If three points are in B the element is skipped

      call mshshift2dcor ( nodesb, numnodesb, imarknodes, imarkelem )

      if ( debug ) call prinin ( nodesb, numnodesb, 'nodesb' )

      deallocate ( imarknodes, imarkelem, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'imarknodes' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshshift2dallw'

      end if  ! ( debug )
      call erclos ( 'mshshift2dallw' )

      end
