      subroutine mshshift2dcor ( nodesb, numnodesb, imarknodes, imarkelem )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-10-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Remove those elements from set B where all nodes are inside the obstacle
!     If a node in set B is only in elements that are completely inside the
!     obstacle or are just removed it is removed from the set B
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     triangle
!     adaptation
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nodesb(*), numnodesb, imarknodes(*), imarkelem(*)

!     imarkelem     i/o   Array of length nelem to mark all elements as being
!                         inside or outside the obstacle
!                         0: element is either inside the obstacle or
!                            contains at least one node of the boundary
!                         1: element is completely outside the obstacle
!                         2: element is completely outside the obstacle
!                            or all nodes belong to set B
!                         3: all nodes of element belong to set B (not used)
!     imarknodes    i/o   Work array of length npoint to mark nodes
!                         -1  node inside obstacle
!                          0  node outside obstacle
!                          1  node belongs to set B
!                          2  node belongs to set B but must be skipped
!     nodesb        i/o   Set of nodes B of nodes on edges that are intersected
!                         where the node closest to the boundary is used
!     numnodesb     i/o   Number of nodal points in array nodesb
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numedges, numelmsT, lensetE
      integer, allocatable :: setE(:), setT(:)
      logical plotdebug, removenodes

!     lensete        Declared length of array setE
!     numedges       number of edges in set E
!     numelmst       number of elements in set T
!     plotdebug      If true plots must be made for debugging
!     removenodes    if true nodes  must be removed from nodesb
!     sete           Set of edges that have all nodes in B and are at the
!                    boundary of setT
!     sett           triangles with all points outside the obstacle
!                    or in set B
!                    Triangles with all nodes in B are skipped
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MSHSHIFT2DCR1  Find all triangles with all points inside the obstacle
!                    or in set B. Store in setT
!     MSHSHIFT2DCR2  Find the set of edges that have all nodes in B and are at
!                    the boundary of setT
!                    Only those edges that must be removed of the set B are
!                    stored
!     MSHSHIFT2DCR3  Remove those nodes from nodesB that are in edges that are
!                    lying in 2 triangles with imarkelem >=2
!     PLOTMARKELNOD  Plot information from arrays imarkelem and imarknodes
!                    defining the status of elements and nodes in case of
!                    an obstacle
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshshift2dcor' )
      debug = .false. .and. ioutp>=0
      plotdebug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshshift2dcor'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Find all triangles with all points inside the obstacle
!         or in set B
!         Triangles with all nodes in B are skipped
!         Store in setT

      allocate ( setT(nelem), stat = error )
      if ( error/=0 ) call eralloc ( error, nelem, 'setT' )
      if ( ierror/=0 ) go to 1000

      call mshshift2dcr1 ( numnodesb, nodesb, numelmsT, &
                           setT, imarkelem, imarknodes )

      if ( debug ) call prinin ( nodesB, numnodesb, 'nodesB/1' )

!     --- Find the set of edges that have all nodes in B and are at the
!         boundary of setT. Store in set E

      lensetE = 3*numnodesb
      allocate ( setE(lensetE), stat = error )
      if ( error/=0 ) call eralloc ( error, lensetE, 'setE' )
      if ( ierror/=0 ) go to 1000

      call mshshift2dcr2 ( numelmsT, setT, imarknodes, &
                           setE, numedges, lensetE, &
                           removenodes, nodesb, numnodesb )

      if ( plotdebug ) call plotmarkelnod ( imarknodes, imarkelem )

      deallocate ( setT, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'setT' )
      if ( ierror/=0 ) go to 1000

!     --- Remove those nodes from nodesB that are in edges that are
!         lying in 2 triangles with imarkelem >=2, &
!         i.e. all nodes of the triangles are either outside the obstacle
!         or are member of setB

      call mshshift2dcr3 ( numnodesb, nodesb, imarknodes, &
                           setE, numedges, imarkelem, removenodes )
      if ( debug ) call prinin ( nodesB, numnodesb, 'nodesB/2' )

      deallocate ( setE, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'setE' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshshift2dcor'

      end if  ! ( debug )
      call erclos ( 'mshshift2dcor' )

      end
