      subroutine mshshifttest ( npoint, imark, skipnod )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 19-09-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Test if all nodes that have to be shifted are moved to the interface
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     level_set
!     check
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint, imark(npoint), skipnod(npoint)

!     imark          i    Marks nodal points
!                         0: node must not be moved
!                         1: node must be moved
!     npoint         i    Number of nodal points
!     skipnod        i    Array of npoint long in which the nodes are marked
!                         which are not allowed to be moved
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nstore1, nstore2
      integer, allocatable :: istore1(:), istore2(:)

!     error          Return value of allocate or deallocate
!     i              Counting variable
!     istore1        work array to store a set of nodes
!     istore2        work array to store a set of nodes
!     nstore1        Number of nodes in istore1
!     nstore2        Number of nodes in istore2
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3113   nodes are marked as not to be moved but are moved
!    3114   nodes are marked as to be moved but are kept in place
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshshifttest' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshshifttest'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- create temporary arrays

      allocate ( istore1(npoint), istore2(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, 2*npoint, 'istore1' )
      if ( ierror/=0 ) go to 1000

      nstore1 = 0
      nstore2 = 0

      do i = 1, npoint

         if ( imark(i)>0 .and. skipnod(i)==0 ) then

!        --- Node should be moved but has not

            nstore1 = nstore1+1
            istore1(nstore1) = i

         else if ( imark(i)==0 .and. skipnod(i)==3 ) then

!        --- Node has been moved but should not

            nstore2 = nstore2+1
            istore2(nstore2) = i

         end if
      end do  ! i = 1, npoint

      if ( nstore1>0 ) then

!     --- There are nodes that have not been moved

         call errsub ( 3114, 0, 0, 0 )
         call prinin ( istore1, nstore1, 'nodes' )

      end if  ! ( nstore1>0 )

      if ( nstore2>0 ) then

!     --- There are nodes that have not been moved

         call errsub ( 3113, 0, 0, 0 )
         call prinin ( istore2, nstore2, 'nodes' )

      end if  ! ( nstore2>0 )

!     --- Remove temporary arrays

      deallocate ( istore1, istore2, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'istore1' )

1000  call erclos ( 'mshshifttest' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshshifttest'

      end if  ! ( debug )

      end
