      subroutine mshsmoothquad ( coornew, isurnr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-02-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Adapt displacement of nodes of quadratic elements
!     such that mid-side points remain in the middle
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     2d
!     3d
!     smoothing
!     free_surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isurnr
      double precision coornew(3,*)

!     coornew       i/o   Array containing the new coordinates of the nodes
!     isurnr         i    Surface sequence number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numcurvs, surstart, i, icurnr, k1, k2, k, &
              nodp, istart, inpelm, ielem, istar3
      integer, allocatable :: iwork(:)

!     i              Counting variable
!     icurnr         Curve sequence number
!     ielem          Element number
!     inpelm         number of points per element
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     istart         starting address of surface in kmeshn
!     iwork          integer work array of length npoint to mark points that
!                    may be moved (0) or not (1)
!     k              Counting variable
!     k1             First point of curve in KMESH part m
!     k2             Last point of curve in KMESH part m
!     nodp           Nodal point sequence number
!     numcurvs       Number of curves in surface
!     surstart       Starting address of surface in kmeshs
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIINFSURF     Find starting address of surface isurnr in array kmeshn
!                    Also find number of element and number of points per
!                    element in isurnr
!     MSHADAPMIDS    Adapt coordinates of mid side points of one element such
!                    that they are in the centre of the side
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2910   Not enough space
!    2911   Not possible to deallocate
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array iwork
!     Mark all nodes on the curves by -1
!     For all elements in surface do
!        Adapt midside coordinates if node not on boundary
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshsmoothquad' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshsmoothquad'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( iwork(npoint), stat = error )
      if ( error/=0 ) then

!     --- error#0, so not enough space

         call errint ( npoint, 1 )
         call errint ( error, 2 )
         call errchr ( 'iwork', 1 )
         call errsub ( 2910, 2, 0, 1 )
         go to 900

      end if  ! ( error/=0 )

!     --- Clear array iwork

      iwork = 0

!     --- Mark all nodes on the curves by -1

      numcurvs = kmeshs(isurnr+1)-kmeshs(isurnr)
      surstart = kmeshs(isurnr)
      istar3 = kmeshm(2)-1   ! starting address of node numbers
      do i = 1, numcurvs

         icurnr = abs(kmeshs(surstart+i))  ! curve number
         k1 = kmeshm(4+icurnr)
         k2 = kmeshm(5+icurnr)-1
         do k = k1, k2
            nodp = kmeshm(istar3+k)         ! node on curve
            if ( nodp>0 ) iwork(nodp) = -1  ! mark node
         end do  ! k = k1, k2

      end do  ! i = 1, numcurvs

!     --- Find starting address of surface in kmeshn

      call iniinfsurf ( kmeshn, istart, inpelm, inelem, nsurfs, &
                        isurnr )
      istart = istart+1  ! real starting address

!     --- For all elements in surface do

      do ielem = 1, inelem

!     --- Adapt midside coordinates if node not on boundary

         call mshadapmids ( kmeshn(istart), ishape, iwork, coornew )
         if ( ierror/=0 ) go to 1000
         istart = istart+inpelm     ! raise istart

      end do  ! ielem = 1, inelem

900   deallocate ( iwork, stat = error )
      if ( error/=0 ) then

!     --- error#0, problem with deallocate

         call errint ( error, 1 )
         call errchr ( 'iwork', 1 )
         call errsub ( 2911, 1, 0, 1 )

      end if  ! ( error/=0 )

1000  call erclos ( 'mshsmoothquad' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshsmoothquad'

      end if  ! ( debug )

      end
