      subroutine mshsmoothsurf ( isurnr, coornew )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.5  date 11-12-2017 New storage of intmat
!        version  1.4  date 17-12-2015 New call to incommat
!        version  1.3  date 07-08-2014 use seprmprobinfo
!
!   copyright (c) 2009-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute displacement of surface nodes, given the displacement
!     of the boundary by Laplacian smoothing
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     2d
!     3d
!     smoothing
!     free_surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemnsub
      use sepmodulemain
      use sepmoduleintmat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isurnr
      double precision coornew(3,*)

!     coornew        i    Array containing the new coordinates of the nodes
!     isurnr         i    Surface sequence number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iincommt(20), irhsdlc(5), isol(5), idim, isurfaces(1), &
              nsurfaces, nnodes
      integer, target ::  intmatlc
      integer, pointer :: kproblc
      integer, allocatable :: nodes(:)

!     idim           coordinate direction
!     iincommt       Integer input array containing information concerning
!                    the structure of the large matrix
!                    See subroutine read00
!     intmatlc       Standard SEPRAN array of length 5 containing
!                    information concerning the structure of the matrix.
!                    See the Programmer's Guide 13.5 for a
!                    complete description.
!     irhsdlc          Standard SEPRAN array, containing information of the
!                    large vector
!     isol           Standard SEPRAN array containing information of the
!                    solution array
!     isurfaces      Array containing the set of surfaces
!     kproblc        Standard SEPRAN array, containing information of
!                    the problem
!     nnodes         Number of nodes
!     nodes          Contains the node numbers of the surface
!     nsurfaces      Number of surfaces in array isurfaces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MSHSMOOTHADCR  Adapt coordinates of surface using the displacement vector
!     MSHSMOOTHBC    Fill boundary conditions due to displacement of boundary
!                    in solution vector
!     MSHSMOOTHMAT   Build matrix and zero right-hand side for surface smoothing
!     MSHSMOOTHQUAD  Adapt displacement of nodes of quadratic elements
!                    such that mid-side points remain in the middle
!     MSHSMOOTHSOLV  Solve linear system corresponding to displacement of
!                    surface nodes
!     MSHSMOOTHSRPRB Fill array kprob with respect to surface smoothing
!     PRINTSURFACE   Print the contents of array kmeshn for one surface
!     SEPCOMMAT      Actual body of COMMAT and MATSTRUC
!     SEPRMINTMT     Remove array with sequence number iseq from intmt
!     SEPRMMATRIX    Remove array with sequence number iseq from realmt
!     SEPRMPROBINFO  Remove array with sequence number kproblc from kp
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
!     SEPSURFSNOD    Store all nodes in the surfaces stored in array surfaces
!                    in array nodes
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Define array kprob for this surface
!     Create matrix for smoothing
!     For all coordinate directions
!        Fill boundary conditions
!        Solve system of equations
!        Adapt coordinates
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshsmoothsurf' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshsmoothsurf'
         write(irefwr,1) 'isurnr', isurnr
         call printsurface ( isurnr )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Define array kprob for this surface

      kproblc => kprob2d(maxmesh+1)
      kprob => kproblc
      call mshsmoothsrprb ( isurnr, kmeshs, kmeshs(nsurfs+2), ishape )
      if ( ierror/=0 ) go to 1000

!     --- Create corresponding array intmat

      iincommt = 0
      iincommt(2) = 5    ! symmetric compact matrix
      iincommt(5) = 1    ! iprob
      intmatlc = 0

      intmat => intmatlc

      call sepcommat ( iincommt )

!     --- Create matrix for smoothing

      irhsdlc = 0
      call mshsmoothmat ( irhsdlc )

!     --- allocate arrays

      allocate ( nodes(npoint), stat = error)
      if ( error/=0 ) call eralloc ( error, npoint, 'nodes' )
      if ( ierror/=0 ) go to 1000

!     --- Store all nodes of surface in array nodes

      nsurfaces = 1           ! only one surface
      isurfaces(1) =  isurnr  ! store surface number
      call sepsurfsnod ( nsurfaces, nodes, nnodes, isurfaces )

!     --- For all coordinate directions

      isol = 0
      do idim = 1, 3

!     --- Fill boundary conditions

         call mshsmoothbc ( idim, isol, coornew, nodes, nnodes )

!        --- Solve system of equations

         call mshsmoothsolv ( isol, irhsdlc )

!        --- Adapt coordinates

         call mshsmoothadcr ( idim, isol, coornew, nodes, nnodes )

      end do  ! idim = 1, 3

!     --- In case of quadratic elements, the mid points should
!         remain in the middle of the elements

      if ( ishape/=3 .and. ishape/=5 ) then

!     --- extra nodes in element, hence adapt the new coordinates

         call mshsmoothquad ( coornew, isurnr )

      end if  ! ( ishape/=3 .and. ishape/=5 )

!     --- Delete temporary arrays

      call seprmsolinfo ( isol(1) )  ! solution
      call seprmsolinfo ( irhsdlc(1) )  ! right-hand side
      call seprmmatrix ( stiffmat )
      call seprmintmt ( intmatlc )
      call seprmprobinfo ( kproblc )

      kprob => kprob2d(meshnr)

      deallocate ( nodes, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'nodes' )

1000  if ( debug ) then

!     --- Debug information

         call printsurface ( isurnr )
         write(irefwr,*) 'End mshsmoothsurf'

      end if  ! ( debug )
      call erclos ( 'mshsmoothsurf' )

      end subroutine mshsmoothsurf
