      subroutine mshvolbcord1 ( bcord, nsubsf, iunder, iupper )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 23-08-2010 Remove arrays iwork/work from parm list
!        version  1.0  date 18-07-2008
!
!   copyright (c) 2008-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill coordinates of boundary of volume into array bcord
!     One surface only
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     volume
!     boundary
!     coordinate
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nsubsf, iunder, iupper
      double precision bcord(*)

!     bcord         i/o   array containing the coordinates of the boundaries
!     iunder         i    Lower bound in array bcord
!     iupper         i    Upper bound in array bcord
!     nsubsf         i    Number of subsurfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ih, ih1, index, istart, isubsf, isur, &
              jstart, leng1, npoloc
      integer, allocatable :: iwork(:)
      double precision, allocatable :: work(:)

!     i              Counting variable
!     ih             Help variable to store a constant temporarily
!     ih1            Help variable to store a constant temporarily
!     index          Record sequence number
!     istart         First position used at a part
!     isubsf         Sequence number of subsurface
!     isur           surface number
!     iwork          integer work array
!     jstart         starting position
!     leng1          length indicator points
!     npoloc         Number of nodes in subsurface
!     work           Work array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     DCOPY          Copy double precision dx to double precision dy
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL1        Print 2d real vector
!     READFL         Read integer array from a direct access SEPRAN file
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2910   Not enough space
!    2911   Not possible to deallocate
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshvolbcord1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshvolbcord1'
         write(irefwr,1) 'isurnr', isurnr
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      length = abs(kmeshh(1,isurnr+1))*ndim

      iupper = iunder+length-1
      if ( nsubsf==0 ) then

!     --- nsubsf = 0, No subsurfaces

         call dcopy ( length, kmhsur(isurnr)%coorsur, 1, bcord(iunder), 1 )

      else

!     --- Surface consists of subsurfaces
!         Read information of subsurfaces and store in bcord

         istart = kmeshr(isurnr)
         jstart = kmeshr(nsurfs+1) - nsurfs-1 + istart
         do isubsf = 1, nsubsf

            isur = abs ( kmeshr(istart+isubsf) )
            if ( debug ) then

!           --- Debug statements

               write(irefwr,*) 'Subsurface ', isubsf, ' is ', isur

            end if
            index = kmeshr(jstart+isubsf)
            npoloc = kmeshh(1,isur+1)
            leng1 = npoloc*ndim

            allocate ( iwork(npoloc), work(leng1), stat = error )
            if ( error/=0 ) then

!           --- error#0, so not enough space

               call errint ( npoloc+2*leng1, 1 )
               call errint ( error, 2 )
               call errchr ( 'iwork', 1 )
               call errsub ( 2910, 2, 0, 1 )
               go to 1000

            end if  ! ( error/= 0 )

!           --- Read local numbering and store in array iwork of length
!               npoloc

            call readfl ( index, 1, npoloc, 1, 3, iwork )
            if ( debug ) call prinin ( iwork, npoloc, 'iwork' )

!           --- read local array coor and store in array work
!               Length: npoloc * ndim

            call dcopy ( leng1, kmhsur(isur)%coorsur, 1, work, 1 )

!           --- Put co-ordinates from local array coor into global bcord

            do i = 1, npoloc
               ih = 3 * (i-1)
               ih1 = 3 * (iwork(i)-1)+iunder-1
               bcord(ih1+1:ih1+3) = work(ih+1:ih+3)
            end do  ! i = 1, npoloc

            deallocate ( iwork, work, stat = error )
            if ( error/= 0 ) then

!           --- error#0, problem with deallocate

               call errint ( error, 1 )
               call errchr ( 'iwork', 1 )
               call errsub ( 2911, 1, 0, 1 )
               go to 1000

            end if  ! ( error/= 0 )

         end do  ! isubsf = 1, nsubsf

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iunder, iupper', iunder, iupper
         write(irefwr,1) 'iunder/3, iupper/3', iunder/3+1, iupper/3
         call prinrl1 ( bcord(iunder), (iupper-iunder)/3+1, 3, 'bcord' )
         write(irefwr,*) 'End mshvolbcord1'

      end if  ! ( debug )
      call erclos ( 'mshvolbcord1' )

      end
