      subroutine sepadapcent3d
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Adapt coordinates of 3d mesh once the mesh has been deformed
!     Goal is to keep mid-side points ate the centre of the sides
!     and centre points in the centroid
!     Old subroutine mshadapcent3d
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     3d
!     smoothing
!     free_surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istart, inpelm, ielgrp, irelem, isurnr, &
              ikelmc, i, nodp
      integer, allocatable :: imark(:)

!     i              Counting variable
!     ielgrp         element group number
!     ikelmc         position in kmesh part c
!     imark          Array of size npoint to indicate if nodes may be moved
!                    (0) or not (1)
!     inpelm         number of points per element
!     irelem         Relative element sequence number with respect to
!                    element group
!     istart         Starting address i kmeshn
!     isurnr         Surface sequence number
!     nodp           Nodal point sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHADAPCENT12  Adapt coordinates of quadratic tetrahedron
!     MSHADAPCENT14  Adapt coordinates of tri-quadratic hexahedron
!     MSHADAPCENT17  Adapt coordinates of tri-linear hexahedron of shape 17
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
!    2910   Not enough space
!    2911   Not possible to deallocate
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepadapcent3d' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepadapcent3d'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( imark(npoint), stat = error )
      if ( error/=0 ) then

!     --- error#0, so not enough space

         call errint ( npoint, 1 )
         call errint ( error, 2 )
         call errchr ( 'imark', 1 )
         call errsub ( 2910, 2, 0, 1 )
         go to 900

      end if  ! ( error/=0 )

!     --- First mark all nodes on the surfaces
!         They may not be moved

      imark = 0

      istart = 2*nsurfs
      do isurnr = 1, nsurfs

         inpelm = kmeshn(isurnr)
         inelem =  kmeshn(isurnr+nsurfs)

         do i = 1, inelem*inpelm

            nodp = kmeshn(istart+i)
            if ( nodp>0 ) imark(nodp) = 1

         end do  ! i = 1, inelem*inpelm

         istart = istart+inelem*inpelm

      end do  ! isurnr = 1, nsurfs

!     --- Next move centroid nodes, depending on shape number

      ikelmc = 1             ! starting address in kmeshc
      do ielgrp = 1, nelgrp

         ishape = kmeshf(ielgrp)
         inelem = Topgrp(ielgrp)%inelem
         inpelm = Topgrp(ielgrp)%inpelm

         select case (ishape)

            case(1:11,13)

!           --- Shape = 1 to 11, 13, linear elements, no action

               ikelmc = ikelmc+inelem*inpelm

            case(12,15,16)

!           --- Shape = 12, 16, 16, quadratic tetrahedron, adapt coordinates

               do irelem = 1, inelem

                  call mshadapcent12 ( coor, kmeshc(ikelmc), imark, &
                                       ishape )
                  ikelmc = ikelmc+inpelm

               end do  ! irelem = 1, inelem

            case(14)

!           --- Shape = 14, triquadratic hexahedron, adapt coordinates

               do irelem = 1, inelem

                  call mshadapcent14 ( coor, kmeshc(ikelmc), imark )
                  ikelmc = ikelmc+inpelm

               end do  ! irelem = 1, inelem

            case(17)

!           --- Shape = 17, trilinear hexahedron with extra centroid, &
!               adapt centre

               do irelem = 1, inelem

                  call mshadapcent17 ( coor, kmeshc(ikelmc), imark )
                  ikelmc = ikelmc+inpelm

               end do  ! irelem = 1, inelem

            case default

!           --- Shape number not implemented (error)

               call errint ( ishape, 1 )
               call errsub ( 800, 1, 0, 0 )
               ikelmc = ikelmc+inelem*inpelm

         end select  ! case (ishape)

      end do  ! ielgrp = 1, nelgrp

900   deallocate ( imark, stat = error )
      if ( error/=0 ) then

!     --- error#0, problem with deallocate

         call errint ( error, 1 )
         call errchr ( 'imark', 1 )
         call errsub ( 2911, 1, 0, 1 )

      end if  ! ( error/=0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepadapcent3d'

      end if  ! ( debug )
      call erclos ( 'sepadapcent3d' )

      end
