      subroutine seplevelcrel3d ( levelset, lenicrossobst, &
                                  lenrcrossobst, iobst )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute all information concerning kmesh ad parts 3 and 4, i.e.
!     information of the cross sections of elements with level set function 0
!     and store in arrays icrossobst and rcrossobst
!     3d version
!     Old subroutine mshlevelcrel3d
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     element
!     crossing
!     cross_section
!     crossing_point
!     intersection
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer lenicrossobst, lenrcrossobst, iobst
      double precision  :: levelset(*)

!     iobst          i    Obstacle sequence number
!     lenicrossobst  o    Computed length of array icrossobst
!     lenrcrossobst  o    Computed length of array rcrossobst
!     levelset       i    Array of length npoint containing the value of the
!                         level set function in each node
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iielem, ielem, &
              ielgrp, inodes(3,12), numintersect, &
              iverts(12), istartrcross, iedge, istart, &
              istartedge,  istarticr, istarticross, nintersect, &
              numnodes, nvert, numinelmobst, numbnelmobst, &
              numbndobst, istart2, numcross, numelms
      double precision rverts(12)
      integer, allocatable :: iwork(:), iwork1(:), iwork2(:), iedges(:)
      save istarticross, istartrcross, numintersect

!     i              Counting variable
!     iedge          edge sequence number
!     iedges         work array of length nedges
!                    iedge(i) = 0 means that edge i has not been treated
!                    yet,  iedge(i) # 0 means that it has
!     ielem          Element number
!     ielgrp         element group number
!     iielem         Relative element number in element group
!     inodes         Array of length 3*nvert containing the node numbers
!                    of the edges as well as the edge number
!                    inodes(1,i) = first node of edge i
!                    inodes(2,i) = last node of edge i
!                    inodes(3,i) = edge number
!     istart         Starting address
!     istart2        Last position used in iwork2
!     istartedge     Starting position of edge in array nmeshd
!     istarticr      Starting address of cross section in array icrossobst
!     istarticross   Starting address of obstacle in array icrossobst
!     istartrcross   Starting address of cross section in array rcrossobst
!     iverts         Array of length nintersect containing the edge
!                    numbers of the edges that intersect the obstacle
!     iwork          Integer work array of length nelem
!                    iwork(i) = 1, means that element i is partly in
!                    obstacle i, otherwise iwork(i) = 0
!     iwork1         Integer work array of length npoint
!                    iwork1(i) = 1, means that node i is on the
!                    obstacle, iwork1(i) = 0, means not
!     iwork2         integer work array of size 4*nelem
!                    In iwork2 the second part of iobstcross is stored, &
!                    i.e. the information with respect to the elements
!     nintersect     Number of intersections found
!     numbndobst     Number of nodal points on the boundary of the obstacle
!     numbnelmobst   Number of elements that are partly within the obstacle
!     numcross       Number of cross sections of the obstacle with edges
!     numelms        Number of elements found that are crossed by the obstacle
!     numinelmobst   Number of elements that are completely within the obstacle
!     numintersect   Number of intersections
!     numnodes       Number of nodal points in array nodes
!     nvert          Number of edges to be considered
!     rverts         Contains the factors corresponding to the intersection
!                    points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     INIZEROLEV     Compute number of nodes in zero level set and starting
!                    position in ipointobst
!     MSHLEVELINT2D  Find the intersection of all sides in the element with
!                    the level set function value 0
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array icrossobst
!     For iobst := 1 (1) numobst
!        For ielgrp := 1 (1) nelgrp do
!           inelem := number of elements in element group ielgrp
!           ishape := shape number of elements in element group ielgrp
!           if ( ishape corresponds to 2 dimensional element ) then
!              For iielem := 1 (1) inelem
!                 ielem := element number
!                 If ( ielemobst(ielem) = -iobst ) then
!                    Compute the intersections of the element with the
!                    obstacles and store information in iwork and work
!        Update arrays icrossobst and rcrossobst
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seplevelcrel3d' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seplevelcrel3d'

      end if
      if ( ierror/=0 ) go to 1000

!     --- allocate work arrays

      allocate ( iwork(inelem), iwork1(npoint), iwork2(4*inelem), &
                 iedges(nedges), stat = error )
      if ( error/=0 ) call eralloc ( error, 5*inelem+npoint, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Clear array icrossobst

      icrossobst(1:numobst) = 0
      istarticross = numobst+1
      istartrcross = 0
      iwork = 0
      iwork1 = 0

      numintersect = 0

!     --- Fill array iwork

      istart = ielemobst(iobst)
      numinelmobst = ielemobst(istart)
      numbnelmobst = ielemobst(istart+1)+ielemobst(istart+2)
      istart = istart+2+numinelmobst
      iwork(ielemobst(istart+1:istart+numbnelmobst)) = 1

!     --- Fill array iwork1

      call inizerolev ( ipointobst, istart, numbndobst, iobst )
      iwork1(ipointobst(istart+1:istart+numbndobst)) = 1

!     --- Clear array iedges

      iedges(1:nedges) = 0

      istarticr = istarticross

!     --- For ielgrp := 1 (1) nelgrp do

      ielem = 0
      istart2 = 0
      numelms = 0
      do ielgrp = 1, nelgrp

!     --- inelem := number of elements in element group ielgrp

         inelem = Topgrp(ielgrp)%inelem
         ishape = sepgetshape ( ielgrp )
         if ( ishape>10 .and.ishape/=134 ) then

!        --- Three-dimensional shape
!            For iielem := 1 (1) inelem

            do iielem = 1, inelem

!           --- ielem := element number

               ielem = ielem+1

               if ( iwork(ielem)==1 ) then

!              --- iwork(ielem) = 1
!                  Compute the intersections of the element with the
!                  obstacles and store information in iwork and work

                  istartedge = nmeshd(ielem)
                  nedpel = nmeshd(istartedge)
                  nvert = 0
                  numcross = 0
                  do i = 1, nedpel
                     iedge = abs(nmeshd(istartedge+i))
                     if ( iedges(iedge)==0 ) then

!                    --- new edge found

                        nvert = nvert+1
                        istart = nmeshc(iedge)
                        numnodes = nmeshc(istart)
                        inodes(1,nvert) = nmeshc(istart+1)
                        inodes(2,nvert) = nmeshc(istart+numnodes)
                        inodes(3,nvert) = iedge
                        iedges(iedge) = -1

                     else if ( iedges(iedge)>0 ) then

!                    --- already existing edge found that contains intersection

                        numcross = numcross+1
                        iwork2(istart2+2+numcross) = iedges(iedge)

                     end if

                  end do

                  if ( nvert>0 ) then

!                 --- There are edges to be considered

                     call inichkarlen ( nvert, 12, 'inodes' )
                     if ( ierror/=0 ) go to 1000
                     call mshlevelint2d ( nvert, inodes, iverts, rverts, &
                                          nintersect, iwork1, levelset )
                     call inichkarlen ( nintersect, 12, 'iverts' )
                     if ( ierror/=0 ) go to 1000

                     do i = 1, nintersect
                        istartrcross = istartrcross+1
                        rcrossobst(istartrcross) = rverts(i)
                        icrossobst(istarticr+1) = iverts(i)
                        icrossobst(istarticr+2) = istartrcross
                        istarticr = istarticr+2
                        iedges(iverts(i)) = i+numintersect
                        numcross = numcross+1
                        iwork2(istart2+2+numcross) = i+numintersect
                     end do
                     numintersect = numintersect+nintersect

                  end if
                  if ( numcross>0 ) then

!                 --- element contains intersections

                     iwork2(istart2+1) = ielem
                     iwork2(istart2+2) = numcross
                     istart2 = istart2+2+numcross
                     numelms = numelms+1

                  end if

               end if

            end do

         else

!        --- One-dimensional shape

            ielem = ielem+inelem

         end if

      end do

!     --- Clear array iwork

      istart = ielemobst(iobst)
      istart = istart+2+numinelmobst
      iwork(ielemobst(istart+1:istart+numbnelmobst)) = 0

!     --- Clear array iwork1

      call inizerolev ( ipointobst, istart, numbndobst, iobst )
      iwork1(ipointobst(istart+1:istart+numbndobst)) = 0

!     --- Fill rest of icrossobst and raise istarticross

      icrossobst(iobst) = istarticross
      icrossobst(istarticross) = numintersect
      istarticross = istarticross+1+2*numintersect
      icrossobst(istarticross) = numelms
      icrossobst(istarticross+1:istarticross+istart2) = &
         iwork2(1:istart2)
      istarticross = istarticross+1+istart2

      lenrcrossobst = istartrcross
      lenicrossobst = istarticross

      deallocate ( iwork, iwork1, iwork2, iedges, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End seplevelcrel3d'

      end if
      call erclos ( 'seplevelcrel3d' )

      end
