      subroutine er0072 ( index1, jnpelm, jnelem, usol, ichint, ichfun, &
                          number, value, isurf, ldim, istart )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-01-2016 Use sepmoduleelem
!        version  2.5  date 30-07-2013 New call to eli013/14
!        version  2.4  date 18-02-2003 Correction for empty elements
!
!   copyright (c) 1991-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    subroutine er0072 computes the integral over one surface
!    ER0072 is a help subroutine for subroutine INTBOU
!    Complex case, the real equivalent is subroutine ER0071
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     complex
!     integral
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmodulekprob
      use sepmodulekmesh
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer index1(*), jnpelm, jnelem, ldim, &
              ichint, number, istart, isurf, ichfun
      double complex usol(*), value(*)

!     ichfun     i    indication of the value of f.
!                     Possibible values:
!                     0         f = 1
!                     >0        f = func ( ichfun, x, y, z ) (real case)
!                               or
!                               f = cfunc ( ichfun, x, y, z ) (complex
!                                   case)
!                               at this moment only the combination f#1
!                               and ichint=1 is permitted
!     ichint     i    indicates the kind of integral to be computed.
!                     possibilities:
!                     1          /  f u  ds
!                     2          /  f u.n  ds
!                     3          /  f u.t  ds
!                     4          /  f n. sigma .n  ds
!                     5          /  f n. sigma .t  ds
!                     6          /  f n. sigma     ds
!                     7          /  f u  n  ds
!                     9          /  f ds
!     ldim       i    This parameter indicates what type of boundary
!                     integral must be computed
!                     Possible values:
!                     1: curves
!                     2: surfaces
!                     3: contact
!                     4: no_contact
!     index1     i    Contains the nodal point numbers of the nodes in the
!                     elements of the surface
!     isurf      i    Surface number
!     jnelem     i    Number of elements in the surface
!     jnpelm     i    Number of point in the element
!     istart     i    Starting address in kprobw2
!     number     i    the number th  unknown in each nodal point of the
!                     vector corresponding to ivectr is used
!     usol       i    Solution vector
!     value      o    array containing the computed value of the integral
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ipindx, j, inusol, inunkp
      double complex usollc(27), valloc(1)
      double precision phiksi(84)

!     i              General loop variable
!     inusol         Position of unknown in array USOL
!     ipindx         Starting address in array index1
!     j              General loop variable
!     phiksi         Work array for elp subroutines
!     usollc         Local solution vector for the element
!                    usolbn may be considered as a two-dimensional array:
!                    usolbn(i,j), i=1(1)m, j=1(1)inpelm, where i refers to the
!                    the vector in each point and j refers to the point in the
!                    element
!     valloc         Array to store the local integrals
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELI012         Compute boundary integral over linear surface triangle
!                    Complex version
!     ELI013         Compute boundary integral over bilinear surface
!                    quadrilateral Complex version
!     ELI014         Compute boundary integral over quadratic surface triangle
!                    Complex version
!     ELI015         Compute boundary integral over biquadratic surface
!                    quadrilateral Complex version
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    267    Incorrect number of nodes per element
!    1567   Unknown not present in node
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'er0072' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from er0072'

      end if

      ndim = 3
      inpelm = jnpelm
      nelemcac  = jnelem
      npointcac = npoint
      if ( debug) &
         write(irefwr,*) 'ndim, inpelm, nelem, npointcac, indprpi, indprfi ', &
                          ndim, inpelm, nelemcac, npointcac, indprpi, indprfi

      allocate ( work(432) )

!     --- Compute number of components of the vector u that must be used for
!         the integration per point

      if ( ichint==1 ) then
         m = 1
      else if ( ichint==2 ) then
         m = 3
      else if ( ichint==4 ) then
         m = 6
      else if ( ichint==9 ) then
         m = 1
         do i = 1, inpelm
            usollc(i) = (1d0,0d0)
         end do
      else if ( ichint==10 ) then
         m = 6
      else
         m = 3
      end if
      if ( debug ) write(irefwr,*) 'ichint, m ', ichint, m
      if ( inpelm<3 .or. inpelm==5 .or. inpelm>7 .and. &
           inpelm/=9 ) then
         call errint ( inpelm, 1 )
         call errsub ( 267, 1, 0, 0 )
      end if
      if ( ierror/=0 ) go to 1000

!     --- Loop over elements

      ifirst = 0
      do ielem = 1, nelemcac

!     --- Copy the value of usol into usollc

         if ( ldim==2 .or. ldim==5 ) then

!        --- ldim = 2 surfaces

            ipindx = (ielem-1)*inpelm

         else

!        --- ldim>2 contacts

            ipindx = (kprobwp2(istart-1+ielem)-1)*inpelm

         end if
         if ( index1(ipindx+1)>0 ) then

!        --- Array index1 contains nodes

            if ( debug ) then

!           --- Debug information

               write(irefwr,*) 'ipindx ', ipindx
               call prinin ( index1(ipindx+1), inpelm, 'index1' )

            end if
            if ( ifirst==0 ) then

!           --- Check if the required degrees of freedom are available in all
!               nodes
!               If not: error

               if ( indprpi/=0 ) then

!              --- indprpi # 0, array kprobpi contains the information

                  do i = 1, inpelm
                     do j = 1, m
                        if ( kprobpi(index1(ipindx+i),j+number-1)==0 ) &
                           then
                           call errint ( j+number-1, 1 )
                           call errint ( i, 2 )
                           call errint ( isurf, 3 )
                           call errsub ( 1567, 3, 0, 0 )
                        end if
                     end do
                  end do

               else if ( indprfi/=0 ) then

!              --- indprfi # 0, array kprobfi contains the information

                  do i = 1, inpelm
                     inunkp = kprobfi(index1(ipindx+i)+1)- &
                              kprobfi(index1(ipindx+i))
                     if ( number+m-1>inunkp ) then
                        call errint ( m+number-1, 1 )
                        call errint ( i, 2 )
                        call errint ( isurf, 3 )
                        call errsub ( 1567, 3, 0, 0 )
                     end if
                  end do

               end if

            end if

            valloc(1) = (0d0,0d0)
            if ( indprpi/=0 ) then

!           --- indprpi # 0, array kprobpi contains the information

               do i = 1, inpelm
                  do j = 1, m
                     usollc((i-1)*m+j) = &
                        usol(kprobpi(index1(ipindx+i),j+number-1))
                  end do
               end do

            else if ( indprfi/=0 ) then

!           --- indprfi # 0, array kprobfi contains the information

               do i = 1, inpelm
                  inusol = kprobfi(index1(ipindx+i)) + number - 1
                  do j = 1, m
                     usollc((i-1)*m+j) = usol(inusol+j)
                  end do
               end do

            else if ( ichint/=9 ) then

!           --- indprfi = 0 and indprpi = 0, i.e. no renumbering, constant number
!               of degrees of freedom

               do i = 1, inpelm
                  inusol = (index1(ipindx+i)-1)*nphysi+number-1
                  do j = 1, m
                     usollc((i-1)*m+j) = usol(inusol+j)
                  end do
               end do

            end if

            if ( inpelm==3 ) then
               call eli012 ( ichint, ichfun, m, usollc, coor, &
                             index1(ipindx+1), valloc )
            else if ( inpelm==4 ) then
               call eli013 ( ichint, ichfun, m, usollc, coor, &
                             index1(ipindx+1), valloc, phiksi )
            else if ( inpelm==6 ) then
               call eli014 ( ichint, ichfun, m, usollc, coor, &
                             index1(ipindx+1), valloc, phiksi )
            else if ( inpelm==9 ) then
               call eli015 ( ichint, ichfun, m, usollc, coor, &
                             index1(ipindx+1), valloc )
            end if
            value(1) = value(1) + valloc(1)
            ifirst = 1

         end if

      end do

      deallocate ( work )

1000  if ( debug ) write(irefwr,*) 'End er0072'
      call erclos ( 'er0072' )

      end
