      subroutine intcop04 ( usol1, usol2, iunk, minnode, &
                            maxnode, npointnw, minmax, &
                            iwork, indprp, kprobf, kprobp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-06-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpolate usol1 into usol2 using the information stored in the arrays
!     imap and rmap
!     Special case that not all nodes are filled
! **********************************************************************
!
!                       KEYWORDS
!
!     interpolation
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemap
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npointnw, iunk, minnode, maxnode, &
              minmax, iwork(*), indprp, kprobf(*), &
              kprobp(npointnw,*)
      double precision usol1(*), usol2(*)

!     indprp         i    Indication if array kprob part p has been filled (1)
!                         or not (0)
!     iunk           i    Position of degree of freedom in node
!     iwork          i    integer work array
!     kprobf         i    KPROB part f: contains the number of unknowns per
!                         point accumulated (mesh 2)
!     kprobp         i    SEPRAN array KPROB part p containing the positions of
!                         the unknowns per point (mesh 2)
!     maxnode        i    Maximum nodal point number in part
!     minmax         i    If 1 the interpolated solution is restricted to the
!                         range uold_iun(min) <= unew_iun <= uold_iun(max)
!                         for each physical unknown iun, if 0 no restriction
!     minnode        i    Minimum nodal point number in part
!     npointnw       i    Number of points in new mesh
!     usol1          i    Input solution vector corresponding to mesh 1
!     usol2          o    Output solution vector corresponding to mesh 2
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nodp, k, ih, ih1
      double precision sum, u, minval, maxval
      logical errorfound

!     errorfound     If true an error has been found
!     ih             Help variable to store a constant temporarily
!     ih1            Help variable to store a constant temporarily
!     k              Counting variable
!     maxval         Maximum value of unknown
!     minval         Minimum value of unknown
!     nodp           Nodal point sequence number
!     sum            Help parameter to compute a sum
!     u              Help parameter to store the value of the solution
!                    in a point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRWAR         Print warnings
!     PRINMAP1       Actual printing of array imap
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     430   Possibility not implemented in mesh interpolation
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intcop04' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intcop04'
         write(irefwr,*) 'iunk, minmax, indprp ', iunk, minmax, indprp
         call prinmap1 ( npointnw )

      end if
      errorfound = .false.

      if ( minmax.eq.1 ) then

!     --- minmax = 1, restrict values

         minval = rinfin
         maxval = -rinfin
         do nodp = minnode, maxnode

            if ( indprp.gt.0 ) then

!           --- Array kprobp has been filled

               ih = kprobp(nodp,iunk)

            else

!           --- Array kprobp has not been filled

               ih1 = kprobf(nodp)
               if ( kprobf(nodp+1)-ih1.ge.iunk ) then
                  ih = ih1+iunk
               else
                  ih = 0
               end if  ! ( kprobf(nodp+1)-ih1.ge.iunk )

            end if  ! ( indprp.gt.0 )

            if ( ih.gt.0 ) then

!           --- ih > 0, interpolation must be done in this node

               if ( imap(nodp,1).eq.0 ) then

!              --- No interpolation for this node

                  errorfound = .true.
                  if ( debug ) write(irefwr,*) 'nodp, ih, imap ', &
                     nodp, ih, imap(nodp,1)

               else

!              --- Standard case

                  do k = 1, imap(nodp,1)
                     u = usol1(imap(nodp,k+1))
                     minval = min(u,minval)
                     maxval = max(u,maxval)
                  end do

               end if  ! ( imap(nodp,1).eq.0 )

            end if  ! ( ih.gt.0 )

         end do  ! nodp = minnode, maxnode

         do nodp = minnode, maxnode

            if ( indprp.gt.0 ) then

!           --- Array kprobp has been filled

               ih = kprobp(nodp,iunk)

            else

!           --- Array kprobp has not been filled

               ih1 = kprobf(nodp)
               if ( kprobf(nodp+1)-ih1.ge.iunk ) then
                  ih = ih1+iunk
               else
                  ih = 0
               end if  ! ( kprobf(nodp+1)-ih1.ge.iunk )

            end if  ! ( indprp.gt.0 )

            if ( ih.gt.0 ) then

!           --- ih > 0, interpolation must be done in this node

               sum = 0d0
               do k = 1, imap(nodp,1)
                  if ( iwork(imap(nodp,k+1)).ne.1 ) then

!                 --- iwork has not been filled with 1

                     errorfound = .true.
                     if ( debug ) then
                        write(irefwr,*) 'nodp, k, imap ', &
                          nodp, k, imap(nodp,1)
                        write(irefwr,*) 'imap, iwork ', &
                          imap(nodp,k+1), iwork(imap(nodp,k+1))
                     end if  ! ( debug )

                  end if  ! ( iwork(imap(nodp,k+1)).ne.1 )
                  sum = sum + rmap(nodp,k)*usol1(imap(nodp,k+1))

               end do
               usol2(nodp) = max(minval,min(sum,maxval))

            end if  ! ( ih.gt.0 )

         end do  ! nodp = minnode, maxnode

      else

!     --- No need to restrict

         do nodp = minnode, maxnode

            if ( indprp.gt.0 ) then

!           --- Array kprobp has been filled

               ih = kprobp(nodp,iunk)

            else

!           --- Array kprobp has not been filled

               ih1 = kprobf(nodp)
               if ( kprobf(nodp+1)-ih1.ge.iunk ) then
                  ih = ih1+iunk
               else
                  ih = 0
               end if  ! ( kprobf(nodp+1)-ih1.ge.iunk )

            end if  ! ( indprp.gt.0 )

            if ( ih.gt.0 ) then

!           --- ih > 0, interpolation must be done in this node

               if ( imap(nodp,1).eq.0 ) then

!              --- No interpolation for this node

                  errorfound = .true.
                  if ( debug ) write(irefwr,*) 'nodp, ih, imap ', &
                     nodp, ih, imap(nodp,1)

               end if  ! ( imap(nodp,1).eq.0 )
               sum = 0d0
               do k = 1, imap(nodp,1)

                  if ( iwork(imap(nodp,k+1)).ne.1 ) then

!                 --- iwork has not been filled with 1

                     errorfound = .true.
                     if ( debug ) then
                        write(irefwr,*) 'nodp, k, imap ', &
                          nodp, k, imap(nodp,1)
                        write(irefwr,*) 'imap, iwork, usol1 ', &
                          imap(nodp,k+1), iwork(imap(nodp,k+1)), &
                          usol1(imap(nodp,k+1))
                     end if  ! ( debug )

                  end if  ! ( iwork(imap(nodp,k+1)).ne.1 )

                  sum = sum + rmap(nodp,k)*usol1(imap(nodp,k+1))

               end do

               usol2(ih) = sum

            end if  ! ( ih.gt.0 )

         end do  ! nodp = minnode, maxnode

      end if

      if ( errorfound ) call errwar ( 430, 0, 0, 0 )

      call erclos ( 'intcop04' )
      if ( debug ) write(irefwr,*) 'End intcop04'

      end
