      subroutine intmap3d412 ( kmeshc, inelem, kprobf, &
                               usol, usolcop, ielgrp, inpelm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-12-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy solution in mesh with non-constant number of unknowns
!     per point to solution array with constant number of unknowns
!     The solution to be copied is not present in all nodes
!     Center nodes of elements and faces are skipped
!     Elements of shape 12, 15 or 16
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     3d
!     tetrahedron
!     quadratic
!     vertex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduletypes
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpelm, kmeshc(inpelm,*), inelem, &
              kprobf(*), ielgrp
      double precision usol(*), usolcop(*)

!     ielgrp         i    element group number
!     inelem         i    Number of elements in a specific element group
!     inpelm         i    Number of nodes in elements
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!                         kmeshc is only related to this element group
!     kprobf         i    KPROB part f: contains the number of unknowns per
!                         point accumulated
!     usol           i    Solution vector
!     usolcop       i/o   Solution vector to which the solution must be
!                         copied. usolcop has nunkp unknowns per node
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ielem, node, node2, node3, imidside(6,3), k1, k2, k3
      logical present

!     i              Counting variable
!     ielem          Element number
!     imidside       Array of size 6 * 3 containing:
!                    (i,1):  i^th mid side point
!                    (i,2) and (i,3) neighbor vertices of the midside point
!     k1             Relative node number of midside point
!     k2             Relative node number of first neighbor
!     k3             Relative node number of second neighbor
!     node           Node number
!     node2          Node number of first neighbor
!     node3          Node number of second neighbor
!     present        If true the unknowns are present in the midside points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     583   The required degree of freedom is not available in the vertices
!           of the element
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intmap3d412' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intmap3d412'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Check if unknown is present in all vertices

      do i = 1, 4

         node = kmeshc(ivertx12(i),1)

         if ( kprobf(node+1)-kprobf(node)==0 ) then

!        --- Unknown not present in vertex

            call errint ( 1, 1 )
            call errint ( 1, 2 )
            call errint ( ielgrp, 3 )
            call errwar ( 583, 3, 0, 0 )
            go to 1000

         end if  ! ( kprobf(node+1)-kprobf(node)==0 )

      end do  ! i = 1, 4

!     --- Check if unknown is present in the midside points
!         First fill array imidside

      do i = 1, 3
         imidside(1,i) = 2*i
         imidside(2,i) = 2*i-1
         imidside(3,i) = 2*i+1
         imidside(4,i) = i+6
         imidside(5,i) = 2*i-1
         imidside(6,i) = 10
      end do  ! i = 1, 3
      imidside(3,3) = 1

      present = .true.
      do i = 1, 6

         node = kmeshc(imidside(i,1),1)
         if ( kprobf(node+1)-kprobf(node)==0 ) present = .false.

      end do  ! i = 1, 6

      if ( present ) then

!     --- Copy solution in vertices and midside points

         do ielem = 1, inelem

            do i = 1, 10

               node = kmeshc(i,ielem)
               usolcop(node) = usol(kprobf(node)+1)

            end do  ! i = 1, 10

         end do  ! ielem = 1, inelem

      else

!     --- Copy solution in vertices

         do ielem = 1, inelem

            do i = 1, 4

               node = kmeshc(ivertx12(i),ielem)
               usolcop(node) = usol(kprobf(node)+1)

            end do  ! i = 1, 4

         end do  ! ielem = 1, inelem

!        --- Interpolate solution in midside points

         do i = 1, 6

            k1 = imidside(i,1)
            k2 = imidside(i,2)
            k3 = imidside(i,3)
            do ielem = 1, inelem

               node = kmeshc(k1,ielem)
               node2 = kmeshc(k2,ielem)
               node3 = kmeshc(k3,ielem)

               usolcop(node) = 0.5d0*(usolcop(node2)+usolcop(node3))

            end do  ! ielem = 1, inelem

         end do  ! i = 1, 6

      end if ! ( present )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End intmap3d412'

      end if  ! ( debug )
      call erclos ( 'intmap3d412' )

      end
