      subroutine intmshm11 ( npoint1, coor1, npoint2, coor2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 19-04-2013 Adaptation for same points
!        version  1.0  date 02-01-2001
!
!   copyright (c) 2000-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Make a formal interpolation from one mesh to another.
!   In fact the mapping between the meshes is computed, &
!   without knowing the function (1D version)
! **********************************************************************
!
!                       KEYWORDS
!
!     interpolation
!     mesh
!     1d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemap
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint1, npoint2
      double precision coor1(*), coor2(*)

!     coor1          i    Array containing the coordinates of the first mesh
!     coor2          i    Array containing the coordinates of the second mesh
!     npoint1        i    number of points in first mesh
!     npoint2        i    number of points in second mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, inext, j, kgrad1(npoint1), kgrad2(npoint2)
      double precision alpha1, x, x1, x2, dx, eps

!     alpha1         multiplication factor
!     dx             length of vector in x-direction
!     eps            Local accuracy
!     i              Counting variable
!     inext          Next point in curve
!     j              Counting variable
!     kgrad1         Work array of length  npoint1 used for the sorting of
!                    coor1. kgrad1(i) contains the index of the i-th
!                    coordinate in the sorted sequence
!     kgrad2         Work array of length  npoint2 used for the sorting of
!                    coor2. kgrad2(i) contains the index of the i-th
!                    coordinate in the sorted sequence
!     x              x co-ordinate of point
!     x1             x-coordinate of first point
!     x2             x-coordinate of last point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ2        Print an integer two-dimensional array
!     PRINRL2        Print a real two-dimensional array
!     PRSORT         Sort real array for increasing sequence
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Sort the coordinate arrays
!     Set first interval (x1,x2,inext)
!     for i := 1 (1) npoint2 do
!        j := kgrad2(i);  x:=coor(j)
!        if ( x>x2 ) then
!           find new interval and raise inext
!        Fill imap and rmap
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intmshm11' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intmshm11'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Sort the coordinate arrays

      call prsort ( coor1, kgrad1, npoint1 )
      call prsort ( coor2, kgrad2, npoint2 )

!     --- Set first interval

      inext = 1
      x1 = coor1(kgrad1(inext))
      x2 = coor1(kgrad1(inext+1))
      eps = 1d-6*(coor1(kgrad1(npoint1))-x1)
      inext = 2
      dx = 1d0/(x2-x1)

!     --- loop over all nodes in coor2

      do i = 1, npoint2
         j = kgrad2(i)
         x = coor2(j)

!        --- adapt interval if necessary

         do while ( x>x2+eps .and. inext<npoint1 )
            inext = inext+1
            x1 = x2
            x2 = coor1(kgrad1(inext))
            dx = 1d0/(x2-x1)
         end do

!        --- Fill imap and rmap

         alpha1 = (x-x1)*dx

         if ( abs(alpha1)<1d-10 ) then

!        --- alpha1 = 0, hence one point only

            imap(j,1) = 1
            imap(j,2) = kgrad1(inext-1)
            rmap(j,1) = 1d0

         else if ( abs(alpha1-1d0)<1d-10 ) then

!        --- alpha1 = 1, hence one point only

            imap(j,1) = 1
            imap(j,2) = kgrad1(inext)
            rmap(j,1) = 1d0

         else

!        --- General case

            imap(j,1) = 2
            imap(j,2) = kgrad1(inext-1)
            imap(j,3) = kgrad1(inext)
            rmap(j,1) = 1d0-alpha1
            rmap(j,2) = alpha1

         end if  ! ( abs(alpha1)<1d-10 )

      end do

1000  if ( debug ) then

!     --- Debug information

         call prinin2 ( imap, npoint2, 3, 'imap' )
         call prinrl2 ( rmap, npoint2, 2, 'rmap' )
         write(irefwr,*) 'End intmshm11'

      end if  ! ( debug )
      call erclos ( 'intmshm11' )

      end
