      subroutine prgat1 ( xcint, fcint, nseg, kseg, ncint )
! ======================================================================
!
!        programmer    Jos van Kan
!        version  2.0  date 14-10-2016 New parameter list
!        version  1.1  date 21-01-1995 New call to prsort
!        version  1.0  date 11-03-1991
!
!   copyright (c) 1993-2016  "Ingenieursbureau  SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Complex version of prgath
!   prgat1 gathers the points for one value of slevel and  puts
!   them SORTED back into the array xcint and fcint. Double points
!   are removed. NSEG contains the number of segments, KSEG(I)
!   contains the number of points in the I-th segment.
!   The routine performs identically to prgat1, but uses the work
!   arrays xwork, fwork & iwork as a scratch pad.
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ncint, nseg, kseg(*)
      double precision xcint(ncint)
      double complex  fcint(ncint)

!     fcint   i    array of interpolated function values
!     kseg    o    array containing number of points of
!                  ith segment in ith place.
!     ncint   i    number of points of xcint and fcint on input.
!                  Since a lot are double, the number of points
!                  on output (sum over i kseg (i)) should be
!                  a lot less.
!     nseg    o    number of segments per line
!     xcint   i    array of interpolated x- values
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision eps, eps1, xh1, xh2, xm1, xm2, xwork(ncint)
      double complex :: fwork(ncint)
      integer idint, i, iwork (ncint)

!     eps     tolerance. intervals that are smaller are removed
!     eps1    1d-3 * eps. To identify coinciding midpoints
!     fwork   Work array of length ncint to store the sorted f-values
!     i       Counting variable
!     idint   counter to double intervals
!     iwork   Work array of length ncint to store the indices temporarily
!     xh1     auxiliary variables to store endpoints and midpoints of intervals
!     xh2     auxiliary variables to store endpoints and midpoints of intervals
!     xm1     auxiliary variables to store endpoints and midpoints of intervals
!     xm2     auxiliary variables to store endpoints and midpoints of intervals
!     xwork   Work array of length ncint to store the sorted x-values
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     PRSORT  Sort real array for increasing sequence
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      nseg = 0

!     --- If there are no points the segment is empty and we are done.

      if (ncint.eq.0) return

!     --- To sort the intervals we set up the midpoints as a sortkey in xwork

      do 100 i = 1, ncint / 2
         xwork (i) = .5d0 * (xcint (2 * i - 1) + xcint (2 * i))
100   continue

!     --- Now we sort the intervals by this sortkey, putting the sorting
!         permutation vector in iwork. The upper half of iwork is used as
!         workarray by prsort.

      call prsort ( xwork, iwork, ncint / 2 )

!     --- Next we remove all double intervals (the midpoints will coincide)

      eps = 1d-6 * (xcint (2 * iwork (ncint / 2)) - xcint (2 * iwork (1) &
            - 1))
      eps1 = 1d-3 * eps
      idint = 0
      xwork (1) = xcint (2 * iwork (1) - 1)
      xwork (2) = xcint (2 * iwork (1))
      fwork (1) = fcint (2 * iwork (1) - 1)
      fwork (2) = fcint (2 * iwork (1))
      xm1 = .5d0 * (xwork (1) + xwork (2))
      do 110 i = 2, ncint / 2
         xh1 = xcint (2 * iwork (i) - 1)
         xh2 = xcint (2 * iwork (i))
         xm2 = .5d0 *  (xh1 + xh2)
         if (xm2 - xm1.lt.eps1) then

!        --- Midpoints of two successive intervals coincide hence this interval
!            --- is
!            double and has to be removed

            idint = idint + 1
            ncint = ncint - 2
         else
            xwork (2 * (i - idint) - 1) = xh1
            xwork (2 * (i - idint)) = xh2
            fwork (2 * (i - idint) - 1) = fcint (2 * iwork (i) - 1)
            fwork (2 * (i - idint)) = fcint (2 * iwork (i))
         end if
         xm1 = xm2
110   continue

!     --- Next remove the small intervals

      idint = 0
      do 120 i = 1, ncint/2 - 1
         if ((xwork (2 * i) - xwork (2 * i - 1)).gt.eps) then
            xwork (2 * (i - idint) + 1) = xwork (2 * i + 1)
            xwork (2 * (i - idint)) = xwork (2 * i)
            fwork (2 * (i - idint) + 1) = fwork (2 * i + 1)
            fwork (2 * (i - idint)) = fwork (2 * i)
         else if ((xwork (2 * i + 1) - xwork (2 * i)).lt.eps1) then

!        --- small interval, can be joined with next interval

            idint = idint + 1
         else if (i.ne.1 .and. (xwork (2 * i - 1) - xwork &
               (2 * i - 2)).lt.eps1) then

!        --- small interval, can be joined with previous interval

            idint = idint + 1
            xwork (2 * (i - idint) + 1) = xwork (2 * i + 1)
            xwork (2 * (i - idint)) = xwork (2 * i)
            fwork (2 * (i - idint) + 1) = fwork (2 * i + 1)
            fwork (2 * (i - idint)) = fwork (2 * i)
         end if
120   continue

!     --- last point has still to be treated

      if (ncint.gt.2 ) then
         if (xwork (ncint) - xwork (ncint - 1).lt.eps .and. (xwork &
            (ncint - 1) - xwork (ncint - 2)).lt.eps1) then
            idint = idint + 1
         end if
      end if
      xwork (ncint - 2 * idint) = xwork (ncint)
      fwork (ncint - 2 * idint) = fwork (ncint)
      ncint = ncint - 2 * idint

!     --- now comes the splitting into segments

      nseg = 1
      kseg (1) = 1
      xcint (1) = xwork (1)
      fcint (1) = fwork (1)
      do 130 i = 2, ncint - 2, 2
         kseg (nseg) = kseg (nseg) + 1
         if (xwork (i + 1) - xwork (i).lt..5 * eps) then

!        --- internal point. has to be added to segment.

            xcint (i / 2 + nseg) = xwork (i)
            fcint (i / 2 + nseg) = fwork (i)
         else

!        --- Boundary point. has to be added and next segment started.

            xcint (i / 2 + nseg) = xwork (i)
            fcint (i / 2 + nseg) = fwork (i)
            nseg = nseg + 1
            kseg (nseg) = 1
            xcint (i / 2 + nseg) = xwork (i + 1)
            fcint (i / 2 + nseg) = fwork (i + 1)
         end if
130   continue

!     --- Now add last point

      kseg (nseg) = kseg (nseg) + 1
      xcint (ncint / 2 + nseg) = xwork (ncint)
      fcint (ncint / 2 + nseg) = fwork (ncint)
      end
