      subroutine printerpolnod3 ( inode, coor, dphidx, &
                                  npoint, xh, yh, zh, ielem, kmeshtri )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-02-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if node is in element, if so interpolate
!     3d linear tetrahedrons only
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     interpolation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemap
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inode, npoint, kmeshtri(4), ielem
      double precision coor(3,*), dphidx(4,3), &
                       xh(4), yh(4), zh(4)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!                         Contains coordinates of mesh in which inode is lying
!     dphidx         i    Contains the derivatives of the basis functions
!     ielem          i    Element number
!     inode          i    Node sequence number
!                         Interpolation must be performed in this node
!     kmeshtri       i    Contains 3 nodes of the triangle
!     npoint         i    Number of nodal points in new mesh (size of imap2)
!     xh             i    x-coordinates of vertices
!     yh             i    y-coordinates of vertices
!     zh             i    z-coordinates of vertices
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer l, icount
      double precision xgauss(4), dx, dy, dz

!     dx             distance in x-direction
!     dy             distance in y-direction
!     dz             distance in z-direction
!     icount         Counter
!     l              General loop variable
!     xgauss         baricentric co-ordinates of the point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'printerpolnod3' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from printerpolnod3'
         write(irefwr,1) 'inode', inode
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- compute baricentric co-ordinates of inode

      dx = coor(1,inode) - xh(4)
      dy = coor(2,inode) - yh(4)
      dz = coor(3,inode) - zh(4)
      xgauss(1) = dphidx(1,1)*dx+dphidx(1,2)*dy+dphidx(1,3)*dz
      xgauss(2) = dphidx(2,1)*dx+dphidx(2,2)*dy+dphidx(2,3)*dz
      xgauss(3) = dphidx(3,1)*dx+dphidx(3,2)*dy+dphidx(3,3)*dz
      xgauss(4) = 1d0 - xgauss(1) - xgauss(2) - xgauss(3)

      if ( debug ) call prinrl ( xgauss, 4, 'xgauss' )

!     --- Check if node is in element

      do l = 1, 4
         if ( xgauss(l)<-0.01d0 .or. &
              xgauss(l)>1.01d0 ) goto 1000
      end do

!     --- all baricentric co-ordinates in [0,1]
!         Compute interpolation
!         Fill imap2 and rmap2

      icount = 0
      do l = 1, 4
         if ( abs(xgauss(l))>1d-15 ) then

!        --- Node has non-zero contribution

            icount = icount+1
            imap2(inode,icount+1) = kmeshtri(l)
            rmap2(inode,icount) = xgauss(l)

         end if  ! ( abs(xgauss(l))>1d-15 )
      end do  ! l = 1, 4

      imap2(inode,1) = icount
      imap2(inode,icount+2) = ielem

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End printerpolnod3'

      end if  ! ( debug )
      call erclos ( 'printerpolnod3' )

      end
