      subroutine prlin2 ( fval, coor, knodp, nelem, npoint, angdeg, &
                          orgx, orgy, nseg, kseg, xcint, fcint, ncint )
! ======================================================================
!
!        programmer    Jos van Kan
!        version  2.0  date 14-10-2016 New parameter list
!        version  1.3  date 30-11-2012 Long integers
!        version  1.2  date 27-11-1992 Minor change in declarations
!
!   copyright (c) 1991-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!  Complex version of prlin1
!  prlin2 is a SEPRAN subroutine to do the following:
!  Given a mesh consisting of triangles with only tree nodal points
!  and function values (one only) in those nodal points, to calculate
!  a set of interpolated values along a given line.
!  The angle of that line with respect to the x-axis is given
!  by angdeg in degrees. The new origin by (orgx, orgy).
!  The output is organized as follows:
!  any given line may consist of a number of separate SEGMENTS, for
!  example if an L-shaped region is cut like this:
!             ***\*****
!\   *
!\  *
!\ *
!\*
!\
!*\
!*       * \
!***\*****
!\   *
!\  *
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nelem, npoint, ncint,  knodp(3,nelem), kseg(*), nseg
      double complex fval(npoint), fcint(ncint)
      double precision coor(2,npoint), xcint(ncint), orgx, orgy, &
             angdeg

!     angdeg         i    angle of rotation in degrees
!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     fcint          o    array of interpolated function values
!     fval           i    function values in nodes
!     knodp          i    nodal points of elements
!     kseg           o    number of points per segment
!     ncint          o    declared length of fcint and xcint (must be
!                         larger than sum_j kseg (j))
!     nelem          i    Number of elements
!     npoint         i    Number of nodal points
!     nseg           o    number of segments per line
!     orgx           i    coordinates of new origin
!     orgy           i    coordinates of new origin
!     xcint          o    array of interpolated x- values
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nhelp, niout, iout(nelem)
      double precision slevel

!     i              Counting variable
!     iout           integer array with element numbers that
!                    intersect the line y = slevel.
!     nhelp          Number of entries that have been stored in ihelp
!     niout          number of elements intersecting this line
!     slevel         value of y coordinate to intersect the elements
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERWARN         Print warnings
!     PRELI1         calculates which elements intersect this line. (unsorted
!                    element field)
!     PRGAT1         removes doubles and sorts.
!     PRROTA         rotate coordinates
!     PRSEG1         calculates intersection points
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!  Algorithm
!
!  The coordinate system is first shifted to orgx, orgy and rotated
!  over angdeg degrees.
!
!  1. The intersection points (if any) of the elements with the given
!     line are calculated, together with the interpolated function
!     values. All internal points now will be double, boundary
!     points single.
!  2. These points are sorted and the double points removed. The number
!     of segments per line and points per segment are stored in nseg and
!     kseg respectively.
!
!  The coordinate system is rotated back to clean up the mess.
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- the line consists of two segments. This number of segments is stored
!         in nseg. The number of interpolated points per segment is stored in
!         kseg (j), in which j is increased by one for each new segment. The
!         coordinate system is rotated over angdeg degrees, and shifted, so that
!         intersection line is y = 0 in the new coordinates.
!         The (rotated and shifted) x-coordinates of interpolation are stored in
!         xcint, the corresponding interpolated function values in fcint.
!         rotate over angdeg degrees
!         Shift origin

      do i = 1, NPOINT
         coor ( 1,i ) = coor ( 1,i ) - orgx
         coor ( 2,i ) = coor ( 2,i ) - orgy
      end do
      call prrota (coor, angdeg, NPOINT)
      slevel = 0d0

!     --- calculate which elements intersect this line

        call preli1 (slevel, knodp, NELEM, coor, iout, NIOUT)
        if ( niout==0 ) then

!       --- No intersection found, warning and return

           call erwarn ( 'intp2d  (prlin2)', 752, 1, 0, 0, 0 )
           return
         end if

!        --- calculate in which points this line intersects and interpolate
!            function values

      call prseg1 ( slevel, coor, fval, knodp, iout, NIOUT, &
                    xcint, fcint )

!     --- sort the points, remove the doubles and partition into segments

      nhelp = 2*NIOUT
      call prgat1 (xcint, fcint, nseg, kseg, nhelp )

!     --- now rotate back the coordinate system

      call prrota (coor, -angdeg, NPOINT)

!     --- And back shift the origin

      do i = 1, NPOINT
         coor (1, i) = coor (1,i) + orgx
         coor (2, i) = coor (2,i) + orgy
      end do
      end
