      subroutine prreducinters ( inpsep, isol, numsol, rinsep, values )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 30-04-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if end point of line is given and if so reduce array usol
!     to part defined by this length if length smaller than original length
! **********************************************************************
!
!                       KEYWORDS
!
!     reduction
!     length
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: inpsep(*), numsol
      integer, intent(inout) :: isol(numsol)
      double precision, intent(in) :: rinsep(*), values(3)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         Only positions 8-10 are used in this subroutine
!                         See subroutine readscintersin for a description
!     isol          i/o   Standard SEPRAN array containing information of the
!                         solution arrays
!     numsol         i    Number of solution vectors in isol
!     rinsep         i    Array containing the real input for this subroutine
!                         See subroutine readscintersin for a description
!     values         i    Double precision or complex array in which the real
!                         or complex information concerning the line is stored
!                         See subroutine readscprinint (rinsep)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, numsubcurvs, numnodes, ientry
      double precision, allocatable :: work(:,:)
      logical :: reduction

!     i              Counting variable
!     ientry         The last entry of work that is used
!     numnodes       Number of nodal points in array nodes
!     numsubcurvs    Number of subcurves of intersection
!     reduction      If true array work is reduced
!     work           Work array to store the s-coodinates as well as
!                    the function values of the intersection
!                    (.,1) the s-coordinates
!                    (.,2..) the values
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     ERFILL119      Fill array outvec with the information of array ivectr
!                    of type 119
!     ERFILLBCK119   Fill array isol of type 119 with the contents
!                    of array work
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ1        print 2d integer array
!     PRREDUCEWORK   Check if end point of line is given and if so reduce
!                    array work to part defined by this length if length
!                    smaller than original length
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3179   Not the same length for all vectors
!    3186   More than 1 subcurve
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prreducinters' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prreducinters'
         call prinin1 ( isol, numsol, 5, 'isol' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( inpsep(8)==0 .and. inpsep(9)==0 .and. inpsep(10)==0 ) &
         go to 1000  ! skip since no endpoint given

!     --- Check if they all have the same length

      isoltemp => ks(isol(1))%isol
      length = size ( isoltemp )  ! length of array
      do i = 2, numsol
         isoltemp => ks(isol(i))%isol
         if ( length/=size ( isoltemp ) ) then

!        --- Error not the same number of nodes in the various arrays

            call errsub ( 3179, 0, 0, 0 )
            go to 1000

         end if  ! ( length/=size ( isoltemp ) )
      end do  ! i = 1, numsol

!     --- Compute starting address as well as some extra information

      numsubcurvs = ks(isol(1))%n   ! number of subcurves
      isoltemp => ks(isol(1))%isol
      numnodes = sum(isoltemp(1:numsubcurvs)) ! number of nodes

      if ( numsubcurvs>1 ) then

!     --- Error not yet implemented

         call errint ( numsubcurvs, 1 )
         call errsub ( 3186, 1, 0, 0 )
         go to 1000

      end if  ! ( numsubcurvs>1 )

!     --- Copy vectors in temporary vector work

      allocate ( work(numnodes,numsol+1), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, numnodes*(numsol+1), 'work' )
      if ( ierror/=0 ) go to 1000

      call erfill119 ( isol, numsol, numsubcurvs, work, numnodes )

!     --- Reduce work to part with length length

      call prreducework ( work, numnodes, numsol, reduction, &
                          inpsep, rinsep, values, ientry )

      if ( reduction ) then

!     --- reduction is true, redefine isol

         call erfillbck119 ( isol, numsol, numsubcurvs, work, numnodes, &
                             ientry )

      end if  ! ( reduction )

!     --- remove temporary array

      deallocate ( work, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'work' )
      if ( ierror/=0 ) go to 1000

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prreducinters'

      end if  ! ( debug )
      call erclos ( 'prreducinters' )

      end subroutine prreducinters
