      subroutine putuser ( iincop, isol, kmeshlc, kproblc, funcy, inodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.4  date 29-05-2014 remove call to time subroutine
!        version  1.3  date 30-11-2012 Long integers
!        version  1.2  date 13-04-2002 New call to erput01
!
!   copyright (c) 1998-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   subroutine putuser is used to put the solution from a user array
!   and store it in a SEPRAN solution array
!   Actual body of putuser
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     user_vector
!     sepran_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulemain
      use sepmodulekmesh
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iincop(*), isol, kmeshlc, kproblc, inodes(*)
      double precision funcy(*)

!     funcy      i    user array of type 117, where the degrees of freedom
!                     of the points are stored
!                     funcy must be an array of the type produced by COMPCR or
!                     COMPSR
!     iincop     i    Dummy array, may be used in the future
!                     At this moment the first position must be equal to 0
!     inodes     i    integer array containing the node numbers corresponding to
!                     the arrays funcy in the same sequence from
!                     position 6
!                     This must be an array of type 128 as produced by compsr
!     isol       o    Standard SEPRAN solution array of type 110 or 115
!     kmeshlc    i    Standard SEPRAN array containing information of the mesh
!     kproblc    i    Standard SEPRAN array containing information of the
!                     problem
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer idegfd, nnodes, ih

!     idegfd         Degree of freedom sequence number
!     ih             Help variable to store a constant temporarily
!     nnodes         Number of nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERPUT01        Copy values from user array into solution array
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     SEPACTSOLBF2   compute pointers to degrees of freedom of vectors
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     287   Array IINCOP(1) has incorrect value
!     610   array isol corresponds to a complex array
!    2017   Array has not been filled properly
!    2018   The number of degrees of freedom per point in array FUNCY is
!           incorrect
!    2019   Array FUNCY is incorrect
!    2020   Arrays FUNCY and INODES do not correcpond to each other
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Test input
!     Activate ISOL
!     Set starting addresses
!     Copy array
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'putuser' )

!     --- Test input

      if ( iincop(1)/=0 ) then

!     --- iincop(1) is not equal to zero

         call errint ( iincop(1), 1 )
         call errint ( 0, 2 )
         call errwar ( 287, 2, 0, 0 )

      end if
      if ( inodes(2)/=128 ) then

!     --- inodes is not of type 128

         call errint ( 2, 1 )
         call errint ( inodes(2), 2 )
         call errint ( 128, 3 )
         call errchr ( 'inodes', 1 )
         call errsub ( 2017, 3, 0, 1 )

      end if
      ih = nint(funcy(2))
      if ( ih/=117 ) then

!     --- funcy is not of type 117

         call errint ( 2, 1 )
         call errint ( ih, 2 )
         call errint ( 117, 3 )
         call errchr ( 'funcy', 1 )
         call errsub ( 2017, 3, 0, 1 )
      end if
      kprob2d(1) = kproblc
      kprob => kprob2d(1)
      kmesh2d(1) = kmeshlc
      kmesh => kmesh2d(1)
      call sepfillkmesh(kmesh)

!     --- Activate ISOL

      call sepactsolbf2 ( isol )
      if ( ks(isol)%compl/=0 ) then

!     --- Array isol is complex

         call errsub ( 610, 0, 0, 0 )

      end if
      idegfd = nint(funcy(4))
      if ( idegfd<1 .or. idegfd>nphysi ) then

!     --- error 2018:IDEGFD>nphysi

         call errint ( idegfd, 1 )
         call errint ( nphysi, 2 )
         call errsub ( 2018, 2, 0, 0 )

      end if
      if ( ierror/=0 ) go to 1000
      nnodes = inodes(5)
      ih = nint(funcy(5))
      if ( ih/=nnodes*idegfd ) then

!     --- error 2019: FUNCY inconsistent

         call errint ( idegfd, 1 )
         call errint ( nnodes, 2 )
         call errint ( ih, 3 )
         call errsub ( 2019, 3, 0, 0 )

      end if
      if ( nnodes/=inodes(5) ) then

!     --- error 2020: INODES and FUNCY do not correspond to each other

         call errint ( inodes(5), 1 )
         call errint ( nnodes, 2 )
         call errsub ( 2020, 2, 0, 0 )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Set starting addresses

      usoltemp => ks(isol)%sol
      call erput01 ( usoltemp, inodes, funcy )

1000  call erclos ( 'putuser' )
      end
