      subroutine sepderbodyint ( list )
! ======================================================================
!
!        programmer    Jaap van der Zanden/Guus Segal
!        version  7.2  date 06-08-2015 New definition istold
!        version  7.1  date 05-12-2012 Long integers
!        version  7.0  date 15-11-2012 Extra parameters
!
!   copyright (c) 1990-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Actual body of subroutine to0100, to compute derived quantities
!     or special integrals along boundaries over derivatives
!     the effect of weights in nodal points is taken into account
!     by an averaging procedure
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     integral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulemain
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulesmf
      use sepmodulebuild
      use sepmoduleoldvecs
      use sepmodulecpack
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: list(*)

!     list           i    Contains list of all admissible elements
!                         See subroutine TO0101
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ikelmc, ikmcrf, ielmrf, &
              nel, lcoun, ipkproba

!                    for an element group
!     i              Counting variable
!     ielmrf         Reference for the computation of the element number
!     ikelmc         position in kmesh part c
!     ikmcrf         Help variable to compute a position in KMESH part c
!     ipkproba       starting address in kprob part a
!     lcoun          Help parameter to compute position in array list
!     nel            Number of admissible elements in an element group
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     SEPCOPYELEM    Copy information of elements in type element
!     SEPDERELGRP    Initialize some parameters for one element group
!     SEPDERVDALLOC  De-allocate temporary arrays in for subroutine to0112
!     SEPDERVINDEX   Fill arrays index2, index3 and index4 for subroutine deriv
!     SEPELDERV      Compute element vector and weights for derivatives
!     SEPNULLIFYBASE Nullify all pointers associated with B
!     TO0102         Compute line integral over a gradient for one element along
!                    a curve
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     211   notvec = 1  for relative element number more than 1
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepderbodyint' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepderbodyint'
         write(irefwr,1) 'iseqi1', iseqi1
         call prinin ( isolut2d(iseqi1), 5, 'isolut2d/1' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      kproba => kprobpart%kproba
      indpra = kprobloc(14)
      kprobc => kprobpart%kprobc

      valintegral = 0d0

!     --- outer loop over all groups of elements

      ikmcrf = 0
      ielmrf = 0
      lcoun  = 1
      isubr  = 0
      ipkproba = 1
      imas = 0     ! no mass matrix in case of derivatives

      B => B1
      call sepnullifybase  ! nullify all pointers in B

      if ( debug ) write(irefwr,1) 'nelgrp', nelgrp

      do ielgrp = 1, nelgrp

!     --- set notvec = 0  (element vector of this group not zero)

         notvec = 0
         if ( indpra>0 ) kprobatmp => kproba(ielgrp)%arr
         inelem = sepgetinelem(ielgrp)
         inpelm = sepgetinpelm(ielgrp)
         itype = kprobc(ielgrp)

         if ( debug ) then
            write(irefwr,1) 'ielgrp, itype, iskip', &
                             ielgrp, itype, &
                             skipelgrps(ielgrp)
         end if  ! ( debug )

         if ( itype<=0 .or. skipelgrps(ielgrp)==0 ) then

!        --- element group ielgrp must be skipped
!            Raise parameters

            ikmcrf = ikmcrf+inpelm*inelem
            ielmrf = ielmrf+inelem

         else

            ifirst = 0
            vector = .true.

!           --- Compute icount for input vector

            icount = sepfillcountvc ( isolut2d(iseqi1), ielgrp )

!           --- Compute icount for output vector

            icouno = sepfillcountvc ( SMF%ioutvc, ielgrp )

!           --- Fill information for element group in E and allocate some
!               element arrays

            call sepderelgrp
            if ( debug ) write(irefwr,1) 'inelem', inelem

            nel = skipelgrps(ielgrp)
            do irelem = 1, nel

!           --- fill arrays index1, index2, index3, index4, and index5

               E%first = ifirst
               ielem = list(lcoun)
               ikelmc = ikmcrf + (ielem-ielmrf-1)*inpelm
               index1 => kmeshc(ikelmc+1:ikelmc+inpelm)

!              --- Fill arrays index2/3/4/5

               call sepdervindex ( ikelmc, ipkproba, SMF%ioutvc )

!              --- Copy information into type elem

               call sepcopyelem

!              --- Create element vector as well as weight vector

               if ( irelem==nel ) E%last=1  ! mark last element
               call sepelderv
               if ( ierror/=0 ) go to 1000

               if ( notvec/=0 ) then

!              --- skip other elements of this group since the element vector
!                  is zero

                  if ( irelem>1 ) then
                     call errint ( ielgrp, 1 )
                     call errint ( irelem, 2 )
                     call errsub ( 211, 2, 0, 0 )
                     go to 1000
                  end if
                  ikmcrf = ikmcrf+inpelm*inelem
                  ielmrf = ielmrf+inelem
                  go to 500

               end if
               ifirst = 1
               call to0102 ( list(lcoun) )
               lcoun = lcoun + 3

            end do

         end if

500      ipkproba = ipkproba+inpelm
         call sepdervdalloc

      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,2) 'valintegral', valintegral
         write(irefwr,*) 'End sepderbodyint'

      end if  ! ( debug )
      call erclos ( 'sepderbodyint' )

      end subroutine sepderbodyint
