      subroutine sepderintegral ( icurvs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-01-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute integrals over derivatives
!
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     integral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesmf
      use sepmodulemain
      use sepmodulebuild
      use sepmoduleoldvecs
      use sepmoduleelm
      use sepmodulefiles
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: icurvs(*)

!     icurvs         i    Integer array containing the curve numbers for the
!                         computation of the boundary integral.
!                         A negative curve number indicates that the curve
!                         must be computed in the reversed direction.
!                         The curves including their sign must form one
!                         contiguous curve
!                         The number of curves is stored in array iinder
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, allocatable :: list(:,:)
      integer, target :: ioutvc

!                    they are not
!     ioutvc         Standard SEPRAN array to be used only to store
!                    information to compute icouno
!     list           Array of length 3*nnodes-1, containing the list of all
!                    curves to be considered in common. For each two points
!                    only one element is found.
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INITDR         Initialize parameters for subroutine TO0100 and store these
!                    parameters in common CSUBWK. Perform some tests
!     PRININ         print 1d integer array
!     SEPASSEMSTREAC Reserve space in array IBUFFR with respect to
!                    subroutine sepcompreac
!                    Fill array ipoint in common CSUBWK
!     SEPDERBODYINT  Actual body of subroutine to0100, to compute derived
!                    quantities or special integrals along boundaries
!                    over derivatives
!     SEPDERINTCURVS Fill array list with information about elements for
!                    boundary integrals
!     SEPDERINTOUT   Fill array ioutvc with information that is used to
!                    compute icouno
!     SEPDERSTARTINT Perform some tests for subroutine sepderintegral
!     SEPDERTYPE     Analyze information from backing storage (file 1), as
!                    stored in istfl1 for standard elements with standard
!                    element number > 99 for given icheld
!     SEPFILLMATSTR2 Fill parameters in module sepmodulematstr
!     SEPREACSTART1  Start of assembly process
!                    Make all initializations, fill modules and declare arrays
!     SEPRMMATSTR2   nullify arrays in module sepmodulematstr
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepderintegral' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepderintegral'
         call prinin ( iinder, 20, 'iinder' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Perform some checks
!         Make initializations

      call sepderstartint
      if ( ierror/=0 ) go to 1000

!     --- Start of assembly process
!          Make some initializations, fill modules and declare arrays

      call sepreacstart1
      if ( ierror/=0 ) go to 1000

!     --- Initialize some parameters for the element groups

      call sepassemstreac

      ioutvc = 0
      SMF%ioutvc => ioutvc

!         Perform some checks
!         Make initializations

      call initdr ( icurvs )

      if ( ierror/=0 ) go to 1000

      if ( debug ) write(irefwr,1) 'ichderiv/2', ichderiv

!     --- test if for given icheld the parameters ichderiv, ivec are equal
!         for all itype of the problem
!         allocate temporary space for array of length 2*nrec1

      call sepfillmatstr2

      if ( debug ) write(irefwr,1) 'ichderiv, ichtst/1', ichderiv, ichtst

      call sepdertype

!     --- Fill array ioutvc with information to compute icouno

      call sepderintout ( ioutvc )

!     --- First find all nodes at the curves in icurvs
!         Create array list

      allocate ( list(3,npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, 3*npoint, 'list' )
      if ( ierror/=0 ) go to 1000
      call sepderintcurvs ( list, icurvs )

      if ( ierror/=0 ) go to 1000

!     --- Actual computations

      if ( numol0==0 ) numold = 1
      call sepderbodyint ( list )
      if ( ierror/=0 ) go to 1000

!     --- Delete temporary "permanent" arrays

      deallocate ( istold, istold1, oldsols, indoldsols, list, stat=error )
      if ( error/=0 ) call erdealloc ( error, 'istold' )

      call seprmmatstr2  ! nullify arrays in modulematstr

      call seprmsolinfo ( ioutvc )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepderintegral'

      end if  ! ( debug )
      call erclos ( 'sepderintegral' )

      end subroutine sepderintegral
