      subroutine sepintcoor ( isol, solint, coorlc, nunks, ncoor, &
                              iinmaplc, maplc )
! ======================================================================
!
!        programmer    Bas van Rens
!        version  1.0  date 01-06-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpolate the solution as stored in array isol
!     from nodal points in array kmesh onto points with coordinates
!     stored in coorlc, store the result solint
!     Old subroutine intcoor
! **********************************************************************
!
!                       KEYWORDS
!
!     interpolation
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulemeshinf
      use sepmodulemnsub
      use sepmodulemain
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isol, ncoor, nunks, iinmaplc(*)
      double precision coorlc(ndim,ncoor), solint(nunks,ncoor)
      type (mapping) :: maplc

!     coorlc     i    array of length ndim x ncoor containing the co-ordinates
!                     of the points in which interpolation must be performed
!     iinmaplc     i    Input array defined by the user
!                     iinmaplc gives some extra information about the
!                     interpolation
!                     Array iinmaplc must be filled as follows:
!                     1:    Last entry that has been filled by the user
!                     2:    infmap, indicates if and how array map is used
!                           Possible values:
!                           0:  Array map is not used
!                           1:  Information about the formal mapping is stored
!                               in array map
!                           2:  Array map has been filled before for these two
!                               meshes and the information is reused for the
!                               mapping
!                           Default value : 0
!                     3:    minmax, indicates if the interpolation is free (0)
!                           or that the interpolated values are restricted to
!                           the range of the original array
!                           In that case we get:
!                           uold_iun(min)<=unew_iun<=uold_iun(max)
!                           for each physical unknown iun
!                           Default value : 0
!     isol       i    Standard SEPRAN array containing information of the
!                     solution array
!     maplc      i    Mapping array
!     ncoor      i    Number of points in solint
!     nunks      i    Number of unknowns per point in solint
!     solint     o    Array to be filled by the interpolation
!                     The size of the array is nunks x ncoor
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nmap=3
      integer iinmapcp(nmap), icurvs(1), iseqkm1, iseqkm2, &
              i, isol2(5), infmap, minmax, inunkp, iinputmsh(2)
      type (mapping) :: maploc
      integer, pointer :: kprob2, kmesh2

!     i              Counting variable
!     icurvs         Dummy array
!     iinmapcp       Copy of array iinmaplc extended with defaults
!     iinputmsh      Input array for subroutine intmshm2
!     infmap         iinmaplc(2)
!     inunkp         Number of degrees of freedom in point
!     iseqkm1        Sequence number with respect to km of first mesh
!     iseqkm2        Sequence number with respect to km of second mesh
!     isol2          temporary standard array that will contain the solution
!     kmesh2         standard kmesh but only filled with the necessary
!                    information to call intmsh2m and let it do the work
!     kprob2         standard kprob but only filled with the necesary
!                    information to call intmsh2m and let it do the work
!     maploc         Local array map
!     minmax         iinmaplc(3)
!     nmap           Length of array iinmapcp
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     DCOPY          Copy double precision dx to double precision dy
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     ERTRACE        Prints trace back of subroutines
!     INTMSHM2       Make a formal interpolation from one mesh to another (2D)
!     INTMSHM3       Make a formal interpolation from one mesh to another (3D)
!     INTSOLMP       Interpolate array isol1 into array isol2
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
!     SEPCLEAR       ?
!     SEPCRMESHINFO  create a new array in km
!     SEPCRPROBINFO  create a new array in kp
!     SEPRMMESHINFO  Remove array with sequence number iseq from km
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1080   NUNKS too large
!    1081   NUNKS too small
!    1655   Element of array out of range
!    1657   Array map does not correspond to kmesh1 and kmesh2
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Check arrays kmesh and kprob
!   Make a new mesh based on array coor (kmesh2)
!   Make a formal mapping
!   Make a new problem description (kprob2)
!   Interpolate and store the result in isol2
!   Copy the solution into solint
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepintcoor' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepintcoor'
         call ertrace
         write(irefwr,1) 'nmap, nunks, ncoor, ndim', nmap, nunks, ncoor, ndim
    !     call prinin ( iinmaplc, iinmaplc(1), 'iinmaplc' )
    !     call prinrl1 ( coorlc, ncoor, ndim, 'coorlc' )
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Copy array iinmaplc into iinmapcp and extend with defaults

      do i = 1, nmap
         iinmapcp(i) = 0
      end do
      do i = 2, min(nmap,iinmaplc(1))
         iinmapcp(i) = iinmaplc(i)
      end do
      infmap = iinmapcp(2)
      if ( infmap<0 .or. infmap>2 ) then
         call errint ( 2, 1 )
         call errint ( iinmaplc(2), 2 )
         call errint ( 0, 3 )
         call errint ( 2, 4 )
         call errchr ( 'iinmaplc', 1 )
         call errchr ( 'infmap', 2 )
         call errsub ( 1655, 4, 0, 2 )
      end if
      minmax = iinmapcp(3)
      if ( minmax<0 .or. minmax>1 ) then
         call errint ( 3, 1 )
         call errint ( minmax, 2 )
         call errint ( 0, 3 )
         call errint ( 1, 4 )
         call errchr ( 'iinmaplc', 1 )
         call errchr ( 'minmax', 2 )
         call errsub ( 1655, 4, 0, 2 )
      end if
      if ( ierror/=0 ) go to 1000

      iseqkm1 = iseqkm

!     --- Fill necessary parts of kmesh2

      kmesh2 => kmesh2d(maxmesh+1)
      kprob2 => kprob2d(maxmesh+1)
      call sepcrmeshinfo ( kmesh2 )
      iseqkm2 = iseqkm
      km(iseqkm2)%kmesh(8) = ncoor
      km(iseqkm2)%kmesh(27) = 0

      if ( debug ) write(irefwr,1) 'iseqkm1, iseqkm2', iseqkm1, iseqkm2

!     --- find the correct part of kprob

      call sepactsolbf2 ( isol )

!     --- Create kprob2

      call sepcrprobinfo ( kprob2, 1 )
      kprobpart => kp(iseqkp)%kprobsub(1)
      kprobloc => kprobpart%kprobloc

!     --- Fill the coordinates in part i

      allocate ( km(iseqkm2)%coor(ndim,ncoor) )
      km(iseqkm2)%coor = coorlc

!     --- Call intmshm2 to do the setup work

      isol2 = 0

!     --- Fill the necessary parts of kprob2
!         trick to evade the iprob problem in ini087

      kprobloc(40) = 1  ! nprob = 1

!     --- copy the necessary parts of kprob into kprob2

      if ( nunks>nphysi ) then

!     --- nunks too large

         call errint ( nunks, 1 )
         call errint ( nphysi, 2 )
         call errsub ( 1080, 2, 0, 0 )

      else if ( nunks<nphysi ) then

!     --- nunks too small

         call errint ( nunks, 1 )
         call errint ( nphysi, 2 )
         call errwar ( 1081, 2, 0, 0 )

      end if
      nphysi      = min(nphysi,nunks)
      kprobloc(4)  = nphysi
      kprobloc(33) = nphysi
      kprobloc(5)  = nphysi*ncoor
      kprobloc(33) = nphysi

      call sepclear ( maploc )

!     --- Call intsolmp to do the actual work

      if ( debug ) write(irefwr,1) 'infmap, ndim', infmap, ndim

      iinputmsh(1) = 1       ! ichoice for intmshm2
      iinputmsh(2) = 0       ! idim for intmshm2
      if ( infmap==0 ) then

!     --- Array map is not used

         if ( ndim==2 ) then
            call intmshm2 ( kmesh, kmesh2, maploc, iinputmsh, icurvs )
         else
            call intmshm3 ( kmesh, kmesh2, maploc, 1 )
         end if
         call intsolmp ( meshnr, maxmesh+1, maploc, isol, &
                         isol2, minmax, 1 )

         if ( associated(maploc%imap) ) deallocate ( maploc%imap, maploc%rmap )

      else if ( infmap==1 ) then

!     --- Array map must  be filled

         if ( ndim==2 ) then
            call intmshm2 ( kmesh, kmesh2, maplc, iinputmsh, icurvs )
         else
            call intmshm3 ( kmesh, kmesh2, maplc, 1 )
         end if
         call intsolmp ( meshnr, maxmesh+1, maplc, isol, &
                         isol2, minmax, 1 )

      else

!     --- Array map is reused, check its contents

         if ( maplc%iseqkm1/=iseqkm1 ) then

!        --- Array maplc does not correspond to arrays kmesh1 and kmesh2

            call errsub ( 1657, 0, 0, 0 )

         end if
         if ( ierror/=0 ) go to 1000
         call intsolmp ( meshnr, maxmesh+1, maplc, isol, &
                         isol2, minmax, 1 )

      end if

!     --- copy the solution into solint

      usoltemp => ks(isol2(1))%sol
      length = nphysi*ncoor
      if ( ks(isol)%compl==1 ) length = 2*length
      call dcopy ( length, usoltemp, 1, solint, 1 )

!     --- Delete superfluous arrays

      call seprmsolinfo ( isol2(1) )
      call seprmmeshinfo ( kmesh2 )
      iseqkm = iseqkm1
      coor => km(iseqkm)%coor

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepintcoor'

      end if
      call erclos ( 'sepintcoor' )

      end subroutine sepintcoor
