      subroutine sepintpolms ( meshin, meshout, iinmaplc, isol1, isol2, &
                               maplc, icurvs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-11-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     This subroutine interpolates the solution as stored in array isol1
!     from nodal points in array kmesh1 onto nodal points in array kmesh2
!     and stores the result in array isol2
!     Information about the formal mapping may be stored in array maplc
!     Old subroutine intpolms
! **********************************************************************
!
!                       KEYWORDS
!
!     interpolation
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesol
      use sepmodulekmeshv
      use sepmodulemain
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer meshin, meshout, isol1, isol2, iinmaplc(*), &
              icurvs(*)
      type (mapping) :: maplc

!     icurvs         i    Integer array containing the user points, curve or
!                         surface numbers that must be printed.
!                         Pos. 1 indicates how array icurvs is filled.
!                         Possible values:
!                          0   only one curve is used. The curve number is
!                              stored in icurvs(2)
!                         -1   The solution is printed along the curves
!                              icrv1 to icrv2. icrv1 is stored in icurvs(2) and
!                              icrv2 in icurvs(3).
!                         n>0  The curve numbers numbers are stored in
!                              icurvs(2) to icurvs(n+1). That sequence is used.
!     iinmaplc       i    Input array defined by the user
!                         iinmaplc gives some extra information about the
!                         interpolation
!                         Array iinmaplc must be filled as follows:
!                         1:    Last entry that has been filled by the user
!                         2:    infmap, indicates if and how array map is used
!                               Possible values:
!                               0:  Array map is not used
!                               1:  Information about the formal mapping is
!                                   stored in array map
!                               2:  Array map has been filled before for these
!                                   two meshes and the information is reused
!                                   for the mapping
!                               Default value : 0
!                         3:    minmax, indicates if the interpolation is free
!                               (0) or that the interpolated values are
!                               restricted to the range of the original array
!                               In that case we get:
!                               uold_iun(min)<=unew_iun<=uold_iun(max)
!                               for each physical unknown iun
!                               Default value : 0
!                         4:    ichoice    Choice parameter indicating if the
!                               new solution vector has the same structure as
!                               isol1 (0,10) or that it must be of type 110
!                               (1,11)
!                               if>9 points outside the original region are
!                               not interpolated
!                               Default value : 0
!                         5:    idim    Choice parameter indicating if all
!                                       surfaces curves, volumes are used (0)
!                                       or a restricted set stored in icurvs (1)
!                               Default value : 0
!     isol1          i    Standard SEPRAN array containing information of the
!                         vector to be interpolated
!     isol2          o    Standard SEPRAN array containing information of the
!                         interpolated vector
!     maplc         i/o   contains information of the mapping
!     meshin         i    Sequence number of first mesh to be used
!     meshout        i    Sequence number of last mesh to be used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nmap
      parameter (nmap=5)
      integer iinmapcp(nmap), infmap, minmax, ndimlc, &
              itypvc, ichoice, idim, iinputmsh(2), iseqkm1, iseqkm2, &
              kmesh1, kmesh2
      type (mapping) :: maploc
      double precision :: timebefore(2)

!     ichoice        Choice parameter indicating if the new
!                    solution vector has the same structure as isol1
!                    (0,10) or that it must be of type 110 (1,11)
!                    if>9 points outside the original region are
!                    not interpolated
!     idim           See iinmaplc(5)
!     iinmapcp       Copy of array iinmaplc extended with defaults
!     iinputmsh      Input array for subroutine intmshm2
!     infmap         iinmaplc(2)
!     iseqkm1        Sequence number with respect to km of first mesh
!     iseqkm2        Sequence number with respect to km of second mesh
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     kmesh1         Standard SEPRAN array containing information of mesh 1
!     kmesh2         Refers to mesh2
!     maploc         Local array map
!     minmax         iinmaplc(3)
!     ndimlc         Dimension of the space.
!     nmap           Length of array iinmapcp
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLMN         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERSETTIME      Get the CPU time at start of subroutine
!     ERTRACE        Prints trace back of subroutines
!     INTMSHM1       Make a formal interpolation from one mesh to another (1D)
!     INTMSHM2       Make a formal interpolation from one mesh to another (2D)
!     INTMSHM3       Make a formal interpolation from one mesh to another (3D)
!     INTSOLMP       Interpolate array isol1 into array isol2
!     PRININ         print 1d integer array
!     SEPCOPYVC      Copy one SEPRAN vector into another one
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      60   Input solution array has not been filled or is of the wrong
!     419   NDIM # 2
!    1299   The dimension in KMESH2 is different from that in KMESH1
!    1655   Element of array out of range
!    1657   Array map does not correspond to kmesh1 and kmesh2
!    2765   Value of integer parameter not yet implemented for this ndim
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Perform some tests
!     Copy array iinmaplc into iinmapcp and extend with defaults
!     If ( ndim=2 ) then
!        if ( infmap=0 ) then
!           Compute mapping using a local array map (maploc)
!           Interpolate using maploc
!        else if ( infmap=1 ) then
!           Compute mapping and store in map
!           Interpolate using map
!        else ( infmap=2 )
!           Interpolate using map
!     else ( ndim=3 )
!        Perform old interpolation
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepintpolms' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepintpolms'
         call ertrace
         write(irefwr,1) 'meshin, meshout', meshin, meshout
         call prinin ( iinmaplc, iinmaplc(1), 'iinmaplc' )
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      kmesh1 = kmesh2d(meshin)
      kmesh2 = kmesh2d(meshout)

      iseqkm1 = kmesh1
      iseqkm2 = kmesh2

      if ( debug ) write(irefwr,1) 'iseqkm1, iseqkm2', iseqkm1, iseqkm2

      if ( itime>0 ) call ersettime ( timebefore )
      if ( ierror/=0 ) go to 1000
      ndimlc = km(iseqkm1)%kmesh(6)
      if ( km(iseqkm2)%kmesh(6)/=ndimlc ) then
         call errint ( km(iseqkm2)%kmesh(6), 1 )
         call errint ( ndimlc, 1 )
         call errsub ( 1299, 2, 0, 0 )
      end if
      if ( ierror/=0 ) go to 1000

      if ( debug ) write(irefwr,1) 'ndimlc', ndimlc

!     --- Copy array iinmaplc into iinmapcp and extend with defaults

      iinmapcp(1:nmap) = 0
      idim = min(nmap,iinmaplc(1))
      iinmapcp(2:idim) = iinmaplc(2:idim)
      infmap = iinmapcp(2)
      if ( infmap<0 .or. infmap>2 ) then
         call errint ( 2, 1 )
         call errint ( iinmaplc(2), 2 )
         call errint ( 0, 3 )
         call errint ( 2, 4 )
         call errchr ( 'iinmaplc', 1 )
         call errchr ( 'infmap', 2 )
         call errsub ( 1655, 4, 0, 2 )
      end if
      minmax = iinmapcp(3)
      ichoice = iinmapcp(4)
      idim = iinmapcp(5)
      if ( debug ) &
         write(irefwr,1) 'infmap, minmax, ichoice, ndimlc, idim', &
                          infmap, minmax, ichoice, ndimlc, idim
      if ( ichoice<0 .or. ichoice>11 .or. &
           ichoice<=9 .and. ichoice>=2 ) then
         call errint ( 4, 1 )
         call errint ( ichoice, 2 )
         call errint ( 0, 3 )
         call errint ( 1, 4 )
         call errchr ( 'iinmaplc', 1 )
         call errchr ( 'ichoice', 2 )
         call errsub ( 1655, 4, 0, 2 )
      end if

      if ( idim/=0 .and. ndimlc/=2 ) then

!     --- Value of idim not yet implemented for this dimension

         call errint ( idim, 1 )
         call errint ( ndimlc, 2 )
         call errchr ( 'idim', 1 )
         call errsub ( 2765, 2, 0, 1 )

      end if  ! ( idim/=0 .and. ndimlc/=2 )

      itypvc = ks(isol1)%typevec
      if ( itypvc/=110 .and. itypvc/=115) then
         call errint ( itypvc, 1 )
         call errsub ( 60, 1, 0, 0 )
      end if
      if ( ierror/=0 ) go to 1000

      if ( infmap==2 .and. maplc%iseqkm1==0 ) then

!     --- Array maplc should be used but its first position is zero
!         Copy array

          call sepcopyvc ( isol1, isol2 )
          go to 1000

      end if  ! ( infmap==1 .and. maplc(1)==0 )

      if ( ndimlc==1 ) then

!     --- ndimlc = 1

         if ( infmap==0 ) then

!        --- Array maplc is not used

            nullify ( maploc%imap, maploc%rmap )
            call intmshm1 ( iseqkm1, kmesh2, maploc )
            call intsolmp ( meshin, meshout, maploc, &
                            isol1, isol2, minmax, ichoice )
            deallocate ( maploc%imap, maploc%rmap )

         else if ( infmap==1 ) then

!        --- Array maplc will be filled

            call intmshm1 ( iseqkm1, kmesh2, maplc )
            call intsolmp ( meshin, meshout, maplc, &
                            isol1, isol2, minmax, ichoice )

         else

!        --- Array maplc is reused, check its contents

            if ( maplc%iseqkm1/=iseqkm1 .or. &
                 maplc%iseqkm2/=iseqkm2 ) then

!           --- Array maplc does not correspond to arrays kmesh1 and kmesh2

               call errsub ( 1657, 0, 0, 0 )
            end if
            if ( ierror/=0 ) go to 1000
            call intsolmp ( meshin, meshout, maplc, &
                            isol1, isol2, minmax, ichoice )

         end if

      else if ( ndimlc==2 ) then

!     --- ndimlc = 2

         if ( ichoice>9 ) then
            iinputmsh(1) = 2       ! ichoice for intmshm2, interpolate only in
                                   ! mesh
         else
            iinputmsh(1) = 1       ! ichoice for intmshm2, interpolate for
                                   ! all points
         end if  ! ( ichoice>9 )
         iinputmsh(2) = idim    ! idim for intmshm2

         if ( infmap==0 ) then

!        --- Array maplc is not used

            nullify ( maploc%imap, maploc%rmap )
            call intmshm2 ( kmesh1, kmesh2, maploc, iinputmsh, icurvs )

            call intsolmp ( meshin, meshout, maploc, &
                            isol1, isol2, minmax, ichoice )
            deallocate ( maploc%imap, maploc%rmap )

         else if ( infmap==1 ) then

!        --- Array maplc is filled

            call intmshm2 ( kmesh1, kmesh2, maplc, iinputmsh, icurvs )
            call intsolmp ( meshin, meshout, maplc, &
                            isol1, isol2, minmax, ichoice )

         else

!        --- Array maplc is reused, check its contents

            if ( maplc%iseqkm1/=iseqkm1 .or. &
                 maplc%iseqkm2/=iseqkm2 ) then

!           --- Array maplc does not correspond to arrays kmesh1 and kmesh2

               call errsub ( 1657, 0, 0, 0 )

            end if
            if ( ierror/=0 ) go to 1000

            call intsolmp ( meshin, meshout, maplc, &
                            isol1, isol2, minmax, ichoice )

         end if

      else if ( ndimlc==3 ) then

!     --- ndimlc = 3

         if ( infmap==0 ) then

!        --- Array maplc is not used

            nullify ( maploc%imap, maploc%rmap )
            call intmshm3 ( kmesh1, kmesh2, maploc, 1 )
            call intsolmp ( meshin, meshout, maploc, &
                            isol1, isol2, minmax, ichoice )
            deallocate ( maploc%imap, maploc%rmap )

         else if ( infmap==1 ) then

!        --- Array maplc is filled

            call intmshm3 ( kmesh1, kmesh2, maplc, 1 )
            call intsolmp ( meshin, meshout, maplc, &
                            isol1, isol2, minmax, ichoice )

         else

!        --- Array maplc is reused, check its contents

            if ( debug ) &
               write(irefwr,1) 'maplc%iseqkm', maplc%iseqkm1, maplc%iseqkm2
            if ( maplc%iseqkm1/=iseqkm1 .or. &
                 maplc%iseqkm2/=iseqkm2 ) then

!           --- Array maplc does not correspond to arrays kmesh1 and kmesh2

               call errsub ( 1657, 0, 0, 0 )

            end if
            if ( ierror/=0 ) go to 1000

            call intsolmp ( meshin, meshout, maplc, &
                            isol1, isol2, minmax, ichoice )

         end if

      else

!     --- ndimlc>3

         call errint ( ndimlc, 1 )
         call errint ( 2, 2 )
         call errint ( 3, 3 )
         call errsub ( 419, 3, 0, 0 )

      end if
1000  if ( debug ) write(irefwr,*) 'End sepintpolms'
      call erclmn ( 'sepintpolms', timebefore )
      ierror = 0
      end subroutine sepintpolms
