      subroutine sepprinov ( ivectr, jdim, text, xjdim, jpoint )
! ======================================================================
!
!        programmer    Jaap van der Zanden
!        version  1.0  date 26-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     subroutine sepprinov is used to print SEPRAN solution arrays
!     where the co-ordinates are ordered in increasing order
!     If required only a part of the region is printed
!     Old subroutine prinov
! **********************************************************************
!
!                       KEYWORDS
!
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      character *(*) text
      integer ivectr(*), jdim, jpoint
      double precision xjdim(*)

!     ivectr  i    standard sepran array denoting usol or vector of special
!                  structure defined in nodal points
!     jdim    i    Input parameter for subroutine sepprinov, with the following
!                  meaning:
!                  0  The complete region is printed. The nodes are ordered in
!                     first instance with respect to increasing x1-co-ordinate
!                     then with respect to increasing x2-co-ordinate
!                     and finally with respect to increasing x3-co-ordinate.
!                     The heading above the print is suppressed
!                  1  The complete region is printed. The nodes are ordered in
!                     first instance with respect to increasing x1-co-ordinate
!                     then with respect to increasing x2-co-ordinate
!                     and finally with respect to increasing x3-co-ordinate.
!                  2  The complete region is printed. The nodes are ordered in
!                     first instance with respect to increasing x2-co-ordinate
!                     then with respect to increasing x3-co-ordinate
!                     and finally with respect to increasing x1-co-ordinate.
!                  3  The complete region is printed. The nodes are ordered in
!                     first instance with respect to increasing x3-co-ordinate
!                     then with respect to increasing x1-co-ordinate
!                     and finally with respect to increasing x2-co-ordinate.
!                  -n  See JDIM = n, however only the points with
!                      x jdim = XJDIM(i,1), i = 1, jpoint are selected
!                 10+n See JDIM = n, however only the part of the region given
!                      by xn>=XJDIM(1,1) is printed.
!               --10-n See JDIM = n, however only the part of the region given
!                      by xn<=XJDIM(1,1) is printed.
!                 20+n See JDIM = n, however only the part of the region given
!                      by XJDIM(1,1)<= xn<=XJDIM(1,2) is printed.
!                 30+n See JDIM = n, however only the part of the region given
!                      by XJDIM(1,1)<= x1 <=XJDIM(2,1), &
!                      XJDIM(2,1)<= x2 <=XJDIM(2,2)
!                      XJDIM(1,3)<= x3 <=XJDIM(2,3) is printed.
!     jpoint  i    jpoint is only be used for IDIM = -1, -2 or -3, otherwise
!                  jpoint = 2 ( jdim>30 ), or 1 (rest of the cases)
!     text    i    String variable containing some text
!     xjdim   i    Input array for subroutine sepprinov, see JDIM
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nparm
      parameter ( nparm=12 )
      integer i, inppri(nparm), icurvs(1)
      double precision xminsv, yminsv, zminsv, xmaxsv, ymaxsv, zmaxsv, &
                       rinpri(2)

!     i       Counting variable
!     icurvs  Dummy parameter with respect to seper0075
!     inppri  Integer array in which the user indicates which quantities
!             must be printed by subroutine PRINBN
!             The entries of INPPRI must be filled as follows
!             1:  Indicates calling subroutine:
!                 1: PRINVC
!                 2: PRINCR
!             2:  ICHOIS, indicates the type of print
!                 Possibilities:
!                 0:   The nodal point number is printed followed by the
!                      values in the nodal point
!                 1:   The nodal point number is printed followed by the
!                      co-ordinates and then the values in the nodal point
!             3:  IDIM, indicates if userpoints (idim=1), curves (idim=2)
!                 or surfaces (idim=3) must be printed.
!                 The user point, curve or surface numbers must be stored
!                 in array ICURVS
!             4:  IQUANTITY, indicates which quantity must be printed
!                 Possible values:
!                 0:  The contents of the vector are printed as stored in
!                     the vector corresponding to IVECTR i.e. without
!                     transformation.
!                 1:  The normal component of a vector along the curve
!                     or surface is printed.
!                 2:  The tangential component(s) of a vector along the
!                     curve or surface is printed.
!             5:  NUMBER, indicates which degree of freedom in each point
!                 must be printed. This parameter must be seen in connection
!                 with IQUANTITY.
!                 Possible values:
!                 0:  If iquantity=0, all degrees of freedom are printed
!                     in the nodes defined by icurvs, &
!                     if iquantity>0, the vector from which the normal or
!                     tangential components must be computed is supposed
!                     to be a n-dimensional vector with first degree of
!                     freedom 1, second degree of freedom 2 and third
!                     degree of freedom 3. In this case it is necessary
!                     that IDIM = NDIM (dimension of space)
!                 i>0: If iquantity=0, the degree of freedom i is printed
!                     if iquantity>0, the vector from which the normal or
!                     tangential components must be computed is supposed
!                     to be a n-dimensional vector with first degree of
!                     freedom i, second degree of freedom i+1 and third
!                     degree of freedom i+2. In this case it is necessary
!                     that IDIM = NDIM (dimension of space)
!             6:  ISEQ, indicates the sequence in which the co-ordinates
!                 along the surface must be ordered in case idim=3
!                 Possible values:
!                 1:  increasing x, for fixed x increasing y, &
!                     for fixed y increasing z
!                 2:  increasing y, for fixed y increasing z, &
!                     for fixed z increasing x
!                 3:  increasing z, for fixed z increasing x, &
!                     for fixed x increasing y
!             7:  IREGION, indicates if the complete region must be printed
!                 (0) or that the printing is restricted to a sub region, &
!                 defined by the parameters xmin, xmax, ymin, ymax, zmin, &
!                 zmax in the comon block CPLOT.
!                 Possible values:
!                 0:  The complete region is printed
!                 1:  The region is restricted to xmin<=x<=xmax, &
!                     ymin<=y<=ymax, zmin<=z<=zmax
!                -1:  The region is restricted to x=xmin
!                -2:  The region is restricted to y=ymin
!                -3:  The region is restricted to z=zmin
!                -11: The region is restricted to x>=xmin
!                -12: The region is restricted to y>=ymin
!                -13: The region is restricted to z>=zmin
!                -21: The region is restricted to x<=xmin
!                -22: The region is restricted to y<=ymin
!                -23: The region is restricted to z<=zmin
!             8:  IHEADER, indicates if the header must be printed (0) or
!                 not (1)
!     nparm   Number of parameters in inppri
!     rinpri  dummy
!     xmaxsv  Help variable to save the value of xmax
!     xminsv  Help variable to save the value of xmin
!     ymaxsv  Help variable to save the value of ymax
!     yminsv  Help variable to save the value of ymin
!     zmaxsv  Help variable to save the value of zmax
!     zminsv  Help variable to save the value of zmin
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     SEPER0075      Print the values of a SEPRAN array along curves or surfaces
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      inppri = 0
      inppri(1) = nparm
      inppri(2) = 1
      inppri(6) = 1
      inppri(9) = 1
      inppri(10) = 1
      if ( jdim==0 ) then
         inppri(8) = 1
      else
         inppri(6) = mod(abs(jdim),10)
      end if
      xminsv = xmin
      yminsv = ymin
      zminsv = zmin
      xmaxsv = xmax
      ymaxsv = ymax
      zmaxsv = zmax
      if ( jdim<0 .and. jdim>=-3 ) then

!     --- jdim<0, loop over jpoint cases

         inppri(7) = jdim
         do i = 1, jpoint
            if ( jdim==-1 ) then
               xmin = xjdim(i)
               xmax = xjdim(i)
            else if ( jdim==-2 ) then
               ymin = xjdim(i)
               ymax = xjdim(i)
            else
               zmin = xjdim(i)
               zmax = xjdim(i)
            end if
            call seper0075 ( inppri, ivectr, icurvs, text, rinpri )
            inppri(8) = 1
         end do

      else
         xmin = -rinfin
         ymin = -rinfin
         zmin = -rinfin
         xmax =  rinfin
         ymax =  rinfin
         zmax =  rinfin
         if ( jdim>30 ) then
            xmin = xjdim(1)
            xmax = xjdim(2)
            ymin = xjdim(3)
            ymax = xjdim(4)
            zmin = xjdim(5)
            zmax = xjdim(6)
         else if ( jdim>20 ) then
            if ( jdim==21 ) then
               xmin = xjdim(1)
               xmax = xjdim(2)
            else if ( jdim==22 ) then
               ymin = xjdim(1)
               ymax = xjdim(2)
            else
               zmin = xjdim(1)
               zmax = xjdim(2)
            end if
         else if ( jdim>10 ) then
            if ( jdim==11 ) then
               xmin = xjdim(1)
            else if ( jdim==12 ) then
               ymin = xjdim(1)
            else
               zmin = xjdim(1)
            end if
         else if ( jdim<=-10 ) then
            if ( jdim==-11 ) then
               xmax = xjdim(1)
            else if ( jdim==-12 ) then
               ymax = xjdim(1)
            else
               zmax = xjdim(1)
            end if
         end if
         if ( jdim>=0 .and. jdim<=3 ) then
            inppri(7) = 0
         else
            inppri(7) = 1
         end if
         call seper0075 ( inppri, ivectr, icurvs, text, rinpri )
      end if
      xmin = xminsv
      ymin = yminsv
      zmin = zminsv
      xmax = xmaxsv
      ymax = ymaxsv
      zmax = zmaxsv
      end subroutine sepprinov
