      subroutine sepsortcoor ( iseq, nnodes, inodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-11-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Order nodal points in region with respect to increasing co-ordinates
!     depending on the parameter iseq
! **********************************************************************
!
!                       KEYWORDS
!
!     ordering
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nnodes, inodes(nnodes), iseq

!     inodes        i/o   Array containing the nodal point numbers.
!                         At output the nodes are sorted
!     iseq           i    indicates the sequence in which the co-ordinates
!                         along the surface must be ordered in case idim=3
!                         Possible values:
!                         1:  increasing x, for fixed x increasing y,
!                             for fixed y increasing z
!                         2:  increasing y, for fixed y increasing z,
!                             for fixed z increasing x
!                         3:  increasing z, for fixed z increasing x,
!                             for fixed x increasing y
!     nnodes         i    Number of nodes in inodes
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iwork(nnodes,3), inodsq(nnodes)
      double precision work(nnodes)

!     i              Counting variable
!     inodsq         Work array of length nnodes
!     iwork          integer work array of length 2*nnodes
!     work           Work array of length nnodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
!     PRSORT         Sort real array for increasing sequence
!     PRSW03         rearrange the second and third co-ordinate such that an
!                    increasing sequence is created for nodes with equal
!                    first co-ordinate
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepsortcoor' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepsortcoor'
         write(irefwr,1) 'iseq, nnodes, ndim', iseq, nnodes, ndim
         call prinin ( inodes, nnodes, 'inodes' )
         call prinrl1 ( coor, nnodes, ndim, 'coor' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Sort with respect to co-ordinates according to iseq

      work = coor(iseq,inodes)
      if ( debug ) call prinrl ( work, nnodes, 'work/1' )
      call prsort ( work, iwork, nnodes )
      if ( debug ) call prinin ( iwork, nnodes, 'iwork/1' )
      inodsq = inodes(iwork(:,1))
      if ( debug ) call prinin ( inodsq, nnodes, 'inodsq/1' )

      call prsw03 ( -iseq, ndim, nnodes, inodsq, coor, iwork, &
                    iwork(1,2), work )

!     --- Copy back to inodes

      inodes = inodsq

1000  if ( debug ) then

!     --- Debug information

         call prinin ( inodes, nnodes, 'inodes' )
         write(irefwr,*) 'End sepsortcoor'

      end if  ! ( debug )
      call erclos ( 'sepsortcoor' )

      end subroutine sepsortcoor
