      subroutine sepstream ( ichois, ichels, ivec, istrm, startnode, &
                             strm0, inpvec, ielhlp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-11-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     a stream function or potential is computed
!     for 2-dim problems from inpvec (line elements excluded)
!     Old subroutine to0040
! **********************************************************************
!
!                       KEYWORDS
!
!     compute
!     incompressible
!     stream_function
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulemain
      use sepmodulekmesh
      use sepmodulekprob
      use sepmodulecactl
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istrm, inpvec, ielhlp(*), ichois, &
              ichels, ivec, startnode
      double precision strm0

!     ichels         i    Choice parameter for the element subroutine
!     ichois         i    Choice parameter
!                         ichois is 0, do not skip element groups,
!                         ichois is 1, skip element groups as indicated by
!                         ielhlp
!     ielhlp         i    This array of length nelgrp+5 is only used if
!                         allelements = 1
!                         ielhlp(ielgrp+5) = 0 means elements of element group
!                         ielgrp are skipped, ielhlp(ielgrp+5) = 1, means that
!                         they are assembled
!     inpvec         i    Input vector from which the stream function must be
!                         computed
!     istrm          o    Output vector containing information of the stream
!                         function
!     ivec           i    Sequence number of vector of special structure
!     startnode      i    Starting node number
!     strm0          i    Value of stream function in startnode
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ivctst, knpelm, knodp, lstep, nunkpsav, &
                 numelm, itypvo, inusol, inunkp
      integer, allocatable :: iwork(:)
      double precision :: dum(1), psi
      double precision, pointer :: stream(:)

!                    they are not
!     dum            General double precision dummy variable
!     inunkp         Number of degrees of freedom in point
!     inusol         Position of unknown in array USOL
!     itypvo         Defines type of output vector (110, 115 or 116)
!     ivctst         Test value of ivec
!     iwork          integer work array
!     knodp          Node number
!     knpelm         Number of nodes in element
!     lstep          Step variable for vertices. Usually lstep = 1,
!                    but when the solution must be computed in the vertices
!                    and the element is quadratic then lstep = 2
!     numelm         Number of elements
!     nunkpsav       Help parameter to save the value of nunkp
!     psi            Value of stream function in node
!     stream         stream function
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINLG         Print and checks length of user arrays
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
!     SEPCREASOLBF1  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
!     SEPFILLKPROB   Fill parameters in module sepmodulekprob
!     SEPPOINTKMNEL  Fill array pointnel with number of elements in previous
!                    element groups accumulated
!     SEPSTREAM1     Find start point (knodp) for computation of stream function
!                    corresponding element (ielem) and start value (psi)
!     SEPSTREAM2     yields the element and group in which knodp is lying
!     SEPSTREAM3     mark elements that must be skipped by setting iwork = -2
!                    compute number elements (numelm) remaining
!     SEPSTREAM4     Compute knpelm and lstep as function of ielgrp
!     SEPSTREAM5     Check if start node of stream function is a vertex
!     SEPSTREAM6     Compute stream function in all nodes except start node
!     SEPSTREAMCHK   test if the parameter ivec is equal
!                    for all itype of the problem
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!       1   ICHOIS is incorrect
!     222   input vector is complex:  not yet allowed.
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- initializations and tests

      call eropen ( 'sepstream' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepstream'
         write(irefwr,1) 'ichois, nunkp, inpvec', ichois, nunkp, inpvec
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ichois<0 .or. ichois>1 ) then
         call errint ( ichois, 1 )
         call errsub ( 1, 1, 0, 0 )
         go to 1000
      end if
      if ( ks(inpvec)%compl==1 ) then

!     --- input vector is complex (not allowed)

         call errsub ( 222, 0, 0, 0 )
         go to 1000

      end if

!     --- Activate modules kmesh and kprob

      iprob  = ks(inpvec)%iprob
      call sepactsolbf2 ( inpvec )
      indprflc = indprfi
      kprobflc => kprobfi

 !     call sepfillkprob

      kproba => kprobpart%kproba
      indpra = kprobloc(14)
      indprf = kprobloc(19)
 !     indprfi = indprf
      kprobc => kprobpart%kprobc
      kprobf => kprobpart%kprobf
!      kprobfi => kprobf
      nunkpsav = nunkp

      jcart = ks(inpvec)%jcart+1
      npointcac = npoint
      nelemcac = nelem

      if ( debug ) write(irefwr,1) 'iprob, nunkp, jcart', iprob, nunkp, jcart

!     --- test length of ielhlp

      if ( ichois==1 ) call prinlg('ielhlp',nelgrp+5,dum,ielhlp)

!     --- test if the parameter ivec is equal
!         for all itype of the problem

      ivctst = ivec
      call sepstreamchk ( ivec, ivctst, ichois, ielhlp )

!     --- define output vector

      if (ivctst==0) then
         itypvo=110
      else
         itypvo=115
      end if
      call sepcreasolbf1 ( istrm, itypvo, ivctst, 'stream', 0 )

!     --- clear the output vector and associate pointers wirh ibuffr/buffer

      stream => ks(istrm)%sol
      stream = 0d0

      velocity => ks(inpvec)%sol

!     --- create and clear/fill temporary space for iwork and other help arrays

      allocate ( iwork(nelem), pointnel(nelgrp+1), stat = error )
      if ( error/=0 ) call eralloc ( error, nelem, 'iwork' )
      if ( ierror/=0 ) go to 1000

      iwork = -3

!     --- Fill array pointnel

      call seppointkmnel

!     --- store strm0 and find the first element in which startnode lies
!         (skip line element groups and other forbidden groups)

      call sepstream1 ( ichois, startnode, knodp, psi, strm0, ielhlp )
      if ( ierror/=0 ) go to 1000

!     --- find a "good" starting element to start with

      call sepstream2 ( ichois, ielhlp, knodp )
      if ( ierror/=0 ) go to 1000

!     --- put psi (=strm0) in output vector

      nunkp = nunkpo
      indprf = indprfo
      kprobf => kprobfo
      inusol = sepgetinusol ( knodp, inunkp )+1
      stream(inusol) =  psi
      nunkp = nunkpsav

!     --- mark elements that must be skipped by setting iwork = -2
!         compute number elements (numelm) remaining

      iwork(ielem) = 0  ! element is used
      call sepstream3 ( ichois, numelm, ielhlp, iwork )
      if ( ierror/=0 ) go to 1000

      call sepstream4 ( knpelm, lstep ) ! compute knpelm and lstep

!     --- Check if start node is vertex

      call sepstream5 ( knpelm, lstep, knodp )

      call sepstream6 ( iwork, ichels, stream, velocity, &
                        inpvec, istrm, knodp, numelm, nunkpo )
      nunkp = nunkpsav

!     --- put strm0 in output vector once more

      inusol = sepgetinusol ( knodp, inunkp )+1
      stream(inusol) =  psi

!     --- delete work array

      deallocate ( iwork, pointnel, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  call erclos ( 'sepstream' )
      end subroutine sepstream
