      subroutine plafscanfile ( ipict, xminrd, xmaxrd, yminrd, ymaxrd, &
                                numdigits )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 02-09-2013 Extra parameter numdigits
!        version  2.0  date 31-05-2004 Rename subroutine
!        version  1.0  date 17-05-2003
!
!   copyright (c) 2003-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read all records from a sepran neutral file and detect minimum and maximum
!     x and y values
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     plot
!     neutral_file
!     scan
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulemach
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ipict, numdigits
      real xminrd, xmaxrd, yminrd, ymaxrd

!     ipict          i    Picture number (also file number)
!     numdigits      i    Number of digits in plot file
!     xmaxrd         o    Maximum x-value read in file
!     xminrd         o    Minimum x-value read in file
!     ymaxrd         o    Maximum y-value read in file
!     yminrd         o    Minimum y-value read in file
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ios, iref, lastps, ierrtn
      character * 60 namefl

!     ierrtn         Error return code with the following meaning:
!                    0: No error has been found
!                    1: The file with reference number |irefpl| is empty
!                    2: An incorrect record has been read in the plot file
!                    3: The plot file does not end with the correct record
!                    4: The plot file contains a string with too much characters
!                    5: The plot file contains a polygon consisting of too many
!                       points
!     ios            IOSTAT variable, returns from reading or writing
!     iref           File reference number
!     lastps         Last non-blank position in namefl
!     namefl         Name of default SEPRAN neutral plotting file
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PLAFSCAN       Read all records from a sepran neutral file and detect
!                    minimum and maximum x and y values
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     184   Error in opening of plot file
!    1709   Plot file is empty
!    1710   Plot file is incorrect
!    1711   Plot file end with incorrect record
!    1712   Too much characters in a string
!    1713   Too many nodes in a polygon
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plafscanfile' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plafscanfile'

      end if

!     --- construct name of plot file by concatenation of namepl and picture
!         number

      lastps = index ( namepl, ' ' ) - 1
      namefl = ' '
      namefl(1:lastps) = namepl(1:lastps)
      namefl(lastps+1:lastps+1) = '.'
      if ( numdigits==3 ) then

!     --- file name is of the form  name.xxx

         write ( namefl(lastps+2:lastps+4), 100 ) ipict
100      format ( i3.3 )
         lastps = lastps + 4
      else

!     --- file name is of the form  name.xxxx

         write ( namefl(lastps+2:lastps+5), 110 ) ipict
110      format ( i4.4 )
         lastps = lastps + 5

      end if  ! ( numdigits==3 )

!     --- Open plot file

      iref = abs(irefpl)
      if ( irefpl<0 ) then

!     --- irefpl<0, file is unformatted

         open ( unit=iref, file=namefl(1:lastps), iostat=ios, &
                status='old', form='unformatted' )

      else

!     --- irefpl>0, file is formatted

         open ( unit=iref, file=namefl(1:lastps), iostat=ios, &
                status='old' )

      end if

      if ( ios/=0 ) then

!     --- Error on opening of file
!         Presumably the file does not exist

         call errchr ( namefl(1:lastps), 1 )
         call errint ( ios, 1 )
         call errsub ( 184, 1, 0, 1 )
         go to 1000

      end if

!     --- Read and plot PLOT file

      call plafscan ( xminrd, xmaxrd, yminrd, ymaxrd, ierrtn )
      if ( ierrtn>0 ) then

!     --- Error in PLOT file

         call errchr ( namefl(1:lastps), 1 )
         call errsub ( 1708+ierrtn, 0, 0, 1 )

      end if

!     --- Close plot file

      close ( unit = iref )

1000  call erclos ( 'plafscanfile' )
      if ( debug ) write(irefwr,*) 'End plafscanfile'

      end
