      subroutine plbc25 ( contln, usol, cont, compl, indcol, incol, &
                          rmult, special, iplots, ncont, nelemt )
! ======================================================================
!
!        programmer    Guus Segal
!        version  8.1  date 23-01-2015 Use subroutines
!        version  8.0  date 31-10-2003 Extra parameter kmesh
!        version  7.2  date 03-02-2003 Debug statements
!
!   copyright (c) 1988-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Compute some parameters for subroutine PLCN00 and fill contour levels
!
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     3d
!     plot
!     plot_contour
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical compl
      integer indcol(*), incol(*), special, iplots(*), ncont, nelemt
      double precision contln(*), cont(*), usol(*), rmult

!     compl          i    indication whether the arithmetic is real (false)
!                         or complex (true)
!     cont           o    Array with the contour levels
!     contln         i    User array of length ncntln + 5 (if used) in which the
!                         user is supposed to have stored the contour levels
!                         from position 6
!     incol          o    This array is only used if isubr=2
!                         It is filled with the colours of the levels
!     indcol         i    This array is only used when isubr=2 and jsmoot>0
!                         If JSMOOT=1 it is a variable length array of length:
!                         number of contour levels plus 5
!                         indcol must be filled by the user with the colours
!                         of the various contour levels in increasing sequence.
!                         indcol must be filled from position 6
!                         If JSMOOT=2 it is a variable length array of length 6
!                         In INDCOL(6) the user must store the minimum colour
!                         required
!                         If JSMOOT=3 it is a variable length array of length 7
!                         The minimum colour must be stored in INDCOL(6) the
!                         maximum one in INDCOL(7)
!                         If the range of the colours is less than the available
!                         range of colours the colour numbering is repeated
!     iplots         i    Integer control array in which the user defines which
!                         choices he wants for this subroutine
!                         See subroutine PLCN00 for a description
!     ncont          i    Actual number of contour levels
!     nelemt         i    Number of triangles in new triangular mesh
!     rmult          i    Multiplication factor for the solution
!                         Since the solution is not actually multiplied by this
!                         factor but only the legenda is adapted, it is also
!                         necessary to correct the contour values in some cases
!     special        i    Indicates which special situation is present
!                         Possible values:
!                         0     standard
!                         1     solution vector is of type 116 and a coloured
!                               2d plot is required
!                         2     solution vector is of type 110 or 115 and a
!                               coloured 2d plot is required, whereas there are
!                               no unknowns in the vertices
!     usol           i    Vector from which the contours must be plotted
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision amin, amax, stepx, dummy
      integer i, ichois, icompt, idummy(1), jindcl, m, n, ncntln, number

!     amax           Maximum value in solution array
!     amin           Minimum value in solution array
!     dummy          Dummy parameter
!     i              Counting variable
!     ichois         Choice parameter
!     icompt         See IPLOTS(2)
!     idummy         Dummy parameter
!     jindcl         See IPLOTS(5)
!     m              Dummy parameter
!     n              Dummy parameter
!     ncntln         Parameter to compute the number of contour levels
!     number         The number-th unknown in each nodal point of the array
!                    is considered. If the array is complex number=2*i-1
!                    corresponds to the real part of array i and number=2*i
!                    to the imaginary part of array i
!     stepx          Step value between maximum and minimum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     DCOPY          Copy double precision dx to double precision dy
!     DSCAL          Blas routine to scale a vector
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INI023         Compute minimum and maximum value in solution array
!     PL0004         Round minimum and maximum values
!     PLBCMINMAX     Compute minimum and maximum value of usol(number,*)
!     PLOTFILLCOLORS Fill array with sequence numbers of colors
!     PRINLG         Print and checks length of user arrays
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     493   Contour levels are not in ascending sequence
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plbc25' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plbc25'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if

      ichois = iplots(2)
      number = iplots(3)
      ncntln = iplots(4)
      jindcl = iplots(5)
      icompt = iplots(21)
      if ( debug ) then
         write(irefwr,1) 'ichois, number, ncntln, jindcl, icompt', &
                          ichois, number, ncntln, jindcl, icompt
         write(irefwr,*) 'special, compl ', special, compl
      end if

!     --- Fill contour levels in array icont

      if ( ncntln>0 ) call prinlg ( 'contln' , -ncont-5, contln, &
           idummy )

      if ( icompt>0 ) then

!     --- icompt>0, hence only the mininmum and/or the maximum value are
!                     given

         if ( icompt<=2 ) then

!        --- icompt<=2  Compute minimum and maximum value

            if ( special==0 ) then

               call ini023 ( usol, number, amin, amax, compl )

            else if ( special==1 ) then

               call plbcminmax ( usol, number, nphysi, amin, amax )
            else

               amin = minval(usol(1:nelemt))
               amax = maxval(usol(1:nelemt))

            end if

         end if
         if ( icompt==1 .or. icompt==3 ) amin = contln(6)
         if ( icompt==2 .or. icompt==3 ) amax = contln(7)

      else if ( ncntln>0 ) then

!     --- icompt = 0  and  ncntln>0   Copy contln into ibuffr
!         Multiplication of the solution by rmult is the same as
!         division of the contour levels by rmult

         call dcopy ( ncntln, contln(6), 1, cont, 1 )
         if ( abs(rmult-1d0)<sqreps ) call dscal ( ncntln, 1d0/rmult, &
              cont, 1 )

      else

!     --- icompt = 0  and  ncntln<=0  Compute minimum and maximum value

         if ( special==0 ) then

!        --- ispecial = 0, standard case

            if ( debug ) then

!           --- Debug information

               write(irefwr,1) 'indprfi, indprpi, npoint', &
                                indprfi, indprpi, npoint
               write(irefwr,1) 'number, nphysi', number, nphysi
               call prinrl ( usol, npoint, 'usol')

            end if

            call ini023 ( usol, number, amin, amax, compl )

         else if ( special==1 ) then

            call plbcminmax ( usol, number, nphysi, amin, amax )
         else

           amin = minval(usol(1:nelemt))
           amax = maxval(usol(1:nelemt))

         end if
         if ( debug ) write(irefwr,2) 'amin, amax', amin, amax

      end if

      if ( ncntln<=-10000 ) then

!     --- ncntln = -10000 or -10001, use negative and positive values

         if ( amin<0d0 ) then

!        --- amin<0d0, subdivide the space between 0 and amin

            ncont = iplots(24)
            if ( amax>=0d0 ) then

!           --- amax>=0d0

               if ( ncntln==-10001 ) then
                  amin = amin * rmult
                  dummy   = 0d0
                  call pl0004 ( amin, dummy, m, n )
                  amin = amin / rmult
               end if
               stepx = ( -amin ) / ( ncont-1d0 )
               do i = 1, ncont
                  cont(i) = amin + (i-1d0) * stepx
               end do

            else

!           --- amax<0d0

               if ( ncntln==-10001 ) then
                  amin = amin * rmult
                  amax = amax * rmult
                  call pl0004 ( amin, amax, m, n )
                  amin = amin / rmult
                  amax = amax / rmult
               end if
               stepx = ( amax-amin ) / ( ncont-1d0 )
               do i = 1, ncont
                  cont(i) = amin + (i-1d0) * stepx
               end do

            end if

         else

!        --- No negative values available, decrease ncont

            ncont = 0

         end if

         if ( amax>0d0 ) then

!        --- amax>0d0, subdivide the space between 0 and amax

            if ( amin<=0d0 ) then

!           --- amin<=0d0

               if ( ncntln==-10001 ) then
                  amax = amax * rmult
                  dummy   = 0d0
                  call pl0004 ( dummy, amax, m, n )
                  amax = amax / rmult
               end if
               stepx = ( amax ) / ( iplots(23)-1d0 )
               do i = 1, iplots(23)
                  cont(i+ncont-1) = (i-1d0) * stepx
               end do
               ncont = ncont-1+iplots(23)

            else

!           --- amin>0d0

               if ( ncntln==-10001 ) then
                  amin = amin * rmult
                  amax = amax * rmult
                  call pl0004 ( amin, amax, m, n )
                  amin = amin / rmult
                  amax = amax / rmult
               end if
               ncont = iplots(23)
               stepx = ( amax-amin ) / ( ncont-1d0 )
               do i = 1, ncont
                  cont(i) = amin + (i-1d0) * stepx
               end do

            end if

         end if

      else if ( icompt>0 .or. ncntln<=0 ) then

!     --- Either icompt>0 or ncntln<=0
!         Compute contour levels

         if ( ncntln<0 .and. ncntln>=-1000 ) then

!        --- Contour levels must be rounded
!            Since muliplication of the solution by a constant is equivalent
!            to multiplication of the legenda by this value, the multiplied
!            values must be rounded and corrected afterwards

            amin = amin * rmult
            amax = amax * rmult
            call pl0004 ( amin, amax, m, n )
            amin = amin / rmult
            amax = amax / rmult

            if ( abs(amax-amin)<=sqreps*(abs(amax)+abs(amin)) ) &
               amin = amax

         end if
         stepx = ( amax-amin ) / ( ncont-1d0 )
         do i = 1, ncont
            cont(i) = amin + (i-1d0) * stepx
         end do

      end if

      if ( ichois==2 ) then

!     --- ichois = 2, fill coloured contour levels
!         Copy array indcol in INCOL

         call plotfillcolors ( incol, indcol, ncont, jindcl )

!        --- check sequence of contour levels

         if ( icompt==0 .and. ncntln>0 ) then

            do i = 2, ncntln
               if ( contln(i+5)<=contln(i+4) ) then
                  call errint ( i, 1 )
                  call erreal ( contln(i+5), 1 )
                  call erreal ( contln(i+4), 2 )
                  call errsub ( 493, 1, 2, 0)
               end if
            end do

         end if

      end if

      if ( debug ) write(irefwr,*) 'End plbc25'
      call erclos ( 'plbc25' )

      end
