      subroutine plelem ( ncontr, coor, kmeshc, ilow, nelem, inpelm, &
                          nboun, circle, ndim, fillcl )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.0  date 15-09-2010 Remove work arrays
!        version  4.4  date 25-04-2010 New call to plaf05
!        version  4.3  date 14-02-2003 Debug statements
!
!   copyright (c) 1984-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    plot elements for one sequence number
!    when jmark<3 the elements are provided with element numbers
!    If this subroutine is changed also change plelempar
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     plot_mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical ncontr, circle, fillcl
      integer nelem, inpelm, kmeshc(inpelm,nelem), ilow, nboun, ndim
      double precision coor(ndim,*)

!     circle         i    indication whether the element is two-dimensional
!                         (true) or not (false)
!                         when circle is true the first and last point are
!                         connected
!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     fillcl         i    Indicates if the elements must be filled with a colour
!                         (true) or that only the boundaries must be plotted
!                         (false)
!                         If fillcl = true, the colour is given by dcolor, the
!                         boundaries get the default colour. Otherwise the
!                         colour of the boundaries is defined by dcolor
!     ilow           i    elements are numbered from element number ilow
!     inpelm         i    number of points per element
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     nboun          i    number of points in element to be plotted
!                         nboun<0 defines special situations:
!                         -1:  The element is a quadrilateral consisting of
!                              boundary points only. On each side the number of
!                              boundary points is equal to inpelm/4+1
!                              The numbering of the nodes is in the direction of
!                              the axis, i.e. first the under boundary, then the
!                              under side points etc. Finally the upper boundary
!                              from left to right
!                         -2:  The element is a quadrilateral consisting of
!                              boundary points and internal points, where the
!                              number of points on a side is equal to
!                              sqrt(inpelm). Internal points are
!                              created by taking the product of directions.
!                              The numbering is the same as in the case -1
!     ncontr         i    indication whether the elements are contracted (false)
!                         or not (true)
!     ndim           i    Dimension of the space.
!     nelem          i    Number of elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielem, j, n, nodp, icolsv, imark
      double precision c1, c2
      double precision, allocatable :: x1(:,:)
      logical plnumb

!     c1             Help variable
!     c2             Help variable
!     icolsv         Help parameter to save the colour parameter dcolor
!                    temporarily
!     ielem          Element number
!     imark          jmark modulo 10
!     j              Counting variable
!     n              number of basis functions for each component
!     nodp           Nodal point sequence number
!     plnumb         If true element numbers must be plotted
!     x1             work array of length 2*nboun
!                    If fillcl = true, the length of this work array
!                    should be 4 * nboun
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
!     PLAF05         Plot polygon
!     PLAFEP         General SEPRAN low level plotting subroutine
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plelem' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plelem'
         call ertrace
         write(irefwr,1) 'nelem, inpelm, nboun', nelem, inpelm, nboun
         call prinin1 ( kmeshc, nelem, inpelm, 'kmeshc' )
  1      format ( a, 1x, (10i6) )

      end if

      if ( nboun>0 ) then
         n = nboun
      else if ( nboun==-1 ) then
         n = inpelm
      else
         c1 = inpelm
         c1 = sqrt(c1)
         n = nint(c1)
         n = 4 * (n-1)
      end if  ! ( nboun>0 )
      n = max(n,inpelm)

      allocate ( x1(2,n), stat = error )
      if ( error/=0 ) call eralloc (  error, 2*n, 'x1' )
      if ( ierror/=0 ) go to 1000

      imark = mod(jmark,10)
      plnumb = imark<=2 .or. imark==6
      if ( ndim==2 ) then

!     --- ndim = 2, two-dimensional elements

         do ielem = 1, nelem

!        --- store coordinates of boundary points in arrays x1

            if ( nboun==-1 ) then

!           --- nboun = -1, special situation
!               First the under boundary

               n = inpelm/4+1
               do j = 1, n
                  nodp = kmeshc(j,ielem)
                  x1(1,j) = coor(1,nodp)
                  x1(2,j) = coor(2,nodp)
               end do

!              --- Next the sides

               do j = 1, n-2
                  nodp = kmeshc(n+2*j-1,ielem)
                  x1(1,inpelm+1-j) = coor(1,nodp)
                  x1(2,inpelm+1-j) = coor(2,nodp)
                  nodp = kmeshc(n+2*j,ielem)
                  x1(1,n+j) = coor(1,nodp)
                  x1(2,n+j) = coor(2,nodp)
               end do

!              --- Upper boundary

               do j = 1, n
                  nodp = kmeshc(inpelm+1-j,ielem)
                  x1(1,j+2*n-2) = coor(1,nodp)
                  x1(2,j+2*n-2) = coor(2,nodp)
               end do
               n = inpelm

            else if ( nboun==-2 ) then

!           --- nboun = -2, special situation
!               First the under boundary

               c1 = inpelm
               c1 = sqrt(c1)
               n = nint(c1)
               do j = 1, n
                  nodp = kmeshc(j,ielem)
                  x1(1,j) = coor(1,nodp)
                  x1(2,j) = coor(2,nodp)
               end do

!              --- Next the sides

               do j = 1, n-2
                  nodp = kmeshc(j*n+1,ielem)
                  x1(1,4*n-3-j) = coor(1,nodp)
                  x1(2,4*n-3-j) = coor(2,nodp)
                  nodp = kmeshc(n*(j+1),ielem)
                  x1(1,n+j) = coor(1,nodp)
                  x1(2,n+j) = coor(2,nodp)
               end do

!              --- Upper boundary

               do j = 1, n
                  nodp = kmeshc(inpelm+1-j,ielem)
                  x1(1,j+2*n-2) = coor(1,nodp)
                  x1(2,j+2*n-2) = coor(2,nodp)
               end do
               n = 4 * (n-1)

            else

!           --- Standard situation, iboun>0

               do j = 1, nboun
                  nodp = kmeshc(j,ielem)
                  x1(1,j) = coor(1,nodp)
                  x1(2,j) = coor(2,nodp)
               end do
               n = nboun

            end if
            if ( .not. ncontr .or. plnumb ) then

!           --- ncontr = false, or plnumb = true,  compute centroid

               c1 = 0d0
               c2 = 0d0
               do j = 1, n
                  c1 = c1+x1(1,j)
                  c2 = c2+x1(2,j)
               end do
               c1 = c1/n
               c2 = c2/n
               if ( plnumb ) &
                  call plafep ( 9, c1, c2, -0.06d0, 0d0, ielem+ilow-1 )
               if ( .not. ncontr ) then

!              --- ncontr = false  contract the elements

                  do j = 1, n
                     x1(1,j) = (c1+4d0*x1(1,j))*.2d0
                     x1(2,j) = (c2+4d0*x1(2,j))*.2d0
                  end do
               end if

            end if

!           --- plot element

            if ( fillcl ) call plaf05 ( 1, dcolor, n, x1, 2 )
            call plafep(4, x1(1,1), x1(2,1), 0d0, 0d0, 0)
            if ( fillcl ) then
               icolsv = dcolor
               dcolor = 0
            end if
            if ( n>1 ) then
               do j = 2, n
                  call plafep(5, x1(1,j), x1(2,j), 0d0, 0d0, 0)
               end do
               if ( circle) call plafep(5, x1(1,1), x1(2,1), 0d0, 0d0,0)
               if ( fillcl ) dcolor = icolsv
            end if
         end do
      else

!     --- ndim = 1, one-dimensional elements

         do ielem = 1, nelem

!        --- store coordinates of boundary points in arrays x1

            do j = 1, inpelm
               x1(1,j) = coor(1,kmeshc(j,ielem))
            end do
            if ( .not. ncontr .or. jmark>2 ) then

!           --- ncontr = false, or jmark<3,  compute centroid

               c1 = 0.5d0 * (x1(1,inpelm)-x1(1,1))
               if ( plnumb ) &
                  call plafep ( 9, c1, 0d0, -0.06d0, 0d0, ielem+ilow-1 )
               if ( .not. ncontr ) then

!              --- ncontr = false  contract the elements

                  do j = 1, inpelm
                     x1(1,j) = (c1+4d0*x1(1,j))*.2d0
                  end do
               end if
            end if
            call plafep(4, x1(1,1), 0d0, 0d0, 0d0, 0)

!           --- plot element

            if ( inpelm>1 ) then
               do j = 2, inpelm
                  call plafep(5, x1(1,j), 0d0, 0d0, 0d0, 0)
               end do
            end if

         end do

      end if

      deallocate ( x1, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'x1' )

1000  if ( debug ) write(irefwr,*) 'End plelem'
      call erclos ( 'plelem' )

      end
