      subroutine plobstaclegrid ( rmeshx, nx, ny, iobgrid, iobst )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 09-03-2013 Correction length text
!        version  1.2  date 02-10-2012 New definition of iobgrid
!        version  1.1  date 25-04-2010 New call to plaf05
!
!   copyright (c) 2001-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Plot all cells of the grid overlapping an obstacle using a colour
!    to distinguish between inner and outside cells and boundary cells
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     obstacle
!     grid
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nx, ny, iobgrid(0:nx-1,0:ny-1), iobst
      double precision rmeshx(6,*)

!     iobgrid        i    array of length nx x ny that is filled as follows:
!                         The region covered is defined by xmin1, xmax, ymin1, &
!                         ymax
!                         The cell size is dx x dy
!                         Cell i,j refers to the coordinates:
!                         (xmin1+(i-1)dx,ymin1+(j-1)dy) x (xmin1+idx,ymin1+jdy)
!                         iobgrid(i,j) = 0: cell outside obstacle
!                         iobgrid(i,j) = k>0: boundary part k crosses cell
!                         iobgrid(i,j) = -1: cell completely in obstacle
!     iobst          i    Obstacle sequence number
!     nx             i    number of points in x direction
!     ny             i    number of points in y direction
!     rmeshx         i    Real part of kmesh part x (i = obstacle number)
!                         rmeshx(1,i) = xmin1
!                         rmeshx(2,i) = xmax
!                         rmeshx(3,i) = ymin1
!                         rmeshx(4,i) = ymax
!                         rmeshx(5,i) = dx
!                         rmeshx(6,i) = dy
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision polyg(2,4), xmin1, ymin1, dx, dy, x, y
      integer iscolr, i, j

!     dx             length of vector in x-direction
!     dy             length of vector in y-direction
!     i              Counting variable
!     iscolr         Help variable to store the value of dcolor temporarily
!     j              Counting variable
!     polyg          Array containing the co-ordinates of the point in the
!                    polygon in the following way:
!                    polyg(1,i) contains the x-coordinate of the first
!                    point
!                    polyg(2,i) contains the y-coordinate of the first
!                    point
!     x              x co-ordinate of point
!     xmin1           Minimum x-value
!     y              y co-ordinate of point
!     ymin1           Minimum y-value
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     PLAF05         Plot polygon
!     PLAFP7         Plot text using absolute co-ordinates
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Compute coordinates of rectangle
!     Compute dx, dy
!     Plot all elements with iobgrid = 0 (yellow)
!     Plot all elements with iobgrid = -1 (black)
!     Plot all elements with iobgrid>0 (green)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Compute coordinates of rectangle
!         Compute dx, dy

      xmin1 = rmeshx(1,iobst)
      ymin1 = rmeshx(3,iobst)
      dx   = rmeshx(5,iobst)
      dy   = rmeshx(6,iobst)
      iscolr = dcolor

!     --- Plot all elements with iobgrid = 0 (yellow)

      dcolor = 20
      do i = 0, nx-1
         do j = 0, ny-1
            if ( iobgrid(i,j)==0 ) then

               x = xmin1+i*dx
               y = ymin1+j*dy
               polyg(1,1) = x
               polyg(2,1) = y
               polyg(1,2) = x+dx
               polyg(2,2) = y
               polyg(1,3) = x+dx
               polyg(2,3) = y+dy
               polyg(1,4) = x
               polyg(2,4) = y+dy
               call plaf05 ( 1, dcolor, 4, polyg, 2 )

            end if
         end do
      end do

!     --- Plot all elements with iobgrid = -1 (red)

      dcolor = 11
      do i = 0, nx-1
         do j = 0, ny-1
            if ( iobgrid(i,j)==-1 ) then

               x = xmin1+i*dx
               y = ymin1+j*dy
               polyg(1,1) = x
               polyg(2,1) = y
               polyg(1,2) = x+dx
               polyg(2,2) = y
               polyg(1,3) = x+dx
               polyg(2,3) = y+dy
               polyg(1,4) = x
               polyg(2,4) = y+dy
               call plaf05 ( 1, dcolor, 4, polyg, 2 )

            end if
         end do
      end do

!     --- Plot all elements with iobgrid>0 (green)

      dcolor = 65
      do i = 0, nx-1
         do j = 0, ny-1
            if ( iobgrid(i,j)>0 ) then

               x = xmin1+i*dx
               y = ymin1+j*dy
               polyg(1,1) = x
               polyg(2,1) = y
               polyg(1,2) = x+dx
               polyg(2,2) = y
               polyg(1,3) = x+dx
               polyg(2,3) = y+dy
               polyg(1,4) = x
               polyg(2,4) = y+dy
               call plaf05 ( 2, dcolor, 4, polyg, 2 )

            end if
         end do
      end do

      dcolor = iscolr
      call plafp7 ( 15d0, 19.5d0, 'grid covering the obstacle', 0d0, &
                    0.25d0, 26 )
      dcolor = 11
      call plafp7 ( 15d0, 19d0, '-- internal', 0d0, 0.25d0, 11 )
      dcolor = 20
      call plafp7 ( 15d0, 18.5d0, '-- external', 0d0, 0.25d0, 11 )
      dcolor = 65
      call plafp7 ( 15d0, 18d0, '-- boundary', 0d0, 0.25d0, 11 )
      dcolor = iscolr

      end
