      subroutine plot3mobst ( iplots, rplots )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 29-05-2014 Extension of timebefore
!        version  1.1  date 25-11-2012 Long integers
!        version  1.0  date 18-04-2002
!
!   copyright (c) 2002-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!  subroutine plot3mobst is used to plot a three-dimensional obstacle mesh with
!  colors. In fact this subroutine creates a hidden line plot on a screen.
!  Remark:  this subroutine is only fitted for display plotting, not
!           for a pen plotter.
!           Subroutine PLAFP8 (fill colored surfaces) must have been
!           implemented for the plot package
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     color
!     mesh
!     plot
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iplots(*)
      double precision rplots(*)

!     iplots  i   Integer array in which the following quantities must be
!                 stored:
!                  1:  Position of last item that has been filled by the user
!                  2:  choice parameter for type of projection
!                      Possible values:
!                       0 : parallel projection plot
!                       1 : perspective projection plot
!                      10 : parallel projection plot (View point automatic)
!                      11 : perspective projection plot (View point automatic)
!                      Default value: 1
!                  3:  color number of farthest triangle
!                      Default value: 1
!                  4:  color number of nearest triangle
!                      Default value: 16
!                  5:  Indication if boundaries must be plotted (1) or not (0)
!                      Default value: 1
!                  6:  Indication whether x-y is basic plane (1), z-x is basic
!                      plane (2) or y-z is basic plane (3)
!                      Default value: 1
!                  7:  iskip:  Indication if element groups must be skipped (1)
!                              or not (0)
!                      Default value: 0
!                  8:  numrotations:  Number of rotations
!                      Default value: 1
!                  9:  Use specific volumes stored from position 15 (>0) or not
!                      (0)
!                 10:  Plot elements per layer (1) or not (0)
!                 11:  idefstart, defines how the start level must
!                      be computed
!                      1: Start with set of nodes with largest
!                         distance to view point
!                      2: Start with set of surfaces indicated in
!                         positions 12 and 13
!                 12:  ifirst, first surface number
!                 13:  ilast, last surface number
!                 15...14+nelgrp:  ihelp  Integer input array of length NELGRP
!                                       that is only used if ISKIP = 1
!                                       It indicates if element group ielgrp
!                                       must be skipped ihelp(ielgrp) = 0
!                                       or not ihelp(ielgrp) = 1
!     rplots  i   Double precision input array that must be filled as follows:
!                 1:    plotfm
!                 2:    yfact
!                 3-5:  eye-point
!                 6-8:  projection point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ilplot(13), npelmlc, nelgrplc, npointlc, nelemlc, idum(1), jv, &
              lenrl, iobst, neleml, iorien, i, irotation, &
              numrotations, jtimessav
      integer :: kmeshalc(1), kmeshglc(1)
      logical perspc
      double precision dum(3), xmean(3), viewpoint(3), &
                       viewp(3)
      double precision, allocatable :: coornw(:)
      double precision :: timebefore(2)

!     coornw         coordinate array for projected points
!                    (1,i) contains the x-coordinate of the projected point i
!                    (2,i) contains the y-coordinate of the projected point i
!                    (3,i) contains the distance of point i to
!                          the view-point
!     dum            General dummy double precision array
!     i              Counting variable
!     idum           General integer dummy variable
!     ilplot         Copy of iplots, if necessary extended with defaults
!     iobst          Obstacle sequence number
!     iorien         Direction of rotation:
!                     Possible values:
!                     1:  rotate in x-y plane
!                     2:  rotate in y-z plane
!                     3:  rotate in z-x plane
!     irotation      Loop variable for rotation
!     jtimessav      Help variable to save the value of jtimes
!     jv             Indicates if the view point has been filled (1) or not (0)
!     kmeshalc         kmesh part a
!     kmeshglc         kmesh part g
!     lenrl          Number of reals in one row of rmeshx
!     neleml         Number of "line"-elements in mesh
!     nelemlc        Number of elements in the mesh
!     nelgrplc       Number of element groups
!     npelmlc        Maximum number of nodes in an element
!     npointlc       Number of nodal points in the mesh
!     numrotations   Number of rotations
!     perspc         indicates if a perspective (true) or a parallel projection
!                    (false) must be applied
!     viewp          Array containing the first view point
!     viewpoint      Array containing actual view point
!     xmean          Array containing the mean values of the co-ordinates
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLMN         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERSETTIME      Get the CPU time at start of subroutine
!     PL0011         Fill copy of IPLOTS and extend with defaults
!     PL0014         Compute projection of points onto projection plane
!     PLAFEP         General SEPRAN low level plotting subroutine
!     PLFILLVIEWP    Fill array viewpoint depending on the rotation
!     PLMORE         User subroutine to plot extra information in the plot
!     PLOTSF         Make a three-dimensional hidden line plot of a 3D surface,
!                    using colored surfaces. (Display only)
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     581   The mesh is not three-dimensional
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Check input
!   Copy iplots into ilplot and fill defaults (PL0011)
!   Activate permanent arrays
!   Create surface vertex elements only
!   Perform actual procedure (PLOTSF)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plot3mobst' )
      if ( itime>0 ) call ersettime ( timebefore )
      if ( ndim/=3 ) then
         call errint ( ndim, 1 )
         call errsub ( 581, 1, 0, 0 )
      end if
      lenrl = 3*ndim

!     --- Fill local array ilplot and check rplots

      call pl0011 ( iplots, rplots, ilplot )

!     --- Define work space and activate permanent arrays

      jtimessav = jtimes

      if ( ierror/=0 ) go to 1000

!     --- Compute actual plot

      npelmlc = 3
      iobst = 1

      nelemlc  = kmeshx(11,iobst)
      npointlc = kmeshx(12,iobst)
      nelgrplc = 1

      allocate ( coornw(3*npointlc), stat = error )
      if ( error/=0 ) call eralloc ( error, npointlc, 'coornw' )
      if ( ierror/=0 ) go to 1000

      kmeshalc(1) = npelmlc
      kmeshglc(1) = nelemlc

      jv = 1 - iplots(2)/10
      perspc = mod(iplots(2),10)==1
      iorien = ilplot(6)
      numrotations = ilplot(8)

!     --- Loop over the rotations

      do irotation = 1, numrotations

         if ( numrotations==1 ) then
            jtimes = jtimessav
         else if ( irotation==numrotations .and. &
                   jtimessav>=10 ) then
            jtimes = 11
         end if
         if ( irotation==1 .and. jv==1 ) then

!        --- First rotation, set viewpoint

            do i = 1, 3
               viewpoint(i) = rplots(2+i)
            end do

         end if

         call plfillviewp ( irotation, jv, viewp, viewpoint, iorien, &
                            numrotations )

         call pl0014 ( coorobst, viewpoint, jv, coornw, &
                       npointlc, xmean, perspc, 3, dum, 1, rplots(1), &
                       rplots(2), iorien )
         if ( irotation==1 .and. jv==0 ) then

!        --- First step and view point has not yet been filled

            do i = 1, 3
               viewp(i) = viewpoint(i)
            end do

         end if

         neleml = 0

         call plotsf ( coornw, npointlc, kmeshalc, &
                       kmeshglc, nelgrplc, kmeshcobst, &
                       ilplot, dum, dum, idum(1), &
                       idum, 0, 0, neleml, idum )

         if ( mod(jtimes,10)<1 .or. mod(jtimes,10)>2 ) then
            call plmore
            call plafep ( 2, 0d0, 0d0, 0d0, 0d0, 0 )
         end if

      end do

      deallocate ( coornw, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'coornw' )

1000  if ( ierror/=0 ) then

!     --- ierror # 0  Reset ierror and nerror
!                     Return to main program

         ierror = 0
         nerror = 0
         if ( jtimes==11 ) jtimes = 1
         go to 1010

      end if

      if ( iwarn/=0 ) then

!     --- iwarn # 0  Reset iwarn and nwarn
!                    Return to main program

         iwarn = 0
         nwarn = 0

      end if

1010  call erclmn ( 'plot3mobst', timebefore )

      end subroutine plot3mobst
