      subroutine plotfuncscalar ( nfunc, funcx, funcy, ny, rinput, &
                                  inpsep, colours, iscalars )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-03-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Plot one or more time-histories of scalars in a function plot
!     The function values along the t axis are the same
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     function
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons
      use sepmoduleplot
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: nfunc, ny, inpsep(*), colours(*), &
                             iscalars(ny)
      double precision, intent(in) :: funcx(nfunc), &
                        funcy(nfunc,ny), rinput(*)

!     colours        i    Contains the set of colors to be used for each
!                         See readscplottim for a description
!     funcx          i    Set of values in x-direction
!     funcy          i    ny sets of values in y-direction
!     inpsep         i    The integer input read is stored in array INPSEP
!                         See readscplottim for a description
!     iscalars       i    Contains the sequence numbers of the scalars
!                         to be used
!     nfunc          i    Number of values per function
!     ny             i    number of functions to be plotted
!     rinput         i    Real input array for subroutine
!                         See readscplottim for a description
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ncolours, nlegenda, lentext, ncol, cols(ny), i
      double precision yfact, plotfm
      character (len=80) :: textleg(ny)

!     cols           Local array with set of color numbers
!     i              Counting variable
!     lentext        Number of characters in textleg
!     ncol           Number of colors in cols
!     ncolours       Number of colors stored in colours
!     nlegenda       Number of items in legenda
!     plotfm         Plotting factor. When PLOTFM<0, then the length of
!                    the plot is computed by - (xmax-xmin) * PLOTFM
!                    and the width by - (ymax-ymin) * YFACT * PLOTFM
!                    else the length of the plot is equal to PLOTFM cm
!                    When by some mistake the user has given | PLOTFM | a
!                    value smaller than 1 cm, then the length is chosen
!                    equal to the default length stored in common block
!                    consta
!     textleg        Contains the texts to be plotted in the legenda
!     yfact          Multiplication for the y-co-ordinates. With this
!                    parameter the user can influence the scale in
!                    y-direction
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PLAFEP         General SEPRAN low level plotting subroutine
!     PLOTAXIS       plot the axis of a function plot
!     PLOTFUNC11     Actual plot of function values
!     PLOTFUNCLEG1   Plot text legenda for function plot
!     PLOTSTSTART    Start a new plot
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
!     PRPLOTMINMAX   Compute minimum and maximum x and y values for a plot
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Compute minimum and maximum x and y-values
!     Compute scales
!     Plot scales
!     Plot function
!     Plot legenda if necessary
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plotfuncscalar' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plotfuncscalar'
         write(irefwr,1) 'nfunc, ny', nfunc, ny
         write(irefwr,1) 'kiesx, kiesy', kiesx, kiesy
         call prinrl ( funcx, nfunc, 'funcx' )
         call prinrl1 ( funcy, ny, nfunc, 'funcy' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ncolours = inpsep(3) ! number of colours given
      nlegenda = 0

      jtest = 0   ! do not clip

      plotfm = rinput(1)
      yfact  = rinput(2)

      if ( debug ) write(irefwr,2) 'plotfm, yfact', plotfm, yfact

!     --- Compute xmin, xmax, ymin, ymax

      call prplotminmax ( nfunc, funcx, funcy, ny )

!     --- Plot start

      call plotststart ( yfact, plotfm )

!     --- Plot scales

      textx = 'Time'
      if ( ny==1 ) then

!     --- 1 scalar

         lentext = len(chscalars(iscalars(1)))
         texty = chscalars(iscalars(1))(1:lentext)
         call plotaxis
         ncol = ncolours
         cols(1) = colours(1)

      else

!     --- More scalars

         texty = 'values'
         call plotaxis

         if ( ncolours==0 ) then

!        --- No colors given, use defaults

            ncol = ny
            do i = 1, ny
               cols(i) = 1+5*i
            end do  ! i = 1, ny

         else

!        --- copy colors

            ncol = ncolours
            cols(1:ncolours) = colours(1:ncolours)

         end if  ! ( ncolours==0 )

!        --- Fill text for legenda

         do i = 1, ny
            textleg(i) = chscalars(iscalars(i))
         end do  ! i = 1, ny
         nlegenda = ny

      end if  ! ( ny==1 )

!     --- Plot function

      call plotfunc11 ( nfunc, funcx, funcy, ny, ncol, cols )

!     --- Plot legenda if necessary

      if ( nlegenda>0 ) &
          call plotfuncleg1 ( nlegenda, colours, textleg )

!     --- close plot

      call plafep ( 2, 0d0, 0d0, 0d0, 0d0, 0 )

1000  call erclos ( 'plotfuncscalar' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End plotfuncscalar'

      end if  ! ( debug )

      end
