      subroutine plthid ( coor, npoint, nelem, inpelm, &
                          idrie, plotfm, yfact, &
                          v, procor, ichois, neleml, keleml, isitua )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  7.3  date 12-09-2013 Add plottime
!        version  7.2  date 17-10-2000 Layout
!        version  7.1  date 06-07-1995 Reset coor
!
!   copyright (c) 1987-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Make plot with hidden lines removed of a 3-D object.
!     It is assumed that the boundary of the object is given by either
!     triangles or rectangles. If only the 3-D elements are given the
!     user has to determine first the constituting triangles and
!     quadrilaterals.
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     hidden_line
!     plot_mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint, nelem, inpelm, &
              idrie(*), ichois, neleml, keleml(*), isitua
      double precision plotfm, yfact, coor(3,*), v(*), procor(*)

!     coor    i    array of length ndim x npoint containing the co-ordinates
!                  of the nodal points with respect to the global numbering
!     ichois  i    parameter to distinguish between the following
!                  possibilities
!                   0: perspective projection plot
!                   1: parallel projection plot
!                  10: perspective projection plot (View point automatic)
!                  11: parallel projection plot (View point automatic)
!     idrie   i    help array for triangles , length
!                  inpelm = 3 : nelem * 3
!                  inpelm = 4 : nelem * 6
!     inpelm  i    number of points per element
!     isitua  i    indicator for coordinate basis
!                    1  :  x - y - z
!                    2  :  z - x - y
!                    3  :  y - z - x
!     keleml  i    array with "line"-elements
!     nelem   i    Number of elements
!     neleml  i    Number of "line"-elements
!     npoint  i    Number of nodal points
!     plotfm  i    Plotting factor. When PLOTFM<0, then the length of
!                  the plot is computed by - (xmax-xmin) * PLOTFM
!                  and the width by - (ymax-ymin) * YFACT * PLOTFM
!                  else the length of the plot is equal to PLOTFM cm
!                  When by some mistake the user has given | PLOTFM | a value
!                  smaller than 1 cm, then the length is chosen equal to
!     procor  i    coordinate array for projected points
!                  procor(1,i) contains the x-coordinate of the projected point
!                  procor(2,i) contains the y-coordinate of the projected point
!                  procor(3,i) contains the distance of point to the view-point
!                  i is the point number
!     v       i    view point
!     yfact   i    Multiplication for the y-co-ordinates. With this
!                  parameter the user can influence the scale in y-direction
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision eps, xmean(3), xa, xb, xc, vact(3), &
                       xd, ya, yb, yc, yd, &
                       abc, acd, abd, bcd, xs, ys, he, xz, yz, view(20)
      integer i, ndrie, iline, j, i1, i2, i3, i4, &
              npunt, iget, jv, ichoic
      logical perspc
      integer, allocatable :: line(:), ihelp(:), kmeshc(:)
      double precision, allocatable :: heltri(:), help(:)

!     abc     Help variable to store computed area of projected face
!     abd     Same as abc
!     acd     Same as abc
!     bcd     Same as abc
!     eps     Local accuracy
!     he      Help variable to store intermediate computational result
!     help    help array,  length is (length array line)/2
!     heltri  help array for the sorted triangles
!     i       Counting variable
!     i1      First node number of line, triangle or quadrilateral
!     i2      Second node of line, triangle or quadrilateral
!     i3      Third node of triangle or quadrilateral
!     i4      Fourth node of quadrilateral
!     ichoic  Choice parameter for pl0014
!     iget    Help variable to classify projected quadrilateral
!     ihelp   help array of 2 * inpelm * nelem positions
!     iline   Number of lines in line array
!     j       Counting variable
!     jv      Help variable for projection : parallel or perspective
!     kmeshc  kmesh part c, contains the nodal points corresponding to the
!             elements
!     line    help array for lines , length
!             inpelm = 3 : 6 * nelem
!             inpelm = 4 : 8 * nelem
!     ndrie   Number of extra triangles from quadrilaterals
!     npunt   Total number of points in projected mesh
!     perspc  Logical to indicate projection method
!     vact    computed (corrected) view point
!     view    help array of 20 positions
!     xa      First coordinate node 1
!     xb      First coordinate node 2
!     xc      First coordinate node 3
!     xd      First coordinate node 4
!     xmean   Array containing the mean values of the co-ordinates
!     xs      x-coordinate crossing point
!     xz      Help variable to store first coordinate tilted crossing
!             point
!     ya      Second coordinate node 1
!     yb      Second coordinate node 2
!     yc      Second coordinate node 3
!     yd      Second coordinate node 4
!     ys      y-coordinate crossing point
!     yz      Help variable to store second coordinate tilted crossing
!             point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     PL0014         Compute projection of points onto projection plane
!     PLAFEP         General SEPRAN low level plotting subroutine
!     PLBC39         Reset co-ordinates array after a call of PL0014
!     PLOTTIME       Plot the actual time in a plot depending on the parameters
!                    itimdep and t in common ctimen
!     PLT350         Subroutine to plot projection with hidden lines of 3-D
!                    objects
!     PLT361         Subroutine to place end points of line in array line
!     PRSORT         Sort real array for increasing sequence
!     TOPQ01         Subroutine to cancel all double lines in array line
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Compute projection and compute axiscross for 3d plot
!     Fill lines and triangles arrays depending on basis elements
!     In case of quadrilaterals extra triangles have to be created but
!     no extra lines
!     Add extra "line"-elements to line array
!     Cancel double lines
!     Select all lines with respect to their x-coordinate from small
!     to large
!     Determine which lines are visible and make plot (PLT350)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      allocate ( line(2*inpelm*nelem+10+2*neleml), &
                 ihelp(2*inpelm*nelem+10), kmeshc(inpelm*nelem), &
                 heltri(2*nelem), help(2*inpelm*nelem+20), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, 5*inpelm*nelem+2*neleml, 'line' )
      if ( ierror/=0 ) go to 1000

      eps =  10 * epsmac

!     --- Compute projection and compute axis for plot

      perspc = mod(ichois,10)==0
      jv = 1 - ichois / 10
      ichoic = 1
      call pl0014 ( coor, v, jv, procor, npoint, xmean, perspc, 2, &
                    vact, ichoic, plotfm, yfact, isitua )

!     --- Form lines and triangles
!         (First initiate number of lines )

      iline = 0
      if ( inpelm==3 ) then

!     --- No new points

           npunt = npoint

!          --- Rearrange array idrie

           do i = 1 , nelem
               i1 = idrie( 3 * i - 2 )
               i2 = idrie( 3 * i - 1 )
               i3 = idrie( 3 * i     )
               xa = procor(2*i1-1)
               xb = procor(2*i2-1)
               xc = procor(2*i3-1)
               help(i) = min( xa,xb,xc )
               kmeshc(3*i-2) = i1
               kmeshc(3*i-1) = i2
               kmeshc(3*i  ) = i3
           end do
           call prsort( help, ihelp, nelem )
           do j = 1 , nelem
               i = ihelp( j )
               heltri(j) = help(i)
               idrie( 3*j-2 ) = kmeshc ( 3*i-2 )
               idrie( 3*j-1 ) = kmeshc ( 3*i-1 )
               idrie( 3*j   ) = kmeshc ( 3*i   )
           end do

!          --- Lines

           do i = 1 , nelem
               i1 = idrie( 3*i-2 )
               i2 = idrie( 3*i-1 )
               i3 = idrie( 3*i   )
               call plt361( i1, i2, iline, line )
               call plt361( i2, i3, iline, line )
               call plt361( i3, i1, iline, line )
           end do
           ndrie = nelem

      else if ( inpelm==4 ) then

!     --- Fill line and triangles)

           ndrie = 0

!          --- Rearrange quadrilaterals

           do i = 1 , nelem
                j = 4 * ( i - 1 )
               i1 = idrie( j + 1 )
               i2 = idrie( j + 2 )
               i3 = idrie( j + 3 )
               i4 = idrie( j + 4 )
               xa = procor( 2*i1-1 )
               xb = procor( 2*i2-1 )
               xc = procor( 2*i3-1 )
               xd = procor( 2*i4-1 )
               help(i) = min( xa,xb,xc,xd )
               kmeshc( j+1 ) = i1
               kmeshc( j+2 ) = i2
               kmeshc( j+3 ) = i3
               kmeshc( j+4 ) = i4
           end do
           call prsort( help,ihelp,nelem )
           ndrie = 0
           npunt = npoint
           do j = 1 , nelem
               i = ihelp(j)
               heltri( ndrie+1 ) = help( i )
               heltri( ndrie+2 ) = help( i )
               i1 = kmeshc( 4*i - 3 )
               i2 = kmeshc( 4*i - 2 )
               i3 = kmeshc( 4*i - 1 )
               i4 = kmeshc( 4*i     )
               xa = procor(2*i1-1)
               ya = procor(2*i1  )
               xb = procor(2*i2-1)
               yb = procor(2*i2  )
               xc = procor(2*i3-1)
               yc = procor(2*i3  )
               xd = procor(2*i4-1)
               yd = procor(2*i4  )

!              --- Compute surfaces of projection triangles

               abc = xa*(yb-yc) + xb*(yc-ya) + xc*(ya-yb)
               acd = xa*(yc-yd) + xc*(yd-ya) + xd*(ya-yc)
               abd = xa*(yb-yd) + xb*(yd-ya) + xd*(ya-yb)
               bcd = xb*(yc-yd) + xc*(yd-yb) + xd*(yb-yc)
               iget = 1111
               if ( abc>0 ) iget = iget + 1000
               if ( acd>0 ) iget = iget +  100
               if ( abd>0 ) iget = iget +   10
               if ( bcd>0 ) iget = iget +    1
               if ( iget==2222 .or. iget==2221 .or. &
                    iget==2212 .or. iget==1121 .or. &
                    iget==1112 .or. iget==1111 ) then
                    ndrie = ndrie + 1
                    idrie(3*ndrie-2) = i1
                    idrie(3*ndrie-1) = i2
                    idrie(3*ndrie  ) = i3
                    ndrie = ndrie + 1
                    idrie(3*ndrie-2) = i1
                    idrie(3*ndrie-1) = i3
                    idrie(3*ndrie  ) = i4
               else if ( iget==2122 .or. iget==1222 .or. &
                    iget==2111 .or. iget==1211 ) then
                    ndrie = ndrie + 1
                    idrie(3*ndrie-2) = i1
                    idrie(3*ndrie-1) = i2
                    idrie(3*ndrie  ) = i4
                    ndrie = ndrie + 1
                    idrie(3*ndrie-2) = i2
                    idrie(3*ndrie-1) = i3
                    idrie(3*ndrie  ) = i4
               else if ( iget==2121 .or. iget==1212 ) then
                    if ( abs(xa-xd)<eps ) then
                         xs = xa
                         ys = yb + (yc-yb)*(xs-xb)/(xc-xb)
                    else if ( abs(xb-xc)<eps ) then
                         xs = xb
                         ys = ya + (yd-ya)*(xs-xa)/(xd-xa)
                    else
                         xs =(ya*xd-yd*xa)/(xd-xa)-(yb*xc-yc*xb)/(xc-xb)
                         he =(yc-yb)/(xc-xb)-(yd-ya)/(xd-xa)
                         xs = xs / he
                         ys = ya + (yd-ya)*(xs-xa)/(xd-xa)
                    end if
                    npunt = npunt + 1
                    xz = 0.01d0 * ( xa + xb + xs ) / 3d0 + 0.99d0 * xs
                    yz = 0.01d0 * ( ya + yb + ys ) / 3d0 + 0.99d0 * ys
                    procor(2*npunt-1) = xz
                    procor(2*npunt  ) = yz
                    coor(1,npunt) = (coor(1,i1) + coor(1,i4) ) / 2d0
                    coor(2,npunt) = (coor(2,i1) + coor(2,i4) ) / 2d0
                    coor(3,npunt) = (coor(3,i1) + coor(3,i4) ) / 2d0
                    ndrie = ndrie + 1
                    idrie(3*ndrie-2) = i1
                    idrie(3*ndrie-1) = i2
                    idrie(3*ndrie  ) = npunt
                    npunt = npunt + 1
                    xz = 0.01d0 * ( xc + xd + xs ) / 3d0 + 0.99d0 * xs
                    yz = 0.01d0 * ( yc + yd + ys ) / 3d0 + 0.99d0 * ys
                    procor(2*npunt-1) = xz
                    procor(2*npunt  ) = yz
                    coor(1,npunt) = (coor(1,i1) + coor(1,i4) ) / 2d0
                    coor(2,npunt) = (coor(2,i1) + coor(2,i4) ) / 2d0
                    coor(3,npunt) = (coor(3,i1) + coor(3,i4) ) / 2d0
                    ndrie = ndrie + 1
                    idrie(3*ndrie-2) = i3
                    idrie(3*ndrie-1) = i4
                    idrie(3*ndrie  ) = npunt
               else if ( iget==2112 .or. iget==1221 ) then
                    if ( abs(xa-xb)<eps ) then
                         xs = xa
                         ys = yd + (yc-yd)*(xs-xd)/(xc-xd)
                    else if ( abs(xd-xc)<eps ) then
                         xs = xd
                         ys = ya + (yb-ya)*(xs-xa)/(xb-xa)
                    else
                         xs =(ya*xb-yb*xa)/(xb-xa)-(yd*xc-yc*xd)/(xc-xd)
                         he =(yc-yd)/(xc-xd)-(yb-ya)/(xb-xa)
                         xs = xs / he
                         ys = ya + (yb-ya)*(xs-xa)/(xb-xa)
                    end if
                    npunt = npunt + 1
                    xz = 0.01d0 * ( xa + xd + xs ) / 3d0 + 0.99d0 * xs
                    yz = 0.01d0 * ( ya + yd + ys ) / 3d0 + 0.99d0 * ys
                    procor(2*npunt-1) = xz
                    procor(2*npunt  ) = yz
                    coor(1,npunt) = (coor(1,i1) + coor(1,i2) ) / 2d0
                    coor(2,npunt) = (coor(2,i1) + coor(2,i2) ) / 2d0
                    coor(3,npunt) = (coor(3,i1) + coor(3,i2) ) / 2d0
                    ndrie = ndrie + 1
                    idrie(3*ndrie-2) = i1
                    idrie(3*ndrie-1) = i4
                    idrie(3*ndrie  ) = npunt
                    npunt = npunt + 1
                    xz = 0.01d0 * ( xc + xb + xs ) / 3d0 + 0.99d0 * xs
                    yz = 0.01d0 * ( yc + yb + ys ) / 3d0 + 0.99d0 * ys
                    procor(2*npunt-1) = xz
                    procor(2*npunt  ) = yz
                    coor(1,npunt) = coor(1,npunt-1)
                    coor(2,npunt) = coor(2,npunt-1)
                    coor(3,npunt) = coor(3,npunt-1)
                    ndrie = ndrie + 1
                    idrie(3*ndrie-2) = i2
                    idrie(3*ndrie-1) = i3
                    idrie(3*ndrie  ) = npunt
               end if

!              --- Place sides in line array

               call plt361(i1,i2,iline,line)
               call plt361(i2,i3,iline,line)
               call plt361(i3,i4,iline,line)
               call plt361(i4,i1,iline,line)
          end do

      end if

!     --- Add extra "line"-elements in line array

      do i = 1 , neleml
          i1 = keleml(2*i-1)
          i2 = keleml(2*i  )
          call plt361( i1, i2, iline, line )
      end do

!     --- Cancel double lines

      call topq01(line,iline,npunt,ihelp,help)

!     --- Select all lines with respect to their x-coordinate

      do i = 1 , iline
          i1 = line(2*i-1)
          i2 = line(2*i  )
          xa = procor( 2*i1-1 )
          xb = procor( 2*i2-1 )
          if ( xa<xb ) then
               help( i ) = xa
          else
               help( i ) = xb
               line(2*i-1) = i2
               line(2*i  ) = i1
          end if
      end do
      call prsort(help,ihelp,iline )

!     --- Compute and make plot

      call plt350(coor,procor,idrie,ndrie,heltri,line,iline,vact, &
                  view,ihelp,2,isitua)

!     --- Reset coor:

      call plbc39 ( coor, npoint, xmean )

      call plottime  ! plot time label if time dependent

      call plafep ( 2, 0d0, 0d0, 0d0, 0d0, 0 )  ! close plot

      deallocate ( line, ihelp, kmeshc, heltri, help, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'line' )

1000  end
