      subroutine pltjan ( xcint, nseg, kseg, nlines, nextra, kextra, &
                          ncomp, dy )
! ======================================================================
!
!        programmer    Jos van Kan
!        version  1.2  date 05-05-1998 Correct bug in 900 loop
!        version  1.1  date 23-11-1995 Patch up
!        version  1.0  date 20-01-1995
!
!   copyright (c) 1995-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!  pltjan analyses xcint and inserts the boundary structure in
!  nextra kextra. Specifically nextra and kextra contain infor- &
!  mation how to connect the segments in the projection. This is
!  important for correctly implementing the visibility properties
!  at the boundary.
!  The way in which successive components are connected is
!  indicated in ncomp. This is important in plotting non convex
!  regions. This information is used in the basic plotting algorithm for
!  nonconvex regions (bplgnc).
!  The specific layout is as follows:
!  for every segment in kseg nextra has two entries (one for each
!  endpoint) and kextra has four entries: two for each endpoint.
!  nextra (*, ksgcnt) contains a number N, 0<=N<=9.
!  N shows the number of extra points to be added if N is 0 or 1.
!  If N > 1, (2, 3 or 6) it shows that 2 extra points must be added.
!  kextra (j, *, ksgcnt) is filled according to the value of nextra
!  (j,ksgcnt)
!     0 : kextra is undefined
!     1 : 1 contains the index of the extra point in xcint
!         2 contains y position (-1, 0 or 1)
!     2 : 1 & 2 contain indices in xcint for 0, -1
!     3 : 1 & 2 contain indices in xcint for 1, -1
!     6 : 1 & 2 contain indices in xcint for 1, 0
!  ncomp (*, ksgcnt) shows the segment number of the lower (*=1) or
!  the higher (*=2) connected component. If more than one component
!  connects, the highest segment number is taken.
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulepltj
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nlines, kseg (*), nseg (nlines), nextra (2,*), &
              kextra (2, 2, *), ncomp (2, *)
      double precision dy, xcint (*)

!     dy      i    y difference between two successive lines
!     kextra  o    array containing information about forked endings of the
!                  segments.
!     kseg    i    number of points per segment
!     ncomp   i    arry containing lower (1, segnum) and upper (2, segnum)
!                  components of segment segnum.
!     nextra  o    array containing information about segment endings and
!                  whether segments have already been loaded into the
!                  complist(1, segnum) > 10 or have already been plotted
!                  (2, segnum) > 10.
!     nlines  i    number of lines
!     nseg    i    number of segments per line
!     xcint   i    array of interpolated x- values
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical revrse
      integer iseglo, iseghi, inxelo, inxehi, inxslo, inxshi, strslo, &
              strshi, finslo, finshi, inc, strlin, finlin, nsgcnt, &
              ksgcnt, ilines, iseg, isgcnt, ksgclo

!     finlin  ?
!     finshi  ?
!     finslo  ?
!     ilines  Counting variable
!     inc     1 = forward loading, -1 = backward loading (reverse = true)
!     inxehi  ?
!     inxelo  ?
!     inxshi  ?
!     inxslo  ?
!     iseg    counter to number of segments
!     iseghi  ?
!     iseglo  ?
!     isgcnt  ?
!     ksgclo  ?
!     ksgcnt  ?
!     nsgcnt  ?
!     revrse  Boolean to indicate whether lines are plotted in reverse
!             order
!     strlin  ?
!     strshi  ?
!     strslo  ?
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     PLTJEX  ?
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!       For all lines do
!          {while either line contain still segments do
!       REGULAR:
!          get next segment of hi & lo line
!       Free segments:
!          while (end lo < begin hi or hi exhausted) do
!             {flag new lo segment
!             get new lo segment}
!          while (end hi < begin lo or lo exhausted) do
!             {flag new hi segment
!             get new hi segment}
!          goto free segments
!       SEGBEGIN:
!          if (pr (begin lo) < pr (begin hi)) then
!             hi gets extra point at -1
!          else
!             lo gets extra point at 1
!          end if
!       connecting segments:
!          while (nextbegin lo < end hi) do
!             {lo gets extra point at 0
!             get next lo segment}
!          while (nextbegin hi < end lo) do
!             {hi get extra point at 0
!             get next hi segment}
!          goto connecting segments
!       SEGEND:
!          if (pr (endlo) < pr (end hi)) then
!             lo gets extra point at 1
!          else
!             hi gets extra point at -1
!          end if
!          end while
!       end for
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ('pltjan')

!     --- Initialize counters & Increments

      if (cth - scfi * sth.lt.0) then
         revrse = .true.
      else
         revrse = .false.
      end if
      ksgcnt = 0
      nsgcnt = 0
      do 50 ilines = 1, nlines
         nsgcnt = nsgcnt + nseg (ilines)
         do 50 iseg = nsgcnt + 1 - nseg (ilines), nsgcnt
50          ksgcnt = ksgcnt + kseg (iseg)
      do 60 isgcnt = 1, nsgcnt
         nextra (1, isgcnt) = 0
         nextra (2, isgcnt) = 0
         kextra (1, 1, isgcnt) = 0
         kextra (1, 2, isgcnt) = 0
         kextra (2, 1, isgcnt) = 0
         kextra (2, 2, isgcnt) = 0
         ncomp (1, isgcnt) = 0
         ncomp (2, isgcnt) = 0
60    continue
      if (revrse) then
         strlin = nlines
         finlin = 1
         strshi = nsgcnt
         finshi = nsgcnt + 1
         inc = - 1
      else
         ksgcnt = 1
      strlin = 1
         finlin = nlines
         strshi = 1
         finshi = 0
         inc = 1
      end if

!     --- For all lines ...

      do 900 ilines = strlin, finlin, inc
         strslo = strshi
         finslo = finshi
         strshi = finshi + inc
         finshi = finshi + inc * nseg (ilines)
         ksgclo = ksgcnt
         do 100 iseg = strslo, finslo, inc
100         ksgcnt = ksgcnt + inc * kseg (iseg)
         inxelo = ksgclo - inc
         inxehi = ksgcnt - inc
         iseglo = strslo - inc
         iseghi = strshi - inc

!        --- Initialize all indices
!            Next Segment

110      if (iseglo.eq.finslo .or. iseghi.eq.finshi) goto 800

!        --- End Regular case : 800

         iseglo = iseglo + inc
         iseghi = iseghi + inc
         inxshi = inxehi + inc
         inxehi = inxehi + inc * kseg (iseghi)
         inxslo = inxelo + inc
         inxelo = inxelo + inc * kseg (iseglo)

!        --- Setting of indices

!            ---Non Generic Case: Component birth or Component Death

         if (nseg (ilines - inc).ne.nseg (ilines)) then

!        --- First we test for unconnected segments

120      continue
            if (inc * (xcint (inxelo) - xcint (inxshi)).lt.0) then

!           --- Low segment is unconnected
!               Get next lo segment

               if (iseglo.eq.finslo) goto 800
               iseglo = iseglo + inc
               inxslo = inxelo + inc
               inxelo = inxelo + inc * kseg (iseglo)
               goto 120
            end if
            if (inc * (xcint (inxehi) - xcint (inxslo)).lt.0) then

!           --- High segment is unconnected
!               Get next hi segment

               if (iseghi.eq.finshi) goto 800
               iseghi = iseghi + inc
               inxshi = inxehi + inc
               inxehi = inxehi + inc * kseg (iseghi)
               goto 120
            end if
         end if

!        --- now for the regular case

          if ((xcint (inxslo) -xcint (inxshi)) * (cth - scfi * sth) &
             - inc * dy * (sth + scfi * cth).lt.0d0) then

!         --- HI GETS EXTRA POINT AT -1

                call pltjex (nextra, kextra, iseghi, 1, inxslo, -1)
       else

!      --- LO GETS EXTRA POINT AT 1

         call pltjex (nextra, kextra, iseglo, 1, inxshi, 1)
       end if
140      continue

!        --- Segment end
!            --- Exception handling: Component birth or component death

       if (nseg (ilines -inc).ne.nseg (ilines)) then

!      --- First the exception handling : lo and hi segments do not
!          correspond.

            if (iseglo.eq.finslo) then
               goto 145

!              --- skip to test for hi segments

            else if (inc * (xcint (inxelo + inc) - xcint (inxehi)) &
            .lt.0) then

!           --- Next lo segment also corresponds to this hi segment

!                LO GETS EXTRA POINT AT 0

               call pltjex (nextra, kextra, iseglo, 2, inxelo + inc, 0)
               ncomp (2, iseglo) = iseghi
               iseglo = iseglo + inc
               inxslo = inxelo + inc
               inxelo = inxelo + inc * kseg (iseglo)
               goto 140
            end if
145         continue
            if (iseghi.eq.finshi) then
               goto 150
            else if (inc * (xcint (inxehi + inc) - xcint (inxelo)) &
              .lt.0)  then

!           --- Next hi segment corresponds to this lo segment

!                 HI GETS EXTRA POINT AT 0

               call pltjex (nextra, kextra, iseghi, 2, inxehi +inc, 0)
               ncomp (1, iseghi) = iseglo
               iseghi = iseghi + inc
               inxshi = inxehi + inc
               inxehi = inxehi + inc * kseg (iseghi)
               goto 140
            end if
       end if

!      --- End of exception handling
!          Now comes the regular case

150   continue
         if ((xcint (inxelo) -xcint (inxehi)) * (cth - scfi * sth) &
                - inc * dy * (sth + scfi * cth).lt.0d0) then

!        --- LO GETS EXTRA POINT AT 1

            call pltjex (nextra, kextra, iseglo, 2, inxehi, 1)
         else

!        --- HI GETS EXTRA POINT AT -1

            call pltjex (nextra, kextra, iseghi, 2, inxelo, -1)
         end if
         ncomp (2, iseglo) = iseghi
         ncomp (1, iseghi) = iseglo
         goto 110
800      continue

!        --- hi or lo segment exhausted.

900   continue
      call erclos ('pltjan')
      end
