      subroutine pltr00 ( icase, kmeshc, coor, velo, x, y, nelem, t, &
                          tend, itel, user, iuser, ihulp, ielold, &
                          nsteps, maxsteps, partout, icallfn, &
                          difeps, itypeinteg )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  3.3  date 24-12-2014 Jump in case of error
!        version  3.2  date 09-06-2001 Correction for small steps
!        version  3.1  date 28-12-2000 Reduce partout
!
!   copyright (c) 1991-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     subroutine to compute the track of a particle in a velocity field.
!     if needed the porosity for each element group is determined utilizing
!     arrays user, iuser, ihulp and ielold.
! **********************************************************************
!
!                       KEYWORDS
!
!     particle_tracking
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision x, y, coor(2,*), velo(2,*), t, tend, user(*), &
                       partout(3,*), difeps
      integer nelem, kmeshc(3,*), itel(*), nsteps, maxsteps, icallfn, &
              icase, iuser(*), ihulp(*), ielold(*), itypeinteg

!     coor       i    coordinate array
!     difeps     i    reference accuracy for errorterm
!     icallfn    i    Indicates if for each position in the particle trace
!                     a user subroutine FNPARTIC must be called (1) or not (0)
!     icase      i    indicator whether
!                     1 : velo per element, no porosity
!                     2 : velo per element, porosity
!                     3 : velo per nodal point, no porosity
!                     4 : velo per nodal point, porosity
!     ielold     i    helparray in which is stored from which
!                     original element each element is taken
!     ihulp      i    helparray in which is stored in which element group
!                     each triangle is situated
!     itel            helparray to count how many times an element
!                     appears in the trace of one particle
!     itypeinteg i    Defines the type of integration aplied
!                     Possible values:
!                     1:  Heun (explicit)
!                     2:  Runge-Kutta 4 (explicit)
!                     3:  Trapezoid rule (implicit)
!     iuser      i    helparray in which integer variables are stored
!     kmeshc     i    element array (triangles)
!     maxsteps   i    Maximum number of positions that may be stored in partout
!     nelem      i    number of elements
!     nsteps    i/o   Contains the number of positions stored in partout
!     partout   i/o   Array containing the actual information concerning the
!                     particle trajectories
!                     partout(1,i) contains t_i
!                     partout(2-3,i) contains x_i and y_i
!     t         i/o   at input starting time, at output final time
!     tend       i    endtime of integration
!     user       i    helparray to store real values
!     velo       i    velocity array
!     x          i    x-coordinate of startlocation
!     y          i    y-coordinate of startlocation
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ielem, i1, i2, i3, iresul, io1, io2, io3, ital, igroep, &
              ilne, istart, istag, ielemo, ielemv, ivel, ivelol
      double precision dlen, dx, dy, dref, xl(3), yl(3), u, v, &
                       ul(3), vl(3), up, vp, upl(3), vpl(3), &
                       ski, eta, det, porost, uo, vo
      logical repeat, herh

!     det          determinant value
!     dlen         length of track in element
!     dref         reference distance for plotting
!     dx           distance
!     dy           distance
!     eta          local coordinate
!     herh         while variable for iteration
!     i            loop variable
!     i1           node number
!     i2           node number
!     i3           node number
!     ielem        element number
!     ielemo       old element number ( in sequence -1 )
!     ielemv       old element number ( in sequence -2 )
!     igroep       groupe number of triangle considered
!     ilne         line indicator
!     io1          node number
!     io2          node number
!     io3          node number
!     iresul       indicator for point belonging to element
!     istag        indicator for stagnation point
!     istart       helpvariable for porositydetermination
!     ital         local counter
!     ivel         number of element for velocity
!     ivelol       number of old element for velocity
!     porost       porosity
!     repeat       repeat variable
!     ski          local coordinate
!     u            velocity component
!     ul           array with velocity components in the three nodes
!     uo           old velocity component
!     up           velocity component
!     upl          array with velocity components in the three nodes
!     v            velocity component
!     vl           array with velocity components in the three nodes
!     vo           old velocity component
!     vp           velocity component
!     vpl          velocity components
!     xl           local x-coordinate array
!     yl           local y-coordinate array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    deconcatenate subroutine name from sequence of names
!     EROPEN    concatenate subroutine name to sequence of names
!     ERRINT    Put integer in error message
!     ERRSUB    give error message
!     ERRWAR    give warning
!     FNPARTIC  User function subroutine that is called in eah step of the
!               particle tracer
!     PLTR01    determine in which element particle is situated
!     PLTR02    compute track for case 2 in triangle
!     PLTR03    compute track in triangle for case 3
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     937   give message that porosity is zero
!    1638   Particle array too small
!    2224   Porosity not constant in element has not yet been implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Determine elementnumber of triangle
!     While not outside region of interest
!        Compute track in triangle, determine next triangle
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen( 'pltr00' )

!     --- Determine reference length for plotting and printing

      dx = xmax - xmin
      dy = ymax - ymin
      dref = sqrt( dx * dx + dy * dy ) / 2d2

!     --- Set counterarray to zero

      do i = 1, nelem
         itel( i ) = 0
      end do
      ielem  = 0
      ielemo = 0
      i1     = 0
      i2     = 0
      i3     = 0
      repeat = .true.
100   if ( repeat ) then

!     --- determine elementnumber of triangle

         ielemv = ielemo
         ielemo = ielem
         io1    = i1
         io2    = i2
         io3    = i3
         ielem = 0
         do i = 1 , nelem
            i1 = kmeshc( 1, i )
            i2 = kmeshc( 2, i )
            i3 = kmeshc( 3, i )

!           --- Check whether element is neighbour element

            ital  = 0
            if ( i1==io1 .or. i1==io2 .or. i1==io3 ) ital=ital+1
            if ( i2==io1 .or. i2==io2 .or. i2==io3 ) ital=ital+1
            if ( i3==io1 .or. i3==io2 .or. i3==io3 ) ital=ital+1
            if ( ital/=2 ) goto 150
            xl(1) = coor(1,i1)
            xl(2) = coor(1,i2)
            xl(3) = coor(1,i3)
            yl(1) = coor(2,i1)
            yl(2) = coor(2,i2)
            yl(3) = coor(2,i3)
            iresul = 0
            call pltr01( x, y, iresul, xl, yl, ski, eta, det )
            if ( iresul==1 ) then
               ielem  = i

!              --- Element found, jump to next section

               goto 300
            end if
150      end do

!        --- Element not yet found, &
!            Second action, run through all elements

         ielem = 0
         do i = 1 , nelem
            i1 = kmeshc( 1, i )
            i2 = kmeshc( 2, i )
            i3 = kmeshc( 3, i )
            xl(1) = coor(1,i1)
            xl(2) = coor(1,i2)
            xl(3) = coor(1,i3)
            yl(1) = coor(2,i1)
            yl(2) = coor(2,i2)
            yl(3) = coor(2,i3)
            iresul=0
            call pltr01( x, y, iresul, xl, yl, ski, eta, det )
            if ( iresul==1 ) then
               ielem   = i

!              --- Element found, jump to next section

               goto 300
            end if
         end do

!        --- No element found, point outside, track ready

         repeat = .false.
         goto 100

!        --- Set line indicator to zero

300      ilne = 0

!        --- Check whether particle has been in element before for case
!            that velocities are constant over one element, in order to
!            circumvent the situation that a particle will travel zig-zag
!            along the common boundary

         if ( ielem==ielemv .and. icase<3 ) then
            ilne = 0

!           --- Element has been visited before, check whether there is a
!               common line between the last two elements

            if ( i1==io1 .or. i1==io2 .or. i1==io3 ) then
                 ilne = ilne + 1
            end if
            if ( i2==io1 .or. i2==io2 .or. i2==io3 ) then
                 ilne = ilne + 2
            end if
            if ( i3==io1 .or. i3==io2 .or. i3==io3 ) then
                 ilne = ilne + 4
            end if
         end if

!        --- Check whether particle is trapped in element

         itel( ielem ) = itel( ielem ) + 1

!        --- Element found, determine elementgroup for this triangle

         igroep = 0
         herh = .true.
400      if ( herh ) then

!        --- Find element group number

             if ( ielem>ihulp(igroep+1) ) then
                  igroep = igroep + 1
             else
                  herh   = .false.
             end if
             goto 400
         end if
         if ( icase==2 .or. icase==4 ) then

!        --- Determine porosity for this element

            istart = iuser( 5+igroep )
            if ( iuser(istart)==0 ) then

!           --- Error, zero porosity

               call errsub ( 937, 0, 0, 0 )
               go to 1000

            else if ( iuser(istart)<-5 ) then

!           --- Porosity is constant

               porost = user(-iuser(istart))
            else
               call errwar ( 2224, 0, 0, 0 )
               goto 1000
            end if

         else

!        --- No porosity problem

            porost = 1d0

         end if

!        --- Determine global velocities

         if ( icase==1 .or. icase==2 ) then

!        --- Velocity has been given per element

            ivel = ielold(ielem)
            u  = velo( 1, ivel ) / porost
            v  = velo( 2, ivel ) / porost

         else if ( icase==3 .or. icase==4 ) then

!        --- Velocity has been given per point

            ul(1) = velo( 1, i1 ) / porost
            ul(2) = velo( 1, i2 ) / porost
            ul(3) = velo( 1, i3 ) / porost
            vl(1) = velo( 2, i1 ) / porost
            vl(2) = velo( 2, i2 ) / porost
            vl(3) = velo( 2, i3 ) / porost

         end if

!        --- Determine track

         if ( icase==1 .or. icase==2 ) then

!        --- Transform to local velocities

            up  =   ( (yl(3)-yl(1))*u - (xl(3)-xl(1))*v ) / det
            vp  = - ( (yl(2)-yl(1))*u - (xl(2)-xl(1))*v ) / det
            if ( ilne/=0 ) then

!           --- Particle should travel along boundary

               ivelol = ielold( ielemo )
               u = velo( 1,ivelol ) / porost
               v = velo( 2,ivelol ) / porost
               uo  =   ( (yl(3)-yl(1))*u - (xl(3)-xl(1))*v ) / det
               vo  = - ( (yl(2)-yl(1))*u - (xl(2)-xl(1))*v ) / det

!              --- Check whether particle should travel along boundary

               if ( ilne==3 ) then

!              --- Along ski-boundary

                  up = ( up + uo ) / 2d0
                  vp = 0d0
               else if ( ilne==5 ) then

!              --- Along eta-boundary

                  up = 0d0
                  vp = ( vp + vo ) / 2d0
               else if ( ilne==6 ) then

!              --- Along ski,eta boundary

                  uo  =  ( up + uo ) / 2d0
                  vo  =  ( vp + vo ) / 2d0
                  up  =  ( uo - vo ) / 2d0
                  vp  =  ( vo - uo ) / 2d0
               end if
            end if

!           --- determine track of particle in this element

            istag = 0
            call pltr03 ( up, vp, ski, eta, t, tend, x, y, xl, yl, dref, &
                          nsteps, partout, maxsteps, icallfn, repeat )
            if ( ierror/=0 ) go to 1000

         else if ( icase==3 .or. icase==4 ) then

!        --- Compute local velocities

            upl(1) =   ((yl(3)-yl(1))*ul(1) - (xl(3)-xl(1))*vl(1)) / det
            upl(2) =   ((yl(3)-yl(1))*ul(2) - (xl(3)-xl(1))*vl(2)) / det
            upl(3) =   ((yl(3)-yl(1))*ul(3) - (xl(3)-xl(1))*vl(3)) / det
            vpl(1) = - ((yl(2)-yl(1))*ul(1) - (xl(2)-xl(1))*vl(1)) / det
            vpl(2) = - ((yl(2)-yl(1))*ul(2) - (xl(2)-xl(1))*vl(2)) / det
            vpl(3) = - ((yl(2)-yl(1))*ul(3) - (xl(2)-xl(1))*vl(3)) / det

!           --- determine track of particle in this element

            istag = 0
            call pltr02 ( upl, vpl, ski, eta, t, tend, x, y, &
                          xl, yl, dref, dlen, istag, nsteps, partout, &
                          maxsteps, itypeinteg, difeps, icallfn, &
                          repeat )

         end if
         if ( istag==0 ) then

!        --- No stagnation point

            if ( partout(1,nsteps)<t-1d-3*tend ) then

               nsteps = nsteps + 1
               if ( nsteps>maxsteps ) then
                  call errint ( nsteps, 1 )
                  call errsub ( 1638, 1, 0, 0 )
                  repeat = .false.
                  go to 100
               end if
               partout(1,nsteps) = t
               partout(2,nsteps) = x
               partout(3,nsteps) = y

            end if

            if ( icallfn>0 ) &
                 call fnpartic ( t, x, y, 0d0, .false., repeat )
         end if
         if ( t>=tend .or. istag==1 ) repeat = .false.
         goto 100
      end if

!     --- Return to pltrac

1000  call erclos( 'pltr00' )
      end
