      subroutine pltr03 ( up, vp, ski, eta, t, tend, x, y, xl, yl, &
                          dref, nsteps, partout, maxsteps, &
                          icallfn, repeat )
! ======================================================================
!
!         programmer   Niek Praagman
!         version 3.0  date  31-03-1996 Extra parameters nsteps and partout
!         version 2.0  date  22-01-1993 redesign
!         version 1.2  date  18-12-1991 correction for time writing added
!         version 1.1  date  10-07-1991 small accuracy error removed in
!                                       determination crossing points
!         version 1.0  date  08-03-1991
!
!
!   copyright (c) 1991-1996  "ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard or soft copy granted only by written license obtained
!   from "ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system (e.g. , in memory, disk or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
!
!                       DESCRIPTION
!
!     subroutine for particle-tracking in the unit-triangle while the
!     velocity is constant over the triangle
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     particle_tracking
!     unit_triangle
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      double precision up, vp, ski, eta, t, tend, x, y, dref, &
                       xl(3), yl(3), partout(3,*)
      integer nsteps, maxsteps, icallfn
      logical repeat

!     dref       i    reference distance
!     eta       i/o   first coordinate of startlocation at input
!                     of end location at output
!     icallfn    i    Indicates if for each position in the particle trace
!                     a user subroutine FNPARTIC must be called (1) or not (0)
!     maxsteps   i    Maximum number of positions that may be stored in partout
!     nsteps    i/o   Contains the number of positions stored in partout
!     partout   i/o   Array containing the actual information concerning the
!                     particle trajectories
!                     partout(1,i) contains t_i
!                     partout(2-3,i) contains x_i and y_i
!     repeat    i/o   If true the process has not yet been finished
!     ski       i/o   second coordinate of startlocation at input
!                     of end location at output\
!     t         i/o   actual time
!     tend       i    time at which the integration should terminate
!     up         i    u-velocity in the triangle
!     vp         i    v-velocity in the triangle
!     x         i/o   global x-coordinate of particle
!     xl         i    global x-coordinates of corners of triangle
!     y         i/o   global y-coordinate of particle
!     yl         i    global y-coordinates of corners of triangle
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision afst, dt, dx, dy, eps, etaold, skiold, xold, &
                       yold, t0, t1, t2, t3
      integer ja

!     afst        local distance
!     dt          time step
!     dx          local x-distance
!     dy          local y-distance
!     eps         accuracy parameter
!     etaold      old local eta coordinate
!     ja          local indicator
!     skiold      old local ski coordinate
!     t0          time of crossing boundary
!     t1          time of crossing line ski = 0
!     t2          time of crossing line eta = 0
!     t3          time of crossing line ski+eta = 1
!     xold        x-coordinate old position
!     yold        y-coordinate old position
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
!     FNPARTIC  User function subroutine that is called in eah step of the
!               particle tracer
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1638   Particle array too small
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'pltr03' )
      eps = 1d-4
      xold = x
      yold = y

100   continue

!     --- determine stepsize

      dt = 2d0 / sqrt( up * up + vp * vp )

      if ( t+dt.gt.tend ) dt = (tend - t) * (1d0 + eps)

!     --- set old values

      skiold = ski
      etaold = eta

      ski    = ski + dt * up
      eta    = eta + dt * vp

!     --- determine global position

      x = xl(1) + (xl(2)-xl(1))*ski + (xl(3)-xl(1))*eta
      y = yl(1) + (yl(2)-yl(1))*ski + (yl(3)-yl(1))*eta

      dx = x - xold
      dy = y - yold

      afst = sqrt( dx * dx + dy * dy )

      ja = 0

      if ( afst.gt.dref ) then

           xold = x
           yold = y

           ja = 1

      end if

      if ( ( -eps.lt.ski .and. 1d0+eps.gt.ski )  .and. &
           ( -eps.lt.eta .and. 1d0+eps.gt.eta )  .and. &
           ( -eps.lt.1d0-ski-eta ) )  then

!     --- Adjust time

         t = t + dt

         if ( ja.eq.1 ) then

!        --- New point

            nsteps = nsteps + 1
            if ( nsteps.gt.maxsteps ) then
               call errint ( nsteps, 1 )
               call errsub ( 1638, 1, 0, 0 )
               repeat = .false.
               go to 1000
            end if
            partout(1,nsteps) = t
            partout(2,nsteps) = x
            partout(3,nsteps) = y
            if ( icallfn.gt.0 ) &
                 call fnpartic ( t, x, y, 0d0, .false., repeat )
            if ( .not.repeat ) go to 1000

         end if
         if ( t.ge.tend ) goto 1000

         goto 100

      end if

!     --- Outside triangle, interpolate to obtain crossing point

      if ( ski.gt.0 ) then

         t1 = 2d0

      else

         if ( abs( skiold - ski ).lt.eps ) then
            t1 = 2d0
         else
            t1 = skiold / ( skiold - ski )
         end if

      end if

      if ( eta.gt.0 ) then

         t2 = 2d0

      else

         if ( abs( etaold - eta ).lt.eps ) then
            t2 = 2d0
         else
            t2 = etaold / ( etaold - eta )
         end if

      end if

      if ( ski+eta.lt.1d0 ) then

         t3 = 2d0

      else

         if ( abs( ski+eta-skiold-etaold ).lt.eps ) then
            t3 = 2d0
         else
            t3 = ( 1d0-skiold-etaold ) / ( ski+eta-skiold-etaold )
         end if

      end if

      t0 = min( t1, t2, t3 )
      t0 = t0 + 1d-4

      ski = skiold + t0 * ( ski - skiold )
      eta = etaold + t0 * ( eta - etaold )

!     --- Adjust time in accordance with the interpolation

      t = t + t0 * dt

!     --- Determine global position

      x = xl(1) + (xl(2)-xl(1))*ski + (xl(3)-xl(1))*eta
      y = yl(1) + (yl(2)-yl(1))*ski + (yl(3)-yl(1))*eta

      nsteps = nsteps + 1
      if ( nsteps.gt.maxsteps ) then
         call errint ( nsteps, 1 )
         call errsub ( 1638, 1, 0, 0 )
         repeat = .false.
         go to 1000
      end if
      partout(1,nsteps) = t
      partout(2,nsteps) = x
      partout(3,nsteps) = y
      if ( icallfn.gt.0 ) &
           call fnpartic ( t, x, y, 0d0, .false., repeat )

1000  call erclos ( 'pltr03' )

      end
