      subroutine pltr16 ( plotfm, yfact, nview )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  1.3  date 25-04-2010 New call to pltr15
!        version  1.2  date 06-04-1996 New call to pltr15
!        version  1.1  date 11-02-1993 More possibilities for nview
!
!   copyright (c) 1991-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Help subroutine to compute framecube for 3-D plotting
!     Compute xmin, xmax, ymin, ymax, zmin, zmax
!     Start plot and plot cube of extreme values dependent on nview
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     parallel_plot
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nview
      double precision plotfm, yfact

!     nview     i   Indicator for direction of eye-point
!     plotfm    i   Plotting factor. When PLOTFM<0, then the length of
!                   the plot is computed by - (xmax-xmin) * PLOTFM
!                   and the width by - (ymax-ymin) * YFACT * PLOTFM
!                   else the length of the plot is equal to PLOTFM cm
!                   When by some mistake the user has given | PLOTFM | a value
!                   smaller than 1 cm, then the length is chosen equal to
!                   the default length stored in common block consta
!     yfact     i   Multiplication factor for the y-co-ordinates. With this
!                   parameter the user can influence the scale in y-direction
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ip, istart, iend
      double precision disx, disy, xminl, xmaxl, xminh, xmaxh, &
                       yminh, ymaxh, yminl, ymaxl, &
                       zminh, zmaxh, zminl, zmaxl, coorpl(3,8)

!     coorpl     Array containing the co-ordinates of the curve to be plotted
!     disx       local distance x-dirextion
!     disy       local distance y-direction
!     i          loop variable
!     iend       helpvariable to indicate where nodenumbers stop
!     ip         nodal point number
!     istart     helpvariable to indicate where nodenumbers start
!     xmaxh      local help variable
!     xmaxl      local variable to store maximal x-value
!     xminh      local helpvariable
!     xminl      local variable to store minimal x-value
!     ymaxh      local help variable
!     ymaxl      local variable to store maximal y-value
!     yminh      local help variable
!     yminl      local variable to store minimal y-value
!     zmaxh      local help variable
!     zmaxl      local variable to store maximal z-value
!     zminh      local help variable
!     zminl      local variable to store minimal z-value
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS   Deconcatenate name from string of calling subroutines
!     EROPEN   Concatenate name to string of calling subroutines
!     PL0000   Compute length and width of plot
!     PLAFEP   General low level SEPRAN plot subroutine
!     PLTR15   Subroutine to perform a PLAFEP action in 3D while the
!              eye direction is taken into account
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen( 'pltr16' )

!     --- Compute extreme values of the mesh considered

      istart = kmeshm(2)
      iend   = kmeshm(3) - 1
      xminh =  rinfin
      xmaxh = -rinfin
      yminh =  rinfin
      ymaxh = -rinfin
      zminh =  rinfin
      zmaxh = -rinfin
      do 100 i = istart , iend
          ip    =  kmeshm( i )
          xminh  =  min ( xminh,  coor(1,ip) )
          yminh  =  min ( yminh,  coor(2,ip) )
          zminh  =  min ( zminh,  coor(3,ip) )
          xmaxh  =  max ( xmaxh,  coor(1,ip) )
          ymaxh  =  max ( ymaxh,  coor(2,ip) )
          zmaxh  =  max ( zmaxh,  coor(3,ip) )
100   continue

!     --- Set the local values

      xminl = xminh
      yminl = yminh
      zminl = zminh
      xmaxl = xmaxh
      ymaxl = ymaxh
      zmaxl = zmaxh

!     --- Compute extreme values of projection, this is dependent on nview, &

      if ( nview==1 ) then
           xmax =  xmaxh
           xmin =  xminh
           ymax =  ymaxh
           ymin =  yminh
           zmax =  zmaxh
           zmin =  zminh
      else if ( nview==2 ) then
           xmax =  ymaxh
           xmin =  yminh
           ymax = -xminh
           ymin = -xmaxh
           zmax =  zmaxh
           zmin =  zminh
      else if ( nview==3 ) then
           xmax = -xminh
           xmin = -xmaxh
           ymax = -yminh
           ymin = -ymaxh
           zmax =  zmaxh
           zmin =  zminh
      else if ( nview==4 ) then
           xmax = -yminh
           xmin = -ymaxh
           ymax =  xmaxh
           ymin =  xminh
           zmax =  zmaxh
           zmin =  zminh
      end if
      xmin = xmin + 0.77d0 * ymin
      xmax = xmax + 0.77d0 * ymax
      ymin = zmin + 0.45d0 * ymin
      ymax = zmax + 0.45d0 * ymax
      disx  = ( xmax - xmin ) / 10d0
      disy  = ( ymax - ymin ) / 10d0
      xmin  = xmin - disx
      xmax  = xmax + disx
      ymin  = ymin - disy
      ymax  = ymax + disy

!     --- Compute aleng and width

      call pl0000 ( plotfm, yfact )

!     --- start picture

      call plafep( 1, aleng, width, 0d0, 0d0, 0 )

!     --- Plot the enveloping cube

      coorpl(1,1) = xminl
      coorpl(2,1) = yminl
      coorpl(3,1) = zminl
      coorpl(1,2) = xmaxl
      coorpl(2,2) = yminl
      coorpl(3,2) = zminl
      coorpl(1,3) = xmaxl
      coorpl(2,3) = ymaxl
      coorpl(3,3) = zminl
      coorpl(1,4) = xminl
      coorpl(2,4) = ymaxl
      coorpl(3,4) = zminl
      coorpl(1,5) = xminl
      coorpl(2,5) = yminl
      coorpl(3,5) = zminl
      coorpl(1,6) = xminl
      coorpl(2,6) = yminl
      coorpl(3,6) = zmaxl
      coorpl(1,7) = xmaxl
      coorpl(2,7) = yminl
      coorpl(3,7) = zmaxl
      coorpl(1,8) = xmaxl
      coorpl(2,8) = yminl
      coorpl(3,8) = zminl
      call pltr15 ( 8, coorpl, nview, 3 )
      coorpl(1,1) = xmaxl
      coorpl(2,1) = yminl
      coorpl(3,1) = zmaxl
      coorpl(1,2) = xmaxl
      coorpl(2,2) = ymaxl
      coorpl(3,2) = zmaxl
      coorpl(1,3) = xmaxl
      coorpl(2,3) = ymaxl
      coorpl(3,3) = zminl
      call pltr15 ( 3, coorpl, nview, 3 )
      coorpl(1,1) = xmaxl
      coorpl(2,1) = ymaxl
      coorpl(3,1) = zmaxl
      coorpl(1,2) = xminl
      coorpl(2,2) = ymaxl
      coorpl(3,2) = zmaxl
      coorpl(1,3) = xminl
      coorpl(2,3) = ymaxl
      coorpl(3,3) = zminl
      call pltr15 ( 3, coorpl, nview, 3 )
      coorpl(1,1) = xminl
      coorpl(2,1) = ymaxl
      coorpl(3,1) = zmaxl
      coorpl(1,2) = xminl
      coorpl(2,2) = yminl
      coorpl(3,2) = zmaxl
      call pltr15 ( 2, coorpl, nview, 3 )
      call erclos( 'pltr16' )
      end
