      subroutine pltrac ( iinpar, rinpar, startp, iuser, user )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  4.9  date 24-12-2014 Jump in case of error
!        version  4.8  date 29-04-2013 Initialize iporig
!        version  4.7  date 25-11-2012 Long integers
!
!   copyright (c) 1992-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subroutine to compute, print and or plot the trace of a number
!     of particles in a velocity field
!     The mesh is given by KMESH and the velocity field is in IVSOL.
! **********************************************************************
!
!                       KEYWORDS
!
!     particle_tracking
!     plot
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmodulework
      use sepmodulemain
      use sepmodulekprob
      use sepmodulekmesh
      use sepmoduleplot
      use sepmoduledebug
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinpar(*), iuser(*)
      double precision rinpar(*), user(*), startp(ndim,*)

!     iinpar     i    Integer array of length 9 to be filled by the user
!                     with information concerning the output
!                     The array must be filled as follows
!                      1:  Highest entry number of the array that has been
!                          filled by the user
!                      2:  NPART, number of particles to be traced
!                          Default :  1
!                      3:  Parameter indicating whether the trajectories
!                          must be printed or not. Values
!                           -1 : Trajectories are not printed
!                            0 : Trajectories are printed to the standard
!                                output file (IREFWR)
!                           >0 : Trajectories are printed to the file with
!                                reference number IINPAR(3)
!                                This file must have been opened before
!                          Default : -1
!                      4:  Indicates how the plots must be made. Values
!                            0 : No plots are made
!                            1 : The particle trajectories are plotted
!                            2 : Both mesh and trajectories are plotted
!                                (2D only)
!                          Default :  1
!                      5:  Indicates the type of parallel projection (3D only)
!                          Values
!                            1 : Projection on plane y=constant seen from y=  oo
!                            2 : Projection on plane x=constant seen from x=  oo
!                            3 : Projection on plane y=constant seen from y= -oo
!                            4 : Projection on plane x=constant seen from x= -oo
!                          Default value : 1
!                      6:  Indicates whether the vector to be plotted is a
!                          standard velocity vector or that fluxes are given.
!                          Values
!                            0 : The vector is a standard velocity vector
!                                defined per nodal point or element.
!                                No porosity is given.
!                            1 : The vector is a flow vector corresponding to a
!                                porous medium. In this case also the porosity
!                                is required and the arrays IUSER and USER must
!                                be used.
!                            2 : See 0, however now fluxes through the edges
!                                (2D) or faces (3D) are given in isol. This is
!                                for example the case if element type 650 is
!                                used.
!                            3 : See 1, fluxes in a porous medium
!                          Default value : 0
!                      7:  Indicates whether markers must be plotted at the
!                          trajectories at fixed time intervals. Values
!                            0 : No markers are plotted
!                            1 : Markers are plotted, tstep_mark is used
!                                No other values are yet available
!                          Default value : 0
!                      8:  Indicates if the output must be stored in array
!                          or not
!                          Possible values
!                          0: ioutput is not used
!                          1: ioutput is used, all computed values are stored
!                          2: ioutput is used, the time step tstep_out is used
!                          Default value : 0
!                      9:  Indicates if for each position in the particle trace
!                          a user subroutine FNPARTIC must be called (1) or
!                          not (0)
!                          With this subroutine the user may manipulate the
!                          particle process a little bit.
!                          Default value : 0
!                     10:  Indicates if tstep_print is used for the printing
!                          of the particles (1) or not (0)
!                     11:  Defines type of time integration
!                          Possible values:
!                          1: Heun
!                          2: Runge Kutta (4)
!                          3: Trapezoid rule
!                          Default value : 1
!                     12:  Indicates if EPS_TIME_STEP is given to define the
!                          accuracy for the determination of the time step (1)
!                          or not (0)
!                          Default value : 0
!     iuser      i    Help array to fill porosity values. See also user
!     rinpar     i    Double precision array of length 5 to be filled by the
!                     user with the following information
!                       1  :  plotfm
!                       2  :  yfact
!                       3  :  tbegin  Starting time for tracing particles
!                       4  :  tend    End time for tracing particles
!                       5  :  tstep_mark  Time interval for placing markers.
!                                     Only used if iinpar(7)>0
!                       6  :  tstep_print  Time interval for printing.
!                                     Only used if iinpar(10)>0
!                       7  :  tstep_out  Time interval for output.
!                                     Only used if iinpar(8)>1
!                       8  :  eps_time_step: accuracy to define time step
!                                     Only used if iinpar(12) = 1
!     startp     i    Two dimensional double precision array of size ndim*npart,
!                     where ndim is the dimension of space and npart is given in
!                     iinpar(2). Startp must be filled by the user with the
!                     coordinates of the starting points of the particles,
!                     in the following way
!                         xi = startp( 1,i )
!                         yi = startp( 2,i )
!                       [ zi = startp( 3,i ) ]
!     user       i    Array to be filled with porosities for each element group
!                     if iinpar(6) = 1 or 3. For each element group one value is
!                     needed. (Filling may be performed with the help of
!                     routines FILCOF and FIL100)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision plotfm, t, tbegin, tend, xt, yfact, yt, zt, &
                       tstepmark, tstepprint, tstepout, difeps, curves(1)
      integer id, idoubl, iinner, ichois(5), ikader, ikelmc, jpos, &
              iveck, itemp, ilow, imark, indexe, i, &
              iprint, iplot, ikeuze, icurvs(1), &
              ikeus, nboun, npart, nview, ivul, iskip, istoutp, &
              iskpar(1), iresul(5,3), jchois, number(4), &
              special, maxsteps, nsteps, icallfn, jmarksv, &
              istepprint, lenout, ilevel, iaccuracy, itypeinteg, kpos, &
              nelgrpsav, nelemsav, istart
      logical compl, ncontr, circle, contin
      double precision, allocatable :: part(:), partout(:), temp(:)
      integer, pointer :: iorig(:), itel(:)

!     circle         logical to indicate whether boundary of element is closed
!     compl          indicator whether variable is complex or not
!     contin         If true the process must be continued
!     curves         array containing the coordinates of the curves
!     difeps         Defines the accuracy to be used to define the time step
!     i              loop variable
!     iaccuracy      If 0 difeps is set to the default value otherwise it is
!                    extracted from rinpar(8)
!     icallfn        Indicates if for each position in the particle trace
!                    a user subroutine FNPARTIC must be called (1) or not (0)
!     ichois         choice array
!     icurvs         array containing the number of points in the curves
!                    accumulated
!     id             loop variable
!     idoubl         indicator whether lines have to be plotted double
!     iinner         indicator for plotting inner boundaries
!     ikader         help variable to store jkader temporarily
!     ikelmc         local parameter for kmesh part c
!     ikeus          help parameter for several temporary actions
!     ikeuze         indicator how many positions have been filled by the user
!     ilevel         dummy
!     ilow           counter for elements
!     imark          local plot parameter
!     indexe         infor array pointer
!     iorig          help array with the original element numbers
!     iplot          plotting indicator
!     iprint         print indicator
!     iresul         result array of topqu0
!     iskip          variable to indicate whether element groups should be
!                    skipped
!     iskpar         help array for iskip, length nelgrp
!     istart         start position
!     istepprint     Indicates if tstep_print is used for the printing
!                    (1) or not (0)
!     istoutp        If 1 the output must be stored in particinf and partic
!     itel           help array to count how many times an element
!                    appears in the trace of one particle
!     itemp          problem type number (standard element)
!     itypeinteg     Defines the type of integration aplied
!                    Possible values:
!                    1:  Heun (explicit)
!                    2:  Runge-Kutta 4 (explicit)
!                    3:  Trapezoid rule (implicit)
!     iveck          indicator whether velocity is on elements (1) or nodes (2)
!     ivul           indicator whether array with helpnumbers porosity has to
!                    be filled (1) or not (0)
!     jchois         choice parameter
!     jmarksv        Help variable to store the value of jmark
!     jpos           Position in array
!     kpos           Position in array
!     lenout         Length of output array
!     maxsteps       maximum number of positions in a particle trace
!     nboun          help variable for kelmb
!     ncontr         logical helpvariable for routine PLELEM
!     nelemsav       Help parameter to save the old value of nelem
!     nelgrpsav      Help parameter to save the old value of nelgrp
!     npart          number of particles to be tracked
!     nsteps         number of points in a particle trajectory
!     number         counter
!     nview          indicator for direction of viewpoint in 3-D case
!                    1 from x =  inf (default)
!                    2 from y =  inf
!                    3 from x = -inf
!                    4 from y = -inf
!     part          Array containing the actual information concerning the
!                    particle trajectories
!                    part(1,i) contains t_i
!                    part(2-3,i) contains x_i and y_i
!     partout       Array containing the actual information concerning the
!                    particle trajectories
!                    partout(1,i) contains t_i
!                    partout(2-3,i) contains x_i and y_i
!                    The difference with part is that this array may
!                    contain the values in by the user chosen time
!                    intervals, while part contains the time steps
!                    detected by the computation
!     plotfm         plot factor for scaling
!     special        Indicates which special situation is present
!                    Possible values:
!                    0     standard
!                    1     solution vector is of type 116 and a coloured
!                          2d plot is required
!                    2     solution vector is of type 110 or 115 and a coloured
!                          2d plot is required, whereas there are no unknowns
!                          in the vertices
!     t              actual time
!     tbegin         begin time for tracking
!     temp           work array to store the particle positions
!     tend           end time for tracking
!     tstepmark      time step wth respect to markers
!     tstepout       time step wth respect to output
!     tstepprint     time step wth respect to print
!     xt             x-position at time t
!     yfact          scaling factor for plots
!     yt             y-position at time t
!     zt             z-position at time t
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     FNPARTIC       User function subroutine that is called in eah step of the
!                    particle tracer
!     PL0003         Open plot, plots frame work and computes plotting
!     PLAFEP         General SEPRAN low level plotting subroutine
!     PLELEM         Plot elements for one element group (2d)
!     PLTR00         Actual particle tracking routine (2D)
!     PLTR10         Actual particle tracking routine (3D)
!     PLTR16         Plot reference cadre in case of 3-D
!     PLTRACOUT      Print and/or plot the particle trajectories
!                    If necessary the values are stored in an output array
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     SEPACTSOLBF2   compute pointers to degrees of freedom of vectors
!     SEPPLOTBN      Plot boundary of region
!     SEPRMSOLINFO   Remove array with sequence number ivectlc from ks
!     TO0069         Perform particle tracking
!     TOPQ10         Determine the number of elements to be created
!     TOPQ11         Determine all the triangles/tetrahedrons
!     TOPQU0         Subdivide mesh if necessary. Map unknowns into an organized
!                    array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      60   Input solution array has not been filled or is of the wrong type
!    2220   Parameter out of range
!    2221   Particle tracking for flux problem has not yet been implemented
!    2222   markers to be placed on the trajectories have not yet been
!           implemented in 3D
!    2223   velocities are not constant per element
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Check which situation and check all parameters (defaults)
!     Compute large array of triangles (ndim = 2) or
!     tetrahedrons (ndim = 3). Depending on the specified solution
!     vector (velocity). Keep for each element the element group number
!     because this may be important for the porosity.
!     Plot cadre and in case of 2d mesh.
!     Compute and print/plot npart traces
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen( 'pltrac' )
      debug = debug_pltrac
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from pltrac'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nelgrpsav = nelgrp
      nelemsav = nelem
      kprobc => kprobpart%kprobc

!     --- Initiate all index values

      indexe = 0
      nullify(iorig, itel)

!     --- Set default indicator values

      npart  =  1
      iprint = -1
      iplot  =  2
      nview  =  1
      ikeus  =  0
      imark  =  0
      istoutp = 0
      icallfn = 0
      istepprint = 0
      itypeinteg = 1
      iaccuracy = 0
      iresul = 0

      ikeuze = iinpar( 1 )

      if ( ikeuze>1 ) npart   = iinpar( 2)
      if ( ikeuze>2 ) iprint  = iinpar( 3)
      if ( ikeuze>3 ) iplot   = iinpar( 4)
      if ( ikeuze>4 ) nview   = iinpar( 5)
      if ( ikeuze>5 ) ikeus   = iinpar( 6)
      if ( ikeuze>6 ) imark   = iinpar( 7)
      if ( ikeuze>7 ) istoutp = iinpar( 8)
      if ( ikeuze>8 ) icallfn = iinpar( 9)
      if ( ikeuze>9 ) istepprint = iinpar(10)
      if ( ikeuze>10 ) itypeinteg = iinpar(11)
      if ( ikeuze>11 ) iaccuracy = iinpar(12)
      if ( debug ) then

!     --- debug is true, print input

         call prinin ( iinpar,ikeuze,'iinpar')
         call prinrl ( rinpar, 8, 'rinpar' )

      end if

!     --- Check value of ikeus for case of fluxes

      itemp  = kprobc( 1 )

!     --- Check value of itemp
!         Flux method, fluxes in the middles of all sides

      if ( itemp==650 ) then

         if ( ikeus/=2 .and. ikeus/=3 ) then

!        --- Error detected

            call errint ( itemp, 1 )
            call errsub ( 2221, 1, 0, 0 )
            goto 1000

         end if

      end if

!     --- Check print value (-1 no print,
!                             0 print on standard output iprint=irefwr
!                            >0 print on ref iprint )

      if ( iprint==0 ) then

         iprint = irefwr

      else if ( iprint<0 ) then

         iprint = 0

      end if

!     --- Check value of NVIEW

      if ( nview<1 .or. nview>4 ) then

         call errint ( 5, 1 )
         call errint ( nview, 2 )
         call errint ( 1, 3 )
         call errint ( 2, 4 )
         call errchr ( 'iinpar', 1 )
         call errchr ( 'nview', 2 )
         call errwar ( 2220, 4, 0, 2 )

         nview     = 1

      end if

!     --- Check value of IMARK

      if ( ndim==3 .and. imark/=0 ) then

         call errwar ( 2222, 0, 0, 0 )
         imark  = 0

      end if

!     --- Determine values of rinpar

      plotfm = rinpar(1)
      yfact  = rinpar(2)
      tbegin = rinpar(3)
      tend   = rinpar(4)
      difeps = 0.01d0
      if ( imark>0 ) tstepmark  = rinpar(5)
      if ( istepprint>0 ) tstepprint  = rinpar(6)
      if ( istoutp>1 ) tstepout  = rinpar(7)
      if ( iaccuracy>0 ) difeps  = rinpar(8)

!     --- If a flux field is given call TO0069

      maxsteps = 20000
      if ( ikeus>1 ) then

!     --- Fluxes

         ichois(1) = 5
         ichois(2) = 1
         ichois(3) = iprint
         ichois(4) = iplot
         ichois(5) = nview

         call sepactsolbf2 ( isolut)
         usol => ks(isolut)%sol

         if ( icount==0 ) then
            if ( ndim==2 ) then
               icount = 3
            else if ( ndim==3 ) then
               icount = 4
            end if
         end if

         call to0069 ( usol, startp, npart, tbegin, tend, &
                       plotfm, yfact, ichois, user, iuser, &
                       icount, maxsteps )

         go to 1000

      end if

!     --- Standard situation with two possibilities
!         Determine iveck

      if ( ks(isolut)%typevec==116 ) then

!     --- Velocity vector is given on all the elements
!         In the PLTR routines it is assumed that velocities are only given
!         in the centroids of the elements (Check!)

         length = ndim * nelem

         if ( length/=ks(isolut)%nusol ) then

!        --- Not correct velocity field terminate execution

            call errwar ( 2223, 0, 0, 0 )
            goto 1000

         end if

         iveck = 1

!        --- With or without porosity

         if ( ikeus==1 ) iveck = 2

      else if ( ks(isolut)%typevec==110 .or. ks(isolut)%typevec==115 ) then

!     --- Velocity vector is given in the nodal points

         iveck = 3

!        --- With or without porosity

         if ( ikeus==1 ) iveck = 4

      else

!     --- Error, type not recognized

         call errint ( ks(isolut)%typevec, 1 )
         call errwar ( 60, 1, 0, 0 )

         goto 1000

      end if

!     --- Velocity field is given either in points or on elements
!         Determine the correct pointers for an array with elements, an array
!         with velocities and eventually the porosities
!         indexe help array for new elements (to be created and filled
!                                             in TOPQU0)

      length    = nelgrp + 1

      allocate ( iwork1(length) )

!     --- Fill array number for the velocity components
!         (number of components is equal to ndim)

      number(1) = ndim
      number(2) = 1
      number(3) = 2
      number(4) = 3

      compl     = .false.

      jchois    = 1
      iskip     = 0
      ivul      = 1

!     --- Divide in linear triangles (2D) or tetrahedrons (3D)

      if ( ks(isolut)%typevec==110 .or. ks(isolut)%typevec==115 ) then

         if ( ndim==2 ) jchois = 1
         if ( ndim==3 ) jchois = 6
         special = 0
         ilevel = 1
         iresul = 0
         call topqu0 ( jchois, number, iskip, iskpar, iresul, &
                       compl, ivul, special, ilevel )

!        --- Result of division
!            Local Elements array

         indexe = iresul(1,2)

!        --- Number of elements for this local mesh

         nelemcac = iresul(3,2)

      else if ( ks(isolut)%typevec==116 ) then

!     --- Run through all groups of elements and divide in either
!         triangles (2D) or in tetrahedrons (3D),
!         remember the element number of the original element
!         Compute length needed for triangle\tetrahedron array and
!         help trapping array

         call topq10 ( nelemcac )

!        --- Create element help array

         length = 3 * nelemcac

         allocate ( kmeshctmp(length) )
         indexe = length

!        --- Create velocity help array

         length = nelemcac

         allocate ( iorig(length) )

         if ( indexe==0 ) kmeshctmp => kmeshc

!        --- Fill the (help) mesh array (triangles/tetrahedrons)

         call topq11 ( kmeshctmp, ivul , iwork1, iorig )

!        --- Old solution can be used

         usolt => ks(isolut)%sol

      end if

!     --- Create local help array for trapping (length nelemcac)

      allocate ( itel(nelemcac) )
      if ( istoutp==1 ) then

!     --- istoutp = 1, make space at the end of buffer to store the particle
!         trajectories temporarily

         lenout = (ndim+1)*maxsteps

      else if ( istoutp>1 ) then

!     --- istoutp>1, make space at the end of buffer to store the particle
!         trajectories temporarily

         lenout = (ndim+1)*(int((tend-tbegin)/tstepout)+1)

      else

         lenout = 0

      end if
      if ( ierror/=0 ) go to 1000

      allocate ( iwork(npart), temp(max(lenout*npart,1)), stat = error )
      if ( error/=0 ) call eralloc ( error, lenout*npart+npart, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Determine coordinates, curves, numbering and velo's

      allocate ( part((ndim+1)*(maxsteps+1)), &
                 partout((ndim+1)*(maxsteps+1)), stat = error )
      if ( error/=0 ) call eralloc ( error, ndim*maxsteps, 'part' )
      if ( ierror/=0 ) go to 1000

      if ( indexe==0 ) kmeshctmp => kmeshc
      kpos = 1

!     --- Open the plot

      ikader = jkader
      jmarksv  = jmark

!     --- Plot cadre without axis

      jkader =  -1
      jmark  =   5

      dcolor = 0

      textx  = 'x-axis'
      texty  = 'y-axis'

!     --- Plot cadre in COLOR 0 if IPLOT>0

      if ( iplot>=1 .and. ndim==2 ) then

         call pl0003 ( jkader, 1, plotfm, yfact, 0 )

      else if ( iplot>=1 .and. ndim==3 ) then

!     --- Plot reference cadre

         call pltr16 ( plotfm, yfact, nview )

      end if

!     --- Plot elements in COLOR 1 if IPLOT>1

      dcolor = 1

      if ( iplot>1 .and. ndim==2 ) then

         ilow    =  1
         ikelmc  =  0
         ncontr  = .true.

         do ielgrp = 1 , nelgrp

            ishape = kmeshf(ielgrp)
            circle = .true.
            if ( ishape<=0 ) circle = .false.
            inelem = sepgetinelem ( ielgrp )
            inpelm = sepgetinpelm ( ielgrp )
            nboun = inpelm
            if(kelmb/=0 ) nboun = kmeshb(ielgrp)

            istart = 1 + ikelmc

            call plelem ( ncontr, coor, kmeshctmp(istart), &
                          ilow, inelem, inpelm, nboun, &
                          circle, ndim, .false. )

            ilow   = ilow + inelem
            ikelmc = ikelmc + inpelm * inelem

         end do

      end if

!     --- Compute and plot the particle tracks

      if ( ndim==2 ) then

         dcolor = 3

         do id = 1, npart

!        --- Determine coordinates of initial point of track

            xt = startp( 1, id )
            yt = startp( 2, id )

!           --- Change color

            dcolor = dcolor + 1
            contin = .true.

            if ( dcolor>8 ) dcolor = 3

!           --- Set begin time

            t = tbegin

!           --- Put values in buffer

            part(1) = t
            part(2) = xt
            part(3) = yt
            nsteps = 1
            if ( icallfn>0 ) &
                call fnpartic ( t, xt, yt, 0d0, .true., contin )

!           --- Determine track of this particle

            if ( contin ) then
               call pltr00 ( iveck, kmeshctmp, coor, &
                             usolt, xt, yt, nelemcac, t, tend, &
                             itel, user, iuser, iwork1, iorig, nsteps, &
                             maxsteps, part, icallfn, difeps, &
                             itypeinteg )
            end if
            if ( ierror/=0 ) go to 1000

            call pltracout ( part, ndim, iprint, iplot, &
                             istoutp, partout, istepprint, &
                             tstepout, tstepmark, tstepprint, nsteps, &
                             id, imark, nview )

            if ( istoutp==1 ) then

!           --- Store array part in temp from position kpos

               iwork(id) = nsteps
               do i = 0, 3*nsteps-1
                  temp(kpos+i) = part(1+i)
               end do
               kpos = kpos+3*nsteps

            else if ( istoutp>1 ) then

!           --- Store array partout in temp from position kpos

               iwork(id) = nsteps
               do i = 0, 3*nsteps-1
                  temp(kpos+i) = partout(1+i)
               end do
               kpos = kpos+3*nsteps

            end if

         end do

         if ( iplot>=1 ) then

!        --- Plot boundaries with color 9 and close plot dataset

            idoubl = 1
            iinner = 0
            dcolor = 9

            call sepplotbn ( idoubl, iinner, .false. )

            call plafep ( 2, 0d0, 0d0, 0d0, 0d0, 0 )

         end if

      else if ( ndim==3 ) then

         dcolor = 3

         do id = 1, npart

!        --- Determine coordinates of initial point of track

            xt = startp( 1,id )
            yt = startp( 2,id )
            zt = startp( 3,id )

!           --- Change colour

            dcolor = dcolor + 1

            if ( dcolor>8 ) dcolor = 3

!           --- Set begin time

            t = tbegin
            contin = .true.

!           --- Put values in buffer

            part(1) = t
            part(2) = xt
            part(3) = yt
            part(4) = zt
            nsteps = 1
            if ( icallfn>0 ) &
                call fnpartic ( t, xt, yt, zt, .true., contin )

!           --- Determine track of this particle

            call pltr10 ( iveck, kmeshctmp, coor, &
                          usolt, xt, yt, zt, nelemcac, t, tend, &
                          itel, user, iuser, &
                          iwork1, iorig, nsteps, &
                          maxsteps, part, icallfn )

            call pltracout ( part, ndim, iprint, iplot, &
                             istoutp, partout, istepprint, &
                             tstepout, tstepmark, tstepprint, nsteps, &
                             id, imark, nview )

            if ( istoutp>0 ) then

!           --- Store array partout in temp

               iwork(id) = nsteps
               do i = 0, 4*nsteps-1
                  temp(kpos+i) = partout(1+i)
               end do
               kpos = kpos+4*nsteps

            end if

         end do

         if ( iplot>=1 ) call plafep ( 2, 0d0, 0d0, 0d0, 0d0, 0 )

      end if
      if ( istoutp>0 ) then

!     --- Store array in particinf and partic

         lenout = kpos-1
         if ( debug ) write(irefwr,1) 'lenout, npart', lenout, npart
         if ( associated(particinf) ) deallocate(particinf, partics)
         allocate ( particinf(2,npart+1), partics(lenout) )
         partics = temp(1:lenout)
         particinf(1,1) = npart
         particinf(2,1) = ndim
         jpos = 1
         do i = 1, npart
            nsteps = iwork(i)
            particinf(1,i+1) = nsteps
            particinf(2,i+1) = jpos
            jpos = jpos+nsteps
         end do

      end if

!     --- Reset values

      dcolor = 0
      jmark  = jmarksv
      jkader = ikader

      deallocate ( part, partout, iwork, temp, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'part' )

!     --- Free temporary arrays
!         Check whether new triangle/tetrahedron array has been
!         created or the old one has been used

      if ( indexe/=0 ) deallocate ( kmeshctmp )
      if ( associated(iwork1) ) deallocate(iwork1)
      if ( associated(iorig) ) deallocate(iorig)
      if ( associated(itel) ) deallocate(itel)

      kmeshc => km(iseqkm)%kmeshc
      nelgrp = nelgrpsav
      nelem = nelemsav
      inelem = nelem
      if ( iresul(2,3)==1 ) call seprmsolinfo ( iresul(1,3) )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End pltrac'

      end if  ! ( debug )
      call erclos ( 'pltrac' )

      end subroutine pltrac
