      subroutine prcomplevels ( contln, ncntln, icompt, lencontln, &
                                inpsep, rinsep, imult )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-01-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute levels for contour plots and store in array contln
!     The number of contour levels ncntln is computed
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     contour_level
!     level
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: lencontln, inpsep(*), imult
      integer, intent(out) :: ncntln, icompt
      double precision, intent(in) :: rinsep(*)
      double precision, intent(out) :: contln(lencontln)

!     contln         o    User array of length ncntln + 5 (if used) in which the
!                         user is supposed to have stored the contour levels
!                         from position 6
!     icompt         o    indicates how the contour levels must be
!                         computed from NCNTLN in the case NCNTLN<=0
!                         0:  The minimum and maximum level are computed
!                         1:  The minimum level is given in contln(6), the
!                             maximum level must be computed
!                         2:  The maximum level is given in contln(7), the
!                             minimum level must be computed
!                         3:  The minimum level is given in contln(6), the
!                             maximum level in contln(7)
!     imult          i    Multiplication factor for default number
!                         of levels
!                         1: 10 levels
!                         2: 20 levels
!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine readscplotcn
!     lencontln      i    Declared length of array contln
!     ncntln         o    number of contour levels
!     rinsep         i    Array containing the real input for this subroutine
!                         See subroutine readscplotcn
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ipos
      double precision :: minlevel, maxlevel

!     ipos           Position in array rinsep
!     maxlevel       maximum contour level
!     minlevel       minimum contout level
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcomplevels' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcomplevels'
         call prinin ( inpsep, 20, 'inpsep' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Initialize contln

      contln = 0d0
      contln(1) = lencontln
      icompt = 0

      ncntln = inpsep(4)   ! number of contour levels

      minlevel = 1d50
      maxlevel =-1d50
      if ( inpsep(5)>0 ) minlevel = rinsep(inpsep(5))
      if ( inpsep(6)>0 ) maxlevel = rinsep(inpsep(6))

      if ( minlevel<1d49 .or. maxlevel>-1d49 ) then

!     --- minlevel or maxlevel given, set ncntln

         if ( ncntln>0 ) then
            ncntln  = -ncntln-1001
         else
            ncntln = -imult*10-1001
         end if
         if ( minlevel<1d49 ) then

!        --- minlevel given

            if ( maxlevel>-1d49 ) then

!           --- minlevel and maxlevel given

               icompt = 3
               contln(6) = minlevel
               contln(7) = maxlevel

            else

!           --- minlevel given and maxlevel not given

               icompt = 1
               contln(6) = minlevel

            end if

         else

!        --- maxlevel given

            icompt = 2
            contln(7) = maxlevel

         end if

      end if

      if ( inpsep(11)>0 ) then

!     --- levels given explicitly

         ncntln = inpsep(4)   ! number of contour levels
         ipos = inpsep(11)
         call inichkarlen ( ncntln+5, lencontln, 'contln' )
         if ( ierror/=0 ) go to 1000
         contln(6:ncntln+5) = rinsep(ipos:ipos+ncntln-1)

      end if  ! ( inpsep(11)>0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ncntln', ncntln
         write(irefwr,2) 'minlevel, maxlevel', minlevel, maxlevel
         call prinrl ( contln, ncntln+5, 'contln' )
         write(irefwr,*) 'End prcomplevels'

      end if  ! ( debug )
      call erclos ( 'prcomplevels' )

      end subroutine prcomplevels
