      subroutine sepintpolline ( inpsep, rinsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-09-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute intersection of line (p1,p2) with surface S1
!     In case of 2d mesh, the intersection with the whole mesh is computed
!     Is is assumed that the mesh is activated and subarrays have been
!     stored in module sepmodulemesh
! **********************************************************************
!
!                       KEYWORDS
!
!     intersection
!     mesh
!     line
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmoduleplotfn
      use sepmoduleint2d
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: inpsep(*)
      double precision, intent(in) :: rinsep(*)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine readscplotfn
!     rinsep         i    Array containing the real input for this subroutine
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: isurnr, istart, inpelm, mult, i, iielem, j, k, &
                 nsubel, istartsur, nodp
      double precision :: normallc(3), trans(3,3)
      logical :: planefound

!     i              Counting variable
!     iielem         Relative element number in element group
!     inpelm         number of points per element
!     istart         First position used in kmeshn
!     istartsur      Start address of surface in inpsep
!     isurnr         Surface sequence number
!     j              Counting variable
!     k              Counting variable
!     mult           multiplication factor.
!                    Number of subtriangles per element
!     nodp           Nodal point sequence number
!     normallc       Normal on surface
!     nsubel         number of sub elements an element is subdivided into
!     planefound     If true the surface is a plane
!     trans          array containing the inverse of the transformation
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     ERFILLNODSKMC1 Fill array nodes for elements 1 to inelem
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     INIINFSURF     Find starting address of surface isurnr in array kmeshn
!                    Also find number of element and number of points per
!                    element in isurnr
!     MSHCHPLANE     Check if all nodes in surface are lying in a plane
!     MSHTRANS3D2D   Compute the transformation from 3d to 2d for a plane
!                    given the normal
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
!     SEP2DTRIAN     Make a reference of the 2d array ibuffer2d to the pointer
!                    trian2d
!     SEPINTSECTLINE Compute intersection of a line with a set of triangles
!     SEPMAPPNT3D2D  Map coordinates stored in x3d onto surface and store in x2d
!                    using the tranformation stored in trans
!     SEPNORMALSURF  Compute the normal onto the first element of surface
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2711   Option not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
!     if ( ndim==2 ) then
!        subdivide mesh into triangles
!        compute intersection with line
!     else
!        subdivide mesh into triangles
!        Map surface onto 2d mesh
!        Map line on this mesh
!        compute intersection with line
!        Map intersection onto original coordinates
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepintpolline' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepintpolline'
         write(irefwr,1) 'ndim', ndim
         call prinin ( inpsep, 16, 'inpsep' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( debug ) write(irefwr,1) 'ndim', ndim

      if ( ndim==2 ) then

!     --- ndim = 2, two-dimensional mesh, use complete mesh

         call errsub ( 2711, 0, 0, 0 )
         go to 1000

      else

!     --- ndim = 3, three-dimensional mesh, use surface

         allocate ( iwork(npoint), nodes(npoint), stat = error )
         if ( error/=0 ) call eralloc ( error, 2*npoint, 'iwork' )
         if ( ierror/=0 ) go to 1000

         iwork = 0

         istartsur = inpsep(2)+inpsep(1)
         isurnr = inpsep(istartsur)
         if ( debug ) write(irefwr,1) 'isurnr', isurnr

!        --- Map surface onto 2d mesh
!            First compute normal onto first element

         call iniinfsurf ( kmeshn, istart, inpelm, inelem, nsurfs, isurnr )
         if ( debug ) &
            write(irefwr,1) 'istart, inpelm, inelem, nsurfs', &
                             istart, inpelm, inelem, nsurfs

         call sepnormalsurf ( kmeshn(istart+1), coor, normallc, isurnr, inpelm )
         if ( debug ) call prinrl ( normallc, 3, 'normal' )

!        --- For all nodes inode in surface fill nodes

         nnodes = 0
         call erfillnodskmc1 ( nodes, nnodes, kmeshn(istart+1), inelem, &
                               inpelm, iwork )
         if ( debug ) call prinin ( nodes, nnodes, 'nodes' )

!        --- Check if all elements have the same normal

         planefound = .false.
         call mshchplane ( nnodes, nodes, coor, isurnr, normallc, planefound )
         if ( ierror/=0 ) go to 1000

!        --- subdivide surface into linear triangles

         if ( inpelm==3 ) then

!        --- No subdivision necessary, use kmeshn

            allocate ( triangles(inelem*inpelm), stat = error )
            if ( error/=0 ) call eralloc ( error, inelem*inpelm, 'triangles' )
            if ( ierror/=0 ) go to 1000

            triangles = kmeshn(istart+1:istart+inelem*inpelm)
            mult = 1

         else

!        --- Subdivision necessary, allocate triangles and fill

            if ( inpelm==4 ) then
               mult = 2
               istart =  ishstr(5)
            else if ( inpelm==6 .or. inpelm==7 ) then
               mult = 4
               istart =  ishstr(4)
            else
               mult = 8
               istart =  ishstr(6)+20
            end if  ! ( inpelm==4 )
            nsubel = mult*inpelm
            inelem = inelem*mult

            allocate ( triangles(inelem*3), stat = error )
            if ( error/=0 ) call eralloc ( error, inelem*3, 'triangles' )
            if ( ierror/=0 ) go to 1000

              do i = 1, nsubel
                 do iielem = 1, inelem
                    triangles((iielem-1)*nsubel+i) = &
                       kmeshc((iielem-1)*inpelm+ishsub(istart+i))
                 end do  ! iielem = 1, inelem
              end do  ! i = 1, nsubel

         end if  ! ( inpelm==3 )

         call sep2dtrian ( triangles, inelem )

!        --- Renumber triangles to local numbering

         triangles(1:inelem*3) = iwork(triangles(1:inelem*3))

!        --- Compute the transformation from 3d to 2d

         call mshtrans3d2d ( normallc, trans )
         if ( debug ) then
            call prinrl ( normallc, 3, 'n')
            call prinrl1 ( trans, 3, 3, 'trans' )
         end if  ! ( debug )

!        --- Map surface onto 2d surface

         allocate ( coor2d(2,nnodes), work(2,nnodes), stat = error )
         if ( error/=0 ) call eralloc ( error, 2*nnodes, 'coor2d' )
         if ( ierror/=0 ) go to 1000

         do i = 1, nnodes
            nodp = nodes(i)
            do k = 1, 2
               coor2d(k,i) = sum(coor(1:3,nodp)*trans(k,1:3))
            end do  ! k = 1, 2
         end do  ! i = 1, nnodes

         if ( debug ) call prinrl1 ( coor2d, nnodes, 2, 'coor2d' )

!        --- Map line on this surface

         istart = inpsep(5)
         call sepmappnt3d2d ( rinsep(istart), trans, x1 )
         call sepmappnt3d2d ( rinsep(istart+3), trans, x2 )

!        --- compute intersection with line

         call sepintsectline

         deallocate ( triangles, iwork, nodes, coor2d, work, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'iwork' )

      end if  ! ( ndim==2 )

1000  call erclos ( 'sepintpolline' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepintpolline'

      end if  ! ( debug )

      end subroutine sepintpolline
