      subroutine sepplotdt ( idgfd1, idgfd2, isol, plotfm, yfact, multfact )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-06-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Plot destorted mesh
!     Old subroutine plotdt
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     plot
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulecactl
      use sepmodulekprob
      use sepmodulekmesh
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer idgfd1, idgfd2, isol
      double precision plotfm, yfact, multfact

!     idgfd1  i   the displacement u,v is given by the idegfd1 th and
!                 idgfd2 th degree of freedom in the nodal point
!     idgfd2  i   see idgfd1
!     isol    i   array containing information of the solution vector
!     multfact  i   the co-ordinates of the distorted mesh are given by:
!                 xnew = x + multfact * u
!                 ynew = y + multfact * v
!     plotfm  i   length of the plot in cm's
!     yfact   i   scale factor: all y-co-ordinates are multiplied
!                 by yfact before plotting
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical compl
      integer ikader, iuser(1)
      double precision, pointer :: coordt(:,:)
      double precision :: timebefore(2)

!     compl          indication whether the arithmetic is real (false)
!                    or complex (true)
!     coordt         Contains coorindates of shifted mesh
!     ikader         reserved
!     iuser          reserved
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLMN         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERSETTIME      Get the CPU time at start of subroutine
!     PL0003         Open plot, plots frame work and computes plotting
!     PL0010         ?
!     PLAFEP         General SEPRAN low level plotting subroutine
!     PLELM1         Plot elements in 2D mesh
!     SEPACTSOLBF2   compute pointers to degrees of freedom of vectors
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      62   dimension of space (ndim) has wrong value
!      90   array isol has not been filled, or the boundary conditions have
!           not been stored in the solution vector
!     214   parameter idgfd1 has a non-positive value
!     608   factor has wrong value
!     609   idgfd2 has wrong value
!     610   array isol corresponds to a complex array
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepplotdt' )
      if ( itime>0 ) call ersettime ( timebefore )
      if ( ks(isol)%typevec/=110 .and. ks(isol)%typevec/=115 ) then
         call errint ( ks(isol)%typevec, 1 )
         call errsub ( 90, 1, 0, 0 )
      end if
      if ( multfact<=0d0 ) then
         call erreal ( multfact, 1 )
         call errsub ( 608, 0, 1, 0 )
      end if
      call sepactsolbf2 ( isol )

      compl = ks(isol)%compl==1
      if ( idgfd1<=0 .or. idgfd1>nphysi ) then
         call errint ( idgfd1, 1 )
         call errsub ( 214, 1, 0, 0 )
      end if
      if ( idgfd2<=0 .or. idgfd2>nphysi ) then
         call errint ( idgfd2, 1 )
         call errint ( nphysi, 2 )
         call errsub ( 609, 2, 0, 0 )
      end if
      if ( compl ) call errsub ( 610, 0, 0, 0 )
      if ( ndim/=2 ) then
         call errint ( ndim, 1 )
         call errsub ( 62, 1, 0, 0 )
      end if

      allocate ( coordt(2,npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, 2*npoint, 'coordt' )
      if ( ierror/=0 ) go to 1000

      usoltemp => ks(isol)%sol
      if ( ierror/=0 ) go to 1000

      call pl0010 ( coor, usoltemp, kprobfi, indprfi, coordt, &
                    npoint, multfact, idgfd1, idgfd2, nphysi )
      textx = 'x'
      texty = 'y'
      coor => coordt
      call pl0003 ( ikader, 2, plotfm, yfact, 0 )

!     --- plot elements

      iuser(1) = 0
      call plelm1 ( iuser, coordt, .false., .true., .false. )

!     --- close plot dataset

      call plafep ( 2, 0d0, 0d0, 0d0, 0d0, 0 )
      jkader = ikader

      deallocate ( coordt, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'coordt' )

1000  ierror = 0
      call erclmn ( 'sepplotdt', timebefore )
      end subroutine sepplotdt
