      subroutine topsubint2d1 ( usol, usolnew, weight, skipelgrps, iwork )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 26-01-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpolate solution from original mesh to triangular mesh
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     subdivision
!     surface
!     2d
!     triangle
!     interpolation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: skipelgrps(2,*)
      integer, intent(out) :: iwork(npoint)
      double precision, intent(in) :: usol(npoint)
      double precision, intent(out) :: usolnew(npoint), weight(npoint)

!     iwork          o    integer work array to mark nodes that have been
!                         treated completely (iwork(node)=1)
!                         For those nodes no interpolation is required, but
!                         the value is copied directly
!     skipelgrps     i    array of size 2 x nelgrp containing information
!                         for the subdivision of the elements in triangles
!                         (1,i) mult, i.e. number of triangles each element
!                               of element group i must be subdivided
!                         (2,i) typesubdiv, defines special subdivisions
!     usol           i    Solution vector
!     usolnew        o    Temporary array usol
!     weight         o    array containing the weights for those values that
!                         must be averaged
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ielgrp, inpelm, iskip, ikelmc, &
                 typesubdiv, i, inode
      logical :: fillweight

!     fillweight     if true the weight vector must be filled
!     i              Counting variable
!     ielgrp         element group number
!     ikelmc         position in kmesh part c
!     inode          Node sequence number
!     inpelm         number of points per element
!     iskip          Indicates if element groups must be skipped (0)
!                    or not (>0)
!     typesubdiv     Defines type of subdivision and interpolation
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     TOPSUBINT2D2   Interpolate solution from original mesh to triangular mesh
!                    case typesubdiv = 1
!     TOPSUBINT2D3   Interpolate solution from original mesh to triangular mesh
!                    case typesubdiv = 2
!     TOPSUBINT2D4   Interpolate solution from original mesh to triangular mesh
!                    case typesubdiv>2
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topsubint2d1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from topsubint2d1'
         call prinrl ( usol, npoint, 'usol' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ikelmc = 0
      fillweight = .false.

      weight = 0d0
      iwork = 0

!     --- Loop over element groups

      do ielgrp = 1, nelgrp

         ishape = kmeshf(ielgrp)   ! shape of element
         inpelm = Topgrp(ielgrp)%inpelm
         inelem = Topgrp(ielgrp)%inelem

         typesubdiv = skipelgrps(2,ielgrp)
         iskip = skipelgrps(1,ielgrp)

         if ( debug ) &
           write(irefwr,1) 'ielgrp, inpelm, ishape, iskip, typesubdiv', &
                            ielgrp, inpelm, ishape, iskip, typesubdiv

         if ( iskip>0 ) then

!        --- group must not be skipped

            select case (typesubdiv)

            case(0)

!           --- unknown present in all nodes, copy value
!               Set iwork equal to 1 to mark as filled

               do i = 1, inelem*inpelm

                  inode = kmeshc(ikelmc+i)
                  if ( iwork(inode)==0 ) then

!                 --- node has not been filled yet

                     iwork(inode)= 1  ! mark as filled
                     usolnew(inode) = usol(inode)  ! copy value

                  end if  ! ( iwork(inode)==0 )

               end do  ! i = 1, inelem*inpelm

            case(1)

!           --- unknown not present in all nodes, interpolate value
!               and fill weight

               call topsubint2d2 ( kmeshc(ikelmc+1), inelem, inpelm, &
                                   ishape, usol, usolnew, weight, &
                                   npoint, iwork, fillweight )

            case(2)

!           --- unknown not present in all nodes, interpolate value
!               and fill weight

               call topsubint2d3 ( kmeshc(ikelmc+1), inelem, inpelm, &
                                   ishape, usol, usolnew, weight, &
                                   npoint, iwork, fillweight )

            case(3,4)

!           --- unknown not present in all nodes, interpolate value
!               and fill weight

               call topsubint2d4 ( kmeshc(ikelmc+1), inelem, inpelm, &
                                   typesubdiv, usol, usolnew, weight, &
                                   npoint, iwork, fillweight )

            end select  ! case (typesubdiv)

         end if  ! ( iskip>0 )

         ikelmc = ikelmc+inelem*inpelm

      end do  ! ielgrp = 1, nelgrp

      if ( fillweight ) then

!     --- weights have been filled, compute mean value

         do i = 1, npoint

            if ( iwork(i)==0 .and. weight(i)>0d0 ) then

!           --- weight has been filled

               usolnew(i) = usolnew(i)/weight(i)

            end if  ! ( iwork(i)==0 .and. weight(i)>0d0 )

         end do  ! i = 1, npoint

      end if  ! ( fillweight )

1000  call erclos ( 'topsubint2d1' )
      if ( debug ) then

!     --- Debug information

         call prinrl ( usolnew, npoint, 'usolnew' )
         write(irefwr,*) 'End topsubint2d1'

      end if  ! ( debug )

      end subroutine topsubint2d1
