      subroutine topsubint2d3 ( kmeshc, inelem, inpelm, ishape, usol, &
                                usolnew, weight, npoint, iwork, &
                                fillweight )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 26-01-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpolate solution from original mesh to triangular mesh
!     case typesubdiv = 2
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     subdivision
!     surface
!     2d
!     triangle
!     interpolation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: inelem, inpelm, &
                             kmeshc(inpelm,inelem), ishape, npoint
      double precision, intent(in) :: usol(npoint)
      double precision, intent(inout) :: usolnew(npoint), weight(npoint)
      integer, intent(inout) :: iwork(npoint)
      logical, intent(inout) :: fillweight

!     fillweight    i/o   if true the weight vector must be filled
!     inelem         i    Number of elements in a specific element group
!     inpelm         i    number of points per element
!     ishape         i    Shape number of elements
!     iwork         i/o   integer work array
!                         treated completely (iwork(node)=1)
!                         For those nodes no interpolation is required, but
!                         the value is copied directly
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     npoint         i    Number of nodal points
!     usol           i    Solution vector
!     usolnew       i/o   Temporary array usol
!     weight        i/o   array containing the weights for those values that
!                         must be averaged
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ielem, inode, i
      double precision :: value

!     i              Counting variable
!     ielem          Element number
!     inode          Node sequence number
!     value          value of function in node
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topsubint2d3' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from topsubint2d3'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      select case ( ishape )
      case(6)

!     --- bi-quadratic quadrilateral

         do ielem = 1, inelem

            do i = 2, 8, 2
               inode = kmeshc(i,ielem)
               if ( iwork(inode)==0 ) then

!              --- node has not been filled yet

                  iwork(inode)= 1  ! mark as filled
                  usolnew(inode) = usol(inode)  ! copy value

               end if  ! ( iwork(inode)==0 )

            end do  ! i = 1, 8

            inode = kmeshc(9,ielem)
            iwork(inode)= 1  ! mark as filled
            usolnew(inode) = usol(inode)  ! copy value

!           --- Next interpolate vertices

            inode = kmeshc(1,ielem)
            if ( iwork(inode)==0 ) then

!           --- node has not been filled yet

               usolnew(inode) =  usolnew(inode)+ &
                                 usol(kmeshc(2,ielem))+ &
                                 usol(kmeshc(8,ielem))- &
                                 usol(kmeshc(9,ielem))
               weight(inode) = weight(inode)+1d0
               fillweight = .true.

            end if  ! ( iwork(inode)==0 )

            inode = kmeshc(3,ielem)
            if ( iwork(inode)==0 ) then

!           --- node has not been filled yet

               usolnew(inode) =  usolnew(inode)+ &
                                 usol(kmeshc(2,ielem))+ &
                                 usol(kmeshc(4,ielem))- &
                                 usol(kmeshc(9,ielem))
               weight(inode) = weight(inode)+1d0
               fillweight = .true.

            end if  ! ( iwork(inode)==0 )

            inode = kmeshc(5,ielem)
            if ( iwork(inode)==0 ) then

!           --- node has not been filled yet

               usolnew(inode) =  usolnew(inode)+ &
                                 usol(kmeshc(6,ielem))+ &
                                 usol(kmeshc(4,ielem))- &
                                 usol(kmeshc(9,ielem))
               weight(inode) = weight(inode)+1d0
               fillweight = .true.

            end if  ! ( iwork(inode)==0 )

            inode = kmeshc(7,ielem)
            if ( iwork(inode)==0 ) then

!           --- node has not been filled yet

               usolnew(inode) =  usolnew(inode)+ &
                                 usol(kmeshc(6,ielem))+ &
                                 usol(kmeshc(8,ielem))- &
                                 usol(kmeshc(9,ielem))
               weight(inode) = weight(inode)+1d0
               fillweight = .true.

            end if  ! ( iwork(inode)==0 )

         end do  ! ielem = 1, inelem

      case(7)

!     --- quadratic triangle with extra point
!         unknown in center

         do ielem = 1, inelem

!        --- interpolate other points

            value = usol(kmeshc(7,ielem))

            do i = 1, 6

               inode = kmeshc(i,ielem)
               if ( iwork(inode)==0 ) then

!              --- node has not been filled yet

                  usolnew(inode) =  usolnew(inode)+value
                  weight(inode) = weight(inode)+1d0
                  fillweight = .true.

               end if  ! ( iwork(inode)==0 )

            end do  ! i = 1, 6

         end do  ! ielem = 1, inelem

      end select  ! case ( ishape )

1000  call erclos ( 'topsubint2d3' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End topsubint2d3'

      end if  ! ( debug )

      end subroutine topsubint2d3
