      subroutine ercreatsub ( isol, inpcre, rincre, islold, ifirst, ipstin, &
                              ivecsq, nv, kvec, isubr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 05-13-2013 Extension with change_vector
!        version  1.1  date 05-12-2012 Long integers
!        version  1.0  date 03-01-2012
!
!   copyright (c) 2012-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Basis subroutine for the creation of one vector
!
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     create
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulevecs
      use sepmoduleplot
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: isol(*), inpcre(*), &
                 islold(*), ifirst, ipstin, ivecsq, nv, kvec, isubr
      double precision rincre(*)

!     ifirst         i    Value of first position of array INPCRE
!     inpcre         i    Array containing the integer input
!                         See subroutine creatv01
!     ipstin        i/o   Starting address in array INPCRE
!     islold        i/o   User input array in which the user puts information
!                         of all preceding solutions
!                         Integer array of length 5 x numold
!     isol          i/o   Standard SEPRAN array containing information of
!                         the solution vectors
!     isubr          i    Indicates the type of calling subroutine
!                         Possible values
!                         1:  By the user or CREATE
!                         2:  By PRESDF or PRESTM
!                         8:  By ercret standard
!                         9:  By ercret, use old vector
!     ivecsq         i    Sequence number of vector in isol
!     kvec           i    Loop variable over the vectors to be created
!     nv             i    Number of vectors to be created
!     rincre         i    Real input array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ichvc, icompl, ifill, &
              istartvc, itypvc, ivec, ivecold, &
              jpstin, k, kdeg, kfill, kfun, maxphys, mvec, n
      logical new, bndcon, obstaclefound

!     bndcon         If true boundary conditions must be filled, otherwise
!                    a vector must be created
!     ichvc          Indication of the type of vector to be created.
!                    Possible values
!                    0: Vector either exists or is a solution vector
!                    1: Vector is a solution vector
!                    2: Vector is an array of special structure
!                    3: Vector is an array of special structure defined per
!                       element
!                    4: Vector is an array of special structure defined per node
!                       per element
!     icompl         Indication if the vector to be created is real (0) or
!                    complex
!     ifill          Indicates how the vector must be filled.
!                    Possible values:
!                    -1  The corresponding vector is not filled, &
!                        only created
!                     0  The corresponding vector is created and made
!                        equal to zero
!                    m>0 The filling of the vector is defined by m commands
!     istartvc       Starting position of sequence numbers of vectors in inpcre
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     ivec           Gives the sequence number of the array of special
!                    structure if
!                    ICHVC = 2, or the number of degrees of freedom per element
!                    if
!                    ICHVC = 3 or the number of degrees of freedom per node if
!                    ICHVC = 4
!     ivecold        ivec for old vector
!     jpstin         Help variable to compute new value of ipstin partly
!     k              Counting variable
!     kdeg           Indication which degrees of freedom must be filled
!                    Possibilities:
!                    0:    All degrees of freedom
!                    >0:   Only the KDEGth degree of freedom
!                    <0:   The -KDEG degrees of freedom stored in array
!                          IARDEG must be filled
!     kfill          Indication how the nodal points must be filled into
!                    array NODES
!                    See subroutine prfillnodes
!     kfun           Indication of how the degrees of freedom must be
!                    filled.
!                    Possibilities:
!                    0:  Not filled
!                    <0:  Constant value stored in RINCRE(-KFUN)
!                    >0:  Function subroutine
!     maxphys        Maximum number of unknowns per point
!     mvec           Number of old vectors to be used in FUNCVECT
!     n              number of commands
!     new            Indication if the vector to be created is new (true) or old
!                    (false)
!     obstaclefound  If true values must be given in points in an obstacle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     CREATV116      Fill arrays of type 116 or 126
!     CREATV801      Fill arrays of type 801
!     CREATVSPEC     Fill array usol with values in the case that 1000<fkun<2000
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERCREATGLOB    Fill part global unknowns of vector
!     ERCREATSTAND   Fill one vector for one option only (standard case)
!     ERCREATSUB1    Compute some parameters for one loop in create vector
!     ERCREATSUB12   Fill array vector in case kfill = 12
!     ERCREATSUB2    Determine type of vector to be created
!     ERCREATSUB3    print information of create vector if necessary
!     ERCREATSUB4    Compute some parameters concerning the vector to be filled
!     ERCREATSUB5    Compute some parameters that are necessary to fill the
!                    vector
!     ERCREATSUB7    Raise ipstin as function of kfill and jpstin
!     ERCREATSUB8    Treatment of periodical boundary conditions while filling a
!                    vector
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPFILLKPROB   Fill parameters in module sepmodulekprob
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ercreatsub' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ercreatsub'
!        call ertrace
         write(irefwr,1) 'ipstin, ifirst', ipstin, ifirst
         call prinin ( isol, 5, 'isol' )
         call prinin ( inpcre(ipstin), 10, 'inpcre' )
  1      format ( a, 1x, (10i8) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Set bndcon

      bndcon = isubr==2 .or. isubr==4 .or. isubr==6 .or. isubr==7

!     --- Compute ICHVC, IPROB, IVEC, ICOMPL, IFILL and itypvc

      call ercreatsub1 ( isol, inpcre, ifirst, ipstin, ivecsq, &
                         nv, kvec, isubr, ichvc, iprob, ivec, &
                         icompl, ifill, itypvc )
      if ( ierror/=0 ) go to 1000

      if ( debug ) &
         write(irefwr,1) 'ichvc, iprob, ivec, icompl, ifill, itypvc, ifirst', &
                          ichvc, iprob, ivec, icompl, ifill, itypvc, ifirst

!     --- Compute starting address in kprob

      iseqkp = kprob
      kprobpart => kp(iseqkp)%kprobsub(iprob)
      kprobloc => kprobpart%kprobloc
      call sepfillkprob
      if ( indprf>0 ) kprobf => kprobpart%kprobf

      kprobvp1 => kp(iseqkp)%kprobsub(1)%kprobvp1
      kprobvp2 => kp(iseqkp)%kprobsub(1)%kprobvp2
      indprr = kprobloc(37)
      if ( isubr/=2 .and. indprr==0 ) then
         indprh = 0
      else
         kprobh => kprobpart%kprobh
         kprobhinv => kprobpart%kprobhinv
         indprh = kprobloc(50)
      end if  ! ( isubr/=2 )


!     --- Determine type of vector to be created
!         new defines if the vector already exists or must be created (true)
!         iprob, icompl, ivec may be adapted in case of an old vector

      call ercreatsub2 ( ichvc, itypvc, new, isol, &
                         icompl, kvec, isubr, ivec )
      if ( ierror/=0 ) go to 1000

      if ( debug ) then
         write(irefwr,*) 'new ', new
         write(irefwr,1) 'kvec, ivec', kvec, ivec
         call prinin ( isol(kvec), 5, 'isol' )
      end if  ! ( debug )

!     --- Recompute starting address in kprob (iprob may have been changed)

      kprobpart => kp(iseqkp)%kprobsub(iprob)
      kprobloc => kprobpart%kprobloc

!     --- print information of create vector if necessary

      call ercreatsub3 ( isubr, kvec, ifill )
      if ( ierror/=0 ) go to 1000

!     --- Compute some parameters concerning the vector to be filled
!         If vector is new create space and clear vector
!         In case of boundary conditions, clear last part of vector
!         If necessary array ipointobst is activated

      call ercreatsub4 ( isol, itypvc, ivec, &
                         icompl, kvec, new, ifill, bndcon, &
                         isubr, inpcre, ifirst, ipstin, obstaclefound )
      if ( ierror/=0 ) go to 1000

!     --- Set starting address of vector to be created

      usoltemp => ks(isol(kvec))%sol

!     --- ifill>0, Loop over IFILL cases, each fills a part of the vector

      do k = 1, ifill

!     --- Compute some parameters that are necessary to fill the vector
!         Compute jpstin (position in inpcre)

         call ercreatsub5 ( ipstin, jpstin, ifirst, inpcre, n, &
                            istartvc, kdeg, kfun, kfill, ivecold, &
                            mvec, islold, maxphys )

         if ( debug ) &
            write(irefwr,1) 'k, ipstin, jpstin, itypvc, kfill', &
                             k, ipstin, jpstin, itypvc, kfill

         if ( itypvc==116 ) then

!        --- itypvc = 116

            call creatv116 ( isol, nphyso, itypvc, kfill, kfun, &
                             kdeg, icompl, inpcre, rincre, jpstin, &
                             kvec, istartvc, maxphys, mvec )

         else if ( itypvc==801 ) then

!        --- itypvc = 801

            call creatv801 ( usoltemp, nphyso, kfill, kfun, &
                             kdeg, icompl, inpcre, rincre, jpstin, &
                             ivec )

         else if ( kfill==12 ) then

!        --- kfill = 12, use file with node numbers

            call ercreatsub12 ( isol, kvec, kdeg, jpstin, icompl, inpcre )

         else if ( kfill==15 ) then

!        --- kfill = 15, use unknowns corresponding to global element group

            call ercreatglob ( usoltemp, kdeg, &
                               kfun, inpcre(jpstin), rincre )

         else if ( kfill==0 .and. kfun>1000 .and. kfun<2000 ) then

!        --- kfill = 0 and 1000< kfun<2000
!            special function (only in case of all nodes)

            call creatvspec ( kfun, usoltemp, icompl )

         else

!        --- Standard case

            call ercreatstand ( kfill, n, inpcre, jpstin, kfun, rincre, &
                                ipstin, mvec, maxphys, bndcon, &
                                icompl, kdeg, kvec, ivecold, &
                                isol, istartvc, islold )

         end if  ! ( itypvc==116 )
         if ( ierror/=0 ) go to 1000

!        --- raise ipstin

         call ercreatsub7 ( inpcre, ipstin, jpstin, kfill, n )

!        --- Treatment of periodical boundary conditions

         call ercreatsub8 ( itypvc, kvec, isol, icompl )

      end do  ! k = 1, ifill

      indprh = kprobloc(50)

1000  if ( debug ) then

!     --- Debug information

 !        call printv ( isol, kmesh, kprob, 'sol' )
         write(irefwr,*) 'End ercreatsub'

      end if  ! ( debug )
      call erclos ( 'ercreatsub' )

      end subroutine ercreatsub
