      subroutine ervl16 ( n, m, stiff, smas, &
                          z, y, nr, iseed )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.2  date 13-05-2010 Fortran 90
!        version  2.1  date 17-03-1991 Z.Vlasek: amas renamed to smas, &
!                                      new parameter iseed, deletion of
!                                      intmas
!
!   copyright (c) 1991-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   purpose: to perform once again the Lanczos process
!            and transform the eigenvectors of the tridiagonal matrix
!            to eigenvectors of the original problem
!        adapted from subroutine lsvlan of J.v.Katz and H.A.v.d.Vorst
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemat
      use sepmodulematstr
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer n, m, nr, iseed
      double precision z (m,nr), y(n,nr), smas(*)
      type (rmatrix) :: stiff

!     iseed      i
!     m          i    the maximum number of columns of the process to be
!                     computed
!     n          i    the size of the matrix
!     nr         i
!     smas       i    Mass matrix
!     stiff      i    Stiffness matrix
!     y          i
!     z          i
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k
      double precision temp, alphaj, betaj, betamx, q0(n), q1(n), u(n), uw(n)

!     alphaj
!     betaj
!     betamx
!     i          Counting variable
!     j          Counting variable
!     k          Counting variable
!     q0         vector of length n
!     q1         the first column of q, a real vector, not assumed
!                to be of norm 1.
!     temp
!     u          vector of length n
!     uw         vector of length n
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision dnrm2

!     DAXPY          Overwrite double precision dy with double precision
!                    da*dx + dy
!     DCOPY          Copy double precision dx to double precision dy
!     DNRM2          Blas routine to compute the 2-norm of a vector
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERVL11         solve   sqrt(m)^(-1) s sqrt(m)^(-1)    a  =  b
!     PRANDOM        generates a random vector in parallel
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   process used: this subroutine implements a refined version of the
!   paige-style lanczos process.
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ervl16' )
!     --- initialization
!         step 1:  fill q1 and normalize it to unit length

      if ( iseed==0 ) then

!     --- default initial vector

        do i = 1, n
          q1(i) = i
        end do

      else

!     --- random initial vector, it must be the same as in ervl20 !!

        call prandom ( n, q1 )
        q1 = 2d0*q1-1d0

      end if

!     --- Clear array y

      y = 0d0

      temp = 1d0/dnrm2(n,q1,1)
      q1 = q1*temp

!     --- compute q2

      do j = 1,nr
         call daxpy ( n, z(1,j), q1, 1, y(1,j), 1)
      end do  ! j = 1,nr

      call ervl11(q1, u, n, stiff, smas, uw)
      alphaj = dot_product ( u, q1 )
      u = u - alphaj*q1

!     --- for use in an iterative lanczos process, since beta may be
!         very small, we reorthogonalize q2 with respect to q1 in order
!         to avoid loss of a lot of our orthogonality right off.

      temp = dot_product ( u, q1 )
      u = u - temp*q1

!     --- now normalize q2

      betaj = dnrm2(n,u,1)
      temp = 1d0/betaj
      q0 = q1
      call dcopy(n,q1,1,q0,1)
      q1 = u * temp

      do j = 1,nr
         call daxpy ( n, z(2,j), q1, 1, y(1,j), 1)
      end do  ! j = 1,nr

      betamx = betaj
      k = 2
80    if (k>m) goto 900

!     --- compute previous alpha and direction of next vector

      call ervl11(q1, u, n, stiff, smas, uw)
      call daxpy(n,-betaj,q0,1,u,1)
      alphaj = dot_product ( u, q1 )
      call daxpy(n,-alphaj,q1,1,u,1)

!     --- normalize next vector

      betaj = dnrm2(n,u,1)
      if (betaj<epsmac*betamx) then
         m=k
         goto 900
      end if
      temp = 1d0/betaj
      if (betaj>betamx) betamx = betaj
      call dcopy(n,q1,1,q0,1)
      q1 = u*temp
      if (k<m) then

         do j = 1,nr
            call daxpy ( n, z(k+1,j), q1, 1, y(1,j), 1)
         end do  ! j = 1,nr

      end if

      k = k + 1
      go to 80
900   call erclos ( 'ervl16' )
      end
