      subroutine prdecosm ( n, amat )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 08-03-2002 Remove work array from parameter list
!        version  1.0  date 19-07-1997
!
!   copyright (c) 1997-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute the L D L^T  decomposition of a real symmetric full matrix
!
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     real
!     symmetric
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer n
      double precision amat(n,n)

!     amat    i/o   in this array the matrix to be decomposed is stored at input
!                   At output the decomposition overwrites the input
!                   The docmposition is only stored in the lower triangle
!     n        i    Size of matrix amat
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k
      double precision rowsum, r(n)

!     i            Counting variable
!     j            Counting variable
!     k            Counting variable
!     r            Work array of length n to store intermediate variables
!     rowsum       Row norm of matrix
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision dasum

!     DASUM     Blas subroutine computing the sum of absolute values of vector
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERREAL    Put real in error message
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
!     ERRWAR    Print warnings
!     INSTOP    Stop the program
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     104   matrix has small pivots
!    1535   matrix is singular
!    1820   The matrix found in the solver is completely zero
! **********************************************************************
!
!                       PSEUDO CODE
!
!                      Method
!               T
!      The L D L   decomposition of the matrix A is based on the following
!      formulas:
!              n               T
!      a   =  sum   l    d    l
!       ij    k=1    ik   kk   kj
!      this implies:
!                        j-1
!      l   = ( a   -     sum  l    d    l   ) / d      ( j=1 (1) i-1 )
!       ij      ij       k=1   ik   kk   jk      jj
!                    i-1
!      d   =   a   - sum  l    d    l
!       ii      ii   k=1   ik   kk   ik
!      For the practical implementation we use the work array r to store
!      products l   d
!                ik  kk
!      Hence the final algorithm becomes:
!      For i := 1  (1)  n
!         For j := 1  (1) i-1
!                              j-1
!            l   = ( a   -     sum   r   l   ) / d
!             ij      ij       k=1    k   jk      jj
!            r   = l   d
!             j     ij  jj
!         end for j
!                       i-1
!         d   =   a   - sum  r  l
!          ii      ii   k=1   k  ik
!      end for i
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- compute row norm of part of the matrix in order to be able to check
!         the stopping criterium for the pivot process

      rowsum = 0d0
      do i = 1, n
         rowsum = max ( rowsum, dasum ( n, amat(1,i), 1 ) )
      end do
      if ( rowsum.le.0d0 ) then
         call eropen ( 'prdecosm' )
         call errsub ( 1820, 0, 0, 0 )
         call erclos ( 'prdecosm' )
         call instop
      end if
      rowsum = rowsum*epsmac

!     --- decomposition algorithm
!         Row loop

      do i = 1, n

!     --- Loop over columns

         do j = 1, i-1

            r(j) = amat(i,j)
            do k = 1, j-1
               r(j) = r(j) - r(k)*amat(j,k)
            end do
            amat(i,j) = r(j) / amat(j,j)

         end do

!        --- Diagonal element

         do k = 1, i-1
            amat(i,i) = amat(i,i) - r(k)*amat(i,k)
         end do

!        --- Check pivot element:

         if ( abs(amat(i,i)).le.0d0 ) then
            call eropen ( 'prdecosm' )
            call errint ( i, 1 )
            call errsub ( 1535, 1, 0, 0 )
            call erclos ( 'prdecosm' )
            call instop
         else if ( abs(amat(i,i)).lt.rowsum ) then
            call eropen ( 'prdecosm' )
            call errint ( i, 1 )
            call erreal ( amat(i,i), 1 )
            call erreal ( rowsum, 2 )
            call errwar ( 104, 1, 2, 0 )
            call erclos ( 'prdecosm' )
         end if

      end do

!     --- end of decomposition

      end
