      subroutine prfullgg ( amat, nrow )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 29-10-1999
!
!   copyright (c) 1999-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute the decomposition of a symmetric positive definite
!     full matrix
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     positive_definite
!     real
!     symmetric
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nrow
      double precision amat(nrow,nrow)

!     amat          i/o   in this array the parts of the matrix to be decomposed
!                         are stored
!     nrow           i    Number of rows of the matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k
      double precision rowsum, am, sum

!     am             Help parameter to store the diagonal of the matrix before
!                    computing the square root
!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     rowsum         Row norm of matrix
!     sum            Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision dasum

!     DASUM          Blas subroutine computing the sum of absolute values of
!                    vector
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     INSTOP         Stop the program
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     104   matrix has small pivots
!    1535   matrix is singular
!    1820   The matrix found in the solver is completely zero
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Only the lower part of the matrix amat = a_ij is used
!    The matrix is splitted into a Cholesky decomposition (G G^T)
!    From aij = sum g(ik) g^T(kj), where the sum is from 1 to n
!    and gij = 0 if j>i, we get that the sum must be taken from 1 to min(i,j)
!    is from 1 to j
!    So:
!    aij = sum (i=1 to j ) gik gjk
!   Hence
!    gij = ( aij - sum(k=1 to j-1) gik gjk ) / gjj  (j<i)
!    gii = sqrt ( aii - sum(k=1 to i-1) gik^2)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- compute row norm of part of the matrix in order to be able to check
!         the stopping criterium for the pivot process

      rowsum = 0d0
      do j = 1, nrow
         rowsum = max(rowsum,dasum(nrow,amat(1,j),1))
      end do
      if ( abs(rowsum).le.0d0 ) then
         call eropen ( 'prfullgg' )
         call errsub ( 1820, 0, 0, 0 )
         call erclos ( 'prfullgg' )
         call instop
      end if
      rowsum = rowsum*epsmac

!     --- decomposition

      do i = 1, nrow
         do j = 1, i-1
            sum = 0d0
            do k = 1, j-1
               sum = sum+amat(i,k)*amat(j,k)
            end do
            amat(i,j) = (amat(i,j)-sum)/amat(j,j)
         end do

!        --- g(ii)

         sum = 0d0
         do k = 1, i-1
            sum = sum+amat(i,k)**2
         end do
         am = amat(i,i)-sum
         if ( am.le.0d0 ) then
            call eropen ( 'prfullgg' )
            call errint ( i, 1 )
            call errsub ( 1535, 1, 0, 0 )
            call erclos ( 'prfullgg' )
            call instop
         else if ( am.lt.rowsum ) then
            call eropen ( 'prfullgg' )
            call errint ( i, 1 )
            call erreal ( am, 1 )
            call erreal ( rowsum, 2 )
            call errwar ( 104, 1, 2, 0 )
            call erclos ( 'prfullgg' )
         end if
         amat(i,i) = sqrt(am)

      end do

!     --- end of decomposition

      if ( ierror.ne.0 ) call instop
      end
