      subroutine prov00enthal ( ichois, irhsd, isol, &
                                eps, iwork, ireadenthal )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.5  date 11-12-2017 New storage of intmat
!        version  1.4  date 21-04-2015 New definition intmat
!        version  1.3  date 31-03-2015 Replace er0061/2 by inicopindarr1/2
!
!   copyright (c) 2004-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     solution of the system of linear equations amat * x = y
!     by the overrelaxation process in the special case of an enthalpy equation
!     The matrix amat must be stored using the structure of jmetod = 9
!     depending on the contents of array iwork the solution must satisfy
!     constraints of the shape  a(x)<= u(x)<=b(x)
!     This subroutie is called by overcs and overrl, in fact repaces them both
! **********************************************************************
!
!                       KEYWORDS
!
!     constraint
!     linear_solver
!     overrelaxation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulesolve
      use sepmodulematr
      use sepmodulekprob
      use sepmoduleintmat
      use sepmodulematstr
      use sepmodulekmesh
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer irhsd(*), isol(3), iwork(*), ichois, ireadenthal
      double precision eps

!     eps            i    |eps| is the required accuracy
!                         if eps>0, it concerns the absolute accuracy, eps<0
!                         means reative accuracy
!     ichois         i    choice parameter. Possibilities:
!                         ichois = ikeuze + 10*iprint with
!                         ikeuze:
!                         0:  the initial vector is filled by the subroutine
!                             its value is equal to zero, except for the
!                             boundary conditions
!                         1:  the vector is supposed to be filled by the user
!                         iprint:
!                         0:  no special print output
!                         1:  extra information of iteration process is printed
!     ireadenthal    i    Sequence number of input block for enthalpy
!     irhsd          i    standard sepran array containing information
!                         concerning the right-hand-side vector
!     isol          i/o   standard sepran array containing information
!                         concerning the solution vector usol
!                         depending of the value of ichois usol must be filled
!                         with an initial estimate or not
!                         boundary conditions must always have been filled in
!                         usol
!     iwork          i    user array of length 4 containing the following
!                         information:
!                         iwork(2) = 0:  no constraint
!                         iwork(2) = 1:  u(x)>=a(x)
!                         iwork(2) = 2:  u(x)<=b(x)
!                         iwork(2) = 3:  a(x)<=u(x)<=b(x)
!                         iwork(3):  information of a(x):
!                                  >0:   a(x) is constant, with value in work(1)
!                                  =0:   a(x) = 0
!                                  <0:   a(x) is variable; a function subroutine
!                                        funccf is needed. Ichois in funccf is
!                                        equal to - iwork(2)
!                         iwork(4):  information of b(x):
!                                  >0:   b(x) is constant, with value in work(2)
!                                  =0:   b(x) = 0
!                                  <0:   b(x) is variable; a function subroutine
!                                        funccf is needed. Ichois in funccf is
!                                        equal to - iwork(3)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer niinpent, nrinpent
      parameter ( niinpent=10, nrinpent=10 )
      integer ikeuze, n, number, leninp, lenrnp, &
              jcart, iinenthalpy(niinpent)
      double precision rinenthalpy(nrinpent), dum(1)
      double precision, pointer :: mass(:)
      double precision, pointer :: temp(:), enthalpylc(:)
      double precision, pointer :: tempcp(:), enthalpycp(:)

!     dum            General double precision dummy variable
!     enthalpycp     renumbered copy of enthalpy
!     enthalpylc     renumbered copy of enthalpy
!     iinenthalpy    Integer input array with respect to enthalpy
!                    computations
!                    For a definition of this array see subroutine
!                    readenthalpy (array iinput)
!     ikeuze
!     jcart          Type of co-ordinate system
!                    1:  Cartesian co-ordinates
!                    2:  Axi-symmetric co-ordinates
!                    3:  Polar co-ordinates
!     leninp         Computed length of integer input array
!     lenrnp         Computed length of real input array
!     mass           diagonal mass-matrix
!     n              number of degrees of freedom
!     niinpent       Declared length of array iinenthalpy
!     nrinpent       Declared length of array rinenthalpy
!     number         The number-th unknown in each nodal point of the array
!                    is considered. If the array is complex number=2*i-1
!                    corresponds to the real part of array i and
!                    number=2*i to the imaginary part of array i
!     rinenthalpy    Real input array with respect to enthalpy
!                    computations
!                    For a definition of this array see subroutine
!                    readenthalpy (array iinput)
!     temp           temperature vector
!     tempcp         renumbered copy of temp
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     IINPUTSEQ      Read contents of input block with sequence number iseqnr
!                    corresponding to main keyword name
!     INICOPINDARR1  Copy old indexed arrayold into new array arraynew
!     INICOPINDARR2  Copy old arrayold into new indexed array arraynew
!     PROV00CHECK    Test the input for subroutine prov00 and set some
!                    parameters
!     PROVENTHALPY   actual overrelaxation process for the enthalpy method
!     PRSPRR         Fill part of USOL corresponding to "right" periodical
!                    boundary conditions ( real case )
!     SEPRENUMSOLBCK Renumber solution vector back to original form
!                    Real case
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     491   array isol does not correspond to array kprob
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prov00enthal' )

!     --- tests and initializations

      if ( alam<=0d0 .or. alam>=1d0 ) alam = 0.5d0
      call prov00check ( isol, irhsd, ichois, ikeuze, eps, iwork, &
                         jcart, n, number )
      if ( ierror/=0 ) go to 1000

!     --- Special statements for enthalpy

      if ( ks(isol(2))%nusol/=nusol ) then
         call errint ( ks(isol(2))%nusol, 1 )
         call errint ( nusol, 2 )
         call errsub ( 491, 2, 0, 0 )
      end if


!     --- Read input for enthalpy problem and store in arrays
!         iinenthalpy, rinenthalpy

      leninp = niinpent
      lenrnp = nrinpent
      call iinputseq ( 26, ireadenthal, iinenthalpy, &
                       rinenthalpy, leninp, lenrnp, &
                       'ENTHALPY_INTEGRATION' )
      if ( ierror/=0 ) go to 1000

      iseqmt = stiffmat(1)
      S_ii => realmt(iseqmt)%S_ii
      iseqmt = massmat
      mass => realmt(iseqmt)%S_ii

      rhsd => ks(irhsd(1))%sol
      enthalpylc => ks(isol(1))%sol
      enthalpy => enthalpylc
      temp => ks(isol(2))%sol

      if ( indprh>0 ) then

!     --- In case of renumbering it is necessary to renumber rhsd
!         and usol

         allocate ( tempcp(nusol), rhsdcp(nusol), enthalpycp(nusol), &
                    stat = error )
         if ( error/=0 ) call eralloc ( error, 2*nusol, 'tempcp' )
         if ( ierror/=0 ) go to 1000
         call inicopindarr1 ( nrusol, rhsdcp, rhsd, kprobhinv )
         call inicopindarr1 ( nrusol, tempcp, temp, kprobhinv )
         call inicopindarr1 ( nrusol, enthalpycp, enthalpy, kprobhinv )
         enthalpy => enthalpycp
         rhsd => rhsdcp

      else

!     --- No renumbering, copy pointers

         tempcp => temp
         enthalpycp => enthalpy
         rhsdcp => rhsd

      end if
      if ( ierror/=0 ) go to 1000

      if ( ikeuze==0 ) enthalpy(1:n) = 0d0

!     --- Perform the actual non-linear overrelaxation process

      call proventhalpy ( S_ii, irowsii, icolsii, intdia, rhsd, enthalpycp, &
                          n, eps, ndim, iprint, tempcp, &
                          mass, rinenthalpy )
      if ( ierror/=0 ) go to 1000

      if ( npboun>0 ) then

!     --- npboun>0; compute effect of periodical boundary conditions

        call prsprr ( enthalpycp )
        call prsprr ( tempcp )

      end if
      if ( ierror/=0 ) go to 1000
      ks(isol(1))%jcart = jcart
      ks(isol(2))%jcart = jcart

      if ( npltra>0 ) then

!     --- npltra     local transformations;  back transformation

         if ( nusol>nrusol .and. indprh>0 ) then

!        --- indprh>0, fill boundary conditions in usol

            call inicopindarr1 ( nusol-nrusol-npboun, &
                                 tempcp(1+nrusol+npboun), &
                                 temp, kprobhinv(1+nrusol+npboun) )
            call inicopindarr1 ( nusol-nrusol-npboun, &
                                 enthalpycp(1+nrusol+npboun), &
                                 enthalpy, kprobhinv(1+nrusol+npboun) )
            temp => tempcp
            enthalpy => enthalpycp

         end if
         if ( indprh==0 ) enthalpy => enthalpylc

         indprhlc = indprh
         indprh = 0
         call seprenumsolbck ( temp, 1, dum )
         call seprenumsolbck ( enthalpy, 1, dum )
         indprh = indprhlc
      end if
      if ( ierror/=0 ) go to 1000

      if ( indprh>0 ) then

!     --- indprh>0, reorder usol

         if ( npltra==0 ) then
            nrusol = nusol-nbound
         else
            nrusol = nusol
         end if
         enthalpy => enthalpylc
         call inicopindarr2 ( nrusol, temp, tempcp, kprobhinv )
         call inicopindarr2 ( nrusol, enthalpy, enthalpycp, kprobhinv )

         deallocate ( tempcp, rhsdcp, enthalpycp, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'tempcp' )

      end if  ! ( indprh>0 )

1000  call erclos ( 'prov00enthal' )

      end subroutine prov00enthal
