      subroutine sepaptmatvec ( A, x, y  )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-07-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute y = A P^T x as part of projection
!
! **********************************************************************
!
!                       KEYWORDS
!     conjugate_gradients
!     projection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleproj
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix), intent(in) :: A
      double precision, intent(in) :: x(A%neq)
      double precision, intent(out) :: y(A%nproj)

!     A             i/o   Matrix A
!     x              i    Vector to be multiplied
!     y              o    result of multiplication
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j, idegfd, numneigh, iblock, icontext, nsend
      double precision :: h, work(A%nproj), usend(A%nproj)

!     h              Help variable to store a constant temporarily
!     i              Counting variable
!     iblock         Block sequence number
!     icontext       Tag for sending and receiving, is used as check
!     idegfd         Degree of freedom sequence number
!     j              Counting variable
!     nsend          Help variable for receiving
!     numneigh       number of neighbors of a point
!     usend          Help array to be used to send reals to other processor
!     work           Work array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     DAXPY          Overwrite double precision dy with double precision
!                    da*dx + dy
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIRECEIVEMPI  Receive information from other node (mpi)
!     INISENDMPI     Send information to other node (mpi)
!     PRINRL         Print 1d real vector
!     SEPSOLVSM      Solve system of symmetric equations (full matrix), where
!                    the matrix has been decomposed by PRDECOSM
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepaptmatvec' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepaptmatvec'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( parallel ) then

!     --- Parallel case

         work = 0d0
         do j = 1, A%neq
            work(inodenr) = work(inodenr) + &
                            innercontr(j)*(A%projAU(j)*x(j))
         end do  ! j = 1, A%neq

!           --- Now the contributions of the neighboring nodes

         numneigh = blockunkinfo(1)
         do i = 1, numneigh

            iblock =  blockunkinfo(2*i)

            do j = A%projrowsa(i)+1, A%projrowsa(i+1)

               idegfd = projcolsa(j)
               work(iblock) = work(iblock) + &
                         innercontr(idegfd)*(A%projAU(A%neq+j)*x(idegfd))

            end do

         end do

!        --- send all contributions to the first node

         icontext = 13

         if ( inodenr==1 ) then

!        --- Receive next contributions from the other nodes and sum up

            do j = 2, A%nproj
               nsend = A%nproj
               call inireceivempi ( usend, nsend, SPDOUBLE, iblock, &
                                    icontext )
               do i = 1, A%nproj
                  work(i) = work(i) + usend(i)
               end do
            end do

         else

!        --- Send contributions to the first node

            call inisendmpi ( work, A%nproj, SPDOUBLE, 1, icontext )

         end if

         if ( debug ) call prinrl ( work, A%nproj, 'work/1' )

!        --- On the first node we multiply by the inverse of E and store
!            in the same positions: work = E^-1 work. Thereafter
!            this vector is sent back to all processors.

         icontext = 14

         if ( inodenr==1 ) then

            call sepsolvsm ( A%nproj, A%projE, work, work )

!           --- Send contributions to the other nodes

            do j = 2, A%nproj
               call inisendmpi ( work, A%nproj, SPDOUBLE, j, &
                                 icontext )
            end do

         else

!        --- Receive the solution vector from node 1

            nsend = A%neq
            call inireceivempi ( work, nsend, SPDOUBLE, iblock, &
                                 icontext )
         end if

         if ( debug ) call prinrl ( work, A%nproj, 'work/2' )

!        --- Compute x = P^T x: x = x - U work
!            First the contribution of the own vector

         y = x
         call daxpy ( A%neq, -work(inodenr), A%projU, 1, y, 1 )

!        --- Now the contributions of the neighboring nodes

         do i = 1, numneigh

            iblock =  blockunkinfo(2*i)

            do j = A%projrows(i)+1, A%projrows(i+1)

               idegfd = A%projcols(j)
               y(idegfd) = y(idegfd)-work(iblock)*A%projU(A%neq+j)

            end do

         end do

      else

!     --- Serial case

         do i = 1, A%nproj

            h = 0d0

            do j = A%projrows(i)+1, A%projrows(i+1)
               idegfd = A%projcols(j)
               h = h + A%projAU(j)*x(idegfd)
            end do

            y(i) = h

         end do

      end if  ! ( parallel )

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( y, A%neq, 'y' )
         write(irefwr,*) 'End sepaptmatvec'

      end if  ! ( debug )
      call erclos ( 'sepaptmatvec' )

      end subroutine sepaptmatvec
