      subroutine sepcbicgstabl ( A, x, b )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-07-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Solve system of equations by the bi-cgstab(l) method
!     complex case
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     iterative
!     linear_solver
!     preconditioning
!     complex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecmat
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (cmatrix), intent(inout) :: A
      double complex, intent(inout) :: x(A%neq), b(A%neq)

!     A              i    Matrix A
!     b              i    Right-hand side
!     x              o    Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ldim, nmax, countres, info, &
                 priter(10), ifiter, j, k, i, imin, ireturn, iter, n
      double complex :: alpha, omega, sigma, rho, rhoold, &
                        beta, kappa0, kappal, varrho, hatgamma
      double precision :: rnorm, rmin, rferr, bnorm, residu(10)
      integer, allocatable :: iwork(:)
      double complex, allocatable :: rbar(:), res(:,:), pvec(:,:), &
                                     vecxp(:), work(:), xmin(:), &
                                     zy0(:), zyl(:), zy(:), &
                                     zy1(:), zmat(:,:), zzmat(:,:)
      logical :: restart, repeat1, finished

!     alpha          Factor alpha in the CG-process ( (r,r) / (p,Ap) )
!     beta           Factor beta in the CG-process ( (r ,r ) / (r   ,r   ) )
!                                                      i  i      i-1  i-1
!     bnorm          Norm of the right-hand side (b,b)
!     countres       Counter for number of restarts
!                    they are not
!     finished       Indicates if the loop has finished (true) or not
!                    (false)
!     hatgamma       gamma^ in method
!     i              Counting variable
!     ifiter         Starting value of parameter iter
!     imin           Iteration with minimal norm
!     info           Return value of lapack subroutine
!     ireturn        Return code:
!                    0: continue process
!                    1: restart process
!                    2: end of process
!     iter           Iteration sequence number
!     iwork          integer work array
!     j              Counting variable
!     k              Counting variable
!     kappa0         Parameter Kappa_0 in method
!     kappal         Parameter Kappa_l in method
!     ldim           Parameter l in bicgstab(l)
!     n              size of matrix and vectors
!     nmax           maximum number of iterations that may be performed by
!                    the CG-process.
!     omega          parameter in method
!     priter         Output array to print residuals
!     pvec           Set of arrays in method
!     rbar           Shadow array
!     repeat1         Logical variable indicating if the accuracy check must be
!                    carried out twice (true) or once (false), because the
!                    cnorm of the solution could not be computed
!     res            set of arrays in method, the first one is the residual
!     residu         Output array to print residuals
!     restart        If true the recomputation of rho has not been started
!     rferr          Maximal accuracy possible for this matrix, due to round-off
!     rho            Inner product ( r0bar, r )
!     rhoold         Inner product ( r0bar, r ) from the previous iteration
!     rmin           minimal cnorm of residual
!     rnorm          cnorm of residual
!     sigma          parameter sigmain method
!     varrho         Parameter in method
!     vecxp          Work array to store the initial value of x
!     work           Work array
!     xmin           Work array of length n to store the vector x with
!                    the smallest residual norm
!     zmat           Matrix R R^t
!     zy             work array for Householder process
!     zy0            work array for Householder process
!     zy1            work array for Householder process
!     zyl            work array for Householder process
!     zzmat          Copy of zamt. Is used for inversion
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     PRINCOMPL      Print 1d complex vector
!     PRINCOMPL1     Print 2d complex vector
!     PRINTITER      Fill arrays priter and residu
!     SEPCCGESTEPS   Print first line of iteration output
!                    Compute first estimate of accuracy  eps
!     SEPCCGSTART    Fill starting value in array x and compute initial residual
!     SEPCCHECKACCUR Check on accuracy reached in step iter-1 of bi-cgstab
!     SEPCMAXACC     Compute maximum accuracy possible in iterative solver
!     ZGETRF         computes an LU factorization of a general M-by-N complex
!                    matrix A
!                    Lapack subroutine
!     ZGETRS         solves a system of linear equations * A * X = B or
!                    A' * X = B * with a general N-by-N complex matrix A using
!                    the LU factorization computed by zgetrf
!                    Lapack subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     676   The CG-process has terminated because the number of iterations
!           exceeds the maximum value
!    1400   Bi-CGSTAB process has terminated because sigma = 0
!    2988   Small system singular
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcbicgstabl' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcbicgstabl'
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get parameters

      n = A%neq

      bnorm = cnorm(b,n)
      if ( bnorm<=0d0 ) then

!     --- Right-hand side is equal to zero

         x = 0d0
         go to 1000  ! rhs is equal to 0

      end if  ! ( bnorm<=0d0 )

      ldim = A%ninner
      allocate(rbar(n), res(n,ldim+1), pvec(n,ldim+1), vecxp(n), &
               work(n), zy0(ldim+1), zyl(ldim+1), iwork(ldim), &
               zy(ldim+1), zy1(ldim+1), zmat(ldim+1,ldim+1), &
               zzmat(ldim+1,ldim+1), xmin(n), stat = error)
      if ( error/=0 ) call eralloc ( error, ldim+10*n, 'rbar' )
      if ( ierror/=0 ) go to 1000

!     --- Set constants

      nmax = A%nmax       ! maximum number of iterations
      if ( nmax==0 ) nmax = 10*n

      restart = .true.

!     --- Compute maximum accuracy possible

      call sepcmaxacc ( A, b, work, bnorm, rferr )

!     --- Print heading if iprint>0

      if ( A%iprint>0 ) write ( irefwr,100 ) ldim
100   format(//' Output for the Bi-CGSTAB(',i2,' ) method', &
               ' (complex non-symmetric matrix)'//)

      ifiter = 1
      countres = 0

!     --- compute start vector and residual

      call sepccgstart ( A, x, b, res, work )

!     --- Set residual r = b, it will be updated when x # 0, label 110
!         is used when the method is restarted

110   if ( ifiter>1 ) then

!     --- Not the first iteration, compute residual

         res(:,1) = cresidual ( A, x, b )
      else

!     --- First iteration, store x in xmin

         xmin = x

      end if  ! ( ifiter>1 )

      if ( debug ) call princompl ( res(1,1), n, 'residual' )
      countres = countres+1

      vecxp = x
      x = 0d0

!     --- Copy r into rbar

      rbar = res(:,1)
      pvec(:,1) = 0d0

!     --- Set constants
!         Norm of the initial residual r

      rnorm = cnorm ( res(1,1), n )
      rmin  = rnorm
      imin  = ifiter
      if ( debug ) write(irefwr,2) 'rnorm', rnorm

!     --- Print first line of iteration output
!         Compute first estimate of accuracy  eps

      call sepccgesteps ( A, b, work, rnorm, rferr, repeat1, ifiter )

      if ( ifiter==1 .and. rnorm<A%eps ) then

!     --- first iteration, but already accuracy reached
!         process finished

         nmax = 1
         A%numiter = 1
         if ( A%iprint>=1 ) then

!        --- iprint>=1  Print information

            write ( irefwr,150 ) nmax, rnorm

         end if  ! ( A%iprint>=1 )
         go to 1000

      end if  ! ( ifiter==1 .and. rnorm<A%eps )

      alpha = 0d0
      omega = 1d0
      sigma = 1d0
      rhoold = 1d0

      finished = .false.

!     --- While loop check on accuracy ( nmax+1 is used because
!         the residual of the previous iteration is always checked )

      iter = ifiter
      do while ( .not. finished .and. iter<=nmax+1 )

!     --- Check on accuracy reached in iter-1

         call sepccheckaccur ( A, x, b, res, iter, ifiter, &
                               rnorm, repeat1, rferr, &
                               imin, ireturn, nmax, rmin, priter, &
                               residu, xmin, work, vecxp )
         if ( ireturn==1 ) then
            go to 110  ! restart method
         else if ( ireturn==2 ) then
            finished = .true.
            exit  ! method has finished
         end if  ! ( ireturn==1 )

         if ( iter>nmax ) then
            finished = .true.
            exit  ! do not start (nmax+1)-th iteration
         end if  ! ( iter>nmax )

!        --- First the BiCGstab part

         rhoold = -omega*rhoold
         do k = 1, ldim

!        --- Compute rho = ( rbar, r )

            rho = dot_product ( rbar, res(:,k) )
            if ( debug ) write(irefwr,2) 'rho', rho

            if ( iter>1 .and. restart .and. &
                 abs(rhoold)*epsmac*1d2>abs(rho) ) then

!           --- rho too small, restart the process

               ifiter = iter

               restart = .false.
               x = x + vecxp
               go to 110

            end if  ! ( iter>1 .and. restart .and. .. )

            beta = alpha * rho / rhoold
            rhoold = rho
            do j = 1, k
               pvec(:,j) = res(:,j) - beta*pvec(:,j)
            end do

            if ( debug ) then
               write(irefwr,2) 'beta', beta
               call princompl1 ( pvec, k, n, 'p' )
            end if  ! ( debug )

!           --- p(k+1) = A p(k) (Preconditioned)

            work = pvec(:,k)/A

            if ( A%eistat ) then
               pvec(:,k+1) = pvec(:,k) + A%S * work
               pvec(:,k+1) = A/pvec(:,k+1)
               pvec(:,k+1) = pvec(:,k+1)+work
            else
               pvec(:,k+1) = A*work
            end if  ! ( A%eistat )
            iter = iter+1
            if ( debug ) call princompl ( pvec(1,k+1), n, 'v' )

!           --- Compute sigma = ( r0bar, v ),  alpha = rho / sigma

            sigma = dot_product ( rbar, pvec(:,k+1) )
            if ( abs(sigma)<=0d0 ) then

!           --- Sigma = 0, Process can not be continued

               if ( countres<=10 ) then

!              --- countres<=10 restart the process

                  ifiter = iter
                  go to 110

               else

!              --- countres>10 terminate

                  call errsub ( 1400, 0, 0, 0 )

                  A%numiter = iter
                  A%itererr = 2

                  go to 1000

               end if  ! ( countres>=10 )

            end if  ! ( abs(sigma)<=0d0 )

            alpha = rho / sigma
            if ( debug ) write(irefwr,2) 'alpha, sigma', alpha, sigma

            x = x + alpha*pvec(:,1)
            if ( debug ) call princompl ( x(1), n, 'x' )
            do j = 1, k
               res(:,j) = res(:,j) - alpha * pvec(:,j+1)
            end do
            if ( debug ) call princompl1 ( res, k, n, 'res' )

            work = res(:,k)/A

            if ( A%eistat ) then
               res(:,k+1) = res(:,k) + A%S * work
               res(:,k+1) = A/res(:,k+1)
               res(:,k+1) = res(:,k+1)+work
            else
               res(:,k+1) = A*work
            end if  ! ( A%eistat )

            if ( debug ) call princompl ( res(1,1+k), n, 't' )
            iter = iter+1
            rnorm  = cnorm ( res(1,1), n )

         end do  ! k = 1, ldim

!        --- Next the convex polynomial part
!            First store R'R in zmat

         do i = 1, ldim+1
            do j = 1, i
               zmat(i,j) = dot_product ( res(:,i), res(:,j) )
               zmat(j,i) = zmat(i,j)
            end do  ! j = 1, i
         end do  ! i = 1, ldim+1
         zzmat = zmat     ! Copy of zmat used for inversion

!        --- Store LU factorization of zzmat(2,2) ... zmat(l,l)
!            Pivots are stored in iwork

         call zgetrf (ldim-1, ldim-1, zzmat(2,2), ldim+1, iwork, info)
         if ( info/=0 ) then
            call errint ( info, 1 )
            call errsub ( 2988, 1, 0, 0 )
            go to 1000
         end if  ! ( info/=0 )

!        --- tilde r0 and tilde rl (small vectors, stored in zy0 and zyl)

         zy0(1) = -1d0
         zy0(2:ldim) = zmat(2:ldim,1)
         zy0(ldim+1) = 0d0

!        --- Solve zy0(2:l) = zmat(2:l,2:l) using the LU decomposition

         call zgetrs ( 'n', ldim-1, 1, zzmat(2,2), ldim+1, iwork, &
                       zy0(2), ldim+1, info )
         if ( info/=0 ) then
            call errint ( info, 1 )
            call errsub ( 2988, 1, 0, 0 )
            go to 1000
         end if  ! ( info/=0 )

         zyl(1) = 0d0
         zyl(2:ldim) = zmat(2:ldim,1+ldim)
         zyl(ldim+1) = -1d0

!        --- Solve zyl(2:l) = zmat(2:l,2:l) using the LU decomposition

         call zgetrs ( 'n', ldim-1, 1, zzmat(2,2), ldim+1, iwork, &
                       zyl(2), ldim+1, info )
         if ( info/=0 ) then
            call errint ( info, 1 )
            call errsub ( 2988, 1, 0, 0 )
            go to 1000
         end if  ! ( info/=0 )

!        --- Convex combination
!            zy1 = A zy0
!            kappa_0^2 = (zy0,zy1)

         do i = 1, ldim+1
            zy1(i) = dot_product ( zmat(:,i), zy0 )
         end do  ! i = 1, n
         kappa0 = sqrt ( dot_product ( zy0, zy1 ) )

!        --- zy = A zyl
!            kappa_l^2 = (zyl,zy)

         do i = 1, ldim+1
            zy(i) = dot_product ( zmat(:,i), zyl )
         end do  ! i = 1, ldim+1
         kappal = sqrt ( dot_product ( zyl, zy ) )

!        --- varrho = (zyl,zy1)/(kappa0*kappal)

         varrho = dot_product ( zyl, zy1 )/(kappa0*kappal)
         hatgamma =  varrho/abs(varrho)*max(abs(varrho),0.7d0) &
              * (kappa0/kappal)

         zy0 = zy0 - hatgamma*zyl

!        --- Update

         omega = zy0(ldim+1)
         if ( debug ) write(irefwr,2) 'omega', omega

!        --- pvec(1:l) = pvec(1:l)-(pvec(2:l+1),zy0(2:l+1))

         do i = 1, ldim
            pvec(1:n,1) = pvec(1:n,1) - zy0(1+i)*pvec(1:n,1+i)
            x(1:n) = x(1:n) + zy0(1+i)*res(1:n,i)
            res(1:n,1) = res(1:n,1) - zy0(1+i)*res(1:n,i+1)
         end do
         if ( debug ) then
            call princompl ( x(1), n, 'x' )
            call princompl ( res(1,1), n, 'r' )
         end if  ! ( debug )

!        --- Compute cnorm of residual

         rnorm = cnorm ( res(1,1), n )

         if ( rnorm<rmin ) then
            xmin = x
            rmin = rnorm
            imin = iter+1
         end if

         if ( A%iprint>1 ) &
            call printiter ( A%iprint, iter, rnorm, priter, residu, &
                             ldim*2 )

      end do  ! while ( .not. finished .and. iter<=nmax+1 )

      if ( .not. finished ) then

!     --- If the subroutine reaches this stage, too much  iterations have
!         been carried out

         if ( A%iprint==2 .and. iter>20 ) then

!        --- iprint = 2 Print final residuals

            do j = max(1,33-iter), 10
               write ( irefwr, 450 ) priter(j), residu(j)
450            format( i10, ss,1pe15.2e2 )
            end do

         end if

         if ( A%iprint>=1 ) then

!        --- iprint>=1  Print information

            write ( irefwr,150 ) nmax, rmin
            write ( irefwr,160 ) imin-1
150         format(' The number of executed iterations in the', &
                   ' Bi-CGSTAB process is', i5/ &
                   ' The 2-norm of the residual is:', ss,1pe12.2e2 )
160         format (' The minimal residual is obtained at', &
                    ' iteration', i5 )

         end if

         call errint ( nmax, 1 )

         A%numiter=nmax
         A%itererr=1

         if ( A%aterror==1 ) then

            call errwar ( 676, 1, 0, 0 )

            if ( A%aterror>=1 .and. imin>1 ) then

               x = xmin

            end if

         else

            call errsub ( 676, 1, 0, 0 )

         end if

      end if  ! ( .not. finished )

      deallocate(rbar, res, pvec, vecxp, work, zy0, zyl, &
                 zy, zy1, zmat, zzmat, xmin, iwork, stat = error)
      if ( error/=0 ) call erdealloc ( error, 'rbar' )
      A%numiter = iter

1000  call erclos ( 'sepcbicgstabl' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepcbicgstabl'

      end if  ! ( debug )

      end subroutine sepcbicgstabl
