      subroutine sepccgnorms ( x, y, A, bnorm, bsnorm, xnorm, work )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 08-04-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute some norms necessary for the termination criterion of
!     the cg process
!     complex case
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     iterative
!     linear_solver
!     preconditioning
!     complex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecmat
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (cmatrix), intent(in) :: A
      double complex, intent(in) :: y(A%neq)
      double complex, intent(out) :: work(A%neq), x(A%neq)
      double precision, intent(out) :: bnorm, bsnorm, xnorm

!     A              i    Matrix A
!     bnorm          o    Norm of the right-hand side (b,b)
!     bsnorm         o    Preconditioned cnorm of the right-hand side
!                         (M^(-1)b,M^(-1)b)
!     work           o    work vector to store intermediate result
!     x              o    Solution vector
!     xnorm          o    Norm of the estimated solution
!     y              i    Right-hand side
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

!                    they are not
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRWAR         Print warnings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2168   Right-hand side equal to 0
!    2445   Norm of right-hand side is small
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepccgnorms' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepccgnorms'
         write(irefwr,*) 'eistat, precon ', A%eistat, A%precon
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( A%eistat ) then

         work = A/y
         bnorm = cnorm(work,A%neq)**2

      else

         bnorm = cnorm(y,A%neq)**2

      end if

      if ( bnorm<=0d0 ) then

!     --- bnorm = 0, hence y = 0

         call errwar ( 2168, 0, 0, 0 )
         x = 0d0
         go to 1000

      else if ( bnorm< 1d-24 ) then

!     --- Norm of right-hand side<10^-12, produce a warning

         call erreal ( sqrt(bnorm), 1 )
         call errwar ( 2445, 0, 1, 0 )

      end if

      bsnorm = bnorm
      if ( debug ) then
         write(irefwr,2) 'bnorm', bnorm
         write(irefwr,*) 'eistat, precon ', A%eistat, A%precon
         write(irefwr,1) 'mod(inpsol(11),10)', A%iaccuracy
      end if  ! ( debug )

      if ( A%precon .and. A%iaccuracy==0 ) then

!     --- precon = true,  preconditioned case (relative error with respect to x)
!         work = M^(-1) * y:  bsnorm = ||work||

         work = y/A
         bsnorm = cnorm(work,A%neq)**2

      end if

      xnorm  = bsnorm

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,2) 'xnorm', xnorm
         write(irefwr,*) 'End sepccgnorms'

      end if  ! ( debug )
      call erclos ( 'sepccgnorms' )

      end subroutine sepccgnorms
