      subroutine sepcdivmat ( A, x, y, n )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-06-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute y = A^(-1) x, where A is a preconditioning matrix
!     a complex symmetric compact storage is used, hence only D and L
!     The preconditioning matix has been created before
!     In other words a part of the ILU is executed (The U part)
!     This routine is meant for eistenstat preconditioning
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     iterative
!     linear_solver
!     preconditioning
!     complex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecmat   ! definition of matrices
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: n
      type (cmatrix), intent(in) :: A
      double complex, intent(in) :: x(n)
      double complex, intent(out) :: y(n)

!     A              i    Matrix A
!     n              i    size of x
!     x              i    Solution vector
!     y              o    Result  y = A^(-1) x
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iprec

!                    they are not
!     iprec          type of preconditioning
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPCSCMATDIVL  Compute y = A^(-1) x, where A is a preconditioning matrix
!                    a symmetric compact storage is used, hence only D and L
!                    Eisenstat preconditioning
!     SEPCSCMATDIVEL Compute y = A^(-1) x, where A is a preconditioning matrix
!                    a symmetric compact storage is used, hence only D and L
!                    Eisenstat preconditioning
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2684   Value of iprec not implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcdivmat' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcdivmat'
         write(irefwr,1) 'iprec', A%type_prec
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iprec = A%type_prec

      if ( iprec==2 .or. iprec==4 .or. iprec==7 ) then

!     --- Eisenstat preconditioning, only L y = x

         if ( A%symmetric ) then

!        --- symmetric compact matrix

            call sepcscmatdivL ( A, x, y )

         else

!        --- row-compact matrix

            call sepcscmatdiveL ( A, x, y )

         end if  ! ( A%symmetric )

      else

!     --- Not available

         call errint ( iprec, 1 )
         call errsub ( 2684, 1, 0, 0 )

      end if  ! ( iprec<=1 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepcdivmat'

      end if  ! ( debug )
      call erclos ( 'sepcdivmat' )

      end subroutine sepcdivmat
