      subroutine sepcg ( A, x, b )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 02-04-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Apply the preconditioned method of conjugate gradients to solve the
!     linear system of equations A x = y     ( A x = b )
!     where the matrix A and the preconditioning matrix M should be symmetric
!     and positive definite
!     This subroutine contains the following algorithms:
!     -  Standard conjugate gradient method
!     -  Preconditioned conjugate gradient method
!     -  Eisenstat implementation of the preconditioned conjugate gradient
!        method
!     Both in the case of the preconditioned conjugate gradients method, &
!     as in the case of the Eisenstat implementation, the matrix and right-hand
!     side are supposed to be premultiplied by the inverse of the diagonal D
!     of the preconditioning matrix in the following way:  ( A x = b )
!               -1        -1
!          A = D   A     D
!                   orig
!               -1
!          b = D   b
!                   orig
!     If the initial estimate x is given it must also be premultiplied by D:
!          x = D  x
!                  orig
!                                                        -1
!     At output the solution x must be premultiplied by D   in order to get
!     the solution of the original problem:
!                  -1
!         x     = D   x
!          orig
!     In the case of Eisenstat the matrix A is supposed to contain the
!     matrix K = D  - 2I in the first n positions, where D  is the diagonal
!                 A                                       A
!     of A after the premultiplication
!     Original subroutine: Rik Kaasschieter and Henk van der Vorst
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     iterative
!     linear_solver
!     preconditioning
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix), intent(inout) :: A
      double precision, intent(inout) :: x(A%neq), b(A%neq)

!     A             i/o   Matrix A
!     b             i/o   Right-hand side
!     x             i/o   Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: n, isqrn, nmax, iter
      double precision :: bnorm, rho, rhotest, r0norm, lambda, sqrn, &
                          rferr, bsnorm, xnorm, lamminsav, &
                          sigma, beta, alpha, rhosta
      double precision, allocatable :: eigvl(:)
      double precision, allocatable :: res(:), p(:), work(:), pbar(:)
      logical :: repeat1, finished, extradebug

!     alpha          Factor alpha in the CG-process ( (r,r) / (p,Ap) )
!     beta           Factor beta in the CG-process ( (r_i,r_i) / (r_i-1,r_i-1) )
!     bnorm          Norm of the right-hand side (b,b)
!     bsnorm         Preconditioned norm of the right-hand side
!                    (M^(-1)b,M^(-1)b)
!     eigvl          Real 10 x 2 array to store the factors alpha and beta
!                    of the first 10 iterations in eigvl(i,1)
!                    resp. eigvl(i,2)
!     extradebug     If true extra debugging statements are carried out
!     finished       if true the iteration has finished
!     isqrn          Nearest integer to sqrt(n)
!     iter           Loop variable
!     lambda         Approximation of smallest eigenvalue
!     lamminsav      Help parameter to store the value of alammin
!     n              number of free degrees of freedom in cpu
!                    because unknowns on interfaces are summed up.
!     nmax           maximum number of iterations
!     p              Work array of length n to store the direction vector
!     pbar           Work array of length n to store the vector p^_. This
!                    vector is only used for the Eisenstat preconditioning
!     r0norm         norm of start residual
!     repeat1        Logical variable indicating if the accuracy check must be
!                    carried out twice (true) or once (false), because the
!                    norm of the solution could not be computed
!     res            Work array of length n to store the residual vector
!     rferr          Maximal accuracy possible for this matrix, due to round-off
!     rho            dot product between M^(-1) r and r at old level, i.e.
!                    M^(-1) norm of residual
!     rhosta         rho at new level
!     rhotest        norm of residual used to terminate CG
!     sigma          Factor sigma in the CG-process (dot product of p and Ap)
!     sqrn           Help variable to compute sqrt(n)
!     work           Work array of length n to store the help vectors
!     xnorm          Norm of the estimated solution
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRSUB         Error messages
!     ERTRACE        Prints trace back of subroutines
!     PRINRL         Print 1d real vector
!     SEPCGACCUR     rhotest is smaller than maximal accuracy possible
!                    It is useless to continue the process, print
!     SEPCGCHECKEPS  Check if eps must be recomputed
!     SEPCGEPS       Compute eps as function of eps1 eps2 and norms
!     SEPCGLAMDA     Update lambda for cg if iter<=ntreigv
!     SEPCGNORMS     Compute some norms necessary for the termination criterion
!                    of the cg proces
!     SEPCGPRINT     Perform extended print for subroutine sepcg
!     SEPCGPRINT1    Perform extended print for subroutine prcg during iteration
!     SEPCGPRINT2    Perform printing CG process when iteration has not
!                    converged
!     SEPCGRFERR     Check error level, i.e. compute new value for rferr
!     SEPCGSTART     Fill starting value in array x and compute initial residual
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     675   The CG-process has terminated because the matrix is not positive
!           definite
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcg' )
      debug = .false. .and. ioutp>=0
      extradebug = .false.

!     --- Get parameters

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcg'
   !      call ertrace
         write(irefwr,1) 'n', A%neq
         write(irefwr,*) 'A%precon, A%eistat ', A%precon, A%eistat
         if ( extradebug ) call prinrl ( b, A%neq, 'b' )
  !       call sepprinmat6 ( A )
  !       call sepprinmat6p ( A )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get parameters

      n = A%neq                      ! number of free unknowns
      nmax = A%nmax                  ! maximum number of iterations
      if ( nmax==0 ) nmax=10*n
      lamminsav = A%alammin

      if ( debug ) &
         write(irefwr,1) 'iprec, iprint, nproj', &
                          A%type_prec, A%iprint, A%nproj

!     --- Allocate array to store eigenvalues

      allocate ( eigvl(2*A%ntreigv), res(A%neq), p(A%neq), &
                 work(A%neq), pbar(A%neq), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, 2*A%ntreigv+4*A%neq, 'eigvl' )
      if ( ierror/=0 ) go to 1000

!     --- Compute norm of right-hand side (squared)
!         Some other norms with respect to the termination criterion are
!         computed as well

      call sepcgnorms ( x, b, A, bnorm, bsnorm, xnorm, work )

      if ( bnorm<=0d0 ) go to 900

!     --- Print heading if iprint>0

      if ( A%iprint>0 ) write ( irefwr, 100 )

100   format(//' Output for the CG method (symmetric matrix)'//)

!     --- Set start vector and initial residu

      call sepcgstart ( A, x, b, res, work )

!     --- Compute the direction vector p and the square of the norm of the
!         residual (rho)

      p = res
      if ( A%precon ) p = res/A

      rho = dotprod(p,res, A)

      if ( A%iaccuracy==0 ) then
         rhotest = dotprod(p,p, A)       ! termination based on x
      else
         rhotest = dotprod(res,res, A)   ! termination based on res
      end if
      r0norm = rhotest

      if ( A%iprint>=1 ) then

!     --- iprint>=1,  Print information

         call sepcgprint ( res, rhotest, bnorm, bsnorm, r0norm, A )

      end if

!     --- Set constants

      lambda = 0d0
      if ( A%iaccuracy==0 ) lambda = A%alammin

      sqrn = A%nglob

      isqrn  = nint ( sqrt ( sqrn ) )
      rferr  = bnorm * epsmac **2

!     --- First estimate of eps  ( eps = eps1 + ||b|| eps2 )

      call sepcgeps ( A, repeat1, bnorm, bsnorm, r0norm )
      if ( debug ) then
         write(irefwr,2) 'bnorm, epsmac, rferr', bnorm, epsmac, rferr
         write(irefwr,2) 'bsnorm, r0norm', bsnorm, r0norm
         write(irefwr,2) 'eps, rho', A%eps, rho
      end if  ! ( debug )

!     --- While loop check on accuracy ( nmax+1 is used because
!         the residual of the previous iteration is always checked )

      do iter = 1, nmax+1

!     --- Check on accuracy reached in iter-1

         if ( debug ) write(irefwr,1) 'iter', iter

         finished = iter>1 .and. rhotest<=A%eps .and. &
                    lambda<=A%alammin .or. rhotest<=0d0

         if ( finished ) then

!        --- The accuracy of the process seems to be reached
!            Check if eps must be recomputed

            call sepcgcheckeps ( x, repeat1, xnorm, res, &
                                 lambda, iter, finished, &
                                 rhotest, .false., A, b )

            if ( debug ) then
               write(irefwr,*) 'finished', finished
               write(irefwr,2) 'eps', A%eps
            end if  ! ( debug )

            if ( finished ) go to 900  ! accuracy has been reached

         end if  ! ( finished )

!        --- Accuracy not yet reached
!            First check error level if iter is multiple of sqrt(n)

         if ( iter<A%ntreigv ) repeat1 = .true.
         if ( mod(iter,isqrn )==0 ) then

!        --- iter is multiple of sqrt(n)
!            Check error level, i.e. compute new value for rferr

            call sepcgrferr ( A, x, b, res, rferr, iter, work )

         end if  ! ( mod(iter,isqrn )==0 )
         if ( debug ) write(irefwr,2) 'rhotest, rferr', rhotest, rferr

         if ( rhotest<rferr ) then

!        --- rhotest is smaller than maximal accuracy possible
!            It is useless to continue the process

            call sepcgaccur ( res, rferr, A, lambda, iter, .false. )
            go to 900

         end if  ! ( rhotest<rferr )

!        --- do not start (nmax+1)-th iteration

         if ( iter>nmax ) exit

!        --- Compute Ap, store in work

         if ( A%eistat ) then

!        --- In case of Eisenstat Ap = E^(-T) p + E^(-1) p + K E^(-T) p
!            If necessary projection is carried out

            pbar = p/A
            work = p + A%S(A%diag) * pbar
            work = A/work
            work = work+pbar

         else

            work = A*p

         end if  ! ( A%eistat )

         if ( A%nproj>0 .and. A%deflation ) work = A .PROJT. work

         if ( extradebug ) call prinrl ( work, A%neq, 'Ap' )

!        --- Compute sigma

         sigma = dotprod(p,work, A)
         if ( sigma<=0d0 ) then

!        --- Sigma<=0, matrix not positive definite

            call erreal ( sigma, 1 )
            call errsub ( 675, 0, 1, 0 )

            A%numiter = iter
            A%itererr = 2

            go to 900

         end if

         alpha = rho / sigma
         if ( debug ) write(irefwr,2) 'alpha, sigma', alpha, sigma

!        --- Update x and r

         if ( A%eistat ) then

!        --- Eisenstat

            x = x + alpha*pbar

         else

!        --- Normal case

            x = x + alpha*p

         end if  ! ( A%eistat )

         res = res - alpha*work

         if ( extradebug ) then
            call prinrl ( x, A%neq, 'x' )
            call prinrl ( res, A%neq, 'res' )
         end if  ! ( extradebug )

!        --- Compute new dot product (r,r), store in rhostr

        if ( A%precon ) then

!        --- Preconditioning, store M^(-1) r into work
            work = res/A
            rhosta = dotprod(res,work, A)

         else

            rhosta = dotprod(res,res, A)

         end if

         beta = rhosta/rho
         rho  = rhosta
         if ( debug ) write(irefwr,2) 'beta, rho', beta, rho

         if ( A%iaccuracy==0 .and. A%precon ) then
            rhotest =  dotprod(work,work, A)
         else
            rhotest = dotprod(res,res, A)
         end if
         if ( debug ) write(irefwr,2) 'rhotest', rhotest

!        --- Update p:  p_(k+1) = M^(-1) r_(k+1) + beta p_k

         if ( A%precon ) then

!        --- Preconditioning,  M^(-1) r has been stored in work

            p = work + beta * p

         else

!        --- no preconditioning

            p = res + beta * p

         end if
         if ( debug ) write(irefwr,*) 'after update p'

         if ( extradebug ) call prinrl ( p, A%neq, 'p' )

         if ( A%iaccuracy==0 .and. iter<=A%ntreigv+1 ) then

!        --- Update lambda if iter<=A%ntreigv

            call sepcglamda ( iter, alpha, beta, eigvl, &
                              lambda, A, finished )
            if ( finished ) go to 900  ! error found

         end if

         if ( A%iprint>=2 ) then

!        --- iprint>=2,  Maximal output

            call sepcgprint1 ( res, iter, rhotest, bsnorm, &
                               lambda, xnorm, A, bnorm, r0norm )

         end if

      end do ! iter = 1, nmax+1

!     --- If the subroutine reaches this stage, too much  iterations have
!         been carried out

      call sepcgprint2 ( res, .false., A, lambda, nmax )

900   if ( A%nproj>0 .and. A%deflation ) then

!     --- Project vector x
!         First x := Proj(x)

         x = A .PROJ. x

!        --- Next x := x + V E^(-1) V^T y

          work = A .CPROJ. b
          x = x+work

      end if

!     --- Deallocate arrays

      deallocate ( eigvl, res, p, work, pbar, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'eigvl' )
      A%alammin = lamminsav

1000  if ( debug ) then

!     --- Debug information

         if ( extradebug ) call prinrl ( x, A%neq, 'x' )
         write(irefwr,*) 'End sepcg'

      end if  ! ( debug )
      call erclos ( 'sepcg' )

      end subroutine sepcg
