      subroutine sepcgmresr ( A, x, b )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-07-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Solve linear system of equation by the preconditioned restarted
!     GMRES method
!     complex case
! **********************************************************************
!
!                       KEYWORDS
!
!     iterative
!     linear_solver
!     preconditioning
!     complex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecmat
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (cmatrix), intent(inout) :: A
      double complex, intent(inout) :: x(A%neq), b(A%neq)

!     A              i    Matrix A
!     b              i    Right-hand side
!     x              o    Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: imax, nglobal, nmax, &
                 iter, inner, j, ninner, ihulp, imin, itnmin, ntrunk
      double complex, allocatable :: work(:,:)
      double precision :: bnorm, rnorm, rferr, bmin
      double complex :: t
      logical :: repeat1, finished

!     bmin           Variable used in the J&R truncation strategy.
!     bnorm          Norm of the right-hand side (b,b)
!                    they are not
!     finished       Indicates if the loop has finished (true) or not
!                    (false)
!     ihulp          Auxiliary variable.
!     imax           Parameter controlling the size of the subspaces, thus
!                    determining the restarts of the outer iteration
!                    process.
!                    The iterations restart after imax iterations, &
!                    with the latest iterate as new starting vector. The
!                    parameter should be larger than zero.
!     imin           Variable used in the J&R truncation strategy.
!     inner          Loop variable for the inner iteration.
!     iter           Iteration sequence number
!     itnmin         Auxiliary variable.
!     j              Counting variable
!     nglobal        Global number of unknowns. This is an overestimate, &
!                    because unknowns on interfaces are summed up.
!     ninner         Number of inner iterations
!     nmax           maximum number of iterations that may be performed by
!                    the CG-process. In any way no more than the number of
!                    degrees of freedom iterations are carried out.
!     ntrunk         The residual is made orthogonal to ntrunk vectors.
!     repeat1        Logical variable indicating if the accuracy check must be
!                    carried out twice (true) or once (false), because the
!                    cnorm of the solution could not be computed
!     rferr          Maximal accuracy possible for this matrix, due to
!                    round-off
!     rnorm          Norm of residual
!     t              Auxiliary variable.
!     work           Work array to store help vectors
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRWAR         Print warnings
!     SEPCCGESTEPS   Print first line of iteration output
!                    Compute first estimate of accuracy  eps
!     SEPCCGSTART    converged
!     SEPCMAXACC     Compute maximum accuracy possible in iterative solver
!     SEPCPOLGMR     innerloop for GMRESR.
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     677   The maximal accuracy with respect to the condition of the
!           matrix is reached
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The subroutine GMRESR is used to solve an unsymmetric system of
!     linear equations of the shape A.x = f. Starting from an initial
!     vector, in each iteration a vector is sought which minimizes the
!     residual vector A.x-f in a certain subspace. The process restarts
!     after a fixed number of iteration, with the latest approximation
!     as a new starting vector. Furthermore, only ntrunk vectors are
!     used to orthogonalize the search direction. The method consists
!     of an inner and outer iteration loop. In the inner loop we do
!     a number of iterations with the original GMRES algorithm to
!     obtain a good search direction. In the outer loop the residual
!     is made orthogonal with respect to the search directions. In
!     the case that the value of 'inner' is equal to 1 the GMRESR
!     method is identical with the GCR method.
!     The method is described in:
!     GMRESR: a family of nested GMRES methods, &
!     H.A. van der Vorst & C. Vuik, &
!     TUD Report 91-80.
!     and
!     Further experiences with GMRESR, &
!     C. Vuik, &
!     TUD Report 92-12.
!     The work array can be interpreted as follows:
!       work(.,j)          contains u(j), &
!       work(.,ntrunk+j)   contains c(j), &
!       work(.,2*ntrunk+3) contains the residual, &
!     where the vectors u(j), and c(j) are given on page 3 of TUD
!     Report 91-80.
!     The GCR method is described in:
!     Variational iterative methods for non symmetric systems of linear
!     equations, &
!     S.C. Eisenstat, H.C. Elman & M.H. Schultz, &
!     SIAM J. Numer. Anal., 20 (1983), pp.345-357.
!     Both in the case of the preconditioned GMRES method
!     as in the case of the Eisenstat implementation, the matrix and right-hand
!     side are supposed to be premultiplied by the inverse of the diagonal D
!     of the triangular factors from the preconditioning matrix in the
!     following way:  ( A x = b )
!               -1        -1
!          A = D   A     D
!                   orig
!               -1
!          b = D   b
!                   orig
!     If the initial estimate x is given it must also be premultiplied by D, &
!     before sepcgmresr is called:
!          x = D  x
!                  orig
!                                                        -1
!     At output the solution x must be premultiplied by D   in order to get
!     the solution of the original problem, after go to 1000 from sepcgmresr:
!                  -1
!         x     = D   x
!          orig
!     In the case of Eisenstat the matrix amat is supposed to contain the
!     matrix H = D  - 2I in the first n positions, where D  is the diagonal
!                 A                                       A
!     of A after the premultiplication
!     Check error level:
!     If ||r||<100*epsmac*||b||
!     then maximal accuracy has been reached.
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcgmresr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcgmresr'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      bnorm = cnorm(b,A%neq)
      if ( bnorm<=0d0 ) then

!     --- Right-hand side is equal to zero

         x = 0d0
         go to 1000  ! rhs is equal to 0

      end if  ! ( bnorm<=0d0 )

      nglobal = A%neq                ! number of global unknowns
      ntrunk = A%ntrunk
      imax = A%imax                  ! Maximal dimension of Krylov space
      nmax = A%nmax                  ! maximum number of iterations
      if ( nmax==0 ) nmax=10*nglobal
      ninner = max(1,min(A%ninner,100))

      if ( debug ) &
         write(irefwr,1) 'iprec, iprint', A%type_prec, A%iprint

      bmin   = 0d0

      allocate ( work(A%neq,2*ntrunk+ninner+10), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, A%neq*2*ntrunk+ninner+10, 'work' )
      if ( ierror/=0 ) go to 1000

!     --- Compute maximum accuracy possible

      call sepcmaxacc ( A, b, work, bnorm, rferr )
      if ( debug ) write(irefwr,2) 'rferr', rferr

      work(:,2*ntrunk+4) = 0d0

!     --- Print heading if iprint>0

      if ( A%iprint>0 ) then
         if ( ninner>1 ) then
            write ( irefwr, 100 )
         else
            write ( irefwr, 101 )
         end if
      end if

100   format(//' Output for the GMRESR method', &
               ' (non-symmetric matrix)',//)
101   format(//' Output for the GCR method', &
               ' (non-symmetric matrix)',//)

      if ( A%iprint>0 ) then
         if ( ninner>1 ) then
            write ( irefwr, 105 ) imax, ntrunk, ninner
         else
            write ( irefwr, 106 ) imax, ntrunk
         end if
      end if

105   format(' The GMRESR method restarts after ',i5,' iterations.',/, &
             ' There are ',i5,' truncation vectors used.',/, &
             ' The degree of the polynomial is less than',i5)
106   format(' The GCR method restarts after ',i5,' iterations.',/, &
             ' There are ',i5,' truncation vectors used.')

!     --- Set residual r = b, it will be updated when x # 0

      call sepccgstart ( A, x, b, work(1,1), work(1,2) )

      work(:,2*ntrunk+3) = work(:,1)

!     --- Set constants
!         Norm of the initial residual r

      rnorm = cnorm(work(:,1),A%neq)

!     --- Print first line of iteration output
!         Compute first estimate of accuracy  eps

      iter = 1
      inner = 0
      call sepccgesteps ( A, b, work(:,2), rnorm, rferr, repeat1, iter )
      if ( debug ) write(irefwr,2) 'rnorm, eps', rnorm, A%eps

      finished = .false.

!     --- While loop check on accuracy ( nmax+1 is used because
!         the residual of the previous iteration is always checked )

      do while ( .not. finished .and. iter<nmax )

         if ( debug ) write(irefwr,1) 'iter', iter

         finished = iter>1 .and. rnorm<=A%eps .or. rnorm<=0d0


         do while ( finished )

!        --- The accuracy of the process seems to be reached
!            Check if eps must be recomputed

            if ( repeat1 ) then

!           --- repeat is true, recompute eps

               repeat1 = .false.
               A%eps = A%eps1 + cnorm(x,A%neq) * A%eps2

!              --- Check error level.

               if ( A%eps<rferr ) then

!              --- eps is smaller than maximal accuracy possible
!                  It is useless to continue the process after rnorm
!                  is less than rferr

                  A%eps = rferr

                  if ( A%iprint>=0 ) then
                     call erreal ( rferr, 1 )
                     call errwar ( 677, 0, 1, 0 )
                  end if

               end if
               if ( debug ) write(irefwr,2) 'eps', A%eps

               finished = rnorm<=A%eps

            end if

            if ( finished .and. .not. repeat1 ) then

!           --- Process is ended

            if ( A%iprint>=1 ) then

!           --- iprint>=1  Print information

               if ( ninner>1 ) then
                  write ( irefwr,150 ) iter-1, rnorm
               else
                  write ( irefwr,151 ) iter-1, rnorm
               end if

150            format(' The number of executed iterations in the', &
                      ' GMRESR process is', i5/ &
                      ' The 2-norm of the residual is:', ss,1pe12.2e2 )
151            format(' The number of executed iterations in the', &
                      ' GCR process is', i5/ &
                      ' The 2-norm of the residual is:', ss,1pe12.2e2 )

            end if

               A%numiter = iter-1
               A%itererr = 0

               go to 500

            end if  ! ( finished .and. .not. repeat1 ) then

         end do  ! while ( finished )

!        --- Accuracy not yet reached
!            do not start (nmax+1)-th iteration

         if ( iter>nmax ) then
            finished = .true.
            exit
         end if  ! ( iter>nmax )
         inner = inner+1
         itnmin = min(inner-1,ntrunk)

         if ( ninner==1 ) then

!        --- First inner iteration

            work(:,2*ntrunk+1) = work(:,2*ntrunk+3)

         else

!        --- Next inner iteration

            call sepcpolgmr ( A, work(:,2*ntrunk+1), &
                              work(:,2*ntrunk+3), &
                              work(:,2*ntrunk+5) )

         end if  ! ( ninner==1 )

!        --- Preconditioned matrix-vector multiplication

         work(:,2*ntrunk+5) = work(:,2*ntrunk+1)/A

         if ( A%eistat ) then
            work(:,2*ntrunk+2) = work(:,2*ntrunk+1) + &
                                 A%S * work(:,2*ntrunk+5)
            work(:,2*ntrunk+2) = A/work(:,2*ntrunk+2)
            work(:,2*ntrunk+2) = work(:,2*ntrunk+2)+work(:,2*ntrunk+5)
         else
            work(:,2*ntrunk+2) = A*work(:,2*ntrunk+5)
         end if  ! ( A%eistat )

         do j = 1, itnmin
            t = -dot_product ( work(:,ntrunk+j), work(:,2*ntrunk+2) )
            work(:,2*ntrunk+1) = work(:,2*ntrunk+1) + t*work(:,j)
            work(:,2*ntrunk+2) = work(:,2*ntrunk+2) + t*work(:,ntrunk+j)

            if ( abs(t)<=bmin.or.j==1 ) then

               bmin = abs(t)
               imin = j
            end if

         end do
         t = cnorm ( work(:,2*ntrunk+2),A%neq )
         t = 1.0/t

         if ( inner>ntrunk ) then
            ihulp = imin
         else
            ihulp = inner
         end if

         work(:,ihulp) = work(:,2*ntrunk+1)*t
         work(:,ntrunk+ihulp) = work(:,2*ntrunk+2)*t

!        --- update x and r

         t = dot_product ( work(:,ntrunk+ihulp), work(:,2*ntrunk+3) )
         work(:,2*ntrunk+4) = work(:,2*ntrunk+4) + t*work(:,ihulp)
         work(:,2*ntrunk+3) = work(:,2*ntrunk+3)-t*work(:,ntrunk+ihulp)
         rnorm = cnorm ( work(1,2*ntrunk+3),A%neq )

!        --- Check for termination of inner iterations.

         if ( inner==imax .or. rnorm<A%eps .or. iter==nmax ) then

            work(:,2*ntrunk+5) = work(:,2*ntrunk+4)/A
            x = x + work(:,2*ntrunk+5)

            inner = 0

            work(:,2*ntrunk+4) = 0d0

            work(:,1) = b
            work(:,2) = A*x

            if ( A%eistat ) then

!           --- In the case of Eisenstat there is  amat = A - 2 * I, &
!               so t(*) must be corrected

               work(:,2) = work(:,2) + 2d0*x

            end if

!           --- Update r

            work(:,1) = work(:,1) - work(:,2)

!           --- Premultiply residual by L^(-1) in the case of Eisenstat

            if ( A%eistat ) work(:,1) = A/work(:,1)
            rnorm = cnorm ( work,A%neq )
            work(:,2*ntrunk+3) = work(:,1)

         end if

         if ( A%iprint>=2 ) then

!        --- iprint>=2,  Maximal output

            write ( irefwr, 350 ) iter, rnorm
350         format( i10, ss,1pe15.2e2 )

         end if

         iter = iter+1

      end do  ! while ( .not. finished .and. iter<nmax )

500   deallocate ( work, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'work' )



1000  call erclos ( 'sepcgmresr' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepcgmresr'

      end if  ! ( debug )

      end subroutine sepcgmresr
