      subroutine sepcopytoprof ( A )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-10-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Initialize the LU-decomposition matrix of A for a profile method
!     This is done by copying the matrix from row-compact storage to
!     profile storage (asymmetric)
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     matrix_structure
!     linear_solver
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesolve
      use sepmoduleintmat
      use sepmodulematr
      use sepmodulemat
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix) :: A

!     A             i/o   Matrix A
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      type (intmatrix), pointer :: IM
      integer :: i, ip, icol, jpos, ii
      integer, pointer :: rows(:), cols(:)
      type (globmat), pointer :: Ag

!     Ag             Refers to matrix A as type globmat
!     IM             Refers to intmt(intmat)
!     cols           Refers to column numbers
!     i              Counting variable
!     icol           column number
!     ii             Position of diagonal element A(ii)
!     ip             Loop over columns in row
!     jpos           Position in array
!     rows           Refers to start of column numbers per row
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPPRINMAT2    Print inverse matrix that is stored as profile real matrix
!     SEPPRINMAT6    print compact matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcopytoprof' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcopytoprof'
         call sepprinmat6 ( A )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(iseqim)
      Ag => realmt(iseqmt)
      A%Mrows => IM%rowsiiprec
      A%Mcols => IM%colsiiprec
      rows => IM%rowsii
      cols => IM%colsii

      if ( associated(Ag%matinv) ) deallocate(Ag%matinv)
      lenpremat = A%Mrows(A%neq)+A%neq-A%Mcols(A%neq)
      allocate(Ag%matinv(lenpremat))
      A%MD => Ag%matinv
      A%MD = 0d0

!     --- Loop over rows

      do i = 1, A%neq

         ii = A%Mrows(i)  ! diagonal element i
         do ip = rows(i)+1, rows(i+1)  ! elements in row
            icol = cols(ip)        ! column number

!           --- Compute position in matinv

            if ( icol<=i ) then

!           --- Lower triangular part (including diagonal)

               jpos = ii-i+icol

            else

!           --- Upper triangular part

               jpos = A%Mrows(icol)+icol-i

            end if  ! ( icol<=i )
            A%MD(jpos) = A%S(ip)  ! copy

         end do  ! ip = rows(i)+1, rows(i+1)

      end do  ! i = 1, A%neq

1000  if ( debug ) then

!     --- Debug information

         call sepprinmat2 ( A )
         write(irefwr,*) 'End sepcopytoprof'

      end if  ! ( debug )
      call erclos ( 'sepcopytoprof' )

      end subroutine sepcopytoprof
