      subroutine sepcprojEinv ( A, x, y )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-12-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute y = E^-1 x, where E is the matrix (AP)^T P as part
!     of projection. P is the matrix of projection vectors
!     It is assumed that E has been decomposed before
!     Complex version of sepprojEinv
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     projection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleproj
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (cmatrix), intent(in) :: A
      double complex, intent(in) :: x(A%nproj)
      double complex, intent(out) :: y(A%nproj)

!     A             i/o   Matrix A
!     x              i    Vector to be multiplied
!     y              o    result of multiplication
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double complex :: usend(A%nproj)
      integer :: nsend, icontext, iblock, j

!     iblock         Block sequence number
!     icontext       Tag for sending and receiving, is used as check
!     j              Counting variable
!     nsend          Help variable for receiving
!     usend          Help array to be used to send reals to other processor
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIRECEIVEMPI  Receive information from other node (mpi)
!     INISENDMPI     Send information to other node (mpi)
!     PRINRL         Print 1d real vector
!     sepcsolvsm      Solve system of symmetric equations (full matrix), where
!                    the matrix has been decomposed by PRDECOSM
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcprojEinv' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcprojEinv'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( parallel ) then

!     --- Parallel case
!         On the first node we multiply by the inverse of E and store
!         in the same positions: y = E^-1 y. Thereafter
!         this vector is sent back to all processors.

         icontext = 14

         if ( inodenr==1 ) then

            call sepcsolvsm ( A%nproj, A%projE, y, y )

!           --- Send contributions to the other nodes

            do j = 2, A%nproj
               call inisendmpi ( y, A%nproj*2, SPDOUBLE, j, icontext )
            end do

         else

!        --- Receive the solution vector from node 1

            nsend = A%nproj
            call inireceivempi ( y, nsend, SPDOUBLE, iblock, &
                                 icontext )
         end if

         if ( debug ) call prinrl ( y, A%nproj*2, 'y/2' )

      else

!     --- Serial case

         call sepcsolvsm ( A%nproj, A%projE, y, y )

      end if  ! ( parallel )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepcprojEinv'

      end if  ! ( debug )
      call erclos ( 'sepcprojEinv' )

      end subroutine sepcprojEinv
