      subroutine sepcscmatdiv ( A, usol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 08-06-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute y = A^(-1) x, where A is a preconditioning matrix
!     a symmetric compact storage is used, hence only D and U
!     The preconditioning matrix has been created before
!     In other words the ILU inverse of A is stored
!     complex case
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     real
!     direct_method
!     nested_dissection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulecmat
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (cmatrix) :: A
      double complex :: usol(A%neq)

!     A             i/o   Matrix A
!     usol          i/o   solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, k, ii, mk, jp
      integer, pointer :: diag(:), row(:), col(:), rowmat(:)
      integer, allocatable :: rowdiff(:)
      double complex, pointer :: mat(:)
      double complex :: yi, som

!     col            Column numbers of matrix
!     diag           Contains the position of the diagonal elements
!                    in the rows of the matrix (umsymmetric storage)
!     i              Counting variable
!     ii             Position of diagonal element i in matrix
!     jp             Loop variable
!     k              Counting variable
!     mat            elements of matrix
!     mk             Column number
!     row            starting addresses (-1) of rows in array cols
!     rowdiff        Correction of row because we are dealing with a
!                    symmetric matrix
!     rowmat         tarting addresses (-1) of rows in array mat
!     som            Help parameter to compute a sum
!     yi             usol(i)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     SEPPRINMAT6P   print compact preconditioning matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!      L L^T u = f
!      First L y = f, hence y = L^-1 f ( store in usol)
!      Next L^T u = y
!
!      L y = f:  sum_k=1^i L_ik y_k = f_i
!      Hence y_i = ( f_i - sum_k=1^i-1 L_ik y_k ) / L_ii
!
!      L^T u = y:  sum_k=i+1^n L_ki u_i = y_i
!      Hence u_i = ( y_i - sum_k=i+1^n L_ki u_i ) / L_ii
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcscmatdiv' )
      debug = .false. .and. ioutp>=0

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcscmatdiv'
         call prinrl ( usol, A%neq, 'rhsd' )
         call sepprinmat6p ( A )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      mat => A%MD

      row => A%Mrows
      col => A%Mcols
      diag => A%Mdiag

!        --- compute y, store in usol
!            L * y = f   (rhsd)

      if ( A%scaling==0 ) usol = usol/mat(diag)
      do i = 1, A%neq

         yi = usol(i)
         do jp = row(i)+2, row(i+1)
            k = col(jp)
            usol(k) = usol(k)-mat(jp)*yi
         end do  ! k = row(i)+1, diag(i)-1

      end do

!     --- compute x. store in usol
!          t
!         L  * x = y  (usol)

      do i = A%neq, 1, -1

         do k = row(i)+2, row(i+1)
            usol(i) = usol(i)-mat(k)*usol(col(k))
         end do  ! k = row(i)+1, diag(i)-1

      end do
      if ( A%scaling==0 ) usol = usol/mat(diag)

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( usol, 5, 'usol' )
         write(irefwr,*) 'End sepcscmatdiv'

      end if  ! ( debug )
      call erclos ( 'sepcscmatdiv' )

      end subroutine sepcscmatdiv
