      subroutine seploctrnrl ( itrans, usol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-10-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute (back-)transformation when local transformations are
!     performed
!     real version
!     Old subroutine SEPLOCTRN
! **********************************************************************
!
!                       KEYWORDS
!
!     transformation
!     real
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: itrans
      double precision, intent(inout) :: usol(*)

!     itrans     i    Indicates if transformation from standard vector (i.e.
!                     usual degrees of freedom, for example Cartesian) to
!                     transformed vector (i.e. with transformed degrees of
!                     freedom in points with local transform, for example normal
!                     and tangential components) is defined (1) or that the
!                     backwards transformation to standard vector must be
!                     computed (2)
!     usol      i/o   solution array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPLOCTRNRLNOR Compute (back-)transformation in case of local
!                    transformations ( real case no renumbering )
!     SEPLOCTRNRLREN Compute (back-)transformation in case of local
!                    transformations ( real case and renumbering )
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!       The tranformation is given by u = R un,
!
!       with R depending on maxtrn:
!
!
!       In R^2:
!                | nx   -ny |              | r11   r12 |
!       2:   R = |          |   4,8:   R = |           |
!                | ny    nx |              | r21   r22 |
!
!
!       this means:
!
!       2:
!            |  nx un - ny ut |             | nx u1 + ny u2 |
!       u  = |                |       un =  |               |
!            |  ny un + nx ut |             |-ny u1 + nx u2 |
!
!       4,8:
!            |  r11 un + r12 ut |            1 | r22 u1 - r12 u2 |
!       u  = |                  |       un = - |                 |
!            |  r21 un + r22 ut |            d |-r21 u1 + r11 u2 |
!
!                                       d = r11 r22 - r12 r21
!
!       In R^3:
!
!       See 4:, but now extended to three dimensions
!
!    if ( itrans=1 ) then
!       perform forward transformation
!    else
!       perform backtransformation
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seploctrnrl' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seploctrnrl'
         write(irefwr,1) 'itrans, npltra, ndim, maxtrn, indprh', &
                          itrans, npltra, ndim, maxtrn, indprh
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( indprh==0 ) then
         call seploctrnrlnor ( itrans, usol )
      else
         call seploctrnrlren ( itrans, usol )
      end if  ! ( indprh==0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End seploctrnrl'

      end if  ! ( debug )
      call erclos ( 'seploctrnrl' )

      end subroutine seploctrnrl
