      subroutine sepmakeprec ( A )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-04-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create preconditioning matrix if necessary
!     Compute parameter iprec
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     preconditioning
!     create
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemat
      use sepmodulekprob
      use sepmodulematr
      use sepmodulesolve
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix) :: A

!     A             i/o   Structure of type matrix containing the necessary
!                         information of the matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: lengmtprec, lengL, i, iend, istart

!     i              Counting variable
!     iend           Last position used at a part
!     istart         First position used at a part
!     lengL          Length of lower tridiagonal of matrix
!     lengmtprec     Length of preconditioning matrix
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
!     SEPCREAPREC    computes the preconditioning matrix for the CG-
!                    CGS- and Bi-CGSTAB process. (real case)
!     SEPPRINMAT6    print compact matrix
!     SEPPRINMAT6P   print compact preconditioning matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1405   This combination of the storage scheme and the preconditioner
!           is not implemented.
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepmakeprec' )
      debug = .false.! .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepmakeprec'
         write(irefwr,1) 'A%type_prec, A%neq', A%type_prec, A%neq
    !     call sepprinmat6 ( A )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get parameters

      if ( A%type_prec==0 ) go to 1000  ! no preconditioning required: skip

      if ( debug ) &
         write(irefwr,1) 'iprob, iseqintmat', iprob, iseqintmat

!     --- Check if the structure of preconditioning matrix and original
!         matrix differ (iseqprerd>0 and ILU)

      if ( debug ) write(irefwr,1) 'iseqpre, ipreco', iseqpre, A%type_prec

!     --- Compute length of preconditioning matrix

      lengL = A%Mrows(1+A%neq)
      lengmtprec = A%Mrows(1+A%neq)

      if ( debug ) write(irefwr,1) 'lengmtprec', lengmtprec

      length = 0

      if ( debug ) write(irefwr,1) 'keep', A%keep

!     --- In case the preconditioning matrix does not exist and keep =2
!         we have to reset keep to 1 in order to make a new one

      if ( A%keep==2 .and. .not. associated(realmt(iseqmt)%matinv) ) &
         A%keep = 1

      if ( A%keep<2 ) then

!     --- New preconditioning matrix

         if ( A%type_prec==1 .or. A%type_prec==2 .or. A%type_prec==7 ) then

!        --- Diagonal matrix

            length = A%neq

         else if ( A%type_prec==3 .or. A%type_prec==4 ) then

!        --- ILU preconditioning

            length = lengmtprec

         else if ( A%type_prec>=8 ) then

!        --- Preconditioning is not yet available

            call errint ( A%type_storage, 1 )
            call errint ( A%type_prec, 2 )
            call errwar ( 1405, 2, 0, 0 )
            A%type_prec = 2
            length = A%neq

         end if

!        --- Make array permanent

         allocate ( realmt(iseqmt)%matinv(length), stat = error )
         if ( error/=0 ) call eralloc ( error, length, 'premat' )
         if ( ierror/=0 ) go to 1000

         Pre_ii => realmt(iseqmt)%matinv

      end if  ! ( A%keep<2 )

      if ( debug ) write(irefwr,1) 'length', length

      lenpremat = length

!     --- associate preconditioning matrix with A

      iend = A%neq
      A%MD => Pre_ii
      A%DLshift => Dlapshift  ! diagonal laplace shift

      if ( A%keep<2 ) call sepcreaprec ( A )  !  Create preconditioning matrix

1000  if ( debug ) then

!     --- Debug information

    !     call sepprinmat6p ( A )
         write(irefwr,*) 'End sepmakeprec'

      end if  ! ( debug )
      call erclos ( 'sepmakeprec' )

      end subroutine sepmakeprec
