      recursive subroutine sepmatvec ( A, x, y )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-06-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Matrix vector multiplication for serial matrices
!
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     matrix_vector_multiplication
!     real
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesimple
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix), intent(in) :: A
      double precision, intent(in) :: x(A%neq)
      double precision :: y(A%neq)

!     A              i    Matrix A
!     x              i    vector to be multiplied
!     y              o    result Ax
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical :: glob

!     glob           If true sepparglob must be called
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERTRACE        Prints trace back of subroutines
!     SEPDMATVEC     Matrix vector multiplication D u = y (simple)
!     SEPGMATVEC     Matrix vector multiplication G p = y (simple)
!     SEPMATVEC30    Matrix vector multiplication for serial matrices
!                    Special case: finite difference matrix
!     SEPMATVECSIM   Compute y = A x, where A is a matrix with simple storage
!     SEPNRCMATVEC   Compute y = A x, where A is non-symmetric row compact
!                    matrix
!     SEPNSCMATVEC   Compute y = A x, where A is non-symmetric compact matrix
!     SEPNSPMATVEC   Compute y = A x, where A is unsymmetric profile matrix
!     SEPPARGLOB     Send information to neighboring processors in order to give
!                    all local arrays the global value in the common points
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2917   Option not available
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepmatvec' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepmatvec'
         call ertrace
         write(irefwr,*) 'parallel, A%simple ', parallel, A%simple
         write(irefwr,1) 'typecoupl, type_storage', typecoupl, A%type_storage
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )

      if ( ierror/=0 ) go to 1000

      if ( debug ) write(irefwr,1) 'matmultsp', A%matmultsp

      if ( A%matmultsp==2 ) then

!     --- Special matrix matrix vector multiplication for parallel simple
!         y = - D Q^-1 G x

         call sepgmatvec ( x, worky2 )     ! G*x
         worky2 = worky2/S_Qprime             ! Q^-1 G*x
         call sepdmatvec ( worky2, worky1 )! D Q^-1 G x
         y = worky1
         go to  1000

      end if  ! ( A%matmultsp==1 )

      glob = parallel .and. typecoupl==0

      if ( A%simple ) then

!     --- simple matrix

         call sepmatvecsim ( x, y )
         glob = .false.

      else if ( A%type_storage==30 ) then

!     --- Special case, finite difference matrix

         call sepmatvec30 ( A%S, x, y, A%neq, A%rows )

      else if ( A%symmetric ) then

!     --- symmetric compact matrix

         call sepnscmatvec ( A, x, y )

      else

!     ---  Row-compact storage (a-symmetric)

         call sepnrcmatvec ( A, x, y )

      end if  ! ( A%simple )

      if ( A%matmultsp==1 ) then

!     --- Special matrix matrix vector multiplication for AL method
!         First step y = Q x
!         Next gamma y = y + gamma G M^-1 D x

         call sepdmatvec ( x, worky1 )
         worky1 = worky1*S_PMD
         call sepgmatvec ( worky1, worky2 )
         y = y+worky2

      end if  ! ( A%matmultsp==1 )

!     --- Give y the same value in common points of different parallel blocks
!         in case of Neumann-Neumann coupling

      if ( glob ) then
         length = size(A%sendinfo)
         call sepparglob ( A, y, A%neq )
      end if  ! ( glob ) then

1000  if ( debug ) then

!     --- Debug information

  !       call prinrl ( y, A%neq, 'rhsd' )
         write(irefwr,*) 'End sepmatvec'

      end if  ! ( debug )
      call erclos ( 'sepmatvec' )

      end subroutine sepmatvec
