      subroutine sepmaver ( matr, ix, iy, ichoice )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  2.1  date 11-12-2017 New storage of intmat
!        version  2.0  date 26-02-2016 Complete new subroutine
!        version  1.19 date 11-04-2015 Remove ibcmat
!
!   copyright (c) 1999-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     perform matrix vector multiplication
!     amat * x = y
!     depending on the value of intmat(1) one of the subroutines
!     er0013, er0014, er0015, er0016 is called
!     the effect of boundary conditions is taken into account (ichoice = odd)
!     or not (ichoice = even) (subroutine er0017, er0018)
!     when matr is of type isol, the matrix is supposed to be a
!     diagonal matrix and intmat is not used
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_vector_multiplication
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulematr
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer matr, ix, iy, ichoice

!     ichoice   i  choice parameter. Possibilities:
!                 0 boundary conditions are not taken into account
!                   boundary conditions in y are left unchanged
!                 1 boundary conditions are taken into account
!                   boundary conditions in y are left unchanged
!                 2 boundary conditions are not taken into account
!                   boundary conditions in y are copied from x
!                 3 boundary conditions are taken into account
!                   boundary conditions in y are copied from x
!                 4 boundary conditions are not taken into account
!                   boundary conditions in y are made equal to 0
!                 5 boundary conditions are taken into account
!                   boundary conditions in y are made equal to 0
!                 6 the vector y is made equal to the boundary conditions
!                   without the matrix-vector multiplication.
!                   the positions corresponding to the boundary conditions
!                   get the value 0
!                 7 The existing vector y is updated with minus the effect
!                   of the boundary conditions.
!                   Hence yf = yf - Sfp xp
!                   This is typically meant for an update of a right-hand
!                   side vector where the effect of the boundary conditions
!                   has not been taken into account
!                 8 The existing vector y is filled with the reaction
!                   forces that result from boundary conditions.
!                   Hence yp = Spf xf + Spp xp.  This option is only
!                   available if ibcmat=1 and is only useful if
!                   xf has already been computed.  The part corresponding to
!                   yf is left unchanged, which is a fast option if
!                   only the reaction forces are required.
!                 9 Combination of 1 and 8. Less fast then 8
!                10 Same as 8, now the rest of the vector is set equal to 0
!     ix       i  standard sepran array containing information of the
!                 vector x
!     iy       o  sepran array containing information of the vector y
!     matr     i  standard sepran array containing information
!                 concerning the matrix
!                 matr may be of type isol.
!                 then the matrix is supposed to be diagonal.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, allocatable :: ytemp(:)
      double complex, allocatable :: yctemp(:)

!     yctemp         temporary result vector (complex)
!     ytemp          temporary result vector (real)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
!     PRINRL         Print 1d real vector
!     SEPMAVER1      Matrix vector multiplication y_i = S_ii*x_i (real)
!     SEPMAVER2      Matrix vector multiplication y_i = S_ii*x_i (complex)
!     SEPMAVERP1     First part of Matrix vector multiplication
!     SEPMAVERP2     Last part of Matrix vector multiplication
!     SEPMULTBCOND1  Compute y = y + S_ip x_p in real case
!     SEPMULTBCOND2  Compute y = y + S_ip x_p in complex case
!     SEPMULTREAC1   Compute reaction forces for real matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepmaver' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepmaver'
         call ertrace
         write(irefwr,1) 'ichoice, ix, iy, matr', ichoice, ix, iy, matr
         write(irefwr,1) 'ks(ix)%compl', ks(ix)%compl
         write(irefwr,1) 'ks(ix)%iprob', ks(ix)%iprob
         write(irefwr,*) 'realmt(matr)%complexmat', realmt(matr)%complexmat
         write(irefwr,*) 'realmt(matr)%diagonal', realmt(matr)%diagonal
         write(irefwr,1) 'size(irows)', size(realmt(matr)%S_i%rows)
  1      format ( a, 1x, (10i11) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Create or activate iy and define Smat, xri/ci, yri/ci

      call sepmaverp1 ( matr, ix, iy, ichoice )

      select case (ichoice)
      case (0)

!     --- y_i = S_ii*x_i; y_b unchanged

         Smat%inclbc = .false.
         if ( Smat%complexmat ) then
            call sepmaver2 ( Smat, xci, yci )
         else
            call sepmaver1 ( Smat, xri, yri )
         end if  ! ( Smat%compl )

      case (1)

!     --- y_i = S_ii*x_i + S_ip*x_p; y_b unchanged

         Smat%inclbc = nbound>0 .and. .not. Smat%diagonal
         if ( Smat%complexmat .or. ks(ix)%compl==1 ) then
            call sepmaver2 ( Smat, xci, yci )
            if ( Smat%inclbc ) call sepmultbcond2 ( Smat, xci, yci )
         else
            call sepmaver1 ( Smat, xri, yri )
            if ( Smat%inclbc ) call sepmultbcond1 ( Smat, xri, yri )
         end if  ! ( Smat%compl )

      case (2)

!     --- y_i = S_ii*x_i; y_b = x_b

         Smat%inclbc = .false.
         if ( Smat%complexmat .or. ks(ix)%compl==1 ) then
            call sepmaver2 ( Smat, xci, yci )
            yci(nusol-nbound+1:nusol) = xci(nusol-nbound+1:nusol)
         else
            call sepmaver1 ( Smat, xri, yri )
            if ( nbound>0 ) yrp = xrp
         end if  ! ( Smat%compl )

      case (3)

!     --- y_i = S_ii*x_i + S_ip*x_p; y_b = x_b

         Smat%inclbc = nbound>0 .and. .not. Smat%diagonal
         if ( Smat%complexmat .or. ks(ix)%compl==1 ) then
            call sepmaver2 ( Smat, xci, yci )
            yci(nusol-nbound+1:nusol) = xci(nusol-nbound+1:nusol)
            if ( Smat%inclbc ) call sepmultbcond2 ( Smat, xci, yci )
         else
            call sepmaver1 ( Smat, xri, yri )
            if ( nbound>0 ) yrp = xrp
            if ( Smat%inclbc ) call sepmultbcond1 ( Smat, xri, yri )
         end if  ! ( Smat%compl )

      case (4)

!     --- y_i = S_ii*x_i; y_b = 0

         Smat%inclbc = .false.
         if ( Smat%complexmat .or. ks(ix)%compl==1 ) then
            call sepmaver2 ( Smat, xci, yci )
            yci(nusol-nbound+1:nusol) = 0d0
         else
            call sepmaver1 ( Smat, xri, yri )
            if ( nbound>0 ) yrp = 0d0
         end if  ! ( Smat%compl )

      case (5)

!     --- y_i = S_ii*x_i + S_ip*x_p; y_b = 0

         Smat%inclbc = nbound>0 .and. .not. Smat%diagonal
         if ( Smat%complexmat .or. ks(ix)%compl==1 ) then
            call sepmaver2 ( Smat, xci, yci )
            yci(nusol-nbound+1:nusol) = 0d0
            if ( Smat%inclbc ) call sepmultbcond2 ( Smat, xci, yci )
         else
            if ( nbound>0 ) yrp = 0d0
            call sepmaver1 ( Smat, xri, yri )
            if ( Smat%inclbc ) call sepmultbcond1 ( Smat, xri, yri )
         end if  ! ( Smat%compl )

      case (6)

!     --- y_i = S_ip*x_p; y_b = 0

         Smat%inclbc = nbound>0 .and. .not. Smat%diagonal
         Smat%intmul = .false.
         if ( Smat%complexmat .or. ks(ix)%compl==1 ) then
            call sepmaver2 ( Smat, xci, yci )
            yci(nusol-nbound+1:nusol) = 0d0
            if ( Smat%inclbc ) call sepmultbcond2 ( Smat, xci, yci )
         else
            if ( nbound>0 ) yrp = 0d0
            call sepmaver1 ( Smat, xri, yri )
            if ( Smat%inclbc ) call sepmultbcond1 ( Smat, xri, yri )
         end if  ! ( Smat%compl )
         Smat%intmul = .true.

      case (7)

!     --- y_i = y_i - S_ip*x_p

         Smat%inclbc = nbound>0 .and. .not. Smat%diagonal
         if ( .not. Smat%inclbc ) go to 1000
         if ( Smat%complexmat .or. ks(ix)%compl==1 ) then
            allocate ( yctemp(nusol) )
            yctemp = 0d0
            call sepmultbcond2 ( Smat, xci, yctemp )
            yci(1:nusol) = yci(1:nusol)-yctemp(1:nusol)
            deallocate ( yctemp )
         else
            allocate ( ytemp(nusol) )
            ytemp = 0d0
            call sepmultbcond1 ( Smat, xri, ytemp )
            yri(1:nrusol) = yri(1:nrusol)-ytemp(1:nrusol)
            deallocate ( ytemp )
         end if  ! ( Smat%compl )

      case (8,10)

!     --- Compute reaction force:
!         y_p = S_pf x_f + S_pp x_p. y_f is left unchanged

         call  sepmultreac1 ( Smat, xri, yri )

      case (9)

!     --- Compute reaction force:
!         y_p = S_pf x_f + S_pp x_p. y_f is left unchanged
!         y_i = S_ii*x_i + S_ip*x_p; y_b unchanged

         if ( Smat%complexmat .or. ks(ix)%compl==1 ) then
            allocate ( yctemp(nusol) )
            yctemp = 0d0
            call sepmultbcond2 ( Smat, xci, yctemp )
            yci(1:nusol) = yci(1:nusol)+yctemp(1:nusol)
            deallocate ( yctemp )
         else
            allocate ( ytemp(nusol) )
            ytemp = 0d0
            call sepmultbcond1 ( Smat, xri, ytemp )
            yri(1:nrusol) = yri(1:nrusol)+ytemp(1:nrusol)
            deallocate ( ytemp )
         end if  ! ( Smat%compl )
         call  sepmultreac1 ( Smat, xri, yri )

      end select ! case (ichoice)

!     --- Renumber back and  delete local arrays

      if ( ichoice<=10 ) call sepmaverp2 ( ix, iy, ichoice )

      if ( parallel .and. ichoice>7 .and. associated(sendwork) ) then

         deallocate ( xglob, sendwork, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'sendwork' )

      end if  ! ( parallel )

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( ks(iy)%sol, nusol, 'y' )
         write(irefwr,*) 'End sepmaver'

      end if  ! ( debug )
      call erclos ( 'sepmaver' )

      end
