      subroutine sepnestdec ( A )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-06-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute lU decomposition for system of real linear equations
!     Use Gauss elimination (nested dissection)
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     real
!     direct_method
!     nested_dissection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesolve
      use sepmoduleintmat
      use sepmodulemat
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix) :: A

!     A             i/o   Matrix A
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPNESTDECLLT  Compute L D L^T decomposition for system of real linear
!                    equations (nested dissection)
!                    Symmetric matrix
!     SEPNESTLU      Compute the preconditioning matrix in case of row-compact
!                    storage. Incomplete LU-decomposition
!                    Real matrix
!     SEPPRECIGGT    compute the preconditioning matrix for the CG-process
!                    iprec = 3,  Incomplete LU-decomposition
!     SEPPRINTCOND   Print information about the condition number of
!                    the large matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepnestdec' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepnestdec'
         write(irefwr,1) 'ipos', ipos
  1      format ( a, 1x, (10i8) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(intmat)
      if ( debug ) write(irefwr,*) 'IM%symmetric ', IM%symmetric

!     --- Compute decomposition

      if ( IM%symmetric ) then

!     --- symmetric matrix

         if ( ipos==1 ) then

!        --- Matrix positive definite

            copyprec = .false.
            call seppreciggt ( 1d60, A )

         else

!        --- General symmetric matrix

            call sepnestdecllt ( A )

         end if  ! ( ipos==1 )

      else

!     --- asymmetric matrix

         call sepnestlu ( A )

      end if  ! ( IM%symmetric )

      if ( A%iprint>0 ) call sepprintcond ( A )

      A%decomp = .false.
      lenpremat = 1

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepnestdec'

      end if  ! ( debug )
      call erclos ( 'sepnestdec' )

      end subroutine sepnestdec
