      subroutine sepparglob ( A, rhsd, n )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-03-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Send information to neighboring processors in order to give all
!     local arrays the global value in the common points
!     Old subroutine prparglob1
! **********************************************************************
!
!                       KEYWORDS
!
!     parallel
!     global
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematstr
      use sepmodulemat
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix) :: A
      integer :: n
      double precision, intent(inout) :: rhsd(n)

!     A              o    Structure of type matrix containing the necessary
!                         information of the matrix
!     n              i    Size of array rhsd
!     rhsd          i/o   Real vector of length n containing the global
!                         vector to be used in parallel computations
!                         At input it contains the local values
!                         At output it contains the global values
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: j, k, numneigh, nneigh, sendstart, iblock, &
              seqblocks(iacnodes), icontext
      double precision :: usend(n)
      integer, pointer :: send_buffer(:)
      integer, save :: send_buffer_size

!                    they are not
!     iblock         Block sequence number
!     icontext       Tag for sending and receiving, is used as check
!     j              loop counter
!     k              loop counter
!     nneigh         Number of overlapping unknowns
!     numneigh       number of neighbouring blocks
!     send_buffer    send buffer
!     send_buffer_size  Length of the send buffer
!     sendstart      starting address in array sendinfo
!     seqblocks      Work array of length nblocks (=nsurfs) which
!                    is used to find the sequence of block numbers
!                    in array blockunkinfo
!                       seqblocks(iblock) = 0: block iblock not present
!                       seqblocks(iblock) = i: block iblock is the i-th block
!                    in blockunkinfo
!     usend          Work array of length n to store the double precision data
!                    to be send to another block
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
!     INIBARRIERMPI  Synchronize parallel sending and receiving (mpi)
!     INIBSENDMPI    Send information to other node (mpi buffered)
!     INIBUFATTMPI   Attach mpi buffer if it exists
!     INIBUFDETMPI   Detach mpi buffer if it exists
!     INIBUFSIZEMPI  Extend send_buffer_size with mpi overhead
!     INIRECEIVEMPI  Receive information from other node (mpi)
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepparglob' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepparglob'
         call ertrace
         write(irefwr,1) 'n', n
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      icontext = 93

      numneigh = blockunkinfo(1)
      if ( debug ) then
         write(irefwr,1) 'numneigh', numneigh, size(A%sendinfo)
         call prinin ( blockunkinfo, 3*numneigh, 'blockunkinfo' )
         call prinin ( A%sendinfo, min(10,size(A%sendinfo)), 'sendinfo' )
      end if

!     --- Store sequence of block numbers in array seqblocks

      seqblocks = 0

      do j = 1, numneigh
         iblock =  blockunkinfo(2*j)
         seqblocks(iblock) = j
      end do

!     --- Synchronize communication by calling mpi_barrier
!         This avoids that something is written to other nodes, &
!         before reading by those other nodes has been done

      call inibarriermpi

      send_buffer_size = 0
      do j = 1, numneigh
         sendstart = blockunkinfo(2*j+1)
         nneigh = A%sendinfo(sendstart)
         if ( debug ) &
            write(irefwr,1) 'j, sendstart, nneigh', &
                             j, sendstart, nneigh
         send_buffer_size = send_buffer_size + nneigh*8
      end do

!     --- Compute actual buffer size using mpi information
!         allocate array and attach buffer

      call inibufsizempi ( send_buffer_size, numneigh )

      allocate(send_buffer(send_buffer_size))

      call inibufattmpi ( send_buffer, send_buffer_size )

      do j = 1, numneigh
         iblock = blockunkinfo(2*j)
         sendstart = blockunkinfo(2*j+1)
         nneigh = A%sendinfo(sendstart)
         do k = 1, nneigh
            usend(k) = rhsd(A%sendinfo(sendstart+k))
         end do

         if ( debug ) then
            write(irefwr,1) 'j, iblock, nneigh', j, iblock, nneigh
            write(irefwr,1) 'sendstart', sendstart
            call prinrl ( usend, nneigh, 'usend/1' )
         end if  ! ( debug )

!        --- Send array usend

         call inibsendmpi ( usend, nneigh, SPDOUBLE, iblock, icontext )

      end do

      call inibarriermpi

      do j = 1, numneigh

!     --- Receive next array and store in usend
!         Get corresponding block number

         nneigh = n
         call inireceivempi ( usend, nneigh, SPDOUBLE, iblock, &
                              icontext )

         sendstart = blockunkinfo(2*seqblocks(iblock)+1)

         if ( debug ) then
            write(irefwr,1) 'j, iblock, nneigh', j, iblock, nneigh
            write(irefwr,1) 'sendstart', sendstart
            call prinrl ( usend, nneigh, 'usend/2' )
         end if  ! ( debug )

         do k = 1, nneigh
            rhsd(A%sendinfo(sendstart+k)) = &
                rhsd(A%sendinfo(sendstart+k)) + usend(k)
         end do

      end do

      if (associated(send_buffer)) then

!     --- deattach send_buffer and deallocate

         call inibufdetmpi ( send_buffer, send_buffer_size )
         deallocate(send_buffer)

      end if  ! (associated(send_buffer))

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepparglob'

      end if  ! ( debug )
      call erclos ( 'sepparglob' )

      end subroutine sepparglob
