      subroutine sepprecildlt ( amax, A )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-03-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute the preconditioning matrix for the SYMMLQ-process
!     iprec = 3,  Incomplete LDL^T-decomposition
!     Old subroutine prcg01precldlt
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     preconditioning
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulesolve
      use sepmodulematstr
      use sepmodulemat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix) :: A
      double precision, intent(in) :: amax

!     A             i/o   Structure of type matrix containing the necessary
!                         information of the matrix
!     amax           i    maximal element of a matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, k, j, l, mj, ml, id, kd
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!                    they are not
!     i              Counting variable
!     id             Position of diagonal element i
!     j              Help variable to store pointers
!     k              Counting variable
!     kd             Position of diagonal elementk
!     l              Help variable to store pointers
!     mj             Help variabe to store positions in A%cols
!     ml             Help variabe to store positions in A%cols
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     SEPCOPYTOPREC5 Copy matrix in preconditioning matrix
!     SEPPRINMAT6    Print matrix that is stored as row-compact real matrix
!     SEPPRINMATDC6  Print preconditioning matrix that is stored as
!                    row-compact real matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     674   The desired preconditioning matrix does not exist
!    2236   The desired preconditioning matrix does not exist, lumping is tried
!    2419   Inconsistency in matrix structure
! **********************************************************************
!
!                       PSEUDO CODE
!
!       first step ( explained on complete decomposition ):
!         goal: A = L D L^T, diag(L)=1
!         do i=1,n
!           do j=1,i-1
!              L(i,j) = [ A(i,j) - sum L(i,k)*d(k,k)*L(j,k) ] / D(j)
!                                  k<j
!           end do
!           D(i) = [ A(i,i) - sum L(i,k)*d(i,i)*L(i,k) ]
!                             k<i
!         end do
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepprecildlt' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepprecildlt'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )
         call sepprinmat6 ( A )

      end if
      if ( ierror/=0 ) go to 1000

      IM => intmt(iseqim)

!     --- Copy matrix into preconditioning matrix

300   if ( copyprec ) call sepcopytoprec5 ( A )

!     --- Adapt diagonal in case of shifted laplace

      if ( IM%lapshift ) A%MD(A%diag) = A%MD(A%diag)-(1d0+lapshift)*A%DLshift

      if ( A%adapt>0d0 ) A%MD(A%Mdiag) = max(A%MD(A%Mdiag),A%adapt)

      if ( A%prlump==1 ) then

!     --- Positive off-diagonal elements are lumped to the main
!         diagonal

         do i = 1, A%neq
            if ( A%rows(i)<A%Mrows(i+1)-1 ) then
               id = A%Mdiag(i)
               do k = A%Mrows(i) + 1, A%Mrows(i+1)-1
                  if ( A%MD(k)>0d0 ) then
                     A%MD(id) = A%MD(id) + A%MD(k)
                     kd = A%Mdiag(A%Mcols(k))
                     A%MD(kd) = A%MD(kd) + A%MD(k)
                     A%MD(k) = 0d0
                  end if
               end do
            end if
         end do

      end if

!     --- end of lumping

      do i = 1, A%neq

         if ( debug ) &
            write(irefwr,1) 'i, A%Mrows(i), A%Mrows(i+1)', &
                             i, A%Mrows(i), A%Mrows(i+1)
         id = A%Mdiag(i)
         if ( A%Mrows(i)<A%Mrows(i+1)-1 ) then

            if ( A%Mcols( A%Mrows(i)+1 )>=i ) then
               call errint ( A%Mcols( A%Mrows(i)+1 ), 1 )
               call errint ( i, 2 )
               call errsub ( 2419, 2, 0, 0 )
            end if

!           --- l_i1 = a_i1/d11
!               d_ii = d_ii - l_i1^2/d11

            k = A%Mrows(i)+1   ! Position of first non-zero column in row i
            kd = A%Mdiag(A%Mcols(k))
            A%MD(k) = A%MD(k)/A%MD(kd)
            A%MD(id) = A%MD(id) - A%MD(k)**2 * A%MD(kd)

            if ( debug ) then

!           --- Debug information

               write(irefwr,1) 'k, kd', k, kd
               write(irefwr,2) 'A%MD(k), A%MD(kd)', A%MD(k), A%MD(kd)

            end if  ! ( debug ) then

!           --- Loop over rest of elements in row

            do k = A%Mrows(i) + 2, A%Mrows(i+1)-1
               j = A%Mrows(i)+1
               l = A%Mrows ( A%Mcols(k) ) + 1
               if ( debug ) &
                  write(irefwr,1) 'k, j, l, A%Mrows( A%Mcols(k)+1 )', &
                                   k, j, l, A%Mrows( A%Mcols(k)+1 )
310            if ( j<=k-1 .and. l<=A%Mrows( A%Mcols(k)+1 ) ) then
                  mj = A%Mcols(j)
                  ml = A%Mcols(l)
                  if ( debug ) write(irefwr,1) 'j, l, mj, ml', &
                                                j, l, mj, ml
                  if ( mj==ml ) then

!                 --- mj = ml

                     A%MD(k) = A%MD(k) - A%MD(j) * A%MD(l) * &
                                A%MD(A%Mdiag(mj))
                     if ( debug ) then
                        write(irefwr,1) 'A%Mcols(mj)', A%Mcols(mj)
                        write(irefwr,2) 'A%MD(k)', A%MD(k)
                     end if  ! ( debug )
                  end if  ! ( mj==ml )

                  if ( mj<=ml ) j = j+1
                  if ( mj>=ml ) l = l+1
                  go to 310
               end if

               if ( A%Mcols(k)>=i ) then
                  call errint ( A%Mcols(k), 1 )
                  call errint ( k, 2 )
                  call errsub ( 2419, 2, 0, 0 )
               end if

               kd = A%Mdiag(A%Mcols(k))
               A%MD(k) = A%MD(k) / A%MD(kd)
               A%MD(id) = A%MD(id) - A%MD(k) ** 2*A%MD(kd)
               if ( debug ) then
                  write(irefwr,1) 'A%Mcols(k)', A%Mcols(k)
                  write(irefwr,2) 'A%MD(k), A%MD(i)', &
                                   A%MD(k), A%MD(i)
               end if  ! ( debug )
            end do
         end if

         if ( A%prlump/=1 .and. &
              ( A%MD(id)<=0d0 .or. A%MD(id)>=amax ) ) then

!        --- If A%MD(i) becomes too small or too big make prlump = 1
!            and start again.

             call errwar ( 2236, 0, 0, 0)
             A%prlump = 1
             goto 300

         end if

!        --- If A%MD(i) becomes too small or too big and prlump = 1
!            give an error message and stop

         if ( A%MD(id)<=0d0 ) then
            call errint ( i, 1 )
            call erreal ( A%MD(id), 1 )
            call errsub ( 674, 1, 1, 0)
            go to 1000
         end if

      end do

!     --- |   now ( explained on complete decomposition):        |
!         |      A  =  ( L + D ) * ( D + U ), diag(L)=diag(U)=0  |
!         |                                                      |
!         |   let  L = D * low,  U = upp * D, then               |
!         |                                                      |
!         |      A  =  D * ( low + 1 ) * ( 1 + up ) * D ,        |
!         |                                                      |
!         |   which is suitable for later calculations           |
!         |                                                      |
!         |   D should be stored as it is,  strictly triangular  |
!         |   matrices  low = inv(D) * L,  upp = U * inv(D),     |
!         |   will be passed to the calling process:             |

1000  if ( debug ) then

!     --- Debug information

         call sepprinmatdc6 ( A )
         write(irefwr,*) 'End sepprecildlt'

      end if
      call erclos ( 'sepprecildlt' )

      end subroutine sepprecildlt
