      subroutine sepprofsolun ( A, usol, rhsd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-03-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     solve system of linear equations by gaussian elimination
!     where the matrix has been decomposed by subroutine sepprofun
!     This implies that the matrix is supposed to be a real
!     profile matrix
!     Old subroutine prspun
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     real
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix) :: A
      double precision :: usol(A%neq)
      double precision, intent(in) :: rhsd(A%neq)

!     A              i    Matrix A
!     rhsd           i    right-hand-side vector
!     usol           o    solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: pj, i, ii

!     i              Counting variable
!     ii             Position of diagonal element ii
!     pj             Column number of at most left non-zero element in row i
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision :: ddot

!     DAXPY          Overwrite double precision dy with double precision
!                    da*dx + dy
!     DDOT           (CDOT)     Inner product of two vectors
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!      The L U decomposition of the matrix A has been computed by
!      subroutine PROFCS
!      Now the system of equations:   A x = f must be solved, hence:
!
!                                                    T
!         L  U  x = f   or:     U  x  =  y
!                               L  y  =  f
!
!
!      First:  L y = f
!
!             i                                   i-1
!      f  =  sum   l    y      or:    y   = f   - sum   l   y
!       i    k=1    ik   k             i     i    k=P    ik  k
!                                                    i
!
!
!      Finally  U x = y
!
!             n                                      n
!      y  =  sum   u    x      or:    x   = ( y   - sum   u   x ) / u
!       i    k=i    ik   k             i       i    k=i+1  ik  k     ii
!
!
!      Because of the profile storage it is not easy to get u   so the
!                                                            ik
!      order must be reversed:   x:=y
!
!      i = n (-1) 1
!
!         x  = x  / u
!          i    i    ii
!
!         j = P  (1) i-1
!              i
!
!             x  = x  - u   x
!              j    j    ji  i
!
!
!
!      Hence the final algorithm becomes:
!
!
!      For i := 1 (1) n
!
!                   i-1
!         x  = f  - sum  l   x
!          i    i   k=P   ij  j
!                      i
!
!      end for i
!
!
!      For i := n (-1) 1
!
!         x  = x  / u
!          i    i    ii
!
!         For j := P  (1) i-1
!                   i
!
!            x  = x  - u   x
!             j    j    ji  i
!
!         end for j
!
!      end for i
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepprofsolun' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepprofsolun'
         call ertrace
         write(irefwr,1) 'n', A%neq
         write(irefwr,*) 'associated(A%Mrows) ', &
                          associated(A%Mrows), size(A%Mrows)
         write(irefwr,*) 'associated(A%Mcols) ', &
                          associated(A%Mcols), size(A%Mcols)
         write(irefwr,*) 'associated(A%MD) ', &
                          associated(A%MD), size(A%MD)
    !     call sepprinmat2 ( A )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!      --- Loop over rows

      do i = 1, A%neq

         pj = A%Mcols(i)
         if ( pj>=i ) then

!        --- pj >= i, no summation

            usol(i) = rhsd(i)

         else

!        --- pj < i, summation

            usol(i) = rhsd(i) - ddot ( i-pj, A%MD(A%Mrows(i)-i+pj), &
                                        1, usol(pj), 1 )

         end if

      end do

!     --- Finally the back substitution
!         Hence Ux = y

      do i = A%neq, 1, -1
         ii = A%Mrows(i)
         usol(i) = usol(i) / A%MD(ii)
         pj = A%Mcols(i)
         if ( i>pj ) call daxpy ( i-pj, -usol(i), &
                          A%MD(ii+1), -1, usol(pj), 1 )
      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepprofsolun'

      end if  ! ( debug )
      call erclos ( 'sepprofsolun' )
      end subroutine sepprofsolun
