      subroutine sepprofun ( A )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-03-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute the L U  decomposition of a non-symmetric real matrix.
!     The matrix is supposed to be stored as a profile matrix.!
!     Old subroutine profun
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     real
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesolve
      use sepmodulemat
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix) :: A

!     A             i/o   Matrix A
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: pi, p, ii, k1, k2, i, j, jj, ij, ji
      double precision :: rowsum

!     i              Counting variable
!     ii             Position of i th diagonal element in array A%MD
!     ij             Position of element ij in array A%MD
!     j              Counting variable
!     ji             Position of element ji in array A%MD
!     jj             Position of j th diagonal element in array A%MD
!     k1             Help variable to store a quantity
!     k2             Help variable to store a quantity
!     p              Max (pi, pj)
!     pi             Column number of at most left non-zero element in row i
!     rowsum         Row norm of matrix
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision dasum, ddot

!     DASUM          Blas subroutine computing the sum of absolute values of
!                    vector
!     DDOT           (CDOT)     Inner product of two vectors
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     SEPPRINMAT2    Print inverse matrix that is stored as profile real matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     104   matrix has small pivots
!    1535   matrix is singular
!    1820   The matrix found in the solver is completely zero
! **********************************************************************
!
!                       PSEUDO CODE
!
!      The L U   decomposition of the matrix A is based on the following
!      formulas:
!              n
!      a   =  sum   l    u                  l   = 1
!       ij    k=1    ik   kj                 kk
!      this implies:
!                     j-1
!      u   = a   -    sum      l   u       ( j=P  (1) i-1 )
!       ji    ji   k=max(P ,P ) jk  ki          i
!                         i  j
!                        j-1
!      l   = ( a   -     sum      l    u   ) / d      ( j=P  (1) i-1 )
!       ij      ij   k=max(P ,P )  ik   kj      jj         i
!                           i  j
!                         i-1
!      u  = d   =   a   - sum  l    u
!       ii   ii      ii   k=P   ik   ki
!                            i
!      Here P   denotes the column number of the first non-zero element in row i
!            i
!      The decomposition is stopped if |d  |<||A|| * epsmac
!                                        ii
!      For the matrix norm we use the maximum row norm, i.e.
!      ||A|| = max  sum  |a  |
!               i    j     ij
!     Pseudo code
!     Compute row norm of matrix and multiply by machine accuracy
!     For i = 1 (1) n
!         For j = P  (1) i-1
!                  i
!             P = max ( P , P )
!                        i   j
!             If ( P<j ) then
!                 adapt u   with inner product
!                        ji
!                 adapt l   with inner product
!                        ij
!             l   = l   / d
!              ij    ij    ii
!         adapt d   with inner product
!                ii
!         if (d  <||A|| epsmac ) then
!              ii
!            Give error message and stop the program
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepprofun' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepprofun'
         write(irefwr,1) 'nrusol', nrusol
         call sepprinmat2 ( A )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- compute row norm of part of the matrix in order to be able to check
!         the stopping criterion for the pivot process

      rowsum = 0d0
      do  i = 1, nrusol
         ii = A%Mrows(i)
         k1 = i-A%Mcols(i)
         rowsum = max ( rowsum, dasum (2*k1+1, A%MD(ii-k1), 1) )
      end do
      if ( rowsum==0d0 ) then
         call errsub ( 1820, 0, 0, 0 )
         go to 1000
      end if
      rowsum = rowsum*epsmac
      if ( debug ) write(irefwr,2) 'rowsum', rowsum

!     --- decomposition algorithm
!         Row loop

      do i = 1, nrusol

         ii = A%Mrows(i)
         pi = A%Mcols(i)
         if ( pi<i ) then

!        --- pi<i

            k1 = ii+i
            k2 = ii-i

!           --- Loop over columns

            do j = pi, i-1

               p = max ( pi, A%Mcols(j) )
               jj = A%Mrows(j)
               ij = k2+j
               if ( p<j ) then

!              --- p<j

                  ji=k1-j
                  A%MD(ji) = A%MD(ji) - ddot ( j-p, A%MD(jj-j+p), 1, &
                                        A%MD(ji+1), -1 )
                  A%MD(ij) = A%MD(ij) - ddot ( j-p, A%MD(k2+p), 1, &
                                        A%MD(jj+1), -1 )
               end if
               A%MD(ij) = A%MD(ij)/A%MD(jj)

            end do

!           --- Diagonal element

            A%MD(ii) = A%MD(ii) - ddot ( i-pi, A%MD(k2+pi), &
                                  1, A%MD(ii+1), -1 )
         end if

!        --- Check pivot element:

         if ( skipzeros ) then

!        --- skipzeros = true, hence if the diagonal element is too small
!            it is replaced by the machine accuracy (scaled by rowum)

            if ( abs(A%MD(ii))<rowsum ) A%MD(ii) = rowsum

         else

!        --- skipzeros = false, check singular matrix

            if ( abs(A%MD(ii))<=0d0 ) then

!           --- matrix is singular, error

               call errint ( i, 1 )
               call errsub ( 1535, 1, 0, 0 )
               go to 1000

            else if ( abs(A%MD(ii))<rowsum ) then

!           --- matrix is probably singular, warning

               call errint ( i, 1 )
               call erreal ( A%MD(ii), 1 )
               call erreal ( rowsum, 2 )
               call errwar ( 104, 1, 2, 0 )

            end if

         end if  ! ( skipzeros )

      end do  ! i = 1, nrusol

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'L U decomposition'
         call sepprinmat2 ( A )
         write(irefwr,*) 'End sepprofun'

      end if  ! ( debug )
      call erclos ( 'sepprofun' )

      end subroutine sepprofun
