      subroutine sepscalmatbck ( A, diag, x, y )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-04-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Rescale the original large sparse matrix, &
!     the right-hand side and if necessary the solution vector
!     with the the diagonal of the preconditioning matrix.
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     iterative
!     linear_solver
!     preconditioning
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemat
      use sepmodulematstr
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix), intent(inout) :: A
      double precision, intent(inout) :: x(A%neq), y(A%neq)
      double precision, intent(inout) :: diag(A%neq)

!     A             i/o   Matrix A
!     diag          i/o   Diagonal of scaling matrix
!     x             i/o   Solution vector
!     y             i/o   Right-hand side
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iprec, n, i, j, idegfd
      double precision :: h
      double precision, pointer :: prdiagsav(:)

!     h              Help variable to store a constant temporarily
!     i              General loop variable
!     idegfd         Column number
!     iprec          Type of preconditioning used. Possible values:
!                    0    No preconditioning
!                    1    Diagonal scaling
!                    2    Eisenstat preconditioning
!                    3    Incomplete LU decomposition
!                    4    Symmetric Gauss-Seidel preconditioning
!                    5    Pre-assembled preconditioning according to
!                         Kaasschieter
!                    7    Eisenstat preconditioning (MILU variant)
!     j              General loop variable
!     n              number of equations
!     prdiagsav      A copy of the diagonal of the preconditioning matrix
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPPARADAPPDIAG Copy diagonal of preconditioning matrix in prdiagsav
!                    Adapt diagonal of preconditioning matrix
!     SEPSCALMATPRBC Rescale the projection vectors with the inverse of the
!                    diagonal of the preconditioning matrix.
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2917   Option not available
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepscalmatbck' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepscalmatbck'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iprec = A%type_prec  ! type of preconditioning
      n = A%neq  ! number of rows

      if ( debug ) then
         write(irefwr,1) 'iprec, n, A%scaling', iprec, n, A%scaling
      end if  ! ( debug )

      if ( parallel .and. typecoupl==0 ) then

!     --- a copy is made of the local array A%MD into prdiagsav
!         communication is established
!         allocate space for work array

         allocate ( prdiagsav(n), stat = error )
         if ( error/=0 ) call eralloc ( error, n, 'prdiagsav' )
         if ( ierror/=0 ) go to 1000

         call sepparadappdiag ( diag, prdiagsav, A )

      end if

!     --- iprec = 2, 4, 7, Eisenstat version, recompute the diagonal

      if ( iprec==2 .or. iprec==4 .or. iprec==7 ) then

!     --- IPREC =2,  Eisenstat   Change diagonal of the large matrix

         do i = 1, A%neq
            j = A%diag(i)
            A%S(j) = A%S(j) + 2d0
         end do  ! i = 1, A%neq

      end if

!     --- Recompute right-hand side, solution vector and diagonal

      if ( A%scaling==2 ) then

!     --- scaling = 2, symmetric scaling

         do i = 1, A%neq
            j = A%diag(i)
            A%S(j) = A%S(j)*diag(i)**2
         end do  ! i = 1, A%neq

      else

!     --- scaling = 1, 3, row or column scaling

         do i = 1, A%neq
            j = A%diag(i)
            A%S(j) = A%S(j)*diag(i)
         end do  ! i = 1, A%neq

      end if

!     --- row or symmetric scaling, pre-multiply right-hand side

      if ( A%scaling<=2 ) y = y * diag

!     --- column or symmetric scaling, pre-multiply solution vector

      if ( A%scaling>=2 ) x = x / diag

!     --- Rescale rest of matrix

      select case ( A%scaling)

         case(1)

!         --- iscaling = 1, row scaling

            do i = 1, n

               h = diag(i)
               do j = A%rows(i)+1,A%diag(i)-1
                  A%S(j) = A%S(j) * h
               end do  ! j = A%rows(i)+1,A%diag(i)-1
               do j = A%diag(i)+1,A%rows(i+1)
                  A%S(j) = A%S(j) * h!diag(A%cols(j))
               end do  ! A%diag(i)+1,A%rows(i+1)

            end do  ! i = 1, n

         case(2)

!         --- iscaling = 2, symmetric scaling

            do i = 1, n

               h = diag(i)
               do j = A%rows(i)+1,A%diag(i)-1
                  A%S(j) = A%S(j) * (h*diag(A%cols(j)))
               end do  ! j = A%rows(i)+1,A%diag(i)-1
               do j = A%diag(i)+1,A%rows(i+1)
                  A%S(j) = A%S(j) * (h*diag(A%cols(j)))
               end do  ! A%diag(i)+1,A%rows(i+1)

            end do  ! i = 1, n

         case(3)

!         --- iscaling = 3, column scaling

            do i = 1, n

               do j = A%rows(i)+1,A%diag(i)-1
                  A%S(j) = A%S(j) * diag(A%cols(j))
               end do  ! j = A%rows(i)+1,A%diag(i)-1
               do j = A%diag(i)+1,A%rows(i+1)
                  A%S(j) = A%S(j) * diag(A%cols(j))
               end do  ! A%diag(i)+1,A%rows(i+1)

            end do  ! i = 1, n

         case default

            call errint ( A%scaling, 1 )
            call errsub ( 2917, 1, 0, 0 )

      end select  ! case ( A%scaling)

      if ( A%nproj>0 ) call sepscalmatprbc ( A, diag )

      if ( parallel .and. typecoupl==0 ) then

!     --- the local array prdiag is copied back from prdiagsav

         diag = prdiagsav

         deallocate ( prdiagsav, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'prdiagsav' )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepscalmatbck'

      end if  ! ( debug )
      call erclos ( 'sepscalmatbck' )

      end subroutine sepscalmatbck
