      subroutine sepsimplegcr ( normrhsd, A, usol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-04-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Solve a system of linear equations of special form using the SIMPLE method
!     The method is accelerated by the GCR process
!     Old subroutine prsimplegcr
! **********************************************************************
!
!                       KEYWORDS
!
!     simple
!     linear_solver
!     matrix
!     real
!     krylov
!     conjugate_gradients
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecpu
      use sepmodulesimple
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision normrhsd
      type (rmatrix) :: A
      double precision :: usol(A%neq)

!     A             i/o   Structure of type matrix containing the necessary
!                         information of the matrix
!     normrhsd       i    Norm of right-hand side vector
!     usol           o    solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer k, iter, niteru, niterp, &
              itimesav, niteruacc, niterpacc, &
              niteracc, niterutot, niterptot, ngcr
      double precision, allocatable :: svec(:,:), vvec(:,:), work(:), &
                                       rhsup(:), ps(:), rud(:)
      double precision rnorm, rferr, normprec
      logical converged, extradebug
      double precision :: timebefore(2)

!     converged      If true the process is converged
!     iter           Iteration sequence number
!     itimesav       Help parameter to save the parameter itime
!     k              Counting variable
!     ngcr           maximum number of GCR iterations
!     niteracc       Accumulated number of GCR iterations
!     niterp         Number of iterations to solve the p-equations
!     niterpacc      Accumulated number of iterations to solve the p-equations
!     niterptot      Total number of p iterations in all GCR steps
!     niteru         Number of iterations to solve the u-equations
!     niteruacc      Accumulated number of iterations to solve the u-equations
!     niterutot      Total number of u iterations in all GCR steps
!     normprec       Norm of preconditioned residual
!     ps             Work array to store p* &
!     rferr          Maximal accuracy possible for this matrix, due to
!                    round-off
!     rhsup          Right-hand side vector
!     rnorm           norm of residual
!     rud            Work array of length nu+np to store right-hand side
!                    of intermediate equation
!     svec           Work array of size ngcr x n_u to store the vectors
!                    s during the iteration
!     vvec           Work array of size ngcr x n_u to store the vectors
!                    v during the iteration
!     work           Work array of length nu to store intermediate vectors
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKCLK         print elapsed time
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     ERSETTIME      Get the CPU time at start of subroutine
!     PRINRL         Print 1d real vector
!     SEPGCRCYCLE    Perform one step of the gcr process
!     SEPLSCITER     Perform a lsc iteration as part of some outer loop
!     SEPSIMPLECONVG Compute norm of residual and adapt accuracy if necessary
!     SEPSIMPLEITER  Perform a simple iteration as part of some outer loop
!     SEPSIMPLESTART Compute initial norm, adapted eps and print start
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     676   The CG-process has terminated because the number of iterations
!           exceeds the maximum value
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Start with The matrices Q, R, D and the vectors u, p, ru and r_p
!     For k := 0 (1) ngcr do
!        Perform one step of simple to compute du and dp
!        s^(k+1) := (du,dp), r = (ru,r_p)
!        v^(k+1) := A s^(k+1)
!        For i := 1 (1) k do
!           alpha := ( v^(k+1), v^i)
!           v^(k+1) := v^(k+1)- alpha v^i
!           s^(k+1) := s^(k+1)- alpha s^i
!        v^(k+1) := v^(k+1)/||v^(k+1)||
!        s^(k+1) := s^(k+1)/||v^(k+1)||
!        alpha := ( r, v^(k+1))
!        (u,p) := (u,p) + alpha s^(k+1)
!        r := r - alpha v^(k+1)
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data niteruacc /0/, niterpacc /0/, niteracc /0/
! ======================================================================
!
      call eropen ( 'sepsimplegcr' )
      call ersettime ( timebefore )
      debug = .false.
      extradebug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepsimplegcr'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ngcr = inpsol(13,1)   ! (maximum) number of gcr iterations
      allocate(svec(nu+np,0:ngcr+1) , vvec(nu+np,0:ngcr+1), work(nu+np), &
               rhsup(nu+np), ps(np), rud(nu+np), stat = error)
      if ( error/=0 ) call eralloc ( error, (nu+np)*(ngcr+3), 'svec' )
      if ( ierror/=0 ) go to 1000

      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'nu, np ', nu, np
         if ( extradebug ) call prinrl ( ru, nu+np, 'ru' )

      end if  ! ( debug )

      converged = .false.
      itimesav = itime
      itime = 0
      niterutot = 0
      niterptot = 0

!     --- Print heading if iprint>0

      if ( A%iprint>0 ) then
         if ( inpsol(3,1)==6 ) then
            write ( irefwr, 100 )   ! simple
100         format(//' Output for the Simple method'//)
         else if ( inpsol(3,1)==8 ) then
            write ( irefwr, 101 )   ! simpler
101         format(//' Output for the Simpler method'//)
         else if ( inpsol(3,1)==12 ) then
            write ( irefwr, 102 )   ! hsimple
102         format(//' Output for the HSimpler method'//)
         else if ( inpsol(3,1)==13 ) then
            write ( irefwr, 103 )   ! msimple
103         format(//' Output for the MSimpler method'//)
         else if ( inpsol(3,1)==14 ) then
            write ( irefwr, 104 )   ! lsc
104         format(//' Output for the LSC method'//)
         else
            write ( irefwr, 105 )   ! qlsc
105         format(//' Output for the QLSC method'//)
         end if  ! ( inpsol(3,1)==6 )
      end if  ! ( A%iprint>0 ) then

!     --- Compute initial norm, adapted eps and print start
!         Produce matlab output if required

      call sepsimplestart ( rnorm, rferr, normrhsd, A )

      if ( extradebug ) call prinrl (resup, nu+np, 'ru/1' )

      if ( A%irelerr==2 .and. rnorm<=A%eps .or. normrhsd<1d-30 ) then

!     --- Initial residual already small

         go to 1000

      end if

!     --- For k := 0 (1) maxiter-1 do

      do k = 0, A%nmax-1

!     --- Perform one step of simple to compute du and dp
!         s^(k+1) := (du,dp), r = (ru,r_p)

         iter = k
         if ( A%typesolv<14 ) then

!        --- SIMPLE iteration

            call sepsimpleiter ( svec(1,0), normrhsd, niterp, &
                                 iter, work, rhsup, ps )

         else

!        --- LSC iteration

            call seplsciter ( svec(1,0), normrhsd, niterp, &
                              work, rhsup, ps, rud )

         end if  ! ( inpsol(3,1)<14 )
         if ( ierror/=0 ) go to 1000

         if ( extradebug ) call prinrl (ru, nu+np, 'ru/2' )
         niteru = Q%numiter       ! number of iterations carried out
         niterutot = niterutot+niteru
         niterptot = niterptot+niterp

         normprec = norm ( svec(1,0), A )
         if ( debug ) write(irefwr,2) 'normprec', normprec

         if ( iter==0 .and. A%irelerr==1 ) then

!        --- ireler = 1 and iter=0, we have to adapt the accuracy

            A%eps = A%eps*normprec

         end if  ! ( iter==0 .and. A%irelerr==1 )

!        --- Perform one step of the gcr process.

         vvec(:,0) = A*svec(:,0)
         if ( ierror/=0 ) go to 1000

         call sepgcrcycle ( svec, vvec, ngcr, k, rnorm, resup, usol, A )
         if ( ierror/=0 ) go to 1000

  !       if ( debug ) call prinrl (ru, nu+np, 'ru/3' )

         call sepsimpleconvg ( iter, rferr, converged, niteru, niterp, &
                               rnorm, normprec, A )

         if ( converged ) go to 500

      end do  ! k = 1, ngcr

!     --- If the subroutine reaches this stage, too much iterations have
!         been carried out

      call errint ( iter, 1 )
      if ( inpsol(22,1)==1) then
         inpsol(1,1) = iter
         call errwar ( 676, 1, 0, 0 )
      else
         call errsub ( 676, 1, 0, 0 )
      end if

500   niteracc = niteracc + iter
      niteruacc = niteruacc + niterutot
      niterpacc = niterpacc + niterptot

      if ( A%iprint>=1 ) then

!     --- iprint = 1  Print information

         write ( irefwr, 510 ) iter, rnorm, niterutot, niterptot, &
                               niteracc, niteruacc, niterpacc
510      format('The number of executed iterations in the', &
                ' GCR-process is', i5/ &
                'The 2-norm of the residual is:', ss,1pe12.2e2/ &
                'The number of u iterations is ', i6/ &
                'The number of p iterations is ', i6/ &
                'The accumulated number of GCR-iterations is', i8/ &
                'The accumulated number of u-iterations is', i8/ &
                'The accumulated number of p-iterations is', i8 )

      end if

!     --- Fill information in citerinfo
!         This is only used for performance checking

      niterout = niteracc
      niteruout = niteruacc
      niterpout = niterpacc
      call askclk
      timesubr = tim3-timebefore(1)

1000  deallocate ( svec, vvec, work, rhsup, ps, rud, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'svec' )

      if ( debug ) then
         if ( extradebug ) call prinrl ( usol, nu, 'u' )
         if ( extradebug ) call prinrl ( usol(nu+1), np, 'p' )
         write(irefwr,*) 'End sepsimplegcr'
      end if  ! ( debug )
      itime = itimesav

      call erclos ( 'sepsimplegcr' )

      end
