      subroutine sepsimplemat ( A )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-04-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create the matrices as well as the preconditioning matrices with respect
!     to the SIMPLE algorithm
!     Old subroutine prsimplemat
! **********************************************************************
!
!                       KEYWORDS
!
!     simple
!     matrix
!     preconditioning
!     krylov
!     linear_solver
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulesimple
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix) :: A

!     A              o    Structure of type matrix containing the necessary
!                         information of the matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: lmax, istart, iend
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     iend           Last position used at a part
!     istart         First position used at a part
!     lmax           Maximal length of the blocks
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRECCOPY       Copy matrix amat into matrix premat, where premat may have
!                    a different structure.
!                    Both matrices are stored as compact matrix
!     SEPCOPYMATR    Copy real R matrix in type matrix (simple)
!     SEPPARGLOB     Send information to neighboring processors in order to give
!                    all local arrays the global value in the common points
!     SEPPRINMATDC6  Print preconditioning matrix that is stored as
!                    row-compact real matrix
!     SEPSIMPLEMATPM Create the preconditioning matrix for PM depending on iprec
!     SEPSIMPLEMATQ  Create the preconditioning matrix for Q depending on iprec
!     SEPSIMPLEMATR  Create the matrix R = D Q'^-1 G for the SIMPLE algorithm
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The simple algorithm can be formulated as follows:
!
!     Let the system of equations be:
!
!     | Q    G | |u|  = |f_u|
!     | D    C | |p|    |f_p|
!
!     input:  Q, G, D, f_u, f_p
!     output: du, dp
!
!     solution:
!           Q du = f_u
!           Q' = diag(Q)
!      (C - D Q'^-1 G) dp = f_2 - D du
!           du = du - Q'^-1 G dp
!
!     In this subroutine we create the matrix R = D Q'^-1 G
!     as well the preconditioning matices corresponding to R and Q
!
!     Code
!     Extract parameters from input
!     Activate arrays
!     Compute starting addresses
!     Extract some information from intmat-1
!     Create the matrix R
!     create the preconditioning matrix for Q depending on iprec
!     create the preconditioning matrix for R depending on iprec
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepsimplemat' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepsimplemat'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )

      IM => intmt(iseqim)

!     --- Create space for the matrix R

      allocate ( S_R(sizedgmat), stat = error )
      if ( error/=0 ) call eralloc ( error, sizedgmat, 'S_R' )
      if ( ierror/=0 ) go to 1000

!     --- Store information of R matrix in type matrix (R)

      call sepcopymatr

      if ( debug ) write(irefwr,1) 'A%typesolv', A%typesolv

      if ( A%typesolv==13 .or. A%typesolv==14 ) then

!     --- typesolv = 13, hence Msimple or lsc
!         Use diagonal of velocity mass matrix instead of velocity matrix

         S_Qprime => S_UD

      else

!     --- typesolv # 13, 14 other methods
!         Use diagonal of velocity matrix

         if ( parallel ) then

!        --- Parallel case
!            The diagonal of the Q-matrix is stored as separate array, since
!            we need a global diagonal, not a local one

            allocate ( S_Qprime(nu), stat = error )
            if ( error/=0 ) call eralloc ( error, nu, 'S_Qprime' )
            if ( ierror/=0 ) go to 1000

            S_Qprime = S_Q(intdiaQ)
            length = size(Q%sendinfo)
            call sepparglob ( Q, S_Qprime, Q%neq )

         else

!        --- Serial case

            allocate ( S_Qprime(nu), stat = error )
            S_Qprime = S_Q(intdiaQ)

         end if   ! ( parallel )

      end if  ! ( typesolv==13 .or. typesolv==14 )

!     --- Copy C-matrix into R

      if ( debug ) write(irefwr,*) 'before preccopy'
      call preccopy ( np, irowsPM, irowsDG, S_C, &
                      S_R, icolsPM, icolsDG )

!     --- Reverse sign

      S_R = -S_R

!     --- Create the matrix R

      if ( debug ) write(irefwr,*) 'before sepsimplematr'
      call sepsimplematr

      if ( debug ) then

!     --- Debug information, print matrix

         write(irefwr,*) 'R(DG) matrix'
         call sepprinmatdc6 ( R )

      end if  ! ( debug )

!     --- create the preconditioning matrix for Q depending on iprec

      call sepsimplematq

!     --- create the preconditioning matrix for R depending on iprec

      call sepsimplematpm ( .true., 3 )

      qmatprime => S_Qprime

1000  if ( debug ) then
         write(irefwr,*) 'End sepsimplemat'
      end if  ! ( debug )
      call erclos ( 'sepsimplemat' )

      end
