      subroutine sepsolvebfibt ( rhspress, niters, A )
! ======================================================================
!
!        programmer    Mehfooz ur Rehman
!        version  1.0  date 05-05-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Solve a system of linear equations of special form using the schur
!     complement method
!     The method is accelerated by the GCR process
!     Old subroutine prsolvebfibt
! **********************************************************************
!
!                       KEYWORDS
!
!     simple
!     linear_solver
!     matrix
!     real
!     krylov
!     conjugate_gradients
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesimple
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: niters(*)
      double precision :: rhspress(np)
      type (rmatrix) :: A

!     A             i/o   Structure of type matrix containing the necessary
!                         information of the matrix
!     niters         i    ?
!     rhspress      i/o   Right-hand side for p-equation
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: k, iter, itimesav, uiter, ngcr
      double precision, allocatable :: svec(:,:), vvec(:,:)
      double precision  :: rnorm, eps
      logical :: converged

!     converged      If true the process is converged
!     eps            Local accuracy
!     iter           Iteration sequence number
!     itimesav       Help parameter to save the parameter itime
!     k              Counting variable
!     ngcr           Maximum number of GCR iterations
!     rnorm          norm of residual
!     svec           Work array of size ngcr x n_u to store the vectors
!                    s during the iteration
!     uiter          ?
!     vvec           Work array of size ngcr x n_u to store the vectors
!                    v during the iteration
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     PRININ1        print 2d integer array
!     PRINRL         Print 1d real vector
!     SEPSCHURGCRCYCL Perform one step of the gcr process wrt schur complement
!                    method
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     676   The CG-process has terminated because the number of iterations
!           exceeds the maximum value
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Start with The matrices Q, R, D and the vectors u, p, ru and r_p
!     For k := 0 (1) ngcr do
!        Perform one step of simple to compute du and dp
!        s^(k+1) := (du,dp), r = (ru,r_p)
!        v^(k+1) := A s^(k+1)
!        For i := 1 (1) k do
!           alpha := ( v^(k+1), v^i)
!           v^(k+1) := v^(k+1)- alpha v^i
!           s^(k+1) := s^(k+1)- alpha s^i
!        v^(k+1) := v^(k+1)/||v^(k+1)||
!        s^(k+1) := s^(k+1)/||v^(k+1)||
!        alpha := ( r, v^(k+1))
!        (u,p) := (u,p) + alpha s^(k+1)
!        r := r - alpha v^(k+1)
!
!     The GCR steps are performed in prschurgcrcycle
!     In this subroutine we first premultiply by the inverse of the
!     pressure mass matrix, which is used as preconditioner
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepsolvebfibt' )
      ngcr = inpsol(13,1)
      debug = .false.
      if ( debug ) then

!     --- debug inforamtion

         write(irefwr,*) 'Debug information from sepsolvebfibt'
         call prinin1 ( inpsol, 5, ninpsol1, 'inpsol' )

      end if
      if ( ierror/=0 ) go to 1000

      allocate(svec( np,0:ngcr+1) , vvec(np,0:ngcr+1), &
               stat = error)
      if ( error/=0 ) call eralloc ( error, np*ngcr, 'svec' )
      if ( ierror/=0 ) go to 1000

      converged = .false.
      itimesav = itime
      itime = 0

      if ( R%nmax==0 ) R%nmax = np

      rnorm = norm ( rhspress, R )
      eps = R%eps * rnorm

      if ( R%irelerr/=0 .and. rnorm<=eps ) then

!     --- Initial residual already small

         go to 1000

      end if

!     --- Print heading if iprint>0

      iter = 0

!     --- For k := 0 (1) R%nmax-1 do

      do k = 0, R%nmax-1

!     --- Perform one step of simple to compute du and dp
!         s^(k+1) := (du,dp), r = (ru,r_p)

         iter = iter+1

!        --- Perform a preconditioning step by premultiplying
!            by the inverse of the pressure mass matrix
!            This is done by an inaccurate solve

         svec(:,0) = rhspress/R
         if ( debug ) call prinrl ( svec(1,0), np, 'svec(1,0)/1' )

         if ( ierror/=0 ) go to 1000

!        --- Perform one step of the gcr process.

         if ( debug ) write(irefwr,*) 'before prschurgcrcycle'
         call sepschurgcrcycl ( svec, vvec, ngcr, &
                                k, rnorm, rhspress, dp,  A )
         niters(1) =  niters(1) + Q%numiter

         if ( ierror/=0 ) go to 1000
         converged = rnorm<=eps

         if ( converged ) go to 500

      end do  ! k = 1, R%nmax-1

!     --- If the subroutine reaches this stage, too much iterations have
!         been carried out

      call errint ( iter, 1 )
      if ( inpsol(22,3)==1) then
         inpsol(1,3) = iter
         call errwar ( 676, 1, 0, 0 )
      else
         call errsub ( 676, 1, 0, 0 )
      end if

500   niters(2) = iter

!     --- Since we have to solve -D Q^-1G p = r, we multiply by -1

      dp = -dp

1000  deallocate ( svec, vvec, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'work' )

      call erclos ( 'sepsolvebfibt' )
      if ( debug ) write(irefwr,*) 'End sepsolvebfibt'
      itime = itimesav

      end
