      subroutine solvelitref ( isol, irhsd, matr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 24-10-2008
!
!   copyright (c) 2008-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     iterative refinement and/or
!     Compute and print the residual
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematr
      use sepmodulesolve
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isol(*), irhsd, matr

!     irhsd          i    Standard SEPRAN array, containing information of the
!                         large vector
!     isol          i/o   Standard SEPRAN array containing information of the
!                         solution array
!     matr           i    Standard SEPRAN array, containing information of the
!                         large matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iinvec(6), icor, iresid
      double precision rinvec(2), residu
      logical :: finished

!     finished       Indicates if the loop has finished (true) or not
!                    (false)
!     icor           Refers to correction vector
!     iinvec         User input array for subroutine MANVEC
!                    For a description see subroutine MANVEC
!     iresid         Refers to residual vector
!     residu         Norm of residual
!     rinvec         Real user input array for subroutine MANVEC
!                    The contents depend on iinvec
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRESIDUAL      Compute residual of system of linear equations
!     SEPCOPYVC      Copy one SEPRAN vector into another one
!     SEPMANVEC      Vector manipulation subroutine
!     SEPRMMATRIX    Remove array with sequence number iseq from realm
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
!     SEPSOLDIR      Actual body of subroutine SOLVE
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'solvelitref' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from solvelitref'
         write(irefwr,1) 'iseqmt2, iseqsol', iseqmt2, iseqsol
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      icor = 0
      iresid = 0

!     --- while loop iterative refinement

      niter = 0
      finished = .false.

300   if ( .not. finished ) then

!     --- Compute and print the residual
!         First res := Ax

         niter = niter + 1
         call presidual ( iseqmt2, isol(iseqsol), &
                          irhsd, iresid, 2 )

!        --- Compute norm of residual

         iinvec(1) = 6
         iinvec(2) = 5
         iinvec(3) = 0
         iinvec(4) = 1
         iinvec(5) = 0
         iinvec(6) = 2

         call sepmanvec ( iinvec, rinvec, iresid, iresid, iresid )

!        --- Print norm of residual

         if ( inpsol(12,1)==1 ) then

!        --- absolute norm

            write ( irefwr, 310 ) rinvec(1)
310         format ( ' Norm of residual is ', ss, 1pe10.2e2 )

         else if ( inpsol(12,1)==2 ) then

!        --- Compute norm of right-hand side

            call sepmanvec ( iinvec, rinvec(2), irhsd, irhsd, irhsd )

            if ( rinvec(2)<epsmac**3 ) then

               write ( irefwr, 320 ) rinvec(2), epsmac**3
320            format ( ' The norm of the right-hand side is ', &
                        ss, 1pe10.2e2, ', which is less than ', &
                        1pe10.2e2/ &
                        'The relative norm is not computed ')
               write ( irefwr, 310 ) rinvec(1)

            else

               write ( irefwr, 330 ) rinvec(1) / rinvec(2)
330            format( ' Norm of residual divided by norm of right-', &
                       'hand side is ', ss, 1pe10.2e2 )
            end if

         end if

         if ( inpsol(3,1)==0 .and. inpsol(5,1)>0 ) then

!        --- Iterative refinement

            if ( niter>=inpsol(5,1) ) finished = .true.
            if ( icor==0 ) call sepcopyvc ( irhsd, icor )

!           --- Solve correction = A^(-1) res

            call sepsoldir ( matr, icor, iresid )

!           --- u := u + correction

            iinvec(1) = 3
            iinvec(2) = 27
            iinvec(3) = 0
            residu = rinvec(1)
            rinvec(1) = 1d0
            rinvec(2) = 1d0

            call sepmanvec ( iinvec, rinvec, isol(iseqsol), icor, isol )
            if ( inpsol(6,1)>=2 ) then
               write(irefwr,340) niter, residu
340            format(' Iteration ', i4,'  Residual is ', ss, &
                      1pe11.2e2 )
            end if
            if ( residu<rinsol(1,1) ) then
               if ( inpsol(6,1)>=1 ) write(irefwr,350) niter
350            format(' Iterative improvement finished after ',i4, &
                      ' iterations' )
               finished = .true.
            end if

         else

!        --- No iterative refinement

            finished = .true.

         end if

         go to 300

      end if
      inpsol(1,1) = niter

!     --- Delete copied matrix

      if ( iseqmt2>0 ) call seprmmatrix ( iseqmt2 )
      if ( icor>0 ) call seprmsolinfo ( icor )
      call seprmsolinfo ( iresid )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End solvelitref'

      end if  ! ( debug )
      call erclos ( 'solvelitref' )

      end
