      subroutine to0082 ( vecinp, vecout, idegfd, iskip, iskippoints )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 26-01-2011 New parameter list
!        version  2.0  date 23-09-2002 Extra parameter idegfd
!        version  1.2  date 23-09-2002 Correction address vecout
!
!   copyright (c) 1991-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     subroutine to0082 is used to map a SEPRAN vector of one structure
!     onto a SEPRAN vector of a different structure
!     Real case, the complex equivalent is subroutine TO0087
!     Restriction: this subroutine may only be used if both arrays
!                  correspond to the same mesh and the number of degrees
!                  of freedom per point is identical
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision vecinp(*), vecout(*)
      integer idegfd, iskip, iskippoints(*)

!     idegfd         i    Mapping must be stored from the idegfd-th
!                         unknown per point in the array vecout
!     iskip          i    Indicates if element groups must be skipped (>0)
!                         or not (0)
!     iskippoints    i    If iskippoints(i)=0, node i must be skipped, else
!                         iskippoints(i)>0, node i must not be skipped
!     vecinp         i    Input vector to be mapped
!     vecout         o    Output vector, result of the mapping
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inusol, jdegfd

!     jdegfd         Maximum of 1 and idegfd
!     inusol         Number of unknowns in vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     DCOPY          Copy double precision dx to double precision dy
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     TO00821        Map one vector onto another (real case)
!     TO00822        Map one vector onto another (real case)
!     TO00823        Map one vector onto another (real case)
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1743   The number of unknowns per point is constant for the input
!           vector but not for the output vector
!    2278   Renumbering of unknowns has not yet been implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
!   If ( number of unknowns per point in input vector is constant ) then
!      Check if number of unknowns per point in output vector is constant
!      Compute total number of unknowns
!   else ( number of unknowns per point in input vector is not constant )
!      Check if number of unknowns per point in output vector is not constant
!      Compare kprobfi and kprobfo
!      Compute total number of unknowns
!   If ( input vector is not renumbered ) then
!      If ( output vector is not renumbered ) then
!         Copy input vector into output vector
!      else ( output vector is renumbered ) then
!         Map input vector into output vector
!   else ( input vector is renumbered ) then
!      If ( output vector is not renumbered ) then
!         Map input vector into output vector
!      else ( output vector is renumbered ) then
!         Map input vector into output vector
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'to0082' )

      debug = .false.
      jdegfd = max(1,idegfd)

      if ( debug ) then

!     --- debug information

         write(irefwr,*) 'Debug information from to0082'
         write(irefwr,1) 'indprfi, indprfo, nphysi, nphyso, jdegfd', &
                          indprfi, indprfo, nphysi, nphyso, jdegfd
  1      format ( a, 1x, (10i6) )

      end if

      if ( indprfi==0 ) then

!     --- indprfi = 0, number of unknowns per point is constant

         if ( indprfo/=0 ) then
            call errsub ( 1743, 0, 0, 0 )
            go to 1000
         end if

         if ( idegfd>0 .and. jdegfd<=nphyso .or. iskip>0 ) then

!        --- Vectors have different number of degrees of freedom

            call to00821 ( vecout, vecinp, nphyso, nphysi, jdegfd, &
                           npoint, iskip, iskippoints )
            go to 1000

         else

!        --- Standard case, both vectors have the same structure

            inusol = npoint * nphysi

         end if

      else

!     --- indprfi>0, number of unknowns per point is not constant

         if ( indprfo==0 ) then

!        --- Special case, output vector has constant number of unknowns
!            input vector not

            call to00822 ( vecout, vecinp, nphyso, jdegfd, &
                           npoint, iskip, iskippoints, kprobfi )
            go to 1000

         else

!        --- Both vectors have different number of unknowns per point

            call to00823 ( vecout, vecinp, kprobfo, jdegfd, &
                           npoint, iskip, iskippoints, kprobfi )
            go to 1000

         end if

      end if

!     --- Both vectors have the same structure
!         The whole vector must be copied

      call dcopy ( inusol, vecinp, 1, vecout, 1 )

1000  call erclos ( 'to0082' )
      end
