      subroutine to0097 ( iineig, rineig, iuserloc, userloc, eigvals )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 24-12-2014 New definition clinsl
!        version  1.1  date 05-12-2012 Long integers
!        version  1.0  date 07-09-1999
!
!   copyright (c) 1999-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Compute eigenvalues and eigenvectors of a matrix build by
!   the finite element method
! **********************************************************************
!
!                       KEYWORDS
!
!     eigenvalue
!     matrix
!     eigenvector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulemain
      use sepmoduleuser
      use sepmoduletime
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iineig(*)
      integer, target :: iuserloc(*)
      double precision rineig(*), eigvals(*)
      double precision, target :: userloc(*)

!     eigvals    o    Array of length neigv where all eigenvalues are stored
!     iineig     i    Input array in which all integer user information with
!                     respect to this subroutine is stored
!                      1:  IEIGVEC   If 1 the eigenvectors must be computed, &
!                                    if 0 not
!                      2:  IPRINT    Defines the amount of output produced
!                                    Possible values:
!                                    0:  No information about the iteration
!                                        process is printed except in the case
!                                        of an error
!                                    1:  The number of iterations performed is
!                                        printed
!                                    2:  Some extra information about the
!                                        iteration process is printed for each
!                                        iteration
!                      3:  ISEQ_COEF Defines the sequence number of the
!                                    input for the coefficients
!                      4:  INITIAL   Defines how the initial vector must be
!                                    filled
!                                    0: ev(i) = i
!                                    >0: random, the value is used to define the
!                                        random values
!                      5:  IMAS      If 0 the mass matrix is diagonal, if 1 it
!                                    is a profile matrix
!                      6:  MAXITER   Maximum number of iterations
!                                    ( when 0 a default value is used )
!                      7:  MAXINTV   Maximum number of e.v. intervals
!                                    ( when 0 a default value is used )
!                      8:  MAXPNT    Maximum number of points x(*) used
!                                    in P+R process to catch the eigenvalues
!                                    ( when 0 a default value is used )
!                      9:  NEIGV     Number of eigenvalues to be computed
!                     10:  IUSEUSR   This parameter indicates if the arrays
!                                    IUSER and USER from the parameter list are
!                                    used (1) or if the parameter ISEQFILCOF is
!                                    used (0)
!                     11:  NUMVEC    Maximum number of vectors stored in isol
!                     12:  IVECTOR   First vector sequence number for the
!                                    storage of the eigenvectors
!     iuserloc   i    In this array together with array user information
!                     concerning coefficients etc. for the differential
!                     equation must be stored.
!                     For standard problems that are contained in the library, &
!                     the arrays IUSER and USER must be filled according to
!                     fixed rules (see the manual Standard Problems).
!                     When the user uses his own elements he may fill and use
!                     IUSER and USER in his own way, since these arrays are only
!                     used by the element subroutines.
!     rineig     i    Contains the real input for the solver
!                     At this moment he input is defined as follows
!                     1:  Global accuracy eps
!     userloc     i   See array iuserlc
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iinbldlc(15), i, imas, &
              iprob, iseqcf, isubr, iuseusr, &
              ivectr, numveclc, neigv
      double precision eps

!     eps        Local accuracy
!     i          Counting variable
!     iinbldlc   User input array for subroutine BUILD
!     imas       Indicates the type of mass matrix (1=diagonal, 2=consistent)
!     iprob      Actual problem number
!     iseqcf     Sequence number corresponding to the coefficients
!     isubr      Indicates the type of calling subroutine
!                Possible values
!                1:  CREATE
!     iuseusr    See iineig
!     ivectr     Sequence number of first eigenvector in isol
!                If 0 no eigenvectors are computed
!     neigv      Number of eigenvalues
!     numveclc     Number of vectors stored in isol
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERLINS         Help subroutine to call FILCOF, which fills coefficients in
!                    IUSER and USER
!     EROPEN         Produces concatenated name of local subroutine
!     SEPBUILD       Build matrix and right-hand side
!     SEPEIGVAL      Find eigenvalues and if necessary eigenvectors of the
!                    generalized eigenvalue problem
!     SEPFILLSMF     Fill all input for subroutine SEPBUILD in SMF
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'to0097' )

      if ( ierror/=0 ) go to 1000

!     --- get parameters from array iineig

      iseqcf  = iineig(3)
      imas    = iineig(5)
      neigv   = iineig(9)
      iuseusr = iineig(10)
      numveclc  = iineig(11)
      ivectr  = iineig(12)
      iprob = intmt(intmat)%iprob
      eps = rineig(1)
      isubr = 3

      if ( iuseusr>0 ) then

!     --- iuseusr>0

         iuser => iuserloc(1:iuserlc(1))
         user => userloc(1:nint(userlc(1)))

      end if  ! ( iuseusr>0 )

      if ( iuseusr==0 ) then

!     --- Do not use the arrays IUSER and USER from the parameter list
!         Fill coefficients in IBUFFR and BUFFER

         if ( ioutp>=0 .and. itimloop<=1 .and. iseqcf>0 .and. &
              nitercom<=1 ) write ( irefwr, 110 ) iseqcf
110      format ( ' Fill coefficients with sequence number ', i3 )
         if ( iseqcf>0 ) call erlins ( isubr, iprob, iseqcf )

!        --- Copy arrays IUSER and USER to end of IBUFFR.  In this way BUILD
!            cannot overwrite the positions used by these arrays

         if ( associated(iuser1(iprob)%iuser) ) then

!        --- iuser1(iprob)>0, IUSER and USER are filled

            iuser => iuser1(iprob)%iuser
            user => iuser1(iprob)%user

         end if

      end if

!     --- The system of equations is built  (call of build)

      iinbldlc(1) = 15
      if ( ioutp>=0 .and. nitercom<=1 .and. itimloop<=1 ) &
         write ( irefwr, 140 )
140   format ( ' Build matrix for', &
               ' eigenvalue/eigenvector computation' )
      iinbldlc(2) = 14
      iinbldlc(3) = 1
      iinbldlc(4) = imas+1
      iinbldlc(5) = 0
      iinbldlc(6) = 0
      iinbldlc(7) = 0
      iinbldlc(8) = 0
      iinbldlc(9) = 0
      iinbldlc(11) = 0
      iinbldlc(10) = numveclc
      iinbldlc(12) = 1
      iinbldlc(13) = 1
      iinbldlc(14) = 0
      iinbldlc(15) = 0

      stiffmat => stiffm

      call sepfillsmf ( iinbldlc, irhsd, isolut2d, isolut2d )

      call sepbuild

!     --- Compute eigenvalues/vectors

      call sepeigval ( eigvals, neigv, isolut2d(ivectr), &
                       stiffmat, massmt, iineig, eps )

1000  call erclos ( 'to0097' )
      end subroutine to0097
