      subroutine to0105noncons ( vec1, vec2, vec3, npoint, indprfvec1, &
                                 indprfvec3, kprobfvec1, kprobfvec3, &
                                 ndegfd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 27-12-2007 Extra parameter ndegfd
!        version  1.0  date 12-12-2005
!
!   copyright (c) 2005-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the point-wise inner product of two vectors, each which ncomp
!     degrees of freedom per point in some points
! **********************************************************************
!
!                       KEYWORDS
!
!     algebraic_manipulation
!     inner_product
!     pointwise
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint, indprfvec1, indprfvec3, &
              kprobfvec1(*), kprobfvec3(*), ndegfd
      double precision vec1(*), vec2(*), vec3(*)

!     indprfvec1     i    Indicates if input vector 1 has a constant number of
!                         degrees of freedom (0) or not (>0)
!     indprfvec3     i    Indicates if input vector 2 has a constant number of
!                         degrees of freedom (0) or not (>0)
!     kprobfvec1     i    Kprob part f for input vector 1
!                         Contains number of unknowns per point accumulated
!     kprobfvec3     i    Kprob part f for input vector 2
!                         Contains number of unknowns per point accumulated
!     ndegfd         i    Number of components of the vectors that must be
!                         used in the inner product
!     npoint         i    Number of nodal points
!     vec1           i    first input vector with ncomp component per point
!     vec2           i    second input vector with ncomp component per point
!     vec3           o    output vector with one component per point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, inunkp, jstart, ncomp, istart
      double precision sum

!     i              Counting variable
!     inunkp         Number of degrees of freedom in point in vec3
!     istart         Last position used at a part
!     j              Counting variable
!     jstart         Last position used at a part
!     ncomp          number of degrees of freedom in vec1/2
!     sum            Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2793   The input vector has a non-constant number of degrees of freedom, &
!           the output vector has exactly one unknown per point
!    2795   The number of degrees of freedom in node in output vector is
!           incorrect.
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'to0105noncons' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from to0105noncons'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if
      if ( ierror/=0 ) go to 1000

      if ( indprfvec1>0 .and. indprfvec3>0 ) then

!     --- indprfvec1>0 and indprfvec3>0, input and output vector have a
!         non-constant number of degrees of freedom

         do i = 1, npoint

            istart = kprobfvec1(i)
            ncomp = kprobfvec1(i+1)-istart
            jstart = kprobfvec3(i)
            inunkp = kprobfvec3(i+1)-jstart
            if ( debug ) &
               write(irefwr,1) 'i, istart, ncomp, jstart, inunkp', &
                                i, istart, ncomp, jstart, inunkp
            if ( ncomp>0 ) then

!           --- Degrees of freedom in node

               sum = 0d0
               do j = 1, min(ndegfd,ncomp)
                  sum = sum + vec1(j+istart)*vec2(j+istart)
               end do

               if ( debug ) then

!              --- Debug information

                  call prinrl ( vec1(1+istart), ncomp, 'vec1' )
                  call prinrl ( vec2(1+istart), ncomp, 'vec2' )
                  write(irefwr,2) 'sum', sum

               end if  ! ( debug )

               if ( inunkp==1 ) then

!              --- One degree of freedom in output vector

                  vec3(jstart+1) = sum

               else

!              --- Not one degree of freedom in output vector

                  call errint ( i, 1 )
                  call errint ( inunkp, 2 )
                  call errint ( 1, 3 )
                  call errsub ( 2795, 3, 0, 0 )
                  go to 1000

               end if  ! ( inunkp==1 )

            else if ( inunkp>0 ) then

!           --- Not zero degrees of freedom in output vector

               call errint ( i, 1 )
               call errint ( inunkp, 2 )
               call errint ( 0, 3 )
               call errsub ( 2795, 3, 0, 0 )
               go to 1000

            end if  ! ( ncomp>0 )

         end do  ! i = 1, npoint

      else if ( indprfvec1>0  ) then

!     --- indprfvec1>0 and indprfvec3=0, input vector has a
!         non-constant number of degrees of freedom
!         output vector has only one unknown per point
!         First clear output vector

         do i = 1, npoint
            vec3(i) = 0d0
         end do  ! i = 1, npoint

         do i = 1, npoint

            istart = kprobfvec1(i)
            ncomp = kprobfvec1(i+1)-istart
            if ( debug ) &
               write(irefwr,1) 'i, istart, ncomp', &
                                i, istart, ncomp
            if ( ncomp>0 ) then

!           --- Degrees of freedom in node

               sum = 0d0
               do j = 1, min(ndegfd,ncomp)
                  sum = sum + vec1(j+istart)*vec2(j+istart)
               end do

               if ( debug ) then

!              --- Debug information

                  call prinrl ( vec1(1+istart), ncomp, 'vec1' )
                  call prinrl ( vec2(1+istart), ncomp, 'vec2' )
                  write(irefwr,2) 'sum', sum

               end if  ! ( debug )

               vec3(i) = sum

            end if  ! ( ncomp>0 )

         end do  ! i = 1, npoint

      else

!     --- one of indprfvec1 and indprfvec3 = 0, the other not
!         This situation is not permitted

         call errsub ( 2793, 0, 0, 0 )

      end if  ! ( indprfvec1==0 .and. indprfvec3==0 )

1000  call erclos ( 'to0105noncons' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End to0105noncons'

      end if

      end
