      subroutine to0106 ( ichois, idegfd, ivec1, ivec2, &
                          ivec3, const, iconst )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.2  date 15-11-2012 Long integers
!        version  2.1  date 17-05-2008 to0023-->tolincomb
!        version  2.0  date 14-04-1999 Extra parameters ibuffr/buffer
!
!   copyright (c) 1995-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    TO0106 is a vector manipulation subroutine
!    It is used for input vectors of type 119 only
!    It is called by the SEPRAN subroutines MANVEC, VECMAN, ALGEBR, ANORM,
!    DIFFVC and MANIVC
!    TO0106 is able to manipulate both vectors of common type (110, 115, 116)
!    as vectors of the special type 119
! **********************************************************************
!
!                       KEYWORDS
!
!     algebraic_manipulation
!     manipulation
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision const(*)
      integer ichois, idegfd(*), ivec1, ivec2, ivec3, &
              iconst(*)

!    const    i/o  In- and output real array containing constants for the
!                  manipulation depending on ichois
!    ichois    i   Choice parameter indicating what type of function
!                  is chosen by the user.  Possible values:
!                   1:    Subtract:  VEC3 = VEC1 - CONST(1)
!                   2:    CONST(1) is average value of VEC1
!                         At output IPOINT is number of points
!                   3:    Linear combination
!                         VEC3 = CONST(1) * VEC1 + CONST(2) * VEC2
!                   4:    VEC1(IDEGFD) in IPOINT is stored in CONST(1)
!                   5:    VEC1(IDEGFD) in IPOINT = CONST(1)
!                   6:    The minimum value of VEC1 is stored in CONST(1)
!                         The maximum value of VEC1 is stored in CONST(2)
!                         Real vectors only
!                   7:    The inner product VEC1 + VEC2 is stored in CONST(1)
!                         The number of points used is stored in IPOINT
!                   8:    VEC3 = CONST(1) * |VEC1|
!                         VEC1 is complex, VEC3 is real
!                   9:    VEC3 = CONST(1) * arg( VEC1 )
!                         VEC1 is complex, VEC3 is real
!                         If ICONST(1)=1, VEC3 = 180/pi * CONST(1) * arg( VEC1 )
!                  10:    The inner product VEC1 + VEC2 is stored in CONST(1)
!                         The number of points used is stored in IPOINT
!                         Boundary conditions are excluded
!                  11:    The minimum and maximum values of the array
!                         represented by IVEC1 are computed.
!                         Besides that the co-ordinates for which these
!                         values are reached are stored in const
!                         Restriction: Only one unknown may be considered
!                                      per point
!                         At output CONST(1) contains the minimum, CONST(2)
!                         the maximum value, CONST(3) - (5), contain the
!                         co-ordinates for which the minimum is reached and
!                         CONST(6) - (8), contain the co-ordinates for which
!                         the maximum is reached. Real vectors only.
!                   12:   See 11, however, the minimum and maximum values of the
!                         absolute value of the array are used
!                         (Real and complex)
!                   13:   VEC3 is a function of VEC1 and VEC2. The user
!                         subroutine FUNALG or FUNALC is called.
!                   14:   VEC3(i) = sqrt ( VEC1(idegfd(1) **2 + &
!                                                        i
!                                          VEC1(idegfd(2) **2  )
!                                                        i
!                         The dimension must be stored in ICONST(1)
!                         The type of output vector is found by the
!                         subroutine
!                   15:   CONST(1) = || VEC1 - VEC 2 ||
!                         The type of norm must be given in ICONST(1).
!                         Possible values:
!                         1       l1-norm
!                         2       l2-norm
!                         3       max-norm
!                         4       l1-norm / number of degrees of freedom
!                         5       l2-norm / number of degrees of freedom
!                        10       max-norm  the nodal point number
!                                 corresponding to the maximum is stored in
!                                 iconst(2)
!                   16:   CONST(1) = || VEC1 ||
!                         For the type of norms available: see ICHOIS = 15
!                   17:   VEC3 is a function of VEC1(.,i) (i=1,...,NVEC)
!                         The user subroutine FUNVEC is called.
!                         NVEC must be stored in ICONST(1)
!                   18:   VEC3 = conjg(VEC1)
!                   19:   VEC3 is a function of VEC1 and VEC2. The SEPRAN
!                         subroutine FUNADA is called.
!                   21:   VEC3 is a vector of one unknown in each point
!                         The IDEGFD-th degree of freedom from VEC1 is
!                         used as input
!                         It is supposed that this degree of freedom is
!                         available in each point
!                   22:   VEC3 is Real part of VEC1 multiplied by CONST(1)
!                   23:   VEC3 is Imaginary part of VEC1 multiplied by CONST(1)
!                   24:   Linear combination
!                         VEC3 = CONST(1) * VEC1 + CONST(2) * VEC2
!                         with constraint: vec3(i)>=const(3)
!    iconst   i/o  In- and output integer array containing constants for the
!                  manipulation depending on ichois
!    idegfd    i   In this array given by the user, he must store the degrees
!                  of freedom that are requested for the special purpose.
!                  In most applications only one degree of freedom per point
!                  variable but not by a constant.
!                  If the variable has value 0, then all degrees of freedom
!                  are considered.
!                  If the variable has value -1, then all degrees of freedom
!                  are considered, except those corresponding to essential
!                  boundary conditions
!                  The length of the array depends on ichois:
!                  If ichois = 1-13:  Length 1
!                                14:         2 or 3 (See iconst(1))
!                             15-16:         1
!                             17,18:     ?
!                                19:         1
!    ivec1     i   First input vector to be manipulated
!                  ivec1 must be a SEPRAN array of length 5, which may be
!                  of type 110, 115, 116 or 119
!    ivec2     i   Second input vector to be manipulated (if necessary)
!                  If only one array is needed, ivec2 may be replaced by a
!                  dummy
!    ivec3     i   Third input vector to be manipulated (if necessary)
!                  If only one or two arrays are needed, ivec3 may be replaced
!                  by a dummy
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, indvc2, indvc3, numveclc, lenvc1, lenvc3, ipvec1, &
              ipvec2, ipvec3, ipveccoor, idum
      logical compli, new, notper, usevc2, usevc3
      double precision amin, amax, coorlc(6)

!     amax    Maximum value in solution array
!     amin    Minimum value in solution array
!     compli  Indication if IVEC1 corresponds to a complex array (true) or to
!             a real one (false)
!     coorlc  array of length ndim x npoint containing the co-ordinates
!             of the nodal points with respect to the global numbering
!     i       Counting variable
!     idum           General dummy variable
!     indvc2  Help variable to compute starting address of array IVEC2
!     indvc3  Help variable to compute starting address of array IVEC3
!     ipvec1  Starting address of array VECTOR1
!     ipvec2  Starting address of array VECTOR2
!     ipvec3  Starting address of array VECTOR3
!     ipveccoor      Starting address of array vec2(coor)
!     lenvc1  Length of vector 1 in words
!     lenvc3  Length of vector 2 in words
!     new     Indication if the output vector is new (true) and hence should
!             be cleared or not (false)
!     notper  If true this possibility is not permitted
!     numveclc  Number of subarrays (type 119 only)
!     usevc2  If true vector 2 is used
!     usevc3  If true vector 3 is used
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPCRSOLINFO   Create a new vector in ks
!     TO0023         Perform some vector manipulations for real vectors
!     TO0024         Perform some vector manipulations for complex vectors
!     TO0031         Perform some vector manipulations for real vectors
!     TO0032         Perform some vector manipulations for complex vectors
!     TOLINCOMB      Store linear combination a vec1 + b vec2 into vec3
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1574   combination ichois and itypvc not allowed
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data coorlc /6*0d0/
! ======================================================================
!
      call eropen ( 'to0106' )

!     --- Check type of ivec1

      compli = ks(ivec1)%compl/=0

!     --- Reserve space

      indprfi = 0
      indprhi = 0
      indprpi = 0
      ipvec1 = 1
      ipvec2 = 1
      ipvec3 = 1

      notper = ichois==4  .or.  ichois==5  .or.  ichois==8  .or. &
               ichois==9  .or.  ichois>=15 .and. ichois<=17 .or. &
               ichois>=21 .and. ichois==23 .or. ichois>=26

      if ( notper ) then

!     --- Value of ichois not permitted for this type of vector

         call errint ( ichois, 1 )
         call errint ( 119, 2 )
         call errsub ( 1574, 2, 0, 0 )

      end if

      nbound = 0
      nphysi = 1

      indvc2 = 0
      usevc2 = ichois==3  .or. ichois==7 .or. ichois==10 .or. &
               ichois==13 .or. ichois==15 .or. ichois==19 .or. &
               ichois==24 .or. ichois==25
      usevc3 = ichois==1  .or. ichois==3 .or. ichois==13 .or. &
               ichois==18 .or. ichois==19 .or. ichois==24 .or. &
               ichois==25

      if ( usevc2 ) then

!     --- Second vector is used

         indvc2 = ivec2(5)

      end if
      indvc3 = 0
      if ( usevc3 ) then

!     --- Third vector must be created
!         Check if ivec3 has been filled before (new=false) or not (true)

         new = ivec3<=0 .or. ivec3>lastsol
         if ( .not. new ) new = solarr(ivec3)==0
!         if ( .not. new ) new = ivec3(2)/=ks(ivec1)%typevec .or. &
!               ks(ivec3)%compl/=ks(ivec1)%compl .or. &
!               ivec3(4)/=ivec1(4) .or. ivec3(5)/=ivec1(5)

         if ( new ) ivec3 = 0
         lenvc1 = size ( ks(ivec1)%isol )
         call sepcreasolbf1 ( ivec3, ks(ivec1)%typevec, ks(ivec1)%ivec, &
                              'ivec3', ks(ivec1)%compl )
         vectout => ks(ivec3)%sol

!        --- Clear ivec3 if it is a new vector

         if ( new ) vectout = 0d0

      end if

      if ( ierror/=0 ) go to 1000

!     --- Set some constants depending on the type of vector

      numveclc = ks(ivec1)%n
      ivectin => ks(ivec1)%isol
      vectin => ks(ivec1)%sol
      ipveccoor = 1
      if ( indvc2>0 ) vectin2 => ks(indvc2)%sol
      if ( indvc3>0 ) then

!        --- Copy first part of VEC1 into VEC3

         ks(ivec3)%isol(1:numveclc) = ks(ivec1)%isol(1:numveclc)
         vectout => ks(ivec3)%sol

      end if  ! ( indvc3>0 )

      nphysi = max(nphysi,1)
      if ( ichois==11 .or. ichois==12 ) then

!     --- ichois = 11, 12   minimum and maximum value and corresponding points

         ndim = 1
         do i = 1, numveclc
            nusol = ivectin(i)
            ipvec1 = ipveccoor + nusol
            if ( compli ) then
               call to0032 ( ichois, vectin(ipvec1), vectin(ipveccoor), &
                             coorlc, amin, amin, amax, ndim, idegfd(1), &
                             npoint )
            else
               call to0031 ( ichois, vectin(ipvec1), vectin(ipveccoor), &
                             coorlc, amin, amax, ndim, idegfd(1), npoint )
            end if
            if ( i==1 ) then
               const(1) = amin
               const(2) = amax
               const(3) = coorlc(1)
               const(6) = coorlc(4)
            else
               if ( const(1)>amin ) then
                  const(3) = coorlc(1)
                  const(1) = amin
               end if
               if ( const(2)<amax ) then
                  const(6) = coorlc(4)
                  const(2) = amax
               end if
            end if
            ipveccoor = ipvec1 + nusol
         end do

      else

!     --- ichois # 11, 12

         if ( compli ) then

!        --- Complex case

            do i = 1, numveclc
               nusol = ivectin(i)
               ipvec1 = ipveccoor + nusol
               ipvec2 = ipvec2 + nusol
               ipvec3 = ipvec3 + nusol
               call to0024 ( ichois, vectin(ipvec1), vectin2(ipvec2), &
                             vectout(ipvec3), const(1), const(3), &
                             iconst(1), nusol, nbound, 0, &
                             iconst, const, idum )
               ipveccoor = ipvec1 + nusol * 2
               ipvec2 = ipvec2 + nusol * 2
               ipvec3 = ipvec3 + nusol * 2
            end do

         else if ( ichois==3 .or. ichois==24 ) then

!        --- Real case, ichois = 3 or 24

            do i = 1, numveclc
               nusol = ivectin(i)
               ipvec1 = ipveccoor + nusol
               ipvec2 = ipvec2 + nusol
               ipvec3 = ipvec3 + nusol
               call tolincomb ( vectin(ipvec1), vectin2(ipvec2), &
                                vectout(ipvec3), const(1), const(2), &
                                const(2), nusol, kprobh, &
                                indprhi,  .true., ichois )
               ipveccoor = ipvec1 + nusol
               ipvec2 = ipvec2 + nusol
               ipvec3 = ipvec3 + nusol
            end do

         else

!        --- Real case

            do i = 1, numveclc
               nusol = ivectin(i)
               ipvec1 = ipveccoor + nusol
               ipvec2 = ipvec2 + nusol
               ipvec3 = ipvec3 + nusol
               u1 => vectin(ipvec1:ipvec1+nusol-1)
               u2 => vectin(ipvec2:ipvec2+nusol-1)
               u3 => vectin(ipvec3:ipvec3+nusol-1)
               call to0023 ( ichois, const(1), const(2), &
                             const(2), iconst(1), nusol, nbound, &
                             indprhi, kprobhinvi, iconst )
               ipveccoor = ipvec1 + nusol
               ipvec2 = ipvec2 + nusol
               ipvec3 = ipvec3 + nusol
            end do

         end if

      end if

1000  call erclos ( 'to0106' )
      end subroutine to0106
