      subroutine tofict00 ( iinbld, ielhlp, elemmtloc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 24-07-2015 Use sepmodulebuild
!        version  2.4  date 11-04-2015 New call to tofict02
!        version  2.3  date 25-11-2002 Reorganization (JdH)
!
!   copyright (c) 2000-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill part of matrix corresponding to fictitious unknowns
!
! **********************************************************************
!
!                       KEYWORDS
!
!     assembly
!     matrix
!     fictitious_domain
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iinbld(*), ielhlp(*)
      double precision, intent(inout) :: elemmtloc(*)

!     elemmtloc           work array to store local element matrix
!     ielhlp         i    This array of length nelgrp+5 is only used if
!                         allelements = 1
!                         ielhlp(ielgrp+5) = 0 means elements of element group
!                         ielgrp are skipped, ielhlp(ielgrp+5) = 1, means that
!                         they are assembled
!     iinbld         i    User input array for subroutine BUILD
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: maxlambda=9
      integer :: i, icurnr, inunkpgr, istar3, inusolgr, &
                 j, istart, isurnr, &
                 lnindex1, lnindex2, intarbld(50), inforelement(2,2), i1, &
                 i2, icurve1, icurve2, ielemfluid, ielemlowfluid, &
                 ielemlowstruct, ielemstruct, ifluidgrp1, igrp, ikelmc, &
                 ikelmcfluid, ikelmcstruct, inpelmfluid, inpelmstruct, &
                 inunkp_elm, iposkproby, iregionlam, ishapefluid, &
                 ishapelam, ishapestruct, istructgrp, k, nfictpnt, &
                 nlambda, nusolclassic, ishape_curve
      double precision :: weightlc(maxlambda), xgauss(3*maxlambda), &
                          philambda(maxlambda*maxlambda)

!     i              Counting variable
!     i1             First point of curve in KMESH part m
!     i2             Last point of curve in KMESH part m
!     icurnr         Curve sequence number
!     icurve1        First curve number of a set
!     icurve2        Last curve number of a set
!     ielemfluid     Element sequence number of fluid element
!     ielemlowfluid  Lowest element sequence number of fluid elements
!     ielemlowstruct Lowest element sequence number of structural elements
!     ielemstruct    Element sequence number of structural element
!     ifluidgrp1     First fluid element group number
!     igrp           Boundary element group number read
!     ikelmc         position in kmesh part c
!     ikelmcfluid    Position in kmeshc part c with respect to fluid elements
!     ikelmcstruct   Position in kmeshc part c with respect to structural
!                    element
!     inforelement   Array of size 2 x 2 containing the following
!                    information
!                    (1,.) refers to the structural element
!                    (2,.) refers to the fluid element
!                    (.,1) contains the shape number of the element
!                    (.,2) contains the number of points in the element
!     inpelmfluid    number of points per fluid element
!     inpelmstruct   number of points per structural element
!     intarbld       Integer array to be used by build and subroutines to
!                    pass information.
!                    For a description see subroutine TO0050
!     inunkp_elm     Number of unknowns per point
!     inunkpgr       Number of global unknowns in a group
!     inusolgr       Position of first fictitious unknown in a group with
!                    respect to solution array
!     iposkproby      Position in kprob y
!     iregionlam     type of region where the lambda's are defined:
!                     1:  curves
!                     2:  surfaces
!     ishape_curve   Shape of structure curve
!     ishapefluid    shape number for fluid element
!     ishapelam      Shape of lambda basis functions
!     ishapestruct   shape number for structural element
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     istart         First position used at a part
!     istructgrp     Structural element group
!     isurnr         Surface sequence number
!     j              Counting variable
!     k              Counting variable
!     lnindex1       Length of array index1
!     lnindex2       Length of array index2
!     maxlambda      Maximum number of lambda's allowed per element
!     nfictpnt       Total number of points with fictitious unknowns
!     nlambda        number of lambda points in the lambda element
!     nusolclassic   Number of free classical unknowns, i.e ficitious unknowns
!                    and global unknowns excluded
!     philambda      Contains the values of the lambda basis function
!                    in the gauss point
!     weightlc       Help array to store weights
!     xgauss         array of length m x ndim containing the co-ordinates
!                    of the gauss points.
!                    xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                    zg_i = xgauss (i,3)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELM921         Compute element matrix for fictitious unknowns
!     ELMFICTLAM     Compute the integration points and basis functions for the
!                    Lagrangian multipliers
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     TOFICT01       Fill arrays index2 and index3 with the positions of the
!                    unknowns in the nodal points of an element
!     TOFICT02       Add element matrix corresponding to fictitious unknowns to
!                    large matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Initializations
!    For all fictitious groups do
!       Find type of elements
!       If ( type of elements = curves ) then
!          For all curves required do
!             Build element matrix and vector
!             Store matrix and vector into large matrix
!       else if ( type of elements = surfaces ) then
!          For all surfaces required do
!             Build element matrix and vector
!             Store matrix and vector into large matrix
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'tofict00' )

!     --- Initializations

      matrix = addmat
      vector = addvec
      nfictpnt = 0
      inpelm = npelm
      ielem  = nelem
      nusolclassic = nusol-nglunk-nfictunk
      inusolgr = nrusol-nglunk-nglcoupl-nfictunk+1
      lnindex1 = npelm*2
      lnindex2 = lnindex1*ndim

!     --- Clear iwork

      iwork = 0

!     --- Clear element matrix

      elemmtloc(1:lnindex2*27) = 0d0
      if ( ndim==2 ) then

!     --- ndim = 2

         istar3 = kmeshm(2)-1

      end if

!     --- Loop over fictitious groups

      do igrp = 1, nfictgrp

!     --- Get some relevant parameters

         iposkproby = kproby(10+igrp)-1
         itype      = kproby(iposkproby+1)
         inunkpgr   = kproby(iposkproby+2)
         iregionlam = kproby(iposkproby+3)
         ishapelam  = kproby(iposkproby+4)
         icurve1    = kproby(iposkproby+5)
         icurve2    = kproby(iposkproby+6)
         istructgrp = kproby(iposkproby+7)
         ifluidgrp1 = kproby(iposkproby+8)
         inunkp_elm = kproby(iposkproby+10)
         nlambda    = inunkp_elm/inunkpgr

!        --- Reset inpelm because of inpelm = inpelmstruct + inpelmfluid

         inpelm = npelm

!        --- Compute starting address of structural elements in kmeshc

         ikelmcstruct = 0
         ielemlowstruct = 1
         do i = 1, istructgrp-1
            inpelm = sepgetinpelm(i)
            inelem = sepgetinelem(i)
            ikelmcstruct = ikelmcstruct+inelem*inpelm
            ielemlowstruct = ielemlowstruct+inelem
         end do

         inpelmstruct = sepgetinpelm(istructgrp)
         ishapestruct = sepgetshape(istructgrp)

!        --- Compute starting address of fluid elements in kmeshc

         ikelmcfluid = 0
         ielemlowfluid = 1
         do i = 1, ifluidgrp1-1
            inpelm = sepgetinpelm(i)
            inelem = sepgetinelem(i)
            ikelmcfluid = ikelmcfluid+inelem*inpelm
            ielemlowfluid = ielemlowfluid+inelem
         end do
         inpelmfluid = sepgetinpelm(ifluidgrp1)
         ishapefluid = sepgetshape(ifluidgrp1)
         inforelement(1,1) = ishapestruct
         inforelement(1,2) = inpelmstruct
         inforelement(2,1) = ishapefluid
         inforelement(2,2) = inpelmfluid
         icount = ndim*(inpelmfluid+inpelmstruct)

         if ( iregionlam==1 ) then

!        --- region is of type curve

            if ( ishapestruct==3 .or. ishapestruct==5 ) then

!           --- ishape = 3 or 5 (linear case)

               ishape_curve = 1

            else if ( ishapestruct==4 .or. ishapestruct==6 ) then

!           --- ishape = 4 or 6 (quadratic case)

               ishape_curve = 2

            else

!           --- error: shape number not yet implemented

               call errint ( ishapestruct, 1 )
               call errsub ( 800, 1, 0, 0 )
               go to 1000

            end if

            ifirst = 0

!           --- loop over curves

            do icurnr = icurve1, icurve2

               i1 = kmeshm(4+icurnr)
               i2 = kmeshm(5+icurnr)-1

!              --- Find structural and fluid element containing the lambdas
!                  Loop over elements along curve

               do i = i1, i2-1, ishape_curve

!              --- Fill information with respect to lambda element

                  ielem = ielem+1
                  call elmfictlam ( coor,kmeshm(istar3+i),ishape_curve, &
                                    ndim, xgauss,  philambda, weightlc, &
                                    ishapelam, .true. )

!                 --- Loop over lambda points in element

                  do j = 1, ishapelam
                     nfictpnt = nfictpnt+1
                     ielemstruct = kprobfict(1,nfictpnt)

!                    --- Loop over the nodes in the structural element
!                        Store the nodes in index1

                     ikelmc = ikelmcstruct+ &
                              (ielemstruct-ielemlowstruct)*inpelmstruct

                     do k = 1, inpelmstruct
                        index1(k) = kmeshc(ikelmc+k)
                     end do

!                    --- Loop over the nodes in the fluid element
!                        Store the nodes in index1

                     ielemfluid = kprobfict(2,nfictpnt)
                     ikelmc = ikelmcfluid+ &
                              (ielemfluid-ielemlowfluid)*inpelmfluid

                     do k = 1, inpelmfluid
                        index1(k+inpelmstruct) = kmeshc(ikelmc+k)
                     end do

!                    --- Fill array index2 with the sequence numbers of the
!                        unknowns and compute icount

                     inpelm = inpelmstruct+inpelmfluid
                     if ( itype==921 ) then

!                    --- itype = 921, standard case: ndim unknowns per point

                        ndimlc = ndim

                     else

!                    --- itype = 923, special case: 1 unknowns per point

                        ndimlc = 1

                     end if
                     call tofict01

!                    --- Fill element matrix

                     call elm921 ( coor, elemmtloc, index1, &
                                   index1(inpelmstruct+1), inforelement, &
                                   xgauss,j, weightlc(j), &
                                   philambda, nlambda )

!                    --- add element matrix to large matrix

                     call tofict02 ( inpelm*ndimlc, inunkp_elm, &
                                     inusolgr, iwork, elemmtloc )

                  end do

!                 --- Raise parameters

                  nusolclassic = nusolclassic+inunkp_elm
                  inusolgr = inusolgr+inunkp_elm
                  ifirst = 1

               end do

            end do

         else

!        --- region is of type surface

            if ( ishapestruct==11 ) then

!           --- ishape = 11

               ishape_curve = 3

            else if ( ishapestruct==12 ) then

!           --- ishape = 12

               ishape_curve = 4

            else if ( ishapestruct==13 ) then

!           --- ishape = 13

               ishape_curve = 5

            else if ( ishapestruct==14 ) then

!           --- ishape = 14

               ishape_curve = 6

            else

!           --- error: shape number not yet implemented

               call errint ( ishapestruct, 1 )
               call errsub ( 800, 1, 0, 0 )
               go to 1000

            end if

!           --- Find starting address in kmesh part n

            istart = 2*nsurfs+1
            do isurnr = 1, icurve1-1
               istart = istart+kmeshn(nsurfs+isurnr)*kmeshn(isurnr)
            end do

!           --- loop over surfaces

            ifirst = 0
            do isurnr = icurve1, icurve2

               inelem = kmeshn(nsurfs+isurnr)
               inpelm = kmeshn(isurnr)

!              --- Find structural and fluid element containing the lambdas
!                  Loop over elements along surface

               do i = 1, inelem

!              --- Fill information with respect to lambda element

                  ielem = ielem+1
                  inpelm = kmeshn(isurnr)

                  call elmfictlam ( coor, kmeshn(istart), ishape_curve, &
                                    ndim, xgauss,  philambda, weightlc, &
                                    ishapelam, .true. )
                  istart = istart+inpelm

!                 --- Loop over lambda points in element

                  do j = 1, nlambda
                     nfictpnt = nfictpnt+1
                     ielemstruct = kprobfict(1,nfictpnt)

!                    --- Loop over the nodes in the structural element
!                        Store the nodes in index1

                     ikelmc = ikelmcstruct+ &
                              (ielemstruct-ielemlowstruct)*inpelmstruct
                     do k = 1, inpelmstruct
                        index1(k) = kmeshc(ikelmc+k)
                     end do

!                    --- Loop over the nodes in the fluid element
!                        Store the nodes in index1

                     ielemfluid = kprobfict(2,nfictpnt)
                     ikelmc = ikelmcfluid+ &
                              (ielemfluid-ielemlowfluid)*inpelmfluid
                     do k = 1, inpelmfluid
                        index1(k+inpelmstruct) = kmeshc(ikelmc+k)
                     end do

!                    --- Fill array index2 with the sequence numbers of the
!                        unknowns and compute icount

                     inpelm = inpelmstruct+inpelmfluid
                     if ( itype==921 ) then

!                    --- itype = 921, standard case: ndim unknowns per point

                        ndimlc = ndim

                     else

!                    --- itype = 923, special case: 1 unknowns per point

                        ndimlc = 1

                     end if
                     call tofict01
!                    --- Fill element matrix

                     call elm921 ( coor, elemmtloc, index1, &
                                   index1(inpelmstruct+1), inforelement, &
                                   xgauss, j, weightlc(j), &
                                   philambda, nlambda )

!                    --- add element matrix to large matrix

                     call tofict02 ( inpelm*ndimlc, inunkp_elm, &
                                     inusolgr, iwork, elemmtloc )

                  end do

!                 --- Raise parameters

                  nusolclassic = nusolclassic+inunkp_elm
                  inusolgr = inusolgr+inunkp_elm
                  ifirst = 1

               end do

            end do

         end if

      end do

1000  call erclos ( 'tofict00' )

      end subroutine tofict00
