      subroutine tofict02 ( jcount, inunkp_elm, inusolgr, iwork1, elemmtloc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 11-04-2015 new parameter list
!        version  1.2  date 14-08-2002 Extra error message
!        version  1.1  date 14-04-2001 Correction boundary conditions
!
!   copyright (c) 2000-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Add element matrix corresponding to fictitous unknowns to large matrix
!
! **********************************************************************
!
!                       KEYWORDS
!
!     assembly
!     matrix
!     fictitious_domain
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulematstr
      use sepmodulebuild
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: jcount, inunkp_elm, inusolgr
      integer, intent(inout) :: iwork1(*)
      double precision, intent(inout) :: elemmtloc(inunkp_elm,*)

!     elemmtloc      i    Element matrix
!     inunkp_elm     i    Number of unknowns per point
!     inusolgr       i    Position of first global unknown in a group with
!                         respect to solution array
!     iwork1               integer work array of length nusol
!     jcount         i    Number of degrees of freedom in one row of elemmtloc
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical :: bouncond
      integer :: i, j, imat, inusol, ishift, idiag, jnusol, k, &
                 jpos, ktel, lengpu, lengup
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     bouncond       If true boundary conditions have been found
!                    they are not
!     i              Counting variable
!     idiag          Position of diagonal element in matrix
!     imat           Position in array amat
!     inusol         Position of unknown in array USOL
!     ishift         Shift of specific variable with respect to diagonal
!     j              Counting variable
!     jnusol         Position of unknown in array USOL
!     jpos           Position in array
!     k              Counting variable
!     ktel           Global sequence number of prescribed degree of freedom, &
!                    where the first prescribed d.o.f. has sequence number 1
!     lengpu         length of matrix S_up
!     lengup         length of matrix S_up+S_u per
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2270   integer incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'tofict02' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from tofict02'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(iseqim)
      irowsii => IM%rowsii
      icolsii => IM%colsii

      S_ii => Smat%S_ii
      S_ip => Smat%S_ip

      bouncond = .false.
      if ( IM%symmetric .and. .not. IM%mumps ) then

!     --- symmetric method

         do j = 1, inunkp_elm
            inusol = inusolgr+j-1

!           --- Store inverse of icol into iwork1

            do i = irowsii(inusol)+1, irowsii(inusol+1)
               iwork1(icolsii(i)) = i
            end do

            do i = 1, jcount
               if ( index2(i)<=nrusol ) then

!              --- Internal unknown, lower part of matrix

                  imat = iwork1(index2(i))
                  if ( imat<=0 ) then

!                 --- imat = 0 , this is a programming error

                     call errint ( imat, 1 )
                     call errchr ( 'imat', 1 )
                     call errsub ( 2270, 1, 0, 1 )
                     go to 1000

                  end if

                  S_ii(imat) = S_ii(imat)+elemmtloc(j,i)

               else

!              --- Boundary condition found

                  bouncond = .true.

               end if

            end do

!           --- Clear iwork1

            do i = irowsii(inusol)+1, irowsii(inusol+1)
               iwork1(icolsii(i)) = 0
            end do

         end do

      else

!     --- row-compact storage

         do j = 1, inunkp_elm
            inusol = inusolgr+j-1

!           --- Store inverse of icol into iwork1

            do i = irowsii(inusol)+1, irowsii(inusol+1)
               iwork1(icolsii(i)) = i
            end do

            do i = 1, jcount
               if ( index2(i)<=nrusol ) then

!              --- Internal unknown, lower part of matrix

                  imat = iwork1(index2(i))
                  if ( imat<=0 ) then

!                 --- imat = 0 , this is a programming error

                     call errint ( imat, 1 )
                     call errchr ( 'imat', 1 )
                     call errsub ( 2270, 1, 0, 1 )
                     go to 1000

                  end if

                  S_ii(imat) = S_ii(imat)+elemmtloc(j,i)  ! upper part

                  jnusol = index2(i)
                  do k = irowsii(jnusol+1), irowsii(jnusol)+1, -1
                     imat = k
                     if ( icolsii(k)==inusol ) exit
                  end do  ! k = irowsii(inusol+1), irowsii(inusol)+1, -1
                  S_ii(imat) = S_ii(imat)+elemmtloc(j,i)  ! lower part

               else

!              --- Boundary condition found

                  bouncond = .true.

               end if

            end do

!           --- Clear iwork1

            do i = irowsii(inusol)+1, irowsii(inusol+1)
               iwork1(icolsii(i)) = 0
            end do

         end do

      end if

      lengup = size(icolsip)
      lengpu = 2*lengup

      if ( bouncond ) then

!     --- boundcond = true, store part corresponding to boundary conditions
!         in amat part b

         do i = 1, jcount
            if ( index2(i)>nrusol ) then

!           --- Boundary unknown, store in matrix part bc

               inusol = index2(i)

!              --- Find starting position of row inusol in icol
!                  Find end position of row inusol in icol

               ktel = inusol-nrusol
               do k = irowsip(ktel)+1, irowsip(ktel+1)
                  iwork1(icolsip(k)) = k
               end do

               do j = 1, inunkp_elm

                  jnusol = inusolgr+j-1
                  imat = iwork1(jnusol)

!                 --- imat may be 0, since it is possible that some of
!                     the nodes of the element are not connected to boundary
!                     conditions while some are
!                     This is for example the case if a boundary element
!                     intersects a boundary fluid element, and some Lagrangian
!                     points are in the boundary element and others not

                  if ( imat>0 ) S_ip(imat) = S_ip(imat)+elemmtloc(j,i)

               end do  ! j = 1, inunkp_elm

               do k = irowsip(ktel)+1, irowsip(ktel+1)
                  iwork1(icolsip(k)) = 0
               end do

               ktel = inusol-nrusol
               do k = irowsp(ktel)+1, irowsp(ktel+1)
                  iwork1(icolsp(k)) = k
               end do

               do j = 1, inunkp_elm

                  jnusol = inusolgr+j-1
                  imat = iwork1(jnusol)

!                 --- imat may be 0, since it is possible that some of
!                     the nodes of the element are not connected to boundary
!                     conditions while some are
!                     This is for example the case if a boundary element
!                     intersects a boundary fluid element, and some Lagrangian
!                     points are in the boundary element and others not

                  if ( imat>0 ) then
                     imat = imat!+lengpu
                     S_pp(imat) = S_pp(imat)+elemmtloc(j,i)
                  end if  ! ( imat>0 )

               end do  ! j = 1, inunkp_elm

               do k = irowsp(ktel)+1, irowsp(ktel+1)
                  iwork1(icolsp(k)) = 0
               end do

            end if

         end do

      end if
1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End tofict02'

      end if  ! ( debug )
      call erclos ( 'tofict02' )

      end subroutine tofict02
