      subroutine tomanvec45_1 ( vecinp, vecout, indpfi, kprbfi, indpfo, &
                                kprbfo, npoint, vector, ndim, nunkp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 26-09-2001
!
!   copyright (c) 2001-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Perform vector manipulations for subroutine manvec in case of ichois = &
!    a vector is expanded per point by multiplying by a vector
! **********************************************************************
!
!                       KEYWORDS
!
!     manipulation
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision vecinp(*), vecout(*), vector(*)
      integer indpfi, kprbfi(*), indpfo, kprbfo(*), &
              npoint, ndim, nunkp

!     indpfi         i    indication if kprob part f has been filled (>0)
!                         or not (0)  input vector
!     indpfo         i    Indication if array KPROB part f has been filled (>0)
!                         or not (0)  Output vector only
!     kprbfi         i    Array KPROB part f for input vector
!     kprbfo         i    Array KPROB part f for output vector
!     ndim           i    Dimension of the space.
!     npoint         i    Number of nodal points
!     nunkp          i    Maximum number of unknowns in a point (output vector)
!     vecinp         i    Input vector to be mapped
!     vecout         o    Output vector, result of the mapping
!     vector         i    Contains the ndim components of the vector to be used
!                         in the manipulation
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, onunkp, inunkp, istart, ostart

!     i              Counting variable
!     inunkp         Number of degrees of freedom in point (input vector)
!     istart         First position used at a part (input vector)
!     j              Counting variable
!     onunkp         Number of degrees of freedom in point (output vector)
!     ostart         First position used at a part (output vector)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2279   Number of unknowns in point in input vector too small
!    2280   Number of unknowns in point in output vector incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'tomanvec45_1' )
      if ( indpfi.eq.0 ) then

!     --- indpfi = 0, number of unknowns per point (input) is constant

         if ( indpfo.eq.0 ) then

!        --- indpfo = 0, number of unknowns per point (output) is constant

             do i = 1, npoint
                do j = 1, ndim
                   vecout((i-1)*nunkp+j) = vecinp(i)*vector(j)
                end do
             end do

         else

!        --- indpfo # 0, number of unknowns per point (output) is  not constant

            do i = 1, npoint
               istart = kprbfo(i)
               inunkp = kprbfo(i+1)-istart
               if ( inunkp.eq.ndim ) then

!              --- ndim components in output vector

                  do j = 1, ndim
                     vecout(istart+j) = vecinp(i)*vector(j)
                  end do

               else

!              --- Not enough components in output vector

                  call errint ( i, 1 )
                  call errint ( inunkp, 2 )
                  call errint ( ndim, 3 )
                  call errsub ( 2280, 3, 0, 0 )

               end if
            end do

         end if

      else

!     --- indpfi > 0, number of unknowns per point is not constant

         if ( indpfo.eq.0 ) then

!        --- indpfo = 0, number of unknowns per point (output) is constant

            do i = 1, npoint
               istart = kprbfi(i)
               inunkp = kprbfi(i+1)-istart
               if ( inunkp.ge.1 ) then

!              --- enough components in input vector

                  do j = 1, ndim
                     vecout((i-1)*nunkp+j) = vecinp(istart+1)*vector(j)
                  end do

               else

!              --- Not enough components in input vector

                  call errint ( i, 1 )
                  call errint ( inunkp, 2 )
                  call errint ( 1, 3 )
                  call errsub ( 2279, 3, 0, 0 )

               end if

            end do

         else

!        --- indpfo > 0, number of unknowns per point (output) is  not constant

            do i = 1, npoint
               istart = kprbfi(i)
               inunkp = kprbfi(i+1)-istart
               ostart = kprbfo(i)
               onunkp = kprbfo(i+1)-ostart
               if ( onunkp.eq.ndim ) then

!              --- ndim degree of freedom in output vector

                  if ( inunkp.ge.1 ) then

!                 --- enough components in input vector

                     do j = 1, ndim
                        vecout(ostart+j) = vecinp(istart+1)*vector(j)
                     end do

                  else

!                 --- Not enough components in input vector

                     call errint ( i, 1 )
                     call errint ( inunkp, 2 )
                     call errint ( 1, 3 )
                     call errsub ( 2279, 3, 0, 0 )

                  end if

               else if ( onunkp.ne.0 .or. onunkp.eq.0 .and. &
                         inunkp.ne.0 ) then

!              --- Not enough components in output vector

                  call errint ( i, 1 )
                  call errint ( onunkp, 2 )
                  call errint ( ndim, 3 )
                  call errsub ( 2280, 3, 0, 0 )

               end if

            end do

         end if

      end if

      call erclos ( 'tomanvec45_1' )
      end
