      subroutine tomanvec46 ( ivec1, ivec2, iconst, ichoice )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.2  date 05-12-2012 Long integers
!        version  2.1  date 10-03-2004 ini060 --> ini060text
!        version  2.0  date 21-09-2002 Extra parameter ichoice
!
!   copyright (c) 2001-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subtract the normal component of a vector defined on one surface (3D)
!     or curve (2D) and store the result in another vector
!     The input vector must be a vector with ndim components
!     The normal on this curve or surface is computed and the normal
!     component of u in the n direction is subtracted from u
!     If ichoice = 47: compute u.n n
! **********************************************************************
!
!                       KEYWORDS
!
!     manipulation
!     vector
!     normal_vector
!     2d
!     3d
!     surface
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ivec1, ivec2, iconst(*), ichoice

!     ichoice        i    Choice parameter, see manvecbf
!     iconst         i    Contains some constants required to compute the vector
!                         1:  surface or curve number
!     ivec1          i    input vector to be manipulated
!                         ivec1 must be a SEPRAN array of length 5, which may be
!                         of type 110, 115, 116 or 119
!     ivec2         i/o   Output vector, may be the same as input vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer icurvs(2), isurnr, ivec, normaldr
      double precision, allocatable :: norm(:), weight(:)

!     icurvs         array containing information of the surface numbers along
!                    which the normal must be computed
!     isurnr         Surface sequence number
!     ivec           Sequence number of vector of special structure
!     norm           Array of length ndim * npoint containing the normal
!                    in the nodal points
!     normaldr       If 1 the outward directed normal is required, if -1
!                    the inward directed normal
!     weight         Help array to store the weights
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPACTSOLBF2   compute pointers to degrees of freedom of vectors
!     SEPCOMNOR      Compute the averaged normals in all points on the surfaces
!                    isrf1 to isrf2
!     SEPCREASOLBF1  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
!     TOMANVEC46_1   Subtract the normal component of a vector defined on one
!                    surface (3D) or curve (2D)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     222   input vector is complex:  not yet allowed.
!     419   NDIM # 2
!    2276   nphys in output vector<ndim
!    2282   surface number out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Check type of ivec1
!     Activate input vector and corresponding arrays
!     Perform some checks
!     Activate or create output vector and corresponding arrays
!     Compute normals on surface isurnr (array defined in all points)
!     Perform the actual action on the solution vector
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'tomanvec46' )

!     --- Check type of ivec1

      if ( ks(ivec1)%compl/=0 ) then
         call errsub ( 222, 0, 0, 0 )
         go to 1000
      end if

!     --- Activate input vector and corresponding arrays

      call sepactsolbf2 ( ivec1 )
      if ( nphysi<ndim ) then

!     --- Number of degrees of freedom incorrect

         call errint ( nphysi, 1 )
         call errint ( ndim, 2 )
         call errsub ( 2276, 2, 0, 0 )

      end if
      if ( ndim/=3 ) then

!     --- ndim # 3

         call errint ( ndim, 1 )
         call errint ( 3, 2 )
         call errint ( 3, 3 )
         call errsub ( 419, 3, 0, 0 )

      end if

!     --- Activate or create output vector and corresponding arrays

      iprob = ks(ivec1)%iprob
      ivec = ks(ivec1)%ivec
      call sepcreasolbf1 ( ivec2, ks(ivec1)%typevec, ivec, 'vector', &
                           ks(ivec1)%compl )

      isurnr = iconst(1)
      if ( isurnr<1 .or. isurnr>nsurfs ) then

!     --- surface number out of range

         call errint ( isurnr, 1 )
         call errint ( nsurfs, 2 )
         call errsub ( 2282, 2, 0, 0 )

      end if

!     --- Compute normals on surface isurnr (array defined in all points)
!         First reserve space

      allocate ( norm(3*npoint), weight(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'norm' )
      if ( ierror/=0 ) go to 1000

!     --- Compute starting addresses

      vectin => ks(ivec1)%sol
      if ( ivec1==ivec2 ) then
         vectin2 => vectin
         nphyso = nphysi
         indprfo = indprfi
         kprobfo => kprobfi
      else
         vectin2 => ks(ivec2)%sol
      end if
      if ( ierror/=0 ) go to 1000

      icurvs(1) = 0
      icurvs(2) = isurnr
      normaldr  = 1
      call sepcomnor ( norm, weight, icurvs, normaldr )

!     --- Perform the actual action on the solution vector

      call tomanvec46_1 ( vectin, vectin2, indprfo, &
                          kprobfo, nphyso, npoint, norm, &
                          weight, ichoice )

      deallocate ( norm, weight, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'norm' )

1000  call erclos ( 'tomanvec46' )
      end subroutine tomanvec46
