      subroutine erpsiboun ( npoint, usol, psi, coor, linear, jcart, &
                             idirec, anorm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-03-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the stream function along a boundary
!
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     curve
!     stream_function
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleconsts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint, linear, jcart, idirec
      double precision usol(npoint,2), psi(npoint), coor(2,npoint), &
                       anorm(2,npoint)

!     anorm          i    In this array the normal vector is stored
!                         anorm(j,i) contains the j th component of the normal
!                         vector in the i-th integration point
!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     idirec         i    Defines in which direction the curves must be
!                         considered
!                         idirec = 1, from low to high, idirec = -1: reversed
!                         order
!     jcart          i    Defines type of coordinate system
!                         1:  Cartesian
!                         2:  Axi-symmetric
!     linear         i    Defines type of elements along the boundary
!                         1:  Linear
!                         2:  Quadratic
!     npoint         i    Number of nodal points along the boundary
!     psi            o    Contains the stream function in the nodes of the
!                         boundary
!     usol           i    Solution vector of size npoint x 2
!                         Contains the velcoity vector in all boundary points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k
      double precision x0, x1, x2, y0, y1, y2, u0, u1, u2, v0, v1, v2, &
                       h, wg(3), phig1(9), xksi1(3), phig2(9), xksi2(3), &
                       dxdks1(3), dxdks2(3), dydks1(3), dydks2(3), &
                       un0, un1, sums1, sums2, p1k, p1k3, p1k6, p2k, &
                       p2k3, p2k6, ug11, ug12, ug21, ug22, xg11, xg12, &
                       xx1, xx2, yy1, yy2

!     dxdks1         ?
!     dxdks2         ?
!     dydks1         ?
!     dydks2         ?
!     h              Length of element
!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     p1k            ?
!     p1k3           ?
!     p1k6           ?
!     p2k            ?
!     p2k3           ?
!     p2k6           ?
!     phig1          ?
!     phig2          ?
!     sums1          ?
!     sums2          ?
!     u0             ?
!     u1             ?
!     u2             Second component of velocity vector
!     ug11           ?
!     ug12           ?
!     ug21           ?
!     ug22           ?
!     un0            Normal velocity component in first point of element
!     un1            Normal velocity component in second point of element
!     v0             ?
!     v1
!     v2
!     wg             Array containing the weights to be applied for the
!                    computation of the normals
!     x0             x-coordinate of first point of element
!     x1             x-coordinate of second point of element
!     x2             x-coordinate of last point of element
!     xg11           ?
!     xg12           ?
!     xksi1          ?
!     xksi2          ?
!     xx1            ?
!     xx2            ?
!     y0             y-coordinate of first point of element
!     y1             y-coordinate of second point of element
!     y2             y-coordinate of last point of element
!     yy1            ?
!     yy2            ?
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!                       x(i)
!    psi(i) - psi(0) = /     u.n dx
!                      x(0)
!
!   Remark: the method for quadratic elements is far too complicated
!           Use simple formulas in the future
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data wg /0.5555555555555556d0, 0.8888888888888889d0, &
               0.5555555555555556d0/
      data phig1 /0.687298334620742d0 ,0.0d0, -0.087298334620742d0, &
                  0.4d0               ,1.0d0,  0.4d0, &
                 -0.087298334620742d0 ,0.0d0, 0.687298334620742d0/
      data xksi1/-0.774596669241483d0 ,0.0d0, 0.774596669241483d0/
      data phig2 /0.837298334620742d0 ,0.375d0, 0.062701665379258d0, &
                  0.212701665379258d0 ,0.75d0 , 0.987298334620742d0, &
                 -0.05d0             ,-0.125d0,-0.05d0/
      data xksi2/-0.887298334620742d0 ,-0.5d0 ,-0.112701665379258d0/
! ======================================================================
!
      call eropen ( 'erpsiboun' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erpsiboun'
         write(irefwr,*) 'linear, jcart, idirec ', &
                          linear, jcart, idirec

      end if

      if ( linear.eq.1 ) then

!     --- linear elements

         if ( idirec.eq.1 ) then

!        --- Forwards direction

            psi(1) = 0d0
            if ( jcart.eq.1 ) then

!           --- jcart = 1, cartesian coordinates, trapezoid rule

               un0 = usol(1,1)*anorm(1,1) + usol(1,2)*anorm(2,1)
               x0 = coor(1,1)
               y0 = coor(2,1)
               do i = 2, npoint
                  un1 = usol(i,1)*anorm(1,i) + usol(i,2)*anorm(2,i)
                  x1 = coor(1,i)
                  y1 = coor(2,i)
                  h = sqrt((x1-x0)**2+(y1-y0)**2)
                  psi(i) = psi(i-1) + h*0.5d0*(un0+un1)
                  x0 = x1
                  y0 = y1
                  un0 = un1
               end do  ! i = 2, npoint

            else

!           --- jcart = 2, axi-symmetric coordinates, trapezoid rule

               un0 = usol(1,1)*anorm(1,1) + usol(1,2)*anorm(2,1)
               x0 = coor(1,1)
               y0 = coor(2,1)
               do i = 2, npoint
                  un1 = usol(i,1)*anorm(1,i) + usol(i,2)*anorm(2,i)
                  x1 = coor(1,i)
                  y1 = coor(2,i)
                  h = sqrt((x1-x0)**2+(y1-y0)**2)
                  psi(i) = psi(i-1) + h*pi*(un0*x0+un1*x1)
                  x0 = x1
                  y0 = y1
                  un0 = un1
               end do  ! i = 2, npoint

            end if  ! ( jcart.eq.1 )

         else

!        --- Backwards direction

            psi(npoint) = 0d0
            if ( jcart.eq.1 ) then

!           --- jcart = 1, cartesian coordinates

               un0 = usol(npoint,1)*anorm(1,npoint) + &
                     usol(npoint,2)*anorm(2,npoint)
               x0 = coor(1,npoint)
               y0 = coor(2,npoint)
               do i = npoint-1, 1, -1
                  un1 = usol(i,1)*anorm(1,i) + usol(i,2)*anorm(2,i)
                  x1 = coor(1,i)
                  y1 = coor(2,i)
                  h = sqrt((x1-x0)**2+(y1-y0)**2)
                  psi(i-1) = psi(i) + h*0.5d0*(un0+un1)
                  x0 = x1
                  y0 = y1
                  un0 = un1
               end do  ! i = 2, npoint

            else

!           --- jcart = 2, axi-symmetric coordinates

               un0 = usol(npoint,1)*anorm(1,npoint) + &
                     usol(npoint,2)*anorm(2,npoint)
               x0 = coor(1,npoint)
               y0 = coor(2,npoint)
               do i = npoint-1, 1, -1
                  un1 = usol(i,1)*anorm(1,i) + usol(i,2)*anorm(2,i)
                  x1 = coor(1,i)
                  y1 = coor(2,i)
                  h = sqrt((x1-x0)**2+(y1-y0)**2)
                  psi(i-1) = psi(i) + h*pi*(un0*x0+un1*x1)
                  x0 = x1
                  y0 = y1
                  un0 = un1
               end do  ! i = 2, npoint

            end if  ! ( jcart.eq.1 )

         end if  ! ( idirec.eq.1 )

      else

!     --- quadratic elements, can be much simpler

         if ( idirec.eq.1 ) then

!        --- Forwards direction

            psi(1) = 0d0
            if ( jcart.eq.1 ) then

!           --- jcart = 1, cartesian coordinates Simpsons rule

               do i = 2, npoint, 2
                  x0 = coor(1,i-1)
                  x1 = coor(1,i)
                  x2 = coor(1,i+1)
                  y0 = coor(2,i-1)
                  y1 = coor(2,i)
                  y2 = coor(2,i+1)
                  u0 = usol(i-1,1)
                  u1 = usol(i,1)
                  u2 = usol(i+1,1)
                  v0 = usol(i-1,2)
                  v1 = usol(i,2)
                  v2 = usol(i+1,2)
                  xx1 = x0-2d0*x1+x2
                  yy1 = y0-2d0*y1+y2
                  xx2 = -0.5d0*(x0-x2)
                  yy2 = -0.5d0*(y0-y2)
                  psi(i+1) = psi(i-1)+ &
                             (yy2*(u0+4d0*u1+u2) - yy1*(u0-u2) &
                             -xx2*(v0+4d0*v1+v2) + xx1*(v0-v2))/3d0
                  psi(i) = psi(i-1)+ &
                           (yy2*(5d0*u0+8d0*u1-u2) &
                           -yy1*(3.5d0*u0+3*u1-0.5d0*u2) &
                           -xx2*(5d0*v0+8d0*v1-v2) &
                           +xx1*(3.5d0*v0+3*v1-0.5d0*v2))/12d0
               end do

            else

!           --- jcart = 2, axi-symmetric coordinates

               do i = 2, npoint, 2
                  x0 = coor(1,i-1)
                  x1 = coor(1,i)
                  x2 = coor(1,i+1)
                  y0 = coor(2,i-1)
                  y1 = coor(2,i)
                  y2 = coor(2,i+1)
                  u0 = usol(i-1,1)
                  u1 = usol(i,1)
                  u2 = usol(i+1,1)
                  v0 = usol(i-1,2)
                  v1 = usol(i,2)
                  v2 = usol(i+1,2)
                  xx1 = x0-2d0*x1+x2
                  yy1 = y0-2d0*y1+y2
                  xx2 = -0.5d0*(x0-x2)
                  yy2 = -0.5d0*(y0-y2)
                  do j=1,3
                     dxdks1(j)=xksi1(j)*xx1+xx2
                     dxdks2(j)=xksi2(j)*xx1+xx2
                     dydks1(j)=xksi1(j)*yy1+yy2
                     dydks2(j)=xksi2(j)*yy1+yy2
                  end do

                  sums1=0.0d0
                  sums2=0.0d0
                  do k=1,3

!                 --- compute u1,u2,x1 in integration points on both intervals

                     p1k =phig1(k)
                     p1k3=phig1(k+3)
                     p1k6=phig1(k+6)
                     p2k =phig2(k)
                     p2k3=phig2(k+3)
                     p2k6=phig2(k+6)
                     ug11=u0*p1k+u1*p1k3+u2*p1k6
                     ug12=u0*p2k+u1*p2k3+u2*p2k6
                     ug21=v0*p1k+v1*p1k3+v2*p1k6
                     ug22=v0*p2k+v1*p2k3+v2*p2k6
                     xg11=x0*p1k+x1*p1k3+x2*p1k6
                     xg12=x0*p2k+x1*p2k3+x2*p2k6
                     sums1=sums1+(ug11*dydks1(k)- &
                                  ug21*dxdks1(k))*xg11*wg(k)
                     sums2=sums2+(ug12*dydks2(k)- &
                                  ug22*dxdks2(k))*xg12*wg(k)

                  end do
                  psi(i+1) = psi(i-1)+sums1*2d0*pi
                  psi(i) = psi(i-1)+sums2*pi
               end do

            end if  ! ( jcart.eq.1 )

         else

!        --- Backwards direction

            psi(npoint) = 0d0
            if ( jcart.eq.1 ) then

!           --- jcart = 1, cartesian coordinates

               do i = npoint-1, 2, -2
                  x0 = coor(1,i+1)
                  x1 = coor(1,i)
                  x2 = coor(1,i-1)
                  y0 = coor(2,i+1)
                  y1 = coor(2,i)
                  y2 = coor(2,i-1)
                  u0 = usol(i+1,1)
                  u1 = usol(i,1)
                  u2 = usol(i-1,1)
                  v0 = usol(i+1,2)
                  v1 = usol(i,2)
                  v2 = usol(i-1,2)
                  xx1 = x0-2d0*x1+x2
                  yy1 = y0-2d0*y1+y2
                  xx2 = -0.5d0*(x0-x2)
                  yy2 = -0.5d0*(y0-y2)
                  psi(i-1) = psi(i+1)+ &
                             (yy2*(u0+4d0*u1+u2) - yy1*(u0-u2) &
                             -xx2*(v0+4d0*v1+v2) + xx1*(v0-v2))/3d0
                  psi(i) = psi(i+1)+ &
                           (yy2*(5d0*u0+8d0*u1-u2) &
                           -yy1*(3.5d0*u0+3*u1-0.5d0*u2) &
                           -xx2*(5d0*v0+8d0*v1-v2) &
                           +xx1*(3.5d0*v0+3*v1-0.5d0*v2))/12d0
               end do

            else

!           --- jcart = 2, axi-symmetric coordinates

               do i = npoint-1, 2, -2
                  x0 = coor(1,i+1)
                  x1 = coor(1,i)
                  x2 = coor(1,i-1)
                  y0 = coor(2,i+1)
                  y1 = coor(2,i)
                  y2 = coor(2,i-1)
                  u0 = usol(i+1,1)
                  u1 = usol(i,1)
                  u2 = usol(i-1,1)
                  v0 = usol(i+1,2)
                  v1 = usol(i,2)
                  v2 = usol(i-1,2)
                  xx1 = x0-2d0*x1+x2
                  yy1 = y0-2d0*y1+y2
                  xx2 = -0.5d0*(x0-x2)
                  yy2 = -0.5d0*(y0-y2)
                  do j=1,3
                     dxdks1(j)=xksi1(j)*xx1+xx2
                     dxdks2(j)=xksi2(j)*xx1+xx2
                     dydks1(j)=xksi1(j)*yy1+yy2
                     dydks2(j)=xksi2(j)*yy1+yy2
                  end do

                  sums1=0.0d0
                  sums2=0.0d0
                  do k=1,3

!                 --- compute u1,u2,x1 in integration points on both intervals

                     p1k =phig1(k)
                     p1k3=phig1(k+3)
                     p1k6=phig1(k+6)
                     p2k =phig2(k)
                     p2k3=phig2(k+3)
                     p2k6=phig2(k+6)
                     ug11=u0*p1k+u1*p1k3+u2*p1k6
                     ug12=u0*p2k+u1*p2k3+u2*p2k6
                     ug21=v0*p1k+v1*p1k3+v2*p1k6
                     ug22=v0*p2k+v1*p2k3+v2*p2k6
                     xg11=x0*p1k+x1*p1k3+x2*p1k6
                     xg12=x0*p2k+x1*p2k3+x2*p2k6
                     sums1=sums1+(ug11*dydks1(k)- &
                                  ug21*dxdks1(k))*xg11*wg(k)
                     sums2=sums2+(ug12*dydks2(k)- &
                                  ug22*dxdks2(k))*xg12*wg(k)

                  end do
                  psi(i-1) = psi(i+1)+sums1*2d0*pi
                  psi(i) = psi(i+1)+sums2*pi
               end do

            end if  ! ( jcart.eq.1 )

         end if  ! ( idirec.eq.1 )

      end if  ! ( linear.eq.1 )
      call erclos ( 'erpsiboun' )
      if ( debug ) then

!     --- Debug information

         call prinrl ( psi, npoint, 'psi' )
         write(irefwr,*) 'End erpsiboun'

      end if

      end
