      subroutine prcompdistnod1 ( node, coor, ndim, nx, origin, dx, &
                                  grid, markintf, nmeshc, nmeshl, &
                                  ninterf, phi, isigphi, coorinter )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 18-09-2012 Extend length of array set
!        version  1.0  date 24-08-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute distance of node to interface
!     It concerns the case of a non-adapted mesh
!     This subroutine is the equivalent of prcompdistnod
! **********************************************************************
!
!                       KEYWORDS
!
!     level_set
!     distance
!     interface
!     2d
!     node
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer node, ndim, nx(ndim), grid(*), markintf(*), nmeshc(*), &
              nmeshl(*), ninterf, isigphi
      double precision coor(ndim,*), origin(ndim), dx(ndim), phi(*), &
                       coorinter(ndim,*)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     coorinter      i    Contains coordinates of the interface nodes
!                         in the local numbering of the interface
!     dx             i    contains the difference between xmax, xmin etc
!     grid           i    The domain is subdivided in nx(1) x nx(2) cells
!                         of size dx(1) x dx(2)
!                         In grid the number of interface nodes per cell is
!                         stored as well as the node numbers in the folwoing way
!                         Pos. 1 starting address of node numbers
!                         (nx(1)*nx(2)+1)))
!                         The number of nodes in cell ij is defined by
!                         grid(start+1)-grid(start). where
!                         start = (j)-1)*nx(1)+i
!                         The corresponding node numbers are stored in grid
!                         from position start
!     isigphi        i    sign of phi (1 if phi>0), (-1 if phi<0)
!     markintf      i/o   Array of size nedges or nfaces to mark the edges or
!                         faces of the interface (1) else 0
!     ndim           i    Dimension of the space.
!     ninterf        i    Number of nodes in interface
!     nmeshc         i    nmesh part c, contains local node numbers
!                         corresponding to local edges of interface
!     nmeshl         i    nmesh part l, contains edge numbers corresponding
!                         to local nodes of interface
!     node           i    Node number
!     nx             i    number of points in ndim directions of the grid
!     origin         i    Contains the coordinates of the origin of the region
!     phi           i/o   contains the level set function phi
!                         (distance)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer indi(ndim), nset, numfound, ishift
      double precision mindist
      integer, allocatable :: set(:,:), nodes(:)
      logical found

!     error          Return value of allocate or deallocate
!     found          If true the set of cells contains nodes of the
!                    interface
!     indi           contains ijk-index of cell in which point is lying
!     ishift         Shift with respect to center cell
!     mindist        Minimum distance
!     nodes          Contains the node numbers of the interface that are lying
!                    in the set of cells we are considering
!     nset           Number of cells in set
!     numfound       number of nodes in array nodes
!     set            contains the ijk innidces of the set of cells
!                    we consider
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      logical prsetmindist

!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
!     PRINRL         Print 1d real vector
!     PRMINDIST1     Find minimum distance of node to set of nodes in interface
!     PRNODECELLIJ   Find ij cell index of node
!     PRSETCELLS     make a new set of cells, given the index of the center cell
!                    and the shift
!     PRSETMINDIST   Return with the number of nodes of the interface that are
!                    inside the set of cells
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Compute cell index ij of node cell
!     found := false
!     set := cell_ij
!     min_dist := infinity
!     while ( not found ) do
!        found := set contains nodes of interface
!        if ( found  ) then
!           min_dist := min ( min_dist, &
!                       minimum distance to all segments of interface in set )
!        set := all neighbor cells of present set (not yet treated)
!
!     found := set contains nodes of interface
!     if ( found  ) then
!        min_dist := min ( min_dist, &
!                    minimum distance to all segments of interface in set )
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcompdistnod1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcompdistnod1'
         write(irefwr,1) 'node, ndim, isigphi', node, ndim, isigphi
         write(irefwr,1) 'nx', nx
         call prinrl ( coor(1,node), ndim, 'x' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Compute cell index ij of node cell and position in grid

      call prnodecellij ( indi, nx, origin, dx, ndim, coor(1,node) )

      if ( debug ) call prinin ( indi, ndim, 'indi' )

!     --- found := false
!         set := cell_ij
!         min_dist := infinity

      length = 10*ndim**2*maxval(nx)
      if ( debug ) write(irefwr,1) 'length', length

      allocate ( set(ndim,length), nodes(ninterf), stat = error )
      if ( error/=0 ) call eralloc ( error, ninterf+length, 'set' )
      if ( ierror/=0 ) go to 1000

      found = .false.
      nset = 1
      set(1:ndim,1) = indi
      mindist = rinfin

      ishift = 1

!     --- while ( not found ) do

      do while ( .not. found )

!     --- found := set contains nodes of interface

         found = prsetmindist ( set, nset, ndim, nx, grid, &
                                numfound, nodes )

         if ( debug ) then
            write(irefwr,*) 'found', found
            call prinin ( nodes, numfound, 'nodes' )
         end if  ! ( debug )

         if ( found  ) then

!        --- Nodes of interface are part of the set
!            min_dist := min ( min_dist, &
!                        minimum distance to all segments of interface in set )

            call prmindist1 ( numfound, nodes, ndim, mindist, coor, &
                              node, markintf, nmeshc, nmeshl, &
                              coorinter )

            if ( debug ) write(irefwr,2) 'mindist', mindist

         end if  ! ( found  )
         if ( ierror/=0 ) go to 1000

!        --- set := all neighbor cells of present set

         call prsetcells ( set, nset, ndim, ishift, nx, indi, length )
         if ( ierror/=0 ) go to 1000

         if ( debug ) then
            write(irefwr,*) 'found/1', found
            call prinin1 ( set, nset, ndim, 'set' )
         end if  ! ( debug )

      end do  ! while ( .not. found )

!     --- found := set contains nodes of interface

      found =  prsetmindist ( set, nset, ndim, nx, grid, &
                              numfound, nodes )
      if ( ierror/=0 ) go to 1000

      if ( debug ) write(irefwr,*) 'found/2', found
      if ( found  ) then

!     --- Nodes of interface are part of the set
!         min_dist := min ( min_dist, &
!                     minimum distance to all segments of interface in set )

         call prmindist1 ( numfound, nodes, ndim, mindist, coor, &
                           node, markintf, nmeshc, nmeshl, coorinter )

         if ( debug ) write(irefwr,2) 'mindist', mindist

      end if  ! ( found  )
      if ( ierror/=0 ) go to 1000

      if ( debug ) then
         write(irefwr,1) 'node', node
         write(irefwr,2) 'philold, phinew', &
                          phi(node), isigphi*sqrt(mindist)
      end if  ! ( debug )

      phi(node) = isigphi*sqrt(mindist)

      deallocate ( set, nodes, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'set' )
      if ( ierror/=0 ) go to 1000

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prcompdistnod1'

      end if  ! ( debug )
      call erclos ( 'prcompdistnod1' )

      end
