      subroutine prcompinvmat ( iinputinv, rinputinv )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 11-12-2012 Long integers
!        version  1.0  date 25-10-1999
!
!   copyright (c) 1999-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute the matrix S for the inverse problem
!
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix
!     inverse_problem
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemnsub
      use sepmodulemain
      use sepmodulekmesh
      use sepmoduletime
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinputinv(*)
      double precision rinputinv(*)

!     iinputinv      i    Integer input array for subroutine prcompinvmat
!                         Contents
!                          1: Largest entry that has been filled by the user
!                          2: iseqsl:  sequence number of input for linear
!                                      solver
!                          3: iseqcf:  sequence number of input for coefficients
!                          4: iveccapacity: sequence number of the capacity
!                                           vector in array isolut
!                          5: ibegincv: curve number of first electrode
!                          6: iendcv : curve number of last electrode
!                          7: ivecsol: sequence number of the solution
!                                      vector in array isolut
!                          8: ivecreact: sequence number of the reaction
!                                        vector in array isolut
!                          9: ielgrploc: element group sequence number
!                                        corresponding to inverse problem
!                         10: iveceps: sequence number in array solut of
!                                      array in which the values of epsilon
!                                      are stored
!     rinputinv      i    Double precision input array for subroutine
!                         prcompinvmat
!                         Contents
!                          1: value of delta_eps
!                          2: value of regularization parameter mu
!                          3: value of eps_ref
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iinputcap(8), ibegincv, iendcv, iseqcf, &
              iseqsl, nelectrodes, ielem, ielgrp, ielgrploc, ikelmc, &
              inpelm, irelem, iveccapacity, nelemlc, &
              iveceps, ivecreact, ivecsol, lengcvec, &
              itimloopsave
      double precision amax, constant, epsilon0, epsilon1
      double precision, pointer :: volumes(:)

!     amax           Maximum value in array
!     constant       Value of constant
!     epsilon0       Value of epsilon_0
!     epsilon1       Value of epsilon_1
!     ibegincv       First curve number
!     ielem          Element number
!     ielgrp         element group number
!     ielgrploc      local element group number
!     iendcv         Last curve number
!     iinputcap      Integer input array for subroutine compcapacity
!     ikelmc         position in kmesh part c
!     inpelm         number of points per element
!     irelem         Relative element sequence number with respect to
!                    element group
!     iseqcf         Sequence number corresponding to the coefficients
!     iseqsl         Sequence number corresponding to the linear solver
!     itimloopsave   Help variable to store the value of itimloop
!     iveccapacity   Sequence number of capacity vector in isol
!     iveceps        Sequence number of array epsilon in isol
!     ivecreact      Sequence number of reaction vector in isol
!     ivecsol        Sequence number of solution vector in isol
!     lengcvec       Length of the capacity vectors
!     nelectrodes    Number of electrodes
!     nelemlc        Local number of elements
!     volumes        volumes of the elements
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRCOMPCAPACITY Compute the capacity of all electrodes and store in array c
!     PRCOMPINVMT0   Fill matrix S for the inverse problem with one row
!     PRCOMPVOL      Compute volume of all elements and store in array volumes
!     PRININ         print 1d integer array
!     SEPCREASOLVEC  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2133   Sequence number of vector too large
! **********************************************************************
!
!                       PSEUDO CODE
!
!     ielgrp := element group corresponding to region where epsilon must
!               be detected
!     N := number of electrodes
!     epsilon_0 := eps_ref
!     epsilon_1 := epsilon_0+delta_eps
!     Compute length of vector C: N (N-1)/2
!     Set epsilon vector equal to epsilon_0
!     Compute vector C_0 for this epsilon
!     Set epsilon vector equal to epsilon_1
!     Compute vector C_1 for this epsilon
!     Compute maximum volume of elements in element group ielgrp (A_max)
!     For all elements ielem in element group ielgrp do
!        epsilon vector := epsilon_0
!        epsilon vector(ielem) := epsilon_1
!        Compute vector C_ielem
!        A := volume of element ielem
!        S_ielem := A_max/A (C_ielem-C_0)/(C_1-C_0)
!                (component-wise)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcompinvmat' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcompinvmat'
         write(irefwr,1) 'numvecmn, maxvc1', numvecmn, maxvc1
         call prinin ( iinputinv, iinputinv(1), 'iinputinv' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- initializations

      iseqsl = 1
      iseqcf = 1
      ibegincv = 1
      iendcv   = 2
      iveccapacity = 3
      ivecsol = 1
      ivecreact = 2
      iveceps = 4
      ielgrploc = 1
      if ( iinputinv(1)>1 ) iseqsl = iinputinv(2)
      if ( iinputinv(1)>2 ) iseqcf = iinputinv(3)
      if ( iinputinv(1)>3 ) iveccapacity = iinputinv(4)
      if ( iinputinv(1)>4 ) ibegincv = iinputinv(5)
      if ( iinputinv(1)>5 ) iendcv = iinputinv(6)
      if ( iinputinv(1)>6 ) ivecsol = iinputinv(7)
      if ( iinputinv(1)>7 ) ivecreact = iinputinv(8)
      if ( iinputinv(1)>8 ) ielgrploc = iinputinv(9)
      if ( iinputinv(1)>9 ) iveceps = iinputinv(10)
      if ( iveccapacity>maxvc1 ) then

!     --- error: iveccapacity too large

         call errint ( iveccapacity, 1 )
         call errint ( maxvc1, 2 )
         call errchr ( 'capacity', 1 )
         call errsub ( 2133, 2, 0, 1 )

      end if
      if ( ivecsol>maxvc1 ) then

!     --- error: ivecsol too large

         call errint ( ivecsol, 1 )
         call errint ( maxvc1, 2 )
         call errchr ( 'solution', 1 )
         call errsub ( 2133, 2, 0, 1 )

      end if
      if ( ivecreact>maxvc1 ) then

!     --- error: ivecreact too large

         call errint ( ivecreact, 1 )
         call errint ( maxvc1, 2 )
         call errchr ( 'reaction vector', 1 )
         call errsub ( 2133, 2, 0, 1 )

      end if
      if ( iveceps>maxvc1 ) then

!     --- error: iveceps too large

         call errint ( iveceps, 1 )
         call errint ( maxvc1, 2 )
         call errchr ( 'epsilon vector', 1 )
         call errsub ( 2133, 2, 0, 1 )

      end if
      numvecmn = max(numvecmn,iveceps,ivecreact,ivecsol,iveccapacity)
      nelectrodes = iendcv-ibegincv+1
      epsilon0 = rinputinv(3)
      epsilon1 = epsilon0+rinputinv(1)

!     --- Compute length of vector C: N (N-1)/2

      lengcvec = nelectrodes*(nelectrodes-1)/2

!     --- Set epsilon vector equal to epsilon_1

      call sepcreasolbf1 ( isolut2d(iveceps), 116, 1, 'epsilon', 0 )
      veceps => usoltemp
      veceps = epsilon1

!     --- Compute vector C_1 for this epsilon

      itimloopsave = itimloop
      iinputcap(1) = 8
      iinputcap(2) = iseqsl
      iinputcap(3) = iseqcf
      iinputcap(4) = iveccapacity
      iinputcap(5) = ibegincv
      iinputcap(6) = iendcv
      iinputcap(7) = ivecsol
      iinputcap(8) = ivecreact
      call prcompcapacity ( iinputcap )
      if ( debug ) write(irefwr,*) 'after prcompcapacity/1'

!     --- Copy into c1

      allocate ( vecc1(size(capacity)) )
      vecc1 = capacity

!     --- Set epsilon vector equal to epsilon_0

      veceps= epsilon0

!     --- Compute vector C_0 for this epsilon

      itimloop = max(2,itimloop)
      call prcompcapacity ( iinputcap )
      if ( debug ) write(irefwr,*) 'after prcompcapacity/2'

!     --- Copy into c0

      allocate ( vecc0(size(capacity)) )
      vecc0 = capacity

!     --- Compute maximum volume of elements in element group ielgrp (A_max)
!         Create array volumes

      allocate ( volumes(nelem) )

!     --- compute volumes of all elements

      call prcompvol ( volumes )
      if ( debug ) write(irefwr,*) 'after prcompvol'

!     --- Store maximum in amax

      amax = maxval ( volumes )
      if ( debug ) write(irefwr,2) 'amax', amax

!     --- For all elements ielem in element group ielgrp do

      ikelmc = 0
      ielem = 0
      do ielgrp = 1, ielgrploc-1
         inelem = Topgrp(ielgrp)%inelem
         inpelm = Topgrp(ielgrp)%inpelm
         ikelmc = ikelmc+inelem*inpelm
         ielem = ielem+inelem
      end do

!     --- Create matrix S

      inelem = Topgrp(ielgrploc)%inelem
      allocate ( matrixs(inelem*lengcvec) )

      if ( debug ) write(irefwr,1) 'inelem', inelem

      nelemlc = inelem
      do irelem = 1, nelemlc

!     --- epsilon vector(ielem) := epsilon_1

         ielem = ielem+1
         veceps(ielem) = epsilon1

!        --- Compute vector C_ielem

         call prcompcapacity ( iinputcap )

!        --- A := volume of element ielem
!            S_ielem := A_max/A (C_ielem-C_0)/(C_1-C_0)
!                (component-wise)

         constant = amax/(epsilon1-epsilon0)
         call prcompinvmt0 ( matrixs, capacity, &
                             vecc0, vecc1, constant, &
                             volumes, ielem, irelem, nelemlc, lengcvec )

!        --- reset epsilon vector

         veceps(ielem) = epsilon0

      end do
      itimloop = itimloopsave

      if ( debug ) write(irefwr,*) 'after loop'

!     --- delete array volumes

      deallocate ( volumes )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prcompinvmat'

      end if  ! ( debug )
      call erclos ( 'prcompinvmat' )

      end subroutine prcompinvmat
