      subroutine prfillintfnmc1 ( iwork, npoint, numbndobst, ipointobst, &
                                  nmeshl, imark, nmeshcglob, istore, &
                                  numedges, numintersect, lenistore )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 29-05-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Mark all local edges that have a global node on the interface
!     Store all local edges that have two global nodes on the interface
!     Hence all local edges on the interface that are also global edges
!     are stored
! **********************************************************************
!
!                       KEYWORDS
!
!     interface
!     level_set
!     node
!     edge
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint, iwork(npoint), numbndobst, ipointobst(numbndobst), &
              nmeshl(*), imark(*), nmeshcglob(*), lenistore, numedges, &
              numintersect, istore(2,lenistore)

!     imark         i/o   Integer array to mark edges that have been used
!                         not used (0), used (1)
!     ipointobst     i    Array containing information about the nodes in an
!                         obstacle
!                         See PG Section 24.2 for a description
!                         In this case only the part with respect to the
!                         interface nodes is considered, hence the starting
!                         address must be adapted
!     istore         o    Work array to store nodes of global edges that are
!                         completely inside the interface, i.e. both nodes
!                         are global nodes on the interface
!                         It concerns the relative node numbers with respect
!                         to the interface nodes as stored in ipointobst
!                         + numintersect
!     iwork         i/o   integer work array to mark all global nodes on
!                         the interface
!                         iwork(i) = 0: node not on interface
!                         iwork(i) > 0: the node is on the interface.
!                         The value indicates the sequence number in array
!                         ipointobst
!                         At input iwork must be zero
!     lenistore      i    Declared length of arrays istore
!     nmeshcglob     i    Global array nmeshc, containing global node numbers
!                         corresponding to global edges
!     nmeshl         i    nmesh part l, contains edge numbers corresponding
!                         to nodes (global numbers)
!     npoint         i    Number of nodal points
!     numbndobst     i    Number of nodal points on the boundary of the obstacle
!     numedges       o    number of edges that are stored in istore
!     numintersect   i    Number of intersections with edges
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, inode, nedgeslc, nmlstart, j, iedge, nmcstart, nnodes, &
              node1, node2, jnode, ih

!     i              Counting variable
!     iedge          edge sequence number
!     ih             Help variable to store a constant temporarily
!     inode          Node sequence number of node on interface
!     j              Counting variable
!     jnode          Node sequence number of second node on interface
!     nedgeslc       number of edges
!     nmcstart       starting address in nmeshcglob
!     nmlstart       starting address in nmeshl
!     nnodes         Number of nodes in edge
!     node1          Node of edge
!     node2          Node of edge
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prfillintfnmc1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prfillintfnmc1'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Set iwork equal to relative node number for all nodes on the interface
!         relative with respect to the sequence in ipointobst

      do i = 1, numbndobst
         iwork(ipointobst(i)) = i  ! seq. number of interface node
      end do  ! i = 1, numbndobst

      if ( debug ) call prinin ( iwork, npoint, 'iwork' )

!     --- Store all edges that are on the interface in istore

      do i = 1, numbndobst

         inode = ipointobst(i)         ! node on interface
         nmlstart = nmeshl(inode)      ! starting address in nmeshl
         nedgeslc = nmeshl(nmlstart)     ! number of edges connected to node

         if ( debug ) &
            write(irefwr,1) 'i, inode, nedgeslc, iwork(inode)', &
                             i, inode, nedgeslc, iwork(inode)

         do j = 1, nedgeslc              ! Loop over edges

            iedge = nmeshl(nmlstart+j) ! neighbor edge of node found

            if ( imark(iedge)==0 ) then

!           --- New edge found

               nmcstart = nmeshcglob(iedge) ! starting address in nmeshcglob
               nnodes = nmeshcglob(nmcstart) ! number of nodes in edge

               node1 = nmeshcglob(nmcstart+1)      ! start node
               node2 = nmeshcglob(nmcstart+nnodes) ! end node

               if ( debug ) &
                  write(irefwr,1) 'j, iedge, node1, node2', &
                                   j, iedge, node1, node2

!              --- Find which of the nodes is the other one
!                  edge is defined by inode and jnode

               if ( node1==inode ) then
                  jnode = node2   ! inode = node1
               else
                  jnode = node1   ! inode = node2
               end if  ! ( node1==inode )

               ih = iwork(jnode)   ! Reference in interface
               if ( ih>0 ) then

!              --- Other node in interface, store edge

                  numedges = numedges+1
                  istore(1,numedges) = iwork(inode)+numintersect
                  istore(2,numedges) = ih+numintersect

               end if  ! ( ih>0 )

               imark(iedge) = 1  ! mark edge as used

            end if  ! ( imark(iedge)==0 )

         end do  ! j = 1, nedgeslc

      end do  ! i = 1, numbndobst

1000  if ( debug ) then

!     --- Debug information

         call prinin1 ( istore, numedges, 2, 'istore' )
         write(irefwr,*) 'End prfillintfnmc1'

      end if  ! ( debug )
      call erclos ( 'prfillintfnmc1' )

      end
