      subroutine prfillmass2d ( massmat, linear, kmeshm, ncurvs, coor, npoint )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-05-2010
!
!   copyright (c) 2010-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill 2d mass matrix for outer boundary
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mass_matrix
!     outer_boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshm(*), ncurvs, npoint
      double precision massmat(npoint), coor(*)
      logical linear

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     kmeshm         i    KMESH part m, contains nodal point numbers of the
!                         curves
!     linear         i    Indicates whether the elements are linear
!                         (true) or not (false)
!     massmat        o    mass matrix
!     ncurvs         i    Number of curves in mesh
!     npoint         i    Size of massmat
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ninner, istar3, istar4, iwork(ncurvs), k1, k2, j, &
              index1(3), icurnr
      double precision x(3,2), w(3), dphidx(3,3), xgauss(3,2), phi(3,3), &
                       tang(3,2), qmat(2,2,3), phider(3,3)

!     dphidx         Dummy
!     icurnr         Curve sequence number
!     index1         Array of length inpelm containing the point numbers
!                    of the nodal points in the element
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     istar4         Starting address of KMESHM part 4 in KMESH part M
!     iwork          integer work array of length ncurvs
!                    iwork(i) = 1, if curve i inner curve
!     j              Counting variable
!     k1             First point of curve in KMESH part m
!     k2             Last point of curve in KMESH part m
!     ninner         Number of inner curves
!     phi            dummy
!     phider         dummy
!     qmat           dummy
!     tang           dummy
!     w              array of length m containing the weights for
!                    integration
!     x              contains coordinates
!     xgauss         dummy
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELP610         Compute basis functions and weights for linear line element
!     ELP616         Compute basis functions and weights for quadratic line
!                    element R2
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear mass matrix
!     For all elements along outer boundary do
!        Compute element mass matrix
!        Add to large matrix
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prfillmass2d' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prfillmass2d'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Clear mass matrix

      massmat = 0

      ninner = kmeshm(1)    ! number of inner boundaries
      istar3 = kmeshm(2)-1  ! start of kmeshm part 3
      istar4 = kmeshm(3)-1  ! start of kmeshm part 4

!     --- mark all inner curves

      iwork = 0             ! all curves are defined as outer
      do icurnr = 1, ninner
         iwork(kmeshm(istar4+icurnr)) = 1   ! inner boundary
      end do  ! icurnr = 1, ninner

!     --- Set some parameters for the element subroutines

      ifirst = 0  ! first call
      jtrans = 0  ! no transformation
      iweigh = 1  ! weight must be computed
      irule = 1   ! Newton Cotes rule
      icoor = 1   ! Fill coordinates in x
      iphi = 0    ! Do not fill phi
      igausp = 0  ! do not fill Gauss points

!     --- For all elements along outer boundary do
!            Compute element mass matrix
!            Add to large matrix

      if ( linear ) then

!     --- Linear elements

         m = 2  ! number of integration points per element
         n = 2  ! number of nodes per element

         do icurnr = 1, ncurvs

            if ( iwork(icurnr)==0 ) then

!           --- iwork(icurnr=0), so outer boundary

               k1 = kmeshm(4+icurnr)    ! start of curve
               k2 = kmeshm(5+icurnr)-1  ! end of curve

               do j = k1, k2-1          ! loop over elements

                  index1(1) = kmeshm(istar3+j)
                  index1(2) = kmeshm(istar3+j+1)
                  call elp610 ( coor, x, w, dphidx, xgauss, phi, index1, &
                                qmat, tang )
                  ifirst = 1

                  massmat(index1(1:2)) = massmat(index1(1:2))+w(1:2)

               end do  ! j = k1, k2-1

            end if  ! ( iwork(icurnr)==0 )

         end do  ! icurnr = 1, ncurvs

      else

!     --- Quadratic elements

         m = 3  ! number of integration points per element
         n = 3  ! number of nodes per element

         do icurnr = 1, ncurvs

            if ( iwork(icurnr)==0 ) then

!           --- iwork(icurnr=0), so outer boundary

               k1 = kmeshm(4+icurnr)    ! start of curve
               k2 = kmeshm(5+icurnr)-1  ! end of curve

               do j = k1, k2-1, 2       ! loop over elements

                  index1(1) = kmeshm(istar3+j)
                  index1(2) = kmeshm(istar3+j+1)
                  index1(3) = kmeshm(istar3+j+2)
                  call elp616 ( coor, x, w, dphidx, xgauss, phi, index1, &
                                qmat, tang, phider )
                  ifirst = 1

                  massmat(index1(1:3)) = massmat(index1(1:3))+w(1:3)

               end do  ! j = k1, k2-1

            end if  ! ( iwork(icurnr)==0 )

         end do  ! icurnr = 1, ncurvs

      end if  ! ( linear )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prfillmass2d'

      end if  ! ( debug )
      call erclos ( 'prfillmass2d' )

      end
