      subroutine prmakedistex1 ( interf, ninterf, iwork, &
                                 grid, isigphi, origin, nx, dx )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 26-04-2013 Set nfaces/nedges after computing
!        version  1.2  date 11-12-2012 Long integers
!        version  1.1  date 18-09-2012 Jump in case of errors
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Make phi a distance function for all nodes next to the interface
!     and all of its neighbors
!     Points that satisfy | ||grad phi||-1 |<eps are excluded!
!     Only points with phi>0 or phi<0 are considered
! **********************************************************************
!
!                       KEYWORDS
!
!     level_set
!     distance
!     interface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(*), ninterf, &
              interf(ninterf), grid(*), isigphi, nx(ndim)
      double precision origin(ndim), dx(ndim)

!     dx             i    contains the difference between xmax, xmin etc
!     grid           i    The domain is subdivided in nx(1) x nx(2) cells
!                         of size dx(1) x dx(2)
!                         In grid the number of interface nodes per cell is
!                         stored as well as the node numbers in the folwoing way
!                         Pos. 1 starting address of node numbers
!                         (nx(1)*nx(2)+1)))
!                         The number of nodes in cell ij is defined by
!                         grid(start+1)-grid(start). where
!                         start = (j)-1)*nx(1)+i
!                         The corresponding node numbers are stored in grid
!                         from position start
!     interf         i    Contains node numbers of interface
!     isigphi        i    sign of phi (1 if phi>0), (-1 if phi<0)
!     iwork         i/o   integer work array to mark the nodes
!                         -10:  node with phi<0 outside the band
!                          -1:  node with phi<0 inside the band
!                           0:  node on interface
!                           1:  node with phi>0 inside the band
!                          10:  node with phi>0 outside the band
!     ninterf        i    number of nodes in array interf
!     nx             i    number of points in ndim directions of the grid
!     origin         i    Contains the coordinates of the origin of the region
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nset1, nset2, nfaceslc, iseq
      integer, allocatable :: set1(:), set2(:), markintf(:), setwk(:), &
                              markintfedge(:)

!     iseq           Dummy
!     markintf       Array of size nedges or nfaces to mark the edges or
!                    faces of the interface (1) else 0
!     markintfedge   Array of size nedges to mark the edges
!                    of the interface (1) else 0  (3d only)
!     nfaceslc       Either nfaces (3d) or nedges (2d)
!     nset1          number of nodes in set1
!     nset2          Number of nodes in set 2
!     set1           Contains all neighbors of the interface (one side only)
!     set2           Contains all neighbors of set1 not in the interface
!     setwk          Help array to store the values of iwork in set1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ININEIGHMARK   Find all marked neighbors of a set of nodes and store
!                    in array neighbors
!     ININEIGHMARK2  Reduce set of nodes to those nodes that satisfy
!                    grad(phi) not close to 1
!     PRCOMPDIST     Compute distance of set of nodes to interface
!                    Store in array phi with sign isigphi
!     PRININ         print 1d integer array
!     PRMARKINTERF   Mark all edges or faces in interface
!     PRMARKINTFEDGE Mark all edges in 3d interface
!     SEPFILLKMESHAA Fill first part of array KMESH part aa
!     SEPFILLKMESHAD Fill array kmesh part d
!     SEPFILLKMESHE  Fill array kmesh part e
!     SEPFILLNMESHA  Fill array nmesha
!     SEPFILLNMESHC  Fill array nmeshc
!     SEPFILLNMESHG  Fill array nmeshg
!     SEPFILLNMESHK  Fill array nmesh part k
!     SEPFILLNMESHL  Fill array nmesh part l
!     SEPFILLOBFACES Fill kmesh part ad subpart 9: iobstfaces
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prmakedistex1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prmakedistex1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Activate arrays

      if ( kelme==0 ) call sepfillkmeshe
      if ( kelmaa==0 ) call sepfillkmeshaa
      if ( nmesh(33)==0 ) call sepfillnmeshc
      if ( nmesh(42)==0 ) call sepfillnmeshl
      if ( kelmad==0 ) call sepfillkmeshad ( iseq )
      if ( kmeshad(9)==0 ) call sepfillobfaces

      if ( ndim==3 ) then
         if ( nmesh(31)==0 ) call sepfillnmesha
         if ( nmesh(37)==0 ) call sepfillnmeshg
         if ( nmesh(41)==0 ) call sepfillnmeshk
         nfaceslc = nfaces
      else
         nfaceslc = nedges
      end if  ! ( ndim==3 )

!     --- allocate arrays

      allocate ( set1(npoint), set2(npoint), markintf(nfaceslc), &
                 markintfedge(nedges), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, 2*npoint+nfaceslc+nedges, 'set1' )
      if ( ierror/=0 ) go to 1000

      markintf = 0  ! clear array
      markintfedge = 0  ! clear array

!     --- Set 1 = all direct neighbors in nodes of interface

      nset1 = npoint
      call inineighmark ( interf, ninterf, npoint, kmeshe, &
                          nset1, set1, iwork )
      if ( ierror/=0 ) go to 1000

      if ( debug ) call prinin ( set1, nset1, 'set1' )

      allocate ( setwk(nset1), stat = error )
      if ( error/=0 ) call eralloc ( error, nset1, 'setwk' )
      if ( ierror/=0 ) go to 1000

      setwk = iwork(set1(1:nset1))  ! store values of iwork in setwk
      iwork(set1(1:nset1)) = 2   ! Mark nodes in set1 in iwork with a 2

!     --- Set 2 = all direct neighbors of the previous set not already marked

      nset2 = npoint
      call inineighmark ( set1, nset1, npoint, kmeshe, nset2, &
                          set2, iwork )
      if ( ierror/=0 ) go to 1000

      if ( debug ) call prinin ( set2, nset2, 'set2' )

      iwork(set1(1:nset1)) = setwk

!     --- Reduce all sets to those nodes that satisfy grad(phi) not close to 1

      call inineighmark2 ( nset1, set1, iwork )
      call inineighmark2 ( nset2, set2, iwork )

      if ( debug ) then
         call prinin ( set1, nset1, 'set1/2' )
         call prinin ( set2, nset2, 'set2/2' )
      end if  ! ( debug )

      deallocate ( setwk, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'setwk' )
      if ( ierror/=0 ) go to 1000

!     --- Mark all edges or faces in interface

      call prmarkinterf ( markintf, obstfaces, nfaceslc )
      if ( ierror/=0 ) go to 1000

      if ( ndim==3 ) then

!     --- ndim = 3, mark edges in interface

         call prmarkintfedge ( obstfaces, markintfedge, nmeshb )

      end if  ! ( ndim==3 )
      if ( ierror/=0 ) go to 1000

!     --- Compute distance for all nodes in set 1

      if ( nset1>0 ) then

!     --- nset1>0, there are nodes to be adapted

         call prcompdist ( set1, nset1, coor, ndim, nx, &
                           origin, dx, grid, markintf, &
                           nmeshc, nmeshl, &
                           ninterf, obstfaces, &
                           phi1, isigphi, markintfedge, &
                           nmeshb, nmeshk, &
                           nmeshg, nfaceslc, nedges )

      end if  ! ( nset1>0 )
      if ( ierror/=0 ) go to 1000

!     --- Compute distance for all nodes in set 2

      if ( nset2>0 ) then

!     --- nset2>0, there are nodes to be adapted

         call prcompdist ( set2, nset2, coor, ndim, nx, &
                           origin, dx, grid, markintf, &
                           nmeshc, nmeshl, ninterf, obstfaces, &
                           phi1, isigphi, markintfedge, &
                           nmeshb, nmeshk, nmeshg, nfaceslc, nedges )

      end if  ! ( nset2>0 )

      deallocate ( set1, set2, markintf, markintfedge, &
                   stat = error )
      if ( error/=0 ) call erdealloc ( error, 'set1' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prmakedistex1'

      end if  ! ( debug )
      call erclos ( 'prmakedistex1' )

      end subroutine prmakedistex1
