      subroutine prmakedistexac ( interf, ninterf, iwork, eps, &
                                  nnodes, h )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 11-12-2012 Long integers
!        version  1.1  date 18-09-2012 Jump in case of errors
!        version  1.0  date 21-08-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Make phi a distance function for all nodes next to the interface
!     and all of its neighbors
!     Points that satisfy | ||grad phi||-1 |<eps are excluded
!     The exact distance is computed
! **********************************************************************
!
!                       KEYWORDS
!
!     level_set
!     distance
!     interface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nnodes, iwork(*), ninterf, interf(ninterf)
      double precision eps, h

!     eps            i    Local accuracy with respect to distance function
!     h              i    step size for the rectangular grid
!     interf         i    Contains node numbers of interface
!     iwork         i/o   integer work array to mark the nodes
!                         -10:  node with phi<0 outside the band
!                          -1:  node with phi<0 inside the band
!                           0:  node on interface
!                           1:  node with phi>0 inside the band
!                          10:  node with phi>0 outside the band
!     ninterf        i    number of nodes in array interf
!     nnodes         i    Number of nodes in nodes
!     nodes          i    Set of nodes in the given band around the interface
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nx(ndim), ntot
      double precision origin(ndim), dx(ndim), xmaxar(ndim)
      integer, allocatable :: grid(:)

!     dx             stepsize of grid in all directions
!     grid           The domain is subdivided in nx(1) x nx(2) cells
!                    of size dx(1) x dx(2)
!                    In grid the number of interface nodes per cell is
!                    stored as well as the node numbers in the folwoing way
!                    Pos. 1 starting address of node numbers
!                    (nx(1)*nx(2)+1)))
!                    The number of nodes in cell ij is defined by
!                    grid(start+1)-grid(start). where
!                    start = (j)-1)*nx(1)+i
!                    The corresponding node numbers are stored in grid
!                    from position start
!     ntot           number of cells in grid
!     nx             number of points in ndim directions of the grid
!     origin         Contains the coordinates of the origin of the region
!     xmaxar         Contains the coordinates xmax, ymax ..
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRFILLGRIDNOD  Fill information of a set of nodes in rectangular 2d or 3d
!                    grid
!     PRGRIDSIZE     Compute nx, dx and ntot from information in kmeshquant
!                    and h
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRMAKEDISTEX1  Make phi a distance function for all nodes next to the
!                    interface and all of its neighbors
!     PRMAKEDISTMK1  Mark all nodes in band around interface with
!                    | ||grad phi||-1 | < eps with -10 or 10
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Mark all points in array nodes with ||grad phi|| = 1
!     Set 1 = all direct neighbors in nodes of interface
!     Set 2 = all direct neighbors of the previous set not already marked
!     Compute distance for all nodes in set 1
!     Compute distance for all nodes in set 2
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prmakedistexac' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prmakedistexac'
         write(irefwr,2) 'h', h
         call prinin ( interf, ninterf, 'interf' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Activate arrays

      if ( ierror/=0 ) go to 1000

!     --- Make a rectangular grid over the whole region and store the number
!         of nodes of the interface per cell
!         First compute number of cells and define hx etc.

      call prgridsize ( kmeshquant, ndim, origin, h, dx, nx, &
                        npoint, ntot, xmaxar )
      if ( ierror/=0 ) go to 1000

      if ( debug ) then
         call prinin ( nx, ndim, 'nx' )
         call prinrl ( origin, ndim, 'origin' )
         call prinrl ( dx, ndim, 'dx' )
         call prinrl ( xmaxar, ndim, 'xmaxar' )
         write(irefwr,1) 'ntot, lengrid', ntot, ntot+1+ninterf
      end if  ! ( debug )

      allocate ( grid(ntot+1+ninterf), stat = error )
      if ( error/=0 ) call eralloc ( error, ntot+1+ninterf, 'grid' )
      if ( ierror/=0 ) go to 1000

!     --- Fill array grid

      call prfillgridnod ( interf, ninterf, &
                           grid, nx, origin, xmaxar )
      if ( ierror/=0 ) go to 1000

!     --- Mark all points in array nodes with ||grad phi|| = 1 (iwork=10)

      call prmakedistmk1 ( nnodes, nodes, grad, iwork, eps, ndim )
      if ( ierror/=0 ) go to 1000

      if ( debug ) call prinin ( iwork, npoint, 'iwork' )

!     --- Compute distance for all nodes with positive phi

      call prmakedistex1 ( interf, ninterf, iwork, grid, 1, &
                           origin, nx, dx )
      iwork(1:npoint) = -iwork(1:npoint)
      if ( ierror/=0 ) go to 1000

!     --- Compute distance for all nodes with negative phi

      call prmakedistex1 ( interf, ninterf, iwork, grid, -1, &
                           origin, nx, dx )

      deallocate ( grid, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'set1' )
      if ( ierror/=0 ) go to 1000

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prmakedistexac'

      end if  ! ( debug )
      call erclos ( 'prmakedistexac' )

      end subroutine prmakedistexac
