      subroutine prmakedistexnw ( iwork, eps, nnodes, h, iphi, kproblc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 06-05-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Make phi a distance function for all nodes next to the interface
!     and all of its neighbors
!     Points that satisfy | ||grad phi||-1 |<eps are excluded
!     The exact distance is computed
!     It concerns the case of a non-adapted mesh
!     This subroutine is the equivalent of prmakedistexac
! **********************************************************************
!
!                       KEYWORDS
!
!     level_set
!     distance
!     interface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulemain
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nnodes, iwork(*), kproblc(*)
      integer, target :: iphi
      double precision eps, h

!     eps            i    Local accuracy with respect to distance function
!     h              i    step size for the rectangular grid
!     iphi           i    Sepran solution array containing references to
!                         the distance function phi
!     iwork         i/o   integer work array to mark the nodes
!                         -10:  node with phi<0 outside the band
!                          -1:  node with phi<0 inside the band
!                           0:  node on interface
!                           1:  node with phi>0 inside the band
!                          10:  node with phi>0 outside the band
!     kproblc        i    Standard SEPRAN array, containing information of
!                         the problem
!     nnodes         i    Number of nodes in nodes
!     nodes          i    Set of nodes in the given band around the interface
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nx(ndim), ntot, numintersect, &
              iobst, nedginterf, numbndobst, ninterfnew, &
              lennmeshc, lennmeshl, numedgeintf, numnodeintf
      double precision origin(ndim), dx(ndim), xmaxar(ndim)
      integer, allocatable :: grid(:), nmeshcloc(:), nmeshlloc(:), &
                              nmeshctmp(:), nmeshltmp(:)
      double precision, allocatable :: coorinter(:,:)

!     coorinter      Contains coordinates of the interface nodes
!                    in the local numbering of the interface
!     dx             stepsize of grid in all directions
!     grid           The domain is subdivided in nx(1) x nx(2) cells
!                    of size dx(1) x dx(2)
!                    In grid the number of interface nodes per cell is
!                    stored as well as the node numbers in the folwoing way
!                    Pos. 1 starting address of node numbers
!                    (nx(1)*nx(2)+1)))
!                    The number of nodes in cell ij is defined by
!                    grid(start+1)-grid(start). where
!                    start = (j)-1)*nx(1)+i
!                    The corresponding node numbers are stored in grid
!                    from position start
!     iobst          Obstacle sequence number
!     lennmeshc      Length of array nmeshc
!     lennmeshl      Length of array nmeshl
!     nedginterf     Number of edges in interface
!     ninterfnew     number of nodes in interface extended by interface nodal
!                    points
!     nmeshcloc      Local array nmeshc for the interface only, &
!                    using a local numbering
!     nmeshctmp      Local array nmeshc for the interface only, &
!                    using a local numbering (First estimate)
!     nmeshlloc      Local array nmeshl for the interface only, &
!                    using a local numbering
!     nmeshltmp      Local array nmeshl for the interface only, &
!                    using a local numbering (First estimate)
!     ntot           number of cells in grid
!     numbndobst     Number of nodal points on the boundary of the obstacle
!     numedgeintf    Number of edges at the interface
!     numintersect   Number of intersections of interface with edges
!     numnodeintf    Number of nodes at the interface
!     nx             number of points in ndim directions of the grid
!     origin         Contains the coordinates of the origin of the region
!     xmaxar         Contains the coordinates xmax, ymax ..
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRFILLGRIDNINT Fill information of a set of nodes in rectangular
!                    2d or 3d grid
!     PRFILLINTERF   Fill coordinates of interface nodes into array coorinter
!     PRFILLINTFNMSC Fill nmeshc for interface (2d) (Node numbers of the edges)
!                    The local numbering of the interface is used
!     PRFILLINTFNMSL Fill nmeshl for interface (2d)
!                    Edge numbers of edges connected to a node
!                    The local numbering of the interface is used
!     PRGETNINTERF   Get number of nodes in interface for obstacle iobst
!     PRGRIDSIZE     Compute nx, dx and ntot from information in kmeshquant
!                    and h
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
!     PRMAKEDISTEX3  Make phi a distance function for all nodes next to the
!                    interface and all of its neighbors
!     PRMAKEDISTMK1  Mark all nodes in band around interface with
!                    | ||grad phi||-1 | < eps with -10 or 10
!     SEPFILLKMADL3  Fill array kmesh ad  part 3: icrossobst,rcrossobst
!                    in case of level set
!     SEPFILLKMESHAA Fill first part of array KMESH part aa
!     SEPFILLNMESHC  Fill array nmeshc
!     SEPFILLNMESHL  Fill array nmesh part l
!     SEPPRCROSSOBST Print the contents of the arrays icrossobst and rcrossobst
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Mark all points in array nodes with ||grad phi|| = 1
!     Set 1 = all direct neighbors in nodes of interface
!     Set 2 = all direct neighbors of the previous set not already marked
!     Compute distance for all nodes in set 1
!     Compute distance for all nodes in set 2
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prmakedistexnw' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prmakedistexnw'
         write(irefwr,2) 'h', h
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get parameters

      if ( kelmad==0 ) call sepfillkmeshaa
      if ( nmesh(33)==0 ) call sepfillnmeshc
      if ( nmesh(42)==0 ) call sepfillnmeshl

!     --- Compute the intersections of the zero value of the level set
!         with the mesh

      isolut => iphi
      call sepfillkmadl3
      if ( debug ) then
         write(irefwr,1) 'nlevelsets, kelmad', nlevelsets, kelmad
         call prinin ( kmeshad(1), 10, 'kmeshad' )
         call sepprcrossobst
      end if  ! ( debug )

!     --- Create a temporary array coorinter containing the
!         the coordinates of the interface

      iobst = 1
      call prgetninterf ( iobst, numintersect, nedginterf, numbndobst )
      if ( debug ) &
         write(irefwr,1) 'numintersect, numbndobst', &
                          numintersect, numbndobst

      ninterfnew = numintersect+numbndobst

      allocate ( coorinter(ndim,ninterfnew), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, ninterfnew*ndim, 'coorinter' )
      if ( ierror/=0 ) go to 1000

      call prfillinterf ( iobst, numintersect, coorinter, ninterfnew )

      if ( debug ) &
         call prinrl1 ( coorinter, ninterfnew, ndim, 'coorinter' )

!     --- Activate arrays

      if ( ierror/=0 ) go to 1000

!     --- Make a rectangular grid over the whole region and store the number
!         of nodes of the interface per cell
!         First compute number of cells and define hx etc.

      call prgridsize ( kmeshquant, ndim, origin, h, dx, nx, &
                        npoint, ntot, xmaxar )
      if ( ierror/=0 ) go to 1000

      if ( debug ) then
         call prinin ( nx, ndim, 'nx' )
         call prinrl ( origin, ndim, 'origin' )
         call prinrl ( dx, ndim, 'dx' )
         call prinrl ( xmaxar, ndim, 'xmaxar' )
         write(irefwr,1) 'ntot, lengrid', ntot, ntot+1+numintersect
      end if  ! ( debug )

      allocate ( grid(ntot+1+ninterfnew), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, ntot+1+numintersect, 'grid' )
      if ( ierror/=0 ) go to 1000

!     --- Fill array grid with information of nodes of interface
!         The position of each node in the grid is stored

      call prfillgridnint ( ninterfnew, grid, nx, origin, &
                            xmaxar, coorinter )
      if ( ierror/=0 ) go to 1000

!     --- Mark all points in array nodes with ||grad phi|| = 1 (iwork=10)

      call prmakedistmk1 ( nnodes, nodes, grad, iwork, eps, ndim )
      if ( ierror/=0 ) go to 1000

      if ( debug ) call prinin ( iwork, npoint, 'iwork' )

!     --- Create local arrays nmeshcloc and nmeshlloc which
!         correspond to the interface only

      lennmeshl = size ( nmeshl )+2*numbndobst
      lennmeshc = lennmeshl ! nmeshcloc overestimated

      allocate ( nmeshctmp(lennmeshc), stat = error )
      if ( error/=0 ) call eralloc ( error, lennmeshc, 'nmeshctmp' )
      if ( ierror/=0 ) go to 1000

!     --- Create local arrays nmeshcloc, nmeshlloc with respect to interface
!         First create temporary array nmeshctmp

      call prfillintfnmsc ( nmeshctmp, lennmeshc, iobst, &
                            numedgeintf, numnodeintf )
      if ( ierror/=0 ) go to 1000

!     --- Create nmeshcloc and copy nmeshctmp into nmeshcloc

      allocate ( nmeshcloc(lennmeshc), nmeshltmp(lennmeshl), &
                 stat = error )
      if ( error/=0 ) &
         call eralloc ( error, lennmeshc+lennmeshl, 'nmeshcloc' )

      nmeshcloc(1:lennmeshc) = nmeshctmp(1:lennmeshc)  ! copy

!     --- Remove nmeshctmp

      deallocate ( nmeshctmp, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'nmeshctmp' )
      if ( ierror/=0 ) go to 1000

      call prfillintfnmsl ( nmeshltmp, lennmeshl, nmeshcloc, &
                            numedgeintf, numnodeintf )
      if ( ierror/=0 ) go to 1000

!     --- Create nmeshlloc and copy nmeshltmp into nmeshlloc

      allocate ( nmeshlloc(lennmeshl), stat = error )
      if ( error/=0 ) call eralloc ( error, lennmeshl, 'nmeshloc' )

      nmeshlloc(1:lennmeshl) = nmeshltmp(1:lennmeshl)  ! copy

!     --- Remove nmeshltmp

      deallocate ( nmeshltmp, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'nmeshltmp' )
      if ( ierror/=0 ) go to 1000

!     --- Compute distance for all nodes with positive phi

      call prmakedistex3 ( numintersect, iwork, grid, 1, &
                           origin, nx, dx, coorinter, nedginterf, &
                           nmeshcloc, nmeshlloc, numnodeintf )
      iwork(1:npoint) = -iwork(1:npoint)
      if ( ierror/=0 ) go to 1000

!     --- Compute distance for all nodes with negative phi

      call prmakedistex3 ( numintersect, iwork, grid, -1, &
                           origin, nx, dx, coorinter, nedginterf, &
                           nmeshcloc, nmeshlloc, numnodeintf )

      deallocate ( grid, coorinter, nmeshcloc, nmeshlloc, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'set1' )
      if ( ierror/=0 ) go to 1000

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prmakedistexnw'

      end if  ! ( debug )
      call erclos ( 'prmakedistexnw' )

      end subroutine prmakedistexnw
