      subroutine prparaddvec ( usol, indprf, kprobf, nunkp, nusol, &
                               npoint, blockinfo, transportinfo )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 24-03-2013 Use inibsendmpi
!        version  1.1  date 21-10-2005 Fixed array dimensions and sync
!        version  1.0  date 11-04-2002
!
!   copyright (c) 2002-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Add common parts of a SEPRAN solution vector or vector of special
!     structure in case of parallel program
!     Results of each block are added to all neigbour blocks
!     In this way the global vector is identical on all processors
! **********************************************************************
!
!                       KEYWORDS
!
!     parallel
!     boundary
!     common_point
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecpack
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer indprf, kprobf(*), nunkp, nusol, npoint, blockinfo(*), &
              transportinfo(*)
      double precision usol(*)

!     blockinfo      i    Array with global block information for surface
!                         number isurnr
!                         Contents:
!                         Pos. 1  m ( number of neihgbour blocks)
!                         Pos. 2*i   block number of i-th neighbour
!                         Pos. 2*i+1 starting address of i-th neighbour in
!                                    array transinfo
!                         Length: 2*m+1
!     indprf         i    indication if kprob part f has been filled (>0)
!                         or not (0)
!     kprobf         i    KPROB part f: contains the number of unknowns per
!                         point accumulated
!     npoint         i    Number of nodal points
!     nunkp          i    Maximum number of unknowns in a point
!     nusol          i    Total number of unknowns
!     transportinfo  i    Array containing information of the transport of
!                         unknowns between neighbouring blocks
!                         The starting address of each neighbour block is
!                         stored in array blockinfo
!                         For each neighbour block the following information
!                         is stored
!                         Pos. 1 number of nodes n on common side of both blocks
!                         Pos. 2 ... n+1    local node numbers in block i
!                         Pos. n+2 ... 2n+1 local node numbers in neighbour
!                                           block j
!     usol          i/o   Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer isend(npoint+1), numneigh, j, iblock, seqblocks(iacnodes), &
              sendstart, nneigh, k, icount, istart, jstart, i, nodp, &
              inunkp, icontext, lastsent, nneigh1
      double precision usend(nusol+npoint+2)
      integer, dimension(:), pointer :: send_buffer
      integer, save :: send_buffer_size

!     debug          Print debug statements
!     i              Counting variable
!     iblock         Block sequence number
!     icontext       Tag for sending and receiving, is used as check
!     icount         Counter
!     inunkp         Number of degrees of freedom in isol1
!     isend          Work array to send the number of degrees of freedom
!                    for each boundary point
!                    Maximum length: largest neighbour part in transinfo
!                    (< npoint)
!     istart         First position used at a part
!     j              Counting variable
!     jstart         First position used at a part
!     k              Counting variable
!     lastsent       counter
!     nneigh         Number of neighbours
!     nneigh1        Size of the received array
!     nodp           Nodal point sequence number
!     numneigh       number of neighbour blocks of a block
!     send_buffer    send buffer
!     send_buffer_size  Length of the send buffer
!     sendstart      starting address in array sendinfo
!     seqblocks      Work array of length nblocks (=nsurfs) which
!                    is used to find the sequence of block numbers
!                    in array blockinfo
!                    seqblocks(iblock) = 0: block iblock not present
!                    seqblocks(iblock) = i: block iblock is the i-th block
!                    in blockinfo
!     usend          Help array to be used to send reals to other processor
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIBARRIERMPI  Synchronize parallel sending and receiving (mpi)
!     INIBSENDMPI    Send information to other node (mpi buffered)
!     INIBUFATTMPI   Attach mpi buffer if it exists
!     INIBUFDETMPI   Detach mpi buffer if it exists
!     INIBUFSIZEMPI  Extend send_buffer_size with mpi overhead
!     INIRECEIVEMPI  Receive information from other node (mpi)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     numneigh := number of neighbour blocks
!     for all neighbour blocks iblock do
!        Store starting addresses of blocks in array seqblocks
!     For all neighbour blocks iblock do
!        for all common nodes nodp do
!           store number of unknowns in array isend
!           store unknowns in array usend
!        Send arrays isend and usend to other blocks
!     For all neighbour blocks iblock do
!        Receive arrays isend and usend from block iblock
!        for all common nodes nodp do
!           add unknowns to solution array
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prparaddvec' )

      debug = .false.

!     --- numneigh := number of neighbour blocks

      numneigh = blockinfo(1)

!     --- for all neighbour blocks iblock do
!            Store starting addresses of blocks in array seqblocks

      seqblocks = 0

      do j = 1, numneigh
         iblock =  blockinfo(2*j)
         seqblocks(iblock) = j
      end do

!     --- Synchronize communication by calling mpi_barrier
!         This avoids that something is written to other nodes, &
!         before reading by those other nodes has been done

      call inibarriermpi

      send_buffer_size = 0
      do j = 1, numneigh
         sendstart = blockinfo(2*j+1)
         nneigh = transportinfo(sendstart)
         send_buffer_size = send_buffer_size + nneigh*nunkp*8
      end do

!     --- Compute actual buffer size using mpi information
!         allocate array and attach buffer

      call inibufsizempi ( send_buffer_size, numneigh )

      allocate(send_buffer(send_buffer_size))

      call inibufattmpi ( send_buffer, send_buffer_size )

!     --- For all neighbour blocks iblock do

      do j = 1, numneigh
         iblock = blockinfo(2*j)
         sendstart = blockinfo(2*j+1)
         nneigh = transportinfo(sendstart)
         usend(1) = nneigh

!        --- for all common nodes nodp do
!               store number of unknowns in array isend
!               store unknowns in array usend

         if ( indprf==0 ) then

!        --- indprf = 0, the number of unknowns per point is constant

            do k = 1, nneigh
               nodp = transportinfo(sendstart+k)
               do i = 1, nunkp
                  if ( debug ) then
                     write(irefwr,*) nodp,' sending: glob',2+i+(k-1)* &
                           nunkp,' loc', i+(nodp-1)*nunkp, &
                           usol(i+(nodp-1)*nunkp)
                     flush(irefwr)
                  end if
                  lastsent = 2+i+(k-1)*nunkp
                  usend(lastsent) = usol(i+(nodp-1)*nunkp)
               end do
            end do
            usend(2) = -nunkp
            icount = 0

         else

!        --- indprf>0, the number of unknowns per point is not constant

            icount = 0
            do k = 1, nneigh
               nodp = transportinfo(sendstart+k)
               istart = kprobf(nodp)
               inunkp = kprobf(nodp+1)-istart
               isend(k) = icount
               do i = 1, inunkp
                  lastsent = 2+i+icount
                  usend(lastsent) = usol(i+istart)
               end do
               icount = icount + inunkp
            end do
            usend(2) = icount

!           --- Place isend at the end of usend

            istart = 2+icount
            do k = 1, nneigh
               lastsent = istart+k
               usend(lastsent) = isend(k)
            end do

         end if

!        --- Send arrays isend and usend to other blocks

         icontext = 3
         if ( debug ) then
            write(irefwr,*)'sending',lastsent,'numbers.',usend(1),j
            flush(irefwr)
         end if
         call inibsendmpi ( usend, lastsent, SPDOUBLE, iblock, &
                            icontext )

      end do

!     --- For all neighbour blocks iblock do

      do j = 1, numneigh

!     --- Receive arrays isend and usend from block iblock
!         for all common nodes nodp do
!            add unknowns to solution array

         icontext = 3
         nneigh1 = nusol+npoint+2
         call inireceivempi ( usend, nneigh1, SPDOUBLE, iblock, &
                              icontext )

         sendstart = blockinfo(2*seqblocks(iblock)+1)
         nneigh = nint(usend(1))
         icount = nint(usend(2))

!        --- Copy the end of usend to isend

         if ( icount>0 ) then

!        --- icount>0: isend has been stored in usend

            istart = 2+icount
            do k = 1, nneigh
               isend(k) = nint(usend(istart+k))
            end do
            isend(nneigh+1) = icount

         else

!        --- icount<0: isend has not been stored in usend

            inunkp = -icount
            do k = 0, nneigh
               isend(k+1) = k*inunkp
            end do

         end if

         if ( indprf==0 ) then

!        --- indprf = 0, the number of unknowns per point is constant

            do k = 1, nneigh
               nodp = transportinfo(sendstart+k)
               jstart = isend(k)
               inunkp = min(isend(k+1)-jstart,nunkp)
               istart = (nodp-1)*nunkp
               do i = 1, inunkp
                  if ( debug ) then
                     write(irefwr,*) k,nodp, iblock, sendstart, &
                        ' adding: glob', 2+i+jstart, ' loc', &
                         i+istart, usol(i+istart),usend(2+i+jstart), &
                        usol(i+istart)+usend(2+i+jstart)
                  end if
                  usol(i+istart) = usol(i+istart)+usend(2+i+jstart)
               end do
            end do

         else

!        --- indprf>0, the number of unknowns per point is not constant

            do k = 1, nneigh
               nodp = transportinfo(sendstart+k)
               jstart = isend(k)
               istart = kprobf(nodp)
               inunkp = kprobf(nodp+1)-istart
               inunkp = min(isend(k+1)-jstart,inunkp)
               do i = 1, inunkp
                  usol(i+istart) = usol(i+istart)+usend(2+i+jstart)
               end do
            end do

         end if

      end do

!     --- Synchronize communication by calling mpi_barrier
!         This avoids that something is written to other nodes, &
!         before reading by those other nodes has been done

      call inibarriermpi

      if (associated(send_buffer)) then

!     --- deattach send_buffer and deallocate

         call inibufdetmpi ( send_buffer, send_buffer_size )
         deallocate(send_buffer)

      end if  ! (associated(send_buffer))

      call erclos ( 'prparaddvec' )
      end
