      subroutine prpartpnlncon2 ( finished, inpsep, rinsep, numcomps )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-04-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     One time step for the integration of a particle is carried out
!     It concerns a system of concentration equations with a nonlinear
!     set of mixed boundary conditions
!     The interface concentration csol is computed in a loop until
!     convergence is reached
!     more dimensional case
! **********************************************************************
!
!                       KEYWORDS
!
!     particle
!     level_set
!     time_dependent
!     non_linear
!     2d
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons
      use sepmodulevecs
      use sepmodulekprob
      use sepmodulemain
      use sepmoduletime
      use sepmodulekmesh
      use sepmodulepost
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numcomps, inpsep(*)
      double precision rinsep(*)
      logical finished

!     finished      i/o   Indicates if the loop has been finished (true) or not
!                         (false)
!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine readscpnlnconc for a description
!     numcomps       i    Number of components
!     rinsep         i    Array containing the real input for this subroutine
!                         See subroutine readscpnlnconc for a description
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer maxiter, iloop, inpseploc(7), ilev, i, j, itermethod, &
              iseqconc, iseqvn, dimsub, substitutional(numcomps), &
              istore, iseqg, numvecsav, iseqiter, minloop, &
              nnodes, nparts, istart, mmipointobst, iprint, &
              itypvc, ivec, iseqcs, idummy
      double precision, allocatable :: g(:,:), cboun(:,:), vn(:), &
                                       vstore(:,:), csstore(:,:)
      double precision diff, bigM, eps, xp(numcomps), k(numcomps), &
                       cpart(numcomps), cs(numcomps), diffprev, &
                       h, xs(numcomps), factor, kinf, dummy(1)
      logical zeroval

!     bigm           Parameter M in the method
!     cboun          Values of the concentrations at the boundary
!     cpart          Value of the concentrations in the particle
!     cs             mole fractions along the interface
!     csstore        Help array to store all cs values in the interface
!     diff           Help parameter to compute the difference of two
!                    parameters
!     diffprev       Previous value of diff (prior iteration)
!     dimsub         Dimension of the set of substitutional elements
!     dummy          Dummy parameter
!     eps            Accuracy with respect to the iteration process
!                    Computation is stopped once |factor-1|<eps
!     factor         Multiplication factor (must go to 1)
!     g              right-hand side for the boundary condition
!     h              Help variable to store a constpstant temporarily
!     i              Counting variable
!     idummy         Dummy parameter
!     ilev           Sequence number of level being considered
!     iloop          Loop parameter, counts number of iterations
!     inpseploc      Local array inpsep for various subroutines
!     iprint         Defines the amount of output to be printed
!     iseqconc       Sequence number of concentration vectors
!     iseqcs         Sequence number of vector cs
!     iseqg          Sequence number of right-hand side g
!     iseqiter       Sequence number of numiter in which the number of
!                    iterations is stored
!     iseqvn         Sequence number of boundary velocity
!     istart         dummy
!     istore         Local counter for Aitken's method
!     itermethod     Type of iteration method
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     ivec           Sequence number of vector of special structure
!     j              Counting variable
!     k              parameter k in the method
!     kinf           parameter kinf in the method
!     maxiter        maximum number of iterations
!     minloop        Minimum number of iterations in loop
!     mmipointobst   memory management sequence number of array ipointobst
!     nnodes         Number of nodes in nodes
!     nparts         dummy
!     numvecsav      Help variable to store the value of numvec
!     substitutional Set of substitutional elements
!     vn             Normal velocity
!     vstore         Help array to store three consecutive values of vn
!     xp             mole fractions within the precipitate
!     xs             see method
!     zeroval        if true at least one of the xs entries is zero
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     INIZEROLEV     Compute number of nodes in zero level set and starting
!                    position in ipointobst
!     PRAITKEN       Perform Aitken acceleration to an iteration process
!     PRGETVALARR    Copy the contents of a sepran solution array into a user
!                    array
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRINRL2        Print a real two-dimensional array
!     PRPARTPNLNCONP Extract some information from inpsep with respect
!                    to prpartpnlnconc
!     PRPUTVALARR    Copy the contents of a user array into a sepran
!                    solution array
!     PUTSCAL        Put the value of the scalar iseq in the block constpstants
!     SEPCOPYVC      Copy one SEPRAN vector into another one
!     SEPCREASOLBF1  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
!     SEPFILLNODES   Fill array nodes with nodal points as defined in array
!                    inpsep
!     SEPFLPOINTOBST Fill array kmesh ad  part 1: ipointobst
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
!     SEPTIMEFREE    Actual body of subroutine TIMEFREE
! **********************************************************************
!
!                       I/O
!
!   Depending on the values in IREAD input may be read from the standard
!   input file by subroutine READFRST
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     In each time step we have to solve a set of convection-diffusion
!     equations for each particle component
!     At the interface the following boundary conditions must be satisfied:
!     D_e dc_e/dn + vn c_e = g_e, &
!     where D_e is the diffusion coefficient in the convection-diffusion e
!           c_e is the concentration of component e
!           vn is the unknown interface velocity and g_e the right-hand side
!     g_e is equal to c_e^p * vn, with c_e^p the particle concentration
!     To compute vn the following relations must be satisfied:
!
!     x_e^p = c_e^p/sum_e(c_e^p) is the mole fraction within the precipitate
!     c_e^s = c_e^p/K_e vn + c_e the mole fraction along the interface
!         with c_e the concentration at the interface and K_e the given
!         interface reaction speed
!     M is the maximum total concentration of substitutional elements
!     M = cinf(1)/xinf(1) = sum(cinf(S))-sum(cinf)
!        with cinf and xinf given and S the set of substitutional elements
!     I is the set of interstitial elements, the complement of S
!     The equations to be satisfied are
!     prod((x_e^s)^x_e^p) = kinf exp ( zeta kappa ),      (1)
!     with
!     x_e^s = c_e^s / (sum(c_I^s) + M)
!     zeta = gamma V_m / (R_g T ) is given
!     kappa is the curvature at the interface
!     convergence is reached if (1) is satisfied accurately enough
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prpartpnlncon2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prpartpnlncon2'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d14.6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Compute some parameters

      call prpartpnlnconp ( inpsep, numcomps, xp, bigM, maxiter, &
                            eps, rinsep, iseqconc, iseqvn, cpart, k, &
                            substitutional, dimsub, iseqg, kinf, &
                            itermethod, iseqiter, iprint, iseqcs )

!     --- Create arrays g, without filling
!         First get information form the concentration

      itypvc = 110
      ivec = 0
      iprob = ks(isolut2d(iseqconc))%iprob

      do i= 1, numcomps
         call sepcreasolbf1 ( isolut2d(iseqg+i-1), itypvc, ivec, 'g', 0 )
      end do  ! i= 1, numcomps

      debug = iprint>=3

      if ( debug ) &
         write(irefwr,1) 'iprint, itermethod', iprint, itermethod

!     --- Find number of nodes along interface
!         activate or create array ipointobst

      call sepflpointobst

      ilev = 1
      call inizerolev ( ipointobst, istart, nnodes, ilev )

      if ( debug ) write(irefwr,1) 'nnodes', nnodes

      allocate ( nodes(nnodes), cboun(nnodes,numcomps), vn(nnodes), &
                 vstore(nnodes,3), g(nnodes,numcomps), &
                 csstore(nnodes,numcomps), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, nnodes*(2+numcomps), 'nodes' )
      if ( ierror/=0 ) go to 1000

!     --- Find nodes at boundary

      inpseploc(1) = 1
      call sepfillnodes ( 20, 0, inpseploc, &
                          nodes, nnodes, idummy, nparts )
      if ( ierror/=0 ) go to 1000

!     --- Store value of vn at boundary in array vn

      call prgetvalarr ( isolut2d(iseqvn), nnodes, 1, vn )
      if ( ierror/=0 ) go to 1000

!     --- Copy old concentration into temporary array concold

      numvecsav = numvec
      numvec = numvec+numcomps
      call inichkarlen ( numvec, maxvc1, 'isolut2d' )
      if ( ierror/=0 ) go to 1000

      if ( debug ) write(irefwr,1) 'numvecsav', numvecsav

      do i = 1, numcomps
         call sepcopyvc ( isolut2d(iseqconc+i-1), &
                         isolut2d(numvecsav+i) )
      end do  ! i = 1, numcomps
      if ( ierror/=0 ) go to 1000

!     --- loop until convergence

      diff = 1d3
      diffprev = diff
      iloop = 0
      minloop = 2

      if ( iprint>=2 ) write(irefwr,100)
100   format ( 'In iteration process for particle non-linear', &
               'concentration'// &
               'iter   |factor-1|  quotient' )

      do while ( diff>eps .and. iloop<maxiter .or. iloop<=minloop )

!     --- Find concentration at boundary

         do i = 1, numcomps
            call prgetvalarr ( isolut2d(iseqconc+i-1), &
                               nnodes, 1, cboun(1,i) )
         end do  ! i = 1, numcomps

         istore = mod(iloop,3)+1

         if ( debug ) then
            write(irefwr,1) 'iloop, istore', iloop, istore
            call prinrl ( vn, nnodes, 'vn' )
            call prinrl2 ( cboun, nnodes, numcomps, 'cboun' )
         end if  ! ( debug )

!        --- Loop over nodes

         diff = 0d0
         do j = 1, nnodes

!        --- First compute the mole fractions c_e^s defined by
!            c_e^s = c_e^p/K_e vn + ce, &
!            with ce the concentration along the boundary

            cs = cpart/k * vn(j) + cboun(j,:)
            csstore(j,:) = cs(:)
            if ( debug ) write(irefwr,2) 'cs', cs

!           --- next x_e^s defined by
!               x_e^s =  c_e^s / (sumI(c^s) + M)

            h = bigM+sum(cs(1:numcomps))- &
                sum(cs(substitutional(1:dimsub)))
            xs = cs/h
            if ( debug ) write(irefwr,2) 'xs', xs

!           --- Compute product (x_e^s)^x_e^p

            zeroval = .false.
            i = 0
            do while ( .not. zeroval .and. i<numcomps )

               i = i+1
               if ( abs(xs(i))<=0d0 ) then
                  zeroval = .true.
               else
                  xs(i) = xs(i)**xp(i)
               end if  ! ( abs(xs(i))<=0d0 )

            end do  ! while ( .not. zeroval )

            if ( zeroval .or. vn(j)==0d0 ) then
               factor = 0d0
               vn(j) = 1d-3
            else
               factor = kinf/product(xs(1:numcomps))
               vn(j) = vn(j)*factor
            end if  ! ( zeroval )
            diff = max(diff,abs(factor-1d0))

            if ( debug ) &
               write(irefwr,2) 'vn, factor, diff', vn(j), factor, diff

         end do  ! j = 1, nnodes

         if ( itermethod>1 ) then

!        --- itermethod>1, use Aitken's extrapolation

            vstore(:,istore) = vn

!           --- Apply Aitken's method after each 3 steps
!               However, skip the first three steps to avoid influence
!               of the wrong initial guess

            if ( iloop>3 .and. istore==3 ) then

!           --- 3 values in a row

               call praitken ( nnodes, vn, vstore, itermethod-1 )

            end if  ! ( iloop==3 )

         end if  ! ( itermethod>1 )

!        --- Store value of vn at boundary in array sepran array vn

         call prputvalarr ( isolut2d(iseqvn), &
                            nodes, nnodes, 1, vn, 'vn' )
         if ( ierror/=0 ) go to 1000

!        --- Put right-hand side of boundary condition in sepran array g

         do i = 1, numcomps
            g(:,i) = vn*cpart(i)
         end do  ! i = 1, numcomps

         do i = 1, numcomps
            call prputvalarr ( isolut2d(iseqg+i-1), nodes, &
                               nnodes, 1, g(1,i), 'g' )
         end do  ! i = 1, numcomps

!        --- Copy old concentration back into concentration

         do i = 1, numcomps
            call sepcopyvc ( isolut2d(numvecsav+i), &
                            isolut2d(iseqconc+i-1) )
         end do  ! i = 1, numcomps

         inpseploc(1) = inpsep(5)         ! iseqtime
         inpseploc(2) = iseqconc+5000

         if ( debug ) call prinin ( inpseploc, 2, 'inpseploc' )

         call septimefree ( finished, inpseploc, dummy )

         maxcalltim = 2  ! set maximum number of calls per timestep>1
                         ! in this way reading of coefficients is triggered
                         ! in the next call

         iloop = iloop+1

         if ( iloop<3 ) diff = 1d0
         if ( debug ) call prinrl2 ( g, nnodes, numcomps, 'g' )

         if ( iprint>=2 ) write(irefwr,110) iloop, diff, diff/diffprev
110      format ( i5, 2d12.3 )

         diffprev = diff

      end do  ! while ( diff>eps and iloop<maxiter )

      numvec = numvecsav

      h = iloop
      if ( iseqiter>0 ) call putscal ( 'num_iter', iseqiter, h )

      if ( iprint>=1 ) write(irefwr,120) iloop, diff
120   format ( 'number of executed iterations = ', i5, &
               '  |factor-1| = ', d12.3 )

      if ( iseqcs>0 ) then

!     --- Store value of cs at boundary in array sepran array vn

         do i = 1, numcomps
            call prputvalarr ( isolut2d(iseqcs+i-1), nodes, &
                               nnodes, 1, csstore(1,i), 'cs' )
         end do  ! i = 1, numcomps
         if ( ierror/=0 ) go to 1000

      end if  ! ( iseqcs>0 )

      deallocate ( nodes, vn, cboun, vstore, g, csstore, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'nodes' )

!     --- Remove arrays g

      do i = 1, numcomps
         call seprmsolinfo ( isolut2d(iseqg+i-1) )
      end do  ! i = 1, numcomps

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prpartpnlncon2'

      end if  ! ( debug )
      call erclos ( 'prpartpnlncon2' )

      end subroutine prpartpnlncon2
