      subroutine sepnestdecllt ( A )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-06-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute L D L^T decomposition for system of real linear equations
!     (nested dissection)
!     Symmetric matrix
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     real
!     direct_method
!     nested_dissection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmoduleintmat
      use sepmodulemat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix) :: A

!     A             i/o   Matrix A
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j, k, kk, ip, jp, ii
      integer, pointer :: point(:), point1(:), row(:), col(:), &
                          rowhor(:), colhor(:)
      double precision, pointer :: mat(:), c(:)
      double precision :: Lik, rowsum

!     Lik            Element L_ik of decomposed matrix
!     c              Work array to store one column of decomposition matrix
!     col            Column numbers of matrix
!     colhor         contains column numbers per horizontal row
!     i              Row index
!     ii             position of element D(i)
!     ip             Loop variable
!     j              Counting variable
!     jp             Loop variable
!     k              Row index
!     kk             position of element D(k)
!     mat            elements of matrix
!     point          pointer array referring to next element in a row
!     point1         pointer array referring to next element to be used
!                    in last summation over columns
!     row            row numbers of matrix
!     rowhor         contains starting addresses (-1) of horizontal rows in
!                    colhor
!     rowsum         Row norm of matrix
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision :: dasum

!     DASUM          Blas subroutine computing the sum of absolute values of
!                    vector
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     SEPCOMMATREV   Fill arrays irowsnew and icolsnew with upper triangular
!                    part if icolsii and irowsii contain the lower triangular
!                    part and vice versa
!     SEPPRINCOMPSTR print part of array intmat corresponding to internal
!                    unknowns. Compact storage
!     SEPPRINMAT6    print compact matrix
!     SEPPRINMAT6P   print compact preconditioning matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     104   matrix has small pivots
!    1535   matrix is singular
!    1820   The matrix found in the solver is completely zero
!    3229   Internal error in SEPRAN: col(point1(i))<=k.
! **********************************************************************
!
!                       PSEUDO CODE
!
!      The L D L   decomposition of the matrix A is based on the following
!      formulas:
!              n               T
!      a   =  sum   l    d    l
!       ij    k=1    ik   kk   kj
!      this implies:
!
!     For i = 1 to n
!        d_ii = a_ii - sum_k=1^k=i-1 (l_ik d_kk l_ik)
!        For j = i+1 to n do
!           l_ij = a_ij - sum_k=1^k=i-1 (l_ik d_kk l_jk) / d_jj
!! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepnestdecllt' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepnestdecllt'
         call sepprinmat6 ( A )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      row => A%Mrows
      col => A%Mcols
      mat => A%MD

      allocate ( point(A%neq+1), point1(A%neq+1), c(A%neq), &
                 rowhor(A%neq+1), colhor(size(col)) )
      point = row+2
      point1 = point
      c = 0d0

!     --- compute row norm of part of the matrix in order to be able to check
!         the stopping criterion for the pivot process

      rowsum = 0d0
      do  k = 1, A%neq
         rowsum = max ( rowsum,  &
                  2d0*dasum (A%rows(k+1)-A%rows(k)-1, A%S(A%rows(k)+1), 1) &
                        +abs(A%S(A%rows(k)+1)) )
      end do

      if ( rowsum<=0d0 ) then
         call errsub ( 1820, 0, 0, 0 )
         go to 1000
      end if
      rowsum = rowsum*epsmac

!     --- Store row structure of matrix in row/colhor

      call sepcommatrev ( rowhor, colhor, row, col, nrusol )
      if ( debug ) call sepprincompstr ( A%neq, rowhor, colhor )

!     --- Loop over rows

      do k = 1, A%neq

         kk = row(k)+1      ! position of L(k,k)

!        --- Copy column k of mat in c

         do i = row(k)+2, row(k+1)
            c(col(i)) = mat(i)
         end do  ! i = row(k)+1, row(k+1)

!        --- For all non-zero rows k do

         do ip = rowhor(k)+1, rowhor(k+1)-1

            i = colhor(ip)  ! row number
            ii = row(i)+1

            if ( abs(mat(ii))<=0d0 ) then

!           --- matrix is singular, error

               call errint ( i, 1 )
               call errsub ( 1535, 1, 0, 0 )
               go to 1000

            else if ( abs(mat(ii))<rowsum ) then

!           --- matrix is probably singular, warning

               call errint ( i, 1 )
               call erreal ( mat(ii), 1 )
               call erreal ( rowsum, 2 )
               call errwar ( 104, 1, 2, 0 )

            end if

            jp = point(i)
            Lik = mat(jp)  ! element L(i,k)
            mat(jp) = mat(jp)/mat(ii)
            mat(kk) = mat(kk)-Lik*mat(jp)
            Lik = mat(jp)  ! element L(i,k)

!           --- For each i < k: all columns j>k for which L(i,j)#0

            if ( col(point1(i))<=k ) point1(i) = point1(i)+1
            if ( point1(i)<=row(i+1) ) then
               if ( col(point1(i))<=k ) then
                  call errsub ( 3229, 0, 0, 0 )
                  go to 1000
               end if  ! ( col(point1(i))<=k )
            end if  ! ( point1(i)<=row(i+1) )

            do jp = point1(i), row(i+1)
               j = col(jp)
               c(j) = c(j)-Lik*mat(jp)   ! c_j = c_j - L(k,i)*L(k,j)
            end do  ! i = point(j), row(j+1)
            point(i) = point(i)+1
            point1(i) = point1(i)+1

         end do  ! j = 1, k

!        --- Store c in mat and reset c

         do ip = row(k)+2, row(k+1)
            i = col(ip)
            mat(ip) = c(i)

            c(i) = 0d0
         end do  ! i = row(k)+2, row(k+1)
         c(k) = 0d0

      end do  ! j = 1, A%neq

      deallocate ( point, point1, c, rowhor, colhor )

1000  if ( debug ) then

!     --- Debug information

         call sepprinmat6p ( A )
         write(irefwr,*) 'End sepnestdecllt'

      end if  ! ( debug )
      call erclos ( 'sepnestdecllt' )

      end subroutine sepnestdecllt
