      subroutine seppjdset
! ======================================================================
!
!        programmer     Gjalt Wijma
!        version  1.0  date 14-06-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     The original input, subdivided in element groups, is regrouped
!     in sets. All element groups with the same integer property which
!     are coupled (have at least one node in common), are combined into
!     a set. Furthermore the connections between sets is saved.
!     For each set is determined whether it is a sand set or a shale
!     set by looking at the integer properties of the element groups.
!     For each set, the number of neighbors is determined, if the
!     number of neighbors is equal to one, then the set is an inclusion.
!     This new data structure is meant for use in the deflated ICCG
!     algorithm.
!     Old subroutine prpjdset
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     projection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleproj
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ielgrp, icongrp, ncongrp, i, j, neighbr, iset, &
                 jcongrp(nelgrp), congrp(nelgrp)

!     congrp         Integer array, contains information which element
!                    groups are coupled to the set, is filled during the
!                    process. CONGRP(i) =
!                    1 element group i is coupled but not yet evaluated
!                   -1 element group has already been evaluated
!     i              Loop variable
!     icongrp        Counting variable
!     ielgrp         Loop variable
!     iset           Counting variable
!     j              Loop variable
!     jcongrp        Integer array, contains the same information as CONGRP
!                    but now sequentially
!                    CONGRP(NCONGRP) is the next elementgroup to be evaluated
!     ncongrp        Number of element groups in JCONGRP
!     neighbr        neighbor group of current element group
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRININ2        Print an integer two-dimensional array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Goal: Combining element groups into sets.
!
!     Note that two element groups are considered connected if
!     they are connected by at least one node.
!
!     A set is defined as a collection of connected element
!     groups that have the same integer property.
!
!     Semi PSD: seppjdset
!
!     Initalization:
!
!     Set the array GROUPSET everywhere zero:
!     GROUPSET(1..nELGRP):=0
!
!     Find out for each element group which set it belongs to
!     and determine the connections between the sets.
!     Loop over all element groups:
!     DO iELGRP := 1, nELGRP
!        IF there not already a set assigned to iELGRP, &
!        GROUPSET(iELGRP) = 0, THEN
!           Make a new set: nSET := nSET+1.
!
!           The type of the new set is stored in array SANDSET:
!           SANDSET(nSET):= type of element group iELGRP.
!
!           Because iELGRP is the only element group in the new
!           set we set CONGRP(1..nELGRP) to zero. Since element
!           group iELGRP has not been fully checked yet we set
!           CONGRP(iELGRP) := 1. The first group to examine is
!           iELGRP: therefore jCONGRP(1) := iELGRP. For the moment
!           there is only one group to examine: nJCONGRP := 1.
!
!           Loop until there are no more groups left to check out:
!           DO until nJCONGRP = 0
!              We take the last group from jCONGRP: jELGRP:= &
!              jCONGRP(nCONGRP). Array jCONGRP is shortened with
!              one: nJCONGRP=nJCONGRP-1. We check whether the group
!              and the set are of the same type: sand (1) or
!              shale (0).
!              IF they are of the same type THEN
!                 Group jELGRP belongs to the current set:
!                 GROUPSET(jELGRP) := nSET.
!                 All groups that are connected to jELGRP and have
!                 not been checked out yet (see CONGRP) are added
!                 to the back of jCONGRP and nCONGRP is updated.
!                 For the groups that have been added, CONGRP is
!                 set to one.
!              ELSE
!                 Check whether group jELGRP already has been
!                 assigned to a set: iSET := GROUPSET(jELGRP)>0.
!                 IF this is the case THEN
!                    The connection between that set and the
!                    current one is saved in CONSET.
!                    CONSET(nSET,iSET):=1, CONSET(iSET,nSET):=1.
!                    The number of connected sets is updated:
!                    nCONSET(iSET+1) and nCONSET(nSET+1) are both
!                    increased with one.
!                 END IF
!              END IF
!              Group iELGRP has now been completely checked out:
!              CONGRP(iELGRP):=-1.
!           END DO
!        END IF
!     END DO
!
!     Loop over all newly created sets, check whether they
!     are an inclusion and transform the array CONSET
!     into an index array for the compact array CCONSET.
!     DO iSET := 1, nSET
!        Check for each set the number of neighbors. If an area has
!        only one neighbor then it is an inclusion:
!        INCLUSN(iSET) := 1. Transform the arry nCONSET into an
!        index array: nCONSET(iSET+1) := nCONSET(iSET+1) + &
!        nCONSET(iSET).
!     END DO
!
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seppjdset' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seppjdset'
         call prinin ( intprop, nelgrp, 'intprop' )
         call prinin ( kmshv1, nelgrp+1, 'kmshv1' )
         call prinin ( kmshv2, kmshv1(nelgrp+1), 'kmshv2' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( print_lv>=3 ) then
         write(irefwr,*) ' '
         write(irefwr,*) '   Number of element groups: ', nelgrp
         write(irefwr,*) '   Number of integer properties: ', nintprop
         write(irefwr,*) ' '
         write(irefwr,*) '   [ element group : integer property ]'
         do i=1,nelgrp
            write(irefwr,*) '   ', i, ' : ', intprop(1,i)
         end do
         write(irefwr,*) ' '
      end if

!     --- number of sets is zero
!         number of element groups belonging to sets is zero
!         PART I of MKSET
!         determine set number for each element group and
!         save connections between sets

      nset = 0
      nconset = 0

!     --- initializing data to zero

      groupset(1:nelgrp) = 0

      do ielgrp = 1, nelgrp

         if ( groupset(ielgrp)==0 ) then

!        --- a new set is made

            nset = nset+1
            sandset(nset) = intprop(1,ielgrp)
            nconset(nset+1) = 0
            conset(nset,:)= 0
            congrp = 0
            jcongrp = 0

            congrp(ielgrp) = 1
            jcongrp(1) = ielgrp
            ncongrp = 1
100         if ( ncongrp>0 ) then
               icongrp = jcongrp(ncongrp)
               if ( intprop(1,icongrp)==sandset(nset)) then

!              --- group belongs to current set

                  groupset(icongrp) = nset

!                 --- check the neighboring groups

                  do i = kmshv1(icongrp)+1, kmshv1(icongrp+1)
                     neighbr = kmshv2(i)
                     if ( congrp(neighbr)==0 ) then

!                    --- neighbor has not been checked out yet,
!                        so it can be added

                        congrp(neighbr) = 1
                        jcongrp(ncongrp) = neighbr
                        ncongrp = ncongrp+1
                     end if
                  end do
               else

                  iset = groupset(icongrp)
                  if ( iset>0 ) then

!                 --- set is connected to another set

                     if ( conset(nset,iset)==0 ) then
                        nconset(nset+1) = nconset(nset+1)+1
                        conset(nset,iset) = 1
                        nconset(iset+1) = nconset(iset+1)+1
                        conset(iset,nset) = 1
                     end if

                  end if

               end if

!              --- icongrp has been checked

               congrp(icongrp) = -1
               ncongrp = ncongrp - 1
               go to 100

            end if

         end if

      end do

      if ( debug ) then
         call prinin ( congrp, nelgrp, 'congrp' )
         call prinin2 ( conset, nelgrp, nelgrp, 'conset' )
         call prinin ( nconset, nelgrp+1, 'nconset' )
      end if  ! ( debug )

!     --- PART II of MKSET
!         In this part the data is determined which sets inclusions
!         are. The connections between sets are saved in a compact
!         form into array IBUFFR.

      nconset(1) = 0
      do iset = 1, nset

         if ( nconset(iset+1)==1 ) then

!        --- set iSET is een inclusie!

            inclusn(iset) = 1
         else
            inclusn(iset) = 0
         end if

!        --- Creating an index array:

         nconset(iset+1) = nconset(iset+1) + nconset(iset)

      end do

      if ( debug .or. print_lv>=3 ) then
         write(irefwr,*) '   [ element group : set ] '
         do i = 1, nelgrp
            write(irefwr,*) '   ', i, ' : ', groupset(i)
         end do
         write(irefwr,*) ' '
         write(irefwr,*) '   [ set : property ]'
         do i = 1, nset
            if ( sandset(i)==1 ) then
               if ( inclusn(i)==1 ) then
                  write(irefwr,*) '   ', i, ' :  sand inclusion'
               else
                  write(irefwr,*) '   ', i, ' :  sand set'
               end if
            else
               if ( inclusn(i)==1 ) then
                  write(irefwr,*) '   ', i, ' :  shale inclusion'
               else
                  write(irefwr,*) '   ', i, ' :  shale set'
               end if
            end if
         end do
         write(irefwr,*) ' '
      end if

1000  if ( debug ) then

!     --- Debug information

         call prinin ( groupset, nelgrp, 'groupset' )
         call prinin2 ( conset, nelgrp, nelgrp, 'conset' )
         call prinin ( nconset, nelgrp+1, 'nconset' )
         call prinin ( inclusn, nelgrp, 'inclusn' )
         call prinin ( sandset, nelgrp, 'sandset' )
         write(irefwr,*) 'End seppjdset'

      end if  ! ( debug )
      call erclos ( 'seppjdset' )

      end subroutine seppjdset
