      subroutine seppjmate
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 29-06-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the projection matrix E and vectors Au for cg
!     Old subroutine prpjmate
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     projection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmoduleproj
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, idegfd, j, row, col, iproj, i1, i2, &
                 indxconn, jproj
      integer cmap(nrusol)
      double precision work1(nrusol)
      double precision :: som
      type (globmat), pointer :: S
      type (intmatrix), pointer :: IM
      double precision, pointer :: mat(:)
      integer, pointer :: diag(:)

!     IM             Refers to intmt(intmat)
!     S              Refers to global matrix
!     cmap         - Integer array containing information on the index nummer
!                    of a degree of freedom in array INTPRJ or PROJVC.
!                    CMAP( iDEGFD ) = index number of iDEGFD in array projrows
!                    or array PROJVC.
!     col            Column number
!     diag           Positions of the diagonal elements of the matrix
!     i              Counting variable
!     i1             Dummy variable
!     i2             Dummy variable
!     idegfd         Number of a degree of freedom
!     indxconn       Index of a degree of freedom connected to iDEGFD into
!                    array PROJVC.
!     iproj          Number of a projection vector
!     j              Counting variable
!     jproj          Number of a projection vector
!     mat            elements of matrix
!     row            Row number
!     som            Help variable
!     work1          Temporary non-sparse version of a (projection) vector.
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRDECOSM       Compute LDL^T decomposition of symmetric full matrix
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
!     SEPCOMMATREV   Fill arrays irowsnew and icolsnew with upper triangular
!                    part if icolsii and irowsii contain the lower triangular
!                    part and vice versa
!     SEPMATREV      Copy matrix from rowwise to columnwise storage
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Goal: Creating the sparse vectors Au and the matrix E
!     Note that the projection vectors, u1...uNPROJ, are stored in
!     sparse form. This sparse form is kept while doing a matrix
!     vector product. When two sparse vectors are multiplied first
!     one of them is made non-sparse.
!     Semi PSD: prpjGCON
!     Compute the vectors Au by multiplying each projection vector
!     with the large matrix. The resulting vectors are stored in projAU.
!     The matrix E is computed as :  E_ij = (Au_i)^T u_j
!     Finally the choleski decomposition of E is made in PRDECOSM. The
!     result is stored in projE2d.
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seppjmate' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seppjmate'
         write(irefwr,1) 'nproj, nnproj, nrusol', nproj, nnproj, nrusol
         call prinin ( projrows, nproj+1, 'projrows' )
         call prinin ( projcols, nnproj, 'projcols' )
  !      call pr0004 ( 5, nrusol, irowsii, icolsii )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( debug ) then
         write(irefwr,1) 'nnproj, nrusol', nnproj, nrusol
         call prinrl ( projU, nnproj, 'projU' )
      end if  ! ( debug )

      S => realmt(iseqmt)
      IM => intmt(S%intmatref)

!     --- Change to row-wise storage

      allocate ( irowsii(size(IM%rowsii)), icolsii(size(IM%colsii)), &
                 mat(size(IM%colsii)), diag(size(IM%rowsii)-1) )
      call sepcommatrev ( irowsii, icolsii, IM%rowsii, IM%colsii, nrusol )
      call sepmatrev ( irowsii, mat, IM%rowsii, IM%colsii, S%S_ii, nrusol )
      diag = irowsii(2:size(irowsii))

!     --- Clear array projAU

      projAU = 0d0

!     --- Compute the vectors Au

      cmap(1:nrusol) = 0

      do iproj = 1, nproj

!     --- Store the index number of each degree of freedom

         do i1 = projrows(iproj)+1,projrows(iproj+1)
            row = projcols(i1)
            cmap(row) = i1
         end do

!        --- Loop over all non-zero elements in the vectors Au.

         do i1 = projrows(iproj)+1, projrows(iproj+1)
            row = projcols(i1)

!           --- Multiply by the main diagonal entry

            projAU(i1) = projAU(i1)+&
               mat(diag(row))*projU(i1)

!           --- loop over all entries in row ROW

            som = 0d0

            do i2 = irowsii(row)+1, irowsii(row+1)-1
               col = icolsii(i2)

!              --- Determine index number of degree of freedom COL
!                  in array PROJVC.

               indxconn = cmap(col)

               if ( indxconn>0 ) then
                  som = som+mat(i2) * &
                   projU(indxconn)

                  projAU(indxconn) = projAU(indxconn)+&
                   mat(i2)*projU(i1)
               end if
            end do
            projAU(i1) = projAU(i1)+som
         end do

         do i1 = projrows(iproj)+1,projrows(iproj+1)
            row = projcols(i1)
            cmap(row) = 0
         end do

      end do

      if ( debug .or. print_lv>=4 ) then
         write(irefwr,*) ' '
         do i=1,nproj
            write(irefwr,*) '   Sparse vector Au: ', i
            write(irefwr,*) '   [ degree of freedom : value ]'
            do j = projrows(i)+1,projrows(i+1)
               write(irefwr,100) projcols(j), projAU(j)
            end do
            write(irefwr,*) ' '
         end do
100      format( '    ', i4, ' : ', 1pe12.5e2 )
      end if

!     --- Compute the matrix E

      do iproj = 1, nproj
         do jproj = iproj, nproj
            projE2d(iproj,jproj) = 0d0
            work1(1:nrusol) = 0d0
            do i1 = projrows(iproj)+1,projrows(iproj+1)
               idegfd = projcols(i1)
               work1(idegfd) = projU(i1)
            end do
            do i1 = projrows(jproj)+1, projrows(jproj+1)
               idegfd = projcols(i1)
               projE2d(iproj,jproj) = projE2d(iproj,jproj)+&
                work1(idegfd) * projAU(i1)
            end do
            projE2d (jproj,iproj) = projE2d(iproj,jproj)
         end do
      end do

      if ( debug .or. print_lv>=4 ) then
         write(irefwr,*) '   Matrix E'
         write(irefwr,*) '   [ i : j : E(i,j) ]'
         do i=1,nproj
            do j=1,nproj
              write(irefwr,110) i,  j, projE2d(i,j)
            end do
         end do
110      format( '    ', i4, ' : ', i4, ' : ', 1pe12.5e2 )
         write(irefwr,*) ' '
      end if

!     --- Compute the Cholesky decomposition of E
!         and store in the same positions

      call prdecosm ( nproj, projE2d )
      deallocate ( irowsii, icolsii, mat, diag )

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( projE2d, nproj, nproj, 'projE2d' )
         write(irefwr,*) 'End seppjmate'

      end if  ! ( debug )
      call erclos ( 'seppjmate' )

      end subroutine seppjmate
