      subroutine seppjspar
! ======================================================================
!
!        programmer     Gjalt Wijma
!        version  1.0  date 14-06-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     In this subroutine the sparse projection vectors are assembled
!     from the constant parts and the subsolutions.
!     Old subroutine prpjspar
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     projection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleproj
      use sepmodulekprob
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer idxprojv, iproj, iset, i1, i2, &
              idegfd, inodp, conpoint, condegfd, i, j
      integer, allocatable :: checkdgf(:)

!     checkdgf       Integer help array. If CHECKDGF( iDEGFD ) = 1 then
!                    degree of freedom iDEGFD belongs to the sparse
!                    projection vector.
!     condegfd       Degree of freedom connected to iDEGFD.
!     conpoint       Nodal point connected to iNODP.
!     i              Counting variable
!     i1             Counting variable
!     i2             Counting variable
!     idegfd         Number of a degree of freedom
!     idxprojv       Index into array RTSPROJV, see above.
!     inodp          Number of a nodal point
!     iproj          Number of a projection vector
!     iset           Set number.
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Goal: Creating sparse projection vectors from the
!           sub-solutions and the constant parts.
!
!     Semi PSD: seppjspar
!
!     We start with assembling all projection vectors set by set.
!
!     Loop over all projection vectors:
!     DO iPROJ = 1, nPROJ
!        Loop over all sets:
!        DO iSET = 1, nSET
!           IF the projection vector is one in this set, &
!           ITSPROJV(iproj,iset) = 1, THEN
!              Copy all degrees of freedom in this set to
!              array INTPRJ and copy an equal number of ones
!              to array PROJVC. Update the number of degrees
!              of freedom in this projection vector in array
!              INTPRJ.
!
!              The degrees of freedom that are added are marked
!              in array CHECKDGF
!           ELSE IF the projection vector is equal to a sub- &
!           solution in this set, ITSPROJV(iproj,iset) < 0, &
!           THEN
!              Copy all entries in the subsolution that are larger
!              than the threshold value from array RTSPROJV to
!              array PROJVC.
!
!              Copy all accompanying degrees of freedom to array
!              INTPRJ and update the number of degrees of freedom
!              in this projection vector in array INTPRJ.
!
!              The degrees of freedom that are added are marked
!              in array CHECKDGF
!           END IF
!        END DO
!
!        Next all neighbors of the degrees of freedom in the
!        projection vector are added. This is done so that
!        the product of the large matrix with the projection
!        vector, Au, is defined in the same degrees of freedom
!        as the projection vector itself.
!
!        LOOP over all degrees of freedom of the projection vector
!           LOOP over all neighbor degrees of freedom
!              IF a neighbor degree of freedom is not defined
!              for this projection vector, CHECKDGF( iDEGFD ) = 0, &
!              THEN
!                 This degree of freedom is added to array INTPRJ.
!                 A zero is added to array PROJVC.
!                 The number of degrees of freedom in this
!                 projection vector is updated.
!                 The degree of freedom that is added is marked
!                 in array CHECKDGF
!              END IF
!           END LOOP
!        END LOOP
!     END DO
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seppjspar' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seppjspar'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( checkdgf(nusol), stat = error )
      if ( error/=0 ) call eralloc ( error, nusol, 'checkdgf' )
      if ( ierror/=0 ) go to 1000

      if ( debug .or. print_lv>=3 ) then
         write(irefwr,*) ' '
         write(irefwr,*) 'Estimated total length of sparse', &
                          ' projection vectors:', nnproj
         write(irefwr,2) 'Threshold value: ', threshold
         write(irefwr,*) ' '
         do iproj=1,nproj
            write(irefwr,1) '   projection vector:', iproj
            write(irefwr,*) '   [ set : type of solution ]'
            do iset =1,nset
               if ( itsprojv(iproj,iset)==1 ) then
                  write(irefwr,*) '   ', iset, ' :  constant one'
               else if ( itsprojv(iproj,iset)<0 ) then
                  write(irefwr,*) '   ', iset, ' :  sub-solution'
               else
                  write(irefwr,*) '   ', iset, ' :  constant zero'
               end if
            end do
            write(irefwr,*) ' '
         end do
      end if

      idxprojv = 0
      projrows(1) = 0

!     --- Loop over all projection vectors

      do iproj = 1, nproj

         checkdgf = 0

         projrows(iproj+1) = projrows(iproj)

!        --- Loop over all sets

         do iset = 1, nset

!        --- Check if the current projection vector is non-zero on
!            this set...

            if ( itsprojv(iproj,iset)==1 ) then

!           --- The projection vector is one in this set

               do i1 = nsetnode(iset)+1, nsetnode(iset+1)
                  inodp = setnode(i1)
                  idegfd = kprobh(inodp)

                  checkdgf(idegfd) = 1
                  projrows(iproj+1) = projrows(iproj+1)+1
                  idxprojv = idxprojv+1
                  projcolstmp(idxprojv) = idegfd

                  projUtmp(idxprojv) = 1d0
               end do

            else if ( itsprojv(iproj,iset)<0 ) then

!           --- The projection vector is equal to one of the
!               partial solutions on this set.

               i1 = abs(itsprojv(iproj,iset))
               do i2 = nsetnode(iset)+1, nsetnode(iset+1)
                  inodp = setnode(i2)
                  idegfd = kprobh(inodp)

!                 --- Check if the value of the partial solution is
!                     above the threshold factor.

                  if ( rtsprojv(i1) - threshold.gt.0 ) then

                     checkdgf(idegfd) = 1
                     projrows(iproj+1) = projrows(iproj+1)+1
                     idxprojv = idxprojv+1
                     projcolstmp(idxprojv) = idegfd
                     projUtmp(idxprojv) = rtsprojv(i1)

                  end if
                  i1 = i1+1
               end do
            end if
         end do

!        --- Include the neighbors into the sparsity pattern. This
!            is done so that A u has the same sparsity storage as u,
!            where u is a projection vector.

         do i1 = projrows(iproj)+1, projrows(iproj+1)
            idegfd = projcolstmp(i1)
            inodp = kprobhinv(idegfd)

!           --- Loop over all neighbors of this nonzero entry

            do i2 = kmshe1(inodp)+1, kmshe1(inodp+1)
               conpoint = kmshe2(i2)
               condegfd = kprobh(conpoint)

               if ( checkdgf(condegfd)==0 .and. condegfd<=nrusol ) then

!              --- The connected degree of freedom is added to the
!                  sparsity pattern. Of course is the projection
!                  vector equal to zero in the connected degree of
!                  freedom.

                  checkdgf(condegfd) = 1
                  projrows(iproj+1) = projrows(iproj+1)+1
                  idxprojv = idxprojv+1
                  projcolstmp(idxprojv) = condegfd
                  projUtmp(idxprojv) = 0d0
               end if
            end do
         end do

      end do

!     --- Update estimated length of the projection vectors

      nnproj = projrows(nproj+1)

      if ( debug .or. print_lv>=3 ) then
         write(irefwr,*) 'Exact total length of sparse', &
                          ' projection vectors:', nnproj
         write(irefwr,*) ' '
      end if
      if ( print_lv>=4 ) then
         do i=1,nproj
            write(irefwr,*) '   Sparse projection vector: ', i
            write(irefwr,*) '   [ degree of freedom : value ]'
            do j = projrows(i)+1,projrows(i+1)
               write(irefwr,110) projcolstmp(j),projUtmp(j)
            end do
            write(irefwr,*) ' '
         end do
110      format( '    ', i4, ' : ', 1pe12.5e2 )
      end if

      deallocate ( checkdgf, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'checkdgf' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'nproj', nproj
         call prinin ( projrows, nproj+1, 'projrows' )
         write(irefwr,*) 'End seppjspar'

      end if  ! ( debug )
      call erclos ( 'seppjspar' )

      end subroutine seppjspar
