      subroutine seppjspar4
! ======================================================================
!
!        programmer    Fred Vermolen
!        version  1.0  date 27-06-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     In this subroutine the sparse projection vectors are assembled
!     from the constant parts. For the interface nodes an averaged
!     overlap is used, such that there is no overlap when the element
!     group corresponds to shale layer. For details see Report 01-13 of
!     the department of applied mathematical analysis.
!     Old subroutine prpjspar4
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     projection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleproj
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer idxprojv, i1, i2, i3, istart_ielgrp, nsand, &
              nbnusol, nnodes, inodp, nintnod, &
              nbnode, ielgrp, inusol, istart_node, nofelgr, &
              elgrpi, intnodes(npoint), checkdgf(npoint), &
              dcn

!     checkdgf       list that indicates whether a neighboring node
!                    outside the present element group has been
!                    checked or not checkdgf(inusol) is 1 means that
!                    the node has been checked
!     dcn            help variable to indicate that a neighboring
!                    interface must be added to the initialization
!                    of the projection vector
!                    they are not
!     elgrpi         element group number?
!     i1             Counting variable
!     i2             Counting variable
!     i3            Counting variable
!     idxprojv       index of projection vector during filling
!     ielgrp         element group number
!     inodp          Nodal point number
!     intnodes       integer array containing all nodes at the
!                    interface of an element group
!     inusol         Position of unknown in array USOL
!     istart_ielgrp  Starting position of nodes for element group
!     istart_node    starting index for node_elgrps to indicate in which
!                    element groups an interface node is located
!     nbnode         node numbers of neighboring nodes in neighboring
!                    element groups
!     nbnusol        degree of freedom of node nbnode
!     nintnod        number of interface nodes of an element group
!     nnodes         Number of nodes in an element group
!     nofelgr        number of element groups
!     nsand          number of sand layers where an interface node is in
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Goal: Creating sparse projection vectors from the constant parts
!           and the average overlaps at the interface nodes
!
!     Semi PSD: seppjspar4
!
!     We start with assembling all projection vectors set by set.
!
!     LOOP over all elementgroups:
!        Determine the number of nodes in the element group
!        Determine the list of nodes in the element group
!        Number of interface nodes = 0
!        LOOP over all interface nodes in the element group
!           Copy the position of the corresponding unknown to the array INTPRJ
!           Determine the number of element groups the node lies in
!           IF the node lies in one element group THEN
!              projUtmp = 1
!           ELSE
!              IF the current element group has property 0 (shale) THEN
!                 projUtmp = 0
!              ELSE
!              Determine the number of element groups the node lies in
!              Determine the list of element groups the node lies in
!              LOOP over all element groups the node lies in
!                 IF the element group has property 1 (sand) THEN
!                    increment the number of sandlayers in the interface node
!                    NSAND = NSAND + 1
!                 END IF
!              END LOOP
!              projUtmp = 1 / NSAND
!              Increment the number of interface nodes
!              Add the interface node to the list of interface nodes
!           END IF
!        END LOOP
!
!        Next all neighbors of the degrees of freedom in the
!        projection vector are added. This is done so that
!        the product of the large matrix with the projection
!        vector, Au, is defined in the same degrees of freedom
!        as the projection vector itself. In the following routine
!        we use the list of interface nodes since only the neighbors
!        of the interface nodes are involved.
!
!        LOOP over all interface nodes
!           Determine the number of neighbors of the interface node
!           Determine the list of neighbors of the interface node
!           LOOP over all neighboring nodes
!              Determine the element group the neighbor lies in
!              IF neighbor lies in different element group than current one
!              THEN
!                 Copy the position of the corresponding unknown to
!                               the array INTPRJ
!                 projUtmp = 0
!              END IF
!           END LOOP
!           Clean entry in list of interface nodes for current interface node
!        END LOOP
!
!     END LOOP
!
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seppjspar4' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seppjspar4'
         write(irefwr,1) 'print_lv, npoint', print_lv, npoint
         call prinin ( ielgrp_nodes, nelgrp, 'ielgrp_nodes' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      idxprojv = 0
      projrows(1) = 0

      if (print_lv==1) print_lv = 0

!     --- First the array that contains interface nodes is initialized

      intnodes(1:npoint) = 0
      checkdgf(1:npoint) = 0

!     --- Loop over all element groups

      do ielgrp = 1, nelgrp
         projrows(ielgrp+1) = projrows(ielgrp)
         istart_ielgrp = ielgrp_nodes(ielgrp)
         nnodes = ielgrp_nodes( istart_ielgrp )
         nintnod = 0

!        --- Loop over nodes in the element group

         do i1 = 1,nnodes
            inodp = ielgrp_nodes( istart_ielgrp+i1 )
            inusol = kprobh( inodp )
            if ( inusol<=nrusol ) then

!           --- inusol<=nrusol, i.e. non-prescribed unknown

               idxprojv = idxprojv + 1
               projcolstmp(idxprojv ) = inusol
               projrows(ielgrp+1) = projrows(ielgrp+1) + 1

!              --- If the node is contained by this element group only,
!                  then the entry in the projection vector is one.
!                  If the node is also contained in other element groups,
!                  then: 1) the node should be added to the array containing
!                           the interface nodes (INTNODES),
!                        2) the corresponding entry in the projection vector
!                           becomes one divided by the number of sand layers
!                           that contain the interface node.

               if   ( node_elgrps( inodp )>0 ) then
                  projUtmp(idxprojv) = 1d0
               else
                  nintnod = nintnod + 1
                  intnodes( nintnod ) = inodp
                  if (intprop(1,ielgrp)==0) then
                     projUtmp(idxprojv) = 0d0
                  else
                     nsand = 0
                     istart_node = -node_elgrps( inodp )
                     nofelgr = node_elgrps( istart_node )
                     intnodes( nintnod ) = inodp
                     do i2 = 1,nofelgr
                        elgrpi = node_elgrps( istart_node+i2 )
                        if (intprop(1,elgrpi)==1) then
                           nsand = nsand + 1
                        end if
                     end do
                     projUtmp(idxprojv) = 1d0 / nsand
                  end if
               end if
            end if
         end do

!        --- The Projection vector has been filled for the element group ielgrp.
!            Now we include the neighbors of the interface nodes into the
!            sparsity pattern by setting the corresponding entries of the
!            projection vector equal to zero. Here we use the array INTNODES
!            that
!            contains the interface nodes. After use the particular entry in
!            the array INTNODES is cleared. Furthermore, for this purpose
!            the array CHECKDGF is used to monitor whether a particular
!            neighboring point has been incorporated into the sparsity
!            pattern, i.e. each neighboring point is incorporated only once.
!            The neighbors are only incorporated into the sparsity pattern
!            whenever the element group has a projection vector that has
!            non-zero entries on the interfaces.

         if (intprop(1,ielgrp)==1) then
            do i1 = 1,nintnod
               inodp = intnodes( i1 )
               do i2 = kmeshe( inodp ) + 1,kmeshe( inodp+1 )
                  nbnode = kmeshe( npoint+1+i2 )
                  if (node_elgrps( nbnode )<0 ) then
                     istart_node = -node_elgrps(nbnode)
                     dcn = 1
                     do i3 = 1,node_elgrps(istart_node)
                        if (node_elgrps(istart_node+i3)==ielgrp) then
                           dcn = 0
                        end if
                     end do
                     if ((dcn==1).and.(checkdgf( nbnode )==0)) then
                        checkdgf( nbnode ) = 1
                        nbnusol = kprobh( nbnode )
                        if ( nbnusol<=nrusol ) then
                           projrows(ielgrp+1) = projrows(ielgrp+1) + 1
                           idxprojv = idxprojv + 1
                           projcolstmp(idxprojv ) = nbnusol
                           projUtmp(idxprojv) = 0d0
                        end if
                     end if
                  end if
                  if ( ( node_elgrps( nbnode )>0 ) .and. &
                     ( node_elgrps( nbnode )/=ielgrp ) .and. &
                      (checkdgf( nbnode )==0 )) then
                     checkdgf( nbnode ) = 1
                     nbnusol = kprobh( nbnode )
                     if ( nbnusol<=nrusol ) then
                        projrows(ielgrp+1) = projrows(ielgrp+1) + 1
                        idxprojv = idxprojv + 1
                        projcolstmp(idxprojv ) = nbnusol
                        projUtmp(idxprojv) = 0d0
                     end if
                  end if
                end do
            end do
         end if

         do i1=1,nintnod
            intnodes( i1 ) = 0
         end do

         do i1=1,npoint
            checkdgf(i1) = 0
         end do

      end do

      if ( projrows(nelgrp+1)>nnproj ) then

!     --- Arrays too small

         call errint ( projrows(nelgrp+1), 1 )
         call errint ( nnproj, 2 )
         call errchr ( 'nnproj', 1 )
         call errsub ( 550, 2, 0, 1 )

      end if
      nnproj = projrows( nelgrp+1 )
      nproj  = nelgrp

1000  call erclos ( 'seppjspar4' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End seppjspar4'

      end if  ! ( debug )

      end
