      subroutine septimenth2eu ( finished, iseqenthalpy, iinenthalpy, &
                                 rinenthalpy )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 15-12-2017 New storage of intmat
!        version  1.0  date 12-11-2015
!
!   copyright (c) 2015-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Time integration of a discretized partial differential equation
!    of first order in time with an impicit modified theta method
!              .
!          M * u + S * u = F
!              -       -   -
!    It has been assumed that the matrices M and S are not dependent
!    of the solution vector u. Hence the system is linear . M and
!    S may of course depend on place and time.
!    Solution method used is the modified theta method. The user
!    has to give the stepsize tstep.
!    Special subroutine for enthalpy in combination with Newton
!     Old subroutine prtimenthal2eu
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     enthalpy
!     time_integration
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulemnsub
      use sepmodulemain
      use sepmoduleintmat
      use sepmoduletime
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iseqenthalpy, iinenthalpy(*)
      double precision, intent(in) :: rinenthalpy(*)
      logical, intent(inout) :: finished

!     finished      i/o   Indicates if the time loop has been finished (true)
!                         or not  (false)
!     iinenthalpy    i    Integer input array with respect to enthalpy
!                         computations
!                         For a definition of this array see subroutine
!                         readenthalpy (array iinput)
!     iseqenthalpy   i    Sequence number of input for block enthalpy
!     rinenthalpy    i    Real input array with respect to enthalpy
!                         computations
!                         For a definition of this array see subroutine
!                         readenthalpy (array iinput)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: maxprbloc=10
      integer :: ideltatemp, i, ieqn, &
                 isolprec(maxprbloc), niter
      double precision :: tsteps
      logical :: first, finishtm, linear
      character (len=25) :: text
      save ideltatemp, tsteps, first, isolprec, niter

!     finishtm       Indicates if the time loop has been finished (true)
!                    or not (false)
!     first          If true it concerns the first time step
!     i              Counting variable
!     ideltatemp     Sepran array of length 5 containing information
!                    concerning the temperature difference during the
!                    iteration
!     ieqn           Equation sequence number
!     isolprec       Solution at preceding level
!     linear         If true the linear time-iteration is used, if false an
!                    iteration per time step is performed
!     maxprbloc      Maximum number of coupled problems allowed
!     maxprbloc5     5*maxprbloc
!     niter          Iteration sequence number
!     text           text with respect to writing or plotting
!     tsteps         help variable to save the value of tstep
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRCOMPENTHALP2 Compute the enthalpy, given the temperature
!                    Newton approach
!     PRINTHIS       Print time history
!     SEPCOPYVEC     copy Sepran array
!     SEPTIMBOUNCOND Fill boundary conditions in array isol and check if the
!                    array corresponds to the correct problem
!     SEPTIMIINBLD   Fill some parameters in iinbld and initialize
!                    some other parameters for time integration
!     SEPTIMRESET    reset all arrays with respect to time integration that
!                    are no longer used
!     SEPTIMTHETA1   Start the time integration for suboutine septimtheta
!     SEPTIMTHETA2   Set some parameters during a time loop
!     SEPTIMTHETAFRC Extrapolate the solution from old and intermediate time
!                    step to next time step in case of theta method
!     SEPTIMUSER     Fill arrays iuser and user for a specific
!                    problem in a sequence of problems
!     SEPTMENTHALPY2 Perform one step in an Euler implicit scheme for the
!                    enthalpy equation
!                    Newton approach
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Differential equations to be solved:
!        .
!     M  u   +  S  u   =  f
!     Application of Euler
!     -Euler part (predictor):
!       n+o     n
!      u    -  u
!       j       j           n+o      n+o
!    M ----------- +  S  u     =  f          (n is actual time-level)
!          dt               j        j         (n+o is time at t+theta dt)
!    - Extrapolation step
!     n+1     1     n+o            n
!    u    = -----( u   - (1-theta)u )
!           theta
!    or. &
!                 n+o        n         n+o
!   ( M + dt S ) u     =  M u    + dt f
!                 j          j         j
!                             n                 n+1
!   M and S may depend on:   u    k>=j  and  u      k<j (Silechi approach)
!                             k                 k
!   If a stationary solution is required, the process is stopped if
!       k+1    k      1-lambda               k+1
!   || u    - u  ||<-------- (eps_abs + ||u   || eps_rel)
!                      lambda
!                     k+1   k
!                 || u   - u ||
!   with lambda = ------------- &
!                     k   k-1
!                 || u - u ||
!   Check input
!   ifrac := 1
!   stationary := false
!   While ( ifrac>1 or t<tout and not stationary ) do
!      if ( nfrac=1 ) then
!         t := t + theta dt
!      else ( nfrac>1 )
!         t := t + theta_(ifrac+1) dt
!         theta := theta_(ifrac+1) / (theta_ifrac + theta_(ifrac+1) )
!      Copy isolut2d into isol
!      For iprob := 1 (1) ncopeqtim do
!         Build matrices M(iprob) and S(iprob)
!         Store boundary conditions into usol(iprob)
!         Build right-hand side f without the effect of boundary conditions
!            i.e. the part fi
!         help := M un, i.e. Mii ui + Mir ur
!         help := help + dt f
!         if ( icons=0 or t=t0+dt ) then
!            M_help = M + dt S
!         help1 := M_helpir ur (n+1)
!         help  := help - help1
!         Solve M_help isol(iprob) = help
!         Copy isol(iprob) into isolut2d(iprob)
!      If ( extrapolation required ) then
!         extrapolate solution
!         if ( nfrac=1 ) then
!            t := t + (1-theta) dt
!         else
!            t := t + theta_ifrac dt
!            ifrac := modulo(ifrac+2,nfrac)
!      If ( stationary solution required ) then
!         estimate lambda
!         check stopping criterion
!   If ( t>=tend ) then
!      Delete all arrays created
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data ideltatemp /0/, isolprec/maxprbloc*0/
! ======================================================================
!
      call eropen ( 'septimenth2eu' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from septimenth2eu'
         write(irefwr,*) 'iseqenthalpy ', iseqenthalpy

      end if

!     --- Start of process
!         Set parameters

      ncopeqtim = 2
      Teq(1)%iseqfr = iinenthalpy(3)  ! temperature
      Teq(2)%iseqfr = iinenthalpy(2)  ! enthalpy
      ireadbctim = iinenthalpy(5)
      call  septimtheta1 ( finishtm, first, linear )

!     --- In ideltatemp we store the temperature difference
!         Copy temperature vector into temperature difference

      call sepcopyvec ( isolut2d(iinenthalpy(3)), ideltatemp, 0 )
      tsteps = tstep

!     --- While loop over time-steps

      do while ( .not. finishtm )

!     --- t<tout
!         Clear array dT stored in ideltatemp

         usol => ks(ideltatemp)%sol
         usol = 0d0

!        --- Copy solution at old time level into isolprec

         do i = 1, ncopeqtim

            call sepcopyvec ( isolut2d(Teq(i)%iseqfr), isolprec(i), 0 )

!           --- set some parameters, which are used in the time loop

            call septimtheta2 ( tsteps, i )

         end do  ! i = 1, ncopeqtim

!        --- Put boundary conditions into temperature

         ncopeqtim = 1
         if ( itimbc<2 .and. (first .or. itimbc==0) ) then

!        --- Either first call, or boundary conditions are time-dependent
!            Mark that for the ncopeqtim problems
!            the boundary conditions are only given for the temperature (1)

            isol2d => isolut2d(iinenthalpy(3):size(isolut2d))
            isolut2dred => isolut2d
            call septimbouncond ( first )

!           --- we have to recompute the enthalpy if the temperature
!               boundary conditions have been changed

            call prcompenthalp2 ( numvecmn, iinenthalpy, 41 )

         end if  ! ( itimbc<2 .and. (first .or. itimbc==0) )

!        --- While loop for non-linear iteration
!            Loop over all problems

         ncopeqtim = 1
         ieqn = 1

!        --- First set the correct parameters in iinbld

         call septimiinbld ( first, 1 )

!        --- Compute starting addresses for the arrays iuser and user

         call septimuser ( first, 1 )

!        --- Perform one step Euler implicit
!            special case of enthalpy equation
!            an adapted version of prtm00 is used
!            iuser and user come from the input file

         call septmenthalpy2 ( first, isolprec, iinenthalpy, &
                               rinenthalpy, ideltatemp )

         ncopeqtim = 2  ! in the enthalpy case we have
                     ! 2 vectors (temp and enthalpy)

         if ( nfrac>1 .or. abs(1d0-theta)>100d0*epsmac ) then

!        --- extrapolate to final time

            call septimthetafrc ( isolut2d, isolprec )

         end if  ! ( nfrac>1 .or. abs(1d0-theta)>100d0*epsmac )

         if ( nhist>0 ) then

!        --- print time histor.

            call printhis ( nhist, nodhist, t, ncopeqtim, &
                            isolut2d(iinenthalpy(3)), first )

         end if
         if ( ifrac==1 .and. t>=tout-0.0001d0*tstep .or. &
              .not. complrun ) finishtm = .true.
         first = .false.

      end do

!     --- While loop has been finished, reset quantities

      tstep = tsteps
      iflag = 3

      if ( t>=tend-0.00001*tstep ) then

!     --- t = tend   reset all arrays

         call septimreset ( irhsd, ideltatemp, isolprec, finished )

      end if  ! ( t>=tend-0.00001*tstep )

      if ( debug ) write(irefwr,*) 'End septimenth2eu'
      call erclos ( 'septimenth2eu' )

      end
