      subroutine septimenth4eu ( iinenthalpy, finished )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 15-12-2017 New storage of intmat
!        version  1.0  date 12-11-2015
!
!   copyright (c) 2015-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Time integration of the enthalpy equation using Newton linearization
!    The method of Fachinotti et al is used
!    No iteration
!    Simplified version of prtimenthal3eu
!     Old subroutine prtimenthal4eu
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     enthalpy
!     time_integration
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematr
      use sepmodulevecs
      use sepmodulemain
      use sepmoduleintmat
      use sepmodulesolve
      use sepmoduletime
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iinenthalpy(*)
      logical, intent(inout) :: finished

!     finished      i/o   Indicates if the time loop has been finished (true)
!                         or not  (false)
!     iinenthalpy    i    Integer input array with respect to enthalpy
!                         computations
!                         For a definition of this array see subroutine
!                         readenthalpy (array iinput)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ideltatemp, iseqtemp, normtype
      double precision :: eps, resnorm1, resnorm
      logical :: first, converged
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     converged      If true, the Newton Raphson process is terminated
!     eps            Required level of accuracy
!     first          If true it concerns the first step
!     ideltatemp     Search direction vector
!     iseqtemp       Sequence number of temperature in solution vector
!     normtype       Type of norm used. 1: infinity norm 2: l2-nor. &
!     resnorm        Norm of the residual
!     resnorm1       Norm of the initial residual
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPCOPINPSOL   Copy arrays inpsl1 and rinsl1 into inpsol and rinsol
!                    into sepmodulesolve
!     SEPCOPYVC      Copy one SEPRAN vector into another one
!     SEPLINSOL      Body of linear solver
!     SEPRMMATRIX    Remove array with sequence number iseq from realmt
!     SEPRMSOLINFO   Remove array with sequence number ivectlc from ks
!     SEPTIMEENTH3E1 Build matrix and right-hand side for enthalpy equation
!                    solved by Newtons method
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'septimenth4eu' )
      debug = .false.

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from septimenth4eu'
         write(irefwr,110) 't, tout, tend, tstep', t, tout, tend, tstep
100      format ( a, 1x, (10i6) )
110      format ( a, 1x, (5d12.4) )

      end if
      if ( ierror/=0 ) go to 1000

      intmat => intmatar(1)
      ideltatemp = 0

!     --- Set some parameters

      eps = max(absaccurtim,relaccurtim)
      iseqtemp = iinenthalpy(3) ! sequence number of temperature in isolut2d
      first = t<=t0+0.1d0*tstep ! if true first time step
      iprint = iinenthalpy(8)   ! print level
      converged = .false.
      normtype = 1

      if ( debug ) then
         write(irefwr,100) 'iseqtemp', iseqtemp
         write(irefwr,110) 'eps', eps
      end if  ! ( debug )

!     --- Adapt array inpsoltim

      if ( first ) then

!     --- First call, set some parameters in inpsoltim

         IM => intmt(intmat)
         if ( IM%direct .and. .not. IM%nested .and. .not. IM%mumps ) then

!        --- direct method

            if ( inpsoltim(3)>0 ) inpsoltim(3) = 0

         else

!        --- iterative method
!            use old solution and keep preconditioning if matrix is constant

            if ( inpsoltim(3)==0 ) inpsoltim(3) = 1
            inpsoltim(9)  = 1

         end if ! ( IM%direct .and. .not. IM%nested .and. .not. IM%mumps )

      end if ! ( first )

!     --- Initialize array ideltatemp by copying the old temperature

      call sepcopyvc ( isolut2d(iseqtemp), ideltatemp )
      if ( ierror/=0 ) go to 1000

!     --- Furthermore we have to make the essential boundary conditions
!         equal to 0. This can be done by setting the whole vector to 0

      deltatemp => ks(ideltatemp)%sol
      deltatemp = 0d0

!     --- Test for initial guess being a root. Use more stringent test than
!         simply eps. First we build the right hand side vectors and form the
!         right hand side (residual).
!         Build matrix and right-hand side

      call septimeenth3e1 ( iinenthalpy, 1 )

!     --- add both right-hand sides and store residual in irhsd(1)

      rhsd2 => ks(irhsd(2))%sol
      rhsd1 => ks(irhsd(1))%sol
      rhsd1 = rhsd2 + rhsd1

!     --- Initial guess is a root? If so, set converged to true
!         Keep in mind that irhsd(1) contains -1d0*residual !!!
!         Multiply by tstep!

      if (normtype==2) then

         resnorm = norm2(rhsd1)/size(rhsd1)

      else

         resnorm = maxval (abs(rhsd1))

      end if

      resnorm1 = resnorm*tstep
      if (debug) write(irefwr,*) 'resnorm1', resnorm1

      if (resnorm1<1d-6*eps) converged = .true.

!     --- While loop for one time-step
!         First raise time step

      t = t + tstep  ! new time level
      if ( iprint>1 ) then

         write(irefwr,*) 't, tstep', t, tstep
         write(irefwr,*) &
             'Iteration   2-Norm of residual   inf-Norm of residual'

      end if ! ( iprint>1 )

      if (converged) go to 1000

!     --- Build matrix and right-hand side
!         both R1 and R2 are build.

      call septimeenth3e1 ( iinenthalpy, 1 )

!     --- now add both right-hand sides and store in irhsd(1)

      rhsd1 = rhsd2 + rhsd1

!     --- Solve system of equations

      ireadinpsol = -1
      iseqsol = 1
      iseqreac = 0
      iseqintmat = 1
      keepmat = 0
      iseqsolut = 1
      iseqmt = stiffmat(1)
      call sepcopinpsol ( inpsoltim, rinsoltim )
      call seplinsol ( stiffmat, ideltatemp, irhsd )
      if ( ierror/=0 ) go to 1000

!     --- Add ideltatemp to temp

      temperature => ks(isolut2d(iseqtemp))%sol
      temperature = deltatemp + temperature

!     --- While loop has been finished, reset quantities

      iflag = 3

      if ( t>=tend-0.00001*tstep ) then

!     --- t = tend   reset all arrays

         call seprmsolinfo ( ideltatemp )
         call seprmmatrix ( stiffmat )
         call seprmsolinfo ( irhsd(1) )
         call seprmsolinfo ( irhsd(2) )
         finished = .true.

      end if  ! ( t>=tend-0.00001*tstep )

1000  if ( debug ) then
         write(irefwr,*) 'End septimenth4eu'
      end if  ! ( debug )
      call erclos ( 'septimenth4eu' )

      end
