      subroutine septimeulstep ( first, isol, islold, irhsdlc, &
                                 isolprec, ieqn )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 11-12-2017 New storage of intmat
!        version  1.0  date 03-11-2015
!
!   copyright (c) 2015-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Perform one time step for an Euler implicit discretization
!     One equation only
!     The mass matrix, right-hand side and the stiffness matrix must
!     have been filled before.
!     The essential boundary conditions at the new time level must have
!     been stored in the solution vector
!     Old subroutine prtimeulerstep
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     time_integration
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesolve
      use sepmodulekprob
      use sepmodulevecs
      use sepmodulematr
      use sepmoduleintmat
      use sepmoduletime
      use sepmodulecactl
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) ::  islold(*), irhsdlc, isolprec, ieqn
      integer, intent(inout) :: isol(*)
      logical, intent(in) :: first

!     first          i    If true it concerns the first call in a row,
!                         otherwise it concerns a subsequent call
!     ieqn           i    Equation sequence number
!     irhsdlc        i    Standard SEPRAN array, containing information of the
!                         large vector
!     islold         i    User input array in which the user puts information
!                         of all preceding solutions
!     isol          i/o   SEPRAN array containing information of the solution
!                         array. At input the boundary condition must have been
!                         filled in isol
!     isolprec       i    Array referring to the previous time-level
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      type (globmat), pointer :: MASS, MS
      type (rmatrix), pointer :: A
      logical :: extradebug
      type (intmatrix), pointer :: IM
      double precision, pointer :: Massm(:)

!     A              Refers to stiffness + dt * mass matrix
!     IM             Refers to intmt(intmat)
!     MASS           Refers to mass matrix
!     MS             Refers to stiffness + dt * mass matrix
!     Massm         Mass matrix
!     extradebug     If true extra debugging statements are carried out
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     SEPCOMPREAC1   Actual computing of reaction force
!     SEPEFFBOUNCD   Compute effect of boundary conditions in Au and
!                    subtract from f
!     SEPFILLPARMEUL Fill some parameters in modules for subroutine
!                    septimeulstep
!     SEPLINSOLREAL  Solve system of linear equations (real matrix)
!     SEPMATMULT     Apply matrix vector multiplication y = A * x
!     SEPPRINTMAT    Print matrix
!     SEPPRINTV      Print solution vector or vector of special structure
!     SEPRENUMSOLBCK Renumber solution vector back to original form
!                    Real case
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Differential equations to be solved:
!
!        .
!     M  u   +  S  u   =  f
!
!     Application of Euler
!
!     -Euler part (predictor):
!
!       n+1     n
!      u    -  u            n+1      n
!    M ----------- +  S  u     =  f          (n is actual time-level)
!          dt
!
!
!   Effect of boundary conditions:
!
!   We split the matrix into an internal part and a part due to
!   boundary conditions. The Euler part can then be written as:
!
!                        .
!   |  M       M   |  |  u  |      |  S       S   |  |  u  |      |  f  |
!   |   ii      ir |  |   i |      |   ii      ir |  |   i |      |   i |
!   |              |  |  .  |   +  |              |  |     |  =   |     |
!   |  M       M   |  |  u  |      |  S       S   |  |  u  |      |  f  |
!   |   ri      rr |  |   r |      |   ri      rr |  |   r |      |   r |
!
!   The last row is deleted, hence the discretized equations become:
!
!                    n+1       n+1        n+1            n            n+1   n
!   ( M /dt  + S  ) u    =  ( f    - S   u   )  + M /dt u   - M/dt ( u   - u  )
!      ii       ii   i         i      ir  r        ii    i     ir     r     r
!
!
!
!   Store boundary conditions into usol
!   Build matrix M and S
!   Subdivide M by dt
!   Build right-hand side f without the effect of boundary conditions
!      i.e. the part fi
!   rhsd2 := M un, i.e. Mii ui + Mir ur
!   rhsd2 := rhsd2 + f
!   M_help = M + S
!   help1 := M_helpir ur (n+1)
!   rhsd2  := rhsd2 - help1
!   Solve M_help isol = rhsd2
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'septimeulstep' )
      debug = .false.
      extradebug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from septimeulstep'
         write(irefwr,1) 'isolprec, massmat, stiffmat', &
                          isolprec, massmat, stiffmat
         write(irefwr,2) 'dt', dt
  1      format ( a, 1x, (10i11) )
  2      format ( a, 1x, (5d12.4) )
         if ( extradebug ) then
            write(irefwr,*) 'stiffmat'
            call sepprintmat ( stiffmat )
            write(irefwr,*) 'massmat'
            call sepprintmat ( massmat )
         end if  ! ( extradebug )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      MASS => realmt(massmat)  ! refers to mass matrix
      Massm => MASS%S_ii

      IM => intmt(MASS%intmatref)

!     --- Multiply mass matrix by 1/dt

      Massm = Massm/dt

!     --- Fill some parameters in modules for subroutine septimeulstep
!         Create solution array at new time level if it does not exist
!         Create S+M/dt, store in Mattot
!         Store old solution in uold and new solution in usol

      call sepfillparmeul ( first, isol, isolprec, ieqn, irhsdlc, islold )

!     --- Allocate rhsd2

      allocate ( rhsd2(nusol) )

!     --- determine new solution at time level t (which is in fact t+dt)
!         rhsd2 :=  M * u(old)

      call sepmatmult ( MASS, uold, rhsd2 ) ! rhsd2 :=  M * u(old)

      if ( debug ) call prinrl ( rhsd2, nrusol, 'M uold' )

      if ( Teq(ieqn)%iconsrhs/=2 ) rhsd2 = rhsd2 + rhsd

      if ( debug ) call prinrl ( rhsd2, nrusol, 'M uold + f' )

!     --- Reset mass matrix by multiplying by dt

      Massm = Massm*dt

      MS => realmt(totmat)  ! S + M/dt
      iseqmt = totmat
      A => MS%S_i           ! contents of S + M/dt

!     --- Compute effect of boundary conditions and add to right-hand side
!         help1 := MS ub (n+1), rhsd2 = rhsd2-help1

      if ( nbound>0 ) call sepeffbouncd ( MS, usol, rhsd2 )

      if ( debug ) call prinrl ( rhsd2, nrusol, 'rhsd2' )

!     --- solve system to obtain usol

      if ( IM%nested ) then
         A%Mrows => IM%rowsiiprec
         A%Mcols => IM%colsiiprec
         A%Mdiag => IM%diagprec
      end if  ! ( IM%nested )
      call seplinsolreal ( A, usol, rhsd2 )
      if ( debug ) call prinrl ( usol, nrusol, 'usol' )

      deallocate ( rhsd2 )

      if ( indprh>0 ) then

!      --- indprh > 0, renumber usol back and store information in isol(ieqn)

         usolbuf => ks(isol(ieqn))%sol
         call seprenumsolbck ( usol, 1, xglob )

      end if  ! ( indprh>0 )

      if ( parallel ) then

         deallocate ( xglob, sendwork, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'sendwork' )

      end if  ! ( parallel )
      if ( ierror/=0 ) go to 1000

      if ( debug ) call sepprintv ( isol(ieqn), 'isol' )

!     --- Compute reaction forces, store in islold(iseqreactim)

      if ( iseqreactim/=0 ) call sepcompreac1 ( usolbuf, reac, rhsd )

      if ( indprh>0 ) then

!     --- indprh > 0, deallocate arrays

         deallocate ( usol, uold )
         if ( Teq(ieqn)%iconsrhs/=2 ) deallocate(rhsd)

      end if  ! ( indprh>0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End septimeulstep'

      end if  ! ( debug )
      call erclos ( 'septimeulstep' )

      end subroutine septimeulstep
