      subroutine septimpresscr ( iinpresscor, first )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-10-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute one time step for the incompressible
!     Navier-Stokes equations
!     pressure correction is applied
!     Old subroutine prtimpresscorr
! **********************************************************************
!
!                       KEYWORDS
!
!     time_integration
!     navier_stokes_equation
!     pressure_correction
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulebuild
      use sepmoduletime
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iinpresscor(*)
      logical, intent(inout) :: first

!     first         i/o   If true it concerns the first step
!     iinpresscor    i    Integer input array for the pressure correction
!                         process
!                         See subroutine readpresscor
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: joutp, jtime
      logical :: first_step

!     first_step     If true it concerns the first time step
!     joutp          Help varaibale to store the value of ioutp
!     jtime          Help varaibale to store the value of itime
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRTIMPRESSCOR1 Compute pressure in pressure correction step and correct
!                    velocity
!     SEPPRINTV      Print solution vector or vector of special structure
!     SEPTIMCHECK    Perform some checks with respect to time integration input
!                    Fill array iinbldtim
!     SEPTIMPRESSPR  Compute the prediction step in the pressure correction
!                    equation for the incompressible Navier-Stokes equations
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The pressure correction method applied consists of the following steps
!
!     Start given p^n and u^n at previous time level
!     Set p equal to p^n
!     Compute u* by solving corrector equations, i.e. with given p and without
!     incompressibility
!     Compute pressure correction by solving Laplacian equation
!     Correct u and p to get u^(n+1) and p^(n+1)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'septimpresscr' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from septimpresscr'
         call prinin ( isolut2d, 2, 'isolut2d' )
         call prinin ( iinpresscor, 7, 'iinpresscor' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- set ioutp and itime temporarily

      joutp = ioutp
      ioutp = min(0,ioutp)
      jtime = itime
      itime = 0

!     --- Define parameters and fill arrays

      call septimcheck ( first )
      if ( ierror/=0 ) go to 1000
      if ( keeptim>1 ) t = t - tstep

!     --- Start given p^n and u^n at previous time level
!         Set p equal to p^n
!         Compute u* by solving corrector equations, i.e. with given p and
!         without
!         incompressibility

      if ( debug ) then

!     --- Debug information

         call sepprintv ( isolut2d(1), 'u^n' )
         call sepprintv ( isolut2d(2), 'p^n' )

      end if  ! ( debug )

      t = t + tstep   ! raise t, necessary for correct boundary conditions
      dt = tstep

      call septimpresspr ( first, iinpresscor, first_step )
      if ( ierror/=0 ) go to 1000

      if ( debug ) &
         call sepprintv ( isolut2d(1), 'u*' )

!     --- Compute pressure correction by solving Laplacian equation
!         Correct u and p to get u^(n+1) and p^(n+1)

      call prtimpresscor1 ( iinpresscor, tstep, ichtim, first_step )
      if ( ierror/=0 ) go to 1000
      if ( debug ) then

!     --- Debug information

         call sepprintv ( isolut2d(1), 'u^n+1' )
         call sepprintv ( isolut2d(2), 'p^n+1' )

      end if  ! ( debug )

!     --- reset ioutp and itime

      first = .false.
      ioutp = joutp
      itime = jtime

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End septimpresscr'

      end if
      call erclos ( 'septimpresscr' )

      end
