      subroutine septimrout ( first, finished )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 13-12-2017 New storage of intmat
!        version  1.0  date 28-10-2015
!
!   copyright (c) 2015-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     General time integration of a discretized partial differential equation
!     of first or second order in time
!     Old subroutine pr1000
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     time_integration
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmoduletime
      use sepmoduleinput
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical, intent(inout) :: first, finished

!     finished      i/o   Indicates if the time loop has been finished (true)
!                         or not (false)
!     first         i/o   If true it concerns the first time step
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: numarrays, iseqarrsav, iseqarrmin, iseqarrmax, iseqfr, &
                 joutp, jtime

!     iseqarrmax     Maximum value of iseqarray in loop
!     iseqarrmin     Minimum value of iseqarray in loop
!     iseqarrsav     Help parameter to store old value of iseqarray
!     iseqfr         See array iintim
!     joutp          Help variable to store the value of ioutp
!     jtime          Help variable to store the value of itime
!     numarrays      Number of indexed vectors in vector array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIREDUCEVEC   Compute the reduced value of ivecin which indicates the
!                    first sequence number of a (series of) vectors, as well as
!                    the dimension of the series
!     SEPTIMCHECK    Perform some checks with respect to time integration input
!                    Fill array iinbldtim
!     SEPTIMEULEXPL  Explicit Euler. Step-size given by the user
!     SEPTIMEURKFSS  Runge-Kutta-Fehlberg with self-selecting step-size
!     SEPTIMEUTRSS   Combination of Euler/Trapezoid rule with self-selecting
!                    step-size
!     SEPTIMNEWMARK  time integration of second order time-derivatives
!                    Step-size given by the user
!                    Newmark method
!     SEPTIMSECOND   Central differences. Step-size given by the user. Second
!                    order time-derivatives
!     SEPTIMTHETA    Time integration of a discretized partial differential
!                    equation of first order in time with an implicit
!                    modified theta method
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'septimrout' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from septimrout'
         write(irefwr,1) 'numoldtim', numoldtim
         write(irefwr,2) 't, t0, tstep, tout', t, t0, tstep, tout
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- set ioutp and itime temporarily

      joutp = ioutp
      ioutp = min(0,ioutp)
      jtime = itime
      itime = 0

!     --- Define parameters and fill arrays

      if ( keeptim>1 ) t = t - tstep

      iseqarrsav = iseqarray
      if ( iseqarray==0 ) then

!     --- iseqarray = 0, check if loop is necessary

         call inireducevec ( iseqfrtim, iseqfrtim, numarrays )
         iseqarrmin = 1
         iseqarrmax = numarrays

      else

!     --- iseqarray # 0, use iseqarray

         iseqarrmin = iseqarray
         iseqarrmax = iseqarray

      end if  ! ( iseqarray==0 )

      if ( debug ) &
         write(irefwr,1) 'iseqarrmin, iseqarrmax, iseqfr', &
                          iseqarrmin, iseqarrmax, iseqfrtim

!     --- Loop over an array of vectors

      do iseqarray = iseqarrmin, iseqarrmax

         iseqfr = iseqfrtim+iseqarray-1
         if ( .not. nocomputationtim ) Teq(1)%iseqfr = iseqfr
         massmat => massmt(iseqarray)
         stiffmat => stiffm(iseqarray:maxprb)
         totmat => mattot(iseqarray)

!        --- Check some parameters and fill arrays iinbldtim and inpsoltim

         call septimcheck ( first )
         if ( ierror/=0 ) go to 1000

!        --- Make a choice depending on ichtim

         if ( debug ) &
            write(irefwr,1) 'ichtim, iseqarray', ichtim, iseqarray
         if ( nocomputationtim ) then

!        --- Special case in which no computation must be carried out

            t = t + tstep
            if ( t>=tend-0.1*tstep ) then
               finished = .true.
            end if

         else

            select case (ichtim)

            case(1)

!           --- combination Euler / Trapezoid ; stepsize is determined by the
!               integration routine

              call septimeutrss

           case(3)

!           --- Runge-Kutta-Fehlberg with self-selecting stepsize, 1-2 order

               call septimeurkfss

            case (5, 6, 9, 10, 12 )

!           --- Implicit theta method (Euler, Crank Nicolson, fractional step)
!               Stepsize is given by user.

               call septimtheta ( finished )

            case(7)

!           --- Explicit method of Euler Stepsize is given by user

               call septimeulexpl ( finished )

            case(11)

!           --- Second order time-derivative, central differences
!               Step size selected by the user

               call septimsecond ( finished )

            case(14, 15)

!           --- Second order time-derivative, Newmark (14)
!               or generalized alpha (15)
!               Step size selected by the user

            call septimnewmark ( finished )

            case default

!           --- No time integration, problem is solved as stationary one

               t = t + tstep
               if ( t>=tend-0.1*tstep ) then
                  finished = .true.
               end if

            end select  ! case(ichtim)
            if ( ierror/=0 ) go to 1000

         end if  ! ( nocomputationtim>0 )
         if ( ierror/=0 ) go to 1000

         if ( numarrays>1 .and. iseqarray<numarrays ) then

!        --- More than one array, but not the last one

            t = t - tstep  ! reset t

         end if  ! ( numarrays>1 .and. iseqarray<numarrays )

      end do  ! iseqarray = iseqarrmin, iseqarrmax

      iseqarray = iseqarrsav  ! reset iseqarray
      if ( .not. nocomputationtim ) Teq(1)%iseqfr = iseqfr ! reset iseqfr

!     --- reset ioutp and itime

      first = .false.
      ioutp = joutp
      itime = jtime

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End septimrout'

      end if  ! ( debug )
      call erclos ( 'septimrout' )

      end subroutine septimrout
