      subroutine toaddblockmat ( elemmat, icnt )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.4  date 24-07-2015 New calls of subroutines
!        version  3.3  date 21-04-2015 Extra debug
!        version  3.2  date 11-12-2012 Long integers
!
!   copyright (c) 2008-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Add block matrices in case of block preconditioners for the
!     Navier-Stokes equations
! **********************************************************************
!
!                       KEYWORDS
!
!     assembly
!     matrix
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmoduleelm
      use sepmodulematstr
      use sepmodulebuild
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(out) :: icnt
      double precision, intent(in) :: elemmat(*)
      type (intmatrix), pointer :: IM

!     elemmat        i    Element matrix
!     icnt           o    Number of non-prescribed unknowns in element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: nu, np, nwork
      integer, allocatable :: jwork(:)

!     jwork          integer work array of size nu
!     np             Number of pressure unknowns
!     nu             Number of velocity unknowns
!     nwork          Declared length of jwork
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     TOFILLCMAT     Fill the C-matrix for the block preconditioning
!     TOFILLDMAT     Fill the matrices D and G for the SIMPLE algorithm
!     TOFILLPRESMMAT Add pressure element mass matrix to pressure mass matrix
!     TOFILLPRMASSMT Fill the Pressure mass matrix for the Schur complement
!                    algorithm
!     TOFILLQMAT     Fill the matrix Qmat for the SIMPLE algorithm
!     TOFILLVELMMAT  Add diagonal of element velocity mass matrix to
!                    global velocity mass matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'toaddblockmat' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from toaddblockmat'
         call prinin ( irowsii, 10, 'irowsii' )
  1      format ( a, 1x, (10i8) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(intmat)

!     --- jmethod = 13 to 16, i.e. special simple structure
!         Build the matrices Q and D separately

      nu = IM%nu
      np = IM%np

      if ( debug ) write(irefwr,1) 'nu, np', nu, np

      nwork = max(nu,np)

      allocate ( jwork(nwork), stat=error )
      if ( error/=0 ) call eralloc ( error, nu, 'jwork' )
      if ( ierror/=0 ) go to 1000

      jwork = 0

!     --- Fill Q-matrix

      call tofillqmat ( elemmat, jwork, icnt )

!     --- Fill D and G -matrices

      call tofilldmat ( elemmat, jwork )

!     --- Fill C-matrix

      call tofillcmat ( elemmat, jwork, icnt, nwork )
      if ( debug ) write(irefwr,*) 'after tofillcmat'

      if ( ielgrp<=nelgrp ) then

!     --- internal element
!         Fill weighted pressure mass matrix

         call tofillprmassmt ( jwork, nwork )

!        --- Fill diagonal of pressure mass matrix

         call tofillpresmmat

      end if  ! ( ielgrp>nelgrp )

      if ( debug ) write(irefwr,*) 'before deallocate'

      deallocate ( jwork, stat=error )
      if ( error/=0 ) call erdealloc ( error, 'jwork' )

!     --- Fill diagonal of velocity mass matrix

      if ( debug ) write(irefwr,1) 'icount', icount
      call tofillvelmmat

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End toaddblockmat'

      end if  ! ( debug )
      call erclos ( 'toaddblockmat' )

      end
