      subroutine tosendrecfi ( work, storeinfo1, lenwork, lenstore, amatfi )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-05-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Send and receive matrices amatfi to/from neighbor blocks
!
! **********************************************************************
!
!                       KEYWORDS
!
!     assembly
!     matrix
!     real
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematstr
      use sepmodulebuild
      use sepmodulekprob
      use sepmodulecpack
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: lenwork, lenstore
      double precision, intent(in) :: amatfi(*)
      double precision, intent(inout) :: work(lenwork)
      double precision, intent(out) :: storeinfo1(lenstore)

!     amatfi         i    Matrix part fi i.e. interface rows coupled
!                         to internal unknowns
!     lenstore       i    Declared length of array storeinfo1
!     lenwork        i    Declared length of array work
!     storeinfo1     o    Array to store all information that is received
!                         from other blocks
!                         Comparable to storeinfo in subroutine incommatpar3
!     work          i/o   Work array for transportation
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: numlow, numupp, icontext, iblock, ilow, ihigh, ninterf, &
                 istart, i
      integer, pointer :: send_buffer(:)
      integer, save :: send_buffer_s

!                    they are not
!     i              Counting variable
!     iblock         Block sequence number
!     icontext       Tag for sending and receiving, is used as check
!     ihigh          largest index of interface unknown+1
!     ilow           smallest index of interface unknown
!     istart         First position used at a part
!     ninterf        Number of unknowns in interf (interface)
!     numlow         Number of neighbor blocks with lower number
!     numupp         Number of neighbor blocks with higher number
!     send_buffer    Buffer for sending messages
!     send_buffer_s  Length of array send_buffer
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIBARRIERMPI  Synchronize parallel sending and receiving (mpi)
!     INIBSENDMPI    Send information to other node (mpi buffered)
!     INIBUFATTMPI   Attach mpi buffer if it exists
!     INIBUFDETMPI   Detach mpi buffer if it exists
!     INIBUFSIZEMPI  Extend send_buffer_size with mpi overhead
!     INIRECEIVEMPI  Receive information from other node (mpi)
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'tosendrecfi' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from tosendrecfi'
         write(irefwr,1) 'lenstore, lenwork', lenstore, lenwork
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      numlow = blockinfor(1)  ! number of neighbors with smaller number
      numupp = blockinfor(2)  ! number of neighbors with larger number

      storeinfo1 = 0d0

!     --- Synchronize communication by calling mpi_barrier
!         This avoids that something is written to other nodes, &
!         before reading by those other nodes has been done

      call inibarriermpi

!     --- Compute length of send buffer

      send_buffer_s = 0

      do i = 1, numlow

         iblock = blockinfor(2+i)      ! neighbor block
         ilow = ninterfdeg(iblock)+1   ! smallest index of interface unknown
         ihigh = ninterfdeg(iblock+1)+1 ! largest index of interface unknown+1
         ninterf = ihigh-ilow          ! number of interface unknowns in block
         length = 1
         if ( ninterf>0 ) length = irowsfi(ihigh)-irowsfi(ilow)
         send_buffer_s = send_buffer_s+length

      end do  ! i = 1, numlow

!     --- Multiply by 8 (doubles) and add 100 for safety

      send_buffer_s = send_buffer_s*8+100

!     --- Compute actual buffer size using mpi information
!         allocate array and attach buffer

      call inibufsizempi ( send_buffer_s, max(2,numlow) )

      allocate(send_buffer(send_buffer_s))

      call inibufattmpi ( send_buffer, send_buffer_s )

!     --- Send part of Sfi to blocks with smaller number

      icontext = 54
      do i = 1, numlow

         iblock = blockinfor(2+i)      ! neighbor block
         ilow = ninterfdeg(iblock)+1   ! smallest index of interface unknown
         ihigh = ninterfdeg(iblock+1)+1 ! largest index of interface unknown+1
         ninterf = ihigh-ilow          ! number of interface unknowns in block
         length = 1

         if ( debug ) &
            write(irefwr,1) 'i, iblock, ilow, ihigh, ninterf', &
                             i, iblock, ilow, ihigh, ninterf

         if ( ninterf>0 ) then

!        --- ninterf>0: there are interface unknowns

            length = irowsfi(ihigh)-irowsfi(ilow)
            istart = irowsfi(ilow)+1

            if ( debug ) &
               write(irefwr,1) 'length, istart', length, istart

            work(1:length) = amatfi(istart:istart+length-1)

            if ( debug ) then
               write(irefwr,1) 'i, iblock, length', i, iblock, length
               call prinrl ( work, length, 'work' )
            end if  ! ( debug )

!           --- Send work to iblock

            call inibsendmpi ( work, length, SPDOUBLE, iblock, &
                               icontext )

         else

!        --- Send 0 to iblock

            work(1) = 0
            call inibsendmpi ( work, length, SPDOUBLE, iblock, &
                               icontext )

         end if  ! ( ninterf>0 )

      end do  ! i = 1, numlow

      call inibarriermpi

!     --- Receive part of icolsfi from blocks with larger number

      do i = 1, numupp

         length = lenwork
         call inireceivempi ( work, length, SPDOUBLE, iblock, icontext )

         istart = storeinfo(iblock)
         if ( istart>0 ) then
            ninterf = storeinfo(istart)
            istart = istart+2*ninterf+1
            storeinfo1(istart+1:istart+length) = work(1:length)
         end if  ! ( istart>0 )

      end do  ! i = 1, numupp

      if (associated(send_buffer)) then

!     --- deattach send_buffer and deallocate

         call inibufdetmpi ( send_buffer, send_buffer_s )
         deallocate(send_buffer)

      end if  ! (associated(send_buffer))

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( storeinfo1, lenstore, 'storeinfo1' )
         write(irefwr,*) 'End tosendrecfi'

      end if  ! ( debug )
      call erclos ( 'tosendrecfi' )

      end subroutine tosendrecfi
