      subroutine ch0003 ( renum, lenprf, nbwid, irenum )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 12-06-2015 Change of parameter list
!        version  1.3  date 17-01-2007 Set lenprf=0 in case of integer overflow
!        version  1.2  date 11-10-2002 Adaptation to prevent integer overflow
!
!   copyright (c) 1987-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute size of profile for renumbered or non-renumbered mesh
!     One unknown per point is assumed
!     The matrix is supposed to be symmetrical
!     At output not the actual profile size is computed but a simplified one, &
!     that can be used to compare renumbering routines
!     The diagonal is not counted for
! **********************************************************************
!
!                       KEYWORDS
!
!     renumbering
!     reordering
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in), optional :: irenum(*)
      integer (kind=8), intent(out) :: lenprf
      integer, intent(out) :: nbwid
      logical, intent(in) :: renum

!     irenum         i    Array containing the information of the inverse of the
!                         renumbering, i.e.
!                         irenum(i) contains the new number of node i
!     lenprf         o    Computed size of the profile according to the
!                         restrictions mentioned above
!     nbwid          o    Computed band width according to the restrictions
!                         mentioned above
!     renum          i    Logical indicating if the renumbered size must be
!                         computed (true) or the original size (false)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, ineigh, inodp, maxdis, neighb
      logical :: compute

!     compute        If true lenprf must be computed, if false computation is
!                    stopped because of integer overflow
!     i              Counting variable
!     ineigh         set of new neighbours
!     inodp          Nodal point number
!     maxdis         Local band width
!     neighb         node number of neighbour
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2270   integer incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ch0003' )
      lenprf = 0
      nbwid = 0
      compute = .true.
      if ( renum ) then

!     --- renum is true;   mesh is renumbered

         do i = 1, npoint

            inodp = irenum(i)
            maxdis = 0
            do ineigh = kmshe1(i)+1, kmshe1(i+1)
               neighb = irenum(kmshe2(ineigh))
               maxdis = max ( maxdis, inodp-neighb )
            end do
            nbwid = max ( nbwid, maxdis )
            if ( compute ) lenprf = lenprf + maxdis
            if ( maxdis<0 ) then

!           --- internal error: maxdis<0

               call errint ( maxdis, 1 )
               call errchr ( 'maxdis', 1 )
               call errsub ( 2270, 1, 0, 1 )
               go to 1000

            end if

         end do

      else

!     --- renum is false;   no renumbering

         do i = 1,npoint

            maxdis = 0
            do ineigh = kmshe1(i)+1, kmshe1(i+1)
               neighb = kmshe2(ineigh)
               maxdis = max ( maxdis, i-neighb )
            end do
            nbwid = max ( nbwid, maxdis )
            if ( compute ) lenprf = lenprf + maxdis
            if ( maxdis<0 ) then

!           --- internal error: maxdis<0

               call errint ( maxdis, 1 )
               call errchr ( 'maxdis', 1 )
               call errsub ( 2270, 1, 0, 1 )
               go to 1000

            end if

         end do

      end if

1000  call erclos ( 'ch0003' )

      end
