      subroutine incm01basis ( iwork, icheck, istore )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 06-01-2016 Use sepmodulematstr
!        version  3.0  date 17-12-2015 New parameter list
!        version  2.1  date 28-06-2012 Fortran 90
!
!   copyright (c) 2007-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute first part of contact matrix, i.e. fill array INTMAT part 1.
!     For each internal row it is stored how many degrees of freedom are coupled
!     Basis subroutine performs the main part
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulematstr
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(3,*), icheck(*), istore(nusol,2)

!     icheck        i/o   work array of length nusol to indicate if a degree of
!                         is present in array istore (1) or not (0)
!                         At input icheck must be 0
!                         The output is not important
!                         ihelp(i) = 0, it does not
!     istore        i/o   work array of length nusolx2 to store the
!                         non-prescribed neighbor degrees of freedom of a point
!                         (i,1) contains the degree of freedom
!                         (i,2) contains the position of the unknown in the node
!                         At input istore must be 0
!                         At output istore is filled, but in some cases the
!                         contents must be changed
!     iwork          i    work array of length 3*npoint to indicate if a point
!                         is periodical or not
!                         If iwork(1,i) = 0 then point is not periodical
!                         For a chain of periodical points, iwork is
!                         filled as follows:
!                         iwork(1,first_node_in_chain) = sequence number in
!                         iwork(2,.)
!                         iwork(1,next_nodes in chain) = -first_node_in_chain
!                         iwork(2,.) contains for each
!                         chain of periodical points the number of points that
!                         are connected to the first point. These numbers are
!                         accumulated
!                         iwork(2,1) = 1
!                         iwork(2,i+1)-iwork(2,i) = number of points connected
!                         to nodal point i
!                         iwork(3,.) contains the nodal points
!                         connected to the first point in a chain in the
!                         sequence given by iwork(2,.)
!                         Hence iwork(3,.) positions iwork(2,i) to
!                         iwork(2,i+1)-1
!                         contain all nodes from the chain starting with node i
!                         iwork is used in subroutine incm02
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer node, nfdeg, numnei, ncoupled, icoupled, inusol, &
              ihelpunk(20), knunkp, iseqlc, imark(nunkp), inunkp, &
              nfdegmid, nrep, inext, i, &
              inunkpmin, nodp1, ilowhelp, &
              numneimid, kfree(2,nunkp)
      integer, allocatable :: ihelp(:)
      logical diffunk, extraneigh

!     diffunk        If true there are coupled nodes with boundary conditions
!                    that are not periodical
!     extraneigh     If true extra fill-in for one row of extra neighbors is
!                    required
!     i              Counting variable
!     icoupled       Number of coupled nodes that have been treated for this
!                    specific node
!     ihelp          integer work array of length npoint+1
!                    Is used to indicate if a point belongs
!                    to the set of periodical points corresponding to a
!                    specific periodic point in the following way:
!                    ihelp(i) = 1, point i corresponds to the periodic
!                    point
!     ihelpunk       Work array indicating if unknowns in coupled nodes are
!                    periodical (0) or not (1)
!     ilowhelp       Lower boundary of array ihelp
!     imark          Work array of length nunkp to indicate which degrees
!                    of freedom are coupled or decoupled per node
!                    imark(i) = 0, degree of freedom is coupled to all others
!                    imark(i) = 1, degree of freedom is coupled to all
!                                  degrees of freedom with imark(i) = 0
!     inext          Next point
!     inunkp         Number of degrees of freedom in point
!     inunkpmin      Lowest degree of freedom that must be taken into account
!     inusol         Position of unknown in array USOL
!     iseqlc           Sequence number
!     kfree          work array of length nunkpx2 to store the non-prescribed
!                    degrees of freedom in a node
!                    (1,i) contains the degree of freedom
!                    (2,i) contains the position of the unknown in the node
!     knunkp         number of degrees of freedom in node
!     ncoupled       Number of coupled nodes that are connected to this
!                    specific node including the node
!     nfdeg          Number of free degrees of freedom in a point
!     nfdegmid       Number of free degrees of freedom corresponding to the
!                    first ndim+1 unknowns
!     node           Node number
!     nodp1          Node sequence number
!     nrep           ?
!     numnei         Number of neighbors
!     numneimid      Number of standard neighbors
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INCM01CLEAR    Clear array icheck
!     INCM01HELP     if point is periodical mark all points in the chain in
!                    array ihelp
!                    Fill array ihelpunk with information about periodical nodes
!                    Compute diffunk and ncoupled
!     INCM01IMAT1    Fill array irowsii
!     INCM01IMAT2    Fill array irowsii
!                    Special case of decoupled unknowns
!     INCM01NEIGH    consider the neighbors of node
!                    put real degrees of freedom in istore
!                    Mark those one used in icheck
!                    Repeat for periodical node
!     INCM01NODE     put real degrees of freedom of nodal point node into kfree
!                    Mark unknowns in icheck
!     INCM01PERIOD   Adapt icheck, kfree and istore for periodical nodes
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For node = 1 (1) NUMBER OF NODES do
!        If ( iwork(1,node)>=0 ) then
!           Store free unknows in array kfree
!           If ( there are free unknowns in node ) then
!              If ( node is not periodical ) then
!                 Store all free unknowns coupled to point in ISTORE
!              else (node is periodical) then
!                 Store all free unknowns coupled to point and to corresponding
!                 periodical nodal points in ISTORE
!              Store number of free unknowns in irowsii
!              Raise nblocks
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'incm01basis' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from incm01basis'
  1      format ( a, 1x, (10i6) )
         write(irefwr,1) 'npoint, nrusol, ndim, nunkp', &
                          npoint, nrusol, ndim, nunkp

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( ihelp(0:npoint), stat=error )
      if ( error/=0 ) call eralloc ( error, npoint+1, 'ihelp' )
      if ( ierror/=0 ) go to 1000

      ihelp = 0
      ilowhelp = 0

      inunkpmin = 1
      diffunk = .false.
      nfdegmid = 0

!     --- Fill array imark
!         degrees of freedom that are decoupled get value 1, all others 0

      imark = 0
      imark(decoupled(1:numdec)) = 1

      if ( extrafillin<=3 ) then

!     --- Standard situation with respect to fill-in

         extraneigh = mod(extrafillin/2,2)==1

      else

         extraneigh = .true.
         if ( extrafillin==7 ) inunkpmin = ndim+2

      end if
      if ( debug ) then
         write(irefwr,1) 'inunkpmin, ndim, extrafillin', &
                          inunkpmin, ndim, extrafillin
         write(irefwr,*) 'extraneigh ', extraneigh
         if ( indprf>0 ) call prinin ( kprobf, npoint+1, 'kprobf' )
         if ( indprr>0 ) call prinin ( kprobr, nusol, 'kprobr' )
      end if  ! ( debug )

!     --- Loop over all nodes

      do node = 1, npoint

         nfdeg = 0
         numnei = 0
         if ( debug ) &
            write(irefwr,1) 'node, iwork(1,node)', node, iwork(1,node)

         if ( iwork(1,node)>=0 ) then

!        --- Point is not a periodical node, or is the basis point of a chain
!            if point is periodical mark all points in the chain in array ihelp
!            Fill array ihelpunk with information about periodical nodes
!            Compute diffunk, iseqlc, inusol, icoupled and ncoupled

            call incm01help ( iwork, node, ihelp, icoupled, ncoupled, &
                              knunkp, ihelpunk, diffunk, &
                              iseqlc, inusol, ilowhelp, inunkp )

!           --- put real degrees of freedom of nodal point node into kfree
!               Mark unknowns in icheck

            call incm01node ( kfree, icheck, nfdeg, nfdegmid, &
                              inusol, istore, numnei, ihelpunk, inunkp )

         end if  ! ( iwork(1,node)>=0 )
         if ( debug ) write(irefwr,1) 'nfdeg, nfdegmid', nfdeg, nfdegmid

!        --- Find neighbor unknowns of free degrees of freedom

         if ( nfdeg==0 ) go to 500

!        --- nfdeg>0, so there are nfdeg free unknowns in node

         if ( iwork(1,node)==0 ) then

!        --- node is not periodical

            nrep  = 1

         else

            nrep = iwork(2,iseqlc+1) - iwork(2,iseqlc) + 1
            inext = iwork(2,iseqlc)
            if ( debug ) write(irefwr,1) 'nrep, inext', nrep, inext

         end if

         nodp1  = node

!        --- consider the neighbors of node
!            put real degrees of freedom in istore
!            Mark those ones used in icheck
!            Repeat for periodical node

250      call incm01neigh ( kfree, ihelp, nodp1, &
                            icheck, numnei, istore, nrep, diffunk, &
                            inext, iwork, extraneigh, &
                            knunkp, numneimid, inunkpmin, &
                            ihelpunk, ilowhelp )
         if ( debug ) then

            write(irefwr,1) 'nfdeg, nfdegmid, numnei, numneimid', &
                             nfdeg, nfdegmid, numnei, numneimid
            call prinin1 ( kfree, nfdeg, 2, 'kfree')

         end if  ! ( debug )

!        --- Fill array irowsii

         if ( numdec==0 ) then

!        --- Standard case: all degrees of freedom per node are coupled

            call incm01imat1 ( nfdegmid, kfree, nfdeg, &
                               numnei, istore, numneimid )

         else

!        --- Special case: not all degrees of freedom per node are coupled

            call incm01imat2 ( nfdegmid, kfree, nfdeg, &
                               numnei, istore, numneimid, imark )

         end if  ! ( numdec==0 )
         if ( debug ) call prinin ( irowsii, nrusol+1, 'irowsii/1' )

!        --- Clear icheck

         call incm01clear ( numnei, icheck, istore, kfree, nfdeg, &
                            diffunk, node, nodp1, iwork, iseqlc )

!        --- Reset ihelp

500      if ( iwork(1,node)>0 ) then

!        --- Basis point of chain

            do i = iwork(2,iseqlc), iwork(2,iseqlc+1) - 1
               ihelp(iwork(3,i)) = 0
            end do
            ihelp(node) = 0

         end if

         if ( debug ) then
            write(irefwr,*) 'diffunk', diffunk
            write(irefwr,1) 'icoupled, ncoupled', icoupled, ncoupled
         end if  ! ( debug )

550      if (  diffunk .and. icoupled<ncoupled ) then

!        --- The node is periodical with degrees of freedom that are not
!            coupled
!            Loop over all coupled nodes for the non-coupled degrees of freedom
!            First mark all unknowns corresponding to the other points
!            in the chain

            call incm01period ( node, nodp1, icheck, iwork, icoupled, &
                                nfdeg, numnei, kfree, iseqlc, &
                                istore, nrep, ihelpunk )

            if ( nfdeg==0 ) then

!           --- No free degrees of freedom, skip

               go to 550

            else

!           --- Free degrees of freedom, repeat

               go to 250

            end if

         end if

!        --- Clear icheck

         icheck( istore(1:numnei,1) ) = 0
         icheck( kfree(1,1:nfdeg) ) = 0

      end do  ! node = 1, npoint

      deallocate ( ihelp, stat=error )
      if ( error/=0 ) call erdealloc ( error, 'ihelp' )

1000  if ( debug ) then

!     --- Debug information

         call prinin ( irowsii, nrusol+1, 'irowsii' )
         write(irefwr,*) 'End incm01basis'

      end if  ! ( debug )
      call erclos ( 'incm01basis' )

      end
