      subroutine incm01help ( iwork, node, ihelp, icoupled, ncoupled, &
                              knunkp, ihelpunk, diffunk, iseqlc, &
                              inusol, ilowhelp, inunkp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-01-2016 Use sepmodulematstr
!        version  2.0  date 17-12-2015 New parameter list
!        version  1.0  date 12-10-2007
!
!   copyright (c) 2007-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     if point is periodical mark all points in the chain in array ihelp
!     Fill array ihelpunk with information about periodical nodes
!     Compute diffunk and ncoupled
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematstr
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(3,*), node, ilowhelp, icoupled, ncoupled, inunkp, &
              knunkp, ihelpunk(*), iseqlc, inusol, ihelp(ilowhelp:*)
      logical diffunk

!     diffunk        o    If true there are coupled nodes with boundary
!                         conditions that are not periodical
!     icoupled       o    If 0 the momentum equations are coupled, if 1 they
!                         are not
!     ihelp          o    integer work array of length max(1+nrusol,npoint+1)
!                         In the case of periodical boundary conditions it is
!                         used as work array to compute the chain
!                         Futhermore it is used to indicate if a point belongs
!                         to the set of periodical points corresponding to a
!                         specific periodic point in the following way:
!                         ihelp(i) = 1, point i corresponds to the periodic
!                         point
!                         ihelp(i) = 0, it does not
!     ihelpunk       o    Work array indicating if unknowns in coupled nodes are
!                         periodical (0) or not (1)
!     ilowhelp       i    Lower boundary of array ihelp
!     inunkp         i    Number of degrees of freedom in point
!     inusol         o    Position of first unknown of node in array USOL (-1)
!     iseqlc         o    Sequence number in chain
!     iwork          i    work array of length 3*npoint to indicate if a point
!                         is periodical or not
!                         See subroutine incm01basis
!     knunkp         o    number of degrees of freedom in node
!     ncoupled       o    Number of coupled nodes that are connected to this
!                         specific node including the node
!     node           i    Node number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, jnunkp, ineigh, jnusol, knusol, j

!     i              Counting variable
!     ineigh         neighbor
!     j              Counting variable
!     jnunkp         Number of degrees of freedom in point
!     jnusol         Position of unknown in array USOL
!     knusol         Position of unknown in array USOL
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2034   Number of unknowns in periodical nodes not the same
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'incm01help' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from incm01help'
         write(irefwr,1) 'node', node
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( iwork(1,node)>0 ) then

!     --- Node is basis point of a chain
!         Mark all points in the chain in array ihelp

         iseqlc = iwork(1,node)
         do i = iwork(2,iseqlc), iwork(2,iseqlc+1) - 1
            ihelp(iwork(3,i)) = 1
         end do
         ihelp(node) = 1
         ncoupled = iwork(2,iseqlc+1)-iwork(2,iseqlc)+1
         icoupled = 0

      end if  ! ( iwork(1,node)>0 )

      inusol = sepgetinusol ( node, inunkp )
      knunkp = inunkp
      if ( debug ) &
         write(irefwr,1) 'inusol, inunkp', inusol, inunkp

!     --- Initialize parameters for the case of periodical nodes, with
!         degrees of freedom which are not all coupled

      do i = 1, inunkp
         ihelpunk(i) = 0
      end do
      if ( iwork(1,node)>0 .and. inunkp>1 ) then

!     --- iwork(1,node)>0, hence periodical node
!         inunkp>1, hence the number of unknowns per point is more
!         than 1
!         Check if all free degrees of freedom in the chain of nodes
!         have the same sequence number
!         If not this means that some are not coupled
!         Loop over all other nodes in the chain

         jnunkp = inunkp
         do j = iwork(2,iseqlc), iwork(2,iseqlc+1)-1
            ineigh = iwork(3,j)

!           --- Loop over all other unknowns in the neihbour node

            do i = 1, inunkp

               jnusol = sepgetinusol ( ineigh, jnunkp )
               if ( jnunkp/=inunkp ) then

!              --- Not yet implemented

                  call errint ( node, 1 )
                  call errint ( inunkp, 2 )
                  call errint ( ineigh, 3 )
                  call errint ( jnunkp, 4 )
                  call errsub ( 2034, 4, 0, 0 )

               end if
               knusol = kprobr(inusol+i)
               if ( knusol<=nrusol ) then

!              --- Free unknown in basis point found
!                  Check if unknown in other points has the same sequence
!                  number

                  if ( knusol/=kprobr(jnusol+i) ) then

!                 --- Sequence number is different, store in ihelpunk

                     ihelpunk(i) = 1
                     diffunk = .true.

                  end if

               else

!              --- Prescribed unknown in basis point found
!                  Check if unknown in other points are also prescribed

                  if ( kprobr(jnusol+i)<=nrusol ) then

!                 --- Unknown in neighbouring node is not prescribed
!                     Store in ihelpunk

                     ihelpunk(i) = 1
                     diffunk = .true.

                  end if

               end if

            end do

         end do

      end if  ! ( iwork(1,node)>0 .and. inunkp>1 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'diffunk ', diffunk
         write(irefwr,1) 'ncoupled ', ncoupled
         write(irefwr,*) 'End incm01help'

      end if  ! ( debug )
      call erclos ( 'incm01help' )

      end
