      subroutine incommatextend
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-01-2016 Use sepmodulematstr
!        version  2.0  date 17-12-2015 New parameter list
!
!   copyright (c) 2015-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Update the integer description of S_ii in the parallel case if
!     necessary
!     Check if the internal matrices should be adapted because
!     there are elements in other blocks that make connections in this
!     block
!     This is for example the case if an element of 4 nodes is completely
!     outside the block but 3 of the nodes are inside
!     The connection between diagonal nodes might not be recognized
!     In first instance we fill array irowsiiext, thus checking if this
!     situation occurs
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulecpack
      use sepmodulematstr
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, allocatable :: irowsiiext(:), iwork(:), &
                              irowsiiold(:) , icolsiiold(:)
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     icolsiiold     starting address of icolsiiold
!     irowsiiext     contains the number of extra elements required in
!                    the matrix S_ii per row accumulated
!     irowsiiold     old version of irowsii
!     iwork          integer work array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INCOMMATDIAG   Store position of diagonal elements of array icolsii into
!                    intdia in case of row-compact storag
!     INCOMMATPAR11  Fill array irowsiiext with number of extra elements per
!                    row accumulated
!     INCOMMATPAR12  Fill new arrays irowsii and icolsii in case extar elements
!                    are required in the parallel case
!     SEPSORTCOLS    Sort column numbers per row of internal matri
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'incommatextend' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from incommatextend'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(intmat)

!     --- Get parameters

      allocate ( irowsiiext(0:nrusol),  iwork(nrusol), stat = error )
      if ( error/=0 ) call eralloc ( error, 2*nrusol, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Check if the internal matrices should be adapted because
!         there are elements in other blocks that make connections in this
!         block
!         This is for example the case if an element of 4 nodes is completely
!         outside the block but 3 of the nodes are inside
!         The connection between diagonal nodes might not be recognized
!         In first instance we fill array irowsiiext, thus checking if this
!         situation occurs


      call incommatpar11 ( iwork, irowsiiext )

      if ( irowsiiext(nrusol)>0 ) then

!     --- irowsiiext(nrusol)>0, hence extra elements must be added to
!         matrix S_ii
!         Copy old irowsii and icolsii in temporary array
!         and extend icolsii

         if ( associated(icolsii) ) then
            length = max(size(icolsii), 1)
         else
            length = 1
         end if  ! ( associated(icolsii) )
         allocate ( irowsiiold(0:nrusol), icolsiiold(length), &
                    stat = error )
         if ( error/=0 ) &
            call eralloc ( error, nrusol+length, 'irowsiiold' )
         if ( ierror/=0 ) go to 1000

         irowsiiold(0:nrusol) = irowsii(1:nrusol+1)
         if ( length>1 ) icolsiiold(1:length) = icolsii(1:length)

         if ( length==1 ) length = 0
         length = length+irowsiiext(nrusol)  ! new length icolsii

!        --- Delete old array icolsii and create new one

         if ( associated(icolsii) ) deallocate ( icolsii, stat = error )

         if ( length>0 ) then

            allocate ( icolsii(length), stat = error )
            if ( error/=0 ) call eralloc ( error, length, 'isolsii' )
            if ( ierror/=0 ) go to 1000

         end if  ! ( length>0 )
         call incommatpar12 ( iwork, irowsiiold, &
                              icolsiiold, irowsiiext )

!        --- Sort columns per row in increasing sequence

         call sepsortcols

!        --- In case of row-compact storage fill intdia with positions
!            of diagonal elements in icolsii

         if ( debug ) write(irefwr,*) 'before incommatdiag'
         call incommatdiag

!        --- Remove temporary arrays

         deallocate ( irowsiiold, icolsiiold, stat = error )
         if ( error/=0 ) call erdealloc ( error, '' )

      end if  ! ( irowsiiext(nrusol)>0 )

      deallocate ( iwork, irowsiiext, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End incommatextend'

      end if  ! ( debug )
      call erclos ( 'incommatextend' )

      end subroutine incommatextend
