      subroutine incommatpar4 ( leniwork )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-01-2016 Use sepmodulematstr
!        version  2.0  date 17-12-2015 New parameter list
!
!   copyright (c) 2015-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     send information of icolsfi to all neighbor blocks with
!     lower number and receive from neighbors with higher number
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematstr
      use sepmodulekprob
      use sepmodulecpack
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: leniwork

!     leniwork       i    Computed length of array iwork
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: numlow, numupp, ilow, ihigh, ninterf, i, iblock, &
                 icontext, istart, iwork(leniwork)
      integer, pointer :: send_buffer(:)
      integer, save :: send_buffer_s

!     i              Counting variable
!     iblock         Block sequence number
!     icontext       Tag for sending and receiving, is used as check
!     ihigh          largest index of interface unknown+1
!     ilow           smallest index of interface unknown
!     istart         First position used at a part
!     iwork          integer work array
!     ninterf        Number of unknowns in interf (interface)
!     numlow         Number of neighbor blocks with lower number
!     numupp         Number of neighbor blocks with higher number
!     send_buffer    Buffer for sending messages
!     send_buffer_s  Length of array send_buffer
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIBARRIERMPI  Synchronize parallel sending and receiving (mpi)
!     INIBSENDMPI    Send information to other node (mpi buffered)
!     INIBUFATTMPI   Attach mpi buffer if it exists
!     INIBUFDETMPI   Detach mpi buffer if it exists
!     INIBUFSIZEMPI  Extend send_buffer_size with mpi overhead
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     INIRECEIVEMPI  Receive information from other node (mpi)
!     PRININ         print 1d integer array
!     PRININTMAT1    Print array storeinfo
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'incommatpar4' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from incommatpar4'
         write(irefwr,1) 'inodenr', inodenr
         if ( ninterface>0 ) then
            call prinin ( irowsfi, ninterface, 'irowsfi' )
            length = irowsfi(ninterface)
            call prinin ( icolsfi, length, 'icolsfi' )
         end if  ! ( ninterface>0 )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Synchronize communication by calling mpi_barrier
!         This avoids that something is written to other nodes, &
!         before reading by those other nodes has been done

      call inibarriermpi

      numlow = blockinfor(1)  ! number of neighbors with smaller number
      numupp = blockinfor(2)  ! number of neighbors with larger number

      if ( debug ) write(irefwr,1) 'numlow, numupp', numlow, numupp

!     --- Compute length of send_buffer

      send_buffer_s = 0
      do i = 1, numlow

         iblock = blockinfor(2+i)      ! neighbor block
         ilow = ninterfdeg(iblock)     ! smallest index of interface unknown
         ihigh = ninterfdeg(iblock+1)  ! largest index of interface unknown+1
         ninterf = ihigh-ilow          ! number of interface unknowns in block
         length = 1
         if ( ninterf>0 ) length = irowsfi(ihigh)-irowsfi(ilow)
         call inichkarlen ( length, leniwork, 'iwork' )
         send_buffer_s = send_buffer_s+length

      end do  ! i = 1, numlow

!     --- Multiply by 4 (integers) and add 100 for safety

      send_buffer_s = send_buffer_s*4+100

!     --- Compute actual buffer size using mpi information
!         allocate array and attach buffer

      call inibufsizempi ( send_buffer_s, max(2,numlow) )

      allocate(send_buffer(send_buffer_s))

      call inibufattmpi ( send_buffer, send_buffer_s )
      if ( ierror/=0 ) go to 1000

!     --- Send part of icolsfi to blocks with smaller number

      icontext = 53
      do i = 1, numlow

         iblock = blockinfor(2+i)      ! neighbor block
         ilow = ninterfdeg(iblock)     ! smallest index of interface unknown
         ihigh = ninterfdeg(iblock+1)  ! largest index of interface unknown+1
         ninterf = ihigh-ilow          ! number of interface unknowns in block
         length = 1

         if ( debug ) &
            write(irefwr,1) 'i, iblock, ilow, ihigh, ninterf', &
                             i, iblock, ilow, ihigh, ninterf

         if ( ninterf>0 ) then

!        --- ninterf>0: there are interface unknowns

            length = irowsfi(ihigh)-irowsfi(ilow)
            istart = irowsfi(ilow)+1

            if ( debug ) &
               write(irefwr,1) 'length, istart', length, istart

            iwork(1:length) = icolsfi(istart:istart+length-1)

            if ( debug ) call prinin ( iwork, length, 'iwork' )

!           --- Translate column numbers to global numbers

            iwork(1:length) = kprobhglob(iwork(1:length))

            if ( debug ) then
               write(irefwr,1) 'i, iblock, length', i, iblock, length
               call prinin ( iwork, length, 'iwork' )
            end if  ! ( debug )

!           --- Send iwork to iblock

            call inibsendmpi ( iwork, length, SPINT, iblock, icontext )

         else

!        --- Send 0 to iblock

            iwork(1) = 0
            call inibsendmpi ( iwork, length, SPINT, iblock, icontext )

         end if  ! ( ninterf>0 )

      end do  ! i = 1, numlow

      call inibarriermpi

!     --- Receive part of icolsfi from blocks with larger number

      do i = 1, numupp

         length = leniwork
         call inireceivempi ( iwork, length, SPINT, iblock, icontext )

         istart = storeinfo(iblock)
         if ( istart>0 ) then
            ninterf = storeinfo(istart)
            istart = istart+2*ninterf+1
            storeinfo(istart+1:istart+length) = iwork(1:length)
         end if  ! ( istart>0 )

      end do  ! i = 1, numupp

      if (associated(send_buffer)) then

!     --- deattach send_buffer and deallocate

         call inibufdetmpi ( send_buffer, send_buffer_s )
         deallocate(send_buffer)

      end if  ! (associated(send_buffer))

      call inibarriermpi

1000  if ( debug ) then

!     --- Debug information

         call prinintmat1 ( storeinfo )
         write(irefwr,*) 'End incommatpar4'

      end if  ! ( debug )
      call erclos ( 'incommatpar4' )

      end subroutine incommatpar4
