      subroutine init07comcrv ( iinp, ielshape, natcomcrv )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-03-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill number of nodes per element and element shapes for
!     boundary groups of type complete curve in array ielshape
!     New input
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_element
!     element_shape
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinp, ielshape(2,*), natcomcrv

!     ielshape      i/o   Array of length 2*(nelgrp+numnat) containing:
!                         (1,i) number of nodes in element group i
!                         (2,i) shape number of elements in element group i
!                         if i>nelgrp: i-nelgrp = boundary group number
!                         otherwise it is the element group number
!     iinp          i/o   last position used in array iinputkp
!     natcomcrv      i    number of parts where complete_curve elements are
!                         defined for natural boundary conditions
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer j, ibngrp, inum, inpelm, nparts
      integer, allocatable :: iwork(:), ipoint(:)

!     ibngrp         Actual boundary element group number
!     inpelm         number of points per element
!     inum           Number of curves
!     ipoint         Array of length nparts+1 containing the starting
!                    addresses in array iwork of the various parts
!                    the boundary consists of.
!                    ipoint(1) = 1, &
!                    number of nodes at part i = ipoint(i+1)-ipoint(i)
!     iwork          integer work array of length npoint to store
!                    the nodes along the curves
!     j              Counting variable
!     nparts         number of parts the boundary consists of
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     SEPCURVENODES  Store all nodes in one curves in array inodes and gaps in
!                    these curves in array ipoint
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     271   boundary element group number has wrong value
!    1083   Shape number for boundary group different from before
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'init07comcrv' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from init07comcrv'
         write(irefwr,1) 'natcomcrv, iinp', natcomcrv, iinp
         call prinin ( iinputkp(iinp), 4, 'iinputkp' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do j = 1, natcomcrv

         ibngrp = iinputkp(iinp+1)
         if ( ibngrp<=0 .or. ibngrp>numnat ) then
            call errint ( ibngrp, 1 )
            call errint ( numnat, 2 )
            call errint ( j, 3 )
            call errchr ( 'complete_curve', 1 )
            call errsub ( 271, 3, 0, 1 )
         end if
         if ( debug ) &
            write(irefwr,1) 'j, ibngrp, iinp', j, ibngrp, iinp
         ishape = 0
         inum = iinputkp(iinp+2)

!        --- Find nodes of curves and store in iwork

         allocate ( iwork(npoint), ipoint(ncurvs+1), stat = error )
         if ( error/=0 ) call eralloc ( error, npoint, 'iwork' )
         if ( ierror/=0 ) go to 1000

         call sepcurvenodes ( iinputkp(iinp+2), ipoint, iwork, nparts )
         if ( debug ) call prinin ( iwork, inpelm, 'iwork' )
         inpelm = ipoint(nparts+1)

         deallocate ( iwork, ipoint, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'iwork' )

         if ( ielshape(2,ibngrp+nelgrp)/=0 ) then

!        --- check if shapes coincide

            if ( ielshape(2,ibngrp+nelgrp)/=ishape ) then
               call errint ( ibngrp, 1 )
               call errint ( ishape, 2 )
               call errint ( ielshape(2,ibngrp+nelgrp), 3 )
               call errint ( j, 4 )
               call errchr ( 'complete_curve', 1 )
               call errsub ( 1083, 4, 0, 1 )
            end if

         end if
         ielshape(1,ibngrp+nelgrp) = inpelm
         ielshape(2,ibngrp+nelgrp) = ishape
         iinp = iinp+2+inum

      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End init07comcrv'

      end if  ! ( debug )
      call erclos ( 'init07comcrv' )

      end
