      subroutine prob01crobst ( kcrosselem, rcrosselem, nintersect, &
                                iinp, ikprbi, iposgrp, natobstcross, iworkcross )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.2  date 24-10-2012 New call to princrossobst
!        version  3.1  date 15-03-2012 New input
!        version  3.0  date 27-09-2010 Make work arrays local
!
!   copyright (c) 2000-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Determine the elements corresponding to cross sections with an
!    obstacle
! **********************************************************************
!
!                       KEYWORDS
!
!     fictitious_domain
!     element
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      use sepmoduledebug
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kcrosselem(2,*), nintersect, &
              iinp, ikprbi, iposgrp(*), natobstcross, iworkcross(*)
      double precision rcrosselem(*)

!     iinp          i/o   last position used in array iinputkp
!     ikprbi        i/o   last position used in array kprob part i
!     iposgrp       i/o   work array of length numnat in which the starting
!                         addresses of the element groups in KPROB part i are
!                         stored
!                         In this subroutine the length with respect to an
!                         element group is added to the existing array
!     iworkcross     o    integer work array of length natobstcross that
!                         contains the number of elements per set of boundary
!                         elements corresponding to the intersections
!     kcrosselem     o    array of size nfictunk x 2 with the following contents
!                         kcrosselem(i,1) contains the first node of the element
!                         kcrosselem(i,2) contains the second node of the
!                         element
!     natobstcross   i    number of parts where cross_section_obstacle elements
!                         are defined
!     nintersect     o    Total number of points with fictitious unknowns
!     rcrosselem     o    Contains the factors defining the intersection point
!                         per element
!                         The intersection point is defined by x1 + fac *(x2-x1)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iedge, igrp, iobst, ipos, istart, i, j, istar3, k1, k2, &
              istarticross, itypeexcl, inum, nelemlc, &
              node1, node2, numbndobst, numinnobst, icurnr, &
              numintersect, numnodes, numpntobst, ibngrp, itypeskip, &
              nintersectbef
      integer, allocatable :: iwork(:), iwork1(:)
      logical debugextra

!     debugextra     Second debug parameter for extra output
!     i              Counting variable
!     ibngrp         Actual boundary element group number
!     icurnr         Curve sequence number
!     iedge          edge sequence number
!     igrp           Boundary element group number read
!     inum           Number of curves
!     iobst          Obstacle sequence number
!     ipos           Position in array
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     istart         First position used at a part
!     istarticross   Starting address of obstacle in array icrossobst
!     itypeexcl      Defines how intersection faces are excluded
!                    Possible values:
!                    1: the face is not used as an element if the node outside
!                       the object is already part of an ficitious element
!                    2: the face is not used as an element if one of the nodes
!                       is already part of an ficitious element
!     itypeskip      Defines if boundaries are skipped
!                    0: do not skip
!                    1: skip given curves
!     iwork          integer work array of length npoint
!                    iwork(i) = 1, means that node i is already used in
!                    a ficitious domain element, &
!                    iwork(i) = 0, means not yet used
!     iwork1         integer work array of length npoint
!                    iwork1(i) = 1, means that node i is inside the
!                    obstacle, iwork1(i) = 0 not
!     j              Counting variable
!     k1             First point of curve in KMESH part m
!     k2             Last point of curve in KMESH part m
!     nelemlc        Local number of elements
!     nintersectbef  Previous value of nintersect
!     node1          Node of face
!     node2          Node of face
!     numbndobst     Number of nodal points on the boundary of the obstacle
!     numinnobst     Number of points that are near the boundary of the obstacle
!     numintersect   Number of intersections
!     numnodes       Number of nodla points in array nodes
!     numpntobst     Number of nodal points inside the obstacle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
!     PRINRL         Print 1d real vector
!     SEPPRCROSSOBST Print the contents of the arrays icrossobst and rcrossobst
!     SEPPRINIPOBST  Print the contents of array ipointobst
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     nintersect := 0
!     Clear arrays iwork and iwork1
!     For igrp := 1 (1) natobstcross
!        iobst := obstacle number
!        itypeexcl := type of exclusion mechanism
!        if ( itypeexcl=1 ) then
!           Set iwork1 for all nodes in obstacle
!           For all intersection faces of the obstacle do
!              node1 and node2 are nodes of face
!              if ( node1 outside obstacle ) then
!                 if ( iwork(node1) = 0 ) then
!                    iwork(node1) := 1
!                    add element to array kcrosselem
!                    nintersect := nintersect+1
!              else ( node2 outside obstacle )
!                 if ( iwork(node2) = 0 ) then
!                    iwork(node2) := 1
!                    add element to array kcrosselem
!                    nintersect := nintersect+1
!           Clear iwork1
!        else ( itypeexcl=2 )
!           For all intersection faces of the obstacle do
!              node1 and node2 are nodes of face
!              if ( iwork(node1) = 0 and iwork(node2) = 0 ) then
!                 iwork(node1) := 1
!                 iwork(node2) := 1
!                 add element to array kcrosselem
!                 nintersect := nintersect+1
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob01crobst' )
      debug = .false.
      debugextra = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob01crobst'
         call sepprcrossobst
         call sepprinipobst ( ipointobst )
  1      format ( a, 1x, (10i6) )

      end if

!     --- nintersect := 0

      allocate ( iwork(npoint), iwork1(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, 2*npoint, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Clear arrays iwork and iwork1

      iwork = 0
      iwork1 = 0

      nelemlc = 0

!     --- For igrp := 1 (1) natobstcross

      nintersect = 0
      do igrp = 1, natobstcross

!     --- iobst := obstacle number
!         itypeexcl := type of exclusion mechanism

         ibngrp    = iinputkp(iinp+1)
         iobst     = iinputkp(iinp+2)
         itypeexcl = iinputkp(iinp+3)
         itypeskip = iinputkp(iinp+4)
         inum      = iinputkp(iinp+5)
         nintersectbef = nintersect
         if ( debug ) &
            write(irefwr,1) 'ibngrp, iobst, itypeexcl, itypeskip, inum', &
                             ibngrp, iobst, itypeexcl, itypeskip, inum

!        --- Set iwork1 for all nodes in obstacle

         istart = ipointobst(iobst)
         numpntobst = ipointobst(istart)
         numinnobst = ipointobst(istart+1)
         numbndobst = ipointobst(istart+2)
         istart = istart+3
         do i = 1, numinnobst+numbndobst+numpntobst
            iwork1(ipointobst(istart+i)) = 1
         end do

         if ( debug ) then

!        --- Debug information

            write(irefwr,1)'igrp, ibngrp, iobst, itypeexcl, itypeskip', &
                            igrp, ibngrp, iobst, itypeexcl, itypeskip
            write(irefwr,1) 'inum, istart', inum, istart
            write(irefwr,1) 'numpntobst, numinnobst,  numbndobst', &
                             numpntobst, numinnobst,  numbndobst
            if ( debugextra ) call prinin ( iwork1, npoint, 'iwork1' )

         end if

         if ( itypeskip==1 ) then

!        --- itypeskip = 1, curves ifirst to ilast may not be part of the
!            cross-section elements
!            In order to fullfill this demand we make iwork(i) = 1 for
!            all nodes on these curves

            istar3 = kmeshm(2)-1
            do i = 1, inum

               icurnr = iinputkp(iinp+5+i)
               k1 = kmeshm(4+icurnr)
               k2 = kmeshm(5+icurnr)-1
               do j = k1, k2
                  iwork(kmeshm(istar3+j)) = 1
               end do

            end do

         end if
         if ( debugextra ) call prinin ( iwork, npoint, 'iwork' )

         if ( itypeexcl==1 ) then

!        --- itypeexcl=1
!            For all intersection faces of the obstacle do

            istarticross = icrossobst(iobst)
            numintersect = icrossobst(istarticross)
            if ( debug ) write(irefwr,*) 'istarticross, numintersect ', &
               istarticross, numintersect
            do i = 1, numintersect

!           --- node1 and node2 are nodes of face

               iedge = icrossobst(istarticross+1)
               ipos  = icrossobst(istarticross+2)
               istarticross = istarticross+2
               istart = nmeshc(iedge)
               numnodes = nmeshc(istart)
               node1 = nmeshc(istart+1)
               node2 = nmeshc(istart+numnodes)
               if ( debug ) then

!              --- Debug information

                  write(irefwr,*) 'i, iedge, numnodes, node1, node2 ', &
                                   i, iedge, numnodes, node1, node2
                  write(irefwr,*) 'iwork(node1), iwork(node2) ', &
                                   iwork(node1), iwork(node2)

               end if

               if ( iwork1(node1)==0 ) then

!              --- node1 outside obstacle

                  if ( iwork(node1)==0 ) then

!                 --- iwork(node1) = 0
!                     iwork(node1) := 1
!                     add element to array kcrosselem
!                     nintersect := nintersect+1

                     iwork(node1) = 1
                     nintersect = nintersect+1
                     kcrosselem(1,nintersect) = node1
                     kcrosselem(2,nintersect) = node2
                     rcrosselem(nintersect) = rcrossobst(ipos)

                  end if

               else if ( iwork1(node2)==0 ) then

!              --- node2 outside obstacle

                  if ( iwork(node2)==0 ) then

!                 --- iwork(node2) = 0
!                     iwork(node2) := 1
!                     add element to array kcrosselem
!                     nintersect := nintersect+1

                     iwork(node2) = 1
                     nintersect = nintersect+1
                     kcrosselem(1,nintersect) = node2
                     kcrosselem(2,nintersect) = node1
                     rcrosselem(nintersect) = 1d0-rcrossobst(ipos)

                  end if

               end if

            end do

         else

!        --- itypeexcl=2
!            For all intersection faces of the obstacle do

            istarticross = icrossobst(iobst)
            numintersect = icrossobst(istarticross)

            if ( debug ) write(irefwr,*) 'istarticross, numintersect ', &
               istarticross, numintersect

            do i = 1, numintersect

!           --- node1 and node2 are nodes of face

               iedge = icrossobst(istarticross+1)
               ipos  = icrossobst(istarticross+2)
               istarticross = istarticross+2
               istart = nmeshc(iedge)
               numnodes = nmeshc(istart)
               node1 = nmeshc(istart+1)
               node2 = nmeshc(istart+numnodes)
               if ( debug ) then
                  write(irefwr,*) 'i, iedge, node1, node2 ', &
                                   i, iedge, node1, node2
                  write(irefwr,*) 'iwork(node1), iwork(node2) ', &
                                   iwork(node1), iwork(node2)
               end if  ! ( debug )

               if ( iwork(node1)==0 .and. iwork(node2)==0 ) then

!              --- iwork(node1) = 0 and iwork(node2) = 0
!                  iwork(node1) := 1
!                  iwork(node2) := 1
!                  add element to array kcrosselem
!                  nintersect := nintersect+1

                  iwork(node1) = 1
                  iwork(node2) = 1
                  if ( iwork1(node1)==0 ) then

!                 --- node1 outside obstacle

                     nintersect = nintersect+1
                     kcrosselem(1,nintersect) = node1
                     kcrosselem(2,nintersect) = node2
                     rcrosselem(nintersect) = rcrossobst(ipos)

                  else if ( iwork1(node2)==0 ) then

!                 --- node2 outside obstacle

                     nintersect = nintersect+1
                     kcrosselem(1,nintersect) = node2
                     kcrosselem(2,nintersect) = node1
                     rcrosselem(nintersect) = 1d0-rcrossobst(ipos)

                  end if
                  if ( debug ) then
                     write(irefwr,*) 'iwork1 ', &
                        iwork1(node1), iwork1(node2)
                     write(irefwr,*) 'nintersect, kcrosselem ', &
                        nintersect, kcrosselem(1,nintersect), &
                        kcrosselem(2,nintersect)
                  end if  ! ( debug )

               end if

            end do

         end if
         kprobb(ibngrp) = kprobb(ibngrp)+nintersect-nintersectbef
         ikprbi = ikprbi+(nintersect-nintersectbef)*2
         iposgrp(ibngrp) = iposgrp(ibngrp)+(nintersect-nintersectbef)*2
         iworkcross(igrp) = nintersect-nintersectbef
         nelemlc = nintersect-nintersectbef+nelemlc
         if ( debug ) then
            write(irefwr,*) 'ibngrp, nintersect, igrp, nelem ', &
                             ibngrp, nintersect, igrp, nelemlc
            write(irefwr,*) 'nintersectbef ',nintersectbef
         end if  ! ( debug )

!        --- Clear iwork1

         istart = ipointobst(iobst)
         numpntobst = ipointobst(istart)
         numinnobst = ipointobst(istart+1)
         numbndobst = ipointobst(istart+2)
         istart = istart+3+numpntobst
         do i = 1, numinnobst+numbndobst
            iwork1(ipointobst(istart+i)) = 0
         end do

         iinp = iinp+5+inum

      end do

      nintersect = nelemlc

      deallocate ( iwork, iwork1, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug .or. printobst ) then

!     --- debug is true, print kproby, kcrosselem and rcrosselem

         write(irefwr,1) 'iinp', iinp
         call prinin1 ( kcrosselem, nintersect, 2, 'kcrosselem' )
         call prinrl ( rcrosselem, nintersect, 'rcrosselem' )

      end if
      call erclos ( 'prob01crobst' )

      end
