      subroutine prob04obst ( iinput, iinp, nkbnd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.4  date 15-03-2012 Extra debug
!        version  1.3  date 29-11-2004 Correction for points in obstacle and
!                                      on boundary
!        version  1.2  date 11-01-2004 New definition ipointobst
!
!   copyright (c) 2000-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute the number of nodal points with prescribed boundary
!    conditions for nodes in obstacles
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
!     problem_definition
!     obstacle
!     essential_boundary_condition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iinp, nkbnd

!     iinp          i/o   last position used in array iinput
!     iinput         i    integer input array
!     nkbnd         i/o   Number of points with essential boundary conditions
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, nesallobst, nesinnobst, iobst, iseq, &
              nesbnobst, istart, numpntobst, numinnobst, numbndobst, &
              nesonbnobst, nesinonobst, numinonobst

!     i              Counting variable
!     iobst          Obstacle sequence number
!     iseq           Sequence number
!     istart         First position used at a part
!     j              Counting variable
!     nesallobst     Number of obstacles where all nodes are prescribed
!     nesbnobst      Number of obstacles where all but the inner nodes are
!                    prescribed
!     nesinnobst     Number of obstacles where all inner nodes are prescribed
!     nesinonobst    Number of obstacles where all special nodes on the
!                    boundary of an obstacle are prescribed
!     nesonbnobst    Number of obstacles where all the nodes on the boundary of
!                    the obstacle are prescribed
!     numbndobst     Number of nodal points on the boundary of the obstacle
!     numinnobst     Number of points that are near the boundary of the obstacle
!     numinonobst    Number of nodal points on the boundary of the obstacle
!                    which are only in elements completely inside the obstacle
!     numpntobst     Number of nodal points inside the obstacle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     SEPFILLKMAD2   Fill array kmesh ad  part 2: ielemobst
!     SEPFILLKMESHAD Fill array kmesh part d
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2209   Obstacle number out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
!      nesallobst  := iinput(45)
!      nesinnobst  := iinput(41)
!      nesbnobst   := iinput(42)
!      nesbonnobst := iinput(43)
!      nesinonobst := iinput(46)
!      for i := 1 (1) nesallobst do
!         iobst := obstacle number
!         for all points in obstacle do
!            nkbnd := nkbnd+1
!      for i := 1 (1) nesinnobst do
!         iobst := obstacle number
!         for all points in obstacle do
!            nkbnd := nkbnd+1
!      for i := 1 (1) nesbnobst do
!         iobst := obstacle number
!         for all points near the boundary of the obstacle do
!            nkbnd := nkbnd+1
!      for i := 1 (1) nesonbnobst do
!         iobst := obstacle number
!         for all points on the boundary of the obstacle do
!            nkbnd := nkbnd+1
!      for i := 1 (1) nesinonobst do
!         iobst := obstacle number
!         for all points on the boundary of the obstacle do
!            nkbnd := nkbnd+1
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob04obst' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob04obst'
         write(irefwr,1) 'iinp', iinp
         call prinin ( iinput(iinp), 3, 'iinput' )
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      if ( kelmad==0 ) call sepfillkmeshad ( iseq )
      if ( kmeshad(1)==0 ) call sepfillkmad2

!     --- nesallobst  := iinput(45)
!         nesinnobst  := iinput(41)
!         nesbnobst   := iinput(42)
!         nesbonnobst := iinput(43)
!         nesinonobst := iinput(46)

      nesallobst  = iinput(45)
      nesinnobst  = iinput(41)
      nesbnobst   = iinput(42)
      nesonbnobst = iinput(43)
      nesinonobst = iinput(46)
      if ( debug ) write(irefwr,1) &
         'nesallobst, nesinnobst, nesbnobst, nesonbnobst, nesinonobst', &
          nesallobst, nesinnobst, nesbnobst, nesonbnobst, nesinonobst

!     --- for i := 1 (1) nesallobst do

      do i = 1, nesallobst

!     --- iobst := obstacle number

         iobst = iinput(iinp+2)
         iinp = iinp+2
         if ( iobst<1 .or. iobst>numobst ) then

!        --- iobst out of range

            call errint ( iobst, 1 )
            call errint ( numobst, 2 )
            call errsub ( 2209, 2, 0, 0 )

         end if

!        --- all nodes

         istart = ipointobst(iobst)
         numpntobst = ipointobst(istart)
         numinnobst = ipointobst(istart+1)
         numbndobst = ipointobst(istart+2)
         numinonobst = ipointobst(istart+3)
         istart = istart+3
         do j = 1, numpntobst+numinnobst+numbndobst+numinonobst
            nkbnd = nkbnd+1
         end do

      end do

!     --- for i := 1 (1) nesinnobst do

      do i = 1, nesinnobst

!     --- iobst := obstacle number

         iobst = iinput(iinp+2)
         iinp = iinp+2
         if ( iobst<1 .or. iobst>numobst ) then

!        --- iobst out of range

            call errint ( iobst, 1 )
            call errint ( numobst, 2 )
            call errsub ( 2209, 2, 0, 0 )

         end if

!        --- internal nodes

         istart = ipointobst(iobst)
         numpntobst = ipointobst(istart)
         numinonobst = ipointobst(istart+3)
         do j = 1, numpntobst+numinonobst
            nkbnd = nkbnd+1
         end do

      end do

!     --- for i := 1 (1) nesbnobst do

      do i = 1, nesbnobst

!     --- iobst := obstacle number

         iobst = iinput(iinp+2)
         iinp = iinp+2
         if ( iobst<1 .or. iobst>numobst ) then

!        --- iobst out of range

            call errint ( iobst, 1 )
            call errint ( numobst, 2 )
            call errsub ( 2209, 2, 0, 0 )

         end if

!        --- boundary nodes

         istart = ipointobst(iobst)
         numinnobst = ipointobst(istart+1)
         numbndobst = ipointobst(istart+2)
         nkbnd = nkbnd+numinnobst+numbndobst

      end do

!     --- for i := 1 (1) nesonbnobst do

      do i = 1, nesonbnobst

!     --- iobst := obstacle number

         iobst = iinput(iinp+2)
         iinp = iinp+2
         if ( iobst<1 .or. iobst>numobst ) then

!        --- iobst out of range

            call errint ( iobst, 1 )
            call errint ( numobst, 2 )
            call errsub ( 2209, 2, 0, 0 )

         end if

!        --- boundary nodes

         istart = ipointobst(iobst)
         numbndobst = ipointobst(istart+2)
         nkbnd = nkbnd+numbndobst

      end do

!     --- for i := 1 (1) nesinonobst do

      do i = 1, nesinonobst

!     --- iobst := obstacle number

         iobst = iinput(iinp+2)
         iinp = iinp+2
         if ( iobst<1 .or. iobst>numobst ) then

!        --- iobst out of range

            call errint ( iobst, 1 )
            call errint ( numobst, 2 )
            call errsub ( 2209, 2, 0, 0 )

         end if

!        --- special boundary nodes

         istart = ipointobst(iobst)
         numinonobst = ipointobst(istart+3)
         nkbnd = nkbnd+numinonobst
         if ( debug ) write(irefwr,*) 'i, numinonobst ', i, numinonobst

      end do  ! i = 1, nesinonobst

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'nkbnd ', nkbnd
         write(irefwr,*) 'End prob04obst'

      end if
      call erclos ( 'prob04obst' )

      end
