      subroutine prob04vlmnew ( iinput, iinp, nkbnd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-04-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the number of nodal points with prescribed boundary
!     conditions for a set of volumes
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
!     problem_definition
!     volume
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iinp, nkbnd

!     iinp          i/o   last position used in array iinput
!     iinput         i    integer input array
!                         See subroutine prord3
!     nkbnd         i/o   Number of points with essential boundary conditions
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inpelm, iskipbn, istvlm, i, &
              istart, jinp, nsurfsact, nvolumes, ivolnr
      integer, allocatable :: iwork(:)

!     i              Counting variable
!     inpelm         number of points per element
!     iskipbn        Defines if the outer boundary must be skipped
!                    Possible values:
!                    0:  the outer boundary must not be skipped
!                    1:  the complete outer boundary must be skipped
!                    2:  the inner boundary must be skipped as well as some
!                        explicitly given outer surfaces
!                    3:  the outer boundary must be skipped except some
!                        explicitly given outer surfaces
!     istart         First position used at a part
!     istvlm         Starting address of volumes in iinput
!     ivolnr         Surface sequence number
!     iwork          integer work array of length npoint
!                    See subroutine prob04vlmels for a description
!     jinp           last position used in array iinput
!     nsurfsact      Actual number of surfaces to be used
!     nvolumes       Number of volumes in array ivolumes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PROB04VLMELS   Compute the number of nodal points with prescribed boundary
!                    conditions for a set of nodes corresponding to volumes
!     SEPEXTKMESHT   Compute starting and ending address of elements in kmeshc
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2281   the option skip surfaces has not yet been implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob04vlmnew' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob04vlmnew'
         write(irefwr,1) 'iinp', iinp
         call prinin ( iinput(iinp+1), 5+iinput(iinp+2)+iinput(iinp+1), &
                       'iinput' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( iwork(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'nodes' )
      if ( ierror/=0 ) go to 1000

      inpelm = iinput(iinp+2)
      nvolumes = iinput(iinp+3)
      istvlm = iinp+3
      iinp = iinp+3+nvolumes

      if ( debug ) &
         write(irefwr,1) 'inpelm, nvolumes, istvlm, iinp', &
                          inpelm, nvolumes, istvlm, iinp

!     --- clear array iwork

      iwork = 0

!     --- compute iskipbn

      jinp = iinp+inpelm+2
      iskipbn = iinput(jinp-1)
      nsurfsact = iinput(jinp)

      if ( iskipbn>=1 .or. nsurfsact>0 ) then

!     --- iskipbtruen>0, or ncurves>0: noy yet implemented

         call errsub ( 2281, 0, 0, 0 )
         go to 1000

      end if  ! ( iskipbn>=1 .or. nsurfsact>0 )

!     --- Loop over volumes

      do i = 1, nvolumes

!     --- compute the number of elements  inelem

         ivolnr = iinput(istvlm+i)

         if ( debug ) write(irefwr,1) 'i, ivolnr', i, ivolnr

         call sepextkmesht ( ivolnr, ivolnr, istart, inpelm )
         istart = istart-1

!        --- Compute nkbnd for all nodes in volumes

         call prob04vlmels ( inelem, inpelm, kmeshc, istart, &
                             iinput, iinp, iwork, nkbnd )

      end do  ! i = 1, nvolumes

      iinp = jinp+nsurfsact

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'nodes' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prob04vlmnew'

      end if  ! ( debug )
      call erclos ( 'prob04vlmnew' )

      end
