      subroutine prob05 ( iskippoint, nodes, ipoint, nesintersect )
! ======================================================================
!
!        programmer    Guus Segal
!        version 14.0  date 13-10-2013 Complete new parameter list
!        version 13.2  date 20-06-2013 New definition file ref numbers
!        version 13.1  date 03-04-2012 New calls to prob05sf/cv
!
!   copyright (c) 1984-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!       Fill KPROB part g with information concerning the essential
!       boundary conditions
!       This subroutine is called by PROBDF
!       It replaces a part of the previous subroutines PROB05, PROB06, PROB08,
!       PROB11, PROB12 and PROB13
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iskippoint(*), nesintersect, &
              nodes(npoint,nesintersect), ipoint(nesintersect)

!     ipoint         i    Array of length nesintersect containing the
!                         number of nodes for each intersection
!     iskippoint     i    Array of length nskip+1 containing the points to
!                         be skipped
!                         Position 1 contains nskip (number of points to be
!                                    skipped
!                         The next positions contain the node numbers to be
!                         skipped
!     nesintersect   i    Number of intersections where essential boundary
!                         conditions are given
!     nodes          i    Array of size nesintersect x npoint
!                         nodes(i,1:ipoint(i)) contains the nodal points
!                         corresponding to intersection i
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer maxobst, iinp, ivers

!     iinp           last position used in array iinputkp
!     ivers          Version sequence number
!     maxobst        maximum number of active obstacles
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ1        print 2d integer array
!     PROB05CAV      Fill KPROB part g with information concerning the essential
!                    conditions in cavity points
!     PROB05CONT     Fill essential boundary conditions for contact
!     PROB05CV       Fill essential boundary conditions for a curve
!     PROB05ELEM     Fill essential boundary conditions for elements
!     PROB05ELGRP    Fill essential boundary conditions for element groups
!     PROB05INTERS   Fill essential boundary conditions for intersections
!     PROB05LEVSET   Fill essential boundary conditions  in case of level
!                    set functions
!     PROB05NODAL    Fill essential boundary conditions for nodes
!     PROB05NODP     Fill essential boundary conditions for user points
!     PROB05OBST     Fill essential boundary conditions for an obstacle
!     PROB05OBSTCV   Fill essential boundary conditions for obstacle curves
!     PROB05SF       Fill essential boundary conditions for a surface
!     PROB05SKIP     Fill essential boundary conditions for skip elements
!     PROB05UP       Fill essential boundary conditions for user points
!     PROB05VLM      Fill KPROB part g with information concerning the essential
!                    boundary conditions for a volume
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob05' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob05'
         write(irefwr,1) 'npoint', npoint
 1       format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      nkbnd = 0
      iinp  = intarprb(4)-1
      ivers  = iinputkp(15)
      if ( debug ) write(irefwr,1) 'iinp, ivers', iinp, ivers

!     --- User points

      if ( iinputkp(6)>0 ) call prob05up ( iinp )
      if ( debug ) write(irefwr,1) 'iinp after prob05up', iinp, nkbnd

!     --- Curves

      if ( iinputkp(7)>0 ) call prob05cv ( iinp )
      if ( debug ) write(irefwr,1) 'iinp after prob05cv', iinp, nkbnd

!     --- Surfaces

      if ( iinputkp(8)>0 ) call prob05sf ( iinp )
      if ( debug ) write(irefwr,1) 'iinp after prob05sf1', iinp, nkbnd

!     --- Volumes

      if ( iinputkp(39)>0 ) call prob05vlm ( iinp )
      if ( debug ) write(irefwr,1) 'iinp after prob05vlm1', iinp, nkbnd

!     --- Nodal points

      if ( iinputkp(10)>0 ) call prob05nodp ( iinp )
      if ( debug ) write(irefwr,1) 'iinp after prob05nodp', iinp, nkbnd

!     --- Elements

      if ( iinputkp(11)>0 ) call prob05elem ( iinp)
      if ( debug ) write(irefwr,1) 'iinp after prob05elem', iinp, nkbnd

!     --- Element groups

      if ( iinputkp(12)>0 ) call prob05elgrp ( iinp)
      if ( debug ) write(irefwr,1) 'iinp after prob05elgrp', iinp, nkbnd

!     --- Obstacles

      if ( ivers<=1 ) go to 1000  ! jump in case of first version
      maxobst = numobst

      if ( iinputkp(16)>0 .and. maxobst>0 .and. kelmh>0 ) then
         if ( kmeshh(2,1)>0 ) call prob05obstcv ( iinp)
      end if  ! ( iinputkp(16)>0 .and. maxobst>0 .and. kelmh>0 )
      if ( debug ) write(irefwr,1) 'iinp after prob05obstcv', iinp, nkbnd

      if ( ivers>=6 .and. &
           (iinputkp(45)>0 .or. iinputkp(41)>0 .or. &
            iinputkp(42)>0 .or. iinputkp(43)>0 .or. &
            iinputkp(48)>0 .and. nlevelsets>0) ) then

!     --- points in obstacles are prescribed

         call prob05obst ( iinp)

      end if
      if ( debug ) write(irefwr,1) 'iinp after prob05obst2', iinp, nkbnd

!     --- Contact and no_contact

      if ( iinputkp(19)>0 .or. iinputkp(22)>0 ) call prob05cont ( iinp)
      if ( debug ) write(irefwr,1) 'iinp after prob05cont', iinp, nkbnd

!     --- Nodal points

      if ( iinputkp(28)>0 ) call prob05nodal ( iinp )
      if ( debug ) write(irefwr,1) 'iinp after prob05nodal', iinp, nkbnd

!     --- Skip points

      if ( kprobloc(53)>0 ) then

!     --- kprob part aa has been filled, so possibly points must be skipped

         call prob05skip ( iskippoint )

      end if
      if ( debug ) write(irefwr,1) 'iinp after prob05skip', iinp, nkbnd

!     --- Level set functions

      if ( nlevelsets>0 .and. iinputkp(48)>0 ) then

!     --- There are level set functions

         call prob05levset ( iinp )

      end if  ! ( nlevelsets>0 .and. iinputkp(48)>0 )
      if ( debug ) write(irefwr,1) 'iinp after prob05levset1', iinp, nkbnd

!     --- Cavity boundary conditions

      if ( kprobloc(60)>0 .and. iinputkp(49)>0 ) then

!     --- There are parts where we have cavitation
!         All those nodes are prescribed

         call prob05cav ( iinp )

      end if  ! ( kprobloc(60)>0 .and. iinputkp(49)>0 )
      if ( debug ) write(irefwr,1) 'iinp after prob05cav1', iinp, nkbnd

!     --- Intersection boundary conditions

      if ( iinputkp(56)>0 ) &
         call prob05inters ( iinp, nodes, &
                             ipoint, nesintersect, npoint )
      if ( debug ) write(irefwr,1) 'iinp after prob05inters', iinp

1000  if ( debug ) then

!     --- Debug information

         call prinin1 ( kprobg, nkbnd, 2, 'kprobg' )
         write(irefwr,*) 'End prob05'

      end if
      call erclos ( 'prob05' )

      end
