      subroutine prob09curv ( iinp, nloccv )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-10-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute npltra and store in kprob with respect to curves
!
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     local_transform
!     transformation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinp, nloccv

!     iinp          i/o   last position used in array iinputkp
!     nloccv         i    Number of curves with local transforms
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jdgfd1, jdgfd2, jdgfd3, i, &
              jstep, iexclu, ilast, ifirst, iinsign, kstep, n, &
              kstart, kend, nodp, inunkp, k, jtrans, j
      integer, allocatable :: iwork(:), iwork1(:)

!     i              Counting variable
!     iexclu         Indicates which points are excluded
!     ifirst         First curve number
!     iinsign        Sign of curve number
!     ilast          Last curve number
!     inunkp         Number of degrees of freedom in point
!     iwork          integer work array of length npoint
!     iwork1         integer work array of length npoint
!     j              Counting variable
!     jdgfd1         Degree of freedom
!     jdgfd2         Degree of freedom
!     jdgfd3         Degree of freedom
!     jstep          Step in loop
!     jtrans         Type of local transformation
!                    1:  standard transformation
!                    2:  symmetric transformation
!                    3:  non-symmetric transformation
!     k              Counting variable
!     kend           End value of parameter
!     kstart         Start value of parameter
!     kstep          Step in loop
!     n              number of nodes in curves
!     nodp           Nodal point sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRTS00         Check icurv1, icurv2
!     SEPER0026      fill nodal points of curves icrv1 to icrv2 in array iwork
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     335   degree of freedom has wrong value
!     337   at least two degrees of freedom have same value
!     559   The parameter l for the definition of local transforms has
!           an incorrect value
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob09curv' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob09curv'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- get parameters

      jdgfd3 = 0
      maxtrn = 2
      if ( ndim==3 ) maxtrn = 9

!     --- allocate work space

      allocate ( iwork(npoint), iwork1(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, 2*npoint, 'iwork' )
      if ( ierror/=0 ) go to 1000

      do i = 1, nloccv

         jdgfd1 = iinputkp(iinp+1)
         jdgfd2 = iinputkp(iinp+2)
         if ( ndim==3 ) jdgfd3 = iinputkp(iinp+3)
         jstep = iinputkp(iinp+ndim+1)
         if ( jstep>=0 ) then

!        --- jstep>=  0,  compute iexclu

            iexclu = jstep/100
            if ( iexclu>3 ) then
               call errint ( iexclu, 1 )
               call errint ( jstep, 2 )
               call errsub ( 559, 2, 0, 0 )
            end if
            jstep = jstep-100*iexclu

         else

            iexclu = 0

         end if

         ifirst = iinputkp(iinp+ndim+2)
         ilast = iinputkp(iinp+ndim+3)
         if ( debug ) &
            write(irefwr,1) 'ndim, ifirst, ilast',ndim, ifirst, ilast
         if ( jdgfd1==0 ) jdgfd1 = 1
         if ( jdgfd2==0 ) jdgfd2 = 2
         if ( ndim==3 .and. jdgfd3==0 ) jdgfd3 = 3
         iinputkp(iinp+1) = jdgfd1
         iinputkp(iinp+2) = jdgfd2
         if ( ndim==3 ) iinputkp(iinp+3) = jdgfd3
         iinp = iinp+ndim+3
         if ( jdgfd1<0 .or. jdgfd1>nunkp ) then
            call errint ( 1, 1 )
            call errint ( jdgfd1, 2 )
            call errint ( nunkp, 3 )
            call errint ( ifirst, 4 )
            call errsub ( 335, 4, 0, 0 )
         end if
         if ( jdgfd2<0 .or. jdgfd2>nunkp ) then
            call errint ( 2, 1 )
            call errint ( jdgfd2, 2 )
            call errint ( nunkp, 3 )
            call errint ( ifirst, 4 )
            call errsub ( 335, 4, 0, 0 )
         end if
         if ( jdgfd1==jdgfd2 ) then
            call errint ( ifirst, 1 )
            call errsub ( 337, 1, 0, 0 )
         end if
         if ( ndim==3 ) then
            if ( jdgfd3<0 .or. jdgfd3>nunkp ) then
               call errint ( 3, 1 )
               call errint ( jdgfd3, 2 )
               call errint ( nunkp, 3 )
               call errint ( ifirst, 4 )
               call errsub ( 335, 4, 0, 0 )
            end if
            if ( jdgfd1==jdgfd2 .or. jdgfd1==jdgfd3 .or. &
                 jdgfd2==jdgfd3 ) then
               call errint ( ifirst, 1 )
               call errsub ( 337, 1, 0, 0 )
            end if
         end if
         if ( ilast==0 ) ilast = ifirst
         if ( ifirst<0 ) then
            ifirst = -ifirst
            iinsign = -1
         else
            iinsign = 1
         end if
         if ( ilast<0 ) ilast = -ilast
         iinputkp(iinp) = ilast
         call prts00 ( ifirst, ilast, ncurvs )
         iinp = iinp+1
         jtrans = iinputkp(iinp)
         if ( jtrans==2 ) then
            iinp = iinp+ndim**2
            maxtrn = max(maxtrn,ndim**2)
         else if ( jtrans==3 ) then
            iinp = iinp+2*ndim**2
            maxtrn = 2*ndim**2
         else if ( jtrans==4 ) then
            maxtrn = max(maxtrn,ndim**2)
         end if
         kstep = abs(jstep)+1
         call seper0026 ( ifirst, ilast, kstep, n, iwork1, .true., kmeshm )
         if ( iinsign==-1 ) then

!        --- Reverse sequence in iwork1

            do k = 1, n
               iwork(k) = iwork1(n+1-k)
            end do
            do k = 1, n
               iwork1(k) = iwork(k)
            end do

         end if
         iwork = 0
         if ( jstep>=0 ) then

!        --- jstep>=  0   end points of intervals
!            Correct for initial and end point in the case that iexclude>0

            if ( iexclu==0) then

!           --- iexclu = 0  No excluding of nodal points

               kstart = 1
               kend = n

            else if ( iexclu==1) then

!           --- iexclu = 1 Exclude initial point

               kstart = 2
               kend = n

            else if ( iexclu==2) then

!           --- iexclu = 2 Exclude end point

               kstart = 1
               kend = n-1

            else if ( iexclu==3) then

!           --- iexclu = 3 Exclude initial and end point

               kstart = 2
               kend = n-1

            end if

            do k = kstart, kend, kstep
               nodp = iwork1(k)
               if ( nodp>0 .and. iwork(nodp)==0 ) then
                  if ( indprf>0 ) then

!                 --- indprf>0  kprob part f contains the number of unknowns

                     inunkp = kprobf(nodp+1)-kprobf(nodp)
                     if ( jdgfd1>inunkp .or. jdgfd2>inunkp ) goto 200
                     if ( ndim==3 .and. jdgfd3>inunkp ) goto 200

                  end if
                  iwork(nodp) = 1
                  npltra = npltra+1
               end if
200         end do

         else

!        --- jstep <0      points between intervals

            n = n-1
            do k = 1, n, kstep
               do j = 1, kstep-1
                  nodp = iwork1(k+j)
                  if ( nodp>0 .and. iwork(nodp)==0 ) then
                     if ( indprf>0) then

!                    --- indprf>0  kprob part f contains the number of
!                        unknowns

                        inunkp = kprobf(nodp+1)-kprobf(nodp)
                        if ( jdgfd1>inunkp .or. jdgfd2>inunkp ) goto 270
                        if ( ndim==3 .and. jdgfd3>inunkp ) goto 270
                     end if
                     iwork(nodp) = 1
                     npltra = npltra+1
                  end if

270            end do

            end do

         end if

      end do  ! i = 1, nloccv

      deallocate ( iwork, iwork1, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prob09curv'

      end if  ! ( debug )
      call erclos ( 'prob09curv' )

      end
