      subroutine prob10cv2d ( iwork, iwork1, iwork2, &
                              kstart, kend, kstep, jdgfd1, jdgfd2, &
                              jtrans, iinp, n, &
                              jdgfd3, ifirst, ilast, mappinglc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 26-11-2007 Give warning in case of double transform
!        version  2.0  date 24-09-2007 Remove array curves
!        version  1.0  date 23-10-2006
!
!   copyright (c) 2006-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill kprob part n for curves with degrees of freedom
!     and transformation factors (coortr)
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     local_transform
!     transformation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(*), iwork1(*), iwork2(*), &
              kstart, kend, kstep, jdgfd1, jdgfd2, &
              jtrans, iinp, jdgfd3, n, &
              ifirst, ilast, mappinglc(*)

!     ifirst         i    First curve number
!     iinp          i/o   last position used in array iinputkp
!     ilast          i    Last curve number
!     iwork         i/o    integer work array of length npoint
!     iwork1         i    integer work array length max(npoint, ncurvs)
!     iwork2        i/o   integer work array of length npoint
!                         iwork2 is used to mark nodes that have been
!                         transformed already
!                         iwork2(i) contains the number of times a node
!                         is transformed
!     jdgfd1         i    Degree of freedom
!     jdgfd2         i    Degree of freedom
!     jdgfd3         i    Degree of freedom
!     jtrans         i    Type of local transformation
!                         1:  standard transformation
!                         2:  symmetric transformation
!                         3:  non-symmetric transformation
!     kend           i    End value of parameter
!     kstart         i    Start value of parameter
!     kstep          i    Step in loop
!     mappinglc        i    Array of size npointglobal
!                         Contains the local nodal point numbers corresponding
!                         to nodes in the global mesh
!                         If gobal node is not present in local mesh the
!                         corresponding value in mappinglc is 0
!     n              i    number of nodes in curve
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer k, nodp, inusol, inunkp, ideg, iprec, inext, j, ifunc, l, &
              nodpglob
      double precision z

!     ideg           Degree of freedom
!     ifunc          Choice parameter with respect to function calls
!     inext          Next point in curve
!     inunkp         Number of degrees of freedom in point
!     inusol         Position of unknown in array USOL
!     iprec          Surface number of surface to be copied etc.
!     j              Counting variable
!     k              Counting variable
!     l              General loop variable
!     nodp           Nodal point sequence number (local mesh)
!     nodpglob       Nodal point sequence number (global mesh)
!     z              z co-ordinate of point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision functr

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
!     FUNCTR         User function subroutine to compute the local
!                    transformation
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
!     PRINRL1        Print 2d real vector
!     PROB10TR       Compute local transformation for points on a curve
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2909   Double transformation
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob10cv2d' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob10cv2d'
         if ( ipstkmeshpar>0 ) &
            call prinin ( mappinglc, kmeshpar, 'mappinglc' )
         write(irefwr,1) 'kstart, kend, kstep', kstart, kend, kstep
         write(irefwr,1) 'indprf, jdgfd1, jdgfd2', indprf, jdgfd1, jdgfd2
         write(irefwr,1) 'npltra, maxtrn, jtrans', npltra, maxtrn, jtrans
         write(irefwr,1) 'indprh, n, nunkp', indprh, n, nunkp
         write(irefwr,1) 'ifirst, ilast', ifirst, ilast
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      z = 0d0

      do k = kstart, kend, kstep

         if ( ipstkmeshpar>0 ) then

!        --- parallel case, use array mappinglc

            nodpglob = iwork1(k)
            nodp = mappinglc(nodpglob)
            if ( k==1 ) then
               iprec = nodpglob
            else
               iprec = iwork1(k-1)
            end if
            if ( k==n) then
               inext = nodpglob
            else
               inext = iwork1(k+1)
            end if

         else

!        --- serial case

            nodp = iwork1(k)
            nodpglob = nodp
            if ( k==1 ) then
               iprec = nodp
            else
               iprec = iwork1(k-1)
            end if
            if ( k==n) then
               inext = nodp
            else
               inext = iwork1(k+1)
            end if

         end if  ! ( ipstkmeshpar>0 )
         if ( debug ) &
            write(irefwr,1) 'k, nodp, nodpglob, iprec, inext', &
                             k, nodp, nodpglob, iprec, inext

         if ( nodp<=0 ) go to 200  ! skip non-positive nodes

         if ( iwork(nodp)==0 ) then

!        --- node not used in this part

            if ( iwork2(nodp)>0 ) then

!           --- Node has been tranformed before, give warning and skip

               call errint ( nodp, 1 )
               call errwar ( 2909, 1, 0, 0 )
               go to 200

            end if  ! ( iwork2(nodp)>0 )

            if ( indprf>0) then

!           --- indprf>0  kprob part f contains the number of unknowns

               inusol = kprobf(nodp)
               inunkp = kprobf(nodp+1)-inusol
               if ( jdgfd1>inunkp .or. jdgfd2>inunkp ) goto 200
               if ( ndim==3 .and. jdgfd3>inunkp ) goto 200

            else

               inusol = (nodp-1)*nunkp

            end if

            iwork(nodp) = 1
            iwork2(nodp) = iwork2(nodp)+1

            npltra = npltra+1
            ideg = inusol+jdgfd1
            if ( indprh/=0) ideg = kprobh(ideg)
            kprobn(1,npltra) = ideg
            ideg = inusol+jdgfd2
            if ( indprh/=0) ideg = kprobh(ideg)
            kprobn(2,npltra) = ideg

            if ( ndim==3 ) then
               ideg = inusol+jdgfd3
               if ( indprh/=0) ideg = kprobh(ideg)
               kprobn(3,npltra) = ideg
            end if

            if ( jtrans==1 .or. jtrans==4 ) then

!           --- Fill array coortr

               call prob10tr ( maxtrn, jtrans, coortr, npltra, &
                               ndim, coorglob, iprec, inext, &
                               nodpglob, k, ifirst, ilast )
               if ( ierror/=0 ) go to 1000

            else

!           --- compute R and V as given by the user

               do j = 1, ndim**2*(jtrans-1)
                  ifunc = iinputkp(iinp+j)
                  if ( ifunc<0 ) then
                     if ( ndim==3 ) z = coorglob(3,nodpglob)
                     coortr(j,npltra) = functr ( -ifunc, &
                           coorglob(1,nodpglob), coorglob(2,nodpglob), z )
                  else
                     coortr(j,npltra) = rinputkp(ifunc)
                  end if
               end do

            end if
            if ( jtrans==2 .and. maxtrn==2*ndim**2 ) then
               do l = 1, ndim**2
                  coortr(ndim**2+l,npltra) = coortr(l,npltra)
               end do
            end if

         end if

200   end do  ! k = kstart, kend, kstep

1000  if ( debug ) then

!     --- Debug information

         call prinin1 ( kprobn, npltra, ndim, 'kprobn' )
         call prinrl1 ( coortr, npltra, maxtrn, 'coortr' )
         write(irefwr,*) 'End prob10cv2d'

      end if  ! ( debug )
      call erclos ( 'prob10cv2d' )

      end
