      subroutine prob10cvplane ( iwork1, iwork, coor, planefound, &
                                 kstart, kend, kstep, normal )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 04-12-2006 Correction loop
!        version  1.0  date 23-10-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if all nodes in curve are in a plane
!     compute normal on this plane
! **********************************************************************
!
!                       KEYWORDS
!
!     plane
!     normal_vector
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork1(*), iwork(*), kstart, kend, kstep
      double precision coor(3,*), normal(3)
      logical planefound

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     iwork         i/o   integer work array of length npoint
!                         At input and output it is 0
!                         In between all nodes of the curve are stored in iwork
!     iwork1         i    integer work array containing the node numbers of
!                         the curve
!                         The nodes iwork1(kstart), iwork1(kstart+kstep), &
!                         iwork1(kstart+2*kstep), ... , iwork1(kend) are used
!     kend           i    End value of nodes in iwork1
!     kstart         i    Start value of nodes in iwork1
!     kstep          i    Step in loop to be used in iwork1
!     normal         o    Contains normal on plane
!     planefound    i/o   if true at output all nodes are in a plane
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer j, k, nnodes
      double precision t1(3), t2(3), h, innerprod

!     h              length of vector
!     innerprod      inner product of 2 vectors
!     j              Counting variable
!     k              Counting variable
!     nnodes         Number of nodes in curve
!     t1             First tangential vector
!     t2             Second tangential vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELOUTR         Compute outer product of two vectors
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHCHPLANE     Check if all nodes in surface are lying in a plane
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob10cvplane' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob10cvplane'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Store all nodes of curve in iwork

      nnodes = 0
      do k = kstart, kend, kstep
         nnodes = nnodes+1
         iwork(nnodes) =  iwork1(k)
      end do  ! k = kstart, kend, kstep
      if ( debug ) call prinin ( iwork, nnodes, 'iwork' )

!     --- Compute 2 tangential vectors in plane
!         The first one between nodes 1 and 2

      do k = 1, 3
         t1(k) = coor(k,iwork(2))-coor(k,iwork(1))
      end do  ! k = 1, 3
      h = 1d0/sqrt(t1(1)**2+t1(2)**2+t1(3)**2)  ! inverse length of t1
      do k = 1, 3
         t1(k) = t1(k)*h       ! normalized
      end do  ! k = 1, 3

!     --- The second one, not in the same direction

      do k = 2, nnodes-1

         do j = 1, 3
            t2(j) = coor(j,iwork(k+1))-coor(j,iwork(k))
         end do  ! j = 1, 3
         h = 1d0/sqrt(t2(1)**2+t2(2)**2+t2(3)**2)  ! inverse length of t2
         do j = 1, 3
            t2(j) = t2(j)*h       ! normalized
         end do  ! j = 1, 3
         innerprod =  t1(1)*t2(1)+t1(2)*t2(2)+t1(3)*t2(3)
         if ( abs(innerprod)<0.9d0 ) go to 200  ! next tangent found

      end do  ! k = 2, nnodes

!     --- If the program arives here, the curve is straight

      planefound = .false.
      go to 900              ! jump to end

!     --- Compute normal as outer product of 2 tangential vectors

200   call eloutr ( t1, t2, normal )

      if ( debug ) then
         call prinrl ( t1, 3, 't1' )
         call prinrl ( t2, 3, 't2' )
         call prinrl ( normal, 3, 'normal' )
      end if  ! ( debug )

      h = 1d0/sqrt(normal(1)**2+normal(2)**2+normal(3)**2)  ! inverse length
      do k = 1, 3
         normal(k) = normal(k)*h       ! normalized
      end do  ! k = 1, 3

!     --- Check if all nodes are in the plane

      call mshchplane ( nnodes, iwork, coor, 1, normal, &
                        planefound )

!     --- Clear iwork

900   do k = 1, nnodes
         iwork(k) =  0
      end do  ! k = 1, nnodes

1000  call erclos ( 'prob10cvplane' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prob10cvplane'

      end if  ! ( debug )

      end
