      subroutine prob10surf ( iinp, nlocsf, mappinglc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-10-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill kprob part n with degrees of freedom with respect to surfaces
!     and transformation factors (coortr)
! **********************************************************************
!
!                       KEYWORDS
!
!     surface
!     local_transform
!     transformation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulework
      use sepmodulekmesh
      use sepmodulecpack
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinp, nlocsf, mappinglc(*)

!     iinp          i/o   last position used in array iinputkp
!     mappinglc        i    contains global node numbers for mesh
!     nlocsf         i    Number of surfaces with local transforms
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iskipbn, istart, numcurvs, &
              jdgfd1, jdgfd2, isrf1, isrf2, isurnr, itang, jdgfd3, &
              k, nodp, j, jtrans, inpelm, inunkp, inusol, &
              normaldr, ielem, jinp, nodpglob, &
              ncurvsact, icurvs(3)
      integer, allocatable :: isurfaces(:), icurvind(:)
      double precision, allocatable :: tangent(:), weight(:), norm(:,:)
      double precision t1(3)
      logical skip

!     i              Counting variable
!     icurvind       Integer work array of length ncurvs that is filled as
!                    follows
!                    icurvind(i) = -1, at the boundary of the surfaces
!                    icurvind(i) = 1, or not in the surfaces icurvind(i)=0
!     icurvs       Integer array containing the surface numbers
!                    Pos. 1 indicates how array icurvs is filled.
!                    Possible values:
!                     0   only one surface is used. The
!                         surface number is stored in icurvs(2)
!                    -1   The solution is used along the
!                         surfaces icrv1 to icrv2. icrv1 is stored in
!                         icurvs(2) and icrv2 in icurvs(3).
!                    n>0  The surface numbers are stored
!                         in icurvs(2) to icurvs(n+1).
!                         That sequence is used.
!     ielem          Element sequence number
!     inpelm         Number of nodes per element
!     inunkp         Number of degrees of freedom in first point
!     inusol         Position of unknown in array USOL
!     iskipbn        If 1 the outer boundary of the surfaces must be skipped
!     isrf1          First surface number
!     isrf2          Last surface number
!     istart         First position used at a part
!     isurfaces      Array containing the surfaces from which information
!                    must be found
!     isurnr         Surface sequence number
!     itang          Defines how the first tangential vector must be defined
!                    0: arbitrary
!                    1: By a line through points p1 and p2
!                    2: By (1,0,0)  (xdir)
!                    3: By (0,1,0)  (ydir)
!                    4: By (0,0,1)  (zdir)
!                    10: By a set of curves in the mesh
!     j              Counting variable
!     jdgfd1         Degree of freedom
!     jdgfd2         Degree of freedom
!     jdgfd3         Degree of freedom
!     jinp           last position used in array iinputkp with respect to
!                    iprec
!     jtrans         Type of local transformation
!                    1:  standard transformation
!                    2:  symmetric transformation
!                    3:  non-symmetric transformation
!     k              Counting variable
!     ncurvsact      Actual number of curves to be used
!     nodp           Nodal point sequence number (local mesh)
!     nodpglob       Nodal point sequence number (global mesh)
!     norm           Array of length ndim * npoint containing the normal
!                    in the nodal points
!     normaldr       If 1 the outward directed normal is required, if -1 the
!                    inward directed normal
!     numcurvs       Number of curves
!     skip           If true the next part must be skipped
!     t1             First tangential vector
!     tangent        Work array of length 3 * number of points in curve
!                    In this array the tangential vectors corresponding
!                    to each node in curve icurnr are stored
!     weight         Work array of length npoint to compute the weights
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERCOMNOR2      Store the computed normals in array norm with the
!                    correct sign
!     ERCURVENODTAN  Set iwork(i) > 0 for all nodes in a (compound) curve
!                    Store the corresponding tangential vectors in
!                    array tangent
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     ERTRACE        Prints trace back of subroutines
!     PR0008         Print information of curves
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
!     PRININNONZERO  Print the non-zero contents of an integer array
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
!     PROB10SF01     fill kprob part n with degrees of freedom with respect to
!                    surfaces and transformation factors (coortr)
!                    in one specific point
!     PROB10SFUSP    Find the user points P1 and P2
!                    fill the tangential direction in t1
!     SEPCOMNOR      Compute the averaged normals in all points on the surfaces
!                    isrf1 to isrf2
!     SEPOUTCR       Find all curves surrounding a set of surfaces
!     SEPPROB09S1    Mark all points on external curves with a -1 in iwork
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2013   ITANG has incorrect value
!    2780   parallel version not yet available
!    2909   Double transformation
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob10surf' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob10surf'
         call ertrace
  1      format ( a, 1x, (10i7) )
         call pr0008 ( kmeshmglob, ncurvs )
         write(irefwr,1) 'iinp, mmnormal', iinp, mmnormal
         call prinin ( iinputkp(iinp+1), 40, 'iinputkp' )
         write(irefwr,1) 'nunkp, nsurfs, maxtrn, nlocsf', &
                          nunkp, nsurfs, maxtrn, nlocsf

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( isurfaces(nsurfs), iwork(npointglob), icurvind(ncurvs), &
                 tangent(3*kmeshmglob(3)), weight(npointglob), &
                 norm(3,npointglob), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, nsurfs+5*npointglob+ncurvs+3*kmeshmglob(3), &
                        'isurfaces' )
      if ( ierror/=0 ) go to 1000

      do i = 1, nlocsf

!     --- Clear array iwork

         iwork = 0
         if ( mmnormal>=0 ) norm = 0d0

         ncurvsact = 0
         jdgfd1 = iinputkp(iinp+1)
         jdgfd2 = iinputkp(iinp+2)
         jdgfd3 = iinputkp(iinp+3)
         isrf1 = iinputkp(iinp+4)
         isrf2 = iinputkp(iinp+5)
         jtrans = iinputkp(iinp+6)
         iskipbn = mod(iinputkp(iinp+7),10)
         inpelm = kmeshnglob(isrf1)
         if ( debug ) then
            write(irefwr,1) 'i, iinp', i, iinp
            write(irefwr,1) &
               'jdgfd1, jdgfd2, jdgfd3, isrf1, isrf2, jtrans, iskipbn', &
                jdgfd1, jdgfd2, jdgfd3, isrf1, isrf2, jtrans, iskipbn
         end if  ! ( debug )

!        --- Set normaldr
!            The value stored in iinputkp is 0:outer, 1: inner
!            To get 1: outer, -1: inner, we multiply it by -2 and add 1

         normaldr = -2*(iinputkp(iinp+7)/10)+1
         if ( jtrans==1 ) then
            itang = iinputkp(iinp+8)
            if ( itang==10 ) numcurvs = iinputkp(iinp+9+inpelm)
         else
            itang = 0
         end if  ! ( jtrans==1 )
         iinp = iinp+8
         if ( debug ) write(irefwr,1) 'itang, iinp', itang, iinp
         if ( itang==1 ) then

!        --- Find the user points P1 and P2, and fill the tangential direction
!            in t1

            call prob10sfusp ( kmeshlglob, iinp, jtrans, inpelm, &
                               coorglob, t1 )
            if ( debug ) then
               write(irefwr,1) 'iinp after prob10sfusp', iinp
               call prinrl ( t1, 3, 't1' )
            end if  ! ( debug )

         else if ( itang>1 .and. itang<5 ) then

!        --- itang = 2, 3, 4 first tangential vector in itang-1 direction

            t1 = 0d0
            t1(itang-1) = 1d0
            if ( debug ) then
               write(irefwr,1) 'iinp itang/1', iinp
               call prinrl ( t1, 3, 't1' )
            end if  ! ( debug )

         else if ( itang==10 ) then

!        --- itang = 10, i.e. along curves the tangential vector is given
!                       the other two directions are used as "normal"
!                       directions
!                       Set iwork = 1 for all nodes in this curve
!                       and store the tangential vectors in tangent

            if ( parallel ) then

!           --- Parallel computation, we need global coordinates for
!               the transformations: not yet implemented

               call errsub ( 2780, 0, 0, 0 )
               go to 1000

            end if  ! ( parallel )

            if ( debug ) write(irefwr,1) 'numcurvs', numcurvs

!           --- fill iwork

            if ( debug ) then
               write(irefwr,1) 'iinp, inpelm', iinp, inpelm
               call prinin ( iinputkp(iinp+1+inpelm), &
                             max(2,1+iinputkp(iinp+1+inpelm)), 'iinputkp' )
            end if  ! ( debug )

            call ercurvenodtan ( iinputkp(iinp+1+inpelm), tangent )

         else if ( jtrans==1 .and. itang/=0 ) then

!        --- itang # 1, has not yet been implemented

            call errsub ( 2013, 0, 0, 0 )
            go to 1000

         end if  ! ( itang==1 )

!        --- Find all nodes in surfaces with local transformations

         if ( debug ) write(irefwr,1) 'iskipbn, iinp', iskipbn, iinp
         if ( iskipbn>=1 ) then

!        --- Boundary must be skipped, &
!            first find curves which correspond to boundary of surfaces

            do j = 1, isrf2-isrf1+1
               isurfaces(j) = isrf1+j-1
            end do ! j = 1, isrf2-isrf1+1
            call sepoutcr ( icurvind, isurfaces, isrf2-isrf1+1 )

!           --- Next mark all points on the outer curves by -1

            if ( iskipbn>1 ) then

!           --- iskipbn>1, extra curves required

               jinp = iinp+inpelm
               if ( jtrans==2 ) then
                  jinp = jinp+9
                  maxtrn = max(maxtrn,9)
               else if ( jtrans==3 ) then
                  jinp = jinp+18
                  maxtrn = max(maxtrn,18)
               end if  ! ( jtrans==2 )

               if ( debug ) &
                  write(irefwr,1) 'iinp, jinp, inpelm, maxtrn', &
                                   iinp, jinp, inpelm, maxtrn

               if ( itang>0 .and. itang<10 ) then

!              --- 1<=itang<=10, 2 extra positions

                  jinp = jinp+2

               else if ( itang>9 ) then

!              --- itang>=10, 1+numcurvs extra positions

                  jinp = jinp+1+numcurvs

               end if  ! ( itang>0 .and. itang<10 )
               if ( debug ) &
                  write(irefwr,1) 'itang, jinp, iinp', itang, jinp, iinp
               ncurvsact = iinputkp(jinp+1)
               jinp = jinp+2
               if ( debug ) &
                  write(irefwr,1) 'ncurvsact, jinp', ncurvsact, jinp

            else

!           --- iskipbn<=1, no extra curves required

               ncurvsact = 0
               jinp = iinp

            end if  ! ( iskipbn>1 )
            if ( debug ) &
               write(irefwr,1) 'itang, jinp, iinp', itang, jinp, iinp

!           --- Mark that local nodal points in iwork must be marked
!               Hence we use kmeshm instead of kmeshmglob

            call sepprob09s1 ( icurvind, iwork, iskipbn, &
                               ncurvsact, iinputkp(jinp) )
            if ( debug ) call prininnonzero ( iwork, npointglob, 'iwork' )

         end if  ! ( iskipbn>=1 )
         if ( debug ) &
            write(irefwr,1) 'iinp, jinp after iskipbn', iinp, jinp

!        --- Compute all normal vectors on the surfaces

         if ( mmnormal==0 ) then

!        --- mmnormal = 0, i.e. normals have not yet been computed

            if ( parallel ) then

!           --- Parallel computation, we need global coordinates for
!               the transformations: not yet implemented

               call errsub ( 2780, 0, 0, 0 )
               go to 1000

            end if  ! ( parallel )
            icurvs(1) = -1
            icurvs(2) = isrf1
            icurvs(3) = isrf2
            call sepcomnor ( norm, weight, icurvs, normaldr )

         else if ( mmnormal>0 ) then

!        --- mmnormal>0, i.e. normals have already been computed
!            Store in array norm with correct sign

            call ercomnor2 ( norm, normaldr, isrf1, isrf2 )

         end if

!        --- Find all points in surfaces with iwork(i) = 0 and fill coortr
!            and kprobn

         istart = 2*nsurfs
         do k = 1, isrf1-1
            istart = istart+kmeshnglob(k)*kmeshnglob(nsurfs+k)
         end do !  k = 1, isrf1-1

!        --- Loop over surfaces

         do isurnr = isrf1, isrf2

            inelem = kmeshnglob(nsurfs+isurnr)

!           --- Loop over elements in surface

            do ielem = 1, inelem

!           --- Loop over nodes in element

               do j = 1, inpelm

                  if ( iinputkp(iinp+j)==1 ) then

!                 --- Node must be taken into account

                     nodpglob = kmeshnglob(j+istart)
                     if ( parallel ) then
                        nodp = mappinglc(nodpglob)
                     else
                        nodp = nodpglob
                     end if  ! ( parallel )
                     if ( nodp>0 .and. iwork(nodp)>=0 ) then

                        if ( iwork2(nodp)>0 ) then

!                       --- Node has been tranformed before, &
!                           give warning and skip

                           call errint ( nodp, 1 )
                           call errwar ( 2909, 1, 0, 0 )
                           go to 200

                        end if  ! ( iwork2(nodp)>0 )

                        if ( indprf>0 ) then

!                       --- indprf>0  kprob part f contains the number
!                                       of unknowns

                           inusol = kprobf(nodp)
                           inunkp = kprobf(nodp+1)-inusol
                           skip = jdgfd1>inunkp .or. &
                                  jdgfd2>inunkp .or. &
                                  jdgfd3>inunkp

                        else

!                       --- indprf = 0  kprob part f is not used, &
!                           compute inusol directly

                           inusol = nunkp*(nodp-1)
                           skip = .false.

                        end if

                        if ( .not. skip ) then

!                       --- Node must not be skipped, fill information

                           if ( debug ) then
                              write(irefwr,1) 'nodp, nodpglob, inusol', &
                                               nodp, nodpglob, inusol
                              write(irefwr,1)'jdgfd1, jdgfd2, jdgfd3', &
                                              jdgfd1, jdgfd2, jdgfd3
                           end if  ! ( debug )
                           call prob10sf01 ( inusol, jdgfd1, jdgfd2, jdgfd3, &
                                             jtrans, norm, isurnr, ielem, &
                                             nodp, j, iinp, itang, coorglob, &
                                             inpelm, iwork, tangent, t1 )

                        end if  ! ( .not. skip )

                        iwork(nodp) = -10  ! mark node as used
                        iwork2(nodp) = iwork2(nodp)+1
                             ! count number of times
                     end if

                  end if

200            end do ! j = 1, inpelm
               istart = istart+inpelm

            end do ! ielem = 1, inelem

         end do ! isurnr = isrf1, isrf2

         if ( debug ) &
            write(irefwr,1) 'iinp, jtrans, inpelm', iinp, jtrans, inpelm

         iinp = iinp + (jtrans-1)*9 + inpelm
         if ( debug ) write(irefwr,1) 'iinp', iinp

         if ( itang>0 .and. itang<10 ) then

!        --- 1<=itang<=9, 2 extra positions

            iinp = iinp+2

         else if ( itang>9 ) then

!        --- itang>=10, numcurvs+1 extra positions

            iinp = iinp+1+numcurvs

         end if  ! ( itang>0 .and. itang<10 )

         if ( debug ) &
            write(irefwr,1) 'iinp, jtrans, inpelm, itang, ncurvsact', &
                             iinp, jtrans, inpelm, itang, ncurvsact
         if ( ncurvsact>0 ) iinp = iinp+1+ncurvsact
         if ( debug ) write(irefwr,1) 'iinp', iinp

      end do ! i = 1, nlocsf

      deallocate ( isurfaces, iwork, icurvind, tangent, weight, norm, &
                   stat = error )
      if ( error/=0 ) call erdealloc ( error, 'isurfaces' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iinp', iinp
         call prinin1 ( kprobn, npltra, 3, 'kprobn' )
         call prinrl1 ( coortr, npltra, maxtrn, 'coortr' )
         write(irefwr,*) 'End prob10surf'

      end if  ! ( debug )
      call erclos ( 'prob10surf' )

      end subroutine prob10surf
