      subroutine probcurvnumelm ( icurnr, nelemb, ibngrp, lenkmeshm3 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 14-03-2007 Correction for parallel
!        version  1.1  date 24-11-2004 Fortran 90
!        version  1.0  date 12-10-2002
!
!   copyright (c) 2002-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute number of line elements along a (compound) curve
!
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     boundary_condition
!     boundary
!     natural
!     line_element
!     compound_curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer icurnr, nelemb, ibngrp, lenkmeshm3

!     ibngrp         i    Actual boundary element group number
!     icurnr         i    Curve sequence number
!     lenkmeshm3     i    Length of kmesh m part 3
!     nelemb         o    Number of elements along curve that are present
!                         in processor
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i1, i2, nsubcv, ic, j1, j2, npoints, jcurnr, &
              firstnode, lastnode, prevlastnode, istar3, kcurnr, &
              iwork(lenkmeshm3), inpelm, iw, j, k, nelemlc
      logical present

!     firstnode      First node on curve
!     i1             First point of curve in KMESH part m
!     i2             Last point of curve in KMESH part m
!     ic             Relative subcurve number
!     inpelm         number of points per element
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     iw             Address
!     iwork          integer work array
!     j              Counting variable
!     j1             First point of curve in KMESH part m
!     j2             Last point of curve in KMESH part m
!     jcurnr         curve sequence number of subcurve
!     k              Counting variable
!     kcurnr         |jcurnr| i.e. absolute value of curve sequence number
!     lastnode       Last node on curve
!     nelemlc        Local number of elements
!     npoints        Number of points
!     nsubcv         Number of subcurves of a curve
!     present        If true the element is lying in the processor
!     prevlastnode   Last node on previous curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     159   number of nodal points along the curves not correct for the
!           creation of line elements in boundary element group
!     603   number of nodal points along the curves not correct for the
!           creation of line elements in boundary element group
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Compute begin and end position of curve in kmeshm part 3
!     Compute number of elements
!     Correct for double points on compound curves
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'probcurvnumelm' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from probcurvnumelm'
  1      format ( a, 1x, (10i6) )

      end if

!     --- Extract parameters

      istar3 = kmeshm(2)-1

      nelemb = 0

!     --- Compute begin and end position of curve in kmeshm part 3

      i1 = kmeshm(4+icurnr)
      i2 = kmeshm(5+icurnr)-1
      inpelm = ishape+1

!     --- Check if curve is a compound curve

      if ( kelmq==0 ) then

!     --- There are no compound curves

         nsubcv = 1

      else

!     --- There are compound curves

         nsubcv = max(1,kmeshq(icurnr+1) - kmeshq(icurnr))

      end if

      if ( nsubcv<=1 ) then

!     --- Curve is single, no special precautions

         nelemlc = (i2-i1)/max(1,ishape)
         if ( ishape==0 ) nelemlc = nelemlc+1

         if ( ishape>0 ) then
            if ( i2-i1/=nelemlc*ishape ) then

!           --- Error, number of nodes does not match with number of elements

               call errint ( icurnr, 1 )
               call errint ( ibngrp, 2 )
               call errint ( nelemlc, 3 )
               call errint ( i2-i1, 4 )
               call errint ( nelemlc*ishape, 5 )
               call errsub ( 159, 5, 0, 0 )
            end if  ! ( i2-i1/=nelemlc*ishape )

         end if  ! ( ishape>0 )

         iw = 1
         do j = i1, i2
            iwork(iw+j-i1) = kmeshm(istar3+j)
         end do

!        --- Compute number of active elements

         iw = 0
         do k = 1, nelemlc

            present = .true.  ! indicates whether element is in processor

            do j = 1, inpelm
               if ( iwork(iw+j)<=0 ) present = .false. ! node not in processor
            end do  ! j = 1, inpelm

            if ( present ) nelemb = nelemb+1

            iw = iw+max(1,ishape)

         end do  ! k = 1, nelemlc

      else

!     --- Curve is compound, i.e. double nodes must be removed and
!         gaps must be tested

         nelemlc = 0

!        --- Loop over subcurves

         do ic = 1, nsubcv

!        --- Compute subcurve number, first point last point and number
!            of nodes

            jcurnr = kmeshq( kmeshq(icurnr)+ic )
            kcurnr = abs(jcurnr)
            j1 = kmeshm(4+kcurnr)
            j2 = kmeshm(5+kcurnr)-1
            if ( jcurnr>0 ) then

!           --- jcurnr>0, curve must be used in forward direction

               firstnode = kmeshm(istar3+j1)
               lastnode = kmeshm(istar3+j2)

            else

!           --- jcurnr<0, curve must be used in backward direction

               firstnode = kmeshm(istar3+j2)
               lastnode = kmeshm(istar3+j1)

            end if  ! ( jcurnr>0 )
            if ( debug ) then
               write(irefwr,1) 'ic, jcurnr, j1, j2', ic, jcurnr, j1, j2
               write(irefwr,1) 'firstnode, lastnode', &
                                firstnode, lastnode
            end if

!           --- Check if this curve is the start of a new series
!               or the next in a row

            if ( ic==1 ) then

!           --- First curve, hence always next in a row
!               Set npoints

               npoints = j2-j1
               if ( debug ) write(irefwr,1) 'npoints', npoints
               iw = 1
               if ( jcurnr>0 ) then

!              --- jcurnr>0, curve must be used in forward direction

                  do j = j1, j2
                     iwork(iw+j-j1) = kmeshm(istar3+j)
                  end do

               else

!              --- jcurnr<0, curve must be used in backward direction

                  do j = j2, j1, -1
                     iwork(iw+j2-j) = kmeshm(istar3+j)
                  end do

               end if  ! ( jcurnr>0 )
               if ( debug ) then
                  write(irefwr,1) 'iw, npoints', iw, npoints
               end if
               iw = iw+j2-j1+1

            else if ( firstnode==prevlastnode ) then

!           --- first node is equal to previous last node
!               Hence the curve is the next in a row
!               Raise npoints

               npoints = npoints+j2-j1
               if ( debug ) write(irefwr,1) 'npoints', npoints
               iw = iw-1
               if ( jcurnr>0 ) then

!              --- jcurnr>0, curve must be used in forward direction

                  do j = j1, j2
                     iwork(iw+j-j1) = kmeshm(istar3+j)
                  end do

               else

!              --- jcurnr<0, curve must be used in backward direction

                  do j = j2, j1, -1
                     iwork(iw+j2-j) = kmeshm(istar3+j)
                  end do

               end if  ! ( jcurnr>0 )

               if ( debug ) then
                  write(irefwr,1) 'iw, npoints', iw, npoints
                  call prinin ( iwork(iw), j2-j1+1, 'iwork' )
               end if

               iw = iw+j2-j1+1

               if ( debug )  write(irefwr,1) 'iw', iw

            else

!           --- first node is not equal to previous last node
!               Hence the curve is the start of a new row
!               Raise and check nelemlc

               if ( ishape>0 ) then

                  if ( mod(npoints,ishape)/=0 ) then

!                 --- The number of points along the curves does not
!                     correspond to the shape number

                     call errint ( icurnr, 1 )
                     call errint ( ibngrp, 2 )
                     call errint ( npoints/ishape, 3 )
                     call errint ( npoints, 4 )
                     call errint ( ishape*(npoints/ishape), 5 )
                     call errint ( jcurnr, 6 )
                     call errsub ( 603, 6, 0, 0 )

                  end if

               end if  ! ( ishape>0 )
               if ( debug ) write(irefwr,1) 'npoints', npoints
               npoints = npoints/max(1,ishape)
               if ( ishape==0 ) npoints = npoints+1
               nelemlc = nelemlc+npoints
               if ( debug ) write(irefwr,1) 'nelemlc, npoints', &
                                             nelemlc, npoints
               iw = 0
               do k = 1, npoints

                  present = .true.  ! indicates whether element is in processor

                  do j = 1, inpelm
                     if ( iwork(iw+j)<=0 ) present = .false. ! not in processor
                  end do  ! j = 1, inpelm

                  if ( present ) nelemb = nelemb+1

                  iw = iw+max(1,ishape)

               end do  ! k = 1, npoints

!              --- Reset npoints and iw

               npoints = j2-j1
               iw = 1
               if ( jcurnr>0 ) then

!              --- jcurnr>0, curve must be used in forward direction

                  do j = j1, j2
                     iwork(iw+j-j1) = kmeshm(istar3+j)
                  end do

               else

!              --- jcurnr<0, curve must be used in backward direction

                  do j = j2, j1, -1
                     iwork(iw+j2-j) = kmeshm(istar3+j)
                  end do

               end if
               if ( debug ) then
                  write(irefwr,1) 'iw, npoints', iw, npoints
                  call prinin ( iwork(iw), j2-j1+1, 'iwork' )
               end if
               iw = j2-j1+2
               if ( debug )  write(irefwr,1) 'iw', iw

            end if
            prevlastnode = lastnode

         end do

!        --- Check last part that is left

         if ( ishape>0 ) then

            if ( mod(npoints,ishape)/=0 ) then

!           --- The number of points along the curves does not
!               correspond to the shape number

               call errint ( icurnr, 1 )
               call errint ( ibngrp, 2 )
               call errint ( npoints/ishape, 3 )
               call errint ( npoints, 4 )
               call errint ( ishape*(npoints/ishape), 5 )
               call errint ( jcurnr, 6 )
               call errsub ( 603, 6, 0, 0 )

            end if

         end if
         npoints = npoints/max(1,ishape)
         if ( ishape==0 ) npoints = npoints+1
         nelemlc = nelemlc+npoints
         if ( debug ) write(irefwr,1) 'nelemlc, npoints', nelemlc, npoints
         iw = 0
         do k = 1, npoints

            present = .true.  ! indicates whether element is in processor

            do j = 1, inpelm
               if ( iwork(iw+j)<=0 ) present = .false. ! node not in processor
            end do  ! j = 1, inpelm

            if ( present ) nelemb = nelemb+1

            iw = iw+max(1,ishape)

         end do  ! k = 1, npoints

      end if

      call erclos ( 'probcurvnumelm' )

      end
