      subroutine probnumunk ( nunkp, nusol, indprf, kprobf, kprobf2d, &
                              iwork, npoint, nnodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-02-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute number of unknwons per point as well as total
!     number of unknowns in vector restricted to a set of surfaces
! **********************************************************************
!
!                       KEYWORDS
!
!     map
!     surface
!     2d
!     3d
!     problem_definition
!     solution_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nunkp, nusol, indprf, kprobf(*), kprobf2d(0:*), iwork(*), &
              npoint, nnodes

!     indprf        i/o   indication if kprob part f has been filled (>0)
!                         or not (0)
!                         At input with respect to 3d mesh
!                         At output with respect to 2d mesh
!     iwork          i    Contain the local node numbers corresponding to the
!                         global ones
!                         if zero node is not local
!                         length of array iwork is npoint
!     kprobf         i    KPROB part f: contains the number of unknowns per
!                         point accumulated
!     kprobf2d       o    Kprob f with respect to 2d mesh
!     nnodes         i    Number of nodes in 2d mesh
!     npoint         i    Number of nodal points in 3d mesh
!     nunkp         i/o   Maximum number of unknowns in a point
!                         At input with respect to 3d mesh
!                         At output with respect to 2d mesh
!     nusol          o    Total number of unknowns in 2d mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nunkpmin, nunkpmax, inunkp, nodp

!     i              Counting variable
!     inunkp         Number of degrees of freedom in point
!     nodp           Nodal point sequence number
!     nunkpmax       Maximum value of inunkp in 2d mesh
!     nunkpmin       Minimum value of inunkp in 2d mesh
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     if ( indprf=0 ) then
!        nusol := nnodes*nunkp
!     else ( indprf>0 )
!        nunkp := maximum number of unknowns per point in surface
!        nunkpmin := minimum number of unknowns per point in surface
!        if ( nunkpmin==nunkp ) then
!           nusol := nnodes*nunkp
!        else ( nunkpmin # nunkp )
!           Fill kprobf and compute nusol
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'probnumunk' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from probnumunk'

      end if
      if ( ierror/=0 ) go to 1000

      if ( indprf==0 ) then

!     --- indprf=0, constant number of unknowns per point
!         nusol := nnodes*nunkp

          nusol = nnodes*nunkp

      else

!     --- indprf>0, so number of unknowns not constant in 3d region
!         nunkp := maximum number of unknowns per point in surface
!         nunkpmin := minimum number of unknowns per point in surface

         nunkpmin = nunkp
         nunkpmax = 0

         do i = 1, npoint

            if ( iwork(i)>0 ) then

!           --- Node in surface

               nodp = iwork(i)      ! nodal point in surface
               inunkp = kprobf(i+1)-kprobf(i)  ! number of unknowns in node
               nunkpmin = min(nunkpmin,inunkp)
               nunkpmax = max(nunkpmax,inunkp)
               kprobf2d(nodp) = inunkp  ! Put inunkp into kprobf2d

            end if  ! ( iwork(i)>0 )

         end do  ! i = 1, npoint

         if ( nunkpmin==nunkpmax ) then

!        --- nunkpmin==nunkpmax
!            nusol := nnodes*nunkp

             nusol = nnodes*nunkpmax
             nunkp = nunkpmax
             indprf = 0

         else

!        --- nunkpmin # nunkpmax
!            Fill kprobf and compute nusol

            kprobf2d(0) = 0
            do i = 1, nnodes
               kprobf2d(i) = kprobf2d(i)+kprobf2d(i-1)  ! accumulate
            end do  ! i = 1, nnodes
            nusol = kprobf2d(nnodes)

         end if  ! ( nunkpmin==nunkp ) then

      end if  ! ( indprf==0 )

1000  call erclos ( 'probnumunk' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End probnumunk'

      end if

      end
