      subroutine prord0user ( ielgrp, iinplc, iinputlc, numveclc, nsear )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 29-10-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read information for user elements
!
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: ielgrp
      integer, intent(inout) :: iinplc, iinputlc(*), numveclc, &
                                nsear

!     ielgrp         i    element group number
!     iinplc        i/o   Last position used in array iinputlc
!     iinputlc      i/o   See subroutine prord0
!     nsear         i/o   Number of specific degrees of freedom
!     numveclc        i/o   Number of vectors of special structure
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: inpelm, inum, i, inunkp, isear, ivec
      logical :: finished

!     finished       Indicates if the loop has finished (true) or not
!                    (false)
!     i              Counting variable
!     inpelm         number of points per element
!     inum           Number of numbers read in a record
!     inunkp         Number of degrees of freedom in point
!     isear          Number of specific degrees of freedom
!     ivec           Sequence number of vector of special structure
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      22   number of degrees of freedom in nodal points not correct
!      31   Incorrect number of points with degrees of freedom are given
!           in element
!      32   Number of numbers found in record corresponding to vector
!           of special structure incorrect
!      33   sequence number of vector of special structure is incorrect
!      34   Incorrect number of points with degrees of freedom given in
!           element for array of special structure
!     540   sequence number of record corresponding to definition of physical
!           degrees of freedom is incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!

!     code     Table of local keywords
!              1: NUMDEGFD
!              2: VEC
!              3: NUMBER
! ======================================================================
!
      call eropen ( 'prord0user' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prord0user'
         write(irefwr,1) 'ielgrp', ielgrp
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nncode = 3
      code(1) = 'NUMDEGFD'
      code(2) = 'VEC'
      code(3) = 'NUMBER'
      ncode(1:3) = 5
      ncode(2) = 3
      ncodmx(1) = 8
      ncodmx(2) = 3
      ncodmx(3) = 6
      itype(1) = 28
      itype(2:3) = 33
      tabmin(1:nncode) = 1d2
      codemain(nncodemain+1) = codesub(1)
      ncodemain(nncodemain+1) = ncodesub(1)
      nmain = -nncodemain-1

      finished = .false.

      do while ( .not. finished )

         inpelm = sepgetinpelm ( ielgrp )

         if ( debug ) &
            write(irefwr,1) 'ielgrp, inpelm', ielgrp, inpelm

!        --- Read rest of information until a main keyword or ELGRP is found

         table(1:nncode) = -1d20
         tabmax(1:3) = 100d0
         iretur = 0
         if ( debug ) write(irefwr,*) 'before inxt02/3'
         call sepreadsetkeyw
         icode = iretur
         if ( ierror/=0 ) go to 1000

         if ( debug ) then
            write(irefwr,1) 'icode', icode
            write(irefwr,2) 'table(1)', table(1)
            call prinrl ( tabmax, 3, 'tabmax' )
         end if  ! ( debug )

         if ( icode>0 ) then

!        --- icode>0, end of information

            iinplc = iinplc+1
            iinputlc(iinplc) = -1
            finished = .true.

         else if ( table(1)>0d0 ) then

!        --- numdegfd found

            iposut = nint(table(1))
            inum = nint(tabmax(1))
            iinputlc(iinplc+1) = 1
            iinputlc(iinplc+2) = inum
            iinplc = iinplc+2
            do i = 1, inum
               iinputlc(iinplc+i) = nint(tabout(iposut+i-1))
            end do
            if ( debug ) &
               call prinin ( iinputlc(iinplc+1), inum, 'numdegfd' )

            if ( inum==1 ) then

!           --- inum  =  1

               inunkp = iinputlc(iinplc+1)
               if ( debug ) write(irefwr,1) 'inunkp', inunkp

               if ( inunkp<=0 ) then
                  call errint ( inunkp, 1 )
                  call errsub ( 22, 1, 0, 0 )
               end if

            else

!           --- inum # 1

               if ( inum/=inpelm ) then
                  call errint ( inum, 1 )
                  call errint ( inpelm, 2 )
                  call errsub ( 31, 2, 0, 0 )
               end if

            end if
            iinplc = iinplc+inum

         else if ( table(2)>0d0 ) then

!        --- VEC found,  hence vector of special structure

            iposut = nint(table(2))
            inum = nint(tabmax(2))
            iinputlc(iinplc+1) = 2
            iinputlc(iinplc+2) = inum
            iinplc = iinplc+2
            do i = 1, inum
               iinputlc(iinplc+i) = nint(tabout(iposut+i-1))
            end do
            if ( inum<2 )  then
               call errint ( inum, 1 )
               call errsub ( 32, 1, 0, 0 )
            else if ( inum>2 .and. inum/=inpelm+1 ) then
               call errint ( inum-1, 1 )
               call errint ( inpelm, 2 )
               call errsub ( 34, 2, 0, 0 )
               call errcrd
            end if
            ivec = iinputlc(iinplc+1)
            if ( ivec<=0 .or. ivec>100 ) then
               call errint ( ivec, 1 )
               call errint ( 100, 2 )
               call errsub ( 33, 2, 0, 0 )
               call errcrd
            else
               numveclc = max(numveclc,ivec)
            end if
            iinplc = iinplc+inum

         else if ( table(3)>0d0 ) then

!        --- NUMBER found,  hence physical degrees of freedom

            iposut = nint(table(3))
            inum = nint(tabmax(3))
            iinputlc(iinplc+1) = 3
            iinputlc(iinplc+2) = inum
            iinplc = iinplc+2
            do i = 1, inum
               iinputlc(iinplc+i) = nint(tabout(iposut+i-1))
            end do
            if ( inum<=2 )  then
               call errint ( inum, 1 )
               call errsub ( 32, 1, 0, 0 )
            else if ( inum>2 .and. inum/=inpelm+1 ) then
               call errint ( inum, 1 )
               call errint ( inpelm+1, 2 )
               call errsub ( 34, 2, 0, 0 )
               call errcrd
            end if
            isear = iinputlc(iinplc+1)
            if ( isear<=0 .or. isear>100 ) then
               call errint ( isear, 1 )
               call errint ( 100, 2 )
               call errsub ( 540, 2, 0, 0 )
               call errcrd
            else
               nsear = max(nsear,isear)
            end if
            iinplc = iinplc+inum

         end if

      end do  ! while ( .not. finished )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prord0user'

      end if  ! ( debug )
      call erclos ( 'prord0user' )

      end subroutine prord0user
