      subroutine prord1type ( iinplc, iinputlc, iwork, &
                              inum, ibngrp, numnat, ntype )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 21-10-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read information of type numbers for boundary elements
!
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(out) :: inum, ibngrp
      integer, intent(inout) :: iinplc, iinputlc(*), iwork(*), numnat, ntype

!     ibngrp         o    boundary element group number
!     iinplc        i/o   Last position used in array iinputlc
!     iinputlc      i/o   See subroutine prord0
!     inum           o    Number of numbers read in a record
!     iwork         i/o   See subroutine prord0
!     ntype         i/o   Number of different type numbers to be used for one
!                         group
!     numnat        i/o   number of boundary element groups
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode2=8
      integer :: ncode2(nncode2), ncodmx2(nncode2)
      integer :: ibngrpfirst, ibngrplast, nelgrp
      character (len=25) :: code2(nncode2)

!     code2          Table with keywords
!     ibngrpfirst    first boundary element group number
!     ibngrplast     last boundary element group number
!     ncode2         Table containing the number of significant characters
!                    in the keyword table code2
!     ncodmx2        Table containing the maximal number of  characters in
!                    the keyword table code2
!     nelgrp         Number of element groups
!     nncode2        Number of entries in code2
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      67   ielgrp has wrong value
!     162   Number of type numbers read in record is incorrect
!     163   element group number read is not the next element group number
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code2 / 'BNGRP', 'LAPLACE', 'POISSON', 'DIFFUSION_EQUATION', &
                  'CONVECTION_DIFFUSION', &
                  'TYPE', 'GENERAL_ELLIPTIC_EQUATION', 'DELTA_FUNCTION' /
      data ncode2  / 3, 4, 4, 4, 6,  4,11, 5 /
      data ncodmx2 / 5, 7, 7,18,20,  4,25,14 /

!     code2    Table of local keywords
!              1: BNGRP
!              2: LAPLACE
!              3: POISSON
!              4: DIFFUSION_EQUATION
!              5: CONVECTION_DIFFUSION
!              6: TYPE
!              7: GENERAL_ELLIPTIC_EQUATION
!              8: DELTA_FUNCTION
! ======================================================================
!
      call eropen ( 'prord1type' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prord1type'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call sepcopycode ( code2, nncode2, ncode2, ncodmx2 )

      itype(1:nncode) = 2
      tabmin(1:nncode) = 0d0
      tabmax(1:nncode) = 0d0
      table(1:nncode) = -1d0

!     --- Read information of types

      itype(1) = 41       ! ibngrp i to j
      tabmin(1) = 21d0
      tabmax(1) = 1d6
      table(1) = -3 ! type

      itype(6) = 11       ! type
      tabmin(6) = 1d0
      tabmax(6) = 999d0
      nmain = -nncodemain
      iretur = icode

      if ( debug ) write(irefwr,*) 'before inxt02/1'
      call sepreadsetkeyw
      icode = iretur
      if ( debug ) then
         write(irefwr,1) 'icode, ierror', icode, ierror
         call prinrl ( table, nncode, 'table' )
      end if  ! ( debug )

      if ( ierror/=0 ) go to 1000

      if ( icode>0 ) then

!     --- Main keyword found

         if ( icode==7 ) then

!        --- Problem found, get next item

            call asknxt

         end if
         go to 1000  ! repeat

      end if

      if ( table(1)<0d0 ) then

!     --- not a boundary element group number found

         ibngrpfirst = 1
         ibngrplast = 1
         iposut = 0
         inum = 0

         if ( table(6)>0d0 ) then

!        --- type found

            iposut = nint(table(6))
            inum = nint(tabmax(6))

         end if  ! ( table(6)>0d0 )

      else

!     --- boundary element group number found

         iposut = nint(table(1))
         inum = nint(tabmax(1))-2
         ibngrpfirst = nint(tabout(iposut))
         ibngrplast = nint(tabout(iposut+1))

      end if  ! ( table(1)<0 )

      if ( debug ) &
         write(irefwr,1) 'ibngrpfirst, ibngrplast, inum, iposut', &
                          ibngrpfirst, ibngrplast, inum, iposut
      do ibngrp = ibngrpfirst, ibngrplast

         if ( debug ) write(irefwr,1) 'ibngrp', ibngrp

         if ( ibngrp<1 .or. ibngrp>1000 ) then

!        --- ibngrp out of range

            call errint ( ibngrp, 1 )
            call errint ( 1000, 2 )
            call errsub ( 67, 2, 0, 0 )

         else if ( iwork(ibngrp)/=0 ) then

!        --- ibngrp has been used already

            call errint ( ibngrp, 1 )
            call errsub ( 163, 1, 0, 0 )

         else

            iwork(ibngrp) = iinplc+1
            numnat = max(numnat,ibngrp)

         end if

      end do  ! ibngrp = ibngrpfirst, ibngrplast
      ibngrp = ibngrpfirst
      iinputlc(iinplc+1) = ibngrpfirst
      iinputlc(iinplc+2) = ibngrplast
      iinplc = iinplc+2
      iinputlc(iinplc+1:iinplc+20) = 0

      if ( inum>0 ) then

         if ( table(6)<0d0 ) iposut = iposut+2
         ntype = max(ntype,inum)  ! number of types found

         iinputlc(iinplc+1:iinplc+inum) = &
            nint(tabout(iposut:iposut+inum-1))

         if ( debug ) then
            write(irefwr,1) 'iinplc, inum, iposut', iinplc, inum, iposut
            call prinrl ( tabout(iposut-2), inum+2, 'tabout' )
         end if  ! ( debug )

      else if ( table(2)>0d0 ) then

!     --- table(2)>0d0, Laplace

         inum = 1
         iinputlc(iinplc+1) = 700

      else if ( table(3)>0d0 ) then

!     --- table(3)>0d0, Poisson

         inum = 1
         iinputlc(iinplc+1) = 701

      else if ( table(4)>0d0 ) then

!     --- table(4)>0d0, diffusion

         inum = 1
         iinputlc(iinplc+1) = 702

      else if ( table(5)>0d0 ) then

!     --- table(5)>0d0, convection diffusion

         inum = 1
         iinputlc(iinplc+1) = 703

      else if ( table(7)>0d0 ) then

!     --- table(7)>0d0, general elliptic equation

         inum = 1
         iinputlc(iinplc+1) = 705

      else if ( table(8)>0d0 ) then

!     --- table(8)>0d0, delta function

         inum = 1
         iinputlc(iinplc+1) = 802

      else

!     --- No type number found

         call errint ( inum, 1 )
         call errint ( 20, 2 )
         call errint ( ibngrp, 3 )
         call errsub ( 162, 3, 0, 0 )
         call errcrd

      end if  ! ( inum>0 )

1000  if ( debug ) then

!     --- Debug information

         call prinin ( iinputlc(iinplc+1), 20, 'iinputlc' )
         write(irefwr,1) 'ntype', ntype
         write(irefwr,*) 'End prord1type'

      end if  ! ( debug )
      call erclos ( 'prord1type' )

      end subroutine prord1type
