      subroutine prordperbc ( iinput, maxlen, iinp, rinput, lengrl, maxrl )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 14-01-2013 correction surfaces
!        version  1.0  date 06-06-2012
!
!   copyright (c) 2012-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read records for definition of periodical boundary conditions
!
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     problem_definition
!     periodical
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), lengrl, maxlen, maxrl, iinp
      double precision rinput(*)

!     iinp          i/o   Last position used in IINPUT
!     iinput        i/o   integer array to be filled with information concerning
!                         periodical boundary conditions
!                         Contents (from position iinp+1)
!                         1: ilast    last address used by this part
!                         2: numpoints
!                         3: numcurves
!                         4: numsurfs
!                         followed by:
!                         5: numpoints starting addresses for each point pair
!                         6: numcurves starting addresses for each curve pair
!                         7: numsurfs starting addresses for each surface pair
!                         For each point pair the following information is
!                         stored
!                         1:  ifirst   first user point
!                         2:  ilast    last user point
!                         3:  number of different parts with information
!                             of the degrees of freedom
!                         Followed by information per part:
!                         1:  inunkp, defines degrees of freedom in bit pattern
!                         2:  if not 0 the position of constant in rinput
!                         3:  if not 0 the position of factor in rinput
!                         For each curve pair the following information is
!                         stored
!                         1:  ifirst   first user point
!                         2:  ilast    last user point
!                         3:  l        parameter l
!                         4:  number of different parts with information
!                             of the degrees of freedom
!                         Followed by information per part (see points)
!                         For each surface pair the following information is
!                         stored
!                         1    first surface number
!                         2    last surface number
!                         3    if>0, number of curves that are excluded
!                              if 0 no curves are excluded
!                              if -1 all curves are excluded
!                         4 ...  curve numbers that are excluded
!                              Finally number of different parts with info
!                              of the degrees of freedom
!                              Followed by information per part (see points)
!     lengrl        i/o   Number of items stored in rinput
!     maxlen         i    Maximal space available for array IINPUT
!     maxrl          i    Maximum number of reals that are allowed in the input
!     rinput        i/o   Real input array
!                         Filling depends on iinput
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode2=9, ncodsb=3, lensurfs=100, &
                            maxinpdeg=20, maxcurves=10, maxpoints=10
      integer ncode2(nncode2), ncodmx2(nncode2), nscods(ncodsb)
      integer numpoints, numcurves, numsurfs, &
              icountdeg, iinputsrf(lensurfs), iinpsrf, &
              iinputdeg(5,maxinpdeg), iinputcrv(3,maxcurves), &
              itypepos, iinputpnt(2,maxpoints)
      character (len=8) code2(nncode2)
      character (len=1) codesb(ncodsb)

!     code2          Table with keywords
!     codesb         Table with subkeywords
!     icountdeg      number of parts stored in iinputdeg
!     iinpsrf        Last position used in iinputsrf
!     iinputcrv      Array of size 3 x maxcurves in which the input read
!                    is stored
!                    Contents
!                    (1,.)  first curve number
!                    (2,.)  last curve number
!                    (3,.)  parameter l
!     iinputdeg      Array of size 5 x maxinpdeg in which the input read
!                    is stored
!                    Contents
!                    (1,.)  itypepos of last position record read
!                    (2,.)  sequence number of corresponding point, curve
!                           or surface
!                    (3,.)  inunkp, defines degrees of freedom in bit pattern
!                    (4,.)  if not 0 the position of constant in rinput
!                    (5,.)  if not 0 the position of factor in rinput
!     iinputpnt      Array of size 2 x maxpoints in which the input read
!                    is stored
!                    Contents
!                    (1,.)  first user point number
!                    (2,.)  last user point number
!     iinputsrf      Array of size lensurfs in which the input read
!                    is stored
!                    Contents per surface pair starting at a position iinpsrf+1
!                    1    first surface number
!                    2    last surface number
!                    3    if>0, number of curves that are excluded
!                         if 0 no curves are excluded
!                         if -1 all curves are excluded
!                    4 ...  curve numbers that are excluded
!     itypepos       Type of place record that has been read as last record
!                    1: user points
!                    2: curves
!                    3: surfaces
!     lensurfs       Extra length needed to store the new surfaces
!     maxcurves      Size of array iinputcrv
!     maxinpdeg      Size of array iinputdeg
!     maxpoints      Size of array iinputpnt
!     ncode2         Table containing the number of significant characters
!                    in the keyword table code2
!     ncodmx2        Table containing the maximal number of  characters in
!                    the keyword table code2
!     ncodsb         Number of entries in codesb
!     nncode2        Number of entries in code2
!     nscods         Number of sub keywords
!     numcurves      Number of curves in array icurves
!     numpoints      Number of points pairs read
!     numsurfs       number of surfaces pairs read
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRORDPERBCCRV  Interpret records for definition of periodical boundary
!                    conditions. Part curves
!     PRORDPERBCFIN  Rearrange input read in sub arrays for periodical boundary
!                    conditions and store in array iinput
!     PRORDPERBCPNT  Interpret records for definition of periodical boundary
!                    conditions. Part user points
!     PRORDPERBCSRF  Interpret records for definition of periodical boundary
!                    conditions. Part surfaces
!     PRORDPERDEGFD  Interpret records for definition of periodical boundary
!                    conditions. Part degfdi and factors
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYCODESUB Copy codemn2 and related arrays into codesub
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!    input is read from the input file.
!    The keyword read is PERIODICAL_BOUNDARY_CONDITIONS
!    The following texts are recognized:
!        POINTS(P1,P2)
!        CURVES l (C1,C2)
!        SURFACES (S1,S2)[, EXCLUDE ( [ALL] [Ci, Ck, ... ] )
!          constant=c1, factor=c2, degfdi, degfdj , ... (may be repeated)
!        The last part may be part of the curves, points or surface record
!        or may be stored as a new record in which case it refers to the
!        last position record read
!    main keywords ( see prorea )
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3107   Not points, curves or surfaces found following periodical boundary
!           conditions
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code2 / 'POINTS', 'CURVES', 'SURFACES', 'EXCLUDE', 'ALL', &
                   'CONSTANT', 'FACTOR', 'DEGFD', 'C' /
      data ncode2  / 3, 3, 3, 3, 3,    3, 3, 3, 1 /
      data ncodmx2 / 6, 6, 8, 8, 3,    8, 6, 5, 1 /
      data codesb / 'P', 'C', 'S' /
      data nscods / 1, 1, 1 /

!     code2     Table of local keywords
!              1: POINTS
!              2: CURVES
!              3: SURFACES
!              4: EXCLUDE
!              5: ALL
!              6: CONSTANT
!              7: FACTOR
!              8: DEGFD
!              9: C
!     codesb   Table of local subkeywords
!              1: P
!              2: C
!              3: S
! ======================================================================
!
      call eropen ( 'prordperbc' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prordperbc'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call sepcopycode ( code2, nncode2, ncode2, ncodmx2 )
      call sepcopycodesub ( codesb, ncodsb, nscods )

      itype(1:nncode) = 1
      itype(1) = 18    ! POINTS
      itype(2) = 31    ! CURVES
      itype(3) = 18    ! SURFACES
      itype(4:5) =  2  ! EXCLUDE, ALL
      itype(8:9) = 14  ! DEGFD, C

      icode = 0
      numpoints = 0  ! number of parts with user points read
      numcurves = 0  ! number of parts with curves read
      numsurfs = 0   ! number of parts with surfaces read
      itypepos = 0   ! indicates last position part read
      iinpsrf = 0    ! last position used in iinputsrf
      icountdeg = 0  ! number of parts in iinputdeg

      ityprd = 1  ! read next line

      do while ( icode<=0 )

!     --- Read keyword and subkeywords

         table(1:nncode) = -1d0
         table(1) = -1  ! P
         table(2) = -2  ! C
         table(3) = -3  ! S
         table(6:7) = -1d20

         tabmin(1:nncode) = -1d9
         tabmin(1:3) = 2d0            ! number of user points, curves, surfaces
         tabmin(8) = 1d1              ! DEGFD
         tabmin(9) = ncurvs        ! C

         tabmax(1:nncode) = 1d9
         tabmax(1) = nuspnt   ! P
         tabmax(2) = ncurvs   ! C
         tabmax(3) = nsurfs   ! S
         tabmax(8) = 1d2              ! DEGFD
         tabmax(9) = tabmin(9)        ! C

         errorcode = 0
         iretur = 0
         icode = 0
         nmain = nncodemain

         call sepreadsetkeyw

         icode = iretur
         if ( debug ) then
            write(irefwr,1) 'icode', icode
            call prinrl ( table, nncode, 'table' )
            call prinrl ( tabmin, nncode, 'tabmin' )
         end if  ! ( debug )
         if ( ierror/=0 ) go to 1000

         if ( icode>0 ) exit   ! main keyword found

!        --- interpret information read regarding the

         if ( table(1)>0d0 ) then

!        --- Points record found

            itypepos = 1  ! point record found
            call prordperbcpnt ( table, code, ncodmx2, tabmax, tabout, &
                                 numpoints, maxpoints, iinputpnt )

         else if ( table(2)>0d0 ) then

!        --- Curve record found

            itypepos = 2  ! curve record found
            call prordperbccrv ( table, code, ncodmx2, tabmax, tabout, &
                                 numcurves, maxcurves, iinputcrv, &
                                 tabmin )

         else if ( table(3)>0d0 ) then

!        --- Surface record found

            itypepos = 3  ! surface record found
            call prordperbcsrf ( table, code, ncodmx2, tabmax, tabout, &
                                 numsurfs, lensurfs, iinputsrf, &
                                 iinpsrf )

         end if  ! ( table(1)>0d0 )
         if ( ierror/=0 ) go to 1000

!        --- Next interpret the part with degfd, constant and factor
!            This is only possible if a position record has been read

         if ( itypepos==0 ) then

!        --- Error, none of the options points, curves, surfaces found

            call errsub ( 3107, 0, 0, 0 )
            go to 1000

         end if  ! ( itypepos==0 )

         call prordperdegfd ( icountdeg, iinputdeg, itypepos, &
                              numpoints, numcurves, numsurfs, &
                              maxinpdeg, table, tabmax, tabout, &
                              lengrl, rinput, maxrl )

      end do  ! while ( icode<=0 )

!     --- Rearrange input for periodical boundary conditions
!         and compute numperbcs
!         Store information in iinput

      call prordperbcfin ( iinput, iinp, numpoints, &
                           numcurves, numsurfs, iinputpnt, &
                           iinputcrv, iinputsrf, iinputdeg, &
                           icountdeg, maxlen, rinput )

      iinput(52) = numpoints+numcurves+numsurfs  ! number of per bc's

1000  if ( debug ) then

!     --- Debug information

         call prinin ( iinput, iinp, 'iinput' )
         write(irefwr,*) 'End prordperbc'

      end if  ! ( debug )
      call erclos ( 'prordperbc' )

      end
