      subroutine prordperdeg1 ( icountdeg, iinputdeg )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-06-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Rearrange array iinputdeg into a natural sequence
!
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     problem_definition
!     periodical
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer icountdeg, iinputdeg(5,icountdeg)

!     icountdeg     i/o   Number of sets of information stored in iinputdeg
!                         The value at input and output are the same but
!                         the parameters is refilled during computation
!     iinputdeg     i/o   Array of size 5 x maxinpdeg in which the input read
!                         is stored
!                         At input:
!                         (1,.)  itypepos of last position record read
!                         (2,.)  sequence number of corresponding point, curve
!                                or surface
!                         (3,.)  inunkp, defines degs of freedom in bit pattern
!                         (4,.)  if not 0 the position of constant in rinput
!                         (5,.)  if not 0 the position of factor in rinput
!                         At output the array is stored in the sequence
!                         points, curves, surfaces all in the same sequence
!                         as the input pairs are stored
!                         (1,.) contains the number of repetitions for each
!                               point, curve of surface
!                               only for the first one in a series
!                         (2..4,.) contain the same as previously (3..5,.)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iinputcopp(3,icountdeg), iinputcopc(3,icountdeg), &
              iinputcops(3,icountdeg), numcrv, numpnt, numsrf, iseqpnt, &
              iseqcrv, iseqsrf, iinputseqp(icountdeg), i, irepeat, &
              iinputseqc(icountdeg), iinputseqs(icountdeg)

!     i              Counting variable
!     iinputcopc     contains a copy of positions 3..5 of iinputdeg for curves
!     iinputcopp     contains a copy of positions 3..5 of iinputdeg for points
!     iinputcops     contains a copy of positions 3..5 of iinputdeg for surfs
!     iinputseqc     contains number of repetitions of degrees of freedom per
!                    curve
!     iinputseqp     contains number of repetitions of degrees of freedom per
!                    point
!     iinputseqs     contains number of repetitions of degrees of freedom per
!                    surface
!     irepeat        number of repetitions
!     iseqcrv        counter for curves
!     iseqpnt        counter for points
!     iseqsrf        counter for surfaces
!     numcrv         number of curves
!     numpnt         number of points
!     numsrf         number of surfaces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prordperdeg1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prordperdeg1'
         call prinin1 ( iinputdeg, icountdeg, 5, 'iinputdeg/input' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- First copy all points to iinputcopp, curves to iinputcopc etc

      numpnt = 0  ! number of points
      numcrv = 0  ! number of surves
      numsrf = 0  ! number of surfaces

      iseqpnt = 0 ! sequence number of point pair
      iseqcrv = 0 ! sequence number of curve pair
      iseqsrf = 0 ! sequence number of surface pair

      do i = 1, icountdeg

         select case ( iinputdeg(1,i) )

         case(1)

!        --- case points, copy to iinputcopp

            numpnt = numpnt+1
            iinputcopp(1:3,numpnt) = iinputdeg(3:5,i)

            if ( iseqpnt<iinputdeg(2,i) ) then

!           --- New point found

               iseqpnt = iseqpnt+1
               iinputseqp(iseqpnt) = 1  ! number of times point appears

            else

!           --- Same point found

               iinputseqp(iseqpnt) = iinputseqp(iseqpnt)+1

            end if  ! ( iseqpnt<iinputdeg(2,i) )

         case(2)

!        --- case curves, copy to iinputcopc

            numcrv = numcrv+1
            iinputcopc(1:3,numcrv) = iinputdeg(3:5,i)

            if ( debug ) then
               write(irefwr,1) 'numcrv, iseqcrv', numcrv, iseqcrv
               write(irefwr,1) 'iinputdeg(2,i)', iinputdeg(2,i)
               call prinin ( iinputcopc(1,numcrv), 3, 'iinputcopc' )
            end if  ! ( debug )

            if ( iseqcrv<iinputdeg(2,i) ) then

!           --- New curve found

               iseqcrv = iseqcrv+1
               iinputseqc(iseqcrv) = 1  ! number of times curve appears

            else

!           --- Same curve found

               iinputseqc(iseqcrv) = iinputseqc(iseqcrv)+1

            end if  ! ( iseqcrv<iinputdeg(2,i) )
            if ( debug ) &
               write(irefwr,1) 'iinputseqc(iseqcrv)', &
                                iinputseqc(iseqcrv)

         case(3)

!        --- case surfaces, copy to iinputcops

            numsrf = numsrf+1
            iinputcops(1:3,numsrf) = iinputdeg(3:5,i)

            if ( iseqsrf<iinputdeg(2,i) ) then

!           --- New surface found

               iseqsrf = iseqsrf+1
               iinputseqs(iseqsrf) = 1  ! number of times surface appears

            else

!           --- Same surface found

               iinputseqs(iseqsrf) = iinputseqs(iseqsrf)+1

            end if  ! ( iseqsrf<iinputdeg(2,i) )

         end select  ! case ( iinputdeg(1,i) )

      end do  ! i = 1, icountdeg

!     --- Refill array iinputdeg

      icountdeg = 0
      numpnt = 0
      do i = 1, iseqpnt
         irepeat = iinputseqp(i)  ! number of inputs for this point
         iinputdeg(1,icountdeg+1) = irepeat
         iinputdeg(2:4,icountdeg+1:icountdeg+irepeat) = &
            iinputcopp(1:3,numpnt+1:numpnt+irepeat)
         numpnt = numpnt+irepeat
         icountdeg = icountdeg+irepeat
      end do  ! i = 1, iseqpnt

      numcrv = 0
      do i = 1, iseqcrv
         irepeat = iinputseqc(i)  ! number of inputs for this curve
         iinputdeg(1,icountdeg+1) = irepeat
         iinputdeg(2:4,icountdeg+1:icountdeg+irepeat) = &
            iinputcopc(1:3,numcrv+1:numcrv+irepeat)
         numcrv = numcrv+irepeat
         icountdeg = icountdeg+irepeat
      end do  ! i = 1, iseqcrv

      numsrf = 0
      do i = 1, iseqsrf
         irepeat = iinputseqs(i)  ! number of inputs for this surface
         iinputdeg(1,icountdeg+1) = irepeat
         iinputdeg(2:4,icountdeg+1:icountdeg+irepeat) = &
            iinputcops(1:3,numsrf+1:numsrf+irepeat)
         numsrf = numsrf+irepeat
         icountdeg = icountdeg+irepeat
      end do  ! i = 1, iseqsrf

1000  call erclos ( 'prordperdeg1' )
      if ( debug ) then

!     --- Debug information

         call prinin1 ( iinputdeg, icountdeg, 5, 'iinputdeg/output' )
         write(irefwr,*) 'End prordperdeg1'

      end if  ! ( debug )

      end
