      subroutine sepcommatboun ( isum, isumnp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 30-01-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill parts of array intmat with respect to boundary conditions
!     Old subroutine incommatboun
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
!     boundary_condition
!     periodical
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulematstr
      use sepmodulekprob
      use sepmoduleintmat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(inout) :: isum, isumnp

!     isum          i/o   Integer variable to compute a sum
!     isumnp        i/o   length of array imat part 2a minus the periodical
!                         boundary conditions part
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: leng2
      integer, allocatable :: iwork(:)
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     iwork          integer work array to store an array temporarily
!     leng2          Length of array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INCM04         Compute length of INTMAT part 2 for periodical boundary
!                    conditions
!     INCM05         Fill INTMAT part 2 with respect to periodical boundary
!                    conditions
!     INI011         Compute integer information of matrix corresponding to
!                    essential boundary conditions
!     INI014         Compute length of arrays to be used in INI011
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcommatboun' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcommatboun'
         write(irefwr,1) 'nbound, npboun, nusol, nrusol, npoint', &
                          nbound, npboun, nusol, nrusol, npoint
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(intmat)

!     --- compute second part of array INTMAT corresponding to boundary
!         conditions

      isum = 0

      if ( nbound>0 ) then

!     --- Essential boundary conditions, allocate irowsip

         allocate ( IM%rowsip(nbound+1), stat = error )
         if ( error/=0 ) call eralloc ( error, nbound+1, 'irowsip' )
         if ( ierror/=0 ) go to 1000
         irowsip => IM%rowsip

      end if  ! ( nbound>0 )

      if ( npboun>0 ) then

!     --- essential and/or periodical boundary conditions
!         reserve temporary space for irowsic

         allocate ( IM%rowsic(npboun+1), stat = error )
         if ( error/=0 ) call eralloc ( error, npboun+1, 'irowsic' )
         if ( ierror/=0 ) go to 1000
         irowsic => IM%rowsic

      end if
      if ( nbound>0 ) then

!     --- nbound>0, hence there are essential boundary conditions
!         Do some checking of data and fill array irowsip

         call ini014

         isum = irowsip(nbound+1)
         isumnp = isum
         if ( debug ) write(irefwr,1) 'isum', isum

      end if

      if ( npboun>0 ) then

!     --- Periodical boundary conditions, compute length of arrays

         call incm04

         isum = isum+irowsic(npboun+1)
         if ( debug ) write(irefwr,*) 'isum ', isum

      end if

      if ( nbound>0 ) then

!     --- nbound>0, essential boundary conditions

         leng2 = irowsip(nbound+1)
         if ( debug ) write(irefwr,*) 'leng2 ', leng2
         if ( leng2>0 ) then

!        --- leng2 > 0, allocate icolsip

            allocate ( IM%colsip(leng2), stat = error )
            if ( error/=0 ) call eralloc ( error, leng2, 'icolsip' )
            if ( ierror/=0 ) go to 1000
            icolsip => IM%colsip

         end if  ! ( leng2>0 )

!        --- Copy irowsip, positions 1 to nbound to
!            iwork so that it can be changed in ini011

         allocate ( iwork(nbound), stat = error )
         if ( error/=0 ) call eralloc ( error, nbound, 'iwork' )
         if ( ierror/=0 ) go to 1000

         iwork(1:nbound) = irowsip(1:nbound)

!        --- Fill intmat part 2 (icolsip)

         call ini011 ( iwork )

         deallocate ( iwork, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'iwork' )
         if ( ierror/=0 ) go to 1000

      end if

      if ( npboun>0 ) then

!     --- Periodical boundary conditions
!         fill part of array INTMT2 with connections corresponding to the
!         "right" part of the periodical unknowns.

         leng2 = irowsic(npboun+1)
         if ( debug ) write(irefwr,*) 'leng2 ', leng2

!        --- Copy intmat part 1a, positions 1 to nbound to
!            iwork so that it can be changed in ini011

         allocate ( iwork(npboun), IM%colsic(leng2), stat = error )
         if ( error/=0 ) call eralloc ( error, npboun+leng2, 'iwork' )
         if ( ierror/=0 ) go to 1000
         icolsic => IM%colsic

         iwork(1:npboun) = irowsic(1:npboun)

         call incm05 ( iwork )

         if ( debug ) then
            write(irefwr,1) 'nusol,nbound,npboun', &
                             nusol,nbound,npboun
         end if

         deallocate ( iwork, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'iwork' )
         if ( ierror/=0 ) go to 1000

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepcommatboun'

      end if  ! ( debug )
      call erclos ( 'sepcommatboun' )

      end subroutine sepcommatboun
