      subroutine sepcommatdg
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-02-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Construct the matrix structure of the matrix DG given the structure
!     of the matrix D
!     Old subroutine incommatdg
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
!     simple
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulesimple
      use sepmodulematstr
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, allocatable :: iwork(:), intcol1(:), intcol2(:), jwork(:)
      integer i, j, k, istore(np), icol, icol1, icount, ndg, &
              jcount, jstore(np), ibegin, iend
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     i              Counting variable
!     ibegin         Starting value for loop counter
!     icol           column number (velocity)
!     icol1          column number (pressure)
!     icount         Counter
!     iend           End value for loop counter
!     intcol1        First part of the integer matrix structure of the
!                    columns-wise storage
!                    Contains pointers to part 2
!                    Length: nu+1
!     intcol2        Second part of the integer matrix structure of the
!                    columns-wise storage
!                    Contains row numbers
!                    Length: same as icolsG
!     istore         integer work array of length nu to store column numbers
!     iwork          integer work array of length np to store sequence numbers
!                    of column numbers in matrix
!     j              Counting variable
!     jcount         Counter
!     jstore         integer work array of length nu to store column numbers
!     jwork          integer work array of length np to store sequence numbers
!                    of column numbers in matrix
!     k              Counting variable
!     ndg            Number of entries in lower triangular part of DG
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     CHSORT         Sort integer array for increasing sequence
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INCOMMATCOL    Make a columnwise copy of the matrix structure, &
!                    where the present orientation is row-wise
!     PR0004         Print INTMAT corresponding to internal matrix
!     SEPINTMATDG    Transform symmetric compact storage scheme to row-compact storage
!                    Special application DG-matrix for simple
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2910   Not enough space
!    2911   Not possible to deallocate
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Make copy of irowsG/2d but now columnwise orientation
!     for i := 1 (1) np do
!        for j := 1 (1) i-1 do
!           if  (row d_i. and row d_j. have common column numbers ) then
!              add ij to matrix DG
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcommatdg' )

      IM => intmt(intmat)

      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcommatdg'
         write(irefwr,1) 'nu, np', nu, np
         write(irefwr,*) 'Structure of matrix D'
         call pr0004 ( 13, np, IM%rowsG, IM%colsG )
  1      format ( a, 1x, (10i6) )

      end if

      irowsG => IM%rowsG
      icolsG => IM%colsG

!     --- Create space for intcol1 and intcol2, iwork, etc

      allocate ( IM%rowsDG(np+1), stat = error )
      if ( error/=0 ) go to 100
      allocate(iwork(np), jwork(np), stat = error)
      if ( error/=0 ) go to 100
      allocate(intcol1(nu+1), stat = error)
      if ( error/=0 ) go to 100
      length = IM%rowsG(np+1)
      allocate(intcol2(length), stat = error)
100   if ( error/=0 ) then

!     --- space for these arrays could not be allocated

         call errint ( 2*np+1+nu+1, 1 )
         call errint ( error, 2 )
         call errchr ( 'iwork', 1 )
         call errsub ( 2910, 2, 0, 1 )
         go to 1000

      end if  ! (error/=0)

!     --- Store organization of L column-wise instead of row-wise

      irowsDG => IM%rowsDG
      call incommatcol ( irowsG, icolsG, np, &
                         intcol1, intcol2, nu )

!     --- Clear arrays iwork and jwork

      iwork = 0
      jwork = 0

!     --- for i := 1 (1) np do

      irowsDG(1) = 0
      irowsDG(2) = 0
      ndg = 0
      ibegin = irowsii(nu+1)+1

!     --- Loop over rows in matrix

      do i = 2, np

!     --- Make iwork unequal 0 for column numbers in di.

         icount = 0     ! number of coupled degrees of freedom
         jcount = 0     ! number of elements in row

!        --- First store elements corresponding to pressure mass matrix

         iend = irowsii(nu+1+i)
         do j = ibegin, iend
            if ( icolsii(j)>nu ) then
               icol = icolsii(j)-nu
               jwork(icol) = 1
               jcount = jcount+1
               jstore(jcount) = icol
            end if  ! ( icolsii(j)>nu )
         end do  ! j = ibegin,iend
         ibegin = iend+1

!        --- Next the other elements

         do j = irowsG(i)+1, irowsG(i+1)

            icol = icolsG(j)     ! column number (velocity)

!           --- Loop over all unknowns coupled to icol

            do k = intcol1(icol)+1, intcol1(icol+1)

               icol1 = intcol2(k)     ! column number (pressure)
               if ( iwork(icol1)==0 ) then

!              --- iwork(icol1)=0, hence new unknown

                  iwork(icol1) = 1       ! mark column
                  icount = icount+1      ! store column into istore
                  istore(icount) = icol1

                  if ( icol1<i .and. jwork(icol1)==0 ) then

!                 --- Matrix is symmetrical so only column numbers less than i
!                     are stored

                     jcount  = jcount+1
                     jstore(jcount) = icol1

                  end if  ! ( icol<i )

               end if  ! ( iwork(icol1)==0 )

            end do  ! k = intcol1(icol)+1, intcol1(icol+1)

         end do  ! j = irowsG(i)+1, irowsG(i+1)

!        --- Clear iwork

         do j = 1, icount
            iwork(istore(j)) = 0
         end do  ! j = 1, icount
         do j = 1, jcount
            jwork(jstore(j)) = 0
         end do  ! j = 1, icount

!        --- sort jstore, indices in istore

         call chsort ( jstore, istore, jcount )

!        --- store sorted elements in icolsDG

         do j = 1, jcount
            icolsDG(ndg+j) = jstore(istore(j))
         end do ! j = 1, jcount
         ndg = ndg+jcount

         irowsDG(i+1) = ndg

      end do   ! i = 1, np
      deallocate(iwork, jwork, stat = error)
      if ( error/=0 ) go to 200
      deallocate(intcol1, stat = error)
      if ( error/=0 ) go to 200
      deallocate(intcol2, stat = error)
200   if ( error/=0 ) then

!     --- problem with deallocate

         call errint ( error, 1 )
         call errchr ( 'iwork', 1 )
         call errsub ( 2911, 1, 0, 1 )
         go to 1000

      end if  ! (error/=0)

      allocate ( IM%colsDG(ndg), IM%diagDG(np) )
      IM%colsDG = icolsDG(1:ndg)
      deallocate ( icolsDG )

 !    if ( .not. IM%Gsymmetric ) call sepintmatdg
      call sepintmatdg

1000  if ( debug ) then

         write(irefwr,*) 'Structure of matrix DG'
         call pr0004 ( 13, np, IM%rowsDG, IM%colsDG )
         write(irefwr,*) 'End sepcommatdg'

      end if
      call erclos ( 'sepcommatdg' )

      end subroutine sepcommatdg
