      subroutine sepcommatrev ( irowsnew, icolsnew, irowsii, icolsii, nrusol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 31-05-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill arrays irowsnew and icolsnew with upper triangular part if
!     icolsii and irowsii contain the lower triangular part and vice versa
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
!     nested_dissection
! **********************************************************************
!
!                       MODULES USED
!
  !    use sepmodulekprob
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: nrusol, irowsii(nrusol+1), icolsii(*)
      integer, intent(out) :: irowsnew(nrusol+1), icolsnew(*)

!     icolsii        i    Contains the input column structure
!     icolsnew       o    Contains the output column structure
!     irowsii        i    Contains the input row structure
!     irowsnew       o    Contains the output row structure
!     nrusol         i    Number of free unknowns
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j, m, ih, ih1
      integer, pointer :: point(:)

!     i              Counting variable
!     ih             Help variable to store a constant temporarily
!     ih1            Help variable to store a constant temporarily
!     j              Counting variable
!     m              Counting variable
!     point          pointer array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcommatrev' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcommatrev'
         write(irefwr,1) 'nrusol', nrusol
         call prinin ( irowsii, nrusol+1, 'irowsii' )
         call prinin ( icolsii, irowsii(nrusol+1), 'icolsii' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill irowsnew

      irowsnew = 0
      do i = 1, nrusol

         do j = irowsii(i)+1, irowsii(i+1)
            irowsnew(icolsii(j)) = irowsnew(icolsii(j))+1
         end do

      end do

!     --- Accumulate irowsnew

      ih = 0
      do i = 1, nrusol
         ih1 = ih+irowsnew(i)
         irowsnew(i) = ih
         ih = ih1
      end do
      irowsnew(nrusol+1) = ih

      if ( debug ) call prinin ( irowsnew, nrusol+1, 'irowsnew' )

      allocate ( point(nrusol) )

      point = irowsnew(1:nrusol)  ! pointer to irowsnew in icolsnew
      icolsnew(1) = 1
      do i = 1, nrusol
         do j = irowsii(i)+1, irowsii(i+1)
            m = icolsii(j)
            point(m) = point(m)+1
            icolsnew(point(m)) = i     ! upper part of matrix
         end do
      end do

      deallocate ( point )

1000  if ( debug ) then

!     --- Debug information

         call prinin ( irowsnew, nrusol+1, 'irowsnew' )
         call prinin ( icolsnew, irowsnew(nrusol+1), 'icolsnew' )
         write(irefwr,*) 'End sepcommatrev'

      end if  ! ( debug )
      call erclos ( 'sepcommatrev' )

      end subroutine sepcommatrev
