      subroutine sepobstmarkel ( lenielemobst )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Mark all elements in the mesh with information if they are
!     within an obstacle or outside an obstacle
!     Store this information in array ielemobst
!     Negative values in ipointobst are set
!     Old subroutine mshobstmarkel
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     element
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: lenielemobst

!     lenielemobst   o    Computed length of array ielemobst
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, inpelm, iielem, ielem,  &
              ielgrp, ikelmc, iobst, inodp, istart, istart1, istartelem, &
              nbnpelm, ninpelm, numbndobst, numbnelmobst, &
              numinnobst, numinelmobst,  numpntobst
      integer, allocatable :: iwork(:), iwork1(:)

!     i              Counting variable
!     ielem          Element number
!     ielgrp         element group number
!     iielem         Relative element number in element group
!     ikelmc         position in kmesh part c
!     inodp          Nodal point
!     inpelm         number of points per element
!     iobst          Obstacle sequence number
!     istart         First position used at a part
!     istart1        First position used at a part
!     istartelem     Starting address in element array
!     iwork          integer work array of length npoint
!                    In this array all nodes in or on the obstacle are
!                    marked as follows
!                    iwork(i) = 0: node i outside obstacle
!                    iwork(i) = 1: node i in obstacle
!                    iwork(i) = 2: node i in obstacle near boundary
!                    iwork(i) = 3: node i in obstacle on boundary
!     iwork1         integer work array of length maximum (nelem,npoint)
!                    This array is used to store element numbers
!                    respectively node numbers
!     nbnpelm        Number of points in element that are on the boundary of
!                    the obstacle
!     ninpelm        Number of points in element that are inside
!                    the obstacle
!     numbndobst     Number of nodal points inside the obstacle
!     numbnelmobst   Number of elements that are partly within the obstacle
!     numinelmobst   Number of elements that are completely within the obstacle
!     numinnobst     Number of points that are near the boundary of the obstacle
!     numpntobst     Number of nodal points inside the obstacle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     SEPPRINIELOBST Print the contents of array ielemobst
!     SEPPRINIPOBST  Print the contents of array ipointobst
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array iwork
!     For iobst := 1 (1) numobst do
!        Fill array iwork(i) with a 1 if node i is in the obstacle
!           and with a 3 when it is on the boundary
!        For ielgrp := 1 (1) nelgrp do
!           inelem := number of elements in element group ielgrp
!           For iielem := 1 (1) inelem
!              ielem := element numbr
!              If (all nodes of element in obstacle iobst) then
!                fill element in ielemobst
!              else if (some nodes of element in obstacle iobst and all others
!                       not ) then
!                fill element in iwork1
!        Fill rest of ielemobst for this obstacle
!        Store points near the boundary in iwork1 and clear iwork
!        Store points near the boundary in ipointobst
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepobstmarkel' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepobstmarkel'
         write(irefwr,1) 'numobst', numobst
  1      format ( a, 1x, (10i6) )
         call sepprinipobst ( ipointobst )

      end if
      if ( ierror/=0 ) go to 1000

!     --- allocate work arrays

      allocate ( iwork(npoint), iwork1(max(npoint,inelem)), &
                 stat = error )
      if ( error/=0 ) &
         call eralloc ( error, npoint+max(npoint,inelem), 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Clear array iwork

      iwork = 0
      istartelem = numobst+1

!     --- Loop over obstacles

      do iobst = 1, numobst

!     --- Fill array iwork(i) with a 1 if node i is in the obstacle
!         and with a 3 when it is on the boundary

         istart = ipointobst(iobst)-1
         numpntobst = ipointobst(istart+1)
         numinnobst = 0
         numbndobst = ipointobst(istart+3)
         i = istart+4+numpntobst
         iwork(ipointobst(istart+5:i)) = 1
         iwork(ipointobst(i+1:i+numbndobst)) = 3
         numinelmobst = 0
         numbnelmobst = 0
         ielemobst(iobst) = istartelem

!        --- For ielgrp := 1 (1) nelgrp do

         ikelmc = 0
         ielem = 0

         do ielgrp = 1, nelgrp

!        --- inelem := number of elements in element group ielgrp

            inelem = Topgrp(ielgrp)%inelem
            inpelm = Topgrp(ielgrp)%inpelm
            ishape = sepgetshape ( ielgrp )

            if ( ndim==1 .or. ndim==2 .and. ishape>=3 .or. ishape>10 ) then

!           --- For iielem := 1 (1) inelem

               do iielem = 1, inelem

!              --- ielem := element number
!                  If (all nodes of element in obstacle iobst) then
!                    fill element in ielemobst
!                  else if (some nodes of element in obstacle iobst and all
!                           others not ) then
!                    fill element in iwork1

                  ielem = ielem+1
                  nbnpelm = 0
                  ninpelm = 0
                  do i = 1, inpelm
                     inodp = kmeshc(ikelmc+i)
                     if ( iwork(inodp)==3 ) then

!                    --- Node on obstacle found

                        nbnpelm = nbnpelm+1

                     else if ( iwork(inodp)>0 ) then

!                    --- Node in obstacle found

                        ninpelm = ninpelm+1

                     end if

                  end do

                  if ( ninpelm+nbnpelm==inpelm ) then

!                 --- all points in obstacle, hence element completely in
!                     obstacle

                     numinelmobst = numinelmobst+1
                     ielemobst(istartelem+2+numinelmobst) = ielem

                  else if ( nbnpelm+ninpelm>0 ) then

!                 --- some points in obstacle, hence element partly in obstacle

                     numbnelmobst = numbnelmobst+1
                     iwork1(numbnelmobst) = ielem
                     do i = 1, inpelm
                        inodp = kmeshc(ikelmc+i)
                        if ( iwork(inodp)==1 ) then

!                       --- Mark node as near the boundary

                           iwork(inodp) = 2

                        end if
                     end do

                  end if

                  ikelmc = ikelmc+inpelm

               end do  ! iielem = 1, inelem

            else

               ikelmc=ikelmc+inpelm*inelem
               ielem = ielem+inelem

            end if  ! ( ishape>=3 )

         end do  ! ielgrp = 1, nelgrp

!        --- Fill rest of ielemobst for this obstacle

         ielemobst(istartelem) = numinelmobst
         ielemobst(istartelem+1) = 0
         ielemobst(istartelem+2) = numbnelmobst
         i = istartelem+2+numinelmobst
         ielemobst(i+1:i+numbnelmobst) = iwork1(1:numbnelmobst)
         istartelem = istartelem+3+numinelmobst+numbnelmobst

!        --- Store points near the boundary in iwork1 and clear iwork

         istart1 = istart+4
         do i = 1, numpntobst
            inodp = ipointobst(istart+4+i)
            if ( iwork(inodp)==1 ) then

!           --- Internal node

               istart1 = istart1+1
               ipointobst(istart1) = inodp

            else

!           --- Near boundary node

               numinnobst = numinnobst+1
               iwork1(numinnobst) = inodp

            end if
            iwork(inodp) = 0

         end do

         i = istart+4+numpntobst
         iwork(ipointobst(i+1:i+numbndobst)) = 0

!        --- Store points near the boundary in ipointobst

         numpntobst = numpntobst-numinnobst
         ipointobst(istart1+1:istart1+numinnobst) = iwork1(1:numinnobst)
         ipointobst(istart+1) = numpntobst
         ipointobst(istart+2) = numinnobst

      end do  ! iobst = 1, numobst
      lenielemobst = istartelem

      deallocate ( iwork, iwork1, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         call sepprinielobst ( ielemobst )
         write(irefwr,*) 'End sepobstmarkel'

      end if
      call erclos ( 'sepobstmarkel' )
      end
