      subroutine sepouteredges ( edgeelems, edgelist, lenedgelist )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-04-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Make a list of all edges corresponding to 1 surface element only
!     Old subroutine topouteredges
! **********************************************************************
!
!                       KEYWORDS
!
!     edge
!     element
!     element
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer edgeelems(*), edgelist(*), lenedgelist

!     edgeelems      i    Contains the element numbers an edge is connected with
!                         in the following way:
!                         Position 1 to nedges contains starting addresses
!                         for each edge
!                         For each edge from the starting position the number
!                         of elements is stored followed by the element numbers
!     edgelist       o    List of edges that correspond to 1 element only
!     lenedgelist    o    Number of elements in array edgelist
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer icount, iedge, istart, numelms, iwork(nelem), i, &
              ielem, nsurfelms, ielgrp

!     i              Counting variable
!     icount         Counter
!     iedge          edge sequence number
!     ielem          Element number
!     ielgrp         element group number
!     istart         First position used at a part
!     iwork          integer work array of length nelem
!                    iwork(i) = 0 if element i is surface element, else
!                    if element i is line element iwork(i) = 1
!     nsurfelms      Number of surface elements connected to edge
!     numelms        Number of elements connected to edge
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array iwork(1:nelem)
!     Set iwork(i) = 1, if element i is line element
!     icount := 0
!     for iedge := 1 (1) nedges do
!        if ( number of elements corresponding to edge = 1 ) then
!           icount := icount+1
!           edgelist(icount) := iedge
!        else ( number of elements corresponding to edge > 1 ) then
!           if ( only one of the elements is surface element ) then
!              icount := icount+1
!              edgelist(icount) := iedge
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepouteredges' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepouteredges'

      end if

!     --- Clear array iwork(1:nelem)

      iwork = 0

!     --- Set iwork(i) = 1, if element i is line element

      ielem = 0
      do ielgrp = 1, nelgrp
         ishape = kmeshf(ielgrp)
         inelem = Topgrp(ielgrp)%inelem
         if ( ishape<3 ) then

!        --- line element found, set iwork to 1

            do i = 1, inelem
               iwork(ielem+i) = 1
            end do

         end if
         ielem = ielem+inelem

      end do
      if ( debug ) call prinin ( iwork, nelem, 'iwork' )

!     --- icount := 0

      icount = 0

!     --- for iedge := 1 (1) nedges do

      do iedge = 1, nedges

         istart = edgeelems(iedge)         ! starting address of iedge
         numelms =  edgeelems(istart)      ! number of elements corresponding to
                                           ! iedge
         if ( numelms==1 ) then

!        --- number of elements corresponding to edge = 1
!            icount := icount+1
!            edgelist(icount) := iedge

            icount = icount+1
            if ( icount>lenedgelist ) then

!           --- Array edgelist too small

               call errint ( icount, 1 )
               call errint ( lenedgelist, 2 )
               call errchr ( 'edgelist', 1 )
               call errsub ( 550, 2, 0, 1 )
               go to 1000

            end if
            edgelist(icount) = iedge

         else  ! ( numelms/=1 )

!        --- number of elements corresponding to edge > 1

            nsurfelms = 0
            do i = 1, numelms
               if ( iwork(edgeelems(istart+i))==0 ) &
                  nsurfelms = nsurfelms+1
            end do
            if ( debug ) write(irefwr,*) 'iedge, numelms, nsurfelms ', &
               iedge, numelms, nsurfelms

            if ( nsurfelms==1 ) then

!           --- only one of the elements is surface element
!               icount := icount+1
!               edgelist(icount) := iedge

               icount = icount+1
               if ( icount>lenedgelist ) then

!              --- Array edgelist too small

                  call errint ( icount, 1 )
                  call errint ( lenedgelist, 2 )
                  call errchr ( 'edgelist', 1 )
                  call errsub ( 550, 2, 0, 1 )
                  go to 1000

               end if
               edgelist(icount) = iedge

            end if  ! ( nsurfelms==1 )

         end if  ! ( numelms==1 )

      end do  ! iedge = 1, nedges
      lenedgelist = icount

1000  if ( debug ) then

!     --- Debug information

         call prinin ( edgelist, lenedgelist, 'outer edges' )
         write(irefwr,*) 'End sepouteredges'

      end if
      call erclos ( 'sepouteredges' )

      end
