      subroutine sepqu8obst ( ntria, npelnw, &
                              nelext, vertex, kline, neleml, iskip, &
                              iskpar, ilevel, itriacolour )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   copyright (c) 2001-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Store all triangles or quadrilaterals of a three-dimensional mesh
!     in array KELEM
!     If the mesh consists of triangles and quadrilaterals, the quadrilaterals
!     are subdivided into triangles
!     All elements must be either linear or quadratic
!     Surface elements that belong to two volume elements are stored twice
!     Later on they can be removed in order to get the outer surface only
!     Old subroutine sepqu8obst
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     boundary
!     mesh
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduletypes
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ntria, npelnw, nelext, &
              kline(*), neleml, iskip, iskpar(*), ilevel, itriacolour(*)

      logical vertex

!     ilevel         i    Defines the level of elements to be plotted if
!                         kmesh(53)>0
!     iskip          i    Indicates if element groups must be skipped (>0)
!                         or not (0)
!                         If <0 volumes must be skipped
!     iskpar         i    array of length NELGRP that is only used if ISKIP > 0
!                         It indicates if element group ielgrp must be skipped
!                         iskpar(ielgrp) = 0 or not iskpar(ielgrp) = 1
!     itriacolour    o    Same as elemobst but now for the triangles
!     kline          o    Array for extra line elements
!     neleml         i    Estimated number of elements by user (parsurf)
!     nelext         i    Number of boundary elements
!     npelnw         o    Number of points per element in KELEM
!     ntria          o    number of triangles or quadrilaterals after removal of
!                         double ones
!     vertex         i    indicates if only the vertices of the surface elements
!                         have to be filled (true) or all points (false)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical triang, bound, sevenp, quadrt
      integer ilow, ihigh, inpelm, ielgrp, ikelmc, ikmeshcnew, i, j, &
              nsurfe, istare, npelmc, istart, k, ielem

!     bound          Indicates if boundary elements are present (true) or not
!                    (false)
!     i              Counting variable
!     ielem          Element number
!     ielgrp         element group number
!     ihigh          Indicates the highest dimensionality in the grid.
!                    If ihigh=0 only 2D elements are present, otherwise (1)
!                    there are also 3D elements
!     ikelmc         position in kmesh part c
!     ikmeshcnew     position in array KELEM
!     ilow           Indicates the lowest dimensionality in the grid.
!                    If ilow=0 only 3D elements are present, otherwise (1)
!                    there are also 2D elements
!     inpelm         number of points per element
!     istare         Starting address of a type of element in array IBOUN3
!     istart         First position used at a part
!     j              Counting variable
!     k              Counting variable
!     npelmc         Number of points in the surface elements as stored
!                    in IBOUN3
!     nsurfe         Number of surfaces of a three-dimensional element as
!                    stored in IBOUN3
!     quadrt         Indicates if quadratic elements have been found
!                    (true) or not
!     sevenp         Indicates if elements with 7 points have been found
!                    (true) or not (false)
!     triang         Indicates if triangular elements have been found
!                    (true) or not (false)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2204   All element groups are skipped
! **********************************************************************
!
!                       PSEUDO CODE
!
!     determine whether triangles or quadrilaterals are used
!     and check whether only surfaces or surfaces and volumes are
!     considered
!     run through all groups
!         if line elements
!           place nodes in special "line" array
!         else if ( ishape==3 .or. ishape.eq.5 .or. .not. vertex .and. &$
!                 ( ishape>=4 .and. ishape<8 ) ) then
!           linear triangles or quadrilaterals
!           or quadratic ones place all nodes in element array
!           (if spectral element use special values)
!         else if ( ishape>=4 .and. ishape<=7 ) then
!           quadratic triangles or quadrilaterals (place vertices only)
!         else if ( ishape==11 .or. ishape.eq.13 .or. .not. vertex
!             .and. ishape>10 ) then
!           linear tetrahedron or hexahedron or 3D element with all points
!           (if spectral element use special values)
!         else if ( ishape>=12 ) then
!           3D elements, vertices only
!        end if
!     Do the same loop once again for surface elements if surfaces and
!     volumes are considered in order to cancel later the correct
!     elements
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepqu8obst' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepqu8obst'
         write(irefwr,1) 'nelgrp', nelgrp
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000


      ilow   = 0
      ihigh  = 0
      neleml = 0

!     --- determine whether triangles or quadrilaterals are used

      triang = .false.
      quadrt = .false.
      sevenp = .false.

      do ielgrp = 1, nelgrp

         ishape = sepgetshape ( ielgrp )
         if ( debug ) write(irefwr,1) 'ielgrp, ishape', ielgrp, ishape

         if ( ishape>=3 .and. ishape<10 ) ilow  = 1
         if ( ishape>=10 ) ihigh = 1

         if ( ishape== 3 .or. ishape.eq. 4 .or. ishape.eq. 7 .or. &
              ishape== 8 .or. ishape.eq.11 .or. ishape.eq.12 .or. &
              ishape>=15 .or. ishape==18 ) triang = .true.
         if ( ishape== 4 .or. ishape.eq. 6 .or. ishape.eq. 7 .or. &
              ishape== 8 .or. ishape.eq.12 .or. ishape>=14 ) &
              quadrt = .true.
         if ( ishape== 7 .or. ishape.eq.15 .or. ishape.eq.16 ) &
              sevenp = .true.

      end do

      bound  = ilow + ihigh==2
      ikelmc = 0
      ikmeshcnew = 0
      ntria  = 0
      nelext = 0

      if ( triang ) then
         if ( .not. vertex ) then
            if ( sevenp ) then
               npelnw = 7
            else if ( quadrt ) then
               npelnw = 6
            else
               npelnw = 3
            end if
         else
            npelnw = 3
         end if
      else if ( quadrt .and. .not. vertex ) then
         npelnw = 9
      else
         npelnw = 4
      end if

      if ( debug ) write(irefwr,1) 'npelnw, iskip', npelnw, iskip

      ielem = 0
      if ( iskip>=0 ) then

!     --- Standard case, run through all groups

         do ielgrp = 1, nelgrp

            ishape = sepgetshape ( ielgrp )
            inelem = Topgrp(ielgrp)%inelem
            inpelm = Topgrp(ielgrp)%inpelm

            if ( debug ) &
               write(irefwr,1) 'ishape, inelem, inpelm', &
                                ishape, inelem, inpelm
            if ( iskip>0 ) then

!           --- Skip element groups if necessary

               if ( iskpar(ielgrp)==0 ) then

!              --- Skip this group

                  ikelmc = ikelmc + inelem * inpelm
                  ielem = ielem+inelem
                  go to 100

               end if

            end if

            if ( ishape<3 ) then

!           --- line elements, some elements may be skipped
!               place nodes in special "line" array

               if ( ishape==1 ) then

!              --- Linear lines

                  do i = 1, inelem
                     ielem = ielem+1
                     if ( elemskip(ielem)==ilevel ) then

!                    --- Element must not be skipped

                        kline(2*neleml+1) = kmeshc(ikelmc+1)
                        kline(2*neleml+2) = kmeshc(ikelmc+2)
                        neleml = neleml+1

                     end if
                     ikelmc = ikelmc+2
                  end do

               else if ( ishape==2 ) then

!              --- Quadratic lines (vertices only)

                  do i = 1, inelem
                     ielem = ielem+1
                     if ( elemskip(ielem)==ilevel ) then

!                    --- Element must not be skipped

                        kline(2*neleml+1) = kmeshc(ikelmc+1)
                        kline(2*neleml+2) = kmeshc(ikelmc+3)
                        neleml = neleml+1

                     end if
                     ikelmc = ikelmc+3
                  end do

               end if

            else if ( ishape==3 .or. ishape.eq.5 .or. .not. vertex &
                       .and.( ishape>=4 .and. ishape<8 ) ) then

!           --- linear triangles or quadrilaterals
!               or quadratic ones all points

               if ( debug ) then
                  write(irefwr,1) 'inelem, ielem, ilevel', inelem, ielem, ilevel
                  write(irefwr,1) 'ikelmc, ikmeshcnew', ikelmc, ikmeshcnew
                  call prinin ( elemskip, inelem, 'elemskip' )
                  call prinin1 ( kmeshc(ikelmc+1), inelem, inpelm, 'kmeshc' )
                  call prinin ( elemobst, inelem, 'elemobst' )
               end if  ! ( debug )

               do i = 1, inelem
                  ielem = ielem+1
                  if ( elemskip(ielem)==ilevel ) then

!                 --- Element must not be skipped

                     do j = 1, inpelm
                        kmeshctmp(ikmeshcnew+j) = kmeshc(ikelmc+j)
                     end do
                     ikmeshcnew = ikmeshcnew + npelnw
                     ntria  = ntria  + 1
                     if ( bound ) nelext = nelext + 1
                     itriacolour(ntria) = elemobst(ielem)

                  end if
                  ikelmc = ikelmc+inpelm
               end do

            else if ( ishape>=4 .and. ishape<=7 ) then

!           --- quadratic triangles or quadrilaterals (vertices only)

               do i = 1, inelem
                  ielem = ielem+1
                  if ( elemskip(ielem)==ilevel ) then

!                 --- Element must not be skipped

                     do j = 1, npelnw
                        kmeshctmp(ikmeshcnew+j) = kmeshc(ikelmc+2*j-1)
                     end do
                     ikmeshcnew = ikmeshcnew + npelnw
                     ntria  = ntria  + 1
                     if ( bound ) nelext = nelext + 1
                     itriacolour(ntria) = elemobst(ielem)

                  end if
                  ikelmc = ikelmc+inpelm
               end do

            else if ( ishape==11 .or. ishape.eq.13 .or. ishape>=17 &
                      .or. .not. vertex .and. ishape>10 ) then

!           --- linear tetrahedron or hexahedron or 3D element with all points

               nsurfe = itypbs(2,ishape-10)
               istare = itypbs(3,ishape-10)-1

               if ( debug ) then
                  write(irefwr,1) 'inelem, ielem, ilevel', inelem, ielem, ilevel
                  write(irefwr,1) 'ikelmc, ikmeshcnew', ikelmc, ikmeshcnew
                  write(irefwr,1) 'nsurfe, istare', nsurfe, istare
                  call prinin ( elemskip, inelem, 'elemskip' )
                  call prinin1 ( kmeshc(ikelmc+1), inelem, inpelm, 'kmeshc' )
                  call prinin ( elemobst, inelem, 'elemobst' )
               end if  ! ( debug )

               do i = 1, inelem
                  ielem = ielem+1
                  if ( elemskip(ielem)==ilevel ) then

!                 --- Element must not be skipped

                     do j = 1, nsurfe * npelnw
                        kmeshctmp(ikmeshcnew+j) = &
                           kmeshc(ikelmc+iboun3(istare+j))
                     end do
                     ikmeshcnew = ikmeshcnew + nsurfe * npelnw
                     do  j = 1, nsurfe
                        itriacolour(ntria+j) = elemobst(ielem)
                     end do
                     ntria  = ntria  + nsurfe

                  end if
                  ikelmc = ikelmc+inpelm
               end do

            else if ( ishape>=12 ) then

!           --- 3D elements, vertices only

               npelmc = itypbs(1,ishape-10)
               nsurfe = itypbs(2,ishape-10)
               istare = itypbs(3,ishape-10)-1

               do i = 1, inelem
                  ielem = ielem+1
                  if ( elemskip(ielem)==ilevel ) then

!                 --- Element must not be skipped

                     istart = istare
                     do j = 1, nsurfe
                        do k = 1, npelnw
                           kmeshctmp(ikmeshcnew+k) = &
                           kmeshc(ikelmc+iboun3(istart+2*k-1))
                        end do
                        ikmeshcnew = ikmeshcnew + npelnw
                        ntria  = ntria  + 1
                        itriacolour(ntria) = elemobst(ielem)
                        istart = istart + npelmc
                     end do

                  end if
                  ikelmc = ikelmc + inpelm
               end do

            end if

100      end do

         if ( nelext==0 ) go to 1000

         ikelmc = 0

!        --- extra for boundary elements

         do ielgrp = 1, nelgrp

            ishape = sepgetshape ( ielgrp )
            inelem = Topgrp(ielgrp)%inelem
            inpelm = Topgrp(ielgrp)%inpelm

            if ( ishape==3 .or. ishape.eq.5 .or. .not. vertex .and. &
                 ( ishape>=4 .and. ishape<8 ) ) then

!           --- linear triangles or quadrilaterals
!               or quadratic ones all points

               do i = 1, inelem
                  ielem = ielem+1
                  if ( elemskip(ielem)==ilevel ) then

!                 --- Element must not be skipped

                     do j = 1, inpelm
                        kmeshctmp(ikmeshcnew+j) = kmeshc(ikelmc+j)
                     end do
                     ikmeshcnew = ikmeshcnew + npelnw
                     ntria  = ntria  + 1
                     itriacolour(ntria) = elemobst(ielem)

                  end if
                  ikelmc = ikelmc+inpelm
               end do

            else if ( ishape>=4 .and. ishape<=7 ) then

!           --- quadratic triangles or quadrilaterals (vertices only)

               do i = 1, inelem
                  ielem = ielem+1
                  if ( elemskip(ielem)==ilevel ) then

!                 --- Element must not be skipped

                     do j = 1, npelnw
                        kmeshctmp(ikmeshcnew+j) = kmeshc(ikelmc+2*j-1)
                     end do
                     ikmeshcnew = ikmeshcnew + npelnw
                     itriacolour(ntria) = elemobst(ielem)
                     ntria  = ntria  + 1

                  end if
                  ikelmc = ikelmc+inpelm
               end do

            else

               ikelmc = ikelmc + inelem * inpelm

            end if

         end do
         if ( ntria==0 ) then

!        --- Error no active element groups found

            call errsub ( 2204, 0, 0, 0 )

         end if

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepqu8obst'

      end if  ! ( debug )
      call erclos ( 'sepqu8obst' )

      end subroutine sepqu8obst
