      function sepstartnode()
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-05-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find start node in a set of nodes (pseudo-peripheral node) stored in set
!     Old subroutine chstartnode
! **********************************************************************
!
!                       KEYWORDS
!
!     renumbering
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmodulekmesh
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: sepstartnode

!     sepstartnode   o    Starting node number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: levellength, newnode, levellennew, i, startnode, j
      logical :: finished

!     finished       Indicates if the loop has finished (true) or not
!                    (false)
!     i              Node number
!     j              Loop variable
!     levellength    Length of rooted level structure
!     levellennew    New length of rooted level structure
!     newnode        If true a new node has been created
!     startnode      Starting node number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: sepmindeg

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     SEPMINDEG      Find node with minimum degree in a set
!     SEPROOTLEVEL   Find rooted level structure corresponding to node
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3207   No starting node found
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Start with arbitrary node (r)
!     Construct rooted level structure and length l(r)
!     finished := false
!     while ( not finished ) do
!        Find node x with min degree in last level
!        Construct rooted level structure and length l(x)
!        if ( l(x)>l(r) ) then
!           r := x;  l(r):=l(x)
!        else
!           finished := true
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepstartnode' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepstartnode'
         write(irefwr,1) 'npoint', npoint
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Start with arbitrary node (r)

      startnode = 0
      do j = 1, size(set)
         i = set(j)
         if ( marknodes(i)>-10 ) then

!        --- new node found

            startnode = i
            exit

         end if  ! ( marknodes(i)>-10 )

      end do  ! i = ifirst, ilast

      if ( startnode==0 ) then

!     --- No starting node could be found, the set is empty

         call errsub ( 3207, 0, 0, 0 )
         go to 1000

      end if  ! ( startnode==0 )

!     --- Clear array marknodes

      do j = 1, size(set)
         i = set(j)
         if ( marknodes(i)>-10 ) marknodes(i) = 0
      end do  ! i = 1, npoint

!     --- Construct rooted level structure and length l(r)

      call seprootlevel ( startnode, levellength )
      if ( ierror/=0 ) go to 1000

!     --- finished := false

      finished = size(lastlevel)==0

!     --- while ( not finished ) do

      do while ( .not. finished )

!     --- Find node x with min degree in last level

         newnode = sepmindeg()
         if ( ierror/=0 ) go to 1000

!        --- Clear array marknodes

         do j = 1, size(set)
            i = set(j)
            if ( marknodes(i)>-10 ) marknodes(i) = 0
         end do  ! i = 1, npoint

!        --- Construct rooted level structure and length l(x)

         call seprootlevel ( newnode, levellennew )
         if ( ierror/=0 ) go to 1000

         if ( levellennew>levellength ) then

!        --- l(x)>l(r)
!            r := x;  l(r):=l(x)

            levellength = levellennew
            startnode = newnode

         else

!        --- l(x)<=l(r)

           finished = .true.

         end if  ! ( levellennew>levellength )

      end do  ! while ( .not. finished )

      sepstartnode = startnode

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepstartnode'

      end if  ! ( debug )
      call erclos ( 'sepstartnode' )

      end function sepstartnode
