      subroutine topbv1
! ======================================================================
!
!        programmer    Guus Segal
!        version 13.0  date 14-04-2015 Use subroutines
!        version 12.4  date 14-01-2013 extra debug
!        version 12.3  date 15-06-2012 Use allocate for work arrays
!
!   copyright (c) 1984-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fills array kprob parts h, r and s
!     KPROB h contains and the renumbered sequence of the unknowns
!     and the inverse of it
!     Warning:  TOPBV1 uses memory management subroutines, and should therefore
!               handled with care
! **********************************************************************
!
!                       KEYWORDS
!
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulemain
      use sepmodulecpack
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical period
      integer, parameter :: lenrenum=20
      integer i, maxren, nelgrplc, lenistore, lenkprobab, &
              irenum(lenrenum)
      integer, allocatable :: iwork(:), istoreleft(:)
      double precision, allocatable :: jumps(:), factors(:)

!     factors        Array containing the factors for periodical boundary
!                    conditions
!     i              Counting variable
!     irenum         This array contains the levels of the renumbering of all
!                    unknowns per point
!                    So if iren(i)=j then unknown i is numbered in the j-th
!                    level
!                    Unknowns with sequence number larger than maxren
!                    correspond always to the largest level
!     istoreleft     Contains all periodical right neighbor unknowns
!                    for all left periodical neighbor unknowns
!                    Pos. 1 to nusol+1 contain pointers to the second part
!                    istoreleft(1) contains nusol+1
!                    The number of neighbors of unknown i is stored in
!                    istoreleft(i+1)-istoreleft(i)
!                    The second part contains the neighbors
!     iwork          integer work array of length nusol in which the information
!                    about the essential boundary conditions is stored
!                    if iwork(i) = 0, then unknown i is not prescribed
!                    if iwork(i) = 1, then unknown i is prescribed
!     jumps          Array of size nusol containing the jumps
!                    for the periodical boundary conditions
!     lenistore      Estimated length of arrays istoreleft and istoreright
!     lenkprobab     Length of array kprobab
!     lenrenum       Declared length of array irenum
!     maxren         Maximum number of renumbered unknowns
!     nelgrplc       Number of element groups
!     period         Indicates if periodical boundary conditions are
!                    present (true) or not (false)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     PRININ         print 1d integer array
!     SEPPRINKPROB1  Print array KPROB for one problem
!     TOPBV1KPROBR   Fill arrays kprob parts r and s
!     TOPBV1NOKPROBH Array kprobh will not be filled since no renumbering
!                    required
!                    Remove superfluous arrays
!     TOPBV1PERIOD   Nodal points of which the unknowns are connected through
!                    periodical boundary conditions (possibly with jump) are
!                    marked
!     TOPBV1REN      Perform renumbering if necessary and fill array irenum
!     TOPBV2         Fill array IWORK with numbers of prescribed degrees of
!                    freedom
!     TOPBV3         Fill KPROB part h
!     TOPBVPERIOD    Fill array kprob part ab with pairs of periodical degrees
!                    of freedom
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1878   There are no free unknowns in this problem left
!    2550   Parallel in combination with periodical boundary conditions
!           not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topbv1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from topbv1'
         call prinin ( kprobloc, 80, 'kprob' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- initializations

      kprobc => kprobpart%kprobc
      kprobf => kprobpart%kprobf
      kprobg => kprobpart%kprobg
      kprobp => kprobpart%kprobp
      nunkp  = kprobloc(4)
      nusol  = kprobloc(5)
      nkbnd  = kprobloc(9)
      indprx = kprobloc(11)
      indprf = kprobloc(19)
      nphys = kprobloc(33)
      indprp = kprobloc(35)
      period = iinputkp(15)>1  ! version number input larger than 1
      if ( period ) period = iinputkp(52)>0  ! periodical bc's

      if ( .not. period ) then

!     --- iinputkp(52) = 0, so no periodical boundary conditions according to
!         the new description
!         However, in the old description we use connection elements
!         with type number -1

         do i = 1, nelgrp
            if ( kprobc(i)==-1) period = .true.
         end do  ! i = 1, nelgrp

      else if ( parallel ) then

!     --- Parallel in combination with periodical boundary conditions not yet
!         implemented

         call errsub ( 2550, 0, 0, 0 )
         go to 1000

      end if  ! ( .not. period )

      if ( debug ) then
         write(irefwr,*) 'period ', period
         write(irefwr,1) 'kelmj, nkbnd, nunkp, indprx', &
                          kelmj, nkbnd, nunkp, indprx
      end if  ! ( debug )

!     --- Check if the arrays should be filled

      if ( kelmj==0 .and. indprf==0 .and. nkbnd==0 .and. &
           nunkp==1 .and. .not. period .and. indprx==0 .and. &
           .not. parallel ) then

!     --- In this special case kprobh is not filled,
!         neither are the arrays kprobr and kprobs

         call topbv1nokprobh
         go to 1000

      else if ( .not. period ) then

         kprobloc(34) = 0

      end if

!     --- reserve work space
!         This work space is stored at the end of the buffer since due to
!         a call to topol addresses may be changed

      allocate ( iwork(nusol), stat = error )
      if ( error/=0 ) call eralloc ( error, nusol, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Fill positions in iwork corresponding to prescribed boundary
!         conditions

      call topbv2 ( iwork )

      if ( period ) then

!     --- Mark nodal points with periodical boundary conditions in iwork

         lenistore = 3*nusol
         allocate ( jumps(nusol), factors(nusol), &
                    istoreleft(lenistore), stat = error )
         if ( error/=0 ) &
            call eralloc ( error, 5*nusol, 'jumps' )
         if ( ierror/=0 ) go to 1000

         call topbv1period ( iwork, jumps, factors, istoreleft, &
                             lenistore )

      end if

      if ( period ) then

!     --- Periodical boundary conditions, store left and right degrees of
!         freedom in kprob part AB
!         First reserve temporary space for kprobab

         lenkprobab = 3*nusol
         allocate ( kprobab(lenkprobab), stat = error )
         if ( error/=0 ) call eralloc ( error, lenkprobab, 'kprobab' )
         if ( ierror/=0 ) go to 1000
         call topbvperiod ( lenkprobab, istoreleft )
         if ( ierror/=0 ) go to 1000

!        --- Make array kprobab permanent

         if ( associated(kprobpart%kprobab) ) &
            deallocate(kprobpart%kprobab )
         allocate(kprobpart%kprobab(lenkprobab))
         kprobpart%kprobab = kprobab(1:lenkprobab)
         deallocate ( kprobab )
         kprobab => kprobpart%kprobab
         kprobloc(58) = lenkprobab
         indprab = lenkprobab

         deallocate ( istoreleft, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'istoreleft' )
         if ( ierror/=0 ) go to 1000

      end if

!     --- Reserve space for new permanent arrays

      if ( associated(kprobpart%kprobh) ) deallocate(kprobpart%kprobh)
      if ( associated(kprobpart%kprobhinv) ) deallocate(kprobpart%kprobhinv)
      allocate(kprobpart%kprobh(nusol), kprobpart%kprobhinv(nusol))
      kprobh => kprobpart%kprobh
      kprobhinv => kprobpart%kprobhinv
      kprobloc(50) = nusol
      indprh = kprobloc(50)

!     --- Perform renumbering if necessary and fill array irenum

      call topbv1ren ( irenum, lenrenum, maxren )

!     --- Fill array kprobh and renumber KPROBP

      call topbv3 ( iwork, maxren, irenum )
      nbound = kprobloc(10)
      npboun = kprobloc(34)

      if ( nusol-nbound-npboun<=0 ) then

!     --- Warning: no unknowns for this problem

         call errint ( nusol, 1 )
         call errint ( nbound, 2 )
         call errint ( npboun, 3 )
         call errwar ( 1878, 3, 0, 0 )

      end if

      if ( npboun==0 ) then

!     --- npboun, no periodical boundary conditions, kprobrlc  =  kprobh

         if ( kprobloc(38)>0 ) then
            deallocate ( kprobpart%kprobs, &
                         kprobpart%rprobs )
            kprobloc(38) = 0
         end if  ! ( kprobloc(38)>0 )
         kprobloc(37) = 0
         kprobpart%kprobr => kprobpart%kprobh
         kprobr => kprobpart%kprobr
         indprr = indprh

      else

!     --- npboun>0, so there are periodical boundary conditions
!         Read standard array rinputkp
!         Fill arrays kprob parts r and s

         call topbv1kprobr ( iwork, factors, jumps )

         deallocate ( jumps, factors, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'jumps' )
         if ( ierror/=0 ) go to 1000

      end if

      if ( kprobloc(50)>0 ) then

!     --- kprobh has been filled, store inverse in second part

         do i = 1, nusol
            kprobhinv(kprobh(i)) = i
         end do

      end if

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         call sepprinkprob1 ( 14 )
         call sepprinkprob1 ( 23 )
         call sepprinkprob1 ( 24 )
         call sepprinkprob1 ( 52 )
         write(irefwr,*) 'End topbv1'

      end if  ! ( debug )
      call erclos ( 'topbv1' )

      end subroutine topbv1
