      subroutine topbv3glob ( nusolclassic, iwork, jump )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-04-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill last part of kprobh for global unknowns
!
! **********************************************************************
!
!                       KEYWORDS
!
!     problem_definition
!     renumbering
!     boundary_condition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulecpack
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: nusolclassic
      integer, intent(inout) ::  iwork(*)
      logical, intent(out) :: jump

!     iwork         i/o   Output of subroutine TOPBV2
!                         iwork(i)=1, means essential boundary condition for
!                                     unknown i
!                         iwork(i)=0, means unknown i is free
!                         iwork(i)<0, means unknown i corresponds to
!                         periodical bc.
!     jump           o    if true nrusol should not be adapted after the call
!     nusolclassic   i    Number of free classical unknowns, i.e fictitious
!                         unknowns
!                         and global unknowns excluded
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, iinp10, inusol, inodp, icoupl, inunkp, j, ihelp(2*nphys)

!     i              Counting variable
!     icoupl         number of coupled degrees of freedom found
!     ihelp          work array
!     iinp10         first pos. iinputkp extra information
!     inodp          Nodal point number
!     inunkp         Number of degrees of freedom in point
!     inusol         Position of unknown in array USOL
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2593   Global unknowns have not yet been implemented in the parallel case
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topbv3glob' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from topbv3glob'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( parallel ) then

!     --- parallel has not yet been implemented

         call errsub ( 2593, 0, 0, 0 )
         go to 1000

      end if  ! ( parallel )

      nphys = max(nphys,nunkp)
      inunkp = nunkp

      jump = .false.

      do i = 1, nglunk
         kprobh(nusolclassic+i) = nrusol+i
      end do
      if ( nglcoupl>0 ) then

!     --- There are coupled unknowns, in this case the last physical degrees
!         must be renumbered
!         First the degrees of freedom that are coupled are stored into
!         ihelp(nphys+1...2*nphys)
!         0 means not coupled, 1 means coupled

         do i = nphys+1, 2*nphys
            ihelp(i) = 0
         end do
         iinp10 = intarprb(10)
         do i = 1, nglcoupl
            ihelp(nphys+iinputkp(iinp10+i)) = 1
         end do

!        --- Next we try to find the degree of freedom in the nodal point
!            with the highest (renumbered) node number
!            The degree of freedom in that position gets a new number
!            iwork is made positive

         nrusol = nrusol+nglunk
         if ( indprp>0 ) then

!        --- indprp>0, physical degrees of freedom

            do i = 1, nphys

               if ( ihelp(nphys+i)>0 ) then

!              --- degree of freedom must be renumbered

                  do j = npoint, 1, -1

                     if ( kelmj>0 ) then
                        inodp = kmeshj(j)
                     else
                        inodp = j
                     end if
                     inusol = kprobp(inodp,i)
                     if ( inusol>0) then

!                    --- Point has been found, check if it is prescribed

                        if ( iwork(inusol)==0 ) then

!                       --- Point has been found, fill arrays and skip

                           nrusol = nrusol+1
                           kprobh(inusol) = nrusol
                           iwork(inusol) = nbound+1
                           go to 100

                        end if

                     end if

                  end do

100            end if

            end do

         else if ( indprf>0 ) then

!        --- kprobp = 0, kprobf has been filled

            icoupl = 0
            do j = npoint, 1, -1

               if ( kelmj>0 ) then
                  inodp = kmeshj(j)
               else
                  inodp = j
               end if
               inusol = kprobf(inodp)
               inunkp = kprobf(inodp+1)-inusol
               do i = 1, inunkp

                  if ( iwork(inusol+i)==0 .and. &
                       ihelp(nphys+i)>0 ) then

!                 --- Point has been found, fill arrays

                     nrusol = nrusol+1
                     kprobh(inusol+i) = nrusol
                     iwork(inusol+i) = nbound+1
                     ihelp(nphys+i) = 0
                     icoupl = icoupl+1
                     if ( icoupl>=nglcoupl ) then
                        jump = .true.
                        go to 1000
                     end if  ! ( icoupl>=nglcoupl )

                  end if

               end do

            end do

         else

!        --- kprobp = 0, indprf = 0

            icoupl = 0
            do j = npoint, 1, -1

               if ( kelmj>0 ) then
                  inodp = kmeshj(j)
               else
                  inodp = j
               end if
               inusol = (j-1)*nunkp
               do i = 1, nunkp

                  if ( iwork(inusol+i)==0 .and. &
                       ihelp(nphys+i)>0 ) then

!                 --- Point has been found, fill arrays

                     nrusol = nrusol+1
                     kprobh(inusol+i) = nrusol
                     iwork(inusol+i) = nbound+1
                     ihelp(nphys+i) = 0
                     icoupl = icoupl+1
                     if ( icoupl>=nglcoupl ) then
                        jump = .true.
                        go to 1000
                     end if  ! ( icoupl>=nglcoupl )

                  end if

               end do

            end do

         end if

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End topbv3glob'

      end if  ! ( debug )
      call erclos ( 'topbv3glob' )

      end subroutine topbv3glob
