      subroutine topbv4
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 09-03-2013 Correction check
!        version  1.1  date 27-02-1999 Correct istart
!        version  1.0  date 15-01-1992
!
!   copyright (c) 1992-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!       Compute renumbering of nodes in the case of spectral elements
!       A simple Cuthill-McKee algorithm based on elements is used
! **********************************************************************
!
!                       KEYWORDS
!
!     renumbering
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulework
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielem, istart, ielgrp, nbound, &
              maxnei, numnei, iielem, i, nneigh, ielstr, ifirst, ilast, &
              ineigh, numnbr, nodp, nnodes, neltot, inpelm, &
              inodp, levels(npoint)
      integer, allocatable :: ielcon(:)
      logical check

!    check     Indicates if a condition is satisfied
!    i         general loop variable
!    ielcon    Work array of length max(npoint,nelem) in which first
!              information about the elements that have been considered is
!              stored.
!              ielcon(i) = 0, means element i has not been treated
!              ielcon(i)>0, means element i belongs to level ielcon(i)
!              In the second part it indicates if a nodal point has been
!              treated (ielcon(nodal_point_number)>0) or not (=0)
!    ielem     element number
!    ielgrp    element group sequence number
!    ielstr    Starting element
!    ifirst    Position of first element of present level in array iwork
!    iielem    relative element number in a group
!    ilast     Position of last element of present level in array iwork
!    ineigh    Neighbour number
!    inodp     Loop variable for nodes
!    inpelm    number of nodes in an element
!    istart    Help variable to compute a starting address
!    levels    Array of length nlevel in which the number of nodes per
!              level is stored accumulated
!    maxnei    Maximum number of neighbours of an element
!    nbound    Number of essential boundary conditions
!    neltot    Total number of elements with neighbours as stored in kmesho
!    nneigh    actual number of neighbours of an element
!    nnodes    Number of nodes
!    nodp      Nodal point sequence number
!    numnbr    number of active neighbours of the elements in a level
!    numnei    number of neighbours of elements in an element group
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   First order elements:
!   Find element with smallest number of neighbours (element_start)
!   Clear array elements_considered
!   ilast := 1
!   iwork(ilast) := element_start
!   ifirst := 1
!   levels(0) := 0
!   levels(1) := 1
!   nlevel := 1
!   elements_considered(element_start) := 1
!   while ilast<nelem do
!      nlevel := nlevel + 1
!      number_neigh := number of neigbours of all elements in
!                  iwork(ifirst to ilast) satisfying
!                  elements_considered(element_number) = 0
!      ifirst := ilast
!      if ( number_neigh=0 ) then
!         find element with smallest number of neighbours satisfying
!              elements_considered(element_number) = 0 (element_start)
!         number_neigh := 1
!         iwork(ilast) := element_start
!      else ( number_neigh>0 )
!         Store these neighbours in iwork positions number to number+nlevel-1
!      ilast := ilast + number_neigh
!      levels(nlevel) := number_neigh + levels(nlevel-1)
!      Set elements_considered for the last number_neigh elements equal to 1
!     Next order nodes:
!     Clear array nodes
!     number_of_nodes := 0
!     For i := 1 (1) nlevel do
!          nstore := number of nodes in all elements in
!                    iwork ( levels(i-1)+1 to levels(i) )
!                    not satisfying nodes(node) = 0
!          Store the nstore nodes in next positions of kmeshj
!          Make array nodes equal to one for these nodes
!          number_of_nodes := number_of_nodes + nstore
!          levels(i) := number_of_nodes
!     If ( not all nodes have been stored in kmeshj ) then
!        nlevel := nlevel + 1
!        levels(nlevel) := npoint
!        Store remaining nodes in kmeshj
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topbv4' )

!     --- initialisations

      nbound = kmsho1(1)
      inelem = nelem
      neltot = inelem

      allocate ( ielcon(max(npoint, nelem)) )

!     --- Fill arrays iwork1 and iwork2 if necessary

      if ( nelgrp>1 ) then

!     --- nelgrp>1

         ielem = 0
         neltot = 0
         istart = 0

         do ielgrp = 1, nelgrp

            nbound = kmsho1(ielgrp)
            inelem = kmeshg(ielgrp)
            if ( nbound>0 ) neltot = neltot + inelem

            do iielem = 1, inelem

               iwork1(ielem+iielem) = ielgrp
               iwork2(ielem+iielem) = istart + (iielem-1)*nbound

            end do

            ielem = ielem + inelem
            istart = istart + inelem*nbound

         end do

      end if

!     --- Find element with smallest number of neighbours (element_start)

      maxnei = nelem + 1
      istart = 0
      ielem  = 0

!     --- Loop over element groups

      do ielgrp = 1, nelgrp

         numnei = kmsho1(ielgrp)
         if ( numnei>0 ) then

!        --- numnei>0

            inelem = Topgrp(ielgrp)%inelem

            do iielem = 1, inelem

               nneigh = 0

               do i = 1, numnei

                  if ( kmsho2(istart+i)/=0 ) nneigh = nneigh + 1

               end do
               istart = istart + numnei
               if ( maxnei>nneigh ) then

!              --- new element found

                  maxnei = nneigh
                  ielstr = ielem+iielem

               end if

            end do

         end if

      end do

!     --- Clear array elements_considered

      do i = 1, inelem
         ielcon(i) = 0
      end do

      ilast = 1
      ifirst = 1
      iwork(ilast) = ielstr
      nlevel = 1
      levels(nlevel) = 1
      ielcon(ielstr) = 1

!     --- while loop ilast<neltot

200   if ( ilast<neltot ) then

!     --- ilast<inelem, not all elements found

         nlevel = nlevel + 1

!        --- Compute the number of neigbours of all elements in
!            iwork(ifirst to ilast) with ielcon(i) = 0

         ielgrp = 1
         numnbr = 0
         do ielem = ifirst, ilast

            if ( nelgrp>1 ) ielgrp = iwork1(ielem)
            numnei = kmsho1(ielgrp)
            if ( nelgrp>1 ) then
               istart = iwork2(ielem)
            else
               istart = (ielem-1) * nbound
            end if

            do i = 1, numnei

               ineigh = mod( kmsho2(istart+i), nelem+1 )
               check = ineigh>0
               if ( check ) check = ielcon(ineigh)==0
               if ( check ) then

!              --- new neighbour found

                  ielcon(ineigh) = nlevel
                  numnbr = numnbr + 1
                  iwork(ilast+numnbr) = ineigh

               end if

            end do

         end do

         if ( numnbr==0 ) then

!        --- No neighbours found, &
!            find element with smallest number of neighbours satisfying
!            elements_considered(element_number) = 0 (element_start)

            maxnei = nelem + 1
            istart = 0
            ielem  = 0

!           --- Loop over element groups

            do ielgrp = 1, nelgrp

               numnei = kmsho1(ielgrp)
               if ( numnei>0 ) then

!              --- numnei>0

                  inelem = Topgrp(ielgrp)%inelem

                  do iielem = 1, inelem

                     nneigh = 0
                     if ( ielcon(ielem+iielem)==0 ) then

!                    --- new element found

                        do i = 1, numnei

                           if ( kmsho2(istart+i)/=0 ) nneigh=nneigh+1

                        end do

                     end if

                     istart = istart + numnei
                     if ( maxnei>nneigh ) then

!                    --- new element found

                        maxnei = nneigh
                        ielstr = ielem+iielem

                     end if

                  end do

               end if

            end do

            numnbr = 1
            iwork(ilast+numnbr) = ielstr

         end if

         ifirst = ilast + 1
         ilast  = ilast + numnbr
         levels(nlevel) = ilast
         go to 200

      end if

!     --- Next order nodes
!         Clear array nodes

      do i = 1, npoint
         ielcon(i) = 0
      end do

      if ( nelgrp>1 ) then

!     --- Fill array iwork2 with respect to kmeshc

         ielem = 0
         istart = 0

         do ielgrp = 1, nelgrp

            inpelm = Topgrp(ielgrp)%inpelm
            inelem = Topgrp(ielgrp)%inelem

            do iielem = 1, inelem

               iwork2(ielem+iielem) = istart + (iielem-1)*inpelm

            end do

            ielem = ielem + inelem
            istart = istart + inelem*inpelm

         end do

      end if

      ifirst = 1
      nnodes = 0

!     --- Loop over levels

      do i = 1, nlevel

         ilast = levels(i)

!        --- Find all nodes in levels i

         do iielem = ifirst, ilast

            ielem = iwork(iielem)
            if ( nelgrp>1 ) then

!           --- nelgrp>1

               istart = iwork2(ielem)
               ielgrp = iwork1(ielem)
               inpelm = Topgrp(ielgrp)%inpelm

            else

!           --- nelgrp = 1

               istart = (ielem-1) * inpelm

            end if
            do inodp = 1, inpelm

               nodp = kmeshc(istart+inodp)
               if ( ielcon(nodp)==0 ) then

!              --- new node has been found

                  nnodes = nnodes + 1
                  kmeshj(nnodes) = nodp
                  ielcon(nodp) = 1

               end if

            end do

         end do

         levels(i) = nnodes

      end do

      if ( nnodes<npoint ) then

!     --- nnodes<npoint, not all nodes have been treated
!         Store remaining points in last level

         do i = 1, npoint

            if ( ielcon(i)==0 ) then
               nnodes = nnodes + 1
               kmeshj(nnodes) = i
            end if

         end do

         nlevel = nlevel + 1
         levels(nlevel) = npoint

      end if
      deallocate ( ielcon )

      call erclos ( 'topbv4' )

      end
