      subroutine topbv7021 ( nodp1, nodp2, indprf, kprobf, idegper, &
                             nunkp, istoreleft, istoreright, rjumpleft, &
                             rmultleft, istartleft, istartright, &
                             rjumploc, rmultloc, iwork, rjumpright, &
                             rmultright, itemp, nusol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-06-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill rest of arrays istoreleft and istoreright
!     Fill jumps in array jumpleft and jumpright and
!     multipication factors in rmultleft and rmultfact
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     periodical
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nodp1, nodp2, indprf, kprobf(*), nunkp, idegper(nunkp), &
              istoreleft(*), istoreright(*), istartleft(*), &
              istartright(*), iwork(*), itemp(*), nusol
      double precision rjumpleft(*), rmultleft(*), rjumploc(*), &
                       rmultloc(*), rmultright(*), rjumpright(*)

!     idegper        i    Array of size nunkp containing a 1 if the
!                         corresponding unknown is periodical and a 0 if not
!     indprf         i    indication if kprob part f has been filled (>0)
!                         or not (0)
!     istartleft    i/o   Array of size nusol that is used to keep track of the
!                         last position used in istoreleft w.r.t. unknown i
!     istartright   i/o   Array of size nusol that is used to keep track of the
!                         last position used in istoreright w.r.t. unknown i
!     istoreleft    i/o   Contains all periodical right neighbour unknowns
!                         for all left periodical neighbour unknowns
!                         Pos. 1 to nusol+1 contain pointers to the second part
!                         istoreleft(1) contains nusol+1
!                         The number of neighbours of unknown i is stored in
!                         istoreleft(i+1)-istoreleft(i)
!                         The second part contains the neighbours
!                         At input the first part must have been filled
!     istoreright   i/o   Contains all periodical left neighbour unknowns
!                         for all right periodical neighbour unknowns
!                         Pos. 1 to nusol+1 contain pointers to the second part
!                         istoreright(1) contains nusol+1
!                         The number of neighbours of unknown i is stored in
!                         istoreright(i+1)-istoreright(i)
!                         The second part contains the neighbours
!                         At input the first part must have been filled
!     itemp         i/o   Temporaray work array of length nusol to indicate
!                         if an unknown corresponds to a "left" periodical
!                         boundary condition (1) or not (0)
!                         At input itemp must have been cleared
!     iwork          i    Semi-permanent work array of length nusol in which
!                         information about essential boundary conditions and
!                         periodical boundary conditions is stored
!                         iwork is filled as follows:
!                         iwork(i) = 0: means unknown i is free
!                         iwork(i) > 0: means unknown i is prescribed
!                                       (essential bc)
!                         iwork(i) < 0: means unknown i refers to unknown
!                                       iwork(i), which means that unknown i
!                                       has the same value, possibly
!                                       extended with a jump
!     kprobf         i    KPROB part f: contains the number of unknowns per
!                         point accumulated
!     nodp1          i    Node sequence number of first point
!     nodp2          i    Node sequence number of second point
!     nunkp          i    Maximum number of unknowns in a point
!     nusol          i    Total number of unknowns
!     rjumpleft     i/o   Array wih the same length as the second part of
!                         istoreleft containing the jumps w.r.t the right part
!     rjumploc       i    Array of size nunkp containing the jumps for each
!                         degree of freedom
!     rjumpright    i/o   Array wih the same length as the second part of
!                         istoreright containing the jumps w.r.t the left part
!     rmultleft     i/o   Array wih the same length as the second part of
!                         istoreleft containing the factors w.r.t the right part
!     rmultloc       i    Array of size nunkp containing the factors for
!                         each degree of freedom.
!     rmultright    i/o   Array wih the same length as the second part of
!                         istoreright containing the factors w.r.t the left part
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer j, istart, inunkp, inusol1, inusol2

!     inunkp         Number of degrees of freedom in point
!     inusol1        Position of degree of freedom in solution vector
!     inusol2        Position of degree of freedom in solution vector
!     istart         First position used at a part
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1835   Essential boundary condition in periodical boundary point
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topbv7021' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from topbv7021'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      inunkp = nunkp

      if ( indprf==0 ) then

!     --- indprf = 0, fixed number of unknowns per point

         inusol1 = (nodp1-1)*nunkp
         inusol2 = (nodp2-1)*nunkp

      else

!     --- indprf>0, variable number of unknowns per point

         inusol1 = kprobf(nodp1)
         inunkp = kprobf(nodp1+1)-inusol1
         inusol2 = kprobf(nodp2)
         inunkp = min(kprobf(nodp2+1)-inusol2,inunkp)

      end if

      do j = 1, inunkp

         if ( idegper(j)==1 ) then

!        --- periodical boundary conditions found, fill arrays

            istart = istartleft(inusol1+j)+1
            istartleft(inusol1+j) = istart
            istoreleft(istart) = inusol2+j
            istart = istart-nusol-1
            rjumpleft(istart) = rjumploc(j)
            rmultleft(istart) = rmultloc(j)
            itemp(inusol1+j) = 1

            istart = istartright(inusol2+j)+1
            istartright(inusol2+j) = istart
            istoreright(istart) = inusol1+j
            istart = istart-nusol-1
            rjumpright(istart) = -rjumploc(j)
            rmultright(istart) = 1d0/rmultloc(j)
            if ( iwork(inusol1+j).gt.0 ) then

!           --- essential boundary condition

               if ( iwork(inusol2+j).le.0 ) then

!              --- periodical boundary condition, but also essential

                  call errint ( nodp1, 1 )
                  call errint ( nodp2, 2 )
                  call errint ( j, 3 )
                  call errwar ( 1835, 3, 0, 0 )
                  if ( debug ) then

!                 --- Debug information

                     write(irefwr,*) 'Periodical bcs in', &
                        'nodes ', nodp1, nodp2, &
                        ' with also essential bcs'
                  end if

               end if
               iwork(inusol2+j) = iwork(inusol1+j)

            else if ( iwork(inusol2+j).gt.0 ) then

               if ( iwork(inusol1+j).le.0 ) then

!              --- periodical boundary condition, but also essential

                  call errint ( nodp1, 1 )
                  call errint ( nodp2, 2 )
                  call errint ( j, 3 )
                  call errwar ( 1835, 3, 0, 0 )
                  if ( debug ) then

!                 --- Debug information

                     write(irefwr,*) 'Periodical bcs in', &
                        'nodes ', nodp1, nodp2, &
                        ' with also essential bcs'
                  end if

               end if
               iwork(inusol1+j) = iwork(inusol2+j)

            end if  ! ( iwork(inusol1+j).gt.0 )

         end if  ! ( idegper(j)==1 )

      end do  ! j = 1, inunkp

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End topbv7021'

      end if  ! ( debug )
      call erclos ( 'topbv7021' )

      end
