      subroutine topbv703 ( iwork, itemp, jumps, factors, &
                            istoreleft, nusol, rjumpleft, &
                            rmultright, rjumpright, rmultleft, &
                            istoreright )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-06-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Nodal points of which the unknowns are connected through periodical
!     boundary conditions (possibly with jump) are marked
!       degrees of freedom corresponding to one of the nodal points
!       are filled in iwork with a negative number referring to the
!       other one
!     This is the final part of subroutines topbv7 and topbv5
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     periodical
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nusol, iwork(*), istoreleft(*), itemp(nusol), &
              istoreright(*)
      double precision jumps(*), factors(*), &
                       rjumpleft(nusol), rmultright(nusol), &
                       rjumpright(nusol), rmultleft(nusol)

!     factors        o    Array containing the factors of the curves
!     istoreleft     i    Contains all periodical right neighbor unknowns
!                         for all left periodical neighbor unknowns
!                         Pos. 1 to nusol+1 contain pointers to the second part
!                         istoreleft(1) contains nusol+1
!                         The number of neighbors of unknown i is stored in
!                         istoreleft(i+1)-istoreleft(i)
!                         The second part contains the neighbors
!     istoreright    i    Contains all periodical left neighbor unknowns
!                         for all right periodical neighbor unknowns
!                         Pos. 1 to nusol+1 contain pointers to the second part
!                         istoreright(1) contains nusol+1
!                         The number of neighbors of unknown i is stored in
!                         istoreright(i+1)-istoreright(i)
!                         The second part contains the neighbors
!     itemp         i/o  Temporaray work array of length nusol to indicate
!                         if an unknown corresponds to a "left" periodical
!                         boundary condition (1) or not (0)
!     iwork         i/o   Semi-permanent work array of length nusol in which
!                         information about essential boundary conditions and
!                         periodical boundary conditions is stored
!                         iwork is filled as follows:
!                         iwork(i) = 0: means unknown i is free
!                         iwork(i)>0: means unknown i is prescribed
!                                       (essential bc)
!                         iwork(i)<0: means unknown i refers to unknown
!                                       iwork(i), &
!                         which means that unknown i has the same value, &
!                         possibly extended with a jump
!                         Subroutine TOPBV2 has already initialised iwork with
!                         non-negative values
!     jumps         i/o   Array of size nusol containing the jumps
!                         for the periodical boundary conditions
!     nusol          i    Total number of unknowns
!     rjumpleft      i    Array wih the same length as the second part of
!                         istoreleft containing the jumps w.r.t the right part
!     rjumpright     i    Array wih the same length as the second part of
!                         istoreright containing the jumps w.r.t the left part
!     rmultleft      i    Array wih the same length as the second part of
!                         istoreleft containing the factors w.r.t the right part
!     rmultright     i    Array wih the same length as the second part of
!                         istoreright containing the factors w.r.t the left part
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ileft, inext, ipos, istarthelp1, istore(nusol), &
              iright, istart, istarthelp, nstore, nhelp, ihelp(nusol)

!     i              Counting variable
!     ihelp          Work array of length nusol that is use to keep track
!                    of all unknowns that are coupled to a specifice
!                    degree of freedom
!                    if ihelp(i) = 1, degree of freedom i is coupled to
!                    istart
!     ileft          Left degree of freedom in periodical couple
!     inext          Next point in curve
!     ipos           Position in array
!     iright         Right degree of freedom in periodical couple
!     istart         First position used at a part
!     istarthelp     First position used at a part
!     istarthelp1    First position used at a part
!     istore         Work array to store neighbours of a specific unknown
!     nhelp          Number of entries that have been stored in ihelp
!     nstore         Counter with respect to array istore
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRININNONZERO  Print the non-zero contents of an integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2428   Inconsistent jumps
!    2429   Inconsistent multiplication factors
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topbv703' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from topbv703'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill array ihelp in the following way:
!         ihelp(i) = 0: unknown is not periodical or has been treated already
!         ihelp(i) = 1: unknown is periodical and has not yet been treated
!         nhelp := number of non-zero entries in ihelp

      ihelp = 0
      do i = istoreleft(1)+1, istoreleft(nusol+1)
         ihelp(istoreleft(i)) = 1
      end do
      nhelp = istoreleft(nusol+1)-istoreleft(1)
      do i = istoreright(1)+1, istoreright(nusol+1)
         ipos = istoreright(i)
         if ( ihelp(ipos)==0 ) then
            ihelp(ipos) = 1
            nhelp = nhelp+1
         end if
      end do

      if ( debug ) then

!     --- Debug statements

         write(irefwr,*) 'nhelp = ', nhelp
         call prininnonzero ( ihelp, nusol, 'ihelp' )

      end if

!     --- While ( nhelp>0 ) do

      istarthelp = 1
      istarthelp1 = 1
      do while ( nhelp>0 )

!     --- istart := smallest unknown with ihelp(istart) = 1
!         First those with itemp(i) = 1, i.e. left points

         do i = istarthelp, nusol
            if ( ihelp(i)==1 .and. itemp(i)==1 ) then
               istart = i
               istarthelp = istart+1
               go to 100
            end if
         end do

!        --- If the program comes in this place, ther are no left points
!            left but still not all periodical boundary conditions treated

         istarthelp = nusol+1
         do i = istarthelp1, nusol
            if ( ihelp(i)==1 ) then
               istart = i
               istarthelp1 = istart+1
               go to 100
            end if
         end do

!        --- ihelp(istart) := 0, nhelp:=nhelp-1

100      ihelp(istart) = 0
         nhelp = nhelp-1

!        --- Store all neighbours of istart in istore
!            nstore := number of entries in istore
!            Fill iwork, jumps, factors for all entries in istore
!            Remove all neighbours from ihelp and decrease nhelp

         nstore = 0
         do i = istoreleft(istart)+1, istoreleft(istart+1)
            iright = istoreleft(i)
            ipos = i-nusol-1
            if ( ihelp(iright)==1 ) then

!           --- new unknown found

               nstore = nstore+1
               istore(nstore) = iright
               if ( iwork(iright)==0 ) iwork(iright) = -istart
               jumps(iright) = rjumpleft(ipos)
               factors(iright) = rmultleft(ipos)
               ihelp(iright) = 0
               nhelp = nhelp-1

            else

!           --- old unknown found, debug factors and jumps

               if ( abs(jumps(iright)+rjumpright(ipos))>epsmac ) then

!              --- error jumps inconsistent

                  call errint ( iright, 1 )
                  call errint ( istart, 2 )
                  call erreal ( rjumpright(ipos), 1 )
                  call erreal ( jumps(iright), 2 )
                  call errsub ( 2428, 2, 2, 0 )

               end if

               if ( abs(factors(iright)*rmultright(ipos)-1d0)> &
                    epsmac )then

!              --- error factors inconsistent

                  call errint ( iright, 1 )
                  call errint ( istart, 2 )
                  call erreal ( rmultright(ipos), 1 )
                  call erreal ( factors(iright), 2 )
                  call errsub ( 2429, 2, 2, 0 )

               end if

            end if

         end do

         do i = istoreright(istart)+1, istoreright(istart+1)
            ileft = istoreright(i)
            ipos = i-nusol-1
            if ( ihelp(ileft)==1 ) then

!           --- new unknown found

               nstore = nstore+1
               istore(nstore) = ileft
               if ( iwork(ileft)==0 ) iwork(ileft) = -istart
               jumps(ileft) = rjumpright(ipos)
               factors(ileft) = rmultright(ipos)
               ihelp(ileft) = 0
               nhelp = nhelp-1

            else

!           --- old unknown found, debug factors and jumps

               if ( abs(jumps(ileft)+rjumpright(ipos))>epsmac ) then

!              --- error jumps inconsistent

                  call errint ( ileft, 1 )
                  call errint ( istart, 2 )
                  call erreal ( rjumpright(ipos), 1 )
                  call erreal ( jumps(ileft), 2 )
                  call errsub ( 2428, 2, 2, 0 )

               end if

               if ( abs(factors(ileft)*rmultright(ipos)-1d0)>epsmac ) then

!              --- error factors inconsistent

                  call errint ( ileft, 1 )
                  call errint ( istart, 2 )
                  call erreal ( rmultright(ipos), 1 )
                  call erreal ( factors(ileft), 2 )
                  call errsub ( 2429, 2, 2, 0 )

               end if

            end if

         end do  ! i-loop

!        --- While ( nstore>0 ) do

         do while ( nstore>0 )

!        --- inext := first unknown in istore
!            remove inext from istore and decrease nstore

            inext = istore(1)
            nstore = nstore-1
            do i = 1, nstore
               istore(i) = istore(i+1)
            end do

!           --- For all neighbours ineigh of inext do

            do i = istoreleft(inext)+1, istoreleft(inext+1)
               iright = istoreleft(i)
               ipos = i-nusol-1
               if ( ihelp(iright)==0 ) then

!              --- iright is already in use, debug jump and factor

                  if ( abs(-jumps(iright)+jumps(inext)+rjumpleft(ipos)) &
                       >epsmac )then

!                 --- error jumps inconsistent

                     call errint ( iright, 1 )
                     call errint ( inext, 2 )
                     call erreal ( rjumpleft(ipos), 1 )
                     call erreal ( -jumps(iright)+jumps(inext), 2 )
                     call errsub ( 2428, 2, 2, 0 )

                  end if

                  if ( abs(factors(inext)/factors(iright)* &
                       rmultright(ipos)-1d0)>epsmac ) then

!                 --- error factors inconsistent

                     call errint ( iright, 1 )
                     call errint ( inext, 2 )
                     call erreal ( rmultleft(ipos), 1 )
                     call erreal ( factors(inext)/factors(iright), 2 )
                     call errsub ( 2429, 2, 2, 0 )

                  end if

              else

!             --- add neighbour to iwork, jumps, factors and istore
!                 nstore := nstore+1
!                 ihelp(ineigh) := 0

                 nstore = nstore+1
                 istore(nstore) = iright
                 if ( iwork(iright)==0 ) iwork(iright) = -istart
                 jumps(iright) = jumps(inext)+rjumpleft(ipos)
                 factors(iright) = factors(inext)*rmultleft(ipos)
                 nhelp = nhelp-1
                 ihelp(iright) = 0

              end if

            end do  ! i-loop

            do i = istoreright(inext)+1, istoreright(inext+1)
               ileft = istoreright(i)
               ipos = i-nusol-1
               if ( ihelp(ileft)==0 ) then

!              --- ileft is already in use, debug jump and factor

                  if ( abs(-jumps(ileft)+jumps(inext)+rjumpright(ipos)) &
                      >epsmac ) then

!                 --- error jumps inconsistent

                     call errint ( ileft, 1 )
                     call errint ( inext, 2 )
                     call erreal ( rjumpright(ipos), 1 )
                     call erreal ( -jumps(ileft)+jumps(inext), 2 )
                     call errsub ( 2428, 2, 2, 0 )

                  end if

                  if ( abs(factors(inext)/factors(ileft)* &
                       rmultright(ipos)-1d0)>epsmac ) then

!                 --- error factors inconsistent

                     call errint ( ileft, 1 )
                     call errint ( inext, 2 )
                     call erreal ( rmultright(ipos), 1 )
                     call erreal ( factors(inext)/factors(ileft), 2 )
                     call errsub ( 2429, 2, 2, 0 )

                  end if

              else

!             --- add neighbour to iwork, jumps, factors and istore
!                 nstore := nstore+1
!                 ihelp(ineigh) := 0

                 nstore = nstore+1
                 istore(nstore) = ileft
                 if ( iwork(ileft)==0 ) iwork(ileft) = -istart
                 jumps(ileft) = jumps(inext)+rjumpright(ipos)
                 factors(ileft) = factors(inext)*rmultright(ipos)
                 nhelp = nhelp-1
                 ihelp(ileft) = 0

              end if

            end do  ! i-loop

         end do  ! while ( nstore>0 )

      end do  ! while ( nhelp>0 )

1000  call erclos ( 'topbv703' )
      if ( debug ) then

!     --- Debug information

         call prinin (iwork, nusol, 'iwork' )
         call prinrl ( jumps, nusol, 'jumps' )
         call prinrl ( factors, nusol, 'factors' )
         write(irefwr,*) 'End topbv703'

      end if  ! ( debug )

      end
