      subroutine topedgeobst ( nelem, kmeshdobst, kmeshcobst, &
                               nmeshdobst, iwork, knelobst, closed )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 27-09-2002 Remove error message
!        version  1.0  date 16-11-2001
!
!   copyright (c) 2001-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Create edges and store edge sequence numbers in array nmeshdobst
!    One obstacle only
! **********************************************************************
!
!                       KEYWORDS
!
!     topology
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nelem, kmeshdobst(*), kmeshcobst(3,*), &
              nmeshdobst(3,*), iwork(*), knelobst(*)
      logical closed

!     closed         o    If true the boundary is closed
!     iwork               integer work array of length nelem to indicate
!                         elements that contain a certain node (>0) or not (0)
!     kmeshcobst     i    Array kmeshc with respect to an obstacle
!                         Contains the nodal point numbers of the triangles
!                         in the obstacle
!     kmeshdobst     i    KMESH part d (only part 1) of the obstacle
!     knelobst       i    Contains kmesh part d2, i.e.
!                         the element numbers of the obstacle
!     nelem          i    Number of elements
!     nmeshdobst          Work array of length 3*nelem containing the edge
!                         sequence numbers of all edges in the elements
!                         of the obstacle
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, raise(3), ielem, nedge, nodp1, nodp2, ih, iseq, &
              inelem, jelem

!     i              Counting variable
!     ielem          Element number
!     ih             Help variable to store a constant temporarily
!     inelem         Number of elements in a specific element group
!     iseq           Sequence number
!     j              Counting variable
!     jelem          Element number
!     k              Counting variable
!     nedge          Number of edges
!     nodp1          Node sequence number
!     nodp2          Node sequence number
!     raise          Raise-operator
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2401   Number of obstacle surfaces connected to obstacle edge # 2
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array nmeshdobst
!     nedge := 0
!     For all elements in the obstacle do
!        For all edges in element do
!           if ( edge has not been treated yet ) then
!              nedge := nedge+1
!              Store nodes of edge in nodp1/nodp2
!              Find all elements containing both nodp1 and nodp2
!              If ( number of elements not 2 ) then
!                 Error: surface not closed
!              Fill edges in both elements
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data raise /2, 3, 1/
! ======================================================================
!
      call eropen ( 'topedgeobst' )

!     --- Clear array nmeshdobst

      do i = 1, nelem
         do j = 1, 3
            nmeshdobst(j,i) = 0
         end do
      end do
      closed = .true.

!     --- Clear array iwork

      do i = 1, nelem
         iwork(i) = 0
      end do

!     --- nedge := 0

      nedge = 0

!     --- For all elements in the obstacle do

      do ielem = 1, nelem

!     --- For all edges in element do

         do j = 1, 3

            if ( nmeshdobst(j,ielem).eq.0 ) then

!           --- edge has not been treated yet
!               nedge := nedge+1
!               Store nodes of edge in nodp1/nodp2

               nedge = nedge+1
               nodp1 = kmeshcobst(j,ielem)
               nodp2 = kmeshcobst(raise(j),ielem)

!              --- Find all elements containing both nodp1 and nodp2

               do i = kmeshdobst(nodp1)+1, kmeshdobst(nodp1+1)
                  iwork(knelobst(i)) = 1
               end do
               inelem = 0
               do i = kmeshdobst(nodp2)+1, kmeshdobst(nodp2+1)
                  ih = knelobst(i)
                  if ( iwork(ih).gt.0 ) then
                     inelem = inelem+1
                     if ( ih.ne.ielem ) jelem = ih
                  end if
               end do

!              --- The next test is removed by Raoul van Loon
!                  by making it false. Why?

               if ( closed .and. inelem.ne.2 ) then

!              --- number of elements not 2
!                  Error: surface not closed

                  call errint ( nedge, 1 )
                  call errint ( inelem, 2 )
                  inelem = 0
                  do i = kmeshdobst(nodp2)+1, kmeshdobst(nodp2+1)
                     ih = knelobst(i)
                     if ( iwork(ih).gt.0 ) then
                        inelem = inelem+1
                        call errint ( ih, inelem+2 )
                     end if
                  end do
                  call errwar ( 2401, inelem+2, 0, 0 )
                  closed = .false.

               else if ( inelem.eq.2 ) then

!              --- Fill edges in both elements

                  nmeshdobst(j,ielem) = nedge
                  iseq = 0
                  do k = 1, 3
                     if ( kmeshcobst(k,jelem).ne.nodp1 .and. &
                          kmeshcobst(k,jelem).ne.nodp2 ) iseq = k
                  end do
                  nmeshdobst(raise(iseq),jelem) = nedge

               end if

!              --- Clear iwork

               do i = kmeshdobst(nodp1)+1, kmeshdobst(nodp1+1)
                  iwork(knelobst(i)) = 0
               end do

            end if

         end do

      end do

      call erclos ( 'topedgeobst' )
      end
