      subroutine toplpj ( ielshape, constunk )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.6  date 24-07-2015 Extra debug
!        version  4.5  date 17-10-2011 fortran 90
!        version  4.4  date 26-06-2008 Extra debug
!
!   copyright (c) 1982-2015 "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    array kprob part f is filled when the number of degrees of
!    freedom in the nodal points is not constant
!    nusol is computed
!    kmesh part c and kprob part f must be stored in-core
! **********************************************************************
!
!                       KEYWORDS
!
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: ielshape(2,*)
      logical constunk

!     constunk   o    If true the number of unknowns per point is constant
!     ielshape   i    Array of length 2*(nelgrp+numnat) containing:
!                     (1,i) number of nodes in element group i
!                     (2,i) shape number of elements in element group i
!                     if i>nelgrp: i-nelgrp = boundary group number
!                     otherwise it is the element group number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer j, ikelmc, inpelm, ielgrp, itype, iielem, nodp, inunkp, &
              idegfd, jnunkp

!     idegfd         Degree of freedom
!     ielgrp         element group number
!     iielem         Relative element number in element group
!     ikelmc         position in kmesh part c
!     inpelm         number of points per element
!     inunkp         Number of degrees of freedom in first point
!     itype          Type number of element
!     j              Counting variable
!     jnunkp         Number of degrees of freedom in a point
!     nodp           Nodal point sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Warnings
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     549   degree of freedom for the creation of boundary conditions of the
!           type u equals unknown constant incorrect
!    1684   Boundary element creates degree of freedom in point, whereas
!           internal element does not
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'toplpj' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from toplpj'
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- clear kprob part j

      kprobf(1:npoint) = 0
      ikelmc = 0
      numnat = kprobloc(12)
      inpelm = npelm
      inelem = nelem
      if ( debug ) &
         write(irefwr,1) 'npoint, inpelm, inelem, nelgrp', &
                          npoint, inpelm, inelem, nelgrp

!     --- First fill kprobf for internal elements

      do ielgrp = 1, nelgrp

         if ( indpra>0 ) kprobatmp => kproba(ielgrp)%arr
         if ( nelgrp>1 ) then

!        --- nelgrp>1

            inpelm = Topgrp(ielgrp)%inpelm
            inelem = Topgrp(ielgrp)%inelem
            itype = kprobc(ielgrp)
            if ( debug ) &
               write(irefwr,1) 'ielgrp, inpelm, inelem, itype, ikelmc', &
                                ielgrp, inpelm, inelem, itype, ikelmc

!           --- Skip empty elements as well as elements corresponding to
!               periodical boundary conditions

            if ( itype==-1 .or. itype==0) then
               ikelmc = ikelmc+inelem*inpelm
               goto 100
            end if

         end if
         if ( debug ) then

!        --- Debug information

            write(irefwr,1) 'ielgrp, inelem, inpelm', &
                             ielgrp,inelem,inpelm

         end if

         do iielem = 1, inelem
            do j = 1, inpelm
               nodp = kmeshc(ikelmc+j)
               inunkp = kprobatmp(j)
               kprobf(nodp) = max(kprobf(nodp),inunkp)
            end do
            ikelmc = ikelmc+inpelm
         end do
100      continue

      end do
      if ( debug ) call prinin ( kprobf, npoint, 'kprobf' )

!     --- Next fill for boundary elements

      ikelmc = 0
      nbcons = 0
      do ielgrp = nelgrp+1, nelgrp+numnat
         inpelm = ielshape(1,ielgrp)
         inelem = kprobb(ielgrp-nelgrp)
         itype = kprobc(ielgrp)
         if ( debug ) &
            write(irefwr,1) 'ielgrp, inpelm, itype, inelem, indprb', &
                             ielgrp, inpelm, itype, inelem, indprb

         if ( itype==-2 ) then

!        --- Boundary conditions of the type u = constant

            nbcons = nbcons+1
            idegfd = kprobo(nbcons)

         else

            if ( indpra>0 ) kprobatmp => kproba(ielgrp)%arr

         end if
         do iielem = 1, inelem

            if ( debug ) then
               write(irefwr,1) 'iielem, inpelm, ikelmc', &
                                iielem, inpelm, ikelmc
               call prinin ( kprobi(ikelmc+1), inpelm, 'kprobi' )
               call prinin ( kprobatmp, inpelm, 'kproba' )
            end if  ! ( debug )

            do j = 1, inpelm
               nodp = kprobi(ikelmc+j)
               inunkp = kprobatmp(j)
               if ( inunkp>0 .and. kprobf(nodp)==0 ) then

                  call errint ( nodp, 1 )
                  call errint ( ielgrp-nelgrp, 2 )
                  call errint ( inunkp, 3 )
                  call errint ( j, 4 )
                  call errint ( iielem, 5 )
                  call errwar ( 1684, 5, 0, 0 )

               end if
               kprobf(nodp) = max(kprobf(nodp),inunkp)
            end do
            if ( itype==-2 ) then

!           --- Boundary conditions of the type u = constant
!               Check idegfd

               nodp = kprobi(ikelmc+j)
               inunkp = kprobf(nodp)
               if ( idegfd>inunkp ) then

!              --- idegfd>inunkp

                  call errint ( idegfd, 1 )
                  call errint ( inunkp, 2 )
                  call errint ( nbcons, 3 )
                  call errsub ( 549, 3, 0, 0 )

               end if

            end if
            ikelmc = ikelmc+inpelm
         end do
      end do

!     --- accumulate part f

      nusol = 0
      constunk = .true.

      inunkp = kprobf(1)
      do j = 1, npoint
         jnunkp = kprobf(j)
         kprobf(j) = nusol
         nusol = nusol+jnunkp
         if ( inunkp/=jnunkp ) constunk = .false.
      end do
      kprobf(npoint+1) = nusol

1000  if ( debug )  then

!     --- Debug information

         write(irefwr,1) 'nusol', nusol
         write(irefwr,*) 'End toplpj'

      end if  ! ( debug )
      call erclos ( 'toplpj' )

      end
