      subroutine el0800in ( iuser, kchois, metup, mrule, ncoef, linear, &
                            jelp, ishift, interpol, iqmat, iqmat1, &
                            iseq, jheat, jwcons )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-01-2016 Use sepmoduleelem
!        version  2.15 date 18-12-2014 Use el0800upw
!        version  2.14 date 20-06-2013 Use el0800ruledef
!
!   copyright (c) 2001-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the number of integration points for this element group
!     using array iuser
!     Elliptic second order equations
! **********************************************************************
!
!                       KEYWORDS
!
!     elliptic_equation
!     initialization
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iuser(*), kchois, metup, mrule, ncoef, jelp, ishift, &
              interpol, iqmat, iqmat1, iseq, jheat, jwcons
      logical linear

!     interpol      i/o   If 0 the standard interpolation is applied, &
!                         if 1 the interpolation is restricted to linear
!                         subelements
!     iqmat         i/o   Indication whether a transformation has to be
!                         performed if the elements are of a lower
!                         dimension than the dimension of the coordinates.
!                         If 0 no tranformation is performed (standard
!                         case) if 1 then the velocities are mapped to the
!                         dimension of the elment
!     iqmat1         o    Indication whether the right-hand side has to be
!                         transformed
!     iseq           i    Sequence number of vector in VECOLD/ISLOLD
!                         In case of a call by ELI800 it is the parameter JDEGFD
!     ishift        i/o   Shift in coefficient sequence number
!     iuser          i    Integer user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
!     jelp           o    Indication of the type of basis function subroutine
!                         must be called by ELel0800inx.
!     jheat          o    Integer parameter indicating if the mass matrix for
!                         the heat equation must be computed (>0) or not ( = 0)
!                         If jheat = 1, then the coefficient for the mass matrix
!                         is constant, otherwise (jheat = 2) it is variable
!     jwcons         o    Indication if the weight function is constant
!                         (1) or not (2)
!     kchois         o    Indication what type of parameters must be computed
!                         Possible values:
!                         0:  Standard method, the calling subroutine is ELM100
!                             It is supposed that an element matrix must be
!                             computed
!                         1:  Subroutine ELD100 calls this subroutine, only
!                             derivatives must be computed
!                         2:  Subroutine ELD100 or ELI100 calls this subroutine, &
!                             a user function defined as first parameter in
!                             IUSER/USER must be filled in elemvc (icheld=9)
!                         3:  Subroutine ELD100 calls this subroutine, &
!                             alpha times the gradient must be stored in ELEMVC
!                             The first 3 parameters of IUSER/USER are used
!     linear         o    Indicates whether the element is a linear simplex
!                         (true) or not (false)
!     metup          o    Upwind parameter
!     mrule          o    Type of integration rule as given by the user
!     ncoef          o    Number of coefficients in the equation
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer massparm

!     massparm       Indicates the largest value of jheat during the
!                    time integration
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inirepscal

!     EL0800ELP      Set parameters depending on shape of element
!     EL0800INM      Compute the parameter m for an element
!     EL0800RULE     Fill the parameters irule and jcart from array iuser
!     EL0800UPW      Compute parameter metup defining the type of upwinding
!     ELCOMM         Initializes the commons celp and celint with standard
!                    values
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERROR42        Print error message 42
!     ERRSUB         Error messages
!     INIREPSCAL     Corrects integer value in case of scalars
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1785   Upwinding has only been implemented for Cartesian and axi-symmetric
!           co-ordinates systems
!    1787   iseq has incorrect value
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el0800in' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el0800in'
         write(irefwr,1) 'imas', imas
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( icheld<0 ) then
         icheli = mod(-icheld,10)
      else
         icheli = 0
      end if
      call elcomm
      if ( icheli==7 ) then
         jcart  = 1
      end if
      metupw = 0
      metup  = 0
      mrule = 0
      massparm = 0

!     --- Compute kchois

      if ( icheld==0 .or. icheld>=23 .and. icheld<=25 .or. &
           icheld>=41 .and. icheld<=45 ) then

!     --- Call by elm800, all coefficients required or
!         call by eld800 with icheld = 41

         kchois = 0

      else if ( icheli>0 .and. icheli/=7 ) then

!     --- call by eli800, one (new) coefficient required

         kchois = 2

      else if ( icheld==21 ) then

!     --- call by eld800, icheld = 21, one (new) coefficient required

         kchois = 4

      else if ( icheld==6 .or. icheld==16 .or. icheld==26 .or. &
                icheld==28 .or. icheld==29 .or. icheld==36 ) then

!     --- call by eld800, icheld = 6, 16, 26, 28, 29, 36
!         The first six standard coefficients are required

         kchois = 3

      else

!     --- call by eld800, icheld<6, or 10<icheld<16, or icheld = 27
!         or call by eli800, icheli = 7
!         i.e. no coefficients required

         kchois = 1

      end if
      if ( debug ) write(irefwr,1) 'kchois', kchois

      if ( ierror/=0 ) go to 1000

!     --- Set parameters depending on the shape numbers

      call el0800elp ( linear, jelp )

      if ( kchois==0 ) then

!     --- standard situation

         if ( itype==800 .or. itype==813 .or. itype==815 ) then
            ncoef = 15
            if ( icheld>=23 .and. icheld<=25 ) ncoef = 19
         else
            ncoef = 12
         end if  ! ( itype==800 )

      else if ( kchois==1 ) then

!     --- kchois = 1, no coefficients required

         ncoef = 0

      else if ( kchois==2 .or. kchois==4 ) then

!     --- kchois = 2 or 4, one coefficient required

         ncoef = 1

      else if ( kchois==3 ) then

!     --- kchois = 3, six coefficients required

         ncoef = 6

      end if
      if ( ierror/=0 ) go to 1000
      if ( debug ) write(irefwr,1) 'ncoef', ncoef

!     --- Fill the arrays IND, MST and ARRAY with information of the
!         coefficient

      if ( ncoef>0 ) then

         ipos = iuser ( ielgrp + 5 )
         if ( ipos<7 ) call error42 ( ielgrp, ipos, ielem, itype )

         if ( kchois==0 ) then

!        --- Standard situation, Compute the parameters jheat, metupw
!            Set jheat equal to imas

            jheat = imas
            massparm = max(jheat,massrule)
            iqmat = inirepscal ( iuser(ipos) )
            iqmat = iqmat/10
            if ( debug ) &
               write(irefwr,1) 'imas, jheat, iqmat, massparm', &
                                imas, jheat, iqmat, massparm

            iqmat1 = 0
            if ( iqmat>1 ) iqmat1 = 1
            if ( iqmat>=1 ) then
               iweigh = iweigh + 10
               iqmat = 1
            end if
            ipos = ipos + 1
            ishift = ishift+1

            call el0800upw ( iuser, metup, jelp, ishift )

         else if ( icheld>0 ) then

!        --- icheld>0, Compute iseq

            if ( iuser(ipos)>0 ) &
               iseq = inirepscal ( iuser(ipos) )+iseq-1
            ipos = ipos + 4
            ishift = ishift+4
            if ( iseq<0 .or. iseq>numold ) then
               call errint ( iseq, 1 )
               call errint ( numold, 2 )
               call errint ( ielem, 3 )
               call errint ( itype, 4 )
               call errint ( ielgrp, 5 )
               call errsub ( 1787, 5, 0, 0 )
            end if
            if ( iseq==0 ) iseq = 1

         end if
         if ( ierror/=0 ) go to 1000

         if ( debug ) &
            write(irefwr,1) 'kchois, icheli, icheld', &
                             kchois, icheli, icheld
         if ( kchois==0 .or. kchois==2 ) then

!        --- Call by BUILD or INTEGR, Compute the parameters jcart and irule

            call el0800rule ( kchois, interpol, ishift, iuser, mrule )

         end if  ! ( kchois==0 .or. kchois==2 )

         if ( metup>0 .and. jcart>2 ) then
            call errint ( jcart-1, 1 )
            call errint ( ielem, 2 )
            call errint ( itype, 3 )
            call errint ( ielgrp, 4 )
            call errsub ( 1785, 4, 0, 0 )
         end if
         if ( ierror/=0 ) go to 1000

         if ( mod(metup,20)>=9 .and. mod(metup,20)<=11 ) then

!        --- special upwind (9: linear, satisfying maximum principle)
!            Always use 1 point Gauss rule

            irule = 2

         else if ( mrule==0 .and. metup>0 ) then

!        --- mrule = 0, and upwind
!            A different integration rule is applied

            if ( imas==1 ) then
               irule = 1
            else if ( jelp==3 .and. mod(metup,20)>6 .or. &
                      jelp==11 .or. jelp==21 ) then
               irule = 2
            else if ( jelp>=3 .and. jelp<=6 ) then
               irule = 3
            end if

         end if

         if ( linear ) then
            jwcons = min(jcart,2)
         else
            jwcons = 2
         end if
         if ( icheld>0 .and. mrule==0 ) irule = 1
         if ( debug ) &
            write(irefwr,1) 'icheld, irule, mrule, jelp, jheat', &
                             icheld, irule, mrule, jelp, jheat
         if ( massparm==2 .and.  mrule==0 .and. &
              (jelp<=5 .or. jelp>=7) ) then

!        --- Non-diagonal mass matrix, shapes 1-5 and >=7
!            Integration rule must be at least 3 to avoid non-positive
!            definite mass matrices

            if ( jelp==2 ) then

!           --- Quadratic 1d, use irule = 4

               irule = 4

            else

!           --- Other cases, use irule = 3

               irule = max ( irule, 3 )

            end if

         else if ( massparm==2 .and. jelp<=6 .and. mrule==0 ) then

!        --- Non-diagonal mass matrix, shape 6
!            Integration rule must be at least 2 to avoid non-positive
!            definite mass matrices

            irule = max ( irule, 2 )

         end if
         if ( icheli>2 .and. icheli<=5 ) irule = 1
         if ( debug ) &
            write(irefwr,1) 'icheli, irule, jdiag', icheli, irule, jdiag

         if ( irule/=1 ) then

!        --- irule>1, Gauss rule

            call el0800inm ( m, jelp, jwcons, irule, igausp, jdiag, &
                             iphi )
            mn = m * n

         else if ( metup>0 ) then

!        --- upwinding, fill array phi

            iphi = 1

         end if

      end if  ! ( ncoef>0 )

1000  call erclos ( 'el0800in' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'irule, imas, jdiag, jheat, m, n', &
                          irule, imas, jdiag, jheat, m, n
         write(irefwr,*) 'End el0800in'

      end if  ! ( debug )

      end
