      subroutine el0801coeff ( iuser, linear, jboun, mrule, jwcons, &
                               ishift )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.3  date 18-12-2014 Use elgetintcoef
!        version  1.2  date 06-02-2014 Extra debug
!
!   copyright (c) 2006-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Initialize coefficients for elements of type 801 and 907
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     navier_stokes_equation
!     pressure_correction
!     initialization
!     parabolic_equation
!     elliptic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iuser(*), jboun, mrule, jwcons, ishift
      logical linear

!     ishift        i/o   Shift in coefficient sequence number
!     iuser          i    Integer user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
!     jboun          o    Defines the type of boundary condition.
!                         Possible values:
!                         2:  Standard mixed (Robbins) boundary condition
!                         3:  Boundary condition of type 3
!                         4:  Boundary condition of type 4 (not yet available)
!     jwcons         o    Indication if the weight function is constant
!                         (1) or not (2)
!     linear         o    Indicates whether the element is a linear simplex
!                         (true) or not (false)
!     mrule          o    Type of integration rule as given by the user
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: nncode
      parameter (nncode=3)
      integer :: ncode(nncode)
      character (len=13) :: code (nncode)

!     code           Table with keywords corresponding to string coefficients
!     ncode          Table containing the number of significant characters
!                    in the keyword table code
!     nncode         Number of entries in code
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inirepscal, elgetintcoef

!     ELGETINTCOEF   Give elgetintcoef the actual integer value corresponding to
!                    iuser(ipos)
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     INIREPSCAL     Corrects integer value in case of scalars
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      42   Array IUSER(.) has value which is too small
!    1546   Boundary conditions of type 5 have only been implemented for
!           line elements
!    1766   Type of integration rule incorrect
!    1767   Type of co-ordinates incorrect
!    1912   Parameter jboun has incorrect value
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code / 'CARTESIAN', 'AXI_SYMMETRIC', 'POLAR' /
      data ncode / 9,13, 5 /
! ======================================================================
!
      call eropen ( 'el0801coeff' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el0801coeff'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ipos = iuser ( ielgrp + 5 )
      if ( ipos<7 ) then
         call errint ( ielgrp+5, 1 )
         call errint ( ipos, 2 )
         call errint ( ielem, 3 )
         call errint ( itype, 4 )
         call errint ( ielgrp, 5 )
         call errwar ( 42, 5, 0, 0 )
         notvec = 1
         notmat = 1
         go to 1000
      end if

      if ( debug ) write(irefwr,1) 'ipos', ipos

      nuser = 0

!     --- Compute the parameters jboun, jcart and irule

      jboun = inirepscal ( iuser(ipos) )
      if ( jboun==0 ) jboun = 2
      ipos = ipos + 1
      ishift = ishift+1
      if ( jboun<2 .or. jboun>6 ) then
         call errint ( jboun, 1 )
         call errint ( ielem, 2 )
         call errint ( itype, 3 )
         call errint ( ielgrp, 4 )
         call errsub ( 1912, 4, 0, 0 )
      end if
      if ( jboun==5 .and. ishape>2 .or. itype==907 .and. jboun/=2 .or. &
           jboun==6 .and. ishape/=1 ) then
         call errint ( ielem, 1 )
         call errint ( itype, 2 )
         call errint ( ielgrp, 3 )
         call errsub ( 1546, 3, 0, 0 )
      end if
      ipos = ipos + 1
      ishift = ishift+1

      mrule = inirepscal ( iuser(ipos) )
      irule = max ( 1, mrule )
      ipos = ipos + 1
      ishift = ishift+1
      if ( mrule<0 .or. mrule>40 ) then
         call errint ( mrule, 1 )
         call errint ( 3, 2 )
         call errint ( ielem, 3 )
         call errint ( itype, 4 )
         call errint ( ielgrp, 5 )
         call errsub ( 1766, 5, 0, 0 )
      end if

      jcart = elgetintcoef ( iuser, ipos, ishift, code, nncode, ncode, &
                             1, 'coordinate_system'  )
      if ( jcart<0 .or. jcart>2 ) then
         call errint ( jcart, 1 )
         call errint ( 4, 2 )
         call errint ( ielem, 3 )
         call errint ( itype, 4 )
         call errint ( ielgrp, 5 )
         call errsub ( 1767, 5, 0, 0 )
      end if
      jcart = jcart + 1

      if ( linear ) then
         jwcons = min(jcart,2)
      else
         jwcons = 2
      end if

1000  call erclos ( 'el0801coeff' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'jcart, irule, jboun', jcart, irule, jboun
         write(irefwr,*) 'End el0801coeff'

      end if  ! ( debug )

      end
