      subroutine el1000 ( sp, div, phix, pp, phi, w, x, xgauss, mcont, &
                          rho, kappa, rhsdiv, fdiv, mdiv, ishape, &
                          psipres, icent, rhonodes )
! ======================================================================
!
!        programmer    Guus Segal/ Jaap van der Zanden
!        version  4.0  date 27-02-2010 Extra parameters
!        version  3.5  date 21-12-2008 Remove mcont = 5
!        version  3.4  date 17-09-2008 Extension with pressure mass matrix
!        version  3.3  date 14-09-2007 Debug statements
!
!   copyright (c) 1995-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute the divergence and the pressure gradient matrix for the
!     incompressible Navier-Stokes equations
!     The divergence matrix is stored in array DIV
!     The transpose of the pressure gradient matrix in SP
!     The pressure matrix is stored in array PP
!     If mcont=1, only the pressure gradient matrix is stored
!     If mcont=2, the Goertler equations are solved
!     If mcont=4, the continuity equation contains a pressure term
!        In that case dp/dx has no contribution (3D-Cartesian coordinates only)
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer mcont, mdiv, ishape, icent
      double precision phix(n,m,ndimlc), w(m), sp(ndimlc*n,npsi), &
                       xgauss(m,*), x(n,*), phi(n,m), rho(m), &
                       div(ndimlc*n,npsi), rhsdiv(*), fdiv(m), &
                       psipres(npsi,m), rhonodes(n)
      double precision pp(ndimlc+1,ndimlc+1), kappa(m)

!     div            o    Divergence matrix
!                         div(i,k) corresponds to the k-th row and i-th column
!                         of the matrix, The k refers to the basis functions of
!                         the pressure, i to the basis functions of the velocity
!                         in the sequence of the solution vector
!     fdiv           i    Contains values of divergence right-hand side to be
!                         integrated (if mdiv=1)
!     icent          i    Index of centroid
!     ishape         i    Shape number of elements
!     kappa          i    Factor for pressure term
!     mcont          i    Defines the type of continuity equation
!                         Possible values:
!                         0: incompressible flow
!                         1: compressible flow: div (rho u ) = 0
!                            Not applicable with penalty function method
!                         2: Goertler equations: No dp/dx, but complete div u
!                         4: the continuity equation contains a pressure term
!     mdiv           i    Defines if divergence right-hand side is used (1)
!                         or not (0)
!     phi            i    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     phix           i    Array of length n x m x ndim  containing the
!                         derivatives of the basis functions in the sequence:
!                         d phi / dx (xg ) = phix (i,k,1);
!                              i        k
!                         d phi / dy (xg ) = phix (i,k,2);
!                              i        k
!                         d phi / dz (xg ) = phix (i,k,3);
!                              i        k
!                         If the element is a linear tetrahedron (n=4), then
!                         the derivatives are constant and only k = 1 is filled.
!     pp             o    Pressure matrix
!     psipres        i    Contains the pressure basis functions
!                         psipres(i,k) refers to i-th basis function in
!                         integration point k
!     rho            i    Contains the density rho in integration points
!     rhonodes       i    density rho in nodal  points
!     rhsdiv         o    Divergence right-hand side (if mdiv=1)
!     sp             i    Gradient of pressure matrix
!                         SP(i,k) corresponds to the k-th row and i-th column
!                         of the matrix, The k refers to the basis functions of
!                         the pressure, i to the basis functions of the
!                         velocity in the sequence of the solution vector
!     w              i    array of length m containing the weights for
!                         integration
!     x              i    array containing the coordinates of the nodal points
!                         according to:
!                         x  = x (i,1);  y  = x (i,2);  z  = x (i,3)
!                          i              i              i
!     xgauss         i    array of length m x ndim containing the co-ordinates
!                         of the gauss points.  array xgauss is only filled when
!                         gauss integration is applied.
!                         xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                         zg_i = xgauss (i,3)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer k, npress

!     k              Counting variable
!     npress         Number of pressure unknowns
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELFILLDIV      Fill div matrix, in case of slightly compressible flow
!                    with respect to Crouzeix Raviart elements
!     ELFILLPRESMMT  Fill pressure mass matrix weighted by coefficient kappa
!     ELFILLRHSDIV   Fill vector rhsdiv i.e. the right-hand side of the
!                    continuity equation with respect to Crouzeix Raviart
!                    elements
!     ELFILLSP       Fill sp matrix, ie. the gradient of the pressure matrix
!                    with respect to Crouzeix Raviart elements
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The gradient matrix is given by:
!     SP = |  SPu  SPv  SPw  |
!     with:
!     SPu(i,k) = / psi(i) dphidx(k) dx
!                x
!     SPv(i,k) = / psi(i) dphidy(k) dx
!                x
!     SPw(i,k) = / psi(i) dphidz(k) dx
!                x
!     The divergence matrix is the same but each term multiplied by rho
!     For a linear approximation of the pressure (np=4), psi is given by
!     psi(1)  = 1
!     psi(2)  = x - x14
!     psi(3)  = y - y14
!     psi(4)  = z - z14
!     The matrix SP is organized in the sequence of the velocity vector
!     i.e. u1, v1, w1, u2, v2 ....
!     In case of axi-symmetric co-ordinates the matrix is given by:
!     SPu(i,k) = / psi(i) (dphidx(k) + phi(k)/x) dx
!                x
!     SPv(i,k) = / psi(i) dphidy(k) dx
!                x
!     In case of polar co-ordinates the matrix is given by:
!     SPu(i,k) = / psi(i) (dphidx(k) + phi(k)/x) dx
!                x
!     SPv(i,k) = / psi(i) dphidy(k)/x dx
!                x
!     PP(i,j) = / kappa psi(i) psi(j) dx
!               x
!     The right-hand side is defined by:
!     rhsdiv(i) = / psi(i) f dx
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el1000' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el1000'
         write(irefwr,1) 'ifirst, inpelm, jcart', &
                          ifirst, inpelm, jcart
         write(irefwr,1) 'mcont, mdiv', mcont, mdiv
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- First fill array sp

      call elfillsp ( sp, phix, phi, w, x, xgauss, ishape, icent )

      if ( mdiv==1 ) then

!     --- mdiv = 1, fill array rhsdiv

         call elfillrhsdiv ( w, rhsdiv, fdiv, psipres )

      end if  ! ( mdiv==1 )

!     --- Perform some extra actions depending on mcont

      select case ( mcont )

      case(0)

!     --- mcont = 0, divergence matrix = pressure matrix

         div = sp

      case(1, 3, 5)

!     --- mcont = 1, fill also the divergence matrix

         npress = npsi
         if ( ishape==4 .or. ishape==6 .or. ishape==7 .or. &
              ishape==14 .or. ishape==16 ) npress = ndimlc+1

         call elfilldiv ( rho, phix, psipres, npress, div, phi, w, &
                          xgauss, rhonodes )

      case(2)

!     --- mcont = 2, fill also the divergence matrix
!                    clear part of sp matrix

         div = sp
         do k = 1, n
            sp(ndimlc*(k-1)+1,1:npsi) = 0d0
         end do

      case(4)

!     --- mcont = 4, fill also the divergence matrix and the
!                    pressure matrix
!         Fill divergence matrix similar to above procedure
!         for MCONT = 2; do NOT multiply by density

         div = sp

!        --- Fill the values of the basis functions in the Gauss points
!            according to:
!            psi = x  - xcent
!            Fill the pressure matrix according to
!            PP(i,j) = / kappa * psi(i) * psi(j) dx
!                      x

         call elfillpresmmt ( psipres, pp, kappa, npsi, w )

      end select  ! case ( mcont )

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( sp, npsi, ndimlc*n, 'sp' )
         call prinrl1 ( div, npsi, ndimlc*n, 'div' )
         write(irefwr,*) 'End el1000'

      end if  ! ( debug )
      call erclos ( 'el1000' )

      end
