      subroutine el1013 ( ishift, jshift, kfirst, jelp, iqmat, ndim )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.3  date 03-02-2009 Extra debug
!        version  2.2  date 12-05-2006 Set idim
!        version  2.1  date 29-07-1998 Correction iqmat (BvR)
!        version  2.0  date 16-07-1998 Additional parameter iqmat (BvR)
!        version  1.1  date 12-01-1997 Extra shape numbers
!        version  1.0  date 12-12-1995
!
!   copyright (c) 1995-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute kfirst with respect to the convection terms
!    Help subroutine for el08xx
! **********************************************************************
!
!                       KEYWORDS
!
!     elliptic_equation
!     initialization
!     parabolic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ishift, jshift, kfirst, jelp, iqmat, ndim

!     iqmat      i    Indication if a transformation to a local axis has to
!                     be performed on the diffusion tensor
!     ishift     i    Shift of coefficient index
!                     The coefficients 1+ishift to 3+ishift are considered
!     jelp       i    Indication of the type of basis function subroutine must
!                     be called by ELel1013x. Possible vaues:
!                      1:  Subroutine ELP610  (Linear line element)
!                      2:  Subroutine ELP611  (Quadratic line element)
!                      3:  Subroutine ELP620  (Linear triangle)
!                      4:  Subroutine ELP624  (Extended quadratic triangle)
!                      5:  Subroutine ELP621  (Bilinear quadrilateral)
!                      6:  Subroutine ELP623  (Biquadratic quadrilateral)
!                     11:  Subroutine ELP630  (Linear tetrahedron)
!                     12:  Subroutine ELP631  (Quadratic tetrahedron)
!                     13:  Subroutine ELP632  (Trilinear brick)
!                     14:  Subroutine ELP633  (Triquadratic brick)
!     jshift     i    Shift with respect to error messages
!                     ishift=6: jshift=5
!                     ishift=18: jshift=8
!     kfirst     o    This parameter indicates the smallest velocity component
!                     that is not equal to zero in the convective terms
!                     For example kfirst = 2, means that the y- &
!                     component of the velocity is unequal to zero
!     ndim       i    Dimension of the space.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, in1, in2, in3

!     i          Counting variable
!     in1        Sequence number of first coefficient
!     in2        Sequence number of secondt coefficient
!     in3        Sequence number of third coefficient
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRINT  Put integer in error message
!     ERRWAR  Print warnings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1768   Coefficient is filled that is not used
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el1013' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el1013'
         write(irefwr,1) 'ishift, jshift, iqmat', ishift, jshift, iqmat
         write(irefwr,1) 'jelp, ndim, ielem, itype, ielgrp', &
                          jelp, ndim, ielem, itype, ielgrp
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      in1 = 1+ishift
      in2 = 2+ishift
      in3 = 3+ishift

      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'in1, in2, in3', in1, in2, in3
         write(irefwr,1) 'ind(in.)', ind(in1), ind(in2), ind(in3)
         write(irefwr,1) 'iqmat, jelp', iqmat, jelp

      end if  ! ( debug )

      kfirst = 1
      idim = ndim
      if (iqmat==0) then
         if ( (jelp<=2) .or. (ndim==1) ) then

!        --- ndim = 1

            do i = in2, in3
               if ( ind(i)/=0 ) then
                  call errint ( i+jshift, 1 )
                  call errint ( ielem, 2 )
                  call errint ( itype, 3 )
                  call errint ( ielgrp, 4 )
                  call errwar ( 1768, 4, 0, 0 )
                  ind(i) = 0
               end if
            end do

            if ( ind(in1)==0 ) then

!           --- u1 = 0

               idim = 0

            end if

         else if ( (jelp>2 .and. jelp<=10 ) .or. &
                   (ndim==2) ) then

!        --- ndim = 2

            if ( ind(in3)/=0 ) then
               call errint ( in3+jshift, 1 )
               call errint ( ielem, 2 )
               call errint ( itype, 3 )
               call errint ( ielgrp, 4 )
               call errwar ( 1768, 4, 0, 0 )
               ind(in3) = 0
            end if
            if ( ind(in1)==0 ) then

!           --- u1 = 0

               if ( ind(in2)==0 ) then

!              --- u1 = 0 and u2 = 0

                  idim = 0

               else

!              --- u1 = 0, u2 # 0

                  idim = 1
                  kfirst = 2

               end if

            else if ( ind(in2)==0 ) then

!           --- u2 = 0 and u1 # 0

               idim = 1

            else

!           --- General

               idim = 2

            end if

         else

!        --- ndim = 3

            if ( ind(in1)==0 ) then

!           --- u1 = 0

               if ( ind(in2)==0 ) then

!              --- u1 = 0 and u2 = 0

                  if ( ind(in3)==0 ) then

!                 --- u1 = 0, u2 = 0 and u3 = 0

                     idim = 0

                  else

!                 --- u1 = 0 and u2 = 0

                     idim = 1
                     kfirst = 3

                  end if

               else

!              --- u1 = 0

                  idim = 2
                  kfirst = 2

               end if

            else if ( ind(in2)==0 .and. ind(in3)==0 ) then

!           --- u2 = 0 and u3 = 0

                  idim = 1

            else

!           --- General

               idim = 3

            end if

         end if

      else

!     --- iqmat = 1

         if ( (jelp<=2) .or. (ndim==1) ) then
            idim = 1
         else if ( (jelp>2 .and. jelp<=10 ) .or. &
                   (ndim==2) ) then
            idim = 2
         else
            idim = 3
         end if

      end if

1000  call erclos ( 'el1013' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End el1013'

      end if  ! ( debug )

      end
