      subroutine el3011 ( dcoef, s, w, phix, mdecl, ndecl, id1, id2, &
                          work, symm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 04-06-1999 Include commons
!        version  1.0  date 12-05-1993
!
!   copyright (c) 1993-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute contribution to element stiffness matrix corresponding to a
!     part of the second derivative part of an ellipticl equation, i.e.
!        s  = s   + / d dphi /dx   d phi  / dx
!         ij   ij   e       i   id1      j     id2
!                               k          k                 k
!           = s   + sum  w  (d(x ) d phi (x ) / dx   d phi (x ) / dx
!              ij    k    k             i         id1      j         id2
!     el3011 is called by several types of element subroutines
!            It must be preceded by ELP subroutines of the 6-series
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer mdecl, ndecl, id1, id2
      double precision dcoef(mdecl), s(ndecl,ndecl), w(mdecl), &
                       phix(ndecl,mdecl,*), work(mdecl)
      logical symm

!     dcoef          i    array of length m in which the values of the
!                         coefficient are stored
!     id1            i    Index of first derivative (see description)
!     id2            i    Index of second derivative (see description)
!     mdecl          i    number of integration points in element, &
!                         same as  m  in common /celint/
!     ndecl          i    number of nodal points in element, &
!                         same as  n  in common /celint/
!     phix           i    Array of length n x m x ndim  containing the
!                         derivatives of the basis functions in the sequence:
!                         d phi / dx (xg ) = phix (i,k,1);
!                              i        k
!                         d phi / dy (xg ) = phix (i,k,2);
!                              i        k
!                         d phi / dz (xg ) = phix (i,k,3);
!                              i        k
!                         If the element is a linear tetrahedron (n=4), then
!                         the derivatives are constant and only k = 1 is filled.
!     s              o    Element matrix to be filled
!                         Since SEPRAN assumes an one-dimensional storage, S
!                         must be stored transposed.
!     symm           i    Indication whether the resulting matrix is symmetrical
!                         (true) or not (false)
!     w              i    array of length m containing the weights for
!                         integration
!     work                Work array of length m , to store products of
!                         dcoef x w temporarily
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision sum
      integer i, j, k

!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     sum            Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if ( jdercn.eq.0 ) then

!     --- jdercn = 0, derivatives constant

         sum = 0d0
         do k = 1, m
            sum = sum + w(k) * dcoef(k)
         end do
         if ( jadd.eq.0 ) then

!        --- jadd = 0, new matrix

            if ( symm ) then

!           --- symmetrical case

               do i = 1, n
                  do j = 1, i
                     s(i,j) = sum*phix(i,1,id1)*phix(j,1,id1)
                  end do
               end do
            else

!           --- non-symmetrical case

               do i = 1, n
                  do j = 1, n
                     s(j,i) = sum*phix(i,1,id1)*phix(j,1,id2)
                  end do
               end do
            end if
         else

!        --- jadd = 1, add to existing matrix

            if ( symm ) then

!           --- symmetrical case

               do i = 1, n
                  do j = 1, i
                     s(i,j) = s(i,j) + sum*phix(i,1,id1)*phix(j,1,id1)
                  end do
               end do
            else

!           --- non-symmetrical case

               do i = 1, n
                  do j = 1, n
                     s(j,i) = s(j,i) + sum*phix(i,1,id1)*phix(j,1,id2)
                  end do
               end do
            end if
         end if
      else

!     --- jdercn = 1, derivatives variable

         do k = 1, m
            work(k) = w(k) * dcoef(k)
         end do
         if ( jadd.eq.0 ) then

!        --- jadd = 0, new matrix

            if ( symm ) then

!           --- symmetrical case

               do i = 1, n
                  do j = 1, i
                     sum = 0d0
                     do k = 1, m
                        sum = sum + work(k)*phix(i,k,id1)*phix(j,k,id1)
                     end do
                     s(i,j) = sum
                  end do
               end do
            else

!           --- non-symmetrical case

               do i = 1, n
                  do j = 1, n
                     sum = 0d0
                     do k = 1, m
                        sum = sum + work(k)*phix(i,k,id1)*phix(j,k,id2)
                     end do
                     s(j,i) = sum
                  end do
               end do
            end if
         else

!        --- jadd = 1, add to existing matrix

            if ( symm ) then

!           --- symmetrical case

               do i = 1, n
                  do j = 1, i
                     sum = 0d0
                     do k = 1, m
                        sum = sum + work(k)*phix(i,k,id1)*phix(j,k,id1)
                     end do
                     s(i,j) = s(i,j) + sum
                  end do
               end do
            else

!           --- non-symmetrical case

               do i = 1, n
                  do j = 1, n
                     sum = 0d0
                     do k = 1, m
                        sum = sum + work(k)*phix(i,k,id1)*phix(j,k,id2)
                     end do
                     s(j,i) = s(j,i) + sum
                  end do
               end do
            end if
         end if
      end if
      if ( symm ) then

!     --- symmetrical case

         do i = 1, n-1
            do j = i+1, n
               s(i,j) = s(j,i)
            end do
         end do
      end if
      end
