      subroutine el3014 ( gamma, elemvc, w, dphids, tang, phi, xgauss, &
                          x, isequv )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 01-03-2004 Extra parameter isequv
!        version  2.0  date 31-03-2003 Extra parameters necessary for
!                                      axi-symmetric coordinates
!        version  1.1  date 12-03-2003 Extension for linear elements
!
!   copyright (c) 1999-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute of surface tension contribution to element vector, i.e.
!     elemvc(.,i)  = elemvc(.,i)  - / gamma t(.) dphi_i/ds ds
!        = elemvc(.,i)  - sum_k w_k gamma(x_k)  dphi_i/ds (x_k)
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     element_vector
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleconsts
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision gamma(m), elemvc(*), w(m), dphids(n,m), &
                       tang(m,2), phi(n,m), xgauss(m,2), x(n,2)
      integer isequv(2,*)

!     dphids         i    Contains the derivatives of the basis functions
!     elemvc        i/o   Element vector
!     gamma          i    array of length m in which the values of the surface
!                         tension parameter gamma in the integration points are
!                         stored
!     isequv         i    Array containing the positions of the velocities in
!                         the element vector in the sequence required by el2005
!     phi            i    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     tang           i    In this array the tangential vector is stored
!                         tang(j,i) contains the j th component of the
!                         tangential vector in the i-th integration point
!     w              i    array of length m containing the weights for
!                         integration
!     x              i    array of length n x ndim containing the co-ordinates
!                         of the points.
!     xgauss         i    array of length m x ndim containing the co-ordinates
!                         of the gauss points.  array xgauss is only filled when
!                         gauss integration is applied.
!                         xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                         zg_i = xgauss (i,3)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision h, sum1, sum2, drdx(m)
      integer i, k

!     drdx           Help array of size m to store dr/dx
!     h              Help variables to compute a sum or to store a quantity
!     i              Counting variables
!     k              Counting variables
!     sum1           Help variable to compute a sum or to store a quantity
!     sum2           Help variable to compute a sum or to store a quantity
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!      The contribution of the surface tension in the Cartesian case is given by
!
!      / gamma K_1 n_i phi_j ds, which can be written as
!
!      - / gamma t_i d phi_j/ds ds (see theoretical manual)
!
!      The contribution in the axisymmetric case is more complicated:
!
!      2 pi / gamma (K_1 +k_2)  n_i phi_j r ds, which can be written as
!
!      - 2 pi / gamma t_i d phi_j/ds r ds
!
!      - 2 pi / gamma t_i phi_j dr/ds ds
!
!      - 2 pi / gamma dz/ds  n_i phi_j ds
!
!     (see theoretical manual)
!
!      In this special case we did not include 2 pi r in the weights
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el3014' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el3014'
         write(irefwr,*) 'n, m, jadd, jdercn, jcart ', &
                         n, m, jadd, jdercn, jcart
         call prinrl ( w, m, 'w' )
         call prinrl ( gamma, m, 'gamma' )
         call prinrl1 ( tang, 2, m, 'tang' )
         call prinrl1 ( x, 2, m, 'xgauss' )
         call prinrl1 ( dphids, m, n, 'dphids' )
         call prinrl1 ( phi, m, n, 'phi' )

      end if

!     --- Check if vector must be added to an existing vector or not
!         If not set vector elemvc equal to zero

      if ( jadd.eq.0 ) then

!     --- jadd = 0, set vector equal to zero

         do i = 1, icount
            elemvc(i) = 0d0
         end do

      end if

!     --- Add vector to elemvc

      if ( jdercn.eq.0 ) then

!     --- Derivatives of basis functions are constant

         if ( jcart.eq.1 ) then

!        --- Cartesian coordinates

            do i = 1, n

               sum1 = 0d0
               sum2 = 0d0
               do k = 1, m
                  h = w(k)*gamma(k) * dphids(i,1)
                  sum1 = sum1 + h*tang(k,1)
                  sum2 = sum2 + h*tang(k,2)
               end do
               elemvc(isequv(1,i)) = elemvc(isequv(1,i)) - sum1
               elemvc(isequv(2,i)) = elemvc(isequv(2,i)) - sum2

            end do

         else

!        --- Axi-symmetric coordinates
!            First compute drdx in the integration points

            do k = 1, m
               sum1 = 0d0
               do i = 1, n
                  sum1 = sum1+x(i,1)*dphids(i,1)
               end do
               drdx(k) = sum1
            end do

!           --- Next term due to kappa_1

            do i = 1, n

               sum1 = 0d0
               sum2 = 0d0
               do k = 1, m
                  h = w(k)*gamma(k) * ( dphids(i,1)*xgauss(k,1) + &
                                        phi(i,k)*drdx(k) )
                  sum1 = sum1 + h*tang(k,1)
                  sum2 = sum2 + h*tang(k,2)
               end do
               elemvc(isequv(1,i)) = elemvc(isequv(1,i)) - sum1*pi2
               elemvc(isequv(2,i)) = elemvc(isequv(2,i)) - sum2*pi2

            end do

!           --- Finally term due to kappa_2

            do i = 1, n

               sum1 = 0d0
               sum2 = 0d0
               do k = 1, m
                  h = w(k)*gamma(k) * phi(i,k)*tang(k,2)
                  sum1 = sum1 + h*tang(k,2)
                  sum2 = sum2 - h*tang(k,1)
               end do
               elemvc(isequv(1,i)) = elemvc(isequv(1,i)) - sum1*pi2
               elemvc(isequv(2,i)) = elemvc(isequv(2,i)) - sum2*pi2

            end do

         end if

      else

!     --- Derivatives of basis functions are variable

         if ( jcart.eq.1 ) then

!        --- Cartesian coordinates

            do i = 1, n
               sum1 = 0d0
               sum2 = 0d0
               do k = 1, m
                  h = w(k)*gamma(k) * dphids(i,k)
                  sum1 = sum1 + h*tang(k,1)
                  sum2 = sum2 + h*tang(k,2)
               end do
               elemvc(isequv(1,i)) = elemvc(isequv(1,i)) - sum1
               elemvc(isequv(2,i)) = elemvc(isequv(2,i)) - sum2
            end do

         else

!        --- Axi-symmetric coordinates
!            First compute drdx in the integration points

            do k = 1, m
               sum1 = 0d0
               do i = 1, n
                  sum1 = sum1+x(i,1)*dphids(i,k)
               end do
               drdx(k) = sum1
            end do

!           --- Next term due to kappa_1

            do i = 1, n
               sum1 = 0d0

               sum2 = 0d0
               do k = 1, m
                  h = w(k)*gamma(k) * ( dphids(i,k)*xgauss(k,1) + &
                                        phi(i,k)*drdx(k) )
                  sum1 = sum1 + h*tang(k,1)
                  sum2 = sum2 + h*tang(k,2)
               end do
               elemvc(isequv(1,i)) = elemvc(isequv(1,i)) - sum1*pi2
               elemvc(isequv(2,i)) = elemvc(isequv(2,i)) - sum2*pi2

            end do

!           --- Finally term due to kappa_2

            do i = 1, n

               sum1 = 0d0
               sum2 = 0d0
               do k = 1, m
                  h = w(k)*gamma(k) * phi(i,k)*tang(k,2)
                  sum1 = sum1 + h*tang(k,2)
                  sum2 = sum2 - h*tang(k,1)
               end do
               elemvc(isequv(1,i)) = elemvc(isequv(1,i)) - sum1*pi2
               elemvc(isequv(2,i)) = elemvc(isequv(2,i)) - sum2*pi2

            end do

         end if

      end if

      call erclos ( 'el3014' )

      if ( debug ) then

!     --- Debug information

         call prinrl1 ( elemvc, n, 2, 'elemvc' )
         write(irefwr,*) 'End el3014'

      end if

      end
