      subroutine el6090 ( u1, u2, u3, s, w1, d1, p1, p2, p3, p4, &
                          p5, p6, p7, p8, p9, work, mod, md )
! ======================================================================
!
!        programmer    Patrick Anderson
!        version  1.0  date 03-01-2000
!
!   copyright (c) 2000-2000  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute contributions to spectral element stiffness matrix
!     corresponding to the convective part of the equation - 3-D case
!     EL6091 is called by the element subroutine ELM600
!            it must be preceded by the subroutines ELP640 and ELP646
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elliptic_equation
!     initialization
!     parabolic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer mod, md
      double precision u1(mod,mod,mod), u2(mod,mod,mod), &
                       u3(mod,mod,mod), s(md,md), w1(mod), &
                       d1(mod,mod), &
                       p1(mod,mod,mod), p2(mod,mod,mod), &
                       p3(mod,mod,mod), p4(mod,mod,mod), &
                       p5(mod,mod,mod), p6(mod,mod,mod), &
                       p7(mod,mod,mod), p8(mod,mod,mod), &
                       p9(mod,mod,mod), work(mod,mod,mod)

!     d1       i   array of length mod x mod contaning the values of the
!                  derivatives of the basisfunctions in the x-direction
!     md       i   number of integration points in the element
!     mod      i   number of integration points in one dimension
!     p1       i   arrays of length mod x mod containing the partial
!                  derivatives of the coordinates due to transformation
!     p2       i   See p1
!     p3       i   See p1
!     p4       i   See p1
!     p5       i   See p1
!     p6       i   See p1
!     p7       i   See p1
!     p8       i   See p1
!     p9       i   See p1
!     s        o   array of length md x md containing the spectral
!                  element stiffness matrix, since SEPRAN assumes an
!                  one-dimensional storage, s must be stored transposed
!     w1       i   array of length mod containing the weights for
!                  numerical integration in the x-direction corresponding
!                  to the reference element
!     u1       i   First component of velocity vector
!     u2       i   Second component of velocity vector
!     u3       i   Third component of velocity vector
!     work         work array of length md, to store products temporarily
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, l, ijk, ljk, mtd
      double precision h1, h2, h3

!     h1         Help variable to compute a product
!     h2         Help variable to compute a product
!     h3         Help variable to compute a product
!     i          Counting variable
!     ijk        position in the matrix s
!     j          Counting variable
!     k          Counting variable
!     l          General loop variable
!     ljk        position in the matrix s
!     mtd        mtd = mod*mod
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      mtd = mod*mod

!     --- first initialize s as zero, if necessary

      if (jadd.eq.0) then
         do  i = 1, md
            do  j = 1, md
               s(i,j) = 0d0
         end do
      end do
      end if

!     --- store first help factor in work

      do i = 1, mod
         do j = 1, mod
            do k = 1, mod
               h1 = u1(i,j,k) * p1 (i,j,k)
               h2 = u2(i,j,k) * p2 (i,j,k)
               h3 = u3(i,j,k) * p3 (i,j,k)
               work(i,j,k) = (h1+h2+h3)*w1(i)*w1(j)*w1(k)
            end do
         end do
      end do

!     --- add first factor of convection matrix to s

      do i = 1, mod
         do j = 1, mod
            do k = 1, mod
               ijk = i + (j-1)*mod + (k-1)*mtd
               do l = 1, mod
                  ljk = l + (j-1)*mod + (k-1)*mtd
                  s(ljk,ijk) = s(ljk,ijk) + &
                               work(i,j,k)*d1(i,l)
               end do
            end do
         end do
      end do

!     --- store second help factor in work

      do i = 1, mod
         do j = 1, mod
            do k = 1, mod
               h1 = u1(i,j,k) * p4 (i,j,k)
               h2 = u2(i,j,k) * p5 (i,j,k)
               h3 = u3(i,j,k) * p6 (i,j,k)
               work(i,j,k) = (h1+h2+h3)*w1(i)*w1(j)*w1(k)
            end do
         end do
      end do

!     --- add second factor of convection matrix to s

      do i = 1, mod
         do j = 1, mod
            do k = 1, mod
               ijk = i + (j-1)*mod + (k-1)*mtd
               do l = 1, mod
                  ljk = i + (l-1)*mod + (k-1)*mtd
                  s(ljk,ijk) = s(ljk,ijk) + &
                               work(i,j,k)*d1(j,l)
               end do
            end do
         end do
      end do

!     --- store third help factor in work

      do i = 1, mod
         do j = 1, mod
            do k = 1, mod
               h1 = u1(i,j,k) * p7(i,j,k)
               h2 = u2(i,j,k) * p8(i,j,k)
               h3 = u3(i,j,k) * p9(i,j,k)
               work(i,j,k) = (h1+h2+h3)*w1(i)*w1(j)*w1(k)
            end do
         end do
      end do

!     --- add third factor of convection matrix to s

      do i = 1, mod
         do j = 1, mod
            do k = 1, mod
               ijk = i + (j-1)*mod + (k-1)*mtd
               do l = 1, mod
                  ljk = i + (j-1)*mod + (l-1)*mtd
                  s(ljk,ijk) = s(ljk,ijk) + &
                               work(i,j,k)*d1(k,l)
               end do
            end do
         end do
      end do
      end
