      subroutine eld800 ( vecloc, work, &
                          phi, psi, phixi, psixi )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.0  date 06-01-2016 Use sepmoduleelem
!        version  4.0  date 21-08-2013 Extra parameters
!        version  3.20 date 09-03-2013 Replace elflr7 by elflr71
!
!   copyright (c) 1992-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element derivative vector and weights for second order elliptic
!     equations (General case) or for local error estimates
!     One, two and three dimensional elements
!     ELD800 is a help subroutine for subroutine DERIV (TO0047)
!     it is called through the intermediate subroutine eld000
!     So:
!     DERIV
!     TO0047
!       -  Loop over elements
!          -  ELD000
!             - ELD800
!    The subroutine is able to handle the following element shapes and cases:
!    linear line element (1D)
!    quadratic line element (1D)
!    linear triangle (2D and 3D)
!    quadratic triangle (2D and 3D)
!    bilinear quadrilateral (2D)
!    biquadratic quadrilateral (2D):
!    linear pyramid (3D)
!    quadratic pyramid (3D)
!    trilinear brick (3D)
!    triquadratic brick (3D)
!    cartesian co-ordinates
!    axi-symmetric co-ordinates (2D only)
!    polar co-ordinates (1D only)
!    Type numbers permitted:
!       325:  Reynolds equation
!       800:  general second order equation with one unknown per point
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     elliptic_equation
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      use sepmoduleoldvecs
      use sepmodulecactl
      use sepmoduleelem
      use sepmoduleelm
      use sepmodulebasefn
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

      double precision vecloc(*), work(*), &
                       phi(*), phixi(*), psi(*), psixi(*)

!     phi           i/o   array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     phixi         i/o   Array of length n x m x ndim containing the values of
!                         the derivatives of the basis functions in the Gauss
!                         points in the reference element.
!                         Array phi is only filled when ifirst = 0 and must be
!                         kept by the program
!     psi           i/o   Array of length n x m containing the values of the
!                         modified basis functions in the integration points
!     psixi         i/o   Array of length nvert x m x ndim containing the values
!                         of the derivatives of the basis functions in the Gauss
!                         points in the reference element
!     vecloc         i    Work array in which all old solution vectors for the
!                         integration points are stored
!     work           Real work array
!                    work consists of the following parts
!                    Starting address    length   Name:
!                    1                   ncoef*27 ARRAY
!                                                 Type 800: ncoef=6,type 325:14)
!                    ipdpdx              27*27*3  DPHIDX
!                    ipdudx              27*3     dudx
!                    ipsmin              27*3     smagin
!                    ipsmno              27*3     smagno
!                    ipwork              27       Work space
!                    Total length: 4806
!                    The subarrays contain the folowing contents:
!                    ARRAY:
!                    array in which the values of the coefficients
!                    in the integration points are stored in the sequence:
!                    Type 800:
!                         1D                  2D            3D
!                    1:   alpha 11         alpha 11       alpha 11
!                    2:                    alpha 12       alpha 12
!                    3:                    alpha 22       alpha 13
!                    4:                                   alpha 22
!                    5:                                   alpha 23
!                    6:                                   alpha 33
!                    Type 325:
!                         1D                2D
!                    1:   h                h
!                    2:   mu               mu
!                    3:   ht               ht
!                    4:   k                k
!                    5:   p0               p0
!                    6:   u_1 1            u_1 1
!                    7:                    u_1 2
!                    8:
!                    9:   rho              rho
!                   10:   RT               RT
!                   11:   f1               f1
!                   12:                    f2
!                   13:   u_2 1            u_2 1
!                   14:                    u_2 2
!                    Type 810:
!                    1:   alpha 11         alpha 11       alpha 11
!                    2:                    alpha 12       alpha 12
!                    3:                    alpha 22       alpha 13
!                    4:                                   alpha 22
!                    5:                                   alpha 23
!                    6:                                   alpha 33
!                    7:   u 1              u 1            u 1
!                    8:                    u 2            u 2
!                    9:                                   u 3
!                   10:   beta             beta           beta
!                   11:   f                f              f
!                   12:   rho              rho            rho
!                   13:   capacity_s       capacity_s     capacity_s
!                   14:   capacity_l       capacity_l     capacity_l
!                   15:   latent_heat      latent_heat    latent_heat
!                   16:   melt_temp        melt_temp      melt_temp
!                    Each coefficient requires exact m (is number of
!                    integration points) positions
!                    DPHIDX:
!                    Array of length inpelm x m x ndim containing the
!                    derivatives of the basis functions in the sequence:
!                    d phi / dx = dphidx (i,k,1);  d phi / dy = dphidx (i,k,2);
!                         i                             i
!                    d phi / dz = dphidx (i,k,3)
!                         i
!                      in node k
!                    DUDX:
!                    Array containing the derivatives in the integration points
!                    SMAGNO:
!                    n x ndim array to store the smoothed vector
!                    a * grad u  in the nodal points
!                    SMAGIN:
!                    m x ndim array to store the smoothed vector
!                    a * grad u  in the integration points
!                    WORK
!                    General work space
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, isub, isup, jelp, kfirst, jheat, icounw, jtemp1, &
              jtemp2, jtemp3, iseq, ipdpdx, ipdudx, ipend, &
              ipsmin, ipsmno, ipwork, ibear, newton, itypln, &
              ncoef, itypeeq, intdiscrule, ipalpha, &
              ipu1, ipu2, ipheight, ipf0, ipfrhsd, iprho, ipgamma, &
              imass_mat
      integer, pointer :: iuser(:), islold(:)
      double precision, pointer :: user(:)
      double precision, pointer :: vecoldlc(:)
      double precision qmat(3,27), tang(3*9), x(81), w(27), xgauss(81), &
                       uold(27), ugauss(27), dum(100)
      save isub, isup, jelp, kfirst, jheat, icounw, ipdpdx, ipdudx, &
           ipsmin, ipsmno, ipwork, iseq, ibear, &
           itypeeq, ipalpha, &
           ipu1, ipu2, ipheight, ipf0, ipfrhsd, iprho, ipgamma

!     dum            General double precision dummy variable
!     i              Counting variable
!     ibear          Indicates type of bearing.
!                    Possible values:
!                    0:  incompressible bearing
!                    1:  compressible bearing (adiabatic)
!     icounw         length of elemwg(*)
!     imass_mat      dummy
!     intdiscrule    dummy
!     ipalpha        Starting position of diffusion tensor in coeffarr
!     ipdpdx         Starting address in array work of dphidx
!     ipdudx         Starting address in array work of dudx
!     ipend          Last position in array
!     ipf0           Starting address of array f0
!     ipfrhsd        Starting address of array frhsd
!     ipgamma        Starting address of array gamma
!     ipheight       Starting address of array height
!     iprho          Starting address of rho
!     ipsmin         Starting address in array work of smagin
!     ipsmno         Starting address in array work of smagno
!     ipu1           Starting address of array u1
!     ipu2           Starting address of array u2
!     ipwork         Starting address in array work of work array
!     iseq           Sequence number of vector in VECOLD/ISLOLD
!     isub           Integer help parameter for do-loops to indicate the
!                    smallest coefficient number that is dependent on space
!     isup           Integer help parameter for do-loops to indicate the
!                    largest coefficient number that is dependent on space
!     itypeeq        if 0 the standard enthalpy equation is solved
!                    if 1 the equation is restricted to a solid temperature
!                    equation
!     itypln         Dummy parameter
!     jelp           Choice parameter to distinguish between the types of
!                    element subroutines to be called.
!     jheat          Integer parameter indicating if the mass matrix for the
!                    heat equation must be computed (>0) or not ( = 0)
!                    If jheat = 1, then the coefficient for the mass matrix is
!                    constant, otherwise (jheat = 2) it is variable
!     jtemp1         ?
!     jtemp2         ?
!     jtemp3         ?
!     kfirst         This parameter indicates the smallest velocity component
!                    that is not equal to zero in the convective terms
!                    For example kfirst = 2, means that the y-
!                    component of the velocity is unequal to zero
!     ncoef          Number of coefficients
!     newton         Dummy parameter
!     qmat           transformation matrix global -> local coordinates
!                    in case of a 3D element
!     tang           In this array the tangential vector is stored
!                    tang(j,i) contains the j th component of the
!                    tangential vector in the i-th integration point
!     ugauss         Preceding solution in Gauss points
!     uold           Array containing the previous iteration (old solution)
!     w              array of length m containing the weights for
!                    integration
!     x              x co-ordinate of point
!     xgauss         array of length m x ndim containing the co-ordinates
!                    of the gauss points.  array xgauss is only filled when
!                    gauss integration is applied.
!                    xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                    zg_i = xgauss (i,3)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL0325         Initialize parameters for subroutine ELM325
!     EL0800         Initialize parameters for subroutine ELD800
!     EL0807         performs the initializations for subroutine ELM807
!     EL0810         performs the initializations for subroutine ELM810
!     EL1001         Fill derivatives and weights for ELD800
!     EL1012         Fill element vector for flow computation of Reynolds
!                    equation
!     EL1017         Fill element vector for flow vector or shear stress
!                    of Reynolds equation
!     EL1017800      Fill element vector for flow vector and shear stress of
!                    Reynolds equation, incompressible bearing
!                    Special case where element 800 is used
!     EL2018         Fill elemvc1 for a-posteriori error estimations
!     EL3007         Fills old solution and derivatives
!     EL3009         Fill series of old solutions in array vecloc
!     ELFLR71        Fill variable coefficients in array ARRAY
!     ELIND0         Fill array ICH in COMMON CELIAR for preceding solutions
!     ELM800BASEFN   Compute the basis functions phi, its derivatives and other
!                    related quantities for various element shapes
!     ELMASSENTHAL   Compute the element-wise inner product of the mass matrix
!                    and a vector, where the vector is defined per
!                    integration point
!     ELRHSDENTHAL   Compute the special part of the right-hand side of the
!                    enthalpy equation in case of a newton linearization
!     ELTEMP2ENTHAL  Compute enthalpy as function of the temperature
!     ELUPDATENTHAL  Update the enthalpy during one iteration of the newton
!                    iteration process
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays IUSER and USER
!     Usually they are filled by FILCOF
!     The sequence is:
!     1:  ISEQ:  Sequence number of vector in VECOLD/ISLOLD from which UOLD and
!         derivatives must be computed.
!         If 0 or 1 the first vector is used
!         If more vectors are needed, these vectors are supposed to be stored
!         sequentially, where the first one is indicated by ISEQ
!     2-5:  Not yet used
!         In the cases ICHELD = 6, 16, 26-29 the positions 6-11 contain
!         information about the matrix A:
!     6:  alpha 11  (1D/2D/3D)
!     7:  alpha 12  (2D/3D)
!     8:  alpha 13  (3D)
!     9:  alpha 22  (2D/3D)
!    10:  alpha 23  (3D)
!    11:  alpha 33  (3D)
!         In the case ICHELD = 21 position 6 contains information about the
!         function f to be filled:
!     6:  Function f
!
!     Meaning of values of icheld
!      1-10: defined in each node
!      1:   dc/dx_i
!      2:   grad c
!      3:   - grad c
!      4:   -
!      5:   (dc/dy, -dc/dx)  (2d only)
!      6:   - A . grad c                Not type 807
!      7:   div (v), where v is a vector
!      8:   -
!      9:   -
!     10:   -
!     11-20: see 10, now defined per element
!     21:    Put function f in sixth coefficient (build vector)
!     22:    flow per node (type 325 only)
!     23:    flow vector (type 325 only)
!     24:    shear stress at bottom (type 325 only)
!     25:    shear stress at top (type 325 only)
!     26-29: Reserved for adaptive mesh refinement
!     31-40: see 10, now weight = 1
!     41:    compute enthalpy from temperature (type 810 only)
!     42:    compute rhside for Nedjar method (type 810 only)
!     43:    update enthalpy (type 810 only)
!     44:    compute Mass matrix times enthalpy (type 810 only)
!     45:    compute vector of elements to refine (types 807/810 only)
!     In case of type 810: see elm800
!
!     Structure of output arrays:
!     Solution array:  1 unknown per point (IVEC=0)
!     Arrays of special structure:
!     IVEC = 5:   NDIM unknowns per point
!     IVEC = -1:  1 unknown per element
!     IVEC = -2:  NDIM unknowns per element
!     IVEC = -7:  1 unknown per integration point
!      41                     -7
!     ICHELD                 IVEC
!       1                      0
!       2                      5
!       3                      5
!       4                      -
!       5                      5
!       6                      5
!       7                      -
!       8                      -
!       9                      -
!      10                      -
!      11                     -1
!      12                     -2
!      13                     -2
!      14                      -
!      15                     -2
!      16                     -2
!      17                      -
!      18                      -
!      19                      -
!      20                      -
!      21                      0
!      22                      5
!      23                      -
!      24                      -
!      25                      -
!      26                      5
!      27                      5
!      28                      0
!      29                     -2
!      41                     -7
!      42                      0
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     406   Value of icheld not allowed
! **********************************************************************
!
!                       PSEUDO CODE
!
!     if ( ifirst=0 ) then
!        Define element independent quantities
!        Compute length of array elemwg
!        Compute pointers in array WORK
!     fi
!     Compute basis functions and related quantities
!     Fill old solution into array UOLD and if necessary the derivatives
!     Compute variable coefficients if necessary
!     Compute derived quantities
!                              METHOD
!       Differential equation:
!         - div alpha grad phi + u + grad phi + beta phi = f
!                                -
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'eld800' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from eld800'
         write(irefwr,1) 'ifirst, icheld', ifirst, icheld
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      iuser => E%iuser
      user => E%user
      islold => E%islold

      if ( ifirst==0 )  then

!     --- ifirst = 0,  compute element independent quantities

         iseq = 0
         if ( itype==325 .and. icheld>=22 .and. icheld<=25 ) then

!        --- special case for Reynolds equation: compute flow

            call  el0325 ( iuser, user, jelp, ibear, newton, kfirst, &
                           isub, isup, work, iseq )
            ncoef = 14

         else if ( itype==325 .and. mod(icheld,10)==6 ) then

!        --- Not implemented

            call errint ( icheld, 1 )
            call errint ( itype, 2 )
            call errint ( ielgrp, 3 )
            call errsub ( 406, 3, 0, 0 )

         else if ( itype==807 ) then

!        --- special case for Newton method in combination
!            with enthalpy compuation

            call el0807 ( iuser, user, jelp, isub, isup, work, iseq, &
                          intdiscrule )
            ncoef = 0
            iseqenth = 0
            iseqtemp = iseq

         else if ( itype==810 ) then

!        --- special case for Nedjar method in combination
!            with enthalpy compuation

            call el0810 ( iuser, user, jelp, itypeeq, jheat, isub, &
                          isup, work, iseq )
            ncoef = 12
            if ( icheld==44 ) ncoef = 0

         else

!        --- standard case

            call  el0800 ( iuser, user, jelp, kfirst, jheat, isub, isup, &
                           work, iseq, itypln, imass_mat )

            if ( icheld==44 ) then
               ncoef = 0
            else if ( icheld>=23 .and. icheld<=25 ) then
               ncoef = 20
               ipalpha = 1
               ipu1 = 1+6*m
               ipf0 = 1+9*m
               ipfrhsd = 1+10*m
               iprho = 1+12*m
               ipgamma = 1+13*m
               ipheight = 1+16*m
               ipu2 = 1+17*m
            else
               ncoef = 6
            end if  ! ( icheld==44 )

         end if
         if ( ierror/=0 ) go to 1000

!        --- Fill array ich (COMMON CELIAR) for preceding solutions

         call elind0 ( islold, numold, isub, isup, iuser )

!        --- Length of elemwg(*) to be filled

         if ( icheld==1 .or. icheld==21 .or. icheld==22 .or. &
              icheld==31 .or. icheld==7 ) then

!        --- icheld = 1, 21 or 31 (one value per node)

            icounw = n

         else if ( icheld==28 ) then

!        --- icheld = 28  (one value per vertex)

            icounw = nvert

         else if ( icheld>10 .and. icheld<=20 .or. &
                   icheld==29 .or. icheld>=41 ) then

!        --- icheld = 11-20 or 29, 41;  (defined per element, hence no weights)

            icounw = 0

         else

!        --- icheld = 2-10, 26, 27, 32-40  (vector defined per node)

            icounw = ndim*n

         end if

!        --- Initialize weight vector, if any

         elemwg(1:icounw) = 1d0

!        --- Fill pointers in array work

         ipdpdx  = 1+ncoef*m
         ipdudx = ipdpdx + 3*n*m
         ipsmin = ipdudx + 3*m
         ipsmno = ipsmin + 3*m
         ipwork = ipsmno + 3*m
         ipend  = ipwork + m
         call inichkarlen ( ipend, 7857, 'work' )
         if ( ierror/=0 ) go to 1000

      end if

!     --- compute basis functions and weights for numerical integration

      call elm800basefn ( coor, x, w, work(ipdpdx), xgauss, phi, &
                          index1, qmat, tang, jelp, phixi, &
                          psi, work(ipwork), psixi )
      if ( ierror/=0 ) go to 1000

!     --- Fill uold ( in nodal points ), it is needed inside el3007
!         Fill dudx ( in integration points ) if necessary
!                   ( when jdercn=0 only in the first point )

      if ( ind(lengthmst)==1 .or. ind(lengthmst)==3 ) &
         call el3007 ( uold, work(ipdudx), phi, &
                       work(ipdpdx), m,  n, ndimlc, ncomp, &
                       index1, dum, ugauss, jtrans, numold, iseq )
      if ( ind(lengthmst)==2 .or. ind(lengthmst)==3 ) &
         call el3009 ( vecloc, phi, n )

!     --- Fill variable coefficients if necessary

      do i = isub, isup

         if ( ind(i)>0 .or. ist(i)>0 ) then

!        --- Coefficient is variable

            call elflr71 ( i, user, work, x, xgauss, &
                          m, n, phi, 1, index1, &
                          iuser, ugauss, vecloc )

         end if

      end do

!     --- Fill elemvc1 and elemwg, content depends on icheld

      select case (icheld)

         case(1:20, 31:40)

!        --- icheld<20 fill elemvc1 by derivatives,
!            elemwg by weights

            call el1001 ( icheld, ix, work(ipdpdx), work(ipdudx), &
                          elemvc1, elemwg, work, ndim, uold )

         case(21)

!        --- icheld = 21; Fill function

            do i = 1, n
               elemvc1(i) = work(i)
            end do

         case(22)

!        --- icheld<22 fill elemvc1 by flow for Reynolds equation

            call el1012 ( elemvc1, uold, work(ipdudx), work(ipdpdx), &
                          phi, w, work, ibear, work(ipwork), &
                          work(ipsmin) )

         case(23:25)

!        --- 23<icheld<25 fill elemvc1 by flow vector or shear stress
!            for Reynolds equation

            if ( itype==800 ) then

               call el1017800 ( icheld, elemvc1, n, ndim, work(ipalpha), &
                                uold, work(ipdudx), work(ipgamma), &
                                work(ipheight), work(iprho), &
                                work(ipfrhsd), work(ipf0), work(ipu1), &
                                work(ipu2) )

            else

               call el1017 ( elemvc1, uold, work(ipdudx), work, ibear, &
                             work(ipwork), icheld, ndim )

            end if  ! ( itype==800 )

         case(26:29)

!        --- icheld = 26 - 29

            if ( icheld==27 .or. icheld==29 ) then

!           --- icheld = 27, 29
!               save (temporary) some values in common

               jtemp1 = jderiv
               jtemp3 = ncomp

!              --- numold=2
!                  Fill smagin and smagno by smoothed  a * grad u

               jderiv = 0
               ncomp = 2

               call el3007 ( work(ipsmno), work(ipwork), &
                             phi, work(ipdpdx), m, n, ndimlc, &
                             ndimlc, index1, dum, work(ipsmin), &
                             jtrans, numold, iseq+1 )

!              --- Restore the values in common

               jderiv = jtemp1
               ncomp = jtemp3

            end if

!           --- Fill elemvc1 and elemwg for a-posteriori error estimations

            if ( icheld==28 ) then

!           --- icheld = 28
!               Get error estimate on element
!               and distribute it to vertices
!               !!! column  iseq+2  must be filled before this spot runs, &
!               by estimates for elements

               iseqtype = oldsols(iseq+2)%iseqtype
               index3n => indoldsols(iseqtype)%index3
               vecoldlc => oldsols(iseq+2)%oldvecs
               do i = 1, n
                  elemvc1 ( i ) = vecoldlc(index3n(1))
               end do

            else

!           --- icheld = 26, 27, 29
!               This part must be changed!!!!!!

               call el2018 ( icheld, work, work(ipdudx), work(ipsmin), &
                             work(ipsmno), m, n, ndimlc, w, phi, &
                             elemvc1, elemwg )

            end if

         case(41)

!        --- icheld = 41, compute enthalpy from temperature

            call eltemp2enthal ( elemvc1, ugauss, work, m, itypeeq )

         case(42)

!        --- icheld = 42, compute rhside for Nedjar method

            call elrhsdenthal ( elemvc1, vecloc, &
                                work, phi, w, itypeeq )

         case(43)

!        --- icheld = 43, update enthalpy

            call elupdatenthal ( elemvc1, vecloc, &
                                 work, itypeeq )

         case(44)

!        --- icheld = 44, compute Mass matrix times enthalpy
!                         in case of other types, use solution (temperature)

            if ( itype/=810 ) iseqenth = iseqin
            call elmassenthal ( elemvc1, vecloc, phi, w )

         case(45)

!        --- icheld = 45, compute vector of elements to refine
!            only available for integers

            call errint ( icheld, 1 )
            call errint ( itype, 2 )
            call errint ( ielgrp, 3 )
            call errsub ( 406, 3, 0, 0 )

      end select  ! case (icheld)

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( elemvc1, icount, 'elemvc1' )
         if ( icounw>0 ) call prinrl ( elemwg, icounw, 'elemwg' )
         write(irefwr,*) 'End eld800'

      end if  ! ( debug )
      call erclos ( 'eld800' )

      end
