      subroutine eldiscint ( massmt, stiffmt, dldtmt, latentvc, &
                             dphidx, temperature, coefarr, x, &
                             intdiscrule )
! ======================================================================
!
!        programmer    John Brusche
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.0  date 26-01-2005
!
!   copyright (c) 2005-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Computes the element matrices (mass, stiffness and dLdT) for
!    elements that are intersected by the free boundary.
!    The element vector for the latent heat contribution is also computed.
!    At the moment this routine is for 1D linear elements only.
!    The element e is divided into two sections: [x1,s] and [s,x2]
!    *-----------x---------------------------------* &
!    x1          xs                                x2
!    The principle of discontinuous integration prescribes that we
!    compute the integrals not over the whole element e but over the
!    two sections seperately and then add the results coherently.
!    The integrals are not computed analytically, but are instead
!    approximated using an integration rule.
!    At the moment only Newton Cotes is implemented.
!    For the entries of the element mass matrix we can write:
!    m_ij = rho*[ c_x1,xs*sum_(k=1)^(s)*w_x1,xs*phi_i(x_k)*phi_j(x_k) + &
!               + c_xs,x2*sum_(k=s)^(2)*w_xs,x2*phi_i(x_k)*phi_j(x_k) ]
!    According to the Sepran definition, phi(i,j) is the value of the
!    ith basis function in the jth integration.
!    We define phi(i,s) to be the value of the ith basis function in the
!    intersection point xs.
!    For the 'left' section [x1,xs] we then have a phi_left as follows:
!               / phi(1,1) phi(1,s) \
!    phi_left = |                   |
!               \ phi(2,1) phi(2,s) /
!    and
!                 / phi(1,s) phi(1,2) \
!    phi_lright = |                   |
!                 \ phi(2,s) phi(2,2) /
!    If we number the intervals (phases) from left to right as 1 and 2
!    then by phi(i,j,k) we can indicate the value of the jth basis
!    function in the kth integration point over the ith interval.
!    For the weights we define w(i,j) as the weigth in the jth integra- &
!    tion point on the ith interval.
!    w(1,j) = dh/2, w(2,j) = (h-dh)/2, &
!    where h is the length of the element and dh the length of the 'left'
!    section [x1,xs].
!    For the coefficients c we define c(i,j) as the c in the jth integra- &
!    tion point on the ith interval.
!    Depending on the values of the temperature T(x1) and T(x2) with
!    respect to the melting temperature Tm, c(i,j) = cs or c(i,j) = cl.
!    For the entries of the element stiffness matrix we can write:
!    s_ij = k_x1,xs*sum_(k=1)^(s)*w_x1,xs*dphidx_i(x_k)*dphidx_j(x_k) + &
!               + k_xs,x2*sum_(k=s)^(2)*w_xs,x2*dphidx_i(x_k)*dphidx_j(x_k)
!    where the dphidx's are the derivatives of the corresponding phi's.
!    Depending on the values of the temperature T(x1) and T(x2) with
!    respect to the melting temperature Tm, k(i,j) = ks or k(i,j) = kl.
!    Part of the Jacobian matrix contains the derivative of the latent
!    heat integral with respect to the temperature: dLdT.
!    The matrix elements for the element e are given by (Fachinotti):
!    dL/dT(i,j) = rho*L*phi(i,s)*phi(j,s)/|dT/dx(xs)|
!    For the latent heat we need to compute
!    L_i = int_omega H(T-Tm)*phi_i dx
!    where H denotes the Heaviside step-funtion. Since this function
!    has value zero in the solid subregion of Omega and is value 1 in
!    the liquid subregion of Omega, only the integral over the liquid
!    part is to be computed:
!    L_i = rho*L*sum_(k=left)^(right)*w_k*phi_i(x_k)
!    where left is 1 or s and right is s or 2 depending on which section
!    of the element contains the liquid
! **********************************************************************
!
!                       KEYWORDS
!
!     free_surface
!     element_matrix
!     element_vector
!     enthalpy
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer          intdiscrule
      double precision temperature(*), coefarr(m,*), &
                       dphidx(n,m,ndim), x(n,3), massmt(n,n), &
                       stiffmt(n,n), dldtmt(n,n), latentvc(n)

!     coefarr        i    Array containing the coefficients of the PDE
!                         Pos. (.,1)   k_solid
!                         Pos. (.,2)   k_fluid
!                         Pos. (.,3)   rho
!                         Pos. (.,4)   c_solid
!                         Pos. (.,5)   c_fluid
!                         Pos. (.,6)   Q
!                         Pos. (.,7)   L
!                         Pos. (.,8)   T_melt
!     dldtmt         o    Element dldt matrix
!     dphidx         i    derivatives of basis functions
!     intdiscrule    i    Numerical integration rule to be used for
!                         discontinuous integration
!                         1: Newton-Cotes, 1st order, 'substraction approach'
!                         2: Newton-Cotes, 2nd order, 'substraction approach'
!                         3: Newton-Cotes, 1st order, &
!                            'subdivision into 3 subtriangles'
!                         4: Newton-Cotes, 1st order, &
!                            'averaging of heat capacity'
!     latentvc       o    Element latent heat vector
!     massmt         o    Element mass matrix
!     stiffmt        o    Element stiffness matrix
!     temperature    i    temperature vector in integration points
!     x              i    x-coordinates of the integration points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELDISCINT1     Perform discontinuous integration 1D
!     ELDISCINT2     Perform discontinuous integration 2D
!     ELDISCINT3     Perform discontinuous integration 3D
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'eldiscint' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from eldiscint'

      end if
      if ( ierror/=0 ) go to 1000

!     --- get parameters

      if (ndim==1) then
         call eldiscint1( massmt, stiffmt, dldtmt, latentvc, &
                          dphidx, temperature, coefarr, m, n, x, &
                          intdiscrule )
      else if (ndim==2) then
         call eldiscint2( massmt, stiffmt, dldtmt, latentvc, &
                          dphidx, temperature, coefarr, x, &
                          intdiscrule )
      else if (ndim==3) then
         call eldiscint3( massmt, stiffmt, dldtmt, latentvc, &
                          dphidx, temperature, coefarr, x, &
                          intdiscrule )
      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End eldiscint'

      end if
      call erclos ( 'eldiscint' )

      end
