      subroutine eldiscint1 ( massmt, stiffmt, dldtmt, latentvc, &
                              dphidx, temperature, coefarr, m, n, x, &
                              intrule )
! ======================================================================
!
!        programmer    John Brusche
!        version  3.1  date 11-06-2010 New call toeldiscint1iso
!        version  3.0  date 30-05-2006 Fully revised layout
!        version  2.0  date 22-02-2006 Added mushy
!        version  1.3  date 21-02-2006 Fixed bug in indices latentvc
!
!   copyright (c) 2005-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Construct element mass matrix, stiffness matrix, derivative latent heat
!   matrix and latent heat vector using discontinous integration.
!   This is the 1D version.
!   Mushy: x_sol and x_liq are determined via linear interpolation
!   T_1      T_sol     T_liq      T_2
!    *---------*---------*---------*     T_1<T_sol<T_m<T_liq<T_2
!           (T_sol - T_1)*(x_2 - x1)
!   x_sol = ------------------------ + x_1
!                 (T_2 - T_1)
!   or with dhs = x_sol - x_1 and h = x_2 - x_1
!         (T_sol - T_1)*h
!   dhs = --------------- &
!           (T_2 - T_1)
!   A similar expression holds for dhl = x_2 - x_liq
!         (T_2- T_liq)*h
!   dhl = --------------- &
!           (T_2 - T_1)
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulemushy
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer          m, n, intrule
      double precision temperature(*), coefarr(m,*), &
                       dphidx(n,m), x(n), massmt(n,n), &
                       stiffmt(n,n), dldtmt(n,n), latentvc(n)

!     coefarr        i    Array containing the coefficients of the PDE
!                         Pos. (.,1)   k_solid
!                         Pos. (.,2)   k_fluid
!                         Pos. (.,3)   rho
!                         Pos. (.,4)   c_solid
!                         Pos. (.,5)   c_fluid
!                         Pos. (.,6)   Q
!                         Pos. (.,7)   L
!                         Pos. (.,8)   T_melt
!     dldtmt         o    Element dldt matrix
!     dphidx         i    derivatives of basis functions
!     intrule       i    Numerical integration rule to be used for
!                         discontinuous integration
!                         1: Newton-Cotes, 1st order, 'substraction approach'
!                         2: Newton-Cotes, 2nd order, 'substraction approach'
!                         3: Newton-Cotes, 1st order, &
!                            'subdivision into 3 subtriangles'
!                         4: Newton-Cotes, 1st order, &
!                            'averaging of heat capacity'
!     latentvc       o    Element latent heat vector
!     m              i    number of integration points
!     massmt         o    Element mass matrix
!     n              i    number of basis functions
!     stiffmt        o    Element stiffness matrix
!     temperature    i    temperature vector in integration points
!     x              i    x-coordinates of the integration points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, mloc, ind(2)
      parameter ( mloc = 2 )
      double precision h, phi(n,mloc)

!     h              width of the element
!     i              Counting variable
!     ind            Integer index array
!     j              Counting variable
!     mloc           local copy of m modified for Newton-Cotes
!     phi            basis functions
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELDISCINT1ISO  1D discontinuous integration, isothermal case
!     ELDISCINT1MUSH 1D discontinuous integration, artificial mushy case
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'eldiscint1' )
      debug = .false.

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from eldiscint1'

      end if

!     --- Initialize

      do i = 1, n
         do j = 1, n
            stiffmt(i,j) = 0d0
            massmt(i,j) = 0d0
            dldtmt(i,j) = 0d0
         end do
            latentvc(i) = 0d0
      end do

!     --- fill local phi using mloc

      do i = 1, n
         do j = 1, mloc
            if (i==j) then
               phi(i,j) = 1d0
            else
               phi(i,j) = 0d0
            end if
         end do
      end do

!     --- compute width h of the element

      h = abs(x(1) - x(2))

!     --- determine node numbers for smallest and largest temperature

      if ( temperature(1)<temperature(2) ) then
         ind(1) = 1
         ind(2) = 2
      else
         ind(1) = 2
         ind(2) = 1
      end if

!     --- distinct between mushy and isothermal

      if (mushy) then

         call eldiscint1mush ( massmt, stiffmt, dldtmt, latentvc, &
                               dphidx, temperature, coefarr, m, n, &
                               intrule, phi, h, ind, mloc )

      else ! isothermal

         call eldiscint1iso ( massmt, stiffmt, dldtmt, latentvc, &
                              dphidx, temperature, coefarr, &
                              intrule, phi, h, mloc )

      end if ! (mushy)

      call erclos ( 'eldiscint1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End eldiscint1'

      end if

      end
