      subroutine eldiscint2iso ( massmt, stiffmt, dldtmt, latentvc, &
                                 dphidx, temperature, coefarr, &
                                 intrule, phi, mloc, x )
! ======================================================================
!
!        programmer    John Brusche
!        version  3.0  date 06-01-2016 Use sepmoduleelem
!        version  2.1  date 04-08-2010 Extension with
!        version  2.0  date 16-06-2010 New parameter list
!
!   copyright (c) 2005-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Construct element mass matrix, stiffness matrix, derivative latent heat
!     matrix and latent heat vector using discontinous integration.
!     This is the isothermal case.
!     The code differs from that using an artificial mushy region, since the
!     dLdT matrix and L-vector are computed differently.
!     This is the 1D version.
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     heat_equation
!     latent_heat
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      use sepmoduleconsts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer          intrule, mloc
      double precision temperature(*), coefarr(m,*), x(n,ndim), &
                       dphidx(n,m,ndim), massmt(n,n), phi(n,mloc), &
                       stiffmt(n,n), dldtmt(n,n), latentvc(n)

!     coefarr        i    Array containing the coefficients of the PDE
!                         Pos. (.,1)   ksolid
!                         Pos. (.,2)   kfluid
!                         Pos. (.,3)   rho
!                         Pos. (.,4)   csolid
!                         Pos. (.,5)   cfluid
!                         Pos. (.,6)   Q
!                         Pos. (.,7)   L
!                         Pos. (.,8)   Tmelt
!     dldtmt         o    Element dldt matrix
!     dphidx         i    derivatives of basis functions
!     intrule       i    Numerical integration rule to be used for
!                         discontinuous integration
!                         1: Newton-Cotes, 1st order, 'substraction approach'
!                         2: Newton-Cotes, 2nd order, 'substraction approach'
!                         3: Newton-Cotes, 1st order, &
!                            'subdivision into 3 subtriangles'
!                         4: Newton-Cotes, 1st order, &
!                            'averaging of heat capacity'
!     latentvc       o    Element latent heat vector
!     massmt         o    Element mass matrix
!     stiffmt        o    Element stiffness matrix
!     temperature    i    temperature vector in integration points
!     x              i    x-coordinates of the integration points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, nnegativet, negativet(2), xhat(mloc), k
      double precision rhol, cs, cl, latentheat, tm, wtriangle1, &
                       ctriangle, kl, ks, gradtemp(ndim), nrmgradt, &
                       ktriangle1, ktriangle, det, det1, wphi1, eps, &
                       sum1, sum2, v(3, mloc), h(2), dh(2), phixs(n,3), &
                       wtriangle, ctriangle1, xs(2,2), phicut(n,3), &
                       dphidxcut(n,ndim), dxs, ind1(5,2), phicut2(n,3), &
                       phicut3(n,3), phi2(n,3), wtriangle2, wtriangle3, &
                       det2, det3, wphi2, wphi3, rhos, rhotriangle, &
                       rhotriangle1, rhoctriangle, rhoctriangle1, r(3), &
                       hmax

!     cl             Specific heat of the liquid
!     cs             Specific heat of the solid
!     ctriangle      Specific heat for 'triangle' part of the element
!     ctriangle1     Specific heat for sub-triangle part of the element
!     det            determinant for cumputing weigth factor of whole element
!     det1           determinant for cumputing weigth factor of a subelement
!     det2           determinant for cumputing weigth factor of a subelement
!     det3           determinant for cumputing weigth factor of a subelement
!     dh             partial lengths of edges
!     dphidxcut      dphidx matrix for intersected element
!     dxs            length of intersection
!     eps            Local accuracy
!     gradtemp       gradient of the temperature
!     h              lengths of edges
!     hmax           Estimate of maximum length of edges
!     i              Counting variable
!     ind1           Integer index array
!     j              Counting variable
!     k              Counting variable
!     kl             Diffusivity of the liquid
!     ks             Diffusivity of the solid
!     ktriangle      Diffusivity for 'triangle' part of the element
!     ktriangle1     Diffusivity for 'triangle1' part of the element
!     latentheat     latent heat
!     mloc           local copy of m modified for Newton-Cotes
!     negativet      node numbers of nodes with temperature less than Tm
!     nnegativet     number of nodes with temperature less than Tm
!     nrmgradt       2-norm of the gradient of temperature T
!     phi            array of length n * m, values of shape
!                    functions in integration points in the sequence:
!                    phii(xg^k) = phi (i,k)
!     phi2           phi matrix for quadratic integration rule
!     phicut         phi matrix for intersected element
!     phicut2        phi matrix for intersected element
!     phicut3        phi matrix for intersected element
!     phixs          phi values in intersection point xs
!     r              mean value of r coordinate in axi-symmetric case
!     rhoctriangle   rho cp for 'triangle' part of the element
!     rhoctriangle1  rho cp for 'triangle1' part of the element
!     rhol           Density of the liquid
!     rhos           Density of the solid
!     rhotriangle    rho for 'triangle' part of the element
!     rhotriangle1   rho for 'triangle1' part of the element
!     sum1           Help parameter to compute a sum
!     sum2           Help parameter to compute a sum
!     tm             Melting temperature
!     v              Help matrix to compute determinants
!     wphi1          Help variable for computing integrals
!     wphi2          Help variable for computing integrals
!     wphi3          Help variable for computing integrals
!     wtriangle      Weight factor for 'triangle' part of the element
!     wtriangle1     Weight factor for sub-triangle part of the element
!     wtriangle2     Weight factor for sub-triangle part of the element
!     wtriangle3     Weight factor for sub-triangle part of the element
!     xhat           Local numbering of nodes
!     xs             coordinates of the intersection points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision ercdet

!     ERCDET         Compute a 3x3 determinant
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'eldiscint2iso' )
      debug = .false.

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from eldiscint2iso'
         call prinrl ( temperature, n, 'temperature' )
         call prinrl1 ( x, ndim, n, 'x' )
  2      format ( a, 1x, (5d12.4) )

      end if

!     --- get parameters

      ks  = coefarr(1,1)
      kl  = coefarr(1,2)
      rhol = coefarr(1,3)
      if ( ind(9)==0 ) then
         rhos  = rhol
      else
         rhos  = coefarr(1,9)
      end if  ! ( ind(9)==0 )
      cs  = coefarr(1,4)
      cl  = coefarr(1,5)
      latentheat = coefarr(1,7)
      Tm  = coefarr(1,8)

!     --- Compute accuracy by multiplying sqreps with coordinate size

      hmax = max((x(2,1)-x(1,1))**2+(x(2,2)-x(1,2))**2, &
                 (x(3,1)-x(2,1))**2+(x(3,2)-x(2,2))**2 )
      eps = sqreps*sqrt(hmax)

      if (debug) then
         write(irefwr,2) 'ks, kl', ks, kl
         write(irefwr,2) 'rhol, rhos', rhol, rhos
         write(irefwr,2) 'cs, cl', cs, cl
         write(irefwr,2) 'latentheat, Tm', latentheat, Tm
         write(irefwr,2) 'eps, hmax', eps, hmax
      end if

!     --- Define temperatures<Tm as negative, otherwise as positive
!         For an intersected triangle1 it holds that the temperature value
!         of one of the three nodes has opposide sign w.r.t. the other two nodes
!         Determine this node, xhat

      nnegativet = 0
      do i = 1, mloc
         if (temperature(i)<Tm) then
            nnegativet = nnegativet + 1
            negativet(nnegativet) = i
         end if
      end do

      if (nnegativet==1) then
         xhat(1) = negativet(nnegativet)
      else
         xhat(1) = 6 - negativet(1) - negativet(2)
      end if

      xhat(2) = mod(xhat(1),mloc)+1
      xhat(3) = mod(xhat(1)+1,mloc)+1

!     --- compute lengths of intersected edges

      do i = 1, 2
         h(i) = sqrt((x(xhat(1),1) - x(xhat(i+1),1))**2 + &
                  (x(xhat(1),2) - x(xhat(i+1),2))**2  )
      end do

!     --- compute estimate intersection positions
!         the most basic way is to use linear interpolation

      do i = 1, 2
         dh(i) = h(i)*(Tm - temperature(xhat(1)))/ &
                      (temperature(xhat(i+1)) - temperature(xhat(1)))
      end do

      if (debug) then
         call prinin ( xhat, 3, 'xhat' )
         call prinrl ( h, 2, 'h' )
         call prinrl ( dh, 2, 'dh' )
      end if  ! (debug)

!     --- compute coordinates of intersection points and length of intersection

      do i = 1, 2
         do j = 1, 2
            if ( abs(x(xhat(1),j)-x(xhat(i+1),j))< eps ) then
               xs(i,j) = x(xhat(1),j)
            else if (x(xhat(1),j)<x(xhat(i+1),j)) then
               xs(i,j) = x(xhat(1),j) + &
                     abs(x(xhat(1),j)-x(xhat(i+1),j))*dh(i)/h(i)
            else
               xs(i,j) = x(xhat(1),j) - &
                     abs(x(xhat(1),j)-x(xhat(i+1),j))*dh(i)/h(i)
            end if
         end do
      end do

      dxs = sqrt((xs(1,1)-xs(2,1))**2 + (xs(1,2)-xs(2,2))**2)
      if ( jcart==2 ) then

!     --- Ad-hoc adaptation in case of axi-symmetric coordinates
!         The weight is multiplied by the average 2 pi r in the element

         dxs = dxs * pi*(xs(1,1)+xs(2,1))

      end if  ! ( jcart==1 )

      if (debug) then
         call prinrl1 ( xs, 2, 2, 'xs' )
         write(irefwr,2) 'dxs', dxs
      end if

!     --- compute det such that |det| = 2*surface whole element

      do i = 1, mloc
         v(1,i) = 1d0
         v(2,i) = x(i,1)
         v(3,i) = x(i,2)
      end do

      det = ercdet(v(1,1), v(1,2), v(1,3))
      if (debug) write(irefwr,2) 'det', det

!     --- compute det1 such that |det1| = 2*surface of subtriangle1

      do i = 1, mloc
         v(1,i) = 1d0
      end do
      do i = 1, 2
         v(i+1,1) = x(xhat(1),i)
         do j = 1, 2
            v(i+1,j+1) = xs(j,i)
         end do
      end do
      r(1:3) = v(2,1:3)

      det1 = ercdet(v(1,1), v(1,2), v(1,3))
      if (debug) write(irefwr,2) 'det1', det1

      if (intrule==3) then

         do i = 1, mloc
            v(1,i) = 1d0
         end do
         do i = 1, 2
            v(i+1,1) = x(xhat(2),i)
            do j = 1, 2
               v(i+1,j+1) = xs(j,i)
            end do
         end do

         det2 = ercdet(v(1,1), v(1,2), v(1,3))
         if (debug) write(irefwr,2) 'det2', det2

         do i = 1, mloc
            v(1,i) = 1d0
         end do
         do i = 1, 2
            v(i+1,1) = xs(2,i)
            do j = 1, 2
               v(i+1,j+1) = x(xhat(j+1),i)
            end do
         end do

         det3 = ercdet(v(1,1), v(1,2), v(1,3))
         if (debug) write(irefwr,2) 'det3', det3

!        --- compute weights

         wtriangle2 = abs(det2)/6d0
         wtriangle3 = abs(det3)/6d0

      end if ! (intrule==3)

!     --- compute weights

      wtriangle1 = abs(det1)/6d0
      wtriangle = abs(det)/6d0

      if ( jcart==2 ) then

!     --- Ad-hoc adaptation in case of axi-symmetric coordinates
!         The weight is multiplied by the average 2 pi r in the element

         wtriangle = wtriangle*2d0*pi*(x(1,1)+x(2,1)+x(3,1))/3d0
         wtriangle1 = wtriangle1*2d0*pi*(r(1)+r(2)+r(3))/3d0

      end if  ! ( jcart==1 )

      if (debug) &
          write(irefwr,2) 'wtriangle1, wtriangle', wtriangle1, wtriangle

!     --- set physical parameters for the subsections of the element

      if (temperature(xhat(1))<Tm) then

!     --- triangle1 is solid, triangle is liquid

         ctriangle1 = cs
         ctriangle  = cl
         ktriangle1 = ks
         ktriangle  = kl
         rhotriangle1 = rhos
         rhotriangle  = rhol

      else ! otherwise

!     --- triangle1 is liquid, triangle is solid

         ctriangle1 = cl
         ctriangle  = cs
         ktriangle1 = kl
         ktriangle  = ks
         rhotriangle1 = rhol
         rhotriangle  = rhos

      end if
      rhoctriangle = rhotriangle*ctriangle
      rhoctriangle1 = rhotriangle1*ctriangle1

!     --- construct phicut

      do i = 1, 2
         ind1(1,i) = dh(i)/h(i)
         ind1(2,i) = 0d0
         ind1(3,i) = 1d0 - dh(i)/h(i)
         ind1(4,i) = dh(i)/h(i)
         ind1(5,i) = 0d0
      end do

      if (debug) then
         call prinrl1 ( ind1, 2, 5, 'ind1' )
         write(irefwr,2) 'rhoctriangle, rhoctriangle1', &
                          rhoctriangle, rhoctriangle1
      end if

      do i = 1, 3
         phixs(i,1) = ind1(3-xhat(1)+i,1)
         phixs(i,2) = ind1(3+xhat(1)-i,2)

!        --- simpson: 1/3*(f(x1) + 4*f(x2) + f(x3)), so multiply by 2 and when
!            constructing the dldt matrix, one retrieves the factor 4

         phixs(i,3) = phixs(i,1) + phixs(i,2)
      end do

      phicut(xhat(1), 1) = phi(xhat(1),xhat(1))
      phicut(xhat(1), 2) = phixs(xhat(1),1)
      phicut(xhat(1), 3) = phixs(xhat(1),2)
      phicut(xhat(2), 1) = phi(xhat(2),xhat(1))
      phicut(xhat(2), 2) = phixs(xhat(2),1)
      phicut(xhat(2), 3) = phixs(xhat(2),2)
      phicut(xhat(3), 1) = phi(xhat(3),xhat(1))
      phicut(xhat(3), 2) = phixs(xhat(3),1)
      phicut(xhat(3), 3) = phixs(xhat(3),2)

      if (intrule==2) then ! NC, 2nd order

         phicut2(xhat(1), 1) = 0.5d0*(phicut(xhat(1), 1) + &
                                        phixs(xhat(1),1))
         phicut2(xhat(1), 2) = 0.5d0*(phicut(xhat(1), 1) + &
                                        phixs(xhat(1),2))
         phicut2(xhat(1), 3) = 0.5d0*(phixs(xhat(1), 1) + &
                                     phixs(xhat(1), 2))
         phicut2(xhat(2), 1) = 0.5d0*(phicut(xhat(2), 1) + &
                                        phixs(xhat(2),1))
         phicut2(xhat(2), 2) = 0.5d0*(phicut(xhat(2), 1) + &
                                        phixs(xhat(2),2))
         phicut2(xhat(2), 3) = 0.5d0*(phixs(xhat(3), 1) + &
                                     phixs(xhat(3), 2))
         phicut2(xhat(3), 1) = 0.5d0*(phicut(xhat(3), 1) + &
                                        phixs(xhat(3),1))
         phicut2(xhat(3), 2) = 0.5d0*(phicut(xhat(3), 1) + &
                                        phixs(xhat(3),2))
         phicut2(xhat(3), 3) = 0.5d0*(phixs(xhat(3), 1) + &
                                     phixs(xhat(3), 2))
         phi2(xhat(1), 1) = 0.5d0
         phi2(xhat(1), 2) = 0.5d0
         phi2(xhat(1), 3) = 0d0
         phi2(xhat(2), 1) = 0.5d0
         phi2(xhat(2), 2) = 0d0
         phi2(xhat(2), 3) = 0.5d0
         phi2(xhat(3), 1) = 0d0
         phi2(xhat(3), 2) = 0.5d0
         phi2(xhat(3), 3) = 0.5d0

      else if (intrule==3) then ! NC, 1st order, subtriangles

         phicut2(xhat(1), 1) = phi(xhat(1),xhat(2))
         phicut2(xhat(1), 2) = phixs(xhat(1),1)
         phicut2(xhat(1), 3) = phixs(xhat(1),2)
         phicut2(xhat(2), 1) = phi(xhat(2),xhat(2))
         phicut2(xhat(2), 2) = phixs(xhat(2),1)
         phicut2(xhat(2), 3) = phixs(xhat(2),2)
         phicut2(xhat(3), 1) = phi(xhat(3),xhat(3))
         phicut2(xhat(3), 2) = phixs(xhat(3),xhat(2))
         phicut2(xhat(3), 3) = phixs(xhat(3),2)
         phicut3(xhat(1), 1) = phi(xhat(1),xhat(3))
         phicut3(xhat(1), 2) = phi(xhat(1),xhat(2))
         phicut3(xhat(1), 3) = phixs(xhat(1),2)
         phicut3(xhat(2), 1) = phi(xhat(2),xhat(3))
         phicut3(xhat(2), 2) = phi(xhat(2),xhat(2))
         phicut3(xhat(2), 3) = phixs(xhat(2),2)
         phicut3(xhat(3), 1) = phi(xhat(3),xhat(3))
         phicut3(xhat(3), 2) = phixs(xhat(3),xhat(2))
         phicut3(xhat(3), 3) = phixs(xhat(3),2)

      end if ! (intrule==2)

!     --- construct dphidxcut, using that dphidx = constant per
!         basis function

      do i = 1, 3
         do j = 1, ndim
            dphidxcut(i, j) = dphidx(i, 1, j)
         end do
      end do

      if (debug) then
         call prinrl1 ( phicut, 3, 3, 'phicut' )
         call prinrl1 ( dphidx, ndim, n, 'dphidx' )
         call prinrl1 ( dphidxcut, ndim, n, 'dphidxcut' )
      end if

!     --- compute gradtemp

      do i = 1, ndim
         sum1 = 0d0
         do j = 1, mloc
            sum1 = sum1 + temperature(j)*dphidxcut(j,i)
         end do
         gradtemp(i) = sum1
      end do

      nrmgradt = sqrt(gradtemp(1)**2 + gradtemp(2)**2)

      if (debug) then
         call prinrl ( gradtemp, ndim, 'gradtemp' )
         write(irefwr,2) 'nrmgradt', nrmgradt
      end if

!     --- fill element mass matrix

      do i = 1, n            ! basis functions
         do j = 1, n         ! basis functions
            sum1 = 0d0
            sum2 = 0d0
            do k = 1, mloc      ! integration points

               if (intrule==1) then ! NC, 1st order, substraction

                  wphi1 = wtriangle1*phicut(i,k)*phicut(j,k)
                  sum1 = sum1 + &
                      rhoctriangle*(wtriangle*phi(i,k)*phi(j,k)-wphi1)
                  sum2 = sum2 + rhoctriangle1*wphi1

               else if ( intrule==2) then ! NC, 2nd order, substraction

                  wphi1 = wtriangle1*phicut2(i,k)*phicut2(j,k)
                  sum1 = sum1 + rhoctriangle* &
                         (wtriangle*phi2(i,k)*phi2(j,k)-wphi1)
                  sum2 = sum2 + rhoctriangle1*wphi1

               else if ( intrule==3) then ! NC, 1st order, subtriangles

                  wphi1 = wtriangle1*phicut(i,k)*phicut(j,k)
                  wphi2 = wtriangle2*phicut2(i,k)*phicut2(j,k)
                  wphi3 = wtriangle3*phicut3(i,k)*phicut3(j,k)
                  sum1 = sum1 + rhoctriangle*(wphi2 + wphi3)
                  sum2 = sum2 + rhoctriangle1*wphi1

               else if ( intrule==4) then ! NC, 1st order, averaging

                  sum1 = sum1 + &
                      (rhoctriangle1*wtriangle1/wtriangle + &
                      rhoctriangle*(wtriangle - wtriangle1)/wtriangle)* &
                      phi(i,k)*phi(j,k)

               end if
            end do
            massmt(i,j) = sum1 + sum2
         end do
      end do

!     --- fill element stiffness matrix

      do i = 1, n            ! basis functions
         do j = 1, n         ! basis functions
            sum1 = 0d0
            sum2 = 0d0
            do k = 1, mloc      ! integration points
               wphi1 = wtriangle1*(dphidxcut(i,1)*dphidxcut(j,1)+ &
                                   dphidxcut(i,2)*dphidxcut(j,2))
               sum1 = sum1 + &
                     ktriangle*(wtriangle*(dphidx(i,1,1)*dphidx(j,1,1)+ &
                                    dphidx(i,1,2)*dphidx(j,1,2))-wphi1)
               sum2 = sum2 + ktriangle1*wphi1
            end do
            stiffmt(i,j) = sum1 + sum2
         end do
      end do

!     --- fill element dldt matrix

      do i = 1, n
         do j = 1, n
            sum1 = 0d0
            if (intrule==2) then
               do k = 1, 3   ! quadratic
                  sum1 = sum1 + rhol*latentheat*(dxs/6d0)* &  ! quadratic
                                  phixs(i,k)*phixs(j,k)/nrmgradt
               end do
            else
               do k = 1, 2   ! linear
                  sum1 = sum1 + rhol*latentheat*(dxs/2d0)*  & ! linear
                                     phixs(i,k)*phixs(j,k)/nrmgradt
               end do
            end if
            dldtmt(i,j) = sum1
         end do
      end do

!     --- fill latent heat element vector
!         latentvc = 0 in solid, so restrict to liquid

      if (temperature(xhat(1))<Tm) then

!     --- trapezium is liquid

         if (debug) write(irefwr,*) 'trapezium is liquid'
         do i = 1, n     ! basis functions
            sum1 = 0d0
            do j = 1, mloc  ! integration points
               wphi1 = wtriangle1*phicut(i,j)
               sum1 = sum1 + &
                      rhol*latentheat*(wtriangle*phi(i,j)-wphi1)
            end do
            latentvc(i) = sum1
         end do

      else

!     --- triangle1 is liquid

         if (debug) write(irefwr,*) 'triangle is liquid'

         do i = 1, n     ! basis functions
            sum1 = 0d0
            do j = 1, mloc  ! integration points
               sum1 = sum1 + rhol*latentheat*wtriangle1*phicut(i,j)
            end do
            latentvc(i) = sum1
         end do

      end if


      call erclos ( 'eldiscint2iso' )
      if ( debug ) then

!     --- Debug information

         call prinrl1 ( massmt, n, n, 'massmt' )
         call prinrl1 ( stiffmt, n, n, 'stiffmt' )
         call prinrl1 ( dldtmt, n, n, 'dldtmt' )
         call prinrl ( latentvc, n, 'latentvc' )
         write(irefwr,*) 'End eldiscint2iso'

      end if

      end
