      subroutine elfillcoeff1 ( number, coeff, iuser, ifunc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 09-03-2013 Correction declaration
!        version  1.1  date 12-12-2012 Long integers
!        version  1.0  date 19-02-2010
!
!   copyright (c) 2010-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fills array coeff with a coefficient, where the coefficient is
!     equal to a solution or derivative of a preceding computation.
!     The value is only stored in all nodal points.
!     If the old solution is not available for all nodes, then it is
!     computed in the missing points by interpolation.
!     At this moment it is assumed that the old solution is at least
!     available in all vertex points
!     Number corresponds to the number th coefficient in common block celiar
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      use sepmoduleoldvecs
      use sepmodulecactl
      use sepmodulecomio
      use sepmoduletypes
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer number, iuser(*), ifunc
      double precision coeff(*)

!     coeff   o      Double precision output array of length n in which the
!                    values of the coefficient in the nodal points are
!                    stored
!     ifunc   o      Output parameter to be used if ind(number)>4000
!                    It indicates if the parameter which with the function
!                    is multiplied is a function (>0) or a constant (<0)
!     iuser   i      Standard array containing integer information of the
!                    coefficients
!     number  i      Parameter indicating which coefficient in common block
!                    celiar must be considered
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ivec, idegfd, i, ndegfd, interpol, j
      double precision sum
      double precision, pointer :: vecoldlc(:)

!     i              Counting variable
!     idegfd         Degree of freedom in VECTOR(IVEC) that must be used
!     interpol       If 0 the standard interpolation is applied, &
!                    if 1 the interpolation is restricted to linear subelements
!     ivec           Sequence number of vector in array VECOLD.
!                    Hence array vecoldlc(IVEC) is considered
!     j              Counting variable
!     ndegfd         Number of degrees of freedom per point in array
!                    vecoldlc(IVEC)
!     sum            Help variable to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inivec

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     INIVEC         Return with vector number corresponding to ivecin
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2049   unknown not present in all nodes in 3D not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elfillcoeff1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elfillcoeff1'
         write(irefwr,1) 'irule,number,inpelm', &
                          irule,number,inpelm
         write(irefwr,1) 'kstep, m', kstep, m
         write(irefwr,1) 'inpelm, n, jdiag', inpelm, n, jdiag
         write(irefwr,1) 'ind(number)', ind(number)
         write(irefwr,1) 'mst(number)', mst(number)
         write(irefwr,1) 'ich(number)', ich(number)
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      interpol = irule/100
      ipos  = ivecnums(number)
      ivec = inivec ( iuser(ipos) )
      iseqtype = oldsols(ivec)%iseqtype
      index3n => indoldsols(iseqtype)%index3
      index4n => indoldsols(iseqtype)%index4
      vecoldlc => oldsols(ivec)%oldvecs
      if ( ind(number)==2004 ) then

!     --- ind(number)=2004, special situation old solution must be
!                           multiplied by a coefficient
!                           The multiplication is carried out in elfillcoeff

         ifunc = iuser(ipos+2)
         if ( debug ) &
            write(irefwr,1) 'ipos, ivec, ifunc', ipos, ivec, ifunc

      end if
      idegfd = max(1,mst(number))

      if ( ich(number)==0 ) then

!     --- ich(number) = 0, constant number of unknowns per element

         sum = vecoldlc(index3n(idegfd))
         do i = 1, n
            coeff(i) = sum
         end do
         go to 1000

      else if ( ich(number)==1 ) then

!     --- in each point there is exactly one unknown

         do i = 1, inpelm
            coeff(i) = vecoldlc ( index3n(i) )
         end do

      else if ( ich(number)==2 ) then

!     --- in each point the number of unknowns is constant

         ndegfd = index4n(1)
         do i = 1, inpelm
            coeff(i) = vecoldlc ( index3n((i-1)*ndegfd+idegfd) )
         end do

      else if ( ich(number)==3 ) then

!     --- There is exactly one unknown in the vertices only

         if ( n==27 ) then

!        --- quadratic hexahedron

            do i = 1, 8
               coeff(ivertx14(i)) = vecoldlc ( index3n(i) )
            end do

         else

!        --- standard situation

            do i = 1, nvert*2, 2
               coeff(i) = vecoldlc ( index3n((i+1)/2) )
            end do

         end if

      else if ( ich(number)==4 ) then

!     --- in each vertex the number of unknowns is constant
!         Outside the vertices there are no unknowns

         ndegfd = index4n(1)
         do i = 1, nvert*2 ,2
            coeff(i) = vecoldlc ( index3n((i-1)*ndegfd+idegfd) )
         end do

      else if ( ich(number)==5 ) then

!     --- The number of unknowns is variable
!         The unknown is present in all nodes

         if ( debug ) then
            write(irefwr,1) 'idegfd', idegfd
            write(irefwr,1) 'index3n(idegfd)', index3n(idegfd)
         end if
         coeff(1) = vecoldlc ( index3n(idegfd) )
         do i = 2, inpelm
            coeff(i) = vecoldlc ( index3n(index4n(i-1)+idegfd) )
         end do
         if ( debug ) call prinrl ( coeff, inpelm, 'coeff' )

      else if ( ich(number)==7 ) then

!     --- The number of unknowns is variable
!         The unknown is present in all nodes except centroid

         coeff(1) = vecoldlc ( index3n(idegfd) )
         do i = 2, inpelm-1
            coeff(i) = vecoldlc ( index3n(index4n(i-1)+idegfd) )
         end do  ! i = 1, inpelm

         if ( inpelm==7 ) then

!        --- inpelm = 7

            coeff(7) = (4d0*(coeff(2)+coeff(4)+coeff(6))- &
                       coeff(1)-coeff(3)-coeff(5))/9d0

         else

!        --- inpelm # 7

            sum = coeff(1)
            do i = 2, inpelm-1
               sum = sum + coeff(i)
            end do
            coeff(inpelm) = sum/inpelm

         end if  ! ( inpelm==7 )

      else if ( ich(number)==8 ) then

!     --- The number of unknowns is variable
!         The unknown is present in all nodes except centroids
!         of tetrahedrons
!         First fill vertices and mid-points

         do i = 1, 10
            coeff(i) = vecoldlc ( index3n(index4n(i-1)+idegfd) )
         end do  ! i = 1, inpelm

!        --- Next fill the 4 centroids of the faces

         coeff(11) = (4d0*(coeff(2)+coeff(4)+coeff(6))- &
                     coeff(1)-coeff(3)-coeff(5))/9d0
         coeff(12) = (4d0*(coeff(2)+coeff(8)+coeff(7))- &
                     coeff(1)-coeff(3)-coeff(10))/9d0
         coeff(13) = (4d0*(coeff(4)+coeff(8)+coeff(9))- &
                     coeff(3)-coeff(5)-coeff(10))/9d0
         coeff(14) = (4d0*(coeff(6)+coeff(7)+coeff(9))- &
                     coeff(1)-coeff(5)-coeff(10))/9d0

         if ( inpelm==15 ) then

!        --- inpelm = 15, centroid of element

            coeff(15) = 0.25d0*(coeff(11)+coeff(12)+coeff(13)+coeff(14))

         end if  ! ( inpelm==15 )

      else if ( ich(number)>2000 ) then

!     --- Special situation of vector of type 129 with ich(number)-2000
!         degrees of freedom per element

         sum = vecoldlc ( index3n(1) )
         do i = 1, n
            coeff(i) = sum
         end do

      else if ( ich(number)>1000 ) then

!     --- Special situation of vector of type 126 with ich(number)-1000
!         degrees of freedom per element

         ndegfd = ich(number)-1000
         do j = 1, ndegfd
            do i = 1, n
               coeff(i+(j-1)*n) = &
                  vecoldlc ( index3n(j+(i-1)*ndegfd) )
            end do
         end do

      else if ( ich(number)>100 ) then

!     --- Special situation of vector of type 116 with ich(number)-100
!         degrees of freedom per element

         do j = 1, ich(number)-100
            sum = vecoldlc ( index3n(j) )
            do i = 1, n
               coeff(i+(j-1)*n) = sum
            end do
         end do

      else

!     --- The number of unknowns is variable
!         The unknown is present in all vertices, but not in all nodes

         coeff(1) = vecoldlc ( index3n(idegfd) )

         if ( inpelm==3 ) then

!        --- inpelm = 3:  quadratic one-dimensional elements

            coeff(3)=vecoldlc( index3n(index4n(4)+idegfd) )

         else if ( inpelm==6 .or. inpelm==7 .or. inpelm==9 ) &
            then

!        --- inpelm = 6, 7 or 9:  quadratic two-dimensional elements

            do i = 3, inpelm-1, 2
               coeff(i) = vecoldlc ( index3n( index4n(2*i-2) &
                                                 +idegfd))
            end do

         else if ( inpelm==27 ) then

!        --- inpelm = 27:  quadratic hexahedron

            do i = 1, 8
               coeff(i) = vecoldlc ( index3n(index4n(i)+idegfd) )
            end do

         else

!        --- This possibility has not yet been implemented in ELFLR8

            call errsub ( 2049, 0, 0, 0 )
            go to 1000

         end if

      end if

      if ( ich(number)==3 .or. ich(number)==4 .or. &
           ich(number)==6 ) then

!     --- The coefficients have only been computed in the vertices
!         Interpolate to the other nodes

         if ( n==3 ) then

!        --- n = 3:  quadratic one-dimensional elements

            coeff(2) = 0.5d0*( coeff(1) + coeff(3) )

         else if ( n==6 .or. n==7 .or. n==9 ) then

!        --- n = 6, 7 or 9:  quadratic two-dimensional elements

            coeff(2*nvert+1) = coeff(1)
            do i = 1, nvert
               coeff(2*i) = 0.5d0 * ( coeff(2*i-1) + coeff(2*i+1) )
            end do

            if ( n==7 ) then

!           --- n = 7, fill 7 th point

               coeff(n) = ( coeff(1) + coeff(3) + coeff(5) ) / 3d0

            else if ( n==9 ) then

!           --- n = 9, fill 9 th point

               coeff(n) = ( coeff(1) + coeff(3) + coeff(5) + coeff(7) ) &
                            * .25d0

            end if

         else if ( n==27 ) then

!        --- n = 27, fill midside points and centroids

            coeff(10) = 0.5d0*(coeff(1)+coeff(19))
            coeff(12) = 0.5d0*(coeff(3)+coeff(21))
            coeff(16) = 0.5d0*(coeff(7)+coeff(25))
            coeff(18) = 0.5d0*(coeff(9)+coeff(27))
            do i = 0, 18, 9
               coeff(2+i) = 0.5d0*(coeff(1+i)+coeff(3+i))
               coeff(4+i) = 0.5d0*(coeff(1+i)+coeff(7+i))
               coeff(6+i) = 0.5d0*(coeff(3+i)+coeff(9+i))
               coeff(8+i) = 0.5d0*(coeff(7+i)+coeff(9+i))
               coeff(5+i) = 0.25d0*(coeff(1+i)+coeff(3+i)+ &
                                   coeff(7+i)+coeff(9+i))
               coeff(2+i) = 0.5d0*(coeff(1+i)+coeff(3+i))
               coeff(4+i) = 0.5d0*(coeff(1+i)+coeff(7+i))
               coeff(6+i) = 0.5d0*(coeff(3+i)+coeff(9+i))
               coeff(8+i) = 0.5d0*(coeff(7+i)+coeff(9+i))
               coeff(5+i) = 0.25d0*(coeff(1+i)+coeff(3+i)+ &
                                   coeff(7+i)+coeff(9+i))
            end do

         else

!        --- This possibility has not yet been implemented in ELFLR8

            call errsub ( 2049, 0, 0, 0 )
            go to 1000

         end if

      end if
      if ( debug ) call prinrl ( coeff, n, 'coeff' )

      if ( inpelm<n .and. ich(number)<100 ) then

!     --- inpelm<n, special situation
!                     At this moment only 6 point triangles with dummy
!                     centroid are allowed

         if ( n==7 ) then
            if ( interpol==0 ) then
               coeff(n) = ( -coeff(1)-coeff(3)-coeff(5) + &
                         4d0 * ( coeff(2)+coeff(4)+coeff(6) ) ) / 9d0
            else
               coeff(n) = ( coeff(2)+coeff(4)+coeff(6) ) /3d0
            end if
         else if ( n==4 ) then
            coeff(n) = ( coeff(1)+coeff(2)+coeff(3) ) /3d0
         else if ( n==5 ) then
            coeff(n) = ( coeff(1)+coeff(2)+coeff(3)+coeff(4) ) * 0.25d0
         end if

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elfillcoeff1'

      end if  ! ( debug )
      call erclos ( 'elfillcoeff1' )

      end
