      function eli325 ( coor2d, index1 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.1  date 12-12-2012 Long integers
!        version  1.0  date 20-12-2010
!
!   copyright (c) 2010-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute some special integrals in case of type number 325
!
! **********************************************************************
!
!                       KEYWORDS
!
!     bearing
!     element_integral
!     integral
!     compressible
!     incompressible
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision eli325, coor2d(*)
      integer index1(*)

!     coor2d           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     eli325         o    Computed integral
!     index1         i    Array of length inpelm containing the point numbers
!                         of the nodal points in the element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision x(27*3), w(27), dphidx(3*27*27), xgauss(81), &
                       phi(27*27), qmat(3,27), tang(3*9), trans(1), &
                       psi(8*27), dpsidx(3*8*27), dudx(1), uold(27), &
                       psiksi(8*27), ugauss(27), diam
      integer jelp
      save jelp, diam

!     diam           Diameter of bearing
!     dphidx         Array of length n x m x ndim  containing the
!                    derivatives
!                    of the basis functions in the sequence:
!                         d phi_i/ dx_j(xg_k) = dphidx (i,k,j);
!                    If the element is a linear simplex, the
!                    derivatives are constant and only k = 1 is filled.
!     dpsidx         Array of length n x m x l  containing the derivatives
!                    of the linear basis functions in the sequence:
!                    d psi / dx (xg ) = psix (i,k,1);
!                         i        k
!                    d psi / dy (xg ) = psix (i,k,2);
!                         i        k
!                    d psi / dz (xg ) = psix (i,k,3);
!                         i        k
!     dudx           Real m x ndim  array in which the values of the
!                    derivatives of the solution vector in the integration
!                    points are stored in the sequence:
!                    du / dx  (x ) = dudx ( k, j )
!                      j     k
!                    If jdercn = 0 in common block celint, then only
!                    dudx (1,*) is filled because dudx is then a constant
!                    per element
!                    dudx is only filled if jderiv>0
!     jelp           Indication of the type of basis function subroutine
!                    must be called by ELxxxx.
!     phi            array of length n * m, values of shape
!                    functions in integration points in the sequence:
!                    phi_i(xg^k) = phi (i,k)
!                    array phi is only filled when gauss integration is
!                    applied
!     psi            Array of length n x m containing the values of the
!                    modified basis functions in the integration points
!     psiksi         Array of length 8 x m x 3 containing the values of the
!                    derivatives of the basis functions in the Gauss points in
!                    the reference element
!     qmat           transformation matrix global -> local coordinates
!                    in case of a 3D element
!     tang           In this array the tangential vector is stored
!                    tang(j,i) contains the j th component of the
!                    tangential vector in the i-th integration point
!     trans          dummy
!     ugauss         Preceding solution in Gauss points
!     uold           Array containing the previous iteration (old solution)
!     w              array of length m containing the weights for
!                    integration
!     x              x co-ordinate of point
!     xgauss         array of length m x ndim containing the co-ordinates
!                    of the gauss points.  array xgauss is only filled when
!                    gauss integration is applied.
!                    xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                    zg_i = xgauss (i,3)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision eli325int, getconst

!     EL3007         Fills old solution and derivatives
!     ELCOMM         Initializes the commons celp and celint with standard
!                    values
!     ELI325INT      Compute actual special integral for type number 325
!     ELI325NM       Fill the parameters jelp, n, m, nm and jdercn for
!                    bearing element
!     ELM800BASEFN   Compute the basis functions phi, its derivatives and other
!                    related quantities for various element shapes
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERREAL         Put real in error message
!     ERRSUB         Error messages
!     GETCONST       Get the value of a constant in the input block constants
! **********************************************************************
!
!                       I/O
!
!     Depending on icheli the following integrals are computed
!
!     40:  f_h = -/ p sin(2x/Diam)
!     41:  f_h = -/ p cos(2x/Diam)
!     42:  f_h = -/ p sin(2x/Diam) (p>=0)
!     43:  f_h = -/ p cos(2x/Diam) (p>=0)
!     44:  f_h = / x p
!     45:  f_h = / y p
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1379   Parameter has wrong value
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'eli325' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from eli325'
         write(irefwr,1) 'icheli', icheli
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      if ( ifirst==0 )  then

!     --- ifirst = 0,  compute element independent quantities

         call elcomm  ! standard choice of parameters
         ideriv = 0   ! do not compute derivatives
         call eli325nm ( jelp )  ! set parameters depending on ishape

         if ( icheli>=40 .and. icheli<=43 ) then

!        --- 40<=icheli<=43, get diam

            diam = getconst('diam')
            if ( diam<=0d0 .or. diam>=1d0 ) then

!           --- diameter has wrong value

               call erreal ( diam, 1 )
               call erreal ( 0d0, 2 )
               call erreal ( 1d0, 3 )
               call errchr ( 'diam', 1 )
               call errsub ( 1379, 0, 3, 1 )
               go to 1000

            end if  ! ( diam<=0d0 .or. diam>=1d0 )

         else

!        --- Other types: diam not required

            diam = 0d0

         end if  ! ( icheli>-40 .and. icheli<=43 )

      end if  ! ( ifirst==0 )  then
      if ( ierror/=0 ) go to 1000

      if ( debug ) write(irefwr,2) 'diam', diam

!     --- compute basis functions and weights for numerical integration

      call elm800basefn ( coor2d, x, w, dphidx, xgauss, phi, &
                          index1, qmat, tang, jelp, dphidx, psi, &
                          dpsidx, psiksi )

      if ( ierror/=0 ) go to 1000

!     --- fill uold

      jtrans = 0
      call el3007 ( uold, dudx, phi, &
                    dphidx, m, n, ndimlc, ncomp, &
                    index1, trans, ugauss, jtrans, numold, iseqin )

!     --- Compute special integral (icheli>=40)

      eli325 = eli325int ( ugauss, w, x, diam )

1000  call erclos ( 'eli325' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End eli325'

      end if  ! ( debug )

      end
