      subroutine elm606 ( swork )
! ======================================================================
!
!        programmer    Luc Timmermans
!        version  3.4  date 09-03-2013 Replace elflr7 by elflr71
!        version  3.3  date 12-12-2012 Long integers
!        version  3.2  date 18-09-2002 New call to el3007
!
!   copyright (c) 1992-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill spectral element matrix and vector for second order elliptic
!     equations or compute residual in the case of finite element
!     preconditioning
!     two unknowns per nodal point
!     two-dimensional quadrilateral elements
!     ELM606 is a help subroutine for subroutine BUILD
!     it is called through the intermediate subroutine EL0000
!     so:
!     BUILD
!       -  loop over spectral elements
!          -  EL0000
!             -  ELM606
!     itype = 606  :   two-dimensional spectral element
!                      two unknowns per nodal point
! **********************************************************************
!
!                       KEYWORDS
!
!     spectral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulebuild
      use sepmodulecactl
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer islold(numold)
      double precision swork(*)

!     swork          spectral element work array
!                    the work array contains several quantities of the high
!                    order spectral element discretization, the first position
!                    of each quantity is pointed at by an integer pointer
!                    total length:  m*m + 24*m + 4*mone
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer isub, isup, jelp, kfirst, jrhs1, jrhs2, method, i, &
              iparry, ipxre, ipx, ipw, ipd1, &
              ipd2, ipdet, ip1, ip2, ip3, ip4, ipwork, ipold, &
              ipmatr, ipmatt, ipmatp,ipvec1, ipvec2,crstrm
      double precision rdum
      save iparry, ipxre, ipx, ipw, ipd1, &
           ipd2, ipdet, ip1, ip2, ip3, ip4, ipwork, ipold, &
           ipmatr, ipmatt, ipmatp,ipvec1, ipvec2,crstrm, &
           isub, isup, jelp, kfirst, jrhs1, jrhs2, method

!     crstrm
!     i         Counting variable
!     ip1       indicate partial derivatives of the coordinates due to
!               the isoparametric transformation, length m
!     ip2       indicate partial derivatives of the coordinates due to
!               the isoparametric transformation, length m
!     ip3       indicate partial derivatives of the coordinates due to
!               the isoparametric transformation, length m
!     ip4       indicate partial derivatives of the coordinates due to
!               the isoparametric transformation, length m
!     iparry    indicates array in which the coefficients are stored
!               length 7 x m
!     ipd1      indicates the values of derivatives of the basisfunctions
!               in the interpolation points in the x-direction correspon- &
!               ding to the reference element, length mone x mone
!     ipd2      indicates the values of 2nd derivatives of the basisfunctions
!               in the interpolation points in the x-direction correspon- &
!               ding to the reference element, length mone x mone
!     ipdet     transformation determinant, length m
!     ipmatp
!     ipmatr    temporary work array to store part of the matrix, &
!               length m x m
!     ipmatt    temporary work array to store part of the matrix, &
!               length m x m
!     ipold     indicates the old solution in the nodal points, length m
!     ipvec1    temporary work array to store part of the vector, &
!               length m
!     ipvec2    temporary work array to store part of the vector, &
!               length m
!     ipw       indicates the Legendre Gauss-Lobatto weights for
!               numerical integration in the x-direction corresponding
!               to the reference element, length mone
!     ipwork    work array, length m
!     ipx       indicates the coordinates of the corresponding points
!               in the element, length m x 2
!     ipxre     indicates the Legendre Gauss-Lobatto points in the
!               x-direction corresponding to the reference element [-1,1], &
!               length mone
!     isub     integer help parameter for do-loops to indicate the smallest
!              coefficient number that is dependent on space
!     isup     integer help parameter for do-loops to indicate the largest
!              coefficient number that is dependent on space
!     jelp     choice parameter to distinguish between the types of element
!              subroutines called, possible values:
!              1: subroutine elp640 (one-dimensional)
!              (more values not yet done)
!     jrhs1    information of the weight function w and the function to
!              be integrated f1, possibilities:
!              0   f1 = 0
!              1   f1 constant, w variable
!              2   general
!     jrhs2    same as jrhs1 for f2
!     kfirst   this parameter indicates the smallest velocity component
!              that is not equal to zero in the convective terms, for
!              example kfirst = 2, means that the y-component of
!              the velocity is unequal to zero
!     method   choice parameter, see USER INPUT
!     rdum     dummy variable
!              (more values not yet done)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL0606  calls the following subroutines:
!     EL3007  Fills old solution and derivatives
!     EL6021  compute the spectral element convection matrix
!     EL6022  compute the spectral element Helmholtz matrix
!     EL6023  compute the spectral element righthand side term
!     EL6024  calls the following subroutines:
!     EL6025  compose full spectral element matrix
!     EL6026  compose full spectral element vector
!     EL6027  compute the spectral element mass matrix (diagonal)
!     EL6029  compute the spectral element Crosstem matrix
!     EL6030  compose full spectral element matrix
!     EL6031  compute the spectral element convection residual
!     EL6032  compute the spectral element Helmholtz residual
!     EL6033  compute the spectral element righthand side residual
!     EL6034  compute contributions to spectral element residue vector
!             corresponding to the diffusive part of the equation in the
!             case of finite element preconditioning
!     EL6035  compute contributions to spectral element stiffness matrix
!             corresponding to the TRANSPOSE diffusive part of the equation
!     ELFLR71 Fill variable coefficients in array ARRAY
!     ELIND0  Fill array ICH in COMMON CELIAR for preceding solutions
!     ELP640  compute basisfunctions, derivatives and weights for
!             numerical integration corresponding to the reference
!             element (element independent)
!     ELP642  compute the integration points, determinant and
!             partial derivatives, two-dimensional case
!             (element dependent)
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
!     INSTOP  Stop the program
! **********************************************************************
!
!                       I/O
!
!     USER INPUT
!     the user provides the coefficients through the arrays E%iuser and
!     USER in the usual way, the sequence is:
!     1:  alpha11
!     2:  alpha12
!     3:  alpha22
!     4:  u1
!     5:  u2
!     6:  beta
!     7:  gamma
!     8:  f1
!     9:  f2
!    10:  method (0: no finite el. preconditioning, fill matrix and vector, &
!                 1: finite el. preconditioning, compute spectral residual)
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1781   The possibility ifunc>10000 not yet available
! **********************************************************************
!
!                       PSEUDO CODE
!
!     differential equation:
!         - div alpha grad c + u . grad c + beta c + gamma L u = f
!                          -   -        -        -           -   - &
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen('elm606')
      if (ifirst==0) then

!     --- ifirst = 0, compute element independent quantities
!         perform initialisations, set iparry

         iparry = 1
         call el0606(E%iuser, E%user, kfirst, jelp, isub, isup, &
                        swork(iparry), 0,crstrm, jrhs1, jrhs2, method)

!        --- set pointers in the workarray

         ipxre  = iparry + 9*m
         ipx    = ipxre + mone
         ipw    = ipx + 2*m
         ipd1   = ipw + mone
         ipd2   = ipd1  + m
         ipdet  = ipd2 + m
         ip1    = ipdet + m
         ip2    = ip1 + m
         ip3    = ip2 + m
         ip4    = ip3 + m
         ipwork = ip4 + m
         ipold  = ipwork + m
         ipmatr = ipold + 2*m
         ipmatt = ipmatr + m*m
         ipmatp = ipmatt + m*m
         ipvec1 = ipmatp + m*m
         ipvec2 = ipvec1 + m

!        --- fill array ich (common celiar) for preceeding solutions

         call elind0( islold, numold, isub, isup, E%iuser)

!        --- jelp = 1 and kstep = 1
!            compute points and weights for numerical integration
!            compute derivatives of the basisfunctions
!            corresponding to the reference element

         call elp640(1, swork(ipxre), swork(ipw), swork(ipd1), &
                     swork(ipd2), mone)
      end if
      if (ierror/=0) go to 1000

!     --- compute integration points, determinant and derivatives

      call elp642(coor, index1, swork(ipx), swork(ipd1), &
                  swork(ipdet), swork(ip1), swork(ip2), swork(ip3), &
                  swork(ip4), mone, m)

!     --- fill variable coefficients if necessary
!         first fill old solution if necessary

      if (ind(lengthmst) ==1 ) then
         call el3007 ( swork(ipold), rdum, rdum, rdum, m, m, &
                       ndimlc, ncomp, index1, rdum, &
                       swork(ipold), jtrans, numold, 1 )
      end if
      do 100 i = isub, isup
         if ((ind(i)>0) .or. (ist(i)>0)) then

!        --- coefficient is variable

            if (ind(i)>10000) then
               call errint ( i, 1 )
               call errsub ( 1781, 1, 0, 0)
            end if
            call elflr71(i, E%user, swork(iparry), &
                        swork(ipx), swork(ipx), m, m, rdum, &
                         1, index1, E%iuser, swork(ipold), swork )
         end if
100   continue
      if ( method==0 ) then

!     --- no fe-preconditioning, build system matrices and vector
!         fill parts of matrix if necessary

         if (matrix) then

!        --- matrix = true, compute spectral element matrix

            jadd = 0

!           --- first second derivative part (diffusive terms)

            if ( jsecnd>0 ) then
               call el6024(swork(iparry), swork(ipmatr), swork(ipw), &
                           swork(ipw), swork(ipd1), swork(ipd1), &
                           swork(ipdet), swork(ip1), swork(ip2), &
                           swork(ip3), swork(ip4), swork(ipwork), &
                           mone, m)
               jadd = 1
            end if
            if ( crstrm>0 ) then
               call el6029(swork(iparry+6*m), swork(ipmatt), swork(ipw), &
                           swork(ipw), swork(ipd1), swork(ipd1), &
                           swork(ipdet), swork(ip1), swork(ip2), &
                           swork(ip3), swork(ip4), swork(ipwork), &
                           mone, m)
               call el6035(swork(iparry), swork(ipmatp), swork(ipw), &
                           swork(ipw), swork(ipd1), swork(ipd1), &
                           swork(ipdet), swork(ip1), swork(ip2), &
                           swork(ip3), swork(ip4), swork(ipwork), &
                           mone, m)
            end if

!           --- then first derivative part (convective terms)

            if (idim>0) then
               call el6021(swork(iparry+(2+kfirst)*m), swork(ipmatr), &
                           idim, kfirst, swork(ipw), swork(ipw), &
                           swork(ipd1), swork(ipd1), swork(ip1), &
                           swork(ip2), swork(ip3), swork(ip4), &
                           swork(ipwork), mone, m)
               jadd = 1
            end if

!           --- finally the constant part (Helmholtz terms)

            if (jzero>0) then
               call el6022(swork(iparry+5*m), swork(ipmatr), &
                           swork(ipw), swork(ipw), swork(ipdet), &
                           mone, m)
            end if

!           --- compose full spectral element matrix

                if (crstrm/=0) then
                   call el6030(E%mat, swork(ipmatr), swork(ipmatt), &
                               swork(ipmatp), m)
                else
                   call el6025(E%mat, swork(ipmatr), m)
                end if
         end if

!        --- fill spectral element mass matrix if necessary

         if (notmas==0) then

!        --- notmas = 0, compute diagonal mass matrix

            call el6027(swork(ipvec1), swork(ipw), swork(ipdet), &
                        mone, m)
            call el6026(E%mmat, swork(ipvec1), swork(ipvec1), m)
         end if

!        --- fill spectral element vector if necessary

         if (vector) then

!        --- vector = true, compute element vector

            jadd = 0
            if (jrhs1>=0) then
               call el6023(swork(iparry+7*m), swork(ipvec1), &
                           swork(ipw), swork(ipw), swork(ipdet), &
                           mone, m)
               jadd = 1
            end if
            if (jrhs2>=0) then
               call el6023(swork(iparry+8*m), swork(ipvec2), &
                           swork(ipw), swork(ipw), swork(ipdet), &
                           mone, m)
               jadd = 1
            end if

!           --- compose full spectral element vector

            call el6026(E%vec, swork(ipvec1), swork(ipvec2), m)
         end if
      else

!     --- fe-preconditioning, build residual vector
!         get old solution

         call el3007 ( swork(ipold), rdum, rdum, rdum, m, m, &
                       ndimlc, ncomp, index1, rdum, &
                       swork(ipold), jtrans, numold, 1 )

!        --- fill parts of vector if necessary

         if (vector) then

!        --- vector = true, compute element vector

            jadd = 0

!           --- first second derivative part (diffusive terms)

            if (jsecnd>0) then
               call el6034(swork(iparry), swork(ipvec1), swork(ipold), &
                           swork(ipw), swork(ipd1), &
                           swork(ipdet), swork(ip1), &
                           swork(ip2), swork(ip3), swork(ip4), &
                           swork(ipwork), mone )
               call el6034(swork(iparry), swork(ipvec2), swork(ipold+m), &
                           swork(ipw), swork(ipd1), &
                           swork(ipdet), swork(ip1), &
                           swork(ip2), swork(ip3), swork(ip4), &
                           swork(ipwork), mone )
               jadd = 1
            end if

!           --- then first derivative part (convective terms)

            if (idim>0) then
               call el6031(swork(iparry+(2+kfirst)*m), swork(ipvec1), &
                        swork(ipold), idim, kfirst, swork(ipw), &
                        swork(ipw), swork(ipd1), swork(ipd1), &
                        swork(ip1), swork(ip2), swork(ip3), &
                        swork(ip4), swork(ipwork), mone, m)
               call el6031(swork(iparry+(2+kfirst)*m), swork(ipvec2), &
                        swork(ipold+m), idim, kfirst, swork(ipw), &
                        swork(ipw), swork(ipd1), swork(ipd1), &
                        swork(ip1), swork(ip2), swork(ip3), &
                        swork(ip4), swork(ipwork), mone, m)
               jadd = 1
            end if

!           --- next the constant part (Helmholtz terms)

            if ( jzero>0 ) then
               call el6032(swork(iparry+5*m), swork(ipvec1), &
                           swork(ipold), swork(ipw), swork(ipw), &
                           swork(ipdet), mone, m)
               call el6032(swork(iparry+5*m), swork(ipvec2), &
                           swork(ipold+m), swork(ipw), swork(ipw), &
                           swork(ipdet), mone, m)
            end if

!           --- finally right-hand side

            if ( jrhs1>=0 ) then
               call el6033(swork(iparry+7*m), swork(ipvec1), &
                           swork(ipw), swork(ipdet), mone, m)
               jadd = 1
            end if
            if ( jrhs2>=0 ) then
               call el6033(swork(iparry+8*m), swork(ipvec2), &
                           swork(ipw), swork(ipdet), mone, m)
               jadd = 1
            end if

!           --- compose full spectral element vector

            call el6026(E%vec, swork(ipvec1), swork(ipvec2), m)
         end if
      end if
1000  if (ierror/=0) call instop
      call erclos('elm606')
      end
