      subroutine elm607 ( swork )
! ======================================================================
!
!        programmer    Peter Minev
!        version  4.0  date 06-01-2016 Use sepmoduleelem
!        version  3.3  date 09-03-2013 Replace elflr7 by elflr71
!        version  3.2  date 12-12-2012 Long integers
!
!   copyright (c) 1994-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fills the diffusion matrix and vector for second order elliptic
!     equations or computes residual in the case of finite element
!     preconditioning -i 1-D, 2-D case and 3-D case
!     ELM607 is a help subroutine for subroutine BUILD
!     it is called through the intermediate subroutine EL0000
!     so:
!     BUILD
!       -  loop over spectral elements
!          -  EL0000
!             -  ELM607
!     itype = 607  : 1-D, 2-D and 3-D spectral element
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elliptic_equation
!     parabolic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulebuild
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision swork(*)

!     swork          spectral element work array
!                    total length:  (14 + ndim) * m + (2 + mone)* mone
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision rdum(1)
      integer iparry,ipx1re,ipx,ipw1,ipd1, method, &
              ipdet, ip1, ip2, ip3, ip4, ip5, ip6, ip7, ip8, ip9, &
              ipwork, ipold
      save iparry,ipx1re,ipx,ipw1,ipd1, method, &
           ipdet, ip1, ip2, ip3, ip4, ip5, ip6, ip7, ip8, ip9, &
           ipwork, ipold

!     ip1        indicate partial derivatives of the coordinates due to
!                the isoparametric transformation, length m
!     ip2        See ip1
!     ip3        See ip1
!     ip4        See ip1
!     ip5        See ip1
!     ip6        See ip1
!     ip7        See ip1
!     ip8        See ip1
!     ip9        See ip1
!     iparry     indicates array in which the coefficients are stored
!                length 2 x m
!     ipd1       indicates the values of derivatives of the basisfunctions
!                in the interpolation points in the x-direction correspon- &
!                ding to the reference element, length mone x mone
!     ipdet      transformation determinant, length m
!     ipold      indicates the old solution in the nodal points, length m
!     ipw1       indicates the Legendre Gauss-Lobatto weights for
!                numerical integration in the x-direction corresponding
!                to the reference element, length mone
!     ipwork     work array, length m
!     ipx        indicates the coordinates of the corresponding points
!                in the element, length m x ndim
!     ipx1re     indicates the Legendre Gauss-Lobatto points in the
!                x-direction corresponding to the reference element [-1,1], &
!                length mone
!     method     choice parameter, see USER INPUT
!     rdum       dummy variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL0607    initialize parameters for subroutine ELM607
!     EL3007    Fills old solution and derivatives
!     EL6015    compute contributions to spectral element residue vector
!               corresponding to the mass matrix
!     EL6016    compute contributions to spectral element stiffness matrix
!               corresponding to the diffusive part of the equation
!     EL6017    compute the spectral element diagonal mass matrix
!     EL6018    compute contributions to spectral element residue vector
!               corresponding to the diffusive part of the equation in the
!               case of finite element preconditioning
!     EL6027    compute the spectral element mass matrix (diagonal)
!     EL6034    compute contributions to spectral element residue vector
!               corresponding to the diffusive part of the equation in the
!               case of finite element preconditioning
!     EL6038    compute the spectral element mass matrix residual - 2-D case
!     EL6044    compute contributions to spectral element stiffness matrix
!               corresponding to the diffusive part of the equation
!     EL6084    compute contributions to spectral element stiffness matrix
!               corresponding to the diffusive part of the equation - 3-D case
!     EL6085    compute contributions to spectral element stiffness residual
!               in the case of matrix-free iterative solver - 3-D case
!     EL6086    compute the spectral element mass residual; 3-D case
!     EL6087    compute the spectral element diagonal mass matrix; 3-D case
!     ELFLR71   Fill variable coefficients in array ARRAY
!     ELP640    compute basisfunctions, derivatives and weights for
!               numerical integration corresponding to the reference
!               element (element independent)
!     ELP642    compute the integration points, determinant and
!               partial derivatives, two-dimensional case
!               (element dependent)
!     ELP646    compute transformation determinant and
!               partial derivatives  in 3-D
!     ELP647    store integration points
!               transform weights and derivatives from the reference element
!     ELSWAP    switch parts of matrix in the case of linear elements
!     ELVSWAP   swap parts of element vector in the the case of a linear
!               spectral element due to different nodal numbering
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
! **********************************************************************
!
!                       I/O
!
!     the user provides the coefficients through the arrays E%iuser and
!     USER in the usual way, the sequence is:
!     1:  nu  viscosity (assumed to be constant at the moment)
!     2:  f source term
!     3:  method (0: no finite el. preconditioning, fill matrix and vector, &
!                 1: finite el. preconditioning, compute stiffness residual
!                 2: finite el. preconditioning, compute stiffness residual
!                    and add to it the mass residual)
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      62   dimension of space (ndim) has wrong value
!    1621   method=0 and vector=0 not allowed
!    1781   The possibility ifunc>10000 not yet available
! **********************************************************************
!
!                       PSEUDO CODE
!
!                 METHOD
!     differential equation:
!         - div nu grad c + c = f
!                              - &
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen('elm607')

      if (ifirst==0) then

!     --- ifirst = 0, compute element independent quantities
!         perform initialisations, set iparry

         iparry = 1
         call el0607 ( E%iuser, E%user, swork(iparry), method )

!        --- set pointers in the workarray

         ipx1re = iparry + m
         ipx    = ipx1re + mone
         ipw1   = ipx + m*ndim
         ipd1   = ipw1 + mone
         ipdet  = ipd1 + mone*mone
         ip1    = ipdet + m
         ip2    = ip1 + m
         ip3    = ip2 + m
         ip4    = ip3 + m
         ip5    = ip4 + m
         ip6    = ip5 + m
         ip7    = ip6 + m
         ip8    = ip7 + m
         ip9    = ip8 + m
         ipwork = ip9 + m
         ipold  = ipwork + m

!        --- compute points and weights for numerical integration
!            compute derivatives of the basisfunctions
!            corresponding to the reference element

         call elp640(1, swork(ipx1re), swork(ipw1), swork(ipd1), &
                     swork(ipwork), mone)
      end if

      if ( ierror/=0 ) go to 1000

!     --- compute integration points, determinant and partial derivatives

      if ( ndim==3 ) then

         call elp646(coor,index1,swork(ipx),swork(ipx+m), &
                     swork(ipx+2*m), &
                     swork(ipd1), swork(ipdet), swork(ip1), &
                     swork(ip2), swork(ip3), swork(ip4), &
                     swork(ip5), swork(ip6), swork(ip7), &
                     swork(ip8), swork(ip9), mone)

      else if ( ndim==2 ) then

         call elp642(coor, index1, swork(ipx), swork(ipd1), &
                     swork(ipdet), swork(ip1), swork(ip2), &
                     swork(ip3), swork(ip4), mone, m)

      else if ( ndim==1 ) then

         call elp647( coor, index1, swork(ipx), swork(ipdet), mone)

      else

         call errint ( ndim, 1 )
         call errsub ( 62, 1, 0, 0 )
         go to 1000

      end if

!     --- load the variable viscosity (if necessary)

      if ((ind(1)>0) .or. (ist(1)>0)) then

!     --- coefficient is variable

         if ( ind(1)>10000 ) then
            call errint ( 1, 1 )
            call errsub ( 1781, 1, 0, 0 )
         end if
         call elflr71(1, E%user, swork(iparry), &
                     swork(ipx), swork(ipx), m, m, rdum, &
                      1, index1, E%iuser, swork(ipold), swork )
      end if

      if ( matrix ) then

!     --- compute the spectral element matrix

         jadd = 0

!        --- second derivative part (diffusive terms)

         if( ndim==3 ) then

            call el6084(swork(iparry), E%mat, swork(ipw1), &
                     swork(ipd1), swork(ipdet), &
                     swork(ip1), swork(ip2), swork(ip3), &
                     swork(ip4), swork(ip5), swork(ip6), &
                     swork(ip7), swork(ip8), swork(ip9), &
                     swork(ipwork), mone, m)

!           --- swap parts of element matrix in the case of a linear element

            if (mone==2) call elswap(E%mat, m, ndim)

         else if ( ndim==2 ) then

            call el6044(swork(iparry), E%mat, swork(ipw1), &
                  swork(ipd1), &
                  swork(ipdet), swork(ip1), swork(ip2), &
                  swork(ip3), swork(ip4), swork(ipwork), &
                  mone, m)

!           --- swap parts of element matrix in the case of a linear element

            if (mone==2) call elswap(E%mat, m, ndim)

         else

            call el6016(swork(iparry), E%mat, swork(ipw1), &
                        swork(ipd1), m, swork(ipwork), swork(ipdet))
         end if

      end if

!     --- fill spectral element mass matrix if necessary

      if (notmas==0 ) then

!     --- notmas = 0, compute diagonal mass matrix

         if ( ndim==3 ) then
             call el6087(E%mmat, swork(ipw1), swork(ipdet), &
                         mone, m)

         else if ( ndim==2 ) then
            jadd = 0
            call el6027(E%mmat, swork(ipw1), swork(ipdet), &
                        mone, m)

         else if( ndim==1 ) then
            call el6017(E%mmat, swork(ipw1), swork(ipdet), m)
         end if

      end if

      if ( vector ) then

         if ( method>0 ) then

!        --- fe-preconditioning, build residual vector
!            get the old solution

            call el3007 ( swork(ipold), rdum, rdum, rdum, m, m, &
                          ndim, ncomp, index1, rdum, &
                          swork(ipold), jtrans, numold, 1 )

!           --- then calculate the residual

            jadd = 0
            if ( ndim==3 ) then
               call el6085(swork(iparry), E%vec, swork(ipw1), &
                     swork(ipd1), swork(ipdet), &
                     swork(ip1), swork(ip2), swork(ip3), &
                     swork(ip4), swork(ip5), swork(ip6), &
                     swork(ip7), swork(ip8), swork(ip9), &
                     swork(ipwork), swork(ipold), mone)
               jadd = 1

            else if ( ndim==2 ) then
               call el6034(swork(iparry), E%vec, swork(ipold), &
                          swork(ipw1), swork(ipd1), &
                          swork(ipdet), swork(ip1), &
                          swork(ip2), swork(ip3), swork(ip4), &
                          swork(ipwork), mone)
               jadd = 1

            else
               call el6018(swork(iparry), E%vec, swork(ipold), &
                           swork(ipdet), swork(ipw1), swork(ipd1), m, &
                           swork(ipwork))
               jadd = 1
            end if

            if (method==2) then

!           --- add the mass residual

               if ( ndim==3 ) then
                  call el6086( E%vec, swork(ipw1), swork(ipdet), &
                               swork(ipold), mone)
               else if ( ndim==2 ) then
                  call el6038( E%vec, swork(ipw1), swork(ipdet), &
                               swork(ipold), mone)
               else
                  call el6015( E%vec, swork(ipold), swork(ipw1), &
                               swork(ipdet), mone)
               end if

            end if

         else

!        --- if method = 0 the right-hand side must me calculated with DERIV

             call errsub ( 1621, 0, 0, 0 )
             go to 1000

         end if

!        --- swap in the case of linear elements

         if ( mone==2.and.ndim>1 ) &
            call elvswap ( E%vec, m, ndim )

      end if

1000  call erclos('elm607')

      end
