      subroutine elm850matrix ( jelp, elemmt, w, dphidx, phi, coor2d, &
                                phixi, x, xgauss )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.2  date 12-12-2012 Long integers
!        version  1.1  date 20-10-2010 new call to el3002
!
!   copyright (c) 2009-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix for stochastic investment problem
!
! **********************************************************************
!
!                       KEYWORDS
!
!     stochastic
!     investment
!     1d
!     2d
!     element_matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulestoc
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer jelp
      double precision elemmt(icount,icount), w(m), dphidx(n,m,ndim), &
                       phi(n,m), coor2d(ndim,*), phixi(*), x(n,ndim), &
                       xgauss(m,ndim)

!     coor2d           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     dphidx         i    Array of length n x m x ndim  containing the
!                         derivatives
!                         of the basis functions in the sequence:
!                              d phi_i/ dx_j(xg_k) = dphidx (i,k,j);
!                         If the element is a linear simplex, the
!                         derivatives are constant and only k = 1 is filled.
!     elemmt         o    Element matrix
!     jelp           i    Indication of the type of basis function subroutine
!                         must be called by ELxxxx.
!     phi            i    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     phixi         i    array of length n x m to store the values of dphi/dxsi
!                         according to:
!                                     k
!                         dphi /dxsi(x ) = phixi(i,k)
!                             i
!     w              i    array of length m containing the weights for
!                         integration
!     x              array of length n x ndim containing the coordinates of
!                    the nodes
!     xgauss         i    array of length m x ndim containing the co-ordinates
!                         of the gauss points.  array xgauss is only filled when
!                         gauss integration is applied.
!                         xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                         zg_i = xgauss (i,3)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer index1(1), iuser(1), metup, kfirst
      double precision alpha(m,6), psi(1), user(1), vecloc(1), &
                       psiupw(1), h, uvel(m,3), beta(m)
      logical second, symm

!     alpha          array of length m x 3 in which the values of
!                    alpha  , alpha  , alpha
!                         11       12       22
!                    are stored in the sequence:
!                                            k
!                    alpha ( k,i ) = alpha (x )   i = 1(1)3, k=1(1)m
!                                         i
!     beta           array of length m containing the values of the
!                    third coefficient in the integration points
!     h              Help variable to store a constant temporarily
!     index1         Dummy
!     iuser          Dummy
!     kfirst         Defines first component for convection
!     metup          Dummy
!     psi            Dummy
!     psiupw         Dummy
!     second         Dummy
!     symm           Indication whether the resulting matrix is symmetrical
!                    (true) or not (false)
!     user           Dummy
!     uvel           Contains the velocity vector
!     vecloc         Dummy
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL3002         Compute integrals of the shape / beta(x) phi (x) phi (x) dx
!                                                   e            i       j
!     ELM800CONV     Compute element matrix for convective terms
!     ELM800DIF      Compute element matrix due to diffusion term
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm850matrix' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elm850matrix'
         write(irefwr,2) 'sigma, sigmax, sigmay', sigma, sigmax, sigmay
         write(irefwr,2) 'mu, mux, muy', mu, mux, muy
         write(irefwr,2) 'r', rstoc
         write(irefwr,1) 'm, n', m, n
         call prinrl ( xgauss, m, 'xgauss' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- First diffusion part: fill coefficients alpha

      if ( jelp<3 ) then

!     --- 1D, alpha = sigma^2 x^2

         h = 0.5d0*sigma**2
         alpha(1:m,1) = h*xgauss(1:m,ixstoc)**2

      else

!     --- 2D, alpha(1,1) = sigmax^2 x^2/2
!             alpha(1,2) = rho*sigmax*sigmay*x*y/2
!             alpha(2,2) = sigmay^2 y^2/2

         h = 0.5d0*sigmax**2
         alpha(1:m,1) = h*xgauss(1:m,1)**2  ! alpha(1,1)
         h = 0.5d0*rho*sigmax*sigmay
         alpha(1:m,2) = h*xgauss(1:m,1)*xgauss(1:m,2) ! alpha(1,2)
         h = 0.5d0*sigmay**2
         alpha(1:m,3) = h*xgauss(1:m,2)**2  ! alpha(2,2)

      end if  ! ( jelp<3 )
      if ( debug ) call prinrl ( alpha, m, 'alpha' )

!     --- Compute diffusion matrix (no upwind)

      jadd = 0
      call elm800dif ( jelp, alpha, elemmt, w, dphidx, psi, &
                       phi, coor2d, index1, &
                       iuser, user, vecloc, second, &
                       phixi, x, xgauss )
      if ( debug ) call prinrl1(elemmt,icount,icount,'elemmt/dif')

!     --- Next convection part: fill coefficients uvel

      if ( jelp<3 ) then

!     --- 1D, u = sigma^2 x-mu x

         h = sigma**2-mu
         uvel(1:m,1) = h*xgauss(1:m,ixstoc)

      else

!     --- 2D, u = x ( sigmax^2 - mux + rho sigmax sigmay/2 )
!             v = y ( sigmay^2 - muy + rho sigmax sigmay/2 )

         h = sigmax**2+0.5d0*rho*sigmax*sigmay-mux
         uvel(1:m,1) = h*xgauss(1:m,1)  ! u
         h = sigmay**2+0.5d0*rho*sigmay*sigmax-muy
         uvel(1:m,2) = h*xgauss(1:m,2)  ! v

      end if  ! ( jelp<3 )
      if ( debug ) call prinrl ( uvel, m, 'u' )

!     --- Compute convection matrix (no upwind)

      metup = 0     ! no upwind
      kfirst = 1    ! start with first component
      idefcr = 0    ! no central differences
      call elm800conv ( elemmt, w, dphidx, psiupw, phi, xgauss, &
                        symm, metup, uvel, kfirst )
      if ( debug ) call prinrl1(elemmt,icount,icount,'elemmt/conv')

!     --- Zeroth order part ( constant r)

      beta = rstoc
      if ( debug ) call prinrl ( beta, m, 'beta' )
      call el3002 ( beta, elemmt, w, phi, m, n, n )
      if ( debug ) call prinrl1(elemmt,icount,icount,'elemmt/zero')

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elm850matrix'

      end if  ! ( debug )
      call erclos ( 'elm850matrix' )

      end
