      subroutine elp611 ( coor2d, x, w, dphidx, xgauss, phi, index1, &
                          qmat, tang )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.0  date 06-01-2016 Use sepmoduleelem
!        version  4.0  date 10-06-2013 Use subroutines
!        version  3.5  date 04-09-2002 Correction dimension xloc
!
!   copyright (c) 1992-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute basis functions and related quantities for quadratic line element
!     Computed are:     weights for numerical integration
!                       values of derivatives of basis functions
!                       values of basis functions in integration points
!     ELP611 is called by several types of element subroutines
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     quadratic
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleconsts
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer index1(3)
      double precision coor2d(ndim,*), x(3,ndim), w(m), dphidx(3,m), &
                       xgauss(m,ndim), phi(3,m), qmat(2,2,3), tang(m,*)

!     coor2d           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     dphidx         o    Array of length n x m x ndim  containing the
!                         derivatives
!                         of the basis functions in the sequence:
!                              d phi_i/ dx_j(xg_k) = dphidx (i,k,j);
!                         If the element is a linear simplex, the
!                         derivatives are constant and only k = 1 is filled.
!     index1         i    Array of length inpelm containing the point numbers
!                         of the nodal points in the element
!     phi            o    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     qmat           o    transformation matrix global -> local coordinates
!                         in case of a 3D element
!     tang           o    In this array the tangential vector is stored
!                         tang(j,i) contains the j th component of the
!                         tangential vector in the i-th integration point
!     w              o    array of length m containing the weights for
!                         integration
!     x             i/o   array of length m x ndim containing the co-ordinates
!                         of the nodal points
!     xgauss         o    array of length m x ndim containing the co-ordinates
!                         of the gauss points.  array xgauss is only filled when
!                         gauss integration is applied.
!                         xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                         zg_i = xgauss (i,3)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision h, phider(3,3), wh(3), dxdxi(3,m)
      integer i, k, iweigo, jcartsav, iqmat
      save iweigo, jcartsav, iqmat, wh, phider

!     dxdxi          work array of length 3xm to store the values of
!                    dx/dxi, dy/dxi and dz/dxi according in the integration
!                    points
!     h              Help variable to store a constant temporarily
!     i              Counting variable
!     iqmat          if 1 the qmat matrix must be filled
!     iweigo         if 1 the weights must be filled
!     jcartsav       Help variable to store jcart temporarily
!     k              Counting variable
!     phider         Contains the derivatives of the basis functions in the
!                    reference element
!                    phider(i,j) = d phi_i/ ds (x^j)
!     wh             Contains the weights in the reference element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELP611CURVED   Fill dxdxi and dphidx for a curved 1d element
!     ELP611FIRST    Fill element independent quantities for a quadratic
!                    line element
!     ELP611QMAT     Fill array qmat for a quadratic line element
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRORELVAR0    Print error message errornr for element
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRINRL2        Print a real two-dimensional array
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!                              METHOD
!        To compute the derivatives of the basis functions, the jacobian
!        of the element and the weights, the following definitions are used:
!        h   =  x2 - x1
!        d phi  / dx   =  -1/h        d phi  / dx   =  1/h
!             1                            2
!                                        |h|       1     2
!        Newton Cotes rule:  / f(x) dx = ---- [ f(x )+f(x ) ]
!                            e            2
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elp611' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elp611'
         write(irefwr,1) 'igausp', igausp
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ifirst==0 ) then

!     --- ifirst = 0  (First call)

         call elp611first ( phi, wh, iweigo, jcartsav, phider, iqmat )

      end if  ! ( ifirst==0 )
      if ( ierror/=0 ) go to 1000

!     --- Store coordinates in x

      do k = 1, ndim
         x(1:3,k) = coor2d(k,index1(1:3))
      end do

!     --- Compute length of element (jacobian and other stuff depending
!                                    on dimension)

      if ( ndim==1 ) then

!     --- ndim = 1, simple case

         h = x(3,1)-x(1,1)

         if ( abs(h)<=0d0 ) then

!        --- Error 105: h is zero

            call errorelvar0 ( 105 )
            call prinin(index1,2,'node numbers')
            call prinrl2(x,2,ndim,'co-ordinates')
            go to 1000

         end if  ! ( hloc<=0d0 )

         w(1:m) = abs(h)*wh(1:m)
         if ( ideriv==1 ) dphidx(:,1:m) = phider(:,1:m)/h

      else

!     --- ndim>1, complicated case
!         Compute w, dphidx, tang

         call elp611curved ( dxdxi, x, phider, w, dphidx, index1 )

      end if  ! ( ndim==1 )

      if ( igausp==1 .or. irule>1 .and. jcartsav==3  ) then

!     --- gauss points must be filled

         if ( irule==1 ) then

!        --- irule = 1  Newton Cotes rule

            xgauss = x

         else

!        --- irule>1  Gauss rule

            do i = 1, ndim
               do k = 1, m
                  xgauss(k,i) = sum ( x(:,i)*phi(:,k) )
               end do  ! k = 1, m
            end do  ! i = 1, ndim

         end if  ! ( irule==1 )

      end if

      if ( ndim>1 .and. iqmat>0 ) then

!     --- Compute qmat

         call elp611qmat ( w, dxdxi, qmat )

      end if  ! ( iqmat>0 )

      if ( ndim>1 .and. icoor>10 ) then

!     --- Compute tang

         do k = 1, ndim
            tang(1:m,k) = dxdxi(k,1:m)/w(1:m)
         end do  ! k = 1, ndim

      end if  ! ( iqmat>0 )

      if ( jcart>1 ) then

!     --- axisymmetric co-ordinates  correct weights

         if ( irule>1 ) then
            w(1:m) = w(1:m)*xgauss(1:m,1)*pi2
         else
            w(1:m) = w(1:m)*x(1:m,1)*pi2
         end if

      end if

1000  call erclos ( 'elp611' )

      if ( debug ) then

!     --- Debug information

         call prinrl ( xgauss, m, 'xgauss' )
         call prinrl ( w, m, 'w' )
         write(irefwr,*) 'End elp611'

      end if  ! ( debug )

      end
