      subroutine elp611qmat ( w, dxdxi, qmat )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.0  date 12-06-2013
!
!   copyright (c) 2013-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array qmat for a quadratic line element
!
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     quadratic
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision w(m), dxdxi(3,m), qmat(2,2,3)

!     dxdxi          i    work array of length 3xm to store the values of
!                         dx/dxi, dy/dxi and dz/dxi in the integration points
!     qmat           o    transformation matrix global -> local coordinates
!                         in case of a 3D element
!     w              i    array of length m containing the weights for
!                         integration
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer k, j
      double precision exi(2), eeta(2), norm, ealpha(2), ebeta(2), h

!     ealpha         base vector in alpha direction
!     ebeta          base vector in beta direction
!     eeta           base vector in eta direction
!     exi            base vector in xi direction
!     h              Help variable to store a constant temporarily
!     j              Counting variable
!     k              Counting variable
!     norm           norm of vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     419   NDIM incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elp611qmat' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elp611qmat'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ndim/=2 ) then

!     --- ndim#2 not yet implemented

         call errint ( ndim, 1 )
         call errint ( 2, 2 )
         call errint ( 2, 3 )
         call errsub ( 419, 3, 0, 0 )
         go to 1000

      end if  ! ( ndim/=2 )

      do k = 1, m

!     --- Define e_xi = x,xi / ||x,xi||

         norm = 1d0/w(k)
         exi(1) = dxdxi(k,1)*norm
         exi(2) = dxdxi(k,2)*norm

!        --- Define e_eta = e_3 x e_xi / ||e_3 x e_xi||

         norm = 1d0/sqrt(exi(1)**2+exi(2)**2)
         eeta(1) = -exi(2)*norm
         eeta(2) = exi(1)*norm

!        --- e_alpha = (e_xi + e_eta)/2 / || (e_xi + e_eta)/2 ||

         ealpha(1) = 5d-1*(exi(1)+eeta(1))
         ealpha(2) = 5d-1*(exi(2)+eeta(2))
         norm = 1d0/sqrt(ealpha(1)**2+ealpha(2)**2)
         ealpha(1) = ealpha(1)*norm
         ealpha(2) = ealpha(2)*norm

!        --- e_beta = e_3(l) x e_alpha / || e_3(l) x e_alpha ||

         norm = 1d0/sqrt(ealpha(1)**2+ealpha(2)**2)
         ebeta(1) =-ealpha(2)*norm
         ebeta(2) = ealpha(1)*norm

!        --- e_1(l) = srqt(2)/2 ( e_alpha - e_beta )
!            e_2(l) = srqt(2)/2 ( e_alpha + e_beta )

         h = 0.5d0*sqrt(2d0)
         do j = 1,2
            qmat(2,j,k) = h*(ealpha(j)-ebeta(j))
            qmat(1,j,k) = h*(ealpha(j)+ebeta(j))
         end do

      end do

1000  call erclos ( 'elp611qmat' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elp611qmat'

      end if  ! ( debug )

      end
