      subroutine elp622curved ( iweigo, x, index1, intrule, dphidx, &
                                dphidxi, dxdxi, iqmat, delta )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.1  date 10-06-2013 Use elpgauss
!        version  1.0  date 27-02-2009
!
!   copyright (c) 2009-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute basis functions and related quantities for quadratic
!     triangle (curved element)
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     quadratic
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iweigo, index1(n), intrule, iqmat
      double precision x(n,ndim), dphidx(n,m,ndim), dphidxi(n,m,2), &
                       dxdxi(m,3,3), delta(7)

!     delta          o    jacobian of element
!     dphidx         o    Array of length n x m x ndim  containing the
!                         derivatives
!                         of the basis functions in the sequence:
!                         d phi / dx (xg ) = dphidx (i,k,1);
!                              i        k
!                         d phi / dy (xg ) = dphidx (i,k,2);
!                              i        k
!                         d phi / dz (xg ) = dphidx (i,k,3);
!                              i        k
!                         If the element is a linear simplex, the
!                         derivatives are constant and only k = 1 is filled.
!     dphidxi        i    contains derivatives in reference element for gauss
!                         rule.
!                         dphidxi(i,k,l) = d phi_i  / d xi_l (x^k)
!     dxdxi          o    work array of length mx3x3 to store the values of
!                         dx/dxi, dx/deta, dy/dxi and dy/deta according to:
!                         dx/dxi(x_k) = dxdxi(k,1,1)
!                         dx/deta(x_k) = dxdxi(k,1,2)
!                         dy/dxi(x_k) = dxdxi(k,2,1)
!                         dy/deta(x_k) = dxdxi(k,2,2) etc.
!     index1         i    Array of length inpelm containing the point numbers
!                         of the nodal points in the element
!     intrule        i    Actual integration rule
!     iqmat          i    if 1 the qmat matrix must be filled
!     iweigo         i    if 1 the weights must be filled
!     x              o    array of length 6 x 2 containing the x- and
!                         y-coordinates of the nodal points with respect to the
!                         local numbering in the sequence:
!                         x  = x (i,1);  y  = x (i,2);
!                          i              i
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, k

!     i              Counting variable
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELP3DTO2DDELTA Compute the jacobian delta and dx/dxi for a 2d element
!                    in 3d
!     ELP3DTO2DDERIV Compute the derivatives dphi/dx for a 2d element in 3d
!     ELPDERIV2D     Compute the Jacobian of an element and store in delta
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!            |  x          y    |
!            |   xi        xi |
!        j  =|                  |
!            |  x          y    |
!            |   eta        eta |
!                         6    k
!        d x / dxi   =  som  x    d phi / d xi
!           i      j     k=1   i        k       j
!                          -1
!        grad    phi   =  j   grad        phi
!            xy     i             xi eta    i
!                                          1   6       k      k
!        newton cotes rule:  / f(x) dx =  --- som [ f(x ) |j(x )| w ]
!                            e             6  k=1                  k
!     In case of 3D elements we use the following formulae:
!     x = som phi (xi,eta) x    dx/dxi = som dphi /dxi x   dx/deta=...
!     -    i     i          -i    -         i      i      -i
!             a                      a                         a
!     dphi /dx = dphi / dxi  dxi/dx   +  dphi / deta  deta/dx
!         i          i                         i
!                                                                        -1
!     | dxi/dx   deta/dx   dzeta/dx  |   | dx/dxi   dy/dxi   dz/dxi |
!     | dxi/dy   deta/dy   dzeta/dy  | = | dx/deta   dy/deta   dz/deta |
!     | dxi/dz   deta/dz   dzeta/dz  |   | dx/dzeta  dy/dzeta  dz/dzeta|
!     zeta is choosen perpendicular to the xi-eta direction, which implies
!     that we may choose:
!     dx / dzeta = dx / dxi times  dx / deta / || dx/dxi x dx/deta ||
!      -            -                -              -         - &
!     where times or x is the outer product
!     The Jacobian of the dx/dxi matrix is given by:
!     J = dx/dzeta ( dy/dxi dz/deta - dy/deta dz/dxi )
!       + dy/dzeta ( dz/dxi dx/deta - dz/deta dx/dxi )
!       + dz/dzeta ( dx/dxi dy/deta - dx/deta dy/dxi )
!     and since dx/dzeta before normalization is given by
!         dx/dzeta = dy/dxi dz/deta - dy/deta dz/dxi
!         dy/dzeta = dz/dxi dx/deta - dz/deta dx/dxi
!         dz/dzeta = dx/dxi dy/deta - dx/deta dy/dxi
!     we see that the normalization factor is equal to
!        sqrt(     ( dy/dxi dz/deta - dy/deta dz/dxi )^2
!                + ( dz/dxi dx/deta - dz/deta dx/dxi )^2
!                + ( dx/dxi dy/deta - dx/deta dy/dxi )^2 ) = J
!      The derivatives dxi/dx and so on follow from the inverse, &
!      which in this 3x3 case leads to:
!      dxi/dx = ( dy/deta  dz/dzeta - dy/dzeta dz/deta  ) / J
!      dxi/dy = ( dy/dzeta dz/dxi  - dy/dxi  dz/dzeta ) / J
!      dxi/dz = ( dy/dxi  dz/deta  - dy/deta  dz/dxi  ) / J
!      deta/dx = ( dz/deta  dx/dzeta - dz/dzeta dx/deta  ) / J
!      deta/dy = ( dz/dzeta dx/dxi  - dz/dxi  dx/dzeta ) / J
!      deta/dz = ( dz/dxi  dx/deta  - dz/deta  dx/dxi  ) / J
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elp622curved' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elp622curved'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( debug ) &
         write(irefwr,1) 'iweigo, ideriv, kstep, m, intrule', &
                          iweigo, ideriv, kstep, m, intrule

      if ( iweigo==1 .or. ideriv==1 ) then

!     --- ideriv = 1  compute derivatives or iweigo = 1: weights
!         compute dx/dxi, dy/dxi, dx/deta, dy/deta;  store in dxdxi

         if ( debug ) &
            write(irefwr,1) 'ndim, kstep, m', ndim, kstep, m

!        --- fill jacobian in delta

         if ( ndim>2 .and. iqmat==0 ) then

!        --- Standard 3D case

            call elp3dto2ddelta ( x, dphidxi, dxdxi, delta, index1 )

         else

!        --- Standard 2D case

            call elpderiv2d ( dxdxi, x, dphidxi, delta, index1 )

         end if

      end if

      if ( debug ) call prinrl ( delta, m, 'delta' )

      if ( ideriv==1 ) then

!     --- ideriv = 1   fill derivatives

         if ( ndim==3 .and. iqmat==0 ) then

!        --- 3D case

            call elp3dto2dderiv ( dxdxi, delta, dphidx, dphidxi )

         else

!        --- standard 2D case

            if ( debug ) then

!           --- debug information

               call prinrl1 ( dphidxi(1,1,1), m, n, 'dphidxi' )
               call prinrl1 ( dphidxi(1,1,2), m, n, 'dpdeta' )

            end if  ! ( debug )

            do k = kstep, m, kstep
               do i = 1, n
                  dphidx(i,k,1) = ( dphidxi(i,k,1)*dxdxi(k,2,2) - &
                            dphidxi(i,k,2)*dxdxi(k,2,1) ) / delta(k)
                  dphidx(i,k,2) = (-dphidxi(i,k,1)*dxdxi(k,1,2) + &
                            dphidxi(i,k,2)*dxdxi(k,1,1) ) / delta(k)
               end do
            end do

         end if

      end if

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( dphidx(1,1,1), m, n, 'dphidx' )
         call prinrl1 ( dphidx(1,1,2), m, n, 'dphidy' )
         if ( ndim==3 ) call prinrl1 ( dphidx(1,1,3), m, n, 'dphidz' )
         write(irefwr,*) 'End elp622curved'

      end if  ! ( debug )
      call erclos ( 'elp622curved' )

      end
