      subroutine elp622first1 ( intrule, dpdxi, phi, weight, lambda, m1 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-05-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill some element independent quantities for a quadratic triangle
!
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     quadratic
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: intrule, m1
      double precision, intent(out) :: weight(m1), dpdxi(6,m1,2), &
                                       phi(6,m1), lambda(3,m1)

!     dpdxi          o    contains derivatives in reference element for gauss
!                         rule.
!                         dpxi(i,k) = d phi_i/ d xi (x^k)
!     intrule        i    Actual integration rule
!     lambda         o    Array containing the linear basis functions in a point
!     m              i    number of integration points
!     phi            o    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     weight         o    Array with the weights for the unit quadratic triangle
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer k, i
      double precision :: h

!     h              Help variable to store a constant temporarily
!     i              Counting variable
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elp622first1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elp622first1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      select case (intrule)

      case(1)

!     --- intrule = 1 : 6 pts Newton Cotes
!         fill d phi / d xsi and  d phi / d eta

         lambda = 0d0
         do i = 1, 3
            lambda(i,2*i-1) = 1d0
         end do  ! i = 1, 3
         lambda(1:2,2) = 0.5d0
         lambda(1,6) = 0.5d0
         lambda(2:3,4) = 0.5d0
         lambda(3,6) = 0.5d0
         weight = 0d0
         h = 1d0/6d0
         weight(2) = h
         weight(4) = h
         weight(6) = h

      case(2)

!     --- intrule = 2 : 4 pts gauss
!         fill d phi / d xsi and  d phi / d eta

         lambda(1:3,1:m1) = alam2(1:3,1:m1)
         weight(1:m1) = wgs2(1:m1)

      case(3)

!     --- intrule = 3 7pts gauss

         lambda(1:3,1:m1) = alam3(1:3,1:m1)
         weight(1:m1) = wgs3(1:m1)

      end select  ! case (intrule)

!     --- Compute phi

      do i = 1, 3
         do k = 1, m1
            phi(2*i-1,k) = lambda(i,k) * (2d0*lambda(i,k)-1d0)
         end do
      end do
      do k = 1, m1
         phi(2,k) = 4d0 * lambda(1,k) * lambda(2,k)
         phi(4,k) = 4d0 * lambda(2,k) * lambda(3,k)
         phi(6,k) = 4d0 * lambda(3,k) * lambda(1,k)
      end do  ! k = 1, m1

!     --- fill d phi / d xsi and  d phi / d eta

      do k = 1, m1
         dpdxi(1,k,1) = -4d0*lambda(1,k)+1d0
         dpdxi(3,k,1) =  4d0*lambda(2,k)-1d0
         dpdxi(5,k,1) =  0d0
         dpdxi(2,k,1) =  4d0*(lambda(1,k)-lambda(2,k))
         dpdxi(4,k,1) =  4d0* lambda(3,k)
         dpdxi(6,k,1) = -dpdxi(4,k,1)
         dpdxi(1,k,2) =  dpdxi(1,k,1)
         dpdxi(3,k,2) =  0d0
         dpdxi(5,k,2) =  4d0*lambda(3,k)-1d0
         dpdxi(4,k,2) =  4d0* lambda(2,k)
         dpdxi(2,k,2) = -dpdxi(4,k,2)
         dpdxi(6,k,2) =  4d0*(lambda(1,k)-lambda(3,k))
      end do  ! k = 1, m1

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elp622first1'

      end if  ! ( debug )
      call erclos ( 'elp622first1' )

      end
