      subroutine elp623 ( coor2d, x, w, dphidx, xgauss, phi, index1, &
                          qmat, psi, dpsidx, dphidxi )
! ======================================================================
!
!        programmer    Guus Segal
!        version  9.0  date 06-01-2016 Use sepmoduleelem
!        version  8.0  date 10-06-2013 Use subroutines
!        version  7.6  date 27-02-2010 Minor change
!
!   copyright (c) 1989-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute basis functions and related quantities for biquadratic
!     quadrilateral
!     local numbering used in this subroutine with respect to jndex1:
!     7----8----9
!     |         |
!     4    5    6    numbering 1
!     |         |
!     1----2----3
!     local numbering with respect to index1:
!     7----6----5
!     |         |
!     8    9    4    numbering 2
!     |         |
!     1----2----3
!     Computed are:     weights for numerical integration
!                       values of derivatives of basis functions
!                       values of basis functions in integration points
!     ELP623 is called by several types of element subroutines
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     quadratic
!     quadrilateral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer index1(9)
      double precision coor2d(ndim,*), x(9,ndim), dphidx(9,m,ndim), &
                       xgauss(m,ndim), phi(9,m), qmat(3,3,*), psi(4,m), &
                       dphidxi(9,9,2), dpsidx(4,9,2), w(*)

!     coor2d           i    array of length 2 x number of nodal points containing
!                         the co-ordinates of the nodal points with respect to
!                         the global numbering
!                         x  = coor2d (1,index1(i));  y  = coor2d (2,index1(i));
!                          i                         i
!     dphidx         o    Array of length 9 x m x 2  containing the derivatives
!                         of the basis functions in the sequence:
!                                      k
!                         d phi / dx (x )= dphidx (i,k,1);
!                              i
!                                      k
!                         d phi / dy (x )= dphidx (i,k,2);
!                              i
!     dphidxi       i/o   Array containing dphi_i/dxi in the integration points
!     dpsidx         o    Array of length n x m x l  containing the derivatives
!                         of the linear basis functions in the sequence:
!                         d psi / dx (xg ) = psix (i,k,1);
!                              i        k
!                         d psi / dy (xg ) = psix (i,k,2);
!                              i        k
!                         d psi / dz (xg ) = psix (i,k,3);
!                              i        k
!     index1         i    Array of length 9 containing the nodal point numbers
!                         of the element
!     phi           i/o   Array of length 9 x m containing the values of the
!                         basis functions in the Gauss points in the sequence:
!                                k
!                         phi (xg ) = phi (i,k)
!                            i
!                         Array phi is only filled when ifirst = 0 and gauss
!                         integration is applied
!     psi           i/o   array of length 4 x m containing the values of the
!                         linear basis functions in the gauss points
!     qmat           o    transformation matrix global -> local coordinates
!                         in case of a 3D element
!     w              o    array of length m containing the weights for the
!                         numerical integration
!     x             i/o   array of length 9 x 2 containing the x- and
!                         y-coordinates of the nodal points with respect to the
!                         local numbering in the sequence:
!                         x  = x (i,1);  y  = x (i,2);
!                          i              i
!     xgauss         o    Array of length m x 2 containing the co-ordinates of
!                         the gauss points.  Array xgauss is only filled when
!                         Gauss integration is applied.
!                         xg  = xgauss (i,1);  yg  = xgauss (i,2);
!                           i                    i
!                            numbering:
!                            4 points                9 points
!                            3------4                7--8--9
!                            |      |                |     |
!                            |      |                4  5  6
!                            |      |                |     |
!                            1------2                1--2--3
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision eps, som, &
                       dxdxi(9,3,3), xg(9,2), &
                       dxidx(9,2,3), psixi(4,9,2), weight(9)
      logical curved, comppsi, compqmat
      integer i, k, intrule, iweigo, iqmat
      save iweigo, iqmat, compqmat, comppsi, xg, psixi, weight

!     comppsi        if true the linear basis functions must be evaluated
!     compqmat       if true qmat must be evaluated
!     curved         indication if the element is curved (true) or not (false)
!     dxdxi          Work array of length 9x3x3 to store the values of dx/dxi, &
!                    dx/deta, dy/dxi and dy/deta according to:
!                            k
!                    dx/dxi(x ) = dxdxi(k,1,1)
!                             k
!                    dx/deta(x ) = dxdxi(k,1,2)
!                            k
!                    dy/dxi(x ) = dxdxi(k,2,1)
!                             k
!                    dy/deta(x ) = dxdxi(k,2,2)
!     dxidx          work array of length 9x2x3 to store the values of dxi/dx, &
!                    dxi/dy, dxi/dz and deta/dx ... according to:
!                            k                           k
!                    dxi/dx(x ) = dxdxi(k,1,1)   dxi/dy(x ) = dxdxi(k,1,2)
!     eps            Local accuracy
!     i              Counting variable
!     intrule        Actual integration rule
!     iqmat          if 1 the qmat matrix must be filled
!     iweigo         if 1 the weights must be filled
!     k              Counting variable
!     psixi          Array of length nvert x m x ndim containing the values
!                    of the derivatives of the basis functions in the Gauss
!                    points in the reference element
!     som            Help parameter to compute a sum
!     weight         Contains the weights for the integration rule in the
!                    reference element
!     xg             Array of length nx2 containing the co-ordinates of the
!                    Gauss points in the reference element for a four point
!                    Gauss rule
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELFILLQMAT     Fill the transformation matrix qmat for the mapping of a
!                    3d surface onto a 2d surface
!     ELP623CURVED   Compute basis functions and related quantities for
!                    biquadratic quadrilateral (curved element)
!     ELP623FIRST    Fill element independent quantities for a
!                    bi-quadratic quadrilateral
!     ELP623STRAIGHT Compute basis functions and related quantities for
!                    biquadratic quadrilateral (straight element)
!     ELPCOOR        fill co-ordinates of nodes in array x
!     ELPGAUSS       Fill Gauss coordinates
!     ELPMAPTANG     Map three-dimensional array dphidx onto array dphids on
!                    surface using mapping array qmat
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     See elp623curved and elp623straight
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elp623' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elp623'
         write(irefwr,1) 'jdiag, iweigh, irule, icoor, ifirst, itype', &
                          jdiag, iweigh, irule, icoor, ifirst, itype
         call prinin ( index1, 9, 'index1' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iweigo = mod(iweigh,10)
      intrule = mod(irule,100)
      iqmat  = iweigh/10
      if ( debug ) write(irefwr,1) 'iqmat', iqmat
      compqmat = .false.

!     --- Check if we need to compute qmat

      if ( iqmat==3 ) then

         compqmat = .true.

!        --- reset iqmat!!

         iqmat = 0

      end if

      if ( ifirst==0 ) then

!     --- ifirst = 0  (First call)
!         initialize some quantities

         call elp623first ( comppsi, xg, dphidxi, &
                            phi, psi, psixi, intrule, weight )

      end if  ! ( ifirst==0 )

      if ( debug ) then
         write(irefwr,*) 'comppsi, compqmat', comppsi, compqmat
         if ( comppsi ) call prinrl1 ( psi, m, 4, 'psi' )
      end if  ! ( debug ) then

!     --- Store coordinates in x

      call elpcoor ( icoor, coor2d, index1, x, n, ndim, n )
      if ( debug ) call prinrl1 ( x, ndim, 9, 'x' )

!     --- Fill gauss points

      call elpgauss ( ndim, intrule, x, xgauss, phi )

      if ( iweigo==1 .or. ideriv==1 .or. igausp==1 ) then

!     --- weights, Gauss points or derivatives must be computed
!         Check if element is straight or curved
!         First find relative accuracy

         if ( debug ) &
            write(irefwr,1) 'iweigo, ideriv, igausp', &
                             iweigo, ideriv, igausp
         if ( (itype>=200 .and. itype<213) .or. &
              (ndim==3 .and. iqmat==0) ) then

!        --- Standard 3D case. Treat as curved

            curved = .true.

         else

!        --- 2D case, compute curved

            eps = sqreps * max(abs(x(1,1)), abs(x(3,1)), abs(x(5,1)), &
                               abs(x(7,1)), abs(x(1,2)), abs(x(3,2)), &
                               abs(x(5,2)), abs(x(7,2)) )
            curved = .false.
            som = x(1,1) + x(3,1) + x(5,1) + x(7,1)
            if ( abs(4d0*x(9,1)-som)>eps .or. &
                 abs(x(2,1)+x(4,1)+x(6,1)+x(8,1)-som)>eps ) then
               curved = .true.
            else
               som = x(1,2)+x(3,2)+x(5,2)+x(7,2)
               if ( abs(4d0*x(9,2)-som)>eps .or. &
                    abs(x(2,2)+x(4,2)+x(6,2)+x(8,2)-som)>eps ) &
                  curved = .true.
            end if
         end if
         if ( debug ) write(irefwr,*) 'curved', curved

      end if  ! ( iweigo==1 .or. ideriv==1 .or. igausp==1 )

      if ( curved ) then

!     --- curved = true,  isoparametric transformation  (bi-quadratic)

         call elp623curved ( iweigo, x, index1, xgauss, w, dxdxi, &
                             dphidx, dphidxi, dpsidx, psixi, comppsi, &
                             iqmat, dxidx, weight )

         if ( compqmat ) then

!        --- compqmat true, hence array qmat is filled and
!            grad phi is mapped onto the surface

!            call elpqmat ( dxdxi, qmat )      ! fill qmat

            call elfillqmat ( 1, dxdxi, m, 3, 1, qmat )  ! fill qmat
            if ( ideriv==1 ) &
               call elpmaptang ( dphidx, qmat )  ! map grad phi onto surface

         end if  ! ( compqmat )

      else

!     --- curved = false,  straight element

         call elp623straight ( iweigo, x, index1, intrule, &
                               xgauss, xg, w, dxdxi, &
                               dphidx, dphidxi, dpsidx, &
                               psixi, comppsi, weight )

      end if  ! ( curved )

      if ( itype>=902 .and. itype<=903 ) then

!     --- Special case of Navier-Stokes element
!         w(1..m ) contains the weights
!         w(m+1 .. m+n) the diagonal of the mass matrix
!         In this case it is sufficient to use the integration rule available

         do i = 1, n
            som = 0d0
            do k = 1, m
               som = som+w(k)*phi(i,k)**2
            end do  ! k = 1, m
            w(m+i)= som
         end do  ! i = 1, n

      end if  ! ( itype>=902 .and. itype<=903 )

1000  call erclos ( 'elp623' )

      if ( debug ) then

!     --- Debug information

         call prinrl ( w, m, 'w' )
         call prinrl1 ( dphidx(1,1,1), m, 9, 'dphidx' )
         call prinrl1 ( dphidx(1,1,2), m, 9, 'dphidy' )
         write(irefwr,*) 'End elp623'

      end if  ! ( debug )

      end
