      subroutine elp623straight ( iweigo, x, index1, intrule, &
                                  xgauss, xg, w, dxidx, &
                                  dphidx, dphidxi, dpsidx, &
                                  psixi, comppsi, weight )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-01-2016 Use sepmoduleelem
!        version  2.0  date 12-08-2013 New parameter list
!        version  1.0  date 25-02-2009
!
!   copyright (c) 2009-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute basis functions and related quantities for biquadratic
!     quadrilateral (straight element)
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     quadratic
!     quadrilateral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleconsts
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iweigo, index1(9), intrule
      double precision x(9,2), xgauss(m,2), xg(m,2), w(m), &
                       dxidx(m,3,3), dphidx(9,m,ndim), dphidxi(9,9,2), &
                       dpsidx(4,9,2), psixi(4,9,2), &
                       weight(m)
      logical comppsi

!     comppsi        i    if true the linear basis functions must be evaluated
!     dphidx         o    Array of length n x m x ndim  containing the
!                         derivatives
!                         of the basis functions in the sequence:
!                         d phi / dx (xg ) = dphidx (i,k,1);
!                              i        k
!                         d phi / dy (xg ) = dphidx (i,k,2);
!                              i        k
!                         d phi / dz (xg ) = dphidx (i,k,3);
!                              i        k
!                         If the element is a linear simplex, the
!                         derivatives are constant and only k = 1 is filled.
!     dphidxi       i/o   Array containing dphi_i/dxi in a point
!     dpsidx         o    Array of length n x m x l  containing the derivatives
!                         of the linear basis functions in the sequence:
!                         d psi / dx (xg ) = psix (i,k,1);
!                              i        k
!                         d psi / dy (xg ) = psix (i,k,2);
!                              i        k
!                         d psi / dz (xg ) = psix (i,k,3);
!                              i        k
!     dxidx          o    work array of length mx3x3 to store the values of
!                         dx/dxi, dx/deta, dy/dxi and dy/deta according to:
!                         dx/dxi(x_k) = dxidx(k,1,1)
!                         dx/deta(x_k) = dxidx(k,1,2)
!                         dy/dxi(x_k) = dxidx(k,2,1)
!                         dy/deta(x_k) = dxidx(k,2,2) etc.
!     index1         i    Array of length inpelm containing the point numbers
!                         of the nodal points in the element
!     intrule        i    Actual integration rule
!     iweigo         i    if 1 the weights must be filled
!     psixi          i    Array of length 9 x m x 2 containing the values of the
!                         derivatives of the basis functions in the Gauss
!                         points in the reference element
!     w              o    array of length m containing the weights for
!                         integration
!     weight        i/o   weights in the reference element
!     x              o    array of length 9 x ndim containing the x- and
!                         y-coordinates of the nodal points with respect to the
!                         local numbering in the sequence:
!                         x_i = x (i,1);  y_i = x (i,2);
!     xg            i/o   Array of length 4x2 containing the co-ordinates of the
!                         Gauss points in the reference element for a four point
!                         Gauss rule
!     xgauss         o    array of length m x ndim containing the co-ordinates
!                         of the gauss points.  array xgauss is only filled when
!                         gauss integration is applied.
!                         xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, k
      double precision a(3,2), wh(m), h

!     a              Help variable to store quantity
!     h              Help variable to store a constant temporarily
!     i              Counting variable
!     k              Counting variable
!     wh             Array of length m containing the values of the
!                    Jacobian in the integration points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELP621DET      Compute the Jacobian determinant of the bilinear
!                    quadrilateral
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL2        Print a real two-dimensional array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!       To compute the derivatives of the basis functions, the jacobian
!       of the element and the weights, the following definitions are used:
!
!
!     Straight boundaries:
!
!         ij       j     i+1    i
!        e   = (-1)  (  x    - x    )   cyclic: i=1,2,3,4 ;   j=1,2
!                        j+1    j+1
!
!              1     2    1          4    1          3    2    1    4
!        x  = x  + (x  - x ) xi + (x  - x ) eta + (x  - x  + x  - x ) xi eta
!         i    i     i    i          i    i          i    i    i    i
!
!                                   i = 1,2
!                  1    12        42
!        So: x  = x  + e   xi - e   eta + A xi eta
!             1    1
!
!                  1    11        41
!            x  = x  - e   xi + e   eta + B xi eta
!             2    2
!
!
!            |   2    1                  4    1          |
!            |  x  - x  + A eta         x  - x  + A xi  |
!            |   1    1                  1    1          |
!        J  =|                                           |
!            |   2    1                  4    1          |
!            |  x  - x  + B eta         x  - x  + B xi  |
!            |   2    2                  2    2          |
!
!
!             1    2    3    4        12    32
!        A = x  - x  + x  - x    = - e   - e
!             1    1    1    1
!
!
!             3    4    2    1       11    31
!        B = x  - x  - x  + x    =  e   + e
!             2    2    2    2
!
!
!       |J| = J  + J  xi + J  eta
!              0    1        2
!
!              41   12    11   42
!        J  = e  * e   - e  * e
!         0
!
!                   12         11
!        J  = B  * e   - A  * e
!         1
!
!                   41         42
!        J  = A  * e   + B  * e
!         2
!
!
!        dphi / dx  = dphi / dxi dxi/dx  + dphi / deta deta/dx
!                 j                      j                      j
!
!                       41
!        dxi/dx  = [  e   + B xi ] / |J|
!               1
!
!                       42
!        dxi/dx  = [  e   - A xi ] / |J|
!               2
!
!                       11
!        deta/dx  = [  e   - B eta ] / |J|
!               1
!
!                       12
!        deta/dx  = [  e   + A eta ] / |J|
!               2
!
!
!                                           9          k      k
!        Newton Cotes rule:  / f(x) dx =   sum [ w  f(x ) |J(x )| ]
!                            e             k=1    k
!
!        with w =w =w =w =1/36; w = w = w = w =4/36;   w = 16/36
!              1  3  7  9        2   4   6   8          5
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elp623straight' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elp623straight'
         call prinrl2 ( x, n, ndim, 'coordinates' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( iweigo==1 .or. ideriv==1 ) then

!     --- weights, or derivatives must be computed
!         Compute Jacobian in the Gauss points
!         Compute the quantities  a_1 until a_3, store in a(1:3,1:2)

         a(1,:) = x(3,:)-x(1,:)
         a(2,:) = x(7,:)-x(1,:)
         a(3,:) = x(5,:)-x(3,:)-a(2,:)

         if ( debug ) call prinrl2 ( a, 3, 2, 'a' )

         call elp621det ( isubr, a, wh, index1, x, xg, intrule )

         if ( intrule==1 ) then

!        --- Recompute wh

            do k = 3, 1, -1
               wh(2*k+1) = wh(k+1)
            end do  ! k = 3, 1, -1
            do k = 1, 3
               wh(2*k) = (wh(2*k-1)+wh(2*k+1))*0.5d0
            end do  ! k = 1, 3
            wh(8) = 0.5d0 * ( wh(7) + wh(1) )
            wh(9) = 0.25d0 * ( wh(2) + wh(4) + wh(6) + wh(8) )

         end if  ! ( intrule==1 )

      end if  ! ( iweigo==1 .or. ideriv==1 )

      if ( iweigo==1 ) then

!     --- iweigo = 1  Fill weights

         w = weight*abs(wh)

         if ( jcart==2 .or. jcart==3 ) then

!        --- Axi-symmetric co-ordinates

            w = w * xgauss(:,1)
            if ( jcart==2 ) w = w*pi2

         end if  ! ( jcart==2 .or. jcart==3 )

      end if  ! ( iweigo==1 )

      if ( ideriv==1 ) then

!     --- ideriv = 1   Fill derivatives

         if ( debug ) write(irefwr,1) 'ideriv, ndim', ideriv, ndim

!        --- store dxi/dx in dxidx

         do k = 1, m
            h = 1d0 / wh(k)
            dxidx(k,1,1) = (a(2,2)+a(3,2)*xg(k,1))*h  ! dxsi/dx
            dxidx(k,1,2) =-(a(2,1)+a(3,1)*xg(k,1))*h  ! dxsi/dy
            dxidx(k,2,1) =-(a(1,2)+a(3,2)*xg(k,2))*h  ! deta/dx
            dxidx(k,2,2) = (a(1,1)+a(3,1)*xg(k,2))*h  ! deta/dy
         end do  ! k = 1, m

!        --- ideriv = 1   Fill derivatives

         do k = 1, m
            do i = 1, n
               dphidx(i,k,1:2) = dxidx(k,1,1:2)*dphidxi(i,k,1)+ &
                                 dxidx(k,2,1:2)*dphidxi(i,k,2)
            end do
         end do

         if ( comppsi ) then

!        --- Compute psi derivatives

            do k = 1, m
               do i = 1, 4
                  dpsidx(i,k,1:2) = dxidx(k,1,1:2)*psixi(i,k,1)+ &
                                    dxidx(k,2,1:2)*psixi(i,k,2)
               end do
            end do

         end if

      end if

1000  call erclos ( 'elp623straight' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elp623straight'

      end if  ! ( debug )

      end
