      subroutine elp624 ( coor2d, x, w, dphidx, xgauss, phi, index1, &
                          qmat, psi, dpsidx, dphidxi )
! ======================================================================
!
!        programmer    Guus Segal
!        version  7.0  date 06-01-2016 Use sepmoduleelem
!        version  6.0  date 12-08-2013 Extra parameters
!        version  5.0  date 10-06-2013 Use subroutines
!
!   copyright (c) 1993-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute basis functions and related quantities for extended
!     quadratic triangle
!     computed are:     weights for numerical integration
!                       values of derivatives of basis functions
!                       values of basis functions in integration points
!     elp624 is called by several types of element subroutines
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleconsts
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer index1(*)
      double precision coor2d(ndim,*), x(7,ndim), w(*), &
                       dphidx(7,m,ndim), &
                       xgauss(m,ndim), phi(7,m), psi(3,m), qmat(3,3,m), &
                       dpsidx(3,7,2), dphidxi(7,7,2)

!     coor2d           i    array of length 2 x number of nodal points containing
!                         the co-ordinates of the nodal points with respect to
!                         the global numbering
!                         x  = coor2d (1,i);  y  = coor2d (2,i);
!                          i                 i
!     dphidx         o    array of length 7 x m x 2  containing the derivatives
!                         of the basis functions in the sequence:
!                                      k
!                         d phi / dx (x )= dphidx (i,k,1);
!                              i
!                                      k
!                         d phi / dy (x )= dphidx (i,k,2);
!                              i
!                         if a newton cotes rule is used and m=3 then the
!                         derivatives are filled in the midside points, else
!                         if m=6 the derivatives are filled in all nodes
!     dphidxi        o    contains derivatives in reference element for gauss
!                         rule.
!                         dphidxi(i,k,l) = d phi_i/ d xi_l (x^k)
!     dpsidx         o    Array of length n x m x l  containing the derivatives
!                         of the linear basis functions in the sequence:
!                         d psi / dx (xg ) = psix (i,k,1);
!                              i        k
!                         d psi / dy (xg ) = psix (i,k,2);
!                              i        k
!                         d psi / dz (xg ) = psix (i,k,3);
!                              i        k
!     index1         i    array of length 7 containing the nodal point numbers
!                         of the element
!     phi            o    array of length 7 x m containing the values of the
!                         basis functions in the gauss points in the sequence:
!                                k
!                         phi (xg ) = phi (i,k)
!                            i
!                         array phi is only filled when ifirst = 0 and gauss
!                         integration is applied
!     psi            o    array of length 3 x m containing the values of the
!                         linear basis functions in the gauss points
!     qmat           o    transformation matrix global -> local coordinates
!                         in case of a 3D element
!     w              o    array of length m containing the weights for the
!                         numerical integration
!     x             i/o   array of length 7 x 2 containing the x- and
!                         y-coordinates of the nodal points with respect to the
!                         local numbering in the sequence:
!                         x  = x (i,1);  y  = x (i,2);
!                          i              i
!     xgauss         o    array of length m x 2 containing the co-ordinates of
!                         the gauss points.  array xgauss is only filled when
!                         gauss integration is applied.
!                         xg  = xgauss (i,1);  yg  = xgauss (i,2);
!                           i                    i
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision h, som, delta(7), e(3,2), &
                       dxdxi(7,3,3), lambda(3,7), weight(7)
      logical curved, comppsi, compqmat
      integer i, k, iweigo, intrule, iqmat
      save iweigo, iqmat, compqmat, lambda, weight

!     comppsi        if true the linear basis functions must be evaluated
!     compqmat       if treu qmat must be filled
!     curved         indication if the element is curved (true) or not (false)
!     delta          work array of length 7 to store the values of the jacobian
!                    in the nodes
!     dxdxi          work array of length 7x3x3 to store the values of dx/dxi, &
!                    dx/deta, dy/dxi and dy/deta according to:
!                             k
!                    dx/dxi(x ) = dxdxi(k,1,1)
!                             k
!                    dx/deta(x ) = dxdxi(k,1,2)
!                             k
!                    dy/dxi(x ) = dxdxi(k,2,1)
!                             k
!                    dy/deta(x ) = dxdxi(k,2,2)
!     e              work array of length 3 x 2, to store the factors e_ij
!                    see elp622straight
!     h              help variable to store a quantity temporarily
!     i              counting variable
!     intrule        Actual intergration rule
!     iqmat          Defines how qmat must be filled
!                    Possible values
!                    0: qmat must not be filled
!                    1: qmat must be filled by subroutine qclsys
!                    2: qmat must be filled assuming a linear transformation
!                       i.e. the surface elements are in a plane
!                    3: qmat must be filled assuming a general transformation
!     iweigo         if 1 the weights must be filled
!     k              counting variable
!     lambda         Array containing the linear basis functions in the
!                    integration points
!     som            help variable to compute a sum
!     weight         Contains the weights for the integration rule in the
!                    reference element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELFILLQMAT     Fill the transformation matrix qmat for the mapping of a
!                    3d surface onto a 2d surface
!     ELP622CURVED   Compute basis functions and related quantities for
!                    quadratic triangle (curved element)
!     ELP622STRDELTA Fill arrays e and dpsidx as well as Jacobian delta
!                    for a straight quadratic triangle
!     ELP624FIRST    Fill element independent quantities for
!                    extended quadratic triangle
!     ELP624STRAIGHT Compute basis functions and related quantities for
!                    extended quadratic triangle (straight element)
!     ELPCOOR        fill co-ordinates of nodes in array x
!     ELPGAUSS       Fill Gauss coordinates
!     ELPMAPTANG     Map three-dimensional array dphidx onto array dphids on
!                    surface using mapping array qmat
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL1        Print 2d real vector
!     PRINRL2        Print a real two-dimensional array
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!                              METHOD
!        to compute the derivatives of the basis functions, the jacobian
!        of the element and the weights, the following definitions are used:
!         ij        i+1    i+2    i+1
!        e   =  (-1)    ( x    - x    )   cyclic: i=1,2,3 ;   j=1,2
!                          j      j
!                    32  11    12 31
!        delta   =  e   e   - e  e
!                             ij
!        d lambda  / dx   =  e   /  delta
!                i     j
!        lamc = lambda lambda lambda
!                     1      2      3
!        phi  =  lambda (2 lambda -1)+3 lamc  i=1,2,3   (basis functions:1,2,3)
!           i          i         i
!        phi  = 4  lambda   lambda - 12 lamc   1<=i<j<=3 (basis functions:4,5,6)
!           ij           i        j
!        phi   = 27 lamc
!           123
!        grad phi  = grad lambda  ( 4 lambda  - 1 ) + 3 grad lamc
!                i              i           i
!        grad phi  = 4 ( grad lambda  lambda  + grad lambda  lambda  ) - &
!                ij                 i       j              j       i
!                   12 grad lamc
!        grad phi    = 27 grad lamc
!                123
!        grad lamc = lambda lambda grad lambda  + lambda lambda grad lambda + &
!                          2      3           1         3      1           2
!                    lambda lambda grad lambda
!                          1      2           3
!       curved boundaries:
!            |  x          y    |
!            |   xi        xi |
!        j  =|                  |
!            |  x          y    |
!            |   eta        eta |
!                         7    k
!        d x / dxi   =  sum  x    d phi / d xi
!           i      j     k=1   i        k       j
!                          -1
!        grad    phi   =  j   grad        phi
!            xy     i             xi eta    i
!                                          7       k      k
!        newton cotes rule:  / f(x) dx =  sum [ f(x ) |j(x )| w ]
!                            e            k=1                  k
!        w  = 1/15     w   = 1/40    w    = 0.225
!         i             ij            123
!
!      To compute qmat, we use the lamina coordinate system as introduced by
!      Hughes:
!
!      Define e_xi = x,xi / ||x,xi||
!      Define e_eta = x,eta / ||x,eta||
!      e_3(l) = e_xi x e_eta / ||e_xi x e_eta||
!      e_alpha = (e_xi + e_eta)/2 / || (e_xi + e_eta)/2 ||
!      e_beta = e_3(l) x e_alpha / || e_3(l) x e_alpha ||
!      e_1(l) = srqt(2)/2 ( e_alpha - e_beta )
!      e_2(l) = srqt(2)/2 ( e_alpha + e_beta )
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elp624' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elp624'
         write(irefwr,1) 'icoor, ideriv, iphi, igausp', &
                          icoor, ideriv, iphi, igausp
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iweigo = mod(iweigh,10)
      iqmat  = iweigh/10
      intrule = mod(irule,100)
      comppsi = jpsi==1

!     --- Check if we need to compute qmat

      if (iqmat==3) then

!     --- iqmat = 3, set comqmat and reset iqmat

         compqmat = .true.
         iqmat = 0

      else

!     --- iqmat # 3, set comqmat

         compqmat = .false.

      end if

      if ( ifirst==0 ) then

!     --- ifirst = 0, make some initializations
!         Set dphidxi and dphidxi in 7-th point

         call elp624first ( intrule, dphidxi, phi, psi, &
                            comppsi, weight, lambda )
         if ( ideriv>0 ) then

!        --- ideriv = 1, set grad phi equal to 0

           dphidx = 0d0

         end if

      end if

!     --- fill co-ordinates of nodes in array x

      call elpcoor ( icoor, coor2d, index1, x, inpelm, ndim, n )

      if ( (itype>=200 .and. itype<213) .or. &
           (ndim==3 .and. iqmat==0) ) then

!     --- Standard 3D case. Treat as curved

         curved = .true.

      else

!     --- 2D compute curved

         curved= .false.
         if ( abs ( x(2,1)+x(4,1)+x(6,1)-x(1,1)-x(3,1)-x(5,1)) > &
              sqreps ) then
            curved = .true.
         else if ( abs (x(2,2)+x(4,2)+x(6,2)-x(1,2)-x(3,2)-x(5,2)) > &
                   sqreps ) then
            curved = .true.
         end if

      end if

      if ( inpelm==6 .and. icoor==1 ) then

!     --- 6 node element, fill seventh point

         if ( curved ) then

!        --- curved element

            x(7,:)=(4d0*(x(2,:)+x(4,:)+x(6,:))-x(1,:)-x(3,:)-x(5,:))/9d0

         else

!        --- straight element

            h = 1d0/3d0
            x(7,:)=(x(1,:)+x(3,:)+x(5,:))*h

         end if

      end if

!     --- Fill gauss points

      call elpgauss ( ndim, intrule, x, xgauss, phi )

      if ( .not. curved ) then

!     --- curved = false,  straight triangle

         call elp624straight ( iweigo, x, index1, intrule, dphidx, &
                               dpsidx, comppsi, delta(1), lambda )

      else

!     --- curved = true,  curved triangle

         if ( comppsi .and. ideriv==1 ) &
            call elp622strdelta ( iweigo, e, x, delta, dpsidx, comppsi, &
                                  index1 )

         call elp622curved ( iweigo, x, index1, intrule, dphidx, &
                             dphidxi, dxdxi, iqmat, delta )

         if ( compqmat ) then

!        --- Fill array qmat
!            Loop over integration points

            call elfillqmat ( 1, dxdxi, m, 3, 1, qmat )
            call elpmaptang ( dphidx, qmat )  ! map grad phi onto surface

         end if

      end if  ! ( .not. curved )

      if ( iweigo==1 ) then

!     --- iweigo = 1  fill weights

         if ( .not. curved ) delta(2:m) = delta(1)
         if ( jcart==1 ) then

!        --- jcart = 1  Cartesian co-ordinates

            w(1:m) = weight(1:m)*abs(delta(1:m))

         else if ( jcart<=4 ) then

!        --- jcart = 2, 3, 4  axi-symmetric co-ordinates or polar co-ordinates

            w(1:m) = abs(delta(1:m))*xgauss(1:m,1)*weight(1:m)
            if ( jcart==2 ) w(1:m) = w(1:m)*pi2

         end if  ! ( jcart==1 )

      end if  ! ( iweigo==1 )

      if ( itype>=902 .and. itype<=903 ) then

!     --- Special case of Navier-Stokes element
!         w(1..m ) contains the weights
!         w(m+1 .. m+n) the diagonal of the mass matrix
!         In this case it is sufficient to use the integration rule available

         do i = 1, n
            som = 0d0
            do k = 1, m
               som = som+w(k)*phi(i,k)**2
            end do  ! k = 1, m
            w(m+i)= som
         end do  ! i = 1, n

      end if  ! ( itype>=902 .and. itype<=903 )

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( x, ndim, 7, 'x' )
         call prinrl1 ( xgauss, ndim, m, 'xgauss' )
         call prinrl2 ( phi, 7, m, 'phi' )
         call prinrl ( w, m, 'w' )
         call prinrl2 ( dphidx(1,1,1), 7, m, 'dphi/dx' )
         call prinrl2 ( dphidx(1,1,2), 7, m, 'dphi/dy' )
         write(irefwr,*) 'End elp624'

      end if  ! ( debug )
      call erclos ( 'elp624' )

      end
