      subroutine elp624straight ( iweigo, x, index1, intrule, dphidx, &
                                  dpsidx, comppsi, delta, lambda )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.0  date 21-08-2013
!
!   copyright (c) 2013-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute basis functions and related quantities for extended quadratic
!     triangle (straight element)
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     quadratic
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iweigo, index1(7), intrule
      double precision lambda(3,m), x(7,2), dphidx(7,m,ndim), &
                       dpsidx(3,m,2), delta
      logical comppsi

!     comppsi        i    if true the linear basis functions must be evaluated
!     delta          o    jacobian of element
!     dphidx         o    Array of length n x m x ndim  containing the
!                         derivatives
!                         of the basis functions in the sequence:
!                         d phi / dx (xg ) = dphidx (i,k,1);
!                              i        k
!                         d phi / dy (xg ) = dphidx (i,k,2);
!                              i        k
!     dpsidx         o    Array of length n x m x l  containing the derivatives
!                         of the linear basis functions in the sequence:
!                         d psi / dx (xg ) = psix (i,k,1);
!                              i        k
!                         d psi / dy (xg ) = psix (i,k,2);
!                              i        k
!     index1         i    Array of length inpelm containing the point numbers
!                         of the nodal points in the element
!     intrule        i    Actual integration rule
!     iweigo         i    if 1 the weights must be filled
!     lambda         i    Array containing the linear basis functions in the
!                         integration points
!     x              o    array of length 9 x ndim containing the x- and
!                         y-coordinates of the nodal points with respect to the
!                         local numbering in the sequence:
!                         x_i = x (i,1);  y_i = x (i,2);
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, k, j
      double precision e(3,2), h

!     e              work array of length 3 x 2, to store the factors
!                    e^ij
!     h              Help variable to store a constant temporarily
!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELP622STRDELTA Fill arrays e and dpsidx as well as Jacobian delta
!                    for a straight quadratic triangle
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL2        Print a real two-dimensional array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elp624straight' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elp624straight'
         write(irefwr,1) 'ideriv, intrule', ideriv, intrule
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill array e, dpsidx and delta

      call elp622strdelta ( iweigo, e, x, delta, dpsidx, comppsi, &
                            index1 )
      if ( debug ) then
         write(irefwr,2) 'delta', delta
         call prinrl2 ( e, 3, 2, 'e' )
      end if  ! ( debug )

      if ( ideriv==1 ) then

!     --- ideriv = 1   fill derivatives

         if ( intrule==1 .or. intrule==4 ) then

!        --- intrule  =  1, 4  newton-cotes rule

            h = 1d0/3d0
            do k = 1, 2

               dphidx(1,1,k)=3*e(1,k)
               dphidx(1,2,k)=e(1,k)+0.75d0*e(3,k)
               dphidx(1,3,k)=-e(1,k)
               dphidx(1,4,k)=-e(1,k)+0.75d0*e(1,k)
               dphidx(1,5,k)=-e(1,k)
               dphidx(1,6,k)=e(1,k)+0.75d0*e(2,k)
               dphidx(1,7,k)=e(1,k)*h
               dphidx(2,1,k)=4*e(2,k)
               dphidx(2,2,k)=-2*e(3,k)-3*e(3,k)
               dphidx(2,3,k)=4*e(1,k)
               dphidx(2,4,k)=2*e(1,k)-3*e(1,k)
               dphidx(2,5,k)=0.0d0
               dphidx(2,6,k)=2*e(2,k)-3*e(2,k)
               dphidx(2,7,k)=-4*e(3,k)*h
               dphidx(3,1,k)=-e(2,k)
               dphidx(3,2,k)=e(2,k)+0.75d0*e(3,k)
               dphidx(3,3,k)=3*e(2,k)
               dphidx(3,4,k)=e(2,k)+0.75d0*e(1,k)
               dphidx(3,5,k)=-e(2,k)
               dphidx(3,6,k)=-e(2,k)+0.75d0*e(2,k)
               dphidx(3,7,k)=e(2,k)*h
               dphidx(4,1,k)=0.0d0
               dphidx(4,2,k)=2*e(3,k)-3*e(3,k)
               dphidx(4,3,k)=4*e(3,k)
               dphidx(4,4,k)=-2*e(1,k)-3*e(1,k)
               dphidx(4,5,k)=4*e(2,k)
               dphidx(4,6,k)=2*e(2,k)-3*e(2,k)
               dphidx(4,7,k)=-4*e(1,k)*h
               dphidx(5,1,k)=-e(3,k)
               dphidx(5,2,k)=-e(3,k)+0.75d0*e(3,k)
               dphidx(5,3,k)=-e(3,k)
               dphidx(5,4,k)=e(3,k)+0.75d0*e(1,k)
               dphidx(5,5,k)=3*e(3,k)
               dphidx(5,6,k)=e(3,k)+0.75d0*e(2,k)
               dphidx(5,7,k)=e(3,k)*h
               dphidx(6,1,k)=4*e(3,k)
               dphidx(6,2,k)=2*e(3,k)-3*e(3,k)
               dphidx(6,3,k)=0.0d0
               dphidx(6,4,k)=2*e(1,k)-3*e(1,k)
               dphidx(6,5,k)=4*e(1,k)
               dphidx(6,6,k)=-2*e(2,k)-3*e(2,k)
               dphidx(6,7,k)=-4*e(2,k)*h
               dphidx(7,1,k)=0.0d0
               dphidx(7,2,k)=6.75d0*e(3,k)
               dphidx(7,3,k)=0.0d0
               dphidx(7,4,k)=6.75d0*e(1,k)
               dphidx(7,5,k)=0.0d0
               dphidx(7,6,k)=6.75d0*e(2,k)
               dphidx(7,7,k)=0.0d0

            end do

         else

!        --- intrule>1  gauss rule

            do j = 1, 2
               do k = 1, m
                  dphidx(7,k,j) = lambda(2,k)*lambda(3,k)*e(1,j) + &
                                  lambda(3,k)*lambda(1,k)*e(2,j) + &
                                  lambda(1,k)*lambda(2,k)*e(3,j)
               end do

               do i = 1, 3
                  do k = 1, m
                     dphidx(2*i-1,k,j) = e(i,j)*(4d0*lambda(i,k)-1d0)+ &
                                         3d0*dphidx(7,k,j)
                  end do
               end do
               do k = 1, m
                  dphidx(2,k,j) = 4d0*(lambda(1,k)*e(2,j)+ &
                                  lambda(2,k)*e(1,j))- &
                                  12d0*dphidx(7,k,j)
                  dphidx(4,k,j) = 4d0*(lambda(2,k)*e(3,j)+ &
                                  lambda(3,k)*e(2,j))- &
                                  12d0*dphidx(7,k,j)
                  dphidx(6,k,j) = 4d0*(lambda(3,k)*e(1,j)+ &
                                  lambda(1,k)*e(3,j))- &
                                  12d0*dphidx(7,k,j)
                  dphidx(7,k,j) = 27d0*dphidx(7,k,j)
               end do
            end do

         end if  ! ( intrule==1 .or. intrule==4 )

      end if  ! ( ideriv==1 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elp624straight'

      end if  ! ( debug )
      call erclos ( 'elp624straight' )

      end
