      subroutine elp630 ( coor2d, x, w, dphidx, xgauss, phi, index1 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 06-01-2016 Use sepmoduleelem
!        version  3.0  date 10-06-2013 Use subroutines
!        version  2.13 date 11-03-2013 Correction declaration
!
!   copyright (c) 1988-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute basis functions and related quantities for linear tetrahedron
!     Computed are:     weights for numerical integration
!                       values of derivatives of basis functions
!                       values of basis functions in integration points
!     ELP630 is called by several types of element subroutines
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     basis_function
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer index1(4)
      double precision coor2d(3,*), x(4,3), w(*), dphidx(4,m,3), &
                       xgauss(m,3), phi(4,m)

!     coor2d    o    array of length ndim x npoint containing the co-ordinates
!                  of the nodal points with respect to the global numbering
!                  x  = coor2d (1,i);  y  = coor2d (2,i);  z  = coor2d (3,i)
!                   i                 i                 i
!     dphidx  o    Array of length n x m x ndim  containing the derivatives
!                  of the basis functions in the sequence:
!                  d phi / dx (xg ) = dphidx (i,k,1);
!                       i        k
!                  d phi / dy (xg ) = dphidx (i,k,2);
!                       i        k
!                  d phi / dz (xg ) = dphidx (i,k,3);
!                       i        k
!     index1  i    Array of length inpelm containing the point numbers
!                  of the nodal points in the element
!     phi     o    Array of length 4 x m containing the values of the
!                  basis functions in the Gauss points in the sequence:
!                         k
!                  phi (xg ) = phi (i,k)
!                     i
!                  Array phi is only filled when ifirst = 0 and gauss
!                  integration is applied
!     w       o    array of length mdecl containing the weights for integration
!     x      i/o   array of length 3 x 4 containing the x- y- and
!                  z-coordinates of the nodal points with respect to the
!                  local numbering in the sequence:
!                  x  = x (i,1);  y  = x (i,2);  z  = x (i,3);
!                   i              i              i
!     xgauss  o    Double precision array of length mdecl x ndim in which the
!                  co-ordinates of the integration points are stored
!                  according to:
!                  x  = xg(i,1);  y  = xg(i,2);  z  = xg(i,3)
!                   i              i              i
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision djacob, dx, dy, dz, e(4,3), wh(10)
      integer i, k, iweigo, intrule
      save iweigo, wh

!     djacob         jacobian of element
!     dx             length of vector in x-direction
!     dy             length of vector in y-direction
!     dz             length of vector in z-direction
!     e              Work array of length 4 x 3, to store the factors
!                     ij
!                    e.        See method
!     i              Counting variable
!     intrule        Actual intergration rule
!     iweigo         if 1 the weights must be filled
!     k              Counting variable
!     wh             Contains the weights in the reference element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELP630FIRST    Fill element independent quantities for a linear
!                    tetrahedron
!     ELP630JACOB    Compute the Jacobian of a tetrahedron
!     ELPGAUSS       Fill Gauss coordinates
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!                              METHOD
!        To compute the derivatives of the basis functions, the jacobian
!        of the element and the weights, the following definitions are used:
!             ij     i+1     i
!        delta   =  x    -  x       cyclic;  i = 1,2,3,4;  j =  1,2,3
!                    j       j
!         ij        i        i+1,j+1     i+2,j+2        i+1,j+2     i+1,j+2
!        e   =  (-1)  [ delta       delta        - delta       delta        ]
!                        11  11        12  12       13  13
!        djacob = - delta   e   - delta   e  - delta   e
!                          ij
!        d phi  / dx   =  e   /  djacob
!             i     j
!                                        |djacob|      1     2     3     4
!        Newton Cotes rule:  / f(x) dx = -------- [ f(x )+f(x )+f(x )+f(x ) ]
!                            e             24
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elp630' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elp630'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      intrule = mod(irule,100)
      if ( ifirst==0 ) then

!     --- ifirst = 0  (First call)

         iweigo = mod(iweigh,10)

         call elp630first ( iweigo, phi, intrule, wh )

      end if  ! ( ifirst==0 )

!     --- Store coordinates in x

      do k = 1, ndim
         x(1:n,k) = coor2d(k,index1(1:n))
      end do

      if ( igausp==1 ) then

!     --- gauss points must be filled

         call elpgauss ( ndim, intrule, x, xgauss, phi )

      end if  ! ( igausp==1 )

      if ( iweigo==1 .or. ideriv==1 ) then

!     --- ideriv = 1  Compute derivatives or iweigo = 1: weights
!         Compute the quantities delta^(ij) ,  e^(ij) and djacob

         call elp630jacob ( e, x, djacob, 0, index1, &
                            ielem, ielgrp, itype, ideriv, n )
         if ( ierror/=0 ) go to 1000

      end if

      if ( iweigo==1 ) then

!     --- iweigo = 1  Fill weights

         w(1:m) = abs(djacob)*wh(1:m)

      end if

      if ( ideriv==1 ) then

!     --- ideriv = 1   Fill derivatives

         do k = 1, 3
            do i = 1, 4
               dphidx(i,1:m,k) = e(i,k)
            end do
         end do

      end if

      if ( interp==1 ) then

!     --- interp = 1   Interpolate

         dx = coor2d(1,1) - x(4,1)
         dy = coor2d(2,1) - x(4,2)
         dz = coor2d(3,1) - x(4,3)
         xgauss(1,1) = dphidx(1,1,1)*dx + dphidx(1,1,2)*dy + &
                       dphidx(1,1,3)*dz
         xgauss(2,1) = dphidx(2,1,1)*dx + dphidx(2,1,2)*dy + &
                       dphidx(2,1,3)*dz
         xgauss(3,1) = dphidx(3,1,1)*dx + dphidx(3,1,2)*dy + &
                       dphidx(3,1,3)*dz
         xgauss(4,1) = 1d0 - xgauss(1,1) - xgauss(2,1) - xgauss(3,1)

      end if

      if ( itype>=902 .and. itype<=903 ) then

!     --- Special case of Navier-Stokes element
!         w(1..m ) contains the weights
!         w(m+1 .. m+n) the diagonal of the mass matrix
!         In this case it is sufficient to use the integration rule available

         do i = 1, n
            w(m+i)= sum(w(1:m)*phi(i,1:m)**2)
         end do  ! i = 1, n

      end if  ! ( itype>=902 .and. itype<=903 )

1000  call erclos ( 'elp630' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elp630'

      end if  ! ( debug )

      end
