      subroutine elp630jacob ( e, x, djacob, isubr, index1, &
                               ielem, ielgrp, itype, ideriv, n )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-06-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the Jacobian of a tetrahedron
!     Also the factors eij, necessary for the derivatives are computed
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!     determinant
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduledebug
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isubr, n, index1(n), ielem, ielgrp, itype, ideriv
      double precision e(n,3), x(n,3), djacob

!     djacob         o    jacobian of element
!     e              o    array of length nx3, to store the factors e^ij
!     ideriv         i    if 1 e must be subdivided by djacb to get the
!                         derivatives
!     ielem          i    Element number
!     ielgrp         i    element group number
!     index1         i    Array of length inpelm containing the point numbers
!                         of the nodal points in the element
!     isubr          i    Indicates the type of calling subroutine
!     itype          i    Table containing information about the numbers behind
!                         the keywords in the table code
!     n              i    number of basis functions for each component
!     x              i    array with the coodinates of the element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer k, ind(5), ind1(4), iven, i, ni
      double precision delta(4,3)

!     delta          Work array of length 4 x 3, to store the factors
!                         ij
!                    delta.    See method
!     i              Counting variable
!     ind            Integer array containing the cyclic permutation of
!                    1,2,3,4
!     ind1           Array of length 4 to store the cyclic permutation of
!                    1 - 3
!     iven           help parameter to compute (-1)
!     k              Counting variable
!     ni             Upper boundary in the computation for the loop
!                    variable i
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL2        Print a real two-dimensional array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     105   Jacobian of element is zero
!    1540   Jacobian zero or changing sign
! **********************************************************************
!
!                       PSEUDO CODE
!
!        To compute the derivatives of the basis functions, the jacobian
!        of the element and the weights, the following definitions are used:
!             ij     i+1     i
!        delta   =  x    -  x       cyclic;  i = 1,2,3,4;  j =  1,2,3
!                    j       j
!         ij        i        i+1,j+1     i+2,j+2        i+1,j+2     i+1,j+2
!        e   =  (-1)  [ delta       delta        - delta       delta        ]
!                        11  11        12  12       13  13
!        djacob = - delta   e   - delta   e  - delta   e
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data ind / 2,3,4,1,2 /, ind1 / 2,3,1,2 /
! ======================================================================
!
      call eropen ( 'elp630jacob' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elp630jacob'

      end if
      if ( ierror/=0 ) go to 1000

!     --- Compute the quantities delta^(ij)

      do i = 1, 4
         delta(i,1:3) = x(ind(i),1:3) - x(i,1:3)
      end do

      if ( ideriv==1 ) then

!     --- ideriv = 1,  compute all factors e^(ij)

         ni = 4

      else

!     --- ideriv = 0,  compute only the factors e^(1j)

         ni = 1

      end if

!     --- Compute the quantities e^(ij)

      iven = -1
      do i = 1, ni
         do k = 1, 3
            e (i,k) = iven * ( delta ( ind(i), ind1(k) ) * &
                      delta ( ind(i+1), ind1(k+1) ) - &
                      delta ( ind(i), ind1(k+1) ) * &
                      delta ( ind(i+1), ind1(k) ) )
         end do
         iven = - iven

      end do

      djacob = -delta(1,1)*e(1,1)-delta(1,2)*e(1,2)-delta(1,3)*e(1,3)

      if ( checksign .and. djacob<0d0 ) then
         write(irefwr,*) 'area of element ', ielem, ' is negative '
         write(irefwr,*) 'area = ', djacob
         call prinin ( index1, n, 'nodes' )
         call prinrl2 ( x, n, 3, 'coordinates' )

      end if

!     --- Check if delta is non-zero

      if ( abs(djacob)<=0d0 ) then

         if ( isubr==0 ) then
            call errint ( ielem, 1 )
            call errint ( itype, 2 )
            call errint ( ielgrp, 3 )
            call errsub ( 105, 3, 0, 0 )
         else
            call errint ( ielgrp, 1 )
            call errint ( ielem, 2 )
            call errchr ( 'volume', 1 )
            call errsub ( 1540, 2, 0, 1 )
         end if
         call prinin(index1,n,'node numbers')
         call prinrl2(x,n,3,'co-ordinates')

      end if

      if ( ideriv==1 ) then

!     --- ideriv = 1,  subdivide e by djacob

         e = e /djacob

      end if  ! ( ideriv==1 )

1000  call erclos ( 'elp630jacob' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elp630jacob'

      end if

      end
