      subroutine elp631 ( coor2d, x, w, dphidx, xgauss, phi, index1, &
                          psi, dpsidx, phiksi )
! ======================================================================
!
!        programmer    Guus Segal
!        version  6.0  date 06-01-2016 Use sepmoduleelem
!        version  5.0  date 10-06-2013 Use subroutines
!        version  4.6  date 30-09-2010 Debug statements
!
!   copyright (c) 1988-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute basis functions and related quantities for quadratic tetrahedron
!     compared to the standard SEPRAN numbering.
!     Computed are:     weights for numerical integration
!                       values of derivatives of basis functions
!                       values of basis functions in integration points
!     ELP631 is called by several types of element subroutines
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     3d
!     quadratic
!     tetrahedron
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer index1(10)
      double precision coor2d(3,*), x(n,3), w(*), dphidx(n,m,3), &
                       xgauss(m,3), phi(10,m), psi(4,m), dpsidx(4,3), &
                       phiksi(n,m,3)

!     coor2d           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     dphidx         o    Array of length n x m x ndim  containing the
!                         derivative
!                         of the basis functions in the sequence:
!                         d phi / dx (xg ) = dphidx (i,k,1);
!                              i        k
!                         d phi / dy (xg ) = dphidx (i,k,2);
!                              i        k
!                         d phi / dz (xg ) = dphidx (i,k,3);
!                              i        k
!                         If the element is a linear simplex, the
!                         derivatives are constant and only k = 1 is filled.
!     dpsidx         o    Array of length n x m x l  containing the derivatives
!                         of the linear basis functions in the sequence:
!                         d psi / dx (xg ) = psix (i,k,1);
!                              i        k
!                         d psi / dy (xg ) = psix (i,k,2);
!                              i        k
!                         d psi / dz (xg ) = psix (i,k,3);
!                              i        k
!     index1         i    Array of length inpelm containing the point numbers
!                         of the nodal points in the element
!     phi            o    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     phiksi        i/o   array of length n x m to store the values of dphi/dxsi
!                         according to:
!                                     k
!                         dphi /dxsi(x ) = phiksi(i,k)
!                             i
!     psi            o    array of length 4 x m containing the values of the
!                         linear basis functions in the gauss points
!     w              i    array of length m containing the weights for
!                         integration
!     x             i/o   array of length 3 x 10 containing the x- y- and
!                         z-coordinates of the nodal points with respect to the
!                         local numbering in the sequence:
!                         x  = x (i,1);  y  = x (i,2);  z  = x (i,3);
!                          i              i              i
!     xgauss         o    array of length m x ndim containing the co-ordinates
!                         of the gauss points.  array xgauss is only filled when
!                         gauss integration is applied.
!                         xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                         zg_i = xgauss (i,3)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision alambd(4,15), wh(15), psiksi(4,3)
      integer k, intern(10), idx(4), i, &
              imidp(6,2), iweigo, intrule, interpol
      logical comppsi, curved
      save intern, imidp, alambd, iweigo, wh, comppsi, psiksi, intrule

!     alambd         Array containing the values of lambda in the integration
!                    points  alambd(i,k) = lambda (xk)
!                                                i
!     comppsi        if true the linear basis functions must be evaluated
!     curved         indication if the element is curved (true) or not
!                    (false)
!     i              Counting variable
!     idx            Array containing the sequence of the vertices
!     imidp          Contains the i,j numbering of the mid side points in the
!                    sequence of intern
!                    So basis function
!                    phi(intern(4+i)) = 4 lambda(imidp(i,1)) lambda(imidp(i,2))
!     intern         Internal numbering of the nodes, vertices first
!     interpol       Indicates how the diagonal of the mass matrix must be
!                    computed
!                    0,1  A Newton Cotes rule is applied
!                    2,3  The standard integration rule is used
!     intrule        Actual intergration rule
!     iweigo         if 1 the weights must be filled
!     k              Counting variable
!     psiksi         Array of length 8 x m x 3 containing the values of the
!                    derivatives of the basis functions in the Gauss points in
!                    the reference element
!     wh             Contains the weights in the reference element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      logical elp631cur

!     ELP631CUR      Check if quadratic tetrahedron is curved
!     ELP631CURVED   Compute derivatives of basis functions for quadratic
!                    tetrahedron. Curved element
!     ELP631FIRST    Fill element independent quantities for a linear
!                    tetrahedron
!     ELP631STRAIGHT Compute derivatives of basis functions for quadratic
!                    tetrahedron. Straight element
!     ELPGAUSS       Fill Gauss coordinates
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!                              METHOD
!        To compute the derivatives of the basis functions, the jacobian
!        of the element and the weights, the following definitions are used:
!             ij     i+1     i
!        delta   =  x    -  x       cyclic;  i = 1,2,3,4;  j =  1,2,3
!                    j       j
!         ij        i        i+1,j+1     i+2,j+2        i+1,j+2     i+2,j+1
!        e   =  (-1)  [ delta       delta        - delta       delta        ]
!                        11  11        12  12       13  13
!        djacob = - delta   e   - delta   e  - delta   e
!                             ij
!        d lambda  / dx   =  e   /  djacob
!                i     j
!        lambda ( x,y,z ) is linear basis function corresponding to
!                         4-point tetrahedron
!        phi    = lambda  ( 2 lambda  - 1 )   i = 1,2,3,4  (vertices)
!           i            i           i
!        phi   = 4 lambda  lambda     1<=i<j<=4   (midside points)
!           ij           i       j
!        grad phi  = grad lambda  ( 4 lambda  - 1 )   i = 1,2,3,4  (vertices)
!                i               i           i
!        grad phi   = 4 [ grad(lambda )lambda  + grad(lambda )lambda ]
!                ij                  i       j              j       i
!                  1<=i<j<=4   (midside points)
!                                        |djacob|      1     2     3     4
!        Newton Cotes rule:  / f(x) dx = -------- [-f(x )-f(x )-f(x )-f(x ) ]
!                            e             120
!            |djacob|      5     6     7     8     9     10
!        + 4 -------- [ f(x )+f(x )+f(x )+f(x )+f(x )+f(x  ) ]
!              120
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data intern / 1, 3, 5, 10, 2, 6, 7, 4, 8, 9 /, &
           imidp  / 1, 1, 1, 2, 2, 3, &
                    2, 3, 4, 3, 4, 4 /
! ======================================================================
!
      call eropen ( 'elp631' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elp631'
         write(irefwr,*) 'irule, jdiag, iweigh', irule, jdiag, iweigh
         call prinin ( index1, n, 'index1' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( index1(10)==index1(1) ) then

!     --- Special case with degenerated elements for plaxis
!         Jump

         w(1:m) = 0d0
         go to 1000

      end if  ! ( index1(10)==-100 )

      interpol = irule-100*intrule

      if ( ifirst==0 ) then

!     --- ifirst = 0  (First call)

         call elp631first ( iweigo, phi, intrule, wh, intern, imidp, &
                            alambd, phiksi, comppsi, psi, psiksi )

      end if  ! ( ifirst==0 )

!     --- Store coordinates in x

      do k = 1, ndim
         x(1:n,k) = coor2d(k,index1(1:n))
      end do

!     --- Fill gauss points

      call elpgauss ( ndim, intrule, x, xgauss, phi )

      if ( iweigo==1 .or. ideriv==1 ) then

!     --- Check if element is straight or curved

         curved = elp631cur ( x, n, intern, imidp, index1 )
         if ( debug ) write(irefwr,*) 'curved ', curved

         if ( curved ) then

!        --- curved = true,  curved tetrahedron

            do i = 1, 4
               idx(i) = i
            end do  ! i = 1, 4
            call elp631curved ( x, index1, w, wh, comppsi, dpsidx, &
                                dphidx, psiksi, phiksi, idx, 1 )

         else

!        --- curved = false,  straight tetrahedron

            call elp631straight ( x, intern, imidp, index1, iweigo, w, &
                                  wh, comppsi, dpsidx, dphidx, alambd )

         end if  ! ( curved )

      end if  ! ( iweigo==1 .or. ideriv==1 )

      if ( itype>=902 .and. itype<=903 ) then

!     --- Special case of Navier-Stokes element
!         w(1..m ) contains the weights
!         w(m+1 .. m+n) the diagonal of the mass matrix
!         In this case it is sufficient to use the integration rule available

         if ( debug ) then
            write(irefwr,1) 'interpol', interpol
            call prinrl ( w, m, 'w/2' )
         end if  ! ( debug )
         if ( interpol>1 ) then

!        --- interpol = 2, 3: Use diagonal correpsponding to integration rule

            do k = 1, n
               w(m+k)= sum(w(1:m)*phi(k,1:m)**2)
            end do  ! k = 1, n

         else

!        --- interpol = 0, 1: Use Newton Cotes diagonal

            w(m+1:2*m)= w(1:m)

         end if  ! ( interpol>1 ) then
         if ( debug ) call prinrl ( w, m+n, 'w/3' )

      end if  ! ( itype>=902 .and. itype<=903 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elp631'

      end if  ! ( debug )
      call erclos ( 'elp631' )

      end
