      function elp631cur ( x, nloc, intern, imidp, index1 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 19-06-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if quadratic tetrahedron is curved
!
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     3d
!     quadratic
!     tetrahedron
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nloc, intern(10), imidp(6,2), index1(10)
      double precision x(nloc,3)
      logical elp631cur

!     elp631cur      o    If true the element is curved
!     imidp          i    Contains the i,j numbering of the mid side points in
!                         the sequence of intern
!                         So basis function
!                         phi(intern(4+i)) = 4 lambda(imidp(i,1))
!                                              lambda(imidp(i,2))
!     index1         i    Array of length inpelm containing the point numbers
!                         of the nodal points in the element
!     intern         i    Internal numbering of the nodes, vertices first
!     n              i    number of basis functions for each component
!     x              i    array of length 3 x 10 containing the x- y- and
!                         z-coordinates of the nodal points with respect to the
!                         local numbering in the sequence:
!                         x  = x (i,1);  y  = x (i,2);  z  = x (i,3);
!                          i              i              i
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, i1, i2
      double precision xmid(3), eps, dist
      logical curved

!     curved         indication if the element is curved (true) or not
!                    (false)
!     dist           Distance between mid point and centre of line
!     eps            Local accuracy
!     i              Counting variable
!     i1             Local sequence number of first end node
!     i2             Local sequence number of first end node
!     xmid           Contains difference between mid point and centre of line
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRININ2        Print an integer two-dimensional array
!     PRINRL         Print 1d real vector
!     PRINRL2        Print a real two-dimensional array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3145   curvature of boundary too large
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elp631cur' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elp631cur'
         call prinin ( intern, 10, 'intern' )
         call prinin2 ( imidp, 6, 2, 'imidp' )
         call prinrl2 ( x, nloc, 3, 'x' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      curved = .false.
      eps = (maxval(abs(x))*sqreps)**2

      if ( debug ) write(irefwr,2) 'eps', eps

      do i = 1, 6

         i1 = intern(imidp(i,1))
         i2 = intern(imidp(i,2))
         xmid(:) = 0.5d0*(x(i1,:)+x(i2,:))-x(intern(i+4),:)

         if ( debug ) then
            write(irefwr,1) 'i, i1, i2, imid', i, i1, i2, intern(i+4)
            call prinrl ( xmid, 3, 'xmid' )
         end if  ! ( debug )

         dist = dot_product(xmid,xmid)
         if ( dist>eps ) then

!        --- mid point not in the middle of vertices
!            check if mid-side points nodes are close to the centre of the line

            curved = .true.

            if ( dist>1d11*eps ) then

!           --- error distance>1d11*eps

               call errint ( index1(intern(i+4)), 1 )
               call errint ( index1(i1), 2 )
               call errint ( index1(i2), 3 )
               call errint ( ielem, 4 )
               call erreal ( dist, 1 )
               call erreal ( 1d11*eps, 2 )
               call errsub ( 3145, 4, 2, 0 )
               call prinin ( index1, 10, 'node numbers')
               call prinrl ( xmid, 3, 'coordinates of diff vector' )

            end if  ! ( dist>1d3*eps )

            exit  ! jump out of loop

         end if  ! ( dot_product(xmid,xmid)>eps )

      end do  ! i = 1, 6

      elp631cur = curved

1000  call erclos ( 'elp631cur' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'curved ', curved
         write(irefwr,*) 'End elp631cur'

      end if  ! ( debug )

      end
