      subroutine elp631first ( iweigo, phi, intrule, wh, intern, imidp, &
                               alambd, phiksi, comppsi, psi, psiksi )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 27-05-2014 Change integeration rules
!        version  1.0  date 18-06-2013
!
!   copyright (c) 2013-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element independent quantities for a quadratic tetrahedron
!
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     basis_function
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iweigo, intrule, intern(10), imidp(6,2)
      double precision phi(n,m), wh(m), alambd(4,m), phiksi(n,m,3), &
                       psi(4,m), psiksi(4,3)
      logical comppsi

!     alambd         o    Contains values of the linar basis functions
!                         in the integration points
!     comppsi        o    if true the linear basis functions must be evaluated
!     imidp          i    Contains the i,j numbering of the mid side points in
!                         the sequence of intern
!                         So basis function
!                         phi(intern(4+i)) = 4 lambda(imidp(i,1))
!                                              lambda(imidp(i,2))
!     intern         i    Internal numbering of the nodes, vertices first
!     intrule        o    Actual integration rule
!     iweigo         o    if 1 the weights must be filled
!     phi            o    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     phiksi         o    array of length n x m to store the values of dphi/dxsi
!                         according to:
!                                     k
!                         dphi /dxsi(x ) = phiksi(i,k)
!                             i
!     psi            o    Array of length n x m containing the values of the
!                         modified basis functions in the integration points
!     psiksi         o    Array of length 8 x m x 3 containing the values of the
!                         derivatives of the basis functions in the Gauss points
!                         in the reference element
!     wh             o    Contains the weights for the integration in the
!                         reference element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, i1, i2
      double precision h
      character (len=21) text

!     h              Help variable to store a constant temporarily
!     i              Counting variable
!     i1             help parameter in computation
!     i2             help parameter in computation
!     j              Counting variable
!     k              Counting variable
!     text           String variable containing some text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRORELVAR     Print error message errornr for element: wrong value of
!                    parameter
!     ERRORELVAR2    Print error message errornr for element: wrong value of
!                    2 parameters
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The linear basis functions in the reference element are defined as
!     lambda(1) = 1 - xi - eta - zeta
!     lambda(2) = xi
!     lambda(3) = eta
!     lambda(4) = zeta
!     The quadratic basis functions are defined as
!     phi_i = lambda(i) ( 2 lambda(i) -1 ):   vertices
!     phi_ij = 4 lambda(i) lambda(j):         mid-side nodes
!     hence the derivatives:
!     grad(phi_i) = grad(lambda(i)) ( 4 lambda(i) -1 )
!     grad(phi_ij) = grad(lambda(i)) lambda(j) + grad(lambda(j)) lambda(i)
!     The derivatives in the reference element for the lambda's are
!                  xi    eta   zeta
!     lambda(1)    -1    -1     -1
!     lambda(2)     1     0      0
!     lambda(3)     0     1      0
!     lambda(4)     0     0      1
!     The sequence numbers of the vertices are:  1 3 5 10
!     and for the mid points we have
!     2:  1,2
!     4:  2,3
!     6:  1,3
!     7:  1,4
!     8:  2,4
!     9:  3,4
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data text / 'Quadratic tetrahedron' /
! ======================================================================
!
      call eropen ( 'elp631first' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elp631first'
         write(irefwr,1) 'irule', irule
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iweigo = mod(iweigh,10)
      comppsi = jpsi==1
      intrule = mod(irule,100)
      if ( debug ) write(irefwr,1) 'intrule, m', intrule, m

      if ( jcart/=1 ) then

!     --- Error 1769: Co-ordinate system not available

         call errorelvar ( jcart-1, 1769, text, 0 )

      end if
      if ( intrule<1 .or. intrule>4 ) then

!     --- Error 1770: Integration rule not available

         call errorelvar ( intrule, 1770, text, 1 )
         intrule = 1

      end if
      if ( ( intrule==1 .and. m/=10 ) .or. &
           ( intrule>=2 .and. intrule<=3 .and. m/=11 ) .or. &
           ( intrule==4 .and. m/=15 ) ) then

!     --- Error 1424: m and integration rule do not match

         call errorelvar2 ( m, intrule, 1424, text, 0 )
         go to 1000

      end if

!     --- Fill array lambda, phi

      select case ( intrule )

      case(1)

!     --- intrule = 1  Newton Cotes rule

         alambd = 0d0
         do i = 1, 4
            alambd(i,intern(i)) = 1d0
            do k = 1, 6
               if ( imidp(k,1)==i .or. imidp(k,2)==i ) then
                  alambd(i,intern(k+4)) = 0.5d0
               end if
            end do  ! k = 1, 6
         end do  ! i = 1, 4

         phi = 0d0
         do i = 1, 10
            phi(i,i) = 1d0
         end do

         h = 1d0/120d0
         wh(intern(1:4)) = -h
         h = 4d0*h
         wh(intern(5:10)) = h

      case(2, 3)

!     --- 11 point Gauss rule; just copy the lambdas

         alambd(1:4,1:m) = alam9(1:4,1:m)

         wh = wgs9

      case(4)

!     --- 15 point Gauss rule; just copy the lambdas

         alambd(1:4,1:m) = alam10(1:4,1:m)
         wh = wgs10

      end select  ! case ( intrule )

      if ( intrule>1 ) then

!     --- Fill array phi_i

         do i = 1, 4
            phi(intern(i),1:m) = alambd(i,1:m)*(2d0*alambd(i,1:m)-1d0)
         end do  ! i = 1, 4
         do i = 1, 6
            i1 = imidp(i,1)
            i2 = imidp(i,2)
            phi(intern(i+4),1:m) = 4d0*alambd(i1,1:m)*alambd(i2,1:m)
         end do  ! i = 1, 6

      end if  ! ( intrule>1 )

      if ( comppsi ) then

!     --- Compute linear basis functions

         psi(1:4,1:m) = alambd(1:4,1:m)

      end if  ! ( comppsi )

!     --- Set igausp to 1 in case of integration rules larger than 1

      if ( intrule>1 .and. iweigo==1 ) igausp = 1

!     --- fill d phi / d xsi,  d phi / d eta and dphi d zeta
!         first d lambda/ dxi

      psiksi(1,1:3) = -1d0
      psiksi(2:4,1:3) = 0d0
      do i = 1, 3
         psiksi(i+1,i) = 1d0
      end do  ! i = 1, 3

      do i = 1, 4
         do k = 1, m
            h = 4d0*alambd(i,k)-1d0
            phiksi(intern(i),k,1:3) = h*psiksi(i,1:3)
         end do  ! k = 1, m
      end do  ! i = 1, 4
      do i = 1, 6
         i1 = imidp(i,1)
         i2 = imidp(i,2)
         do k = 1, m
            do j = 1,3
               h = alambd(i1,k)*psiksi(i2,j)+ &
                   alambd(i2,k)*psiksi(i1,j)
               phiksi(intern(i+4),k,j) = 4d0*h
            end do  ! j = 1,3
         end do  ! k = 1, m
      end do  ! i = 1, 6

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elp631first'

      end if  ! ( debug )
      call erclos ( 'elp631first' )

      end
