      subroutine elp636 ( coor2d, x, w, dphidx, xgauss, phi, index1, &
                          psi )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-01-2016 Use sepmoduleelem
!        version  2.0  date 10-06-2013 Use subroutines
!        version  1.6  date 09-03-2013 Correction declaration
!
!   copyright (c) 1996-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute basis functions and related quantities for extended
!     trilinear brick
!     Computed are:     weights for numerical integration
!                       values of derivatives of basis functions
!                       values of basis functions in integration points
!     ELP636 is called by several types of element subroutines
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer index1(9)
      double precision coor2d(3,*), x(9,3), w(m), dphidx(9,m,3), &
                       xgauss(m,3), phi(9,m), psi(8,m)

!     coor2d     i     array of length 3 x number of nodal points containing
!                    the co-ordinates of the nodal points with respect to
!                    the global numbering
!                    x  = coor2d (1,i);  y  = coor2d (2,i);  z  = coor2d (3,i)
!                     i                 i                 i
!     dphidx   o     Array of length 9 x m x 3  containing the derivatives
!                    of the basis functions in the sequence:
!                    d phi / dx (xg ) = dphidx (i,k,1);
!                         i        k
!                    d phi / dy (xg ) = dphidx (i,k,2);
!                         i        k
!                    d phi / dz (xg ) = dphidx (i,k,3);
!                         i        k
!     index1   i     Array of length 9 containing the nodal point numbers
!                    of the element
!     phi      o     Array of length 9 x m containing the values of the
!                    basis functions in the Gauss points in the sequence:
!                    phi (xg ) = phi (i,k)
!                       i   k
!                    Array phi is only filled when ifirst = 0 and gauss
!                    integration is applied
!     psi      o     array of length 8 x m containing the values of the linear
!                    basis functions in the gauss points in the sequence:
!                           k
!                    psi (xg ) = psi (i,k)
!                       i
!                    array psi is only filled when ifirst = 0 and gauss
!                    integration is applied
!     w        o     array of length m containing the weights for the
!                    numerical integration
!                    During the computation w is temporarily used to store
!                    the Jacobian in the integration points, so even
!                    if there is no need to compute the weights for the
!                    integration, still w should be declared
!     x       i/o    array of length 3 x 8 containing the x- y- and
!                    z-coordinates of the nodal points with respect to the
!                    local numbering in the sequence:
!                    x  = x (i,1);  y  = x (i,2);  z  = x (i,3);
!                     i              i              i
!     xgauss   o     Array of length m x 3 containing the co-ordinates of
!                    the gauss points.  Array xgauss is only filled when Gauss
!                    integration is applied.
!                    xg  = xgauss (i,1);  yg  = xgauss (i,2);
!                      i                    i
!                    zg  = xgauss (i,3);
!                      i
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer indi(8,3), inds(8,3), ih, ih1, i, k, mrule, m2, isign, &
              jsign, j, iweigo, intrule
      double precision weight(8), theta(2,2), hc(3), h, dpdxic, phic, &
                       som, dxdksi(9,3,3), detinv(9,3,3), &
                       dtetds(2), phiksi(9,8,3)
      logical check
      character (len=25) text
      save weight, iweigo, dtetds, theta, phiksi

!     check          Logical variable that is used to indicate if the Jacobian
!                    is zero (true) or not (false)
!     detinv         Work array of length m x 3 x 3, to store the inverse
!                    of the jacobian matrix as follows:
!                     -1    k
!                    J   ( x ) =  detinv ( k, i, j )
!                     ij
!     dpdxic         Value of dphi_c/dxi
!     dtetds         Derivatives of theta in the integration points
!     dxdksi         Work array of length 9 x 3 x 3, to store the factors
!                      i             k
!                    dx  / dalpha  (x )  as follows.
!                                l
!                                    k
!                    dx  / dalpha  (x ) = dxdksi(k, j, i )
!                      i         j
!     h              Help variable to store a quantity temporarily
!     hc             Help array to store a quantity temporarily
!     i              Counting variable
!     ih             Help variable to store a constant temporarily
!     ih1            Help variable to store a constant temporarily
!     indi           Array in which the triples indi, indj and indk are stored
!     inds           Array in which the triples inds1, inds2 and inds3
!                    are stored
!     intrule        Actual intergration rule
!     isign          Help parameters to check the sign of the Jacobian in the
!                    integration points
!     iweigo         if 1 the weights must be filled
!     j              Counting variable
!     jsign          Help parameter to compute sign of the Jacobian
!     k              Counting variable
!     m2             ?
!     mrule          Type of integration rule as given by the user
!     phic           Value of phi_c
!     phiksi         Array of length 9 x m x 3 containing the values of the
!                    derivatives of the basis functions in the Gauss points in
!                    the reference element.
!                    Array phiksi is only filled when ifirst = 0 and must
!                    be kept by the program
!     som            Help variable to compute a sum
!     text           String variable containing some text
!     theta          1D basis funtion in integration points
!     weight         Help variable to compute the weights
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision ddot

!     DDOT           (CDOT)     Inner product of two vectors
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRORELVAR     Print error message errornr for element: wrong value of
!                    parameter
!     ERRORELVAR2    Print error message errornr for element: wrong value of
!                    2 parameters
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     105   Jacobian of element is zero
!     230   Jacobian of element is zero or has changing sign
!    1540   Jacobian zero or changing sign
! **********************************************************************
!
!                       PSEUDO CODE
!
!        To compute the derivatives of the basis functions, the jacobian
!        of the element and the weights, the following definitions are used:
!         -1
!        J   = ( dx   / dalpha     dx   / dalpha    - &
!         ij       i+1        i+1    i+2        i+2
!                dx   / dalpha     dx   / dalpha    ) / det(J)
!                  i+2        i+1    i+1        i+2
!               cyclic;  i = 1,2,3;  j =  1,2,3
!                   3
!        det (J) = sum   dx  / dalpha   ( dx   / dalpha     dx   / dalpha    - &
!                  i=1     1         i      i+1        i+1    i+2        i+2
!                                         dx   / dalpha     dx   / dalpha    )
!                                           i+2        i+1    i+1        i+2
!                        k     3    -1   k                       k
!        d phi  / d x  (x ) = sum  J   (x )  d phi  / d alpha  (x )
!             i      l        j=1   lj            i          j
!        For the Newton-Cotes rule we have the following expressions
!        to compute dxdksi and dphidx:
!                        2k-1                    2k
!        dx  / dalpha  (x    ) = dx  / dalpha  (x  ) = &
!          i         j             i         j
!         ialpha(2,k,j)      ialpha(1,k,j)
!        x               -  x
!         i                  i
!                                         ialpha(1,i,l)
!         d phi              / d alpha  (x             )  =  - 1
!              ialpha(1,i,l)          l
!                                         ialpha(2,i,l)
!         d phi              / d alpha  (x             )  =  - 1
!              ialpha(1,i,l)          l
!                                         ialpha(1,i,l)
!         d phi              / d alpha  (x             )  =  1
!              ialpha(2,i,l)          l
!                                         ialpha(2,i,l)
!         d phi              / d alpha  (x             )  =  1
!              ialpha(2,i,l)          l
!         Therefore the following algorithm may be used to compute dphidx:
!         - Set dphidx equal to zero
!         - Summation:  i = 1 (1) 4; j = 1 (1) 3; l = 1 (1) 3
!           d phi                d phi
!                ialpha(1,i,l)        ialpha(1,i,l)    -1      ialpha(1,i,l)
!           ------------------ = ------------------ - J      (x             )
!           d x                  d x                   jl
!              j                    j
!           d phi                d phi
!                ialpha(1,i,l)        ialpha(1,i,l)    -1      ialpha(2,i,l)
!           ------------------ = ------------------ - J      (x             )
!           d x                  d x                   jl
!              j                    j
!           d phi                d phi
!                ialpha(2,i,l)        ialpha(2,i,l)    -1      ialpha(1,i,l)
!           ------------------ = ------------------ + J      (x             )
!           d x                  d x                   jl
!              j                    j
!           d phi                d phi
!                ialpha(2,i,l)        ialpha(2,i,l)    -1      ialpha(2,i,l)
!           ------------------ = ------------------ + J      (x             )
!           d x                  d x                   jl
!              j                    j
!    The basis functions at the standard unit square are defined by:
!    phi_1 = lambda_1(xi) lambda_1(eta) lambda_1(zeta) - 8 phi_c
!    phi_2 = lambda_2(xi) lambda_1(eta) lambda_1(zeta) - 8 phi_c
!    phi_3 = lambda_2(xi) lambda_2(eta) lambda_1(zeta) - 8 phi_c
!    phi_4 = lambda_1(xi) lambda_2(eta) lambda_1(zeta) - 8 phi_c
!    phi_5 = lambda_1(xi) lambda_1(eta) lambda_2(zeta) - 8 phi_c
!    phi_6 = lambda_2(xi) lambda_1(eta) lambda_2(zeta) - 8 phi_c
!    phi_7 = lambda_2(xi) lambda_2(eta) lambda_2(zeta) - 8 phi_c
!    phi_8 = lambda_1(xi) lambda_2(eta) lambda_2(zeta) - 8 phi_c
!    phi_5 = 64 phi_c
!    phi_c = lambda_1(xi) lambda_2(xi)
!            lambda_1(eta) lambda_2(eta)
!            lambda_1(zeta) lambda_2(zeta)
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data text / 'Extendend Trilinear brick' /
! ======================================================================
!
      call eropen ( 'elp636' )
      intrule = mod(irule,100)
      if ( ifirst==0 ) then

!     --- ifirst = 0  (First call)

         iweigo = mod(iweigh,10)
         if ( jcart/=1 ) then

!        --- Error 1769: Co-ordinate system not available

            call errorelvar ( jcart-1, 1769, text, 0 )

         end if
         if ( intrule<3 .or. intrule>3 ) then

!        --- Error 1770: Integration rule not available

            call errorelvar ( intrule, 1770, text, 1 )
            intrule = 3

         end if
         if ( intrule==3 .and. m/=8 ) then

!        --- Error 1424: m and integration rule do not match

            call errorelvar2 ( m, intrule, 1424, text, 0 )

         end if

!        --- Fill arrays theta and dtetds in integration points

         dtetds(1) = -1d0
         dtetds(2) =  1d0
         do k = 1, 2
            theta(1,k) = 1d0 - alam4(k)
            theta(2,k) = alam4(k)
            w(k) = 0.5d0
         end do
         mrule = 2
         m2 = mrule**2

!        --- fill array indi

         indi(1,1) = 1
         indi(2,1) = 2
         indi(3,1) = 2
         indi(4,1) = 1
         indi(1,2) = 1
         indi(2,2) = 1
         indi(3,2) = 2
         indi(4,2) = 2
         do k =1, 4
            indi(k,3) = 1
            indi(k+4,1) = indi(k,1)
            indi(k+4,2) = indi(k,2)
            indi(k+4,3) = 2
         end do

!        --- fill array inds

         if ( intrule==1 ) then
            do k = 1, 8
               do j = 1, 3
                  inds(k,j) = indi(k,j)
               end do
            end do
         else if ( intrule==2 ) then
            do j = 1, 3
               inds(1,j) = 1
            end do
         else
            do k = 1, mrule
               ih = (k-1)*m2
               do j = 1, mrule
                  ih1 = ih+(j-1)*mrule
                  do i = 1, mrule
                     inds(i+ih1,1) = i
                     inds(i+ih1,2) = j
                     inds(i+ih1,3) = k
                  end do
               end do
            end do
         end if

!        --- Fill array weight

         do k = 1, m
            weight(k) = w(inds(k,1))*w(inds(k,2))*w(inds(k,3))
         end do

         if ( iphi==1 ) then

!        --- fill phi

            do k = 1, m
               phic = theta(1,inds(k,1))*theta(2,inds(k,1))* &
                      theta(1,inds(k,2))*theta(2,inds(k,2))* &
                      theta(1,inds(k,3))*theta(2,inds(k,3))
               h = 8d0*phic
               do i = 1, 8
                  psi(i,k) = theta(indi(i,1),inds(k,1))* &
                             theta(indi(i,2),inds(k,2))* &
                             theta(indi(i,3),inds(k,3))
                  phi(i,k) = psi(i,k) - h
               end do
               phi(9,k) = 8d0*h
            end do

         end if

!        --- fill phiksi

          do k = 1, m
             dpdxic = (2d0*theta(1,inds(k,1))-1d0)* &
                      theta(1,inds(k,2))*theta(2,inds(k,2))* &
                      theta(1,inds(k,3))*theta(2,inds(k,3))
             hc(1) = 8d0*dpdxic
             phiksi(9,k,1) = 64d0*dpdxic
             dpdxic = (2d0*theta(1,inds(k,2))-1d0)* &
                      theta(1,inds(k,3))*theta(2,inds(k,3))* &
                      theta(1,inds(k,1))*theta(2,inds(k,1))
             hc(2) = 8d0*dpdxic
             phiksi(9,k,2) = 64d0*dpdxic
             dpdxic = (2d0*theta(1,inds(k,3))-1d0)* &
                      theta(1,inds(k,1))*theta(2,inds(k,1))* &
                      theta(1,inds(k,2))*theta(2,inds(k,2))
             hc(3) = 8d0*dpdxic
             phiksi(9,k,3) = 64d0*dpdxic
             do i = 1, 8
                phiksi(i,k,1) = dtetds(indi(i,1))* &
                                theta(indi(i,2),inds(k,2))* &
                                theta(indi(i,3),inds(k,3)) - hc(1)
                phiksi(i,k,2) = theta(indi(i,1),inds(k,1))* &
                                dtetds(indi(i,2))* &
                                theta(indi(i,3),inds(k,3)) - hc(2)
                phiksi(i,k,3) = theta(indi(i,1),inds(k,1))* &
                                theta(indi(i,2),inds(k,2))* &
                                dtetds(indi(i,3)) - hc(3)
            end do
         end do

      end if

      if ( icoor==1 ) then

!     --- icoor = 1   Fill co-ordinates of nodes in array x

         do k = 1, 3
            do i = 1, inpelm
               x(i,k) = coor2d(k,index1(i))
            end do
         end do
         if ( inpelm==8 ) then
            do k = 1, 3
               som = 0d0
               do i = 1, inpelm
                  som = som + x(i,k)
               end do
               x(9,k) = som * 0.125d0
            end do
         end if

      end if

!     --- First store dx / dksi etc. in array dxdksi

      do i = 1, m
         dxdksi(i,1,1) = ddot ( 9, x(1,1), 1, phiksi(1,i,1), 1 )
         dxdksi(i,2,1) = ddot ( 9, x(1,1), 1, phiksi(1,i,2), 1 )
         dxdksi(i,3,1) = ddot ( 9, x(1,1), 1, phiksi(1,i,3), 1 )
         dxdksi(i,1,2) = ddot ( 9, x(1,2), 1, phiksi(1,i,1), 1 )
         dxdksi(i,2,2) = ddot ( 9, x(1,2), 1, phiksi(1,i,2), 1 )
         dxdksi(i,3,2) = ddot ( 9, x(1,2), 1, phiksi(1,i,3), 1 )
         dxdksi(i,1,3) = ddot ( 9, x(1,3), 1, phiksi(1,i,1), 1 )
         dxdksi(i,2,3) = ddot ( 9, x(1,3), 1, phiksi(1,i,2), 1 )
         dxdksi(i,3,3) = ddot ( 9, x(1,3), 1, phiksi(1,i,3), 1 )
      end do

      if ( igausp==1 ) then

!     --- igausp = 1   fill gauss points

         do i = 1, m
            xgauss(i,1) = ddot ( 9, x(1,1), 1, phi(1,i), 1 )
            xgauss(i,2) = ddot ( 9, x(1,2), 1, phi(1,i), 1 )
            xgauss(i,3) = ddot ( 9, x(1,3), 1, phi(1,i), 1 )
         end do

      end if

      if ( iweigo==1 .or. ideriv==1 ) then

!     --- ideriv = 1  Compute derivatives or iweigo = 1: weights
!                     Compute determinant in integration points
!                     Store temporarily in array w
!                     compute inverse of Jacobian

         do k = 1, m
            detinv(k,1,1) = dxdksi(k,2,2)*dxdksi(k,3,3)- &
                            dxdksi(k,3,2)*dxdksi(k,2,3)
            detinv(k,2,1) = dxdksi(k,3,2)*dxdksi(k,1,3)- &
                            dxdksi(k,1,2)*dxdksi(k,3,3)
            detinv(k,3,1) = dxdksi(k,1,2)*dxdksi(k,2,3)- &
                            dxdksi(k,2,2)*dxdksi(k,1,3)
            w(k) = dxdksi(k,1,1)*detinv(k,1,1)+dxdksi(k,2,1)* &
                   detinv(k,2,1)+dxdksi(k,3,1)*detinv(k,3,1)
         end do

         if ( ideriv==1 ) then

!        --- ideriv = 1,  compute all factors of inverse

            do k = 1, m
               detinv(k,1,2) = dxdksi(k,2,3)*dxdksi(k,3,1)- &
                               dxdksi(k,3,3)*dxdksi(k,2,1)
               detinv(k,2,2) = dxdksi(k,3,3)*dxdksi(k,1,1)- &
                               dxdksi(k,1,3)*dxdksi(k,3,1)
               detinv(k,3,2) = dxdksi(k,1,3)*dxdksi(k,2,1)- &
                               dxdksi(k,2,3)*dxdksi(k,1,1)
               detinv(k,1,3) = dxdksi(k,2,1)*dxdksi(k,3,2)- &
                               dxdksi(k,3,1)*dxdksi(k,2,2)
               detinv(k,2,3) = dxdksi(k,3,1)*dxdksi(k,1,2)- &
                               dxdksi(k,1,1)*dxdksi(k,3,2)
               detinv(k,3,3) = dxdksi(k,1,1)*dxdksi(k,2,2)- &
                               dxdksi(k,2,1)*dxdksi(k,1,2)
            end do

         end if

!        --- check sign of jacobian

         isign = nint(sign (1d0, w(1)))
         check = .false.
         do i = 1, m
            if ( abs(w(i))<=0d0 ) check = .true.
            jsign = nint(sign(1d0, w(i)))
            if( jsign/=isign ) then

               if ( isubr==0 ) then
                  call errint ( ielem, 1 )
                  call errint ( itype, 2 )
                  call errint ( ielgrp, 3 )
                  call errsub ( 230, 3, 0, 0 )
               else
                  call errint ( ielgrp, 1 )
                  call errint ( ielem, 2 )
                  call errchr ( 'volume', 1 )
                  call errsub ( 1540, 2, 0, 1 )
               end if
               write ( irefwr, 215 ) ((x(k,j),j=1,3), k=1,9)
215            format ( ( 1x, ss, 3(1pe14.5e2) ) )

               go to 1000
            end if
         end do
         if ( check ) then

            if ( isubr==0 ) then
               call errint ( ielem, 1 )
               call errint ( itype, 2 )
               call errint ( ielgrp, 3 )
               call errsub ( 105, 3, 0, 0 )
            else
               call errint ( ielgrp, 1 )
               call errint ( ielem, 2 )
               call errchr ( 'volume', 1 )
               call errsub ( 1540, 2, 0, 1 )
            end if
            write ( irefwr, 215 ) ((x(j,k),j=1,3), k=1,8)

            go to 1000
         end if

      end if

      if ( ideriv==1 ) then

!     --- ideriv = 1   Fill derivatives

         do i = 1, 9
            do k = 1, m
               dphidx(i,k,1) = (phiksi(i,k,1)*detinv(k,1,1)+ &
                                phiksi(i,k,2)*detinv(k,2,1)+ &
                                phiksi(i,k,3)*detinv(k,3,1))/w(k)
               dphidx(i,k,2) = (phiksi(i,k,1)*detinv(k,1,2)+ &
                                phiksi(i,k,2)*detinv(k,2,2)+ &
                                phiksi(i,k,3)*detinv(k,3,2))/w(k)
               dphidx(i,k,3) = (phiksi(i,k,1)*detinv(k,1,3)+ &
                                phiksi(i,k,2)*detinv(k,2,3)+ &
                                phiksi(i,k,3)*detinv(k,3,3))/w(k)
            end do
         end do

      end if

      if ( iweigo==1 ) then

!     --- iweigo = 1  Fill weights

         do k = 1, m
            w(k) = abs( w(k) ) * weight(k)
         end do

      end if

      if ( itype>=902 .and. itype<=903 ) then

!     --- Special case of Navier-Stokes element
!         w(1..m ) contains the weights
!         w(m+1 .. m+n) the diagonal of the mass matrix
!         In this case it is sufficient to use the integration rule available

         do i = 1, n
            som = 0d0
            do k = 1, m
               som = som+w(k)*phi(i,k)**2
            end do  ! k = 1, m
            w(m+i)= som
         end do  ! i = 1, n

      end if  ! ( itype>=902 .and. itype<=903 )

1000  call erclos ( 'elp636' )

      end
