      subroutine elp637intpol ( coor, index1, phi , xcoord, ycoord, &
                                zcoord, xsi, eta, zeta )
! ======================================================================
!
!        programmer    Raoul van Loon
!        version  1.1  date 09-09-2013 Use prsolmat3
!        version  1.0  date 12-02-2003
!
!   copyright (c) 2003-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute the values of the basis function in a specific point
!    extended triquadratic tetrahedron
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     interpolation
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer index1(*)
      double precision coor(3,*), phi(*), xcoord, ycoord, zcoord, &
                       xsi, eta, zeta

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     eta            o    Value of eta in previous iteration
!     index1         i    Array of length inpelm containing the point numbers
!                         of the nodal points in the element
!     phi            o    array of length 4 containing the values of the basis
!                         functions in the point xcoord, ycoord
!     xcoord         i    x-coordinate of point
!     xsi            o    Value of xsi in previous iteration
!     ycoord         i    y-coordinate of point
!     zcoord         i    z-coordinate of point
!     zeta           o    Value of zeta in previous iteration
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer j, iter
      double precision alam(4), gradc, l123, l124, &
                       l134, l234, dpdksi(15), dpdeta(15), dpdzeta(15), &
                       lam, dxsi(3), gradf(3,3), xsi1, eta1, zeta1, &
                       fnewton(3), x(15), y(15), z(15), distance
      logical singular

!     alam           Array containing the values of lambda in the nodal points
!     distance       distance between two quantities
!     dpdeta         array of length 8 to store the values of dphi/deta
!     dpdksi         array of length 8 to store the values of dphi/dxsi
!     dpdzeta        array of length 8 to store the values of dphi/dzeta
!     dxsi           Difference in xsi values
!     eta1           New value of eta
!     fnewton        Contains the values of the funcion on which the Newton
!                    process is carried out
!     gradc          Gradient of lambda_1 * lambda_2 * lambda_3
!                    See pseudo code
!     gradf          Gradient of function in Newton process
!     iter           Iteration sequence number
!     j              Counting variable
!     l123           Help variable corresponding to baricentric coordinate 123
!     l124           Help variable corresponding to baricentric coordinate 124
!     l134           Help variable corresponding to baricentric coordinate 134
!     l234           Help variable corresponding to baricentric coordinate 234
!     lam            Help variable
!     singular       if true the matrix is singular
!     x              x co-ordinate of point
!     xsi1           New value of xsi
!     y              y co-ordinate of point
!     z              z co-ordinate of point
!     zeta1          New value of zeta
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision ddot

!     DDOT           (CDOT)     Inner product of two vectors
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     PRSOLMAT3      Solve system of 3 linear equations S x = f
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2186   The matrix to be inverted is singular
! **********************************************************************
!
!                       PSEUDO CODE
!
!                  SEE ALSO SUBROUTINE ELP637
!
!        lambda(1) = 1 - xsi - eta - zeta
!        lambda(2) = xsi
!        lambda(3) = eta
!        lambda(4) = zeta
!
!        lamc = lambda lambda lambda lambda
!                     1      2      3      4
!
!        --- for vertices i = 1,2,3,4 : by permutation
!
!        phi    = lambda  ( 2 lambda  - 1 ) +3 (lambda lambda lambda + &
!           i           i           i                 i      j      k
!       +            lambda lambda lambda + lambda lambda lambda ) - &
!                       k      l      i        l      i      j
!       +            4 lamc
!
!        --- instead of lambda  ( 2 lambda  - 1 ) also the following
!                              i           i
!            expression can be used:
!            (--- indices by cyclic permutation ---)
!
!            lambda - 2 lambda (lambda +lambda +lambda )
!                  i          i       j       k       l
!
!            because of the relationship:
!
!            lambda + lambda + lambda + lambda = 1
!                  1        2        3        4
!
!        --- for midside points : 1<=i<j<=4
!
!        phi   = 4 lambda  lambda  - 12 ( lambda lambda lambda + &
!           ij           i       j              i      j      k
!       +           lambda lambda lamdba ) + 32 lamc
!                      i      j      l
!
!        --- for midface points : 1<=i<j<k<l<=4
!
!        phi   = 27 lambda lambda lambda  - 108 lamc
!           ijk           i      j      k
!
!        --- centroid tetrahedron
!
!        phi    = 256 lamc
!           c
!
!
!        --- derivatives
!
!        grad lamc = lambda_2 lambda_3 lambda_4 grad lambda_1 + &
!                    lambda_3 lambda_4 lambda_1 grad lambda_2 + &
!                    lambda_4 lambda_1 lambda_2 grad lambda_3 + &
!                    lambda_1 lambda_2 lambda_3 grad lambda_4
!
!        grad phi   = grad lambda  ( 4 lambda  - 1 ) + 3 (
!                i               i           i
!       +        grad lambda lambda lambda + grad lambda lambda lambda + &
!                        i      j      l             j      l      i
!       +        grad lambda lambda lambda + grad lambda lambda lambda + &
!                        l      i      j             i      j      k
!       +        grad lambda lambda lambda + grad lambda lambda lambda + &
!                        j      k      i             k      i      j
!       +        grad lambda lambda lambda + grad lambda lambda lambda + &
!                        i      k      l             k      l      i
!       +        grad lambda lambda lambda ) - 4 grad lamc
!                        l      i      k
!
!        grad phi   = 4 [ grad(lambda )lambda  + grad(lambda )lambda ]
!                ij                  i       j              j       i
!       +      -12 (grad lambda lambda lambda + grad lambda lambda lambda + &
!                           i      j      k             i      j      l
!       +           grad lambda lambda lambda + grad lambda lambda lambda + &
!                           j      k      i             j      l      i
!       +           grad lambda lambda lambda + grad lambda lambda lambda )
!                           k      i      j             l      i      j
!       +      +32 grad lamc
!
!        grad phi   = 27 (grad lambda lambda lambda + &
!                ijk                 i      j      k
!       +                    grad lambda lambda lambda + &
!                                    k      i      j
!       +                    grad lambda lambda lambda ) - 108 grad lamc
!                                    j      k      i
!
!        grad phi = 256 grad lamc
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen('elp637intpol')

!     --- fill coordinates

      do j = 1, 15
         x(j) = coor(1,index1(j))
         y(j) = coor(2,index1(j))
         z(j) = coor(3,index1(j))
      end do

!     --- initialize local coordinates

      xsi  = 5d-1
      eta  = 5d-1
      zeta = 5d-1

      distance = 1d0
      iter = 0
      do while ( distance>1d-6 .and. iter<=5 )

!     --- Stop if the distance is less than 1d-6 or number of
!         iterations is 5

         iter = iter+1

         alam(1)=1d0-xsi-eta-zeta
         alam(2)=xsi
         alam(3)=eta
         alam(4)=zeta

         gradc = alam(1)*alam(2)*alam(3)*alam(4)
         L123 = alam(1)*alam(2)*alam(3)
         L124 = alam(1)*alam(2)*alam(4)
         L134 = alam(1)*alam(3)*alam(4)
         L234 = alam(2)*alam(3)*alam(4)

!        --- Compute the basis functions

         phi(1) = alam(1)*(2d0*alam(1)-1d0)+ &
                  3d0*(L123 + L124 + L134) - 4d0*gradc
         phi(2) = 4d0*alam(1)*alam(2)- &
                  12d0*(L123 + L124) + 32d0*gradc
         phi(3) = alam(2)*(2d0*alam(2)-1d0)+ &
                  3d0*(L123 + L124 + L234) - 4d0*gradc
         phi(4) = 4d0*alam(2)*alam(3)- &
                  12d0*(L123 + L234) + 32d0*gradc
         phi(5) = alam(3)*(2d0*alam(3)-1d0)+ &
                  3d0*(L123 + L134 + L234) - 4d0*gradc
         phi(6) = 4d0*alam(1)*alam(3)- &
                  12d0*(L123 + L134) + 32d0*gradc
         phi(7) = 4d0*alam(1)*alam(4)- &
                  12d0*(L124 + L134) + 32d0*gradc
         phi(8) = 4d0*alam(2)*alam(4)- &
                  12d0*(L124 + L234) + 32d0*gradc
         phi(9) = 4d0*alam(3)*alam(4)- &
                  12d0*(L134 + L234) + 32d0*gradc
         phi(10) =alam(4)*(2d0*alam(4)-1d0)+ &
                  3d0*(L124 + L134 + L234) - 4d0*gradc
         phi(11)= 27d0*L123 - 108d0*gradc
         phi(12)= 27d0*L124 - 108d0*gradc
         phi(13)= 27d0*L234 - 108d0*gradc
         phi(14)= 27d0*L134 - 108d0*gradc
         phi(15)= 256d0*gradc

!        --- Compute the derivatives of the basis functions dphi/dxsi

         lam = alam(1)*alam(3)*alam(4)- &
             alam(2)*alam(3)*alam(4)

         dpdksi(1)  = -4d0*lam + 3d0*(alam(1)*alam(3) + &
                      alam(1)*alam(4) - alam(2)*alam(3) - &
                      alam(2)*alam(4) - alam(3)*alam(4)) - &
                      4d0*alam(1) + 1d0
         dpdksi(3)  = -4d0*lam + 3d0*(alam(1)*alam(3) + &
                      alam(1)*alam(4) + alam(3)*alam(4) - &
                      alam(2)*alam(4) - alam(2)*alam(3)) + &
                      4d0*alam(2) - 1d0
         dpdksi(5)  = -4d0*lam + 3d0*(alam(1)*alam(3) - &
                      alam(2)*alam(3))
         dpdksi(10) = -4d0*lam + 3d0*(alam(1)*alam(4) - &
                      alam(2)*alam(4))
         dpdksi(2)  = 32d0*lam + 12d0*(alam(2)*alam(4) + &
                      alam(2)*alam(3) - alam(1)*alam(4) - &
                      alam(1)*alam(3)) + 4d0*(alam(1)-alam(2))
         dpdksi(4)  = 32d0*lam + 12d0*(alam(2)*alam(3) - &
                      alam(1)*alam(3) - alam(3)*alam(4)) + &
                      4d0*alam(3)
         dpdksi(6)  = 32d0*lam + 12*(alam(2)*alam(3) + &
                      alam(3)*alam(4) - alam(1)*alam(3)) - &
                      4d0*alam(3)
         dpdksi(7)  = 32d0*lam + 12d0*(alam(2)*alam(4) + &
                      alam(3)*alam(4) - alam(1)*alam(4)) - &
                      4d0*alam(4)
         dpdksi(8)  = 32d0*lam + 12d0*(alam(2)*alam(4) - &
                      alam(1)*alam(4) - alam(3)*alam(4)) + &
                      4d0*alam(4)
         dpdksi(9)  = 32d0*lam
         dpdksi(11) = -108d0*lam + 27d0*(alam(1)*alam(3) - &
                      alam(2)*alam(3))
         dpdksi(12) = -108d0*lam + 27d0*(alam(1)*alam(4) - &
                      alam(2)*alam(4))
         dpdksi(13) = -108d0*lam + 27d0*alam(3)*alam(4)
         dpdksi(14) = -108d0*lam - 27d0*alam(3)*alam(4)
         dpdksi(15) = 256d0*lam

!        --- Compute the derivatives of the basis functions dphi/deta

         lam = alam(1)*alam(2)*alam(4)- &
             alam(2)*alam(3)*alam(4)

         dpdeta(1)  = -4d0*lam + 3d0*(alam(1)*alam(2) + &
                      alam(1)*alam(4) - alam(2)*alam(3) - &
                      alam(2)*alam(4) - alam(3)*alam(4)) - &
                      4d0*alam(1) + 1d0
         dpdeta(3)  = -4d0*lam + 3d0*(alam(1)*alam(2) - &
                      alam(2)*alam(3))
         dpdeta(5)  = -4d0*lam + 3d0*(alam(1)*alam(2) + &
                      alam(1)*alam(4) + alam(2)*alam(4) - &
                      alam(3)*alam(4) - alam(2)*alam(3)) + &
                      4d0*alam(3) - 1d0
         dpdeta(10) = -4d0*lam + 3d0*(alam(1)*alam(4) - &
                      alam(3)*alam(4))
         dpdeta(2)  = 32d0*lam + 12d0*(alam(2)*alam(3) + &
                      alam(2)*alam(4) - alam(1)*alam(2)) - &
                      4d0*alam(2)
         dpdeta(4)  = 32d0*lam + 12d0*(alam(2)*alam(3) - &
                      alam(1)*alam(2) - alam(2)*alam(4)) + &
                      4d0*alam(2)
         dpdeta(6)  = 32d0*lam + 12d0*(alam(2)*alam(3) + &
                      alam(3)*alam(4) - alam(1)*alam(2) - &
                      alam(1)*alam(4)) + 4d0*(alam(1)-alam(3))
         dpdeta(7)  = 32d0*lam + 12d0*(alam(2)*alam(4) + &
                      alam(3)*alam(4) - alam(1)*alam(4)) - &
                      4d0*alam(4)
         dpdeta(8)  = 32d0*lam
         dpdeta(9)  = 32d0*lam + 12d0*(alam(3)*alam(4) - &
                      alam(2)*alam(4) - alam(1)*alam(4)) + &
                      4d0*alam(4)
         dpdeta(11) = -108d0*lam + 27d0*(alam(1)*alam(2) - &
                      alam(2)*alam(3))
         dpdeta(12) = -108d0*lam - 27d0*alam(2)*alam(4)
         dpdeta(13) = -108d0*lam + 27d0*alam(2)*alam(4)
         dpdeta(14) = -108d0*lam + 27d0*(alam(1)*alam(4) - &
                      alam(3)*alam(4))
         dpdeta(15) = 256d0*lam

!        --- Compute the derivatives of the basis functions dphi/dzeta

         lam = alam(1)*alam(2)*alam(3)- &
               alam(2)*alam(3)*alam(4)

         dpdzeta(1)  = -4d0*lam + 3d0*(alam(1)*alam(2) + &
                       alam(1)*alam(3) - alam(2)*alam(3) - &
                       alam(2)*alam(4) - alam(3)*alam(4)) - &
                       4d0*alam(1) + 1d0
         dpdzeta(3)  = -4d0*lam + 3d0*(alam(1)*alam(2) - &
                       alam(2)*alam(4))
         dpdzeta(5)  = -4d0*lam + 3d0*(alam(1)*alam(3) - &
                       alam(3)*alam(4))
         dpdzeta(10) = -4d0*lam + 3d0*(alam(1)*alam(2) + &
                       alam(2)*alam(3) + alam(1)*alam(3) - &
                       alam(2)*alam(4) - alam(3)*alam(4)) + &
                       4d0*alam(4) - 1d0
         dpdzeta(2)  = 32d0*lam + 12d0*(alam(2)*alam(3) + &
                       alam(2)*alam(4) - alam(1)*alam(2)) - &
                       4d0*alam(2)
         dpdzeta(4)  = 32d0*lam
         dpdzeta(6)  = 32d0*lam + 12d0*(alam(2)*alam(3) + &
                       alam(3)*alam(4) - alam(1)*alam(3)) - &
                       4d0*alam(3)
         dpdzeta(7)  = 32d0*lam + 12d0*(alam(2)*alam(4) + &
                       alam(3)*alam(4) - alam(1)*alam(2) - &
                       alam(1)*alam(3)) + 4d0*(alam(1)-alam(4))
         dpdzeta(8)  = 32d0*lam + 12d0*(alam(2)*alam(4) - &
                       alam(1)*alam(2) - alam(2)*alam(3)) + &
                       4d0*alam(2)
         dpdzeta(9)  = 32d0*lam + 12d0*(alam(3)*alam(4) - &
                       alam(1)*alam(3) - alam(2)*alam(3)) + &
                       4d0*alam(3)
         dpdzeta(11) = -108d0*lam - 27d0*alam(2)*alam(3)
         dpdzeta(12) = -108d0*lam + 27d0*(alam(1)*alam(2) - &
                       alam(2)*alam(4))
         dpdzeta(13) = -108d0*lam + 27d0*alam(2)*alam(3)
         dpdzeta(14) = -108d0*lam + 27d0*(alam(1)*alam(3) - &
                       alam(3)*alam(4))
         dpdzeta(15) = 256d0*lam

!        --- compute rhs in Newton-Raphson scheme

         fnewton(1) = ddot ( 15, x, 1, phi, 1 )-xcoord
         fnewton(2) = ddot ( 15, y, 1, phi, 1 )-ycoord
         fnewton(3) = ddot ( 15, z, 1, phi, 1 )-zcoord

!        --- Store the transpose of gradf in order to use the fortan standard

         gradf(1,1) = ddot ( 15, x, 1, dpdksi, 1 )
         gradf(2,1) = ddot ( 15, y, 1, dpdksi, 1 )
         gradf(3,1) = ddot ( 15, z, 1, dpdksi, 1 )
         gradf(1,2) = ddot ( 15, x, 1, dpdeta, 1 )
         gradf(2,2) = ddot ( 15, y, 1, dpdeta, 1 )
         gradf(3,2) = ddot ( 15, z, 1, dpdeta, 1 )
         gradf(1,3) = ddot ( 15, x, 1, dpdzeta, 1 )
         gradf(2,3) = ddot ( 15, y, 1, dpdzeta, 1 )
         gradf(3,3) = ddot ( 15, z, 1, dpdzeta, 1 )

!        --- Compute error of local coordinates

         call prsolmat3 ( gradf, fnewton, dxsi, singular )
         if ( singular ) then

!        --- determinant is 0

            call errsub ( 2186, 0, 0, 0 )
            go to 1000

         end if

!        --- Find the new local coordinates

         xsi1 = xsi - dxsi(1)
         eta1 = eta - dxsi(2)
         zeta1 = zeta - dxsi(3)

!        --- Determine the distance between new and old local coordinates

         distance = (xsi1-xsi)**2 + (eta1-eta)**2 + (zeta1-zeta)**2

!        --- Update the old local coordinates

         xsi = xsi1
         eta = eta1
         zeta = zeta1

      end do

1000  call erclos('elp637intpol')
      end
