      subroutine elp638first ( phi, phiksi, weight )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-06-2013
!
!   copyright (c) 2013-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute basis functions and related quantities for trilinear wedge
!     Computed are:     weights for numerical integration
!                       values of derivatives of basis functions
!                       values of basis functions in integration points
!     ELP638 is called by several types of element subroutines
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     wedge
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision phi(6,m), phiksi(6,m,3), weight(m)

!     phi      o     Array of length 6 x m containing the values of the
!                    basis functions in the Gauss points in the sequence:
!                    phi (xg ) = phi (i,k)
!                       i   k
!                    Array phi is only filled when ifirst = 0 and gauss
!                    integration is applied
!     phiksi   o     Array of length 6 x m x 3 containing the values of the
!                    derivatives of the basis functions in the Gauss points in
!                    the reference element.
!                    Array phi is only filled when ifirst = 0 and must be kept
!                    by the program
!     weight   o     Contains weights in the reference element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, k, intrule
      double precision c, xi(m), eta(m), zeta(m)
      character (len=18) text

!     c              help variable to store a constant
!     eta            array that contains the values of eta in the
!                    integration points
!     i              Counting variable
!     intrule        Actual integration rule
!     k              Counting variable
!     text           String variable containing some text
!     xi             array that contains the values of xi in the
!                    integration points
!     zeta           array that contains the values of xeta in the
!                    integration points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRORELVAR     Print error message errornr for element: wrong value of
!                    parameter
!     ERRORELVAR2    Print error message errornr for element: wrong value of
!                    2 parameters
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     To compute the derivatives of the basis functions, the jacobian
!     of the element and the weights, the following definitions are used:
!     The lower triangle is defined by the points 1,2,3, the upper
!     triangle by 4,5,6
!     The basis functions are mapped on a unit wedge in the (xi, eta ,zeta )
!     space with basis functions:
!
!     phi_1 = lambda_1(xi,eta) lambda_1(zeta)
!     phi_2 = lambda_2(xi,eta) lambda_1(zeta)
!     phi_3 = lambda_3(xi,eta) lambda_1(zeta)
!     phi_4 = lambda_1(xi,eta) lambda_2(zeta)
!     phi_5 = lambda_2(xi,eta) lambda_2(zeta)
!     phi_6 = lambda_3(xi,eta) lambda_2(zeta)
!
!     lambda_1(xi,eta) = 1 -xi - eta
!     lambda_2(xi,eta) = xi
!     lambda_3(xi,eta) = eta
!
!     lambda_1(zeta) = 1-zeta
!     lambda_2(zeta) = zeta
!
!     Hence the triangles are in the xi,eta planes and the upwards direction
!     is defined as zeta
!
!     The weights for the numerical integration rule are given as follows:
!     Let J be the Jacobian in the integration points then:
!
!     intrule = 1: (Newton Cotes rule)  w = 1/12 J(x_k)
!     intrule = 2: (One point Gauss rule)  w = 1/2 J(x_k)
!     intrule = 3: (Eight point Gauss rule)  w = 1/2 wgs2(k3) J(x_k)
!        (k3 is k modulo 3)
!
!     The integration points for the numerical integration rule are given as
!     follows:
!
!     intrule = 1: (Newton Cotes rule)  6 vertices
!     intrule = 2: (One point Gauss rule)  1 centroid
!     intrule = 3: (Eight point Gauss rule)
!       (xi_k,eta_k,zeta_k) = alam2(k3) wgs2(kz)
!       (kz = (k-1)/3+1)
!
!     dphi_dxi_j can be computed immediately from the standard basis functions:
!
!     dphi_i/dxi_j        xi              eta            zeta
!         1           -(1-zeta)        -(1-zeta)        -(1-xi-eta)
!         2             1-zeta             0               -xi
!         3                0             1-zeta            -eta
!         4              -zeta            zeta            1-xi-eta
!         5               zeta             0                xi
!         6                0              zeta              eta
!
!     dx/dxi_j = sum_k x_k dphi_k/dxi_j
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data text / 'Trilinear wedge' /
! ======================================================================
!
      call eropen ( 'elp638first' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elp638first'

      end if
      if ( ierror/=0 ) go to 1000

!     --- Get some constants

      intrule = mod(irule,100)
      if ( jcart/=1 ) then

!     --- Error 1769: Co-ordinate system not available

         call errorelvar ( jcart-1, 1769, text, 0 )

      end if
      if ( intrule<1 .or. intrule>3 ) then

!     --- Error 1770: Integration rule not available

         call errorelvar ( intrule, 1770, text, 1 )
         intrule = 1

      end if
      if ( intrule==1 .and. m/=6 .or. &
           intrule==2 .and. m/=1 .or. &
           intrule==3 .and. m/=8 ) then

!     --- Error 1424: m and integration rule do not match

         call errorelvar2 ( m, intrule, 1424, text, 0 )

      end if

!     --- Fill array phi

      xi = 0d0
      eta = 0d0
      zeta = 0d0

      select case ( intrule )

         case(1)

!        --- intrule = 1  Newton Cotes rule

            phi = 0d0
            do i = 1, 6
               phi(i,i) = 1d0
            end do
            c = 1d0/12d0
            weight = c

            xi(2) = 1d0
            xi(5) = 1d0

            eta(3) = 1d0
            eta(6) = 1d0

            zeta(4:6) = 1d0

         case(2)

!        --- intrule = 2  one-point Gauss rule

            c = 1d0/6d0
            phi(:,1) = c
            weight(1) = 0.5d0

            xi(1) = 1d0/3d0
            eta(1) = xi(1)
            zeta(1) = 0.5d0

         case(3)

!        --- 8 point Gauss rule phi_ij = alam2 * alam4

            do i = 1, 3
               phi(i+3,1:4) = alam2(i,1:4) * alam4(1)
               phi(i,1:4) = alam2(i,1:4) * (1d0-alam4(1))
               phi(i+3,5:8) = alam2(i,1:4) * alam4(2)
               phi(i,5:8) = alam2(i,1:4) * (1d0-alam4(2))
            end do
            weight(1:4) = wgs2(1:4) * 0.5d0
            weight(5:8) = weight(1:4)

            xi(1:4) = alam2(2,1:4)
            xi(5:8) = xi(1:4)
            eta(1:4) = alam2(3,1:4)
            eta(5:8) = eta(1:4)
            zeta(1:4) = alam4(1)
            zeta(5:8) = alam4(2)

      end select  ! case ( intrule )

!     --- fill phiksi

      do k = 1, m

         phiksi(1,k,1) = -(1d0-zeta(k))
         phiksi(2,k,1) =  1d0-zeta(k)
         phiksi(3,k,1) =  0d0
         phiksi(4,k,1) = -zeta(k)
         phiksi(5,k,1) =  zeta(k)
         phiksi(6,k,1) =  0d0

         phiksi(1,k,2) = -(1d0-zeta(k))
         phiksi(2,k,2) =  0d0
         phiksi(3,k,2) =  1d0-zeta(k)
         phiksi(4,k,2) = -zeta(k)
         phiksi(5,k,2) =  0d0
         phiksi(6,k,2) =  zeta(k)

         phiksi(1,k,3) = -(1d0-xi(k)-eta(k))
         phiksi(2,k,3) = -xi(k)
         phiksi(3,k,3) = -eta(k)
         phiksi(4,k,3) = 1d0-xi(k)-eta(k)
         phiksi(5,k,3) = xi(k)
         phiksi(6,k,3) = eta(k)

      end do  ! k = 1, m

1000  call erclos ( 'elp638first' )
      if ( debug ) write(irefwr,*) 'End elp638first'

      end
