      subroutine elpup2 ( alpha, u, phi, psi, dphidx, x, nx, second, &
                          dcdx )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.6  date 01-06-2007 Fortran 90
!        version  3.5  date 04-07-2003 Remove common celwrk
!        version  3.4  date 05-10-2002 Correction for 5 points quadrilaterals
!
!   copyright (c) 1993-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute modified basis functions for upwind elements
!    Two dimensional elements, quadrilaterals
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     upwinding
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletime
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nx
      double precision alpha(m,3), u(m,2), phi(n,m), psi(n,m), &
                       dphidx(n,m,2), x(nx,2), dcdx(m,ndimlc)
      logical second

!     alpha    i     Array of length m x 3 containing the factors alpha
!                                                                      ij
!                    in the integration points
!     dcdx     i     Contains the values of dc/dx in the integration points
!                    This is used in case of non-linear upwinding
!     dphidx   i     Array of length n x m x 2 containing the values of the
!                    derivatives of the basis functions in the integration
!                    nodes
!     nx       i     Number of points in the x-direction
!     phi      i     Array of length n x m containing the values of the basis
!                    functions in the integration points
!     psi      o     Array of length n x m containing the values of the modified
!                    basis functions in the integration points
!     second   o     Indicates if the second order term is significant or may
!                    be neglected in the upwinding part
!     u        i     Array of length m x 2 containing the velocity in the
!                    integration points
!     x        i    array containing the coordinates of the nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, k, metupl, indloc(4), j
      double precision xi, beta, xhalf(4,2), uksi, ueta, hksi, heta, &
                       cgnorm, tau1(9), &
                       u1(9,2), tau(9), h(9), unorm(9), eps(9)

!     beta           Parameter beta in method
!     cgnorm         Norm of dphidx in integration point for specific value of i
!     eps            Array of length 9 to store the value of the tensor alpha
!                    in the direction of the velocity in the vertices
!     h              Array of length 9 to store the values of the diameter h
!                    in the vertices
!     heta           Distance in eta direction
!     hksi           Distance in ksi direction
!     i              Counting variable
!     ind            Array containing the cyclic permutation from 1 to 4
!     j              Counting variable
!     k              Counting variable
!     metupl         mod(metupw,100): defines type of upwinding
!                     j      i           j
!     tau            Array of length 9 to store the values of tau in the
!                    integration points
!     tau1           Array of length 9 to store the values of tau parallel in
!                    the integration points
!     u1             Parallel velocity
!     ueta           Velocity component in eta direction
!     uksi           Velocity component in ksi direction
!     unorm          array of length 9 to store the norm of the velocity in the
!                    vertex
!     xhalf          Contains the co-ordinates of the mid-side points
!     xi             Factor xi, see method
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!                              METHOD
!                                                       new
!       The basis functions are modified such that:  phi    = phi + p
!                                                       i        i   i
!       or:
!          new  k          k        k
!       phi   (x ) = phi (x ) + p (x )
!          i            i        i
!       The basis funtions phi are assumed to be filled before
!       p  = tau u . grad phi
!        i       -           i
!                   h xi                    ||u|| h
!       with tau = ------ beta =  -------- &
!                   2||u||                   2 eps
!                      - &
!       xi depends on metupw according to:
!       metupw = 1:  Classical upwind scheme  xi = 1
!                2:  Il'in    xi = coth(beta) - 1/beta
!                                             beta/3     -3<beta<3
!                3:  Doubly assymptotic: xi = &
!                                             sign(beta)  |beta|>3
!                                         -1-1/beta     beta<1
!                4:  critical approx: xi = 0            -1<beta<1
!                                          1-1/beta     beta>1
!       eps is either alpha (if a11 = a22) or alpha in the direction of
!       the velocity
!              T                 2
!       eps = u  alpha  u / ||u||       (tensor alpha)
!             -         -     - &
!       h = max distance in element in the direction of u
!                                                       - &
!       h is computed as follows:
!       h = ( |u   | h    + |u   | h     ) / ||u||
!               ksi   ksi     eta   eta        - &
!       h eta = || x34 - x12 ||      h ksi = || x23 - x41 ||
!       x34 = (x3 + x4)/2     etc.
!              -    - &
!     4 +------------+3
!       |            |
!       |            |
!       |            |
!       +------------+ &
!       1            2
!       u ksi= ux ( x23-x14) + uy ( y23-y14)
!       u eta= ux ( x34-x12) + uy ( y34-y12)
!       First tau and h are computed in the vertices and afterwards tau is
!       interpolated to the integration points
!    In case of the discontinuity capturing methods we define u   as
!                                                              ||
!                                            2
!    u   = u . grad(c) grad(c) / || grad(c)||
!     ||
!    In that case the upwind basis functions become
!       p  = ( tau u  + tau   u  ) . grad phi
!        i        1-       2  -||            i
!    with tau  the tau as described above and tau  is tau   (DC1) or
!            1                                   2       ||
!    max(0,tau  -tau) (DC2)      tau   is the tau based upon u
!             ||                    ||                        ||
!    In this case we limit ourselves to the doubly assymptotic approximation
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data indloc / 2, 3, 4, 1 /
! ======================================================================
!
      call eropen ( 'elpup2' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elpup2'
         call prinrl1 ( phi, m, n, 'phi' )
         call prinrl1 ( dphidx(1,1,1), m, n, 'dphi/dx' )
         call prinrl1 ( dphidx(1,1,2), m, n, 'dphi/dy' )
         call prinrl1 ( u, 2, m, 'u' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill norm of velocity in vertices into array unorm

      metupl = mod ( metupw, 20 )
      do i = 1, m
         unorm(i) = sqrt ( u(i,1)**2 + u(i,2)**2 )
      end do
      if ( debug ) call prinrl ( unorm, m, 'unorm' )

!     --- Store component of alpha tensor in u-direction into eps
!                                            - &

      if ( jsecnd==0 ) then

!     --- jsecnd = 0,   eps = 0

         do i = 1, m
            eps(i) = 0d0
         end do

      else if ( jsecnd==1 ) then

!     --- jsecnd = 1,   eps = alpha = a22  (constant)  a12 = 0

         do i = 1, m
            eps(i) = alpha(1,1)
         end do

      else if ( jsecnd==2 ) then

!     --- jsecnd = 2,   eps = alpha = a22    a12 = 0

         if ( jdiag==1 ) then

!        --- newton cotes rules

            if ( n==4 .or. n==5 ) then

!           --- n = 4, bilinear quadrilateral

               do i = 1, 4
                  eps(i) = alpha(i,1)
               end do

            else

!           --- n = 9, biquadratic quadrilateral

               do i = 1, 9
                  eps(i) = alpha(i,1)
               end do
            end if

         else

!        --- gauss integration

            do i = 1, m
               eps(i) = alpha(i,1)
            end do

         end if

      else

!     --- jsecnd>2,   General case                         T
!                      eps is defined by the formula  eps = u  alpha u
!                                                           -         - &

         if ( jdiag==1 ) then

!        --- newton cotes rules

            if ( n==4 .or. n==5 ) then

!           --- n = 4, linear quadrilateral

               do i = 1, 4
                  if ( unorm(i)>0d0 ) then
                      eps(i) = ( u(i,1)**2*alpha(i,1)+2d0*u(i,1)* &
                                 alpha(i,2)*u(i,2) + u(i,2)**2* &
                                 alpha(i,3) ) / unorm(i)
                  else
                     eps(i) = alpha(i,1)
                  end if
               end do

            else if ( n==9 ) then

!           --- n = 9, biquadratic quadrilateral

               do i = 1, 9
                  if ( unorm(i)>0d0 ) then
                      eps(i) = ( u(i,1)**2*alpha(i,1)+2d0*u(i,1)* &
                                 alpha(i,2)*u(i,2) + u(i,2)**2* &
                                 alpha(i,3) ) / unorm(i)
                  else
                     eps(i) = alpha(i,1)
                  end if
               end do

            end if

         else

!        --- gauss integration rules

            do i = 1, m

              if ( unorm(i)>0d0 ) then
                  eps(i) = ( u(i,1)**2*alpha(i,1)+2d0*u(i,1)* &
                             alpha(i,2)* u(i,2) + u(i,2)**2* &
                             alpha(i,3) ) / unorm(i)
               else
                  eps(i) = alpha(i,1)
               end if

            end do

         end if

      end if
      if ( debug ) call prinrl ( eps, m, 'eps' )

!     --- Compute the distance h in the vertices

      if ( n==4 .or. n==5 ) then

!     --- four-point quadrilateral

         do j = 1, 2
            do i = 1, 4
               xhalf(i,j) = 0.5d0 * ( x(indloc(i),j) + x(i,j) )
            end do
         end do

      else

!     --- nine-point quadrilateral

         do j = 1, 2
            do i = 1, 4
               xhalf(i,j) = x(2*i-1,j)
            end do
         end do

      end if
      do i = 1, m
         if ( unorm(i)>epsmac ) then
            uksi = u(i,1) * ( xhalf(2,1) - xhalf(4,1) ) + &
                   u(i,2) * ( xhalf(2,2) - xhalf(4,2) )
            ueta = u(i,1) * ( xhalf(3,1) - xhalf(1,1) ) + &
                   u(i,2) * ( xhalf(3,2) - xhalf(1,2) )
            h(i) = ( abs(uksi) + abs(ueta) ) / unorm(i)
         else
            hksi = sqrt ( (xhalf(2,1)-xhalf(4,1))**2 + &
                          (xhalf(2,2)-xhalf(4,2))**2 )
            heta = sqrt ( (xhalf(3,1)-xhalf(1,1))**2 + &
                          (xhalf(3,2)-xhalf(1,2))**2 )
            h(i) = 0.5d0 * ( hksi + heta )
         end if
      end do
      if ( debug ) call prinrl ( h, m, 'h' )

!     --- Compute tau in the vertices

      do i = 1, m
         if ( eps(i)>1d-10 .and. metupl<=8 .and. metupl/=6 ) then

!        --- eps(i)>1d-10,  hence second order term plays a role

            beta = unorm(i)*h(i)/(2d0*eps(i))
            if ( beta<1d-10 ) then
               xi = 0d0
            else if ( metupl==1 ) then

!           --- metupl = 1   classical upwind scheme

               xi = 1d0

            else if ( metupl==2 ) then

!           --- metupl = 2   il'in scheme

               xi = 1d0/tanh(beta)-1d0/beta

            else if ( metupl==3 .or. metupl==7 .or. metupl==8 ) then

!           --- metupl = 3, 7 or 8    doubly assymptotic approximation

               if ( abs(beta)>3d0 ) then
                  xi=sign(1d0,beta)
               else
                  xi = beta / 3d0
               end if

            else if ( metupl==4 ) then

!           --- metupl = 4    critical approximation

               if ( beta<-1d0 ) then
                  xi = -1d0 - 1d0 / beta
               else if ( beta>1d0 ) then
                  xi = 1d0 - 1d0 / beta
               else
                  xi = 0d0
               end if

            else if ( metupl==5 ) then

!           --- metupl = 5

               if ( unorm(i)<=1d-5*eps(i) ) then
                  xi = 0d0
               else
                  beta = beta*beta
                  xi = sqrt(beta/(9d0+beta))
               end if
            end if

         else if (metupl==6 ) then

!        --- metupl = 6

            if ( h(i)>epsmac ) then
               beta = (2d0*unorm(i)/h(i))**2 + &
                      (4d0*eps(i)/h(i)**2)**2
            else
               beta = 0d0
            end if
            if ( tstep>0d0 ) beta = beta + 4d0/(tstep*tstep)
            tau(i) = 1d0/sqrt(beta)

         else

            xi = 1d0

         end if
         if ( metupl<=8 .and. metupl/=6 ) then
            if ( unorm(i)>epsmac ) then
               tau(i) = h(i)*xi/(2d0*unorm(i))
            else
               tau(i) = 0d0
            end if
         end if
      end do
      if ( debug ) call prinrl ( tau, m, 'tau' )

!     --- compute new basis functions, it has been assumed that the old
!         basis functions are already stored in array phi

      do k = 1, m
         do i = 1, n
            psi(i,k) = tau(k) * ( u(k,1)*dphidx(i,k,1)+ &
                                  u(k,2)*dphidx(i,k,2) ) + phi(i,k)
         end do
      end do

      if ( jcart==1 ) then

         do k = 1, m
            if ( abs(unorm(k))>0d0 .and. &
                 abs(eps(k))*1d2>unorm(k) ) second = .true.
         end do

      end if

      if ( metupl==7 .or. metupl==8 ) then

!     --- Discontinuity capturing
!         First compute u
!                        ||

         do k = 1, m
            cgnorm = dcdx(k,1)**2 + dcdx(k,2)**2
            if ( cgnorm<epsmac ) then
               u1(k,1) = 0d0
               u1(k,2) = 0d0
            else
               xi = (u(k,1)*dcdx(k,1) + u(k,2)*dcdx(k,2))/cgnorm
               u1(k,1) = xi*dcdx(k,1)
               u1(k,2) = xi*dcdx(k,2)
            end if
            unorm(k) = sqrt ( u1(k,1)**2 + u1(k,2)**2 )
         end do
         if ( jsecnd>2 ) then

!        --- jsecnd>2,   General case                         T
!                         eps is defined by the formula  eps = u  alpha u
!                                                              -         - &

            if ( jdiag==1 ) then

!           --- newton cotes rules

               if ( n==4 .or. n==5 ) then

!              --- n = 4, linear quadrilateral

                  do i = 1, 4
                     if ( unorm(i)>0d0 ) then
                         eps(i) = ( u1(i,1)**2*alpha(i,1)+2d0*u1(i,1)* &
                                    alpha(i,2)*u1(i,2) + u1(i,2)**2* &
                                    alpha(i,3) ) / unorm(i)
                     else
                        eps(i) = alpha(i,1)
                     end if
                  end do

               else if ( n==9 ) then

!              --- n = 9, biquadratic quadrilateral

                  do i = 1, 9
                     if ( unorm(i)>0d0 ) then
                         eps(i) = ( u1(i,1)**2*alpha(i,1)+2d0*u1(i,1)* &
                                    alpha(i,2)*u1(i,2) + u1(i,2)**2* &
                                    alpha(i,3) ) / unorm(i)
                     else
                        eps(i) = alpha(i,1)
                     end if
                  end do
               end if

            else

!           --- gauss integration rules

               do i = 1, m
                 if ( unorm(i)>0d0 ) then
                     eps(i) = ( u1(i,1)**2*alpha(i,1)+2d0*u1(i,1)* &
                                alpha(i,2)* u1(i,2) + u1(i,2)**2* &
                                alpha(i,3) ) / unorm(i)
                  else
                     eps(i) = alpha(i,1)
                  end if
               end do
            end if

         end if

!        --- Compute the distance h in the vertices

         do i = 1, m
            if ( unorm(i)>epsmac ) then
               uksi = u1(i,1) * ( xhalf(2,1) - xhalf(4,1) ) + &
                      u1(i,2) * ( xhalf(2,2) - xhalf(4,2) )
               ueta = u1(i,1) * ( xhalf(3,1) - xhalf(1,1) ) + &
                      u1(i,2) * ( xhalf(3,2) - xhalf(1,2) )
               h(i) = ( abs(uksi) + abs(ueta) ) / unorm(i)
            else
               hksi = sqrt ( (xhalf(2,1)-xhalf(4,1))**2 + &
                             (xhalf(2,2)-xhalf(4,2))**2 )
               heta = sqrt ( (xhalf(3,1)-xhalf(1,1))**2 + &
                             (xhalf(3,2)-xhalf(1,2))**2 )
               h(i) = 0.5d0 * ( hksi + heta )
            end if
         end do

!        --- Compute tau in the vertices

         do i = 1, m
            if ( eps(i)>1d-10 ) then

!           --- eps(i)>1d-10,  hence second order term plays a role

               beta = unorm(i)*h(i)/(2d0*eps(i))
               if ( beta<1d-10 ) then
                  xi = 0d0
               else

!              --- metupl = 3, 7 or 8    doubly assymptotic approximation

                  if ( abs(beta)>3d0 ) then
                     xi=sign(1d0,beta)
                  else
                     xi = beta / 3d0
                  end if
               end if
            else
               xi = 1d0
            end if
            if ( unorm(i)>epsmac ) then
               tau1(i) = h(i)*xi/(2d0*unorm(i))
            else
               tau1(i) = 0d0
            end if
         end do
         if ( metupl==8 ) then
            do k = 1, m
               tau1(k) = max(0d0,tau1(k)-tau(k))
            end do
         end if
         do i = 1, n
            do k = 1, m
               psi(i,k) = tau1(k) * ( u1(k,1)*dphidx(i,k,1)+ &
                                      u1(k,2)*dphidx(i,k,2) ) + psi(i,k)
            end do
         end do
      end if

1000  call erclos ( 'elpup2' )
      if ( debug ) then

!     --- Debug information

         call prinrl1 ( psi, m, n, 'psi' )
         write(irefwr,*) 'End elpup2'

      end if  ! ( debug )

      end
