      subroutine eluderiv ( u, dudx, dphidx, lsub, lsup, lstep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.0  date 13-11-2014
!
!   copyright (c) 2014-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the derivative of a given function in one element
!
! **********************************************************************
!
!                       KEYWORDS
!
!     fill
!     derivative
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: lsub, lsup, lstep
      double precision, intent(in) :: u(n,ncomp), dphidx(n,m,ndim)
      double precision, intent(out) :: dudx(m,ndim,ncomp)

!     dphidx         i    Array of length n x m x ndim  containing the
!                         derivatives
!                         of the basis functions in the sequence:
!                              d phi_i/ dx_j(xg_k) = dphidx (i,k,j);
!                         If the element is a linear simplex, the
!                         derivatives are constant and only k = 1 is filled.
!     dudx           o    Real m x ndim  array in which the values of the
!                         derivatives of the solution vector in the integration
!                         points are stored in the sequence:
!                         du / dx  (x ) = dudx ( k, j )
!                           j     k
!                         If jdercn = 0 in common block celint, then only
!                         dudx (1,*) is filled because dudx is then a constant
!                         per element
!                         dudx is only filled if jderiv>0
!     lstep          i    Step variable for vertices. Usually lstep = 1, &
!                         but when the solution must be computed in the vertices
!                         and the element is quadratic then lstep = 2
!     lsub           i    Under index for the direction of the derivative to be
!                         computed
!     lsup           i    Upper index for the direction of the derivative to be
!                         computed
!     u              i    array of length m containing the values of the
!                         function in the quadrature points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: j, k, l
      double precision :: som

!     j              Counting variable
!     k              Counting variable
!     l              General loop variable
!     som            Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'eluderiv' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from eluderiv'
         write(irefwr,1) 'ncomp, lsub, lsup, jdercn, lstep', &
                          ncomp, lsub, lsup, jdercn, lstep
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( jdercn==0 ) then

!     --- Derivatives of basis functions are constant, hence dudx is constant
!                   n
!         du/dx =  som  u   d phi  / dx
!              k   i=1   i       i     k

         do j = 1, ncomp
            do k = lsub, lsup
               som = dot_product ( u(1:n,j), dphidx(1:n,1,k) )
               do l = 1, m, lstep
                  dudx(l,k+1-lsub,j) = som
               end do
            end do
         end do

      else

!     --- dudx is variable

         do j = 1, ncomp
            do k = lsub, lsup
               do l = 1, m, lstep
                  dudx(l,k+1-lsub,j) = &
                     dot_product ( u(1:n,j), dphidx(1:n,l,k) )
               end do
            end do
         end do

      end if

1000  call erclos ( 'eluderiv' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End eluderiv'

      end if  ! ( debug )

      end subroutine eluderiv
