      subroutine sepelbasefunc
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 17-07-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute information about basis functions
!     Old subroutine elm800basefn
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmodulefunc
      use sepmodulecomio
      use sepmoduleelm
      use sepmodulebasefn
      use sepmoduleconsts
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, ndim, nnodes

!     i              General loop variable
!     ndim           Dimension of space
!     nnodes         number of nodes that are used
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINRL1        Print 2d real vector
!     SEPELP01       Compute information of basis functions
!                    Linear 1d elements
!     SEPELP02       Compute information of basis functions
!                    Quadratic 1d elements
!     SEPELP03       Compute information of basis functions
!                    Linear triangles
!     SEPELP04       Compute information of basis functions
!                    Quadratic triangles
!     SEPELP05       Compute information of basis functions
!                    Bi-linear quadrilaterals
!     SEPELP06       Compute information of basis functions
!                    Bi-quadratic quadrilateral (9 nodes)
!     SEPELP07       Compute information of basis functions
!                    Extended quadratic triangles
!     SEPELP09       Compute information of basis functions
!                    Extended bi-linear quadrilateral (5 nodes)
!     SEPELP10       Compute information of basis functions
!                    Extended linear triangle (4 nodes)
!     SEPELP11       Compute information of basis functions
!                    Linear tetrahedrons
!     SEPELP12       Compute information of basis functions
!                    Quadratic tetrahedrons
!     SEPELP13       Compute information of basis functions
!                    3d Tri-linear hexahedron (8 nodes)
!     SEPELP14       Compute information of basis functions
!                    3d Tri-Quadratic hexahedron (27 nodes)
!     SEPELP16       Compute information of basis functions
!                    Extended tri-Quadratic tetrahedron (15 nodes)
!     SEPELP17       Compute information of basis functions
!                    Extended tri-linear hexahedron (9 nodes)
!     SEPELP18       Compute information of basis functions
!                    Extended linear tetrahedron (5 nodes)
!     SEPELP21       Compute information of basis functions
!                    linear wedge element
!     SEPELP22       Compute information of basis functions
!                    quadratic wedge element
!     SEPELP30       Compute information of basis functions
!                    Third order 1d hermite element (2 nodes)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelbasefunc' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelbasefunc'
         write(irefwr,1) 'type_basisfunc, kstep', B%type_basisfunc, B%kstep
         write(irefwr,1) 'npelm, ndim, ishape', E%npelm, E%ndim, E%ishape
         write(irefwr,1) 'first', E%first
         write(irefwr,1) 'n, m, nd, nd2, nvert', B%n, B%m, B%nd, B%nd2, B%nvert
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      ndim = E%ndim
      nnodes = min(B%n,E%npelm)
      if ( E%ishape==7 ) nnodes = 7

      if ( debug ) write(irefwr,1) 'nnodes', nnodes
      if ( E%first==0 ) then

!     --- First call: allocate basis functions and related quantities

         call sepelallocb

!        --- quantities for module sepmodulefunc

         mquad = B%m
         x => B%xyzg(:,1)
         if ( B%nd>1 ) y => B%xyzg(:,2)
         if ( B%nd>2 ) z => B%xyzg(:,3)
         dimens = B%nd

      end if  ! ( E%first==0 )

!     --- Store coordinates in array xyz

      nnodes = min(B%n,E%npelm)
      do i = 1, ndim
         B%xyz(1:nnodes,i) = E%coor(i,E%nodes(1:nnodes))
      end do  ! i = 1, ndim

      select case (B%type_basisfunc)

         case(-10,0)

!        --- ishape = -10 or 0, Point element, fill array x

            B%w(1) = 1d0
            if ( B%cart==2) B%w(1) = 2d0*pi*B%xyz(1,1)
            B%jdiag = 1

         case(1)

!        --- 1d Linear element (2 nodes)

            call sepelp01

         case(2)

!        --- 1d quadratic element (3 nodes)

            call sepelp02

         case(3)

!        --- 2d Linear triangle (3 nodes)

            call sepelp03

         case(4)

!        --- 2d quadratic triangle (6 nodes)

            call sepelp04

         case(5)

!        --- 2d Bi-linear quadrilateral (4 nodes)

            call sepelp05

         case(6)

!        --- 2d Bi-quadratic quadrilateral (9 nodes)

            call sepelp06

         case(7)

!        --- 2d extended quadratic triangle (7 nodes)

            call sepelp07

         case(9)

!        --- 2d Extended bi-linear quadrilateral (5 nodes)

            call sepelp09

         case(10)

!        --- 2d Extended bi-linear quadrilateral (5 nodes)

            call sepelp10

         case(11)

!        --- 3d Linear tetrahedron (4 nodes)

            call sepelp11

         case(12, 15)

!        --- 3d Quadratic tetrahedron (10 nodes)

            call sepelp12

         case(13)

!        --- 3d Tri-linear hexahedron (8 nodes)

            call sepelp13

         case(14)

!        --- 3d Tri-quadratic hexahedron (27 nodes)

            call sepelp14

         case(16)

!        --- Extended 3d Tri-quadratic hexahedron (15 nodes)

            call sepelp16

         case(17)

!        --- Extended 3d Tri-linear hexahedron (9 nodes)

            call sepelp17

         case(18)

!        --- Extended 3d Linear tetrahedron (5 nodes)

            call sepelp18

         case(21)

!        --- 3d Linear wedge element (6 nodes)

            call sepelp21

         case(22)

!        --- 3d Quadratic wedge element (18 nodes)

            call sepelp22

         case(30)

!        --- Third order 1d hermite element (2 nodes)

            call sepelp30

         case default

!        --- Shape number not available

            call errint ( B%type_basisfunc, 1 )
            call errsub ( 800, 1, 0, 0 )

      end select  ! case (B%type_basisfunc)

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( B%xyz, ndim, nnodes, 'xyz' )
         call prinrl ( B%w, B%m, 'w' )
         write(irefwr,*) 'End sepelbasefunc'

      end if  ! ( debug )
      call erclos ( 'sepelbasefunc' )

      end subroutine sepelbasefunc
