      subroutine sepelcconvect1 ( veloc, ind, jdiag, phi )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-07-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix for convection equation
!     It is assumed that convection coefficients are stored in coefficients
!     1 to 3
!     The derivatives of the basis functions are constant
!     Complex case
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     convection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: ind(3), jdiag
      double precision, intent(in) :: phi(B%n,B%m)
      double complex, target :: veloc(B%m,3)

!     ind            i    See B%ind, contains information about the convection
!                         coefficients, in the same sequence as in coeffs
!     jdiag          i    if 1 the basis functions are 0 or 1 in the quadrature
!                         points
!                         if  0 they are general
!     phi            i    Contains the values of the basis functions in the
!                         quadrature points
!     veloc          i    contains the convection coefficients in the quadrature
!                         points:
!                         (.,1): u
!                         (.,2): v
!                         (.,3): w
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double complex :: h1, hx, hy, hz, workx(B%m), worky(B%m), workz(B%m)
      double precision, pointer :: phix(:), phiy(:), phiz(:)
      double complex, pointer :: u(:), v(:), w(:)
      integer :: i, j, k, m, n, kstep, nd

!     h1             help parameter to store element of convection matrix
!     hx             help parameter to store w*u_x
!     hy             help parameter to store w*u_y
!     hz             help parameter to store w*u_z
!     i              General loop variable
!     j              General loop variable
!     k              Loop variable (quadrature points)
!     kstep          step in loop over quadrature points
!     m              number of quadrature points
!     n              number of basis functions
!     nd             dimension of elements
!     phix           x derivative of basis functions phi
!     phiy           y derivative of basis functions phi
!     phiz           z derivative of basis functions phi
!     u              Contains the u-component of the velocity
!     v              Contains the v-component of the velocity
!     w              Contains the w-component of the velocity
!     workx          Work array to store w*u_x
!     worky          Work array to store w*u_y
!     workz          Work array to store w*u_z
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINCOMPL      Print 1d complex vector
!     PRINCOMPL1     Print 2d complex vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The convection matrix is given by
!
!     / u . grad(phi_j) phi_i with u the velocity vector
!
!     The elements of the element matrix are
!
!     s_ij = sum_k ( w_k  (ux_k dphidx_j(x_k) + uy_k dphidy_j(x_k) +
!                          uz_k dphidz_j(x_k) ) * phi_i(x_k) )
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelcconvect1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelcconvect1'
         write(irefwr,1) 'nd', B%nd
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      phix => B%phider(:,1,1)
      phiy => B%phider(:,1,2)
      phiz => B%phider(:,1,3)

      m = B%m
      n = B%n
      kstep = B%kstep
      nd = B%nd

      u => veloc(:,1)
      v => veloc(:,2)
      w => veloc(:,3)

      if ( debug ) then

         if ( ind(1)/=0 ) call princompl ( u, m, 'u' )
         if ( ind(2)/=0 ) call princompl ( v, m, 'v' )
         if ( ind(3)/=0 ) call princompl ( w, m, 'w' )

      end if  ! ( debug )

      if ( jdiag==1 ) then

!     --- basis functions satisfy phi_i(x_j) = delta_ij

         select case ( nd )

         case(1)

!        --- 1d dimensional element

            do i = 1, n
               hx = B%w(i)*u(i)
               do j = 1, n
                  E%cmat(j,i) = E%cmat(j,i)+hx*phix(j)
               end do  ! j = 1, n
            end do  ! i = 1, n

         case(2)

!        --- 2d dimensional element

            do i = 1, n
               if ( ind(1)/=0 ) hx = B%w(i)*u(i)
               if ( ind(2)/=0 ) hy = B%w(i)*v(i)
               do j = 1, n
                  h1 = 0d0
                  if ( ind(1)/=0 ) h1 = h1+hx*phix(j)
                  if ( ind(2)/=0 ) h1 = h1+hy*phiy(j)
                  E%cmat(j,i) = E%cmat(j,i)+h1
               end do  ! j = 1, n
            end do  ! i = 1, n

         case(3)

!        --- 3d dimensional element

            do i = 1, n
               if ( ind(1)/=0 ) hx = B%w(i)*u(i)
               if ( ind(2)/=0 ) hy = B%w(i)*v(i)
               if ( ind(3)/=0 ) hz = B%w(i)*w(i)
               do j = 1, n
                  h1 = 0d0
                  if ( ind(1)/=0 ) h1 = h1+hx*phix(j)
                  if ( ind(2)/=0 ) h1 = h1+hy*phiy(j)
                  if ( ind(3)/=0 ) h1 = h1+hz*phiz(j)
                  E%cmat(j,i) = E%cmat(j,i)+h1
               end do  ! j = 1, n
            end do  ! i = 1, n

         end select  ! case ( nd )

      else

!     --- Gauss rule  phi_i(x_j) # delta_ij

         if ( ind(1)/=0 ) then
            do k = kstep, m, kstep
               workx(k) = B%w(k) * u(k)
            end do
         end if  ! ( ind(1)/=0 )

         if ( ind(2)/=0 ) then
            do k = kstep, m, kstep
               worky(k) = B%w(k) * v(k)
            end do
         end if  ! ( ind(2)/=0 )

         if ( ind(3)/=0 ) then
            do k = kstep, m, kstep
               workz(k) = B%w(k) * w(k)
            end do
         end if  ! ( ind(2)/=0 )

         select case ( nd )

         case(1)

!        --- 1d dimensional element

            if ( ind(1)/=0 ) then
               do i = 1, n
                  do j = 1, n
                     h1 = 0d0
                     do k = kstep, m, kstep
                        h1 = h1+workx(k)*phix(j)*phi(i,k)
                     end do  ! k = kstep, m, kstep
                     E%cmat(j,i) = E%cmat(j,i)+h1
                  end do  ! j = 1, n
               end do  ! i = 1, n
            end if  ! ( ind(1)/=0 )

         case(2)

!        --- 2d dimensional element

            do i = 1, n
               do j = 1, n
                  h1 = 0d0
                  if ( ind(1)/=0 ) then
                     do k = kstep, m, kstep
                        h1 = h1+workx(k)*phix(j)*phi(i,k)
                     end do  ! k = kstep, m, kstep
                  end if  ! ( ind(1)/=0 )
                  if ( ind(2)/=0 ) then
                     do k = kstep, m, kstep
                        h1 = h1+worky(k)*phiy(j)*phi(i,k)
                     end do  ! k = kstep, m, kstep
                  end if  ! ( ind(2)/=0 )
                  E%cmat(j,i) = E%cmat(j,i)+h1
               end do  ! j = 1, n
            end do  ! i = 1, n

         case(3)

!        --- 3d dimensional element

            do i = 1, n
               do j = 1, n
                  h1 = 0d0
                  if ( ind(1)/=0 ) then
                     do k = kstep, m, kstep
                        h1 = h1+workx(k)*phix(j)*phi(i,k)
                     end do  ! k = kstep, m, kstep
                  end if  ! ( ind(1)/=0 )
                  if ( ind(2)/=0 ) then
                     do k = kstep, m, kstep
                        h1 = h1+worky(k)*phiy(j)*phi(i,k)
                     end do  ! k = kstep, m, kstep
                  end if  ! ( ind(2)/=0 )
                  if ( ind(3)/=0 ) then
                     do k = kstep, m, kstep
                        h1 = h1+workz(k)*phiz(j)*phi(i,k)
                     end do  ! k = kstep, m, kstep
                  end if  ! ( ind(3)/=0 )
                  E%cmat(j,i) = E%cmat(j,i)+h1
               end do  ! j = 1, n
            end do  ! i = 1, n

         end select  ! case ( nd )

      end if  ! ( jdiag==1 )

1000  if ( debug ) then

!     --- Debug information

         call princompl1 ( E%cmat, n, n, 'element matrix' )
         write(irefwr,*) 'End sepelcconvect1'

      end if  ! ( debug )
      call erclos ( 'sepelcconvect1' )

      end subroutine sepelcconvect1
