      subroutine sepelcload ( source )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-07-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the element vector with a load
!     Complex case
! **********************************************************************
!
!                       KEYWORDS
!
!     element_vector
!     elasticity_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebasefn
      use sepmoduleelm
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double complex, intent(in) :: source(B%m)

!     source         i    right-hand side
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, n, kstep, k
      double complex :: work(B%m), h

!     h              Help variable to store a constant temporarily
!     i              General loop variable
!     k              Counting variable
!     kstep          Step in loop
!     n              Number of basis functions
!     work           work array to store help arrays
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINCOMPL      Print 1d complex vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The part of the element matrix is given by
!
!     / source phi_i  d Omega
!
!     The numerical values are given by
!
!     sum_k w(k) {source phi }(x_k)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelcload' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelcload'
         write(irefwr,1) 'jdiag', B%jdiag
         call princompl ( B%w, B%m, 'w' )
         call princompl ( source, B%m, 'source' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      n = B%n
      kstep = B%kstep
      if ( debug ) write(irefwr,1) 'n, kstep', B%n, B%kstep

!     --- Store w source in work

      work = source*B%w

      if ( debug ) call princompl ( work, B%m, 'work' )

      if ( B%jdiag==1 ) then

!     --- Special case: phi_i(x_j) = delta_ij

         if ( kstep==1 ) then
            E%cvec = E%cvec+work
         else
            do k = kstep, n, kstep
               E%cvec(k) = E%cvec(k)+work(k)
            end do  ! k = kstep, n, kstep
         end if  ! ( kstep==1 )

      else

!     --- General case

         do i = 1, n
            h = sum(work(:)*B%phi(i,:))
            E%cvec(i) = E%cvec(i)+h
         end do  ! i = 1, n

      end if  ! ( B%jdiag==1 )

1000  if ( debug ) then

!     --- Debug information

         call princompl ( E%cvec, n, 'element vector' )
         write(irefwr,*) 'End sepelcload'

      end if  ! ( debug )
      call erclos ( 'sepelcload' )

      end subroutine sepelcload
