      double precision function sepelic00 ( )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-06-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute a volume integral over one element depending on icheli
!     Old subroutine elic00
! **********************************************************************
!
!                       KEYWORDS
!
!     element_vector
!     integral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmoduleelem
      use sepmodulebasefn
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

!     sepelic00      o    Computed element integral
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, pointer :: dudx(:), f(:), w(:), u(:)
      double precision :: work(B%m)
      integer i

!     dudx           Real m x ndim  array in which the values of the
!                    derivatives of the solution vector in the quadrature
!                    points are stored in the sequence:
!                    du_j / dx_k (x ) = dudx ( k, j )
!     f              function to be integrated in quadrature points
!     i              Counting variable
!     u              solution to be integrated in quadrature points
!     w              weights for the quadrature
!     work           Work array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelic00' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelic00'
         write(irefwr,1) 'icheli', icheli
         write(irefwr,*) 'derivconst ', B%derivconst
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( icheli/=7 ) then
         f => E%coeffs(:,1)
         u => uprev(:,1)
      end if  ! ( icheli/=7 )

      w => B%w
      m = B%m
      kstep = B%kstep

      if ( debug ) then
         write(irefwr,1) 'm, kstep', m, kstep
         call prinrl ( f, m, 'f' )
         call prinrl ( w, m, 'w' )
         call prinrl ( u, m, 'u' )
      end if  ! ( debug )

!     --- Store f, uf or f du/dx into work

      work = 0d0

      select case ( icheli )
      case (1)

!     --- icheli = 1, only f

         work = f

      case (2)

!     --- icheli = 2, f u

         work = f*u

      case (3:5)

!     --- icheli = 3-5, f du/dx

         dudx => gradoldvec(:,icheli-2,1)

         if ( debug ) call prinrl ( dudx, m, 'dudx' )
         if ( B%derivconst ) then

!        --- Derivative constant

            work = f*dudx(1)

         else

!        --- Derivative variable

            do i = kstep, m, kstep
               work(i) = f(i) * dudx(i)
            end do

         end if

      case (8)

!     --- icheli = 8, f u^2

         do i = kstep, m, kstep
            work(i) = f(i) * u(i)**2
         end do

      end select  ! case ( icheli )

      if ( debug ) call prinrl ( work, m, 'work' )

      if ( icheli==7 ) then

!     --- icheli = 7, compute integral

         sepelic00 = sum(w(kstep:m:kstep))

      else if ( kstep==1 ) then

         sepelic00 = dot_product ( work, w )

      else

         sepelic00 = dot_product ( work(kstep:m:kstep), w(kstep:m:kstep) )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,2) 'sepelic00', sepelic00
         write(irefwr,*) 'End sepelic00'

      end if  ! ( debug )
      call erclos ( 'sepelic00' )

      end function sepelic00
