      subroutine sepelinstrain2 ( eps0, Dw )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-07-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute contribution to element vector due to initial strain (3d)
!     Old subroutine el8103
! **********************************************************************
!
!                       KEYWORDS
!
!     element_vector
!     elasticity_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision, intent(in) :: eps0(m,E%dimD), Dw(E%dimD,E%dimD,m)

!     Dw             i    Dmat * w
!     eps0           i    initial strain in quadrature points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, pointer :: v(:)
      integer :: i
      double precision, pointer :: phix(:,:), phiy(:,:), phiz(:,:)

!     i              Counting variable
!     phix           Array of length n x m x ndim  containing the
!                    derivatives of the basis functions in the sequence:
!                    d phi / dx (xg ) = phix (i,k,1);
!                         i        k
!                    d phi / dy (xg ) = phix (i,k,2);
!                         i        k
!                    d phi / dz (xg ) = phix (i,k,3);
!                         i        k
!                    If the element is a linear tetrahedron (n=4), then
!                    the derivatives are constant and only k = 1 is filled.
!     phiy           dphi/dy
!     phiz           dphi/dz
!     v              Sub element vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelinstrain2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelinstrain2'
         write(irefwr,*) 'derivconst ', B%derivconst
         call prinrl1 ( eps0, 6, m, 'eps0' )
         call prinrl ( E%vec, 3*n, 'vec/1' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      phix => B%phider(:,:,1)
      phiy => B%phider(:,:,2)
      phiz => B%phider(:,:,3)

      if (  B%derivconst ) then

!     --- Derivatives of basis functions constant

         v => E%vec(1:3*n:3)
         do i = 1, n
            v(i) = v(i)+phix(i,1)*sum(Dw(1,1,:)*eps0(:,1) + &
                                      Dw(1,2,:)*eps0(:,2) + &
                                      Dw(1,3,:)*eps0(:,3)) + &
                        phiy(i,1)*sum(Dw(4,4,:)*eps0(:,4)) + &
                        phiz(i,1)*sum(Dw(6,6,:)*eps0(:,6))
         end do  ! i = 1, n
         v => E%vec(2:3*n:3)
         do i = 1, n
            v(i) = v(i)+phiy(i,1)*sum(Dw(2,1,:)*eps0(:,1) + &
                                      Dw(2,2,:)*eps0(:,2) + &
                                      Dw(2,3,:)*eps0(:,3)) + &
                        phix(i,1)*sum(Dw(4,4,:)*eps0(:,4)) + &
                        phiz(i,1)*sum(Dw(5,5,:)*eps0(:,5))
         end do  ! i = 1, n
         v => E%vec(3:3*n:3)
         do i = 1, n
            v(i) = v(i)+phiz(i,1)*sum(Dw(3,1,:)*eps0(:,1) + &
                                      Dw(3,2,:)*eps0(:,2) + &
                                      Dw(3,3,:)*eps0(:,3)) + &
                        phiy(i,1)*sum(Dw(5,5,:)*eps0(:,5)) + &
                        phix(i,1)*sum(Dw(6,6,:)*eps0(:,6))
         end do  ! i = 1, n

      else

!     --- Derivatives of basis functions not constant

         v => E%vec(1:3*n:3)
         do i = 1, n
            v(i) = v(i)+sum(phix(i,:)*(Dw(1,1,:)*eps0(:,1) + &
                                      Dw(1,2,:)*eps0(:,2) + &
                                      Dw(1,3,:)*eps0(:,3))) + &
                        sum(phiy(i,:)*Dw(4,4,:)*eps0(:,4)) + &
                        sum(phiz(i,:)*Dw(6,6,:)*eps0(:,6))
         end do  ! i = 1, n
         v => E%vec(2:3*n:3)
         do i = 1, n
            v(i) = v(i)+sum(phiy(i,:)*(Dw(2,1,:)*eps0(:,1) + &
                                       Dw(2,2,:)*eps0(:,2) + &
                                       Dw(2,3,:)*eps0(:,3))) + &
                        sum(phix(i,:)*Dw(4,4,:)*eps0(:,4)) + &
                        sum(phiz(i,:)*Dw(5,5,:)*eps0(:,5))
         end do  ! i = 1, n
         v => E%vec(3:3*n:3)
         do i = 1, n
            v(i) = v(i)+sum(phiz(i,:)*(Dw(3,1,:)*eps0(:,1) + &
                                       Dw(3,2,:)*eps0(:,2) + &
                                       Dw(3,3,:)*eps0(:,3))) + &
                        sum(phiy(i,:)*Dw(5,5,:)*eps0(:,5)) + &
                        sum(phix(i,:)*Dw(6,6,:)*eps0(:,6))
         end do  ! i = 1, n

      end if  ! (  B%derivconst )

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( E%vec, 3*n, 'vec/2' )
         write(irefwr,*) 'End sepelinstrain2'

      end if  ! ( debug )
      call erclos ( 'sepelinstrain2' )

      end subroutine sepelinstrain2
