      subroutine sepelinstress2 ( sigma0 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-07-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute contribution to element vector due to initial stress (3d)
!     Old subroutine el8104
! **********************************************************************
!
!                       KEYWORDS
!
!     element_vector
!     elasticity_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision, intent(in) :: sigma0(m,6)

!     sigma0         i    initial stress in quadrature points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, pointer :: v(:)
      double precision, pointer :: phix(:,:), phiy(:,:), phiz(:,:)
      integer :: i, nd
      double precision :: sigma(m,6), sig(6)

!     i              Counting variable
!     nd             Dimension of Matrix D
!     phix           Array of length n x m x ndim  containing the
!                    derivatives of the basis functions in the sequence:
!                    d phi / dx (xg ) = phix (i,k,1);
!                         i        k
!                    d phi / dy (xg ) = phix (i,k,2);
!                         i        k
!                    d phi / dz (xg ) = phix (i,k,3);
!                         i        k
!                    If the element is a linear tetrahedron (n=4), then
!                    the derivatives are constant and only k = 1 is filled.
!     phiy           dphi/dy
!     phiz           dphi/dz
!     sig            sum of stress tensor components over quadrature points
!     sigma          stress tensor in quadrature points
!     v              Sub element vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelinstress2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelinstress2'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      phix => B%phider(:,:,1)
      phiy => B%phider(:,:,2)
      phiz => B%phider(:,:,3)
      do i = 1, 6
         sigma(:,i) = B%w(:)*sigma0(:,i)
      end do  ! i = 1, nd

      if (  B%derivconst ) then

!     --- Derivatives of basis functions constant

         do i = 1, 6
           sig(i) = sum(sigma(:,i))
         end do  ! i = 1, 6

         v => E%vec(1:3*n:3)
         do i = 1, n
            v(i) = v(i)-phix(i,1)*sig(1) - phiy(i,1)*sig(4) - &
                        phiz(i,1)*sig(6)
         end do  ! i = 1, n

         v => E%vec(2:3*n:3)
         do i = 1, n
             v(i) = v(i)-phiy(i,1)*sig(2) - phix(i,1)*sig(4) - &
                         phiz(i,1)*sig(5)
         end do  ! i = 1, n

         v => E%vec(3:3*n:3)
         do i = 1, n
             v(i) = v(i)-phiz(i,1)*sig(3) - phiy(i,1)*sig(5) - &
                         phix(i,1)*sig(6)
         end do  ! i = 1, n

      else

!     --- Derivatives of basis functions not constant

         v => E%vec(1:3*n:3)
         do i = 1, n
             v(i) = v(i)-sum(phix(i,:)*sigma(:,1) + &
                             phiy(i,:)*sigma(:,4) + &
                             phiz(i,:)*sigma(:,6) )
         end do  ! i = 1, n

         v => E%vec(2:3*n:3)
         do i = 1, n
             v(i) = v(i)-sum(phiy(i,:)*sigma(:,2) + &
                             phix(i,:)*sigma(:,4) + &
                             phiz(i,:)*sigma(:,5) )
         end do  ! i = 1, n

         v => E%vec(3:3*n:3)
         do i = 1, n
             v(i) = v(i)-sum(phiz(i,:)*sigma(:,3) + &
                             phiy(i,:)*sigma(:,5) + &
                             phix(i,:)*sigma(:,6) )
         end do  ! i = 1, n

      end if  ! (  B%derivconst )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelinstress2'

      end if  ! ( debug )
      call erclos ( 'sepelinstress2' )

      end subroutine sepelinstress2
