      subroutine sepellaplace ( coeff )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-08-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the element matrix for the Laplace equation
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision, intent(in) :: coeff(B%m)

!     coeff          i    coefficient in diffusion equation
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer n, m, nd
      double precision :: h, h1, work(B%m)
      double precision, pointer :: phix(:,:), phiy(:,:), phiz(:,:), mat(:,:)
      integer :: i, j, k

!     h              Help variable to store a constant temporarily
!     h1             Help variable to store a constant temporarily
!     i              General loop variable
!     j              General loop variable
!     k              Loop variable (quadrature points)
!     m              Number of quadrature points
!     mat            sub element matrix
!     n              Number of basis functions
!     nd             Dimension of basis functions
!     phix           Contains x-derivatives of basis functions in quadrature
!                    points
!     phiy           Contains y-derivatives of basis functions in quadrature
!                    points
!     phiz           Contains z-derivatives of basis functions in quadrature
!                    points
!     work           work array to store help arrays
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The element matrix is given by
!
!     s_ij = / grad phi_i grad phi j dOmega
!
!     or with numerical quadrature:
!
!     s_ij = w_k grad phi_i(x^k) grad phi_j(x^k)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepellaplace' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepellaplace'
         write(irefwr,*) 'derivconst ', B%derivconst
  1      format ( a, 1x, (10i8) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      m = B%m
      n = B%n
      nd = B%nd
      phix => B%phider(:,:,1)
      if ( nd>1 ) phiy => B%phider(:,:,2)
      if ( nd>2 ) phiz => B%phider(:,:,3)
      mat => E%submat

      if ( debug ) then
         write(irefwr,1) 'itype, nd', E%itype, nd
         call prinrl ( B%w, m, 'w' )
         call prinrl1 ( phix, m, n, 'dphi/dx' )
         if ( nd>1 ) call prinrl1 ( phiy, m, n, 'dphi/dy' )
         if ( nd>2 ) call prinrl1 ( phiz, m, n, 'dphi/dz' )
      end if  ! ( debug )

      if ( B%derivconst ) then

!     --- Derivatives of basis functions constant

         if ( E%itype==700 .or. E%itype==701 ) then

!        --- Laplace or Poisson equation, coefficient = 1

            h = sum ( B%w )

         else

!        --- Diffusion equation

            h = sum ( B%w*coeff )

         end if  ! ( E%itype==700 .or. E%itype==701 )

         select case ( nd )

         case(1)

!        --- 1d dimensional element

            do i = 1, n
               do j = 1, i
                  h1 = h*phix(i,1)*phix(j,1)
                  mat(i,j) = mat(i,j)+h1
                  if ( j/=i ) mat(j,i) = mat(j,i)+h1
               end do  ! j = 1, i
            end do  ! i = 1, n

         case(2)

!        --- 2d dimensional element

            do i = 1, n
               do j = 1, i
                  h1 = h*(phix(i,1)*phix(j,1)+phiy(i,1)*phiy(j,1))
                  mat(i,j) = mat(i,j)+h1
                  if ( j/=i ) mat(j,i) = mat(j,i)+h1
               end do  ! j = 1, i
            end do  ! i = 1, n

         case(3)

!        --- 3d dimensional element

            do i = 1, n
               do j = 1, i
                  h1 = h*(phix(i,1)*phix(j,1)+phiy(i,1)*phiy(j,1)+ &
                          phiz(i,1)*phiz(j,1))
                  mat(i,j) = mat(i,j)+h1
                  if ( j/=i ) mat(j,i) = mat(j,i)+h1
               end do  ! j = 1, i
            end do  ! i = 1, n

         end select  ! case ( nd )

      else

!     --- Derivatives of basis functions are variable

         if ( E%itype==700 .or. E%itype==701 ) then

!        --- Laplace or Poisson equation

            work = B%w

         else

!        --- Diffusion equation

            if ( B%kstep==1 ) then
               work = B%w*coeff
            else
               do k = B%kstep, m, B%kstep
                  work(k) = B%w(k)*coeff(k)
               end do  ! k = B%kstep, m, B%kstep
            end if  ! ( B%kstep==1 )

         end if  ! ( E%itype==700 .or. E%itype==701 )

         select case ( nd )

         case(1)

!        --- 1d dimensional element

            do i = 1, n
               do j = 1, i
                  h1 = sum ( work(:)*phix(i,:)*phix(j,:) )
                  mat(i,j) = mat(i,j)+h1
                  if ( j/=i ) mat(j,i) = mat(j,i)+h1
               end do  ! j = 1, i
            end do  ! i = 1, n

         case(2)

!        --- 2d dimensional element

            if ( B%kstep==1 ) then

!           --- Standard case, all quadrature points are used

               do i = 1, n
                  do j = 1, i
                     h1 = sum ( work(:)*(phix(i,:)*phix(j,:)+&
                                         phiy(i,:)*phiy(j,:)) )
                     mat(i,j) = mat(i,j)+h1
                     if ( j/=i ) mat(j,i) = mat(j,i)+h1
                  end do  ! j = 1, i
               end do  ! i = 1, n

            else

!           --- Special case of quadratic triangle, where only the mid-points
!               are used

               do i = 1, n
                  do j = 1, i
                     h1 = 0d0
                     do k = B%kstep, m, B%kstep
                        h1 = h1 + work(k)*(phix(i,k)*phix(j,k)+&
                                           phiy(i,k)*phiy(j,k))
                     end do  ! k = B%kstep, m, B%kstep
                     mat(i,j) = mat(i,j)+h1
                     if ( j/=i ) mat(j,i) = mat(j,i)+h1
                  end do  ! j = 1, i
               end do  ! i = 1, n

            end if  ! ( B%kstep==1 ) then

         case(3)

!        --- 3d dimensional element

            do i = 1, n
               do j = 1, i
                  h1 = sum ( work(:)*(phix(i,:)*phix(j,:)+&
                                      phiy(i,:)*phiy(j,:)+&
                                      phiz(i,:)*phiz(j,:)) )
                  mat(i,j) = mat(i,j)+h1
                  if ( j/=i ) mat(j,i) = mat(j,i)+h1
               end do  ! j = 1, i
            end do  ! i = 1, n

         end select  ! case ( nd )

      end if  ! ( B%derivconst )

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( mat, n, n, 'element matrix' )
         write(irefwr,*) 'End sepellaplace'

      end if  ! ( debug )
      call erclos ( 'sepellaplace' )

      end subroutine sepellaplace
