      subroutine sepelm220
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 17-07-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix and vector for simple beam element
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     beam
!     elasticity_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, pointer :: elast(:)
      double precision, pointer :: secmom(:)
      double precision, pointer :: load(:)
      double precision, pointer :: EI(:)

!     EI             Contains product of E and I in quadrature points
!     elast          Contains the elastic modulus in quadrature points (E)
!     load           Contains the load in quadrature points
!     secmom         Contains the second moment of area in quadrature points (I)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     SEPEL0220      Fill element independent quantities for element type 220
!     SEPELBASEFUNC  Compute information about basis functions
!     SEPELBIHAR     Fill the element matrix for the biharmonic equation
!     SEPELDEALLOC   Deallocate coefficients and basis functions for element
!                    subroutines
!     SEPELFILLVAR   Fill space dependent coefficients
!     SEPELLOAD      Fill the element vector with a load
!     SEPELOLDVECS   Fill old vectors for element
!     SEPPRINELMAT   Print element matrix and vector
! **********************************************************************
!
!                       I/O
!
!     The following coefficients are used
!
!     icoef 3 = integration rule
!     icoef 4 = coordinate system
!     coef  6 = E
!     coef  7 = I
!     coef  8 = load
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelm220' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelm220'
         write(irefwr,1) 'first, last', E%first, E%last
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Define element independent quantities
!         only for first element

      if ( E%first==0 ) call sepel0220

!     --- Map relation between coefficients and specific names

      elast => E%coeffs(:,1)
      secmom => E%coeffs(:,2)
      load => E%coeffs(:,3)
      EI => E%coeffs(:,4)

!     --- Fill basis functions

      call sepelbasefunc

!     --- Fill old vectors in quadrature points if necessary

      if ( E%compold>0 ) call sepeloldvecs

      if ( E%isupcfvar>=E%isubcfvar ) then

!     --- Fill variable coefficients if necessary

         call sepelfillvar

         if ( debug ) call prinrl ( load, B%m, 'load' )

      end if  ! ( E%isupcfvar>=E%isubcfvar )

!     --- Compute EI

      if ( E%first==0 .or. E%ind(1)>0 .or. E%ind(2)>0 ) EI = elast*secmom

      if ( debug ) call prinrl ( EI, B%m, 'EI' )

!     --- Fill element matrix

      call sepelbihar ( EI )

!     --- Fill element vector

      if ( E%ind(3)/=0 ) then
         E%subvec => E%vec
         call sepelload ( load )
      else
         E%notvec = 1  ! element vector is zero
      end if  ! ( E%ind(3)/=0 )

!     --- Remove basis functions and coefficients in case of last call

      if ( E%last==1 ) call sepeldealloc

1000  call erclos ( 'sepelm220' )
      if ( debug ) then

!     --- Debug information

         call sepprinelmat
         write(irefwr,*) 'End sepelm220'

      end if  ! ( debug )

      end subroutine sepelm220
