      subroutine sepelm300
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 31-01-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix and vector for Reynolds equation
!     Both compressible and incompressible bearing
!     Old subroutine elm300
! **********************************************************************
!
!                       KEYWORDS
!
!     bearing
!     compressible
!     incompressible
!     element_matrix
!     element_vector
!     lubrication
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, pointer :: h(:), mu(:), diffusion(:)

!     diffusion      Contains the parameter h^3/(12 mu)
!     h              Contains the function h in the equation (film thickness)
!     mu             viscosity
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL1        Print 2d real vector
!     SEPEL0300      Performs initializations for subroutine sepelm300
!     SEPELBASEFUNC  Compute information about basis functions
!     SEPELBEARCONS  Compute constant part of matrix for Reynolds equation
!     SEPELBEARCONV  Compute convection part of Reynolds equation
!     SEPELBEARDIFF  Compute diffusion part of Reynolds equation
!     SEPELBEARRHS   Compute right-hand side part of Reynolds equation
!     SEPELDEALLOC   Deallocate coefficients and basis functions for element
!                    subroutines
!     SEPELFILLVAR   Fill space dependent coefficients
!     SEPELOLDVECS   Fill old vectors for element
!     SEPPRINELMAT   Print element matrix and vector
! **********************************************************************
!
!                       I/O
!
!     The following coefficients are used
!
!     1:  Type of bearing
!         Possible values:
!         0: incompressible bearing
!         1: Compressible bearing (adiabatic)
!         2: incompressible bearing (mass conserving; state variable rho)
!     2:  Type of linearization in case of a compressible bearing
!         Possible values:
!         0: Newton linearization
!         1: Picard linearization
!     3:  Type of numerical integration
!         Possible values:
!         0: default value, defined by element
!     4:  Type of co-ordinate system
!         This parameter consists of 2 parts: jcart and jtransform
!         according to jcart + 10*jtransform
!         Possible values:
!         jcart:
!         0: Cartesian co-ordinates
!         jtransform: transformation from 3d to 2d
!         0:  standard, no special transformation
!         1:  transformation from 3d to 2d, with z constant
!             first coordinate becomes x, second y
!         2:  transformation from 3d to 2d, with x constant
!             first coordinate becomes y, second z
!         3:  transformation from 3d to 2d, with y constant
!             first coordinate becomes x, second z
!         4:  transformation from 3d to 2d, using cylinder co-ordinates
!             xsi = atan2(x,y), eta = z
!     5:  Not yet used (must be 0)
!     6:  h (thickness of the layer)
!     7:  mu (viscosity)
!     8:  ht or pht (squeeze term)
!     9:  k  (constant in equation)
!    10:  p0 (reference pressure)
!    11:  u_1 1 (first component of velocity of "lower surface")
!    12:  u_1 2 (second component of velocity of "lower surface", 2D only)
!    13:  not yet used (must be zero)
!    14:  rho (incompressible case if f#0 or mass conserving: state)
!    15:  RT (compressible case if f#0)
!    16:  f1 (first component of extra force)
!    17:  f2 (second component of extra force)
!    18:  Not yet used (must be 0)
!    19:  u_2 1 (first component of velocity of "upper surface")
!    20:  u_2 2 (second component of velocity of "upper surface", 2D only)
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!       Differential equation:
!    Incompressible bearing:
!    div ( -h^3/(12 mu) (grad p - rho f) +h/2 u ) + ht + k (p-p0 ) = 0
!    with u = u_1 + u_2
!    Both f and u are vectors
!    Compressible bearing (adiabatic)
!    div ( -p h^3/(12 mu) (grad p - p/RT f) + ph/2 u ) + pht + k(p^2-p0^2) = 0
!        The following formulae are used:
!           S   =  S      +  S   +  S
!                   alpha     u      beta
!           F   =  F1 + F2
!        Incompressible bearing:
!           S     (i,j) = / h^3/(12mu) grad phi  . grad phi  de
!            alpha        e                    j           i
!           S     (i,j) = 0
!            u
!           S     (i,j) = / k phi   phi  de
!            beta        e       j     i
!           F1(i)   =  /  (kp0 - ht) phi  de
!                      e                i
!           F2(i)   =  /  (h^3/12mu rho f +h/2 u) . gradphi  de
!                      e                                   i
!        Compressible bearing (Picard linearization):
!           S     (i,j) =  / ph^3/(12mu) grad phi  . grad phi  de
!            alpha         e                     j           i
!           S     (i,j) = / -(h/2 u +ph^3/12mu f/RT) phi . grad phi  de
!            u            e       -                     j          i
!           S     (i,j) = / kp phi   phi  de
!            beta        e        j     i
!           F1(i)   =  /  (k p0^2 - pht) phi  de
!                      e                    i
!           F2(i)   =  0
!        Compressible bearing (Newton linearization):
!           S     (i,j) = S     (i,j) Picard
!            alpha         alpha
!           S     (i,j) = /(-h/2 u + h^3/12mu(-2pf/RT+gradp))phi . grad phi  de
!            u            e                                     j          i
!           S     (i,j) = / 2kp phi   phi  de
!            beta        e         j     i
!           F1(i)   =  /  (k(p0^2+p^2) - pht) phi  de
!                      e                         i
!           F2(i)   =  /  (h^3/12mu p (gradp-pf). gradphi  de
!                      e                                 i
!        Each integral is approximated by a numerical rule of the shape:
!        / int(x) dx = sum  w  int(x )
!        e              k    k      k
!        the factors w  form the weights for the numerical integration
!                     k
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelm300' )
      debug = .false. .and. ioutp>=0

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelm300'
         write(irefwr,1) 'ielem', E%ielem
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      E%mat = 0d0
      E%vec = 0d0

!     --- Define element independent quantities
!         only for first element

      if ( E%first==0 ) call sepel0300

      E%submat => E%mat
      E%subvec => E%vec

!     --- Fill basis functions

      call sepelbasefunc

!     --- Fill old vectors in quadrature points if necessary

      if ( E%compold>0 ) call sepeloldvecs

      if ( debug ) then
         write(irefwr,1) 'isupcfvar, isubcfvar', E%isupcfvar, E%isubcfvar
         write(irefwr,*) 'B%fillqmat ', B%fillqmat
      end if  ! ( debug )

!     --- Fill all coefficients

      if ( E%isupcfvar>=E%isubcfvar ) call sepelfillvar

      h => E%coeffs(:,E%indh)
      mu => E%coeffs(:,E%indvisc)
      diffusion => E%coeffs(:,E%inddiff)

      diffusion = h**3/(12d0*mu)

!     --- Fill parts of matrix if necessary

      call sepelbeardiff  ! diffusion part
      if ( debug ) call prinrl1 ( E%mat, B%n, B%n, 'elemmt/1')

      if ( E%typebear>0 ) call sepelbearconv  ! convection part
      if ( debug ) call prinrl1 ( E%mat, B%n, B%n, 'elemmt/2')

      if ( E%ind(E%indconst)>0 ) call sepelbearcons
      if ( debug ) call prinrl1 ( E%mat, B%n, B%n, 'elemmt/3')

      call sepelbearrhs  ! rhside

      if ( E%typebear==2 ) then

!     --- ibear = 2, mass conserving method
!         For the time being we store the contribution of the term
!         involving drho/dt into the mass matrix stored as
!         diagonal matrix
!         This might be changed in the future

         E%mmat(:,1) = h*B%w

      end if  ! ( E%typebear==2 )

!     --- Remove basis functions and coefficients in case of last call

      if ( debug ) write(irefwr,1) 'last', E%last
      if ( E%last==1 ) call sepeldealloc

1000  if ( debug ) then

!     --- Debug information

         call sepprinelmat
         write(irefwr,*) 'End sepelm300'

      end if  ! ( debug )
      call erclos ( 'sepelm300' )

      end subroutine sepelm300
