      subroutine sepelm701
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-08-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix and vector for Poisson equation
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, pointer :: source(:)

!     source         Contains the source in quadrature points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     SEPEL0701      Fill element independent quantities for element type 701
!     SEPELBASEFUNC  Compute information about basis functions
!     SEPELDEALLOC   Deallocate coefficients and basis functions for element
!                    subroutines
!     SEPELFILLVAR   Fill space dependent coefficients
!     SEPELLAPLACE   Compute the element matrix for the Laplace equation
!     SEPELLOAD      Fill the element vector with a load
!     SEPELMASSMAT   Fill the element mass matrix; one unknown per node
!     SEPELOLDVECS   Fill old vectors for element
!     SEPPRINELMAT   Print element matrix and vector
! **********************************************************************
!
!                       I/O
!
!     The following coefficients are used
!
!     icoef 3 = integration rule
!     icoef 4 = coordinate system
!     coef  6 = source
!     coef  7 = density (rho)
!     coef  8 = heat_capacity (cp)
!     coef  9 = rho cp
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelm701' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelm701'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      E%mat = 0d0
      E%vec = 0d0

!     --- Define element independent quantities
!         only for first element

      if ( E%first==0 ) call sepel0701

      E%submat => E%mat
      E%subvec => E%vec

!     --- Map relation between coefficients and specific names

      source => E%coeffs(:,E%indsource)

!     --- Fill basis functions

      call sepelbasefunc

!     --- Fill old vectors in quadrature points if necessary

      if ( E%compold>0 ) call sepeloldvecs

      if ( debug ) &
        write(irefwr,1) 'isupcfvar, isubcfvar', E%isupcfvar, E%isubcfvar

      if ( E%isupcfvar>=E%isubcfvar ) then

!     --- Fill variable coefficients if necessary

         call sepelfillvar

!        --- Compute rho cp if necessary

         if ( E%ind(E%indrho)/=0 .or. E%ind(E%indcp)/=0 ) then
            E%coeffs(:,E%indrhocp) = E%coeffs(:,E%indrho)*E%coeffs(:,E%indcp)
            E%coeffsnode(:,E%indrhocp) = &
               E%coeffsnode(:,E%indrho)*E%coeffsnode(:,E%indcp)
         end if  ! ( E%ind(E%indrho)/=0 .or. E%ind(E%indcp)/=0 )

         if ( debug ) call prinrl ( source, B%m, 'source' )

      end if  ! ( E%isupcfvar>=E%isubcfvar )

!     --- Fill element matrix

      call sepellaplace ( source )

!     --- Fill element vector

      if ( E%sourceterm ) call sepelload ( source ) ! source term

!     --- Fill element mass matrix if necessary

      if ( E%imas>0 ) call sepelmassmat

!     --- Remove basis functions and coefficients in case of last call

      if ( E%last==1 ) call sepeldealloc

1000  if ( debug ) then

!     --- Debug information

         call sepprinelmat
         write(irefwr,*) 'End sepelm701'

      end if  ! ( debug )
      call erclos ( 'sepelm701' )

      end subroutine sepelm701
