      subroutine sepelm703
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-08-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix and vector for the convection-diffusion equation
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, pointer :: source(:)

!     source         Contains the source in quadrature points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     SEPEL0703      Fill element independent quantities for element type 703
!     SEPELBASEFUNC  Compute information about basis functions
!     SEPELBASEUPW   Compute the modified basis functions psi due to upwind
!     SEPELCONVECT   Compute the element matrix for the convection equation
!     SEPELDEALLOC   Deallocate coefficients and basis functions for element
!                    subroutines
!     SEPELDIFFUSION Compute the element matrix for the diffusion equation
!     SEPELFILLVAR   Fill space dependent coefficients
!     SEPELLOAD      Fill the element vector with a load
!     SEPELMASSMAT   Fill the element mass matrix; one unknown per node
!     SEPELMULTRHO   Multiply velocity by rho cp
!     SEPELOLDVECS   Fill old vectors for element
!     SEPPRINELMAT   Print element matrix and vector
! **********************************************************************
!
!                       I/O
!
!     The following coefficients are used
!
!     icoef 1 = typediff
!     icoef 2 = iupwind
!     icoef 3 = irule (numerical quadrature)
!     icoef 4 = jcart (type of coordinate system)
!     icoef 5 = typeconv
!     coef  6 = D_11
!     coef  7 = D_12
!     coef  8 = D_13
!     coef  9 = D_22
!     coef 10 = D_23
!     coef 11 = D_33
!     coef 12 = U_1
!     coef 13 = U_2
!     coef 14 = U_3
!     coef 15 = source
!     coef 16 = density (rho)
!     coef 17 = heat_capacity (cp)
!     coef 18 = rho cp
!     icoef 20 = applyupw
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelm703' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelm703'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      E%mat = 0d0
      E%vec = 0d0

!     --- Define element independent quantities
!         only for first element

      if ( E%first==0 ) call sepel0703

      E%submat => E%mat
      E%subvec => E%vec

!     --- Map relation between coefficients and specific names

      source => E%coeffs(:,E%indsource)

!     --- Fill basis functions

      call sepelbasefunc

!     --- Fill old vectors in quadrature points if necessary

      if ( E%compold>0 ) call sepeloldvecs

      if ( debug ) &
        write(irefwr,1) 'isupcfvar, isubcfvar', E%isupcfvar, E%isubcfvar

      if ( E%isupcfvar>=E%isubcfvar ) then

!     --- Fill variable coefficients if necessary

         call sepelfillvar

!        --- Compute rho cp if necessary

         if ( E%ind(E%indrho)/=0 .or. E%ind(E%indcp)/=0 ) then
            E%coeffs(:,E%indrhocp) = E%coeffs(:,E%indrho)*E%coeffs(:,E%indcp)
            E%coeffsnode(:,E%indrhocp) = &
               E%coeffsnode(:,E%indrho)*E%coeffsnode(:,E%indcp)
         end if  ! ( E%ind(E%indrho)/=0 .or. E%ind(E%indcp)/=0 )

         if ( debug ) call prinrl ( source, B%m, 'source' )

      end if  ! ( E%isupcfvar>=E%isubcfvar )

!     --- Multiply velocity by rho cp if necessary

      if ( E%ind(E%indrhocp)/=0 ) call sepelmultrho

!     --- Fill upwind basis functions if necessary

      B%fillpsi = E%methodupw>0
      if ( B%fillpsi ) call sepelbaseupw

!     --- Fill element matrix

      if ( E%convection ) call sepelconvect
      if ( E%diffusion ) call sepeldiffusion

!     --- Fill element vector

      if ( E%sourceterm ) call sepelload ( source ) ! source term

!     --- Fill element mass matrix if necessary

      if ( E%imas>0 ) call sepelmassmat

!     --- Reset velocity if necessary

      if ( E%ind(E%indrhocp)/=0 ) &
         E%coeffs(:,E%indvel:E%indvel+2) = E%coeffs(:,E%ncoef+1:E%ncoef+3)

!     --- Remove basis functions and coefficients in case of last call

      if ( E%last==1 ) call sepeldealloc

1000  if ( debug ) then

!     --- Debug information

         call sepprinelmat
         write(irefwr,*) 'End sepelm703'

      end if  ! ( debug )
      call erclos ( 'sepelm703' )

      end subroutine sepelm703
