      subroutine sepelm705
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-08-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix and vector for the general elliptic equation
!     one unknown per point
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: j
      double precision, pointer :: source(:), beta(:)

!     beta           coefficient for zeroth order term
!     j              general loop variable
!     source         Contains the source in quadrature points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPEL0705      Fill element independent quantities for element type 705
!     SEPELBASEFUNC  Compute information about basis functions
!     SEPELBASEUPW   Compute the modified basis functions psi due to upwind
!     SEPELCONVECT   Compute the element matrix for the convection equation
!     SEPELDEALLOC   Deallocate coefficients and basis functions for element
!                    subroutines
!     SEPELDIFFUSION Compute the element matrix for the diffusion equation
!     SEPELLOAD      Fill the element vector with a load
!     SEPELM705CF    Fill coefficients for general elliptic equation
!     SEPELMASSMAT   Fill the element mass matrix; one unknown per node
!     SEPELMASSMAT1  Fill the element mass matrix; one unknown per node
!                    Consistent mass matrix
!     SEPELOLDVECS   Fill old vectors for element
!     SEPELVECGAMMA  Add term div( gamma) to element vector
!     SEPPRINELMAT   Print element matrix and vector
! **********************************************************************
!
!                       I/O
!
!     The following coefficients are used
!
!     icoef 1 = typediff
!     icoef 2 = iupwind
!     icoef 3 = irule (numerical quadrature)
!     icoef 4 = jcart (type of coordinate system)
!     icoef 5 = typeconv
!     coef  6 = D_11
!     coef  7 = D_12
!     coef  8 = D_13
!     coef  9 = D_22
!     coef 10 = D_23
!     coef 11 = D_33
!     coef 12 = U_1
!     coef 13 = U_2
!     coef 14 = U_3
!     coef 15 = beta
!     coef 16 = source
!     coef 17 = density (rho)
!     coef 18 = heat_capacity (cp)
!     coef 19 = gamma_1
!     coef 20 = gamma_2
!     coef 21 = gamma_3
!     coef 22 = rho cp
!     icoef25 = applyupw
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelm705' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelm705'
         write(irefwr,1) 'ielem', E%ielem
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      E%mat = 0d0
      E%vec = 0d0

!     --- Define element independent quantities
!         only for first element

      if ( E%first==0 ) call sepel0705

      E%submat => E%mat
      E%subvec => E%vec

!     --- Map relation between coefficients and specific names

      beta => E%coeffs(:,E%indbeta)
      source => E%coeffs(:,E%indsource)

!     --- Fill basis functions

      call sepelbasefunc

!     --- Fill old vectors in quadrature points if necessary

      if ( E%compold>0 ) call sepeloldvecs

      if ( debug ) then
         write(irefwr,1) 'isupcfvar, isubcfvar', E%isupcfvar, E%isubcfvar
         write(irefwr,*) 'B%fillqmat ', B%fillqmat
      end if  ! ( debug )

!     --- Fill all coefficients

      call sepelm705cf

!     --- Fill upwind basis functions if necessary

      B%fillpsi = E%methodupw>0
      if ( B%fillpsi ) call sepelbaseupw

      if ( matrix ) then

!     --- Fill element matrix

         if ( debug ) &
            write(irefwr,*) 'diffusion, convection, zeroterm', &
                             E%diffusion, E%convection, E%zeroterm

         if ( E%diffusion ) call sepeldiffusion  ! diffusion
         if ( E%convection ) call sepelconvect   ! convection
         if ( E%zeroterm ) call sepelmassmat1 ( beta ) ! zero term

      end if  ! ( matrix )

      if ( vector .and. notvec==0 ) then

!     --- Fill element vector

         if ( E%sourceterm ) call sepelload ( source ) ! source term
         if ( E%gammaterm ) call sepelvecgamma

      end if  ! ( vector )

!     --- Fill element mass matrix if necessary

      if ( E%imas>0 ) call sepelmassmat

!     --- Reset velocity if necessary

      if ( E%ind(E%indrhocp)/=0 ) &
         E%coeffs(:,E%indvel:E%indvel+2) = E%coeffs(:,E%ncoef+1:E%ncoef+3)

!     --- Remove basis functions and coefficients in case of last call

      if ( debug ) write(irefwr,1) 'last', E%last
      if ( E%last==1 ) call sepeldealloc

1000  if ( debug ) then

!     --- Debug information

         call sepprinelmat
         write(irefwr,*) 'End sepelm705'

      end if  ! ( debug )
      call erclos ( 'sepelm705' )

      end subroutine sepelm705
