      subroutine sepelm802
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-02-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix and vector for delta right-hand-side function
!     Both point elements and line elements are permitted
!     Old subroutine elm802
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, pointer :: rho(:)
      double precision :: dx(E%ndim), h
      double precision, target :: x(E%npelm,E%ndim)
      integer :: i

!     dx             length of vector
!     h              length of element
!     i              General loop variable
!     rho            Contains values of delta function
!     x              Coordinates of nodes in element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPEL0802      Fill element independent quantities for element type 802
!     SEPELOLDVECS   Fill old vectors for element
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients of the delta function through
!     the arrays USER and IUSER. The coefficients are stored in the
!     classical way (see Standard Problems of SEPRAN).
!     There is only one coefficient to be given by the user
!     coef 1 = deltarhs(x,[y],[z])
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1771   Shape number not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
!     if delta-point function
!        if matrix then
!           elemmt(1) = 0d0
!        end if
!        if vector then
!           get height of delta function ( rho )
!           elemvc(1) = rho
!        end if
!     else if delta-line function
!        compute coordinates of basis points P
!        if matrix then
!           elemmt(1:4 or 1:9) = 0d0
!        end if
!        if vector then
!           get height of delta function ( rhoP ) in each basis point P
!           compute contribution to elemvc
!        end if
!     end if
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelm802' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelm802'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Define element independent quantities
!         only for first element

      if ( E%first==0 ) call sepel0802

      rho => E%coeffs(:,1)

!     --- Store coordinates in array x

      do i = 1, E%ndim
         x(1:E%npelm,i) = E%coor(i,E%nodes(1:E%npelm))
      end do  ! i = 1, ndim

      B%xyz => x

!     --- Fill old vectors in quadrature points if necessary

      if ( E%compold>0 ) call sepeloldvecs

      if ( E%isupcfvar>=E%isubcfvar ) then

!     --- Fill variable coefficients if necessary

         call sepelfillvar

      end if  ! ( E%isupcfvar>=E%isubcfvar )

!     --- Fill element vector

      select case ( E%ishape )

      case(-10, 0)

!     --- Point delta function

         E%vec = rho

      case(1)

!     --- Linear line delta function

         dx = x(2,:)-x(1,:)
         h = 0.5d0*sqrt(sum(dx(:)**2))
         E%vec = h*rho

      case(2)

!     --- Quadratic line delta function

         dx = x(3,:)-x(1,:)
         h = sqrt(sum(dx(:)**2))/6d0
         E%vec(1) = h*rho(1)
         E%vec(2) = 4d0*h*rho(1)
         E%vec(3) = h*rho(1)

      case default

!     --- shape not available

         call errint ( E%ishape, 1 )
         call errint ( E%ielem, 2 )
         call errint ( E%itype, 3 )
         call errint ( E%ielgrp, 4 )
         call errsub ( 1771, 4, 0, 0 )

      end select  ! case ( E%ishape )

1000  call erclos ( 'sepelm802' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelm802'

      end if  ! ( debug )

      end subroutine sepelm802
