      subroutine sepelm803
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 24-02-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix and vector for second order elliptic equations
!     and related heat equation
!     Type 803: Special non-linear diffusion equation, see SP 3.3.2
!     Old subroutine 803
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elliptic_equation
!     parabolic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, pointer :: source(:), beta(:)

!     beta           coefficient for zeroth order term
!     source         Contains the source in quadrature points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPEL0803      Fill element independent quantities for element type 803
!     SEPELBASEFUNC  Compute information about basis functions
!     SEPELDEALLOC   Deallocate coefficients and basis functions for element
!                    subroutines
!     SEPELDIFF803   Recompute the diffusion tensor for type 803
!     SEPELDIFFUSION Compute the element matrix for the diffusion equation
!     SEPELLOAD      Fill the element vector with a load
!     SEPELM705CF    Fill coefficients for general elliptic equation
!     SEPELMASSMAT   Fill the element mass matrix; one unknown per node
!     SEPELMASSMAT1  Fill the element mass matrix; one unknown per node
!                    Consistent mass matrix
!     SEPELMAT803    Compute extra contribution for element matrix
!                    corresponding to newton linearization for type 803
!     SEPELOLDVECS   Fill old vectors for element
!     SEPELVEC803    Compute extra contribution for element vector
!                    corresponding to newton linearization for type 803
!     SEPPRINELMAT   Print element matrix and vector
! **********************************************************************
!
!                       I/O
!
!     The following coefficients are used
!
!     1-2:  -
!     3:  Type of numerical integration
!         Possible values:
!         0: default value, defined by element
!     4:  Type of co-ordinate system
!         Possible values:
!         0: Cartesian co-ordinates
!         1: Axi-symmetric co-ordinates
!         2: Polar co-ordinates
!     5:  This parameter depends on the type number to be used
!         Possible values:
!         0,1: Picard
!         2:   Newton
!     6-14:  -
!    15:  beta
!    16:  f
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!       Differential equation:
!    rho cp d phi/d t - div alpha grad phi +rho cp u . grad phi + beta phi = f
!                                                  -
!        The following formulae are used:
!           S   =  S      +  S   +  S
!                   alpha     u      beta
!           S     (i,j) = / alpha grad phi  . grad phi   de
!            alpha        e               j           i
!           S     (i,j) = / u . grad phi   phi  de
!            u            e -           j     i
!           S     (i,j) = / beta phi   phi  de
!            beta        e          j     i
!           M (i,j) = / rho cp phi  phi  de
!                     e           i    j
!           F_1(i)    =  /  f phi  de
!                      e       i
!           F_2(i)    =  /  gamma . grad phi_i  de
!                      e
!        Each integral is approximated by a numerical rule of the shape:
!        / int(x) dx = sum  w  int(x )
!        e              k    k      k
!        the factors w  form the weights for the numerical integration
!                     k
!        In the axi-symmetric case they include the factor 2 pi r
!        If itypln=3 (non-linear convection) we apply Newton Raphson, i.e.
!                 n+1                n                                     n
!        g( grad(c   ) ) ~ g ( grad(c ) ) + grad(dc) . dg/d grad(c) (grad(c )), &
!        with:
!                   n+1   n
!             dc = c   - c
!                                                                       n
!        Hence the convective terms get coefficient dg/d grad(c) (grad(c )), &
!        and the right-hand side vector is updated by:
!                  n            n                        n
!        -g( grad(c ) ) + grad(c ) . dg/d grad(c) (grad(c ))
!        .
!        .
!        .
!        We have alpha = alpha(||grad c||) with alpha scalar
!        Picard linearization implies taking alpha at the previous level
!        Newton gives:
!                      n+1       n      n
!        (alpha grad c)   = alpha grad c  + &
!                                  n+1        n                 n
!                           (grad c   - grad c ) d(alpha grad c) /d(grad c)
!                                                          -1
!         d(alpha grad c)/d(grad c) = alpha I + dalpha/dB B  grad c grad c
!                          n
!         with B = ||grad c ||
!         Hence:
!                      n+1       n      n+1
!        (alpha grad c)   = alpha grad c     + &
!                                 n     -1      n        n+1       n
!                           dalpha /dB B  grad c . grad c    grad c  - &
!                                 n            n
!                           dalpha /DB B grad c
!         So in that case we have to add the following term to the matrix:
!                         n     -1       n                    n
!         S (i,j) = dalpha /dB B  (grad c , grad phi ) (grad c , grad phi )
!          extra                                    j                    i
!         and to the right-hand side:
!                         n             n
!         F  (i) =  dalpha /dB B (grad c , grad phi )
!          extra                                   i
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelm803' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelm803'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      E%mat = 0d0
      E%vec = 0d0

!     --- Define element independent quantities
!         only for first element

      if ( E%first==0 ) call sepel0803

      E%submat => E%mat
      E%subvec => E%vec

!     --- Map relation between coefficients and specific names

      beta => E%coeffs(:,E%indbeta)
      source => E%coeffs(:,E%indsource)


!     --- Fill basis functions

      call sepelbasefunc

!     --- Fill old vectors in quadrature points if necessary

      if ( E%compold>0 ) call sepeloldvecs

      if ( debug ) then
         write(irefwr,1) 'isupcfvar, isubcfvar', E%isupcfvar, E%isubcfvar
         write(irefwr,*) 'B%fillqmat, E%typelinear ', B%fillqmat, E%typelinear
      end if  ! ( debug )

!     --- Fill all coefficients

      call sepelm705cf

!     --- Adapt diffusion as function of the linearization
!         and compute two extra coefficients using funcc2

      call sepeldiff803

!     --- Fill element matrix

      if ( debug ) &
         write(irefwr,*) 'diffusion, convection, zeroterm', &
                          E%diffusion, E%convection, E%zeroterm

      if ( E%diffusion ) call sepeldiffusion  ! diffusion
      if ( E%zeroterm ) call sepelmassmat1 ( beta ) ! zero term

      if ( E%typelinear>1 ) call sepelmat803

!     --- Fill element vector

      if ( E%sourceterm ) call sepelload ( source ) ! source term
      if ( E%typelinear>1 ) call sepelvec803

!     --- Fill element mass matrix if necessary

      if ( E%imas>0 ) call sepelmassmat

!     --- Remove basis functions and coefficients in case of last call

      if ( debug ) write(irefwr,1) 'last', E%last
      if ( E%last==1 ) call sepeldealloc

1000  if ( debug ) then

!     --- Debug information

         call sepprinelmat
         write(irefwr,*) 'End sepelm803'

      end if  ! ( debug )
      call erclos ( 'sepelm803' )

      end subroutine sepelm803
