      subroutine sepelm806
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-07-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix and vector for boundary conditions for
!     second order elliptic equations with two unknowns per point
!     One, two and three dimensional elements
!     Old subroutine elm806
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elliptic_equation
!     parabolic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulebuild
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRSUB         Error messages
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
!     SEPEL0806      Fill element independent quantities for element type 806
!     SEPELBASEFUNC  Compute information about basis functions
!     SEPELDEALLOC   Deallocate coefficients and basis functions for element
!                    subroutines
!     SEPELFILLVAR   Fill space dependent coefficients
!     SEPELLOAD1     Fill the element vector with a load
!     SEPELMASSMAT1  Fill the element mass matrix; one unknown per node
!                    Consistent mass matrix
!     SEPELOLDVECS   Fill old vectors for element
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays IUSER and USER
!     Usually they are filled by FILCOF
!     The sequence is:
!     1:  Type of natural boundary condition
!         Possible values:
!         0,2: natural boundary condition of type 2
!         3: natural boundary condition of type 3
!         4: natural boundary condition of type 4
!         5: natural boundary condition of type 5
!     2:  Not yet in use:  set equal to 0
!     3:  Type of numerical integration
!         Possible values:
!         0: default value, defined by element
!     4:  Type of co-ordinate system
!         Possible values:
!         0: Cartesian co-orindates
!         1: Axi-symmetric co-orindates
!         2: Polar co-orindates
!     5:  Not yet in use:  set equal to 0
!     Parameters with respect to the first unknown
!     6:  sigma
!     7:  h
!     8:  alpha 11  (1D/2D/3D)  (The parameters 8-13 are only used for
!     9:  alpha 12  (2D/3D)      boundary conditions of type 4)
!    10:  alpha 13  (3D)
!    11:  alpha 22  (2D/3D)
!    12:  alpha 23  (3D)
!    13:  alpha 33  (3D)
!    14-15: Not yet in use (must be zero)
!    16-25: See 6-15, now with respect to the second unknown
!    In case of boundary conditions of type 5:
!     6:  u1
!     7:  u2
!     8:  u3
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1682   Option not yet available
! **********************************************************************
!
!                       PSEUDO CODE
!
!                              METHOD
!       Boundary conditions:
!       Type 2:   sum  sum alpha   dc/dx n  + sigma c = h
!                  i    j       ij      j i
!       Type 3:   sum  sum alpha   dc/dx n  + sigma dc/dt = h
!                  i    j       ij      j i
!       Type 4:   dc/dn + sigma dc/dt = h
!       Type 5:   special for convective terms of compressible type
!                 See 2: h=0, sigma = u.n
!        The following formulae are used:
!       Type 2:  S(i,j) = /  sigma phi  phi  de
!                         e           i    j
!                f(i)   = /  h phi  de
!                         e       i
!        Each integral is approximated by a numerical rule of the shape:
!        / int(x) dx = sum  w  int(x )
!        e              k    k      k
!        the factors w  form the weights for the numerical integration
!                     k
!        In the axi-symmetric case they include the factor 2 pi r
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelm806' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelm806'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      E%mat = 0d0
      E%vec = 0d0

!     --- Define element independent quantities
!         only for first element

      if ( E%first==0 ) call sepel0806

!     --- Fill basis functions

      call sepelbasefunc

!     --- Fill old vectors in quadrature points if necessary

      if ( E%compold>0 ) call sepeloldvecs

!     --- Fill all coefficients

      if ( E%isupcfvar>=E%isubcfvar ) call sepelfillvar

!     --- Fill parts of matrix if necessary

      if ( matrix .and. E%notmat==0 ) then

!     --- matrix = true    compute element matrix

         select case ( E%typenatbnd )

         case(2)

!        --- Boundary conditions of type 2

            if ( E%zeroterm1 ) then

!           --- First term not zero

               E%submat = 0d0
               call sepelmassmat1 ( E%coeffs(:,1), E%submat )
               E%mat(1:E%icount:2,1:E%icount:2) = E%submat

            end if  ! ( E%zeroterm1 )

            if ( E%zeroterm2 ) then

!           --- First term not zero

               E%submat = 0d0
               call sepelmassmat1 ( E%coeffs(:,11), E%submat )
               E%mat(2:E%icount:2,2:E%icount:2) = E%submat

            end if  ! ( E%zeroterm2 )

         case default

!        --- Boundary conditions of type 5
!            Option not yet implemented

            call errchr ( 'typenatbnd=5', 1 )
            call errsub ( 1682, 0, 0, 1 )
            go to 1000

         end select  ! case ( E%typenatbnd )

      end if  ! ( matrix .and. E%notmat==0 )

!     --- Fill element vector if necessary

      if ( vector .and. notvec/=1 ) then

!     --- vector = true, compute element vector


         if ( E%sourceterm1 ) then
            E%subvec = 0d0
            call sepelload1 ( E%coeffs(:,2), B%jdiag, B%phi, E%subvec )
            E%vec(1:E%icount:2) = E%subvec
         end if  ! ( E%sourceterm )
         if ( E%sourceterm2 ) then
            E%subvec = 0d0
            call sepelload1 ( E%coeffs(:,12), B%jdiag, B%phi, E%subvec )
            E%vec(2:E%icount:2) = E%subvec
         end if  ! ( E%sourceterm )

      end if

      if ( E%last==1 ) then
         call sepeldealloc
         deallocate ( E%subvec, E%submat )
      end if  ! ( E%last==1 )

1000  if ( debug ) then

!     --- Debug information

         call prinrl(E%vec,icount,'E%vec')
         call prinrl1(E%mat,icount,icount,'E%mat')
         write(irefwr,*) 'End sepelm806'

      end if  ! ( debug )
      call erclos ( 'sepelm806' )

      end subroutine sepelm806
