      subroutine sepelmassmat4 ( rho, mat )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-09-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the element mass matrix in case of one unknown per node
!     Consistent mass matrix
!     Matrix is defined by: sum rho psi_i phi_j
!     rho must contain the weights
!     Old subroutine el2010
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     mass_matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision, intent(in) :: rho(m)
      double precision, intent(out) :: mat(n,n)

!     mat            i    matrix to be updated
!     rho            i    density rho cp
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j

!     i              General loop variable
!     j              General loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The element mass matrix has elements
!
!     sum rho phi_j psi_i
!
!     Hence m_ij = sum_k rho(x_k) phi_j(x_k) psi_i(x_k)
!
!     At this moment the transpose of the matrix is stored
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelmassmat4' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelmassmat4'
         write(irefwr,1) 'm, n, jdiag', m, n, B%jdiag
         call prinrl ( rho, m, 'rho' )
         call prinrl1 ( B%phi, m, n, 'phi' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( B%jdiag==1 ) then

!     --- jdiag=1, phi_ik = delta_ik

         if ( E%methodupw>0 .and. E%applyupw==0 ) then

!        --- Upwind is used

            mat = 0d0
            do i = 1, n
               do j = kstep, n, kstep
                  mat(j,i) = rho(j)*B%psi(i,j)
               end do  ! j = kstep, n, kstep
            end do  ! i = 1, n

         else

!        --- No upwind

            mat = 0d0
            do i = kstep, n, kstep
               mat(i,i) = rho(i)
            end do  ! i =

         end if  ! ( E%methodupw>0 .and. E%applyupw==0 )

      else

!     --- jdiag#1, phi general

         if ( E%methodupw>0 .and. E%applyupw==0 ) then

!        --- Upwind is used

            do i = 1, n
               do j = 1, n
                  mat(j,i) = sum ( rho*B%phi(j,:)*B%psi(i,:) )
               end do  ! j = 1, n
            end do  ! i = 1, n

         else

!        --- No upwind

            do i = 1, n
               do j = 1, i
                  mat(j,i) = sum ( rho*B%phi(j,:)*B%phi(i,:) )
                  mat(i,j) = mat(j,i)
               end do  ! j = 1, n
            end do  ! i = 1, n

         end if  ! ( E%methodupw>0 .and. E%applyupw==0 )

      end if  ! ( B%jdiag==1 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelmassmat4'

      end if  ! ( debug )
      call erclos ( 'sepelmassmat4' )

      end subroutine sepelmassmat4
