      subroutine sepelp03
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 17-07-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute information of basis functions
!     Linear triangle (3 nodes)
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      use sepmoduleconsts
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: e1(3,2), delta, h, rc
      double precision, pointer :: xloc(:,:)
      double precision, pointer :: x(:), xg(:)
      integer :: k, i

!     delta          Jacobian of the element
!     e1             Work array of length 3 x 2, to store the factors
!                    e^ij See method
!     h              Help variable to store part of a weight
!     i              Counting variable
!     k              Loop variable
!     rc             Help variable to store part of a weight for axisymmetric
!                    co-ordinates
!     x              x-coordinates of nodal points
!     xg             x-coordinates of Gauss points
!     xloc           co-ordinates of vertices in local (xi-zeta) coordinate
!                    system
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELP620TRANSFO  Compute x as function of jtransfo
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL2        Print a real two-dimensional array
!     SEPELP03FIRST  compute some element independent quantities
!                    Linear triangle (3 nodes)
!     SEPELP03JACOB  Compute Jacobian and derivatives of basis functions
!                    Linear triangle (3 nodes)
!     SEPELP03LOC    Fill local coordinates for triangle in 3d
!     SEPELPGAUSS    Fill coordinates of Gauss points
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!        To compute the derivatives of the basis functions, the jacobian
!        of the element and the weights, the following definitions are used:
!         ij        i+1    i+2    i+1
!        e   =  (-1)    ( x    - x    )   cyclic: i=1,2,3 ;   j=1,2
!                          j      j
!                    32  11    12 31
!        delta   =  e   e   - e  e
!                          ij
!        d phi  / dx   =  e   /  delta
!             i     j
!                                        |delta|       1     2     3
!        Newton Cotes rule:  / f(x) dx = -------- [ f(x )+f(x )+f(x ) ]
!                            e              6
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp03' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp03'
         write(irefwr,1) 'ielem, ndim, jtransfo', E%ielem, E%ndim, E%jtransfo
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( xloc(1:B%n,1:3), B%xloc(1:B%n,1:2) )

      x  => B%xyz(1:B%n,1)
      xg => B%xyzg(:,1)

      if ( E%first==0 ) then

!     --- First call

         call sepelp03first

         if ( debug ) then

!        --- Debug information

            write(irefwr,1) 'm, n, B%rule', B%m, B%n, B%rule
            write(irefwr,*) 'fillqmat', B%fillqmat

         end if  ! ( debug )

      end if  ! ( E%first==0 )

!     --- Fill Gauss points

      call sepelpgauss

      if ( E%ndim>2 .and. E%jtransfo==0 ) then

!     --- Triangle in three-dimensional space
!         transform co-ordinates

         call sepelp03loc

      else

         B%xloc = B%xyz(1:B%n,1:2)

      end if  ! ( ndim>2 .and. jtransfo==0 )

      if ( E%jtransfo>1 ) then

!     --- Transform in a special way

         xloc = B%xyz
         call elp620transfo ( xloc, E%jtransfo )
         B%xloc = xloc(1:B%n,1:2)
         B%xyz(1:B%n,1:2) = B%xloc

      end if  ! ( jtransfo>1 )

!     --- Compute Jacobian and derivatives of basis functions

      call sepelp03jacob ( e1, delta )

!     --- Fill weights

      if ( B%rule==1 .and. B%cart==2 ) then

!     --- jcart = 2  Axi-symmetric co-ordinates

         h = abs ( delta ) / 24d0
         rc = sum(x)
         B%w = h*(x+rc) * pi2

      else

!     --- Standard case

         B%w = abs(delta)*B%wc
         if ( B%cart==2 ) B%w = B%w * xg * pi2  ! axisymmetric

      end if  ! ( B%rule==1 .and. B%cart==2 )

      if ( debug ) call prinrl ( B%w, B%m, 'weights' )

      if ( B%fillderiv ) then

!     --- fillderiv = true, fill first derivatives

         do k = 1, 2
            B%phider(:,1,k) = e1(:,k)
         end do

         do i = 1, 3
            do k = 1, 2
               B%phider(i,2:B%m,k) = B%phider(i,1,k)
            end do  ! k = 1, 2
         end do  ! i = 1, 3
         if ( debug ) then

!        --- Debug information

            write(irefwr,2) 'dphidx', B%phider(1,1,1), B%phider(2,1,1),&
                                      B%phider(3,1,1)
            write(irefwr,2) 'dphidy', B%phider(1,1,2), B%phider(2,1,2),&
                                      B%phider(3,1,2)

         end if

      end if  ! ( B%fillderiv )
      deallocate ( xloc, B%xloc )

      if ( itype>=902 .and. itype<=903 ) then

!     --- Special case of Navier-Stokes element
!         w(1..m ) contains the weights
!         w(m+1 .. m+n) the diagonal of the lumped mass matrix

         if ( B%rule==1 .or. B%cart==2 ) then

!        --- B%rule = 1  Newton Cotes rule, just a copy

            B%w2 = B%w

         else if ( B%cart==1 ) then

!        --- B%cart = 1  Cartesian co-ordinates

            h = abs(delta)/6d0
            B%w2 = h

         end if  ! ( B%cart==1 )

      end if  ! ( itype>=902 .and. itype<=903 )

1000  call erclos ( 'sepelp03' )
      if ( debug ) then

!     --- Debug information

         call prinrl2 ( B%xyz, 3, 2, 'end coordinates' )
         write(irefwr,*) 'End sepelp03'

      end if  ! ( debug )

      end subroutine sepelp03
