      subroutine sepelp05
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 17-07-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute information of basis functions
!     Bi-linear quadrilateral (4 nodes)
!     Old routine: elp621
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      use sepmoduleconsts
      use sepmoduledebug
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: k, i
      double precision :: h, vec(3), wh(m)
      double precision, pointer :: x(:), xg(:), jac(:)

!     h              Help variable to store part of a weight
!     i              Counting variable
!     jac            Array of length m containing the values of the
!                    Jacobian in the vertices or for the Gauss integration
!                    rules work array to store some help variables
!     k              Loop variable
!     x              Contains x co-ordinates of nodes
!     xg             x-coordinates of Gauss points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     SEPELP05FIRST  compute some element independent quantities
!                    Bi-linear quadrilateral (4 nodes)
!     SEPELP05JACDER Compute Jacobian and derivatives of basis functions
!     SEPELPGAUSS    Fill coordinates of Gauss points
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!        To compute the derivatives of the basis functions, the jacobian
!        of the element and the weights, the following definitions are used:
!         ij       j     i+1    i
!        e   = (-1)  (  x    - x    )   cyclic: i=1,2,3,4 ;   j=1,2
!                        j+1    j+1
!              1     2    1          4    1          3    2    1    4
!        x  = x  + (x  - x ) xi + (x  - x ) eta + (x  - x  + x  - x ) xi eta
!         i    i     i    i          i    i          i    i    i    i
!                                   i = 1,2
!                  1    12        42
!        So: x  = x  + e   xi - e   eta + A xi eta
!             1    1
!                  1    11        41
!            x  = x  - e   xi + e   eta + B xi eta
!             2    2
!            |   2    1                  4    1          |
!            |  x  - x  + A eta         x  - x  + A xi  |
!            |   1    1                  1    1          |
!        J  =|                                           |
!            |   2    1                  4    1          |
!            |  x  - x  + B eta         x  - x  + B xi  |
!            |   2    2                  2    2          |
!             1    2    3    4        12    32
!        A = x  - x  + x  - x    = - e   - e
!             1    1    1    1
!             3    4    2    1       11    31
!             2    2    2    2
!       |J| = J  + J  xi + J  eta
!              0    1        2
!              41   12    11   42
!        J  = e  * e   - e  * e
!         0
!                   12         11
!        J  = B  * e   - A  * e
!         1
!                   41         42
!        J  = A  * e   + B  * e
!         2
!        dphi / dx  = dphi / dxi dxi/dx  + dphi / deta deta/dx
!                 j                      j                      j
!                       41
!        dxi/dx  = [  e   + B xi ] / |J|
!               1
!                       42
!        dxi/dx  = [  e   - A xi ] / |J|
!               2
!                       11
!        deta/dx  = [  e   - B eta ] / |J|
!               1
!                       12
!        deta/dx  = [  e   + A eta ] / |J|
!               2
!                                          1   4       k      k
!        Newton Cotes rule:  / f(x) dx =  --- sum [ f(x ) |J(x )| ]
!                            e             4  k=1
!     In case of 3D elements we use the following formulae:
!     x = sum phi (xi,eta) x    dx/dxi = sum dphi /dxi x   dx/deta=...
!     -    i     i          -i    -         i      i      -i
!             a                      a                         a
!     dphi /dx = dphi / dxi  dxi/dx   +  dphi / deta  deta/dx
!         i          i                         i
!                                                                        -1
!     | dxi/dx   deta/dx   dzeta/dx  |   | dx/dxi   dy/dxi   dz/dxi |
!     | dxi/dy   deta/dy   dzeta/dy  | = | dx/deta   dy/deta   dz/deta |
!     | dxi/dz   deta/dz   dzeta/dz  |   | dx/dzeta  dy/dzeta  dz/dzeta|
!     zeta is choosen perpendicular to the xi-eta direction, jacich implies
!     that we may choose:
!     dx / dzeta = dx / dxi times  dx / deta / || dx/dxi x dx/deta ||
!      -            -                -              -         -
!     jacere times or x is the outer product
!     The Jacobian of the dx/dxi matrix is given by:
!     J = dx/dzeta ( dy/dxi dz/deta - dy/deta dz/dxi )
!       + dy/dzeta ( dz/dxi dx/deta - dz/deta dx/dxi )
!       + dz/dzeta ( dx/dxi dy/deta - dx/deta dy/dxi )
!     and since dx/dzeta before normalization is given by
!         dx/dzeta = dy/dxi dz/deta - dy/deta dz/dxi
!         dy/dzeta = dz/dxi dx/deta - dz/deta dx/dxi
!         dz/dzeta = dx/dxi dy/deta - dx/deta dy/dxi
!     we see that the normalization factor is equal to
!        sqrt(     ( dy/dxi dz/deta - dy/deta dz/dxi )^2
!                + ( dz/dxi dx/deta - dz/deta dx/dxi )^2
!                + ( dx/dxi dy/deta - dx/deta dy/dxi )^2 ) = J
!      The derivatives dxi/dx and so on follow from the inverse,
!      which in this 3x3 case leads to:
!      dxi/dx = ( dy/deta  dz/dzeta - dy/dzeta dz/deta  ) / J
!      dxi/dy = ( dz/deta  dx/dzeta - dz/dzeta dx/deta  ) / J
!      dxi/dz = ( dx/deta  dy/dzeta - dx/dzeta dy/deta  ) / J
!      deta/dx = ( dy/dzeta dz/dxi  - dy/dxi  dz/dzeta ) / J
!      deta/dy = ( dz/dzeta dx/dxi  - dz/dxi  dx/dzeta ) / J
!      deta/dz = ( dz/dzeta dy/dxi  - dx/dxi  dy/dzeta ) / J
!     To compute qmat, we use the lamina coordinate system as introduced by
!     Hughes:
!      Define e_xi = x,xi / ||x,xi||
!      Define e_eta = x,eta / ||x,eta||
!      e_3(l) = e_xi x e_eta / ||e_xi x e_eta||
!      e_alpha = (e_xi + e_eta)/2 / || (e_xi + e_eta)/2 ||
!      e_beta = e_3(l) x e_alpha / || e_3(l) x e_alpha ||
!      e_1(l) = srqt(2)/2 ( e_alpha - e_beta )
!      e_2(l) = srqt(2)/2 ( e_alpha + e_beta )
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp05' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp05'
         write(irefwr,1) 'ifirst, n, m', E%first, B%n, B%m
         write(irefwr,1) 'inpelm, jcart', E%npelm, B%cart
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      x => B%xyz(:,1)
      xg => B%xyzg(:,1)
      jac => B%jac

      if ( E%first==0 ) call sepelp05first

!     --- Fill Gauss points

      call sepelpgauss

!     --- Compute Jacobian and derivatives of basis functions

      call sepelp05jacder

      wh = abs(jac)

!     --- Fill weights

      if ( B%cart==1 ) then

!     --- B%cart = 1  Cartesian co-ordinates

         B%w = B%wc*wh

      else if ( B%cart>=2 ) then

!     --- B%cart = 2  Axi-symmetric co-ordinates

         if ( B%rule==1 ) then

!        --- intrule = 1  Newton-Cotes rule

            jac(1:4) = wh(1:4)*x(1:4)
            h = 1d0/36d0
            B%w(1) = h * (4d0*jac(1)+2d0*(jac(2)+jac(4))+jac(3))
            B%w(2) = h * (4d0*jac(2)+2d0*(jac(3)+jac(1))+jac(4))
            B%w(3) = h * (4d0*jac(3)+2d0*(jac(4)+jac(2))+jac(1))
            B%w(4) = h * (4d0*jac(4)+2d0*(jac(1)+jac(3))+jac(2))

         else

!        --- intrule>1  Gauss rule

            B%w = wh*xg*B%wc

         end if  ! ( B%rule==1 )
         if ( B%cart==2 ) B%w = B%w*pi2

      end if  ! ( B%cart==1 )

      if ( debug ) call prinrl ( B%w, B%m, 'weights' )

      if ( itype>=902 .and. itype<=903 ) then

!     --- Special case of Navier-Stokes element
!         w(1..m ) contains the weights
!         B%w2(1 .. m+n) the diagonal of lumped the mass matrix

         if ( B%rule==1 ) then

!        --- intrule = 1  Newton Cotes rule, just a copy

            B%w2= B%w

         else if ( B%cart==1 ) then

!        --- B%cart = 1  Cartesian co-ordinates

            B%w2=0.25d0*wh

         else if ( B%cart==2 ) then

!        --- B%cart = 2  Axi-symmetric co-ordinates

            wh = wh*x
            h = 1d0/36d0*pi2
            B%w2(1) = h * (4d0*wh(1)+2d0*(wh(2)+wh(4))+wh(3))
            B%w2(2) = h * (4d0*wh(2)+2d0*(wh(3)+wh(1))+wh(4))
            B%w2(3) = h * (4d0*wh(3)+2d0*(wh(4)+wh(2))+wh(1))
            B%w2(4) = h * (4d0*wh(4)+2d0*(wh(1)+wh(3))+wh(2))

         end if  ! ( intrule==1 )

      end if  ! ( itype>=900 .and. itype<=903 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'E%iqmat', E%iqmat
         if ( E%iqmat>0 ) then
            vec(1:3) = B%qmat(1,1:3,1)
            call prinrl ( vec, 3, 'normal' )
            vec(1:3) = B%qmat(3,1:3,1)
            call prinrl ( vec, 3, 't1' )
            vec(1:3) = B%qmat(2,1:3,1)
            call prinrl ( vec, 3, 't2' )
            call prinrl ( B%w, B%m, 'w' )
         end if  ! ( iqmat>0 )
         call prinrl ( B%w, B%m, 'w' )
         call prinrl2 ( B%phider(1,1,1), B%n, B%m, 'dphidx' )
         call prinrl2 ( B%phider(1,1,2), B%n, B%m, 'dphidy' )
         call prinrl2 ( B%xyz, B%n, E%ndim, 'x' )
         call prinrl2 ( B%xyzg, B%m, E%ndim, 'xgauss' )
         call prinrl2 ( B%phi, B%n, B%m, 'phi' )
         write(irefwr,*) 'End sepelp05'

      end if  ! ( debug )
      call erclos ( 'sepelp05' )

      end subroutine sepelp05
