      subroutine sepelp06jaccur ( jac )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-09-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute basis functions and related quantities for quadratic
!     quadrilateral (curved element)
!     See elp623curved
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     quadratic
!     quadrilateral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision :: jac(B%m)

!     jac            o    Array of length m containing the values of the
!                         Jacobian in the vertices or for the Gauss integration
!                         rules work array to store some help variables
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j, k, m, n, ndim, ksign, jsign
      double precision, pointer :: x(:,:), dphidx(:,:,:), xg(:,:), &
                                   dphidxi(:,:,:)
      double precision, pointer :: psixi(:,:,:), dpsidx(:,:,:), &
                                   dxdxi(:,:,:)
      double precision :: dxdxin(B%n,3,3), dxidx(B%m,3,3)
      double precision :: det(9)

!     det            array of length modecl x modecl containing the
!                    transformation determinant
!     dphidx         Contains values of dphi/dx in quadrature points
!     dphidxi        Contains values of dphi/dxi in quadrature points
!     dpsidx         Contains values of dpsi/dx in quadrature points,
!                    with psi the linear basis functions
!     dxdxi          work array of length mx3x3 to store the values of
!                    dx/dxi, dx/deta, dy/dxi and dy/deta according to:
!                    dx/dxi(x_k) = dxdxi(k,1,1)
!                    dx/deta(x_k) = dxdxi(k,1,2)
!                    dy/dxi(x_k) = dxdxi(k,2,1)
!                    dy/deta(x_k) = dxdxi(k,2,2) etc.
!     dxdxin         dxdxi in nodes
!     dxidx          work array of length 9x2x3 to store the values of
!                    dxi/dx, dxi/dy, dxi/dz and deta/dx ... according to:
!                    dxi/dx(x^k) = dxdxi(k,1,1)  dxi/dy(x^k) = dxdxi(k,1,2)
!     i              Counting variable
!     j              Counting variable
!     jsign          Help parameter to compute sign of the Jacobian
!     k              Counting variable
!     ksign          Help parameter to compute sign of the Jacobian
!     m              number of quadrature points
!     n              number of basis functions
!     ndim           dimension of space
!     psixi          Contains the values of the derivatives of the
!                    linear basis functions in the Gauss points in
!                    the reference element.
!     x              coordinates of nodes
!     xg             coordinates of Gauss points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL1        Print 2d real vector
!     PRINRL2        Print a real two-dimensional array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     230   Jacobian of element is zero or has changing sign
! **********************************************************************
!
!                       PSEUDO CODE
!
!       To compute the derivatives of the basis functions, the jacobian
!       of the element and the weights, the following definitions are used:
!
!            9
!       x = sum   x  phi
!           ij=1   ij   ij
!
!       phi  (xi,eta) = theta (xi) theta (eta)
!          ij                 i           j
!
!
!       theta (s) = 2 ( s-1/2 ) ( s-1 )
!            1
!
!       theta (s) = -4s ( s-1 )
!            2
!
!       theta (s) = 2s ( s-1/2 )
!            3
!
!       d theta /ds = 4s-3
!              1
!
!       d theta /ds = -8s+4
!              2
!
!       d theta /ds = 4s-1
!              3
!
!            |  x          y    |
!            |   xi        xi |
!        J  =|                  |
!            |  x          y    |
!            |   eta        eta |
!
!                         9    k
!        d x / dxi   =  sum  x    d phi / d xi
!           i      j     k=1   i        k       j
!
!
!                          -1
!        grad    phi   =  J   grad        phi
!            xy     i             xi eta    i
!
!
!   In case of 3D elements the following formulas are used:
!                   a
!   x = sum phi (xi ) x
!        i     i        i
!
!          a                   a
!   dx/dxi = sum d phi / d xi  x
!              i       i          i
!
!   Jacobian = sqrt(d1^2+d2^2+d3^2)
!
!          i+1       1    i+2       2      i+2       1   i+1       2
!   di = dx   / d xi   dx   / d xi  -  dx   / d xi  dx   / d xi
!
!     In case of 3D elements we use the following formulae:
!
!     x = sum phi (xi,eta) x    dx/dxi = sum dphi /dxi x   dx/deta=...
!     -    i     i          -i    -         i      i      -i
!
!             a                      a                         a
!     dphi /dx = dphi / dxi  dxi/dx   +  dphi / deta  deta/dx
!         i          i                         i
!                                                                        -1
!     | dxi/dx   deta/dx   dzeta/dx  |   | dx/dxi   dy/dxi   dz/dxi |
!     | dxi/dy   deta/dy   dzeta/dy  | = | dx/deta   dy/deta   dz/deta |
!     | dxi/dz   deta/dz   dzeta/dz  |   | dx/dzeta  dy/dzeta  dz/dzeta|
!
!     zeta is choosen perpendicular to the xi-eta direction, which implies
!     that we may choose:
!
!     dx / dzeta = dx / dxi times  dx / deta / || dx/dxi x dx/deta ||
!      -            -                -              -         -**
!
!     where times or x is the outer product
!
!     The Jacobian of the dx/dxi matrix is given by:
!
!     J = dx/dzeta ( dy/dxi dz/deta - dy/deta dz/dxi )
!       + dy/dzeta ( dz/dxi dx/deta - dz/deta dx/dxi )
!       + dz/dzeta ( dx/dxi dy/deta - dx/deta dy/dxi )
!
!     and since dx/dzeta before normalization is given by
!
!         dx/dzeta = dy/dxi dz/deta - dy/deta dz/dxi
!         dy/dzeta = dz/dxi dx/deta - dz/deta dx/dxi
!         dz/dzeta = dx/dxi dy/deta - dx/deta dy/dxi
!
!     we see that the normalization factor is equal to
!
!        sqrt(     ( dy/dxi dz/deta - dy/deta dz/dxi )^2
!                + ( dz/dxi dx/deta - dz/deta dx/dxi )^2
!                + ( dx/dxi dy/deta - dx/deta dy/dxi )^2 ) = J
!
!      The derivatives dxi/dx and so on follow from the inverse, &
!      which in this 3x3 case leads to:
!
!      dxi/dx = ( dy/deta  dz/dzeta - dy/dzeta dz/deta  ) / J
!      dxi/dy = ( dy/dzeta dz/dxi  - dy/dxi  dz/dzeta ) / J
!      dxi/dz = ( dy/dxi  dz/deta  - dy/deta  dz/dxi  ) / J
!      deta/dx = ( dz/deta  dx/dzeta - dz/dzeta dx/deta  ) / J
!      deta/dy = ( dz/dzeta dx/dxi  - dz/dxi  dx/dzeta ) / J
!      deta/dz = ( dz/dxi  dx/deta  - dz/deta  dx/dxi  ) / J
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp06jaccur' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp06jaccur'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      x => B%xyz
      xg => B%xyzg
      dphidx => B%phider
      dphidxi => B%phiderc
      psixi => B%lamderc
      dpsidx => B%lamder
      dxdxi => B%dxdxig

      m = B%m
      n = B%n
      ndim = E%ndim

!     --- weights, or derivatives must be computed
!         Compute Jacobian in the Gauss points

      do j = 1, ndim
         do i = 1, m
            dxdxi(i,j,1) = dot_product(x(:,j),dphidxi(:,i,1))
            dxdxi(i,j,2) = dot_product(x(:,j),dphidxi(:,i,2))
         end do
      end do
      if ( debug ) then
         call prinrl1 ( dxdxi(1,1,1), ndim, m, 'dxdxi' )
         call prinrl1 ( dxdxi(1,1,2), ndim, m, 'dydxi' )
      end if  ! ( debug )

!     --- Fill Jacobian in jac

      if ( ndim==3 ) then

!     --- Standard 3D case

         dxdxi(:,1,3) = dxdxi(:,2,1)*dxdxi(:,3,2)-dxdxi(:,3,1)*dxdxi(:,2,2)
         dxdxi(:,2,3) = dxdxi(:,3,1)*dxdxi(:,1,2)-dxdxi(:,1,1)*dxdxi(:,3,2)
         dxdxi(:,3,3) = dxdxi(:,1,1)*dxdxi(:,2,2)-dxdxi(:,2,1)*dxdxi(:,1,2)
         do k = 1, m
            jac(k) = sqrt( dxdxi(k,1,3)**2 + &
                             dxdxi(k,2,3)**2 + &
                             dxdxi(k,3,3)**2 )
            if ( jac(k)<=0d0 ) then
               call errint ( E%ielem, 1 )
               call errint ( E%itype, 2 )
               call errint ( E%ielgrp, 3 )
               call errsub ( 230, 3, 0, 0 )
               call prinin(E%nodes,B%n,'node numbers')
               call prinrl2(x,B%n,2,'co-ordinates')
               go to 1000
            end if
         end do

      else

!     --- Standard 2D case

         do k = 1, m
            jac(k) = dxdxi(k,1,1)*dxdxi(k,2,2) - &
                     dxdxi(k,1,2)*dxdxi(k,2,1)
         end do

      end if

!     --- Check sign of Jacobian

      ksign = nint ( sign ( 1d0,jac(1) ) )
      do k = 2, m
         jsign = nint ( sign ( 1d0, jac(k) ) )
         if ( ksign/=jsign ) then
            call errint ( E%ielem, 1 )
            call errint ( E%itype, 2 )
            call errint ( E%ielgrp, 3 )
            call errsub ( 230, 3, 0, 0 )
            call prinin(E%nodes,B%n,'node numbers')
            call prinrl2(x,B%n,2,'co-ordinates')
            go to 1000
         end if

      end do
      if( ierror/=0 ) go to 1000

      if ( B%fillderiv ) then

!     --- fillderiv = true, fill first derivatives

         if ( ndim==3 ) then

!        --- 3D case

            dxdxi(:,1,3) = dxdxi(:,1,3)/jac
            dxdxi(:,2,3) = dxdxi(:,2,3)/jac
            dxdxi(:,3,3) = dxdxi(:,3,3)/jac
            dxidx(:,1,1) = (dxdxi(:,2,2)*dxdxi(:,3,3) - &
                            dxdxi(:,3,2)*dxdxi(:,2,3))/jac
            dxidx(:,1,2) = (dxdxi(:,3,2)*dxdxi(:,1,3) - &
                            dxdxi(:,1,2)*dxdxi(:,3,3))/jac
            dxidx(:,1,3) = (dxdxi(:,1,2)*dxdxi(:,2,3) - &
                            dxdxi(:,2,2)*dxdxi(:,1,3))/jac
            dxidx(:,2,1) = (dxdxi(:,2,3)*dxdxi(:,3,1) - &
                            dxdxi(:,3,3)*dxdxi(:,2,1))/jac
            dxidx(:,2,2) = (dxdxi(:,3,3)*dxdxi(:,1,1) - &
                            dxdxi(:,1,3)*dxdxi(:,3,1))/jac
            dxidx(:,2,3) = (dxdxi(:,1,3)*dxdxi(:,2,1) - &
                            dxdxi(:,2,3)*dxdxi(:,1,1))/jac
            do j = 1, 3
               do i = 1, 9
                  dphidx(i,:,j) = dphidxi(i,:,1)*dxidx(:,1,j) + &
                                  dphidxi(i,:,2)*dxidx(:,2,j)
               end do
            end do

            if ( B%compqmat ) then

!           --- Special case, fill qmat matrix and map

               E%iqmat = 1
               B%itang = 3
               B%inorm = 1
               call sepelfillqmat  ! Fill qmat
               call sepelpmaptang  ! map dphidx on surface

            end if  ! ( B%compqmat )

         else

!        --- curved = true, curved element

            do i = 1, 9
               dphidx(i,:,1) = ( dphidxi(i,:,1)*dxdxi(:,2,2) - &
                         dphidxi(i,:,2)*dxdxi(:,2,1) ) / jac
               dphidx(i,:,2) = (-dphidxi(i,:,1)*dxdxi(:,1,2) + &
                         dphidxi(i,:,2)*dxdxi(:,1,1) ) / jac
            end do

         end if  ! ( ndim==3 )

      end if  ! ( B%fillderiv )

      if ( B%comppsi ) then

!     --- comppsi true, compute derivatives of linear basis functions

         do i = 1, 4
            dpsidx(i,:,1) = ( psixi(i,:,1)*dxdxi(:,2,2) - &
                              psixi(i,:,2)*dxdxi(:,2,1))/jac
            dpsidx(i,:,2) = ( -psixi(i,:,1)*dxdxi(:,1,2) + &
                               psixi(i,:,2)*dxdxi(:,1,1))/jac
         end do

      end if  ! ( B%comppsi )

      if ( B%fillderivn .and. B%rule>1 ) then

!     --- fillderivn = true, fill first derivatives in nodes

         xg => B%xyzgcn
         dphidx => B%phidern
         dphidxi => B%phidercn
         do j = 1, ndim
            do i = 1, 9
               dxdxin(i,j,1) = dot_product(x(:,j),dphidxi(:,i,1))
               dxdxin(i,j,2) = dot_product(x(:,j),dphidxi(:,i,2))
            end do
         end do
         do k = 1, 9
            det(k) = dxdxin(k,1,1)*dxdxin(k,2,2) - &
                     dxdxin(k,1,2)*dxdxin(k,2,1)
         end do
         do i = 1, 9
            dphidx(i,:,1) = ( dphidxi(i,:,1)*dxdxin(:,2,2) - &
                      dphidxi(i,:,2)*dxdxin(:,2,1) ) / det
            dphidx(i,:,2) = (-dphidxi(i,:,1)*dxdxin(:,1,2) + &
                      dphidxi(i,:,2)*dxdxin(:,1,1) ) / det
         end do

      else if ( B%fillderivn ) then

!     --- fillderivn = true, fill first derivatives in nodes

         B%phidern = B%phider

      end if  ! ( B%fillderiv )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelp06jaccur'

      end if  ! ( debug )
      call erclos ( 'sepelp06jaccur' )

      end subroutine sepelp06jaccur
