      subroutine sepelp09first
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-08-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute some element independent quantities
!     Extended bi-linear quadrilateral (5 nodes)
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: lambda(4), weight(3), h
      double precision, pointer :: xg(:,:), phi(:,:), dpdeta(:,:), dpdxi(:,:)
      integer :: ms, m, k
      character (len=22), parameter :: text = 'Bilinear quadrilateral'

!     dpdeta         contains derivatives in reference element for gauss rule
!                                                   k
!                    dpdeta(i,k) = d phi  / d eta (x )
!                                       i
!     dpdxi          contains derivatives in reference element for gauss rule
!                                                   k
!                    dpdxi(i,k) = d phi  / d xi (x )
!                                      i
!     h              Help variable to store part of a weight
!     k              Loop variable
!     lambda         Contains values of lambda in quadrature points (1d)
!     m              number of quadrature points
!     ms             sqrt(m)
!     text           String variable containing some text
!     x              Contains coordinates of nodes
!     xg             Contains coordinates of Gauss points
!     weight         Help array to store one-dimensional weights
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRORELVAR     Print error message errornr for element: wrong value of
!                    parameter
!     ERRORELVAR2    Print error message errornr for element: wrong value of
!                    2 parameters
!     ERRSUB         Error messages
!     SEPELPLAMBDA1D Fill the one-dimensional linear basis functions in case of
!                    a non-Newton Cotes rule
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1497   Type of integration rule not yet available
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp09first' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp09first'
         write(irefwr,1) 'irule, m, n', B%rule, m, B%n
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      m = B%m

      if ( B%cart>2 ) call errorelvar ( B%cart-1, 1769, text, 0 )

!     --- Fill array xsi and eta for reference element (xg)

      xg =>B%xyzgc
      phi => B%phi
      dpdxi => B%phiderc(:,:,1)
      dpdeta => B%phiderc(:,:,2)

      xg = 0d0

      select case (B%rule)

      case (1)

!     --- Newton Cotes rule

         if ( m/=5 ) call errorelvar2 ( m, B%rule, 1424, text, 0 )
         if ( ierror/=0 ) go to 1000
         xg(2:3,1) = 1d0
         xg(5,1:2) = 0.5d0
         xg(3:4,2) = 1d0
         h = -5d0/12d0
         B%wc(1:4) = h
         B%wc(5) = 8d0/3d0

         B%jdiag = 1   ! basis functions satisfy phi_i(x_j) = delta_ij

      case (2:4)

!     --- Gauss rules

         h = m
         ms = nint(sqrt(h))
         call sepelplambda1d ( lambda, text, B%rule, ms, weight )
         xg(1:ms,1) = lambda(1:ms)
         xg(1:ms,2) = lambda(1)
         B%wc(1:ms) = weight(1:ms)*weight(1)
         if ( ms>1 ) then
            xg(1+ms:2*ms,1) = lambda(1:ms)
            xg(1+ms:2*ms,2) = lambda(2)
            B%wc(1+ms:2*ms) = weight(1:ms)*weight(2)
         end if  ! ( ms>1 )
         if ( ms>2 ) then
            xg(1+2*ms:3*ms,1) = lambda(1:ms)
            xg(1+2*ms:3*ms,2) = lambda(3)
            B%wc(1+2*ms:3*ms) = weight(1:ms)*weight(3)
         end if  ! ( ms>1 )

      case default

!     --- Quadrature not available

         call errint ( B%rule, 1 )
         call errsub ( 1497, 1, 0, 0)
         go to 1000

      end select  ! case (B%rule)

!     --- Fill basis functions in quadrature points

      do k = 1, m
         h = 4d0*(1d0-xg(k,1))*xg(k,1)*(1d0-xg(k,2))*xg(k,2)
         phi(1,k) = (1d0-xg(k,1))*(1d0-xg(k,2)) - h
         phi(2,k) = xg(k,1)*(1d0-xg(k,2)) - h
         phi(3,k) = xg(k,1)*xg(k,2) - h
         phi(4,k) = (1d0-xg(k,1))*xg(k,2) - h
         phi(5,k) = 4d0*h
      end do  ! k = 1, m

!     --- Compute dphi/dxi, store in phiderc

      do k = 1, m
         h = 4d0*xg(k,2)*(1d0-xg(k,2))*(1d0-2d0*xg(k,1))
         dpdxi(1,k) = -(1d0-xg(k,2)) - h
         dpdxi(2,k) =   1d0-xg(k,2) - h
         dpdxi(3,k) =       xg(k,2) - h
         dpdxi(4,k) =      -xg(k,2) - h
         dpdxi(5,k) = 4d0*h
         h = 4d0*xg(k,1)*(1d0-xg(k,1))*(1d0-2d0*xg(k,2))
         dpdeta(1,k) = -(1d0-xg(k,1)) - h
         dpdeta(2,k) =      -xg(k,1) - h
         dpdeta(3,k) =       xg(k,1) - h
         dpdeta(4,k) =   1d0-xg(k,1) - h
         dpdeta(5,k) = 4d0*h
      end do

      B%phi = B%phi

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelp09first'

      end if  ! ( debug )
      call erclos ( 'sepelp09first' )

      end subroutine sepelp09first
