      subroutine sepelp11jacob ( e1, djacob )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-08-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute Jacobian and derivatives of basis functions
!     Linear tetrahedron (4 nodes)
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      use sepmoduledebug
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision, intent(out) :: e1(4,3), djacob

!     djacob         o    Jacobian of the element
!     e1             o    Work array of length 4x3, to store the factors
!                         e^ij
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: delta(4,3)
      integer :: k, ind(5), ind1(4), iven, i, ni

!     delta          Work array of length 4 x 3, to store the factors
!                         ij
!                    delta.    See method
!     i              Counting variable
!     ind            Integer array containing the cyclic permutation of
!                    1,2,3,4
!     ind1           Array of length 4 to store the cyclic permutation of
!                    1 - 3
!     iven           help parameter to compute (-1)
!     k              Counting variable
!     ni             Upper boundary in the computation for the loop
!                    variable i
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL2        Print a real two-dimensional array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     105   Jacobian of element is zero
! **********************************************************************
!
!                       PSEUDO CODE
!
!        To compute the derivatives of the basis functions, the jacobian
!        of the element and the weights, the following definitions are used:
!             ij     i+1     i
!        delta   =  x    -  x       cyclic;  i = 1,2,3,4;  j =  1,2,3
!                    j       j
!         ij        i        i+1,j+1     i+2,j+2        i+1,j+2     i+1,j+2
!        e   =  (-1)  [ delta       delta        - delta       delta        ]
!                        11  11        12  12       13  13
!        djacob = - delta   e   - delta   e  - delta   e
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data ind / 2,3,4,1,2 /, ind1 / 2,3,1,2 /
! ======================================================================
!
      call eropen ( 'sepelp11jacob' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp11jacob'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Compute the quantities delta^(ij)

      do i = 1, 4
         delta(i,1:3) = B%xyz(ind(i),1:3) - B%xyz(i,1:3)
      end do

      if ( B%fillderiv ) then

!     --- ideriv = 1,  compute all factors e^(ij)

         ni = 4

      else

!     --- ideriv = 0,  compute only the factors e^(1j)

         ni = 1

      end if  ! ( B%fillderiv )

!     --- Compute the quantities e^(ij)

      iven = -1
      do i = 1, ni
         do k = 1, 3
            e1(i,k) = iven * ( delta ( ind(i), ind1(k) ) * &
                      delta ( ind(i+1), ind1(k+1) ) - &
                      delta ( ind(i), ind1(k+1) ) * &
                      delta ( ind(i+1), ind1(k) ) )
         end do
         iven = - iven

      end do

      djacob = -delta(1,1)*e1(1,1)-delta(1,2)*e1(1,2)-delta(1,3)*e1(1,3)

      if ( checksign .and. djacob<0d0 ) then
         write(irefwr,*) 'area of element ', E%ielem, ' is negative '
         write(irefwr,*) 'area = ', djacob
         call prinin ( E%nodes, B%n, 'nodes' )
         call prinrl2 ( B%xyz, B%n, 3, 'coordinates' )

      end if

!     --- Check if delta is non-zero

      if ( abs(djacob)<=0d0 ) then

         call errint ( E%ielem, 1 )
         call errint ( E%itype, 2 )
         call errint ( E%ielgrp, 3 )
         call errsub ( 105, 3, 0, 0 )
         call prinin(E%nodes,B%n,'node numbers')
         call prinrl2(B%xyz,B%n,3,'co-ordinates')

      end if

      if ( B%fillderiv ) then

!     --- ideriv = 1,  subdivide e by djacob

         e1 = e1/djacob

      end if  ! ( B%fillderiv )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelp11jacob'

      end if  ! ( debug )
      call erclos ( 'sepelp11jacob' )

      end subroutine sepelp11jacob
