      subroutine sepelp12first ( intern, imidp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-09-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute some element independent quantities
!     Quadratic tetrahedrons
!     Old subroutine elp631first
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     tetrahedron
!     quadratic
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: intern(10), imidp(6,2)

!     imidp          i    Contains the i,j numbering of the mid side points in
!                         the sequence of intern
!                         So basis function
!                         phi(intern(4+i)) = 4 lambda(imidp(i,1))
!                                              lambda(imidp(i,2))
!     intern         i    Internal numbering of the nodes, vertices first
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: m, i, k, irule, j, i1, i2
      double precision :: h, dlamdxi(4,3)
      double precision, pointer :: phi(:,:), lambda(:,:), wh(:)
      double precision, pointer :: phixi(:,:,:)
      character (len=21), parameter :: text = 'Quadratic tetrahedron'

!     dlamdxi        Contains the derivatives of the linear basis functions
!     h              Help variable to store a constant temporarily
!     i              Counting variable
!     i1             help parameter in computation
!     i2             help parameter in computation
!     irule          type of quadrature rule
!     j              Counting variable
!     k              Counting variable
!     lambda         Array containing the values of lambda in the integration
!                    points  lambda(i,k) = lambda_i(xk)
!     m              number of quadrature points
!     phi            contains basis functions
!     phixi          Array of length 10 x m x 3 containing the values of the
!                    derivatives of the basis functions in the Gauss points in
!                    the reference element
!     text           String variable containing some text
!     wh             Contains the weights in the reference element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRORELVAR     Print error message errornr for element: wrong value of
!                    parameter
!     SEPELP12PHIXI  Compute dphi/dxi in quadrature points
!                    Quadratic tetrahedrons
!     SEPELPLAMBDA3D Fill the three-dimensional linear basis functions in case
!                    of a non-Newton Cotes rule
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The linear basis functions in the reference element are defined as
!     lambda(1) = 1 - xi - eta - zeta
!     lambda(2) = xi
!     lambda(3) = eta
!     lambda(4) = zeta
!     The quadratic basis functions are defined as
!     phi_i = lambda(i) ( 2 lambda(i) -1 ):   vertices
!     phi_ij = 4 lambda(i) lambda(j):         mid-side nodes
!     hence the derivatives:
!     grad(phi_i) = grad(lambda(i)) ( 4 lambda(i) -1 )
!     grad(phi_ij) = grad(lambda(i)) lambda(j) + grad(lambda(j)) lambda(i)
!     The derivatives in the reference element for the lambda's are
!                  xi    eta   zeta
!     lambda(1)    -1    -1     -1
!     lambda(2)     1     0      0
!     lambda(3)     0     1      0
!     lambda(4)     0     0      1
!     The sequence numbers of the vertices are:  1 3 5 10
!     and for the mid points we have
!     2:  1,2
!     4:  2,3
!     6:  1,3
!     7:  1,4
!     8:  2,4
!     9:  3,4
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp12first' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp12first'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      phi => B%phi
      phixi => B%phiderc
      wh => B%wc

      m = B%m

      if ( B%cart>1 ) call errorelvar ( B%cart-1, 1769, text, 0 )

      if ( B%rule==1 .or. B%fillderivn ) then

!     --- Fill lambdanc for Newton Cote rule

         lambda => B%lamn
         lambda = 0d0
         do i = 1, 4
            lambda(i,intern(i)) = 1d0
            do k = 1, 6
               if ( imidp(k,1)==i .or. imidp(k,2)==i ) then
                  lambda(i,intern(k+4)) = 0.5d0
               end if
            end do  ! k = 1, 6
         end do  ! i = 1, 4

      end if  ! ( B%rule==1 .or. B%fillderivn )

      lambda => B%lam

!     --- Fill array lambda, phi

      select case ( B%rule )

      case(1)

!     --- intrule = 1  Newton Cotes rule

         lambda = B%lamn
         phi = 0d0
         do i = 1, 10
            phi(i,i) = 1d0
         end do

         h = 1d0/120d0
         wh(intern(1:4)) = -h
         h = 4d0*h
         wh(intern(5:10)) = h

      case(2, 3)

!     --- 11 point Gauss rule; just copy the lambdas

         irule = B%rule  ! save rule
         B%rule = 5  !  11-point Gauss rule
         call sepelplambda3d ( lambda, text )
         B%rule = irule  ! reset rule

      case(4)

!     --- 15 point Gauss rule; just copy the lambdas

         B%rule = 6  !  15-point Gauss rule
         call sepelplambda3d ( lambda, text )
         B%rule = 4  ! reset rule

      end select  ! case ( intrule )

      if ( B%rule>1 ) then

!     --- Fill array phi_i

         do i = 1, 4
            phi(intern(i),:) = lambda(i,:)*(2d0*lambda(i,:)-1d0)
         end do  ! i = 1, 4
         do i = 1, 6
            i1 = imidp(i,1)
            i2 = imidp(i,2)
            phi(intern(i+4),:) = 4d0*lambda(i1,:)*lambda(i2,:)
         end do  ! i = 1, 6

      end if  ! ( intrule>1 )

!     --- fill d phi / d xsi,  d phi / d eta and dphi d zeta
!         first d lambda/ dxi

      dlamdxi(1,1:3) = -1d0
      dlamdxi(2:4,1:3) = 0d0
      do i = 1, 3
         dlamdxi(i+1,i) = 1d0
      end do  ! i = 1, 3

!     --- Compute dphi/dxi

      call sepelp12phixi ( m, phixi, intern, imidp, dlamdxi, lambda )

      do i = 1, m
         B%lamderc(1:4,i,1:3) = dlamdxi(1:4,1:3)
      end do  ! i = 1, m

      if ( B%fillderivn ) then

!     --- Fill phidercn

         if ( B%rule==1 ) then

!        --- Newton Cotes rule

            B%phidercn = B%phiderc

         else

!        --- Gauss rule

            m = 10
            lambda => B%lamn
            call sepelp12phixi ( m, B%phidercn, intern, imidp, dlamdxi, lambda )

         end if  ! ( B%rule==1 )

      end if  ! ( B%fillderivn )

1000  call erclos ( 'sepelp12first' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelp12first'

      end if  ! ( debug )

      end subroutine sepelp12first
