      subroutine sepelp13first
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-09-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute some element independent quantities
!     3d Tri-linear hexahedron (8 nodes)
!     Old subroutine: elp632
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     hexahedron
!     linear
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulebasefn
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: indi(8,3), inds(B%m,3), m, i, j, k, m2, mrule, ih, ih1, n
      double precision theta(2,2), dtetds(2,2), w(2), lambda(2)
      double precision, pointer :: weight(:), phi(:,:), phixi(:,:,:)
      character (len=21), parameter :: text = 'Tri-linear hexahedron'

!     dtetds         Derivatives of theta in the integration points
!     i              Counting variable
!     ih             Help variable to store a constant temporarily
!     ih1            Help variable to store a constant temporarily
!     indi           Array in which the triples indi, indj and indk are stored
!     inds           Array in which the triples inds1, inds2 and inds3 are
!                    stored
!     j              Counting variable
!     k              Counting variable
!     lambda         1-dimensional basis functions in quadrature points
!     m2             Contains the number of quadrature points in one direction
!     mrule          Type of integration rule as given by the user
!     n              number of basis functions
!     phi            contains basis functions
!     phixi          Array of length 8 x m x 3 containing the values of the
!                    derivatives of the basis functions in the Gauss points in
!                    the reference element.
!     text           String variable containing some text
!     theta          1D basis funtion in integration points
!     w              weights for 1-dimensional reference element
!     weight         Contains weights for reference element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRORELVAR     Print error message errornr for element: wrong value of
!                    parameter
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
!     SEPELP13PHIXI  Compute dphi/dxi in quadrature points
!                    3d Tri-linear hexahedron (8 nodes)
!     SEPELPLAMBDA1D Fill the one-dimensional linear basis functions in case of
!                    a non-Newton Cotes rule
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp13first' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp13first'
         write(irefwr,1) 'intrule, m, n', B%rule, B%m, B%n
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      weight => B%wc
      phi => B%phi
      phixi => B%phiderc

      m = B%m
      n = B%n

      if ( B%cart>1 ) call errorelvar ( B%cart-1, 1769, text, 0 )

!        --- Fill arrays theta and dtetds in integration points

      dtetds(1,:) = -1d0
      dtetds(2,:) =  1d0

      select case (B%rule)

      case (1)

!     --- intrule = 1  Newton Cotes rule

         theta(1,1) = 1d0
         theta(1,2) = 0d0
         theta(2,1) = 0d0
         theta(2,2) = 1d0
         w(1) = 0.5d0
         w(2) = 0.5d0
         mrule = 2

      case (2)

!     --- intrule = 2  One point Gauss rule

         theta(1,1) = 0.5d0
         theta(2,1) = 0.5d0
         w(1) = 1d0
         mrule = 1

      case (3)

!     --- intrule = 3  Eight point Gauss rule

         mrule = 2
         call sepelplambda1d ( lambda, text, 3, mrule, w )
         do k = 1, 2
            theta(1,k) = 1d0 - lambda(k)
            theta(2,k) = lambda(k)
         end do

      end select  ! case (intrule)
      m2 = mrule**2

!     --- fill array indi

      indi(1,1) = 1
      indi(2,1) = 2
      indi(3,1) = 2
      indi(4,1) = 1
      indi(1,2) = 1
      indi(2,2) = 1
      indi(3,2) = 2
      indi(4,2) = 2
      do k =1, 4
         indi(k,3) = 1
         indi(k+4,1) = indi(k,1)
         indi(k+4,2) = indi(k,2)
         indi(k+4,3) = 2
      end do

!     --- fill array inds

      select case (B%rule)

      case (1)
         inds = indi
      case(2)
         inds = 1
      case(3)
         do k = 1, mrule
            ih = (k-1)*m2
            do j = 1, mrule
               ih1 = ih+(j-1)*mrule
               do i = 1, mrule
                  inds(i+ih1,1) = i
                  inds(i+ih1,2) = j
                  inds(i+ih1,3) = k
               end do
            end do
         end do
      end select  ! case (intrule)

!     --- Fill array weight

      weight = w(inds(:,1))*w(inds(:,2))*w(inds(:,3))

      do i = 1, n
         phi(i,:) = theta(indi(i,1),inds(:,1))* &
                    theta(indi(i,2),inds(:,2))* &
                    theta(indi(i,3),inds(:,3))
      end do

!     --- fill phixi

      call sepelp13phixi ( phixi, n, m, 2, theta, dtetds, indi, inds )

      if ( B%fillderivn ) then

!     --- Fill also phidercn

         if ( B%rule==1 ) then

!        --- Newton Cotes rule

            B%phidercn = phixi

         else

!        --- Gauss rule, use formulas from Newton Cotes

            theta(1,1) = 1d0
            theta(1,2) = 0d0
            theta(2,1) = 0d0
            theta(2,2) = 1d0
            phixi => B%phidercn

            call sepelp13phixi ( phixi, n, n, 2, theta, dtetds, indi, indi )

         end if  ! ( B%rule==1 )

      end if  ! ( B%fillderivn )

1000  call erclos ( 'sepelp13first' )
      if ( debug ) then

!     --- Debug information

         call prinrl ( weight, m, 'weight' )
         call prinrl1 ( phi, m, 8, 'phi' )
         call prinrl1 ( phixi(1,1,1), m, 8, 'dphidxi_1' )
         call prinrl1 ( phixi(1,1,2), m, 8, 'dphidxi_2' )
         call prinrl1 ( phixi(1,1,3), m, 8, 'dphidxi_3' )
         write(irefwr,*) 'End sepelp13first'

      end if  ! ( debug )

      end subroutine sepelp13first
