      subroutine sepelp22
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-09-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute information of basis functions
!     quadratic wedge element
!     Old subroutine elp639
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     wedge
!     quadratic
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: jac(B%m)
      integer :: k, i

!     i              Counting variable
!     jac            Array of length m containing the values of the
!                    Jacobian in the vertices or for the Gauss integration
!                    rules work array to store some help variables
!     k              Loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
!     SEPELP22FIRST  compute some element independent quantities
!                    quadratic wedge element
!     SEPELP22JACOB  Compute Jacobian and derivatives of basis functions
!                    quadratic wedge element
!     SEPELPGAUSS    Fill coordinates of Gauss points
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     To compute the derivatives of the basis functions, the jacobian
!     of the element and the weights, the following definitions are used:
!     The lower triangle is defined by the points 1..6, the upper
!     triangle by 13..18
!     The basis functions are mapped on a unit wedge in the (xi, eta ,zeta )
!     space with basis functions:
!
!     phi_1..6 = psi_1..6(xi,eta) chi_1(zeta)
!     phi_7..12 = psi_1..6(xi,eta) chi_2(zeta)
!     phi_13..18 = psi_1..6(xi,eta) chi_3(zeta)
!
!     with psi_1..6 the basis functions for a quadratic triangle
!     and ch_1..3 the basis functions for a quadratic line
!
!     Hence the triangles are in the xi,eta planes and the upwards direction
!     is defined as zeta
!
!     The weights for the numerical integration rule are given as follows:
!     Let J be the Jacobian in the integration points then:
!
!     intrule = 1: (Newton Cotes rule)  w = 1/12 J(x_k)
!     intrule = 2: (Four point Gauss rule in xi-eta dir and 2 point
!                   Gauss rule in zeta dir)  w = 1/2 J(x_k)
!     intrule = 3: (Seven point Gauss rule in xi-eta dir and 2 point
!                   Gauss rule in zeta dir)  w = 1/2 J(x_k)
!
!
!     dphi_dxi_j can be computed immediately from the standard basis functions
!
!     dx/dxi_j = sum_k x_k dphi_k/dxi_j
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp22' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp22'
         write(irefwr,1) 'irule, n, m', B%rule, B%n, B%m
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( E%first==0 ) call sepelp22first

      if ( debug ) then
         call prinrl1 ( B%phi, B%m, B%n, 'phi' )
         call prinrl1 ( B%phiderc(1,1,1), B%m, B%n, 'dphi/dxi1' )
         call prinrl1 ( B%phiderc(1,1,2), B%m, B%n, 'dphi/dxi2' )
         call prinrl1 ( B%phiderc(1,1,3), B%m, B%n, 'dphi/dxi3' )
         call prinrl ( B%wc, B%m, 'weight')
      end if  ! ( debug )

!     --- Fill Gauss points

      call sepelpgauss

!     --- Compute Jacobian and derivatives of basis functions

      call sepelp22jacob ( jac )

      if ( debug ) call prinrl ( jac, B%m, 'jacobian' )

!     --- Fill weights

      B%w = abs(jac)*B%wc
      if ( debug ) call prinrl ( B%w, B%m, 'weights' )

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( B%phider(1,1,1), B%m, B%n, 'dphi/dx' )
         call prinrl1 ( B%phider(1,1,2), B%m, B%n, 'dphi/dy' )
         call prinrl1 ( B%phider(1,1,3), B%m, B%n, 'dphi/dz' )
         call prinrl ( B%w, B%m, 'w')
         write(irefwr,*) 'End sepelp22'

      end if  ! ( debug )
      call erclos ( 'sepelp22' )

      end subroutine sepelp22
