      subroutine sepelp22first
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-09-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute some element independent quantities
!     quadratic wedge element
!     Old subroutine elp639first
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     wedge
!     quadratic
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, k, m, m2, m1, indm, j, ih
      double precision, allocatable ::  dphitridxi(:,:,:), phitri(:,:), &
                                        weighttri(:), lambdatri(:,:)
      double precision :: lambdaz(3), wz(3), lam1(3), phiz(3,3), phiderz(3,3)
      double precision :: h
      double precision :: xi(B%m), eta(B%m), zeta(B%m)
      double precision, pointer :: weight(:), phi(:,:)
      character (len=19), parameter :: text = 'Tri-quadratic wedge'

!     dphitridxi     Contains the derivatives of the basis functions in a
!                    quadratic reference triangle
!     eta            Local eta coordinates in quadrature points
!     h              Help variable to store a constant temporarily
!     i              Counting variable
!     ih             Help variable to store a constant temporarily
!     indm           Index in 3d basis function
!     j              Counting variable
!     k              Counting variable
!     lam1           1-lambdaz
!     lambdatri      Contains the values of the linear 2d basis function
!                    in a triangle reference element
!     lambdaz        Contains the values of the linear 1d basis function
!                    in a reference element
!     m              number of quadrature points
!     m1             Number of integration points in xi,eta direction
!     m2             Number of integration points in zeta direction
!     phi            contains basis functions
!     phiderz        Contains the derivatives of the basis functions in a
!                    1d quadratic reference element
!     phitri         Contains the values of the quadratic 2d basis function
!                    in a triangle reference element
!     phiz           Contains the values of the quadratic 1d basis function
!                    in a line reference element
!     text           String variable containing some text
!     weight         Contains weights for reference element
!     weighttri      Contains the weights in the reference triangle
!     wz             Contains the weights in the reference line element
!     xi             Local xi coordinates in quadrature points
!     zeta           Local zeta coordinates in quadrature points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRORELVAR     Print error message errornr for element: wrong value of
!                    parameter
!     ERRORELVAR2    Print error message errornr for element: wrong value of
!                    2 parameters
!     ERRSUB         Error messages
!     SEPELP04DET    compute some element independent quantities
!                    Quadratic triangle (6 nodes)
!     SEPELP21PHIXI  Compute dphi/dxi in quadrature points
!                    linear wedge element
!     SEPELP22PHIXI  Compute dphi/dxi in quadrature points
!                    quadratic wedge element
!     SEPELPLAMBDA1D Fill the one-dimensional linear basis functions in case of
!                    a non-Newton Cotes rule
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1497   Type of integration rule not yet available
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp22first' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp22first'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      weight => B%wc
      phi => B%phi

      if ( B%cart>1 ) call errorelvar ( B%cart-1, 1769, text, 0 )

      m = B%m

      select case (B%rule)

      case (1)

!     --- Newton Cotes rule

         if ( m/=18 ) call errorelvar2 ( m, B%rule, 1424, text, 0 )
         if ( ierror/=0 ) go to 1000

         B%phi = 0d0
         do i = 1, m
            B%phi(i,i) = 1d0
         end do
         lambdaz(1) = 0d0
         lambdaz(2) = 0.5d0
         lambdaz(3) = 1d0
         h = 1d0/6d0
         wz(1) = h
         wz(2) = 4d0*h
         wz(3) = h
         m2 = 3

      case(2, 3)

!     --- intrule = 2, 3  2/3-point Gauss rule

         m2 = B%rule
         call sepelplambda1d ( lambdaz, text, m2+1, m2, wz )

      case default

!     --- Quadrature not available

         call errint ( B%rule, 1 )
         call errsub ( 1497, 1, 0, 0)
         go to 1000

      end select  ! case (B%rule)

!     --- Fill array phiz

      lam1(1:m2) = 1d0-lambdaz(1:m2)
      phiz(1,1:m2) = lam1(1:m2)*(2d0*lam1(1:m2)-1d0)
      phiz(2,1:m2) = 4d0*lambdaz(1:m2)*lam1(1:m2)
      phiz(3,1:m2) = lambdaz(1:m2)*(2d0*lambdaz(1:m2)-1d0)

      phiderz(1,1:m2) = -(4d0*lam1(1:m2)-1d0)
      phiderz(2,1:m2) = 4d0*(lam1(1:m2)-lambdaz(1:m2))
      phiderz(3,1:m2) = 4d0*lambdaz(1:m2)-1d0

!     --- Fill array phitri ( corresponding to triangle)

      m1 = m/m2
      allocate ( dphitridxi(6,m1,2), phitri(6,m1), weighttri(m1), &
                 lambdatri(3,m1), stat = error )
      if ( error/=0 ) call eralloc ( error, 22*m1, 'dphitridxi' )
      if ( ierror/=0 ) go to 1000

      call sepelp04det ( lambdatri, text, B%rule, m1, weighttri, phitri, &
                         dphitridxi )

!     --- Fill arrays phi, phiksi and weight

      do i = 1, m1

         do j = 1, m2

            indm = i+(j-1)*m1
            phi(1:6,indm) = phitri(1:6,i)*phiz(1,j)
            phi(7:12,indm) = phitri(1:6,i)*phiz(2,j)
            phi(13:18,indm) = phitri(1:6,i)*phiz(3,j)

            weight(indm) = weighttri(i)*wz(j)

         end do  ! j = 1, m2

      end do  ! i = 1, m1
      B%phic = B%phi

      call sepelp22phixi ( m, m1, m2, B%phiderc, dphitridxi, phitri, &
                           phiz, phiderz )

      if ( B%comppsi ) then

!     --- Fill B%lamderc

  !       B%lamder => B%phider

         do i = 1, m2
            ih = (i-1)*m1
            zeta(ih+1:ih+m1) = lambdaz(i)
            ih = (i-1)*m1
            xi(ih+1:ih+m1) = lambdatri(2,1:m1)
            eta(ih+1:ih+m1) = lambdatri(3,1:m1)
         end do  ! i = 1, m2
         call sepelp21phixi ( B%m, B%lamderc, xi, eta, zeta )

      end if  ! ( B%comppsi )

      deallocate ( dphitridxi, phitri, weighttri, lambdatri, &
                   stat = error )
      if ( error/=0 ) call erdealloc ( error, 'dphitridxi' )
      if ( ierror/=0 ) go to 1000

      if ( B%fillderivn ) then

!     --- Fill also phidercn

         if ( B%rule==1 ) then

!        --- Newton Cotes rule

            B%phidercn = B%phiderc

         else

!        --- Gauss rule, use formulas from Newton Cotes

            m = 18
            m1 = 6
            m2 = 3

            lambdaz(1) = 0d0
            lambdaz(2) = 0.5d0
            lambdaz(3) = 1d0

            lam1(1:m2) = 1d0-lambdaz(1:m2)
            phiz(1,1:m2) = lam1(1:m2)*(2d0*lam1(1:m2)-1d0)
            phiz(2,1:m2) = 4d0*lambdaz(1:m2)*lam1(1:m2)
            phiz(3,1:m2) = lambdaz(1:m2)*(2d0*lambdaz(1:m2)-1d0)

            phiderz(1,1:m2) = -(4d0*lam1(1:m2)-1d0)
            phiderz(2,1:m2) = 4d0*(lam1(1:m2)-lambdaz(1:m2))
            phiderz(3,1:m2) = 4d0*lambdaz(1:m2)-1d0

            allocate ( dphitridxi(6,m1,2), phitri(6,m1), weighttri(m1), &
                       lambdatri(3,m1), stat = error )
            if ( error/=0 ) call eralloc ( error, 22*m1, 'dphitridxi' )
            if ( ierror/=0 ) go to 1000

            call sepelp04det ( lambdatri, text, 1, m1, weighttri, phitri, &
                               dphitridxi )

            call sepelp22phixi ( m, m1, m2, B%phidercn, dphitridxi, phitri, &
                                 phiz, phiderz )

            deallocate ( dphitridxi, phitri, lambdatri, weighttri, &
                         stat = error )

         end if  ! ( B%rule==1 )

      end if  ! ( B%fillderivn )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelp22first'

      end if  ! ( debug )
      call erclos ( 'sepelp22first' )

      end subroutine sepelp22first
