      subroutine sepelplambda1d ( lambda, text, irule, m, weight )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-08-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the one-dimensional linear basis functions in case of
!     a non-Newton Cotes rule
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: irule, m
      double precision, intent(out) :: lambda(m), weight(m)
      character (len=*) text

!     irule          i    Type of quadrature rule
!     lambda         o    Array containing the linear basis functions in the
!                         integration points
!     m              i    Number of quadrature points
!     text           i    String variable containing some text
!     weight         o    Weights for quadrature
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: k
      double precision :: d

!     d              help parameter to compute a constant
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRORELVAR     Print error message errornr for element: wrong value of
!                    parameter
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1424   m does not correspond to irule
!    1770   Type of integration rule incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelplambda1d' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelplambda1d'
         write(irefwr,1) 'rule', irule
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      select case ( irule )

         case(2)

!        --- irule = 2  one-point Gauss rule

            if ( m/=1 ) call errorelvar2 ( m, irule, 1424, text, 0 )
            if ( ierror/=0 ) go to 1000
            lambda(1) = 0.5d0
            weight(1) = 1d0

         case(3)

!        --- irule = 3  two-point Gauss rule

            if ( m/=2 ) call errorelvar2 ( m, irule, 1424, text, 0 )
            if ( ierror/=0 ) go to 1000
            d = sqrt(3d0)/6d0
            lambda(1) = 0.5d0 - d
            lambda(2) = 0.5d0 + d
            weight(1:2) = 0.5d0

         case(4)

!        --- irule = 4  three-point Gauss rule

            if ( m/=3 ) call errorelvar2 ( m, irule, 1424, text, 0 )
            if ( ierror/=0 ) go to 1000
            d = sqrt(15d0)/10d0
            lambda(1) = 0.5d0 - d
            lambda(2) = 0.5d0
            lambda(3) = 0.5d0 + d
            weight(1)  = 5d0/18d0
            weight(2)  = 8d0/18d0
            weight(3)  = weight(1)

         case(5)

!        --- irule = 5  Simpson's rule

            if ( m/=3 ) call errorelvar2 ( m, irule, 1424, text, 0 )
            if ( ierror/=0 ) go to 1000
            do k = 1, 3
               lambda(k) = (k-1)*0.5d0
            end do

            d = 1d0/6d0
            weight(1) = d
            weight(2) = 4d0*d
            weight(3) = d

         case(6)

!        --- irule = 6  four-point Gauss rule

            if ( m/=4 ) call errorelvar2 ( m, irule, 1424, text, 0 )
            if ( ierror/=0 ) go to 1000
            d = 0.3399810435848563d0*0.5d0
            lambda(1) = 0.5d0 - d
            lambda(2) = 0.5d0 + d
            d = 0.8611363115940526d0*0.5d0
            lambda(3) = 0.5d0 - d
            lambda(4) = 0.5d0 + d
            weight(1:2)  = 0.6521451548625461d0*0.5d0
            weight(3:4)  = 0.3478548451374538d0*0.5d0

         case(7)

!        --- irule = 7  five-point Gauss rule

            if ( m/=5 ) call errorelvar2 ( m, irule, 1424, text, 0 )
            if ( ierror/=0 ) go to 1000
            lambda(1) = 0.5d0
            d = 0.5384693101056831d0*0.5d0
            lambda(2) = 0.5d0 - d
            lambda(3) = 0.5d0 + d
            d = 0.9061798459386640d0*0.5d0
            lambda(4) = 0.5d0 - d
            lambda(5) = 0.5d0 + d
            weight(1)  = 0.5688888888888889d0*0.5d0
            weight(2:3)  = 0.4786286704993665d0*0.5d0
            weight(4:5)  = 0.2369268850561891d0*0.5d0

         case default

!        --- Error value of irule not implemented

            call errorelvar ( irule, 1770, text, 1 )

      end select  ! case ( irule )

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( weight, m, 'wc' )
         call prinrl ( lambda, m, 'lambda' )
         write(irefwr,*) 'End sepelplambda1d'

      end if  ! ( debug )
      call erclos ( 'sepelplambda1d' )

      end subroutine sepelplambda1d
