      subroutine sepelpupw01
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 27-08-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute modified basis functions for upwind elements
!    One-dimensional elements
! **********************************************************************
!
!                       KEYWORDS
!
!     1d
!     basis_function
!     element
!     upwind
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletime
      use sepmodulecomio
      use sepmoduleelm
      use sepmodulebasefn
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: beta, h, xsi(6), tau(6)
      double precision, pointer :: phix(:,:), x(:)
      integer :: i, k

!     beta           factor beta (see method)
!     h              element width (see method)
!     i              Counting variable
!     k              Counting variable
!     phix           x derivative of basis function phi
!     tau            Array of length 6 to store the values of tau in the
!                    integration points
!     x              x-coordinates of nodal points
!     xsi            Array of length m to store the values of xsi in the
!                    integration points (see method)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!       The basis functions are modified such that:  phi    = phi + p
!                                                       i        i   i
!       or:
!          new  k          k        k
!       phi   (x ) = phi (x ) + p (x )
!          i            i        i
!       The basis funtions phi are assumed to be filled before
!       p  = tau u * d phi  / dx
!        i                i
!                   h xi           uh
!       with tau = ------ beta =  -----
!                     2          2 alpha
!       xi depends on metupw according to:
!       metupw = 1:  Classical upwind scheme  xi = 1
!                2:  Il'in    xi = coth(beta) - 1/beta
!                                             beta/3     -3<beta<3
!                3:  Doubly assymptotic: xi =
!                                             sign(beta)  |beta|>3
!                                         -1-1/beta     beta<1
!                4:  critical approx: xi = 0            -1<beta<1
!                                          1-1/beta     beta>1
!       h = element diameter (linear) or half diameter (quadratic)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelpupw01' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelpupw01'
         write(irefwr,1) 'methodupw', E%methodupw
  1      format ( a, 1x, (10i8) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      phix => B%phider(:,:,1)
      x => B%xyz(:,1)

      h = x(2) - x(1)

      if ( debug ) write(irefwr,2) 'h', h

!     --- Compute beta and xsi in the integration points

      if ( E%methodupw<=4 ) then

!     --- methodupw = 1, .. , 4:  old methods

         do i = 1, B%m

            if ( E%ind(1)/=0 .and. E%diffvec(i)>1d-10 ) then

!           --- diffvec(i) > 1d-10,  hence second order term plays a role

               beta = E%uvel(i)*h/(2d0*E%diffvec(i))
               if ( abs(beta)<1d-10 ) then
                  xsi(i) = 0d0
               else if ( E%methodupw==1 ) then

!              --- methodupw = 1   classical upwind scheme

                  xsi(i) = 1d0

               else if ( E%methodupw==2 ) then

!              --- methodupw = 2   il'in scheme

                  xsi(i) = 1d0/tanh(beta)-1d0/beta

               else if ( E%methodupw==3 ) then

!              --- methodupw = 3    doubly assymptotic approximation

                  if ( abs(beta)>3d0 ) then
                     xsi(i)=sign(1d0,beta)
                  else
                     xsi(i) = beta / 3d0
                  end if

               else if ( E%methodupw==4 ) then

!              --- methodupw = 4    critical approxsi(i)mation

                  if ( beta<-1d0 ) then
                     xsi(i) = -1d0 - 1d0 / beta
                  else if ( beta>1d0 ) then
                     xsi(i) = 1d0 - 1d0 / beta
                  else
                     xsi(i) = 0d0
                  end if

               end if

            else

               xsi(i) = 1d0

            end if

         end do

!        --- compute new basis functions, it has been assumed that the old
!            basis functions are already stored in array phi

         if ( B%derivconst ) then

!        --- jdercn = 0,  constant derivatives

            do k = 1, B%m
               do i = 1, B%n
                  B%psi(i,k) = B%phi(i,k) + E%uvel(k)*h*xsi(k)*0.5d0*phix(i,1)
               end do
            end do
         else

!        --- jdercn = 1,  variable derivatives

            do k = 1, B%m
               do i = 1, B%n
                  B%psi(i,k) = B%phi(i,k) + E%uvel(k)*h*xsi(k)*0.5d0*phix(i,k)
               end do
            end do
         end if
      else

!     --- methodupw = 5, 6:  new methods

         if ( E%methodupw==5 ) then

!        --- methodupw = 5

            do i = 1, B%m
               if ( E%ind(1)/=0 .and. E%diffvec(i)>1d-10 ) then

!              --- diffvec(i) > 1d-10,  hence second order term plays a role

                  beta = abs(E%uvel(i))*h/(2d0*E%diffvec(i))
                  if ( abs(E%uvel(i))<=1d-5*E%diffvec(i) ) then
                     tau(i) = 0d0
                  else
                     beta = beta*beta
                     tau(i) = h/(2d0*abs(E%uvel(i)))*sqrt(beta/(9d0+beta))
                  end if
               else if ( E%uvel(i)<=1d-10*E%diffvec(i) ) then
                  tau(i) = 0d0
               else
                  tau(i) = h/(2d0*abs(E%uvel(i)))
               end if
            end do
         else

!        --- methodupw = 6

            if ( E%ind(1)>0 ) then

!           --- second order term not available

               do i = 1, B%m
                  beta = (2d0*abs(E%uvel(i))/h)**2
                  if ( tstep>0d0 ) beta = beta + 4d0/(tstep*tstep)
                  tau(i) = 1d0/sqrt(beta)
               end do
            else

!           --- second order term available

               do i = 1, B%m
                  beta = (2d0*abs(E%uvel(i))/h)**2 + (4d0*E%diffvec(i)/(h*h))**2
                  if ( tstep>0d0 ) beta = beta + 4d0/(tstep*tstep)
                  tau(i) = 1d0/sqrt(beta)
               end do
            end if
         end if
         if ( B%derivconst ) then

!        --- jdercn = 0,  constant derivatives

            do k = 1, B%m
               do i = 1, B%n
                  B%psi(i,k) = B%phi(i,k) + E%uvel(k) * tau(k) * phix(i,1)
               end do
            end do
         else

!        --- jdercn = 1,  variable derivatives

            do k = 1, B%m
               do i = 1, B%n
                  B%psi(i,k) = B%phi(i,k) + E%uvel(k) * tau(k) * phix(i,k)
               end do
            end do
         end if
      end if

!     --- Check if second order upwind term is relevant

      do k = 1, B%m
         if ( abs(E%diffvec(k))*1d2>abs(E%uvel(k)) ) B%secondupw = .true.
      end do

1000  call erclos ( 'sepelpupw01' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelpupw01'

      end if  ! ( debug )

      end subroutine sepelpupw01
