      subroutine sepelvec803
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-03-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute extra contribution for element vector corresponding to
!     newton linearization of special nonlinear equation treated in SP 3.3.2
!     Old subroutine el1020
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!     element_group
!     element_matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleoldvecs
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j, k, n, m
      double precision :: c, fac, som
      double precision, pointer :: gradph(:), dalpdb(:), w(:)
      double precision, pointer :: dcdx(:), dcdy(:), dcdz(:)
      double precision, pointer :: phix(:,:), phiy(:,:), phiz(:,:)
      double precision, pointer :: elemvc(:)

!     c              coefficient
!     dalpdb         d alpha / d (grad c) in integration points
!     dcdx           dc/dx
!     dcdy           dc/dy
!     dcdz           dc/dz
!     elemvc         Element vector
!     fac            coefficient
!     gradph         || grad c ||
!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     m              number of quadrature points
!     n              number of basis functions for each component
!     phix           dphi/dx
!     phiy           dphi/dy
!     phiz           dphi/dz
!     som            Help parameter to compute a sum
!     w              weights for integration
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelvec803' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelvec803'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      n = B%n
      m = B%m

      i = E%indrhocp
      gradph => E%coeffs(:,i+1)
      dalpdb => E%coeffs(:,i+2)
      dcdx => gradoldvec(:,1,1)
      dcdy => gradoldvec(:,2,1)
      dcdz => gradoldvec(:,3,1)
      w => B%w
      phix => B%phider(:,:,1)
      phiy => B%phider(:,:,2)
      phiz => B%phider(:,:,3)
      elemvc => E%vec

      select case ( E%ndim )

      case(1)

!     --- ndim = 1

         c = 0d0
         if ( n.eq.2 ) then

!        --- n = 2, linear elements, hence grad phi and grad c constant

            do k = 1, m
               c = c + dalpdb(k)*gradph(k)*w(k)
            end do
            do i = 1, n
               som = c * dcdx(1)*phix(i,1)
               elemvc(i) = elemvc(i) + som
            end do

         else

!        --- n > 2, non-linear elements

            do i = 1, n
               som = 0d0
               do k = 1, m
                  som = som + dalpdb(k)*gradph(k)*w(k) * &
                              dcdx(k)*phix(i,k)
               end do
               elemvc(i) = elemvc(i) + som
            end do

         end if

      case(2)

!     --- ndim = 2

         if ( n.eq.3 ) then

!        --- n = 3, linear elements, hence grad phi and grad c constant

            c = 0d0
            do k = 1, m
               c = c + dalpdb(k)*gradph(k)*w(k)
            end do
            do i = 1, n
               som = c * (dcdx(1)*phix(i,1)+ &
                          dcdy(1)*phiy(i,1))
               elemvc(i) = elemvc(i) + som
            end do

         else

!        --- n > 3, non-linear elements

            do i = 1, n
               som = 0d0
               do k = 1, m
                  som = som + dalpdb(k)*gradph(k)*w(k) * &
                              (dcdx(k)*phix(i,k)+ &
                               dcdy(k)*phiy(i,k))
               end do
               elemvc(i) = elemvc(i) + som
            end do

         end if

      case(3)

!     --- ndim = 3

         if ( n.eq.4 ) then

!        --- n = 4, linear elements, hence grad phi and grad c constant

            c = 0d0
            do k = 1, m
               c = c + dalpdb(k)*gradph(k)*w(k)
            end do
            do i = 1, n
               som = c * (dcdx(1)*phix(i,1)+ &
                          dcdy(1)*phiy(i,1)+ &
                          dcdz(1)*phiz(i,1))
               elemvc(i) = elemvc(i) + som
            end do

         else

!        --- n > 4, non-linear elements

            do i = 1, n
               som = 0d0
               do k = 1, m
                  som = som + dalpdb(k)*gradph(k)*w(k) * &
                              (dcdx(k)*phix(i,k)+ &
                               dcdy(k)*phiy(i,k)+ &
                               dcdz(k)*phiz(i,k))
               end do
               elemvc(i) = elemvc(i) + som
            end do

         end if

      end select  ! case ( E%ndim )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelvec803'

      end if  ! ( debug )
      call erclos ( 'sepelvec803' )

      end subroutine sepelvec803
