      subroutine sepfilldmat
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 30-06-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill D-matrix for linear elasticity problem
!
! **********************************************************************
!
!                       KEYWORDS
!
!     elasticity_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulecomio
      use sepmodulebasefn
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: factor(B%m), fac1(B%m), fac2(B%m), fac3(B%m)
      double precision, pointer :: nu1(:), Elas1(:), nu2(:), Elas2(:)
      double precision, pointer :: G2(:), Dmat(:,:,:)
      integer :: i, j, nd, igprob

!     Dmat           D matrix
!     Elas1          factor E1
!     Elas2          factor E2
!     G2             factor G2
!     fac1           Multiplication factor
!     fac2           Multiplication factor
!     fac3           Multiplication factor
!     factor         Multiplication factor
!     i              Counting variable
!     j              Counting variable
!     nd             Formal dimension corresponding to n
!     nu1            factor nu1
!     nu2            factor nu2
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfilldmat' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfilldmat'
         write(irefwr,1) 'stress_strain', E%stress_strain
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nu1 => E%coeffs(:,2)
      Elas1 => E%coeffs(:,1)
      Dmat => E%Dmat
      igprob = E%stress_strain+1
      if ( E%itype==256 ) igprob = 100+E%stress_strain

      select case ( igprob )

      case(1)

!     --- igprob = 1  plane stress isotropic

         if ( E%ndim==2 ) then

!        --- ndim = 2

            factor = Elas1/(1d0-nu1**2)
            fac1 = nu1*factor
            Dmat(1,1,:) = factor
            Dmat(2,2,:) = factor
            Dmat(1,2,:) = fac1
            Dmat(3,3,:) = 0.5d0*(1d0-nu1)*factor
            Dmat(2,1,:) = fac1

         else

!        --- ndim = 3

            factor = Elas1*(1d0-nu1)/((1d0+nu1)*(1d0-2d0*nu1))
            fac1 = nu1*factor/(1d0-nu1)
            fac2 = (1d0-2d0*nu1)*factor/(2d0*(1d0-nu1))
            Dmat(1,1,:) = factor
            Dmat(2,2,:) = factor
            Dmat(3,3,:) = factor
            Dmat(1,2,:) = fac1
            Dmat(1,3,:) = fac1
            Dmat(2,1,:) = fac1
            Dmat(2,3,:) = fac1
            Dmat(3,1,:) = fac1
            Dmat(3,2,:) = fac1
            Dmat(4,4,:) = fac2
            Dmat(5,5,:) = fac2
            Dmat(6,6,:) = fac2

         end if

      case(2)

!     --- igprob = 2  plane strain isotropic

         factor = Elas1*(1d0-nu1)/((1d0+nu1)*(1d0-2d0*nu1))
         fac1 = nu1*factor/(1d0-nu1)
         Dmat(1,1,:) = factor
         Dmat(2,2,:) = factor
         Dmat(1,2,:) = fac1
         Dmat(3,3,:) = Elas1/(2d0*(1d0+nu1))
         Dmat(2,1,:) = fac1

      case(3)

!     --- igprob = 3  axi-symmetric stress analysis   isotropic

         factor = Elas1/((1d0+nu1)*(1d0-2d0*nu1))
         fac1 = factor*(1d0-nu1)
         fac2 = nu1*factor
         do i = 1, 3
            Dmat(i,i,:) = fac1
            do j = 1, i-1
               Dmat(i,j,:) = fac2
               Dmat(j,i,:) = fac2
            end do
            Dmat(4,4,:) = Elas1/(2d0*(1d0+nu1))
         end do

      case(4)

!     --- igprob = 4  plane stress orthotropic

         Elas2 => E%coeffs(:,2)
         nu2 => E%coeffs(:,4)
         G2 => E%coeffs(:,5)
         fac1 = Elas1/Elas2
         factor = Elas2/(1d0-fac1*nu2**2)
         Dmat(1,1,:) = factor*fac1
         Dmat(2,2,:) = factor
         Dmat(1,2,:) = factor*fac1*nu2
         Dmat(3,3,:) = G2
         Dmat(2,1,:) = Dmat(1,2,:)

      case(5)

!     --- igprob = 5  plane strain orthotropic

         Elas2 => E%coeffs(:,2)
         nu1 => E%coeffs(:,3)
         nu2 => E%coeffs(:,4)
         G2 => E%coeffs(:,5)
         fac1 = Elas1/Elas2
         factor = Elas2/((1d0+nu1)* &
                          (1d0-nu1-2d0*fac1*nu2**2))
         Dmat(1,1,:) = factor*fac1*(1d0-fac1*nu2**2)
         Dmat(2,2,:) = factor*(1d0-nu1**2)
         Dmat(1,2,:) = factor*fac1*nu2*(1d0+nu1)
         Dmat(2,1,:) = Dmat(1,2,:)
         Dmat(3,3,:) = G2

      case(6)

!     --- igprob = 6  axi-symmetric stress analysis   stratified material

         Elas2 => E%coeffs(:,2)
         nu1 => E%coeffs(:,3)
         nu2 => E%coeffs(:,4)
         G2 => E%coeffs(:,5)
         fac1 = Elas1/Elas2
         factor = Elas2/((1d0+nu1)*(1d0-nu1-2d0*fac1*nu2**2))
         Dmat(1,1,:) = factor*(1d0-nu1**2)
         Dmat(1,2,:) = factor*fac1*nu2*(1d0+nu1)
         Dmat(1,3,:) = Dmat(1,2,:)
         Dmat(2,1,:) = Dmat(1,2,:)
         Dmat(3,1,:) = Dmat(1,2,:)
         Dmat(2,2,:) = factor*fac1*(1d0-fac1*nu2**2)
         Dmat(3,3,:) = Dmat(2,2,:)
         Dmat(2,3,:) = factor*fac1*(nu1+fac1*nu2**2)
         Dmat(3,2,:) = Dmat(2,3,:)
         Dmat(4,4,:) = G2

      case(7)

!        --- igprob = 7  plane stress/strain User defined matrix

         Dmat(1,1,:) = E%coeffs(:,1)
         Dmat(1,2,:) = E%coeffs(:,2)
         Dmat(1,3,:) = E%coeffs(:,4)
         Dmat(2,1,:) = E%coeffs(:,2)
         Dmat(2,2,:) = E%coeffs(:,3)
         Dmat(2,3,:) = E%coeffs(:,5)
         Dmat(3,1,:) = E%coeffs(:,4)
         Dmat(3,2,:) = E%coeffs(:,5)
         Dmat(3,3,:) = E%coeffs(:,6)

     case(8)

!    --- igprob = 8  axisymmetric User defined matrix

         Dmat(1,1,:) = E%coeffs(:,1)
         Dmat(1,2,:) = E%coeffs(:,2)
         Dmat(1,3,:) = E%coeffs(:,3)
         Dmat(1,4,:) = E%coeffs(:,4)
         Dmat(2,1,:) = E%coeffs(:,2)
         Dmat(2,2,:) = E%coeffs(:,5)
         Dmat(2,3,:) = E%coeffs(:,6)
         Dmat(2,4,:) = E%coeffs(:,7)
         Dmat(3,1,:) = E%coeffs(:,3)
         Dmat(3,2,:) = E%coeffs(:,6)
         Dmat(3,3,:) = E%coeffs(:,8)
         Dmat(3,4,:) = E%coeffs(:,9)
         Dmat(4,1,:) = E%coeffs(:,4)
         Dmat(4,2,:) = E%coeffs(:,7)
         Dmat(4,3,:) = E%coeffs(:,9)
         Dmat(4,4,:) = E%coeffs(:,10)

      case(9)

!     --- igprob = 9  3d user defined matrix

         Dmat(1,1,:) = E%coeffs(:,1)
         Dmat(1,2,:) = E%coeffs(:,2)
         Dmat(1,3,:) = E%coeffs(:,3)
         Dmat(1,4,:) = E%coeffs(:,4)
         Dmat(1,5,:) = E%coeffs(:,5)
         Dmat(1,6,:) = E%coeffs(:,6)
         Dmat(2,2,:) = E%coeffs(:,7)
         Dmat(2,3,:) = E%coeffs(:,8)
         Dmat(2,4,:) = E%coeffs(:,9)
         Dmat(2,5,:) = E%coeffs(:,10)
         Dmat(2,6,:) = E%coeffs(:,11)
         Dmat(3,3,:) = E%coeffs(:,12)
         Dmat(3,4,:) = E%coeffs(:,13)
         Dmat(3,5,:) = E%coeffs(:,14)
         Dmat(3,6,:) = E%coeffs(:,15)
         Dmat(4,4,:) = E%coeffs(:,16)
         Dmat(4,5,:) = E%coeffs(:,17)
         Dmat(4,6,:) = E%coeffs(:,18)
         Dmat(5,5,:) = E%coeffs(:,19)
         Dmat(5,6,:) = E%coeffs(:,20)
         Dmat(6,6,:) = E%coeffs(:,21)
         do i = 2, 6
            do j = 1, i-1
               Dmat(i,j,:) =  Dmat(j,i,:)
            end do  ! j = 1, i-1
         end do  ! i = 2, 6

      case(100)

!     --- igprob = 100, shell element

         factor = Elas1/(1d0-nu1**2)
         fac1 = nu1*factor
         fac2 = 0.5d0*(1d0-nu1)*factor
         fac3 = 5d0*fac2/6d0
         Dmat(1,1,:) = factor
         Dmat(2,2,:) = factor
         Dmat(1,2,:) = fac1
         Dmat(3,3,:) = fac2
         Dmat(4,4,:) = fac3
         Dmat(5,5,:) = fac3
         Dmat(2,1,:) = fac1

      end select  ! case (igprob)

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( Dmat, E%dimD, E%dimD, 'D' )
         write(irefwr,*) 'End sepfilldmat'

      end if  ! ( debug )
      call erclos ( 'sepfilldmat' )

      end subroutine sepfilldmat
