      subroutine el0204 ( iuser, user, jelp, igprob, imatlaw, &
                          iusrvc, iusrflg, nd, isub, isup, array, &
                          npres, ndisp, isequ, iseqp, iseqmu, iseqksi, &
                          ncoef )
! ======================================================================
!
!        programmer    Chris van Oijen
!        version  3.0  date 06-01-2016 Use sepmoduleelem
!        version  2.1  date 19-05-2003 Added new shapes (RvL)
!        version  2.0  date 23-08-2001 Complete revision
!
!   copyright (c) 2000-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     This subroutine performs the initializations for subroutine ELM200
!     or ELD200 in the case that IFIRST = 0
!     The parameters in the common blocks celint, celiar and celp
!     get a start value.
!     Array array is initialized with the values of the coefficients
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     initialization
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision user(*), array(*)
      integer iuser(*), jelp, igprob, imatlaw, iusrvc, iusrflg, &
              isub, isup, isequ(*), iseqp(*), iseqmu(*), iseqksi(*), &
              nd, npres, ndisp, ncoef

!     array         o    Real work array in which the values of the
!                        coefficients in the integration points are stored
!     igprob        o    Type of stress strain-relation
!     imatlaw       o    Type of material constitutive equation to be used
!     iseqksi       i    ?
!     iseqmu        i    ?
!     iseqp         o    Sequence for pressure dof's
!     isequ         o    Sequence for displacement dof's
!     isub          o    Integer help parameter for do-loops to indicate the
!                        smallest coefficient number that is dependent on space
!     isup          o    Integer help parameter for do-loops to indicate the
!                        largest coefficient number that is dependent on space
!     iuser         i    Integer user array to pass user information from
!                        main program to subroutine. See STANDARD PROBLEMS
!     iusrflg       i    User flag
!     iusrvc        i    if > 0, a user defined vector is used
!     jelp          o    Indication of the type of basis function subroutine
!                        must be called by ELM100. Possible values:
!                         1:  Subroutine ELP610  (Linear line element)
!                         2:  Subroutine ELP611  (Quadratic line element)
!                         3:  Subroutine ELP620  (Linear triangle)
!                         4:  Subroutine ELP622  (Quadratic triangle)
!                         5:  Subroutine ELP621  (Bilinear quadrilateral)
!                         6:  Subroutine ELP623  (Biquadratic quadrilateral)
!                        11:  Subroutine ELP630  (Linear tetrahedron)
!                        12:  Subroutine ELP631  (Quadratic tetrahedron)
!                        13:  Subroutine ELP632  (Trilinear brick)
!                        14:  Subroutine ELP633  (Triquadratic brick)
!     ncoef         i    Number of coefficients in the equation
!     nd            o    Dimension of tangent stiffness matrix
!     ndisp         o    Number of displacement dof's
!     npres         o    Number of pressure dof's
!     user          i    Real user array to pass user information from
!                        main program to subroutine. See STANDARD PROBLEMS
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, isubcf, kchois, mrule, ishift
      logical funchk, allowed

!     allowed        If true it concerns an allowed possibility
!     funchk         Indicates whether there are coefficients that must be
!                    evaluated by funcc3 (true) or not (false)
!     i              Counting variable
!     ishift         Shift in coefficient sequence number
!     isubcf         Under bound for coefficients check
!     j              Counting variable
!     kchois         Indication what type of parameters must be computed
!                    Possible values:
!                    0:  Standard method, the calling subroutine is ELM200
!                        It is supposed that an element matrix must be computed
!                    1:  Subroutine ELD200 calls this subroutine, only
!                        derivatives must be computed
!                    2:  Subroutine ELI200 calls this subroutine, only
!                        integrals must be computed
!     mrule          Type of integration rule as given by the user
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inirepscal

!     EL0201         Fills sequence of displacement and pressure dof's
!     ELCOMM         Initializes the commons celp and celint with standard
!                    values
!     ELUSR3         Extracts information concerning the coefficients from IUSER
!                    and USER.
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIREPSCAL     Corrects integer value in case of scalars
!     INSTOP         Stop the program
!     PRINLG         Print and checks length of user arrays
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays IUSER and USER
!     Usually they are filled by FILCOF
!     The sequence is:
!
!    +-------+-------------------------------------------------------------+ &
!    | coef  | description                                                 |
!    +-------+-------------------------------------------------------------+ &
!    |   1   | information about the  time-integration                     |
!    |       +---------+----+----------------------------------------------+ &
!    |                 |    | not used                                     |
!    +-------+---------+----+----------------------------------------------+ &
!    |   2   | type of stress-strain relation                              |
!    |       +---------+----+----------------------------------------------+ &
!    |                 |  0 | full 3D                                      |
!    +-------+---------+----+----------------------------------------------+ &
!    |   3   | type of numerical integration                               |
!    |       +---------+----+----------------------------------------------+ &
!    |                 |  0 | default value, defined by element            |
!    |                 |  1 | Newton-Cotes                                 |
!    |                 |  3 | Gauss                                        |
!    +-------+---------+----+----------------------------------------------+ &
!    |   4   | constitutive law  coef = imatlaw                            |
!    |       +---------+----+----------------------------------------------+ &
!    |         imatlaw |  1 | compressible elastic solid                   |
!    |                 |  2 | incompressible elastic solid                 |
!    |                 |  4 | incompressible Mooney-Rivlin                 |
!    |                 | 10 | composite material                           |
!    |                 | 11 | composite material                           |
!    |                 | 99 | user-defined material                        |
!    +-------+---------+----+----------------------------------------------+ &
!    |   5   | user flags    coef = iusrvc + 100*iusrflg                   |
!    |       +---------+----+----------------------------------------------+ &
!    |         iusrvec |  0 | user vector is not filled                    |
!    |                 |  * | user is vector is filled and vector has      |
!    |                 |    | iusrvec degrees of freedom                   |
!    |         iusrflg |  * | user defined flags                           |
!    +-------+---------+---------------------------------------------------+ &
!    |   6   | density                                                     |
!    +-------+-------------------------------------------------------------+ &
!    |   7   | not used                                                    |
!    +-------+-------------------------------------------------------------+ &
!    |   8   | not used                                                    |
!    +-------+-------------------------------------------------------------+ &
!    |   9   | not used                                                    |
!    +-------+-------------------------------------------------------------+ &
!
!    +---------------------------------------------------------------------+ &
!    |    VECTORS NEEDED                                                   |
!    +---------------------------------------------------------------------+ &
!    | The following vectors are needed (in this order)                    |
!    +----+-----------+----------------------------------------------------+ &
!    |  1 |  u        |  incremental solution                              |
!    |  2 |  un       |  old solution at increment n                       |
!    |  3 |  usrvec   |  user defined vector  (if iusrvec >= 1)            |
!    +----+-----------+----------------------------------------------------+ &
!
!
!    +---------------------------------------------------------------------+ &
!    |    CONSTITUTIVE EQUATION PARAMETERS                                 |
!    +---------------------------------------------------------------------+ &
!
!    +---------------------------------------------------------------------+ &
!    |    compressible Neo-Hookean (COEF4 = 1)                             |
!    +-------+-------+-----------------------------------------------------+ &
!            |  10   | shear modulus G                                     |
!            |  11   | bulk modulus K                                      |
!    +-------+-------+-----------------------------------------------------+ &
!    |    incompressible Neo-Hookean (COEF4 = 2)                           |
!    +-------+-------+-----------------------------------------------------+ &
!            |  10   | shear modulus G                                     |
!    +-------+-------+-----------------------------------------------------+ &
!    |    incompressible Mooney-Rivlin (COEF4 = 4)                         |
!    +-------+-------+-----------------------------------------------------+ &
!            |  10   | material parameter c0                               |
!            |  11   | material parameter c1                               |
!    +-------+-------+-----------------------------------------------------+ &
!    |    composite (COEF4 = 10, 11)                                       |
!    +-------+-------+-----------------------------------------------------+ &
!            | 10-29 | see above for matrix material                       |
!            +-------+-----------------------------------------------------+ &
!            | 30-34 | composite and fiber type                            |
!            |       +---+----+--------------------------------------------+ &
!            |           | 30 | type of constitutive equation for matrix   |
!            |           | 31 | not used                                   |
!            |           | 32 | number of layers                           |
!            |           | 33 | not used                                   |
!            |           | 34 | theta (only for COEF4 = 11)                |
!            +-------+---+----+--------------------------------------------+ &
!            | 35-45 | fiber material parameters                           |
!    +-------+-------+-----------------------------------------------------+ &
!    |    user defined material (COEF4 = 99)                               |
!    +-------+-------+-----------------------------------------------------+ &
!            | 10-29 | user material parameters                            |
!            +-------+-----------------------------------------------------+ &
!
!
!    +---------------------------------------------------------------------+ &
!    |    2-PHASE MATERIAL MODEL                                           |
!    +---------------------------------------------------------------------+ &
!
!    +-------+-------------------------------------------------------------+ &
!    | 10-19 | see above for solid phase                                   |
!    +-------+-------------------------------------------------------------+ &
!    | 20-25 | permeability                                                |
!    |       +---+----+----------------------------------------------------+ &
!    |           | 20 | permeability coefficient K11                       |
!    |           | 21 | permeability coefficient K22                       |
!    |           | 22 | permeability coefficient K33                       |
!    |           | 23 | permeability coefficient K12                       |
!    |           | 24 | permeability coefficient K23                       |
!    |           | 25 | permeability coefficient K31                       |
!    +-------+---+----+----------------------------------------------------+ &
!    | 26-27 | time integration                                            |
!    |       +---+----+----------------------------------------------------+ &
!    |           | 26 | delta t                                            |
!    |           | 27 | theta (0=explicit, 1=implicit)                     |
!    +-------+---+----+----------------------------------------------------+ &
!    | 28    | misc                                                        |
!    |       +---+----+----------------------------------------------------+ &
!    |           | 28 | fluid volume fraction                              |
!    +-----------+----+----------------------------------------------------+ &
!
!
!    +---------------------------------------------------------------------+ &
!    |    4-PHASE MATERIAL MODEL                                           |
!    +---------------------------------------------------------------------+ &
!
!    +-------+-------------------------------------------------------------+ &
!    | 10-19 | see above for solid phase                                   |
!    +-------+-------------------------------------------------------------+ &
!    | 20-25 | permeability                                                |
!    |       +---+----+----------------------------------------------------+ &
!    |           | 20 | permeability coefficient K11                       |
!    |           | 21 | permeability coefficient K22                       |
!    |           | 22 | permeability coefficient K33                       |
!    |           | 23 | permeability coefficient K12                       |
!    |           | 24 | permeability coefficient K23                       |
!    |           | 25 | permeability coefficient K31                       |
!    +-------+---+----+----------------------------------------------------+ &
!    | 26-27 | time integration                                            |
!    |       +---+----+----------------------------------------------------+ &
!    |           | 26 | delta t                                            |
!    |           | 27 | theta (0=explicit, 1=implicit)                     |
!    +-------+---+----+----------------------------------------------------+ &
!    | 28-34 | ion properties                                              |
!    |       +---+----+----------------------------------------------------+ &
!    |           | 28 | D+  diffusion coefficient positive ions            |
!    |           | 29 | D-  diffusion coefficient negative ions            |
!    |           | 30 | r                                                  |
!    |           | 31 | V+  molar volume fraction pos. ions                |
!    |           | 32 | V-  molar volume fraction neg. ions                |
!    |           | 33 | z+  valent pos. ions                               |
!    |           | 34 | z-  valent neg. ions                               |
!    +-------+---+----+----------------------------------------------------+ &
!    | 35-45 | misc                                                        |
!    |       +---+----+----------------------------------------------------+ &
!    |           | 35 | cfc    fixed charged density                       |
!    |           | 36 | gamma  osmotic coefficient                         |
!    |           | 37 | phif   volume fraction fluid                       |
!    |           | 38 | cin+   internal concentration pos. ions            |
!    |           | 39 | cin-   internal concentration neg. ions            |
!    |           | 40 | cext+  external concentration pos. ions            |
!    |           | 41 | cext-  external concentration neg. ions            |
!    |           | 42 | T      temperature                                 |
!    |           | 43 | R      constant                                    |
!    |           | 44 | F      constant                                    |
!    |           | 45 | lambda Initial deformation caused by the swollen   |
!    |           |    |        state of the sample at the start of the     |
!    |           |    |        computation                                 |
!    +-----------+----+---------------------------------------------------+ &
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      42   Array IUSER(.) has value which is too small
!     262   ix has wrong value
!     264   jdegfd has wrong value
!     400   The value of stress-strain is incorrect
!     406   Value of icheld not allowed
!    1636   NUMOLD<2, whereas at least two old solutions are required
!    1766   Type of integration rule incorrect
!    1771   Shape number not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Fill some standard values in the common blocks celp and celint

      call eropen ('el0204')
      call elcomm
      ncomp  = ndim
      jcart = 1
      icoor = 2
      ishift = 0

      kchois = 0
      ncoef = 0

!     --- Compute kchois

      if (icheld.eq.0) then

!     --- Call by elm200, all coefficients required

         ncoef = 40
      else if (icheld.gt.0) then

!     --- Call by eld200, all coefficients required

         ncoef = 40
         allowed = (icheld.le.40)

         if (.not. allowed) then

!        --- icheld out of range

            call errint (icheld, 1)
            call errint (itype, 2)
            call errint (ielgrp, 3)
            call errsub (406, 3, 0, 0)
         end if

         if (icheld.eq.1 .or. icheld.eq.21) then
            if (ix.le.0 .or. ix.gt.ndim) then
               call errint (ix, 1)
               call errint (ndim, 2)
               call errint (itype, 3)
               call errint (ielgrp, 4)
               call errsub (262, 4, 0, 0)
            end if
            if (jdegfd.le.0 .or. jdegfd.gt.ndim) then
               call errint (jdegfd, 1)
               call errint (ndim, 2)
               call errsub (264, 2, 0, 0)
            end if
         end if
         ncomp  = ndim
         jderiv = 1
      else if (icheld.lt.0) then

!     --- Call by eli200, one coefficient required

         kchois = 2
         ncoef  = 1
         ncomp = icount/inpelm
         if (jdegfd.lt.1 .or. jdegfd.gt.ncomp) then
            call errint (jdegfd, 1)
            call errint (ncomp, 2)
            call errsub (264, 2, 0, 0)
         end if
      end if

!     --- Set some variables depending on the element type.
!         Number of displacement unknowns is constant. Number of
!         pressure unknowns depends on element type.

      ndisp = ndim*n
      goto (299, 299, 299, 299, 204, 204) itype-199

!     --- Taylor-Hood element 4-phase (ITYPE=204)

 204  if (ishape.eq.6) then
         npres = 4
         npsi = 4
      else if (ishape.eq.4.or.ishape.eq.7) then
         npres = 3
         npsi = 3
      else if (ishape.eq.14.or.ishape.eq.16) then
         npres = 8
         npsi = 8
      end if
      jdiag = 1
      jpsi = 1

 299  continue

!     --- Fill the arrays IND, MST and ARRAY with information of the
!     coefficient

      if (ncoef.gt.0 .or. icheld.gt.0) then

         ipos = iuser (ielgrp + 5)
         if (ipos.lt.7) then
            call errint (ielgrp+5, 1)
            call errint (ipos, 2)
            call errint (ielem, 3)
            call errint (itype, 4)
            call errint (ielgrp, 5)
            call errsub (42, 5, 0, 0)
         end if

         nuser = 0

         if (kchois.eq.0) then

!        --- Standard situation, cmpute the parameter notmas

            ipos = ipos + 1
            if (imas.eq.0) notmas = 1
            ishift = ishift+1
         else if (kchois.eq.1) then
            ipos = ipos + 1
            ishift = ishift+1
         end if

!        --- Fill flag igprob

         igprob = inirepscal(iuser(ipos))
         ipos = ipos + 1
         ishift = ishift+1

         if (igprob.lt.0 ) then
            call errint (igprob, 1)
            call errint (2, 2)
            call errint (ielem, 3)
            call errint (itype, 4)
            call errint (ielgrp, 5)
            call errsub (400, 5, 0, 0)
         end if

         if (kchois.ge.0 .and. kchois.le.2) then

!        --- Call by BUILD or INTEGR, compute the parameters irule and
!            material behaviour

            mrule = inirepscal(iuser(ipos))
            ipos = ipos + 1
            ishift = ishift+1
            if (mrule.lt.0 .or. mrule.gt.3) then
               call errint (mrule, 1)
               call errint (3, 2)
               call errint (ielem, 3)
               call errint (itype, 4)
               call errint (ielgrp, 5)
               call errsub (1766, 5, 0, 0)
            end if

!           --- Material behaviour (passed to el8300)

            imatlaw = mod(inirepscal(iuser(ipos)),100)
            ipos = ipos + 1
            ishift = ishift+1

!           --- Uservector and userflags

            iusrvc = mod(inirepscal(iuser(ipos)),100)
            iusrflg = inirepscal(iuser(ipos))/100

            ipos = ipos + 1
            ishift = ishift+1
         end if
      end if

!     --- Set integration rule

      if (mrule.gt.0) then

!     --- integration rule was given by the user

         irule = mrule
      else if (icheld.ne.0) then

!     --- called by DERIV or INTEGR, so default rule is Newton-Cotes

         irule = 1
      else

!     --- called by BUILD , so default is Gauss

         irule = 3
      end if

!     --- If icheld = 6 then default ix = 2

      if ((icheld.eq.6).and.(ix.eq.1)) then
         ix = 2
      end if

!     --- If icheld = 8 and and ix = 2 then rule for BUILD

      if (icheld.eq.8 .and. ix.eq.2) then
         if (mrule.gt.0) then
            irule = mrule
         else
            irule = 3
         end if
      end if

!     --- Set jdiag = 2 when Gauss integration

      if (irule.ne.1) jdiag = 2

!     --- Set parameters depending on the shape numbers

      goto (198, 198, 198, 104, 198, 106, 107, 198, 198, 198, &
            198, 198, 198, 114, 198, 116, 198, 198), ishape

!     --- Quadratic triangle

 104  jtrans = 0
      jelp = ishape
      nd = 4
      n = 6
      m = n
      if (irule.eq.2) m = 3
      call el0201 (isequ, iseqp)

      do i = 1, 3
         do j = 1, 3
            iseqmu(j+(i-1)*3) = iseqp(i) + j
         end do
         iseqksi(i) = iseqp(i) + 4
      end do
      goto 199

!     --- Biquadratic quadrilateral

 106  jtrans = 0
      jelp = ishape
      nd = 4
      n = 9
      m = n
      if (irule.eq.2) m = 4
      if (irule.eq.4) m = 5
      call el0201 (isequ, iseqp)

!     --- loop over vertices

      do i = 1, 4

!     --- loop over dof 4, 5 and 6

         do j = 1, 3
            iseqmu(j+(i-1)*3) = iseqp(i) + j
         end do

!        --- dof 7

         iseqksi(i) = iseqp(i) + 4
      end do
      goto 199

!     --- Extended quadratic triangle

 107  jtrans = 0
      jelp = ishape
      nd = 4
      n = 7
      m = n
      if (irule.eq.2) m = 3
      call el0201 (isequ, iseqp)
      do i = 1, 3
         do j = 1, 3
            iseqmu(j+(i-1)*3) = iseqp(i) + j
         end do
         iseqksi(i) = iseqp(i) + 4
      end do
      goto 199

!     --- Triquadratic brick

 114     jtrans = 0
         jelp = ishape
         nd = 9
         n = 27
         m = n
         if (irule.eq.2) m = 8

!        --- fill arrays isequ and iseqp depending on type number

         call el0201 (isequ, iseqp)

!        --- fill arrays iseqmu and iseqksi

         do i = 1, 8
            do j = 1, 3
               iseqmu(j+(i-1)*3) = iseqp(i) + j
            end do
            iseqksi(i) = iseqp(i) + 4
         end do
         goto 199

!        --- Ext. Quadratic Tetrahedron

 116     jtrans = 0
         jelp = ishape
         nd = 9
         n = 15
         m = n
         if (irule.eq.2) m = 11
         if (irule.eq.3) m = 15

!        --- fill arrays isequ and iseqp for ishape = 16

         call el0201 (isequ,iseqp)
         do i = 1, 4
            do j = 1, 3
               iseqmu(j+(i-1)*3) = iseqp(i) + j
            end do
            iseqksi(i) = iseqp(i) + 4
         end do
         goto 199

!        --- Shape not available

 198     call errint (ishape, 1)
         call errint (ielem, 2)
         call errint (itype, 3)
         call errint (ielgrp, 4)
         call errsub (1771, 4, 0, 0)

 199  if (ierror.ne.0) goto 1000

!     --- Fill some element constants

      mn = m * n
      iphi = 1
      igausp = 1
      if (inpelm.eq.27) then
         nvert  = 8
      else if (inpelm.eq.6 .or. inpelm.eq.7) then
         nvert = 3
      else if (ishape.eq.6) then
         nvert = 4
      else if (ishape.eq.16) then
         nvert = 4
      else
         nvert = n
      end if

!     --- Check for 2 old solutions when called by BUILD or DERIV

      if (numold.lt.2 .and. icheld.ge.0) then
         call errint (ielem, 1)
         call errint (itype, 2)
         call errint (ielgrp, 3)
         call errsub (1636, 3, 0, 0)
      end if

!     --- Clear array ARRAY

      do i = 1, ncoef*m
         array(i) = 0d0
      end do

!     --- And fill ARRAY with coefficients

      do i = 1, ncoef
         call elusr3 ( i, array(1+(i-1)*m), iuser, user, i+ishift )
      end do

!     --- Test coefficients

      if (ind(1).eq.0) then

!     --- rho has not been given a value, set rho equal to 1

         do i = 1, m
            array(i) = 1d0
         end do
      end if

!     --- Test array lengths, number of nodes and number of degrees of freedom

      call prinlg ( 'iuser', ipos-1, user, iuser )
      call prinlg ( 'user', -nuser, user, iuser )

!     --- Check which parameters depend on the space co-ordinates
!         compute isub and isup

      isubcf = 1
      do i = isubcf , ncoef
         isub = i

!        --- If ind(i) > 0 then parameter i corresponds to a function

         if (ind(i).gt.0) goto 170
      end do
      isub = ncoef + 1

170   isup = ncoef
      do i = ncoef, isub , -1
         isup = i
         if ( ind(i).gt.0 ) goto 190
      end do

!     --- Check for double parameters (functions) to avoid calls for
!         function subroutines
!         ist is used to store the references for the double parameters
!         Set ind(lengthmst)

190   do i = isub, isup
         ist(i) = 0
      end do

      funchk = .false.
      do i = isub, isup
         if (ind(i).gt.0 .and. ind(i).lt.2000) then

!        --- Only the function calls are checked

            do j = i+1,isup
               if (ind(j).eq.ind(i)) then
                  ist(j) = i
                  ind(j) = 0
               end if
            end do
            igausp = 1
         end if
         if (ind(i).gt.1000 .and. ind(i).le.2000) ind(lengthmst) = 1
         if (ind(i).gt.10000) funchk = .true.
         if (ind(i).gt.1000 .and. ind(i).le.2000 .or. ind(i).gt.10000) &
              igausp=1
      end do

!     --- Old solutions need to be computed

      ind(lengthmst) = 3

      if (funchk) ind(lengthmst) = ind(lengthmst) + 2

 1000 if (ierror.ne.0) call instop

      call erclos ( 'el0204' )

      end
