      subroutine el0900coeff ( ncoef, kchois, iuser, user, jtime, &
                               thetdt, jcheld, compress, penalp, &
                               ioldm, mcoefconv, mconv, mcont, &
                               modelrhsd, modelv, interpol, jtype, &
                               ishift, mrule )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.1  date 23-12-2014 Use elgetintcoef
!        version  5.0  date 27-02-2010 Remove array
!        version  4.0  date 21-12-2008 Remove parameter ipenalty
!
!   copyright (c) 2006-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Initialize coefficients for elements of type 900-902 and 916-918
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     navier_stokes_equation
!     initialization
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ncoef, kchois, iuser(*), jtime, jcheld, ioldm, mcoefconv, &
              mconv, mcont, modelrhsd, modelv, interpol, jtype, ishift, &
              mrule
      double precision user(*), thetdt, penalp
      logical compress

!     compress       i    If true the pressure must be computed
!     interpol       o    If 0 the standard interpolation is applied, &
!                         if 1 the interpolation is restricted to linear
!                         subelements
!     ioldm          o    Sequence number of velocity at end of previous time
!                         step in input vector.
!                         If 0 no iterative procedure is adopted for dynamical
!                         analyses.
!     ishift        i/o   Shift in coefficient sequence number
!     iuser          i    Integer user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
!     jcheld         i    Value of icheld corrected for different storage
!                         schemes
!     jtime          o    Integer parameter indicating if the mass matrix for
!                         the time-dependent equation must be computed (>0)
!                         or not (=0)
!                         Possible values:
!                           0:  no mass matrix
!                           1:  diagonal mass matrix with constant coefficient
!                           2:  diagonal mass matrix with variable coefficient
!                          11:  full mass matrix with constant coefficient
!                          12:  full mass matrix with variable coefficient
!                         101:  refers to the special possibility of the theta
!                               method directly applied
!                         102:  refers to the special possibility of the theta
!                               method directly applied with variable rho
!     jtype          i    Same as itype but 916-918 translated into 900-902
!     kchois         i    Indication what type of parameters must be computed
!                         Possible values:
!                         0:  Standard method, the calling subroutine is ELM900
!                             It is supposed that an element matrix must be
!                             computed
!                         1:  Subroutine ELD900 calls this subroutine, only
!                             derivatives must be computed
!                         2:  Subroutine ELI900 calls this subroutine, only
!                             integrals must be computed
!     mcoefconv      o    Defines how the convetive term is treated:
!                         0: standard case
!                         1: special case: the convective term is defined as
!                               c_conv_1 u du/dx + c_conv_2 v du/dy
!                               c_conv_3 u dv/dx + c_conv_4 v dv/dy
!     mcont          o    Defines the type of continuity equation
!                         Possible values:
!                         0: incompressible flow
!                         1: compressible flow: div (rho u ) = 0
!                         2: Goertler equations
!                         4: the continuity equation contains a pressure term
!                         5: special option for pressure mass matrix
!                            see elm903
!     mconv          o    Defines the treatment of the convective terms
!                         Possible values:
!                         0: convective terms are skipped (Stokes flow)
!                         1: convective terms are linearized by Picard iteration
!                         2: convective terms are linearized by Newton iteration
!                         3: convective terms are linearized by the incorrect
!                            Picard iteration
!                         4: convective terms are linearized by successive
!                            substitution
!     modelrhsd      o    Defines how the stress tensor is treated in the rhs
!                         Possible values:
!                         0:    if there is a given stress tensor the reference
!                               to this tensor is stored in position 19
!                         1:    It is supposed that there are 6 given stress
!                               tensor components stored in positions 19-24
!     modelv         o    Defines the type of viscosity model
!                         Possible values:
!                           1:  newtonian liquid
!                           2:  power law liquid
!                           3:  carreau liquid
!                           4:  plastico viscous liquid
!                           5:    special case: the viscous term is defined as
!                                 c_visc_1 d^2 u /dx^2 + c_visc_2 d^2 u /dy^2
!                                 c_visc_3 d^2 v /dx^2 + c_visc_4 d^2 v /dy^2
!                          10:  simplified newtonian liquid, &
!                               with constant viscosity
!                         100:  user provided model depending on grad u
!                         101:  user provided model depending on the second
!                               invariant
!                         102:  user provided model depending on the second
!                               invariant
!                               the co-ordinates and the velocity
!                         103:  user provided model depending on the second
!                               invariant
!                               the co-ordinates, the velocity, and old
!                               solutions
!     mrule         i/o   Type of integration rule as given by the user
!     ncoef          o    Number of coefficients in the equation
!     penalp         o    Parameter eps for the penalty function formulation
!     thetdt         o    1/(theta dt) for the case of the theta method
!                         immediately applied
!     user           i    Real user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode=4, nncode1=5
      integer :: ncode(nncode), ncode1(nncode1), ncodmx1(nncode1)
      character (len=16) :: code(nncode)
      character (len=14) :: code1(nncode1)
      double precision :: array(m)

!     array          Real work array in which the values of the
!                    coefficients in the integration points are stored
!     code           Table with keywords corresponding to string linearization
!     code1          Table with keywords corresponding to string continuity_eq
!     ncode          Table containing the number of significant characters
!                    in the keyword table code
!     ncode1         Table containing the number of significant characters
!                    in the keyword table code1
!     ncodmx1        Maximum number of character of the keywords in code1
!     nncode         Number of entries in code
!     nncode1        Number of entries in code1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: elgetintcoef, sepfilldefstr

!     EL0900COEFF1   Process first 5 parameters of array iuser in case of
!                    elements of type 900
!     ELGETINTCOEF   Give elgetintcoef the actual integer value corresponding to
!                    iuser(ipos)
!     ELUSR3         Extracts information concerning the coefficients from IUSER
!                    and USER.
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPFILLDEFSTR  Give sepfilldefstr either the default value defval or the
!                    value of the variable namestring if it exists
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     838   Penalty parameter is not constant
!    1305   Coefficient has wrong value
!    1915   Penalty parameter not positive
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code / 'NONE', 'PICARD', 'NEWTON', 'INCORRECT_PICARD' /
      data ncode / 4, 6, 6,16 /
      data code1 / 'INCOMPRESSIBLE', 'COMPRESSIBLE', 'GOERTLER', &
                   'PRESSURE', 'MASS_MATRIX' /
      data ncode1  / 5, 4, 4, 4, 4 /
      data ncodmx1 /14,12, 8, 8,11 /

! ======================================================================
!
      call eropen ( 'el0900coeff' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el0900coeff'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call el0900coeff1 ( ncoef, kchois, iuser, jtime, &
                          thetdt, jcheld, compress, ioldm, &
                          modelrhsd, modelv, interpol, &
                          ishift, mrule )

      if ( kchois==0 ) then

!     --- call by BUILD

         mconv = elgetintcoef ( iuser, ipos, ishift, code, nncode, &
                                ncode, 1, 'linearization' )
         metupw = mconv/1000
         mconv = mconv-1000*metupw
         mcoefconv = mconv/100
         mconv = mconv-100*mcoefconv
         mcont = mconv/10
         mconv = mconv-10*mcont

         if ( mcont==0 ) then

!        --- mcont = 0, check if there is string variable with value
!            continuity_eq

            mcont = sepfilldefstr ( 'CONTINUITY_EQ', 0, code1, nncode1, &
                                    ncode1, ncodmx1 )
            if ( mcont>0 ) mcont = mcont-1

         end if  ! ( mcont==0 )

         if ( debug ) then
            write(irefwr,1) 'metupw, mcoefconv, mcont, mconv', &
                             metupw, mcoefconv, mcont, mconv
         end if  ! ( debug )

         if ( mconv<0 .or. mconv>3 .or. mcont>5 .or. &
              mcoefconv>1 .or. metupw>3 ) then

!        --- mconv<0 or mconv>3 or mcont>5 or mcoefconv>1 or metupw>3
!            Such values are not allowed

            call errint ( 5, 1 )
            call errint ( itype, 2 )
            call errint ( ielgrp, 3 )
            call errint ( ielem, 4 )
            call errint ( mconv+10*mcont+100*mcoefconv+ &
                          1000*metupw, 5 )
            call errchr ( 'mconv', 1 )
            call errsub ( 1305, 5, 0, 1 )

         end if
         if ( itype==903 ) then

!        --- Type 903, parameter 6 (penalty) not used

            ipos = ipos + 1
            ishift = ishift+1

         else

!        --- penalty parameter

            call elusr3 ( 1, array, iuser, user, ishift )

            if ( ind(1)>0 ) then
               call errint ( itype, 1 )
               call errint ( 6, 2 )
               call errint ( ielgrp, 3 )
               call errint ( ielem, 4 )
               call errsub ( 838, 4, 0, 0 )
            end if
            penalp = array(1)
            if ( jtype==900 .and. penalp<=0d0 .and. compress ) then
               call erreal ( penalp, 1 )
               call errint ( itype, 1 )
               call errint ( 6, 2 )
               call errint ( ielgrp, 3 )
               call errint ( ielem, 4 )
               call errsub ( 1915, 4, 1, 0 )
            end if

         end if  ! ( itype==903 )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End el0900coeff'

      end if  ! ( debug )
      call erclos ( 'el0900coeff' )

      end subroutine el0900coeff
