      subroutine el0900coeff1 ( ncoef, kchois, iuser, jtime, &
                                thetdt, jcheld, compress, ioldm, &
                                modelrhsd, modelv, interpol, &
                                ishift, mrule )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.2  date 13-01-2015 Extension with models
!        version  1.1  date 23-12-2014 Use elgetintcoef
!        version  1.0  date 20-02-2010
!
!   copyright (c) 2010-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Process first 5 parameters of array iuser in case of elements
!     of type 900
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     navier_stokes_equation
!     initialization
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletime
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ncoef, kchois, iuser(*), jtime, jcheld, ioldm, &
              modelrhsd, modelv, interpol, ishift, mrule
      double precision thetdt
      logical compress

!     compress       i    If true the pressure must be computed
!     interpol       o    If 0 the standard interpolation is applied, &
!                         if 1 the interpolation is restricted to linear
!                         subelements
!     ioldm          o    Sequence number of velocity at end of previous time
!                         step in input vector.
!                         If 0 no iterative procedure is adopted for dynamical
!                         analyses.
!     ishift        i/o   Shift in coefficient sequence number
!     iuser          i    Integer user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
!     jcheld         i    Value of icheld corrected for different storage
!                         schemes
!     jtime          o    Integer parameter indicating if the mass matrix for
!                         the time-dependent equation must be computed (>0)
!                         or not (=0)
!                         Possible values:
!                           0:  no mass matrix
!                           1:  diagonal mass matrix with constant coefficient
!                           2:  diagonal mass matrix with variable coefficient
!                          11:  full mass matrix with constant coefficient
!                          12:  full mass matrix with variable coefficient
!                         101:  refers to the special possibility of the theta
!                               method directly applied
!                         102:  refers to the special possibility of the theta
!                               method directly applied with variable rho
!     kchois         i    Indication what type of parameters must be computed
!                         Possible values:
!                         0:  Standard method, the calling subroutine is ELM900
!                             It is supposed that an element matrix must be
!                             computed
!                         1:  Subroutine ELD900 calls this subroutine, only
!                             derivatives must be computed
!                         2:  Subroutine ELI900 calls this subroutine, only
!                             integrals must be computed
!     modelrhsd      o    Defines how the stress tensor is treated in the rhs
!                         Possible values:
!                         0:    if there is a given stress tensor the reference
!                               to this tensor is stored in position 19
!                         1:    It is supposed that there are 6 given stress
!                               tensor components stored in positions 19-24
!     modelv         o    Defines the type of viscosity model
!                         Possible values:
!                           1:  newtonian liquid
!                           2:  power law liquid
!                           3:  carreau liquid
!                           4:  plastico viscous liquid
!                           5:    special case: the viscous term is defined as
!                                 c_visc_1 d^2 u /dx^2 + c_visc_2 d^2 u /dy^2
!                                 c_visc_3 d^2 v /dx^2 + c_visc_4 d^2 v /dy^2
!                          10:  simplified newtonian liquid, &
!                               with constant viscosity
!                         100:  user provided model depending on grad u
!                         101:  user provided model depending on the second
!                               invariant
!                         102:  user provided model depending on the second
!                               invariant
!                               the co-ordinates and the velocity
!                         103:  user provided model depending on the second
!                               invariant
!                               the co-ordinates, the velocity, and old
!                               solutions
!     mrule         i/o   Type of integration rule as given by the user
!     ncoef          o    Number of coefficients in the equation
!     thetdt         o    1/(theta dt) for the case of the theta method
!                         immediately applied
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode=6, nncode1=3
      integer :: ncode(nncode), ncode1(nncode1)
      character (len=16) :: code (nncode)
      character (len=16) :: code1 (nncode1)

!     code           Table with keywords corresponding to string model
!     code1          Table with keywords corresponding to string
!                    coordinate_system
!     ncode          Table containing the number of significant characters
!                    in the keyword table code
!     ncode1         Table containing the number of significant characters
!                    in the keyword table code1
!     nncode         Number of entries in code
!     nncode1        Number of entries in code1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inirepscal, elgetintcoef

!     ELGETINTCOEF   Give elgetintcoef the actual integer value corresponding to
!                    iuser(ipos)
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIREPSCAL     Corrects integer value in case of scalars
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      42   Array IUSER(.) has value which is too small
!     988   itime = 1 in combination with tstep<=0 not allowed
!    1305   Coefficient has wrong value
!    1541   The combination imas = 1 and ishape=4 is not permitted
!    1766   Type of integration rule incorrect
!    1767   Type of co-ordinates incorrect
!    2438   jtime>0 and imas>0 ( double use of mass matrix)
!    2779   linear elements and time dependent not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code / 'NEWTONIAN', 'POWER_LAW', 'CARREAU', &
                  'PLASTICO_VISCOUS', 'SPECIAL', 'DIAGONAL' /
      data ncode / 9, 9, 7,16, 7,   8 /
      data code1 / 'CARTESIAN', 'AXI_SYMMETRIC', 'POLAR' /
      data ncode1 / 9,13, 5 /
! ======================================================================
!
      call eropen ( 'el0900coeff1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el0900coeff1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      interpol = 0

      if ( ncoef>0 ) then

         ipos = iuser ( ielgrp + 5 )
         if ( ipos<7 ) then
            call errint ( ielgrp+5, 1 )
            call errint ( ipos, 2 )
            call errint ( ielem, 3 )
            call errint ( itype, 4 )
            call errint ( ielgrp, 5 )
            call errsub ( 42, 5, 0, 0 )
            go to 1000
         end if

         nuser = 0

         if ( kchois==0 ) then

!        --- Standard situation, Compute the parameters jtime, irule, jcart,
!                                                       mconv, modelv, penalp

            jtime = inirepscal ( iuser(ipos) )

!           --- jtime consists of 2 parts: jtime_old + 10 * ioldm

            ioldm = jtime/10
            jtime = mod(jtime,10)

            ipos = ipos + 1
            ishift = ishift+1
            if ( imas==0 ) notmas = 1
            if ( jtime==0 ) then

               jtime = 1+(imas-1)*10

            else

               if ( itype==903 ) jtime = 101
               if ( tstep<=0d0 ) then

!              --- jtime>0 and tstep=0

                  call erreal ( tstep, 1 )
                  call errsub ( 988, 0, 1, 0 )

               else

                  thetdt = 1d0 / (theta * tstep)

               end if
               if ( imas/=0 ) then

!              --- imas>0, hence a mass matrix must be built
!                  However, also the mass matrix is used in the stiffness
!                  matrix. This means a double use of mass matrix and hence
!                  an error message is given.

                  call errint ( jtime , 1 )
                  call errint ( imas, 2 )
                  call errsub ( 2438, 2, 0, 0 )

               end if
               jtime = 101

            end if
            if ( imas==1 .and. (ishape==4 .or. ishape==12) ) then

!           --- error ishape=4 or 12 may not be combined with a diagonal mass
!               matrix

               call errint ( ishape, 1 )
               call errint ( irule, 2 )
               call errint ( imas, 3 )
               call errsub ( 1541, 3, 0, 0 )

            end if

            if ( itype==903 .and. jtime>0 .and. &
                (ishape==3 .or. ishape==11) ) then

!           --- error linear elements and time dependent not yet implemented
!               for type 903

               call errint ( ishape, 1 )
               call errint ( itype, 2 )
               call errsub ( 2779, 2, 0, 0 )

            end if  ! ( jtime>0 .and. (ishape==3 .or. ishape==11) )

            modelv = elgetintcoef ( iuser, ipos, ishift, code, nncode, &
                                    ncode, 0, 'model' )

            if ( modelv==6 ) modelv = 10
            modelrhsd = modelv/1000
            modelv = mod(modelv,1000)
            if ( modelv<0 .or. modelv>5 .and. modelv<100 .and. &
                 modelv/=10 .or. modelv>103 ) then
               call errint ( 2, 1 )
               call errint ( itype, 2 )
               call errint ( ielgrp, 3 )
               call errint ( ielem, 4 )
               call errint ( modelv, 5 )
               call errchr ( 'modelv', 1 )
               call errsub ( 1305, 5, 0, 1 )
            end if
            modelv = max ( modelv, 1 )
            if ( modelrhsd<0 .or. modelrhsd>1 ) then
               call errint ( 2, 1 )
               call errint ( itype, 2 )
               call errint ( ielgrp, 3 )
               call errint ( ielem, 4 )
               call errint ( modelrhsd, 5 )
               call errchr ( 'modelrhsd', 1 )
               call errsub ( 1305, 5, 0, 1 )
            end if
            if ( icheld>0 .and. jcheld/=6 .and. &
                 jcheld/=11 ) then
               if ( ndim==3 .or. .not. compress ) modelv = 0
            end if

         else if ( kchois==1 ) then
            ipos = ipos + 2
            ishift = ishift+2
         end if
         if ( kchois>=0 .and. kchois<=2 ) then

!        --- Call by BUILD or INTEGR, Compute the parameters jcart and irule

            mrule = inirepscal ( iuser(ipos) )
            interpol = mrule/100
            mrule = mod(mrule,100)
            ipos = ipos + 1
            ishift = ishift+1
            if ( mrule<0 .or. mrule>5 ) then
               call errint ( mrule, 1 )
               call errint ( 3, 2 )
               call errint ( ielem, 3 )
               call errint ( itype, 4 )
               call errint ( ielgrp, 5 )
               call errsub ( 1766, 5, 0, 0 )
            end if

            jcart = elgetintcoef ( iuser, ipos, ishift, code1, nncode1, &
                                   ncode1, 1, 'coordinate_system' )
            if ( jcart<0 .or. jcart>2 ) then
               call errint ( jcart, 1 )
               if ( kchois==0 ) then
                  call errint ( 4, 2 )
               else
                  call errint ( 2, 2 )
               end if
               call errint ( ielem, 3 )
               call errint ( itype, 4 )
               call errint ( ielgrp, 5 )
               call errsub ( 1767, 5, 0, 0 )
            end if
            jcart = jcart + 1
            if ( kchois==2 ) then
               ipos = ipos + 1
               ishift = ishift+1
            end if

         end if

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End el0900coeff1'

      end if  ! ( debug )
      call erclos ( 'el0900coeff1' )

      end
