      subroutine el0900gen ( iuser, user, jelp, jtime, penalp, cn, &
                             clamb, mconv, mcont, modelv, isub, isup, &
                             coeffintpnts, thetdt, nveloc, npres, imesh, &
                             mdiv, mcoefconv, modelrhsd, ioldm, coeffnodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.0  date 06-01-2016 Use sepmoduleelem
!        version  4.0  date 27-02-2010 Extra parameters
!        version  3.0  date 21-12-2008 Remove parameter ipenalty
!
!   copyright (c) 2006-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Performs the initializations for subroutine ELM900, ELI900 or
!     ELD900 in the case that IFIRST = 0
!     The parameters in the common blocks celint, celiar and celp
!     get a start value
!     Array array is initialized with the values of the coefficients
!     If this subroutine is changed, also adapt el0260
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     navier_stokes_equation
!     initialization
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision user(*), coeffintpnts(m,*), penalp, cn, clamb, &
                       thetdt, coeffnodes(n,*)
      integer iuser(*), jelp, jtime, isub, isup, mconv, modelv, &
              mcont, nveloc, npres, imesh, mdiv, &
              mcoefconv, modelrhsd, ioldm

!     clamb          o    Parameter lambda with respect to viscosity model
!     cn             o    Power in power law model
!     coeffintpnts   o    Real work array in which the values of the
!                         coefficients in the integration points are stored
!                         See el0900
!     coeffnodes     o    Real array in which the values of the
!                         coefficients in the nodal points are stored
!     imesh          o    Sequence number of mesh velocity in input vector
!                         If 0 no mesh velocity is present
!     ioldm          o    Sequence number of velocity at end of previous time
!                         step in input vector.
!                         If 0 no iterative procedure is adopted for dynamical
!                         analyses.
!     isub           o    Integer help parameter for do-loops to indicate the
!                         smallest coefficient number that is dependent on space
!     isup           o    Integer help parameter for do-loops to indicate the
!                         largest coefficient number that is dependent on space
!     iuser          i    Integer user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
!     jelp           o    Indication of the type of basis function subroutine
!                         must be called. See elm800basefn
!     jtime          o    Integer parameter indicating if the mass matrix for
!                         the time-dependent equation must be computed (>0)
!                         or not (=0)
!                         See el0900
!     mcoefconv      o    Defines how the convetive term is treated:
!                         0: standard case
!                         1: special case: the convective term is defined as
!                               c_conv_1 u du/dx + c_conv_2 v du/dy
!                               c_conv_3 u dv/dx + c_conv_4 v dv/dy
!     mcont          o    Defines the type of continuity equation
!                         Possible values:
!                         0: incompressible flow
!                         1: compressible flow: div (rho u ) = 0
!                         2: Goertler equations
!                         4: the continuity equation contains a pressure term
!     mconv          o    Defines the treatment of the convective terms
!                         Possible values:
!                         0: convective terms are skipped (Stokes flow)
!                         1: convective terms are linearized by Picard iteration
!                         2: convective terms are linearized by Newton iteration
!                         3: convective terms are linearized by the incorrect
!                            Picard iteration
!                         4: convective terms are linearized by successive
!                            substitution
!     mdiv           o    Defines if divergence right-hand side is used (1)
!                         or not (0)
!     modelrhsd      o    Defines how the stress tensor is treated in the rhs
!                         Possible values:
!                         0:    if there is a given stress tensor the reference
!                               to this tensor is stored in position 19
!                         1:    It is supposed that there are 6 given stress
!                               tensor components stored in positions 19-24
!     modelv         o    Defines the type of viscosity model
!                         See el0900
!     npres          o    Number of pressure parameters
!     nveloc         o    Number of velocity parameters
!     penalp         o    Parameter eps for the penalty function formulation
!     thetdt         o    1/(theta dt) for the case of the theta method
!                         immediately applied
!     user           i    Real user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL0410         Initialize parameters for subroutines ELD900/ELI900/ELM900
!                    with respect to type 410
!     EL0900         Initialize parameters for subroutines ELD900/ELI900/ELM900
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el0900gen' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el0900gen'
100      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      if ( itype<500 ) then

!     --- Type numbers 400, 402, 404, 406, 410

         call el0410 ( iuser, user, jelp, jtime, penalp, cn, &
                       clamb, mconv, modelv, isub, isup, coeffintpnts, &
                       thetdt )
         mcont = 0
         imesh = 0
         mdiv = 0
         mcoefconv = 0
         modelrhsd = 0
         ioldm = 0
         npres = 1
         npsi = ndim+1

      else

!     --- Type numbers 900 - 902, 925-927

         call el0900 ( iuser, user, jelp, jtime, penalp, cn, clamb, &
                       mconv, mcont, modelv, isub, isup, coeffintpnts, &
                       thetdt, nveloc, npres, imesh, mdiv, &
                       mcoefconv, modelrhsd, ioldm, coeffnodes )

      end if

      if ( debug ) then

         write(irefwr,100) 'mconv, mcont, modelv, isub, isup', &
                            mconv, mcont, modelv, isub, isup
         write(irefwr,100) 'nveloc, npres, imesh', &
                            nveloc, npres, imesh
         write(irefwr,100) 'mdiv, mcoefconv, modelrhsd, ioldm', &
                            mdiv, mcoefconv, modelrhsd, ioldm

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End el0900gen'

      end if
      call erclos ( 'el0900gen' )

      end
