      subroutine el0910 ( iuser, user, jelp, isub, isup, array, iload, induv )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-01-2016 Use sepmoduleelem
!        version  2.12 date 29-09-2004 Extension with slip in 2d
!        version  2.11 date 31-03-2003 Extension for axi-symmetric surface
!                                      tension
!
!   copyright (c) 1994-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     This subroutine performs the initializations for subroutine ELM910
!     in the case that IFIRST = 0
!     The parameters in the common blocks celint, celiar and celp
!     get a start value
!     Array array is initialized with the values of the coefficients
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     element_matrix
!     element_vector
!     initialization
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      use sepmoduleoldvecs
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision user(*), array(*)
      integer iuser(*), jelp, isub, isup, iload, induv(ndim,*)

!     array          o    Real work array in which the values of the
!                         coefficients in the integration points are stored
!                         in the sequence:
!                                2D             3D     load = 2
!                         1:     Tx             Tx       gamma
!                         2:     Ty             Ty       t1
!                         3:     -              Tz       t2
!                         4:     cn             cx
!                         5:     ct             cy
!                         6:     -              cz
!                         Each coefficient requires exact m (is number of
!                         integration points) positions
!     iload          o    Indicates the type of boundary condition.
!                         Possible values:
!                         0: Prescribed load in co-ordinate directions
!                         1: Prescribed load given in normal and tangential
!                            direction
!                         2: surface tension
!     induv          o    Index array of size ncomp x inpelm that contains
!                         the positions of the degrees of freedom 1 to ncomp
!                         in the element
!                         Hence induv(k,i) contains the position of the k^th
!                         degree of freedom in the i^th node of the element
!                         with respect to the element vector
!     isub           o    Integer help parameter for do-loops to indicate the
!                         smallest coefficient number that is dependent on space
!     isup           o    Integer help parameter for do-loops to indicate the
!                         largest coefficient number that is dependent on space
!     iuser          i    Integer user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
!     jelp           o    Indication of the type of basis function subroutine
!                         must be called by ELxxxx. Possible vaues:
!                          1:  Subroutine ELP610  (Linear line element)
!                          2:  Subroutine ELP611  (Quadratic line element)
!                          3:  Subroutine ELP620  (Linear triangle)
!                          4:  Subroutine ELP624  (Extended quadratic triangle)
!                          5:  Subroutine ELP621  (Bilinear quadrilateral)
!                          6:  Subroutine ELP623  (Biquadratic quadrilateral)
!                         11:  Subroutine ELP630  (Linear tetrahedron)
!                         12:  Subroutine ELP631  (Quadratic tetrahedron)
!                         13:  Subroutine ELP632  (Trilinear brick)
!                         14:  Subroutine ELP633  (Triquadratic brick)
!     user           i    Real user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ncoef, i, j, isubcf, mrule, isum, inunkp, ishift
      logical funchk

!     funchk         Indicates whether there are coefficients that must be
!                    evaluated by funcc3 (true) or not (false)
!     i              Counting variable
!     inunkp         Number of degrees of freedom in point
!     ishift         Shift in coefficient sequence number
!     isubcf         Under bound for coefficients check
!     isum           Integer variable to compute a sum
!     j              Counting variable
!     mrule          Type of integration rule as given by the user
!     ncoef          Number of coefficients in the equation
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inirepscal

!     ELCOMM         Initializes the commons celp and celint with standard
!                    values
!     ELUSR3         Extracts information concerning the coefficients from IUSER
!                    and USER.
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     INIREPSCAL     Replace ivalue by scalar if necessary
!     INSTOP         Stop the program
!     PRINLG         Print and checks length of user arrays
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays IUSER and USER
!     Usually they are filled by FILCOF
!
!     The sequence is:
!
!     In the case of a call by BUILD:
!
!     1:  Type of boundary condition
!         Possible values:
!         0: Prescribed load (in co-ordinate directions)
!         1: Prescribed load (in normal and tangential direction) only in 2D
!         2: Surface tension only in 2D
!         3: Boundary condition of the type c_t v_t + sigma_t = u . t
!            c_t must be given in position 9 and the Cartesian vector u in
!            positions 6 to 8
!            Only in R^3 and in combination with local transforms
!            The first component of the local transformation must correspond to
!            the normal vector and the other 2 to the tangential vectors
!            The tangential vectors are extracted from the local transform
!     2:
!     3:  Type of numerical integration
!         Possible values:
!         0: default value, defined by element
!     4:  Type of co-ordinate system
!         Possible values:
!     5:  Not yet in use:  set equal to 0
!     6:  Tx (1-component in local co-ordinate system)
!         or if iload=2: gamma is coefficient of surface tension
!         or if iload=3: x component of u vector
!     7:  Ty (2-component in local co-ordinate system)
!         or if iload=2 and point element: t1 (first component tangential
!                                              vector)
!         or if iload=3: y component of u vector
!     8:  Tz (z component of load or 3-component in local co-ordinate system)
!         or if iload=2 and point element: t2 (second component tangential
!                                              vector)
!         or if iload=3: z component of u vector
!     9:  cx (1-component in local co-ordinate system of coefficient before ux)
!         or if iload=3: c_t
!    10:  cy (2-component in local co-ordinate system of coefficient before uy)
!    11:  cz (z component of load or 3-component in local co-ordinate system
!             of coefficient before uz)
!    12-15: Not yet in use (must be zero)
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      42   Array IUSER(.) has value which is too small
!      74   Incorrect number of degrees of freedom for this type
!    1304   For coefficient only positive constants are allowed
!    1305   Coefficient has wrong value
!    1766   Type of integration rule incorrect
!    1767   Type of co-ordinates incorrect
!    1768   Coefficient is filled that is not used
!    1771   Shape number not yet implemented
!    1786   Shape number of element in combination with dimension not available
!    2065   Degree of freedom not in element
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Fill some standard values in the common blocks celp and celint

      call eropen ( 'el0910' )
      call elcomm
      jtrans = 0
      ideriv = 0
      jderiv = 0
      ncomp  = ndim
      jind = 0
      ishift = 0

!     --- Set parameters depending on the shape numbers

      if ( itype.eq.914 ) then
         n = inpelm/2
      else
         n = inpelm
      end if
      m = n
      mn = m * n
      if ( itype.eq.914 ) then
         jelp = -ishape-10001
      else
         jelp = ishape
      end if
      if ( itype.eq.914 .and. icount.lt.n*ndim*2 .or. &
           itype.ne.914 .and. icount.lt.n*ndim ) then

!     --- Number of unknowns incorrect

         call errint ( icount, 1 )
         call errint ( n*ndim, 2 )
         call errint ( ielem, 3 )
         call errint ( itype, 4 )
         call errint ( ielgrp, 5 )
         call errsub ( 74, 5, 0, 0 )

      end if
      isum = 0
      iseqtype = oldsols(iseqin)%iseqtype
      index4n => indoldsols(iseqtype)%index4
      do i = 1, inpelm
         inunkp = index4n(i)-isum
         if ( inunkp.lt.ncomp ) then
            call errint ( inunkp, 1 )
            call errint ( i, 2 )
            call errint ( ncomp, 3 )
            call errint ( ielem, 4 )
            call errint ( itype, 5 )
            call errint ( ielgrp, 6 )
            call errsub ( 2065, 6, 0, 0 )
         end if
         do j = 1, ncomp
            induv(j,i) = isum+j
         end do
         isum = isum+inunkp
      end do
      if ( inpelm.eq.1 ) then

!     --- Point element

         jelp = 0
         jdercn = 0

      else if ( jelp.eq.1 .or. jelp.eq.3 ) then

!     --- linear element

         mn = m
         jdercn = 0

      else if ( jelp.eq.2 .or. jelp.ge.4 .and. jelp.le.6 ) then

!     --- Bilinear or quadratic element

         jdercn = 1

      else

!     --- Error 1771: this type of element has not yet been implemented

         call errint ( ishape, 1 )
         call errint ( ielem, 2 )
         call errint ( itype, 3 )
         call errint ( ielgrp, 4 )
         call errsub ( 1771, 4, 0, 0 )

      end if

!     --- Fill the arrays IND, MST and ARRAY with information of the
!     coefficient

      ipos = iuser ( ielgrp + 5 )
      if ( ipos.lt.7 ) then
         call errint ( ielgrp+5, 1 )
         call errint ( ipos, 2 )
         call errint ( ielem, 3 )
         call errint ( itype, 4 )
         call errint ( ielgrp, 5 )
         call errsub ( 42, 5, 0, 0 )
      end if

      nuser = 0

!     --- Compute the parameters iload, jcart and irule

      iload = inirepscal ( iuser(ipos) )
      if ( iload.lt.0 .or. (ndim.eq.2 .and.iload.gt.3) .or. &
           ndim.eq.3 .and. ( iload.ge.1 .and. iload.ne.3 ) ) then
         call errint ( 1, 1 )
         call errint ( itype, 2 )
         call errint ( ielgrp, 3 )
         call errint ( ielem, 4 )
         call errint ( iload, 5 )
         call errchr ( 'iload', 1 )
         call errsub ( 1305, 5, 0, 1 )
         iload = 0
      end if
      if ( iload.eq.1 .or. iload.eq.2 ) icoor = 11
      if ( iload.eq.2 ) then

!     --- Surface tension, we need the derivatives, the gauss points and
!         the basis functions

         ideriv = 1
         iphi = 1
         igausp = 1

      end if
      ipos = ipos + 2
      ishift = ishift+2
      if ( (jelp.lt.1 .and. ndim.gt.2) .or. &
           (jelp.lt.1 .and. ndim.gt.1 .and. iload.ne.2) .or. &
           (jelp.lt.3 .and. ndim.gt.2) ) then
         call errint ( jelp, 1 )
         call errint ( ndim, 2 )
         call errint ( itype, 3 )
         call errint ( ielgrp, 4 )
         call errsub ( 1786, 4, 0, 0 )
      end if

      mrule = inirepscal ( iuser(ipos) )
      if ( jelp.le.0 ) then

!     --- Point element

         irule = 1

      else if ( mrule.eq.0 ) then

!     --- No integration rule given, compute

         if ( iload.eq.2 ) then
            irule = 4
         else if ( ishape.lt.3 ) then
            irule = 1
         else
            irule = 3
         end if

      else

!     --- Integration rule given

         irule = max ( 1, mrule )

      end if

      ipos = ipos + 1
      ishift = ishift+1
      if ( mrule.lt.0 .or. mrule.gt.40 ) then
         call errint ( mrule, 1 )
         call errint ( 3, 2 )
         call errint ( ielem, 3 )
         call errint ( itype, 4 )
         call errint ( ielgrp, 5 )
         call errsub ( 1766, 5, 0, 0 )
      end if

      jcart = inirepscal ( iuser(ipos) )
      if ( jcart.lt.0 .or. jcart.gt.1 ) then
         call errint ( jcart, 1 )
         call errint ( 4, 2 )
         call errint ( ielem, 3 )
         call errint ( itype, 4 )
         call errint ( ielgrp, 5 )
         call errsub ( 1767, 5, 0, 0 )
      end if
      jcart = jcart + 1
      ipos = ipos + 2
      ishift = ishift+2
      if ( irule.ne.1 ) then

!     --- irule > 1, Gauss rule

         igausp = 1
         iphi = 1
         jdiag = 2
         if ( jelp.le.2 ) then
            m = irule - 1
         else if ( jelp.eq.3 .or. jelp.eq.5 ) then
            if ( irule.eq.2 ) then
               m = 1
            else
               m = 4
            end if
         else if ( jelp.eq.4 ) then
            if ( irule.eq.2 ) then
               m = 4
            else
               m = 7
            end if
         else if ( jelp.eq.6 ) then
            if ( irule.eq.2 ) then
               m = 4
            else
               m = 9
            end if
         end if
         mn = m * n

      end if

      ncoef = 6
      do i = 1 , ncoef
         call elusr3 ( i, array(1+(i-1)*m), iuser, user, i+ishift )
      end do

!     --- Test coefficients

      if ( ndim.eq.2 .and. iload.ne.2 ) then

!     --- ndim = 2, the coefficients 3 and 6 are not used

         do i = 3, 6, 3

            if ( ind(i).ne.0 ) then
               call errint ( i+5, 1 )
               call errint ( ielem, 2 )
               call errint ( itype, 3 )
               call errint ( ielgrp, 4 )
               call errwar ( 1768, 4, 0, 0 )
               ind(i) = 0
            end if
         end do

      end if
      do i = 4, 6

         if ( ind(i).lt.0 .and. array(1+(i-1)*m).lt.0d0 ) then
            call errint ( i+5, 1 )
            call errint ( itype, 2 )
            call errint ( ielem, 3 )
            call errint ( ielgrp, 4 )
            call erreal ( array(1+(i-1)*m), 1 )
            call errsub ( 1304, 4, 1, 1 )
            call errchr ( 'c', 1 )
            ind(i) = 0
         end if
      end do

!     --- Test array lengths, number of nodes and number of degrees of freedom

      call prinlg ( 'iuser', ipos-1, user, iuser )
      call prinlg ( 'user', -nuser, user, iuser )

!     --- Check which parameters depend on the space co-ordinates
!         Compute isub and isup

      isubcf = 1

      do i = isubcf , ncoef
         isub = i

!        --- If ind(i) > 0 then paramater i corresponds to a function

         if ( ind(i).gt.0 ) goto 170

      end do

      isub = ncoef + 1

170   isup = ncoef
      do i = ncoef, isub , -1
         isup = i
         if ( ind(i).gt.0 ) goto 190
      end do

!     --- Check for double parameters (functions) to avoid calls for
!         function subroutines
!         ist is used to store the references for the double parameters
!         Set ind(lengthmst)

190   do i = isub, isup
         ist(i) = 0
      end do
      ind(lengthmst) = 0
      funchk = .false.
      do i = isub, isup
         if ( ind(i).gt.0 .and. ind(i).lt.2000 ) then

!        --- only the function calls are checked

            do j = i+1,isup
               if ( ind(j).eq.ind(i) ) then
                  ist(j) = i
                  ind(j) = 0
               end if
            end do
         end if
         if ( ind(i).gt.1000 .and. ind(i).le.2000 ) ind(lengthmst) = 1
         if ( ind(i).gt.10000 ) funchk = .true.
      end do

!     --- Check if matrix and vector must be filled

      if ( ind(1).eq.0 .and. ind(2).eq.0 .and. ind(3).eq.0 ) notvec = 1
      if ( ind(4).eq.0 .and. ind(5).eq.0 .and. ind(6).eq.0 ) notmat = 1

      if ( funchk ) ind(lengthmst) = ind(lengthmst) + 2

      if ( ierror.ne.0 ) call instop
      notmas = 1
      call erclos ( 'el0910' )

      end
