      subroutine el2604 ( phi0, phin, phi, dphidx0, dphidxn, psi, &
                          u, ug, Fd, Jd, Fn, Jn, p, p8, pn, pn8, &
                          mu, mu8, mun, mun8, usrvec, iusrvc, iseqnr, &
                          isequ, iseqp, iseqmu, npres, &
                          ci, cn, ncoef, work )
! ======================================================================
!
!        programmer    Chris van Oijen
!        version  4.0  date 06-01-2016 Use sepmoduleelem
!        version  3.1  date 12-12-2012 Long integers
!        version  3.0  date 19-05-2003 Extra input parameters
!                                      Changed computation pressure (RvL)
!
!   copyright (c) 2000-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Get old displacement vector, deformation gradient, pressures and
!    concentrations.
! **********************************************************************
!
!                       KEYWORDS
!
!     mechanics
!     non_linear
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      use sepmoduleoldvecs
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iseqnr, iusrvc, npres, &
              isequ(*), iseqp(*), iseqmu(*), ncoef
      double precision phi0(m,m), phin(m,m), phi(m,m),dphidx0(m,m,ndim), &
                       dphidxn(m,m,ndim), psi(npres,m), &
                       u(n,ndim), ug(m,ndim), Fd(m,ndim,ndim), Jd(*), &
                       Fn(m,ndim,ndim), Jn(*), pn8(*), p(*), pn(*), &
                       p8(*), mu(3,*), mu8(3,*), mun(3,*), mun8(3,*), &
                       usrvec(m,*), ci(3,m), cn(3,m), work(*)

!     ci             o    Array of 3 x m long that contains the concentrations.
!                         ci(1,i) fixed charge concentration
!                         ci(2,i) cation concentration
!                         ci(3,i) anion concentration
!     cn             o    Array of 3 x m long that contains the old
!                         concentrations.
!                         ci(1,i) fixed charge concentration
!                         ci(2,i) cation concentration
!                         ci(3,i) anion concentration
!                         derivative
!                         of the basis functions in the sequence:
!                         d phi / dx (xg ) = dphidx (i,k,1);
!                              i        k
!                         d phi / dy (xg ) = dphidx (i,k,2);
!                              i        k
!                         d phi / dz (xg ) = dphidx (i,k,3);
!                              i        k
!                         If the element is a linear simplex, the
!                         derivatives are constant and only k = 1 is filled.
!     dphidx0        i    ?
!     dphidxn        i    ?
!     fd             o    Deformation gradient F(k,j,i) = F_ij(x^k)
!     fn             o    Deformation gradient at incr. n Fn(k,j,i) = F_ij(x^k)
!     index3lc         i    Two-dimensional integer array of length NUMOLD x
!                         NINDEX containing the positions of the "old"
!                         solutions in array
!                         VECOLD with respect to the present element
!                         For example VECOLD(INDEX3(i,j)) contains the j th
!                         unknown with respect to the i th old solution vector.
!                         The number i refers to the i th vector corresponding
!                         to IVCOLD in the call of SYSTM2 or DERIVA
!     iseqnr         i    Sequence number with respect to the storage in array
!                         ISEPIN
!     iseqp          i    Sequence for pressure dof's
!     isequ          i    Sequence for displacement dof's
!     iusrvc         i    if>0, a user defined vector is used
!     jd             o    Determinant of deformation gradient
!     jn             o    Determinant of deformation gradient at incr. n
!     mu             o    The chemical potentials in the nodal points
!                         mu(1,i) chemical potential of the fluid
!                         mu(2,i) chemical potential of the cations
!                         mu(3,i) chemical potential of the anions
!     mu8            o    chemical potential in vertices
!     mun            o    Old chemical potentials in the nodal points
!                         mun(1,i) chemical potential of the fluid
!                         mun(2,i) chemical potential of the cations
!                         mun(3,i) chemical potential of the anions
!     mun8           o    old chemical potentials in vertices
!     ncoef          i    Number of coefficients in the equation
!     npres          i    Number of pressure parameters per element
!     p              o    Pressure in nodal points
!     p8             o    Pressure in the vertices
!     phi            i    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     phi0           i    ?
!     phin           i    ?
!     pn             o    Old pressure in the nodal points
!     pn8            o    Old pressure in the vertices
!     psi            o    Array of length n x m containing the values of the
!                         modified basis functions in the integration points
!                         Basis functions for the pressure
!     u              o    Real n x ncomp array in which the values of the
!                         solution vector in the nodal points of the element are
!                         stored in the sequence:
!                         u  (x ) = u(j,i)  j = 1(1)n, i = 1 (1) ncomp
!                          i   j
!                         Array u is always filled
!     ug             o    array of length m x ncomp in which the values of
!                         u in the integration points are stored the sequence:
!                         ug (x ) = u(j,i)  j = 1(1)m, i = 1 (1) ncomp
!                           i   j
!                         ugauss is only filled if jdiag>1 in common celint
!     usrvec         i    user defined vector
!     work                Work array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, idx(8), ncompsav, k
      double precision, pointer :: vecoldlc(:)
      double precision rdum(1)
      double precision pd_tmp(npres), pn_tmp(npres), mud_tmp(3,npres), &
                       mun_tmp(3,npres), un(m,ndim), Temp, Rconst, vm, &
                       vp, Gamma, cfc, nf, nm, np, coef(ncoef), lambda

!     cfc            Initial fixed charge density
!     coef           Element coefficients
!     gamma          Osmotic coefficient
!     i              Counting variable
!     idx            Indices of the vertices (3D)
!     j              Counting variable
!     k              Counting variable
!     lambda         Initial deformation
!     mud_tmp        Temporary array to store the chemical potentials
!     mun_tmp        Temporary array to store the old chemical potentials
!     ncompsav       Help variable to save the value of ncomp
!     nf             Initial volume fraction of fluid
!     nm             Initial volume fraction of anions
!     np             Initial volume fraction of cations
!     pd_tmp         Temporary array to store the pressures
!     pn_tmp         Temporary array to store the old pressures
!     rconst         Gas constant
!     rdum           dummy variable
!     temp           Temperature
!     un             normal component of velocity
!     vm             Molar volume of the anions
!     vp             Molar volume of the cations
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL2005         Compute a local vector and its derivatives from a known
!                    vector
!     EL2700         Compute deformation gradient
!     EL3007         Fills old solution and derivatives
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data idx / 1, 3, 7, 9, 19, 21, 25, 27 /
! ======================================================================
!
!     --- Get the displacements and derivatives on increment n
!         always jind = 0, because we use isequ to look up the
!         correct degree of freedom

      jderiv = 1
      jsol = 1
      jind = 0
      call el2005 ( un, Fn, phi0, dphidx0, isequ, ug, iseqnr+1 )

!     --- Compute deformation gradient from displacement gradient

      call el2700 (Fn, Jn)

!     --- Get the delta displacements and derivatives
!         always jind = 0, because we use isequ to look up the
!         correct degree of freedom

      jderiv = 1
      jsol = 1
      jind = 0
      call el2005 ( u, Fd, phin, dphidxn, isequ, ug, iseqnr )

!     --- Compute deformation gradient from displacement gradient

      call el2700 (Fd, Jd)

!     --- Get other degrees of freedom such as pressure, chem. pot. and
!         electr. pot.

      iseqtype = oldsols(iseqnr+1)%iseqtype
      index3n => indoldsols(iseqtype)%index3
      vecoldlc => oldsols(iseqnr+1)%oldvecs
      pn_tmp(1:npres) = vecoldlc(index3n(iseqp(1:npres)))
      do i = 1, npres
         do j = 1, 3
            mun_tmp(j,i) = vecoldlc(index3n(iseqmu(j+(i-1)*3)))
         end do
      end do
      iseqtype = oldsols(iseqnr)%iseqtype
      index3n => indoldsols(iseqtype)%index3
      vecoldlc => oldsols(iseqnr)%oldvecs
      pd_tmp(1:npres) = vecoldlc(index3n(iseqp(1:npres)))
      do i = 1, npres
         do j = 1, 3
            mud_tmp(j,i) = vecoldlc(index3n(iseqmu(j+(i-1)*3)))
         end do
      end do

!     --- Compute quantities in each integration point
!         the current quantities are computed by x = xn + dx

      do i = 1, m
         p(i)     = 0d0
         pn(i)    = 0d0
         mu(1,i)  = 0d0
         mu(2,i)  = 0d0
         mu(3,i)  = 0d0
         mun(1,i) = 0d0
         mun(2,i) = 0d0
         mun(3,i) = 0d0
         do j = 1, npres
            p(i)     = p(i) + (pn_tmp(j)+pd_tmp(j))*psi(j,i)
            pn(i)    = pn(i) + pn_tmp(j)   *psi(j,i)
            mu(1,i)  = mu(1,i) + (mun_tmp(1,j)+mud_tmp(1,j))*psi(j,i)
            mu(2,i)  = mu(2,i) + (mun_tmp(2,j)+mud_tmp(2,j))*psi(j,i)
            mu(3,i)  = mu(3,i) + (mun_tmp(3,j)+mud_tmp(3,j))*psi(j,i)
            mun(1,i) = mun(1,i) + mun_tmp(1,j)*psi(j,i)
            mun(2,i) = mun(2,i) + mun_tmp(2,j)*psi(j,i)
            mun(3,i) = mun(3,i) + mun_tmp(3,j)*psi(j,i)
         end do
      end do

!     --- get some coefficients

      do i = 1, m
         do k = 1, ncoef
            coef(k) = work(i+m*(k-1))
         end do
         cfc = coef(35-5)
         nf = coef(37-5)
         np = coef(38-5)
         nm = coef(39-5)
         vp = coef(31-5)
         vm = coef(32-5)
         Gamma = coef(36-5)
         Rconst = coef(43-5)
         Temp = coef(42-5)
         lambda = coef(45-5)

!        --- Compute the fixed charge concentrations in the deformed state

         ci(1,i)=-((nf+nm+np)*cfc)/(1d0-nf-nm-np-(Jn(i)*Jd(i)/lambda))
         cn(1,i)=-((nf+nm+np)*cfc)/(1d0-nf-nm-np-(Jn(i)/lambda))
      end do

      do i=1,m

!     --- Compute new ion concentrations and pressures from the chemical
!         potentials

         do j=1,10
            ci(2,i)=-0.5d0*ci(1,i)+0.5d0*(ci(1,i)**2d0+4d0*EXP((vp* &
                (mu(2,i)-p(i))+ &
               vm*(mu(3,i)-p(i)))/(Rconst*Temp)))**(0.5d0)
            ci(3,i)=0.5d0*ci(1,i)+0.5d0*(ci(1,i)**2d0+4d0*EXP((vp* &
               (mu(2,i)-p(i))+ &
               vm*(mu(3,i)-p(i)))/(Rconst*Temp)))**(0.5d0)
               p(i)=Rconst*Temp*Gamma*(ci(3,i)+ci(2,i))+mu(1,i)
         end do

!        --- Compute old concentrations and pressures from the chemical
!            --- potentials

         do j=1,10
            cn(2,i)=-0.5d0*cn(1,i)+0.5d0*(cn(1,i)**2d0+4d0*EXP((vp* &
                (mun(2,i)-pn(i))+ &
               vm*(mun(3,i)-pn(i)))/(Rconst*Temp)))**(0.5d0)
              cn(3,i)=0.5d0*cn(1,i)+0.5d0*(cn(1,i)**2d0+4d0*EXP((vp* &
               (mun(2,i)-pn(i))+ &
               vm*(mun(3,i)-pn(i)))/(Rconst*Temp)))**(0.5d0)
            pn(i)=Rconst*Temp*Gamma*(cn(3,i)+cn(2,i))+mun(1,i)
         end do
      end do

!     --- Compute quantities in integration points on the vertices

      if (ishape==4.or.ishape==6.or.ishape==7) then
         idx(1)=1
         idx(2)=3
         idx(3)=5
         idx(4)=7
      end if
      do i = 1, npres
         p8(i)     = p(idx(i))
         pn8(i)    = pn(idx(i))
         mu8(1,i)  = mu(1,idx(i))
         mu8(2,i)  = mu(2,idx(i))
         mu8(3,i)  = mu(3,idx(i))
         mun8(1,i) = mun(1,idx(i))
         mun8(2,i) = mun(2,idx(i))
         mun8(3,i) = mun(3,idx(i))
      end do

!     --- Check whether to retrieve the user defined vector

      if (iusrvc>=1) then

         jderiv = 0
         jsol = 1
         jind = 1
         ncompsav = ncomp
         ncomp = iusrvc

!        --- only the values in the integration points are returned

         call el3007 ( usrvec, rdum, phi, rdum, m, n, &
                       ndim, iusrvc, ind, rdum, usrvec, &
                       jtrans, numold, iseqnr+2 )
         ncomp = ncompsav

      end if

      end
