      function el4027(modelv,etha,cn,cl,param,secinv)
! ======================================================================
!
!        programmer   Jaap van der Zanden
!        version 1.3  date   12-12-1995 add modelv=5,6,7
!        version 1.2  date   22-12-1992 expl. decl.
!        version 1.1  date   22-08-1991 Zdenek: no dimension, cdc, ce
!        version 1.0  date   29-05-1985
!
!
!   copyright (c) 1984-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
!
!   DESCRIPTION
!
!     this subroutine computes the effective viscosity
!     for a generalized newtonian liquid
!     and plastico-viscous liquids
!
!     available are :  modelv=2, power law model
!                      modelv=3, 3 parameter carreau model
!                      modelv=4, 3 parameter plastico-viscous liquid
!                               cn=1 : bingham model
!                               cn=2 : casson model
!                      modelv=5  Papanastasiou plastico-viscous model
!                      modelv=6  bi-viscosity plastico-viscous model
!                      modelv=7  Carreau-Yasuda model
!
! **********************************************************************
!
!          INPUT / OUTPUT  PARAMETERS

      use sepmodulecomio
      implicit none 
      integer modelv
      double precision el4027, etha, cn, cl, param(*), secinv
!
!     cl      i    carreau model parameter lambda, or
!                  bingham (plastico-viscous) model parameter s
!     cn      i    Power in power law and other models
!     el4027  o    effective viscosity
!     etha    i    viscosity parameter etha0
!     modelv  i    type of viscosity model
!     param   i    viscosity model parameters for modelv=6,7
!     secinv  i    second invariant of tensor a1
!
! **********************************************************************
!
!
!     LOCAL  PARAMETERS
!
      double precision expn, factor, secndc, term
!
!     expn    Value of an exponent
!     factor  Multiplication factor
!     secndc  Critical value of second invariant for some models
!     term    Value of a term in an expression
!
! **********************************************************************
!
!   ERROR MESSAGES
!
!     None.
! **********************************************************************
!
!   SUBROUTINES  CALLED
!
!     None.
! **********************************************************************
!
!   PSEUDO  CODE
!
!   trivial
! ======================================================================
!
      if (modelv.eq.2) then

!     --- powerlaw model

         expn=(cn-1.0d0)/2.0d0
         if ((expn.le.0).and.(secinv.lt.epsmac)) then
            el4027=etha*(epsmac**expn)
         else
            el4027=etha*(secinv**expn)
         end if
      else if (modelv.eq.3) then

!     --- 3 parameter carreau liquid

         expn=(cn-1.0d0)/2.0d0
         term=1.0d0+cl*secinv
         el4027=etha*(term**expn)
      else if (modelv.eq.4) then

!     --- 3 parameter plastico-viscous liquid

         expn=0.5d0/cn
         factor=1.0d7

!        --- Below a critical value of the rate of deformation a Newtonian
!            approximation is used with etha_0 = factor * etha_pv
!            according to:
!            C.R. Beverly, R.I. Tanner, &
!            Numerical analysis of three-dimensional Bingham plastic flow, &
!            J. non-Newt. Fluid Mech., vol 42 (1992) 85-115
!            A factor of 1000 appears to be optimal for channel and pipe flow.
!            (this is confirmed by my test-problems)
!            A factor of 10000 appears to be better for outflow in open vessels
!            especially for large values of the yiels stress.
!            For an ideal Bingham model the factor is as high as possible
!            secndc is the square of the critical rate of deformation where
!            the stress t=etha_0 (grad v + (grad v)^T) is equal to the stress
!            according to the model

         secndc=( cl/(etha * (factor**(1d0/cn)-1d0)**cn) )**2
         if (secinv.lt.secndc) then
            el4027=factor*etha
         else
            term=(cl*cl/(etha*etha*secinv))**expn
            el4027=etha*(1.0d0+term)**cn
         end if
      else if (modelv.eq.5) then

!     --- 3 parameter Papanastasiou plastico-viscous liquid
!         T.C. Papanastasiou, Flow of materials with yield, &
!         J. Rheology, vol 31, pp 385-404 (1987).

         term=sqrt(secinv)
         if (term.gt.epsmac) then
            el4027=etha+cl*(1.d0-exp(-cn*term))/term
         else
            el4027=etha+cl*cn
         end if
      else if (modelv.eq.6) then

!     --- 4 parameter bi-viscosity plastico-viscous model

         expn=0.5d0/cn
         factor=param(1)
         secndc=( cl/(etha * (factor**(1d0/cn)-1d0)**cn) )**2
         if (secinv.lt.secndc) then
            el4027=factor*etha
         else
            term=(cl*cl/(etha*etha*secinv))**expn
            el4027=etha*(1.0d0+term)**cn
         end if
      else if (modelv.eq.7) then

!     --- 5 parameter Carreau-Yasuda model
!           see R.B. Bird, R.C. Armstrong, O. Hassager, &
!           Dynamics of polymeric liquids, Vol 1, &
!           2^nd ed., Wiley, 1987, pp 171-172.

         expn=(cn-1.0d0)/param(2)
         term=(1.0d0+(cl*sqrt(secinv))**param(2))**expn
         el4027=param(1)+(etha-param(1))*term
      end if
      end
