      subroutine el4915sp ( s, phix, phi, xgauss, eta, w, swork, work )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 11-05-2001 Clear matrix
!        version  1.0  date 15-04-2001
!
!   copyright (c) 2001-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute the viscosity part of the element matrix for navier stokes
!     and store in matrix s for 2d elements
!     In this case the special form of the Navier-Stokes equations is used
!     based on the given coefficients for the various terms
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision phix(n,m,ndimlc), phi(n,m), eta(m,4), w(m), &
                       s(ndimlc*n,ndimlc*n), swork(n,n), work(*), &
                       xgauss(m,ndimlc)

!     eta     i    Array containing the coeeficients of the viscosity
!                  in the integration points
!     phi     i    array of length 7 x m containing the values of the
!                  basis functions in the gauss points in the sequence:
!                         k
!                  phi (xg ) = phi (i,k)
!                     i
!                  array phi is only filled when ifirst = 0 and gauss
!                  integration is applied
!     phix    i    Array of length n x m x 3  containing the derivatives
!                  of the basis functions in the sequence:
!                  d phi / dx (xg ) = phix (i,k,1);
!                       i        k
!                  d phi / dy (xg ) = phix (i,k,2);
!                       i        k
!                  d phi / dz (xg ) = phix (i,k,3);
!                       i        k
!                  If the element is a linear tetrahedron (n=4), then the
!                  derivatives are constant and only k = 1 is filled.
!     s      i/o   Element matrix to be filled
!     swork        Work array in which a sub element matrix is stored
!     w       i    array of length m containing the weights for integration
!     work    i    Work array of length 2*m
!     xgauss  i    array of length m x ndimlc containing the co-ordinates of
!                  the gauss points.  array xgauss is only filled when gauss
!                  integration is applied.
!                  xg  = xgauss (i,1);  yg  = xgauss (i,2);  zg  = xgauss (i,3)
!                    i                    i                    i
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical symm
      integer i, j

!     i       General loop variable
!     j       General loop variable
!     symm    Indication whether the resulting matrix is symmetrical
!             (true) or not (false)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL3010  Fill small matrix elemmt into larger matrix s
!     EL3011  Compute contribution to element stiffness matrix corresponding to
!             a part of the second derivative part of an elliptic equation
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Computation of the stiffness matrix results in a 3x3 block matrix, each
!     consisting of nxn matrices of the following shape:
!            |   S       S      S     |
!            |    uu      uv     uw   |
!            |                        |
!            |   S       S      S     |
!     S =    |    vu      vv     vw   |
!            |                        |
!            |   S       S      S     |
!            |    wu      wv     ww   |
!     The submatrices are defined by:
!              T
!     S    =  S
!      xy      yx
!     Cartesian co-ordinates:
!     S uu =  eta_1 phix(j) phix(i) + eta_2 phiy(j) phiy(i)
!     S vv =  eta_3 phix(j) phix(i) + eta_4 phiy(j) phiy(i)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el4915sp' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el4915sp'
         write(irefwr,1) 'B%cart, E%ndim, n, m', jcart, ndimlc, n, m
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Clear matrix

      do i = 1, ndimlc*n
         do j = 1, ndimlc*n
            s(i,j) = 0d0
         end do
      end do

!     --- Compute matrices eta phix phix, eta phiy phiy and eta phiz phiz
!         and store in Suu, Svv and Sww

      if ( jcart.eq.1 .and. ndimlc.eq.2 ) then

!     --- 2D Cartesian co-ordinates
!         eta_1 phix phix

         jadd = 0
         symm = .true.
         call el3011 ( eta(1,1), swork, w, phix, m, n, 1, 1, work, &
                       symm )

         call el3010 ( s, swork, 2*n, n, 1, 1, 2, .false., 1d0 )

!        --- eta_2 phiy phiy

         symm = .true.
         call el3011 ( eta(1,2), swork, w, phix, m, n, 2, 2, work, &
                       symm )

         jadd = 1
         call el3010 ( s, swork, 2*n, n, 1, 1, 2, .false., 1d0 )

!        --- eta_3 phix phix

         jadd = 0
         symm = .true.
         call el3011 ( eta(1,3), swork, w, phix, m, n, 1, 1, work, &
                       symm )

         call el3010 ( s, swork, 2*n, n, 2, 2, 2, .false., 1d0 )

!        --- eta_4 phiy phiy

         jadd = 0
         symm = .true.
         call el3011 ( eta(1,4), swork, w, phix, m, n, 2, 2, work, &
                       symm )

         jadd = 1
         call el3010 ( s, swork, 2*n, n, 2, 2, 2, .false., 1d0 )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End el4915sp'

      end if  ! ( debug )
      call erclos ( 'el4915sp' )

      end subroutine el4915sp
