      subroutine el8214 (f, w, s, dpsidx, cn, Fd, Jd, &
                         mu, mun, work, ncoef, npres)
! ======================================================================
!
!        programmer    Raoul van Loon/Chris van Oijen
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.1  date 19-05-2003  Some adjustments
!        version  1.0  date 29-05-2000
!
!   copyright (c) 2000-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the part of the element matrix, that is correlated to
!     the fluid flux out and into the element and its right hand side
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     elasticity_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ncoef, npres
      double precision f(3,*), s(npres,npres,6), dpsidx(npres,m,ndim), &
                       w(*), cn(3,*), Fd(m,ndim,ndim), Jd(*), &
                       mu(3,*), mun(3,npres), work(*)

!     cn             i    Array of 3 x m long that contains the old
!                         concentrations.
!                         ci(1,i) fixed charge concentration
!                         ci(2,i) cation concentration
!                         ci(3,i) anion concentration
!     dpsidx         i    Array of length 8 x m x 3  containing the derivatives
!                         of the linear basis functions in the sequence:
!                         d psi / dx (xg ) = psix (i,k,1);
!                                 i        k
!                         d psi / dy (xg ) = psix (i,k,2);
!                                 i        k
!                         d psi / dz (xg ) = psix (i,k,3);
!                                 i        k
!     f              o     Right hand side correlated to the fluxes
!     fd             i    Deformation gradient F(k,j,i) = F_ij(x^k)
!     jd             i    Determinant of deformation gradient
!     mu             i    The chemical potentials in the nodal points
!                         mu(1,i) chemical potential of the fluid
!                         mu(2,i) chemical potential of the cations
!                         mu(3,i) chemical potential of the anions
!     mun            i    Old chemical potentials in the nodal points
!                         mun(1,i) chemical potential of the fluid
!                         mun(2,i) chemical potential of the cations
!                         mun(3,i) chemical potential of the anions
!     ncoef          i    Number of coefficients in the equation
!     npres          i    Number of pressure parameters per element
!     s              i    Element matrix
!     w              i    array of length m containing the weights for
!                         integration
!     work           i    Work array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, l, q, u, v, maxcoef
      parameter (maxcoef=50)
      double precision f1(3,npres), f2(3,npres), S_wrk(npres,npres,6), &
                       coef(maxcoef), K11, theta, delta_t, Dp, Dm, hind, &
                       Temp, R, nf, np, nm, K_wrk(6), K_inv(6), &
                       Fd_wrk(ndim,ndim)

!     coef           Array containing the element coefficients
!     delta_t        Timestep
!     dm             Diffusion coefficient of the anions
!     dp             Diffusion coefficient of the cations
!     f1             Help array for rhs (latest solution part)
!     f2             Help array for rhs (old solution part)
!     fd_wrk         Deformation gradient
!     hind           Hindrance factor
!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     k11            Permeability
!     k_inv          Inverse of friction matrix
!     k_wrk          Friction matrix
!     l              General loop variable
!     maxcoef        Number of coefficients
!     nf             Initial volume fraction of the fluid
!     nm             Initial volume fraction of the anions
!     np             Initial volume fraction of the cations
!     q              Counting variable
!     r              Gas constant
!     s_wrk          Help array
!     temp           Temperature
!     theta          Constant, responsible for the extent to which
!                    the old solution is taken into account
!     u              Counting variable
!     v              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELSYMINV       Compute the inverse of a symmetric stored 3 x 3 matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Clear the matrices

      do j = 1, 3
         do i = 1, npres
            f1(j,i) = 0d0
            f2(j,i) = 0d0
         end do
      end do

      do k = 1, 6
         do i = 1, npres
            do j = 1, npres
               S_wrk(i,j,k) = 0d0
            end do
         end do
      end do

!     --- First term        T
!                    sum  Bp  * Kab * Bp * mu

      do i = 1, npres
         do j = 1, npres
            f1(1,i) = f1(1,i) + S(i,j,1)*mu(1,j) + &
                                S(i,j,4)*mu(2,j) + &
                                S(i,j,6)*mu(3,j)
            f1(2,i) = f1(2,i) + S(i,j,4)*mu(1,j) + &
                                S(i,j,2)*mu(2,j) + &
                                S(i,j,5)*mu(3,j)
            f1(3,i) = f1(3,i) + S(i,j,6)*mu(1,j) + &
                                S(i,j,5)*mu(2,j) + &
                                S(i,j,3)*mu(3,j)
         end do
      end do

!     --- Second term       T            T
!                    sum  Bp  * F * K * F * Bp * (1/J) *mun

!         --- Loop over the integration points

      do k = 1, m

!     --- get coefficients

         do i = 1, ncoef
            coef(i) = work(1+m*(i-1))
         end do
         K11 = coef(20-5)
         delta_t = coef(26-5)
         theta = coef(27-5)
         Dp = coef(28-5)
         Dm = coef(29-5)
         hind = coef(30-5)
         Temp = coef(42-5)
         R = coef(43-5)
         nf=coef(37-5)
         np=coef(38-5)
         nm=coef(39-5)

!        --- compute inverse P-matrix

         K_inv(4) = -(nf*R*Temp*cn(2,k))/Dp
         K_inv(5) = 0d0
         K_inv(6) = -(nf*R*Temp*cn(3,k))/Dm
         K_inv(1) = (nf**2)/K11 - (1d0-hind)*K_inv(4) - &
                    (1d0-hind)*K_inv(6)
         K_inv(2) = -K_inv(4)/(1d0-hind)
         K_inv(3) = -K_inv(6)/(1d0-hind)

!        --- invert it and compute Kab

         call elsyminv(K_inv, K_wrk)
         K_wrk(1) = K_wrk(1)*nf*nf
         K_wrk(2) = K_wrk(2)*np*np
         K_wrk(3) = K_wrk(3)*nm*nm
         K_wrk(4) = K_wrk(4)*nf*np
         K_wrk(5) = K_wrk(5)*np*nm
         K_wrk(6) = K_wrk(6)*nm*nf

!        --- T
!            --- compute Kab = Fd * Kab * Fd
!            --- compute deformation gradient in current integration point

         do j = 1, ndim
            do i = 1, ndim
               Fd_wrk(i,j) = Fd(k,j,i)
            end do
         end do

!        --- assemble second term

         do q = 1, 6
            do i = 1, npres
               do j = 1, npres
                  do l = 1, ndim
                     do v = 1, ndim
                        do u = 1, ndim
                           S_wrk(i,j,q) = S_wrk(i,j,q) + (1d0-theta)* &
                                          w(k)*(1d0/Jd(k))* &
                                          dpsidx(i,k,l)*Fd_wrk(v,l)* &
                                          K_wrk(q)* &
                                          Fd_wrk(v,u)*dpsidx(j,k,u)* &
                                          delta_t
                        end do
                     end do
                  end do
               end do
            end do
         end do

      end do

      do i = 1, npres
         do j = 1, npres
            f2(1,i) = f2(1,i) + S_wrk(i,j,1)*mun(1,j) + &
                                S_wrk(i,j,4)*mun(2,j) + &
                                S_wrk(i,j,6)*mun(3,j)
            f2(2,i) = f2(2,i) + S_wrk(i,j,4)*mun(1,j) + &
                                S_wrk(i,j,2)*mun(2,j) + &
                                S_wrk(i,j,5)*mun(3,j)
            f2(3,i) = f2(3,i) + S_wrk(i,j,6)*mun(1,j) + &
                                S_wrk(i,j,5)*mun(2,j) + &
                                S_wrk(i,j,3)*mun(3,j)
         end do
      end do

!     --- Assemble total vector

      do k = 1, 3
         do i = 1, npres
            f(k,i) = f1(k,i) + f2(k,i)
         end do
      end do

      end
