      subroutine eld406 ( iuser, user, vecpre, islold, work )
! ======================================================================
!
!        programmer    Jaap van der Zanden / Guus Segal
!        version  4.9  date 09-03-2013 Replace elflr7 by elflr71
!        version  4.8  date 12-12-2012 Long integers
!        version  4.7  date 18-09-2002 New call to el3007
!
!   copyright (c) 1984-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element vector to compute pressure and other derived quantities
!     for two-dimensional Navier-Stokes equations
!     extended quadratic triangle for velocities (itype=406)
!              quadratic triangle for swirl
!     ELD406 is a help subroutine for subroutine DERIVA, &
!     it is called through the intermediate subroutine ELDNS2
!     So:
!     DERIVA
!       -  Loop over elements
!          -  ELDNS2
!             - ELD406
!     itype=406 :    axisymmetric coordinates  7-point triangle with swirl
!                    Penalty function approach
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      use sepmodulebuild
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
!                     /celwrk/
!          Work common block to store local arrays
!          CELWRK contains exactly 1500 real positions, the last 500
!          are reserved for the ELP subroutines
      double precision x, w, dphidx, xgauss, phi, array, amasps, &
                       u, ugauss, un, dudx, trans, transl, qphixx, &
                       etaeff, secinv, dvisd2, omega, sigmalc, rho, cn, &
                       clam, f, eij, dlamdx, psi, dpsidx, dum, worksp, &
                       penal, amas, visprm
      common /celwrk/ x(14), w(7), dphidx(98), xgauss(14), phi(49), &
                      array(56), u(21), ugauss(21), un(21), dudx(42), &
                      trans(24), transl(1), qphixx(147), etaeff(7), &
                      secinv(7), dvisd2(7), omega, sigmalc, rho, &
                      cn, clam, psi(42), f(21), dpsidx(84), amasps(36), &
                      penal(14), amas(49), worksp(210), visprm(3), &
                      eij(6), dlamdx(6), dum(488)
      save   /celwrk/

!          The following parameters are used in CELWRK
!     amas           Mass matrix corresponding to basis funtions phi
!     amasps         Mass matrix corresponding to basis funtions psi
!     array          8 x m array to store the coefficients for each
!                    integration point
!                    Contents:
!                    Pos 1-m:       - &
!                    Pos m+1-2m:    rho
!                    Pos 2m+1-3m:   - &
!                    Pos 3m+1-4m:   omega
!                    Pos 4m+1-5m:   f 1
!                    Pos 5m+1-6m:   f 2
!                    Pos 6m+1-7m:   f 3
!                    Pos 7m+1-8m:   eta
!     clam           Parameter lambda for Generalised Newtonian fluids
!     cn             Parameter n for Generalised Newtonian fluids
!     dlamdx         Array of 6 positions containing the derivatives of the
!                    barycentric co-ordinates
!     dphidx         Array of length n x m x 2  containing the derivatives
!                    of the basis functions in the sequence:
!                    d phi / dx = dphidx (i,k,1);  d phi / dy = dphidx (i,k,2);
!                         i                             i
!                      in node k
!     dpsidx         Array of length n-1 x m x 2  containing the derivatives
!                    of the special basis functions corresponding to the
!                    nodal points in the sequence:
!                    d psi / dx = dpsidx (i,k,1);  d psi / dy = dpsidx (i,k,2);
!                         i                             i
!                      in node k
!     dudx           Array of length 4 x m containing the derivatives of the
!                    preceding velocity in the integration points in the
!                    sequence:
!                    1    - m:  du/dx
!                    m+1  - 2m: du/dy
!                    2m+1 - 3m: dv/dx
!                    3m+1 - 4m: dv/dy
!     dum            488 positions reserved for the ELP subroutines
!     worksp         work space free At least 9 positions are used
!     dvisd2         Array of length m to store the derivative of the viscosity
!                    with respect to the second invariant of the
!                    stress tensor in the integration points
!     eij            Array of 6 positions containing the factors e
!     etaeff         Array of length m to store the effective viscosity in
!                    the integration points
!     f              Array of length 3n to store the right-hand side vector
!                    temporarily
!     omega          Angular velocity of rotating system
!     penal          Help array to store the penalty vector
!     phi      o     Array of length n x m containing the values of the
!                    basis functions in the Gauss points in the sequence:
!                           k
!                    phi (xg ) = phi (i, k)
!                       i
!                    Array phi is only filled when ifirst = 0 and gauss
!                    integration is applied
!     psi            Array of length n-1 x m containing the values of the
!                    basis functions corresponding to the nodal points
!                    in the Gauss points in the sequence:
!                           k
!                    psi (xg ) = psi (i, k)
!                       i
!                    Array psi is only filled when ifirst = 0 and gauss
!                    integration is applied
!     qphixx         Array of length 3 x n x n to store the integrals
!                    corresponding to the second derivative terms on a
!                    reference element
!     rho            Density of the fluid
!     secinv         Array of length m to store the second invariant of the
!                    stress tensor in the integration points
!     sigmalc          Inverse of parameter epsilon in Penalty function method
!     trans          Transformation array for the velocity in the centroid
!                    length: 2 * inpelm
!     transl         Not yet used
!     x              array of length n x 2 containing the x and
!                    y-coordinates of the nodal points with respect to the
!                    local numbering in the sequence:
!                    x  = x (i, 1);  y  = x (i, 2);
!                     i              i
!     xgauss   o     Array of length m x 2 containing the co-ordinates of
!                    the gauss points.  Array xgauss is only filled when Gauss
!                    integration is applied.
!                    xg  = xgauss (i, 1);  yg  = xgauss (i, 2);
!                      i                    i
!     u              Array of length 3n to store the preceding velocity in
!                    the nodal points
!     ugauss         Array of length 3m to store the preceding velocity in
!                    the integration points
!     un             Array of length 3n to store the velocity at a preceding
!                    time level in the nodal points
!     visprm         Viscosity model parameters for modelv=6,7
!     w              array of length m containing the weights for the
!                    numerical integration
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - &
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision user(*), vecpre(*), work(*)
      integer iuser(*), islold(numold)

!     islold   i     User input array in which the user puts information
!                    of all preceding solutions
!                    Integer array of length 5 x numold
!     iuser    i     Integer user array to pass user information from
!                    main program to subroutine. See STANDARD PROBLEMS
!     user     i     Real user array to pass user information from
!                    main program to subroutine. See STANDARD PROBLEMS
!     vecpre   i     Solution at previous time (un)
!     work           Work array to be used by this subroutine
!                    At this moment a working space of 800 suffices
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer induv(18)
      integer method, itcoor, npuv, npuv2, nstart, jtcoor, &
              jtime, mconv, irotat, modelv, ich220, &
              ichint, icurve, ich221, ichiv, i, j, ksign, &
              ipwork, isub, isup, jelp, kderiv, number, ntemp, &
              jcomp, ichh, iaddp, indunk(3)
      double precision delta, pc
      save method, itcoor, npuv, npuv2, nstart, jtcoor, ipwork, &
           jtime, mconv, irotat, modelv, isub, isup, indunk, &
           induv, ichiv, jcomp, iaddp

!     delta   jacobian of element
!     i       Counting variable
!     iaddp   ?
!     ich220  ?
!     ich221  ?
!     ichh    ?
!     ichint  ?
!     ichiv   ?
!     icurve  Type of curve
!     indunk  Defines the new sequence of the unknowns in a global way
!     induv   ?
!     ipwork  Starting address for right-hand side part in array work
!             The first part of array work is reserved for the element
!             matrix
!     irotat  Parameter indicating if the co-ordinate system rotates (10
!             or not (0)
!     isub    Integer help parameter for do-loops to indicate the
!             smallest coefficient number that is dependent on space
!     isup    Integer help parameter for do-loops to indicate the
!             largest coefficient number that is dependent on space
!     itcoor  ?
!     jtime   Parameter indicating the type of time-dependence
!             Possible values:
!             0:  Steady state
!             1:  Instationary equations theta method
!             2 or -2:  Pressure correction scheme
!             3:  Mass matrix only
!             4:  Iterative instationary equations theta method
!     j       Counting variable
!     jcomp   ?
!     jelp    Indication of the type of basis function subroutine must
!             be called by ELM400. Possible vaues:
!             1:  Subroutine ELP220  (Seven-point extended triangle)
!     jtcoor  ?
!     kderiv  ?
!     ksign   Help parameter to compute sign of the Jacobian
!     mconv   With this parameter the type of mconvization of the
!             convective terms is indicated
!             Possible values:
!             0:  No convective terms (Stokes)
!             1:  Standard Picard mconvization
!             2:  Newton mconvization
!             3:  Picard mconvization in the wrong way
!                 (small convergence region, slow convergence)
!             4:  Picard mconvization (only effect on right-hand side)
!     method  Indicates the type of method that is used to solve the
!             Navier-Stokes equations
!             Possible values:
!             1:   Penalty function method
!     modelv  Defines the type of viscosity model
!             Possible values:
!               1:  newtonian liquid
!               2:  power law liquid
!               3:  carreau liquid
!               4:  plastico-viscous liquid
!               5:  Papanastasiou plastico-viscous model
!               6:  bi-viscosity plastico-viscous model
!               7:  Carreau-Yasuda model
!             100:  user provided model depending on grad u
!             101:  user provided model depending on the second invariant
!             102:  user provided model depending on the second invariant, &
!                   the co-ordinates and the velocity
!     npuv    ?
!     npuv2   ?
!     nstart
!     ntemp   ?
!     number  Sequence number of solution array in slold/vecold
!     pc      ?
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL0400    Compute element independent quantities
!     EL3007    Fills old solution and derivatives
!     EL4022    Compute viscous matrix for 2-dim incompressible flow
!     EL4025    Compute (transformed) penalty vector
!     EL4028    Compute  viscosity for a generalized newtonian liquid
!     EL4030    Compute (driving) external forces for RHS vector
!     EL4032    Compute coriolis acceleration and centrifugal force
!     EL4033    Compute convective acceleration for RHS vector
!     EL4034    Compute pressure in centroid for penalty method
!     EL4035    Compute gradient of pressure and fill elemvc1 and elemwg
!     EL4038    Compute derived quantities of a vector
!     ELFLR71   Fill variable coefficients in array ARRAY
!     ELIND0    Fill array ICH in COMMON CELIAR for preceding solutions
!     ELP220    Shape functions for 7-point triangle (7 integration points)
!     ELP221    Shape functions for 6 point triangle (7 integration points)
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRCHR    Put character in error message
!     ERREAL    Put real in error message
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
!     ERRWAR    Print warnings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1291   Warning for pressure for plastico-viscous model
!    1304   For coefficient only positive constants are allowed
!    1781   The possibility ifunc>10000 not yet available
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data induv/1,13,7,2,14,8,3,15,9,4,16,10,5,17,11,6,18,12/
      data indunk / 1, 3, 2 /
! ======================================================================
!
!     --- I / O
!         none

      call eropen ( 'eld406' )
      if ( ifirst==0 ) then

!     --- ifirst = 0, compute element independent quantities

         call el0400 ( iuser, user, jelp, isub, isup, method, mconv, &
                       jtime, modelv, irotat )
         ipwork = 1 + (3*inpelm+2)**2

!        --- Fill array ich (COMMON CELIAR) for preceding solutions

         call elind0 ( islold, numold, isub, isup, iuser )

!        --- Set jsol temporarily equal to -1 in order to indicate that the
!            old versions of the ELP subroutines are used

        jsol = -jsol

!       --- Set some constants, these correspond to the old version
!           npuv    number of points for velocity u1, u2 including centroid
!           nstart  number of test function to start with
!           lrow    total number of degrees of freedom in element

         npuv=inpelm+1
         npuv2=npuv+npuv
         lrow=npuv2 + inpelm
         itcoor = 3
         nstart = npuv
         jtcoor = 1
         jcomp  = 2
         ich220 = jtcoor+30
         if (method==0) ich220= ich220+40
         if ( icheld/=1 ) ich220 = ich220 - jtcoor
         if ( icheld==1 .and. modelv==1 .and. mconv==0 ) then
            jderiv = 0
         else
            jderiv = 1
         end if
         ichiv = 1
         if ( modelv>=4 .and. modelv<=6 .and. icheld==1 ) &
          call errwar ( 1291, 0, 0, 0 )
      end if

!     --- compute basis functions, derivatives of basis functions, etc.

      call elp220 ( ich220, ichint, coor, index1, ksign, &
                    icurve, delta, x, xgauss, eij, dlamdx, w, phi, &
                    dphidx, trans, transl, qphixx )

!     --- swirl, quadratic basis functions

      if ( icheld==1 ) then
         ich221 = 11
      else
         ich221 = 10
      end if
      call elp221 ( ich221, ichh, inpelm, coor, index1, &
                    ksign, icurve, delta, x, xgauss, w, psi, dpsidx )

!     --- Fill variable coefficients if necessary
!         First fill uold if necessary

      number = 0
      if ( jtime==1 .or. jtime==4 ) then

!     --- jtime = 1, 4, time-dependent method
!                    Both un and u n+1 * are necessary

         kderiv = jderiv
         jderiv = 0
         jind = 0
         ncomp = 3
         call el3007 ( un, work, phi, dphidx, m, n, ndimlc, &
                       ncomp, induv, trans, ugauss, jtrans, &
                       numold, 1 )
         jderiv = kderiv
         number = 2
      else
         number = 1
      end if
      if ( number>0 ) then

!     --- Number>0, fill old solution

         jderiv = -jderiv
         jind = -1
         ncomp = 2
         call el3007 ( u, dudx, phi, dphidx, m, n, ndimlc, 3, &
                       indunk, trans, ugauss, jtrans, numold, number )
         ncomp = 1
         jtrans = 0
         ntemp = n
         n = inpelm
         call el3007 ( u(2*ntemp+1), dudx(4*m+1), psi, dpsidx, &
                       m, ntemp, ndimlc, 3, indunk(3), trans, &
                       ugauss(2*m+1), jtrans, numold, number )
         n = ntemp
         ncomp = 3
         jtrans = 1
         jderiv = -jderiv
      end if
      if ( icheld==1 .or. icheld==5 ) then

!     --- icheld = 1 or 5, functions must be computed

         if (ind(8)/=0) ichiv = ichint
         do 150 i = isub, isup
            if ( ind(i)>0 .or. ist(i)>0 ) then

!           --- Coefficient is variable

               if ( ind(i)>10000 )then
                  call errint ( i, 1 )
                  call errsub ( 1781, 1, 0, 0 )
               end if
               call elflr71 ( i, user, array, x, &
                             xgauss, m, n, phi, 1, index1, &
                             iuser, ugauss, work )
               if ( i==8 ) then

!              --- Coefficient 8, i.e. eta
!                  Test if eta is positive

                   do 110 j = 1, m
                      if ( array(j+(i-1)*m)<=0 ) then

!                     --- Error 1304: Eta is not positive

                         call errint ( i, 1 )
                         call errint ( itype, 2 )
                         call errint ( ielgrp, 3 )
                         call errint ( ielem, 4 )
                         call erreal ( array(j+(i-1)*m), 1 )
                         call errchr ( 'eta (0)', 1 )
                         call errsub ( 1304, 4, 1, 1 )
                      end if
110                continue
                end if
            end if
150      continue
      end if
      if ( icheld==1 ) then

!     --- compute effective viscosity

         call el4028 ( 1, itcoor, npuv, ndimlc, &
                       modelv, array(1+7*m), cn, clam, visprm, &
                       work(ipwork), xgauss, ugauss, dudx, etaeff, &
                       secinv, dvisd2 )

!        --- compute derived quantities

      else
         call el4038 ( icheld, ix, jdegfd, itcoor, npuv, &
                       ndimlc, modelv, array(1+7*m), cn, clam, &
                       visprm, x, u, dudx, elemvc1, elemwg, worksp)
         go to 3000
      end if

!     --- compute ndimlc rows of viscous matrix

      call el4022 ( ichiv, itcoor, etaeff, work, w, xgauss, phi, dphidx, &
                    nstart, npuv, npuv, lrow, psi, dpsidx, inpelm, &
                    dlamdx, qphixx, delta, work(ipwork) )

!     --- compute (transformed) penalty vector if required

      iaddp=0
      if ( method==1 ) call el4025 ( jtcoor, ichint, iaddp, work, &
           penal, sigmalc, ksign, delta, eij, w, xgauss, phi, dphidx, &
           npuv, npuv, jcomp, lrow )

!     --- compute right-hand side vector
!         (driving) external forces

      call el4030 ( ichint, itcoor, array(1+4*m), phi, psi, rho, w, &
                    work(ipwork), nstart, npuv, npuv, inpelm, ndimlc, &
                    lrow+inpelm )

!     --- compute influence of convective acceleration for mconv>0

      if (mconv>0) &
          call el4033 ( ichint, itcoor, -rho, ugauss, dudx, &
                        w, xgauss, phi, work(ipwork), &
                        nstart, npuv, npuv, ndimlc, psi, inpelm )

!     --- compute coriolis acceleration and centrifugal force if required

      if ( irotat==1 ) &
          call el4032 ( ichint, itcoor, phi, rho, omega, ugauss, &
                        w, work(ipwork), nstart, npuv, npuv, psi, &
                        inpelm )

!     --- compute pressure in centroid for penalty method

      call el4034 ( pc, sigmalc, delta, penal, u, npuv, ndimlc )

!     --- compute gradient of pressure and fill elemvc1 and elemwg

       call el4035 ( ichint, itcoor, elemvc1, elemwg, pc, phi, xgauss, x, &
                     u, work, work(ipwork), w, npuv, lrow, ndimlc )
3000  call erclos ( 'eld406' )
      end
