      subroutine eld900 ( coor2d, elemvc, iuser, user, index1, &
                          islold, vecloc, coeffnodes, &
                          coeffintpnts, phi, psi, phixi, psixi )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 06-01-2016 Use sepmoduleelem
!        version  3.22 date 12-12-2012 Long integers
!        version  3.21 date 03-02-2010 New call to sepelnsderiv
!
!   copyright (c) 1992-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element derivative vector and weights for Navier-Stokes
!     equations
!     Two and three dimensional elements
!     ELD900 is a help subroutine for subroutine DERIV (TO0047)
!     it is called through the intermediate subroutine eld000
!     So:
!     DERIV
!     TO0047
!       -  Loop over elements
!          -  ELD000
!             - ELD900
!    The subroutine is able to handle the following element shapes and cases:
!    2D:  4 node bilinear quadilateral (Type 900)
!         5 node bilinear quadilateral (Type 901)
!         6 node extended triangle (Type 900)
!         7 node extended triangle (Type 901)
!         9 node biquadratic quadilateral (Types 900 and 901)
!    3D: 27 node brick (Types 900 and 902)
!         15 node tetrahedron (Type 902)
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     element_vector
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/celwrkf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision elemvc(*), coor2d(*), user(*), &
                       vecloc(*), phi(n,m), &
                       coeffnodes(n,*), coeffintpnts(m,*), psi(nvert,m), &
                       phixi(n,m,ndim), psixi(nvert,m,ndim)
      integer iuser(*), index1(*), islold(*)

!     coeffintpnts  i/o   Real array in which the values of the
!                         coefficients in the integration points are stored
!     coeffnodes    i/o   Real array in which the values of the
!                         coefficients in the nodal points are stored
!     coor2d           i    array of length ndim x npointcac containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     elemvc         o    Element vector
!     index1         i    Array of length inpelm containing the point numbers
!                         of the nodal points in the element
!     islold         i    User input array in which the user puts information
!                         of all preceding solutions
!                         Integer array of length 5 x numold
!     iuser          i    Integer user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
!     phi           i/o   array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     phixi         i/o   work array for basis functions
!     psi           i/o   Array of length n x m containing the values of the
!                         modified basis functions in the integration points
!     psixi         i/o   work array for basis functions
!     user           i    Real user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
!     vecloc         i    Work array in which all old solution vectors for the
!                         integration points are stored
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer isub, isup, jelp, jtime, ioldm,indkappa, indcoeffgrad, &
              indcoeffdiv, indetaspecial, indomega, indetha, &
              indgoertler, indcconv, mconv, modelv, jcheld, mult, &
              mcont, icent, nveloc, npres, nsave, imesh, indfdiv, &
              mdiv, mcoefconv, modelrhsd
      double precision qmat(9,m), penalp, cn, clamb, p(npsi), tang(1), &
                       rhsdiv(npsi), rhsdivtr(npsi), dphidx(n,m,ndimlc), &
                       x(n,ndim), w(m+n), xgauss(m,ndim), thetdt, &
                       psipres(npsi,m), sigma(4*n), &
                       dudx(m,ndim,ncomp), pp(ndimlc+1,ndimlc+1), &
                       sp(ndimlc*n,npsi), ugauss(m,ncomp), &
                       unew(n,ncomp), unewgs(m,ncomp), uold(n,ncomp), &
                       dpsidx(nvert,n,ndim), ethaeff(m), &
                       transsp(ndimlc*(n-1),ndimlc), &
                       transdiv(ndimlc*(n-1),ndimlc), &
                       gradv(9,m), div(ndimlc*n,npsi)
      save isub, isup, jelp, jtime, &
           mconv, modelv, penalp, cn, clamb, &
           mult, mcont, nveloc, jcheld, imesh, icent, npres

!     clamb          Parameter lambda with respect to viscosity model
!     cn             Power in power law model
!     div            Divergence matrix
!                    div(i,k) corresponds to the k-th row and i-th column
!                    of the matrix, The k refers to the basis functions of
!                    the pressure, i to the basis functions of the velocity
!                    in the sequence of the solution vector
!     dphidx         Array of length n x m x ndim  containing the derivatives
!                    of the basis functions in the sequence:
!                    d phi / dx (xg ) = dphidx (i,k,1);
!                         i        k
!                    d phi / dy (xg ) = dphidx (i,k,2);
!                         i        k
!                    d phi / dz (xg ) = dphidx (i,k,3);
!                         i        k
!                    If the element is a linear simplex, the
!                    derivatives are constant and only k = 1 is filled.
!     dpsidx         Dummy
!     dudx           Real m x ndim  array in which the values of the
!                    derivatives of the solution vector in the integration
!                    points are stored in the sequence:
!                    du / dx  (x ) = dudx ( k, j )
!                      j     k
!                    If jdercn = 0 in common block celint, then only
!                    dudx (1,*) is filled because dudx is then a constant
!                    per element
!                    dudx is only filled if jderiv>0
!     ethaeff        Contains effective viscosity
!     gradv          Array containing the gradient vector in
!                    the integration points
!     icent          Index of centroid
!     imesh          Sequence number of mesh velocity in input vector
!                    If 0 no mesh velocity is present
!     indcconv       Sequence number of convection coefficients
!     indcoeffdiv    Sequence number of divergence coefficients
!     indcoeffgrad   Sequence number of gradient coefficients
!     indetaspecial  Sequence number of special viscosity
!     indetha        Sequence number of viscosity
!     indfdiv        Sequence number of right-hand side continuity equation
!     indgoertler    Sequence number of goertler coefficients
!     indomega       Sequence number of omega
!     ioldm          Dummy parameter
!     isub           Integer help parameter for do-loops to indicate the
!                    smallest coefficient number that is dependent on space
!     isup           Integer help parameter for do-loops to indicate the
!                    largest coefficient number that is dependent on space
!     jcheld         if ( icheld<= 10 ) then icheld else icheld-20
!     jelp           Indication of the type of basis function subroutine must
!                    be called by ELM100. Possible vaues:
!                     1:  Subroutine ELP610  (Linear line element)
!                     2:  Subroutine ELP611  (Quadratic line element)
!                     3:  Subroutine ELP620  (Linear triangle)
!                     4:  Subroutine ELP622  (Quadratic triangle)
!                     5:  Subroutine ELP621  (Bilinear quadrilateral)
!                     6:  Subroutine ELP623  (Biquadratic quadrilateral)
!                     7:  Subroutine ELP624  (Extended quadratic triangle)
!                    11:  Subroutine ELP630  (Linear tetrahedron)
!                    12:  Subroutine ELP631  (Quadratic tetrahedron)
!                    13:  Subroutine ELP632  (Trilinear brick)
!                    14:  Subroutine ELP633  (Triquadratic  brick)
!                    16:  Subroutine ELP637  (Extended quadratic tetrahedron)
!     jtime          Integer parameter indicating if the mass matrix for the
!                    heat equation must be computed (>0) or not ( = 0)
!                    If jtime = 1, then the coefficient for the mass matrix is
!                    constant, otherwise (jtime = 2) it is variable
!     mcoefconv      Defines how the convetive term is treated:
!                    0: standard case
!                    1: special case: the convective term is defined as
!                          c_conv_1 u du/dx + c_conv_2 v du/dy
!                          c_conv_3 u dv/dx + c_conv_4 v dv/dy
!     mcont          Defines the type of continuity equation
!                    Possible values:
!                    0: incompressible flow
!                    1: compressible flow: div (rho u ) = 0
!     mconv          see USER INPUT in subroutine ELM600
!     mdiv           Defines if divergence right-hand side is used (1) or
!                    not (0)
!     modelrhsd      Defines how the stress tensor is treated in the rhs
!                    Possible values:
!                    0:    if there is a given stress tensor the reference
!                          to this tensor is stored in position 19
!                    1:    It is supposed that there are 6 given stress
!                          tensor components stored in positions 19-24
!     modelv         Defines the type of viscosity model
!                    Possible values:
!                      1:  newtonian liquid
!                      2:  power law liquid
!                      3:  carreau liquid
!                      4:  plastico viscous liquid
!                    100:  user provided model depending on grad u
!                    101:  user provided model depending on the second invariant
!                    102:  user provided model depending on the second invariant
!                          the co-ordinates and the velocity
!     mult           Multiplication factor for the number of unknowns per point
!     npres          Number of pressure parameters
!     nsave          Help parameter to save the value of n
!     nveloc         Number of velocity parameters
!     p              Array to store the values of p in the element
!     penalp         Parameter eps for the penalty function formulation
!     pp             Pressure matrix
!     psipres        Contains the pressure basis functions
!                    psipres(i,k) refers to i-th basis function in
!                    integration point k
!     qmat           transformation matrix global -> local coordinates
!                    in case of a 3D element
!     rhsdiv         Divergence right-hand side (if mdiv=1)
!     rhsdivtr       Transformed divergence right-hand side (if mdiv=1)
!     sigma          Is used to store the stress components
!     sp             Gradient of pressure matrix
!     tang           Dummy
!     thetdt         1/(theta dt) for the case of the theta method
!                    immediately applied
!     transdiv       Contains transformation matrix for divergence
!     transsp        Contains transformation matrix for pressure gradient
!     ugauss         Preceding solution in Gauss points
!     unew           solution vector
!     unewgs         solution vector in Gauss points
!     uold           Array to store the old solution in the nodal points
!     w              array of length m containing the weights for the
!                    numerical integration
!     x              array of length inpelm x 2 containing the x and
!                    y-coordinates of the nodal points with respect to the
!                    local numbering in the sequence:
!                    x  = x (i,1);  y  = x (i,2);
!                     i              i
!     xgauss         Array of length m x 2 containing the co-ordinates of
!                    the gauss points.  Array xgauss is only filled when Gauss
!                    integration is applied.
!                    xg  = xgauss (i,1);  yg  = xgauss (i,2);
!                      i                    i
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL0900GEN      Performs the initializations for subroutine ELM900, ELI900
!                    or ELD900 in the case that IFIRST = 0
!     EL1005         fill of derivatives and weights in elemvc and elemwg
!                    with respect to subroutine ELD250
!     EL1007         Fill element vector for vertices only
!     EL1014         Restrict values of u, x and dudx to centroid
!     EL2005         Compute a local vector and its derivatives from a known
!                    vector
!     EL2006         compute the pressure for the incompressible Navier-Stokes
!                    eq.
!     EL4057         Compute the pressure for the incompressible Navier_Stokes
!                    equation in case of the penalty method
!     ELD900ADDRESS  Compute the parameter jcheld
!                    Compute parameter mult for length of elemvc/wg
!                    Compute starting addresses of subarrays in array
!     ELD900COEFFS   Fill variable coefficients in coefficients array
!                    Fill velocity from prior iteration if necessary
!     ELFILLPSIPRES  Fill the pressure basis functions in the integration points
!                    Crouzeix Raviart elements
!     ELIND0         Fill array ICH in COMMON CELIAR for preceding solutions
!     ELM800BASEFN   Compute the basis functions phi, its derivatives and other
!                    related quantities for various element shapes
!     ELM900ADDRESS1 Compute starting addresses of subarrays in array
!     ELM900DIVGRAD  Compute the divergence and gradient matrix for
!                    Navier-Stokes
!                    Incompressible equations
!     ELM900TRANS    Compute the transformation matrix
!                    Compute the velocity in the centroid and the derivative
!                    of u
!     ELSTRUPD       Update stress term due to initial stress
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
!     SEPELNSDERIV   Compute special derivatives for Navier-Stokes
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays IUSER and USER
!     Usually they are filled by FILCOF
!     The sequence is:
!     1:  ISEQ:  Sequence number of vector in VECOLD/ISLOLD from which UOLD and
!         derivatives must be computed.
!         If 0 or 1 the first vector is used
!         If more vectors are needed, these vectors are supposed to be stored
!         sequentially, where the first one is indicated by ISEQ
!     2:  type of viscosity model used (modelv)
!         Possible values:
!         0,1:  newtonian liquid
!         2:    power law liquid
!         3:    carreau liquid
!         4:    plastico viscous liquid
!       100:    user provided model depending on grad u
!       101:    user provided model depending on the second invariant
!       102:    user provided model depending on the second invariant, the
!               co-ordinates and the velocity
!     3:  Type of numerical integration
!         Possible values:
!         0: default value, defined by element
!     4:  Type of co-ordinate system
!         Possible values:
!         0: Cartesian co-ordinates
!         1: Axi-symmetric co-ordinates
!     5:  Information about the convective terms (mconv)
!         Possible values:
!         0: convective terms are skipped (Stokes flow)
!         1: convective terms are linearized by Picard iteration
!         2: convective terms are linearized by Newton iteration
!         3: convective terms are linearized by the incorrect Picard
!            iteration
!     6-20: Information about the equations and solution method
!     6:  Parameter penalp for the penalty function method
!     7:  density rho
!     8:  angular velocity
!     9:  F1  (Body force in 1-direction)
!    10:  F2  (Body force in 2-direction)
!    11:  F3  (Body force in 3-direction)
!    12:  eta (if modelv = 1, the dynamic viscosity
!                          2, the parameter eta_n
!                          3, the parameter eta_c
!                          4, the parameter eta_pv)
!    13:  n (if modelv = 2-4, the parameter n in the power of the model)
!    14:  lambda (if modelv = 3, the parameter lambda in the viscosity model
!                 if modelv = 4, the parameter s in the viscosity model)
!    15-20: not yet used
!     Remark:  the coefficients 2-20 are only used in the case of icheld = 6
!              or 11, i.e. computation of the stress tensor
!              In the case icheld = 7 and an extended quadratic triangle
!              also all parameters are used
!     icheld                    Cartesian            Cylindrical/Polar
!       1                        du        / d
!                                  jdegfd     ix
!       2                        grad u               2D/Cyl: (r,z)
!                                i.e. du1/dx1,        dur/dr, duz/dr, &
!                                du2/dx1, ...         dur/dz, duz/dz
!                                                     3D/Cyl: (r,z,phi)
!                                                     dur/dr, duz/dr, duphi/dr
!                                                     dur/dr, duz/dz, duphi/dz
!                                                     dur/rdphi-uphi/r, &
!                                                     duz/rdphi, &
!                                                     duphi/rdphi+ur/r
!                                                     2D/Pol: (r,phi)
!                                                     dur/dr, duz/dr, &
!                                                     dur/rdphi-uphi/r, &
!                                                     duphi/rdphi+ur/r
!       3                        - grad u
!       4                        div u
!       5                        curl u                  Cylinder co-or
!                     2D:       du2/dx1-du1/dx2      2D:   duz/dr-dur/dz
!                     3D:       du3/dx2-du2/dx3      3D:   duz/rdphi-duphi/dz
!                               du1/dx3-du3/dx1       duphi/dr+(uphi-dur/dphi)/r
!                               du2/dx1-du1/dx2            dur/dz-duz/dr
!                                                         Polar co-or
!                                                     duphi/dr+(uphi-dur/dphi)/r
!                             | sigma x |                    | sigma r     |
!       6                     | sigma y |                    | sigma z     |
!                     sigma = | sigma z |            sigma = | sigma theta |
!                             | tau xy  |                    | tau rz      |
!                             | tau yz  |                    | tau zphi    |
!                             | tau zx  |                    | tau phir    |
!       7             pressure p
!       8             elongation
!                        ndim = 2, jcart # 4:
!                        e = (u(1)**2*gradu(1,1) + u(2)**2*gradu(2,2) + &
!                             u(1)*u(2)*(gradu(1,2)+gradu(2,1)))/||u||*2
!                        ndim = 3 or jcart = 4:
!                        e = (u(1)**2*gradu(1,1) + u(2)**2*gradu(2,2) + &
!                             u(3)**2*gradu(3,3) + u(1)*u(2)*(gradu(1,2) + &
!                             gradu(2,1)) + u(1)*u(3)*(gradu(1,3)+gradu(3,1)) + &
!                             u(2)*u(3)*(gradu(2,3)+gradu(3,2)))/||u||*2
!       9             deformation
!                        ndim = 2, jcart # 4:
!                        g = (2u(1)u(2)(-gradu(1,1)+gradu(2,2)) + &
!                             u(1)**2-u(2)**2(gradu(1,2)+gradu(2,1)))/||u||*2
!                        ndim = 3 or jcart = 4:
!                        g1 = gradu(1,2) + gradu(2,1)
!                        g2 = gradu(2,3) + gradu(3,2)
!                        g3 = gradu(3,1) + gradu(1,3)
!      10             sqrt(secinv)
!      11             viscous dissipation: 1/2 * t:A1
!      21-31:  See 1-11, however, now defined in the vertices instead of the
!              nodal points
!      32-42:  See 1-11, but now per node per element
!      43-53:  See 1-11, but now per integration point per element
!     Structure of output arrays:
!     Solution array:  ndim unknowns per point (IVEC=0)
!     Arrays of special structure:
!     IVEC = 1:   1 unknowns per point
!     IVEC = 2:   2 unknowns per point
!     IVEC = 3:   3 unknowns per point
!     IVEC = 4:   6 unknowns per point
!     IVEC = 5:   ndim unknowns per point
!     IVEC = 6:   1 unknowns per vertex
!     IVEC = 7:   2 unknowns per vertex
!     IVEC = 8:   3 unknowns per vertex
!     IVEC = 9:   6 unknowns per vertex
!     IVEC =10:   ndim unknowns per vertex
!     IVEC =11:   1/3/6 unknowns per point
!     IVEC =12:   0/1/3 unknowns per point
!     IVEC =13:   1/4/9 unknowns per point
!     IVEC =14:   1/3/6 unknowns per vertex
!     IVEC =15:   0/1/3 unknowns per vertex
!     IVEC =16:   1/4/9 unknowns per vertex
!     ICHELD                 IVEC
!       1                      1
!       2                     13
!       3                     13
!       4                      1
!       5                     12
!       6                      4
!       7                      1
!       8                      1
!       9                     12
!      10                      1
!      11                      1
!      21                      6
!      22                     16
!      23                     16
!      24                      6
!      25                     15
!      26                      9
!      27                      6
!      28                      6
!      29                     15
!      30                      6
!      31                      6
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     if ( ifirst=0 ) then
!        Define element independent quantities
!        Compute pointers in array WORK
!     Compute basis functions and related quantities
!     Fill old solution into array UOLD and if necessary the derivatives
!     Compute variable coefficients if necessary
!     Compute derived quantities
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'eld900' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from eld900'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if
      if ( ierror/=0 ) go to 1000

      if ( ifirst==0 )  then

!     --- ifirst = 0,  compute element independent quantities

         call el0900gen ( iuser, user, jelp, jtime, penalp, cn, &
                          clamb, mconv, mcont, modelv, isub, isup, &
                          coeffintpnts, thetdt, nveloc, npres, imesh, &
                          mdiv, mcoefconv, modelrhsd, ioldm, &
                          coeffnodes )
         if ( inpelm==27 ) then
            icent = 14
         else
            icent = n
         end if

!        --- Fill array ich (COMMON CELIAR) for preceding solutions

         call elind0 ( islold, numold, isub, isup, iuser )

!        --- Compute jcheld and starting addresses

         call eld900address ( jcheld, mult )

      end if
      if ( debug ) then
         write(irefwr,1) 'icheld, jcheld, jtrans', &
                          icheld, jcheld, jtrans
         write(irefwr,1) 'itype, m, n', itype, m, n
      end if  ! ( debug )

!     --- Compute some sequence numbers of coefficients

      call elm900address1 ( indkappa, indcoeffgrad, indcoeffdiv, &
                            indetaspecial, indomega, indetha, &
                            indgoertler, indcconv, mcont, indfdiv )

!     --- compute basis functions and weights for numerical integration

      call elm800basefn ( coor2d, x, w, dphidx, xgauss, phi, &
                          index1, qmat, tang, jelp, phixi, psi, dpsidx, &
                          psixi )
      if ( ierror/=0 ) go to 1000

!     --- Fill variable coefficients if necessary
!         First fill uold if necessary

      call eld900coeffs ( dphidx, phi, ugauss, dudx, uold, &
                          isub, isup, iuser, user, &
                          vecloc, x, xgauss, p, nveloc, &
                          coeffnodes, coeffintpnts )
      if ( debug ) then
         call prinrl ( dudx, n*ndim*ndim, 'dudx/1' )
         write(irefwr,1) 'mcont', mcont
      end if  ! ( debug )

!     --- Compute the divergence and gradient matrix

      if ( jcheld==7 .and. itype/=902 ) then

!     --- Compute the divergence and gradient matrix
!         Store in sp and div
!         pp is not filled and kappa not used
!         Fill array psipress with basis functions for the pressure

         call elfillpsipres ( psipres, npsi, xgauss, x, icent )
         call elm900divgrad ( sp, div, dphidx, pp, phi, w, x, xgauss, &
                              mcont, coeffintpnts(1,1), &
                              coeffintpnts(1,indkappa), &
                              rhsdiv, coeffintpnts(1,indfdiv), mdiv, &
                              ishape, coeffintpnts(1,indcoeffgrad), &
                              coeffintpnts(1,indcoeffdiv), &
                              psipres, npres, icent, coeffnodes(1,1) )
         if ( debug ) then
            write(irefwr,1) 'npres, npsi', npres, npsi
            call prinrl1 ( sp, npsi, ndim*n, 'sp' )
            call prinrl1 ( div, npsi, ndim*n, 'div' )
         end if  ! ( debug )

      end if

      if ( jtrans==1 .and. jcheld==7 ) then

!     --- Compute the transformation matrix
!         Compute the velocity in the centroid and the derivative of u

         call elm900trans ( transdiv, div, mdiv, rhsdivtr, &
                            rhsdiv, transsp, sp, uold, &
                            dudx, phi, dphidx, &
                            ugauss, mcont, mconv, modelv, .true. )
         if ( debug ) &
            call prinrl1 ( transdiv, ndimlc, ndimlc*(n-1), 'trans' )

!        --- Store extra velocity for ALE mesh in unew

         if ( imesh/=0 ) &
            call el2005 ( unew, work, phi, dphidx, index1, &
                          unewgs, imesh )

      end if
      if ( debug ) call prinrl ( dudx, n*ndim*ndim, 'dudx/2' )

!     --- Fill elemvc, content depends on icheld

      if ( jcheld==7 ) then

!     --- icheld = 7, 27, 20, 38 or 49
!         Compute the pressure and store in elemvc

         if ( icheld==7 .or. icheld==20 .or. icheld==38 .or. &
              icheld==49 ) then

!        --- icheld = 7, fill in all nodes

            if ( jtrans==1 ) then

!           --- Special case, the centroid has been eliminated

               call sepelnsderiv ( 6, modelv, coeffintpnts(1,6), ethaeff, cn, &
                             clamb, xgauss, ugauss, dudx, &
                             gradv, 4, &
                             sigma, m, vecloc, maxunk, numold, &
                             m, mcont )
               if ( itype==901 ) elemvc(1) = p(1)
               if ( debug ) write(irefwr,2) 'p(1)', p(1)
               if ( ind(10)/=0 ) then

!              --- An initial stress has been defined
!                  This is an ad-hoc approach and should be adapted once
!                  elm900 is adapted

                  call elstrupd ( sigma, n, coeffintpnts(1,10) )

               end if
               call el2006 ( elemvc, sp, div, w, x, &
                             penalp, uold, ugauss, coeffintpnts(1,3), &
                             phi, coeffintpnts(1,1), dphidx, sigma, &
                             gradv, mconv, xgauss, coeffintpnts(1,2), &
                             icheld, unew, unewgs, &
                             imesh )
               if ( debug ) call prinrl ( elemvc, n, 'elemvc' )

            else

!           --- Standard case

               call el4057 ( elemvc, div, w, x, xgauss, penalp, &
                             uold, p, icheld )

            end if

         else

!        --- icheld = 27, fill in vertices
!            First fill in all nodes

            if ( jtrans==1 ) then

!           --- Special case, the centroid has been eliminated

               call sepelnsderiv ( 6, modelv, coeffintpnts(1,6), ethaeff, cn, &
                             clamb, xgauss, ugauss, dudx, &
                             gradv, 4, &
                             sigma, m, vecloc, maxunk, numold, &
                             m, mcont )
               if ( itype==901 ) elemvc(1) = p(1)
               call el2006 ( work, sp, div, w, &
                             x, penalp, uold, ugauss, coeffintpnts(1,3), &
                             phi, coeffintpnts(1,1), dphidx, sigma, &
                             gradv, mconv, xgauss, coeffintpnts(1,2), &
                             icheld, unew, unewgs, &
                             imesh )
            else

!           --- Standard case

               call el4057 ( work, div, w, x, xgauss, &
                             penalp, uold, p, icheld )

            end if

!           --- Copy in vertices

            call el1007 ( work, elemvc, mult, ishape )

         end if

      else if ( icheld<=5 .or. icheld>=32 .and. icheld<=36 ) then

!     --- 1<=icheld<=5 or 32<=icheld<=36,
!         fill elemvc by derivatives

         call el1005 ( jcheld, ix, jdegfd, uold, dudx, &
                       x, mult, elemvc, m )

      else if ( icheld>=43 .and. icheld<=47 ) then

!     --- 43<=icheld<=47,
!         fill elemvc by derivatives

         call el1005 ( jcheld, ix, jdegfd, ugauss, dudx, &
                       xgauss, mult, elemvc, m )

      else if ( icheld<=11 .or. icheld>36 ) then

!     --- Compute stress tensor, elongation, shear rate, second invariant or
!         viscous dissipation

         call sepelnsderiv ( jcheld, modelv, coeffintpnts(1,6), ethaeff, cn, &
                       clamb, xgauss, uold, dudx, &
                       work, mult, elemvc, m, vecloc, maxunk, &
                       numold, m, mcont )

      else if ( icheld>20 .and. icheld<=25 ) then

!     --- 20<icheld<=25 fill elemvc by derivatives,
!         First fill in all nodes

         call el1005 ( icheld-20, ix, jdegfd, uold, dudx, &
                       x, mult, work, m )

!        --- Copy in vertices

         call el1007 ( work, elemvc, mult, ishape )

      else if ( icheld>11 .and. icheld<=15 ) then

!     --- 12<=icheld<=15
!         fill elemvc by derivatives
!         First set m equal to 1 and compute the arrays u, x and dudx
!         in centroid

         nsave = m
         call el1014 ( uold, dudx, x, ishape )
         m = 1
         call el1005 ( jcheld, ix, jdegfd, uold, dudx, &
                       x, mult, elemvc, nsave )

!        --- reset m

         m = nsave

      else if ( icheld<=20 ) then

!     --- Compute stress tensor, elongation, shear rate, second invariant or
!         viscous dissipation
!         First set n equal to 1 and compute the arrays u, x and dudx
!         in centroid

         nsave = m
         call el1014 ( uold, dudx, x, ishape )
         m = 1
         call sepelnsderiv ( jcheld, modelv, coeffintpnts(1,6), ethaeff, &
                       cn,  clamb, xgauss, uold, dudx, &
                       work, mult, elemvc, m, &
                       vecloc, maxunk, numold, nsave, mcont )

!        --- reset m

         m = nsave

      else

!     --- Compute stress tensor, elongation, shear rate or second invariant
!         First fill in all nodes

         call sepelnsderiv ( jcheld, modelv, coeffintpnts(1,6), ethaeff, cn, &
                       clamb, xgauss, uold, dudx, &
                       work(1+mult*n), mult, work, n, &
                       vecloc, maxunk, numold, m, mcont )

!        --- Copy in vertices

         call el1007 ( work, elemvc, mult, ishape )

      end if

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( elemvc, n, 'elemvc' )
         write(irefwr,*) 'End eld900'

      end if
      call erclos ( 'eld900' )

      end
