      subroutine elfillsp ( sp, phix, phi, w, x, xgauss, ishape, icent )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 08-02-2010
!
!   copyright (c) 2010-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill sp matrix, ie. the gradient of the pressure matrix
!     with respect to Crouzeix Raviart elements
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     navier_stokes_equation
!     gradient
!     pressure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ishape, icent
      double precision phix(n,m,ndimlc), w(m), sp(ndimlc*n,*), &
                       xgauss(m,*), x(n,*), phi(n,m)

!     icent          i    Index of centroid
!     ishape         i    Shape number of elements
!     phi            i    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     phix           i    Array of length n x m x ndim  containing the
!                         derivatives of the basis functions in the sequence:
!                         d phi / dx (xg ) = phix (i,k,1);
!                              i        k
!                         d phi / dy (xg ) = phix (i,k,2);
!                              i        k
!                         d phi / dz (xg ) = phix (i,k,3);
!                              i        k
!                         If the element is a linear tetrahedron (n=4), then
!                         the derivatives are constant and only k = 1 is filled.
!     sp             o    Gradient matrix
!                         SP(i,k) corresponds to the k-th row and i-th column
!                         of the matrix, The k refers to the basis functions of
!                         the pressure, i to the basis functions of the velocity
!                         in the sequence of the solution vector
!     w              i    array of length m containing the weights for
!                         integration
!     x              i    array containing the coordinates of the nodal points
!                         according to:
!                         x  = x (i,1);  y  = x (i,2);  z  = x (i,3)
!                          i              i              i
!     xgauss         i    array of length m x ndim containing the co-ordinates
!                         of the gauss points.  array xgauss is only filled when
!                         gauss integration is applied.
!                         xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                         zg_i = xgauss (i,3)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jdiags, i, j, k
      double precision wmult(m), work(m), sppart(n), sum

!     i              Counting variable
!     j              Counting variable
!     jdiags         Help variable to save jdiag
!     k              Counting variable
!     sppart         Contains part of pressure matrix
!     sum            Help parameter to compute a sum
!     wmult          Contains multiplication vector for basis functions psi
!     work           Work array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL3003         Compute integrals of the shape / f(x) phi (x) dx
!                                                   e         i
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elfillsp' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elfillsp'
         write(irefwr,1) 'jcart, jdiag, ndimlc, n, m, ishape, icent', &
                          jcart, jdiag, ndimlc, n, m, ishape, icent
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      select case ( jcart )

      case(1)

!     --- case 1: cartesian coordinates
!         First psi = 1

         jadd = 0
         jconsf = 2
         jdiags = jdiag
         jdiag = 2
         wmult = 1d0
         do i = 1, ndimlc
            call el3003 ( wmult, sppart, w, phix(1,1,i), m, n )
            do k = 1, n
               sp(ndimlc*(k-1)+i,1) = sppart(k)
            end do
         end do

         if ( ishape==4 .or. ishape==6 .or. ishape==7 .or. &
              ishape==14 .or. ishape==16 ) then

!        --- psi = x  - xcent
!                   j        j

            do j = 1, ndimlc
               wmult = xgauss(1:m,j) - x(icent,j)
               do i = 1, ndimlc
                  call el3003 ( wmult, sppart, w, phix(1,1,i), m, n )
                  do k = 1, n
                     sp(ndimlc*(k-1)+i,j+1) = sppart(k)
                  end do
               end do

            end do
            jdiag = jdiags

         end if

      case(3)

!     --- case 3: polar coordinates

         do i = 1, n
            sum = 0d0
            do k = 1, m
               sum = sum + w(k)*(phix(i,k,1)+phi(i,k)/xgauss(k,1))
            end do
            sp(2*i-1,1) = sum
            sum = 0d0
            do k = 1, m
               sum = sum + w(k)*phix(i,k,2)/xgauss(k,1)
            end do
            sp(2*i,1) = sum

            if ( inpelm>5 ) then

               do j = 1, 2
                  wmult = (xgauss(1:m,j)-x(icent,j))*w
                  sum =0d0
                  do k = 1, m
                     sum = sum + wmult(k)*(phix(i,k,1)+ &
                                 phi(i,k)/xgauss(k,1))
                  end do
                  sp(2*i-1,j+1) = sum
                  sum =0d0
                  do k = 1, m
                     sum = sum + wmult(k)*phix(i,k,2)/xgauss(k,1)
                  end do
                  sp(2*i,j+1) = sum

               end do

            end if

         end do

      case default

!     --- case 2: axi-symmetric coordinates

         do i = 1, n
            sum =0d0
            do k = 1, m
               sum = sum + w(k)*(phix(i,k,1)+phi(i,k)/xgauss(k,1))
            end do
            sp(2*i-1,1) = sum
            sum =0d0
            do k = 1, m
               sum = sum + w(k)*phix(i,k,2)
            end do
            sp(2*i,1) = sum

            if ( inpelm>5 ) then

               do j = 1, 2

                  wmult = (xgauss(1:m,j)-x(icent,j))*w
                  sum =0d0
                  do k = 1, m
                     sum = sum + wmult(k)*(phix(i,k,1)+ &
                                 phi(i,k)/xgauss(k,1))
                  end do
                  sp(2*i-1,j+1) = sum
                  sum =0d0
                  do k = 1, m
                     sum = sum + wmult(k)*phix(i,k,2)
                  end do
                  sp(2*i,j+1) = sum

               end do

            end if

         end do

      end select  ! case ( jcart )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elfillsp'

      end if  ! ( debug )
      call erclos ( 'elfillsp' )

      end
