      subroutine elm210 ( vecloc, work )
! ======================================================================
!
!        programmer    Chris van Oijen
!        version  3.0  date 06-01-2016 Use sepmoduleelem
!        version  2.10 date 29-05-2014 remove call to time subroutine
!        version  2.9  date 12-08-2013 Combine dphi/dxi and dphideta
!
!   copyright (c) 2000-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix and vector for boundary conditions for
!     general nonlinear solids
!     ELM210 is a help subroutine for subroutine BUILD (TO0050)
!     it is called through the intermediate subroutine elmsubs
!     So:
!     BUILD
!     TO0050
!       -  Loop over elements
!          -  ELMSUBS
!             - ELM210
!    The subroutine is able to handle the following element shapes and cases:
!    2D: quadratic line
!    3D: biquadratic quadrilateral
!    3D: Extended biquadratic triangle
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     element_matrix
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulebuild
      use sepmoduleoldvecs
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision vecloc(*), work(*)

!     vecloc          Work array in which all old solution vectors for the
!                     integration points are stored
!     vecold     i    In this array all preceding solutions are stored, i.e.
!                     all solutions that have been computed before and
!                     have been carried to system or deriva by the parameter
!                     E%islold in the parameter list of these main subroutines
!     work       i    Real work array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer maxnu, maxcoef, nisequ, iseqnr
      parameter (nisequ = 27, maxcoef = 15)
      integer isub, isup, jelp
      save isub, isup, jelp
      integer i, j, iload
      integer isequ(nisequ)
      save isequ, iload
      double precision coorn(ndim,inpelm), coor1(ndim,inpelm)
      double precision, pointer :: vecoldlc(:)
      integer ipcoef, ipx, ipxg, ipw, ipphi, ipdphi, ipu, ipug, ipqmat, &
              iptang, ipwork, ifsi, ippsi, ipdpsidx, ipdphidxi, &
              ipdphideta, ipend
      save ipcoef, ipx, ipxg, ipw, ipphi, ipdphi, ipu, ipug, ipqmat, &
           iptang, ipwork, ifsi, ippsi, ipdpsidx, ipdphidxi, &
           ipdphideta, ipend

!     coor1          Coordinates in current configuration
!     coorn          Coordinates in reference configuration
!     i              Counting variable
!     ifsi           Sequence number of old solution vector for
!                    fully coupled fluid-structure interaction
!     iload          Indicates the type of boundary condition.
!     ipcoef         Starting address in array work of coefficients
!     ipdphi         Starting address in array work of deriv. of basis func's
!     ipdphideta     Starting address of array dphideta
!     ipdphidxi      Starting address of array dphidxi
!     ipdpsidx       Starting address of array dpsidx
!     ipend          Last position in array
!     ipphi          Starting address in array work of basis func's
!     ippsi          Starting address in array work of psi
!     ipqmat         Starting address in array work of local rotation matrix
!     iptang         Starting address in array work of tangential load
!     ipu            Starting address in array work of nodal displacement
!     ipug           Starting address in array work of displ. in intgr. points
!     ipw            Starting address in array work of weighing fun's
!     ipwork         Starting address in array work of work array
!     ipx            Starting address in array work of element coordinates
!     ipxg           Starting address in array work of int. points
!     iseqnr         Sequence number with respect to the solution array
!     isequ          Sequence for displacement dof's
!     isub           Integer help parameter for do-loops to indicate the
!                    smallest coefficient number that is dependent on space
!     isup           Integer help parameter for do-loops to indicate the
!                    largest coefficient number that is dependent on space
!     j              Counting variable
!     jelp           Indication of the type of basis function subroutine
!                    must be called by ELxxxx. Possible vaues:
!                     1:  Subroutine ELP610  (Linear line element)
!                     2:  Subroutine ELP611  (Quadratic line element)
!                     3:  Subroutine ELP620  (Linear triangle)
!                     4:  Subroutine ELP624  (Extended quadratic triangle)
!                     5:  Subroutine ELP621  (Bilinear quadrilateral)
!                     6:  Subroutine ELP623  (Biquadratic quadrilateral)
!                     7:  Subroutine ELP624  (Extended quadratic triangle)
!                    11:  Subroutine ELP630  (Linear tetrahedron)
!                    12:  Subroutine ELP631  (Quadratic tetrahedron)
!                    13:  Subroutine ELP632  (Trilinear brick)
!                    14:  Subroutine ELP633  (Triquadratic brick)
!     maxcoef        Maximum number of coefficients
!     maxnu          Maximum number of displacement dof's
!     nisequ         Total length of array isequ
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL0210         performs the initializations for subroutine ELM210
!     EL3003         Compute integrals of the shape / f(x) phi (x) dx
!                                                   e         i
!     EL3023         Compute contribution to element stiffness matrix, resulting
!                    from linearisation of boundary conditions for general
!                    nonlinear solids
!     EL3033         Assemble element stiffness matrix
!     ELFLR71        Fill variable coefficients in array ARRAY
!     ELIND0         Fill array ICH in COMMON CELIAR for preceding solutions
!     ELM910BASEFN   Compute the basis functions phi, its derivatives and other
!                    related quantities for various element shapes
!                    (boundary elements)
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INSTOP         Stop the program
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays E%iuser and USER
!     Usually they are filled by FILCOF
!
!     The sequence is:
!
!    +-------+-------------------------------------------------------------+
!    | coef  | description                                                 |
!    +-------+-------------------------------------------------------------+
!    |   1   | coordinate system for boundary condition                    |
!    |       +---------+----+----------------------------------------------+
!    |                 |  0 | global system                                |
!    |                 |  1 | local system                                 |
!    |                 |  2 | local system and the boundary-integral is    |
!    |                 |    | linearised                                   |
!    |                 |    | (non-symmetric matrix)                       |
!    +-------+---------+----+----------------------------------------------+
!    |   2   | ifsi_solution (integer refers to sequence number of solution|
!    |       |                vector for fully coupled fluid-solid inter.) |
!    +-------+-------------------------------------------------------------+
!    |   3   | type of numerical integration                               |
!    |       +---------+----+----------------------------------------------+
!    |                 |  0 | default value, defined by element            |
!    |                 |  1 | Newton-Cotes                                 |
!    |                 |  3 | Gauss                                        |
!    +-------+---------+----+----------------------------------------------+
!    |   4   | type of coordinate system                                   |
!    |       +---------+----+----------------------------------------------+
!    |                 |  0 | cartesian coordinates                        |
!    |                 |  1 | axisymmetric coordinates                     |
!    +-------+-------------------------------------------------------------+
!    |   5   | not used                                                    |
!    +-------+-------------------------------------------------------------+
!    |   6   | 1-component of stress                                       |
!    +-------+-------------------------------------------------------------+
!    |   7   | 2-component of stress                                       |
!    +-------+-------------------------------------------------------------+
!    |   8   | 3-component of stress                                       |
!    +-------+-------------------------------------------------------------+
!    |   9   | cx                                                          |
!    +-------+-------------------------------------------------------------+
!    |  10   | cy                                                          |
!    +-------+-------------------------------------------------------------+
!    |  11   | cz                                                          |
!    +-------+-------------------------------------------------------------+
!
!    +---------------------------------------------------------------------+
!    |    VECTORS NEEDED                                                   |
!    +---------------------------------------------------------------------+
!    | The following vectors are needed (in this order)                    |
!    +----+-----------+----------------------------------------------------+
!    |  1 |  u        |  incremental solution                              |
!    |  2 |  un       |  old solution at increment n                       |
!    |  3 |  usrvec   |  user defined vector  (if iusrvec>=1)            |
!    +----+-----------+----------------------------------------------------+
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
!                              METHOD
!        The following formulae are used:
!       Type 1:  f(i)   = /  T phi  de
!                 l       e   l   i
!                s(i,j) = /  c phi phi de
!                 l       e   l   i   j
!        Each integral is approximated by a numerical rule of the shape:
!        / int(x) dx = sum  w  int(x )
!        e              k    k      k
!        the factors w  form the weights for the numerical integration
!                     k
!        In the axi-symmetric case they include the factor 2 pi r
!      In case of surface tension we have:
!      f(i) = / gamma/R phi(i) de
!      Because of the relation  n/R = dt/ds  (n normal, t tangent), this
!      can be written as:
!      / gamma dt/ds phi(i) de
!      Partial integration leads to:
!      - / gamma t dphi(i)/ds de + [phi(i) gamma t] end points
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ('elm210')

      if (ifirst==0) then

!     --- ifirst = 0,  compute element independent quantities

         icheld = 0
         call el0210 (E%iuser, E%user, jelp, isub, isup, work, iload, &
                      isequ, index4, ifsi)

!        --- Fill array ich (COMMON CELIAR) for preceding solutions

         call elind0 ( E%islold, numold, isub, isup, E%iuser)
         elemvc = 0d0
         elemmt = 0d0

!        --- Fill pointers in array work

         maxnu=inpelm
         ipcoef     = 1
         ipx        = ipcoef     + maxnu*maxcoef
         ipxg       = ipx        + maxnu*ndim
         ipw        = ipxg       + maxnu*ndim
         ipphi      = ipw        + maxnu
         ipdphi     = ipphi      + maxnu*maxnu
         ipu        = ipdphi     + maxnu*maxnu*ndim
         ipug       = ipu        + maxnu*ndim
         ipqmat     = ipug       + maxnu*ndim
         iptang     = ipqmat     + maxnu*ndim*ndim
         ippsi      = iptang     + maxnu
         ipdpsidx   = ippsi      + maxnu*maxnu
         ipdphidxi  = ipdpsidx   + maxnu*maxnu*ndim
         ipdphideta = ipdphidxi  + maxnu*maxnu*ndim
         ipwork     = ipdphideta + maxnu*maxnu*ndim
         ipend      = ipwork     + maxnu*max(6,maxnu+1)
         B%rule = irule
         B%cart = jcart
         E%ishape = jelp
         call sepelbasefnch

         if ( ipend>13800 ) then

!        --- Array work too small

            call errint ( ipend, 1 )
            call errint ( 13800, 2 )
            call errchr ( 'work', 1 )
            call errsub ( 550, 2, 0, 1 )
            go to 1000

         end if

      end if

!     --- Set iseqnr according to the type of problem

      if ( ifsi>0 ) then
         iseqnr = ifsi
      else
         iseqnr = 1
      end if

!     --- Get coordinates of deformed mesh on increment n (coorn) and
!         compute current and initial coordinates (coor1)

      iseqtype = oldsols(iseqnr)%iseqtype
      index3n => indoldsols(iseqtype)%index3
      vecoldlc => oldsols(iseqnr)%oldvecs
      do i = 1, n
         do j = 1 , ndim
            coorn(j,i) = coor(j,index1(i))
            coor1(j,i) = coorn(j,i) + &
                         vecoldlc(index3n(isequ((i-1)*ndim+j)))
         end do
      end do

!     --- Compute basis functions and get the element coordinates

      call elm910basefn ( coor1, work(ipx), work(ipw), work(ipdphi), &
                          work(ipxg), work(ipphi), index1, work(ipqmat), &
                          work(iptang), jelp, work(ippsi), &
                          work(ipdpsidx), work(ipdphidxi) )
 !     call sepelbasefunc
      if ( ierror/=0 ) goto 1000

!     --- Fill variable coefficients if necessary

      do i = isub, isup
         if (ind(i)>0 .or. ist(i)>0) then

!        --- Coefficient is variable

            call elflr71 (i, E%user, work, work(ipx), &
                         work(ipxg), m, n, work(ipphi), 1, index1, &
                         E%iuser, work(ipug), vecloc )
         end if
      end do

!     --- Fill element vector if necessary

      if (vector .and. notvec/=1) then

!     --- vector = true, compute element vector

         jadd = 0
         goto (500, 501, 501) iload+1

!        --- Load given in global co-ordinate directions

 500     do i = 1, ndim
            if (ind(i)/=0) then

!           --- Component is unequal to zero

               call el3003 (work(1+(i-1)*m), work(ipwork), work(ipw), &
                            work(ipphi), m, n )
               do j = 1, n
                  elemvc(isequ(ndim*(j-1)+i)) = work(ipwork+j-1)
               end do
            end if
         end do
         goto 999

!        --- Load given in normal and tangential directions

 501     do j = 1, m

            work(ipwork+j-1)     = work(ipqmat+(j-1)*ndim*ndim)*work(j)

            work(ipwork+j-1+m)   = work(ipqmat+ndim+(j-1)*ndim*ndim) &
                                        *work(j)

            if (ndim==3) then

            work(ipwork+j-1+2*m) = work(ipqmat+2*ndim+(j-1)*ndim*ndim) &
                                        *work(j)

            end if

         end do

         do i = 1, ndim
            jadd = 0

            call el3003 (work(ipwork+(i-1)*m), work(ipwork+3*m), &
                         work(ipw), work(ipphi), m, n )
            do j = 1, n
               elemvc(isequ(ndim*(j-1)+i)) = work(ipwork+j-1+3*m)
            end do
         end do

 999  end if

!     --- Fill element matrix

      if ( matrix .and. notmat/=1 .and. iload==2 ) then

!     --- Compute element matrix, only for displacement dofs

         call el3023 ( work(1), work(ipwork+3*m), work(ipw), &
                       work(ipphi), work(ipdphidxi), coor1 )

!        --- Fill in the large matrix, based on isequ

         call el3033 ( elemmt, work(ipwork+3*m), isequ )

      end if

1000  if (ierror/=0) call instop

      call erclos ('elm210')

      end
