      subroutine elm900coeffs ( uoldm, dphidx, index1, iseqvel, ioldm, phi, &
                                ugauss, dudx, unew, du0dx, unewgs, &
                                uold, mcont, isub, isup, &
                                imesh, iuser, user, vecloc, x, xgauss, &
                                coeffnodes, coeffintpnts )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 06-01-2016 Use sepmoduleelem
!        version  3.1  date 12-12-2012 Long integers
!        version  3.0  date 17-07-2011 Remove xgauss
!
!   copyright (c) 2006-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill variable coefficients in coefficients array
!     Fill velocity from prior iteration if necessary
!     Help subroutine for elm900
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     navier_stokes_equation
!     initialization
!     coefficient
!     velocity
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer index1(*), ioldm, iseqvel(*), &
              mcont, isub, isup, imesh, iuser(*)
      double precision uoldm(*), dphidx(*), &
                       uold(*), user(*), vecloc(*), x(*), &
                       phi(*), coeffnodes(n,*), ugauss(*), &
                       coeffintpnts(m,*), unew(*), xgauss(m,ndim), &
                       unewgs(*), dudx(*), du0dx(*)

!     coeffintpnts  i/o   Real array in which the values of the
!                         coefficients in the integration points are stored
!     coeffnodes    i/o   Real array in which the values of the
!                         coefficients in the nodal points are stored
!     dphidx         i    Array of length n x m x ndim  containing the
!                         derivatives
!                         of the basis functions in the sequence:
!                         d phi / dx (xg ) = dphidx (i,k,1);
!                              i        k
!                         d phi / dy (xg ) = dphidx (i,k,2);
!                              i        k
!                         d phi / dz (xg ) = dphidx (i,k,3);
!                              i        k
!                         If the element is a linear simplex, the
!                         derivatives are constant and only k = 1 is filled.
!     du0dx          i    See dudx, but now for the extra velocity
!     dudx           i    Real m x ndim  array in which the values of the
!                         derivatives of the solution vector in the integration
!                         points are stored in the sequence:
!                         du / dx  (x ) = dudx ( k, j )
!                           j     k
!     imesh          i    Sequence number of mesh velocity in input vector
!                         If 0 no mesh velocity is present
!     index1         i    Array of length inpelm containing the point numbers
!                         of the nodal points in the element
!     ioldm          i    Sequence number of velocity at end of previous time
!                         step in input vector.
!                         If 0 no iterative procedure is adopted for dynamical
!                         analyses.
!     iseqvel        i    Array containing the positions of the velocities in
!                         the element vector in the sequence required by el2005
!     isub           i    Integer help parameter for do-loops to indicate the
!                         smallest coefficient number that is dependent on space
!     isup           i    Integer help parameter for do-loops to indicate the
!                         largest coefficient number that is dependent on space
!     iuser          i    Integer user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
!     mcont          i    Defines the type of continuity equation
!                         See subroutine elm900
!     phi            i    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     ugauss         i    Preceding solution in Gauss points
!     unew           i    New solution vector
!     unewgs         i    solution vector in integration points
!     uold           i    Array containing the previous iteration (old solution)
!     uoldm          i    Velocity vector at the end of the previous time
!                         step. The rhs contribution of the fluid mass is
!                         computed with this vector for iterative dynamical
!                         analyses. Standard case (no iterations): uold is used.
!     user           i    Real user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
!     vecloc         i    Work array in which all old solution vectors for the
!                         integration points are stored
!     x              i    Contains coordinates of vertices
!     xgauss         i    array of length m x ndim containing the co-ordinates
!                         of the gauss points.  array xgauss is only filled when
!                         gauss integration is applied.
!                         xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                         zg_i = xgauss (i,3)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, jdiagsav, interpol
      double precision work(n)

!     i              Counting variable
!     interpol       If 0 the standard interpolation is applied, &
!                    if 1 the interpolation is restricted to linear
!                    subelements
!     jdiagsav       Help parameter to save the value of jdiag
!     work           Work array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL2005         Compute a local vector and its derivatives from a known
!                    vector
!     EL3009         Fill series of old solutions in array vecloc
!     ELFILLCOEFF    Fills function values in array coeffnodes for the nodal
!                    points and in coeffintpnts for the integration points
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm900coeffs' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elm900coeffs'
         call prinrl1 ( phi, m, n, 'phi' )
         call prinrl1 ( dphidx, m*ndim, n, 'dphidx' )
         write(irefwr,1) 'ind(lengthmst), ioldm', ind(lengthmst), ioldm
         write(irefwr,1) 'iseqin, isub, isup', iseqin, isub, isup
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      interpol = irule/100
      if ( ind(lengthmst)==1 .or. ind(lengthmst)==3 ) then

!     --- velocity vector at the end of previous timestep

         if ( ioldm/=0 ) &
            call el2005 ( uoldm, dudx, phi, &
                          dphidx, iseqvel, ugauss, ioldm )

!        --- most recently computed velocity vector

         call el2005 ( uold, dudx, phi, dphidx, &
                       iseqvel, ugauss, iseqin )
         if ( debug ) then
            call prinrl1 ( uold, ndim, n, 'uold' )
            call prinrl1 ( ugauss, ndim, m, 'ugauss' )
         end if

      end if

!     --- Store extra velocity field for Goertler in wrk1
!         This includes the derivatives

      if ( mcont==2 ) &
         call el2005 ( unew, du0dx, &
                       phi, dphidx, iseqvel, unewgs, imesh )

!     --- Store extra velocity for ALE mesh in wrk1

      if ( imesh/=0 ) &
         call el2005 ( unew, du0dx, &
                       phi, dphidx, iseqvel, unewgs, imesh )
      if ( ind(lengthmst)==2 .or. ind(lengthmst)==3 ) then

!     --- ind(lengthmst) = 2 or 3, hence vecloc must be filled
!         This array is only filled in the nodal points
!         hence jdiag is set to 1 temporarily

         jdiagsav = jdiag  ! save value of jdiag
         if ( interpol==1 ) jdiag = 1
         call el3009 ( vecloc, phi, n )
         jdiag = jdiagsav  ! reset jdiag

      end if  ! ( ind(lengthmst)==2 .or. ind(lengthmst)==3 )

!     --- Fill variable coefficients

      do i = isub, isup
         if ( ind(i)>0 .or. ist(i)>0 ) then

!        --- Coefficient is variable

            call elfillcoeff ( i, user, coeffnodes(1,i), x, &
                               xgauss, phi, 1, iseqvel, &
                               iuser, uold, vecloc, coeffintpnts(1,i) )

         end if

      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elm900coeffs'

      end if
      call erclos ( 'elm900coeffs' )

      end
