      subroutine elm900matrix ( elemmtmp, dphidx, phi, xgauss, &
                                etaspecial, w, etaeff, &
                                mcont, rho, omega, mconv, imesh, &
                                cconv, ugauss, unewgauss, unew, modelv, &
                                symm, mcoefconv, nveloc, du0dx, dudx, &
                                goertler, kappa, psiupw )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 06-01-2016 Use sepmoduleelem
!        version  3.0  date 27-02-2010 Change of parameter list
!        version  2.0  date 19-04-2007 New parameter list
!
!   copyright (c) 2006-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix for the incompressible Navier-Stokes equations
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer mcont, mconv, imesh, modelv, mcoefconv, nveloc
      double precision elemmtmp(*), dphidx(*), phi(*), xgauss(*), &
                       etaspecial(*), w(*), &
                       etaeff(*), rho(*), omega(*), &
                       cconv(*), ugauss(*), unewgauss(*), unew(*), &
                       du0dx(*), dudx(*), goertler(*), kappa(*), &
                       psiupw(*)
      logical symm

!     cconv          i    Array with coefficients c_conv
!                         c_conv_1 = conv(.,1,1)
!                         c_conv_2 = conv(.,2,1)
!                         c_conv_3 = conv(.,1,2)
!                         c_conv_4 = conv(.,2,2)
!     dphidx         i    Array of length n x m x ndim  containing the
!                         derivatives
!                         of the basis functions in the sequence:
!                         d phi / dx (xg ) = dphidx (i,k,1);
!                              i        k
!                         d phi / dy (xg ) = dphidx (i,k,2);
!                              i        k
!                         d phi / dz (xg ) = dphidx (i,k,3);
!                              i        k
!                         If the element is a linear simplex, the
!                         derivatives are constant and only k = 1 is filled.
!     du0dx               See dudx, but now for the extra velocity
!     dudx           i    Real m x ndim  array in which the values of the
!                         derivatives of the solution vector in the integration
!                         points are stored in the sequence:
!                         du / dx  (x ) = dudx ( k, j )
!                           j     k
!                         If jdercn = 0 in common block celint, then only
!                         dudx (1,*) is filled because dudx is then a constant
!                         per element
!                         dudx is only filled if jderiv>0
!     elemmtmp       o    Work array to store (part of) element matrix
!     etaeff         i    Contains effective viscosity
!     etaspecial          Work array to store special form of viscosity
!     goertler       i    Goertler number
!     imesh          i    Sequence number of mesh velocity in input vector
!                         If 0 no mesh velocity is present
!     kappa          i    Parameter for the Goertler equations
!     mcoefconv      i    Defines how the convective term is treated:
!                         0: standard case
!                         1: special case: the convective term is defined as
!                               c_conv_1 u du/dx + c_conv_2 v du/dy
!                               c_conv_3 u dv/dx + c_conv_4 v dv/dy
!     mcont          i    Defines the type of continuity equation
!                         Possible values:
!                         0: incompressible flow
!                         1: compressible flow: div (rho u ) = 0
!                            Not applicable with penalty function method
!     mconv          i    see USER INPUT in subroutine ELM600
!     modelv         i    Defines the type of viscosity model
!                         Possible values:
!                           1, 10:  Newtonian liquid
!                           2:  power law liquid
!                           3:  Carreau liquid
!                           4:  plastico viscous liquid
!                         100:  user provided model depending on grad u
!                         101:  user provided model depending on the second
!                               invariant
!                         102:  user provided model depending on the second
!                               invariant, the co-ordinates and the velocity
!     nveloc         i    Number of velocity parameters
!     omega          i    Contains rotation velcoity omega
!     phi            i    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     psiupw         i    Array of size n*m containing the adapted basis
!                         functions for upwind
!     rho            i    density rho
!     symm           i    Indication whether the resulting matrix is symmetrical
!                         (true) or not (false)
!     ugauss         i    Preceding solution in Gauss points
!     unew           i    New solution vector
!     unewgauss      i    New solution vector in Gauss points
!     w              i    array of length m containing the weights for
!                         integration
!     xgauss         i    array of length m x ndim containing the co-ordinates
!                         of the gauss points.  array xgauss is only filled when
!                         gauss integration is applied.
!                         xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                         zg_i = xgauss (i,3)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision work(n**2), work1(3*m)
      integer i, jdiagsav

!     i              Counting variable
!     jdiagsav       Help parameter to save the value of jdiag
!     work           Work array
!     work1          Work array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL1015         Subtract mesh velocity from velocity
!     EL2010         compute  convective terms of Navier-stokes
!                    for Picard or newton iteration   (k-d)
!     EL2010G        compute  convective terms of Goertler equations
!                    for Picard or newton iteration   (k-d)
!     EL2010SP       compute  convective terms of Navier-stokes
!                    for Picard or newton iteration   (k-d)
!                    special form of Navier-Stokes equations
!     EL4914         Compute contribution of Coriolis term
!     EL4915SP       compute the viscous matrix for 3-dim Navier stokes
!                    (special form)
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL1        Print 2d real vector
!     PRINRL2        Print a real two-dimensional array
!     SEPELNSSTRESS  compute the viscous matrix for 3-dim Navier stokes
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm900matrix' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elm900matrix'
         write(irefwr,1) 'modelv, mconv, mcoefconv, imesh, metupw', &
                          modelv, mconv, mcoefconv, imesh, metupw
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      if ( metupw>0 ) then

!     --- upwind = true, basis functions, not diagonal
!         set jdiag = 2

         jdiagsav = jdiag
         jdiag = 2

      end if  ! ( upwind )

!     --- First viscos terms

      if ( modelv==5 ) then

!     --- Special case the coefficients are given explicitly

         call el4915sp ( elemmtmp, dphidx, phi, xgauss, &
                         etaspecial, w, work, work1 )

      else

!     --- Standard case: Compute the viscous terms

         call sepelnsstress ( elemmtmp, dphidx, phi, xgauss, etaeff, w, &
                              mcont, modelv )

      end if
      if ( debug ) &
         call prinrl1 ( elemmtmp, ndimlc*n, ndimlc*n, 'elemmtmp/1' )

!     --- Next part due to Coriolis accelaration

      if ( ind(2)/=0 ) then

!     --- Coriolis accelaration

         call el4914 ( elemmtmp, phi, rho, w, omega, work, work1 )

      end if

!     --- Finally convective terms

      if ( mconv>0 .and. mconv/=4 ) then

!     --- compute convective terms

         symm = .false.
         if ( imesh>0 .and. mcont/=2 ) then

!        --- Subtract mesh velocity from velocity and put in unew

            call el1015 ( ugauss, unewgauss, unew, m, ndim )

         end if

         if ( mcoefconv>0 ) then

!        --- compute convective terms where the coefficients are stored
!            in the special way

            if ( imesh==0 .or. mcont==2 ) then

!           --- Standard situation

               call el2010sp ( elemmtmp, dphidx, phi, &
                               cconv, w, mconv, ugauss, &
                               dudx, work )

            else

!           --- Subtract mesh velocity from velocity

               call el2010sp ( elemmtmp, dphidx, phi, &
                               cconv, w, mconv, unew, &
                               dudx, work )

            end if

         else if ( mcont==2 ) then

!        --- compute convective terms in case of Goertler equations

            call el2010g ( elemmtmp, dphidx, phi, rho, w, mconv, &
                           ugauss, dudx, xgauss, work, &
                           unew, du0dx, goertler, kappa )

         else

!        --- compute convective terms in standard case

            if ( imesh==0 .or. mcont==2 ) then

!           --- Standard situation, use standard velocity

               if ( debug ) call prinrl1 ( ugauss, ndim, m, 'u' )
               call el2010 ( elemmtmp, dphidx, phi, rho, w, mconv, &
                             ugauss, dudx, xgauss, work, psiupw )
               if ( debug ) call prinrl1 ( elemmtmp, ndim*n, ndim*n, &
                                           'elemmtmp/2' )

            else

!           --- Special case, use adapted velocity

               if ( debug ) call prinrl1 ( unew, ndim, m, 'u' )
               call el2010 ( elemmtmp, dphidx, phi, rho, w, mconv, &
                             unew, dudx, xgauss, work, psiupw )

            end if  ! ( imesh==0 .or. mcont==2 )

         end if

      else if ( imesh>0 ) then

!     --- No convection, only mesh velocity

         mconv = 1
         do i = 1, nveloc
            ugauss(i) = 0d0
         end do
         call el1015 ( ugauss, unewgauss, unew, m, ndim )
         call el2010 ( elemmtmp, dphidx, phi, rho, w, mconv, unew, &
                       dudx, xgauss, work, psiupw )
         mconv = 0

      end if

      if ( metupw>0 ) then

!     --- reset jdiag

         jdiag = jdiagsav

      end if  ! ( upwind )

1000  if ( debug ) then

!     --- Debug information

         call prinrl2 ( elemmtmp, ndimlc*n, ndimlc*n, 'elemmtmp/end' )
         write(irefwr,*) 'End elm900matrix'

      end if
      call erclos ( 'elm900matrix' )

      end
