      subroutine elm900trans ( transdiv, div, mdiv, rhsdivtr, rhsdiv, &
                               transsp, sp, uold, dudx, phi, dphidx, &
                               ugauss, mcont, mconv, modelv, deriv )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 27-02-2010 Change of parameter list
!        version  1.2  date 17-09-2008 Extension with pressure mass matrix
!        version  1.1  date 23-08-2006 New call to el3000
!        version  1.0  date 13-02-2006
!
!   copyright (c) 2006-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the transformation matrix
!     Compute the velocity in the centroid and the derivative of u
! **********************************************************************
!
!                       KEYWORDS
!
!     transformation
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision sp(*), rhsdiv(*), rhsdivtr(*), &
                       transdiv(ndimlc*(n-1),ndimlc), uold(*), dudx(*), &
                       transsp(ndimlc*(n-1),ndimlc), phi(*), dphidx(*), &
                       ugauss(*), div(*)
      integer mdiv, mcont, mconv, modelv
      logical deriv

!     deriv          i    If true the subroutine is called by eld9xx
!     div            i    Divergence matrix
!                         div(i,k) corresponds to the k-th row and i-th column
!                         of the matrix, The k refers to the basis functions of
!                         the pressure, i to the basis functions of the velocity
!                         in the sequence of the solution vector
!     dphidx         i    Array of length n x m x ndim  containing the
!                         derivatives
!                         of the basis functions in the sequence:
!                         d phi / dx (xg ) = dphidx (i,k,1);
!                              i        k
!                         d phi / dy (xg ) = dphidx (i,k,2);
!                              i        k
!                         d phi / dz (xg ) = dphidx (i,k,3);
!                              i        k
!                         If the element is a linear simplex, the
!                         derivatives are constant and only k = 1 is filled.
!     dudx           i    Real m x ndim  array in which the values of the
!                         derivatives of the solution vector in the integration
!                         points are stored in the sequence:
!                         du / dx  (x ) = dudx ( k, j )
!                           j     k
!                         If jdercn = 0 in common block celint, then only
!                         dudx (1,*) is filled because dudx is then a constant
!                         per element
!                         dudx is only filled if jderiv>0
!     mcont          i    Defines the type of continuity equation
!                         Possible values:
!                         0: incompressible flow
!                         1: compressible flow: div (rho u ) = 0
!                            Not applicable with penalty function method
!     mconv          i    see USER INPUT in subroutine ELM600
!     mdiv           i    Defines if divergence right-hand side is used (1)
!                         or not (0)
!     modelv         i    Defines the type of viscosity model
!                         Possible values:
!                           1, 10:  Newtonian liquid
!                           2:  power law liquid
!                           3:  Carreau liquid
!                           4:  plastico viscous liquid
!                         100:  user provided model depending on grad u
!                         101:  user provided model depending on the second
!                               invariant
!                         102:  user provided model depending on the second
!                               invariant, the co-ordinates and the velocity
!     phi            i    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     rhsdiv         i    Divergence right-hand side (if mdiv=1)
!     rhsdivtr       i    Contains values of transformed divergence
!                         right-hand side (if mdiv=1)
!     sp             i    Gradient of pressure matrix
!     transdiv       i    Contains the transformation matrix for the divergence
!     transsp        i    Contains the transformation matrix for the p-gradient
!     ugauss         i    Preceding solution in Gauss points
!     uold           i    Array containing the previous iteration (old solution)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lstep, lsub, lsup

!     lstep          Step variable for vertices. Usually lstep = 1, &
!                    but when the solution must be computed in the vertices
!                    and the element is quadratic then lstep = 2
!     lsub           Under index for the direction of the derivative to be
!                    computed
!     lsup           Upper index for the direction of the derivative to be
!                    computed
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL2000         Compute the transformation matrix to be used in the
!                    elimination process of the centroid point for
!                    incompressible Navier-Stokes
!     EL3000         Compute a local vector and its derivatives from a known
!                    vector
!     ELFILLDERIV    Fill the derivatives of the old solution in nodal points
!     ELFILLUGAUSS   Fill the old solution in Gauss points
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm900trans' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elm900trans'
         write(irefwr,1) 'mcont, modelv', mcont, modelv
         call prinrl1 ( sp, ndimlc+1, ndimlc*n, 'sp' )
         call prinrl1 ( div, ndimlc+1, ndimlc*n, 'div' )
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      call el2000 ( transdiv, div, mdiv, rhsdivtr, rhsdiv )

      if ( debug ) &
         call prinrl1 ( transdiv, ndimlc, ndimlc*(n-1), 'transdiv' )

      if ( mcont/=0 .and. mcont/=5 ) then
         call el2000 ( transsp, sp, 0, rhsdivtr, rhsdiv )
      else
         transsp = transdiv
      end if  ! ( mcont/=0 .and. mcont/=5 )


      if ( debug ) then
         call prinrl1 ( transsp, ndimlc, ndimlc*(n-1), 'transsp' )
         write(irefwr,1) 'mconv, modelv', mconv, modelv
         write(irefwr,1) 'ind(lengthmst)', ind(lengthmst)
      end if  ! ( debug )

      if ( mconv>0 .or. modelv>1 .and. modelv/=10 .or. deriv .and. &
           (ind(lengthmst)==1 .or. ind(lengthmst)==3) ) then

!     --- It is necessary to extract the velocity and to compute the
!         derivatives of u

         call el3000 ( uold, dudx, phi, dphidx, transdiv, &
                       ugauss, mdiv, rhsdivtr, lstep, lsub, lsup )

         if ( jtrans>0 ) then

!        --- jtrans>0, transformation, compute derivatives and u in
!            Gauss points

            if ( jsol/=0 ) then

!           --- Compute solution in Gauss points

               call elfillugauss ( uold, phi, ugauss, n, ncomp, m, jsol, &
                                   jdiag )

            end if  ! ( jsol/=0 )

            if ( jderiv>0 ) then

!           --- jderiv>0,  compute derivatives

               call elfillderiv ( uold, dphidx, dudx, n, ncomp, m, &
                                  jdercn, lsub, lsup, lstep, ndimlc )

            end if  ! ( jderiv>0 )

         end if  ! ( jtrans>0 )

      end if  ! ( mconv>0 .or. modelv>1 .and. modelv/=10 .or. deriv ... )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elm900trans'

      end if
      call erclos ( 'elm900trans' )

      end
