      subroutine elm903gpupw ( sp, phi, psiupw, dthetadx, w )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-05-2007
!
!   copyright (c) 2007-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Update gradient matrix with upwind term due to pressure
!
! **********************************************************************
!
!                       KEYWORDS
!
!     upwinding
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision sp(ndimlc*n,*), phi(n,m), psiupw(n,m), &
                       dthetadx(nvert,m,ndimlc), w(m)

!     dthetadx       i    Contains derivatives of linear basis functions
!     phi            i    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     psiupw         i    Array of size n*m containing the adapted basis
!                         functions for upwind
!     sp            i/o   Gradient of pressure matrix
!                         At input it contains the original grad matrix
!                         At outut it is adapted due to upwind
!     w              i    array of length m containing the weights for
!                         integration
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jdim, i, j, k, ih
      double precision sum

!     i              Counting variable
!     ih             Help variable to store a constant temporarily
!     j              Counting variable
!     jdim           Loop parameter (dimension)
!     k              Counting variable
!     sum            Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL2        Print a real two-dimensional array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The extra term is defined as:
!
!     ( grad p, psi_i - phi_i ), with phi_i the standard basis functions
!     and psi_i the adapted upwind basis functions consisting of phi_i + p_i
!     (p_i is the extra term)
!     So actually it concerns the term (grad p, p_i)
!
!     Using p = sum_j p_j theta_j, with theta_j the pressure basis functions
!     we get:
!
!     sum_j p_j ( grad theta_j, psi_i-phi_i), &
!
!     So the grad matrix is extended with the term
!
!     g_ij = ( grad theta_j, psi_i-phi_i)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm903gpupw' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elm903gpupw'
         write(irefwr,1) 'nvert, npsi, m, n', nvert, npsi, m, n
  1      format ( a, 1x, (10i6) )
         call prinrl2 ( sp, ndimlc*n, npsi, 'Grad-matrix/1' )
         call prinrl2 ( phi, n, m, 'phi' )
         print *,'psiupw(1,2) ', psiupw(1,2)
         call prinrl2 ( psiupw, n, m, 'psiupw' )
         call prinrl2 ( dthetadx(1,1,1), npsi, m, 'dtheta/dx' )
         call prinrl2 ( dthetadx(1,1,2), npsi, m, 'dtheta/dy' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Update G matrix

      do jdim = 1, ndimlc

         do i = 1, n

            do j = 1, npsi

               sum = 0d0

               do k = 1, m
                  sum = sum+dthetadx(j,k,jdim)*(psiupw(i,k)-phi(i,k))* &
                        w(k)
               end do  ! k = 1, m

               ih = (i-1)*ndimlc+jdim
               sp(ih,j) = sp(ih,j) - sum  ! - SP is stored!

            end do  ! j = 1, npsi

         end do  ! i = 1, ndim*n

      end do  ! jdim = 1, ndim

1000  call erclos ( 'elm903gpupw' )
      if ( debug ) then

!     --- Debug information

         call prinrl2 ( sp, ndimlc*n, npsi, 'Grad-matrix/2' )
         write(irefwr,*) 'End elm903gpupw'

      end if  ! ( debug )

      end
