      subroutine elm906
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.1  date 06-08-2014 Set second
!        version  3.0  date 12-08-2013 Extra parameters
!        version  2.1  date 12-12-2012 Long integers
!
!   copyright (c) 2005-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix for the pressure matrix in case of the
!     navier-stokes equations for incompressible flow, using the
!     pressure correction method (Taylor-Hood)
!     Two and three-dimensional elements
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     navier_stokes_equation
!     pressure_correction
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulebuild
      use sepmodulecactl
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
!     phi           i/o   array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     B%phiderc         i/o   Array of length n x m x ndim containing the values of
!                         the derivatives of the basis functions in the Gauss
!                         points in the reference element.
!                         Array phi is only filled when ifirst = 0 and must be
!                         kept by the program
!     psi           i/o   Array of length n x m containing the values of the
!                         modified basis functions in the integration points
!     psixi         i/o   Array of length nvert x m x ndim containing the values
!                         of the derivatives of the basis functions in the Gauss
!                         points in the reference element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, jelp, idummy(1), ind1(8), indexv(8), k, index1sav(27)
      double precision dum(1), alpha(8)
      logical second
      save ind1

!     alpha          array of length m x 3 in which the values of
!                    alpha  , alpha  , alpha
!                         11       12       22
!                    are stored in the sequence:
!                                            k
!                    alpha ( k,i ) = alpha (x )   i = 1(1)3, k=1(1)m
!                                         i
!     dphidx         Array of length n x m x ndim  containing the
!                    derivatives
!                    of the basis functions in the sequence:
!                         d phi_i/ dx_j(xg_k) = dphidx (i,k,j);
!                    If the element is a linear simplex, the
!                    derivatives are constant and only k = 1 is filled.
!     dum            General double precision dummy variable
!     i              Counting variable
!     idummy         Dummy parameter
!     ind1           Array containing the relative position of the vertices
!                    in E%nodes
!     indexv         Contains the nodal points of the vertices
!     jelp           Indication of the type of basis function subroutine
!                    must be called by ELxxxx.
!     k              Counting variable
!     second         dummy
!     w              array of length m containing the weights for
!                    integration
!     x              x co-ordinate of point
!     xgauss         array of length m x ndim containing the co-ordinates
!                    of the gauss points.  array xgauss is only filled when
!                    gauss integration is applied.
!                    xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                    zg_i = xgauss (i,3)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL0906         Initialize some constants with respect to subroutine elm906
!     ELM800BASEFN   Compute the basis functions B%phi, its derivatives and other
!                    related quantities for various element shapes
!     ELM800DIF      Compute element matrix due to diffusion term
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm906' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elm906'
         write(irefwr,1) 'icount, n, ifirst', icount, n, ifirst
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      if ( ifirst==0 )  then

!     --- ifirst = 0,  compute element independent quantities

         call el0906 ( ind1, jelp )
         B%rule = irule
         B%cart = jcart
         E%ishape = jelp
         call sepelbasefnch

!        --- Check array length as defined in elmgrinf

         call inichkarlen (  n, 8, 'ind1' )
         if ( ierror/=0 ) go to 1000

         if ( debug ) write(irefwr,1) 'jelp', jelp

      end if

!     --- Put vertex numbers in indexv

      index1sav(1:size(E%nodes)) = E%nodes
      indexv(1:n) = E%nodes(ind1(1:n))
      E%nodes(1:n) = indexv(1:n)

      if ( debug ) call prinin ( indexv, n, 'indexv' )

!     --- compute basis functions and weights for numerical integration

      call sepelbasefunc
      if ( ierror/=0 ) go to 1000

!     --- Fill parts of matrix if necessary

      if ( vector ) E%vec(1:icount) = 0d0

      if ( matrix ) then

!     --- matrix = true    compute element matrix

         jsecnd = 2  ! constant diffusion (diagonal)
         do k = 1, m
            alpha(k) = 1d0
         end do
         second = .false.
         call elm800dif ( B%type_basisfunc, alpha, E%mat, B%w, B%phider, &
                          dum, B%phi, coor, E%nodes, idummy, dum, &
                          dum, second, B%phiderc, B%xyz, B%xyzg )

      end if  ! ( matrix )

      E%nodes = index1sav(1:size(E%nodes))

1000  if ( debug ) then

!     --- debug = true

         write(irefwr,1) 'element, itype', ielem, itype
         call prinrl1 ( E%mat, icount, icount, 'elemmt' )
         write(irefwr,*) 'End elm906'

      end if
      call erclos ( 'elm906' )

      end
