      subroutine elm910 ( elemmt1d, vecloc, workar, coortr, maxtrn )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 06-01-2016 Use sepmoduleelem
!        version  3.5  date 12-08-2013 Combine dphi/dxi and dphideta
!        version  3.4  date 09-03-2013 Replace elflr7 by elflr71
!
!   copyright (c) 1994-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix and vector for boundary conditions for
!     Navier-Stokes equations
!     Two and three dimensional elements
!     ELM910 is a help subroutine for subroutine BUILD (TO0050)
!     it is called through the intermediate subroutine el0000
!     So:
!     BUILD
!     TO0050
!       -  Loop over elements
!          -  ELMMEC
!             - ELM910
!    The subroutine is able to handle the following element shapes and cases:
!    linear line element (2D)
!    quadratic line element (2D)
!    linear triangle (3D)
!    quadratic triangle (3D)
!    bilinear quadrilateral (3D)
!    biquadratic quadrilateral (3D):
!    cartesian co-ordinates
!    axi-symmetric co-ordinates (2D only)
!    Type numbers 910, 911 and 914
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     element_matrix
!     element_vector
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulebuild
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/celwrkf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer maxtrn
      double precision elemmt1d(*), vecloc(*), workar(*), &
                       coortr(maxtrn,*)

!     coortr         i    kprob part n (local transformations)
!     elemmt1d       o    One dimensional representation of elemmt
!                         (element matrix)
!     maxtrn         i    Maximum number of real variables per point with
!                         respect to the local transformations
!     vecloc              Work array in which all old solution vectors for the
!                         integration points are stored
!     workar              Real work array
!                         work consists of the following parts
!                         Starting address    length   Name:
!                         1                   10*9    ARRAY
!                         Total length: 90
!                         The subarrays contain the folowing contents:
!                         ARRAY:
!                         array in which the values of the coefficients
!                         in the integration points are stored in the sequence:
!                                2D             3D
!                         1:     Tn             Tx
!                         2:     Tt             Ty
!                         3:     -              Tz
!                         4:     cn             cx
!                         5:     ct             cy
!                         6:     -              cz
!                         Each coefficient requires exact m (is number of
!                         integration points) positions
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, isub, isup, jelp, j, iload, induv(27), &
              jcartsav
      double precision uold(9), ugauss(9)
      save isub, isup, jelp, iload, induv
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     i              Counting variable
!     iload          Indicates the type of boundary condition.
!                    Possible values:
!                    0: Prescribed load in co-ordinate directions
!                    1: Prescribed load given in normal and tangential
!                       direction
!                    2: Surface tension only in 2D
!     induv          Index array of size ncomp x inpelm that contains
!                    the positions of the degrees of freedom 1 to ncomp
!                    in the element
!                    Hence induv(k,i) contains the position of the k^th
!                    degree of freedom in the i^th node of the element
!                    with respect to the element vector
!     isub           Integer help parameter for do-loops to indicate the
!                    smallest coefficient number that is dependent on space
!     isup           Integer help parameter for do-loops to indicate the
!                    largest coefficient number that is dependent on space
!     j              Counting variable
!     jcartsav       Help variable to store jcart temporarily
!     jelp           Indication of the type of basis function subroutine
!                    must be called by ELwork. Possible vaues:
!                     1:  Subroutine ELP610  (Linear line element)
!                     2:  Subroutine ELP611  (Quadratic line element)
!                     3:  Subroutine ELP620  (Linear triangle)
!                     4:  Subroutine ELP624  (Extended quadratic triangle)
!                     5:  Subroutine ELP621  (Bilinear quadrilateral)
!                     6:  Subroutine ELP623  (Biquadratic quadrilateral)
!                    11:  Subroutine ELP630  (Linear tetrahedron)
!                    12:  Subroutine ELP631  (Quadratic tetrahedron)
!                    13:  Subroutine ELP632  (Trilinear brick)
!                    14:  Subroutine ELP633  (Triquadratic brick)
!     ugauss         Preceding solution in Gauss points
!     uold           Array containing the previous iteration (old solution)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL0910         Initialize parameters for subroutine ELM910
!     EL3002         Compute integrals of the shape / beta(x) phi (x) phi (x) dx
!                                                   e            i       j
!     EL3003         Compute integrals of the shape / f(x) phi (x) dx
!                                                   e         i
!     EL3007         Fills old solution and derivatives
!     EL3009         Fill series of old solutions in array vecloc
!     EL3013         Fill small matrix elemmtloc of size nesmall x nesmall
!                    into larger matrix elemmt of size nelarge x nelarge
!     EL3014         Fill element vector with surface tension
!     ELCONTANGLE    Computes contribution of contact angle to element vector
!     ELFLR71        Fill variable coefficients in array ARRAY
!     ELIND0         Fill array ICH in COMMON CELIAR for preceding solutions
!     ELM910RHSLD3   Compute the element vector for type 910/911
!                    in case of iload = 3
!     ELMASSDIAGBEL  Compute velocity element mass matrix corresponding to
!                    boundary element, based on parameter sigma in mixed
!                    boundary condition
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
!     SEPELBASEFNCH  Define parameters for choice of basis functions
!     SEPELBASEFUNC  Compute information about basis functions
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays IUSER and USER
!     Usually they are filled by FILCOF
!
!     The sequence is:
!
!     1:  Type of boundary condition
!         Possible values:
!         0: Prescribed load (in co-ordinate directions)
!         1: Prescribed load (in normal and tangential direction (2d))
!         2: Surface tension only in 2D
!         3: Boundary condition of the type c_t v_t + sigma_t = u . t
!            c_t must be given in position 9 and the Cartesian vector u in
!            positions 6 to 8
!            Only in R^3 and in combination with local transforms
!            The first component of the local transformation must correspond to
!            the normal vector and the other 2 to the tangential vectors
!            The tangential vectors are extracted from the local transform
!     2:
!     3:  Type of numerical integration
!         Possible values:
!         0: default value, defined by element
!     4:  Type of co-ordinate system
!         Possible values:
!     5:  Not yet in use:  set equal to 0
!     6:  Tx (1-component in local co-ordinate system)
!         or if iload=2: sigma is coefficient of surface tension
!         or if iload=3: x component of u vector
!     7:  Ty (2-component in local co-ordinate system)
!         or if iload=3: y component of u vector
!     8:  Tz (z component of load or 3-component in local co-ordinate system)
!         or if iload=3: z component of u vector
!     9:  cx (1-component in local co-ordinate system of coefficient before ux)
!         or if iload=3: c_t
!    10:  cy (2-component in local co-ordinate system of coefficient before uy)
!    11:  cz (z component of load or 3-component in local co-ordinate system
!             of coefficient before uz)
!    12-15: Not yet in use (must be zero)
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2238   No local transformation found
! **********************************************************************
!
!                       PSEUDO CODE
!
!                              METHOD
!
!
!        The following formulae are used:
!
!       Type 1:  f(i)   = /  T phi  de
!                 l       e   l   i
!
!                s(i,j) = /  c phi phi de
!                 l       e   l   i   j
!
!        Each integral is approximated by a numerical rule of the shape:
!
!        / int(x) dx = sum  w  int(x )
!        e              k    k      k
!
!
!        the factors w  form the weights for the numerical integration
!                     k
!
!        In the axi-symmetric case they include the factor 2 pi r
!
!
!      In case of surface tension we have:
!
!      f(i) = / gamma/R phi(i) de
!
!      Because of the relation  n/R = dt/ds  (n normal, t B%tangent), this
!      can be written as:
!
!      / gamma dt/ds phi(i) de
!
!      Partial integration leads to:
!
!      - / gamma t dphi(i)/ds de + [phi(i) gamma t] end points
!
!        In the case of iload = 3, we have a three-dimensional problem
!        The boundary condition is c_t v_t + sigma_t = f_t
!        This can only be applied in combination with local transforms
!        n must be the 1 direction and t1, t2 the 2 and 3 directions
!        t1 and t2 may be arbitrary but they must be perpendicular to n
!        The right-hand side f_t is given in the form of three Cartesian
!        co-ordinates.
!        The tangential vectors as stored in coortr are used to transform
!        this to the tangential direction, see also subroutine TO0053
!        The transformation is defined per point in the following way:
!        R is the transformation matrix with elements r_ij stored in coortr
!        in the sequence r_11, r_12, r_13, r21, ...
!        n = ( r11, r21, r31 );  t1 = ( r12, r22, r32 );  t2 = ( r13, r23, r33 )
!        F_1 = 0
!        F_2 = F_cartesian . t1
!        F_3 = F_cartesian . t2
!        The coefficient c_t is the same for both tangential directions
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm910' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elm910'
         write(irefwr,1) 'ielem, itype, icount, inpelm', &
                          ielem, itype, icount, inpelm
         call prinin ( index1, inpelm, 'index1' )
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      IM => intmt(iseqim)

      if ( ifirst==0 )  then

!     --- ifirst = 0,  compute element independent quantities

         icheld = 0
         call el0910 ( E%iuser, E%user, jelp, isub, isup, workar, iload, induv )
         if ( debug ) then
            write(irefwr,*) 'jelp, iload, isub, isup ', &
                             jelp, iload, isub, isup
            call prinin ( induv, ndim*inpelm, 'induv' )
         end if
         B%rule = irule
         B%cart = jcart
         E%ishape = jelp
         E%ktang = 1
         call sepelbasefnch

         if ( iload==3 ) then

!        --- iload = 3, check if local transformations are available

            if ( (ndim==3 .and. maxtrn<9) .or. &
                 (ndim==2 .and. maxtrn<2)  .or. &
                 iworktr(index5(1))<=0 ) then

!           --- maxtrn<9, or no local transform in first point

               call errint ( ielgrp, 1 )
               call errint ( itype, 2 )
               call errsub ( 2238, 2, 0, 0 )

            end if

         end if

!        --- Fill array ich (COMMON CELIAR) for preceding solutions

         call elind0 ( E%islold, numold, isub, isup, E%iuser )
         E%vec = 0d0
         E%mat = 0d0

      end if

!     --- compute basis functions and weights for numerical integration
!         In the case of surface tension we do not multiply the weights
!         by 2 pi r, in order to avoid subdivision by r in el3014

      jcartsav = B%cart
      if ( iload==2 .and. B%cart==2 .and. jelp>0 ) B%cart = 1

!      call elm910basefn ( coor, B%xyz, B%w, B%phider, B%xyzg, B%phi, &
!                          index1, qmat, B%tang, jelp, work(ippsi), dpsidx, &
!                          work(ipdphidxi) )
      call sepelbasefunc
      if ( ierror/=0 ) go to 1000

      B%cart = jcartsav
      if ( ierror/=0 ) go to 1000

!     --- Fill variable coefficients if necessary
!         First fill uold if necessary

      if ( ind(lengthmst)==1 .or. ind(lengthmst)==3 ) &
         call el3007 ( uold, workar, B%phi, B%phider, m, n, &
                       ndimlc, ncomp, induv, workar, ugauss, &
                       jtrans, numold, iseqin )

      if ( ind(lengthmst)==2 .or. ind(lengthmst)==3 ) &
         call el3009 ( vecloc, B%phi, n )

      do i = isub, isup

         if ( ind(i)>0 .or. ist(i)>0 ) then

!        --- Coefficient is variable

            call elflr71 ( i, E%user, workar, B%xyz, B%xyzg, &
                          m, n, B%phi,1, index1, &
                          E%iuser, ugauss, vecloc )

         end if

      end do

!     --- Fill element vector if necessary

      if ( debug ) write(irefwr,*) 'vector, notvec ', vector, notvec
      if ( vector .and. notvec/=1 ) then

!     --- vector = true, compute element vector

         jadd = 0
         if ( debug ) write(irefwr,*) 'iload, m, n ', iload, m, n
         if ( iload==0 ) then

!        --- load given in co-ordinate directions
!            Loop over the ndim components

            do i = 1, ndim

               if ( ind(i)/=0 ) then

!              --- Component is unequal to zero

                  if ( debug ) then
                     write(irefwr,*) 'component ', i
                     call prinrl ( workar(1+(i-1)*m), m, 'coeff' )
                  end if
                  call el3003 ( workar(1+(i-1)*m), work, B%w, B%phi, &
                                m, n )
                  do j = 1, n
                     E%vec(induv(ndim*(j-1)+i)) = work(j)
                  end do

               end if

            end do

         else if ( iload==1 ) then

!        --- Load given in normal and tangential directions
!            use array B%tang

            do j = 1, m
               work(j) = B%tang(j,1)*workar(j+m) + B%tang(j,2)*workar(j)
               work(j+m) = B%tang(j,2)*workar(j+m) - B%tang(j,1)*workar(j)
            end do

            if ( debug ) then

!           --- Debug information

               call prinrl ( B%tang, m, 't_x' )
               call prinrl ( B%tang(1,2), m, 't_y' )
               call prinrl ( workar, m, 'f_n' )
               call prinrl ( workar(1+m), m, 'f_t' )
               call prinrl ( work, m, 'f_x' )
               call prinrl ( work(1+m), m, 'f_y' )

            end if

            do i = 1, ndim

               call el3003 ( work(1+(i-1)*m), work(1+2*m), B%w, &
                             B%phi, m, n )
               do j = 1, n
                  E%vec(induv(ndim*(j-1)+i))=work(j+2*m)
               end do

            end do

         else if ( iload==2 ) then

!        --- Surface tension given

            if ( jelp==0 ) then

!           --- Point element used for contact angle

               call elcontangle ( E%vec, workar(1), workar(2), B%xyz, B%w )

            else

!           --- Line element, use array tang

               call el3014 ( workar, E%vec, B%w, B%phider, B%tang, &
                             B%phi, B%xyzg, B%xyz, induv )

            end if

         else if ( iload==3 ) then

!        --- Special case of iload = 3, i.e. friction boundary condition

            call elm910rhsld3 ( workar, ndim, work, B%phi, E%vec, &
                                induv, iworktr, index5, &
                                coortr, maxtrn, index1, B%w )

         end if

      end if

!     --- Fill element matrix if necessary

      if ( matrix .and. notmat/=1 ) then

!     --- matrix = true, compute element matrix

         if ( iload==3 ) then

!        --- iload = 3, c is stored in coefficient 4

            jadd = 0
            call el3002 ( workar(1+3*m), work, B%w, B%phi, m, n, &
                          n )
            jadd = 1

!           --- Loop over the tangential components

            do i = 2, ndim

               call el3013 ( E%mat, work, icount, n, induv, i, &
                             .false., 1d0 )

            end do

         else

!        --- Standard case three c coefficients
!            Loop over the ndim components

            do i = 1, ndim

               if ( ind(i+3)/=0 ) then

!              --- Component is unequal to zero

                  jadd = 0
                  call el3002 ( workar(1+(i+2)*m), work, B%w, B%phi, &
                                m, n, n )
                  jadd = 1
                  call el3013 ( E%mat, work, icount, n, induv, i, &
                                .false., 1d0 )

               end if

            end do

         end if

      end if

      if ( IM%simple .and. iload<=2 ) then

!     --- Special case for Schur type elements
!         The velocity mass matrix is stored

         call elmassdiagbel ( elemmt1d(icount**2+1), workar(1+3*m), B%w, &
                              induv, ndim, B%phi )

      end if  ! ( IM%simple .and. iload<=2 )

1000  if ( debug ) then

!     --- debug = true

         write(irefwr,1) 'ielem, itype', ielem, itype
         call prinrl(E%vec,icount,'elemvc')
         call prinrl1(E%mat,icount,icount,'elemmt')
         write(irefwr,*) 'End elm910'

      end if
      call erclos ( 'elm910' )

      end
