      subroutine elmassdiagbel ( elemms, sigma, w, iseqvel, ndim, phi )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 13-03-2012 Extra parameter phi
!        version  1.0  date 19-03-2009
!
!   copyright (c) 2009-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute velocity element mass matrix corresponding to boundary
!     element, based on parameter sigma in mixed boundary condition
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     element_matrix
!     velocity
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ndim, iseqvel(ndim,n)
      double precision elemms(icount), w(m), sigma(m,ndim), phi(n,m)

!     elemms         o    Element mass matrix
!     iseqvel        i    Array containing the positions of the velocities in
!                         the element vector in the sequence required by el2005
!     ndim           i    Dimension of the space.
!     phi            i    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     sigma          i    Array containing the parameters "sigma" in the
!                         mixed boundary condition component-wise
!     w              i    array of length m containing the weights for
!                         integration
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, l
      double precision elmat(n,n), sum1
      logical nonzero

!     elmat          Temporary element mass matrix for one component
!                    (non-diagonal)
!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     l              General loop variable
!     nonzero        if true the parameter sigma is non-zero
!     sum1           Help variable to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elmassdiagbel' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elmassdiagbel'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Clear element matrix

      elemms = 0d0

!     --- Check if one of the sigmas is unequal to zero

      nonzero = .false.
      do i = 1, ndim
         if ( ind(3+i)>0 ) then
            nonzero = .true.
         else if ( ind(3+i)<0 .and. sigma(1,i)>1d4 ) then
            nonzero = .true.
         end if  ! ( ind(3+i)>0 )
      end do  ! i = 1, ndim
      if ( .not. nonzero ) go to 1000  ! matrix is zero, jump

!     --- Fill element matrix component-wise

      if ( irule==1 ) then

!     --- irule = 1, hence diagonal matrix

         do i = 1, ndim

            if ( ind(3+i)/=0 ) then

!           --- sigma_i/=0

               do j = 1, n

                  if ( sigma(j,i)>1d4 ) &
                     elemms(iseqvel(i,j)) = w(j)*sigma(j,i)

               end do  ! j = 1, n

            end if  ! ( ind(3+i)/=0 )

         end do  ! i = 1, ndim

      else

!     --- irule>1, hence non-diagonal matrix

         elmat = 0d0

         do l = 1, ndim

            if ( ind(3+l)/=0 ) then

!           --- sigma_l/=0

               do i = 1, n

                  do j = 1, n

                     sum1 = 0d0
                     do k = 1, m
                        sum1 = sum1 + w(k)*sigma(k,l)* phi(i,k)*phi(j,k)
                     end do  ! k = 1, m
                     elmat(i,j) = sum1  ! Create 1 entry in elmat

                  end do  ! j = 1, n

!                 --- Add all off diagonal elements to diagonal

                  elemms(iseqvel(l,i)) = sum(elmat(i,1:n))

               end do  ! i = 1, n

            end if  ! ( ind(3+l)/=0 )

         end do  ! l = 1, ndim

      end if  ! ( irule==1 )

1000  call erclos ( 'elmassdiagbel' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elmassdiagbel'

      end if  ! ( debug )

      end
