      subroutine elstrrhs ( stress, dphidx, elemvclc, w, mloc, nloc, ndimloc, &
                            phi, xgauss, modelrhsd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 08-11-2002 Extra parameters ndimloc ... modelrhsd
!        version  1.0  date 03-12-1998
!
!   copyright (c) 1998-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Compute right-hand side contribution of a given initial stress tensor
!   Ad-hoc test version, only meant for two-dimensional Cartesian Navier- &
!   Stokes
! **********************************************************************
!
!                       KEYWORDS
!
!     stress
!     element_vector
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebuild
      use sepmoduleoldvecs
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nloc, mloc, ndimloc, modelrhsd
      double precision stress(nloc,6), dphidx(nloc,mloc,*), elemvclc(ndimloc,nloc), &
                       w(mloc), phi(nloc,mloc), xgauss(mloc,ndimloc)

!     dphidx         i    Array of length n x m x ndim  containing the
!     derivative
!                         of the basis functions in the sequence:
!                         d phi / dx (xg ) = dphidx (i,k,1);
!                              i        k
!                         d phi / dy (xg ) = dphidx (i,k,2);
!                              i        k
!                         d phi / dz (xg ) = dphidx (i,k,3);
!                              i        k
!                         If the element is a linear simplex, the
!                         derivatives are constant and only k = 1 is filled.
!     elemvclc        i/o   Element vector
!     m              i    number of integration points
!     modelrhsd      i    Defines how the stress tensor is treated in the rhs
!                         Possible values:
!                         0:    if there is a given stress tensor the reference
!                                  to this tensor is stored in position 19
!                         1:    It is supposed that there are 6 given stress
!                                  tensor components stored in positions 19-24
!     n              i    number of basis functions for each component
!     ndimloc         i    Local dimension of space
!     phi            i    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     stress         i    The contents of stress depend on the parameter
!                         modelrhsd
!                         if  modelrhsd = 0 then
!                         Array containing the three values of the stress tensor
!                         in the element
!                         if  modelrhsd = 1 then
!                         Array containing of size n x 6 containing the six
!                         values of the stress tensor in the n nodes of the
!                         element
!     w              i    array of length m containing the weights for
!                         integration
!     xgauss         i    array of length m x ndim containing the co-ordinates
!                         of the gauss points.  array xgauss is only filled when
!                         gauss integration is applied.
!                         xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                         zg_i = xgauss (i,3)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision dpidxint, dpidyint, strssg(6,mloc), stressloc(3)
      integer i, k, j, ivec, number
      double precision, pointer :: vecoldlc(:)

!     dpidxint   Contains the integral of dphi/dx_i over the element
!     dpidyint   Contains the integral of dphi/dy_i over the element
!     i          Counting variable
!     j          Counting variable
!     k          Counting variable
!     strssg     Stress in Gauss integration points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: inivec
      
!     INSTOP         Stop the program
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if ( modelrhsd.eq.0 ) then

!     --- Modelrhsd = 0, stress is stored as 3 constants per element

      number = 10
      ipos  = ivecnums(number)
      ivec = inivec ( E%iuser(ipos) )
      iseqtype = oldsols(ivec)%iseqtype
      index3n => indoldsols(iseqtype)%index3
      vecoldlc => oldsols(ivec)%oldvecs
      stressloc(1:3) = vecoldlc ( index3n(1:3) )
         do i = 1, nloc
            dpidxint = 0d0
            dpidyint = 0d0
            do  k = 1, mloc
               dpidxint = dpidxint + w(k)*dphidx(i,k,1)
               dpidyint = dpidyint + w(k)*dphidx(i,k,2)
            end do
            elemvclc(1,i) = elemvclc(1,i) - &
                          stressloc(1)*dpidxint-stressloc(2)*dpidyint
            elemvclc(2,i) = elemvclc(2,i) - &
                          stressloc(2)*dpidxint-stressloc(3)*dpidyint
         end do

      else

!     --- Modelrhsd = 1, stress is stored completely

         if ( jcart.eq.1 ) then

            if ( ndimloc.eq.2 ) then

!           --- Interpolate stress vector from nodal points to
!               integration points

               do j = 1, 6
                  do k = 1, mloc
                     strssg(j,k) = 0.d0
                     do i = 1, nloc
                        strssg(j,k) = strssg(j,k) + stress(i,j)*phi(i,k)
                     end do
                  end do
               end do

!              --- For the "definition" of the divergence of the stress tensor, &
!                  see subroutine EL2006

               do i = 1, nloc
                  do k = 1, mloc

!                 --- Fill positions corresponding to x-velocities

                     elemvclc(1,i) = elemvclc(1,i) &
                                   + strssg(1,k)*w(k)*dphidx(i,k,1) &
                                   + strssg(4,k)*w(k)*dphidx(i,k,2)

!                    --- Fill positions corresponding to y-velocities

                     elemvclc(2,i) = elemvclc(2,i) &
                                   + strssg(4,k)*w(k)*dphidx(i,k,1) &
                                   + strssg(2,k)*w(k)*dphidx(i,k,2)
                  end do
               end do

            else if ( ndimloc.eq.3 ) then

!           --- Interpolate stress vector from nodal points to
!               integration points

               do j = 1, 6
                  do k = 1, mloc
                     strssg(j,k) = 0.d0
                     do i = 1, nloc
                        strssg(j,k) = strssg(j,k) + stress(i,j)*phi(i,k)
                     end do
                  end do
               end do

!              --- For the "definition" of the divergence of the stress tensor, &
!                  see subroutine EL2006

               do i = 1, nloc
                  do k = 1, mloc

!                 --- Fill positions corresponding to x-velocities

                     elemvclc(1,i) = elemvclc(1,i) &
                                   + strssg(1,k)*w(k)*dphidx(i,k,1) &
                                   + strssg(4,k)*w(k)*dphidx(i,k,2) &
                                   + strssg(6,k)*w(k)*dphidx(i,k,3)

!                    --- Fill positions corresponding to y-velocities

                     elemvclc(2,i) = elemvclc(2,i) &
                                   + strssg(4,k)*w(k)*dphidx(i,k,1) &
                                   + strssg(2,k)*w(k)*dphidx(i,k,2) &
                                   + strssg(5,k)*w(k)*dphidx(i,k,3)

!                    --- Fill positions corresponding to z-velocities

                     elemvclc(3,i) = elemvclc(3,i) &
                                   + strssg(6,k)*w(k)*dphidx(i,k,1) &
                                   + strssg(5,k)*w(k)*dphidx(i,k,2) &
                                   + strssg(3,k)*w(k)*dphidx(i,k,3)
                  end do
               end do
            else
               write(*,*) 'ERROR: value of ndimloc = ', ndimloc, &
                          'is incorrect.'
               call instop
            end if

         else if ( jcart.eq.2 ) then

!        --- 2D axisymmetric
!            Interpolate stress vector from nodal points to
!            integration points

            do j = 1, 6
               do k = 1, mloc
                  strssg(j,k) = 0.d0
                  do i = 1, nloc
                     strssg(j,k) = strssg(j,k) + stress(i,j)*phi(i,k)
                  end do
               end do
            end do

!           --- For the "definition" of the divergence of the stress tensor, &
!               see subroutine EL2006

            do i = 1, nloc
               do k = 1, mloc

!              --- Fill positions corresponding to r-velocities

                  elemvclc(1,i) = elemvclc(1,i) &
                                + strssg(1,k)*w(k)*dphidx(i,k,1) &
                                + strssg(3,k)*w(k)*phi(i,k)/xgauss(k,1) &
                                + strssg(4,k)*w(k)*dphidx(i,k,2)

!                 --- Fill positions corresponding to z-velocities

                  elemvclc(2,i) = elemvclc(2,i) &
                                + strssg(4,k)*w(k)*dphidx(i,k,1) &
                                + strssg(2,k)*w(k)*dphidx(i,k,2)
               end do
            end do

         end if

      end if

      end
