      subroutine sepel0900cf
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 31-08-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill real coefficients for elements of type 900
!     Navier-Stokes equation, penalty approach
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      use sepmodulecactl
      use sepmodulebasefn
      use sepmoduleelm
      use sepmodulecomio
      use sepmoduleoldvecs
      use sepmodulestrings
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: inirepscal

!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIREPSCAL     Corrects integer value in case of scalars
!     PRININ         print 1d integer array
!     SEPELFILLCOF   Fill all real coefficients in E%coeffs
!     SEPELLOWUPP    Find lower and upper bound of variable coefficients
!                    Fill oldvectors if necessary
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     838   Penalty parameter is not constant
!    1915   Penalty parameter not positive
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepel0900cf' )
      debug = .false. .and. ioutp>=0

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepel0900cf'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      E%indrho = 2
      E%indangvel = E%indrho+1
      E%indsource = E%indangvel+1
      E%indeta = E%indsource+3
      E%indpower = E%indeta+1
      E%indkappa = 11
      E%indlambda = E%indpower+1
      E%indgoertler = 11
      E%indrhscomp = 12
      E%indstress = E%indrhscomp+1
      E%indcdiv = E%indstress+1
      E%indcgrad = E%indcdiv+3
      E%indcconv = E%indcgrad+1
      E%indcvisc = E%indcconv+9

      if ( debug ) then
         write(irefwr,1) 'indrho, indsource, indeta, indpower', &
                          E%indrho, E%indsource, E%indeta, E%indpower
         write(irefwr,1) 'indlambda, indrhscomp, indgoertler', &
                          E%indlambda, E%indrhscomp, E%indgoertler
         write(irefwr,1) 'indrhscomp, indstress,  indcdiv', &
                          E%indrhscomp, E%indstress, E%indcdiv
         write(irefwr,1) 'indcgrad, indcconv, indcvisc, indkappa', &
                          E%indcgrad, E%indcconv, E%indcvisc, E%indkappa
      end if  ! ( debug )

      E%ncoef = 36
      allocate ( E%coeffs(m,E%ncoef), E%ind(E%ncoef), E%ist(E%ncoef), &
                 E%mst(E%ncoef), E%ivecnums(E%ncoef), E%ich(E%ncoef), &
                 E%coeffsnode(n,E%ncoef), E%coeffcp(m,E%ncoef), &
                 stat = error )
      if ( error/=0 ) call eralloc ( error, m*E%ncoef, 'coeffs' )
      if ( ierror/=0 ) go to 1000

      E%ich = 0  ! clear ich
      E%ind = 0  ! clear ind
      E%coeffs = 0

      E%ncoef = 9     ! First 9 coefficients
      call sepelfillcof ( 1 )

!     --- Check penalty parameter

      if ( E%ind(1)>0 ) then
         call errint ( itype, 1 )
         call errint ( 6, 2 )
         call errint ( ielgrp, 3 )
         call errint ( ielem, 4 )
         call errsub ( 838, 4, 0, 0 )
      end if
      E%penalp = E%coeffs(1,1)
      if ( itype==900 .and. E%penalp<=0d0 ) then
         call erreal ( E%penalp, 1 )
         call errint ( itype, 1 )
         call errint ( 6, 2 )
         call errint ( ielgrp, 3 )
         call errint ( ielem, 4 )
         call errsub ( 1915, 4, 1, 0 )
      end if

      if ( E%ind(E%indrho)==0 ) then

!     --- rho = 0, make it equal to 1

         E%coeffs(:,E%indrho) = 1d0
         E%ind(E%indrho) = -1

      end if  ! ( E%ind(E%indrho)==0 )

!     --- Mesh velocity

      E%seq_meshvel = inirepscal ( E%iuser(E%ipos) )
      if ( E%seq_meshvel>0 ) indvector(E%seq_meshvel) = 1
      E%ipos = E%ipos+1
      E%ishift = E%ishift+1

      E%ncoef = 36     ! last 26 coefficients
      call sepelfillcof ( 10 )
      do i = 1, E%ncoef
         E%coeffsnode(1:n,i) = E%coeffs(1,i)
      end do  ! i = 1, E%ncoef
      E%coeffcp = E%coeffs

      if ( maxval(E%ind(1:numold))>=10000 ) then

!     --- All vectors are used potentially

         indvector(1:numold) = 1

      else if ( E%typeconvect/=0 .or. E%modelvisc>=100 .or. &
                E%modelvisc>1 .and. E%modelvisc<5 .or. E%typetime>0 ) then

!     --- Navier-Stokes equation or non-newtonian fluid, use old velocity

         indvector(iseqprevsol) = 1
         E%filloldvec = .true.
         E%compold = 1
         if ( jtrans==0 ) E%filloldderiv = .true.
         if ( E%seq_second_old_sol>0 ) indvector(E%seq_second_old_sol) = 1
         if ( E%modelvisc==103 ) indvector(1:numold) = 1

      end if  ! ( E%typeconvect/=0 )

!     --- Find lower and upper bound for space dependent coefficients
!         Fill old vectors in locvecs if necessary

      call sepellowupp

      E%rho => E%coeffs(:,E%indrho)
      E%sourceterm = E%ind(E%indsource)/=0 .or. &
                     E%ind(E%indsource+1)/=0 .or. &
                     E%ind(E%indsource+2)/=0
      if ( E%ind(E%indrhscomp)/=0 ) E%mdiv = 1
      if ( E%filloldvec ) then
         E%ugauss => oldsols(iseqprevsol)%locvecs(1:m,1:E%maxunk)
         uprev => oldsols(iseqprevsol)%locvecsnode(1:n,1:E%maxunk)
      end if  ! ( E%filloldvec )

!     --- Check if element vector is completely zero

      if ( .not. E%sourceterm .and. E%typeconvect==0 .and. &
           E%ind(E%indstress)==0 .and. E%modelvisc<=1 .and. &
           E%mdiv==0 ) E%notvec = 1

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'isupcfvar, isubcfvar', E%isupcfvar, E%isubcfvar
         call prinin ( E%ind, E%ncoef, 'ind' )
         write(irefwr,*) 'End sepel0900cf'

      end if  ! ( debug )
      call erclos ( 'sepel0900cf' )

      end subroutine sepel0900cf
