      subroutine sepel900convct
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-09-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute the convective part of the element matrix for navier-stokes
!     and store in matrix s for 2-d and 3-d elements
!     Old subroutine el2010
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     navier_stokes_equation
!     convection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleoldvecs
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j, k
      double precision :: sum1, rhow(m), rhowr(m), rhowu(m), rhowv(m), &
                          work(m*ndimlc), submat(n,n), wvel(m,E%ndim)
      double precision, pointer :: dudx(:,:,:), s(:,:), u(:,:), phi(:,:)
      double precision, pointer :: dphidx(:,:,:)

!     dphidx         derivatives of the basis functions
!     dudx           derivatives of the solution vector in the quadrature
!                    point
!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     phi            values of shape functions in quadrature points
!     rhow           Contains rho w
!     rhowr          Contains rho w /r
!     rhowu          Contains u rho w /r
!     rhowv          Contains v rho w /r
!     s              sub matrix
!     submat         sub matrix
!     sum1           Help variable to compute a sum
!     u              velocity
!     work           Work array
!     wvel           rho * velocity
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
!     SEPEL900CONV1  Fill part of convection tensor for Navier-Stokes into
!                    matrix S
!     SEPELMASSMAT4  Fill the element mass matrix in case of one unknown per
!                    node. Consistent mass matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Computation of the convective matrix results in a 3x3 block matrix, each
!     consisting of nxn matrices of the following shape:
!            |   S       S      S     |
!            |    uu      uv     uw   |
!            |                        |
!            |   S       S      S     |
!     S =    |    vu      vv     vw   |
!            |                        |
!            |   S       S      S     |
!            |    wu      wv     ww   |
!     The submatrices depend on the parameter mconv.
!     The non-linear term has the shape:
!            | u ux     v uy      w uz |
!            |                         |
!       rho  | u vx     v vy      w vz | = rho u . grad u
!            |                         |
!            | u wx     v wy      w wz |
!     If ( mconv=1 ) then
!      rho u . grad u (n+1)  =  rho u(n) . grad u (n+1)
!     If ( mconv=2 ) then
!      rho u . grad u (n+1)  =  rho u(n) . grad u (n+1) + &
!                               rho u(n+1) . grad u (n) - &
!                               rho u(n) . grad u (n)
!     If ( mconv=3 ) then
!      rho u . grad u (n+1)  =  rho u(n+1) . grad u (n)
!     The submatrices rho u(n) . grad u (n+1) are defined by:
!     S uu =  rho u(n) . gradphi(j) phi(i) = Svv = Sww
!     The submatrices rho u(n+1) . grad u (n) are defined by:
!     S uu =  rho ux(n) phi(j) phi(i)
!     S uv =  rho uy(n) phi(j) phi(i)
!     S uw =  rho uz(n) phi(j) phi(i)
!     S vu =  rho vx(n) phi(j) phi(i)
!     S vv =  rho vy(n) phi(j) phi(i)
!     S vw =  rho vz(n) phi(j) phi(i)
!     S wu =  rho wx(n) phi(j) phi(i)
!     S wv =  rho wy(n) phi(j) phi(i)
!     S ww =  rho wz(n) phi(j) phi(i)
!    In case of cylinder co-ordinates the convective terms become
!    ur dur/dr   + uphi/r dur/dphi   + uz dur/dz   - uphi**2/r
!    ur duphi/dr + uphi/r duphi/dphi + uz duphi/dz + ur uphi/r
!    ur duz/dr   + uphi/r duz/dphi   + uz duz/dz
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepel900convct' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepel900convct'
         write(irefwr,1) 'n, m, jcart, mconv, E%ndim, metupw', &
                          n, m, B%cart, E%typeconvect, E%ndim, E%methodupw
         call prinrl ( E%coeffs(:,E%indrho), m, 'rho' )
         call prinrl1 ( E%submat, E%ndim*n, E%ndim*n, 'submat/1' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      dudx => gradoldvec
      u => E%ugaussloc

!     --- multiply the weight by rho and store in rhow
!         in case of cylinder co-ordinates, store w rho/r in rhowr

      rhow = E%rho * B%w
      if ( B%cart>2 ) rhowr = rhow/B%xyzg(:,1)

      if ( E%typeconvect/=1 ) then

!     --- mconv = 2 or 3, successive substitution part, i.e.
!         rho u(n+1) . grad u (n)

         if ( B%cart<=2 ) then

!        --- jcart = 1, Cartesian co-ordinates
!            or jcart = 2, Axi-symmetrical co-ordinates without swirl

            do i = 1, E%ndim
               do j = 1, E%ndim
                  work(1:m) = dudx(:,j,i)*rhow  ! dudx*rho*w
                  call sepelmassmat4 ( work, submat ) ! work *psi_i*phi_j

!                 --- Add submat to element matrix

                  s => E%submat(j:n*E%ndim:E%ndim,i:n*E%ndim:E%ndim)
                  s = s+submat

               end do  ! j = 1, E%ndim
            end do  ! i = 1, E%ndim
            if ( debug ) &
               call prinrl1 ( E%submat, E%ndim*n, E%ndim*n, 'submat/2' )

         else if ( B%cart==3 ) then

!        --- jcart = 3, Polar co-ordinates

            do i = 1, 2
               work(1:m) = dudx(:,1,i)*rhow  ! dudx*rho*w
               call sepelmassmat4 ( work, submat ) ! work *psi_i*phi_j
               s => E%submat(1:n*E%ndim:E%ndim,i:n*E%ndim:E%ndim)
               s = s+submat
               if ( i==2 ) then
                  work(1:m) = u(:,1)*rhowr  ! u*rho*w
                  call sepelmassmat4 ( work, submat ) ! work *psi_i*phi_j
                  s => E%submat(1:n*E%ndim:E%ndim,i:n*E%ndim:E%ndim)
                  s = s+submat
               end if  ! ( i==2 )
            end do  ! i = 1, 2

            do i = 1, 2
               work(1:m) = dudx(:,2,i)*rhow  ! dudx*rho*w
               call sepelmassmat4 ( work, submat ) ! work *psi_i*phi_j
               s => E%submat(2:n*E%ndim:E%ndim,i:n*E%ndim:E%ndim)
               s = s+submat
               if ( i==1 ) then
                  work(1:m) = u(:,2)*rhowr  ! u*rho*w
                  call sepelmassmat4 ( work, submat ) ! work *psi_i*phi_j
                  s => E%submat(2:n*E%ndim:E%ndim,i:n*E%ndim:E%ndim)
                  s = s+submat
               end if  ! ( i==2 )
            end do  ! i = 1, 2

         end if  ! ( B%cart<=2 )

      end if  ! ( E%typeconvect/=1 )

      if ( E%typeconvect<3 ) then

!     --- mconv = 1 or 2, Picard part, i.e.
!         rho u(n) . grad u (n+1)

         if ( B%cart<=2 ) then

!        --- jcart = 1, Cartesian co-ordinates
!            or jcart = 2, Axi-symmetrical co-ordinates without swirl

            do i = 1, E%ndim
               wvel(:,i) = rhow(:)*u(:,i)
            end do  ! i = 1, E%ndim
            call sepel900conv1 ( wvel, submat )

            do j = 1, E%ndim

               s => E%submat(j:n*E%ndim:E%ndim,j:n*E%ndim:E%ndim)
               s = s+submat

            end do  ! j = 1, E%ndim

         else if ( B%cart==3 ) then

!        --- jcart = 3, Polar co-ordinates

            dphidx => B%phider
            if ( E%methodupw==0 ) then
               phi => B%phi
            else
               phi => B%psi
            end if  ! ( E%methodupw==0 )

            rhowu(:) = rhow(:)*u(:,1)
            rhowv(:) = rhow(:)*u(:,2)

            do i = 1, n
               do j = 1, n
                  submat(j,i) = sum((rhowu * dphidx(j,:,1) + &
                                     rhowv * dphidx(j,:,2))*phi(i,:))
               end do
            end do

            do j = 1, 2

               s => E%submat(j:n*2:2,j:n*2:2)
               s = s+submat

            end do  ! j = 1, 2
            do i = 1, n
               do j = 1, i
                  submat(j,i) = sum(rhowv * phi(j,:) * phi(i,:))
                  submat(i,j) = submat(j,i)
               end do
            end do
            s => E%submat(1:n*E%ndim:E%ndim,2:n*E%ndim:E%ndim)
            s = s+submat
            s => E%submat(2:n*E%ndim:E%ndim,1:n*E%ndim:E%ndim)
            s = s+submat

         end if  ! ( B%cart<=2 )

      end if  ! ( E%typeconvect<3 )

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( E%submat, E%ndim*n, E%ndim*n, 'submat/3' )
         write(irefwr,*) 'End sepel900convct'

      end if  ! ( debug )
      call erclos ( 'sepel900convct' )

      end subroutine sepel900convct
