      subroutine sepel900rhsg ( acof )
! ======================================================================
!
!        programmer    Guus Segal/ Ralf Biertuempfel
!        version  1.0  date 12-08-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute the convective part of the element vector for navier-stokes
!     and add in consisting element vector for a 3-d element
!     Old subroutine el2011g
! **********************************************************************
!
!                       KEYWORDS
!
!     element_vector
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleoldvecs
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecactl
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision acof

!     acof      i    Multiplication factor Adams-Bashfort
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision rhow(m), uconv(m), work(m), elvcsub(n)
      integer i, j, k
      double precision, pointer :: u(:,:), phi(:,:), dphidx(:,:,:), &
                                   dudx(:,:,:), xg(:), vec(:)

!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     rhow           Contains E%rho w
!     uconv          Work array to store coefficient (convection)
!     work           Work array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL3003  Compute integrals of the shape / f(x) phi (x) dx
!                                            e         i
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Computation of the convective vector results in a 3x1 block vector, each
!     consisting of nx1 vectors of the following shape:
!            |   F    |
!            |    u   |
!            |        |
!            |   F    |
!     F =    |    v   |
!            |        |
!            |   f    |
!            |    w   |
!    The sub vectors are defined by:
!    Fu(i) = E%rho ( u du/dx + v du/dy + w du/dz ) phi (i)
!    Fv(i) = E%rho ( u dv/dx + v dv/dy + w dv/dz  + u u) phi (i)
!    Fw(i) = E%rho ( u dw/dx + v dw/dy + w dw/dz ) phi (i)
!    For the expression in cylinder co-ordinates see subroutine el2010
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepel900rhsg' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepel900rhsg'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      u => E%ugauss
      dudx => gradoldvec
      dphidx => B%phider

!     --- multiply the weight by E%rho and store in work(k,1)

      rhow = B%w * E%rho * acof

      if ( B%cart<=2 ) then

!     --- B%cart = 1, Cartesian co-ordinates
!         or B%cart = 2, Axi-symmetrical co-ordinates without swirl

         do i = 1, E%ndim

!        --- Put u dui/dx + v dui/dy in uconv

            uconv(:) = u(:,1) * dudx(:,1,i) &
                       + u(:,2) * dudx(:,2,i)

            if ( E%ndim==3 ) then

!           --- Put third part in work(k,2)

               uconv(:) = uconv(:) + u(:,3) * dudx(:,3,i)
               if (i==2) &
                  uconv(:) = uconv(:) - E%coeffs(:,E%indgoertler) *  &
                             E%coeffs(:,E%indkappa) * (u(:,1)**2)
            end if  ! ( E%ndim==3 )

!           --- Compute contribution to one element vector and store in elvcsub

            work = uconv*rhow
            elvcsub = 0d0
            call sepelload2 ( work, B%jdiag, B%phi, elvcsub )
!            call el3003 ( uconv, elvcsub, rhow, B%phi, m, n )

!           --- Add contribution to element vector

            vec => E%subvec(i:E%ndim*n:E%ndim)
            vec = vec+elvcsub

         end do  ! i = 1, E%ndim

      else

!     --- B%cart = 3, Polar co-ordinates
!         Put u du/dr + v/r du/dy - v**2/r in work(k,2)

         uconv(:) = u(:,1)*dudx(:,1,1) + &
                    (u(:,2)*dudx(:,2,1)-u(:,2)**2)/B%xyzg(:,1)

!        --- Compute contribution to one element vector and store in elvcsub

         work = uconv*rhow
         elvcsub = 0d0
         call sepelload2 ( work, B%jdiag, B%phi, elvcsub )
  !       call el3003 ( uconv, elvcsub, work, B%phi, m, n )

!        --- Add contribution to element vector

         vec => E%subvec(1:E%ndim*n:E%ndim)
         vec = vec+elvcsub

!        --- Put u dv/dr + v/r dv/dy + u*v/r in work(k,2)

         do k = 1, m
            uconv(k) = u(k,1)*dudx(k,1,2) + &
                        (u(k,2)*dudx(k,2,2)+u(k,1)*u(k,2))/B%xyzg(k,1)
         end do

!        --- Compute contribution to one element vector and store in elvcsub

         work = uconv*rhow
         elvcsub = 0d0
         call sepelload2 ( work, B%jdiag, B%phi, elvcsub )
!         call el3003 ( uconv, elvcsub, work, B%phi, m, n )

!        --- Add contribution to element vector

         vec => E%subvec(2:E%ndim*n:E%ndim)
         vec = vec+elvcsub

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepel900rhsg'

      end if  ! ( debug )
      call erclos ( 'sepel900rhsg' )

      end subroutine sepel900rhsg
