      subroutine sepel900rhssp ( acof )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-08-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute the convective part of the element vector for navier-stokes
!     and add in consisting element vector for a 3-d element
!     In this case the special form of the Navier-Stokes equations is used
!     based on the given coefficients for the various terms
!     Old subroutine el2011sp
! **********************************************************************
!
!                       KEYWORDS
!
!     element_vector
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleoldvecs
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecactl
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision acof

!     acof    i    Multiplication factor Adams-Bashfort
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision coeff(m), elvcsub(n)
      integer i, j, k
      double precision, pointer :: u(:,:), dudx(:,:,:), rho(:,:), vec(:)

!     coeff          Work array to store coefficient
!     dudx           derivatives of the solution vector in the quadrature
!                    point
!     elvcsub        sub element vector
!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     u              velcoity
!     vec            sub element vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL3003  Compute integrals of the shape / f(x) phi (x) dx
!                                            e         i
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Computation of the convective vector results in a 3x1 block vector, each
!     consisting of nx1 vectors of the following shape:
!            |   F    |
!            |    u   |
!            |        |
!            |   F    |
!     F =    |    v   |
!            |        |
!            |   f    |
!            |    w   |
!    The sub vectors are defined by:
!    Fu(i) = rho ( u du/dx + v du/dy + w du/dz ) phi (i)
!    Fv(i) = rho ( u dv/dx + v dv/dy + w dv/dz ) phi (i)
!    Fw(i) = rho ( u dw/dx + v dw/dy + w dw/dz ) phi (i)
!    For the expression in cylinder co-ordinates see subroutine el2010
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepel900rhssp' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepel900rhssp'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      u => E%ugauss
      dudx => gradoldvec
      rho => E%coeffs(:,11:10+E%ndim**2)

      if ( B%cart<=2 ) then

!     --- B%cart = 1, Cartesian co-ordinates
!         or B%cart = 2, Axi-symmetrical co-ordinates without swirl

         do i = 1, E%ndim

!        --- Put u dui/dx + v dui/dy + w dui/dz in coeff

            coeff(:) = rho(:,(i-1)*E%ndim+1) * u(:,1) * dudx(:,1,i) + &
                       rho(:,(i-1)*E%ndim+2) * u(:,2) * dudx(:,2,i)

            if ( E%ndim==3 ) then

!           --- Put third part in coeff

               coeff(:) = coeff(:) + rho(:,(i-1)*E%ndim+3)*u(:,3)*dudx(:,3,i)

            end if

!           --- Compute contribution to one element vector and store in elvcsub

            coeff = acof*coeff*B%w
            call sepelload2 ( coeff, B%jdiag, B%phi, elvcsub )
!            el3003 ( coeff, elvcsub, w, phi, m, n )

!           --- Add contribution to element vector

            vec => E%subvec(i:E%ndim*n:E%ndim)
            vec = vec+elvcsub

         end do

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepel900rhssp'

      end if  ! ( debug )
      call erclos ( 'sepel900rhssp' )

      end subroutine sepel900rhssp
