      subroutine sepel900st27 ( s, signdiv )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-08-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Put stiffness matrix and part due to gradient and divergence
!    into element matrix for 27-point Navier-Stokes equation
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebasefn
      use sepmoduleelm
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision s(3*n,3*n)
      integer signdiv

!     s              i    Element matrix without the gradient and divergence
!                         part
!     signdiv        i    Multiplication sign of "continuity equations"
!                         if -1 the stokes matrix is symmetric
!                         if 1 all eigenvalues are in the positive half plane
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer j, icent, npres, isigndiv
      double precision diag(4)

!     diag           Diagonal of the pressure mass matrix
!     icent          Index of centroid
!     isigndiv       signdiv except for mcont = 5, then 1
!     j              Counting variable
!     npres          Number of pressure parameters per element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    The stiffness matrix has the following structure
!     Saa Sab Sac       where a refers to the nodes 1 to 13
!     Sba Sbb Sbc             b refers to node 14
!     Sca Scb Scc             c refers to the nodes 15 to 27
!    The element matrix gets the form:
!     Saa Sab Ga Sac     with G the gradient matrix
!     Sba Sbb Gb Sbc     D the divergence matrix
!     Da  Db  P  Dc      and P the pressure matrix
!     Sca Scb Gc Scc
!   Remember matrices are stored in transposed form
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- First put Stiffness matrix in correct positions

      E%mat(1:3*14,1:3*14) = s(1:3*14,1:3*14)
      E%mat(3*14+5:3*27+4,3*14+5:3*27+4) = s(3*14+1:3*27,3*14+1:3*27)
      E%mat(1:3*14,3*14+5:3*27+4) = s(1:3*14,14*3+1:27*3)
      E%mat(3*14+5:3*27+4,1:14*3) = s(3*14+1:3*27,1:14*3)

!     --- Next put gradient and divergence matrix in the correct positions

      E%mat(1:14*3,14*3+1:14*3+4) = signdiv*E%divmat(1:14*3,1:4)
      E%mat(14*3+5:27*3+4,14*3+1:14*3+4) = signdiv*E%divmat(14*3+1:27*3,1:4)

!     --- gradient matrix

      do j = 1, 4
         E%mat(14*3+j,1:14*3) = -E%gradmat(1:14*3,j)
         E%mat(14*3+j,14*3+5:27*3+4) = -E%gradmat(14*3+1:27*3,j)
      end do  ! j = 1, 4
      E%mat(14*3+1:14*3+4,14*3+1:14*3+4) = 0d0

!     --- Next put pressure matrix in the correct positions

      if ( E%typecontinuity==4 .or. E%typecontinuity==5 ) then

!     --- E%typecontinuity = 4 or 5, add pressure mass matrix

         isigndiv = signdiv
         if ( E%typecontinuity==5 ) isigndiv = 1

         E%mat(14*3+1:14*3+4,14*3+1:14*3+4) = isigndiv*E%pressmat(1:4,1:4)

      end if

!     --- Add penalty parameter to diagonal of pressure part

      if ( E%penalp>0d0 ) then

!     --- E%penalp>0, there is a penalty contribution

         icent = 14
         npres = 4

         diag(1) = sum(B%w(:))

         do j = 1, npres-1
            diag(j+1) = sum(B%w(:)*(B%xyzg(:,j)-B%xyz(icent,j))**2)
         end do

!        --- store the diagonal multiplied by E%penalp in diag

         diag = diag * E%penalp

         do j = 1, npres
            E%mat(14*3+j,14*3+j) =  E%mat(14*3+j,14*3+j) + signdiv*diag(j)
         end do

      end if  ! ( E%penalp>0d0 )

      end subroutine sepel900st27
