      subroutine sepel900visc3d
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-09-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill stress tensor for Navier-Stokes into matrix S
!     Cartesian coordinates (3d)
!     Old subroutine sepelnsstrc3d
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j
      double precision :: dmult(2), work(m), sw(n,n)
      double precision, pointer :: S(:,:), phix(:,:), phiy(:,:), phiz(:,:)

!     S              sub matrix
!     dmult          Array containing multiplication factors
!     i              Counting variable
!     j              Counting variable
!     phix           dphi/dx
!     phiy           dphi/dy
!     phiz           dphi/dz
!     sw             sub matrix
!     work           Work array to store w eta
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Computation of the stiffness matrix results in a 3 x 3 block matrix, each
!     consisting of nxn matrices of the following shape:
!            |   S       S      S     |
!            |    uu      uv     uw   |
!            |                        |
!            |   S       S      S     |
!     S =    |    vu      vv     vw   |
!            |                        |
!            |   S       S      S     |
!            |    wu      wv     ww   |
!     The submatrices are defined by:
!              T
!     S    =  S
!      xy      yx
!     Cartesian co-ordinates:
!     S uu =  2  eta phix phix + eta phiy phiy + eta phiz phiz
!     S uv =     eta phix(j) phiy(i)
!     S uw =     eta phix(j) phiz(i)
!     S vu =     eta phiy(j) phix(i)
!     S vv =  2  eta phiy phiy + eta phix phix + eta phiz phiz
!     S vw =     eta phiy(j) phiz(i)
!     S wu =     eta phiz(j) phix(i)
!     S wv =     eta phiz(j) phiy(i)
!     S ww =  2  eta phiz phiz + eta phix phix + eta phiy phiy
!     In case of E%modelvisc = 10:
!     S uu =  eta phix phix + eta phiy phiy + eta phiz phiz = S vv = S ww
!     Rest is zero
!     2D Cylinder co-ordinates (r, z) without swirl
!     S uu =  2 eta phir phir + eta phiz phiz + 2 eta phi phi /r**2
!     S uv =     eta phir(j) phiz(i)
!     S vu =     eta phiz(j) phir(i)
!     S vv =  2  eta phiz phiz + eta phir phir
!    Goertler equations:
!     S uu =   eta phix phix  +    eta phiy phiy  +    eta phiz phiz
!     S vv =   eta phix phix  +  2 eta phiy phiy  +    eta phiz phiz
!     S ww =   eta phix phix  +    eta phiy phiy  +  2 eta phiz phiz
!     The non-diagonal block matrices are the same as in the 3D cartesian.
!
!    In case of compressible flow the stress term is changed from
!
!    T = eta ( grad v + grad v^T ) into
!
!    T = eta ( grad v + grad v^T ) - 2/3 eta div v I
!
!    with v the velocity
!
!    This means in the Cartesian case that the following extra terms are added:
!
!             |   phi_jx phi_ix   phi_jy phi_ix  phi_jz phi_ix  |
!             |                                                 |
!   -2/3 eta |   phi_jx phi_iy   phi_jy phi_iy  phi_jz phi_iy  |
!             |                                                 |
!             |   phi_jx phi_iz   phi_jy phi_iz  phi_jz phi_iz  |
!             |                                                 |
!    The diagonal terms can be treated by multiplying by 4/3 (=2-2/3)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepel900visc3d' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepel900visc3d'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      phix => B%phider(:,:,1)
      phiy => B%phider(:,:,2)
      phiz => B%phider(:,:,3)
      s => E%submat

      if ( E%modelvisc==10 ) then

!     --- Pure incompressible flow, apply divergence freedom
!         simplified expression

         dmult(1) = 1d0

      else

!     --- Pure incompressible flow, apply divergence freedom

         dmult(1) = 2d0

      end if  ! ( E%modelvisc==10 )
      if ( E%typecontinuity==1 .or. E%typecontinuity==4 ) then

!     --- compressible flow

         dmult(1) = 4d0/3d0
         dmult(2) = -0.5d0*dmult(1)

      end if  ! ( E%typecontinuity==1 .or. E%typecontinuity==4 )

      if ( debug ) call prinrl ( dmult, 2, 'dmult' )

!     --- 3D Cartesian co-ordinates
!         eta phix phix

      work = B%w*E%etaeff

      do i = 1, n
         do j = 1, i
            sw(i,j) = sum(work*phix(i,:)*phix(j,:))
         end do
      end do

      if ( E%typecontinuity==2 ) then

!     --- modified Navier-Stokes equation for Goertler problem

         do i = 1, n
            do j = 1, i
               s(3*i-2,3*j-2) = sw(i,j)
            end do
         end do

      else

!     --- standard Navier-Stokes equations
!         Add to S_uu, S_vv and S_ww

         do i = 1, n
            do j = 1, i
               s(3*i-2,3*j-2) = sw(i,j)*dmult(1)
               s(3*i-1,3*j-1) = sw(i,j)
               s(3*i,3*j) = sw(i,j)
            end do
         end do

      end if

!     --- eta phiy phiy

      do i = 1, n
         do j = 1, i
            sw(i,j) = sum(work*phiy(i,:)*phiy(j,:))
         end do
      end do

!     --- Add to S_uu, S_vv and S_ww

      do i = 1, n
         do j = 1, i
            s(3*i-2,3*j-2) = s(3*i-2,3*j-2)+sw(i,j)
            s(3*i-1,3*j-1) = s(3*i-1,3*j-1)+sw(i,j)*dmult(1)
            s(3*i,3*j) = s(3*i,3*j)+sw(i,j)
         end do
      end do

!     --- eta phiz phiz

      do i = 1, n
         do j = 1, i
            sw(i,j) = sum(work*phiz(i,:)*phiz(j,:))
         end do
      end do

!     --- Add to S_uu, S_vv and S_ww

      do i = 1, n
         do j = 1, i
            s(3*i-2,3*j-2) = s(3*i-2,3*j-2)+sw(i,j)
            s(3*i-1,3*j-1) = s(3*i-1,3*j-1)+sw(i,j)
            s(3*i,3*j) = s(3*i,3*j)+sw(i,j)*dmult(1)
         end do
      end do

      if ( E%modelvisc/=10 ) then

!     --- S uv =  eta phix phiy

         do i = 1, n
            do j = 1, n
               sw(j,i) = sum(work*phix(i,:)*phiy(j,:))
            end do
         end do

!        --- Put in S_uv

         do i = 1, n
            do j = 1, i
               s(3*i-1,3*j-2) = sw(j,i)
               s(3*i-2,3*j-1) = sw(i,j)
            end do
         end do

!        --- S uw =  eta phix phiz

         do i = 1, n
            do j = 1, n
               sw(j,i) = sum(work*phix(i,:)*phiz(j,:))
            end do
         end do

         do i = 1, n
            do j = 1, i
               s(3*i,3*j-2) = sw(j,i)
               s(3*i-2,3*j) = sw(i,j)
            end do
         end do

!        --- S vw =  eta phiy phiz

         do i = 1, n
            do j = 1, n
               sw(j,i) = sum(work*phiy(i,:)*phiz(j,:))
            end do
         end do

         do i = 1, n
            do j = 1, i
               s(3*i,3*j-1) = sw(j,i)
               s(3*i-1,3*j) = sw(i,j)
            end do
         end do

         if ( E%typecontinuity==1 .or. E%typecontinuity==4 ) then

!        --- compressible flow -2/3 eta phi_ix phi_jy

            do i = 1, n
               do j = 1, n
                  sw(j,i) = sum(work*phiy(i,:)*phix(j,:))
               end do
            end do

            do i = 1, n
               do j = 1, i
                  s(3*i-1,3*j-2) = s(3*i-1,3*j-2)+sw(j,i)*dmult(2)
                  s(3*i-2,3*j-1) = s(3*i-2,3*j-1)+sw(i,j)*dmult(2)
               end do
            end do

!           --- S uw = -2/3 eta phix phiz

            do i = 1, n
               do j = 1, n
                  sw(j,i) = sum(work*phiz(i,:)*phix(j,:))
               end do
            end do

            do i = 1, n
               do j = 1, i
                  s(3*i,3*j-2) = s(3*i,3*j-2)+sw(j,i)*dmult(2)
                  s(3*i-2,3*j) = s(3*i-2,3*j)+sw(i,j)*dmult(2)
               end do
            end do

!           --- S vw = -2/3 eta phiy phiz

            do i = 1, n
               do j = 1, n
                  sw(j,i) = sum(work*phiz(i,:)*phiy(j,:))
               end do
            end do

            do i = 1, n
               do j = 1, i
                  s(3*i,3*j-1) = s(3*i,3*j-1)+sw(j,i)*dmult(2)
                  s(3*i-1,3*j) = s(3*i-1,3*j)+sw(i,j)*dmult(2)
               end do
            end do

         end if  ! ( E%typecontinuity==1 .or. E%typecontinuity==4 )

      end if  ! ( E%modelvisc/=10 )

!     --- Fill upper half of matrix

      do i = 1, 3*n
         do j = 1, i
            s(j,i) = s(i,j)
         end do  ! j = 1, i
      end do  ! i = 1, 3*n

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepel900visc3d'

      end if  ! ( debug )
      call erclos ( 'sepel900visc3d' )

      end subroutine sepel900visc3d
