      subroutine sepeldivgrad03
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-09-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute the divergence and the pressure gradient matrix for the
!     incompressible Navier-Stokes equations
!     The divergence matrix is stored in array DIV
!     The transpose of the pressure gradient matrix in SP
!     mcont = 3, i.e. special divergence and gradient matrix
!     Old Subroutine el1000sp
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j, k
      double precision :: work(m), work2(m)

!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     work           Work array
!     work2          Work array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2248   coordinate system not yet available
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The gradient matrix is given by:
!     SP = |  SPu  SPv  SPw  |
!     with:
!     SPu(i,k) = / c_grad psi(i) dphidx(k) dx
!                x
!     SPv(i,k) = / c_grad psi(i) dphidy(k) dx
!                x
!     SPw(i,k) = / c_grad psi(i) dphidz(k) dx
!                x
!     The divergence matrix is defined by:
!     DIVu(i,k) = / c_div(1) psi(i) dphidx(k) dx
!                x
!     DIv(i,k) = / c_div(2) psi(i) dphidy(k) dx
!                x
!     DIw(i,k) = / c_div(3) psi(i) dphidz(k) dx
!                x
!     For a linear approximation of the pressure (np=4), psi is given by
!     psi(1)  = 1
!     psi(2)  = x - x14
!     psi(3)  = y - y14
!     psi(4)  = z - z14
!     The matrix SP is organized in the sequence of the velocity vector
!     i.e. u1, v1, w1, u2, v2 ....
!     In case of axi-symmetric co-ordinates the matrix is given by:
!     SPu(i,k) = / psi(i) (dphidx(k) + phi(k)/x) dx
!                x
!     SPv(i,k) = / psi(i) dphidy(k) dx
!                x
!     In case of polar co-ordinates the matrix is given by:
!     SPu(i,k) = / psi(i) (dphidx(k) + phi(k)/x) dx
!                x
!     SPv(i,k) = / psi(i) dphidy(k)/x dx
!                x
!     The right-hand side is defined by:
!     E%rhsdiv(i) = / psi(i) f dx
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepeldivgrad03' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepeldivgrad03'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      select case ( B%cart )

      case(1)

!     --- Cartesian co-ordinates
!         psi = 1

         do i = 1, E%ndim
            work = B%w(:)*E%coeffs(:,E%indcgrad)
            do j = 1, n
               E%gradmat(i+(j-1)*E%ndim,1) = sum ( work*B%phider(j,:,i) )
            end do  ! j = 1, n

         end do  ! i = 1, E%ndim

!        --- fill the divergence matrix

         do i = 1, E%ndim
            work = B%w(:)*E%coeffs(:,E%indcdiv+i-1)
            do j = 1, n
               E%divmat(i+(j-1)*E%ndim,1) = sum ( work*B%phider(j,:,i) )
            end do  ! j = 1, n
         end do

         if ( E%mdiv/=0 ) then

!        --- mdiv#0, divergence right-hand side

            E%rhsdiv(1) = sum(B%w*E%coeffs(:,E%indrhscomp))

         end if  ! ( E%mdiv/=0 )

         if ( npsi>1 ) then

!        --- psi = x  - xcent
!                   j        j

            do j = 1, E%ndim
               work2 = B%w(:)*B%phipress(j+1,:)
               work = work2*E%coeffs(:,E%indcgrad)
               do i = 1, E%ndim
                  do k = 1, n
                     E%gradmat(i+(k-1)*E%ndim,j+1) = sum ( work*B%phider(k,:,i) )
                  end do  ! k = 1, n
               end do  ! i = 1, E%ndim

!              --- fill divergence

               work = work2*E%coeffs(:,E%indcdiv+j-1)
               do i = 1, E%ndim
                  do k = 1, n
                     E%divmat(i+(k-1)*E%ndim,j+1) = sum ( work*B%phider(k,:,i) )
                  end do  ! k = 1, n
               end do  ! i = 1, E%ndim

               if ( E%mdiv/=0 ) then

!              --- mdiv = 1, divergence right-hand side

                  E%rhsdiv(j+1) = sum(B%w*E%coeffs(:,E%indrhscomp)*work)

               end if  ! ( E%mdiv/=0 )

            end do

         end if

      case default

!     --- error 2248: jcart # 1, not yet implemented

         call eropen ( 'sepeldivgrad03' )
         call errint ( B%cart, 1 )
         call errint ( 1, 2 )
         call errint ( 1, 3 )
         call errint ( ielem, 4 )
         call errint ( itype, 5 )
         call errint ( ielgrp, 6 )
         call errsub ( 2248, 6, 0, 0 )
         call erclos ( 'sepeldivgrad03' )

      end select  ! case ( B%cart )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepeldivgrad03'

      end if  ! ( debug )
      call erclos ( 'sepeldivgrad03' )

      end subroutine sepeldivgrad03
