      subroutine sepelfilldiv
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-09-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill div matrix, in case of slightly compressible flow
!     with respect to Crouzeix Raviart elements
!     Old subroutine elfilldiv
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     navier_stokes_equation
!     divergence
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j, k, jdim
      double precision :: gradrho(m)
      double precision, pointer :: rho(:), rhonodes(:), phi(:,:), &
                                   gradphi(:,:,:), div(:,:), xgauss(:,:), &
                                   w(:), phipress(:,:)

!     div            Divergence matrix
!     gradphi        Contains the gradient of phi
!     gradrho        contains gradient of rho
!     i              Counting variable
!     j              Counting variable
!     jdim           Loop variable over dimensions
!     k              Counting variable
!     phi            values of shape functions in quadrature points
!     phipress       pressure shape functions in quadrature points
!     rho            density rho
!     rhonodes       density rho in nodal  points
!     w              weights for quadrature
!     xgauss         Gauss co-ordinates
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3043   Value of jcart not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelfilldiv' )
      debug = .false. .and. ioutp>=0

      rho => E%rho
      rhonodes => E%coeffsnode(:,E%indrho)
      gradphi => B%phider
      phi => B%phi
      xgauss => B%xyzg
      w => B%w
      phipress => B%phipress

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelfilldiv'
         write(irefwr,1) 'npres, ndim, jcart', npsi, E%ndim, B%cart
         call prinrl ( rho, m, 'rho in integration points' )
         call prinrl ( rhonodes, n, 'rho in nodes' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      select case ( B%cart )

      case(1)

!     --- Case 1: cartesian coordinates
!         DijU = / psi_i ( rho dphi_j/dx + phi_j d rho/dx )

         if ( debug ) then
            write(irefwr,1) 'E%ndim, m, npsi', E%ndim, m, npsi
            call prinrl1 ( phipress, m, npsi, 'phipress' )
         end if  ! ( debug )

         do jdim = 1, E%ndim

            do k = 1, m
               gradrho(k) = sum(rhonodes*gradphi(:,k,jdim))
            end do  ! k = 1, m

            div => E%divmat(jdim:E%ndim*n:E%ndim,:)

            do i = 1, npsi
               do j = 1, n
                  div(j,i) = sum(w*phipress(i,:) * &
                           ( rho * gradphi(j,:,jdim) + &
                             B%phi(j,:) * gradrho ))
               end do  ! j = 1, n
            end do  ! i = 1, npsi
         end do  ! jdim = 1, E%ndim

      case(2)

!     --- Case 2: cylinder coordinates
!         Dij_r = / psi_i ( rho d r phi_j/(rdr) + phi_j d rho/dr )

         do k = 1, m
            gradrho(k) = sum(rhonodes*gradphi(:,k,1))
         end do  ! k = 1, m

         div => E%divmat(1:E%ndim*n:E%ndim,:)
         do i = 1, npsi
            do j = 1, n
               div(j,i) = sum(w * phipress(i,:) * &
                        ( rho*(gradphi(j,:,1)+phi(j,:)/xgauss(:,1))+ &
                          phi(j,:) * gradrho ))
            end do  ! j = 1, n
         end do  ! i = 1, npsi

!        --- Dij_z = / psi_i ( rho dphi_j/dz + phi_j d rho/dz )

         do k = 1, m
            gradrho(k) = sum(rhonodes*gradphi(:,k,2))
         end do  ! k = 1, m

         div => E%divmat(2:E%ndim*n:E%ndim,:)

         do i = 1, npsi
            do j = 1, n
               div(j,i) = sum(w * phipress(i,:) * &
                        ( rho * gradphi(j,:,2) + &
                          phi(j,:) * gradrho ))
            end do  ! j = 1, n
         end do  ! i = 1, npsi

      case(3)

!     --- case 3: polar coordinates
!         Dij_r = / psi_i ( rho d r phi_j/(rdr) + phi_j d rho/dr )

         do k = 1, m
            gradrho(k) = sum(rhonodes*gradphi(:,k,1))
         end do  ! k = 1, m

         div => E%divmat(1:E%ndim*n:E%ndim,:)

         do i = 1, npsi
            do j = 1, n
               div(j,i) = sum(w * phipress(i,:) * &
                        ( rho*(gradphi(j,:,1)+phi(j,:)/xgauss(:,1))+ &
                          phi(j,:) * gradrho ))
            end do  ! j = 1, n
         end do  ! i = 1, npsi

!        --- Dij_z = / psi_i ( rho dphi_j/( r dphi) + phi_j d rho/(r dphi) )

         do k = 1, m
            gradrho(k) = sum(rhonodes*gradphi(:,k,2)/xgauss(:,1))
         end do  ! k = 1, m

         div => E%divmat(2:E%ndim*n:E%ndim,:)

         do i = 1, npsi
            do j = 1, n
               div(j,i) = sum(w * phipress(i,:) * &
                        ( rho * gradphi(j,:,2)/xgauss(k,1) + &
                          phi(j,:) * gradrho ))
            end do  ! j = 1, n
         end do  ! i = 1, npsi

      case default

!     --- not available

         call errint ( B%cart, 1 )
         call errsub ( 3043, 1, 0, 0 )

      end select  ! case ( B%cart )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelfilldiv'

      end if  ! ( debug )
      call erclos ( 'sepelfilldiv' )

      end subroutine sepelfilldiv
