      subroutine sepelm900matp
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-09-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create element matrix for Navier-Stokes equations in case of penalty
!     method using separate submatrices
!     Old subroutine elm900mat
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j, icent, npres
      double precision :: h, diag(E%ndim+1)
      double precision, pointer :: s(:,:)

!     diag           Diagonal of the large matrix
!     h              Help variable to store a constant temporarily
!     i              Counting variable
!     icent          Index of centroid
!     j              Counting variable
!     npres          Number of pressure parameters per element
!     s              Element matrix to be filled
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL1        Print 2d real vector
!     SEPELM900CENT  Eliminate the centroid point for the incompressible
!                    Navier-Stokes equations in case of an extended quadratic
!                    triangle, quadrilateral or 27 node brick
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The Navier-Stokes matrix has the shape:
!            |               T   |
!            |   S        -SP    |    u
!     S   =  |    uu             |
!            |                   |
!            |   DIV        0    |    p
!     In the penalty method the zero matrix is repalced by eps M
!                     T
!     Hence Suu u - SP p = f
!           SP  u        = fd
!     is approximated by
!           Suu u -    SP p  = f
!           DIV u + eps M p  = fd
!                    -1
!    or:  p = 1/eps M   ( fd - DIV u )
!                       T  -1                -1   T
!       ( Suu + 1/eps SP  M   DIV ) u = f + M   SP  fd /eps
!    The matrix M is arbitrary but should approximate the mass matrix for
!    the pressure (related to the pressure basis functions) and must be easily
!    invertable
!    M is chosen to be a diagonal matrix containing the diagonal of the
!    pressure mass matrix, so
!    M(i,i) = / psi  psi  dx
!             x    i    i
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelm900matp' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelm900matp'
         write(irefwr,1) 'itype, jtrans, inpelm', &
                          itype, jtrans, inpelm
         write(irefwr,1) 'ndim, n, m', E%ndim, n, m
         write(irefwr,2) 'penalp', E%penalp
         call prinrl1 ( E%submat, E%ndim*n, E%ndim*n, 'elemsub' )
         call prinrl1 ( E%gradmat, E%npress, E%ndim*n, 'sp' )
         call prinrl1 ( E%divmat, E%npress, E%ndim*n, 'div' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      s => E%submat

!     --- Put divergence and gradient part into matrix
!         Eliminate centroid if necessary
!         Compute penalty contribution (old subroutine el4920)

      if ( jtrans==1 ) then

!     --- jtrans = 1, transformation necessary
!         Add contribution of penalty matrix

         h = 1d0/(E%penalp*sum(B%w)*2d0)
         if ( debug ) write(irefwr,2) 'diag(1)', h

         do i = 1, (n-1)*E%ndim
            do j = 1, (n-1)*E%ndim
               s(j,i) = s(j,i) + E%gradmat(i,1) * h * E%divmat(j,1)
            end do
         end do
         if ( debug ) call prinrl1 ( s, E%ndim*n, E%ndim*n, 's/1' )

!        --- Eliminate centroid

         E%tmpsmat => E%submat
         E%tmpmat => E%mat
         call sepelm900cent

      else

!     --- 2D and 3D case without elimination
!         Compute the diagonal of the mass matrix

         if ( E%ndim==2 ) then
            icent = n
         else
            if ( E%ishape==14 ) then
               icent = 14
            else
               icent = n
            end if
         end if
         if ( E%ishape==4 .or. E%ishape==6 .or. E%ishape==7 .or. &
              E%ishape==14 ) then
            npres = E%ndim+1
         else
            npres = 1
         end if

         diag = 0d0
         diag(1) = sum(B%w)

         do i = 1, npres-1
            diag(i+1) = sum(B%w*(B%xyzg(:,i)-B%xyz(icent,i))**2)
         end do

!        --- store the inverse divided by eps in diag

         do i = 1, npres
            diag(i) = 1d0 / ( diag(i) * E%penalp )
         end do

!        --- compute the penalty matrix and add to the element matrix

         do i = 1, n*E%ndim
            do j = 1, n*E%ndim
               E%mat(j,i) = s(j,i) + &
                  sum(E%gradmat(i,1:npres) * diag(1:npres) * &
                      E%divmat(j,1:npres))
            end do
         end do

      end if  ! ( jtrans==1 )

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( E%mat, icount, icount, 'elemmt' )
         write(irefwr,*) 'End sepelm900matp'

      end if  ! ( debug )
      call erclos ( 'sepelm900matp' )

      end subroutine sepelm900matp
