      subroutine sepelnsstrtens ( elemvc, etaeff, gradv, x, u, a1, &
                                  nelemvc, mcont, ndcomp, mdecl )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 29-06-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute element stress tensor for Navier-Stokes
!
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     navier_stokes_equation
!     viscosity
!     stress
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
     integer, intent(in) :: nelemvc, mcont, ndcomp, mdecl
     double precision, intent(in) :: gradv(9,m), etaeff(m), x(mdecl,*), &
                                     u(mdecl,*), a1(9,m)
     double precision, intent(inout) :: elemvc(ndcomp,m)

!     a1             i    A1 tensor defined by gradv + gradv^T
!     elemvc         o    Element vector
!     etaeff         i    Effective viscosity
!     gradv          i    Array containing the gradient vector in
!                         the quadrature points
!     mcont          i    Defines the type of continuity equation
!                         Possible values:
!                         0: incompressible flow
!                         1: compressible flow: div (rho u ) = 0
!                            Not applicable with penalty function method
!     mdecl          i    number of quadrature points in element,
!                         same as  m  in common /celint/
!     ndcomp         i    Indicates the number of quantities that must be filled
!                         in elemvc and elemwg per nodal point
!     nelemvc        i    Length parameter for elemvc
!     u              i    array of length m x ncomp in which the values of
!                         u in the integration points are stored the sequence:
!                         u  (x ) = u(j,i)  j = 1(1)m, i = 1 (1) ncomp
!                          i   j
!     x              i    array containing the coordinates of the nodal points
!                         according to:
!                         x  = x (i,1);  y  = x (i,2);  z  = x (i,3)
!                          i              i              i
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: edivv(nelemvc)
      integer :: k

!     edivv          Help variable to store divergence in compressible case
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The stress tensor is defined as t = eta A1:A1
!     with A1 = grad u + grad u^T
!     In case of weak compressible flow we have
!       t = eta ( A1:A1 - 2/3 div u I )
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelnsstrtens' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelnsstrtens'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ndimlc==2 .and. (jcart==1 .or. jcart==3) ) then

!     --- ndimlc = 2, Cartesian or Polar co-ordinates

         if ( mcont==1 ) then

!        --- PvK: make sure to add divergence term (weak compressible)

            edivv = -2d0/3d0*(gradv(1,:)+gradv(4,:))
            elemvc(1,:) = etaeff*(a1(1,:)+edivv)
            elemvc(2,:) = etaeff*(a1(2,:)+edivv)
            elemvc(4,:) = etaeff*a1(4,:)

         else

!        --- incompressible

            elemvc(1,:) = etaeff*a1(1,:)
            elemvc(2,:) = etaeff*a1(2,:)
            elemvc(4,:) = etaeff*a1(4,:)

         end if  ! (mcont==1)

      else if ( ndimlc==2 .and. jcart==2 ) then

!     --- ndimlc = 2, Cylinder co-ordinates

         do k = 1, 4
            elemvc(k,:) = etaeff*a1(k,:)
         end do  ! k = 1, 6

         if (mcont==1) then

!        --- Compressible, add -2 eta div I to tensor

            edivv = -2d0/3d0*(gradv(1,:)+gradv(4,:)+gradv(5,:))*etaeff
            elemvc(1,:) = elemvc(1,:)+edivv
            elemvc(2,:) = elemvc(2,:)+edivv
            elemvc(3,:) = elemvc(3,:)+edivv

         end if  ! (mcont==1)

      else

!     --- ndimlc = 3

         do k = 1, 6
            elemvc(k,:) = etaeff*a1(k,:)
         end do  ! k = 1, 6

      end if

1000  call erclos ( 'sepelnsstrtens' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelnsstrtens'

      end if  ! ( debug )

      end subroutine sepelnsstrtens
