      subroutine sepelvisc ( modelv, eta, etheff, cn, clamb, xgauss, u, &
                             dudx, secinv, detdsc, gradv, vecloc, maxunk, &
                             numold )
! ======================================================================
!
!        programmer    Guus Segal/Jaap van der Zanden
!        version  1.2  date 22-02-2006 Extra option modelv
!        version  1.1  date 30-10-1998 Include commons
!        version  1.0  date 15-07-1993 Copied from el4028
!
!   copyright (c) 1993-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     This subroutine computes the effective viscosity depending on the
!     viscosity model used
! **********************************************************************
!
!                       KEYWORDS
!
!     viscosity
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer modelv, numold, maxunk
      double precision eta(m), etheff(m), cn, clamb, xgauss(m,*), &
                       u(m,*), dudx(m,ndimlc,*), secinv(m), &
                       detdsc(m), gradv(9,m), vecloc(numold,maxunk,*)

!     clamb   i    Parameter lambda with respect to viscosity model
!     cn      i    Power in power law model
!     detdsc  o    ?
!     dudx    i    Real m x ndimlc  array in which the values of the
!                  derivatives of the solution vector in the integration
!                  points are stored in the sequence:
!                  du / dx  (x ) = dudx ( k, j )
!                    j     k
!                  If jdercn = 0 in common block celint, then only
!                  dudx (1,*) is filled because dudx is then a constant
!                  per element
!                  dudx is only filled if jderiv>0
!     eta     i    Array containing the viscosity in the integration points
!     etheff  o    Array containing the computed effective viscosity in
!                  the integration points
!     gradv   o    Array containing the gradient vector in
!                  the integration points
!     maxunk  i      Maximum number of unknowns per point
!     modelv  i    Defines the type of viscosity model
!                  Possible values:
!                    1, 10:  newtonian liquid
!                    2:  power law liquid
!                    3:  carreau liquid
!                    4:  plastico viscous liquid
!                  100:  user provided model depending on grad u
!                  101:  user provided model depending on the second invariant
!                  102:  user provided model depending on the second invariant, &
!                        the co-ordinates and the velocity
!     numold  i    Number of vectors that are stored in ISLOLD
!     secinv  o    Array containing the second invariant in the integration
!                  nodes
!     u       i    array of length m x ncomp in which the values of
!                  u in the integration points are stored the sequence:
!                  u  (x ) = u(j,i)  j = 1(1)m, i = 1 (1) ncomp
!                   i   j
!     vecloc  i    In this array the values of the function in the integration
!                  points is stored.
!                  vecloc(i,j,k) corresponds to the kth integration point, &
!                  the ith vector and the jth unkwon corresponding to that
!                  vector
!     xgauss  i    Double precision array of length m x ndimlc in which the
!                  co-ordinates of the integration points are stored
!                  according to:
!                  x  = x (i,1);  y  = x (i,2);  z  = x (i,3)
!                   i              i              i
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer k
      double precision term, expn, epsbng, x3, u2, u3

!                    they are not
!     epsbng         Lower threshold value for the value of a parameter
!     expn           Value of an exponent
!     k              Counting variable
!     term           Value of a term in an expression
!     u2             Second component of velocity vector
!     u3             Third component of velocity vector
!     x3             Third component of co-ordinate vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision fnv000, fnv001, fnv002, fnv003

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     FNV000         User-defined viscosity model (modelv=100)
!     FNV001         User-defined viscosity model (modelv=101)
!     FNV002         User-defined viscosity model (modelv=102)
!     FNV003         User-defined viscosity model (modelv=103)
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
!     none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     234   Computed effective viscosity is less than or equal to zero
!    1430   The viscosity parameter eta is non-positive
!    3143   parameter out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelvisc' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelvisc'
         write(irefwr,1) 'modelv', modelv
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ind(6)>0 ) then

!     --- eta variable, check positivity

         do k = 1, m

            if ( eta(k)<0d0 ) then

               call eropen ( 'sepelvisc' )
               call erreal ( eta(k), 1 )
               call errint ( itype, 1 )
               if ( itype<900 ) then

                  call errint ( 9, 2 )
               else
                  call errint ( 12, 2 )
               end if
               call errint ( ielgrp, 3 )
               call errint ( ielem, 4 )
               call errsub ( 1430, 4, 1, 0 )
               call erclos ( 'sepelvisc' )

            end if

         end do

      end if  ! ( ind(6)>0 )

      select case (modelv)

      case(1, 5, 10)

!      modelv = 1, 10: newtonian fluid
!      copy eta into etheff

         etheff = eta

      case(2)

!     --- modelv = 2, power law model

         expn = 0.5d0 * ( cn-1d0 )
         if ( expn<=0 ) secinv = max ( secinv, epsmac )

         etheff = eta * secinv ** expn

         if ( cn>1d0 ) then

!        --- cn>1, use Newton linearization

            expn = 0.5d0 * ( cn-3d0 )
            if ( expn<=0 ) secinv = max ( secinv, epsmac )

            detdsc = eta * (cn-1d0) * secinv ** expn
            etheff = etheff + detdsc*secinv

         end if

      case(3)

!     --- modelv = 3, three parameter carreau liquid

         expn = 0.5d0 * ( cn-1d0 )
         etheff = eta * (1d0+clamb*secinv) ** expn

      case(4)

!     --- modelv = 3, three parameter plastico-viscous liquid

         expn = 0.5d0/cn
         epsbng = 100d0 * epsmac
         do k = 1, m
            if ( secinv(k)<epsbng ) then

!           --- secinv nearly zero

               if ( secinv(k)>2d0*epsmac ) then
                  secinv(k) = ( secinv(k)-epsmac ) ** 2 / &
                              ( 2d0 * (epsbng-epsmac) )
               else
                  secinv(k) = epsmac
               end if

            end if
            term = ( clamb**2/(eta(k)**2*secinv(k)) ) ** expn
            etheff(k) = eta(k) * ( 1d0 + term ) ** cn
         end do

      case(100)

!     --- modelv = 100, user function fnv000

         do k = 1, m
            etheff(k) = fnv000 ( gradv(1,k) )
         end do

      case(101)

!     --- modelv = 101, user function fnv001

         do k = 1, m
            etheff(k) = fnv001 ( secinv(k) )
         end do

      case(102)

!     --- modelv = 102, user function fnv002

         x3 = 0d0
         u3 = 0d0
         do k = 1, m
            if ( ndimlc==3 ) then

!           --- ndimlc = 3, fill third component of x and u

               x3 = xgauss(k,3)
               u2 = u(k,2)
               u3 = u(k,3)

            else if ( jcart==4 ) then

!           --- jcart = 4, cylindrical co-ordinates with swirl

               u2 = u(k,3)
               u3 = u(k,2)

            else

               u2 = u(k,2)

            end if
            etheff(k) = fnv002 ( xgauss(k,1), xgauss(k,2), x3, &
                                 u(k,1), u2, u3, secinv(k) )
         end do

      case(103)

!     --- modelv = 103, user function fnv003

         x3 = 0d0
         u3 = 0d0
         do k = 1, m
            if ( ndimlc==3 ) then

!           --- ndimlc = 3, fill third component of x and u

               x3 = xgauss(k,3)
               u2 = u(k,2)
               u3 = u(k,3)

            else if ( jcart==4 ) then

!           --- jcart = 4, cylindrical co-ordinates with swirl

               u2 = u(k,3)
               u3 = u(k,2)

            else

               u2 = u(k,2)

            end if
            etheff(k) = fnv003 ( xgauss(k,1), xgauss(k,2), x3, &
                                 u(k,1), u2, u3, secinv(k), &
                                 numold, maxunk, vecloc(1,1,k) )
         end do

      case default

!     --- other values of modelv not implemented

         call errint ( modelv, 1 )
         call errchr ( 'modelv', 1 )
         call errsub ( 3143, 1, 0, 1 )
         go to 1000

      end select  ! case (modelv)
      if ( modelv>=100 ) then

!     --- Check if etheff is positive

         do k = 1, m

            if ( etheff(k)<=0d0 ) then

               call erreal ( etheff(k), 1 )
               call errint ( ielem, 1 )
               call errint ( itype, 2 )
               call errint ( ielgrp, 3 )
               call errsub ( 234, 3, 1, 0 )

            end if

         end do

      end if  ! ( modelv>=100 )

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( etheff, m, 'etheff' )
         write(irefwr,*) 'End sepelvisc'

      end if  ! ( debug )
      call erclos ( 'sepelvisc' )

      end subroutine sepelvisc
