      subroutine el0260 ( iuser, user, jelp, isub, isup,  array, &
                          nveloc, npres )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 06-01-2016 Use sepmoduleelem
!        version  3.2  date 23-08-2001 New call to elusr3
!        version  3.1  date 03-09-2000 Extension for scalars
!
!   copyright (c) 1994-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     This subroutine performs the initializations for subroutine ELM260 or
!     ELD260 in the case that IFIRST = 0
!     The parameters in the common blocks celint, celiar and celp
!     get a start value
!     Array array is initialized with the values of the coefficients
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     rubber_element
!     initialization
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision user(*), array(*)
      integer iuser(*), jelp, isub, isup, nveloc, npres

!     array   o      Real work array in which the values of the coefficients
!                    in the integration points are stored in the sequence:
!                    In the case of eld260 or elm260:
!                         2D                  3D
!                    1:   rho              rho
!                    2:   nu               nu
!                    3:   E                E
!                    4:   f1               f1
!                    5:   f2               f2
!                    6:                    f3
!                    In the case of eli260:
!                    1:   f
!     isub    o    Integer help parameter for do-loops to indicate the
!                  smallest coefficient number that is dependent on space
!     isup    o    Integer help parameter for do-loops to indicate the
!                  largest coefficient number that is dependent on space
!     iuser   i    Integer user array to pass user information from
!                  main program to subroutine. See STANDARD PROBLEMS
!     jelp    o    Indication of the type of basis function subroutine must
!                  be called by ELM100. Possible vaues:
!                  1:  Subroutine ELP610  (Linear line element)
!                  2:  Subroutine ELP611  (Quadratic line element)
!                  3:  Subroutine ELP620  (Linear triangle)
!                  4:  Subroutine ELP622  (Quadratic triangle)
!                  5:  Subroutine ELP621  (Bilinear quadrilateral)
!                  6:  Subroutine ELP623  (Biquadratic quadrilateral)
!                 11:  Subroutine ELP630  (Linear tetrahedron)
!                 12:  Subroutine ELP631  (Quadratic tetrahedron)
!                 13:  Subroutine ELP632  (Trilinear brick)
!                 14:  Subroutine ELP633  (Triquadratic brick)
!     npres   o    Number of pressure parameters
!     nveloc  o    Number of velocity parameters
!     user    i    Real user array to pass user information from
!                  main program to subroutine. See STANDARD PROBLEMS
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ncoef, i, j, isubcf, kchois, mrule, ishift
      double precision elast, anu
      logical funchk

!     anu     Poisson parameter nu
!     elast   Elasticity modulus E
!     funchk  Indicates whether there are coefficients that must be
!             evaluated by funcc3 (true) or not (false)
!     i       Counting variable
!     ishift  Shift in coefficient sequence number
!     isubcf  Under bound for coefficients check
!     j       Counting variable
!     kchois  Indication what type of parameters must be computed
!             Possible values:
!             0:  Standard method, the calling subroutine is ELM260
!                 It is supposed that an element matrix must be computed
!             1:  Subroutine ELD260 calls this subroutine, only
!                 derivatives must be computed
!             2:  Subroutine ELI260 calls this subroutine, only
!                 integrals must be computed
!     mrule   Type of integration rule as given by the user
!     ncoef   Number of coefficients in the equation
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inirepscal

!     ELCOMM         Initializes the commons celp and celint with standard
!                    values
!     ELUSR3         Extracts information concerning the coefficients from IUSER
!                    and USER.
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     INIREPSCAL     Replace ivalue by scalar if necessary
!     INSTOP         Stop the program
!     PRINLG         Print and checks length of user arrays
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays IUSER and USER
!     Usually they are filled by FILCOF
!     The sequence is for the rubber equations is:
!     1:  information about the time-integration
!         Possible values:
!         0: if imas in BUILD=0 then time-independent flow
!            else the mass matrix is built according to imas
!     2:  - &
!     3:  Type of numerical integration
!         Possible values:
!         0: default value, defined by element
!     4:  Type of co-ordinate system
!         Possible values:
!         0: Cartesian co-ordinates
!     5:  - &
!     6-20: Information about the equations and solution method
!     6:  density rho
!     7:  nu  (Poisson ratio)
!     8:  E   (Elasticity modulus)
!     9:  F1  (Body force in 1-direction)
!    10:  F2  (Body force in 2-direction)
!    11:  F3  (Body force in 3-direction)
!    12:  Parameter eps for the penalty function method
!    13-20: not yet used
!     In the case of a call by INTEGR:
!     1:  Type of numerical integration
!         Possible values:
!         0: default value, defined by element
!     2:  Type of co-ordinate system
!         Possible values:
!         0: Cartesian co-orindates
!         1: Axi-symmetric co-orindates
!         2: Polar co-orindates
!     3:  Not yet used (must be 0)
!     4:  f
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      42   Array IUSER(.) has value which is too small
!     262   ix has wrong value
!     264   jdegfd has wrong value
!     406   Value of icheld not allowed
!     555   Poisson ratio nu (or nu1) has wrong value
!    1430   The viscosity parameter etha is non-positive
!    1766   Type of integration rule incorrect
!    1767   Type of co-ordinates incorrect
!    1768   Coefficient is filled that is not used
!    1771   Shape number not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Fill some standard values in the common blocks celp and celint

      call eropen ( 'el0260' )
      call elcomm
      jconsf = 2
      ncomp  = ndim
      kchois = 0
      npres  = 1
      m = 1
      ishift = 0

!     --- Compute kchois

      if ( icheld.eq.0 ) then

!     --- Call by elm260, all coefficients required

         ncoef = 6

      else if ( icheld.gt.0 ) then

!     --- Call by eld260

         if ( icheld.gt.7 .and. icheld.lt.20 .or. icheld.gt.27 ) then

!        --- icheld out of range

            call errint ( icheld, 1 )
            call errint ( itype, 2 )
            call errint ( ielgrp, 3 )
            call errsub ( 406, 3, 0, 0 )

         end if

         if ( icheld.eq.6 .or. icheld.eq.26 ) then
            ncoef = 6
         else
            ncoef = 0
            kchois = 1
         end if

         if ( icheld.eq.1 .or. icheld.eq.21 ) then

            if ( ix.le.0 .or. ix.gt.ndim ) then
               call errint ( ix, 1 )
               call errint ( ndim, 2 )
               call errint ( itype, 3 )
               call errint ( ielgrp, 4 )
               call errsub ( 262, 4, 0, 0 )
            end if
            if ( jdegfd.le.0 .or. jdegfd.gt.ndim ) then
               call errint ( jdegfd, 1 )
               call errint ( ndim, 2 )
               call errsub ( 264, 2, 0, 0 )
            end if

         end if
         ncomp  = ndim
         jderiv = 1

      else if ( icheld.lt.0 ) then

!     --- call by eli260, one (new) coefficient required

         kchois = 2
         ncoef  = 1
         ncomp = icount/inpelm
         if ( jdegfd.lt.1 .or. jdegfd.gt.ncomp ) then
            call errint ( jdegfd, 1 )
            call errint ( ncomp, 2 )
            call errsub ( 264, 2, 0, 0 )
         end if

      end if
      if ( itype.eq.262 ) then
         if ( inpelm.eq.27 ) jind = -2
         if ( ishape.eq.6 .or. ishape.eq.7 .or.ishape.eq.14 ) &
            npres = ndim+1
      else
         jind = 1
      end if

!     --- Fill the arrays IND, MST and ARRAY with information of the
!     coefficient

      if ( ncoef.gt.0 .or. icheld.gt.0 ) then

         ipos = iuser ( ielgrp + 5 )
         if ( ipos.lt.7 ) then
            call errint ( ielgrp+5, 1 )
            call errint ( ipos, 2 )
            call errint ( ielem, 3 )
            call errint ( itype, 4 )
            call errint ( ielgrp, 5 )
            call errsub ( 42, 5, 0, 0 )
         end if

         nuser = 0

         if ( kchois.eq.0 ) then

!        --- Standard situation, Compute the parameter notmas

            ipos = ipos + 2
            ishift = ishift+2
            if ( imas.eq.0 ) notmas = 1

         else if ( kchois.eq.1 ) then
            ipos = ipos + 2
            ishift = ishift+2
         end if

         if ( kchois.ge.0 .and. kchois.le.2 ) then

!        --- Call by BUILD or INTEGR, Compute the parameters jcart and irule

            mrule = inirepscal ( iuser(ipos) )
            ipos = ipos + 1
            ishift = ishift+1
            if ( mrule.lt.0 .or. mrule.gt.3 ) then
               call errint ( mrule, 1 )
               call errint ( 3, 2 )
               call errint ( ielem, 3 )
               call errint ( itype, 4 )
               call errint ( ielgrp, 5 )
               call errsub ( 1766, 5, 0, 0 )
            end if

            jcart = inirepscal ( iuser(ipos) )
            ipos = ipos + 1
            ishift = ishift+1
            if ( jcart.lt.0 .or. jcart.gt.0 ) then
               call errint ( jcart, 1 )
               if ( kchois.eq.0 ) then
                  call errint ( 4, 2 )
               else
                  call errint ( 2, 2 )
               end if
               call errint ( ielem, 3 )
               call errint ( itype, 4 )
               call errint ( ielgrp, 5 )
               call errsub ( 1767, 5, 0, 0 )
            end if
            jcart = jcart + 1
            ipos = ipos + 1
            ishift = ishift+1

         end if

      end if

!     --- Set parameters depending on the shape numbers

      if ( mrule.gt.0 ) then
         irule = mrule
      else if ( icheld.lt.0 .or. ishape.eq.13 .or. ishape.eq.17 ) then
         irule = 1
      else
         irule = 3
      end if
      if ( icheld.gt.0 .and. icheld.ne.6 .and. icheld.ne.26 .or. &
           icheld.eq.20 .or. icheld.lt.-2 ) irule = 1
      if ( irule.ne.1 ) jdiag=2
      if ( itype.eq.260 .and. inpelm.eq.6 .and. &
           (icheld.eq.0 .or. icheld.eq.6 .or. icheld.eq.26 .or. &
            icheld.eq.20) ) then
         n = 7
         jtrans = 1
         jelp = 7
         nveloc = 12
      else if ( itype.eq.261 .and. inpelm.eq.7 ) then
         n = 7
         jtrans = 1
         jelp = 7
         nveloc = 12
      else if ( itype.gt.260 .and. inpelm.eq.5 ) then
         n = 4
         jelp = 5
         nveloc = 8
      else if ( itype.gt.260 .and. ishape.eq.17 ) then
         n = 8
         jelp = 13
         nveloc = 24
      else if ( itype.eq.262 .and. inpelm.eq.27 .or. inpelm.eq.7 ) &
         then
         n = inpelm
         jtrans = 0
         jelp = ishape
         nveloc = ndim*n
      else if ( itype.eq.260 .and. (ishape.ge.4 .and. ishape.lt.7 .or. &
                ishape.eq.13 ) .or. itype.eq.261 .and. &
                ishape.eq.7 .or. ishape.eq.14 ) then
         n = inpelm
         jelp = ishape
         nveloc = ndimlc*n
      else
         call errint ( ishape, 1 )
         call errint ( ielem, 2 )
         call errint ( itype, 3 )
         call errint ( ielgrp, 4 )
         call errsub ( 1771, 4, 0, 0 )
      end if
      m = n
      if ( inpelm.eq.4 ) then
         if ( irule.eq.2 ) then
            m = 1
         else if ( irule.eq.3 ) then
            m = 4
         end if
      else if ( inpelm.eq.6 .or. inpelm.eq.7 ) then
         if ( irule.eq.2 ) then
            m = 4
         else if ( irule.eq.3 ) then
            m = 7
         end if
      else if ( ishape.eq.13 ) then
         if ( irule.eq.2 ) then
            m = 1
         else if ( irule.eq.3 ) then
            m = 8
         end if
      else if ( inpelm.eq.9 .and. irule.eq.2 ) then
         m = 4
      else if ( inpelm.eq.27 .and. irule.eq.2 ) then
         m = 8
      end if

      mn = m * n
      iphi=1
      igausp = 1
      if ( inpelm.eq.27 ) then
         nvert  = 8
      else if ( inpelm.eq.6 .or. inpelm.eq.7 ) then
         nvert = 3
      else if ( ishape.eq.6 ) then
         nvert = 4
      else
         nvert = n
      end if

!     --- Clear array ARRAY

      do i = 1, 6*m
         array(i) = 0d0
      end do

      do i = 1 , ncoef
         call elusr3 ( i, array(1+(i-1)*m), iuser, user, i+ishift )
      end do

!     --- Test coefficients

      if ( ind(1).eq.0 ) then

!     --- rho has not been given a value, set rho equal to 1

         do i = 1, m
            array(i) = 1d0
         end do

      end if

      if ( ind(2).le.0 .and. (icheld.eq.0 .or. icheld.eq.6 &
           .or. icheld.eq.26) ) then

!     --- nu is constant, test whether nu is positive

         anu = array(1+m)
         if ( anu.le.0d0 .or. anu.gt.0.5d0 ) then
            call erreal ( anu, 1 )
            call errint ( ielem, 2 )
            call errint ( itype, 3 )
            call errint ( ielgrp, 4 )
            call errsub ( 555, 3, 1, 0 )
         end if
         anu = min ( anu, 0.49999d0 )
      end if
      if ( ind(3).le.0 .and. (icheld.eq.0 .or. icheld.eq.6 &
           .or. icheld.eq.26) ) then

!     --- E is constant, test whether E is positive

         elast = array(1+2*m)
         if ( elast.le.0d0 ) then

!        --- E <= 0

            call erreal ( elast, 1 )
            call errint ( itype, 1 )
            call errint ( 12, 2 )
            call errint ( ielgrp, 3 )
            call errint ( ielem, 4 )
            call errsub ( 1430, 4, 1, 0 )

         end if

      end if
      if ( icheld.eq.0 .and. ind(4).eq.0 .and. ind(5).eq.0 .and. &
           ind(6).eq.0 ) notvec = 1
      if ( ndimlc.eq.2 .and. ind(6).ne.0 ) then

!     --- Warning f3 # 0, whereas ndim = 2

         call errint ( 11, 1 )
         call errint ( ielem, 2 )
         call errint ( itype, 3 )
         call errint ( ielgrp, 4 )
         call errwar ( 1768, 4, 0, 0 )
         ind(6) = 0

      end if

!     --- Test array lengths, number of nodes and number of degrees of freedom

      call prinlg ( 'iuser', ipos-1, user, iuser )
      call prinlg ( 'user', -nuser, user, iuser )

!     --- Check which parameters depend on the space co-ordinates
!         Compute isub and isup

      isubcf = 1
      if ( icheld.gt.0 .and. .not. (ndim.eq.2 .and. mod(icheld,20).eq.6) &
         ) isubcf = 6

      do i = isubcf , ncoef
         isub = i

!        --- If ind(i) > 0 then paramater i corresponds to a function

         if ( ind(i).gt.0 ) goto 170

      end do

      isub = ncoef + 1

170   isup = ncoef
      do i = ncoef, isub , -1
         isup = i
         if ( ind(i).gt.0 ) goto 190
      end do

!     --- Check for double parameters (functions) to avoid calls for
!         function subroutines
!         ist is used to store the references for the double parameters
!         Set ind(lengthmst)

190   do i = isub, isup
         ist(i) = 0
      end do

      if ( icheld.gt.0 ) then

!     --- Compute derivative quantities

         ind(lengthmst) = 1
         if ( ndim.eq.3 .and. (icheld.eq.6 .or. icheld.eq.26) ) then
            jderiv = 0
         else
            jderiv = 1
         end if

      else if ( icheld.eq.-1 .or. icheld.eq.-2 ) then

!     --- icheld = -1 or -2, do not compute derivatives

         ideriv = 0
         jderiv = 0

      else if ( icheld.lt.-2 ) then

!     --- icheld < -2, derivatives should be computed

         jderiv = -icheld+3

      else

         ind(lengthmst) = 0

      end if

      funchk = .false.
      do i = isub, isup
         if ( ind(i).gt.0 .and. ind(i).lt.2000 ) then

!        --- only the function calls are checked

            do j = i+1,isup
               if ( ind(j).eq.ind(i) ) then
                  ist(j) = i
                  ind(j) = 0
               end if
            end do
            igausp = 1
         end if
         if ( ind(i).gt.1000 .and. ind(i).le.2000 ) ind(lengthmst) = 1
         if ( ind(i).gt.10000 ) funchk = .true.
         if ( ind(i).gt.1000 .and. ind(i).le.2000 .or. ind(i).gt.10000 ) &
              igausp=1
      end do

      if ( icheld.lt.-1 .or. icheld.gt.0 ) ind(lengthmst) = 1
      if ( funchk ) ind(lengthmst) = ind(lengthmst) + 2

      if ( ierror.ne.0 ) call instop

      call erclos ( 'el0260' )

      end
