      subroutine el8003 ( div, w, x, xgauss, elast, anu, u, p )
! =====================================================================
!
!        programmer   Guus Segal
!        version 1.0  date   03-09-1994 Subroutine extracted from el4057
!
!
!   copyright (c) 1994-1994  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                         DESCRIPTION
!
!     compute the pressure for the rubber elements in the centroids
!     of the elements and store in array p (penalty formulation)
! **********************************************************************
!
!                      KEYWORDS
!
!     derivative
!   element_vector
!   navier_stokes_equation
! ********************************************************************
!
!                      COMMON BLOCKS
!
      use sepmodulecactl
      implicit none

! ********************************************************************
!
!                      INPUT / OUTPUT    PARAMETERS
!
      double precision w(m), div(ndimlc*n,*), anu(m), u(n,ndimlc), &
                       xgauss(m,*), x(n,*), elast(m), p(*)

!     anu     i    Poisson parameter nu
!     div     i    Divergence matrix
!                  div(i,k) corresponds to the k-th row and i-th column
!                  of the matrix, The k refers to the basis functions of the
!                  pressure, i to the basis functions of the velocity in the
!                  sequence of the solution vector
!     elast   i    Elasticity modulus E
!     p       o    Array containing the pressure
!                  p(1) contains the pressure in the centroid
!                  p(2-4) contains the gradient of the pressure in the centroid
!     u       i    array of length m containing the values of the
!                  array of length m x ncomp in which the values of
!                  u in the integration points are stored the sequence:
!                  u  (x ) = u(j,i)  j = 1(1)m, i = 1 (1) ncomp
!                   i   j
!     w       i    array of length m containing the weights for integration
!     x       i    array containing the coordinates of the nodal points
!                  according to:
!                  x  = x (i,1);  y  = x (i,2);  z  = x (i,3)
!                   i              i              i
!     xgauss  i    array of length m x ndim containing the co-ordinates of
!                  the gauss points.  array xgauss is only filled when gauss
!                  integration is applied.
!                  xg  = xgauss (i,1);  yg  = xgauss (i,2);  zg  = xgauss (i,3)
!                    i                    i                    i
! **********************************************************************
!
!                    LOCAL PARAMETERS
!
      integer i, j, k, icent, npres
      double precision diag(4), sum

!     diag    Diagonal of pressure mass matrix
!     i       Counting variable
!     icent   Index of centroid
!     j       Counting variable
!     k       Counting variable
!     npres   Number of pressure parameters per element
!     sum     Help parameter to compute a sum
! ********************************************************************
!
!             SUBROUTINES CALLED
!
! ********************************************************************
!
!                    I/O
!
!    none
! ********************************************************************
!
!                    ERROR MESSAGES
!
!   - &
! ********************************************************************
!
!                    PSEUDO CODE
!
!     The "Rubber" matrix has the shape:
!
!            |                T  |
!            |   S        -DIV   |    u
!     S   =  |    uu             |
!            |                   |
!            |   DIV         M   |    p
!
!
!     or:
!                      T
!           Suu u - DIV p  = f
!
!           DIV u +  M p  = 0
!
!                 -1
!    or:  p = -  M   DIV u
!
!                  T  -1
!       ( Suu + DIV  M   DIV ) u = f
!
!    The matrix M is defined by
!
!    M(i,j) = / psi  psi  (E/3(1-2nu)) dx
!             x    i    j
!
! =====================================================================
!
      if ( ifirst.eq.0 .or. ind(2).gt.0 ) then

!     --- First call or nu variable
!         Check nu

         do 50 k = 1, m
            anu(k) = min ( anu(k), 0.49999d0 )
 50      continue

      end if

      if ( ndimlc.eq.2 ) then
         icent = n
      else
         icent = 14
      end if
      if ( n.eq.4 ) then
         npres = 1
      else
         npres = ndimlc+1
      end if

!     --- Compute the diagonal of the mass matrix

      do 100 i = 1, npres
         diag(i) = 0d0
100   continue
      do 110 k = 1, m
         diag(1) = diag(1) + w(k)/elast(k)*(3d0*(1d0-2d0*anu(k)))
110   continue
      do 130 i = 1, npres-1
         do 120 k = 1, m
            diag(i+1) = diag(i+1) + w(k)/ &
               elast(k)*(3d0*(1d0-2d0*anu(k)))* &
               (xgauss(k,i)-x(icent,i))**2
120      continue
130   continue

!     --- store the inverse divided by eps in diag

      do 150 i = 1, npres
         diag(i) = 1d0 / diag(i)
150   continue

!     --- multiply the velocity by the divergence matrix and minus the
!         inverse of the mass matrix divided by eps

      do 230 k = 1, npres
         sum = 0d0
         do 220 j = 1, ndimlc
            do 210 i = 1, n
               sum = sum + div((i-1)*ndimlc+j,k) * u(i,j)
210         continue
220      continue
         p(k) = - diag(k) * sum
230   continue

      end
