      subroutine el8106 ( igprob, elemvc, eps0, sigma0, anu, d, sigma, &
                          eps, mdecl, nd, icheld )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 26-06-2014 Extra parameter ichled
!        version  1.4  date 29-09-2000 Adapt declarations
!        version  1.3  date 20-12-1999 Allow integration points in R^3
!
!   copyright (c) 1992-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the tensor sigma
!     according to:
!       SIGMA = D ( eps - eps ) + SIGMA
!                            0         0
!     At this moment linear elements are assumed with one integration point
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     elasticity_equation
!     stress
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer mdecl, nd, igprob, icheld
      double precision anu(mdecl,*), d(nd,nd,mdecl), sigma(mdecl,*), &
                       eps0(mdecl,*), sigma0(mdecl,*), elemvc(6,*), &
                       eps(mdecl,*)

!     anu     i   Array containing the values of the poisson ratio nu in
!                 the integration points, depending on IGPROB
!                 IGPROB = 1-3:  ANU(1,k) contains nu
!                 IGPROB = 4-6:  ANU(2,k) contains nu1
!                                ANU(3,k) contains nu2
!                                ANU(4,k) contains G2
!     d       i   Array containing the Elasticity matrix D
!                                 k
!                 D(i,j,k) = d  (x )
!                             ij
!     elemvc  o   Array containing the element vector for the eld subroutines
!     eps     i   Array containing the value of the strain epsilon in the
!                 integration points
!                            k
!                 epsilon  (x ) = eps(i,k)
!                         i
!     eps0    i   Array of length m * nd containing the values of eps  in the
!                                                                    0
!                 integration points
!     icheld  i   Choice parameter to distinguish between the various
!                 possibilities of this subroutine. See eld250
!     igprob  i   Type of stress-strain relations
!                 Possible values:
!                 1: plane stress, isotropic material (cartesian)
!                 2: plane strain, isotropic material (cartesian)
!                 3: axi-symmetric stress, isotropic material (axi-symmetric)
!                 4: plane stress, orthotropic material (cartesian)
!                 5: plane strain, orthotropic material (cartesian)
!                 6: axi-symmetric stress, orthotropic material
!                    (axi-symmetric)
!                 7: User defined elasticity matrix (3x3) (cartesian)
!                 8: User defined elasticity matrix (4x4) (axi-symmetric)
!     mdecl   i   Number of integration points in the element
!     nd      i   Dimension of elasticity matrix D
!     sigma   o   Array of length m * nd containing the values of sigma  in the
!                 integration points
!     sigma0  i   Array of length m * nd containing the values of sigma  in the
!                                                                      0
!                 integration points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, k, j
      double precision sum

!     i              General loop variable
!     j              General loop variable
!     k              General loop variable
!     sum            Help variable to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     837   integration rule not yet available
!    3189   option not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el8106' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el8106'
         write(irefwr,1) 'nd, m, n, mdecl, icheld', &
                          nd, m, n, mdecl, icheld
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( nd<=4 ) then

!     --- ndim = 2

         if ( m==1 ) then

!        --- m = 1    linear triangle   or  bilinear quadrilateral

            if ( nd==3 ) then

!           --- igprob = 1, 2, 4, 5 or 7 Plane stress or plane strain

               do i = 1, 3
                  sum = 0d0
                  do j = 1, 3
                     sum = sum + d(i,j,1) * ( eps(1,j)-eps0(1,j) )
                  end do
                  sigma(1,i) = sum + sigma0(1,i)
               end do
               sigma(1,4) = sigma(1,3)
               if( igprob==1 .or. igprob==4 .or. igprob==7 ) then
                  sigma(1,3) = 0d0
               else
                  sigma(1,3) = anu(1,1) * ( sigma(1,1) + sigma(1,2) )
               end if

            else if ( nd==4 ) then

!           --- igprob = 3, 6 or 8  Axi-symmetric stress analysis

               do i = 1, 4
                  sum = 0d0
                  do j = 1, 4
                     sum = sum + d(i,j,1) * ( eps(1,j)-eps0(1,j) )
                  end do
                  sigma(1,i) = sum + sigma0(1,i)
               end do

            end if

            do i = 1, n
               do j = 1, 4
                  elemvc(j,i) = sigma(1,j)
               end do
            end do

         else if ( jdiag==1 ) then

!        --- n # 3   general element    jdiag = 1

            if ( igprob<=2 ) then

!           --- igprob = 1 or 2  Plane stress or plane strain

               do i = 1, 3
                  do k = 1, m
                     sum = 0d0
                     do j = 1, 3
                        sum = sum + d(i,j,k) * ( eps(k,j)-eps0(k,j) )
                     end do
                     sigma(k,i) = sum + sigma0(k,i)
                  end do
               end do
               do k = 1, m
                  sigma(k,4) = sigma(k,3)
               end do

               if( igprob==1 ) then

                  do k = 1, m
                     sigma(k,3) = 0d0
                  end do

               else

                  do k = 1, m
                     sigma(k,3) = anu(k,1) * ( sigma(k,1) + sigma(k,2) )
                  end do

               end if

            else if ( igprob==3 ) then

!           --- igprob = 3  Axi-symmetric stress analysis

               do i = 1, 4
                  do k = 1, m
                     sum = 0d0
                     do j = 1, 4
                        sum = sum + d(i,j,k) * ( eps(k,j)-eps0(k,j) )
                     end do
                     sigma(k,i) = sum + sigma0(k,i)
                  end do
               end do

            end if

            do j = 1, 4
               do i = 1, n
                  elemvc(j,i) = sigma(i,j)
               end do
            end do

         else

!        --- jdiag = 2, integration points do not coincide with nodes

            call eropen ( 'el8106' )
            call errsub ( 837, 0, 0, 0 )
            call erclos ( 'el8106' )

         end if

      else

!     --- ndim = 3

         do k = 1, m
            do i = 1, 6
               sum = 0d0
               do j = 1, 6
                  sum = sum + d(i,j,k) * ( eps(k,j)-eps0(k,j) )
               end do
               sigma(k,i) = sum + sigma0(k,i)
            end do
         end do

         if ( m==1 ) then

!        --- m = 1, copy sigma in elemvc

            do j = 1, nd
               elemvc(j,1:n) = sigma(1,j)
            end do  ! j = 1, nd

         else if ( m==n ) then

!        --- m = n, copy sigma in elemvc

            do j = 1, nd
               elemvc(j,1:n) = sigma(1:n,j)
            end do  ! j = 1, nd

         else if ( icheld==16 .or. icheld==46 ) then

!        --- special case, copy sigma in elemvc per integration point

            do j = 1, nd
               elemvc(j,1:m) = sigma(1:m,j)
            end do  ! j = 1, nd

         else

!        --- other options not yet implemented

            call errint ( m, 1 )
            call errchr ( 'm', 1 )
            call errsub ( 3189, 1, 0, 1 )

         end if  ! ( m==1 )

      end if

1000  call erclos ( 'el8106' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End el8106'

      end if  ! ( debug )

      end
