      subroutine el8301 (Fd, Jd, Fn, Jn, p, coef, D_t, sigma, &
                         ichoice, nd, xcoor, xcoor0)
! ======================================================================
!
!        programmer    Chris van Oijen
!        version  4.0  date 06-01-2016 Use sepmoduleelem
!        version  3.0  date 13-04-2005 Add integer nsig for axi-symmetry (RvL)
!        version  2.1  date 17-12-2002 Use ndim
!                                      extra parameter for elmul01/03
!
!   copyright (c) 2000-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Compressible Neo-Hookean constitutive law
!
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichoice, nd
      double precision D_t(nd,nd), Fd(ndim,ndim), xcoor(*),xcoor0(*), &
                       Fn(ndim, ndim), Jd, Jn, p, coef(*), sigma(*)

!     coef           i    Element coefficients
!     d_t            o    Matrix to be filled
!     fd             i    Deformation gradient F(k,j,i) = F_ij(x^k)
!     fn             i    Deformation gradient at start of timestep
!     ichoice        i    Calculate tang. stifness matrix+stress(0)
!                         or just stress(1)
!     jd             i    Determinant of deformation gradient
!     jn             i    Determinant of deformation gradient at incr. n
!     nd             i    Dimension of tangent stiffness matrix
!     p              i    Hydrostatic pressure
!     sigma          o    Stress to be filled
!     xcoor          i    array of x coordinates in the listpack.
!     xcoor0         i    ?
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, r, s
      double precision F(ndim,ndim), B(3*ndim-3), J, G, K, B4, &
                       D1(nd,nd), D2(nd,nd), temp

!     b              Finger tensor
!     b4             ?
!     d1             Work matrix
!     d2             Work matrix
!     f              Deformation gradient
!     g              Shear modulus
!     i              Counting variable
!     j              Determinant of F
!     k              Bulk modulus
!     r              Counting variable
!     s              Counting variable
!     temp           Temporay variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL2730         ?
!     ELMUL01        ?
!     ELMUL03        ?
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Compute Finger strain
!                         c     c
!         B = Fd * Fn * Fn  * Fd

      call elmul03 (Fd, Fn, F, 0, ndim)
      call elmul01 (F, B, 0, ndim)

      if (jcart==2) then

!     --- axisymmetric case

         B4=(xcoor(1)/xcoor0(1))**2d0

      end if

!     --- Get material parameters

      G = coef(10-5)
      K = coef(11-5)
      J = Jd*Jn

!     --- Compute tangential stiffness

      if (ichoice==0) then

!     --- fill in D_t = D1 + (G/J)*delta(B)

         do r = 1, nd
            do s = 1, nd
               D1(r,s) = 0d0
               D2(r,s) = 0d0
            end do
         end do

!        --- fill in D1

         temp = K*J + (G/3d0)*(J**(-1d0/3d0))
         if (ndim==2) then

          do s = 1, ndim
             D1(1,s) = temp - (G/J)*B(1)
             D1(2,s) = temp - (G/J)*B(2)
             D1(3,s) = - (G/J)*B(3)
             D1(4,s) = - (G/J)*B(3)

!            --- axisymmetric case

             if (jcart==2) then
                D1(5,s) = temp - (G/J)*B4
             end if
          end do

!         --- axisymmetric case

          if (jcart==2) then
             D1(1,nd)= temp - (G/J)*B(1)
             D1(2,nd) = temp - (G/J)*B(2)
             D1(3,nd) = - (G/J)*B(3)
             D1(4,nd) = - (G/J)*B(3)
             D1(5,nd) = temp - (G/J)*B4
          end if

         else if (ndim==3) then

          do s = 1, ndim
            D1(1,s) = temp - (G/J)*B(1)
            D1(2,s) = temp - (G/J)*B(2)
            D1(3,s) = temp - (G/J)*B(3)
            D1(4,s) = - (G/J)*B(4)
            D1(5,s) = - (G/J)*B(5)
            D1(6,s) = - (G/J)*B(6)
            D1(7,s) = - (G/J)*B(4)
            D1(8,s) = - (G/J)*B(5)
            D1(9,s) = - (G/J)*B(6)
          end do

         end if

!        --- fill in D2 = delta(B)

         call el2730 (D2, B, nd, B4)

!        --- assemble D_t = D1 + D2

         do r = 1,nd
            do s = 1, nd
               D_t(r,s) = D1(r,s) + (G/J)*D2(r,s)
            end do
         end do

      end if

!     --- Compute stress
!         sigma = K*(J-1)I + G/J * (B - J^(2/3))I

      if (ndim==2) then

         sigma(1) = K*(J-1d0) + (G/J)*(B(1) - J**(2d0/3d0))
         sigma(2) = K*(J-1d0) + (G/J)*(B(2) - J**(2d0/3d0))
         sigma(3) = (G/J)*B(3)

!        --- axisymmetric case

         if (jcart==2) then

            sigma(4) = K*(J-1d0) + (G/J)*(B4 - J**(2d0/3d0))

         end if

      else if (ndim==3) then

         do i = 1, 3
            sigma(i) = K*(J-1d0) + (G/J)*(B(i) - J**(2d0/3d0))

         end do
         do i = 4,6
            sigma(i) = (G/J)*B(i)
         end do

      end if

!     --- In case of multi-phase material

      if (itype>=203) then
         do i = 1, ndim
            sigma(i) = -p + sigma(i)
         end do
      end if

      end
