      subroutine elm211 ( work )
! ======================================================================
!
!        programmer    Martijn Cox & Niels Driessen
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.4  date 29-05-2014 remove call to time subroutine
!        version  1.3  date 12-08-2013 Combine dphi/dxi and dphideta
!
!   copyright (c) 2004-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix and vector for contact conditions for
!     general nonlinear solids
!     ELM211 is a help subroutine for subroutine BUILD (TO0050)
!     it is called through the intermediate subroutine elmsubs
!     So:
!     BUILD
!     TO0050
!       -  Loop over elements
!          -  ELMSUBS
!             - ELM211
!    The subroutine is able to handle the following element shapes and cases:
!    2D: Quadratic line
!    3D: Biquadratic quadrilateral
!    3D: Extended biquadratic triangle
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     element_matrix
!     element_vector
!     contact
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulebuild
      use sepmoduleoldvecs
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision work(*)

!     work           i    Real work array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer maxnu, maxcoef, nisequ, niseqgam, iseqnr
      parameter (nisequ = 27, niseqgam = 9, maxcoef = 15)
      integer isub, isup, jelp
      save isub, isup, jelp
      integer i, j, iload
      integer isequ(nisequ), iseqgam(niseqgam)
      save isequ, iseqgam, iload
      double precision coorn(ndim,inpelm), coor1(ndim,inpelm), &
                       gamma(inpelm), g(inpelm), &
                       ni(inpelm,ndim), &
                       dni(inpelm,ndim,ndim), rhsg(inpelm), &
                       rhsx(ndim*inpelm), &
                       Kxx(ndim*inpelm,ndim*inpelm), &
                       Kxg(ndim*inpelm,inpelm), &
                       Kgx(inpelm,ndim*inpelm), &
                       Kgg(inpelm,inpelm)
      double precision, pointer :: vecoldlc(:), vecoldlc1(:)
      integer, pointer :: index3n1(:)
      integer ipcoef, ipx, ipxg, ipw, ipphi, ipdphi, ipu, ipug, ipqmat, &
              iptang, ipwork, ifsi, ippsi, ipdpsidx, ipdphidxi, &
              ipdphideta, ipend, igeom
      save ipcoef, ipx, ipxg, ipw, ipphi, ipdphi, ipu, ipug, ipqmat, &
           iptang, ipwork, ifsi, ippsi, ipdpsidx, ipdphidxi, &
           ipdphideta, ipend, igeom

!     coor1          Coordinates in current configuration
!     coorn          Coordinates in reference configuration
!     dni            Change of indentor normal (in nodal points)
!     g              Contact distances (in nodal points)
!     gamma          Contact pressures (in nodal points)
!     i              Counting variable
!     ifsi           Sequence number of old solution vector for
!                    fully coupled fluid-structure interaction
!     igeom          Defines type of geometry for the rigid body
!     iload          Indicates the type of boundary condition.
!     ipcoef         Starting address in array work of coefficients
!     ipdphi         Starting address in array work of deriv. of basis func's
!     ipdphideta     Starting address of array dphideta
!     ipdphidxi      Starting address of array dphidxi
!     ipdpsidx       Starting address of array dpsidx
!     ipend          Last position in array
!     ipphi          Starting address in array work of basis func's
!     ippsi          Starting address in array work of psi
!     ipqmat         Starting address in array work of local rotation matrix
!     iptang         Starting address in array work of tangential load
!     ipu            Starting address in array work of nodal displacement
!     ipug           Starting address in array work of displ. in intgr. points
!     ipw            Starting address in array work of weighing fun's
!     ipwork         Starting address in array work of work array
!     ipx            Starting address in array work of element coordinates
!     ipxg           Starting address in array work of int. points
!     iseqgam        Sequence for contact pressure dof's
!     iseqnr         Sequence number with respect to the solution array
!     isequ          Sequence for displacement dof's
!     isub           Integer help parameter for do-loops to indicate the
!                    smallest coefficient number that is dependent on space
!     isup           Integer help parameter for do-loops to indicate the
!                    largest coefficient number that is dependent on space
!     j              Counting variable
!     jelp           Indication of the type of basis function subroutine
!                    must be called by ELxxxx. Possible vaues:
!     kgg            Element matrix
!     kgx            Element matrix
!     kxg            Element matrix
!     kxx            Element matrix
!     maxcoef        Maximum number of coefficients
!     maxnu          Maximum number of displacement dof's
!     ni             Normal of indentor (in nodal points)
!     niseqgam       Total length of array iseqgam
!     nisequ         Total length of array isequ
!     rhsg           Right hand side for contact pressure dof's
!     rhsx           Right hand side for displacment dof's
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL0211         Fills sequence of displacement and pressure dof's
!     EL3015         Compute contribution to element right hand side
!     EL3024         Compute element matrix for contact element
!     EL3034         Assemble element stiffness matrix for contact element
!     ELIND0         Fill array ICH in COMMON CELIAR for preceding solutions
!     ELM910BASEFN   Compute the basis functions phi, its derivatives and other
!                    related quantities for various element shapes
!                    (boundary elements)
!     ERCLoS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     FUNCMINDIST    calculate contact distances (user defined subroutine)
!     INSTOP         Stop the program
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays E%iuser and USER
!     Usually they are filled by FILCOF
!
!     The sequence is:
!
!    +-------+-------------------------------------------------------------+
!    | coef  | description                                                 |
!    +-------+-------------------------------------------------------------+
!    |   1   | coordinate system for boundary condition                    |
!    |       +---------+----+----------------------------------------------+
!    |                 |  1 | local system                                 |
!    |                 |  2 | local system and the boundary-integral is    |
!    |                 |    | linearised (non-symmetric matrix)            |
!    +-------+---------+----+----------------------------------------------+
!    |   2   | ifsi_solution (integer refers to sequence number of solution|
!    |       |                vector for fully coupled fluid-solid inter.) |
!    +-------+-------------------------------------------------------------+
!    |   3   | type of numerical integration                               |
!    |       +---------+----+----------------------------------------------+
!    |                 |  0 | default value, defined by element (NC)       |
!    |                 |  1 | Newton-Cotes                                 |
!    |                 |  3 | Gauss (not implemented)                      |
!    +-------+---------+----+----------------------------------------------+
!    |   4   | type of geometry for the rigid body                         |
!    +-------+-------------------------------------------------------------+
!    |   5   | not used                                                    |
!    +-------+-------------------------------------------------------------+
!    |   6   | not used                                                    |
!    +-------+-------------------------------------------------------------+
!    |   7   | not used                                                    |
!    +-------+-------------------------------------------------------------+
!    |   8   | not used                                                    |
!    +-------+-------------------------------------------------------------+
!    |   9   | not used                                                    |
!    +-------+-------------------------------------------------------------+
!    |  10   | not used                                                    |
!    +-------+-------------------------------------------------------------+
!    |  11   | not used                                                    |
!    +---------------------------------------------------------------------+
!    |    VECTORS NEEDED                                                   |
!    +---------------------------------------------------------------------+
!    | The following vectors are needed (in this order)                    |
!    +----+-----------+----------------------------------------------------+
!    |  1 |  u        |  incremental solution                              |
!    |  2 |  un       |  old solution at increment n                       |
!    |  3 |  usrvec   |  user defined vector  (if iusrvec>=1)              |
!    |  4 |  icont    |  help array for contact                            |
!    +----+-----------+----------------------------------------------------+
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ('elm211')

      if (ifirst==0) then

!     --- ifirst = 0,  compute element independent quantities

         icheld = 0
         call el0211 (E%iuser, E%user, jelp, isub, isup, work, iload, &
                      isequ, iseqgam, index4, ifsi, igeom)

!        --- Fill array ich (COMMON CELIAR) for preceding solutions

         call elind0 (E%islold, numold, isub, isup, E%iuser)
         elemvc = 0d0
         elemmt = 0d0

!        --- Fill pointers in array work

         maxnu=inpelm
         ipcoef     = 1
         ipx        = ipcoef     + maxnu*maxcoef
         ipxg       = ipx        + maxnu*ndim
         ipw        = ipxg       + maxnu*ndim
         ipphi      = ipw        + maxnu
         ipdphi     = ipphi      + maxnu*maxnu
         ipu        = ipdphi     + maxnu*maxnu*ndim
         ipug       = ipu        + maxnu*ndim
         ipqmat     = ipug       + maxnu*ndim
         iptang     = ipqmat     + maxnu*ndim*ndim
         ippsi      = iptang     + maxnu
         ipdpsidx   = ippsi      + maxnu*maxnu
         ipdphidxi  = ipdpsidx   + maxnu*maxnu*ndim
         ipdphideta = ipdphidxi  + maxnu*maxnu*ndim
         ipwork     = ipdphideta + maxnu*maxnu*ndim
         ipend      = ipwork     + maxnu*max(6,maxnu+1)

         if ( ipend>13800 ) then

!        --- Array work too small

            call errint ( ipend, 1 )
            call errint ( 13800, 2 )
            call errchr ( 'work', 1 )
            call errsub ( 550, 2, 0, 1 )
            go to 1000

         end if
         B%rule = irule
         B%cart = jcart
         E%ishape = jelp
         call sepelbasefnch

      end if

!     --- Set iseqnr according to the type of problem

      if ( ifsi>0 ) then
         iseqnr = ifsi
      else
         iseqnr = 1
      end if

!     --- Get coordinates of deformed mesh on increment n (coorn) and
!         compute current and initial coordinates (coor1)

      iseqtype = oldsols(iseqnr)%iseqtype
      index3n => indoldsols(iseqtype)%index3
      vecoldlc => oldsols(iseqnr)%oldvecs
      do i = 1, n
         do j = 1 , ndim
            coorn(j,i) = coor(j,index1(i))
            coor1(j,i) = coorn(j,i) + &
                         vecoldlc(index3n(isequ((i-1)*ndim+j)))
         end do
      end do

!     --- Get the contact pressures (in nodal points)

      iseqtype = oldsols(iseqnr+1)%iseqtype
      index3n1 => indoldsols(iseqtype)%index3
      vecoldlc1 => oldsols(iseqnr+1)%oldvecs
      iseqtype = oldsols(iseqnr)%iseqtype
      index3n => indoldsols(iseqtype)%index3
      vecoldlc => oldsols(iseqnr)%oldvecs
      do i = 1, n
         gamma(i) = vecoldlc(index3n(iseqgam(i))) + &
                    vecoldlc1(index3n1(iseqgam(i)))
      end do

!     --- Compute basis functions and get the element coordinates

      call elm910basefn ( coor1, work(ipx), work(ipw), work(ipdphi), &
                          work(ipxg), work(ipphi), index1, work(ipqmat), &
                          work(iptang), jelp, work(ippsi), &
                          work(ipdpsidx), work(ipdphidxi) )

      if (ierror/=0) goto 1000

!     --- Fill element vector if necessary

      if (vector .and. notvec/=1) then

!     --- vector = true, compute element vector

!         --- Get contact distance and derived quantities
!         At this moment, the quantities (e.g. gamma, g, ni and dni) are stored
!         in the nodal points. However, after the subroutine funcmindist the
!         quantities are assumed to be stored in the integration points.
!         This is allowed because Newton-Cotes integration is used and it is
!         done for consistency when calculating the element matrices and
!         right hand sides.

         call funcmindist ( coor1, gamma, index1, g, ni, dni, &
                            vecold, index3, igeom )

!        --- RHS for displacements (rhsx) and contact pressure (rhsg)

         call el3015 ( g, gamma, rhsx, rhsg, work(ipw), &
                       work(ipphi), ni )

!        --- assemble rhsx into large elemvc (take care of sign), &
!            based on isequ

         do i = 1, n*ndim
            elemvc(isequ(i)) = -rhsx(i)
         end do

!        --- assemble rhsg into large elemvc (take care of sign), &
!            based on iseqgam

         do i = 1, n
            elemvc(iseqgam(i)) = -rhsg(i)
         end do

         goto 999

 999  end if

!     --- Fill element matrix

      if (matrix .and. notmat/=1 .and. iload==2) then

!     --- Compute element matrices

         call el3024 ( g, coor1, gamma, Kxx, Kxg, Kgx, Kgg, &
                       work(ipw), work(ipphi), work(ipdphidxi), &
                       ni, dni )

!        --- assemble into large elemmt, based on isequ and iseqgam

         call el3034 (elemmt, Kxx, Kxg, Kgx, Kgg, &
                      isequ, iseqgam)

      end if

1000  if (ierror/=0) call instop

      call erclos ('elm211')

      end
