      subroutine elmkbl ( bl, dfgrad, dphidx, phi, xgauss, ugauss, &
                          ndim, nip, nnode )
! ======================================================================
!
!        programmer    Sander Gielen
!        version  2.1  date 14-09-2005 Roy Kerckhoffs, changed
!                                      blwork(6,3,8,*) into (6,3,nnode,nip)
!        version  2.0  date 21-01-1998 Marcel Rutten, added 2D axisymmetry:
!                                      call and  declaration of variables
!                                      adapted
!        version  1.2  date 21-01-1997 changed inner loop: -O3 optimize
!                                      did not work on all machines
!
!   copyright (c) 1996-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Setup the linear strain-displacement matrix for nonlinear analysis.
!   The matrix includes current deformation effects.
! **********************************************************************
!
!                       KEYWORDS
!
!     mechanics
!     non_linear
!     strain
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer          nip, nnode, ndim
      double precision bl(6,3,nnode,nip), dfgrad(nip,ndim,ndim), &
                       dphidx(nnode,nip,ndim), &
                       phi(nnode, nip), &
                       xgauss(nip, ndim), ugauss(nip, ndim)

!     bl         i    bl(i,j,k,l) m:intpt k:node j:dir i:comp
!     dfgrad     i    Reordered deformation tensor
!                     dfgrad(k,j,i) = f_ij(x^k)
!     dphidx     i    Array of length n x m x ndim  containing the derivatives
!                     of the basis functions in the sequence:
!                     d phi / dx (xg ) = dphidx (i,k,1);
!                          i        k
!                     d phi / dy (xg ) = dphidx (i,k,2);
!                          i        k
!                     d phi / dz (xg ) = dphidx (i,k,3);
!                          i        k
!                     If the element is a linear simplex, the
!                     derivatives are constant and only k = 1 is filled.
!     ndim       i    Dimension of the space.
!     nip        i    number of integration points
!     nnode      i    number of nodes
!     phi        i    array of length n * m, values of shape
!                     functions in integration points in the sequence:
!                            k
!                     phi (xg ) = phi (i,k)
!                        i
!                     array phi is only filled when gauss integration is applied
!     ugauss     i    Preceding solution in Gauss points
!     xgauss     i    array of length m x ndim containing the co-ordinates of
!                     the gauss points.  array xgauss is only filled when gauss
!                     integration is applied.
!                     xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                     zg_i = xgauss (i,3)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, l, intrule

!     i          Counting variable
!     intrule    Actual intergration rule
!     j          Counting variable
!     k          Counting variable
!     l          Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      intrule = mod(irule, 100)

      do l = 1,nip
         do k = 1,nnode
            do j=1,ndim

!           --- The first half of Bl is the same for all types of coordinate
!               systems:

               do i=1,ndim
                  bl(i,j,k,l)= dfgrad(l,i,j) * dphidx(k,l,i)

               end do

!              --- The second half of Bl is coordinate system dependent:

               if ( ndim==3 ) then

!              --- 3D Cartesian

                  bl(4,j,k,l)= dfgrad(l,1,j)* dphidx(k,l,2) &
                              +dfgrad(l,2,j)* dphidx(k,l,1)
                  bl(5,j,k,l)= dfgrad(l,2,j)* dphidx(k,l,3) &
                              +dfgrad(l,3,j)* dphidx(k,l,2)
                  bl(6,j,k,l)= dfgrad(l,3,j)* dphidx(k,l,1) &
                              +dfgrad(l,1,j)* dphidx(k,l,3)

               else if ( ndim==2 ) then

!              --- 2D, all coordinate systems:

                     bl(4,j,k,l)= dfgrad(l,1,j)* dphidx(k,l,2) &
                                 +dfgrad(l,2,j)* dphidx(k,l,1)

               end if
            end do

!           --- Fill circumferential part of Bl in case of axisymmetry:

            if ( jcart==2 ) then

               if (nip==1 .and. intrule/=2) then

!              --- Looks like we have reduced integration in a rectangular
!                  --- element
!                  elementon our hands: phi_r = 0.25d0
!                  ( see elp621.f for details )

                 bl(3,1,k,l) = (0.25d0/xgauss(l,1)) * &
                               (1d0 + ugauss(l,1)/xgauss(l,1))
               else

                 bl(3,1,k,l) = (phi(k,l)/xgauss(l,1)) * &
                               (1d0 + ugauss(l,1)/xgauss(l,1))
               end if
            end if

         end do
      end do
      end
