      subroutine elrmhy ( sig, dd )
! ======================================================================
!
!        programmer     Sander Gielen
!        version 1.0    date   02-03-1997
!
!
!
!   copyright (c) 1997-1997  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! **********************************************************************
!
!                       DESCRIPTION
!
!     Remove the hydrostatic parts from the stress tensor and the
!     stiffness tensor.
!     METHOD:
!        P4 = I4 - 1/3 I2 I2
!     4th order projection tensor P4 is composed of 4th order identity
!     tensor and 2nd order identity tensor, giving a deviatoric projection
!        sigd = P4 : sig
!     and
!        dd   = P4 : d : P4
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      double precision sig(6), dd(6,6)

!     dd     i/o   matrix with stiffness components
!     sig    i/o   vector with stress components
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision hyd, t1, t2, t3, t4, t5, t6, t7, t9, t10, &
                       t11, t12, t13, t14, t15, t16, t17, t18, t19, &
                       t20, t21, t22, t24, t25, t26, t27, &
                       t29, t30, t31

!     hyd
!     t1
!     t10
!     t11
!     t12
!     t13
!     t14
!     t15
!     t16
!     t17
!     t18
!     t19
!     t2
!     t20
!     t21
!     t22
!     t24
!     t25
!     t26
!     t27
!     t29
!     t3
!     t30
!     t31
!     t4
!     t5
!     t6
!     t7
!     t9
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The stress tensor and stiffness tensor are stored reduced.
!     The 4th order projection tensor P4 is rewritten in a 6x6 matrix.
!     So:
!        sigd[6] = p[6x6] * sig[6]
!     and:
!        dd[6x6] = p[6x6] * d[6x6] * p[6,6]
!     The code uses the fact that dd is a symmetric 6x6 matrix.
!     Because of the many zeros in the projection matrix, the multiplication
!     has been written out to get an efficient code.
! **********************************************************************
!
!                       MODULES USED
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- remove hydrostatics from stress

      hyd = (sig(1)+sig(2)+sig(3))/3d0
      sig(1) = sig(1) - hyd
      sig(2) = sig(2) - hyd
      sig(3) = sig(3) - hyd

!     --- remove hydrostatics from stiffness

      t1 = 1d0/9d0
      t2 = t1*dd(1,1)
      t3 = t1*dd(1,2)
      t4 = t1*dd(1,3)
      t5 = t1*dd(2,2)
      t6 = t1*dd(2,3)
      t7 = t1*dd(3,3)
      t9 = -2*t2+5*t3-t4-2*t5-t6+t7
      t10 = -2*t2-t3+5*t4+t5-t6-2*t7
      t11 = dd(1,4)/3d0
      t12 = dd(2,4)/3d0
      t13 = dd(3,4)/3d0
      t14 = 2*t11-t12-t13
      t15 = dd(1,5)/3d0
      t16 = dd(2,5)/3d0
      t17 = dd(3,5)/3d0
      t18 = 2*t15-t16-t17
      t19 = dd(1,6)/3d0
      t20 = dd(2,6)/3d0
      t21 = dd(3,6)/3d0
      t22 = 2*t19-t20-t21
      t24 = t2-t3-t4-2*t5+5*t6-2*t7
      t25 = -t11+2*t12-t13
      t26 = -t15+2*t16-t17
      t27 = -t19+2*t20-t21
      t29 = -t11-t12+2*t13
      t30 = -t15-t16+2*t17
      t31 = -t19-t20+2*t21

      dd(1,1) = 4*t2-4*t3-4*t4+t5+2*t6+t7
      dd(1,2) = t9
      dd(1,3) = t10
      dd(1,4) = t14
      dd(1,5) = t18
      dd(1,6) = t22
      dd(2,1) = t9
      dd(2,2) = t2-4*t3+2*t4+4*t5-4*t6+t7
      dd(2,3) = t24
      dd(2,4) = t25
      dd(2,5) = t26
      dd(2,6) = t27
      dd(3,1) = t10
      dd(3,2) = t24
      dd(3,3) = t2+2*t3-4*t4+t5-4*t6+4*t7
      dd(3,4) = t29
      dd(3,5) = t30
      dd(3,6) = t31
      dd(4,1) = t14
      dd(4,2) = t25
      dd(4,3) = t29

!     --- dd(4,4) = dd(4,4)
!         dd(4,5) = dd(4,5)
!         dd(4,6) = dd(4,6)

      dd(5,1) = t18
      dd(5,2) = t26
      dd(5,3) = t30

!     --- dd(5,4) = dd(4,5)
!         dd(5,5) = dd(5,5)
!         dd(5,6) = dd(5,6)

      dd(6,1) = t22
      dd(6,2) = t27
      dd(6,3) = t31

!     --- dd(6,4) = dd(4,6)
!         dd(6,5) = dd(5,6)
!         dd(6,6) = dd(6,6)

      end
