      subroutine sepeldf2gls ( glstr, nip, ndim, jcart )
! ======================================================================
!
!        programmer     Sander Gielen
!        version 1.2    date   21-01-1998 Marcel Rutten,added 2D axisymmetric
!     coordinate
!        version 1.1    date   12-11-1997 removed detf computation
!        version 1.0    date   28-03-1996
!
!
!
!   copyright (c) 1996-1998  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! **********************************************************************
!
!                       DESCRIPTION
!
!     Help routine for nonlinear solid analysis
!     it computes Green-Lagrange strains from the deformation gradient
! **********************************************************************
!
!                       KEYWORDS
!
!    mechanics
!    non_linear
!    strain
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nip, ndim, jcart
      double precision glstr(6,nip)

!     glstr      o    Green-Lagrange strains e(i,k)
!                     k: intpt; i: component
!     f          i    Deformation gradient f(k,j,i) = F_ij( x^k)
!     jcart      i    indication whether the element is a triangle with
!                     cartesian coordinates (1)
!                     or a circular triangle with cylinder symmetric
!                     coordinates (2)
!     ndim       i    Dimension of the space.
!     nip        i    number of integration points
!     ugauss     i    Preceding solution in Gauss points
!     xgauss     i    array of length m x ndim containing the co-ordinates of
!                     the gauss points.  array xgauss is only filled when gauss
!                     integration is applied.
!                     xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                     zg_i = xgauss (i,3)
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k
      double precision, pointer :: f(:,:,:), ugauss(:,:), xgauss(:,:)

!     i          Counting variable
!     j          Counting variable
!     k          Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!                   c
!            E = ( F *F - I )/2
!
!            E  = ( F  F  - delta  )/2 = (f(p,i,k)*f(p,j,k) - delta(i,j))/2
!             ij     ki kj       ij
!
!            detf = DET(F)
!
!     As E is symmetric the components are stored in the sequence
!     glstr(1,k) = E11;       glstr(2,k) = E22;       glstr(3,k) = E33
!     glstr(4,k) = E12 = E21; glstr(5,k) = E23 = E32; glstr(6,k) = E13 = E31
! **********************************************************************
!
!                       MODULES USED
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      f => E%dfgrad
      ugauss => E%ugaussloc
      xgauss => E%xgaussloc

      do k=1,nip

        if ( ndim.eq.3 ) then

!       --- 3D Cartesian: principal and shear strains:

          do j=1,3
            glstr(j,k) = ( f(k,j,1)*f(k,j,1) + f(k,j,2)*f(k,j,2) + &
                       f(k,j,3)*f(k,j,3) - 1d0 )*0.5d0

            i = j+1
            if ( j.eq.3 ) i = 1
            glstr(j+3,k) = ( f(k,j,1)*f(k,i,1) + f(k,j,2)*f(k,i,2) + &
                         f(k,j,3)*f(k,i,3) )*0.5d0
          end do

        else if ( ndim.eq.2 ) then

!       --- 2D principal strains:

          do j = 1, 2

            glstr(j,k) = ( f(k,j,1)*f(k,j,1) + f(k,j,2)*f(k,j,2) &
                       - 1d0 )*0.5d0
          end do

!         --- 2D shear strains, coordinate system dependent:

          if ( jcart.eq.1 )  then

!         --- Cartesian plane strain:

            glstr(3,k) = ( f(k,1,1)*f(k,2,1) + f(k,2,2)*f(k,1,2))*0.5d0

!           --- Axisymmetric:

          else if ( jcart.eq.2 ) then
             glstr(3,k) = ((ugauss(k,1)*ugauss(k,1))/(xgauss(k,1)* &
                        xgauss(k,1))+ &
                     2d0*ugauss(k,1)/xgauss(k,1))*0.5d0
             glstr(4,k) = ( f(k,1,1)*f(k,2,1) + f(k,2,2)*f(k,1,2))*0.5d0
          end if

        end if

      end do

      end
