      subroutine sepelm256
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-08-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix and vector for shell problems
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elasticity_equation
!     parabolic_equation
!     thick_plate
!     plate
!     bending
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPEL0256      Fill element independent quantities for element type 256
!     SEPELBASEFUNC  Compute information about basis functions
!     SEPELDEALLOC   Deallocate coefficients and basis functions for element
!                    subroutines
!     SEPELFILLVAR   Fill space dependent coefficients
!     SEPELMSHELL    Compute element matrix for shell element
!     SEPELOLDVECS   Fill old vectors for element
!     SEPELP105      Compute information of extra basis functions for
!                    shell element
!     SEPELVSHELL    Compute element vector for shell element
!     SEPFILLDMAT    Fill D-matrix for linear elasticity problem
!     SEPPRINELMAT   Print element matrix and vector
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays IUSER and USER
!     Usually they are filled by FILCOF
!
!     The sequence is:
!
!     1:  This parameter is no longer in use.
!         The parameter imas in BUILD defines the type of mass matrix
!         Now the parameter is reserved for future purposes
!
!     2:  Type of stress-strain relations. This parameter is equal to IGPROB-1
!         Possible values:
!         0: isotropic material
!     3:  Type of numerical integration
!         Possible values:
!         0: default value, defined by element (2x2 Gauss)
!
!     4:  Type of load
!         Possible values:
!         0: default value, distributed load in normal direction
!         1: distributed load in Cartesian co-ordinates
!
!     5:  Not yet used (must be 0)
!
!     6-26: Information about the elasticity matrix depending on the
!           value of IGPROB
!         IGPROB = 1
!            6:  E
!            7:  nu
!    27:  h (thickness of the shell)
!         Default value 1
!
!    28:  F1  (distributed load in 1-direction or normal dir)
!    29:  F2  (distributed load in 2-direction)
!    30:  F3  (distributed load in 3-direction)
!
!    31:  epsx 0  (or epsr 0)             initial strain (not yet used)
!    32:  epsy 0  (or epsz 0)
!    33:  epsz 0  (or epstheta 0)
!    34:  gammaxy 0  (or gammarz 0)
!    35:  gammayz 0
!    36:  gammazx 0
!
!    37:  sigmax 0  (or sigmar 0)         initial stress (not yet used)
!    38:  sigmay 0  (or sigmaz 0)
!    39:  sigmaz 0  (or sigmatheta 0)
!    40:  tauxy 0   (or taurz 0)
!    41:  tauyz 0
!    42:  tauzx 0
!
!    43:  rho (parameter for time-derivative)
!         Default: 1
!
!    44:  Not yet used (must be 0)
!    45:  Not yet used (must be 0)
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Method
!
!     Each point contains five unknowns:
!
!     u,v,w:    displacements
!     theta_1:  first component of rotation vector
!     theta_2:  second component of rotation vector
!
!     The stiffness matrix is 5 x 5
!
!     This matrix is defined by the relation:
!
!        k = (B)^T D B
!
!     The B submatrix is constructed by rotations and is not simple since
!     the curvature plays an important role
!
!     The element uses two local coordinate systems:
!     The lamina coordinate system and
!     the fiber coordinate system
!
!     For an isotropic material we have
!
!                      |   1   nu    0          0         0    |
!     D = E / (1-nu^2) |  nu    1    0          0         0    |
!                      |   0    0  (1-nu)/2     0         0    |
!                      |   0    0    0      k(1-nu)/2      0   |
!                      |   0    0    0          0     k(1-nu)/2|
!
!
!     k is a constant to account for the actual nonuniformity of the
!     shearing stress
!     Following Huges we set k=5/6
!
!     In the T1 element the shear stress part of the B-matrix is adapted
!     as described in
!     T.J.R. Hughes and T.E. Tezduyar, Finite elements based upon Mindlin
!     plate theory with particular reference to the four-node bilinear
!     isoparametric element, Journal of applied Mechanics, sept 1981, vol 48, &
!     pages 587-596
!
!     See subroutine EL8111 for the details
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelm256' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelm256'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      E%mat = 0d0
      E%vec = 0d0

!     --- Define element independent quantities
!         only for first element

      if ( E%first==0 ) call sepel0256
      if ( ierror/=0 ) go to 1000

!     --- Fill basis functions

      call sepelbasefunc
      if ( ierror/=0 ) go to 1000

!     --- Fill old vectors in quadrature points if necessary

      if ( E%compold>0 ) call sepeloldvecs

      if ( debug ) then
         write(irefwr,1) 'isupcfvar, isubcfvar', E%isupcfvar, E%isubcfvar
         write(irefwr,*) 'B%fillqmat ', B%fillqmat
      end if  ! ( debug )

!     --- Fill variable coefficients if necessary

      if ( E%isupcfvar>=E%isubcfvar ) call sepelfillvar

!     --- Fill fiber system

      call sepelp105

!     --- Compute D matrix if not constant''

      if ( .not. E%dmatconst ) call sepfilldmat

!     --- Fill element matrix

      if ( matrix ) call sepelmshell

!     --- Fill element vector

      if ( vector .and. E%notvec==0 ) call sepelvshell

!     --- Remove basis functions and coefficients in case of last call

      if ( debug ) write(irefwr,1) 'last', E%last
      if ( E%last==1 ) then
         call sepeldealloc
         deallocate ( E%Dmat, E%Bsh, E%Bs, B%xhat, B%qfib, B%fibjac, &
                      E%subvec )
      end if  ! ( E%last==1 )

1000  if ( debug ) then

!     --- Debug information

         call sepprinelmat
         write(irefwr,*) 'End sepelm256'

      end if  ! ( debug )
      call erclos ( 'sepelm256' )

      end subroutine sepelm256
