      subroutine sepfillbsmat
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-08-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill matrix B^s for plate problem
!     This matrix is made for the T1 element only
!     Old subroutine el8111
! **********************************************************************
!
!                       KEYWORDS
!
!     elasticity_equation
!     thick_plate
!     plate
!     bending
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleconsts
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: e1(4,2), e2(4,2), h(4), alpha(4), g(4,2)
      double precision :: fac1(4), fac2(4)
      double precision, pointer :: x(:,:), phi(:,:), bs(:,:,:,:)
      integer :: k, j

!     alpha          contains inner product of e1 and e2
!     bs             Matrix B^s
!     e1             x(i+1)-x(i)/||x(i+1)-x(i)||
!     e2             x(i)-x(i-1)/||x(i)-x(i-1)||
!     fac1           Multiplication factor
!     fac2           Multiplication factor
!     g              See method
!     h              Help variable to store a constant temporarily
!     icyc           icyc4p(i) = i+1 (cyclic)
!     icyc4m          icyc4m(i) = i-1 (cyclic)
!     j              Counting variable
!     k              Counting variable
!     phi            array of length n * m, values of shape
!                    functions in quadrature points in the sequence:
!                    phi_i(xg^k) = phi (i,k)
!                    array phi is only filled when gauss quadrature is
!                    applied
!     x              coordinates of element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   The matrix b^s is defined as described in
!
!     T.J.R. Hughes and T.E. Tezduyar, Finite elements based upon Mindlin
!     plate theory with particular reference to the four-node bilinear
!     isoparametric element, Journal of applied Mechanics, sept 1981, vol 48, &
!     pages 587-596
!
!    Consider B^s as a matrix with 2 rows
!
!    Define:
!
!    e1(i) = x(i+1)-x(i)/||x(i+1)-x(i)||  cyclic, 2 components, i=1(1)4
!    e2(i) = x(i)-x(i-1)/||x(i)-x(i-1)||  cyclic, 2 components
!    h(i) = ||x(i+1)-x(i)||   scalar
!    alpha(i) = e1(i) . e2(i)     scalar
!    G(i) = 1/(1-alpha(i)^2) phi(i) (e1(i)-alpha(i)e2(i)) - &
!           1/(1-alpha(i+1)^2) phi(i+1) (e2(i+1)-alpha(i+1)e1(i+1))
!
!    Bs(1,i+1) = G(i)/h(i) - G(i+1)/h(i+1)
!    Bs(2,i+1) = (G(i)e2(i+1)_2 - G(i+1)e1(i+1)_2)/2
!    Bs(3,i+1) =-(G(i)e2(i+1)_1 - G(i+1)e1(i+1)_1)/2
!
!   Mark that the definition of theta differs from the Hughes definition in
!   case of plates
!   It is however, the same as in the case of shells
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillbsmat' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillbsmat'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      x => B%xyz
      phi => B%phi
      bs => E%Bs

!     --- h(i) = ||x(i+1)-x(i)||   scalar
!         e1(i) = x(i+1)-x(i)/||x(i+1)-x(i)||  cyclic, 2 components, i=1(1)4

      e1(:,1) = x(icyc4p(:),1)-x(:,1)
      e1(:,2) = x(icyc4p(:),2)-x(:,2)
      h(:) = 1d0/sqrt(e1(:,1)**2+e1(:,2)**2)
      e1(:,1) = e1(:,1)*h(:)
      e1(:,2) = e1(:,2)*h(:)

!     --- e2(i) = x(i)-x(i-1)/||x(i)-x(i-1)||  cyclic, 2 components

      e2(:,1) = -e1(icyc4m(:),1)
      e2(:,2) = -e1(icyc4m(:),2)

!     --- alpha(i) = e1(i) . e2(i)     scalar

      alpha(:) = e1(:,1)*e2(:,1) + e1(:,2)*e2(:,2)

      do k = 1, m

!     --- G(i) = 1/(1-alpha(i)^2) phi(i) (e1(i)-alpha(i)e2(i)) - &
!                1/(1-alpha(i+1)^2) phi(i+1) (e2(i+1)-alpha(i+1)e1(i+1))

         fac1 =  1d0/(1d0-alpha(:)**2)*phi(:,k)
         fac2 =  1d0/(1d0-alpha(icyc4p(:))**2)*phi(icyc4p(:),k)
         g(:,1) = fac1*(e1(:,1)-alpha(:)*e2(:,1)) - &
                  fac2*(e2(icyc4p(:),1)-alpha(icyc4p(:))*e1(icyc4p(:),1))
         g(:,2) = fac1*(e1(:,2)-alpha(:)*e2(:,2)) - &
                  fac2*(e2(icyc4p(:),2)-alpha(icyc4p(:))*e1(icyc4p(:),2))

!        --- Bs(1,i+1) = G(i)/h(i) - G(i+1)/h(i+1)
!            Bs(2,i+1) = (G(i)e2(i+1)_2 - G(i+1)e1(i+1)_2)/2
!            Bs(3,i+1) =-(G(i)e2(i+1)_1 - G(i+1)e1(i+1)_1)/2

         do j = 1, 2
            bs(j,1,icyc4p(:),k) = g(:,j)*h(:) - g(icyc4p(:),j)*h(icyc4p(:))
            bs(j,2,icyc4p(:),k) = (g(:,j)*e2(icyc4p(:),2) - &
                                g(icyc4p(:),j)*e1(icyc4p(:),2))*0.5d0
            bs(j,3,icyc4p(:),k) =-(g(:,j)*e2(icyc4p(:),1) - &
                                g(icyc4p(:),j)*e1(icyc4p(:),1))*0.5d0
         end do

      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepfillbsmat'

      end if  ! ( debug )
      call erclos ( 'sepfillbsmat' )

      end subroutine sepfillbsmat
