!    ***************************************************************
!    *
!    *  oil radial bearing
!    *  boundary  1,2      no flow  (half bearing)
!    *  boundary  5,6      essential boundary conditions
!    *  boundary  3,4 <--> boundary 7,8  periodical boundary conditions
!    *
!    *  Start: Reynolds equation without cavitation
!    *  Second part:  Solution including cavitation constraints
!    *
!    *  The cavitation is taken into account by an iterative
!    *  solution method with constraints
!    *
!    ***************************************************************
!
      program bearing2

!     --- standard declarations

      implicit none
      integer lnmesh, lnprob, lniusr, lnuser
      parameter ( lnmesh=200, lnprob=200, lniusr=100, lnuser=100)
      integer kmesh(lnmesh), kprob(lnprob), intmat(5), matr(5), isol(5),
     +        islold(5), irhsd(5), iuser(lniusr), iwork(3), iinbld(1),
     +        iprob, massmt, ipos, nmax
      double precision user(lnuser), work(1), eps
      logical cavity

!     --- Clear arrays

      kmesh = 0
      kprob = 0
      intmat = 0
      matr = 0
      isol = 0
      islold = 0
      irhsd = 0
      iuser  = 0
      user = 0d0

!     --- Set length in first positions

      kmesh(1) = lnmesh
      kprob(1) = lnprob
      iuser(1) = lniusr
      user(1)  = lnuser

!     --- Start SEPRAN, read mesh

      call sepstr ( kmesh, kprob, intmat )

!     --- fill essential boundary conditions

      call presdf ( kmesh, kprob, isol )

!     --- Build the system of equations and define the parameters

      iprob = 1
      call filcof ( iuser, user, kprob, kmesh, iprob )
      iinbld(1) = 0
      call build ( iinbld, matr, intmat, kmesh, kprob, irhsd,
     +             massmt, isol, islold, iuser, user )

!     --- solve the system of equations
!         cavity is not taken into account
!         matrix is positive definite

      ipos = 1
      call solve ( ipos, matr, isol, irhsd, intmat, kprob )

!     --- compute and print load

      cavity = .false.
      call load ( isol, kmesh, kprob, cavity )

!     --- second part: overrelaxation with constraints in order to take
!         cavitaion into account
!         As initial value the solution from the first part (without cavitation)
!         boundary condition is used.
!         redefine type of method for subroutine overcs
!         and rebuild system of equations

      call commat ( 9, kmesh, kprob, intmat )

      call build ( iinbld, matr, intmat, kmesh, kprob, irhsd,
     +             massmt, isol, islold, iuser, user )

!     --- solve system of equations with the aid of subroutine overcs
!         constrained optimalisation (p>=0)

      iwork(1) = 1
      iwork(2) = 1
      iwork(3) = 1
      work(1)  = 0d0
      eps      = 1d-3
      nmax     = 400

      call overcs ( 1, matr, irhsd, isol, intmat, kmesh, kprob, eps,
     +              nmax, iwork, work )

!     --- compute and print load

      cavity = .true.
      call load ( isol, kmesh, kprob, cavity )

!     --- write result for postprocessings purposes

      call output ( kmesh, kprob, isol )

      end

!     --- function funccf for the computation of h and the
!         arguments in the integral (sin(2x/D) resp cos (2x/D))

      function funccf ( ichois, x, y, z )
      implicit none
      double precision funccf, x, y, z
      integer ichois

      double precision diam, deltar, eps, phi

      diam   = 50d-3
      deltar = 8.2d-5
      eps    = .4d0
      phi    = 2d0*(x/diam)

      if ( ichois.eq.1 ) then

!     --- h(x)   minimal film thickness at x=0

         funccf = deltar*(1d0-eps*cos(phi))

      else if ( ichois.eq.2 ) then

!     --- x-component load

         funccf = -cos(phi)

      else

!     --- y-component load

         funccf = -sin(phi)

      end if

      end

!     --- subroutine load computes and prints the load and the attitude angle

      subroutine load ( isol, kmesh, kprob, cavity )
      implicit none
      integer isol(*), kmesh(*), kprob(*)
      logical cavity
      integer lnuser
      parameter (lnuser=100)
      integer iuser1(lnuser), iuser2(lnuser)
      double precision user1(lnuser), user2(lnuser)
      save iuser1, user1, iuser2, user2

      include 'SPcommon/cmcdpi'

      double precision phi, ftot, fx, fy, outval(3)
      integer iinvol(3), iprob

      data iuser1 /lnuser*0/, iuser2 /lnuser*0/, user1 /lnuser*0d0/,
     +     user2 /lnuser*0d0/

!     --- fill parameters for subroutine volume to compute the integrals

      iuser1(1) = lnuser
      iuser2(1) = lnuser
      user1(1)  = lnuser
      user2(1)  = lnuser
      iprob = 1
      if ( .not. cavity ) call filcof ( iuser1, user1, kprob, kmesh,
     +     iprob )
      iinvol(1) = 3
      iinvol(2) = 2
      iinvol(3) = 1
      call integr ( iinvol, outval, kmesh, kprob, isol, iuser1, user1 )
      fx = outval(1)

      if ( .not. cavity ) call filcof ( iuser2, user2, kprob, kmesh,
     +     iprob )

      call integr ( iinvol, outval, kmesh, kprob, isol, iuser2, user2 )
      fy = outval(1)

      ftot = sqrt(fy*fy+fx*fx)
      phi  = atan(fy/fx)
      if ( cavity ) then
         write ( irefwr, 100 )
100      format ( ' load with effect of cavity' )
      else
         write ( irefwr, 110 )
110      format ( ' load without effect of cavity' )
      end if
      write ( irefwr, 200 ) fx, fy, ftot, phi
200   format(' horizontal component of load',ss,1pe12.3e2/
     +       ' vertical component of load  ',ss,1pe12.3e2/
     +       ' modulus of load             ',ss,1pe12.3e2/
     +       ' attitude angle              ',ss,1pe12.3e2)
!
      end
