c **********************************************************************
c
c *   Solution of the elastohydrodynamic lubrication of an oil pumping
c *   ring seal by SEPRAN
c *
c *   Time-dependent problem
c *
c *   At t=0 the non-linear stationary equations are solved by a Newton
c *   iteration
c *   For t>0 a modified Crank-Nicolson scheme is used.
c *   The non-negative pressure condition is imposed by subroutine OVERCS
c *
c *
c *   Programmers:  Kees Kassels and Guus Segal
c *
c **********************************************************************
c
      program pump
      use sepmoduleoldrouts
      use sepmodulesolve
      use sepmoduletime
      implicit none
      character *80 text
      integer :: kmesh,kprob,intmat,matr,isol,
     .           islold,irhsd,iuser(500), iu1(3),iwork(10)
      double precision :: user(100), u1(3),work(10), u2

      double precision :: a, b, eps, rinput, p, q, pi, value
      integer :: ichois, ichvc, idegfd, idim, ielhlp, iinput, ipnt1,
     +           ipnt2, ipoint, iprint, irotat, istart, istop, istep,
     +           itertp, iu2, jmetod, jstap, maxitr, nstep, ivec
c
c     standard declarations
c
*     itest=0
      kmesh=0
      kprob=0
      iuser = 0
      user=0d0
      intmat = 0
      matr = 0
      isol = 0
      islold = 0
      irhsd = 0
      iuser(1) =500
      user(1)  =100
c
c **********************************************************************
c
c     start sepran
c
      print *,'PROGRAM PUMP'
      istart=0
      irotat=1
      ioutp=0
      itime=0
      call start(istart,irotat,ioutp,itime)
c
c *   mesh generation  input from standard input
c
      ichois=0
      call mesh(ichois,iinput,rinput,kmesh)
c
c *   problem definition  input from standard input
c
      ichois=0
      call probdf(ichois,kprob,kmesh,iinput)
c
c *   define type of solution method
c     jmetod = 2,  direct method, non-symmetrical matrix
c
      jmetod=2
      call commat(jmetod,kmesh,kprob,intmat)
c
c *   essential boundary conditions
c     First all essential boundary conditions are made equal to zero,
c     then the non-zero essential boundary conditions are set (user point 1)
c
      idim=1
      value=6d0
      ipnt1=1
      ipnt2=1
      idegfd=3
      jstap=0
      call bvalue(ichois,idim,kmesh,kprob,isol,value,ipnt1,ipnt2,idegfd,
     .            jstap)
c
c **********************************************************************
c
c *   Compute initial condition at velocity 0 by iteration
c     Start with displacement 0 and linear pressure (function func)
c
c     Set initial time
c
      t0=0
      t=t0
c
c *   Start value
c
      iu1(1)=0
      iu1(2)=0
      iu1(3)=1
      u1(1)=0d0
      u1(2)=0d0
      ichvc=1
      call creavc(ichois,ichvc,ivec,isol,kmesh,kprob,iu1,u1,iu2,u2)
c
c *   iteration by nonlin
c
      itertp=1
      maxitr=10
      eps=1d-3
      iprint=0
      ichois=-1
      call nonlin(itertp,maxitr,eps,iprint,iwork,work,intmat,ichois,
     .            iuser,user,kmesh,kprob,isol,ielhlp)
c
c *  print solution
c
      call prinrv(isol,kmesh,kprob,4,0,'initial condition')
c
c **********************************************************************
c
c *   Compute time-dependent solution with non-constant velocity
c
c     Set time parameters ( two strokes will be computed )
c
      omega=151.84d0
      pi=4d0*atan(1d0)
      tend=4d0*pi/omega
      nstep=80
      tstep=(tend-t0)/nstep
      theta = 0.5d0
c
c *   define type of matrix for subroutine OVERCS (jmetod=9)
c
      jmetod=9
      call commat(jmetod,kmesh,kprob,intmat)
c
c *   copy solution in islold
c
      call copyvc(isol,islold)
c
c
c *   Time iteration by Crank-Nicolson (theta=.5)
c     nstep time steps are carried out
c
      do 100 istep=1,nstep
         t=t+0.5d0*tstep
c
c *     Fill coefficients for differential equation at t+tstep*theta
c
         call filusr(iuser,user,kprob)
c
c *     Build matrix and right-hand side at t+tstep*theta
c
         ichois=-1
         call systm0(ichois,matr,intmat,kmesh,kprob,irhsd,
     .               isol,iuser,user,islold,ielhlp)
c
c *     Solve system of equations by subroutine OVERCS at t+tetha tstep
c
c       Parameters in iwork:
c
c        1:  Degree of freedom that must be limited (3)
c        2:  Indication of type of constraint (1 = p(x)>=a(x))
c        3:  Information of a(x)  (0: a(x)=0)
c        4:  Information of upper bound (Not of interest)
c
         iwork(1)=3
         iwork(2)=1
         iwork(3)=0
         iwork(4)=0
         eps=1d-1
         nmax=1000
c
c *     The overrelaxation process is influenced by the parameters
c       niter1, alam and omega in the common block citer
c
         niter1=-2
         alam=.99d0
         omega=1.6d0
         call overcs(1,matr,irhsd,isol,intmat,kmesh,kprob,eps,
     .               nmax,iwork,work)
c
c *      compute solution at t+tstep  (theta=.5)
c
c        u(n+1) = 2 u(n+1/2) - u(n)
c
         ichois=3
         idegfd=0
         a=2d0
         b=-1d0
         call algebr(ichois,idegfd,isol,islold,islold,kmesh,kprob,a,b,
     .               p,q,ipoint)
         t=t+.5d0*tstep
c
c *     Output of the solution at 10, 20, 30, ... ,nstep steps
c
c *     Of course more output may be printed
c
         if(10*(istep/10).eq.istep) then
c
            text(1:15)='solution at t ='
            write(text(16:27),10) t
 10         format(ss,1pe12.5e2)
            call prinrv(isol,kmesh,kprob,4,0,text(1:27))
         endif
100   continue
c
c
c **********************************************************************
c
c   *  end of program  stop sepran
c
      istop=0
      print *,'END PROGRAM PUMP'
      call finish(istop)
      end
c
c
c
c **********************************************************************
c
c     function func for the computation of the linear start pressure
c
      function func(ichois,r,z,dummy)
      implicit double precision (a-h,o-z)
c
      func=max(0d0,6d0*(3.5d3-z)/3.5d3)
      end
c
c **********************************************************************
c
c     subroutine filnln to define the non-linear system of equations to
c     be solved
c     The parameters for differential equation and natural boundary conditions
c     are stored with help of subroutine filusr
c
      subroutine filnln(niter,iuser,user,kmesh,kprob,isol,relax,differ)
      implicit double precision (a-h,o-z)
      integer :: iuser(*),kmesh,kprob,isol
      double precision :: user(*)
c
      if(niter.eq.0) then
         call filusr(iuser,user,kprob)
      endif
      end
c
c **********************************************************************
c
c     subroutine filusr to define the parameters for the equations to
c     be solved depending on time
c
      subroutine filusr(iuser,user,kprob)
      use sepmoduletime
      implicit double precision (a-h,o-z)
      integer :: iuser(*),kprob,iwork(50)
      double precision :: user(*),work(50)
c
      if(t.eq.t0) then
c
c *     t = t0:  u=0  time-independent
c
         u=0d0
         linear=2
      else
c
c *     t > t0:  u sinus function   time-dependent
c
         linear=4
         omega=151.84d0
         u=3.492d6*(sin(omega*t))
      endif
c
c *   element group 1: Elastic Reynolds element
c
c     arrays iwork and work;  information is stored in iuser and user
c
c     1:  diameter of the rod diam
c     2:  dynamic viscosity visc
c     3:  constant k
c     4:  reference pressure p0
c     5:  velocity u
c     6:  parameter linear
c     7:  theta delta t
c
      visc=.0278d-6
      diam=12d+3
      do 100 i=1,7
         iwork(i)=0
100      work(i)=0d0
      work(1)=diam
      work(2)=visc
      work(5)=u
      iwork(6)=linear
      work(7)=0.5d0*tstep
      nparm=7
      ielgrp=1
      call fil100(ielgrp,iuser,user,kprob,nparm,iwork,work)
c
c *   element group 2: External boundary load  see group 2
c
c     arrays iwork and work;  information is stored in iuser and user
c
c     1:  type of problem ( 3 = axisymmetric stress )
c     2:  ILOAD           ( 1 = load in co-ordinate direction )
c     3:  Tr              ( load in r-direction )
c     4:  Tz              ( load in z-direction )
c
      igprob=3
      iload=1
      tr=0d0
      tz=6d0
c
      iwork(1)=igprob
      iwork(2)=iload
      work(3)=tr
      work(4)=tz
      nparm=4
      ielgrp=2
      call fil100(ielgrp,iuser,user,kprob,nparm,iwork,work)
c
c *   element group 3: External boundary load  see group 2
c
      tr=-11d0
      tz=0d0
c
      work(3)=tr
      work(4)=tz
      ielgrp=3
      call fil100(ielgrp,iuser,user,kprob,nparm,iwork,work)
c
c *   element group 4: Elasticity element
c
c     arrays iwork and work;  information is stored in iuser and user
c
c     2:  type of problem ( 2 = axisymmetric stress )
c     6:  Young's modulus E
c     7:  Poisson ration nu
c    28:  Body force  Fx
c    29:  Body force  Fy
c
      iwork = 0
      work = 0d0
      e=5.27d4
      vnu=.44d0
      fx=0d0
      fy=0d0
      iwork(2)=2
      work(6)=e
      work(7)=vnu
      work(28)=fx
      work(29)=fy
      nparm=45
      ielgrp=4
      call fil100(ielgrp,iuser,user,kprob,nparm,iwork,work)
      end
