/*
 *  Copyright (c) 1991  Philips Export B.V.
 *
 *  Philips Research Laboratories
 *  Eindhoven, The Netherlands
 *
 *  Author    : G.F.M. Haenen
 *              F.W.G.M. Wouters
 *
 *  Modified  : Kees Lemmens, June 9, 1999
 *            - fixed compiler warnings
 *            - window argument was a double array in clip.c
 *              but a struct in other sources !!
 *
 *  Date      : August, 1995, Ronald Aarts, correction heading clip_polygon
 *  Date      : July, 1991
 *
 *  Function  : clipping routines
 *
 *  Comment   :
 *
*/

#include <stdio.h>
#include "plotfun.h"

#define INSIDE    0
#define BELOW     1
#define ABOVE     2

static void append_point(double x,double y,double polygon_x[],
                         double polygon_y[],long *polygon_p)
{
  if (*polygon_p == 0)    /* No previous point in array */
    {
      polygon_x[0] = x;
      polygon_y[0] = y;
      (*polygon_p)++;
    }
  else    /* Previous point in array */
    {
      if (polygon_x[*polygon_p-1] != x ||
          polygon_y[*polygon_p-1] != y)  /* Point must be appended */
        {
          if (*polygon_p == MAXPOLY)
            {
              fprintf(stderr, "Maximum of %d polygon points exceeded\n",
                      MAXPOLY);
              exit(1);
            }
          else
            {
              polygon_x[*polygon_p] = x;
              polygon_y[*polygon_p] = y;
              (*polygon_p)++;
            }
        }
    }
}

static long locate_value(double value,double lower,double upper)
{
   if (value < lower)
     return (BELOW);
  else if (value > upper)
     return (ABOVE);
   else
     return (INSIDE);
}

static long clipx(double window_xl,double window_xh,double *x1,double *y1,
                  double *x2, double *y2)
{
  long loc_1 = locate_value(*x1, window_xl, window_xh);
  long loc_2 = locate_value(*x2, window_xl, window_xh);

  if (loc_1 == loc_2)
    return(loc_1 == INSIDE);
  else
    {
      if (loc_1 == BELOW)
        {
          *y1 = *y1 + (*y2 - *y1) * (window_xl - *x1) / (*x2 - *x1);
          *x1 = window_xl;
        }
      else if (loc_1 == ABOVE)
        {
          *y1 = *y1 + (*y2 - *y1) * (window_xh - *x1) / (*x2 - *x1);
          *x1 = window_xh;
        }

      if (loc_2 == BELOW)
        {
          *y2 = *y1 + (*y2 - *y1) * (window_xl - *x1) / (*x2 - *x1);
          *x2 = window_xl;
        }
      else if (loc_2 == ABOVE)
        {
          *y2 = *y1 + (*y2 - *y1) * (window_xh - *x1) / (*x2 - *x1);
          *x2 = window_xh;
        }

      return(TRUE);
    }
}

static long clipy (double window_yl,double window_yh,double *x1,double *y1,
                   double *x2, double *y2)
{
  long loc_1 = locate_value(*y1, window_yl, window_yh);
  long loc_2 = locate_value(*y2, window_yl, window_yh);

  if (loc_1 == loc_2)
    return(loc_1 == INSIDE);
  else
    {
      if (loc_1 == BELOW)
        {
          *x1 = *x1 + (*x2 - *x1) * (window_yl - *y1) / (*y2 - *y1);
          *y1 = window_yl;
        }
      else if (loc_1 == ABOVE)
        {
          *x1 = *x1 + (*x2 - *x1) * (window_yh - *y1) / (*y2 - *y1);
          *y1 = window_yh;
        }

      if (loc_2 == BELOW)
        {
          *x2 = *x1 + (*x2 - *x1) * (window_yl - *y1) / (*y2 - *y1);
          *y2 = window_yl;
        }
      else if (loc_2 == ABOVE)
        {
          *x2 = *x1 + (*x2 - *x1) * (window_yh - *y1) / (*y2 - *y1);
          *y2 = window_yh;
        }

      return(TRUE);
    }
}

int clip_polygon(double polygon_x[], double polygon_y[],
                 int *polygon_p, window_t *window)
{
    double tmp_x[MAXPOLY], tmp_y[MAXPOLY];
    long   tmp_p;
    long   lpp;

    long   inside, closed;
    double x1, y1, x2, y2;
    long   i;

    /* If invalid polygon then ignore it and return */
    if (*polygon_p < MINPOLY)
        return(FALSE);

    closed = polygon_x[0] == polygon_x[*polygon_p-1] &&
             polygon_y[0] != polygon_y[*polygon_p-1];

    if (!closed)
    {
        polygon_x[*polygon_p] = polygon_x[0];
        polygon_y[*polygon_p] = polygon_y[0];
        (*polygon_p)++;
    }

    tmp_p = 0;
    for (i = 0; i < *polygon_p - 1; i++)    /* Clipping in x */
    {
        x1 = polygon_x[i];
        y1 = polygon_y[i];
        x2 = polygon_x[i+1];
        y2 = polygon_y[i+1];

        inside = clipx(window->xmin, window->xmax, &x1, &y1, &x2, &y2);

        if (inside) /* Vector part inside window exists */
        {
            append_point(x1, y1, tmp_x, tmp_y, &tmp_p);
            append_point(x2, y2, tmp_x, tmp_y, &tmp_p);
        }
    }

    if (tmp_p > 0)  /* close polygon */
        append_point(tmp_x[0], tmp_y[0], tmp_x, tmp_y, &tmp_p);

    *polygon_p = 0;
    if (tmp_p < MINPOLY)  /* Not enough points for continuation */
        return(FALSE);

    for (i = 0; i < tmp_p - 1; i++) /* Clipping in y */
    {
        x1 = tmp_x[i];
        y1 = tmp_y[i];
        x2 = tmp_x[i+1];
        y2 = tmp_y[i+1];

        inside = clipy (window->ymin, window->ymax, &x1, &y1, &x2, &y2);

        if (inside) /* Vector part inside window exists */
        {
            lpp = (long) *polygon_p;
            append_point(x1, y1, polygon_x, polygon_y, &lpp);
            append_point(x2, y2, polygon_x, polygon_y, &lpp);
            *polygon_p = (int)lpp;
        }
    }

    if (*polygon_p > 0 && closed)      /* close polygon */
    {
        lpp = (long) *polygon_p;
        append_point(polygon_x[0], polygon_y[0], polygon_x, polygon_y, &lpp);
        *polygon_p = (int)lpp;
    }

    return(*polygon_p >= MINPOLY);
}

int clip_line(double *x1,double *y1,double *x2,double *y2, window_t *window)
{
   return(clipx(window->xmin, window->xmax, x1, y1, x2, y2) &&
          clipy(window->ymin, window->ymax, x1, y1, x2, y2));
}
