      subroutine sepmanmat ( iinmat, rinmat, matr1, matr2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 24-01-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     General subroutine for manipulations with SEPRAN matrices
!     Old subroutine manmat
! **********************************************************************
!
!                       KEYWORDS
!
!     manipulation
!     matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematr
      use sepmoduleintmat
      use sepmodulekprob
      use sepmodulemain
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iinmat(*), matr1, matr2
      double precision, intent(in) :: rinmat(*)

!   iinmat    i   In this integer input array it is indicated what type of
!                 manipulations must be preformed by subroutine sepmanmat.
!                 For some cases IINMAT acts both as input and
!                 as output array.
!                 The entries of IINMAT must be filled by the user as follows:
!                 1  Number of entries of IINMAT that are filled by the user.
!                    IINMAT(1) must be at least equal to 2.
!                    For positions that are not filled by the user (except the
!                    positions 1 and 2) defaults are used.
!                 2  ICHOIS   General choice parameter indicating what type of
!                             manipulations have to be carried out.
!                             ICHOIS must always have been given a value by
!                             the user.
!                    Description of the possible values of ICHOIS
!                      76     Compare matrix 1 and matrix 2 and print all
!                             differences larger than RINMAT(1)
!   matr1     i   Integer SEPRAN array of length 5 corresponding to matrix 1
!   matr2     i   Integer SEPRAN array of length 5 corresponding to matrix 2
!   rinmat    i   Double precision input and output array. The contents of
!                 RINMAT depend on the contents of IINMAT.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: jmethod1, jmethod2, k, ichois, intmat1, intmat2, iseqkm
      integer :: iprob1, iprob2
      integer, pointer :: irows(:), icols(:)
      type (intmatrix), pointer :: IM1, IM2
      double precision :: timebefore(2)

!     IM1            Refers to intmt(intmat1)
!     IM2            Refers to intmt(intmat2)
!     ichois         Choice parameter, see IINMAP
!     icols          Contains column numbers of non-zero elements
!                    of matrix in sequence defined by irows
!     intmat1        Array INTMAT corresponding to matrix 1
!     intmat2        Array INTMAT corresponding to matrix 2
!     iprob1         Problem number corresponding to intmat1
!     iprob2         Problem number corresponding to intmat2
!     irows          Contains pointers to rows in matrix and icols
!                    irows(1) = 0
!                    number of non-zeros per row is irows(i+1)-irows(i)
!     iseqkm         sequence number of mesh in km
!     jmethod1       Method number corresponding to first array
!     jmethod2       Method number corresponding to second array
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!

!     ERCLMN         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERSETTIME      Get the CPU time at start of subroutine
!     INMT00         Find differences between two matrices
!     INMT01         Print all differences between matrix 1 and 2 exceeding a
!                    threshold value
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPFILLKPROB   Fill parameters in module sepmodulekprob
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!       1   ICHOIS is incorrect
!    1230   Array INTMT1 is incorrect
!    1231   Array INTMT2 is of incorrect type
!    1232   INTMAT1 and INTMAT2 correspond to different problems
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepmanmat' )
      if ( itime>0 ) call ersettime ( timebefore )

!     --- Check input

      ichois = 76
      if ( iinmat(1)>1 ) then
         ichois = iinmat(2)
         if ( ichois/=76 ) then
            call errint ( ichois, 1 )
            call errsub ( 1, 1, 0, 0 )
         end if
      end if
      intmat1 = realmt(matr1)%intmatref
      intmat2 = realmt(matr1)%intmatref
      IM1 => intmt(intmat1)
      IM2 => intmt(intmat2)
      jmethod1 = IM1%jmethod
      jmethod2 = IM2%jmethod
      iprob1 = IM1%iprob
      iprob2 = IM2%iprob
      iseqkp = IM1%iseqkp
      iseqkm = kp(iseqkp)%iseqkm
      call sepfillkmesh(iseqkm)
      if ( jmethod1/=5 ) then
         call errint ( 102, 1 )
         call errint ( jmethod1, 2 )
         call errint ( 5, 3 )
         call errsub ( 1230, 3, 0, 0 )
      end if
      if ( jmethod2/=5 ) then
         call errint ( 102, 1 )
         call errint ( jmethod2, 2 )
         call errint ( 5, 3 )
         call errsub ( 1231, 3, 0, 0 )
      end if
      iprob = iprob1
      kprobloc => kp(iseqkp)%kprobsub(iprob)%kprobloc
      call sepfillkprob
      if ( iprob2/=iprob ) then
         call errint ( iprob, 1 )
         call errint ( iprob2, 2 )
         call errsub ( 1232, 2, 0, 0 )
      end if
      if ( ierror/= 0 ) go to 1000

!     --- Compute constants

      S1_ii => realmt(matr1)%S_ii
      S2_ii => realmt(matr2)%S_ii

      irows => IM1%rowsii
      icols => IM1%colsii

      call inmt00 ( nrusol, irows, icols, S1_ii, S2_ii, rinmat(1) )

!     --- Compare second part of MATR1 and MATR2

      if ( nbound>0 ) then

!     --- Nbound>0, INTMT2 part 2

         S1_ii => realmt(matr1)%S_ip
         S2_ii => realmt(matr2)%S_ip
         irows => IM1%rowsip
         icols => IM1%colsip

         call inmt01 ( S1_ii, S2_ii, irows, &
                       icols, nbound, nusol, rinmat(1) )

         S1_ii => realmt(matr1)%S_pi
         S2_ii => realmt(matr2)%S_pi

         call inmt01 ( S1_ii, S2_ii, irows, &
                       icols, nbound, nusol, rinmat(1) )

         S1_ii => realmt(matr1)%S_pp
         S2_ii => realmt(matr2)%S_pp

         call inmt01 ( S1_ii, S2_ii, irows, &
                       icols, nbound, nusol, rinmat(1) )

      end if

      if ( npboun>0 ) then

!     --- Nbound>0, INTMT2 part 2

         S1_ii => realmt(matr1)%S_ic
         S2_ii => realmt(matr2)%S_ic
         irows => IM1%rowsip
         icols => IM1%colsip

         call inmt01 ( S1_ii, S2_ii, irows, &
                       icols, npboun, nusol, rinmat(1) )

      end if  ! ( npboun>0 )

1000  call erclmn ( 'sepmanmat', timebefore )
      end
