      subroutine writbs ( ichois, inum, numarr, namevc, iarray, &
                          ihelp, typevec )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  1.4  date 13-12-2017 New storage of intmat
!        version  1.3  date 04-06-2014 Allow -102 in intmat(2)
!        version  1.2  date 21-11-2012 Long integers
!
!   copyright (c) 1999-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     writbs writes arrays to  background storage
!
! **********************************************************************
!
!                       KEYWORDS
!
!     backing_storage
!     write
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduledebug
      use sepmodulecomio
      use sepmodulefiles
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      character *(*) namevc
      integer ichois, inum, numarr, iarray, ihelp(*), typevec

!     iarray     i    Array to be written on backing storage.
!                     IARRAY may be both a real or an integer array.
!                     SEPRAN arrays are always integer arrays.
!     ichois     i    Choice parameter. This parameter indicates whether the
!                     array is a SEPRAN array (ICHOIS = 0) or not (ICHOIS # 0)
!                     When IARRAY corresponds to a user array, the value of
!                     |ICHOIS| denotes the length of this array, and the sign of
!                     ICHOIS denotes whether it is a real array (ICHOIS<0) or an
!                     integer array (ICHOIS>0).
!     ihelp      i    integer extra array needed.
!                     Extra arrays necessary for SEPRAN arrays:
!                       IARRAY is KMESH:   IHELP is dummy
!                       IARRAY is KPROB:   IHELP is KMESH
!                       IARRAY is ISOL:    IHELP is KPROB
!                       IARRAY is INTMAT:  IHELP is KPROB
!                       IARRAY is MATR:    IHELP is dummy
!     inum       i    Choice parameter. Possibilities:
!                     When INUM<=0 the array is written on the next free
!                     positions on backing storage.
!                     At most 500 arrays can be written on the permanent file.
!                     When INUM = -1 the subroutine prints a text of the
!                     following form
!                     Array "NAME" is stored as array number "NUMARR"
!                     where "NAME" denotes the value of NAME and "NUMARR"
!                     denotes the value of NUMARR.
!                     When INUM>0  the array is written on positions of a
!                     preceding written array, with array number INUM.
!                     See also NUMARR.
!                     The length of the new and old array may be different.
!                     This possibility may also be used when there is no
!                     preceding array with this number.
!     namevc     i    Name of the array to be written to backing storage.
!                     Name is a character constant of at most 10 characters.
!                     NAME is only used for printing purposes, when INUM = -1.
!     numarr     o    Each array that is written by writbs gets a sequence
!                     number, the first array gets number 1, the second number 2
!                     etc.
!                     At output NUMARR gets the value of this sequence number.
!                     Therefore NUMARR must be a variable and not a constant.
!     typevec    i    Type of sepran vector
!                     0: not a sepran vector
!                     100: kmesh
!                     101: kprob
!                     102: intmat
!                     103: matr
!                     >= 110: vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jchois
      logical :: newarray
      integer, allocatable :: iwork(:)

!     iwork          integer work array to store one record
!     jchois         Absolute value of ichois
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERTRACE        Prints trace back of subroutines
!     INSTOP         Stop the program
!     WRITB2         Write array KMESH
!     WRITB3         Write array KPROB
!     WRITB4         Write array solution type arrays
!     WRITB5         Write array INTMAT
!     WRITB6         Write array MATR
!     WRITB7         Write array integer user array
!     WRITB8         Write array real user array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1242   Array IHELP is not of the type KPROB
!    1715   The array to be written is of the wrong type
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'writbs' )

      allocate ( iwork(nrec2), stat = error )
      if ( error/=0 ) call eralloc ( error, nrec2, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- consider different possibilities

      jchois = abs(ichois)

      if ( ichois>0 ) then

!     --- ichois>0, integer array of user

         call writb7 ( inum, jchois, numarr, namevc, iarray, iwork )

      else if ( ichois<0 ) then

!     --- ichois<0, real array of user

         call writb8 ( inum, jchois, numarr, namevc, iarray, iwork )

      else

!     --- iarray(2) has an undefined value

         select case (typevec)
         case(100)

!        --- kmesh

            call writb2 ( inum, numarr, namevc, iarray, iwork )

         case(101)

!        --- kprob

            call writb3 ( inum, numarr, namevc, iwork )

         case(102)

!        --- intmat

            call writb5 ( inum, numarr, namevc, iarray, iwork )

         case(103)

!        --- matr

            write(irefwr,*) 'writing of matrices not implemented'
            call ertrace
            call instop

         case(110:120)

!        --- iarray(2) = 110, 115, 116, 117, 118, 120, iarray is of i-sol type

            call writb4 ( inum, numarr, namevc, iarray, iwork )

         end select  ! case (buf(iarray(1))%itype)

      end if

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  call erclos ( 'writbs' )
      end subroutine writbs
