      subroutine petscaddmats ( nrusol, irows, icols, kprobhglob, amat )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-08-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Add sepran matrix to empty petsc matrix
!
! **********************************************************************
!
!                       KEYWORDS
!
!     petsc
!     linear_solver
!     matrix
! **********************************************************************
!
!                       MODULES USED
!
      use seppetscmod   ! defines all petsc stuff
      use sepmodulecomio
      use sepmodulecpack
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: nrusol, irows(nrusol+1), icols(*), &
                             kprobhglob(nrusol)
      double precision, intent(in) :: amat(*)

!     amat           i    large matrix in sepran format (row compact storage)
!     icols          i    Contains column numbers of non-zero elements
!                         of matrix in sequence defined by irows
!     irows          i    Contains pointers to rows in matrix and icols
!                         irows(1) = 0
!                         number of non-zeros per row is irows(i+1)-irows(i)
!     kprobhglob     i    Array kprobh with global numbers of free degrees
!                         of freedom (length nrusol)
!     nrusol         i    Number of free unknowns
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: nrow, i, inusol, rowindex(1), ncol, istart
      integer, allocatable :: colindex(:)

!     colindex       Contains the column numbers of the non-zero of the
!                    row of the matrix in c-sense, hence starting with 0
!     i              Counting variable
!     inusol         Position of unknown in array USOL
!     istart         First position used at a part
!     ncol           Number of non-zero elements per row in the lower
!                    triangular matrix
!     nrow           Number of rows to be added (Here 1)
!     rowindex       Row number in c-sense, hence starting with 0
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MATSETVALUES   Inserts or adds a block of values into a matrix (petsc)
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'petscaddmats' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from petscaddmats'
         write(irefwr,1) 'nrusol', nrusol
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nrow = 1  ! number of rows to be copied in 1 call

      allocate ( colindex(nrusol), stat = error )
      if ( error/=0 ) call eralloc ( error, nrusol, 'colindex' )
      if ( ierror/=0 ) go to 1000

!     --- Loop over rows

      do i = 1, nrusol

         inusol = i                ! row index
         istart = irows(i)         ! start in array icols (-1)
         ncol = irows(i+1)-istart  ! number of columns in row

         if ( debug ) &
            write(irefwr,1) 'i, inusol, istart, ncol', &
                             i, inusol, istart, ncol

         if ( parallel ) then

!        --- Parallel case, use global numbers
!            Subtract 1 because petsc uses c numbering (from 0)

            inusol = kprobhglob(inusol)
            colindex(1:ncol) = kprobhglob(icols(istart+1:istart+ncol))-1

         else

!        --- Serial case
!            Subtract 1 because petsc uses c numbering (from 0)

            colindex(1:ncol) = icols(istart+1:istart+ncol)-1

         end if  ! ( parallel )

         rowindex(1) = inusol-1  ! petsc numbering

         if ( debug ) then
            write(irefwr,1) 'rowindex', rowindex
            call prinin ( colindex, ncol, 'colindex' )
            call prinrl ( amat(istart+1), ncol, 'amat' )
         end if  ! ( debug )

         call MatSetValues ( A(iseqmat), nrow, rowindex, ncol, colindex, &
                             amat(istart+1), ADD_VALUES, ierrpetsc )

      end do  ! i = 1, nrusol

      deallocate ( colindex, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'colindex' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End petscaddmats'

      end if  ! ( debug )
      call erclos ( 'petscaddmats' )

      end subroutine petscaddmats
