      subroutine seppetscsol ( Amat, usol, rhsd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-07-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Actual call to petsc solver
!
! **********************************************************************
!
!                       KEYWORDS
!
!     petsc
!     linear_solver
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemat
      use sepmodulesolve
      use sepmodulekprob
      use seppetscmod   ! defines all petsc stuff
      use sepmodulecpack
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix) :: Amat
      double precision :: usol(Amat%neq)
      double precision, intent(in) :: rhsd(Amat%neq)

!     Amat          i/o   Structure of type matrix containing the necessary
!                         information of the matrix
!     rhsd           i    right-hand-side vector
!     usol           o    solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: its, iter, reason
      double precision :: rnorm, dummy
      double precision, pointer :: usollc(:)
      logical :: optionsfile, extradebug

!     dummy          Dummy parameter
!     extradebug     If true extra debugging statements are carried out
!     iter           Iteration sequence number
!     its            Contains number of iterations
!     optionsfile    If true the petsc options file petscrc is used
!                    otherwise the sepran opties
!     reason         Reason why KSP stopped the iteration
!     rnorm          norm of vector
!     usollc         Temporary solution vector (special for petsc)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      external :: seppetscmonitor

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     KSPCREATE      Creates the default KSP context.
!     KSPDESTROY     Destroy solver
!     KSPGETCONVERGEDREASON  Gets the reason the KSP iteration was stopped
!     KSPGETITERATIONNUMBER  Gets the current iteration number
!                    if the KSPSolve() is complete, returns the number
!                    of iterations used
!     KSPGETPC       Returns a pointer to the preconditioner context
!                    set with KSPSetPC().
!     KSPSETINITIALGUESSNONZERO  Tells the iterative solver that the initial
!                    guess is nonzero;
!                    otherwise KSP assumes the initial guess is to be zero
!                    (and thus zeros it out before solving).
!     KSPSETOPERATORS  Sets the matrix associated with the linear system
!                    and a (possibly) different one associated with
!                    the preconditioner
!     KSPSETTOLERANCES  Sets the relative, absolute, divergence, and maximum
!                    iteration tolerances used by the default KSP
!                    convergence testers
!     KSPSOLVE       Solves linear system.
!     KSPVIEW        Prints the KSP data structure.
!     MATDESTROY     Destroy matrix
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     SEPPETSCMONITOR Subroutine to print the iterations
!     SEPPETSCSOL1   Sets the type of iteration method to be used by petsc
!     SEPPETSCSOL2   Sets the type of preconditioner to be used by petsc
!     SEPPETSCSOL3   Print error message because solver has stopped iterating
!     VECDESTROY     Destroy vector
!     VECDUPLICATE   Creates a new vector of the same type as an existing vector
!     VECGETARRAYF90 Accesses a vector array from Fortran90. For default PETSc
!                    vectors, VecGetArrayF90() returns a pointer to the local
!                    data array.
!     VECGHOSTGETLOCALFORM  Obtains the local ghosted representation of a
!                    parallel vector
!     VECGHOSTUPDATEBEGIN  Begins the vector scatter to update the vector from
!                    local representation to global or global representation
!                    to local.
!     VECGHOSTUPDATEEND  Ends the vector scatter to update the vector from
!                    local representation to global or global representation
!                    to local.
!     VECNORM        Computes the vector norm.
!     VECRESTOREARRAYF90  Restores a vector to a usable state after a call
!                    to VecGetArrayF90().
!     VECVIEW        Views a vector object.
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seppetscsol' )
      debug = .false. .and. ioutp>=0
      extradebug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seppetscsol'
         write(irefwr,1) 'nrusol, iseqmat', nrusol, iseqmat
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Create vector x by copying vector b

      call VecDuplicate ( b, x, ierrpetsc )  ! b has same type as x

!     --- Create linear solver context, necessary

      call KSPCreate ( PETSC_COMM_WORLD, sepksp, ierrpetsc )
      call KSPSetFromOptions ( sepksp, ierrpetsc )

      if ( debug ) &
         write(irefwr,1) 'Amat%istartsol, typesolv', Amat%istartsol, typesolv
      if ( Amat%istartsol>0 .and. typesolv/=30 ) then

!     --- istartsol>0, use old solution
!         Not possible with method = preonly
!         Copy vector in usol

         call VecGetArrayF90 ( x, usollc, ierrpetsc )
         usollc(1:nrusol) = usol(1:nrusol)
         call KSPSetInitialGuessNonzero ( sepksp, PETSC_TRUE, ierrpetsc )
         call VecRestoreArrayF90(x,usollc,ierrpetsc)

      end if  ! ( Amat%istartsol>0 )

!     --- Set operators. Here the matrix that defines the linear system
!         also serves as the preconditioning matrix.

      call KSPSetOperators ( sepksp, A(iseqmat), A(iseqmat), ierrpetsc )

!     --- enable setting the preconditioning or iteration method

      call KSPGetPC ( sepksp, seppc, ierrpetsc )

      inquire ( file = 'petscrc', exist = optionsfile )

      if ( debug ) write(irefwr,*) 'optionsfile', optionsfile

      if ( .not. optionsfile ) then

!     --- Options file does not exits, choose type of iteration method

         if ( typesolv>0 ) call seppetscsol1 ( Amat )

!        --- Choose Preconditioner

         call seppetscsol2 ( Amat )

!        --- Set tolerance and maximum number of iterations

         call KSPSetTolerances ( sepksp, rel_accuray, abs_accuray, &
                                 div_accuray, nmax, ierrpetsc )

         if ( Amat%iprint>1 .or. debug ) then

!        --- printlev>=2, hence print the convergence of all iterations
!            The actual printing is done in subroutine seppetscmonitor

            write(irefwr,*) 'iteration      norm'

      !      call KSPMonitorSet(sepksp,seppetscmonitor,PETSC_NULL_OBJECT, &
            call KSPMonitorSet(sepksp,seppetscmonitor,PETSC_NULL_FUNCTION, &
                               PETSC_NULL_FUNCTION,ierrpetsc)

         end if

      end if  ! ( .not. optionsfile )


!     --- Solve the linear system

      call KSPSolve(sepksp,b,x, ierrpetsc)
      if ( extradebug ) &
         call VecView ( x, PETSC_VIEWER_STDOUT_WORLD, ierrpetsc )

      call KSPGetConvergedReason ( sepksp, reason, ierrpetsc )

      if ( reason<0 ) then

!     --- reason<0, iteration has stopped with error

         call seppetscsol3 ( reason )
         go to 900

      end if  ! ( reason<0 )

      if ( parallel ) then

!     --- Parallel case
!         First store ghost values in x

         call VecGhostUpdateBegin ( x, INSERT_VALUES, SCATTER_FORWARD, &
                                    ierrpetsc )
         call VecGhostUpdateEnd ( x, INSERT_VALUES, SCATTER_FORWARD, &
                                  ierrpetsc )

!        --- Next store values in local array xloc

         call vecghostgetlocalform ( x, xloc, ierrpetsc )

         if ( debug ) &
            call VecView ( xloc, PETSC_VIEWER_STDOUT_WORLD, ierrpetsc )

!        --- Copy vector in usol and usol in usol

         call VecGetArrayF90 ( xloc, usollc, ierrpetsc )

         if ( extradebug ) call prinrl ( usollc, nrusol, 'usol' )

!        --- copy usol in output array

         usol(1:nrusol) = usollc(1:nrusol)

         call VecRestoreArrayF90 ( xloc, usollc, ierrpetsc )
         call VecDestroy ( xloc, ierrpetsc )  ! remove loc

      else

!     --- Serial case
!         Copy vector in usol and usol in usol

         call VecGetArrayF90 ( x, usollc, ierrpetsc )

         if ( extradebug ) call prinrl ( usollc, nrusol, 'usol' )

         usol(1:nrusol) = usollc(1:nrusol)     ! copy usol in output array

         call VecRestoreArrayF90 ( x, usollc, ierrpetsc )

      end if  ! ( parallel )

      if ( extradebug ) then

!     --- View solver info; we could instead use the option -ksp_view

         call KSPView(sepksp,PETSC_VIEWER_STDOUT_WORLD, ierrpetsc)
         call VecNorm(x,NORM_2,rnorm,ierrpetsc)
         write(irefwr,2) 'norm', rnorm

      end if  ! ( debug )

!     --- get number of iterations

      call KSPGetIterationNumber(sepksp,its,ierrpetsc)
      niter = its

      if ( Amat%iprint>0 ) then

         write(irefwr,1) 'number of iterations', its

      end if  ! ( Amat%iprint>0 )

!     --- Destroy sepksp, solution vector and right-hand side

900   call VecDestroy ( x, ierrpetsc )
      call VecDestroy ( b, ierrpetsc )
      call MatDestroy ( A(iseqmat), ierrpetsc )
      call KSPDestroy ( sepksp, ierrpetsc )

1000  if ( debug ) then

!     --- Debug information

         if ( extradebug ) call prinrl ( usol, nrusol, 'usol/1' )
         write(irefwr,*) 'End seppetscsol'

      end if  ! ( debug )
      call erclos ( 'seppetscsol' )

      end subroutine seppetscsol

